@@ -19,29 +19,231 @@
 
 
 #include "condor_common.h"
+#include "proc.h"              // for job statuses
+#include "condor_accountant.h" // for PriorityDelta
+#include "condor_config.h"
 #include "analysis.h"
 #include "list.h"
 #include "simplelist.h"
 #include "extArray.h"
+#include "condor_classad_util.h"
 
 #include <iostream>
+#include <sstream>
 
 using namespace std;
+using namespace classad_analysis;
+using namespace classad_analysis::job;
 
 ClassAdAnalyzer::
-ClassAdAnalyzer( )
-{
-	jobReq = NULL;
+ClassAdAnalyzer( bool ras ) :
+  result_as_struct(ras), m_result(NULL), jobReq(NULL) { 
+
+  stringstream std_rank;
+  stringstream preempt_rank;
+  stringstream preempt_prio;
+  
+  std_rank << "MY." << ATTR_RANK << " > MY." << ATTR_CURRENT_RANK;
+  preempt_rank << "MY." << ATTR_RANK << " >= MY." << ATTR_CURRENT_RANK;
+  preempt_prio << "MY." << ATTR_REMOTE_USER_PRIO << " > TARGET." << ATTR_SUBMITTOR_PRIO << " + " << PriorityDelta;
+
+  ParseClassAdRvalExpr(std_rank.str().c_str(), std_rank_condition);
+  ParseClassAdRvalExpr(preempt_rank.str().c_str(), preempt_rank_condition);
+  ParseClassAdRvalExpr(preempt_prio.str().c_str(), preempt_prio_condition);
+  
+  char *preq;
+  if( NULL == ( preq = param( "PREEMPTION_REQUIREMENTS" ) ) ) {
+    // No PREEMPTION_REQUIREMENTS; defaulting to FALSE
+    ParseClassAdRvalExpr( "FALSE", preemption_req );
+  } else {
+    if( ParseClassAdRvalExpr( preq , preemption_req ) ) {
+      // Failed to parse PREEMPTION_REQUIREMENTS; defaulting to FALSE
+      ParseClassAdRvalExpr( "FALSE", preemption_req );
+    }
+#if !defined(WANT_OLD_CLASSADS)
+	ExprTree *tmp_expr = AddTargetRefs( preemption_req, TargetJobAttrs );
+	delete preemption_req;
+	preemption_req = tmp_expr;
+#endif
+    free( preq );
+  }
 }
 
 ClassAdAnalyzer::
 ~ClassAdAnalyzer( )
 {
+	delete std_rank_condition;
+	delete preempt_rank_condition;
+	delete preempt_prio_condition;
+	delete preemption_req;
 	if( jobReq ) {
 		delete jobReq;
 	}
+	
+	if( m_result ) {
+	  delete m_result;
+	  m_result = NULL;
+	}
 }
 
+// Returns "true" if the job is not matched and IDLE, false otherwise
+bool ClassAdAnalyzer::
+NeedsBasicAnalysis( ClassAd *request ) {
+  int status;
+  int matched = false;
+
+  request->LookupInteger( ATTR_JOB_STATUS, status );
+  request->LookupInteger( ATTR_JOB_MATCHED, matched );
+
+  // XXX: are there cases where we need "basic" analysis even though
+  // we're matched?
+  if (matched) {
+    return false;
+  }
+  
+  switch(status) {
+  // XXX:  should we have "is_running/completed/removed" in m_result?
+  // XXX:  should we add "is_held" to m_result/include hold_reason?
+  case RUNNING:
+  case HELD:
+  case REMOVED:
+  case COMPLETED:
+  case TRANSFERRING_OUTPUT:
+    return false;
+  default:
+    return true;
+  }
+}
+
+void ClassAdAnalyzer::
+BasicAnalyze( ClassAd *request, ClassAd *offer ) {
+  // XXX: this code could/should be refactored out (from here and
+  // condor_q.V6/queue.cpp) and into an analysis library
+
+  // NB:  for now, we only use this "basic analysis" if we're
+  // generating a result struct; so return otherwise.  Be sure to
+  // eliminate this check if this code is used elsewhere!
+
+  if (!result_as_struct) { return; }
+
+  char remote_user[128];
+  EvalResult eval_result;
+
+  bool satisfied_std_rank = EvalExprTree(std_rank_condition, offer, request, &eval_result) && eval_result.type == LX_INTEGER && eval_result.i == TRUE;
+  bool satisfied_preempt_prio = EvalExprTree( preempt_prio_condition, offer, request, &eval_result ) && eval_result.type == LX_INTEGER && eval_result.i == TRUE; 
+  bool satisfied_preempt_rank = EvalExprTree( preempt_rank_condition, offer, request, &eval_result ) && eval_result.type == LX_INTEGER && eval_result.i == TRUE;
+  bool satisfied_preempt_req = EvalExprTree( preemption_req, offer, request, &eval_result ) && eval_result.type == LX_INTEGER && eval_result.i == TRUE;
+
+  if (!IsAHalfMatch(request, offer)) {
+    result_add_explanation(classad_analysis::MACHINES_REJECTED_BY_JOB_REQS, offer);
+    return;
+  } 
+
+  if (!IsAHalfMatch(offer, request)) {
+    result_add_explanation(classad_analysis::MACHINES_REJECTING_JOB, offer);
+    return;
+  }
+
+  if (! offer->LookupString( ATTR_REMOTE_USER, remote_user )) {
+    if ( satisfied_std_rank )  {
+      // Machine satisfies job requirements, job satisfies machine
+      // constraints, no remote user
+      result_add_explanation(classad_analysis::MACHINES_AVAILABLE, offer);
+      return;
+    } else {
+      // Standard rank condition failed
+      result_add_explanation(classad_analysis::MACHINES_REJECTING_UNKNOWN, offer);
+      return;
+    }
+  }
+
+  if ( satisfied_preempt_prio )  {
+    if ( satisfied_std_rank ) {
+      // Satisfies preemption priority condition and standard rank
+      // condition; thus available
+      result_add_explanation(classad_analysis::MACHINES_AVAILABLE, offer);
+      return;      
+    } else {
+      if ( satisfied_preempt_rank ) {
+	// Satisfies preemption priority and rank conditions, and ...
+	if (satisfied_preempt_req) {
+	  // ... also satisfies PREEMPTION_REQUIREMENTS:  available
+	  result_add_explanation(classad_analysis::MACHINES_AVAILABLE, offer);
+	  return;      	  
+	} else {
+	  // ... doesn't satisfy PREEMPTION_REQUIREMENTS:  not available
+	  result_add_explanation(classad_analysis::PREEMPTION_REQUIREMENTS_FAILED, offer);
+	  return;
+	}
+      } else {
+	// The comments on the equivalent code path in condor_q
+	// indicate that this case usually implies "some unknown problem"
+	result_add_explanation(classad_analysis::PREEMPTION_FAILED_UNKNOWN, offer);
+	return;
+      }
+    }
+  } else {
+    // Failed preemption priority condition
+    result_add_explanation(classad_analysis::PREEMPTION_PRIORITY_FAILED, offer);
+    return;
+  }
+
+}
+
+void ClassAdAnalyzer::
+ensure_result_initialized(classad::ClassAd *request) {
+    // Set up result object, only if necessary
+
+    // Other parts of this code are written to assume that a
+    // ClassAdAnalyzer can be used to analyze multiple jobs; we make
+    // the same assumption here.  The overall interface of this code
+    // should be marked with a big FIXME.
+
+    if (!result_as_struct) return;
+
+    if (m_result != NULL && !(m_result->job_ad()).SameAs(request)) {
+      delete m_result;
+      m_result = NULL;
+    }
+
+    if (m_result == NULL) {
+      m_result = new classad_analysis::job::result(*request);
+    }
+}
+
+void ClassAdAnalyzer::
+result_add_suggestion(suggestion s) {
+  if (!result_as_struct) return;
+  ASSERT(m_result);
+  
+  m_result->add_suggestion(s);
+}
+
+void ClassAdAnalyzer::
+result_add_explanation(matchmaking_failure_kind mfk, classad::ClassAd resource) {
+  if (!result_as_struct) return;
+  ASSERT(m_result);
+  
+  m_result->add_explanation(mfk, resource);
+}
+
+void ClassAdAnalyzer::
+result_add_explanation(matchmaking_failure_kind mfk, ClassAd *resource) {
+  if (!result_as_struct) return;
+  ASSERT(m_result);
+  
+  m_result->add_explanation(mfk, resource);
+}
+
+void ClassAdAnalyzer::
+result_add_machine(classad::ClassAd resource) {
+  if (!result_as_struct) return;
+  ASSERT(m_result);
+  
+  m_result->add_machine(resource);
+}
+
+
 bool ClassAdAnalyzer::
 AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
 {
@@ -64,6 +266,22 @@ AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
 		return true;
 	}
     explicit_classad  = AddExplicitTargets( converted_classad );
+
+    ensure_result_initialized(explicit_classad);
+    
+    bool do_basic_analysis = NeedsBasicAnalysis(request);
+    offers.Rewind();
+    ClassAd *ad;
+    while((ad = offers.Next())) {
+      classad::ClassAd *new_ad = toNewClassAd(ad);
+      result_add_machine(*new_ad);
+      delete new_ad;
+
+      if (do_basic_analysis) {
+	BasicAnalyze(request, ad);
+      }
+    }
+
 	success = AnalyzeJobReqToBuffer( explicit_classad, rg, buffer );
 
     delete converted_classad;
@@ -113,6 +331,7 @@ AnalyzeJobAttrsToBuffer( ClassAd *request, ClassAdList &offers,
 		return true;
 	}
     explicit_classad  = AddExplicitTargets( converted_classad );
+    ensure_result_initialized(explicit_classad);
 	success = AnalyzeJobAttrsToBuffer( explicit_classad, rg, buffer );
 
     delete converted_classad;
@@ -162,7 +381,7 @@ AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string
 		if( ( *t == '&' ) && ( *( t+1 ) == '&' ) ) {
 			lastAnd = t + 2;
 		}
-		if( t >= lineStart + 80 ) {
+		if( distance( lineStart, t ) >= 80 ) {
 			if( lastAnd != lineStart ) {
 				temp_buffer.replace( lastAnd, lastAnd + 1, 1, '\n' );
 				lineStart = lastAnd + 1;
@@ -296,7 +515,7 @@ AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string
 			// create map from original Condition order to sorted order
 		int numConds = 0;
 		profile->GetNumberOfConditions( numConds );
-		int condMap[numConds];
+		ExtArray<int> condMap ( numConds );
 		int i = 0;
 		while( mapList.Next( index ) ) {
 			condMap[index] = i;
@@ -324,10 +543,12 @@ AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string
 			switch( condition->explain.suggestion ) {
 			case ConditionExplain::REMOVE: {
 				sprintf( suggest, "REMOVE" );
+				result_add_suggestion(suggestion(suggestion::REMOVE_CONDITION, cond_s));
 				break;
 			}
 			case ConditionExplain::MODIFY: {
 				pp.Unparse( value_s, condition->explain.newValue );
+				result_add_suggestion(suggestion(suggestion::MODIFY_CONDITION, cond_s, value_s));
 				strncpy( value, value_s.c_str( ), 63 );
 				sprintf( suggest, "MODIFY TO %s", value );
 				break;
@@ -359,8 +580,8 @@ AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string
 			buffer += "\n";
 			while( profile->explain.conflicts->Next( rawIS ) ) {
 				sortedIS.Init( numConds );
-				IndexSet::Translate( *rawIS, condMap, numConds, numConds,
-									 sortedIS );
+				IndexSet::Translate( *rawIS, condMap.getarray (), 
+									numConds, numConds, sortedIS );
 				buffer += "  conditions: ";
 				bool firstNum = true;
 				for( i = 0; i < numConds; i++ ) {
@@ -379,6 +600,7 @@ AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string
 			}
 		}	
 
+        
 		p++;
 	}
 	return true;
@@ -417,6 +639,7 @@ AnalyzeJobAttrsToBuffer( classad::ClassAd *request, ResourceGroup &offers,
 		string undefAttr = "";
 		adExplain.undefAttrs.Rewind( );
 		while( adExplain.undefAttrs.Next( undefAttr ) ) {
+		  result_add_suggestion(suggestion(suggestion::DEFINE_ATTRIBUTE, undefAttr));
 			buffer += undefAttr;
 			buffer += "\n";
 		}
@@ -487,6 +710,7 @@ AnalyzeJobAttrsToBuffer( classad::ClassAd *request, ResourceGroup &offers,
 				}
 				strncpy( suggest, suggest_s.c_str( ), 64 ); 
 				sprintf( formatted, "%-24s%s\n", attr, suggest );
+				result_add_suggestion(suggestion(suggestion::MODIFY_ATTRIBUTE, attr, suggest_s));
 				tempBuff += formatted;
 			}
 			default: { }
@@ -968,13 +1192,13 @@ SuggestConditionModify( Profile *p, ResourceGroup &rg )
 	ExtArray< string > attrs;
 	ExtArray< ValueRange * > vrs;
 	string attr = "";
-	int vr4Cond[numConds];
+	ExtArray<int> vr4Cond ( numConds );
 	int attrNum = 0;
 	int condNum = 0;
 	int vrNum = 0;
-	classad::Operation::OpKind ops[numConds];
-	Condition *conds[numConds];
-	bool tooComplex[numConds]; 
+	ExtArray<classad::Operation::OpKind> ops ( numConds );
+	ExtArray<Condition*> conds ( numConds );
+    ExtArray<bool> tooComplex ( numConds ); 
 //	classad::Operation::OpKind op1, op2;
 	classad::Value val;
 	p->Rewind( );
@@ -1041,14 +1265,14 @@ SuggestConditionModify( Profile *p, ResourceGroup &rg )
 			for( int i = 0; i < vrs.getsize( ); i++ ){
 				delete vrs[i];
 			}
-			return false;
+            return false;
 		}
 		condNum++;
 	}
 
 	int numVRs = attrs.getsize( );
-	classad::Value *nearestValues[numVRs];
-	for( int i = 0; i < numVRs; i++ ) {
+	ExtArray<classad::Value*> nearestValues ( numVRs );
+    for( int i = 0; i < numVRs; i++ ) {
 		nearestValues[i] = NULL;
 	}
 
@@ -1106,7 +1330,7 @@ SuggestConditionModify( Profile *p, ResourceGroup &rg )
 	classad::Value currPt, currUpper, currLower;
 	BoolValue currBVal, tempBVal;
 	val.SetUndefinedValue( );
-	classad::Value *tempVals[numVRs];
+	ExtArray<classad::Value*> tempVals ( numVRs );
 	for( int i = 0; i < numVRs; i++ ) {
 		tempVals[i] = NULL;
 	}
@@ -1159,12 +1383,7 @@ SuggestConditionModify( Profile *p, ResourceGroup &rg )
 		}
 		sumDist = 0;
 	}
-	for( int i = 0; i < numVRs; i++ ) {
-        if (tempVals[i] != NULL) {
-            delete tempVals[i];
-        }
-	}
-
+	
 	classad::Value condVal;
 	condNum = 0;
   	p->Rewind( );
@@ -1201,8 +1420,12 @@ SuggestConditionModify( Profile *p, ResourceGroup &rg )
   		}
 		condNum++;
  	}
-
-	return true;
+	for( int i = 0; i < numVRs; i++ ) {
+		if (tempVals[i] != NULL) {
+			delete tempVals[i];
+		}
+	}  
+    return true;
 }
 
 
@@ -1396,8 +1619,8 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 
 	int numReqs = reqList.Number( );
 
-	int firstContext[numReqs]; 
-	List< string > jobAttrs, undefAttrs, refdAttrs;
+	ExtArray<int> firstContext ( numReqs );
+    List< string > jobAttrs, undefAttrs, refdAttrs;
 	classad::ClassAd::iterator itr;
 
 		// build list of attributes
@@ -1512,7 +1735,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 				}
 
 					// attribute has been previously encountered.
-				if( attrInRefdAttrs ) {
+				if( attrInRefdAttrs && tempBools) {
 					BoolValue newValue;
 					BoolValue oldValue = ( *tempBools )[attrNum];
 					And( oldValue, conditionValue, newValue );
@@ -1587,8 +1810,8 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 
 		// Convert machine map
 	int numContexts = boolValueArrayList.Number();
-	int machineForContext[numContexts];
- 	int m = 0;
+	ExtArray<int> machineForContext ( numContexts );
+    int m = 0;
   	for( int i = 0; i < numContexts; i++ ) {
   	    if( ( m+1 ) < numReqs) {
   			if( i >= firstContext[m+1] ) {
@@ -1597,7 +1820,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
   	    }
   	    machineForContext[i] = m;
   	}
-
+    
 		// Create BoolTable and ValueRangeTable
 	bt.Init( numContexts, numRefdAttrs );
 	vrt.Init( numContexts, numRefdAttrs );
@@ -1752,8 +1975,8 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 		for( int i = 0; i < hrs->getsize( ); i++ ) {
 			currHR = ( *hrs )[i];
 			currHR->GetIndexSet( hasContext );
-			IndexSet::Translate( hasContext, machineForContext, numContexts,
-								 numReqs, hasMachine );
+			IndexSet::Translate( hasContext, machineForContext.getarray (), 
+								numContexts, numReqs, hasMachine );
 			hasMachine.GetCardinality( currNumContexts );
 			if( currNumContexts > maxNumContexts ) {
 				maxNumContexts = currNumContexts;
@@ -1762,8 +1985,7 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 			}
 		}
 	}
-
-
+    
 	///////////////////////////////////////
 	// STEP 4 - SET UP ATTRIBUTE EXPLAIN //
 	///////////////////////////////////////
@@ -1958,22 +2180,31 @@ AddConstraint( ValueRange *&vr, Condition *condition )
 	if( twoVals ) {
 		Interval *i1 = new Interval( );
 		Interval *i2 = new Interval( );
-		i1->lower.CopyFrom( val1 );
-		i2->lower.CopyFrom( val2 );
-		i1->upper.CopyFrom( val1 );
-		i2->upper.CopyFrom( val2 );
-		i1->openLower = false;
-		i2->openLower = false;
-		i1->openUpper = false;
-		i2->openUpper = false;
-		if( vr->IsInitialized( ) ) {
-			vr->Intersect( i1, i2, false );
+		if(i1) {
+			if(i2) {
+				i1->lower.CopyFrom( val1 );
+				i2->lower.CopyFrom( val2 );
+				i1->upper.CopyFrom( val1 );
+				i2->upper.CopyFrom( val2 );
+				i1->openLower = false;
+				i2->openLower = false;
+				i1->openUpper = false;
+				i2->openUpper = false;
+				if( vr->IsInitialized( ) ) {
+					vr->Intersect( i1, i2, false );
+				}
+				else {
+					vr->Init( i1, i2, false );
+				}
+			} else {
+				delete i1;
+				i1=0;
+			}
 		}
-		else {
-			vr->Init( i1, i2, false );
+		if(i1 && i2) {
+			delete i1;
+			delete i2;
 		}
-		delete i1;
-		delete i2;
 	}
 	else if( op == classad::Operation::NOT_EQUAL_OP || 
 			 op == classad::Operation::ISNT_OP ) {
