@@ -21,6 +21,9 @@
 #include "condor_common.h"
 #include "boolValue.h"
 #include "list.h"
+#include "extArray.h"
+
+using namespace std;
 
 // F && (T|F|U|E) = F
 // E && (T|F|U|E) = E 
@@ -590,12 +593,12 @@ GenerateABVList( List<AnnotatedBoolVector> &result )
 		return false;
 	}
 
-	AnnotatedBoolVector abv;
+	AnnotatedBoolVector *abv = new AnnotatedBoolVector();
 
 	int frequency = 0;
-	bool seen[numCols];
-	bool tempContexts[numCols];
-	bool hasCommonTrue;
+	ExtArray<bool> seen ( numCols );
+	ExtArray<bool> tempContexts ( numCols );
+    bool hasCommonTrue;
 
 	for( int i = 0; i < numCols; i++ ) {
 	    if(!seen[i]) {
@@ -611,17 +614,18 @@ GenerateABVList( List<AnnotatedBoolVector> &result )
 					}
 				}
 			}
-			abv.Init( numRows, numCols, frequency );
+			abv->Init( numRows, numCols, frequency );
 			for( int row = 0; row < numRows; row++ ) {
-				abv.SetValue( row, table[i][row] );
+				abv->SetValue( row, table[i][row] );
 			}
 			for( int col = 0; col < numCols; col++ ) {
-				abv.SetContext( col, tempContexts[col] );
+				abv->SetContext( col, tempContexts[col] );
 				tempContexts[col] = false;
 			}	
 			result.Append( abv );
 		}
 	}
+
 	return true;
 }
 
@@ -634,8 +638,8 @@ GenerateMaxTrueABVList( List<AnnotatedBoolVector> &result )
 
 	AnnotatedBoolVector *abv;
 	int frequency = 0;
-	bool seen[numCols];
-	bool tempContexts[numCols];
+	bool *seen = new bool[numCols];
+	bool *tempContexts = new bool[numCols];
 	for( int col = 0; col < numCols; col ++ ) {
 		seen[col] = false;
 		tempContexts[col] = false;
@@ -681,6 +685,8 @@ GenerateMaxTrueABVList( List<AnnotatedBoolVector> &result )
 			}
 		}
 	}
+    delete [] seen;
+    delete [] tempContexts;
 	return true;
 }
 
