@@ -21,13 +21,12 @@
 #include "condor_debug.h"
 #include "condor_config.h"
 #include "condor_string.h"
-#include "string_list.h"
-#include "MyString.h"
 #include "util_lib_proto.h"
 #include "stat_wrapper.h"
 #include "amazongahp_common.h"
 #include "amazonCommands.h"
 #include "thread_control.h"
+#include "condor_base64.h"
 
 // For gsoap
 #include <stdsoap2.h>
@@ -36,34 +35,6 @@
 #include <wsseapi.h>
 #include "AmazonEC2.nsmap"
 
-// For base64 encoding
-#include <openssl/sha.h>
-#include <openssl/hmac.h>
-#include <openssl/evp.h>
-#include <openssl/bio.h>
-#include <openssl/buffer.h>
-
-// Caller need to free the returned pointer
-static char* base64_encode(const unsigned char *input, int length)
-{
-	BIO *bmem, *b64;
-	BUF_MEM *bptr;
-
-	b64 = BIO_new(BIO_f_base64());
-	bmem = BIO_new(BIO_s_mem());
-	b64 = BIO_push(b64, bmem);
-	BIO_write(b64, input, length);
-	BIO_flush(b64);
-	BIO_get_mem_ptr(b64, &bptr);
-
-	char *buff = (char *)malloc(bptr->length);
-	ASSERT(buff);
-	memcpy(buff, bptr->data, bptr->length-1);
-	buff[bptr->length-1] = 0;
-	BIO_free_all(b64);
-
-	return buff;
-}
 
 void
 AmazonRequest::ParseSoapError(const char* callerstring) 
@@ -81,7 +52,7 @@ AmazonRequest::ParseSoapError(const char* callerstring)
 
 	if( *code ) {
 		// For real error code, refer to 
-		// http://docs.amazonwebservices.com/AWSEC2/2007-08-29/DeveloperGuide/api-error-codes.html
+		// http://docs.amazonwebservices.com/AWSEC2/2008-12-01/DeveloperGuide/index.html?api-error-codes.html
 		//
 		// Client error codes suggest that the error was caused by 
 		// something the client did, such as an authentication failure or 
@@ -134,11 +105,11 @@ AmazonRequest::ParseSoapError(const char* callerstring)
 bool 
 AmazonRequest::SetupSoap(void)
 {
-	if( secretkeyfile.IsEmpty() ) {
+	if( secretkeyfile.empty() ) {
 		dprintf(D_ALWAYS, "There is no privatekey\n");
 		return false;
 	}
-	if( accesskeyfile.IsEmpty() ) {
+	if( accesskeyfile.empty() ) {
 		dprintf(D_ALWAYS, "There is no accesskeyfile\n");
 		return false;
 	}
@@ -150,16 +121,21 @@ AmazonRequest::SetupSoap(void)
 	// Must use canoicalization
 	if( !(m_soap = soap_new1(SOAP_XML_CANONICAL))) {
 		m_error_msg = "Failed to create SOAP context";
-		dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+		dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 		return false;
 	}
 
 	const char* proxy_host_name;
 	int proxy_port = 0;
-	if( get_amazon_proxy_server(proxy_host_name, proxy_port) ) {
+	const char* proxy_user_name;
+	const char* proxy_passwd;
+	if( get_amazon_proxy_server(proxy_host_name, proxy_port, 
+				    proxy_user_name, proxy_passwd) ) {
 		m_soap->proxy_host = proxy_host_name;
 		m_soap->proxy_port = proxy_port; 
-	}
+		m_soap->proxy_userid = proxy_user_name;
+		m_soap->proxy_passwd = proxy_passwd;
+ 	}
 
 	if (soap_register_plugin(m_soap, soap_wsse)) {
 		ParseSoapError("setup WS-Security plugin");
@@ -167,66 +143,83 @@ AmazonRequest::SetupSoap(void)
 	}
 
 	FILE *file = NULL;
-	if ((file = safe_fopen_wrapper(secretkeyfile.Value(), "r"))) {
+	if ((file = safe_fopen_wrapper(secretkeyfile.c_str(), "r"))) {
 		m_rsa_privk = PEM_read_PrivateKey(file, NULL, NULL, NULL);
 		fclose(file);
 
 		if( !m_rsa_privk ) {
-			m_error_msg.sprintf("Could not read private RSA key from: %s",
-					secretkeyfile.Value());
-			dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+			sprintf(m_error_msg,"Could not read private RSA key from: %s",
+					secretkeyfile.c_str());
+			dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 			return false;
 		}
 	} else {
-		m_error_msg.sprintf("Could not read private key file: %s",
-				secretkeyfile.Value());
-		dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+		sprintf(m_error_msg,"Could not read private key file: %s",
+				secretkeyfile.c_str());
+		dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 		return false;
 	}
 
-	if ((file = safe_fopen_wrapper(accesskeyfile.Value(), "r"))) {
+	if ((file = safe_fopen_wrapper(accesskeyfile.c_str(), "r"))) {
 		m_cert = PEM_read_X509(file, NULL, NULL, NULL);
 		fclose(file);
 
 		if (!m_cert) {
-			m_error_msg.sprintf("Could not read accesskeyfile from: %s",
-					accesskeyfile.Value());
-			dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+			sprintf(m_error_msg,"Could not read accesskeyfile from: %s",
+					accesskeyfile.c_str());
+			dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 			return false;
 		}
 	} else {
-		m_error_msg.sprintf("Could not read accesskeyfile file: %s",
-				accesskeyfile.Value());
-		dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+		sprintf(m_error_msg,"Could not read accesskeyfile file: %s",
+				accesskeyfile.c_str());
+		dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 		return false;
 	}
 
+	unsigned short flags = SOAP_SSL_DEFAULT;
+	const char *host_check = getenv("SOAP_SSL_SKIP_HOST_CHECK");
+	if ( host_check && ( host_check[0] == 'T' || host_check[0] == 't' ) ) {
+		flags |= SOAP_SSL_SKIP_HOST_CHECK;
+	}
+	const char *ca_file = getenv("SOAP_SSL_CA_FILE");
+	const char *ca_dir = getenv("SOAP_SSL_CA_DIR");;
+	if (soap_ssl_client_context(m_soap, flags,
+				    NULL,
+				    NULL,
+				    ca_file,
+				    ca_dir,
+				    NULL))
+	{
+	    soap_print_fault(m_soap, stderr);
+	}
+
 	// Timestamp must be signed, the "Timestamp" value just needs
 	// to be non-NULL
 	if( soap_wsse_add_Timestamp(m_soap, "Timestamp", 137)) { 
 		m_error_msg = "Failed to sign timestamp";
-		dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+		dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 		return false;
 	}
 
-	if( soap_wsse_add_BinarySecurityTokenX509(m_soap, "BinarySecurityToken", m_cert)) {
-		m_error_msg.sprintf("Could not set BinarySecurityToken from: %s", 
-				accesskeyfile.Value());
-		dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+	if( soap_wsse_add_BinarySecurityTokenX509(m_soap, "X509Token", m_cert)) {
+		sprintf(m_error_msg,"Could not set BinarySecurityToken from: %s", 
+				accesskeyfile.c_str());
+		dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 		return false;
 	}
 
 	// May be optional
 	if( soap_wsse_add_KeyInfo_SecurityTokenReferenceX509(m_soap, "#X509Token") ) {
 		m_error_msg = "Failed to setup SecurityTokenReference";
-		dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+		dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 		return false;
 	}
 
 	// Body must be signed
 	if( soap_wsse_sign_body(m_soap, SOAP_SMD_SIGN_RSA_SHA1, m_rsa_privk, 0)) {
 		m_error_msg = "Failed to setup signing of SOAP body";
-		dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+		dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 		return false;
 	}
 
@@ -265,9 +258,9 @@ AmazonVMKeypairNames::gsoapRequest(void)
 		return false;
 	}
 
-	if( !check_access_and_secret_key_file(accesskeyfile.GetCStr(),
-				secretkeyfile.GetCStr(), m_error_msg) ) {
-		dprintf(D_ALWAYS, "AmazonVMKeypairNames Error: %s\n", m_error_msg.Value());
+	if( !check_access_and_secret_key_file(accesskeyfile.c_str(),
+				secretkeyfile.c_str(), m_error_msg) ) {
+		dprintf(D_ALWAYS, "AmazonVMKeypairNames Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
@@ -281,7 +274,7 @@ AmazonVMKeypairNames::gsoapRequest(void)
 	int code = -1;		
 	amazon_gahp_release_big_mutex();
 	code = soap_call_ec2__DescribeKeyPairs(m_soap,
-					get_ec2_url(),
+					m_service_url.c_str(),
 					NULL,
 					&request,
 					&response);
@@ -311,27 +304,27 @@ AmazonVMCreateKeypair::gsoapRequest(void)
 		return false;
 	}
 
-	if( !check_access_and_secret_key_file(accesskeyfile.GetCStr(), 
-				secretkeyfile.GetCStr(), m_error_msg) ) {
-		dprintf(D_ALWAYS, "AmazonVMCreateKeyPair Error: %s\n", m_error_msg.Value());
+	if( !check_access_and_secret_key_file(accesskeyfile.c_str(), 
+				secretkeyfile.c_str(), m_error_msg) ) {
+		dprintf(D_ALWAYS, "AmazonVMCreateKeyPair Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
-	if( keyname.IsEmpty() ) {
+	if( keyname.empty() ) {
 		m_error_msg = "Empty_Keyname";
-		dprintf(D_ALWAYS, "AmazonVMCreateKeyPair Error: %s\n", m_error_msg.Value());
+		dprintf(D_ALWAYS, "AmazonVMCreateKeyPair Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
-	if( strcmp(outputfile.Value(), NULL_FILE) ) { 
+	if( strcmp(outputfile.c_str(), NULL_FILE) ) { 
 		has_outputfile = true;
 	}
 
 	// check if output file could be created
 	if( has_outputfile ) { 
-		if( check_create_file(outputfile.GetCStr()) == false ) {
+		if( check_create_file(outputfile.c_str(), 0600) == false ) {
 			m_error_msg = "No_permission_for_keypair_outputfile";
-			dprintf(D_ALWAYS, "AmazonVMCreateKeypair Error: %s\n", m_error_msg.Value());
+			dprintf(D_ALWAYS, "AmazonVMCreateKeypair Error: %s\n", m_error_msg.c_str());
 			return false;
 		}
 	}
@@ -339,12 +332,12 @@ AmazonVMCreateKeypair::gsoapRequest(void)
 	ec2__CreateKeyPairType request;
 	ec2__CreateKeyPairResponseType response;
 
-	request.keyName = keyname.GetCStr();
+	request.keyName = keyname.c_str();
 
 	int code = -1;
 	amazon_gahp_release_big_mutex();
 	code = soap_call_ec2__CreateKeyPair(m_soap,
-					get_ec2_url(),
+					m_service_url.c_str(),
 					NULL,
 					&request,
 					&response);
@@ -353,12 +346,12 @@ AmazonVMCreateKeypair::gsoapRequest(void)
 		if( has_outputfile ) {
 
 			FILE *fp = NULL;
-			fp = safe_fopen_wrapper(outputfile.Value(), "w");
+			fp = safe_fopen_wrapper(outputfile.c_str(), "w", 600);
 			if( !fp ) {
-				m_error_msg.sprintf("failed to safe_fopen_wrapper %s in write mode: "
+				sprintf(m_error_msg,"failed to safe_fopen_wrapper %s in write mode: "
 						"safe_fopen_wrapper returns %s", 
-						outputfile.Value(), strerror(errno));
-				dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+						outputfile.c_str(), strerror(errno));
+				dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 				return false;
 			}
 
@@ -381,29 +374,29 @@ AmazonVMDestroyKeypair::gsoapRequest(void)
 		return false;
 	}
 
-	if( !check_access_and_secret_key_file(accesskeyfile.GetCStr(),
-				secretkeyfile.GetCStr(), m_error_msg) ) {
+	if( !check_access_and_secret_key_file(accesskeyfile.c_str(),
+				secretkeyfile.c_str(), m_error_msg) ) {
 		dprintf(D_ALWAYS, "AmazonVMDestroyKeypair Error: %s\n", 
-				m_error_msg.Value());
+				m_error_msg.c_str());
 		return false;
 	}
 
-	if( keyname.IsEmpty() ) {
+	if( keyname.empty() ) {
 		m_error_msg = "Empty_Keyname";
 		dprintf(D_ALWAYS, "AmazonVMDestroyKeypair Error: %s\n", 
-				m_error_msg.Value());
+				m_error_msg.c_str());
 		return false;
 	}
 
 	ec2__DeleteKeyPairType request;
 	ec2__DeleteKeyPairResponseType response;
 
-	request.keyName = keyname.GetCStr();
+	request.keyName = keyname.c_str();
 
 	int code = -1;
 	amazon_gahp_release_big_mutex();
 	code = soap_call_ec2__DeleteKeyPair(m_soap,
-					get_ec2_url(),
+					m_service_url.c_str(),
 					NULL,
 					&request,
 					&response);
@@ -431,73 +424,79 @@ AmazonVMStart::gsoapRequest(void)
 		return false;
 	}
 
-	if( !check_access_and_secret_key_file(accesskeyfile.GetCStr(),
-				secretkeyfile.GetCStr(), m_error_msg) ) {
-		dprintf(D_ALWAYS, "AmazonVMStart Error: %s\n", m_error_msg.Value());
+	if( !check_access_and_secret_key_file(accesskeyfile.c_str(),
+				secretkeyfile.c_str(), m_error_msg) ) {
+		dprintf(D_ALWAYS, "AmazonVMStart Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
-	if( user_data_file.IsEmpty() == false ) {
-		if( !check_read_access_file( user_data_file.GetCStr()) ) {
-			m_error_msg.sprintf("Cannot read the file for user data(%s)",
-					user_data_file.Value());
+	if( user_data_file.empty() == false ) {
+		if( !check_read_access_file( user_data_file.c_str()) ) {
+			sprintf(m_error_msg,"Cannot read the file for user data(%s)",
+					user_data_file.c_str());
 			return false;
 		}
 	}
 
-	if( ami_id.IsEmpty() ) {
+	if( ami_id.empty() ) {
 		m_error_msg = "Empty_AMI_ID";
-		dprintf(D_ALWAYS, "AmazonVMStart Error: %s\n", m_error_msg.Value());
+		dprintf(D_ALWAYS, "AmazonVMStart Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
 	// userData
-	if( user_data_file.IsEmpty() == false ) {
-		// Need to read file
+	if( user_data_file.empty() == false || user_data.empty() == false ) {
+		char *buffer;
 		int fd = -1;
-		fd = safe_open_wrapper(user_data_file.Value(), O_RDONLY);
-		if( fd < 0 ) {
-			m_error_msg.sprintf("failed to safe_open_wrapper file(%s) : "
-					"safe_open_wrapper returns %s", user_data_file.Value(), 
-					strerror(errno));
-			dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
-			return false;
+		int file_size = 0;
+
+		if ( user_data_file.empty() == false ) {
+			// Need to read file
+			fd = safe_open_wrapper(user_data_file.c_str(), O_RDONLY);
+			if( fd < 0 ) {
+				sprintf(m_error_msg,"failed to safe_open_wrapper file(%s) : "
+						"safe_open_wrapper returns %s", user_data_file.c_str(), 
+						strerror(errno));
+				dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
+				return false;
+			}
+
+			StatWrapper swrap(fd);
+			file_size = swrap.GetBuf()->st_size;
+
+			if (file_size <= 0) {
+				dprintf(D_ALWAYS,
+						"WARNING: UserData file, %s, specified and empty\n",
+						user_data_file.c_str());
+				close( fd );
+				fd = -1;
+				file_size = 0;
+			}
 		}
 
-		int file_size = 0;
-		StatWrapper swrap(fd);
-		file_size = swrap.GetBuf()->st_size;
+		buffer = (char *)malloc( file_size + user_data.size() + 1 );
+		ASSERT( buffer );
 
-		if (file_size <= 0) {
-			dprintf(D_ALWAYS,
-					"WARNING: UserData file, %s, specified and empty\n",
-					user_data_file.Value());
-			base64_userdata = NULL;
-		} else {
-			char *readbuffer = (char*)malloc(file_size);
-			ASSERT(readbuffer);
-
-			int ret = full_read(fd, readbuffer, file_size);
+		if ( user_data.empty() == false ) {
+			strcpy( buffer, user_data.c_str() );
+		}
+		if ( fd != -1 ) {
+			int ret = full_read( fd, buffer + user_data.size(), file_size );
 			close(fd);
 
 			if( ret != file_size ) {
-				m_error_msg.sprintf("failed to read(need %d but real read %d) "
+				sprintf(m_error_msg,"failed to read(need %d but real read %d) "
 									"in file(%s)",
-									file_size, ret, user_data_file.Value());
-				dprintf(D_ALWAYS, "%s\n", m_error_msg.Value());
+									file_size, ret, user_data_file.c_str());
+				dprintf(D_ALWAYS, "%s\n", m_error_msg.c_str());
 
-				free(readbuffer);
-				readbuffer = NULL;
+				free( buffer );
 				return false;
 			}
-	
-			base64_userdata = base64_encode((unsigned char*)readbuffer, file_size);
-			free(readbuffer); readbuffer = NULL;
-		}
-	}else {
-		if( user_data.IsEmpty() == false ) { 
-			base64_userdata = base64_encode((unsigned char*)user_data.GetCStr(), user_data.Length());
 		}
+
+		base64_userdata = condor_base64_encode((unsigned char*)buffer, file_size + user_data.size());
+		free( buffer );
 	}
 
 	ec2__RunInstancesType request;
@@ -506,15 +505,15 @@ AmazonVMStart::gsoapRequest(void)
 	std::string request_keyname;
 
 	// image id
-	request.imageId = ami_id.GetCStr();
+	request.imageId = ami_id.c_str();
 	// min Count
 	request.minCount = 1;
 	// max Count
 	request.maxCount = 1;
 
 	// Keypair
-	if( keypair.IsEmpty() == false ) {
-		request_keyname = keypair.GetCStr();
+	if( keypair.empty() == false ) {
+		request_keyname = keypair.c_str();
 		request.keyName = &request_keyname;
 	}else {
 		request.keyName = NULL;
@@ -562,8 +561,8 @@ AmazonVMStart::gsoapRequest(void)
 	request.addressingType = NULL;
 
 	// instanceType
-	if( instance_type.IsEmpty() == false ) {
-		request.instanceType = instance_type.GetCStr();
+	if( instance_type.empty() == false ) {
+		request.instanceType = instance_type.c_str();
 	}else {
 		request.instanceType = "m1.small";
 	}
@@ -571,7 +570,7 @@ AmazonVMStart::gsoapRequest(void)
 	int code = -1;
 	amazon_gahp_release_big_mutex();
 	code = soap_call_ec2__RunInstances(m_soap,
-					get_ec2_url(),
+					m_service_url.c_str(),
 					NULL,
 					&request,
 					&response);
@@ -596,15 +595,15 @@ AmazonVMStop::gsoapRequest(void)
 		return false;
 	}
 
-	if( !check_access_and_secret_key_file(accesskeyfile.GetCStr(),
-				secretkeyfile.GetCStr(), m_error_msg) ) {
-		dprintf(D_ALWAYS, "AmazonVMStop Error: %s\n", m_error_msg.Value());
+	if( !check_access_and_secret_key_file(accesskeyfile.c_str(),
+				secretkeyfile.c_str(), m_error_msg) ) {
+		dprintf(D_ALWAYS, "AmazonVMStop Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
-	if( instance_id.IsEmpty() ) {
+	if( instance_id.empty() ) {
 		m_error_msg = "Empty_Instance_ID";
-		dprintf(D_ALWAYS, "AmazonVMStop Error: %s\n", m_error_msg.Value());
+		dprintf(D_ALWAYS, "AmazonVMStop Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
@@ -614,7 +613,7 @@ AmazonVMStop::gsoapRequest(void)
 	ec2__TerminateInstancesInfoType instanceSet;
 
 	ec2__TerminateInstancesItemType item;
-	item.instanceId = instance_id.GetCStr();
+	item.instanceId = instance_id.c_str();
 
 	instanceSet.item.push_back( &item );
 
@@ -623,7 +622,7 @@ AmazonVMStop::gsoapRequest(void)
 	int code = -1;
 	amazon_gahp_release_big_mutex();
 	code = soap_call_ec2__TerminateInstances(m_soap,
-					get_ec2_url(),
+					m_service_url.c_str(),
 					NULL,
 					&request,
 					&response);
@@ -645,15 +644,15 @@ AmazonVMStatus::gsoapRequest(void)
 		return false;
 	}
 
-	if( !check_access_and_secret_key_file(accesskeyfile.GetCStr(),
-				secretkeyfile.GetCStr(), m_error_msg) ) {
-		dprintf(D_ALWAYS, "AmazonVMStatus Error: %s\n", m_error_msg.Value());
+	if( !check_access_and_secret_key_file(accesskeyfile.c_str(),
+				secretkeyfile.c_str(), m_error_msg) ) {
+		dprintf(D_ALWAYS, "AmazonVMStatus Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
-	if( instance_id.IsEmpty() ) {
+	if( instance_id.empty() ) {
 		m_error_msg = "Empty_Instance_ID";
-		dprintf(D_ALWAYS, "AmazonVMStatus Error: %s\n", m_error_msg.Value());
+		dprintf(D_ALWAYS, "AmazonVMStatus Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
@@ -663,7 +662,7 @@ AmazonVMStatus::gsoapRequest(void)
 	ec2__DescribeInstancesInfoType instancesSet;
 	ec2__DescribeInstancesItemType item;
 
-	item.instanceId = instance_id.GetCStr();
+	item.instanceId = instance_id.c_str();
 
 	instancesSet.item.push_back( &item );
 
@@ -673,7 +672,7 @@ AmazonVMStatus::gsoapRequest(void)
 	int code = -1;
 	amazon_gahp_release_big_mutex();
 	code = soap_call_ec2__DescribeInstances(m_soap,
-					get_ec2_url(),
+					m_service_url.c_str(),
 					NULL,
 					&request,
 					&response);
@@ -703,7 +702,7 @@ AmazonVMStatus::gsoapRequest(void)
 					continue;
 				}
 
-				if( !strcmp(instance->instanceId.c_str(), instance_id.Value()) ) {
+				if( !strcmp(instance->instanceId.c_str(), instance_id.c_str()) ) {
 
 					status_result.status = instance->instanceState->name.c_str();
 					status_result.instance_id = instance->instanceId.c_str();
@@ -746,9 +745,9 @@ AmazonVMStatusAll::gsoapRequest(void)
 		return false;
 	}
 
-	if( !check_access_and_secret_key_file(accesskeyfile.GetCStr(),
-				secretkeyfile.GetCStr(), m_error_msg) ) {
-		dprintf(D_ALWAYS, "AmazonVMStatusAll Error: %s\n", m_error_msg.Value());
+	if( !check_access_and_secret_key_file(accesskeyfile.c_str(),
+				secretkeyfile.c_str(), m_error_msg) ) {
+		dprintf(D_ALWAYS, "AmazonVMStatusAll Error: %s\n", m_error_msg.c_str());
 		return false;
 	}
 
@@ -763,7 +762,7 @@ AmazonVMStatusAll::gsoapRequest(void)
 	int code = -1;
 	amazon_gahp_release_big_mutex();
 	code = soap_call_ec2__DescribeInstances(m_soap,
-					get_ec2_url(),
+					m_service_url.c_str(),
 					NULL,
 					&request,
 					&response);
