@@ -60,7 +60,7 @@ PipeBuffer stdin_buffer;
 Worker workers[NUMBER_WORKERS];
 
 // this appears at the bottom of this file
-extern "C" int display_dprintf_header(FILE *fp);
+extern "C" int display_dprintf_header(char **buf,int *bufpos,int *buflen);
 
 #ifdef WIN32
 int STDIN_FILENO = fileno(stdin);
@@ -116,15 +116,15 @@ usage()
 	DC_Exit( 1 );
 }
 
-int
+void
 main_init( int argc, char ** const argv )
 {
 
 
 	dprintf(D_FULLDEBUG, "C-GAHP IO thread\n");
 
-	MyString schedd_addr;
-	MyString schedd_pool;
+	std::string schedd_addr;
+	std::string schedd_pool;
 
 	// handle specific command line args
 	int i = 1;
@@ -165,8 +165,7 @@ main_init( int argc, char ** const argv )
 	// the pipe we were passed in via stdin is overlapped-mode
 	// is probably wrong. we therefore create a new pipe with the
 	// read end overlapped and start a "forwarding thread"
-	char* tmp;
-	if ((tmp = daemonCore->InfoCommandSinfulString(daemonCore->getppid())) == NULL) {
+	if (daemonCore->InfoCommandSinfulString(daemonCore->getppid()) == NULL) {
 
 		dprintf(D_FULLDEBUG, "parent is not DaemonCore; creating a forwarding thread\n");
 
@@ -220,7 +219,7 @@ main_init( int argc, char ** const argv )
 					      true	// read end registerable
 					     ) )
 		{
-			return -1;
+			return;
 		}
 
 		workers[i].request_buffer.setPipeEnd(workers[i].request_pipe[1]);
@@ -228,7 +227,7 @@ main_init( int argc, char ** const argv )
 
 		(void)daemonCore->Register_Pipe (workers[i].result_buffer.getPipeEnd(),
 										 "result pipe",
-										 (PipeHandlercpp)&Worker::result_handler,
+										 static_cast<PipeHandlercpp>(&Worker::result_handler),
 										 "Worker::result_handler",
 										 (Service*)&workers[i]);
 	}
@@ -247,14 +246,13 @@ main_init( int argc, char ** const argv )
 	flush_request_tid = 
 		daemonCore->Register_Timer (1,
 									1,
-									(TimerHandler)&flush_pending_requests,
-									"flush_pending_requests",
-									NULL);
+									flush_pending_requests,
+									"flush_pending_requests");
 									
 									  
 
 
-	MyString exec_name;
+	std::string exec_name;
 	char * c_gahp_worker_thread = param("CONDOR_GAHP_WORKER");
 	if (c_gahp_worker_thread) {
 		exec_name = c_gahp_worker_thread;
@@ -263,24 +261,24 @@ main_init( int argc, char ** const argv )
 	else {
 		char * c_gahp_name = param ("CONDOR_GAHP");
 		ASSERT (c_gahp_name);
-		exec_name.sprintf ("%s_worker_thread", c_gahp_name);
+		sprintf(exec_name, "%s_worker_thread", c_gahp_name);
 		free (c_gahp_name);
 	}
 
 	for (i=0; i<NUMBER_WORKERS; i++) {
 		ArgList args;
 
-		args.AppendArg(exec_name.Value());
+		args.AppendArg(exec_name.c_str());
 		args.AppendArg("-f");
 
-		if (schedd_addr.Length()) {
+		if (schedd_addr.length()) {
 			args.AppendArg("-s");
-			args.AppendArg(schedd_addr.Value());
+			args.AppendArg(schedd_addr.c_str());
 		}
 
-		if (schedd_pool.Length()) {
+		if (schedd_pool.length()) {
 			args.AppendArg("-P");
-			args.AppendArg(schedd_pool.Value());
+			args.AppendArg(schedd_pool.c_str());
 		}
 
 		MyString args_string;
@@ -295,7 +293,7 @@ main_init( int argc, char ** const argv )
 
 		workers[i].pid = 
 			daemonCore->Create_Process (
-										exec_name.Value(),
+										exec_name.c_str(),
 										args,
 										PRIV_UNKNOWN,
 										reaper_id,
@@ -322,18 +320,16 @@ main_init( int argc, char ** const argv )
 	fflush(stdout);
 
 	dprintf (D_FULLDEBUG, "C-GAHP IO initialized\n");
-
-	return TRUE;
 }
 
 
 int
 stdin_pipe_handler(Service*, int) {
 
-	MyString* line;
+	std::string* line;
 	while ((line = stdin_buffer.GetNextLine()) != NULL) {
 
-		const char * command = line->Value();
+		const char * command = line->c_str();
 
 		dprintf (D_ALWAYS, "got stdin: %s\n", command);
 
@@ -345,11 +341,11 @@ stdin_pipe_handler(Service*, int) {
 				// Catch "special commands first
 			if (strcasecmp (args.argv[0], GAHP_COMMAND_RESULTS) == 0) {
 					// Print number of results
-				MyString rn_buff;
-				rn_buff+=result_list.number();
+				std::string rn_buff;
+				sprintf( rn_buff, "%d", result_list.number() );
 				const char * commands [] = {
 					GAHP_RESULT_SUCCESS,
-					rn_buff.Value() };
+					rn_buff.c_str() };
 				gahp_output_return (commands, 2);
 
 					// Print each result line
@@ -444,13 +440,13 @@ stdin_pipe_handler(Service*, int) {
 int 
 result_pipe_handler(int id) {
 
-	MyString* line;
+	std::string* line;
 	while ((line = workers[id].result_buffer.GetNextLine()) != NULL) {
 
-		dprintf (D_FULLDEBUG, "Received from worker:\"%s\"\n", line->Value());
+		dprintf (D_FULLDEBUG, "Received from worker:\"%s\"\n", line->c_str());
 
 			// Add this to the list
-		result_list.append (line->Value());
+		result_list.append (line->c_str());
 
 		if (async_mode) {
 			if (!new_results_signaled) {
@@ -680,16 +676,16 @@ flush_request (int worker_id, const char * request) {
 			 request,
 			 worker_id);
 
-	MyString strRequest = request;
+	std::string strRequest = request;
 	strRequest += "\n";
 
-	workers[worker_id].request_buffer.Write(strRequest.Value());
+	workers[worker_id].request_buffer.Write(strRequest.c_str());
 
 	daemonCore->Reset_Timer (flush_request_tid, 0, 1);
 }
 
 
-int flush_pending_requests() {
+void flush_pending_requests() {
 	for (int i=0; i<NUMBER_WORKERS; i++) {
 		workers[i].request_buffer.Write();
 
@@ -698,8 +694,6 @@ int flush_pending_requests() {
 			DC_Exit (1);
 		}
 	}
-
-	return TRUE;
 }
 void
 gahp_output_return (const char ** results, const int count) {
@@ -736,21 +730,18 @@ void Register() {}
 void Reconfig() {}
 
 
-int
-main_config( bool )
+void
+main_config()
 {
 	Reconfig();
-	return TRUE;
 }
 
-int
+void
 main_shutdown_fast()
 {
-
-	return TRUE;	// to satisfy c++
 }
 
-int
+void
 main_shutdown_graceful()
 {
 	daemonCore->Cancel_And_Close_All_Pipes();
@@ -758,8 +749,6 @@ main_shutdown_graceful()
 	for (int i=0; i<NUMBER_WORKERS; i++) {
 		daemonCore->Send_Signal (workers[i].pid, SIGKILL);
 	}
-
-	return TRUE;	// to satify c++
 }
 
 void
@@ -777,7 +766,7 @@ main_pre_command_sock_init( )
 // log entries.
 extern "C"
 int
-display_dprintf_header(FILE *fp)
+display_dprintf_header(char **buf,int *bufpos,int *buflen)
 {
 	static pid_t mypid = 0;
 
@@ -785,7 +774,5 @@ display_dprintf_header(FILE *fp)
 		mypid = daemonCore->getpid();
 	}
 
-	fprintf( fp, "[%ld] ", (long)mypid );
-
-	return TRUE;
+	return sprintf_realloc( buf, bufpos, buflen, "[%ld] ", (long)mypid );
 }
