@@ -22,10 +22,6 @@
 #include "condor_version.h"
 #include "condor_mmap.h"
 
-#if defined(Solaris)
-#include <netconfig.h>		// for setnetconfig()
-#endif
-
 #include "condor_syscalls.h"
 #include "condor_sys.h"
 #include "image.h"
@@ -45,7 +41,7 @@ extern "C" void mydprintf(int foo, const char *fmt, ...);
 extern "C" void report_image_size( int );
 extern "C"	int	SYSCALL(int ...);
 
-#ifdef SAVE_SIGSTATE
+#ifdef DOES_SAVE_SIGSTATE
 extern "C" void _condor_save_sigstates();
 extern "C" void _condor_restore_sigstates();
 #endif
@@ -64,10 +60,7 @@ extern "C" {
 }
 #endif
 
-#if defined(OSF1)
-	extern "C" unsigned int htonl( unsigned int );
-	extern "C" unsigned int ntohl( unsigned int );
-#elif defined(HPUX)
+#if defined(HPUX)
 #	include <netinet/in.h>
 #elif defined(Solaris)
 	#define htonl(x)		(x)
@@ -682,13 +675,6 @@ SegMap::Contains( void *addr )
 }
 
 
-#define USER_DATA_SIZE 256
-#if defined(PVM_CHECKPOINTING)
-extern "C" int user_restore_pre(char *, int);
-extern "C" int user_restore_post(char *, int);
-char	global_user_data[USER_DATA_SIZE];
-#endif
-
 /*
   Given an "image" object containing checkpoint information which we have
   just read in, this method actually effects the restart.
@@ -697,13 +683,8 @@ void
 Image::Restore()
 {
 	int		save_fd = fd;
-	char	user_data[USER_DATA_SIZE];
 
 
-#if defined(PVM_CHECKPOINTING)
-	user_restore_pre(user_data, sizeof(user_data));
-#endif
-
 #if defined(COMPRESS_CKPT)
 	// If the checkpoint header contains an alt heap pointer, then we must
 	// create an alt heap, because the alt heap pointer is going to be
@@ -759,10 +740,6 @@ Image::Restore()
 		// the next gettimeofday() call.
 	_condor_gtodc_init(_condor_global_gtodc);
 
-#if defined(PVM_CHECKPOINTING)
-	memcpy(global_user_data, user_data, sizeof(user_data));
-#endif
-
 		// Now we're going to restore the stack, so we move our execution
 		// stack to a temporary area (in the data segment), then call
 		// the RestoreStack() routine.
@@ -901,10 +878,6 @@ RestoreStack()
 		SetSyscalls( SYS_LOCAL | SYS_MAPPED );
 	}
 
-#if defined(PVM_CHECKPOINTING)
-	user_restore_post(global_user_data, sizeof(global_user_data));
-#endif
-
 #if defined(COMPRESS_CKPT)
 	if (zstr) {
 		int rval = inflateEnd(zstr);
@@ -1565,20 +1538,6 @@ Checkpoint( int sig, int code, void *scp )
 
 	if( MyImage.GetMode() == REMOTE ) {
 		scm = SetSyscalls( SYS_REMOTE | SYS_UNMAPPED );
-#if !defined(PVM_CHECKPOINTING)
-		if ( MyImage.GetFd() != -1 ) {
-			// Here we make _certain_ that fd is -1.  on remote checkpoints,
-			// the fd is always new since we open a fresh TCP socket to the
-			// shadow.  I have detected some buggy behavior where the remote
-			// job prematurely exits with a status 4, and think that it is
-			// related to the fact that fd is _not_ -1 here, so we make
-			// certain.  Hopefully the real bug will be found someday and
-			// this "patch" can go away...  -Todd 11/95
-			
-		dprintf(D_ALWAYS,"WARNING: fd is %d for remote checkpoint, should be -1\n",MyImage.GetFd());
-		MyImage.SetFd( -1 );
-		}
-#endif
 
 		/* now update shadow with CPU time info.  unfortunately, we need
 		 * to convert to struct rusage here in the user code, because
@@ -1594,11 +1553,7 @@ Checkpoint( int sig, int code, void *scp )
 		p_scm = SetSyscalls( SYS_LOCAL | SYS_UNMAPPED );
 		memset(&bsd_usage,0,sizeof(struct rusage));
 		times( &posix_usage );
-#if defined(OSF1)
-		clock_tick = CLK_TCK;
-#else
 		clock_tick = sysconf( _SC_CLK_TCK );
-#endif
 		bsd_usage.ru_utime.tv_sec = posix_usage.tms_utime / clock_tick;
 		bsd_usage.ru_utime.tv_usec = posix_usage.tms_utime % clock_tick;
 		(bsd_usage.ru_utime.tv_usec) *= 1000000 / clock_tick;
