@@ -30,8 +30,12 @@
 #include "condor_socket_types.h"
 #include "subsystem_info.h"
 #include "condor_netdb.h"
-#include "condor_fix_iostream.h"
-#include "condor_fix_fstream.h"
+#include <iostream>
+#include <fstream>
+#include "condor_open.h"
+#include "directory.h"
+#include "basename.h"
+#include "stat_info.h"
 
 #include "server2.h"
 #include "gen_lib.h"
@@ -42,15 +46,12 @@
 #include "classad_collection.h"
 #include "daemon.h"
 #include "protocol.h"
+using namespace std;
 
 XferSummary	xfer_summary;
 
 Server server;
 Alarm  rt_alarm;
-#ifdef WANT_NETMAN
-static bool ManageBandwidth = false;
-static int NetworkHorizon = 300;
-#endif
 
 /* For daemonCore, etc. */
 DECL_SUBSYSTEM( "CKPT_SERVER", SUBSYSTEM_TYPE_DAEMON );
@@ -69,6 +70,7 @@ int ValidateNoPathComponents(char *path);
 
 Server::Server()
 {
+	ckpt_server_dir = NULL;
 	more = 1;
 	req_ID = 0;
 		// We can't initialize this until Server::Init(), after
@@ -94,6 +96,10 @@ Server::Server()
 	reclaim_interval = 0;
 	replication_level = 0;
 	check_parent_interval = 0;
+
+	next_time_to_remove_stale_ckpt_files = 0;
+	remove_stale_ckptfile_interval = 0;
+	stale_ckptfile_age_cutoff = 0;
 }
 
 
@@ -107,13 +113,17 @@ Server::~Server()
 		close(service_req_sd);
 	if (replicate_req_sd >=0)
 		close(replicate_req_sd);
+	if (ckpt_server_dir != NULL) {
+		free(ckpt_server_dir);
+		ckpt_server_dir = NULL;
+	}
 	delete CkptClassAds;
 }
 
 
 void Server::Init()
 {
-	char		*ckpt_server_dir, *collection_log;
+	char		*collection_log;
 	char        log_msg[256];
 	char        hostname[100];
 	int         buf_size;
@@ -154,7 +164,6 @@ void Server::Init()
 		}
 		dprintf(D_ALWAYS, "CKPT_SERVER running in directory %s\n", 
 				ckpt_server_dir);
-		free( ckpt_server_dir );
 	}
 
 	replication_level = param_integer( "CKPT_SERVER_REPLICATION_LEVEL",0 );
@@ -170,6 +179,28 @@ void Server::Init()
 		CkptClassAds = NULL;
 	}
 
+	// How long between periods of checking ckpt files for staleness and
+	// removing them.
+	// Defaults to one day (86400) seconds.
+	// This is used because A) in general, the ckpt server sucks,
+	// and B) the new timeout code in the server interface. If suppose
+	// the checkpoint server goes down and the schedd can't remove the ckpt
+	// files, they are simply leaked into the checkpoint server, forever to
+	// consume space. This will prevent unbounded leakage.
+	// It was faster to implement this on a dying feature of Condor than
+	// to implement the checkpoint server calling back schedds to see if the
+	// job is still around.
+	remove_stale_ckptfile_interval =
+		param_integer("CKPT_SERVER_REMOVE_STALE_CKPT_INTERVAL", 86400, 0, INT_MAX);
+
+	// How long a ckptfile's a_time hasn't been updated to be considered stale
+	// Defaults to 60 days.
+	stale_ckptfile_age_cutoff =
+		param_integer("CKPT_SERVER_STALE_CKPT_AGE_CUTOFF", 5184000, 1, INT_MAX);
+
+	// This has the effect of checking for stale stuff right away on start up 
+	next_time_to_remove_stale_ckpt_files = 0;
+
 	clean_interval = param_integer( "CKPT_SERVER_CLEAN_INTERVAL",CLEAN_INTERVAL );
 	check_parent_interval = param_integer( "CKPT_SERVER_CHECK_PARENT_INTERVAL",120);
 	socket_bufsize = param_integer( "CKPT_SERVER_SOCKET_BUFSIZE",0 );
@@ -179,23 +210,6 @@ void Server::Init()
 
 	max_replicate_xfers = max_store_xfers/5;
 
-#ifdef WANT_NETMAN
-	char		*tmp;				//declare this down here
-									//so the compiler doesn't argue
-	tmp = param( "MANAGE_BANDWIDTH" );
-	if (!tmp) {
-		ManageBandwidth = false;
-	} else {
-		if (tmp[0] == 'T' || tmp[0] == 't') {
-			ManageBandwidth = true;
-		} else {
-			ManageBandwidth = false;
-		}
-		free(tmp);
-		NetworkHorizon = param_integer( "NETWORK_HORIZON",300 );
-	}
-#endif
-
 	if (first_time) {
 		store_req_sd = SetUpPort(CKPT_SVR_STORE_REQ_PORT);
 		restore_req_sd = SetUpPort(CKPT_SVR_RESTORE_REQ_PORT);
@@ -360,6 +374,7 @@ void Server::Execute()
 	time_t         last_reclaim_time;
 	time_t		   last_clean_time;
 	time_t         last_check_parent_time;
+	time_t         last_remove_stale_time;
 	struct timeval poll;
 	int            ppid;
 	
@@ -378,9 +393,18 @@ void Server::Execute()
 	}
     xfer_summary.time_out(current_time, canon_name);
 
+	// determine if I need to clean up any files upon start up. This'll.
+	// ensure I don't get into a "start, leak, crash" problem which will
+	// consume the disk.
+	RemoveStaleCheckpointFiles(ckpt_server_dir);
+	// take into account that I grab the time AFTER I removed the state files
+	// this ensures I don't count the time it took to remove them.
+	last_remove_stale_time = time(NULL);
+
 	while (more) {                          // Continues until SIGUSR2 signal
 		poll.tv_sec = reclaim_interval - ((unsigned int)current_time -
 										  (unsigned int)last_reclaim_time);
+
 		poll.tv_usec = 0;
 
 		if( check_parent_interval > 0 ) {
@@ -395,6 +419,19 @@ void Server::Execute()
 			}
 		}
 
+		// if for whatever reason the user wants a very fast polling time to
+		// check for stale checkpoints, then allow it. This isn't exactly
+		// correct in that occasionally an event could be miscalculated because
+		// I'm not being more careful about the time left up until the next
+		// remove stale cycle. But this code is a hack, we don't have real
+		// timers, and I need to get it done. This should be "good enough" for
+		// now.
+		if ((remove_stale_ckptfile_interval > 0) &&
+			(remove_stale_ckptfile_interval < poll.tv_sec)) 
+		{
+			poll.tv_sec = remove_stale_ckptfile_interval;
+		}
+
 		errno = 0;
 		FD_ZERO(&req_sds);
 		FD_SET(store_req_sd, &req_sds);
@@ -472,6 +509,15 @@ void Server::Execute()
 			}
 			last_check_parent_time = current_time;
 		}
+
+		// and clean up any stale stuff, but only as fast as required in case
+		// of a busy server.
+		if (remove_stale_ckptfile_interval > 0 &&
+			current_time-last_remove_stale_time>=remove_stale_ckptfile_interval)
+		{
+			RemoveStaleCheckpointFiles(ckpt_server_dir);
+			last_remove_stale_time = time(NULL);
+		}
     }
 	free( canon_name );
 	close(service_req_sd);
@@ -1154,7 +1200,7 @@ void Server::Replicate()
 				errno = 0;
 				bytes_read = read(fd, &buf, sizeof(buf));
 				bytes_left = bytes_read;
-				while (bytes_left) {
+				while (bytes_left > 0) {
 					bytes_written = write(server_sd, &buf, bytes_read);
 					assert(bytes_written >= 0);
 					if (bytes_written == 0)
@@ -1252,76 +1298,6 @@ void Server::SendStatus(int data_conn_sd)
   imds.TransferFileInfo(xfer_sd);
 }
 
-#ifdef WANT_NETMAN
-static struct _file_stream_info {
-	struct in_addr shadow_IP;
-	int shadow_pid;
-	struct in_addr startd_IP;
-	bool store_req;
-	int total_bytes;
-	time_t last_update;
-	char user[MAX_NAME_LENGTH];
-} file_stream_info;
-
-extern "C" {
-void
-file_stream_progress_report(int bytes_moved)
-{
-	if (!ManageBandwidth) return;
-
-	time_t current_time = time(0);
-	if (current_time < file_stream_info.last_update + (NetworkHorizon/5))
-		return;
-
-	dprintf(D_ALWAYS, "Sending CkptServerUpdate: %d bytes to go.\n", 
-			file_stream_info.total_bytes-bytes_moved);
-	ClassAd request;
-	char buf[100];
-
-	sprintf(buf, "%s = \"CkptServerUpdate\"", ATTR_TRANSFER_TYPE);
-	request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_USER, file_stream_info.user);
-	request.Insert(buf);
-	sprintf(buf, "%s = 1", ATTR_FORCE);
-	request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_SOURCE,
-			file_stream_info.store_req ?
-			inet_ntoa(file_stream_info.startd_IP) :
-			inet_ntoa(*my_sin_addr()));
-	request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_DESTINATION,
-			file_stream_info.store_req ?
-			inet_ntoa(*my_sin_addr()) :
-			inet_ntoa(file_stream_info.startd_IP));
-	request.Insert(buf);
-	sprintf(buf, "%s = \"<%s:0>\"", ATTR_REMOTE_HOST,
-			inet_ntoa(file_stream_info.startd_IP));
-	request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_REQUESTED_CAPACITY,
-			file_stream_info.total_bytes-bytes_moved);
-	request.Insert(buf);
-	sprintf(buf, "%s = \"%u.%d\"", ATTR_TRANSFER_KEY,
-			ntohl(file_stream_info.shadow_IP.s_addr),
-			file_stream_info.shadow_pid);
- 	request.Insert(buf);
-	Daemon Negotiator(DT_NEGOTIATOR);
-
-    SafeSock sock;
-    sock.timeout(10);
-    if (!sock.connect(Negotiator.addr())) {
-		dprintf(D_ALWAYS, "Couldn't connect to negotiator!\n");
-    }
-
-	Negotiator.startCommand( REQUEST_NETWORK, &sock);
-
-	sock.put(1);
-	request.put(sock);
-	sock.end_of_message();
-	file_stream_info.last_update = time(0);
-}
-}
-#endif
-
 /* check to make sure something being used as a filename that we will
 	be reading or writing isn't trying to do anything funny. This
 	means the filename can't be "." ".." or have a path separator
@@ -1573,15 +1549,6 @@ void Server::ProcessStoreReq(int            req_id,
 			sprintf(pathname, "%s%s/%s/%s", LOCAL_DRIVE_PREFIX, 
 					inet_ntoa(shadow_IP), store_req.owner, 
 					store_req.filename);
-#ifdef WANT_NETMAN
-			memcpy(&file_stream_info.shadow_IP, &shadow_IP,
-				   sizeof(struct in_addr));
-			file_stream_info.shadow_pid = ntohl(store_req.key);
-			file_stream_info.store_req = true;
-			file_stream_info.total_bytes = (int) store_req.file_size;
-			file_stream_info.last_update = time(0);
-			strcpy(file_stream_info.user, store_req.owner);
-#endif
 			ReceiveCheckpointFile(data_conn_sd, pathname,
 								  (int) store_req.file_size);
 			exit(CHILDTERM_SUCCESS);
@@ -1638,9 +1605,6 @@ void Server::ReceiveCheckpointFile(int         data_conn_sd,
 		setsockopt(xfer_sd, SOL_SOCKET, SO_RCVBUF, (char*) &socket_bufsize, 
 				   sizeof(socket_bufsize));
 	}
-#ifdef WANT_NETMAN
-	file_stream_info.startd_IP = chkpt_addr.sin_addr;
-#endif
 	bytes_recvd = stream_file_xfer(xfer_sd, file_fd, file_size);
 	// note that if file size == -1, we don't know if we got the complete 
 	// file, so we must rely on the client to commit via SERVICE_RENAME
@@ -1889,15 +1853,6 @@ void Server::ProcessRestoreReq(int             req_id,
 			  close(store_req_sd);
 			  close(restore_req_sd);
 			  close(service_req_sd);
-#ifdef WANT_NETMAN
-			  memcpy(&file_stream_info.shadow_IP, &shadow_IP,
-					 sizeof(struct in_addr));
-			  file_stream_info.shadow_pid = ntohl(restore_req.key);
-			  file_stream_info.store_req = false;
-			  file_stream_info.total_bytes = (int) chkpt_file_status.st_size;
-			  file_stream_info.last_update = time(0);
-			  strcpy(file_stream_info.user, restore_req.owner);
-#endif
 			  TransmitCheckpointFile(data_conn_sd, pathname,
 									 chkpt_file_status.st_size);
 			  exit(CHILDTERM_SUCCESS);
@@ -1951,9 +1906,6 @@ void Server::TransmitCheckpointFile(int         data_conn_sd,
 			   sizeof(socket_bufsize));
 	}
 
-#ifdef WANT_NETMAN
-	file_stream_info.startd_IP = chkpt_addr.sin_addr;
-#endif
 	bytes_sent = stream_file_xfer(file_fd, xfer_sd, file_size);
 
 	close(xfer_sd);
@@ -2182,6 +2134,172 @@ void Server::ChildComplete()
 	}
 }
 
+void Server::RemoveStaleCheckpointFiles(const char *directory)
+{
+	time_t now;
+	MyString str;
+
+	now = time(NULL);
+
+	if (directory == NULL) {
+		// nothing to do
+		return;
+	}
+
+	// Here we check to see if we need to clean stale ckpts out.
+	if ((remove_stale_ckptfile_interval == 0) ||
+		(now < next_time_to_remove_stale_ckpt_files))
+	{
+		// nothing to do....
+		return;
+	}
+
+	dprintf(D_ALWAYS, "----------------------------------------------------\n");
+	Log("Begin removing stale checkpoint files.");
+
+	StatInfo st(directory);
+	Directory dir(&st, PRIV_CONDOR);
+
+	// do the dirty work.
+	RemoveStaleCheckpointFilesRecurse(directory, &dir,
+		now - stale_ckptfile_age_cutoff, st.GetAccessTime());
+
+	Log("Done removing stale checkpoint files.");
+
+	// set up the next time that I need to remove stale checkpoints 
+	// do this _after_ removing them so I don't count the removal
+	// time towards the interval to the next time I do it.
+	next_time_to_remove_stale_ckpt_files =
+		time(NULL) + remove_stale_ckptfile_interval;
+
+	str.sprintf("Next stale checkpoint file check in %lu seconds.",
+		(unsigned long)remove_stale_ckptfile_interval);
+
+	Log(str.Value());
+}
+
+void Server::RemoveStaleCheckpointFilesRecurse(const char *path, 
+	Directory *dir, time_t cutoff_time, time_t a_time)
+{
+	MyString str;
+	const char *file = NULL;
+	const char *base = NULL;
+
+	char real_path[PATH_MAX];
+	char real_ckpt_server_dir[PATH_MAX];
+
+	if (path == NULL) {
+		// in case of this, do nothing and return.
+		return;
+	}
+
+	if (realpath(path, real_path) < 0) {
+		str.sprintf("Server::RemoveStaleCheckpointFilesRecurse(): Could "
+			"not resolve %s into a real path: %d(%s). Ignoring.\n",
+			path, errno, strerror(errno));
+		return;
+	}
+
+	if (realpath(ckpt_server_dir, real_ckpt_server_dir) < 0) {
+		str.sprintf("Server::RemoveStaleCheckpointFilesRecurse(): Could "
+			"not resolve %s into a real path: %d(%s). Strange..ignoring "
+			"remove request for file under this directory.\n",
+			path, errno, strerror(errno));
+		return;
+	}
+
+	// We're going to do a small safety measure here and if the start of the
+	// path isn't the ckpt_server_dir, we immediately stop since we could have
+	// escaped that directory somehow and could be deleting who knows what
+	// as root. Not Cool.
+	if (
+		// path is shorter than ckpt_server_dir
+		(strlen(real_path) < strlen(real_ckpt_server_dir)) || 
+
+		// OR if path isn't a subdirectory of ckpt_server_dir
+		strncmp(real_path, real_ckpt_server_dir,
+			strlen(real_ckpt_server_dir)) != MATCH ||
+
+		// OR if path is tricksy
+		((strlen(real_path) > strlen(real_ckpt_server_dir)) && 
+			path[strlen(real_ckpt_server_dir)] != '/'))
+	{
+		str.sprintf(
+			"WARNING: "
+			"Server::RemoveStaleCheckpointFilesRecurse(): "
+			"path name %s, whose real path is %s, appears to be outside "
+			"of the ckpt_server_dir of %s. "
+			"Ignoring it.", path, real_path, real_ckpt_server_dir);
+		Log(str.Value());
+
+		return;
+	}
+
+	// if a directory, map myself over it 
+	if (IsDirectory(path)) {
+		Directory files(path, PRIV_CONDOR);
+
+		// Iterate over the directory
+		while(files.Next()) {
+
+			file = files.GetFullPath();
+
+			/* skip these files, especially so we don't recurse into
+				our parent directory.
+			*/
+			if (strcmp(file, ".") == MATCH ||
+				strcmp(file, "..") == MATCH) {
+				continue;
+			}
+
+			/* Here we pass a "back reference" to the directory object here
+				so we can delete the file using the mechanics of the
+				Directory object. A little clunky interface wise, 
+				but easy to implement. If somehow a directory is found
+				that leads outside of the ckpt_server_dir, the above code
+				will catch it.
+			*/
+			RemoveStaleCheckpointFilesRecurse(file, &files, cutoff_time,
+				files.GetAccessTime());
+		}
+
+		// all done with the directory.
+		return;
+	}
+
+	// otherwise is a file so check to see if we need to remove the file
+
+	// skip if any file, anywhere, named TransferLog is found
+	base = condor_basename(path);
+	if (strcmp(base, "TransferLog") == MATCH) {
+		// do nothing, don't even log it since it isn't worth it.
+		return;
+	}
+
+	/* everything else should be checked, though */
+
+	if (a_time <= cutoff_time) {
+		// Another safety rule is that I'll only remove files that
+		// have "cluster" as a prefix. This should describe
+		// the entire set of files I need to worry about--checkpoints
+		// and temporary checkpoints that didn't get renamed for whatever
+		// reason.
+		base = condor_basename(path);
+		// if for whatever reason path is a symlink, we'd end up just removing
+		// the link itself, which is ok, especially if it points to something
+		// outside of ckpt_server_dir. If it is inside it, and named right,
+		// then it'll get cleaned up properly in this or a later pass.
+		if (strncmp(base, "cluster", strlen("cluster")) == MATCH)
+		{
+			dprintf(D_ALWAYS, "Removing stale file: %s\n", path);
+			// reach back into a previous stack frame 
+			if (dir->Remove_Full_Path(path) == false) {
+				dprintf(D_ALWAYS, "Failed to remove stale file %s "
+					"(unknown why)\n", path);
+			}
+		}
+	}
+}
 
 void Server::NoMore(char const *reason)
 {
