@@ -36,6 +36,7 @@ extern "C" void event_mgr (void);
 #include "condor_attributes.h"
 #include "condor_daemon_core.h"
 #include "file_sql.h"
+#include "classad_merge.h"
 
 extern FILESQL *FILEObj;
 
@@ -54,9 +55,9 @@ CollectorEngine::CollectorEngine (CollectorStats *stats ) :
 	StartdAds     (GREATER_TABLE_SIZE, &adNameHashFunction),
 	StartdPrivateAds(GREATER_TABLE_SIZE, &adNameHashFunction),
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 	QuillAds     (GREATER_TABLE_SIZE, &adNameHashFunction),
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 	ScheddAds     (GREATER_TABLE_SIZE, &adNameHashFunction),
 	SubmittorAds  (GREATER_TABLE_SIZE, &adNameHashFunction),
@@ -69,6 +70,7 @@ CollectorEngine::CollectorEngine (CollectorStats *stats ) :
 	CollectorAds  (LESSER_TABLE_SIZE , &adNameHashFunction),
 	NegotiatorAds (LESSER_TABLE_SIZE , &adNameHashFunction),
 	HadAds        (LESSER_TABLE_SIZE , &adNameHashFunction),
+	GridAds       (LESSER_TABLE_SIZE , &adNameHashFunction),
 	LeaseManagerAds(LESSER_TABLE_SIZE , &adNameHashFunction),
 	GenericAds    (LESSER_TABLE_SIZE , &stringHashFunction)
 {
@@ -84,9 +86,9 @@ CollectorEngine::CollectorEngine (CollectorStats *stats ) :
 CollectorEngine::
 ~CollectorEngine ()
 {
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 	killHashTable (QuillAds);
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 	killHashTable (StartdAds);
 	killHashTable (StartdPrivateAds);
@@ -97,10 +99,12 @@ CollectorEngine::
 	killHashTable (StorageAds);
 	killHashTable (CkptServerAds);
 	killHashTable (GatewayAds);
+	killHashTable (CollectorAds);
 	killHashTable (NegotiatorAds);
 	killHashTable (HadAds);
 	killHashTable (XferServiceAds);
 	killHashTable (LeaseManagerAds);
+	killHashTable (GridAds);
 	GenericAds.walk(killGenericHashTable);
 
 	if(m_collector_requirements) {
@@ -177,7 +181,7 @@ scheduleHousekeeper (int timeout)
 
 
 int CollectorEngine::
-invokeHousekeeper (AdTypes adtype)
+invokeHousekeeper (AdTypes adType)
 {
 	time_t now;
 
@@ -185,83 +189,63 @@ invokeHousekeeper (AdTypes adtype)
 	if (now == (time_t) -1)
 	{
 		dprintf (D_ALWAYS, "Error in reading system time --- aborting\n");
-		return FALSE;
+		return 0;
 	}
 
-	switch (adtype)
-	{
-		case STARTD_AD:
-			cleanHashTable (StartdAds, now, makeStartdAdHashKey);
-			break;
-
-#ifdef WANT_QUILL
-		case QUILL_AD:
-			cleanHashTable (QuillAds, now, makeQuillAdHashKey);
-			break;
-#endif /* WANT_QUILL */
-
-		case SCHEDD_AD:
-			cleanHashTable (ScheddAds, now, makeScheddAdHashKey);
-			break;
-
-		case MASTER_AD:
-			cleanHashTable (MasterAds, now, makeMasterAdHashKey);
-			break;
-
-		case STARTD_PVT_AD:
-			cleanHashTable (StartdPrivateAds, now, makeStartdAdHashKey);
-			break;
-
-		case SUBMITTOR_AD:
-			cleanHashTable (SubmittorAds, now, makeScheddAdHashKey);
-			break;
-
-		case LICENSE_AD:
-			cleanHashTable (LicenseAds, now, makeLicenseAdHashKey);
-			break;
-
-		case STORAGE_AD:
-			cleanHashTable (StorageAds, now, makeStorageAdHashKey);
-			break;
-
-		case NEGOTIATOR_AD:
-			cleanHashTable (NegotiatorAds, now, makeStorageAdHashKey);
-			break;
-
-		case HAD_AD:
-			cleanHashTable (HadAds, now, makeHadAdHashKey);
-			break;
-
-		case GENERIC_AD:
-			CollectorHashTable *cht;
+	CollectorHashTable *table=0;
+	CollectorEngine::HashFunc func;
+	if (LookupByAdType(adType, table, func)) {
+		cleanHashTable(*table, now, func);
+	} else {
+		if (GENERIC_AD == adType) {
+			CollectorHashTable *cht=0;
 			GenericAds.startIterations();
 			while (GenericAds.iterate(cht)) {
 				cleanHashTable (*cht, now, makeGenericAdHashKey);
 			}
-			break;
-
-		case XFER_SERVICE_AD:
-			cleanHashTable (XferServiceAds, now, makeXferServiceAdHashKey);
-			break;
-
-		case LEASE_MANAGER_AD:
-			cleanHashTable (LeaseManagerAds, now, makeLeaseManagerAdHashKey);
-			break;
-
-		default:
+		} else {
 			return 0;
+		}
 	}
 
 	return 1;
 }
 
-
-void CollectorEngine::
-toggleLogging (void)
+int
+CollectorEngine::invalidateAds(AdTypes adType, ClassAd &query)
 {
-	log = !log;
-}
+	CollectorHashTable *table=0;
+	CollectorEngine::HashFunc func;
+	if (!LookupByAdType(adType, table, func)) {
+		dprintf (D_ALWAYS, "Unknown type %d\n", adType);
+		return 0;
+	}
 
+	int count = 0;
+	ClassAd  *ad;
+	AdNameHashKey  hk;
+	MyString hkString;
+	(*table).startIterations();
+	while ((*table).iterate (ad)) {
+		if (IsAHalfMatch(&query, ad)) {
+			(*table).getCurrentKey(hk);
+			hk.sprint(hkString);
+			if ((*table).remove(hk) == -1) {
+				dprintf(D_ALWAYS,
+						"\t\tError while removing ad: \"%s\"\n",
+						hkString.Value());
+			} else {
+				dprintf(D_ALWAYS,
+						"\t\t**** Invalidating ad: \"%s\"\n",
+						hkString.Value());
+				delete ad;
+				count++;
+			}
+		}
+	}
+
+	return count;
+}
 
 int (*CollectorEngine::genericTableScanFunction)(ClassAd *) = NULL;
 
@@ -285,103 +269,47 @@ walkGenericTables(int (*scanFunction)(ClassAd *))
 int CollectorEngine::
 walkHashTable (AdTypes adType, int (*scanFunction)(ClassAd *))
 {
-	CollectorHashTable *table;
-	int retval = 1;
+	//int retval = 1;
 
-	ClassAd *ad;
-
-	switch (adType)
-	{
-	  case STARTD_AD:
-		table = &StartdAds;
-		break;
-
-	  case SCHEDD_AD:
-		table = &ScheddAds;
-		break;
-
-#ifdef WANT_QUILL
-	  case QUILL_AD:
-		table = &QuillAds;
-		break;
-#endif /* WANT_QUILL */
-
-	  case MASTER_AD:
-		table = &MasterAds;
-		break;
-
-	  case CKPT_SRVR_AD:
-		table = &CkptServerAds;
-		break;
-
-	  case STARTD_PVT_AD:
-		table = &StartdPrivateAds;	
-		break;
-
-	  case SUBMITTOR_AD:
-		table = &SubmittorAds;
-		break;
-
-	  case LICENSE_AD:
-		table = &LicenseAds;
-		break;
-
-	  case COLLECTOR_AD:
-		table = &CollectorAds;
-		break;
-
-	  case STORAGE_AD:
-		table = &StorageAds;
-		break;
-
-	  case NEGOTIATOR_AD:
-		table = &NegotiatorAds;
-		break;
-
-	  case HAD_AD:
-		table = &HadAds;
-		break;
-
-	  case XFER_SERVICE_AD:
-		table = &XferServiceAds;
-		break;
-
-	  case LEASE_MANAGER_AD:
-		table = &LeaseManagerAds;
-		break;
-
-	  case ANY_AD:
+	if (GENERIC_AD == adType) {
+		return walkGenericTables(scanFunction);
+	} else if (ANY_AD == adType) {
 		return
 			StorageAds.walk(scanFunction) &&
 			CkptServerAds.walk(scanFunction) &&
 			LicenseAds.walk(scanFunction) &&
+			CollectorAds.walk(scanFunction) &&
 			StartdAds.walk(scanFunction) &&
 			ScheddAds.walk(scanFunction) &&
 			MasterAds.walk(scanFunction) &&
 			SubmittorAds.walk(scanFunction) &&
 			NegotiatorAds.walk(scanFunction) &&
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 			QuillAds.walk(scanFunction) &&
 #endif
 			HadAds.walk(scanFunction) &&
-
+			GridAds.walk(scanFunction) &&
 			XferServiceAds.walk(scanFunction) &&
 			LeaseManagerAds.walk(scanFunction) &&
-
 			walkGenericTables(scanFunction);
-	  default:
+	}
+
+	CollectorHashTable *table;
+	CollectorEngine::HashFunc func;
+	if (!LookupByAdType(adType, table, func)) {
 		dprintf (D_ALWAYS, "Unknown type %d\n", adType);
 		return 0;
 	}
 
 	// walk the hash table
-	table->startIterations ();
-	while (table->iterate (ad))
+	ClassAd *ad;
+	(*table).startIterations ();
+	while ((*table).iterate (ad))
 	{
 		// call scan function for each ad
 		if (!scanFunction (ad))
 		{
-			retval = 0;
+			//retval = 0;
 			break;
 		}
 	}
@@ -391,7 +319,7 @@ walkHashTable (AdTypes adType, int (*scanFunction)(ClassAd *))
 
 CollectorHashTable *CollectorEngine::findOrCreateTable(MyString &type)
 {
-	CollectorHashTable *table;
+	CollectorHashTable *table=0;
 	if (GenericAds.lookup(type, table) == -1) {
 		dprintf(D_ALWAYS, "creating new table for type %s\n", type.Value());
 		table = new CollectorHashTable(LESSER_TABLE_SIZE , &adNameHashFunction);
@@ -430,6 +358,15 @@ collect (int command, Sock *sock, sockaddr_in *from, int &insert)
 	// the above includes a timed communication with the client
 	sock->timeout(0);
 
+	// insert the authenticated user into the ad itself
+	const char* authn_user = sock->getFullyQualifiedUser();
+	if (authn_user) {
+		clientAd->Assign("AuthenticatedIdentity", authn_user);
+	} else {
+		// remove it from the ad if it's not authenticated.
+		clientAd->Delete("AuthenticatedIdentity");
+	}
+
 	rval = collect(command, clientAd, from, insert, sock);
 
 	// Don't leak the ad on error!
@@ -449,22 +386,6 @@ collect (int command, Sock *sock, sockaddr_in *from, int &insert)
 
 bool CollectorEngine::ValidateClassAd(int command,ClassAd *clientAd,Sock *sock)
 {
-    // In Version 7.2 only, reject any ads that have extra info packed
-    // into the sinful string (e.g. CCB info), because 7.2 does not
-    // support that and will get all confused.
-    {
-        MyString my_address;
-        clientAd->LookupString( ATTR_MY_ADDRESS, my_address );
-        if( my_address.FindChar('?') >= 0 ) {
-            dprintf(D_ALWAYS,
-                    "REJECTION: ClassAd from %s advertises incompatible"
-                    " IP address (perhaps using CCB or other feature"
-                    " not supported by this version of Condor): %s=%s\n",
-                    (sock ? sock->get_sinful_peer() : "(NULL)"),
-                    ATTR_MY_ADDRESS, my_address.Value());
-            return false;
-        }
-    }
 
 	if( !m_collector_requirements ) {
 			// no need to do any of the following checks if the admin has
@@ -475,6 +396,7 @@ bool CollectorEngine::ValidateClassAd(int command,ClassAd *clientAd,Sock *sock)
 
 	char const *ipattr = NULL;
 	switch( command ) {
+	  case MERGE_STARTD_AD:
 	  case UPDATE_STARTD_AD:
 	  case UPDATE_STARTD_AD_WITH_ACK:
 		  ipattr = ATTR_STARTD_IP_ADDR;
@@ -500,6 +422,7 @@ bool CollectorEngine::ValidateClassAd(int command,ClassAd *clientAd,Sock *sock)
 	  case UPDATE_STORAGE_AD:
 	  case UPDATE_HAD_AD:
 	  case UPDATE_AD_GENERIC:
+      case UPDATE_GRID_AD:
 		  break;
 	default:
 		dprintf(D_ALWAYS,
@@ -584,6 +507,9 @@ collect (int command,ClassAd *clientAd,sockaddr_in *from,int &insert,Sock *sock)
 	{
 	  case UPDATE_STARTD_AD:
 	  case UPDATE_STARTD_AD_WITH_ACK:
+#if !defined(WANT_OLD_CLASSADS)
+		  clientAd->AddTargetRefs( TargetJobAttrs );
+#endif
 		if ( repeatStartdAds > 0 ) {
 			clientAdToRepeat = new ClassAd(*clientAd);
 		}
@@ -668,7 +594,23 @@ collect (int command,ClassAd *clientAd,sockaddr_in *from,int &insert,Sock *sock)
 		}
 		break;
 
-#ifdef WANT_QUILL
+	  case MERGE_STARTD_AD:
+#if !defined(WANT_OLD_CLASSADS)
+		  clientAd->AddTargetRefs( TargetJobAttrs );
+#endif
+		if (!makeStartdAdHashKey (hk, clientAd, from))
+		{
+			dprintf (D_ALWAYS, "Could not make hashkey --- ignoring ad\n");
+			insert = -3;
+			retVal = 0;
+			break;
+		}
+		hashString.Build( hk );
+		retVal=mergeClassAd (StartdAds, "StartdAd     ", "Start",
+							  clientAd, hk, hashString, insert, from );
+		break;
+
+#ifdef HAVE_EXT_POSTGRESQL
 	  case UPDATE_QUILL_AD:
 		if (!makeQuillAdHashKey (hk, clientAd, from))
 		{
@@ -681,7 +623,7 @@ collect (int command,ClassAd *clientAd,sockaddr_in *from,int &insert,Sock *sock)
 		retVal=updateClassAd (QuillAds, "QuillAd     ", "Quill",
 							  clientAd, hk, hashString, insert, from );
 		break;
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 	  case UPDATE_SCHEDD_AD:
 		if (!makeScheddAdHashKey (hk, clientAd, from))
@@ -705,6 +647,12 @@ collect (int command,ClassAd *clientAd,sockaddr_in *from,int &insert,Sock *sock)
 			retVal = 0;
 			break;
 		}
+		// CRUFT: Before 7.3.2, submitter ads had a MyType of
+		//   "Scheduler". The only way to tell the difference
+		//   was that submitter ads didn't have ATTR_NUM_USERS.
+		//   Coerce MyStype to "Submitter" for ads coming from
+		//   these older schedds.
+		clientAd->SetMyTypeName( SUBMITTER_ADTYPE );
 		// since submittor ads always follow a schedd ad, and a master check is
 		// performed for schedd ads, we don't need a master check in here
 		hashString.Build( hk );
@@ -798,7 +746,7 @@ collect (int command,ClassAd *clientAd,sockaddr_in *from,int &insert,Sock *sock)
 		break;
 
 	  case UPDATE_HAD_AD:
-		  if (!makeHadAdHashKey (hk, clientAd, from))
+		if (!makeHadAdHashKey (hk, clientAd, from))
 		{
 			dprintf (D_ALWAYS, "Could not make hashkey --- ignoring ad\n");
 			insert = -3;
@@ -810,6 +758,19 @@ collect (int command,ClassAd *clientAd,sockaddr_in *from,int &insert,Sock *sock)
 							  clientAd, hk, hashString, insert, from );
 		break;
 
+	  case UPDATE_GRID_AD:
+		if (!makeGridAdHashKey(hk, clientAd, from))
+		{
+			dprintf (D_ALWAYS, "Could not make hashkey --- ignoring ad\n");
+			insert = -3;
+			retVal = 0;
+			break;
+		}
+		hashString.Build( hk );
+		retVal=updateClassAd (GridAds, "GridAd  ", "Grid",
+							  clientAd, hk, hashString, insert, from );
+          break;
+
 	  case UPDATE_AD_GENERIC:
 	  {
 		  const char *type_str = clientAd->GetMyTypeName();
@@ -885,6 +846,7 @@ collect (int command,ClassAd *clientAd,sockaddr_in *from,int &insert,Sock *sock)
   	  case QUERY_HAD_ADS:
   	  case QUERY_XFER_SERVICE_ADS:
   	  case QUERY_LEASE_MANAGER_ADS:
+	  case QUERY_GENERIC_ADS:
 	  case INVALIDATE_STARTD_ADS:
 	  case INVALIDATE_SCHEDD_ADS:
 	  case INVALIDATE_MASTER_ADS:
@@ -896,6 +858,7 @@ collect (int command,ClassAd *clientAd,sockaddr_in *from,int &insert,Sock *sock)
 	  case INVALIDATE_HAD_ADS:
 	  case INVALIDATE_XFER_SERVICE_ADS:
 	  case INVALIDATE_LEASE_MANAGER_ADS:
+	  case INVALIDATE_ADS_GENERIC:
 		// these are not implemented in the engine, but we allow another
 		// daemon to detect that these commands have been given
 	    insert = -2;
@@ -917,140 +880,61 @@ lookup (AdTypes adType, AdNameHashKey &hk)
 {
 	ClassAd *val;
 
-	switch (adType)
-	{
-		case STARTD_AD:
-			if (StartdAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-#ifdef WANT_QUILL
-		case QUILL_AD:
-			if (QuillAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-#endif /* WANT_QUILL */
-
-		case SCHEDD_AD:
-			if (ScheddAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case SUBMITTOR_AD:
-			if (SubmittorAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case LICENSE_AD:
-			if (LicenseAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case MASTER_AD:
-			if (MasterAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case CKPT_SRVR_AD:
-			if (CkptServerAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case COLLECTOR_AD:
-			if (CollectorAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case STARTD_PVT_AD:
-			if (StartdPrivateAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case STORAGE_AD:
-			if (StorageAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case NEGOTIATOR_AD:
-			if (NegotiatorAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case HAD_AD:
-			if (HadAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case XFER_SERVICE_AD:
-			if (XferServiceAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		case LEASE_MANAGER_AD:
-			if (LeaseManagerAds.lookup (hk, val) == -1)
-				return 0;
-			break;
-
-		default:
-			val = 0;
+	CollectorHashTable *table;
+	CollectorEngine::HashFunc func;
+	if (!LookupByAdType(adType, table, func)) {
+		return 0;
+	}
+	if (table->lookup(hk, val) == -1) {
+		return 0;
 	}
 
 	return val;
 }
 
+int CollectorEngine::remove (AdTypes t_AddType, const ClassAd & c_query, bool *query_contains_hash_key)
+{
+	int iRet = 0;
+	AdNameHashKey hk;
+	CollectorHashTable * table;  
+	HashFunc makeKey;
+	MyString hkString;
+
+	if( query_contains_hash_key ) {
+		*query_contains_hash_key = false;
+	}
+
+	// making it generic so any would be invalid query can contain these params.
+	if ( LookupByAdType (t_AddType, table, makeKey) )
+	{
+		ClassAd * pAd=0;
+		// try to create a hk from the query ad if it is possible.
+		if ( (*makeKey) (hk, const_cast<ClassAd*>(&c_query), NULL) ) {
+			if( query_contains_hash_key ) {
+				*query_contains_hash_key = true;
+			}
+			if( table->lookup(hk, pAd) != -1 )
+			{
+				hk.sprint( hkString );
+				iRet = !table->remove(hk);
+				dprintf (D_ALWAYS,"\t\t**** Removed(%d) ad(s): \"%s\"\n", iRet, hkString.Value() );
+				delete pAd;
+			}
+		}
+	}
+
+	return ( iRet );
+}
 
 int CollectorEngine::
 remove (AdTypes adType, AdNameHashKey &hk)
 {
-	switch (adType)
-	{
-		case STARTD_AD:
-			return !StartdAds.remove (hk);
-
-#ifdef WANT_QUILL
-		case QUILL_AD:
-			return !QuillAds.remove (hk);
-#endif /* WANT_QUILL */
-
-		case SCHEDD_AD:
-			return !ScheddAds.remove (hk);
-
-		case SUBMITTOR_AD:
-			return !SubmittorAds.remove (hk);
-
-		case LICENSE_AD:
-			return !LicenseAds.remove (hk);
-
-		case MASTER_AD:
-			return !MasterAds.remove (hk);
-
-		case CKPT_SRVR_AD:
-			return !CkptServerAds.remove (hk);
-
-		case STARTD_PVT_AD:
-			return !StartdPrivateAds.remove (hk);
-
-		case COLLECTOR_AD:
-			return !CollectorAds.remove (hk);
-
-		case STORAGE_AD:
-			return !StorageAds.remove (hk);
-
-		case NEGOTIATOR_AD:
-			return !NegotiatorAds.remove (hk);
-
-		case HAD_AD:
-			return !HadAds.remove (hk);
-
-		case XFER_SERVICE_AD:
-			return !XferServiceAds.remove (hk);
-
-		case LEASE_MANAGER_AD:
-			return !LeaseManagerAds.remove (hk);
-
-		default:
-			return 0;
+	CollectorHashTable *table;
+	CollectorEngine::HashFunc func;
+	if (!LookupByAdType(adType, table, func)) {
+		return 0;
 	}
+	return !table->remove(hk);
 }
 	
 				
@@ -1068,13 +952,18 @@ updateClassAd (CollectorHashTable &hashTable,
 	MyString	buf;
 	time_t		now;
 
-	(void) time (&now);
-	if (now == (time_t) -1)
-	{
-		EXCEPT ("Error reading system time!");
-	}	
-	buf.sprintf( "%s = %d", ATTR_LAST_HEARD_FROM, (int)now);
-	ad->Insert ( buf.GetCStr() );
+		// NOTE: LastHeardFrom will already be in ad if we are loading
+		// adds from the offline classad collection, so don't mess with
+		// it if it is already there
+	if( !ad->LookupExpr(ATTR_LAST_HEARD_FROM) ) {
+		(void) time (&now);
+		if (now == (time_t) -1)
+		{
+			EXCEPT ("Error reading system time!");
+		}	
+		buf.sprintf( "%s = %d", ATTR_LAST_HEARD_FROM, (int)now);
+		ad->Insert ( buf.Value() );
+	}
 
 	// this time stamped ad is the new ad
 	new_ad = ad;
@@ -1083,7 +972,7 @@ updateClassAd (CollectorHashTable &hashTable,
 	if ( hashTable.lookup (hk, old_ad) == -1)
     {	 	
 		// no ... new ad
-		dprintf (D_ALWAYS, "%s: Inserting ** \"%s\"\n", adType, hashString.GetCStr() );
+		dprintf (D_ALWAYS, "%s: Inserting ** \"%s\"\n", adType, hashString.Value() );
 
 		// Update statistics
 		collectorStats->update( label, NULL, new_ad );
@@ -1101,70 +990,70 @@ updateClassAd (CollectorHashTable &hashTable,
 	else
     {
 		// yes ... old ad must be updated
-		dprintf (D_FULLDEBUG, "%s: Updating ... \"%s\"\n", adType, hashString.GetCStr() );
+		dprintf (D_FULLDEBUG, "%s: Updating ... \"%s\"\n", adType, hashString.Value() );
 
 		// Update statistics
 		collectorStats->update( label, old_ad, new_ad );
 
 		// Now, finally, store the new ClassAd
-		old_ad->ExchangeExpressions (new_ad);
+		if (hashTable.remove(hk) == -1) {
+			EXCEPT( "Error removing ad" );
+		}
+		if (hashTable.insert(hk, new_ad) == -1) {
+			EXCEPT( "Error inserting ad" );
+		}
 
-		delete new_ad;
+		delete old_ad;
 
 		insert = 0;
-		return old_ad;
+		return new_ad;
 	}
 }
 
-#if 0
+ClassAd * CollectorEngine::
+mergeClassAd (CollectorHashTable &hashTable,
+			   const char *adType,
+			   const char *label,
+			   ClassAd *new_ad,
+			   AdNameHashKey &hk,
+			   const MyString &hashString,
+			   int  &insert,
+			   const sockaddr_in * /*from*/ )
+{
+	ClassAd		*old_ad = NULL;
+
+	insert = 0;
+
+	// check if it already exists in the hash table ...
+	if ( hashTable.lookup (hk, old_ad) == -1)
+    {	 	
+		dprintf (D_ALWAYS, "%s: Failed to merge update for ** \"%s\" because "
+				 "no existing ad matches.\n", adType, hashString.Value() );
+			// We should _NOT_ delete new_ad if we return NULL
+			// because our caller will delete it in that case.
+		return NULL;
+	}
+	else
+    {
+		// yes ... old ad must be updated
+		dprintf (D_FULLDEBUG, "%s: Merging update for ... \"%s\"\n",
+				 adType, hashString.Value() );
+
+			// Do not allow changes to some attributes
+		ClassAd new_ad_copy(*new_ad);
+		new_ad_copy.Delete(ATTR_AUTHENTICATED_IDENTITY);
+		new_ad_copy.Delete(ATTR_MY_TYPE);
+		new_ad_copy.Delete(ATTR_TARGET_TYPE);
+
+		// Now, finally, merge the new ClassAd into the old one
+		MergeClassAds(old_ad,&new_ad_copy,true);
+	}
+	delete new_ad;
+	return old_ad;
+}
+
+
 void
-CollectorEngine::updateAd ( ClassAd *ad ) {
-
-
-
-    updateClassAd ( StartdAds)
-
-    const char      *label = "Start";
-    AdNameHashKey   hashKey;
-    MyString	    key;
-    ClassAd		    *old_ad;
-    time_t		    now;
-
-    /* get the current time */
-    time ( &now );
-
-    if ( (time_t) -1 == now ) {
-        EXCEPT ("Error reading system time!");
-    }
-
-    buf.sprintf( "%s = %d", ATTR_LAST_HEARD_FROM, (int) now );
-	ad->Insert ( buf.GetCStr () );
-
-    /* update statistics */
-    collectorStats->update ( label, NULL, ad );
-
-    /* make a hash key for the ad */
-    if ( !makeStartdAdHashKey ( hashKey, (ClassAd*) &ad, NULL ) ) {
-
-        dprintf (
-            D_FULLDEBUG,
-            "OfflineCollectorPlugin::update: "
-            "failed to hash class ad. Ignoring.\n" );
-
-        return;
-
-    }
-
-    /* store it the ad */
-    hashKey.sprint ( key );
-    if ( -1 == StartdAds.insert ( key, ad ) ) {
-        EXCEPT ( "Error inserting ad (out of memory)" );
-	}
-
-}
-#endif
-
-int
 CollectorEngine::
 housekeeper()
 {
@@ -1175,7 +1064,7 @@ housekeeper()
 	{
 		dprintf (D_ALWAYS,
 				 "Housekeeper:  Error in reading system time --- aborting\n");
-		return FALSE;
+		return;
 	}
 
 	dprintf (D_ALWAYS, "Housekeeper:  Ready to clean old ads\n");
@@ -1186,10 +1075,10 @@ housekeeper()
 	dprintf (D_ALWAYS, "\tCleaning StartdPrivateAds ...\n");
 	cleanHashTable (StartdPrivateAds, now, makeStartdAdHashKey);
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 	dprintf (D_ALWAYS, "\tCleaning QuillAds ...\n");
 	cleanHashTable (QuillAds, now, makeQuillAdHashKey);
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 	dprintf (D_ALWAYS, "\tCleaning ScheddAds ...\n");
 	cleanHashTable (ScheddAds, now, makeScheddAdHashKey);
@@ -1218,6 +1107,9 @@ housekeeper()
 	dprintf (D_ALWAYS, "\tCleaning HadAds ...\n");
 	cleanHashTable (HadAds, now, makeHadAdHashKey);
 
+    dprintf (D_ALWAYS, "\tCleaning GridAds ...\n");
+	cleanHashTable (GridAds, now, makeGridAdHashKey);
+
 	dprintf (D_ALWAYS, "\tCleaning XferServiceAds ...\n");
 	cleanHashTable (XferServiceAds, now, makeXferServiceAdHashKey);
 
@@ -1235,12 +1127,10 @@ housekeeper()
 	event_mgr();
 
 	dprintf (D_ALWAYS, "Housekeeper:  Done cleaning\n");
-	return TRUE;
 }
 
 void CollectorEngine::
-cleanHashTable (CollectorHashTable &hashTable, time_t now,
-				bool (*makeKey) (AdNameHashKey &, ClassAd *, sockaddr_in *) )
+cleanHashTable (CollectorHashTable &hashTable, time_t now, HashFunc makeKey)
 {
 	ClassAd  *ad;
 	int   	 timeStamp;
@@ -1271,7 +1161,12 @@ cleanHashTable (CollectorHashTable &hashTable, time_t now,
 			// then remove it from the segregated table
 			(*makeKey) (hk, ad, NULL);
 			hk.sprint( hkString );
-			dprintf (D_ALWAYS,"\t\t**** Removing stale ad: \"%s\"\n", hkString.GetCStr() );
+			if( timeStamp == 0 ) {
+				dprintf (D_ALWAYS,"\t\t**** Removing invalidated ad: \"%s\"\n", hkString.Value() );
+			}
+			else {
+				dprintf (D_ALWAYS,"\t\t**** Removing stale ad: \"%s\"\n", hkString.Value() );
+			}
 			if (hashTable.remove (hk) == -1)
 			{
 				dprintf (D_ALWAYS, "\t\tError while removing ad\n");
@@ -1282,6 +1177,83 @@ cleanHashTable (CollectorHashTable &hashTable, time_t now,
 }
 
 
+bool
+CollectorEngine::LookupByAdType(AdTypes adType,
+								CollectorHashTable *&table,
+								CollectorEngine::HashFunc &func)
+{
+	switch (adType)
+	{
+		case STARTD_AD:
+			table = &StartdAds;
+			func = makeStartdAdHashKey;
+			break;
+#ifdef WANT_QUILL
+		case QUILL_AD:
+			table = &QuillAds;
+			func = makeQuillAdHashKey;
+			break;
+#endif /* WANT_QUILL */
+		case SCHEDD_AD:
+			table = &ScheddAds;
+			func = makeScheddAdHashKey;
+			break;
+		case SUBMITTOR_AD:
+			table = &SubmittorAds;
+			func = makeScheddAdHashKey;
+			break;
+		case LICENSE_AD:
+			table = &LicenseAds;
+			func = makeLicenseAdHashKey;
+			break;
+		case MASTER_AD:
+			table = &MasterAds;
+			func = makeMasterAdHashKey;
+			break;
+		case CKPT_SRVR_AD:
+			table = &CkptServerAds;
+			func = makeCkptSrvrAdHashKey;
+			break;
+		case STARTD_PVT_AD:
+			table = &StartdPrivateAds;
+			func = makeStartdAdHashKey;
+			break;
+		case COLLECTOR_AD:
+			table = &CollectorAds;
+			func = makeCollectorAdHashKey;
+			break;
+		case STORAGE_AD:
+			table = &StorageAds;
+			func = makeStorageAdHashKey;
+			break;
+		case NEGOTIATOR_AD:
+			table = &NegotiatorAds;
+			func = makeStorageAdHashKey; // XXX
+			break;
+		case HAD_AD:
+			table = &HadAds;
+			func = makeHadAdHashKey;
+			break;
+        case GRID_AD:
+			table = &GridAds;
+			func = makeGridAdHashKey;
+			break;
+		case XFER_SERVICE_AD:
+			table = &XferServiceAds;
+			func = makeXferServiceAdHashKey;
+			break;
+		case LEASE_MANAGER_AD:
+			table = &LeaseManagerAds;
+			func = makeLeaseManagerAdHashKey;
+			break;
+		default:
+			return false;
+	}
+
+	return true;
+}
+
+
 static void
 purgeHashTable( CollectorHashTable &table )
 {
@@ -1301,6 +1273,7 @@ killHashTable (CollectorHashTable &table)
 {
 	ClassAd *ad;
 
+	table.startIterations();
 	while (table.iterate (ad))
 	{
 		delete ad;
