@@ -146,7 +146,8 @@ OfflineCollectorPlugin::update (
 
 	/* make sure the command is relevant to us */
 	if ( UPDATE_STARTD_AD_WITH_ACK != command &&
-		 UPDATE_STARTD_AD != command ) {
+		 UPDATE_STARTD_AD != command &&
+		 MERGE_STARTD_AD != command ) {
 		 return;
 	}
 
@@ -164,14 +165,27 @@ OfflineCollectorPlugin::update (
 		return;
 
 	}
+	MyString s;
+	hashKey.sprint ( s );
+	compressSpaces ( s );
+	const char *key = s.Value ();
 
 	/* report whether this ad is "off-line" or not and update
 	   the ad accordingly. */		
 	int offline  = FALSE,
 		lifetime = 0;
 
-	if ( UPDATE_STARTD_AD_WITH_ACK == command ) {
-		
+	bool offline_explicit = false;
+	if( ad.EvalBool( ATTR_OFFLINE, NULL, offline ) ) {
+		offline_explicit = true;
+	}
+
+	if ( MERGE_STARTD_AD == command ) {
+		mergeClassAd( ad, key );
+		return;
+	}
+	else if ( UPDATE_STARTD_AD_WITH_ACK == command && !offline_explicit ) {
+
 		/* set the off-line state of the machine */
 		offline = TRUE;
 
@@ -210,55 +224,41 @@ OfflineCollectorPlugin::update (
 
 		/* any others? */
 
-	} else {
 
-		/* try and determine if the ad has an offline attribute. 
-		If not, don't consider it a fatal error, we'll just set
-		the ad to be "on-line" bellow. */
-		ad.LookupInteger ( ATTR_OFFLINE, offline );
-
-		/* if the ad was offline, then put it back in the game */
-		if ( TRUE == offline ) {
-
-			/* flag the ad as back "on-line" */
-			offline = FALSE;
-			
-			/* set a sane ad lifetime */
-			lifetime = 30; /* from collector_engine.cpp */
+		dprintf ( 
+			D_FULLDEBUG, 
+			"Machine ad lifetime: %d\n",
+			lifetime );
 
+			/* record the new values as specified above */
+		ad.Assign ( ATTR_OFFLINE, (bool)offline );
+		if ( lifetime > 0 ) {
+			ad.Assign ( ATTR_CLASSAD_LIFETIME, lifetime );
 		}
-
 	}
 
-	dprintf ( 
-		D_FULLDEBUG, 
-		"Machine ad lifetime: %d\n",
-		lifetime );
-
-	/* record the new values as specified above */
-	ad.Assign ( ATTR_OFFLINE, offline );
-	if ( lifetime > 0 ) {
-		ad.Assign ( ATTR_CLASSAD_LIFETIME, lifetime );
-	}
-	
 	_ads->BeginTransaction ();
 
 	ClassAd *p;
-	MyString s;
-	hashKey.sprint ( s );
-	compressSpaces ( s );
-	const char *key = s.Value ();
 
 	/* if it is off-line then add it to the list; otherwise,
 	   remove it. */
 	if ( offline > 0 ) {
 
-		/* don't try to add duplicate ads */
+		/* replace duplicate ads */
 		if ( _ads->LookupClassAd ( key, p ) ) {
 			
-			_ads->AbortTransaction ();
-			return;
+			if ( !_ads->DestroyClassAd ( key ) ) {
+				dprintf (
+					D_FULLDEBUG,
+					"OfflineCollectorPlugin::update: "
+					"failed remove existing off-line ad from the persistent "
+					"store.\n" );
 
+				_ads->AbortTransaction ();
+				return;
+			}
+			dprintf(D_FULLDEBUG, "Replacing existing offline ad.\n");
 		}
 
 		/* try to add the new ad */
@@ -307,6 +307,53 @@ OfflineCollectorPlugin::update (
 
 }
 
+void
+OfflineCollectorPlugin::mergeClassAd (
+	ClassAd &ad,
+	char const *key )
+{
+	ClassAd *old_ad;
+
+	_ads->BeginTransaction ();
+
+	if ( !_ads->LookupClassAd ( key, old_ad ) ) {
+		_ads->AbortTransaction ();
+		return;
+	}
+
+	ad.ResetExpr();
+	ExprTree *expr;
+	const char *attr_name;
+	while (ad.NextExpr(attr_name, expr)) {
+		MyString new_val;
+		MyString old_val;
+
+		ASSERT( attr_name && expr );
+
+		new_val = ExprTreeToString( expr );
+
+		expr = old_ad->LookupExpr( attr_name );
+		if( expr ) {
+			old_val = ExprTreeToString( expr );
+			if( new_val == old_val ) {
+				continue;
+			}
+		}
+
+			// filter out stuff we never want to mess with
+		if( !strcasecmp(attr_name,ATTR_MY_TYPE) ||
+			!strcasecmp(attr_name,ATTR_TARGET_TYPE) ||
+			!strcasecmp(attr_name,ATTR_AUTHENTICATED_IDENTITY) )
+		{
+			continue;
+		}
+
+		_ads->SetAttribute(key, attr_name, new_val.Value());
+	}
+
+	_ads->CommitTransaction ();
+}
+
 void
 OfflineCollectorPlugin::invalidate (
 	int			 command,
