@@ -115,7 +115,7 @@ void ViewServer::Init()
 
 	// Register timer for logging information to history files
 	HistoryTimer = daemonCore->Register_Timer(InitialDelay,HistoryInterval,
-		(Event)WriteHistory,"WriteHistory");
+		WriteHistory,"WriteHistory");
 
 	// Initialize hash tables
 
@@ -392,6 +392,7 @@ int ViewServer::SendListReply(Stream* sock,const MyString& FileName, int FromDat
 			// set the seek to the correct spot and begin searching the file
 		if(fsetpos(fp, search_offset) < 0) {
 				// something failed, so get out of here
+			fclose(fp);
 			return -1;
 		}
 	}
@@ -480,6 +481,7 @@ int ViewServer::SendDataReply(Stream* sock,const MyString& FileName, int FromDat
 			// set the seek to the correct spot and begin searching the file
 		if(fsetpos(fp, search_offset) < 0) {
 				// something failed, so get out of here
+			fclose(fp);
 			return -1;
 		}
 	}
@@ -1065,7 +1067,7 @@ GeneralRecord* ViewServer::GetAccData(AccHash* AccData,const MyString& Key)
 			EXCEPT( "Failed to allocate a GeneralRecord" );
 		}
 		if ( AccData->insert(Key,GenRec) < 0 ) {
-			EXCEPT( "Insert failed: Key=%s", Key.GetCStr() );
+			EXCEPT( "Insert failed: Key=%s", Key.Value() );
 		}
 	}
 	return GenRec;
