@@ -22,6 +22,7 @@
 #include "condor_debug.h"
 #include "condor_config.h"
 #include "condor_ver_info.h"
+#include "condor_open.h"
 
 #include "daemon.h"
 #include "condor_string.h"
@@ -40,6 +41,7 @@
 #include "condor_netdb.h"
 #include "daemon_core_sock_adapter.h"
 #include "subsystem_info.h"
+#include "condor_sinful.h"
 
 #include "counted_ptr.h"
 
@@ -67,7 +69,9 @@ Daemon::common_init() {
 	m_daemon_ad_ptr = NULL;
 	char buf[200];
 	sprintf(buf,"%s_TIMEOUT_MULTIPLIER",get_mySubSystem()->getName() );
-	Sock::set_timeout_multiplier( param_integer(buf,0) );
+	Sock::set_timeout_multiplier( param_integer(buf, param_integer("TIMEOUT_MULTIPLIER", 0)) );
+	dprintf(D_DAEMONCORE, "*** TIMEOUT_MULTIPLIER :: %d\n", Sock::get_timeout_multiplier());
+	m_has_udp_command_port = true;
 }
 
 
@@ -91,7 +95,7 @@ Daemon::Daemon( daemon_t tType, const char* tName, const char* tPool )
 
 	if( tName && tName[0] ) {
 		if( is_valid_sinful(tName) ) {
-			_addr = strnewp( tName );
+			New_addr( strnewp(tName) );
 		} else {
 			_name = strnewp( tName );
 		}
@@ -122,6 +126,30 @@ Daemon::Daemon( const ClassAd* tAd, daemon_t tType, const char* tPool )
 	case DT_SCHEDD:
 		_subsys = strnewp( "SCHEDD" );
 		break;
+	case DT_CLUSTER:
+		_subsys = strnewp( "CLUSTERD" );
+		break;
+	case DT_COLLECTOR:
+		_subsys = strnewp( "COLLECTOR" );
+		break;
+	case DT_NEGOTIATOR:
+		_subsys = strnewp( "NEGOTIATOR" );
+		break;
+	case DT_CREDD:
+		_subsys = strnewp( "CREDD" );
+		break;
+	case DT_QUILL:
+		_subsys = strnewp( "QUILL" );
+		break;
+	case DT_LEASE_MANAGER:
+		_subsys = strnewp( "LEASE_MANAGER" );
+		break;
+	case DT_GENERIC:
+		_subsys = strnewp( "GENERIC" );
+		break;
+	case DT_HAD:
+		_subsys = strnewp( "HAD" );
+		break;
 	default:
 		EXCEPT( "Invalid daemon_type %d (%s) in ClassAd version of "
 				"Daemon object", (int)_type, daemonString(_type) );
@@ -200,11 +228,6 @@ Daemon::deepCopy( const Daemon &copy )
 	}
 	_subsys = strnewp( copy._subsys );
 
-	if( _subsys ) {
-        delete [] _subsys;
-    }
-    _subsys = strnewp( copy._subsys );
-
 	_port = copy._port;
 	_type = copy._type;
 	_is_local = copy._is_local;
@@ -343,6 +366,8 @@ Daemon::idStr( void )
 	const char* dt_str;
 	if( _type == DT_ANY ) {
 		dt_str = "daemon";
+	} else if( _type == DT_GENERIC ) {
+		dt_str = _subsys;
 	} else {
 		dt_str = daemonString(_type);
 	}
@@ -355,7 +380,10 @@ Daemon::idStr( void )
 		buf.sprintf( "%s %s", dt_str, _name );
 	} else if( _addr ) {
 		ASSERT( dt_str );
-		buf.sprintf( "%s at %s", dt_str, _addr );
+		Sinful sinful(_addr);
+		sinful.clearParams(); // too much info is ugly
+		buf.sprintf( "%s at %s", dt_str,
+					 sinful.getSinful() ? sinful.getSinful() : _addr );
 		if( _full_hostname ) {
 			buf.sprintf_cat( " (%s)", _full_hostname );
 		}
@@ -401,70 +429,112 @@ Daemon::display( FILE* fp )
 			 _error ? _error : "(null)" );
 }
 
+bool
+Daemon::nextValidCm()
+{
+	char *dname;
+	bool rval = false;
+
+	do {
+ 		dname = daemon_list.next();
+		if( dname != NULL )
+		{
+			rval = findCmDaemon( dname );
+			if( rval == true ) {
+				locate();
+			}
+		}
+	} while( rval == false && dname != NULL );
+	return rval;
+}
+
+
+void
+Daemon::rewindCmList()
+{
+	char *dname;
+
+	daemon_list.rewind();
+ 	dname = daemon_list.next();
+	findCmDaemon( dname );
+	locate();
+}
+
 
 //////////////////////////////////////////////////////////////////////
 // Communication methods
 //////////////////////////////////////////////////////////////////////
 
 ReliSock*
-Daemon::reliSock( int sec, CondorError* errstack, bool non_blocking, bool ignore_timeout_multiplier )
+Daemon::reliSock( int timeout, time_t deadline, CondorError* errstack, bool non_blocking, bool ignore_timeout_multiplier )
 {
 	if( !checkAddr() ) {
 			// this already deals w/ _error for us...
 		return NULL;
 	}
-	ReliSock* reli;
-	reli = new ReliSock();
-	reli->set_peer_description(idStr());
-	if( sec ) {
-		reli->timeout( sec );
-		if( ignore_timeout_multiplier ) {
-			reli->ignoreTimeoutMultiplier();
-		}
-	}
-	int rc = reli->connect(_addr, 0, non_blocking);
-	if(rc || (non_blocking && rc == CEDAR_EWOULDBLOCK)) {
-		return reli;
-	} else {
-		if (errstack) {
-			errstack->pushf("CEDAR", CEDAR_ERR_CONNECT_FAILED,
-				"Failed to connect to %s", _addr);
-		}
-		delete reli;
+	ReliSock* sock;
+	sock = new ReliSock();
+
+	sock->set_deadline( deadline );
+
+	if( !connectSock(sock,timeout,errstack,non_blocking,ignore_timeout_multiplier) )
+	{
+		delete sock;
 		return NULL;
 	}
+
+	return sock;
+}
+
+SafeSock*
+Daemon::safeSock( int timeout, time_t deadline, CondorError* errstack, bool non_blocking )
+{
+	if( !checkAddr() ) {
+			// this already deals w/ _error for us...
+		return NULL;
+	}
+	SafeSock* sock;
+	sock = new SafeSock();
+
+	sock->set_deadline( deadline );
+
+	if( !connectSock(sock,timeout,errstack,non_blocking) )
+	{
+		delete sock;
+		return NULL;
+	}
+
+	return sock;
 }
 
 
-SafeSock*
-Daemon::safeSock( int sec, CondorError* errstack, bool non_blocking )
+bool
+Daemon::connectSock(Sock *sock, int sec, CondorError* errstack, bool non_blocking, bool ignore_timeout_multiplier )
 {
-	if( !checkAddr() ) {
-			// this already deals w/ _error for us...
-		return NULL;
-	}
-	SafeSock* safe;
-	safe = new SafeSock();
-	safe->set_peer_description(idStr());
+
+	sock->set_peer_description(idStr());
 	if( sec ) {
-		safe->timeout( sec );
-	}
-	int rc = safe->connect(_addr, 0, non_blocking);
-	if(rc || (non_blocking && rc == CEDAR_EWOULDBLOCK)) {
-		return safe;
-	} else {
-		if (errstack) {
-			errstack->pushf("CEDAR", CEDAR_ERR_CONNECT_FAILED,
-				"Failed to connect to %s", _addr);
+		sock->timeout( sec );
+		if( ignore_timeout_multiplier ) {
+			sock->ignoreTimeoutMultiplier();
 		}
-		delete safe;
-		return NULL;
 	}
+
+	int rc = sock->connect(_addr, 0, non_blocking);
+	if(rc || (non_blocking && rc == CEDAR_EWOULDBLOCK)) {
+		return true;
+	}
+
+	if (errstack) {
+		errstack->pushf("CEDAR", CEDAR_ERR_CONNECT_FAILED,
+			"Failed to connect to %s", _addr);
+	}
+	return false;
 }
 
 
 StartCommandResult
-Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, char *version, SecMan *sec_man, bool raw_protocol, char const *sec_session_id )
+Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, char *, SecMan *sec_man, bool raw_protocol, char const *sec_session_id )
 {
 	// This function may be either blocking or non-blocking, depending
 	// on the flag that is passed in.  All versions of Daemon::startCommand()
@@ -474,7 +544,6 @@ Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, S
 	// eventually called in all code paths.
 
 	StartCommandResult start_command_result = StartCommandFailed;
-	bool other_side_can_negotiate = true; //default assumption
 
 	ASSERT(sock);
 
@@ -487,19 +556,7 @@ Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, S
 		sock->timeout( timeout );
 	}
 
-	// look at the version if it is available.  we must disable
-	// negotiation when talking to pre-6.3.3.
-	if (version) {
-		dprintf(D_SECURITY, "DAEMON: talking to a %s daemon.\n", version);
-		CondorVersionInfo vi(version);
-		if ( !vi.built_since_version(6,3,3) ) {
-			dprintf( D_SECURITY, "DAEMON: "
-					 "security negotiation not possible, disabling.\n" );
-			other_side_can_negotiate = false;
-		}
-	}
-
-	start_command_result = sec_man->startCommand(cmd, sock, other_side_can_negotiate, raw_protocol, errstack, 0, callback_fn, misc_data, nonblocking, cmd_description, sec_session_id);
+	start_command_result = sec_man->startCommand(cmd, sock, raw_protocol, errstack, 0, callback_fn, misc_data, nonblocking, cmd_description, sec_session_id);
 
 	if(callback_fn) {
 		// SecMan::startCommand() called the callback function, so we just return here
@@ -511,6 +568,24 @@ Daemon::startCommand( int cmd, Sock* sock, int timeout, CondorError *errstack, S
 	}
 }
 
+Sock *
+Daemon::makeConnectedSocket( Stream::stream_type st,
+							 int timeout, time_t deadline,
+							 CondorError* errstack, bool non_blocking )
+{
+	switch( st ) {
+	case Stream::reli_sock:
+		return reliSock(timeout, deadline, errstack, non_blocking);
+	case Stream::safe_sock:
+		return safeSock(timeout, deadline, errstack, non_blocking);
+	default: break;
+	}
+
+	EXCEPT( "Unknown stream_type (%d) in Daemon::makeConnectedSocket",
+			(int)st );
+	return NULL;
+}
+
 StartCommandResult
 Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, CondorError *errstack, StartCommandCallbackType *callback_fn, void *misc_data, bool nonblocking, char const *cmd_description, bool raw_protocol, char const *sec_session_id )
 {
@@ -523,17 +598,7 @@ Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, C
 	// Also, there's no one to delete the Sock.
 	ASSERT(!nonblocking || callback_fn);
 
-	switch( st ) {
-	case Stream::reli_sock:
-		*sock = reliSock(timeout, errstack, nonblocking);
-		break;
-	case Stream::safe_sock:
-		*sock = safeSock(timeout, errstack, nonblocking);
-		break;
-	default:
-		EXCEPT( "Unknown stream_type (%d) in Daemon::startCommand",
-				(int)st );
-	}
+	*sock = makeConnectedSocket(st,timeout,0,errstack,nonblocking);
 	if( ! *sock ) {
 		if ( callback_fn ) {
 			(*callback_fn)( false, NULL, errstack, misc_data );
@@ -628,7 +693,7 @@ Daemon::sendCommand( int cmd, Sock* sock, int sec, CondorError* errstack, char c
 	if( ! startCommand( cmd, sock, sec, errstack, cmd_description )) {
 		return false;
 	}
-	if( ! sock->eom() ) {
+	if( ! sock->end_of_message() ) {
 		MyString err_buf;
 		err_buf.sprintf( "Can't send eom for %d to %s", cmd,  
 				 idStr() );
@@ -646,7 +711,7 @@ Daemon::sendCommand( int cmd, Stream::stream_type st, int sec, CondorError* errs
 	if( ! tmp ) {
 		return false;
 	}
-	if( ! tmp->eom() ) {
+	if( ! tmp->end_of_message() ) {
 		MyString err_buf;
 		err_buf.sprintf( "Can't send eom for %d to %s", cmd,  
 				 idStr() );
@@ -699,7 +764,7 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 		cmd_sock->timeout( timeout );
 	}
 
-	if( ! cmd_sock->connect(_addr) ) {
+	if( ! connectSock(cmd_sock) ) {
 		MyString err_msg = "Failed to connect to ";
 		err_msg += daemonString(_type);
 		err_msg += " ";
@@ -827,7 +892,7 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 bool
 Daemon::locate( void )
 {
-	bool rval;
+	bool rval=false;
 	char* tmp = NULL;
 
 		// Make sure we only call locate() once.
@@ -852,20 +917,29 @@ Daemon::locate( void )
 		// don't do anything
 		rval = true;
 		break;
+	case DT_GENERIC:
+		rval = getDaemonInfo( GENERIC_AD );
+		break;
 	case DT_CLUSTER:
-		rval = getDaemonInfo( "CLUSTER", CLUSTER_AD );
+		setSubsystem( "CLUSTER" );
+		rval = getDaemonInfo( CLUSTER_AD );
 		break;
 	case DT_SCHEDD:
-		rval = getDaemonInfo( "SCHEDD", SCHEDD_AD );
+		setSubsystem( "SCHEDD" );
+		rval = getDaemonInfo( SCHEDD_AD );
 		break;
 	case DT_STARTD:
-		rval = getDaemonInfo( "STARTD", STARTD_AD );
+		setSubsystem( "STARTD" );
+		rval = getDaemonInfo( STARTD_AD );
 		break;
 	case DT_MASTER:
-		rval = getDaemonInfo( "MASTER", MASTER_AD );
+		setSubsystem( "MASTER" );
+		rval = getDaemonInfo( MASTER_AD );
 		break;
 	case DT_COLLECTOR:
-		rval = getCmInfo( "COLLECTOR" );
+		do {
+			rval = getCmInfo( "COLLECTOR" );
+		} while (rval == false && nextValidCm() == true);
 		break;
 	case DT_NEGOTIATOR:
 		if( !_pool && (tmp = getCmHostFromConfig( "NEGOTIATOR" )) ) {
@@ -878,14 +952,17 @@ Daemon::locate( void )
 		} else {
 				// cool, no NEGOTIATOR_HOST, we can treat it just like
 				// any other daemon 
-			rval = getDaemonInfo ( "NEGOTIATOR", NEGOTIATOR_AD );
+	  		setSubsystem( "NEGOTIATOR" );
+			rval = getDaemonInfo ( NEGOTIATOR_AD );
 		}
 		break;
 	case DT_CREDD:
-	  rval = getDaemonInfo( "CREDD", CREDD_AD );
+	  setSubsystem( "CREDD" );
+	  rval = getDaemonInfo( CREDD_AD );
 	  break;
 	case DT_STORK:
-	  rval = getDaemonInfo( "STORK", ANY_AD, false );
+	  setSubsystem( "STORK" );
+	  rval = getDaemonInfo( ANY_AD, false );
 	  break;
 	case DT_VIEW_COLLECTOR:
 		if( (rval = getCmInfo("CONDOR_VIEW")) ) {
@@ -894,19 +971,32 @@ Daemon::locate( void )
 		} 
 			// If there's nothing CONDOR_VIEW-specific, try just using
 			// "COLLECTOR".
-		rval = getCmInfo( "COLLECTOR" ); 
+		do {
+			rval = getCmInfo( "COLLECTOR" );
+		} while (rval == false && nextValidCm() == true);
 		break;
 	case DT_QUILL:
-		rval = getDaemonInfo( "QUILL", SCHEDD_AD );
+		setSubsystem( "QUILL" );
+		rval = getDaemonInfo( SCHEDD_AD );
 		break;
 	case DT_TRANSFERD:
-		rval = getDaemonInfo( "TRANSFERD", ANY_AD );
+		setSubsystem( "TRANSFERD" );
+		rval = getDaemonInfo( ANY_AD );
 		break;
 	case DT_LEASE_MANAGER:
-		rval = getDaemonInfo( "LEASEMANAGER", LEASE_MANAGER_AD, true );
+		setSubsystem( "LEASEMANAGER" );
+		rval = getDaemonInfo( LEASE_MANAGER_AD, true );
+		break;
+	case DT_HAD:
+		setSubsystem( "HAD" );
+		rval = getDaemonInfo( HAD_AD );
+		break;
+	case DT_KBDD:
+		setSubsystem( "KBDD" );
+		rval = getDaemonInfo( NO_AD );
 		break;
 	default:
-		EXCEPT( "Unknown daemon type (%d) in Daemon::init", (int)_type );
+		EXCEPT( "Unknown daemon type (%d) in Daemon::locate", (int)_type );
 	}
 
 	if( ! rval) {
@@ -939,17 +1029,29 @@ Daemon::locate( void )
 
 
 bool
-Daemon::getDaemonInfo( const char* subsys, AdTypes adtype, bool query_collector)
+Daemon::setSubsystem( const char* subsys )
+{
+	if( _subsys ) {
+		delete [] _subsys;
+	}
+	_subsys = strnewp( subsys );
+
+	return true;
+}
+
+
+bool
+Daemon::getDaemonInfo( AdTypes adtype, bool query_collector )
 {
 	MyString			buf;
 	char				*tmp, *my_name;
 	char				*host = NULL;
 	bool				nameHasPort = false;
 
-	if( _subsys ) {
-		delete [] _subsys;
+	if ( ! _subsys ) {
+		dprintf( D_ALWAYS, "Unable to get daemon information because no subsystem specified\n");
+		return false;
 	}
-	_subsys = strnewp( subsys );
 
 	if( _addr && is_valid_sinful(_addr) ) {
 		dprintf( D_HOSTNAME, "Already have address, no info to locate\n" );
@@ -960,7 +1062,7 @@ Daemon::getDaemonInfo( const char* subsys, AdTypes adtype, bool query_collector)
 		// If we were not passed a name or an addr, check the
 		// config file for a subsystem_HOST, e.g. SCHEDD_HOST=XXXX
 	if( ! _name  && !_pool ) {
-		buf.sprintf( "%s_HOST", subsys );
+		buf.sprintf( "%s_HOST", _subsys );
 		char *specified_host = param( buf.Value() );
 		if ( specified_host ) {
 				// Found an entry.  Use this name.
@@ -993,33 +1095,34 @@ Daemon::getDaemonInfo( const char* subsys, AdTypes adtype, bool query_collector)
 		
 		dprintf( D_HOSTNAME, "Port %d specified in name\n", _port );
 
-		if( is_ipaddr(host, &sin_addr) ) {
+		if(host && is_ipaddr(host, &sin_addr) ) {
 			buf.sprintf( "<%s:%d>", host, _port );
 			New_addr( strnewp(buf.Value()) );
 			dprintf( D_HOSTNAME,
 					"Host info \"%s\" is an IP address\n", host );
 		} else {
 				// We were given a hostname, not an address.
-			dprintf( D_HOSTNAME, "Host info \"%s\" is a hostname, "
-					 "finding IP address\n", host );
-			tmp = get_full_hostname( host, &sin_addr );
-			if( ! tmp ) {
+			if(host) {
+				dprintf( D_HOSTNAME, "Host info \"%s\" is a hostname, "
+						 "finding IP address\n", host );
+				if((tmp = get_full_hostname( host, &sin_addr ))==0) {
 					// With a hostname, this is a fatal Daemon error.
-				buf.sprintf( "unknown host %s", host );
-				newError( CA_LOCATE_FAILED, buf.Value() );
-				if (host) free( host );
+					buf.sprintf( "unknown host %s", host );
+					newError( CA_LOCATE_FAILED, buf.Value() );
+					if (host) free( host );
 
-					// We assume this is a transient DNS failure.  Therefore,
-					// set _tried_locate = false, so that we keep trying in
-					// future calls to locate().
-				_tried_locate = false;
+						// We assume this is a transient DNS failure.  Therefore,
+						// set _tried_locate = false, so that we keep trying in
+						// future calls to locate().
+					_tried_locate = false;
 
-				return false;
-			}
+					return false;
+				}
+			} else return false;
 			buf.sprintf( "<%s:%d>", inet_ntoa(sin_addr), _port );
 			dprintf( D_HOSTNAME, "Found IP address and port %s\n", buf.Value() );
 			New_addr( strnewp(buf.Value()) );
-			New_full_hostname( tmp );
+			if(tmp) New_full_hostname( tmp );
 		}
 
 		if (host) free( host );
@@ -1091,9 +1194,9 @@ Daemon::getDaemonInfo( const char* subsys, AdTypes adtype, bool query_collector)
 		// address of the daemon in question.
 
 	if( _is_local ) {
-		bool foundLocalAd = readLocalClassAd( subsys );
+		bool foundLocalAd = readLocalClassAd( _subsys );
 		if(!foundLocalAd) {
-			readAddressFile( subsys );
+			readAddressFile( _subsys );
 		}
 	}
 
@@ -1108,7 +1211,8 @@ Daemon::getDaemonInfo( const char* subsys, AdTypes adtype, bool query_collector)
 		ClassAd*			scan;
 		ClassAdList			ads;
 
-		if( _type == DT_STARTD && ! strchr(_name, '@') ) { 
+		if( (_type == DT_STARTD && ! strchr(_name, '@')) ||
+			_type == DT_HAD ) { 
 				/*
 				  So long as an SMP startd has only 1 command socket
 				  per startd, we want to take advantage of that and
@@ -1131,6 +1235,8 @@ Daemon::getDaemonInfo( const char* subsys, AdTypes adtype, bool query_collector)
 				*/
 			buf.sprintf( "%s == \"%s\"", ATTR_MACHINE, _full_hostname ); 
 			query.addANDConstraint( buf.Value() );
+		} else if ( _type == DT_GENERIC ) {
+			query.setGenericQueryType(_subsys);
 		} else if ( _name ) {
 			buf.sprintf( "%s == \"%s\"", ATTR_NAME, _name ); 
 			query.addANDConstraint( buf.Value() );
@@ -1195,13 +1301,8 @@ Daemon::getCmInfo( const char* subsys )
 {
 	MyString buf;
 	char* host = NULL;
-	char* tmp;
-	struct in_addr sin_addr;
 
-	if( _subsys ) {
-		delete [] _subsys;
-	}
-	_subsys = strnewp( subsys );
+	setSubsystem( subsys );
 
 	if( _addr && is_valid_sinful(_addr) ) {
 			// only consider addresses w/ a non-zero port "valid"...
@@ -1243,9 +1344,23 @@ Daemon::getCmInfo( const char* subsys )
 
 
 	if( ! host  || !host[0] ) {
-			// this is just a fancy wrapper for param()...
 		free( host );
-		host = getCmHostFromConfig( subsys );
+		host = NULL;
+
+			// this is just a fancy wrapper for param()...
+		char *hostnames = getCmHostFromConfig( subsys );
+		if(!hostnames) {
+			buf.sprintf("%s address or hostname not specified in config file",
+					 subsys ); 
+			newError( CA_LOCATE_FAILED, buf.Value() );
+			_is_configured = false;
+			return false;
+		}
+
+		daemon_list.initializeFromString(hostnames);
+		daemon_list.rewind();
+		host = strdup(daemon_list.next());
+		free( hostnames );
 	}
 
 	if( ! host || !host[0]) {
@@ -1273,26 +1388,49 @@ Daemon::getCmInfo( const char* subsys )
 		return false;
 	} 
 
-	dprintf( D_HOSTNAME, "Using name \"%s\" to find daemon\n", host ); 
+	bool ret = findCmDaemon( host );
+	free( host );
+	return ret;
+}
+
+
+bool
+Daemon::findCmDaemon( const char* cm_name )
+{
+	char* host = NULL;
+	MyString buf;
+	struct in_addr sin_addr;
+	char* tmp;
+
+	dprintf( D_HOSTNAME, "Using name \"%s\" to find daemon\n", cm_name ); 
+
+	Sinful sinful( cm_name );
+
+	if( !sinful.valid() || !sinful.getHost() ) {
+		dprintf( D_ALWAYS, "Invalid address: %s\n", cm_name );
+		buf.sprintf( "%s address or hostname not specified in config file",
+				 _subsys ); 
+		newError( CA_LOCATE_FAILED, buf.Value() );
+		_is_configured = false;
+		return false;
+	}
 
 		// See if it's already got a port specified in it, or if we
 		// should use the default port for this kind of daemon.
-	_port = getPortFromAddr( host );
+	_port = sinful.getPortNum();
 	if( _port < 0 ) {
 		_port = getDefaultPort();
+		sinful.setPort( _port );
 		dprintf( D_HOSTNAME, "Port not specified, using default (%d)\n",
 				 _port ); 
 	} else {
 		dprintf( D_HOSTNAME, "Port %d specified in name\n", _port );
 	}
-	if( _port == 0 && readAddressFile(subsys) ) {
+	if( _port == 0 && readAddressFile(_subsys) ) {
 		dprintf( D_HOSTNAME, "Port 0 specified in name, "
 				 "IP/port found in address file\n" );
 		New_name( strnewp(my_full_hostname()) );
 		New_full_hostname( strnewp(my_full_hostname()) );
-		if( host ) {
-			free( host );
-		}
 		return true;
 	}
 
@@ -1300,7 +1438,7 @@ Daemon::getCmInfo( const char* subsys )
 		// file, so we should store the string we used (as is) in
 		// _name, so that we can get to it later if we need it.
 	if( ! _name ) {
-		New_name( strnewp(host) );
+		New_name( strnewp(cm_name) );
 	}
 
 		// Now that we've got the port, grab the hostname for the rest
@@ -1309,16 +1447,14 @@ Daemon::getCmInfo( const char* subsys )
 		// (which we've already got stashed in _name if we need it),
 		// and finally reset host to point to the host for the rest of
 		// this function.
-	tmp = getHostFromAddr( host );
-	free( host );
-	host = tmp;
-	tmp = NULL;
-
+	if( sinful.getHost() ) {
+		host = strdup( sinful.getHost() );
+	}
 
 
 	if ( !host ) {
 		buf.sprintf( "%s address or hostname not specified in config file",
-				 subsys ); 
+				 _subsys ); 
 		newError( CA_LOCATE_FAILED, buf.Value() );
 		_is_configured = false;
 		return false;
@@ -1326,8 +1462,7 @@ Daemon::getCmInfo( const char* subsys )
 
 
 	if( is_ipaddr(host, &sin_addr) ) {
-		buf.sprintf( "<%s:%d>", host, _port );
-		New_addr( strnewp(buf.Value() ) );
+		New_addr( strnewp( sinful.getSinful() ) );
 		dprintf( D_HOSTNAME, "Host info \"%s\" is an IP address\n", host );
 	} else {
 			// We were given a hostname, not an address.
@@ -1347,9 +1482,10 @@ Daemon::getCmInfo( const char* subsys )
 
 			return false;
 		}
-		buf.sprintf( "<%s:%d>", inet_ntoa(sin_addr), _port );
-		dprintf( D_HOSTNAME, "Found IP address and port %s\n", buf.Value() );
-		New_addr( strnewp(buf.Value() ) );
+		sinful.setHost( inet_ntoa(sin_addr) );
+		dprintf( D_HOSTNAME, "Found IP address and port %s\n",
+				 sinful.getSinful() ? sinful.getSinful() : "NULL" );
+		New_addr( strnewp( sinful.getSinful() ) );
 		New_full_hostname( tmp );
 	}
 
@@ -1384,6 +1520,14 @@ Daemon::initHostname( void )
 		locate();
 	}
 
+		// check again if we already have the info
+	if( _full_hostname ) {
+		if( !_hostname ) {
+			return initHostnameFromFull();
+		}
+		return true;
+	}
+
 	if( ! _addr ) {
 			// this is bad...
 		return false;
@@ -1664,6 +1808,14 @@ Daemon::readLocalClassAd( const char* subsys )
 	return getInfoFromAd( smart_ad_ptr );
 }
 
+bool
+Daemon::hasUDPCommandPort()
+{
+	if( !_tried_locate ) {
+		locate();
+	}
+	return m_has_udp_command_port;
+}
 
 bool 
 Daemon::getInfoFromAd( const ClassAd* ad )
@@ -1671,7 +1823,6 @@ Daemon::getInfoFromAd( const ClassAd* ad )
 	MyString buf = "";
 	MyString buf2 = "";
 	MyString addr_attr_name = "";
-	char *our_network_name = NULL;
 		// TODO Which attributes should trigger a failure if we don't find
 		// them in the ad? Just _addr?
 	bool ret_val = true;
@@ -1681,54 +1832,22 @@ Daemon::getInfoFromAd( const ClassAd* ad )
 		// error messages if we fail  to find the other attributes.
 	initStringFromAd( ad, ATTR_NAME, &_name );
 
-		// Search for the daemon's address. This can be in one of
-		// several attributes in the classad. Search through the possible
-		// attributes until we find a match.
-		// In the future, _addr may become a list of addresses. In that
-		// case, we'd want to include all possible matches in the list.
-	if ( ad->LookupString( ATTR_PRIVATE_NETWORK_IP_ADDR, buf ) &&
-		 ad->LookupString( ATTR_PRIVATE_NETWORK_NAME, buf2 ) &&
-		 ( our_network_name = param( "PRIVATE_NETWORK_NAME" ) ) ) {
-
-		StringList addr_list( buf.Value(), ", " );
-		StringList network_list( buf2.Value(), "," );
-
-		addr_list.rewind();
-		network_list.rewind();
-		if ( !addr_list.isEmpty() && !network_list.isEmpty() &&
-			 strcmp( network_list.next(), our_network_name ) == 0 ) {
-
-			New_addr( strnewp( addr_list.next() ) );
-			found_addr = true;
-			addr_attr_name = ATTR_PRIVATE_NETWORK_IP_ADDR;
-		}
-	}
-	free( our_network_name );
-
-	if ( !found_addr && ad->LookupString( ATTR_PUBLIC_NETWORK_IP_ADDR, buf ) ) {
-		StringList addr_list( buf.Value(), ", " );
-
-		addr_list.rewind();
-		if ( !addr_list.isEmpty() ) {
-			New_addr( strnewp( addr_list.next() ) );
-			found_addr = true;
-			addr_attr_name = ATTR_PUBLIC_NETWORK_IP_ADDR;
-		}
-	}
-
-	if ( !found_addr ) {
 		// construct the IP_ADDR attribute
-		buf.sprintf( "%sIpAddr", _subsys );
-		if ( ad->LookupString( buf.Value(), buf2 ) ) {
-			New_addr( strnewp( buf2.Value() ) );
-			found_addr = true;
-			addr_attr_name = buf;
-		}
+	buf.sprintf( "%sIpAddr", _subsys );
+	if ( ad->LookupString( buf.Value(), buf2 ) ) {
+		New_addr( strnewp( buf2.Value() ) );
+		found_addr = true;
+		addr_attr_name = buf;
+	}
+	else if ( ad->LookupString( ATTR_MY_ADDRESS, buf2 ) ) {
+		New_addr( strnewp( buf2.Value() ) );
+		found_addr = true;
+		addr_attr_name = ATTR_MY_ADDRESS;
 	}
 
 	if ( found_addr ) {
 		dprintf( D_HOSTNAME, "Found %s in ClassAd, using \"%s\"\n",
-				 addr_attr_name.Value(), _addr );
+				 addr_attr_name.Value(), _addr);
 		_tried_locate = true;
 	} else {
 		dprintf( D_ALWAYS, "Can't find address in classad for %s %s\n",
@@ -1740,7 +1859,6 @@ Daemon::getInfoFromAd( const ClassAd* ad )
 		ret_val = false;
 	}
 
-
 	if( initStringFromAd( ad, ATTR_VERSION, &_version ) ) {
 		_tried_init_version = true;
 	} else {
@@ -1825,14 +1943,73 @@ Daemon::New_hostname( char* str )
 }
 
 
-char*
+void
 Daemon::New_addr( char* str )
 {
 	if( _addr ) {
 		delete [] _addr;
 	} 
 	_addr = str;
-	return str;
+
+	if( _addr ) {
+		Sinful sinful(_addr);
+		char const *priv_net = sinful.getPrivateNetworkName();
+		if( priv_net ) {
+			bool using_private = false;
+			char *our_network_name = param( "PRIVATE_NETWORK_NAME" );
+			if( our_network_name ) {
+				if( strcmp( our_network_name, priv_net ) == 0 ) {
+					char const *priv_addr = sinful.getPrivateAddr();
+					dprintf( D_HOSTNAME, "Private network name matched.\n");
+					using_private = true;
+					if( priv_addr ) {
+						// replace address with private address
+						MyString buf;
+						if( *priv_addr != '<' ) {
+							buf.sprintf("<%s>",priv_addr);
+							priv_addr = buf.Value();
+						}
+						delete [] _addr;
+						_addr = strnewp( priv_addr );
+						sinful = Sinful( _addr );
+					}
+					else {
+						// no private address was specified, so use public
+						// address with CCB disabled
+						sinful.setCCBContact(NULL);
+						delete [] _addr;
+						_addr = strnewp( sinful.getSinful() );
+					}
+				}
+				free( our_network_name );
+			}
+			if( !using_private ) {
+				// Remove junk from address that we don't care about so
+				// it is not so noisy in logs and such.
+				sinful.setPrivateAddr(NULL);
+				sinful.setPrivateNetworkName(NULL);
+				delete [] _addr;
+				_addr = strnewp( sinful.getSinful() );
+				dprintf( D_HOSTNAME, "Private network name not matched.\n");
+			}
+		}
+
+		if( sinful.getCCBContact() ) {
+			// CCB cannot handle UDP, so pretend this daemon has no
+			// UDP port.
+			m_has_udp_command_port = false;
+		}
+		if( sinful.getSharedPortID() ) {
+			// SharedPort does not handle UDP
+			m_has_udp_command_port = false;
+		}
+		if( sinful.noUDP() ) {
+			// This address explicitly specifies that UDP is not supported
+			m_has_udp_command_port = false;
+		}
+	}
+
+	return;
 }
 
 char*
@@ -1890,7 +2067,13 @@ Daemon::checkAddr( void )
 			// _error will already be set appropriately
 		return false;
 	}
-	if( _port == 0 ) {
+	if( _port == 0 && Sinful(_addr).getSharedPortID()) {
+			// This is an address with a shared port id but no
+			// SharedPortServer address, so it is only good for
+			// local connections on the same machine.
+		return true;
+	}
+	else if( _port == 0 ) {
 			// if we didn't *just* try locating, we should try again,
 			// in case the address file for the thing we're trying to
 			// talk to has now been written.
@@ -2019,7 +2202,7 @@ Daemon::getTimeOffset( long &offset )
 		//
 	ReliSock reli_sock;
 	reli_sock.timeout( 30 ); // I'm following what everbody else does
-	if( ! reli_sock.connect( this->_addr ) ) {
+	if( ! connectSock(&reli_sock) ) {
 		dprintf( D_FULLDEBUG, "Daemon::getTimeOffset() failed to connect "
 		     				  "to remote daemon at '%s'\n",
 		     				  this->_addr );
@@ -2068,7 +2251,7 @@ Daemon::getTimeOffsetRange( long &min_range, long &max_range )
 		//
 	ReliSock reli_sock;
 	reli_sock.timeout( 30 ); // I'm following what everbody else does
-	if( ! reli_sock.connect( this->_addr ) ) {
+	if( ! connectSock(&reli_sock) ) {
 		dprintf( D_FULLDEBUG, "Daemon::getTimeOffsetRange() failed to connect "
 		     				  "to remote daemon at '%s'\n",
 		     				  this->_addr );
