@@ -146,7 +146,7 @@ public:
 			a problem and the error() string above is set, this result
 			code will specify the type of error encountered.  The
 			CAResult enum and some helper functions to convert it
-			to/from strings are in condor_c++_util/command_string.[Ch]
+			to/from strings are in condor_utils/command_string.[Ch]
 		*/
 	CAResult errorCode( void ) { return _error_code; }
 
@@ -273,30 +273,55 @@ public:
 		  Callers can optionally specify a timeout to use for the
 		  connect().  If there was a failure in connect(), we delete
 		  the object and return NULL.
-		  @param sec Number of seconds for the timeout on connect().
+		  @param timeout Number of seconds for the timeout on connect().
+		  @param deadline Time at which to give up (0 if never).
 		  @return A new ReliSock object connected to the daemon.  
 		  */
-	ReliSock* reliSock( int sec = 0, CondorError* errstack = 0,
-	                    bool non_blocking = false,
+	ReliSock* reliSock( int timeout = 0, time_t deadline = 0,
+						CondorError* errstack = 0, bool non_blocking = false,
 						bool ignore_timeout_multiplier = false );
 
 		/**	Create a new SafeSock object, connected to the daemon.
 		  Callers can optionally specify a timeout to use for the
 		  connect().  If there was a failure in connect(), we delete 
 		  the object and return NULL.
-		  @param sec Number of seconds for the timeout on connect().
+		  @param timeout Number of seconds for the timeout on connect().
+		  @param deadline Time at which to give up (0 if never).
 		  @return A new SafeSock object connected to the daemon.  
 		  */
-	SafeSock* safeSock( int sec = 0, CondorError* errstack = 0,
-	                    bool non_blocking = false );
+	SafeSock* safeSock( int timeout = 0, time_t deadline = 0,
+						CondorError* errstack = 0, bool non_blocking = false );
+
+		/**	Create a new Sock object connected to the daemon.
+		  Callers can optionally specify a timeout to use for the
+		  connect().  If there was a failure in connect(), we delete 
+		  the object and return NULL.
+		  @param timeout Number of seconds for the timeout on connect().
+		  @param deadline Time at which to give up (0 if never).
+		  @return A new Sock object connected to the daemon.  
+		  */
+	Sock *makeConnectedSocket( Stream::stream_type st = Stream::reli_sock,
+							   int timeout = 0, time_t deadline = 0,
+							   CondorError* errstack = NULL,
+							   bool non_blocking = false );
+
+		/**	Connects a socket to the daemon.
+		  Callers can optionally specify a timeout to use for the
+		  connect().  If there was a failure in connect(), we delete 
+		  the object and return NULL.
+		  @param sec Number of seconds for the timeout on connect().
+		             (If 0, then uses timeout already set on socket, if any.)
+		  @return true if connection attempt successful
+		  */
+	bool connectSock(Sock *sock, int sec=0, CondorError* errstack=NULL, bool non_blocking=false, bool ignore_timeout_multiplier=false );
 
 public:
 		/** Send the given command to the daemon.  The caller gives
 		  the command they want to send, the type of Sock they
 		  want to use to send it over, and an optional timeout.  
 		  We then instantiate a new Sock of the right type and
-		  timeout, send the command, and finally, the eom().  The Sock
-		  is then destroyed.
+		  timeout, send the command, and finally, the end_of_message().  
+		  The Sock is then destroyed.
 		  @param cmd The command you want to send.
 		  @param st The type of the Sock you want to use.
 		  @param sec The timeout you want to use on your Sock.
@@ -312,8 +337,8 @@ public:
 		  want us to use to send it over, and an optional timeout.
 		  This method will then put the desired timeout on that sock,
 		  place it in encode() mode, send the command, and finally,
-		  the eom().  The sock is otherwise left alone (i.e. not
-		  destroyed)
+		  the end_of_message().  The sock is otherwise left alone 
+		  (i.e. not destroyed)
 		  @param cmd The command you want to send.
 		  @param sock The Sock you want to use.
 		  @param sec The timeout you want to use on your Sock.
@@ -447,6 +472,14 @@ public:
 		 */
 	void sendBlockingMsg( classy_counted_ptr<DCMsg> msg );
 
+		/**
+		 * returns true if target daemon has a UDP command port.
+		 * Currently, we assume the answer is "yes" unless the target
+		 * daemon is being accessed via CCB or a shared port.
+		 * Eventually, we could be smarter.
+		 */
+	bool hasUDPCommandPort();
+
 		/**
 		 * Contact another daemon and initiate the time offset range 
 		 * determination logic. We create a socket connection, pass the
@@ -475,6 +508,24 @@ public:
 		 **/
 	bool getTimeOffsetRange( long &min_range, long &max_range );
 
+		/**
+		 * Set the name of the subsystem
+		 *
+		 * @param subsys - The subsystem string for this daemon
+		 **/
+	bool setSubsystem( const char* subsys );
+
+		/*
+		 * Interate to the next CM daemon in a case where
+		 * more than 1 central manager is configured
+		 **/
+	bool nextValidCm();
+
+		/*
+		 * Reset the list of CM daemons to the first one
+		 **/
+	void rewindCmList();
+
 protected:
 	// Data members
 
@@ -482,6 +533,7 @@ protected:
 	char* _hostname;
 	char* _full_hostname;
 	char* _addr;
+	bool m_has_udp_command_port;
 	char* _version;
 	char* _platform;
 	char* _pool;
@@ -498,6 +550,8 @@ protected:
 	bool _is_configured; 
 	ClassAd *m_daemon_ad_ptr;
 	SecMan _sec_man;
+	StringList daemon_list;
+
 
 
 		// //////////////////////////////////////////////////////////
@@ -519,21 +573,17 @@ protected:
 		  This does all the real work of finding the right address,
 		  port, ip_addr, etc.  We check for address files, and query
 		  the appropriate collector if that doesn't work.
-		  @param subsys The subsystem string for this daemon
 		  @param adtype The type of ClassAd we'll query.
 		  @parma query_collector Whether to query collector if all else fails
 		  */
-	bool getDaemonInfo( const char* subsys, AdTypes adtype, bool query_collector = true );
+	bool getDaemonInfo( AdTypes adtype, bool query_collector = true );
 
 		/** Helper for central manager daemons (collector and
 		  negotiator).  These are a special case since they have
 		  well-known ports, instead of needing to query the central
-		  manager (of course).   Also, we have to deal with the fact
-		  that the CM might have multiple network interfaces, and we
-		  need to be sure to connect to the right one.  We return
-		  success or failure on whether we found any of the parameters
-		  we were looking for that describe where the CM is.  This is
-		  useful when we're trying to find the condor_view collector,
+		  manager (of course).  Uses findCmDaemon to determine the
+		  parameters for central manager daemons.  This is useful
+		  when we're trying to find the condor_view collector,
 		  b/c if we can't find condor_view-specific entries, we fall
 		  back and try to just find the default collector.  
 		  @param subsys The subsystem string for this daemon
@@ -541,6 +591,16 @@ protected:
 		  */
 	bool getCmInfo( const char* subsys );
 
+		/** Deal with the fact that the CM might have multiple
+		  network interfaces, and we need to be sure to connect
+		  to the right one.  We return success or failure on whether
+		  we found any of the parameters we were looking for that
+		  describe where the CM is.
+		  @param name The hostname of a central manager
+		  @return Whether all the parameters for the CM were found
+		  */
+	bool findCmDaemon( const char* name );
+
 
 		/** Helper to initialize the hostname if we don't have it
 			already, but we do have an IP address.  Usually, when we
@@ -644,7 +704,7 @@ protected:
 	char* New_name( char* );
 	char* New_version( char* );
 	char* New_platform( char* );
-	char* New_addr( char* );
+	void New_addr( char* );
 	char* New_pool( char* );
 
 		/**
@@ -726,7 +786,7 @@ protected:
 };
 
 // Prototype to get sinful string.
-char *global_dc_sinful( void );
+char const *global_dc_sinful( void );
 
 /** Helper to get the *_HOST or *_IP_ADDR param for the appropriate
 	subsystem.  It just returns whatever param() would.  So, if it's
