@@ -39,7 +39,9 @@ static const char* daemon_names[] = {
 	"stork",
 	"quill",
 	"transferd",
-	"lease_manager"
+	"lease_manager",
+	"had",
+	"generic"
 };
 
 extern "C" {
@@ -59,7 +61,7 @@ stringToDaemonType( char* name )
 {
 	int i;
 	for( i=0; i<_dt_threshold_; i++ ) {
-		if( !stricmp(daemon_names[i], name) ) {
+		if( !strcasecmp(daemon_names[i], name) ) {
 			return (daemon_t)i;
 		}
 	}
