@@ -28,8 +28,8 @@ enum daemon_t { DT_NONE, DT_ANY,  DT_MASTER, DT_SCHEDD, DT_STARTD,
 				DT_COLLECTOR, DT_NEGOTIATOR, DT_KBDD, 
 				DT_DAGMAN, DT_VIEW_COLLECTOR, DT_CLUSTER,  
 				DT_SHADOW, DT_STARTER, DT_CREDD, DT_STORK, DT_QUILL,
-				DT_TRANSFERD, DT_LEASE_MANAGER,
-				_dt_threshold_ };
+				DT_TRANSFERD, DT_LEASE_MANAGER, DT_HAD,
+				DT_GENERIC, _dt_threshold_ };
 
 #ifdef __cplusplus
 extern "C" {
