@@ -189,6 +189,7 @@ DCCollector::reconfig( void )
 		use_tcp = false;
 		break;
 	case CONFIG:
+		use_tcp = false;
 		tmp = param( "TCP_UPDATE_COLLECTORS" );
 		if( tmp ) {
 			tcp_collectors.initializeFromString( tmp );
@@ -200,17 +201,9 @@ DCCollector::reconfig( void )
 				break;
 			}
 		}
-		tmp = param( "UPDATE_COLLECTOR_WITH_TCP" );
-		if( ! tmp ) {
-			tmp = param( "UPDATE_COLLECTORS_WITH_TCP" );
-		}		
-		if( tmp ) {
-			if( tmp[0] == 't' || tmp[0] == 'T' || 
-				tmp[0] == 'y' || tmp[0] == 'Y' )
-			{ 
-				use_tcp = true;
-			}
-			free( tmp );
+		use_tcp = param_boolean( "UPDATE_COLLECTOR_WITH_TCP", use_tcp );
+		if( !hasUDPCommandPort() ) {
+			use_tcp = true;
 		}
 		break;
 	}
@@ -223,6 +216,7 @@ DCCollector::reconfig( void )
 		// Set an upper bound of one hour for the collector to be blacklisted.
 	int avoid_time = param_integer("DEAD_COLLECTOR_MAX_AVOIDANCE_TIME",3600);
 	blacklisted.setMaxInterval(avoid_time);
+	blacklisted.setInitialInterval(0);
 
 	parseTCPInfo();
 	initDestinationStrings();
@@ -291,47 +285,27 @@ DCCollector::sendUpdate( int cmd, ClassAd* ad1, ClassAd* ad2, bool nonblocking )
 	}
 
 	// Add start time & seq # to the ads before we publish 'em
-	char	buf [80];
-	sprintf( buf, "%s=%ld", ATTR_DAEMON_START_TIME, (long) startTime );
 	if ( ad1 ) {
-		ad1->InsertOrUpdate( buf );
+		ad1->Assign(ATTR_DAEMON_START_TIME,(long)startTime);
 	}
 	if ( ad2 ) {
-		ad2->InsertOrUpdate( buf );
+		ad2->Assign(ATTR_DAEMON_START_TIME,(long)startTime);
 	}
 	if ( ad1 ) {
-		int	seq = adSeqMan->getSequence( ad1 );
-		sprintf( buf, "%s=%u", ATTR_UPDATE_SEQUENCE_NUMBER, seq );
-		ad1->InsertOrUpdate( buf );
+		unsigned seq = adSeqMan->getSequence( ad1 );
+		ad1->Assign(ATTR_UPDATE_SEQUENCE_NUMBER,seq);
 	}
 	if ( ad2 ) {
-		int	seq = adSeqMan->getSequence( ad2 );
-		sprintf( buf, "%s=%u", ATTR_UPDATE_SEQUENCE_NUMBER, seq );
-		ad2->InsertOrUpdate( buf );
+		unsigned seq = adSeqMan->getSequence( ad2 );
+		ad2->Assign(ATTR_UPDATE_SEQUENCE_NUMBER,seq);
 	}
 
-	// Insert "my address"
-	char	*tmp = global_dc_sinful( );
-	if ( tmp ) {
-		sprintf( buf, "%s=\"%s\"", ATTR_MY_ADDRESS, tmp );
-		// No need to free tmp -- it's static
-
-		if ( ad1 ) {
-			if ( ad1->LookupString( ATTR_MY_ADDRESS, &tmp ) ) {
-				free( tmp );	// tmp from LookupString does need to be freed
-			} else {
-				ad1->InsertOrUpdate( buf );
-			}
-		}
-		if ( ad2 ) {
-			if ( ad2->LookupString( ATTR_MY_ADDRESS, &tmp ) ) {
-				free( tmp );
-			} else {
-				ad2->InsertOrUpdate( buf );
-			}
-		}
+		// Prior to 7.2.0, the negotiator depended on the startd
+		// supplying matching MyAddress in public and private ads.
+	if ( ad1 && ad2 ) {
+		ad2->CopyAttribute(ATTR_MY_ADDRESS,ad1);
 	}
-	
+
 		// We never want to try sending an update to port 0.  If we're
 		// about to try that, and we're trying to talk to a local
 		// collector, we should try re-reading the address file and
@@ -397,7 +371,7 @@ DCCollector::finishUpdate( DCCollector *self, Sock* sock, ClassAd* ad1, ClassAd*
 			return false;
 		}
 	}
-	if( ! sock->eom() ) {
+	if( ! sock->end_of_message() ) {
 		if(self) {
 			self->newError( CA_COMMUNICATION_ERROR,
 			          "Failed to send EOM to collector" );
@@ -481,7 +455,7 @@ public:
 			if(sock) who = sock->get_sinful_peer();
 			dprintf(D_ALWAYS,"Failed to start non-blocking update to %s.\n",who);
 		}
-		else if(!DCCollector::finishUpdate(ud->dc_collector,sock,ud->ad1,ud->ad2)) {
+		else if(sock && !DCCollector::finishUpdate(ud->dc_collector,sock,ud->ad1,ud->ad2)) {
 			char const *who = "unknown";
 			if(sock) who = sock->get_sinful_peer();
 			dprintf(D_ALWAYS,"Failed to send non-blocking update to %s.\n",who);
