@@ -227,6 +227,20 @@ DCSchedd::vacateJobs( StringList* ids, VacateType vacate_type,
 }
 
 
+ClassAd*
+DCSchedd::clearDirtyAttrs( StringList* ids, CondorError * errstack,
+                                         action_result_type_t result_type )
+{
+	if( ! ids ) {
+		dprintf( D_ALWAYS, "DCSchedd::clearDirtyAttrs: "
+				"list of jobs is NULL, aborting\n" );
+		return NULL;
+	}
+	return actOnJobs( JA_CLEAR_DIRTY_JOB_ATTRS, NULL, ids, NULL, NULL,
+				result_type, false, errstack );
+}
+
+
 bool
 DCSchedd::reschedule()
 {
@@ -237,8 +251,8 @@ bool
 DCSchedd::receiveJobSandbox(const char* constraint, CondorError * errstack, int * numdone /*=0*/)
 {
 	if(numdone) { *numdone = 0; }
-	ExprTree *tree = NULL, *lhs = NULL, *rhs = NULL;
-	char *lhstr, *rhstr;
+	ExprTree *tree = NULL;
+	const char *lhstr;
 	int reply;
 	int i;
 	ReliSock rsock;
@@ -315,7 +329,7 @@ DCSchedd::receiveJobSandbox(const char* constraint, CondorError * errstack, int
 	}
 	free( nc_constraint );
 
-	if ( !rsock.eom() ) {
+	if ( !rsock.end_of_message() ) {
 		MyString errmsg;
 		errmsg.sprintf(
 			"Can't send initial message (version + constraint) to schedd (%s)",
@@ -350,7 +364,7 @@ DCSchedd::receiveJobSandbox(const char* constraint, CondorError * errstack, int
 		return false;
 	}
 
-	rsock.eom();
+	rsock.end_of_message();
 
 	dprintf(D_FULLDEBUG,"DCSchedd:receiveJobSandbox: "
 		"%d jobs matched my constraint (%s)\n",
@@ -377,39 +391,20 @@ DCSchedd::receiveJobSandbox(const char* constraint, CondorError * errstack, int
 			return false;
 		}
 
-		rsock.eom();
+		rsock.end_of_message();
 
 			// translate the job ad by replacing the 
 			// saved SUBMIT_ attributes
 		job.ResetExpr();
-		while( (tree = job.NextExpr()) ) {
-			lhstr = NULL;
-			if( (lhs = tree->LArg()) ) { 
-				lhs->PrintToNewStr (&lhstr); 
-			}
+		while( job.NextExpr(lhstr, tree) ) {
 			if ( lhstr && strncasecmp("SUBMIT_",lhstr,7)==0 ) {
 					// this attr name starts with SUBMIT_
 					// compute new lhs (strip off the SUBMIT_)
-				char *new_attr_name = strchr(lhstr,'_');
+				const char *new_attr_name = strchr(lhstr,'_');
 				ASSERT(new_attr_name);
 				new_attr_name++;
-					// compute new rhs (just use the same)
-				rhstr = NULL;
-				if( (rhs = tree->RArg()) ) { 
-					rhs->PrintToNewStr (&rhstr); 
-				}
 					// insert attribute
-				if(rhstr) {
-					MyString newattr;
-					newattr += new_attr_name;
-					newattr += "=";
-					newattr += rhstr;
-					job.Insert(newattr.Value());
-					free(rhstr);
-				}
-			}
-			if ( lhstr ) {
-				free(lhstr);
+				job.Insert(new_attr_name, tree->Copy());
 			}
 		}	// while next expr
 
@@ -451,13 +446,13 @@ DCSchedd::receiveJobSandbox(const char* constraint, CondorError * errstack, int
 		}
 	}	
 		
-	rsock.eom();
+	rsock.end_of_message();
 
 	rsock.encode();
 
 	reply = OK;
 	rsock.code(reply);
-	rsock.eom();
+	rsock.end_of_message();
 
 	if(numdone) { *numdone = JobAdsArrayLen; }
 
@@ -520,7 +515,7 @@ DCSchedd::register_transferd(MyString sinful, MyString id, int timeout,
 	//	ATTR_TREQ_TD_SINFUL
 	//	ATTR_TREQ_TD_ID
 	regad.put(*rsock);
-	rsock->eom();
+	rsock->end_of_message();
 
 	// Get the response from the schedd.
 	rsock->decode();
@@ -534,7 +529,7 @@ DCSchedd::register_transferd(MyString sinful, MyString id, int timeout,
 	//	ATTR_TREQ_INVALID_REQUEST
 	//	ATTR_TREQ_INVALID_REASON
 	respad.initFromStream(*rsock);
-	rsock->eom();
+	rsock->end_of_message();
 
 	respad.LookupInteger(ATTR_TREQ_INVALID_REQUEST, invalid_request);
 
@@ -732,7 +727,7 @@ DCSchedd::requestSandboxLocation(ClassAd *reqad, ClassAd *respad,
 				"Can't send reqad to the schedd\n");
 		return false;
 	}
-	rsock.eom();
+	rsock.end_of_message();
 
 	rsock.decode();
 
@@ -758,7 +753,7 @@ DCSchedd::requestSandboxLocation(ClassAd *reqad, ClassAd *respad,
 			"submission.\n");
 		return false;
 	}
-	rsock.eom();
+	rsock.end_of_message();
 
 	status_ad.LookupInteger(ATTR_TREQ_WILL_BLOCK, will_block);
 
@@ -796,7 +791,7 @@ DCSchedd::requestSandboxLocation(ClassAd *reqad, ClassAd *respad,
 				"Can't receive respond ad from the schedd\n");
 		return false;
 	}
-	rsock.eom();
+	rsock.end_of_message();
 
 	return true;
 }
@@ -887,7 +882,7 @@ DCSchedd::spoolJobFiles(int JobAdsArrayLen, ClassAd* JobAdsArray[], CondorError
 		return false;
 	}
 
-	if( !rsock.eom() ) {
+	if( !rsock.end_of_message() ) {
 		MyString errmsg;
 		errmsg.sprintf(
 			"Can't send initial message (version + count) to schedd (%s)",
@@ -920,7 +915,7 @@ DCSchedd::spoolJobFiles(int JobAdsArrayLen, ClassAd* JobAdsArray[], CondorError
 		rsock.code(jobid);
 	}
 
-	if( !rsock.eom() ) {
+	if( !rsock.end_of_message() ) {
 		MyString errmsg;
 		errmsg.sprintf("Failed while sending job ids to schedd (%s)", _addr);
 
@@ -976,13 +971,13 @@ DCSchedd::spoolJobFiles(int JobAdsArrayLen, ClassAd* JobAdsArray[], CondorError
 	}	
 		
 		
-	rsock.eom();
+	rsock.end_of_message();
 
 	rsock.decode();
 
 	reply = 0;
 	rsock.code(reply);
-	rsock.eom();
+	rsock.end_of_message();
 
 	if ( reply == 1 ) 
 		return true;
@@ -1032,7 +1027,7 @@ DCSchedd::updateGSIcredential(const int cluster, const int proc,
 	PROC_ID jobid;
 	jobid.cluster = cluster;
 	jobid.proc = proc;	
-	if ( !rsock.code(jobid) || !rsock.eom() ) {
+	if ( !rsock.code(jobid) || !rsock.end_of_message() ) {
 		dprintf(D_ALWAYS,"DCSchedd:updateGSIcredential: "
 				"Can't send jobid to the schedd\n");
 		return false;
@@ -1052,7 +1047,7 @@ DCSchedd::updateGSIcredential(const int cluster, const int proc,
 	rsock.decode();
 	reply = 0;
 	rsock.code(reply);
-	rsock.eom();
+	rsock.end_of_message();
 
 	if ( reply == 1 ) 
 		return true;
@@ -1063,6 +1058,8 @@ DCSchedd::updateGSIcredential(const int cluster, const int proc,
 bool 
 DCSchedd::delegateGSIcredential(const int cluster, const int proc, 
 								const char* path_to_proxy_file,
+								time_t expiration_time,
+								time_t *result_expiration_time,
 								CondorError * errstack)
 {
 	int reply;
@@ -1102,7 +1099,7 @@ DCSchedd::delegateGSIcredential(const int cluster, const int proc,
 	PROC_ID jobid;
 	jobid.cluster = cluster;
 	jobid.proc = proc;	
-	if ( !rsock.code(jobid) || !rsock.eom() ) {
+	if ( !rsock.code(jobid) || !rsock.end_of_message() ) {
 		dprintf(D_ALWAYS,"DCSchedd::delegateGSIcredential: "
 				"Can't send jobid to the schedd\n");
 		return false;
@@ -1110,7 +1107,7 @@ DCSchedd::delegateGSIcredential(const int cluster, const int proc,
 
 		// Delegate the gsi proxy
 	filesize_t file_size = 0;	// will receive the size of the file
-	if ( rsock.put_x509_delegation(&file_size,path_to_proxy_file) < 0 ) {
+	if ( rsock.put_x509_delegation(&file_size,path_to_proxy_file,expiration_time,result_expiration_time) < 0 ) {
 		dprintf(D_ALWAYS,
 			"DCSchedd::delegateGSIcredential "
 			"failed to send proxy file %s\n",
@@ -1122,7 +1119,7 @@ DCSchedd::delegateGSIcredential(const int cluster, const int proc,
 	rsock.decode();
 	reply = 0;
 	rsock.code(reply);
-	rsock.eom();
+	rsock.end_of_message();
 
 	if ( reply == 1 ) 
 		return true;
@@ -1234,7 +1231,7 @@ DCSchedd::actOnJobs( JobAction action,
 	}
 
 		// Now, put the command classad on the wire
-	if( ! (cmd_ad.put(rsock) && rsock.eom()) ) {
+	if( ! (cmd_ad.put(rsock) && rsock.end_of_message()) ) {
 		dprintf( D_ALWAYS, "DCSchedd:actOnJobs: Can't send classad\n" );
 		return NULL;
 	}
@@ -1245,7 +1242,7 @@ DCSchedd::actOnJobs( JobAction action,
 		// and it should abort its transaction
 	rsock.decode();
 	ClassAd* result_ad = new ClassAd();
-	if( ! (result_ad->initFromStream(rsock) && rsock.eom()) ) {
+	if( ! (result_ad->initFromStream(rsock) && rsock.end_of_message()) ) {
 		dprintf( D_ALWAYS, "DCSchedd:actOnJobs: "
 				 "Can't read response ad from %s\n", _addr );
 		delete( result_ad );
@@ -1267,7 +1264,7 @@ DCSchedd::actOnJobs( JobAction action,
 		// Tell the schedd we're still here and ready to go
 	rsock.encode();
 	int answer = OK;
-	if( ! (rsock.code(answer) && rsock.eom()) ) {
+	if( ! (rsock.code(answer) && rsock.end_of_message()) ) {
 		dprintf( D_ALWAYS, "DCSchedd:actOnJobs: Can't send reply\n" );
 		delete( result_ad );
 		return NULL;
@@ -1276,7 +1273,7 @@ DCSchedd::actOnJobs( JobAction action,
 		// finally, make sure the schedd didn't blow up trying to
 		// commit these changes to the job queue...
 	rsock.decode();
-	if( ! (rsock.code(reply) && rsock.eom()) ) {
+	if( ! (rsock.code(reply) && rsock.end_of_message()) ) {
 		dprintf( D_ALWAYS, "DCSchedd:actOnJobs: "
 				 "Can't read confirmation from %s\n", _addr );
 		delete( result_ad );
@@ -1303,6 +1300,7 @@ JobActionResults::JobActionResults( action_result_type_t res_type )
 	ar_bad_status = 0;
 	ar_already_done = 0;
 	ar_error = 0;
+	action = JA_ERROR;
 }
 
 
@@ -1579,3 +1577,157 @@ JobActionResults::getResultString( PROC_ID job_id, char** str )
 	return rval;
 }
 
+bool DCSchedd::getJobConnectInfo(
+	PROC_ID jobid,
+	int subproc,
+	char const *session_info,
+	int timeout,
+	CondorError *errstack,
+	MyString &starter_addr,
+	MyString &starter_claim_id,
+	MyString &starter_version,
+	MyString &slot_name,
+	MyString &error_msg,
+	bool &retry_is_sensible)
+{
+	ClassAd input;
+	ClassAd output;
+
+	input.Assign(ATTR_CLUSTER_ID,jobid.cluster);
+	input.Assign(ATTR_PROC_ID,jobid.proc);
+	if( subproc != -1 ) {
+		input.Assign(ATTR_SUB_PROC_ID,subproc);
+	}
+	input.Assign(ATTR_SESSION_INFO,session_info);
+
+	ReliSock sock;
+	if( !connectSock(&sock,timeout,errstack) ) {
+		error_msg = "Failed to connect to schedd";
+		dprintf( D_ALWAYS, "%s\n",error_msg.Value());
+		return false;
+	}
+
+	if( !startCommand(GET_JOB_CONNECT_INFO, &sock, timeout, errstack) ) {
+		error_msg = "Failed to send GET_JOB_CONNECT_INFO to schedd";
+		dprintf( D_ALWAYS, "%s\n",error_msg.Value());
+		return false;
+	}
+
+	if( !forceAuthentication(&sock, errstack) ) {
+		error_msg = "Failed to authenticate";
+		dprintf( D_ALWAYS, "%s\n",error_msg.Value());
+		return false;
+	}
+
+	sock.encode();
+	if( !input.put(sock) || !sock.end_of_message() ) {
+		error_msg = "Failed to send GET_JOB_CONNECT_INFO to schedd";
+		dprintf( D_ALWAYS, "%s\n",error_msg.Value());
+		return false;
+	}
+
+	sock.decode();
+	if( !output.initFromStream(sock) || !sock.end_of_message() ) {
+		error_msg = "Failed to get response from schedd";
+		dprintf( D_ALWAYS, "%s\n",error_msg.Value());
+		return false;
+	}
+
+	if( DebugFlags & D_FULLDEBUG ) {
+		MyString adstr;
+		output.SetPrivateAttributesInvisible(true);
+		output.sPrint(adstr);
+		output.SetPrivateAttributesInvisible(false);
+		dprintf(D_FULLDEBUG,"Response for GET_JOB_CONNECT_INFO:\n%s\n",
+				adstr.Value());
+	}
+
+	bool result=false;
+	output.LookupBool(ATTR_RESULT,result);
+
+	if( !result ) {
+		output.LookupString(ATTR_ERROR_STRING,error_msg);
+		retry_is_sensible = false;
+		output.LookupBool(ATTR_RETRY,retry_is_sensible);
+	}
+	else {
+		output.LookupString(ATTR_STARTER_IP_ADDR,starter_addr);
+		output.LookupString(ATTR_CLAIM_ID,starter_claim_id);
+		output.LookupString(ATTR_VERSION,starter_version);
+		output.LookupString(ATTR_REMOTE_HOST,slot_name);
+	}
+
+	return result;
+}
+
+bool DCSchedd::recycleShadow( int previous_job_exit_reason, ClassAd **new_job_ad, MyString &error_msg )
+{
+	int timeout = 300;
+	CondorError errstack;
+
+	ReliSock sock;
+	if( !connectSock(&sock,timeout,&errstack) ) {
+		error_msg.sprintf("Failed to connect to schedd: %s",
+						  errstack.getFullText());
+		return false;
+	}
+
+	if( !startCommand(RECYCLE_SHADOW, &sock, timeout, &errstack) ) {
+		error_msg.sprintf("Failed to send RECYCLE_SHADOW to schedd: %s",
+						  errstack.getFullText());
+		return false;
+	}
+
+	if( !forceAuthentication(&sock, &errstack) ) {
+		error_msg.sprintf("Failed to authenticate: %s",
+						  errstack.getFullText());
+		return false;
+	}
+
+	sock.encode();
+	int mypid = getpid();
+	if( !sock.put( mypid ) ||
+		!sock.put( previous_job_exit_reason ) ||
+		!sock.end_of_message() )
+	{
+		error_msg = "Failed to send job exit reason";
+		return false;
+	}
+
+	sock.decode();
+
+	int found_new_job = 0;
+	sock.get( found_new_job );
+
+	if( found_new_job ) {
+		*new_job_ad = new ClassAd();
+		if( !(*new_job_ad)->initFromStream( sock ) ) {
+			error_msg = "Failed to receive new job ClassAd";
+			delete *new_job_ad;
+			*new_job_ad = NULL;
+			return false;
+		}
+	}
+
+	if( !sock.end_of_message() ) {
+		error_msg = "Failed to receive end of message";
+		delete *new_job_ad;
+		*new_job_ad = NULL;
+		return false;
+	}
+
+	if( *new_job_ad ) {
+		sock.encode();
+		int ok=1;
+		if( !sock.put(ok) ||
+			!sock.end_of_message() )
+		{
+			error_msg = "Failed to send ok";
+			delete *new_job_ad;
+			*new_job_ad = NULL;
+			return false;
+		}
+	}
+
+	return true;
+}
