@@ -88,16 +88,16 @@ ClaimStartdMsg::ClaimStartdMsg( char const *the_claim_id, ClassAd const *job_ad,
 }
 
 void
-ClaimStartdMsg::cancelMessage() {
-	dprintf(D_ALWAYS,"Canceling request for claim %s\n", description());
-	DCMsg::cancelMessage();
+ClaimStartdMsg::cancelMessage(char const *reason) {
+	dprintf(D_ALWAYS,"Canceling request for claim %s %s\n", description(),reason ? reason : "");
+	DCMsg::cancelMessage(reason);
 }
 
 bool
 ClaimStartdMsg::writeMsg( DCMessenger * /*messenger*/, Sock *sock ) {
 		// save startd fqu for hole punching
 	m_startd_fqu = sock->getFullyQualifiedUser();
-	m_startd_ip_addr = sock->endpoint_ip_str();
+	m_startd_ip_addr = sock->peer_ip_str();
 
 	if( !sock->put_secret( m_claim_id.Value() ) ||
 	    !m_job_ad.put( *sock ) ||
@@ -107,9 +107,10 @@ ClaimStartdMsg::writeMsg( DCMessenger * /*messenger*/, Sock *sock ) {
 		dprintf(failureDebugLevel(),
 				"Couldn't encode request claim to startd %s\n",
 				description() );
+		sockFailed( sock );
 		return false;
 	}
-		// eom() is done by caller
+		// end_of_message() is done by caller
 
 	return true;
 }
@@ -133,6 +134,7 @@ ClaimStartdMsg::readMsg( DCMessenger * /*messenger*/, Sock *sock ) {
 		dprintf( failureDebugLevel(),
 				 "Response problem from startd when requesting claim %s.\n",
 				 description() );	
+		sockFailed( sock );
 		return false;
 	}
 
@@ -143,14 +145,14 @@ ClaimStartdMsg::readMsg( DCMessenger * /*messenger*/, Sock *sock ) {
 	} else {
 		dprintf( failureDebugLevel(), "Unknown reply from startd when requesting claim %s\n",description());
 	}
-		// eom() is done by caller
+		// end_of_message() is done by caller
 
 	return true;
 }
 
 
 void
-DCStartd::asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *description, char const *scheduler_addr, int alive_interval, int timeout, classy_counted_ptr<DCMsgCallback> cb )
+DCStartd::asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *description, char const *scheduler_addr, int alive_interval, int timeout, int deadline_timeout, classy_counted_ptr<DCMsgCallback> cb )
 {
 	dprintf(D_FULLDEBUG|D_PROTOCOL,"Requesting claim %s\n",description);
 
@@ -170,6 +172,7 @@ DCStartd::asyncRequestOpportunisticClaim( ClassAd const *req_ad, char const *des
 	msg->setSecSessionId(cid.secSessionId());
 
 	msg->setTimeout(timeout);
+	msg->setDeadlineTimeout(deadline_timeout);
 	sendMsg(msg.get());
 }
 
@@ -233,7 +236,7 @@ DCStartd::deactivateClaim( bool graceful, bool *claim_is_closing )
 		newError( CA_COMMUNICATION_ERROR, err.Value() );
 		return false;
 	}
-	if( ! reli_sock.eom() ) {
+	if( ! reli_sock.end_of_message() ) {
 		MyString err = "DCStartd::deactivateClaim: ";
 		err += "Failed to send EOM to the startd";
 		newError( CA_COMMUNICATION_ERROR, err.Value() );
@@ -242,7 +245,7 @@ DCStartd::deactivateClaim( bool graceful, bool *claim_is_closing )
 
 	reli_sock.decode();
 	ClassAd response_ad;
-	if( !response_ad.initFromStream(reli_sock) || !reli_sock.eom() ) {
+	if( !response_ad.initFromStream(reli_sock) || !reli_sock.end_of_message() ) {
 		dprintf( D_FULLDEBUG, "DCStartd::deactivateClaim: failed to read response ad.\n");
 			// The response ad is not critical and is expected to be missing
 			// if the startd is from before 7.0.5.
@@ -298,7 +301,6 @@ DCStartd::activateClaim( ClassAd* job_ad, int starter_version,
 		err += "ACTIVATE_CLAIM";
 		err += " to the startd";
 		newError( CA_COMMUNICATION_ERROR, err.Value() );
-		delete tmp;
 		return CONDOR_ERROR;
 	}
 	if( ! tmp->put_secret(claim_id) ) {
@@ -436,7 +438,6 @@ DCStartd::resumeClaim( ClassAd* reply, int timeout )
 	}
 
 	ClassAd req;
-	char buf[1024]; 
 
 		// Add our own attributes to the request ad we're sending
 	req.Assign( ATTR_COMMAND, getCommandString(CA_RESUME_CLAIM) );
@@ -564,7 +565,7 @@ DCStartd::locateStarter( const char* global_job_id,
 }
 
 int
-DCStartd::delegateX509Proxy( const char* proxy )
+DCStartd::delegateX509Proxy( const char* proxy, time_t expiration_time, time_t *result_expiration_time )
 {
 	dprintf( D_FULLDEBUG, "Entering DCStartd::delegateX509Proxy()\n" );
 
@@ -593,7 +594,6 @@ DCStartd::delegateX509Proxy( const char* proxy )
 		               "DELEGATE_GSI_CRED_STARTD "
 		               "to the startd";
 		newError( CA_COMMUNICATION_ERROR, err.Value() );
-		delete tmp;
 		return CONDOR_ERROR;
 	}
 
@@ -646,7 +646,7 @@ DCStartd::delegateX509Proxy( const char* proxy )
 	int rv;
 	filesize_t dont_care;
 	if( use_delegation ) {
-		rv = tmp->put_x509_delegation( &dont_care, proxy );
+		rv = tmp->put_x509_delegation( &dont_care, proxy, expiration_time, result_expiration_time );
 	}
 	else {
 		dprintf( D_FULLDEBUG,
@@ -736,7 +736,7 @@ DCStartd::vacateClaim( const char* name_vacate )
 		newError( CA_COMMUNICATION_ERROR, err.Value() );
 		return false;
 	}
-	if( ! reli_sock.eom() ) {
+	if( ! reli_sock.end_of_message() ) {
 		MyString err = "DCStartd::vacateClaim: ";
 		err += "Failed to send EOM to the startd";
 		newError( CA_COMMUNICATION_ERROR, err.Value() );
@@ -785,7 +785,7 @@ DCStartd::checkpointJob( const char* name_ckpt )
 		newError( CA_COMMUNICATION_ERROR, err.Value() );
 		return false;
 	}
-	if( ! reli_sock.eom() ) {
+	if( ! reli_sock.end_of_message() ) {
 		MyString err = "DCStartd::checkpointJob: ";
 		err += "Failed to send EOM to the startd";
 		newError( CA_COMMUNICATION_ERROR, err.Value() );
