@@ -46,7 +46,7 @@
 #include "Queue.h"
 #include "MapFile.h"
 #ifdef WIN32
-#include "ntsysinfo.h"
+#include "ntsysinfo.WINDOWS.h"
 #endif
 #include "self_monitor.h"
 //#include "stdsoap2.h"
@@ -56,12 +56,14 @@
 #include "daemon.h"
 #include "daemon_list.h"
 #include "limit.h"
+#include "ccb_listener.h"
+#include "condor_sinful.h"
 
 #include "../condor_procd/proc_family_io.h"
 class ProcFamilyInterface;
 
 #if defined(WIN32)
-#include "pipe.WIN32.h"
+#include "pipe.WINDOWS.h"
 #endif
 
 #define DEBUG_SETTABLE_ATTR_LISTS 0
@@ -71,7 +73,6 @@ template <class Key, class Value> class HashTable; // forward declaration
 static const int KEEP_STREAM = 100;
 static const int CLOSE_STREAM = 101;
 static const int MAX_SOCKS_INHERITED = 4;
-static char* EMPTY_DESCRIP = "<NULL>";
 
 /**
    Magic fd to include in the 'std' array argument to Create_Process()
@@ -156,10 +157,14 @@ typedef enum {
 
 const int DCJOBOPT_SUSPEND_ON_EXEC  = (1<<1);
 const int DCJOBOPT_NO_ENV_INHERIT   = (1<<2);
+const int DCJOBOPT_NEVER_USE_SHARED_PORT   = (1<<3);
+const int DCJOBOPT_NO_UDP           = (1<<4);
 
 #define HAS_DCJOBOPT_SUSPEND_ON_EXEC(mask)  ((mask)&DCJOBOPT_SUSPEND_ON_EXEC)
 #define HAS_DCJOBOPT_NO_ENV_INHERIT(mask)  ((mask)&DCJOBOPT_NO_ENV_INHERIT)
 #define HAS_DCJOBOPT_ENV_INHERIT(mask)  (!(HAS_DCJOBOPT_NO_ENV_INHERIT(mask)))
+#define HAS_DCJOBOPT_NEVER_USE_SHARED_PORT(mask) ((mask)&DCJOBOPT_NEVER_USE_SHARED_PORT)
+#define HAS_DCJOBOPT_NO_UDP(mask) ((mask)&DCJOBOPT_NO_UDP)
 
 // structure to be used as an argument to Create_Process for tracking process
 // families
@@ -220,7 +225,6 @@ int BindAnyCommandPort(ReliSock *rsock, SafeSock *ssock);
 bool InitCommandSockets(int port, ReliSock *rsock, SafeSock *ssock,
 						bool fatal);
 
-
 class DCSignalMsg: public DCMsg {
  public:
 	DCSignalMsg(pid_t pid, int s): DCMsg(DC_RAISESIGNAL)
@@ -323,7 +327,8 @@ class DaemonCore : public Service
                           const char *    handler_descrip,
                           Service *       s                = NULL,
                           DCpermission    perm             = ALLOW,
-                          int             dprintf_flag     = D_COMMAND);
+                          int             dprintf_flag     = D_COMMAND,
+                          bool            force_authentication = false);
     
     /** Not_Yet_Documented
         @param command         Not_Yet_Documented
@@ -341,7 +346,8 @@ class DaemonCore : public Service
                           const char *       handler_descrip,
                           Service *          s,
                           DCpermission       perm             = ALLOW,
-                          int                dprintf_flag     = D_COMMAND);
+                          int                dprintf_flag     = D_COMMAND,
+                          bool               force_authentication = false);
     
     /** Not_Yet_Documented
         @param command Not_Yet_Documented
@@ -361,8 +367,9 @@ class DaemonCore : public Service
         @param pid The pid to ask about.  -1 (Default) means
                    the calling process
         @return A pointer into a <b>static buffer</b>, or NULL on error */
-    char* InfoCommandSinfulString (int pid = -1);
+    char const* InfoCommandSinfulString (int pid = -1);
 
+	void daemonContactInfoChanged();
 
     /** Returns the Sinful String <host:port> of the DaemonCore
 		command socket of this process
@@ -376,7 +383,7 @@ class DaemonCore : public Service
 			   is provided only for passing to other processes on the
 			   same machine as an optimization.
         @return A pointer into a <b>static buffer</b>, or NULL on error */
-	char * InfoCommandSinfulStringMyself(bool usePrivateAddress);
+	char const * InfoCommandSinfulStringMyself(bool usePrivateAddress);
 
 		/**
 		   @return Pointer to a static buffer containing the daemon's
@@ -646,15 +653,20 @@ class DaemonCore : public Service
     */
     int Cancel_Socket ( Stream * insock );
 
+		// Returns true if the given socket is already registered.
+	bool SocketIsRegistered( Stream *sock );
+
 		// Call the registered socket handler for this socket
 		// sock - previously registered socket
 		// default_to_HandleCommand - true if HandleCommand() should be called
 		//                          if there is no other callback function
-		// it will index the next registered socket.
 	void CallSocketHandler( Stream *sock, bool default_to_HandleCommand=false );
 
-	/// Cancel and close all registed sockets.
-	int Cancel_And_Close_All_Sockets(void);
+		// Call the registered command handler.
+		// returns the return code of the handler
+		// if delete_stream is true and the command handler does not return
+		// KEEP_STREAM, the stream is deleted
+	int CallCommandHandler(int req,Stream *stream,bool delete_stream=true);
 
 
 	/**
@@ -743,6 +755,12 @@ class DaemonCore : public Service
 			 bool nonblocking_read = false, bool nonblocking_write = false,
 			 unsigned int psize = 4096);
 
+	/** Create a named pipe
+	*/
+	int Create_Named_Pipe( int *pipe_ends,
+			 bool can_register_read = false, bool can_register_write = false,
+			 bool nonblocking_read = false, bool nonblocking_write = false,
+			 unsigned int psize = 4096, const char* pipe_name = NULL);
 	/** Make DaemonCore aware of an inherited pipe.
 	*/
 	int Inherit_Pipe( int p, bool write, bool can_register, bool nonblocking, int psize = 4096);
@@ -810,40 +828,34 @@ class DaemonCore : public Service
         @param deltawhen       Not_Yet_Documented
         @param event           Not_Yet_Documented
         @param event_descrip   Not_Yet_Documented
-        @param s               Not_Yet_Documented
         @return Not_Yet_Documented
     */
     int Register_Timer (unsigned     deltawhen,
-                        Event        event,
-                        const char * event_descrip, 
-                        Service*     s = NULL);
+                        TimerHandler handler,
+                        const char * event_descrip);
 
 	/** Not_Yet_Documented
         @param deltawhen       Not_Yet_Documented
         @param event           Not_Yet_Documented
         @param event_descrip   Not_Yet_Documented
-        @param s               Not_Yet_Documented
         @return Not_Yet_Documented
     */
     int Register_Timer (unsigned     deltawhen,
-                        Event        event,
+                        TimerHandler handler,
 						Release      release,
-                        const char * event_descrip, 
-                        Service*     s = NULL);
+                        const char * event_descrip);
     
     /** Not_Yet_Documented
         @param deltawhen       Not_Yet_Documented
         @param period          Not_Yet_Documented
         @param event           Not_Yet_Documented
         @param event_descrip   Not_Yet_Documented
-        @param s               Not_Yet_Documented
         @return Not_Yet_Documented
     */
     int Register_Timer (unsigned     deltawhen,
                         unsigned     period,
-                        Event        event,
-                        const char * event_descrip,
-                        Service*     s = NULL);
+                        TimerHandler handler,
+                        const char * event_descrip);
 
     /** Not_Yet_Documented
         @param deltawhen       Not_Yet_Documented
@@ -853,10 +865,19 @@ class DaemonCore : public Service
         @return Not_Yet_Documented
     */
     int Register_Timer (unsigned     deltawhen,
-                        Eventcpp     event,
+                        TimerHandlercpp handler,
                         const char * event_descrip,
                         Service*     s);
 
+	/** Thread-safe Register_Timer().
+		This function locks the big fat daemon-core mutex, so it can
+		be safely called from within a thread, as long as there is no
+		way that thread could already have locked the same mutex.
+	 */
+	int Register_Timer_TS (unsigned deltawhen,
+		TimerHandlercpp handler,
+		const char * event_descrip,
+		Service* s);
     /** Not_Yet_Documented
         @param deltawhen       Not_Yet_Documented
         @param period          Not_Yet_Documented
@@ -867,7 +888,7 @@ class DaemonCore : public Service
     */
     int Register_Timer (unsigned     deltawhen,
                         unsigned     period,
-                        Eventcpp     event,
+                        TimerHandlercpp handler,
                         const char * event_descrip,
                         Service *    s);
 
@@ -878,8 +899,8 @@ class DaemonCore : public Service
         @param s               Service object of which function is a member.
         @return                Timer id or -1 on error
     */
-    int Register_Timer (Timeslice    timeslice,
-                        Eventcpp     event,
+    int Register_Timer (const Timeslice &timeslice,
+                        TimerHandlercpp handler,
                         const char * event_descrip,
                         Service*     s);
 
@@ -893,9 +914,32 @@ class DaemonCore : public Service
         @param id The timer's ID
         @param when   Not_Yet_Documented
         @param period Not_Yet_Documented
-        @return Not_Yet_Documented
+        @return 0 if successful, -1 on failure (timer not found)
     */
     int Reset_Timer ( int id, unsigned when, unsigned period = 0 );
+
+    /** Change a timer's period.  Recompute time to fire next based on this
+		new period and how long this timer has been waiting.
+        @param id The timer's ID
+        @param period New period for this timer.
+        @return 0 on success
+    */
+    int Reset_Timer_Period ( int id, unsigned period );
+
+    /** Change a timer's timeslice settings.
+        @param id The timer's ID
+        @param new_timeslice New timeslice settings to use.
+        @return 0 on success
+    */
+    int ResetTimerTimeslice ( int id, Timeslice const &new_timeslice );
+
+    /** Get a timer's timeslice settings.
+        @param id The timer's ID
+        @param timeslice Object to receive a copy of the timeslice settings.
+        @return false if no timeslice associated with this timer
+    */
+    bool GetTimerTimeslice ( int id, Timeslice &timeslice );
+
 	//@}
 
     /** Not_Yet_Documented
@@ -968,7 +1012,8 @@ class DaemonCore : public Service
                used as argv[0].
         @param priv The priv state to change into right before
                the exec.  Default = no action.
-        @param reaper_id The reaper number to use.  Default = 1.
+        @param reaper_id The reaper number to use.  Default = 1. If a
+			   reaper_id of 0 is used, no reaper callback will be invoked.
         @param want_command_port Well, do you?  Default = TRUE.  If
 			   want_command_port it TRUE, the child process will be
 			   born with a daemonCore command socket on a dynamic port.
@@ -1004,6 +1049,9 @@ class DaemonCore : public Service
                mask if a given bit is set ("HAS_DCJOBOPT_...")
 		@param fd_inherit_list An array of fds which you want the child to
 		       inherit. The last element must be 0.
+        ...
+        @param err_return_msg Returned with error message pertaining to
+               failure inside the method.  Ignored if NULL (default).
         @return On success, returns the child pid.  On failure, returns FALSE.
     */
     int Create_Process (
@@ -1021,7 +1069,10 @@ class DaemonCore : public Service
         int           nice_inc             = 0,
         sigset_t      *sigmask             = NULL,
         int           job_opt_mask         = 0,
-        size_t        *core_hard_limit     = NULL
+        size_t        *core_hard_limit     = NULL,
+		int			  *affinity_mask	   = NULL,
+		char const    *daemon_sock         = NULL,
+        MyString      *err_return_msg      = NULL
         );
 
     //@}
@@ -1039,7 +1090,8 @@ class DaemonCore : public Service
     int Suspend_Family(pid_t);
     int Continue_Family(pid_t);
     int Kill_Family(pid_t);
-
+    int Signal_Process(pid_t,int);
+    
 	/** Used to explicitly initialize our ProcFamilyInterface object.
 	    Calling this is not required - if not called, the object
 	    will be initialized on-demand: the first time Create_Process
@@ -1289,7 +1341,7 @@ class DaemonCore : public Service
     SelfMonitorData monitor_data;
 
 	char 	*localAdFile;
-	void	UpdateLocalAd(ClassAd *daemonAd); 
+	void	UpdateLocalAd(ClassAd *daemonAd,char const *fname=NULL); 
 
 		/**
 		   Publish all DC-specific attributes into the given ClassAd.
@@ -1339,24 +1391,57 @@ class DaemonCore : public Service
 		*/
 	void WantSendChildAlive( bool send_alive )
 		{ m_want_send_child_alive = send_alive; }
+	
+		/**
+			Returns true if a wake signal was sent to the master's select.
+			Does nothing and returns false if called by the master or by a thread 
+			that is not a condor_thread. 
+		*/
+	bool Wake_up_select();
+
+		/**
+			Wakes up the main condor select, does not check to see if the caller
+			is a condor_thread or the main thread. returns true if the main thread
+			will wake, or false if unable to send the wake signal.
+		*/
+	bool Do_Wake_up_select();
+
+		/** Registers a socket for read and then calls HandleReq to
+			process a command on the socket once one becomes
+			available.
+		*/
+	void HandleReqAsync(Stream *stream);
+
+		/** Force a reload of the shared port server address.
+			Called, for example, after the master has started up the
+			shared port server.
+		*/
+	void ReloadSharedPortServerAddr();
 
   private:      
 
-	bool m_wants_dc_udp; // do we want a udp comment socket at all?
+		// do and our parents/children want/have a udp comment socket?
+	bool m_wants_dc_udp;
+		// do we ourself want/have a udp comment socket?
+	bool m_wants_dc_udp_self;
 	bool m_invalidate_sessions_via_tcp;
 	ReliSock* dc_rsock;	// tcp command socket
 	SafeSock* dc_ssock;	// udp command socket
+    int m_iMaxAcceptsPerCycle; ///< maximum number of inbound connections to accept per loop
 
     void Inherit( void );  // called in main()
 	void InitDCCommandSocket( int command_port );  // called in main()
+	void SetDaemonSockName( char const *sock_name );
 
     int HandleSigCommand(int command, Stream* stream);
-    int HandleReq(int socki);
-	int HandleReq(Stream *insock);
+    int HandleReq(int socki, Stream* accepted_sock=NULL);
+	int HandleReq(Stream *insock, Stream* accepted_sock=NULL);
 	int HandleReqSocketTimerHandler();
 	int HandleReqSocketHandler(Stream *stream);
     int HandleSig(int command, int sig);
 
+	bool RegisterSocketForHandleReq(Stream *stream);
+
 	friend class FakeCreateThreadReaperCaller;
 	void CallReaper(int reaper_id, char const *whatexited, pid_t pid, int exit_status);
 
@@ -1372,7 +1457,8 @@ class DaemonCore : public Service
                          Service* s, 
                          DCpermission perm,
                          int dprintf_flag,
-                         int is_cpp);
+                         int is_cpp,
+                         bool force_authentication);
 
     int Register_Signal(int sig,
                         const char *sig_descip,
@@ -1436,7 +1522,7 @@ class DaemonCore : public Service
 
 	void Send_Signal(classy_counted_ptr<DCSignalMsg> msg, bool nonblocking);
 
-	MyString GetCommandsInAuthLevel(DCpermission perm);
+	MyString GetCommandsInAuthLevel(DCpermission perm,bool is_authenticated);
 
     struct CommandEnt
     {
@@ -1445,6 +1531,7 @@ class DaemonCore : public Service
         CommandHandlercpp   handlercpp;
         int             is_cpp;
         DCpermission    perm;
+        bool            force_authentication;
         Service*        service; 
         char*           command_descrip;
         char*           handler_descrip;
@@ -1490,11 +1577,15 @@ class DaemonCore : public Service
         char*           handler_descrip;
         void*           data_ptr;
 		bool			is_connect_pending;
+		bool			is_reverse_connect_pending;
 		bool			call_handler;
+		int				servicing_tid;	// tid servicing this socket
+		bool			remove_asap;	// remove when being_serviced==false
     };
     void              DumpSocketTable(int, const char* = NULL);
     int               maxSocket;  // number of socket handlers to start with
-    int               nSock;      // number of socket handlers used
+    int               nSock;      // number of socket handler slots in use use
+	int				  nRegisteredSocks; // number of sockets registered, always < nSock
 	int               nPendingSockets; // number of sockets waiting on timers or any other callbacks
     ExtArray<SockEnt> *sockTable; // socket table; grows dynamically if needed
     int               initial_command_sock;  
@@ -1565,7 +1656,7 @@ class DaemonCore : public Service
 		PidEntry();
 		~PidEntry();
 		int pipeHandler(int pipe_fd);
-		void pipeFullWrite(int pipe_fd);
+		int pipeFullWrite(int pipe_fd);
 
         pid_t pid;
         int new_process_group;
@@ -1579,8 +1670,8 @@ class DaemonCore : public Service
 		LONG deallocate;
 		HANDLE watcherEvent;
 #endif
-        char sinful_string[28];
-        char parent_sinful_string[28];
+        MyString sinful_string;
+        MyString parent_sinful_string;
         int is_local;
         int parent_is_local;
         int reaper_id;
@@ -1593,6 +1684,9 @@ class DaemonCore : public Service
 		/* the environment variables which allow me the track the pidfamily
 			of this pid (where applicable) */
 		PidEnvID penvid;
+		MyString shared_port_fname;
+		//Session ID and key for child process.
+		char* child_session_id;
     };
 
 	int m_refresh_dns_timer;
@@ -1628,8 +1722,6 @@ class DaemonCore : public Service
 	int					_cookie_len, _cookie_len_old;
 	unsigned char		*_cookie_data, *_cookie_data_old;
 
-    void free_descrip(char *p) { if(p &&  p != EMPTY_DESCRIP) free(p); }
-    
     struct in_addr      negotiator_sin_addr;    // used by Verify method
 
 #ifdef WIN32
@@ -1641,11 +1733,15 @@ class DaemonCore : public Service
     int SetFDInheritFlag(int fd, int flag);
 #endif
 
+	// Setup an async_pipe, used to wake up select from another thread.
+	// Implemented on Unix via a pipe, on Win32 via a pair of connected tcp sockets.
 #ifndef WIN32
     int async_pipe[2];  // 0 for reading, 1 for writing
     volatile int async_sigs_unblocked;
+#else
+	ReliSock async_pipe[2];  // 0 for reading, 1 for writing
 #endif
-	volatile bool async_pipe_empty;
+	volatile bool async_pipe_signal;
 
 	// Data memebers for queuing up waitpid() events
 	struct WaitpidEntry_s
@@ -1670,12 +1766,16 @@ class DaemonCore : public Service
     int HungChildTimeout();
     int SendAliveToParent();
     unsigned int max_hang_time;
+	int m_child_alive_period;
     int send_child_alive_timer;
 	bool m_want_send_child_alive;
 
 	// misc helper functions
 	void CheckPrivState( void );
 
+		// invoked by CondorThreads upon thread context switch
+	static void thread_switch_callback(void* & ptr);
+
 		// Call the registered socket handler for this socket
 		// i - index of registered socket
 		// default_to_HandleCommand - true if HandleCommand() should be called
@@ -1683,15 +1783,14 @@ class DaemonCore : public Service
 		// On return, i may be modified so that when incremented,
 		// it will index the next registered socket.
 	void CallSocketHandler( int &i, bool default_to_HandleCommand );
+	static void CallSocketHandler_worker_demarshall(void *args);
+	void CallSocketHandler_worker( int i, bool default_to_HandleCommand, Stream* asock );
+	
 
 		// Returns index of registered socket or -1 if not found.
 	int GetRegisteredSocketIndex( Stream *sock );
 
-    // these need to be in thread local storage someday
-    void **curr_dataptr;
-    void **curr_regdataptr;
     int inServiceCommandSocket_flag;
-    // end of thread local storage
         
 #ifndef WIN32
     static char **ParseArgsString(const char *env);
@@ -1751,6 +1850,15 @@ class DaemonCore : public Service
 
 	char* m_private_network_name;
 
+	class CCBListeners *m_ccb_listeners;
+	class SharedPortEndpoint *m_shared_port_endpoint;
+	MyString m_daemon_sock_name;
+	Sinful m_sinful;     // full contact info (public, private, ccb, etc.)
+	bool m_dirty_sinful; // true if m_sinful needs to be reinitialized
+
+	bool CommandNumToTableIndex(int cmd,int *cmd_index);
+
+	void InitSharedPort(bool in_init_dc_command_socket=false);
 };
 
 #ifndef _NO_EXTERN_DAEMON_CORE
@@ -1774,6 +1882,13 @@ extern void DC_Exit( int status, const char *shutdown_program = NULL );
 */
 extern void DC_Skip_Auth_Init();
 
+/** Call this function (inside your main_pre_dc_init() function) to
+    bypass the core limit initialization in daemoncore.  This is for
+    programs, such as condor_dagman, that are Condor daemons but should
+    run with the user's limits.
+*/
+extern void DC_Skip_Core_Init();
+
 /** The main DaemonCore object.  This pointer will be automatically instatiated
     for you.  A perfect place to use it would be in your main_init, to access
     Daemon Core's wonderful services, like <tt>Register_Signal()</tt> or
@@ -1828,6 +1943,6 @@ int Create_Thread_With_Data(DataThreadWorkerFunc Worker, DataThreadReaperFunc Re
 #define MAX_INHERIT_FDS   32
 
 // Prototype to get sinful string.
-char *global_dc_sinful( void );
+char const *global_dc_sinful( void );
 
 #endif /* _CONDOR_DAEMON_CORE_H_ */
