@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2011, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -19,6 +19,7 @@
 
 
 #include "condor_common.h"
+#include "condor_open.h"
 #include "condor_config.h"
 #include "util_lib_proto.h"
 #include "basename.h"
@@ -35,6 +36,7 @@
 #include "time_offset.h"
 #include "subsystem_info.h"
 #include "file_lock.h"
+#include "directory.h"
 #include "exit.h"
 
 #if HAVE_EXT_GCB
@@ -48,7 +50,7 @@
 #include "condor_daemon_core.h"
 
 #ifdef WIN32
-#include "exphnd.WIN32.h"
+#include "exception_handling.WINDOWS.h"
 #endif
 #if HAVE_EXT_COREDUMPER
 #include "google/coredumper.h"
@@ -59,16 +61,19 @@ extern DLL_IMPORT_MAGIC char **environ;
 
 
 // External protos
-extern int main_init(int argc, char *argv[]);	// old main()
-extern int main_config(bool is_full);
-extern int main_shutdown_fast();
-extern int main_shutdown_graceful();
+extern void main_init(int argc, char *argv[]);	// old main()
+extern void main_config();
+extern void main_shutdown_fast();
+extern void main_shutdown_graceful();
 extern void main_pre_dc_init(int argc, char *argv[]);
 extern void main_pre_command_sock_init();
 
 // Internal protos
-void dc_reconfig( bool is_full );
+void dc_reconfig();
 void dc_config_auth();       // Configuring GSI (and maybe other) authentication related stuff
+int handle_fetch_log_history(ReliSock *s, char *name);
+int handle_fetch_log_history_dir(ReliSock *s, char *name);
+int handle_fetch_log_history_purge(ReliSock *s);
 
 // Globals
 int		Foreground = 0;		// run in background by default
@@ -81,6 +86,8 @@ char*	pidFile = NULL;
 char*	addrFile = NULL;
 static	char*	logAppend = NULL;
 
+static char *core_dir = NULL;
+
 int condor_main_argc;
 char **condor_main_argv;
 time_t daemon_stop_time;
@@ -116,11 +123,15 @@ int runfor = 0; //allow cmd line option to exit after *runfor* minutes
 // It can be set to false by calling the DC_Skip_Auth_Init() function.
 static bool doAuthInit = true;
 
+// This flag tells daemoncore whether to do the core limit initialization.
+// It can be set to false by calling the DC_Skip_Core_Init() function.
+static bool doCoreInit = true;
+
 
 #ifndef WIN32
 // This function polls our parent process; if it is gone, shutdown.
 void
-check_parent()
+check_parent( )
 {
 	if ( daemonCore->Is_Pid_Alive( daemonCore->getppid() ) == FALSE ) {
 		// our parent is gone!
@@ -134,7 +145,7 @@ check_parent()
 
 // This function clears expired sessions from the cache
 void
-check_session_cache()
+check_session_cache( )
 {
 	daemonCore->getSecMan()->invalidateExpiredCache();
 }
@@ -156,7 +167,7 @@ bool global_dc_get_cookie(int &len, unsigned char* &data) {
 }
 
 void
-handle_cookie_refresh()
+handle_cookie_refresh( )
 {
 	unsigned char randomjunk[256];
 	char symbols[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
@@ -171,7 +182,7 @@ handle_cookie_refresh()
 	global_dc_set_cookie (128, randomjunk);	
 }
 
-char* global_dc_sinful() {
+char const* global_dc_sinful() {
 	if (daemonCore) {
 		return daemonCore->InfoCommandSinfulString();
 	} else {
@@ -179,8 +190,7 @@ char* global_dc_sinful() {
 	}
 }
 
-void
-clean_files()
+void clean_files()
 {
 		// If we created a pid file, remove it.
 	if( pidFile ) {
@@ -260,10 +270,24 @@ DC_Exit( int status, const char *shutdown_program )
 		exit_status = DAEMON_NO_RESTART;
 	}
 
+#ifndef WIN32
+	// unregister our signal handlers in case some 3rd-party lib
+	// was masking signals on us...no late arrivals
+	install_sig_handler(SIGCHLD,SIG_DFL);
+	install_sig_handler(SIGHUP,SIG_DFL);
+	install_sig_handler(SIGTERM,SIG_DFL);
+	install_sig_handler(SIGQUIT,SIG_DFL);
+	install_sig_handler(SIGUSR1,SIG_DFL);
+	install_sig_handler(SIGUSR2,SIG_DFL);
+#endif /* ! WIN32 */
+
 		// Now, delete the daemonCore object, since we allocated it. 
-	unsigned long	pid = daemonCore->getpid( );
-	delete daemonCore;
-	daemonCore = NULL;
+	unsigned long	pid = 0;
+	if (daemonCore) {
+		pid = daemonCore->getpid( );
+		delete daemonCore;
+		daemonCore = NULL;
+	}
 
 		// Free up the memory from the config hash table, too.
 	clear_config();
@@ -271,6 +295,11 @@ DC_Exit( int status, const char *shutdown_program )
 		// and deallocate the memory from the passwd_cache (uids.C)
 	delete_passwd_cache();
 
+	if ( core_dir ) {
+		free( core_dir );
+		core_dir = NULL;
+	}
+
 		/*
 		  Log a message.  We want to do this *AFTER* we delete the
 		  daemonCore object and free up other memory, just to make
@@ -285,7 +314,9 @@ DC_Exit( int status, const char *shutdown_program )
 		dprintf( D_ALWAYS, "**** %s (%s_%s) pid %lu EXITING BY EXECING %s\n",
 				 myName, myDistro->Get(), get_mySubSystem()->getName(), pid,
 				 shutdown_program );
+		priv_state p = set_root_priv( );
 		int exec_status = execl( shutdown_program, shutdown_program, NULL );
+		set_priv( p );
 		dprintf( D_ALWAYS, "**** execl() FAILED %d %d %s\n",
 				 exec_status, errno, strerror(errno) );
 #     elif defined(WIN32)
@@ -293,7 +324,9 @@ DC_Exit( int status, const char *shutdown_program )
 				 "**** %s (%s_%s) pid %lu EXECING SHUTDOWN PROGRAM %s\n",
 				 myName, myDistro->Get(), get_mySubSystem()->getName(), pid,
 				 shutdown_program );
+		priv_state p = set_root_priv( );
 		int exec_status = execl( shutdown_program, shutdown_program, NULL );
+		set_priv( p );
 		if ( exec_status ) {
 			dprintf( D_ALWAYS, "**** _execl() FAILED %d %d %s\n",
 					 exec_status, errno, strerror(errno) );
@@ -317,6 +350,12 @@ DC_Skip_Auth_Init()
 	doAuthInit = false;
 }
 
+void
+DC_Skip_Core_Init()
+{
+	doCoreInit = false;
+}
+
 static void
 kill_daemon_ad_file()
 {
@@ -519,19 +558,17 @@ handle_log_append( char* append_str )
 }
 
 
-int
-dc_touch_log_file( Service* )
+void
+dc_touch_log_file( )
 {
 	dprintf_touch_log();
 
 	daemonCore->Register_Timer( param_integer( "TOUCH_LOG_INTERVAL", 60 ),
-				(TimerHandler)dc_touch_log_file, "dc_touch_log_file" );
-
-	return TRUE;
+				dc_touch_log_file, "dc_touch_log_file" );
 }
 
-int
-dc_touch_lock_files(Service *)
+void
+dc_touch_lock_files( )
 {
 	priv_state p;
 
@@ -551,19 +588,17 @@ dc_touch_lock_files(Service *)
 	// reset the timer for next incarnation of the update.
 	daemonCore->Register_Timer(
 		param_integer("LOCK_FILE_UPDATE_INTERVAL", 3600 * 8, 60, INT_MAX),
-		(TimerHandler)dc_touch_lock_files, "dc_touch_lock_files" );
-
-	return TRUE;
+		dc_touch_lock_files, "dc_touch_lock_files" );
 }
 
 
 void
-set_dynamic_dir( char* param_name, const char* append_str )
+set_dynamic_dir( const char* param_name, const char* append_str )
 {
 	char* val;
 	MyString newdir;
 
-	val = param( (char *)param_name );
+	val = param( param_name );
 	if( ! val ) {
 			// nothing to do
 		return;
@@ -578,7 +613,7 @@ set_dynamic_dir( char* param_name, const char* append_str )
 
 		// Now, set our own config hashtable entry so we start using
 		// this new directory.
-	config_insert( (char *) param_name, newdir.Value() );
+	config_insert( param_name, newdir.Value() );
 
 	// Finally, insert the _condor_<param_name> environment
 	// variable, so our children get the right configuration.
@@ -626,8 +661,6 @@ handle_dynamic_dirs()
 	}
 }
 
-static char *core_dir = NULL;
-
 #if HAVE_EXT_COREDUMPER
 void
 linux_sig_coredump(int signum)
@@ -711,7 +744,14 @@ drop_core_in_log( void )
 	char* ptmp = param("LOG");
 	if ( ptmp ) {
 		if ( chdir(ptmp) < 0 ) {
-			EXCEPT("cannot chdir to dir <%s>",ptmp);
+#ifdef WIN32
+			if (MATCH == strcmpi(get_mySubSystem()->getName(), "KBDD")) {
+				dprintf (D_FULLDEBUG, "chdir() to LOG directory failed for KBDD, "
+					     "cannot drop core in LOG dir\n");
+				return;
+			}
+#endif
+    	EXCEPT("cannot chdir to dir <%s>",ptmp);
 		}
 	} else {
 		dprintf( D_FULLDEBUG, 
@@ -720,6 +760,10 @@ drop_core_in_log( void )
 		return;
 	}
 
+	if ( core_dir ) {
+		free( core_dir );
+		core_dir = NULL;
+	}
 	core_dir = strdup(ptmp);
 
 	// in some case we need to hook up our own handler to generate
@@ -741,6 +785,9 @@ drop_core_in_log( void )
 			SetEnv( "_NT_SYMBOL_PATH", binpath );
 			free(binpath);
 		}
+
+		// give the handler our pid
+		g_ExceptionHandler.SetPID ( daemonCore->getpid () );
 	}
 #endif
 	free(ptmp);
@@ -762,9 +809,9 @@ check_core_files()
 	if( (tmp = param("CREATE_CORE_FILES")) ) {
 #ifndef WIN32	
 		if( *tmp == 't' || *tmp == 'T' ) {
-			limit( RLIMIT_CORE, RLIM_INFINITY, CONDOR_SOFT_LIMIT );
+			limit( RLIMIT_CORE, RLIM_INFINITY, CONDOR_SOFT_LIMIT,"max core size" );
 		} else {
-			limit( RLIMIT_CORE, 0, CONDOR_SOFT_LIMIT );
+			limit( RLIMIT_CORE, 0, CONDOR_SOFT_LIMIT,"max core size" );
 		}
 #endif
 		if( *tmp == 'f' || *tmp == 'F' ) {
@@ -796,7 +843,9 @@ handle_off_fast( Service*, int, Stream* stream)
 		dprintf( D_ALWAYS, "handle_off_fast: failed to read end of message\n");
 		return FALSE;
 	}
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGQUIT );
+	if (daemonCore) {
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGQUIT );
+	}
 	return TRUE;
 }
 
@@ -808,7 +857,9 @@ handle_off_graceful( Service*, int, Stream* stream)
 		dprintf( D_ALWAYS, "handle_off_graceful: failed to read end of message\n");
 		return FALSE;
 	}
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGTERM );
+	if (daemonCore) {
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGTERM );
+	}
 	return TRUE;
 }
 
@@ -822,8 +873,10 @@ handle_off_peaceful( Service*, int, Stream* stream)
 		dprintf( D_ALWAYS, "handle_off_peaceful: failed to read end of message\n");
 		return FALSE;
 	}
-	daemonCore->SetPeacefulShutdown(true);
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGTERM );
+	if (daemonCore) {
+		daemonCore->SetPeacefulShutdown(true);
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGTERM );
+	}
 	return TRUE;
 }
 
@@ -846,25 +899,20 @@ handle_set_peaceful_shutdown( Service*, int, Stream* stream)
 
 
 static int
-handle_reconfig( Service*, int cmd, Stream* stream )
+handle_reconfig( Service*, int /* cmd */, Stream* stream )
 {
 	if( !stream->end_of_message() ) {
 		dprintf( D_ALWAYS, "handle_reconfig: failed to read end of message\n");
 		return FALSE;
 	}
-	if( cmd == DC_RECONFIG_FULL ) {
-		dc_reconfig( true );
-	} else {
-		dc_reconfig( false );
-	}		
+	dc_reconfig();
 	return TRUE;
 }
 
 int
-handle_fetch_log( Service *, int, Stream *s )
+handle_fetch_log( Service *, int, ReliSock *stream )
 {
 	char *name = NULL;
-	ReliSock *stream = (ReliSock*) s;
 	int  total_bytes = 0;
 	int result;
 	int type = -1;
@@ -879,13 +927,23 @@ handle_fetch_log( Service *, int, Stream *s )
 
 	stream->encode();
 
-	if(type!=DC_FETCH_LOG_TYPE_PLAIN) {
-		dprintf(D_ALWAYS,"DaemonCore: handle_fetch_log: I don't know about log type %d!\n",type);
-		result = DC_FETCH_LOG_RESULT_BAD_TYPE;
-		stream->code(result);
-		stream->end_of_message();
-        free(name);
-		return FALSE;
+	switch (type) {
+		case DC_FETCH_LOG_TYPE_PLAIN:
+			break; // handled below
+		case DC_FETCH_LOG_TYPE_HISTORY:
+			return handle_fetch_log_history(stream, name);
+		case DC_FETCH_LOG_TYPE_HISTORY_DIR:
+			return handle_fetch_log_history_dir(stream, name);
+		case DC_FETCH_LOG_TYPE_HISTORY_PURGE:
+			free(name);
+			return handle_fetch_log_history_purge(stream);
+		default:
+			dprintf(D_ALWAYS,"DaemonCore: handle_fetch_log: I don't know about log type %d!\n",type);
+			result = DC_FETCH_LOG_RESULT_BAD_TYPE;
+			stream->code(result);
+			stream->end_of_message();
+			free(name);
+			return FALSE;
 	}
 
 	char *pname = (char*)malloc (strlen(name) + 5);
@@ -919,6 +977,11 @@ handle_fetch_log( Service *, int, Stream *s )
 	MyString full_filename = filename;
 	if(ext) {
 		full_filename += ext;
+
+		if( strchr(ext,DIR_DELIM_CHAR) ) {
+			dprintf( D_ALWAYS, "DaemonCore: handle_fetch_log: invalid file extension specified by user: ext=%s, filename=%s\n",ext,full_filename.Value() );
+			return FALSE;
+		}
 	}
 
 	int fd = safe_open_wrapper(full_filename.Value(),O_RDONLY);
@@ -954,16 +1017,134 @@ handle_fetch_log( Service *, int, Stream *s )
 	return total_bytes>=0;
 }
 
+int
+handle_fetch_log_history(ReliSock *stream, char *name) {
+	int result = DC_FETCH_LOG_RESULT_BAD_TYPE;
 
+	const char *history_file_param = "HISTORY";
+	if (strcmp(name, "STARTD_HISTORY") == 0) {
+		history_file_param = "STARTD_HISTORY";
+	}
+
+	free(name);
+	char *history_file = param(history_file_param);
+
+	if (!history_file) {
+		dprintf( D_ALWAYS, "DaemonCore: handle_fetch_log_history: no parameter named %s\n", history_file_param);
+		stream->code(result);
+		stream->end_of_message();
+		return FALSE;
+	}
+	int fd = safe_open_wrapper(history_file,O_RDONLY);
+	free(history_file);
+	if(fd<0) {
+		dprintf( D_ALWAYS, "DaemonCore: handle_fetch_log_history: can't open history file\n");
+		result = DC_FETCH_LOG_RESULT_CANT_OPEN;
+		stream->code(result);
+		stream->end_of_message();
+		return FALSE;
+	}
+
+	result = DC_FETCH_LOG_RESULT_SUCCESS;
+	stream->code(result);
+
+	filesize_t size;
+	stream->put_file(&size, fd);
+
+	stream->end_of_message();
+
+	if(size<0) {
+		dprintf( D_ALWAYS, "DaemonCore: handle_fetch_log_history: couldn't send all data!\n");
+	}
+
+	close(fd);
+	return TRUE;
+}
+
+int
+handle_fetch_log_history_dir(ReliSock *stream, char *paramName) {
+	int result = DC_FETCH_LOG_RESULT_BAD_TYPE;
+
+	free(paramName);
+	char *dirName = param("STARTD.PER_JOB_HISTORY_DIR"); 
+	if (!dirName) {
+		dprintf( D_ALWAYS, "DaemonCore: handle_fetch_log_history_dir: no parameter named PER_JOB\n");
+		stream->code(result);
+		stream->end_of_message();
+		return FALSE;
+	}
+
+	Directory d(dirName);
+	const char *filename;
+	int one=1;
+	int zero=0;
+	while ((filename = d.Next())) {
+		stream->code(one); // more data
+		stream->put(filename);
+		MyString fullPath(dirName);
+		fullPath += "/";
+		fullPath += filename;
+		int fd = safe_open_wrapper(fullPath.Value(),O_RDONLY);
+		if (fd > 0) {
+			filesize_t size;
+			stream->put_file(&size, fd);
+		}
+	}
+
+	free(dirName);
+
+	stream->code(zero); // no more data
+	stream->end_of_message();
+	return 0;
+}
+
+int
+handle_fetch_log_history_purge(ReliSock *s) {
+
+	int result = 0;
+	time_t cutoff = 0;
+	s->code(cutoff);
+	s->end_of_message();
+
+	s->encode();
+
+	char *dirName = param("STARTD.PER_JOB_HISTORY_DIR"); 
+	if (!dirName) {
+		dprintf( D_ALWAYS, "DaemonCore: handle_fetch_log_history_dir: no parameter named PER_JOB\n");
+		s->code(result);
+		s->end_of_message();
+		return FALSE;
+	}
+
+	Directory d(dirName);
+
+	result = 1;
+	while (d.Next()) {
+		time_t last = d.GetModifyTime();
+		if (last < cutoff) {
+			d.Remove_Current_File();
+		}
+	}
+
+    free(dirName);
+
+    s->code(result); // no more data
+    s->end_of_message();
+    return 0;
+}
+
+
+#ifdef WIN32
 int
 handle_nop( Service*, int, Stream* stream)
 {
 	if( !stream->end_of_message() ) {
-		dprintf( D_ALWAYS, "handle_nop: failed to read end of message\n");
+		dprintf( D_FULLDEBUG, "handle_nop: failed to read end of message\n");
 		return FALSE;
 	}
 	return TRUE;
 }
+#endif
 
 
 int
@@ -1069,13 +1250,17 @@ handle_config( Service *, int cmd, Stream *stream )
 		dprintf( D_ALWAYS, "handle_config: failed to read end of message\n");
 		return FALSE;
 	}
-
 	if( config && config[0] ) {
-		to_check = config;
+		to_check = parse_param_name_from_config(config);
 	} else {
-		to_check = admin;
+		to_check = strdup(admin);
 	}
-	if( ! daemonCore->CheckConfigSecurity(to_check, (Sock*)stream) ) {
+	if (!is_valid_param_name(to_check)) {
+		dprintf( D_ALWAYS, "Rejecting attempt to set param with invalid name (%s)\n", to_check);
+		free(admin); free(config);
+		rval = -1;
+		failed = true;
+	} else if( ! daemonCore->CheckConfigSecurity(to_check, (Sock*)stream) ) {
 			// This request is insecure, so don't try to do anything
 			// with it.  We can't return yet, since we want to send
 			// back an rval indicating the error.
@@ -1084,6 +1269,7 @@ handle_config( Service *, int cmd, Stream *stream )
 		rval = -1;
 		failed = true;
 	} 
+	free(to_check);
 
 		// If we haven't hit an error yet, try to process the command  
 	if( ! failed ) {
@@ -1124,41 +1310,53 @@ handle_config( Service *, int cmd, Stream *stream )
 void
 unix_sighup(int)
 {
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGHUP );
+	if (daemonCore) {
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGHUP );
+	}
 }
 
 
 void
 unix_sigterm(int)
 {
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGTERM );
+	if (daemonCore) {
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGTERM );
+	}
 }
 
 
 void
 unix_sigquit(int)
 {
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGQUIT );
+	if (daemonCore) {
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGQUIT );
+	}
 }
 
 
 void
 unix_sigchld(int)
 {
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGCHLD );
+	if (daemonCore) {
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGCHLD );
+	}
 }
 
 
 void
 unix_sigusr1(int)
 {
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGUSR1 );
+	if (daemonCore) {
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGUSR1 );
+	}
 }
 
 void
 unix_sigusr2(int)
 {
-	daemonCore->Send_Signal( daemonCore->getpid(), SIGUSR2 );
+	if (daemonCore) {
+		daemonCore->Send_Signal( daemonCore->getpid(), SIGUSR2 );
+	}
 }
 
 
@@ -1168,7 +1366,7 @@ unix_sigusr2(int)
 
 
 void
-dc_reconfig( bool is_full )
+dc_reconfig()
 {
 		// do this first in case anything else depends on DNS
 	daemonCore->refreshDNS();
@@ -1186,10 +1384,8 @@ dc_reconfig( bool is_full )
 	config(0);
 
 		// See if we're supposed to be allowing core files or not
-	check_core_files();
-
-	if( DynamicDirs ) {
-		handle_dynamic_dirs();
+	if ( doCoreInit ) {
+		check_core_files();
 	}
 
 		// If we're supposed to be using our own log file, reset that here. 
@@ -1213,9 +1409,7 @@ dc_reconfig( bool is_full )
 	daemonCore->reconfig();
 
 	// Clear out the passwd cache.
-	if( is_full ) {
-		clear_passwd_cache();
-	}
+	clear_passwd_cache();
 
 	// Re-drop the address file, if it's defined, just to be safe.
 	drop_addr_file();
@@ -1241,18 +1435,25 @@ dc_reconfig( bool is_full )
 	}
 
 	// call this daemon's specific main_config()
-	main_config( is_full );
+	main_config();
 }
 
 int
 handle_dc_sighup( Service*, int )
 {
 	dprintf( D_ALWAYS, "Got SIGHUP.  Re-reading config files.\n" );
-	dc_reconfig( true );
+	dc_reconfig();
 	return TRUE;
 }
 
 
+void
+TimerHandler_main_shutdown_fast()
+{
+	main_shutdown_fast();
+}
+
+
 int
 handle_dc_sigterm( Service*, int )
 {
@@ -1285,7 +1486,7 @@ handle_dc_sigterm( Service*, int )
 			free( tmp );
 		}
 		daemonCore->Register_Timer( timeout, 0, 
-									(TimerHandler)main_shutdown_fast,
+									TimerHandler_main_shutdown_fast,
 									"main_shutdown_fast" );
 		dprintf( D_FULLDEBUG, 
 				 "Started timer to call main_shutdown_fast in %d seconds\n", 
@@ -1295,6 +1496,12 @@ handle_dc_sigterm( Service*, int )
 	return TRUE;
 }
 
+void
+TimerHandler_dc_sigterm()
+{
+	handle_dc_sigterm(NULL, SIGTERM);
+}
+
 
 int
 handle_dc_sigquit( Service*, int )
@@ -1313,13 +1520,14 @@ handle_dc_sigquit( Service*, int )
 }
 
 void
-handle_gcb_recovery_failed( Service * /*ignore*/ )
+handle_gcb_recovery_failed( )
 {
 	dprintf( D_ALWAYS, "GCB failed to recover from a failure with the "
 			 "Broker. Performing fast shutdown.\n" );
 	main_shutdown_fast();
 }
 
+#if HAVE_EXT_GCB
 static void
 gcb_recovery_failed_callback()
 {
@@ -1327,9 +1535,10 @@ gcb_recovery_failed_callback()
 		// DaemonCore is blocked on a select() or CEDAR is blocked on a
 		// network operation. So we register a daemoncore timer to do
 		// the real work.
-	daemonCore->Register_Timer( 0, (TimerHandler)handle_gcb_recovery_failed,
+	daemonCore->Register_Timer( 0, handle_gcb_recovery_failed,
 								"handle_gcb_recovery_failed" );
 }
+#endif
 
 // This is the main entry point for daemon core.  On WinNT, however, we
 // have a different, smaller main which checks if "-f" is ommitted from
@@ -1343,6 +1552,7 @@ int main( int argc, char** argv )
 {
 	char**	ptr;
 	int		command_port = -1;
+	char const *daemon_sock_name = NULL;
 	int 	http_port = -1;
 	int		dcargs = 0;		// number of daemon core command-line args found
 	char	*ptmp, *ptmp1;
@@ -1623,6 +1833,27 @@ int main( int argc, char** argv )
 			}
 			//call Register_Timer below after intialized...
 			break;
+		case 's':
+				// the c-gahp uses -s, so for backward compatibility,
+				// do not allow abbreviations of -sock
+			if( strcmp("-sock",*ptr) ) {
+				done = true;
+				break;
+			}
+			else {
+				ptr++;
+				if( *ptr ) {
+					daemon_sock_name = *ptr;
+					dcargs += 2;
+				} else {
+					fprintf( stderr, 
+							 "DaemonCore: ERROR: -sock needs another argument.\n" );
+					fprintf( stderr, 
+							 "   Please specify a socket name.\n" );
+					exit( 1 );
+				}
+			}
+			break;
 		case 't':		// log to Terminal (stderr)
 			Termlog = 1;
 			dcargs++;
@@ -1662,7 +1893,9 @@ int main( int argc, char** argv )
 	}
 
 		// See if we're supposed to be allowing core files or not
-	check_core_files();
+	if ( doCoreInit ) {
+		check_core_files();
+	}
 
 		// If we want to kill something, do that now.
 	if( wantsKill ) {
@@ -1770,7 +2003,7 @@ int main( int argc, char** argv )
 			}
 			// Close the /dev/null descriptor _IF_ it's not stdin/out/err
 			if ( fd_null > 2 ) {
-				close( fd );
+				close( fd_null );
 			}
 		}
 		// and detach from the controlling tty
@@ -1864,7 +2097,7 @@ int main( int argc, char** argv )
 
 	if (global_config_source != "") {
 		dprintf(D_ALWAYS, "Using config source: %s\n", 
-				global_config_source.GetCStr());
+				global_config_source.Value());
 	} else {
 		const char* env_name = EnvGetName( ENV_CONFIG );
 		char* env = getenv( env_name );
@@ -1895,20 +2128,14 @@ int main( int argc, char** argv )
 		// right now: a) we are running as a service, and services are 
 		// born without a console, or b) the user did not specify "-f" 
 		// or "-t", and thus we called FreeConsole() above.
-	AllocConsole();
+
+		// for now, don't create a console for the kbdd, as that daemon
+		// is now a win32 app and not a console app.
+	BOOL is_kbdd = (0 == strcmp(get_mySubSystem()->getName(), "KBDD"));
+	if(!is_kbdd)
+		AllocConsole();
 #endif
 
-		// Avoid possibility of stale info sticking around from previous run.
-		// For example, we had problems in 7.0.4 and earlier with reconnect
-		// shadows in parallel universe reading the old schedd ad file.
-	kill_daemon_ad_file();
-
-		// Now that we have our pid, we could dump our pidfile, if we
-		// want it. 
-	if( pidFile ) {
-		drop_pid_file();
-	}
-
 #ifndef WIN32
 		// Now that logging is setup, create a pipe to deal with unix 
 		// async signals.  We do this after logging is setup so that
@@ -1918,6 +2145,11 @@ int main( int argc, char** argv )
 		 fcntl(daemonCore->async_pipe[1],F_SETFL,O_NONBLOCK) == -1 ) {
 			EXCEPT("Failed to create async pipe");
 	}
+#else
+	if ( daemonCore->async_pipe[1].connect_socketpair(daemonCore->async_pipe[0])==false )
+	{
+		EXCEPT("Failed to create async pipe socket pair");
+	}
 #endif
 
 #if HAVE_EXT_GCB
@@ -1927,7 +2159,39 @@ int main( int argc, char** argv )
 
 	main_pre_command_sock_init();
 
+		/* NOTE re main_pre_command_sock_init:
+		  *
+		  * The Master uses main_pre_command_sock_init to check for
+		  * the InstanceLock. Any operation that is distructive before
+		  * this point will possibly change the state/environment for
+		  * an already running master.
+		  *
+		  *  In the pidfile case, a second Master will start, drop its
+		  *  pid in the file and then exit, see GT343.
+		  *
+		  *  In the DAEMON_AD_FILE case, which isn't actually a
+		  *  problem as of this comment because the Master does not
+		  *  drop one, there would be a period of time when the file
+		  *  would be missing.
+		  *
+		  * PROBLEM: TRUNC_*_LOG_ON_OPEN=TRUE will get to delete the
+		  * log file before getting here.
+		  */
+
+		// Now that we have our pid, we could dump our pidfile, if we
+		// want it. 
+	if( pidFile ) {
+		drop_pid_file();
+	}
+
+		// Avoid possibility of stale info sticking around from previous run.
+		// For example, we had problems in 7.0.4 and earlier with reconnect
+		// shadows in parallel universe reading the old schedd ad file.
+	kill_daemon_ad_file();
+
+
 		// SETUP COMMAND SOCKET
+	daemonCore->SetDaemonSockName( daemon_sock_name );
 	daemonCore->InitDCCommandSocket( command_port );
 
 		// Install DaemonCore signal handlers common to all daemons.
@@ -1956,7 +2220,7 @@ int main( int argc, char** argv )
 	if ( runfor ) {
 		daemon_stop_time = time(NULL)+runfor*60;
 		daemonCore->Register_Timer( runfor * 60, 0, 
-				(TimerHandler)handle_dc_sigterm, "handle_dc_sigterm" );
+				TimerHandler_dc_sigterm, "handle_dc_sigterm" );
 		dprintf(D_ALWAYS,"Registered Timer for graceful shutdown in %d minutes\n",
 				runfor );
 	}
@@ -1970,18 +2234,18 @@ int main( int argc, char** argv )
 		// Also note: we do not want the master to exibit this behavior!
 	if ( ! get_mySubSystem()->isType(SUBSYSTEM_TYPE_MASTER) ) {
 		daemonCore->Register_Timer( 15, 120, 
-				(TimerHandler)check_parent, "check_parent" );
+				check_parent, "check_parent" );
 	}
 #endif
 
 	daemonCore->Register_Timer( 0,
-				(TimerHandler)dc_touch_log_file, "dc_touch_log_file" );
+				dc_touch_log_file, "dc_touch_log_file" );
 
 	daemonCore->Register_Timer( 0,
-				(TimerHandler)dc_touch_lock_files, "dc_touch_lock_files" );
+				dc_touch_lock_files, "dc_touch_lock_files" );
 
 	daemonCore->Register_Timer( 0, 5 * 60,
-				(TimerHandler)check_session_cache, "check_session_cache" );
+				check_session_cache, "check_session_cache" );
 	
 
 	// set the timer for half the session duration, 
@@ -1990,10 +2254,12 @@ int main( int argc, char** argv )
 	int cookie_refresh = (param_integer("SEC_DEFAULT_SESSION_DURATION", 3600)/2)+1;
 
 	daemonCore->Register_Timer( 0, cookie_refresh, 
-				(TimerHandler)handle_cookie_refresh, "handle_cookie_refresh");
+				handle_cookie_refresh, "handle_cookie_refresh");
  
 
 	if( get_mySubSystem()->isType( SUBSYSTEM_TYPE_MASTER ) ||
+		get_mySubSystem()->isType( SUBSYSTEM_TYPE_COLLECTOR ) ||
+		get_mySubSystem()->isType( SUBSYSTEM_TYPE_NEGOTIATOR ) ||
 		get_mySubSystem()->isType( SUBSYSTEM_TYPE_SCHEDD ) ||
 		get_mySubSystem()->isType( SUBSYSTEM_TYPE_STARTD ) ) {
         daemonCore->monitor_data.EnableMonitoring();
@@ -2006,7 +2272,7 @@ int main( int argc, char** argv )
 
 	daemonCore->Register_Command( DC_RECONFIG_FULL, "DC_RECONFIG_FULL",
 								  (CommandHandler)handle_reconfig,
-								  "handle_reconfig()", 0, ADMINISTRATOR );
+								  "handle_reconfig()", 0, WRITE );
 
 	daemonCore->Register_Command( DC_CONFIG_VAL, "DC_CONFIG_VAL",
 								  (CommandHandler)handle_config_val,
@@ -2043,14 +2309,22 @@ int main( int argc, char** argv )
 								  (CommandHandler)handle_set_peaceful_shutdown,
 								  "handle_set_peaceful_shutdown()", 0, ADMINISTRATOR );
 
+#ifdef WIN32
+		// DC_NOP is for waking up select.  There is no need for
+		// security here, because anyone can wake up select anyway.
 	daemonCore->Register_Command( DC_NOP, "DC_NOP",
 								  (CommandHandler)handle_nop,
-								  "handle_nop()", 0, READ );
+								  "handle_nop()", 0, ALLOW );
+#endif
 
 	daemonCore->Register_Command( DC_FETCH_LOG, "DC_FETCH_LOG",
 								  (CommandHandler)handle_fetch_log,
 								  "handle_fetch_log()", 0, ADMINISTRATOR );
 
+	daemonCore->Register_Command( DC_PURGE_LOG, "DC_PURGE_LOG",
+								  (CommandHandler)handle_fetch_log_history_purge,
+								  "handle_fetch_log_history_purge()", 0, ADMINISTRATOR );
+
 	daemonCore->Register_Command( DC_INVALIDATE_KEY, "DC_INVALIDATE_KEY",
 								  (CommandHandler)handle_invalidate_key,
 								  "handle_invalidate_key()", 0, ALLOW );
