@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2011, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -24,7 +24,7 @@
 #include "condor_debug.h"
 #include "condor_daemon_core.h"
 
-static char* DEFAULT_INDENT = "DaemonCore--> ";
+static const char* DEFAULT_INDENT = "DaemonCore--> ";
 
 static	TimerManager*	_t = NULL;
 
@@ -38,6 +38,9 @@ static	TimerManager*	_t = NULL;
 	*/	
 const int MAX_FIRES_PER_TIMEOUT = 3;
 
+extern void **curr_dataptr;
+extern void **curr_regdataptr;
+
 
 TimerManager::TimerManager()
 {
@@ -46,10 +49,12 @@ TimerManager::TimerManager()
 		EXCEPT("TimerManager object exists!");
 	}
 	timer_list = NULL;
+	list_tail = NULL;
 	timer_ids = 0;
-	in_timeout = FALSE;
+	in_timeout = NULL;
 	_t = this; 
-	did_reset = -1;
+	did_reset = false;
+	did_cancel = false;
 }
 
 TimerManager::~TimerManager()
@@ -57,49 +62,44 @@ TimerManager::~TimerManager()
 	CancelAllTimers();
 }
 
-int TimerManager::NewTimer(Service* s, unsigned deltawhen, Event event, const char* event_descrip,
-						   unsigned period, int id)
-{
-	return( NewTimer(s,deltawhen,event,(Eventcpp)NULL,(Release)NULL,(Releasecpp)NULL,event_descrip,period,NULL,id,0) );
-}
-
-int TimerManager::NewTimer(Service* s, unsigned deltawhen, Event event, 
+int TimerManager::NewTimer(unsigned deltawhen, TimerHandler handler, 
 						   Release release, const char* event_descrip,
-						   unsigned period, int id)
+						   unsigned period)
 {
-	return( NewTimer(s,deltawhen,event,(Eventcpp)NULL,release,(Releasecpp)NULL,event_descrip,period,NULL,id,0) );
+	return( NewTimer(NULL,deltawhen,handler,(TimerHandlercpp)NULL,release,(Releasecpp)NULL,event_descrip,period,NULL) );
 }
 
-int TimerManager::NewTimer(unsigned deltawhen, Event event, const char* event_descrip,
-						   unsigned period, int id)
+int TimerManager::NewTimer(unsigned deltawhen, TimerHandler handler, const char* event_descrip,
+						   unsigned period)
 {
-	return( NewTimer((Service *)NULL,deltawhen,event,(Eventcpp)NULL,(Release)NULL,(Releasecpp)NULL,event_descrip,period,NULL,id,0) );
+	return( NewTimer((Service *)NULL,deltawhen,handler,(TimerHandlercpp)NULL,(Release)NULL,(Releasecpp)NULL,event_descrip,period,NULL) );
 }
 
-int TimerManager::NewTimer(Service* s, unsigned deltawhen, Eventcpp event, const char* event_descrip,
-						   unsigned period, int id)
+int TimerManager::NewTimer(Service* s, unsigned deltawhen, TimerHandlercpp handler, const char* event_descrip,
+						   unsigned period)
 {
 	if ( !s ) {
 		dprintf( D_DAEMONCORE,"DaemonCore NewTimer() called with c++ pointer & NULL Service*\n");
 		return -1;
 	}
-	return( NewTimer(s,deltawhen,(Event)NULL,event,(Release)NULL,(Releasecpp)NULL,event_descrip,period,NULL,id,1) );
+	return( NewTimer(s,deltawhen,(TimerHandler)NULL,handler,(Release)NULL,(Releasecpp)NULL,event_descrip,period,NULL) );
 }
 
-int TimerManager::NewTimer (Service* s,Timeslice timeslice,Eventcpp event,const char * event_descrip,int id)
+int TimerManager::NewTimer (Service* s,const Timeslice &timeslice,TimerHandlercpp handler,const char * event_descrip)
 {
-	return NewTimer(s,0,(Event)NULL,event,(Release)NULL,(Releasecpp)NULL,event_descrip,0,&timeslice,id,1);
+	return NewTimer(s,0,(TimerHandler)NULL,handler,(Release)NULL,(Releasecpp)NULL,event_descrip,0,&timeslice);
 }
 
 
 // Add a new event in the timer list. if period is 0, this event is a one time
 // event instead of periodical
-int TimerManager::NewTimer(Service* s, unsigned deltawhen, Event event, Eventcpp eventcpp,
-		Release release, Releasecpp releasecpp, const char *event_descrip, unsigned period, 
-		Timeslice *timeslice,int id, int is_cpp)
+int TimerManager::NewTimer(Service* s, unsigned deltawhen,
+						   TimerHandler handler, TimerHandlercpp handlercpp,
+						   Release release, Releasecpp releasecpp,
+						   const char *event_descrip, unsigned period, 
+						   const Timeslice *timeslice)
 {
 	Timer*		new_timer;
-	Timer*		timer_ptr;
 
 	dprintf( D_DAEMONCORE, "in DaemonCore NewTimer()\n" );
 	new_timer = new Timer;
@@ -108,166 +108,163 @@ int TimerManager::NewTimer(Service* s, unsigned deltawhen, Event event, Eventcpp
 		return -1;
 	}
 
-	new_timer->handler = event;
-	new_timer->handlercpp = eventcpp;
+	new_timer->handler = handler;
+	new_timer->handlercpp = handlercpp;
 	new_timer->release = release;
 	new_timer->releasecpp = releasecpp;
 	new_timer->period = period;
 	new_timer->service = s; 
-	new_timer->is_cpp = is_cpp;
 
 	if( timeslice ) {
-		new_timer->timeslice = *timeslice;
-		new_timer->has_timeslice = true;
-		deltawhen = new_timer->timeslice.getTimeToNextRun();
+		new_timer->timeslice = new Timeslice( *timeslice );
+		deltawhen = new_timer->timeslice->getTimeToNextRun();
 	}
 	else {
-		new_timer->has_timeslice = false;
+		new_timer->timeslice = NULL;
 	}
 
+	new_timer->period_started = time(NULL);
 	if ( TIMER_NEVER == deltawhen ) {
 		new_timer->when = TIME_T_NEVER;
 	} else {
-		new_timer->when = deltawhen + time(NULL);
+		new_timer->when = deltawhen + new_timer->period_started;
 	}
 	new_timer->data_ptr = NULL;
 	if ( event_descrip ) 
 		new_timer->event_descrip = strdup(event_descrip);
 	else
-		new_timer->event_descrip = EMPTY_DESCRIP;
+		new_timer->event_descrip = strdup("<NULL>");
 
 
-	if(id >= 0)	{
-		if ( CancelTimer(id, false /* don't delete data_ptr */ ) == -1 ) {
-			dprintf( D_DAEMONCORE, "cannot find timer id %d\n",id);
-			daemonCore->free_descrip( new_timer->event_descrip);
-			delete new_timer;
-			return -1;
-		}
-		new_timer->id = id;
-		dprintf(D_DAEMONCORE,"Renewing timer id %d for %d secs\n",id,deltawhen);
-	} else {
-		new_timer->id = timer_ids++;		
-	}
+	new_timer->id = timer_ids++;		
 
 
-	if ( timer_list == NULL ) {
-		// list is empty, place ours in front
-		timer_list = new_timer;
-		new_timer->next = NULL;
-	} else {
-		// list is not empty, so keep timer_list ordered from soonest to
-		// farthest (i.e. sorted on "when").
-		// Note: when doing the comparisons, we always use "<" instead
-		// of "<=" -- this makes certain we "round-robin" across 
-		// timers that constantly reset themselves to zero.
-		if ( new_timer->when < timer_list->when ) {
-			// make the this new timer first in line
-			new_timer->next = timer_list;
-			timer_list = new_timer;
-		} else {
-			Timer* trail_ptr = NULL;
-			for (timer_ptr = timer_list; timer_ptr != NULL; 
-				 timer_ptr = timer_ptr->next ) 
-			{
-				if (new_timer->when < timer_ptr->when) {
-					break;
-				}
-				trail_ptr = timer_ptr;
-			}
-			ASSERT( trail_ptr );
-			new_timer->next = timer_ptr;
-			trail_ptr->next = new_timer;
-		}
-	}
+	InsertTimer( new_timer );
 
 	DumpTimerList(D_DAEMONCORE | D_FULLDEBUG);
 
 	// Update curr_regdataptr for SetDataPtr()
-	daemonCore->curr_regdataptr = &(new_timer->data_ptr);
+	curr_regdataptr = &(new_timer->data_ptr);
 
 	dprintf(D_DAEMONCORE,"leaving DaemonCore NewTimer, id=%d\n",new_timer->id);
 
 	return	new_timer->id;
 }
 
-int TimerManager::ResetTimer(int id, unsigned when, unsigned period)
+int TimerManager::ResetTimerPeriod(int id,unsigned period)
+{
+	return ResetTimer(id,0,period,true);
+}
+
+bool TimerManager::ResetTimerTimeslice(int id, Timeslice const &new_timeslice)
+{
+	return ResetTimer(id,0,0,false,&new_timeslice)==0;
+}
+
+bool TimerManager::GetTimerTimeslice(int id, Timeslice &timeslice)
+{
+	Timer *timer_ptr = GetTimer( id, NULL );
+	if( !timer_ptr || !timer_ptr->timeslice ) {
+		return false;
+	}
+	timeslice = *timer_ptr->timeslice;
+	return true;
+}
+
+int TimerManager::ResetTimer(int id, unsigned when, unsigned period,
+							 bool recompute_when,
+							 Timeslice const *new_timeslice)
 {
 	Timer*			timer_ptr;
-	Event			handler;
-	Eventcpp		handlercpp;
-	Release			release;
-	Releasecpp		releasecpp;
-	Service*		s; 
-	int				is_cpp;
-	char*			event_descrip;
-	void*			data_ptr;
-	int				reset_dataptr = FALSE;
+	Timer*			trail_ptr;
 
-	dprintf( D_DAEMONCORE, "In reset_timer(), id=%d, time=%d, period=%d\n",id,when,period);
+	dprintf( D_DAEMONCORE,
+			 "In reset_timer(), id=%d, time=%d, period=%d\n",id,when,period);
 	if (timer_list == NULL) {
 		dprintf( D_DAEMONCORE, "Reseting Timer from empty list!\n");
 		return -1;
 	}
 
-	// now find the correct timer entry and set timer_ptr to point to it
-	if (timer_list->id == id) {
-		// timer to remove is the first on the list
-		timer_ptr = timer_list;
-	} else {
-		// timer to remove is not first on the list, so find it
-		for (timer_ptr = timer_list; timer_ptr != NULL && timer_ptr->id != id; 
-			 timer_ptr = timer_ptr->next) ;
-
-		if ( timer_ptr == NULL ) {
-			dprintf( D_DAEMONCORE, "Timer %d not found\n",id );
-			return -1;
-		}
+	timer_ptr = timer_list;
+	trail_ptr = NULL;
+	while ( timer_ptr && timer_ptr->id != id ) {
+		trail_ptr = timer_ptr;
+		timer_ptr = timer_ptr->next;
 	}
 
-	handler = timer_ptr->handler;
-	handlercpp = timer_ptr->handlercpp;
-	release = timer_ptr->release;
-	releasecpp = timer_ptr->releasecpp;
-	s = timer_ptr->service; 
-	is_cpp = timer_ptr->is_cpp;
-	event_descrip = timer_ptr->event_descrip;
-	data_ptr = timer_ptr->data_ptr;
-	if ( daemonCore->curr_dataptr == &(timer_ptr->data_ptr) )
-		reset_dataptr = TRUE;
-
-	// note that this call to NewTimer() will first call CancelTimer on the tid, 
-	// then create a new timer with the same tid.
-	if ( NewTimer(s, when, handler, handlercpp, release, releasecpp, 
-			event_descrip, period, NULL, id, is_cpp) != -1 ) {
-		// and dont forget to renew the users data_ptr
-		daemonCore->Register_DataPtr(data_ptr);
-		// and if a handler was resetting _its own_ tid entry, reset curr_dataptr to new value
-		if ( reset_dataptr == TRUE )
-			daemonCore->curr_dataptr = daemonCore->curr_regdataptr;
-		// now clear curr_regdataptr; the above NewTimer should appear "transparent"
-		// as far as the user code/API is concerned
-		daemonCore->curr_regdataptr = NULL;
-		// set flag letting Timeout() know if a timer handler reset itself.  note
-		// this is probably redundant since our call to NewTimer above called
-		// CancelTimer, and CancelTimer already set the did_reset flag.  But
-		// what the hell.
-		if ( did_reset == id )
-			did_reset = -1;
-		// DumpTimerList(D_DAEMONCORE | D_FULLDEBUG);
-		return 0;
-	} else {
+	if ( timer_ptr == NULL ) {
+		dprintf( D_ALWAYS, "Timer %d not found\n",id );
 		return -1;
 	}
+	if ( new_timeslice ) {
+		if( timer_ptr->timeslice == NULL ) {
+			timer_ptr->timeslice = new Timeslice( *new_timeslice );
+		}
+		else {
+			*timer_ptr->timeslice = *new_timeslice;
+		}
+
+		timer_ptr->when = timer_ptr->timeslice->getNextStartTime();
+	}
+	else if ( timer_ptr->timeslice ) {
+		dprintf( D_DAEMONCORE, "Timer %d with timeslice can't be reset\n",
+				 id );
+		return 0;
+	} else if( recompute_when ) {
+		time_t old_when = timer_ptr->when;
+
+		timer_ptr->when = timer_ptr->period_started + period;
+
+			// sanity check
+		int wait_time = (int)timer_ptr->when - (int)time(NULL);
+		if( wait_time > (int64_t)period ) {
+			dprintf(D_ALWAYS,
+					"ResetTimer() tried to set next call to %d (%s) %ds into"
+					" the future, which is larger than the new period %d.\n",
+					id,
+					timer_ptr->event_descrip ? timer_ptr->event_descrip : "",
+					wait_time,
+					period);
+
+				// start a new period now to restore sanity
+			timer_ptr->period_started = time(NULL);
+			timer_ptr->when = timer_ptr->period_started + period;
+		}
+
+		dprintf(D_FULLDEBUG,
+				"Changing period of timer %d (%s) from %u to %u "
+				"(added %ds to time of next scheduled call)\n",
+				id, 
+				timer_ptr->event_descrip ? timer_ptr->event_descrip : "",
+				timer_ptr->period,
+				period,
+				(int)timer_ptr->when - (int)old_when);
+	} else {
+		timer_ptr->period_started = time(NULL);
+		if ( when == TIMER_NEVER ) {
+			timer_ptr->when = TIME_T_NEVER;
+		} else {
+			timer_ptr->when = when + timer_ptr->period_started;
+		}
+	}
+	timer_ptr->period = period;
+
+	RemoveTimer( timer_ptr, trail_ptr );
+	InsertTimer( timer_ptr );
+
+	if ( in_timeout == timer_ptr ) {
+		// We're inside the handler for this timer. Let Timeout() know
+		// the timer has already been reset for its next call.
+		did_reset = true;
+	}
+
+	return 0;
 }
 
-int TimerManager::CancelTimer(int id, bool release_data_ptr /* = true */ )
+int TimerManager::CancelTimer(int id)
 {
 	Timer*		timer_ptr;
 	Timer*		trail_ptr;
-	Service		*service;
-	Release		release;
-	Releasecpp	releasecpp;
 
 	dprintf( D_DAEMONCORE, "In cancel_timer(), id=%d\n",id);
 	if (timer_list == NULL) {
@@ -275,68 +272,27 @@ int TimerManager::CancelTimer(int id, bool release_data_ptr /* = true */ )
 		return -1;
 	}
 
-	// now find the correct timer entry and set timer_ptr to point to it
-	if (timer_list->id == id) {
-		// timer to remove is the first on the list
-		if (timer_list->next == NULL) {
-			// timer to remove is first and is the only one on the list
-			timer_ptr = timer_list;
-			timer_list = NULL;			
-		} else {
-			// timer to remove is first but not the only one
-			timer_ptr = timer_list;
-			timer_list = timer_list->next;
-		}
+	timer_ptr = timer_list;
+	trail_ptr = NULL;
+	while ( timer_ptr && timer_ptr->id != id ) {
+		trail_ptr = timer_ptr;
+		timer_ptr = timer_ptr->next;
+	}
+
+	if ( timer_ptr == NULL ) {
+		dprintf( D_ALWAYS, "Timer %d not found\n",id );
+		return -1;
+	}
+
+	RemoveTimer( timer_ptr, trail_ptr );
+
+	if ( in_timeout == timer_ptr ) {
+		// We're inside the handler for this timer. Don't delete it,
+		// since Timeout() still needs it. Timeout() will delete it once
+		// it's done with it.
+		did_cancel = true;
 	} else {
-		// timer to remove is not first on the list, so find it
-		for (timer_ptr = timer_list; timer_ptr != NULL && timer_ptr->id != id; 
-			 timer_ptr = timer_ptr->next) {
-			trail_ptr = timer_ptr;
-		}
-
-		if ( timer_ptr == NULL ) {
-			dprintf( D_ALWAYS, "Timer %d not found\n",id );
-			return -1;
-		}
-
-		// remove from the linked list
-		trail_ptr->next = timer_ptr->next;
-	}
-
-	// free event_descrip
-	if ( timer_ptr->event_descrip ) {
-		daemonCore->free_descrip( timer_ptr->event_descrip);
-		timer_ptr->event_descrip = 0;
-	}
-
-	// free the data_ptr
-	service		= timer_ptr->service; 
-	release		= timer_ptr->release;
-	releasecpp	= timer_ptr->releasecpp;
-	if ( release_data_ptr ) {
-		if ( timer_ptr->is_cpp ) {
-			if ( releasecpp ) {
-				(service->*releasecpp)(timer_ptr->data_ptr);
-			}
-		} else {
-			if ( release ) {
-				(*release)(timer_ptr->data_ptr);
-			}
-		}
-	}
-
-	// set curr_dataptr to NULL if a handler is removing itself. 
-	if ( daemonCore->curr_dataptr == &(timer_ptr->data_ptr) )
-		daemonCore->curr_dataptr = NULL;
-	if ( daemonCore->curr_regdataptr == &(timer_ptr->data_ptr) )
-		daemonCore->curr_regdataptr = NULL;
-
-
-	delete timer_ptr;
-	
-	// set flag letting Timeout() know if a timer handler reset itself
-	if ( did_reset == id ) {
-		did_reset = -1;
+		DeleteTimer( timer_ptr );
 	}
 
 	return 0;
@@ -345,29 +301,20 @@ int TimerManager::CancelTimer(int id, bool release_data_ptr /* = true */ )
 void TimerManager::CancelAllTimers()
 {
 	Timer		*timer_ptr;
-	Service		*service;
-	Release		release;
-	Releasecpp	releasecpp;
 
 	while( timer_list != NULL ) {
 		timer_ptr = timer_list;
 		timer_list = timer_list->next;
-		service = timer_ptr->service; 
-		release = timer_ptr->release;
-		releasecpp = timer_ptr->releasecpp;
-		if ( timer_ptr->is_cpp ) {
-			if ( releasecpp ) {
-				(service->*releasecpp)(timer_ptr->data_ptr);
-			}
-		} else {
-			if ( release ) {
-				(*release)(timer_ptr->data_ptr);
-			}
+		if( in_timeout == timer_ptr ) {
+				// We get here if somebody calls exit from inside a timer.
+			did_cancel = true;
+		}
+		else {
+			DeleteTimer( timer_ptr );
 		}
-		daemonCore->free_descrip(timer_ptr->event_descrip);
-		delete timer_ptr;
 	}
 	timer_list = NULL;
+	list_tail = NULL;
 }
 
 // Timeout() is called when a select() time out.  Returns number of seconds
@@ -377,22 +324,12 @@ void TimerManager::CancelAllTimers()
 int
 TimerManager::Timeout()
 {
-	int				current_id;
-	unsigned		period;						// for periodic events
-	Event			handler;
-	Eventcpp		handlercpp;
-	Release			release;
-	Releasecpp		releasecpp;
-	Service*		s; 
 	int				result, timer_check_cntr;
 	time_t			now, time_sample;
-	int				is_cpp;
-	char*			event_descrip;
-	void*			data_ptr;
 	int				num_fires = 0;	// num of handlers called in this timeout
 
-	if ( in_timeout == TRUE ) {
-		dprintf(D_DAEMONCORE,"DaemonCore Timeout() called and in_timeout is TRUE\n");
+	if ( in_timeout != NULL ) {
+		dprintf(D_DAEMONCORE,"DaemonCore Timeout() called and in_timeout is non-NULL\n");
 		if ( timer_list == NULL ) {
 			result = 0;
 		} else {
@@ -403,7 +340,6 @@ TimerManager::Timeout()
 		}
 		return(result);
 	}
-	in_timeout = TRUE;
 		
 	dprintf( D_DAEMONCORE, "In DaemonCore Timeout()\n");
 
@@ -429,6 +365,8 @@ TimerManager::Timeout()
 	{
 		// DumpTimerList(D_DAEMONCORE | D_FULLDEBUG);
 
+		in_timeout = timer_list;
+
 		// In some cases, resuming from a suspend can cause the system
 		// clock to become temporarily skewed, causing crazy things to 
 		// happen with our timers (particularly for sending updates to
@@ -454,58 +392,49 @@ TimerManager::Timeout()
 			}
 		}
 
-		current_id = timer_list->id;
-		period = timer_list->period;
-		handler = timer_list->handler;
-		handlercpp = timer_list->handlercpp;
-		release = timer_list->release;
-		releasecpp = timer_list->releasecpp;
-		s = timer_list->service; 
-		is_cpp = timer_list->is_cpp;
-		event_descrip = strdup(timer_list->event_descrip);
-		data_ptr = timer_list->data_ptr;
-		Timeslice timeslice = timer_list->timeslice;
-		bool has_timeslice = timer_list->has_timeslice;
-
 		// Update curr_dataptr for GetDataPtr()
-		daemonCore->curr_dataptr = &(timer_list->data_ptr);
+		curr_dataptr = &(in_timeout->data_ptr);
 
 		// Initialize our flag so we know if ResetTimer was called.
-		did_reset = current_id;
+		did_reset = false;
+		did_cancel = false;
 
 		// Log a message before calling handler, but only if
 		// D_FULLDEBUG is also enabled.
 		if (DebugFlags & D_FULLDEBUG) {
 			dprintf(D_COMMAND, "Calling Timer handler %d (%s)\n",
-					current_id, event_descrip);
+					in_timeout->id, in_timeout->event_descrip);
 		}
 
-		if( has_timeslice ) {
-			timeslice.setStartTimeNow();
+		if( in_timeout->timeslice ) {
+			in_timeout->timeslice->setStartTimeNow();
 		}
 
 		// Now we call the registered handler.  If we were told that the handler
 		// is a c++ method, we call the handler from the c++ object referenced 
 		// by service*.  If we were told the handler is a c function, we call
 		// it and pass the service* as a parameter.
-		if ( timer_list->is_cpp ) {
-			(s->*handlercpp)();			// typedef int (*Eventcpp)(int)
+		if ( in_timeout->handlercpp ) {
+			// typedef int (*TimerHandlercpp)()
+			((in_timeout->service)->*(in_timeout->handlercpp))();
 		} else {
-			(*handler)(s);				// typedef int (*Event)(Service*,int)
+			// typedef int (*TimerHandler)()
+			(*(in_timeout->handler))();
 		}
 
-		if( has_timeslice ) {
-			timeslice.setFinishTimeNow();
+		if( in_timeout->timeslice ) {
+			in_timeout->timeslice->setFinishTimeNow();
 		}
 
 		if (DebugFlags & D_FULLDEBUG) {
-			if( has_timeslice ) {
+			if( in_timeout->timeslice ) {
 				dprintf(D_COMMAND, "Return from Timer handler %d (%s) - took %.3fs\n",
-						current_id, event_descrip, timeslice.getLastDuration() );
+						in_timeout->id, in_timeout->event_descrip,
+						in_timeout->timeslice->getLastDuration() );
 			}
 			else {
 				dprintf(D_COMMAND, "Return from Timer handler %d (%s)\n",
-						current_id, event_descrip);
+						in_timeout->id, in_timeout->event_descrip);
 			}
 		}
 
@@ -513,33 +442,41 @@ TimerManager::Timeout()
 		daemonCore->CheckPrivState();
 
 		// Clear curr_dataptr
-		daemonCore->curr_dataptr = NULL;
+		curr_dataptr = NULL;
 
-		/* Watch out for cancel_timer() called in the users handler with only 
-		 * one item in list which makes timer_list go to NULL */
-		if (timer_list != NULL  && did_reset == current_id) {
+		if ( did_cancel ) {
+			// Timer was canceled inside its handler. All we need to do
+			// is delete it.
+			DeleteTimer( in_timeout );
+		} else if ( !did_reset ) {
 			// here we remove the timer we just serviced, or renew it if it is 
-			// periodic.  note calls to CancelTimer are safe even if the user's
-			// handler called CancelTimer as well...
-			if ( period > 0 || has_timeslice ) {
-				// timer is periodic, renew it.  note that this call to NewTimer() 
-				// will first call CancelTimer on the expired timer, then renew it.
-				if ( NewTimer(s, period, handler, handlercpp, release, releasecpp,
-						event_descrip, period, has_timeslice ? &timeslice : NULL, 
-						current_id, is_cpp) != -1 ) {
-					// and dont forget to renew the users data_ptr
-					daemonCore->Register_DataPtr(data_ptr);
-					// now clear curr_dataptr; the above NewTimer should appear "transparent"
-					// as far as the user code/API is concerned
-					daemonCore->curr_dataptr = NULL;
+			// periodic.
+
+			// If a new timer was added at a time in the past
+			// (possible when resetting a timeslice timer), then
+			// it may have landed before the timer we just processed,
+			// meaning that we cannot assume prev==NULL in the call
+			// to RemoveTimer() below.
+
+			Timer *prev = NULL;
+			ASSERT( GetTimer(in_timeout->id,&prev) == in_timeout );
+			RemoveTimer( in_timeout, prev );
+
+			if ( in_timeout->period > 0 || in_timeout->timeslice ) {
+				in_timeout->period_started = time(NULL);
+				in_timeout->when = in_timeout->period_started;
+				if ( in_timeout->timeslice ) {
+					in_timeout->when += in_timeout->timeslice->getTimeToNextRun();
+				} else {
+					in_timeout->when += in_timeout->period;
 				}
+				InsertTimer( in_timeout );
 			} else {
 				// timer is not perodic; it is just a one-time event.  we just called
-				// the handler, so now remove the timer from out list.  
-				CancelTimer(current_id);
+				// the handler, so now just delete it. 
+				DeleteTimer( in_timeout );
 			}
 		}
-		free( event_descrip );
 	}  // end of while loop
 
 
@@ -557,15 +494,17 @@ TimerManager::Timeout()
 	}
 
 	dprintf( D_DAEMONCORE, "DaemonCore Timeout() Complete, returning %d \n",result);
-	in_timeout = FALSE;
+	in_timeout = NULL;
 	return(result);
 }
 
+#define IS_ZERO(_value_) \
+	(  ( (_value_) >= -0.000001 ) && ( (_value_) <= 0.000001 )  )
 
 void TimerManager::DumpTimerList(int flag, const char* indent)
 {
-	Timer	*timer_ptr;
-	char	*ptmp;
+	Timer		*timer_ptr;
+	const char	*ptmp;
 
 	// we want to allow flag to be "D_FULLDEBUG | D_DAEMONCORE",
 	// and only have output if _both_ are specified by the user
@@ -589,27 +528,27 @@ void TimerManager::DumpTimerList(int flag, const char* indent)
 			ptmp = "NULL";
 
 		MyString slice_desc;
-		if( !timer_ptr->has_timeslice ) {
+		if( !timer_ptr->timeslice ) {
 			slice_desc.sprintf("period = %d, ", timer_ptr->period);
 		}
 		else {
 			slice_desc.sprintf_cat("timeslice = %.3g, ",
-								   timer_ptr->timeslice.getTimeslice());
-			if( timer_ptr->timeslice.getDefaultInterval() ) {
+								   timer_ptr->timeslice->getTimeslice());
+			if( !IS_ZERO(timer_ptr->timeslice->getDefaultInterval()) ) {
 				slice_desc.sprintf_cat("period = %.1f, ",
-								   timer_ptr->timeslice.getDefaultInterval());
+								   timer_ptr->timeslice->getDefaultInterval());
 			}
-			if( timer_ptr->timeslice.getInitialInterval() ) {
+			if( !IS_ZERO(timer_ptr->timeslice->getInitialInterval()) ) {
 				slice_desc.sprintf_cat("initial period = %.1f, ",
-								   timer_ptr->timeslice.getInitialInterval());
+								   timer_ptr->timeslice->getInitialInterval());
 			}
-			if( timer_ptr->timeslice.getMinInterval() ) {
+			if( !IS_ZERO(timer_ptr->timeslice->getMinInterval()) ) {
 				slice_desc.sprintf_cat("min period = %.1f, ",
-								   timer_ptr->timeslice.getMinInterval());
+								   timer_ptr->timeslice->getMinInterval());
 			}
-			if( timer_ptr->timeslice.getMaxInterval() ) {
+			if( !IS_ZERO(timer_ptr->timeslice->getMaxInterval()) ) {
 				slice_desc.sprintf_cat("max period = %.1f, ",
-								   timer_ptr->timeslice.getMaxInterval());
+								   timer_ptr->timeslice->getMaxInterval());
 			}
 		}
 		dprintf(flag, 
@@ -649,3 +588,106 @@ void TimerManager::Start()
 		}		
 	}
 }
+
+void TimerManager::RemoveTimer( Timer *timer, Timer *prev )
+{
+	if ( timer == NULL || ( prev && prev->next != timer ) ||
+		 ( !prev && timer != timer_list ) ) {
+		EXCEPT( "Bad call to TimerManager::RemoveTimer()!\n" );
+	}
+
+	if ( timer == timer_list ) {
+		timer_list = timer_list->next;
+	}
+	if ( timer == list_tail ) {
+		list_tail = prev;
+	}
+	if ( prev ) {
+		prev->next = timer->next;
+	}
+}
+
+void TimerManager::InsertTimer( Timer *new_timer )
+{
+	if ( timer_list == NULL ) {
+		// list is empty, place ours in front
+		timer_list = new_timer;
+		list_tail = new_timer;
+		new_timer->next = NULL;
+			// since we have a new first timer, we must wake up select
+		daemonCore->Wake_up_select();
+	} else {
+		// list is not empty, so keep timer_list ordered from soonest to
+		// farthest (i.e. sorted on "when").
+		// Note: when doing the comparisons, we always use "<" instead
+		// of "<=" -- this makes certain we "round-robin" across 
+		// timers that constantly reset themselves to zero.
+		if ( new_timer->when < timer_list->when ) {
+			// make the this new timer first in line
+			new_timer->next = timer_list;
+			timer_list = new_timer;
+			// since we have a new first timer, we must wake up select
+			daemonCore->Wake_up_select();
+		} else if ( new_timer->when == TIME_T_NEVER ) {
+			// Our new timer goes to the very back of the list.
+			new_timer->next = NULL;
+			list_tail->next = new_timer;
+			list_tail = new_timer;
+		} else {
+			Timer* timer_ptr;
+			Timer* trail_ptr = NULL;
+			for (timer_ptr = timer_list; timer_ptr != NULL; 
+				 timer_ptr = timer_ptr->next ) 
+			{
+				if (new_timer->when < timer_ptr->when) {
+					break;
+				}
+				trail_ptr = timer_ptr;
+			}
+			ASSERT( trail_ptr );
+			new_timer->next = timer_ptr;
+			trail_ptr->next = new_timer;
+			if ( trail_ptr == list_tail ) {
+				list_tail = new_timer;
+			}
+		}
+	}
+}
+
+void TimerManager::DeleteTimer( Timer *timer )
+{
+	// free the data_ptr
+	if ( timer->releasecpp ) {
+		((timer->service)->*(timer->releasecpp))(timer->data_ptr);
+	} else if ( timer->release ) {
+		(*(timer->release))(timer->data_ptr);
+	}
+
+	// free event_descrip
+	free( timer->event_descrip );
+
+	// set curr_dataptr to NULL if a handler is removing itself. 
+	if ( curr_dataptr == &(timer->data_ptr) )
+		curr_dataptr = NULL;
+	if ( curr_regdataptr == &(timer->data_ptr) )
+		curr_regdataptr = NULL;
+
+	delete timer->timeslice;
+	delete timer;
+}
+
+Timer *TimerManager::GetTimer( int id, Timer **prev )
+{
+	Timer *timer_ptr = timer_list;
+	if( prev ) {
+		*prev = NULL;
+	}
+	while ( timer_ptr && timer_ptr->id != id ) {
+		if( prev ) {
+			*prev = timer_ptr;
+		}
+		timer_ptr = timer_ptr->next;
+	}
+
+	return timer_ptr;
+}
