@@ -35,11 +35,13 @@
 #include "prioritysimplelist.h"
 #include "throttle_by_category.h"
 #include "MyString.h"
+#include "dagman_recursive_submit.h"
+#include "jobstate_log.h"
 
 // NOTE: must be kept in sync with Job::job_type_t
 enum Log_source{
-  CONDORLOG,
-  DAPLOG
+  CONDORLOG = Job::TYPE_CONDOR,
+  DAPLOG = Job::TYPE_STORK
 };
 
 // Which layer of splices do we want to lift?
@@ -62,14 +64,19 @@ class OwnedMaterials
 	public:
 		// this structure owns the containers passed to it, but not the memory 
 		// contained in the containers...
-		OwnedMaterials(ExtArray<Job*> *a) :
-				nodes (a) {};
+		OwnedMaterials(ExtArray<Job*> *a, ThrottleByCategory *tr,
+				bool reject, MyString firstRejectLoc ) :
+				nodes (a), throttles (tr), _reject(reject),
+				_firstRejectLoc(firstRejectLoc) {};
 		~OwnedMaterials() 
 		{
 			delete nodes;
 		};
 
 	ExtArray<Job*> *nodes;
+	ThrottleByCategory *throttles;
+	bool _reject;
+	MyString _firstRejectLoc;
 };
 
 //------------------------------------------------------------------------
@@ -97,7 +104,7 @@ class Dag {
 			   have an error determining the job log files
 		@param useDagDir run DAGs in directories from DAG file paths
 		       if true
-		@param maxIdleJobProcs the maximum number of idle job procss to
+		@param maxIdleJobProcs the maximum number of idle job procs to
 			   allow at one time (0 means unlimited)
 		@param retrySubmitFirst whether, when a submit fails for a node's
 		       job, to put the node at the head of the ready queue
@@ -110,10 +117,16 @@ class Dag {
 			   job procs are prohibited
 		@param submitDepthFirst whether ready nodes should be submitted
 			   in depth-first (as opposed to breadth-first) order
-		@param fundUserLogs whether or not log files for the submit files
-				should be recursively dug out of the dag file and any
-				splices it contains. Usually this is true for the root
-				dag, and false for any splices brought in by the root dag.
+		@param The user log file to be used for nodes whose submit files do
+				not specify a log file.
+		@param isSplice is a boolean which lets the dag object know whether
+				of not it is a splicing dag, or the toplevel dag. We don't
+				wan't to allocate some regulated resources we won't need
+				if we are a splice. It is true for a top level dag, and false
+				for a splice.
+		@param spliceScope a string containing the names of all splices
+				on the "path" to the top-level DAG (e.g., "A+B+"), or
+				"root" for the top-level DAG.
     */
 
     Dag( /* const */ StringList &dagFiles,
@@ -124,7 +137,9 @@ class Dag {
 		 bool retryNodeFirst, const char *condorRmExe,
 		 const char *storkRmExe, const CondorID *DAGManJobId,
 		 bool prohibitMultiJobs, bool submitDepthFirst,
-		 bool findUserLogs = true );
+		 const char *defaultNodeLog, bool generateSubdagSubmits,
+		 const SubmitDagDeepOptions *submitDagDeepOpts,
+		 bool isSplice = false, const MyString &spliceScope = "root" );
 
     ///
     ~Dag();
@@ -137,10 +152,6 @@ class Dag {
 		_abortOnScarySubmit = abortOnScarySubmit;
 	}
 
-	/** Initialize log files, lock files, etc.
-	*/
-	void InitializeDagFiles( bool deleteOldLogs );
-
     /** Prepare DAG for initial run.  Call this function ONLY ONCE.
         @param recovery specifies if this is a recovery
         @return true: successful, false: failure
@@ -163,8 +174,7 @@ class Dag {
     */
 
     
-    //    bool DetectLogGrowth();
-    bool DetectCondorLogGrowth();         //<--DAP 
+    bool DetectCondorLogGrowth();
     bool DetectDaPLogGrowth();            //<--DAP
 
     /** Force the Dag to process all new events in the condor log file.
@@ -250,6 +260,16 @@ class Dag {
 	*/
 	void ProcessNotIdleEvent(Job *job);
 
+	/** Process a held event for a job.
+		@param The job corresponding to this event.
+	*/
+	void ProcessHeldEvent(Job *job, const ULogEvent *event);
+
+	/** Process a released event for a job.
+		@param The job corresponding to this event.
+	*/
+	void ProcessReleasedEvent(Job *job);
+
     /** Get pointer to job with id jobID
         @param the handle of the job in the DAG
         @return address of Job object, or NULL if not found
@@ -284,7 +304,7 @@ class Dag {
 		@param allowEvents what "bad" events to treat as non-fatal (as
 			   opposed to fatal) errors; see check_events.h for values.
     */
-	void SetAllowEvents( int allowEvents);
+	void SetAllowEvents( int allowEvents );
 
     /// Print the list of jobs to stdout (for debugging).
     void PrintJobList() const;
@@ -313,17 +333,32 @@ class Dag {
     /** @return the number of PRE scripts currently running
      */
     inline int NumPreScriptsRunning() const
-		{ return _preScriptQ->NumScriptsRunning(); }
+	{
+		// Do not call this function when the dag is being used to parse
+		// a splice.
+		ASSERT( _isSplice == false ); 
+		return _preScriptQ->NumScriptsRunning();
+	}
 
     /** @return the number of POST scripts currently running
      */
     inline int NumPostScriptsRunning() const
-		{ return _postScriptQ->NumScriptsRunning(); }
+	{
+		// Do not call this function when the dag is being used to parse
+		// a splice.
+		ASSERT( _isSplice == false ); 
+		return _postScriptQ->NumScriptsRunning(); 
+	}
 
     /** @return the total number of PRE/POST scripts currently running
      */
     inline int NumScriptsRunning() const
-		{ return NumPreScriptsRunning() + NumPostScriptsRunning(); }
+	{
+		// Do not call this function when the dag is being used to parse
+		// a splice.
+		ASSERT( _isSplice == false ); 
+		return NumPreScriptsRunning() + NumPostScriptsRunning();
+	}
 
 	/** @return the number of nodes currently in the status
 	 *          Job::STATUS_PRERUN.
@@ -343,7 +378,35 @@ class Dag {
 	inline int ScriptRunNodeCount() const
 		{ return _preRunNodeCount + _postRunNodeCount; }
 
-	inline bool Done() const { return NumNodesDone() == NumNodes(); }
+		/** Determine whether the DAG has finished running (whether
+			successfully or unsuccessfully).
+	    	(If no jobs are submitted and no scripts are running, but the
+		    dag is not complete, then at least one job failed, or a cycle
+			exists.)
+			@return true iff the DAG is finished
+		*/
+	inline bool FinishedRunning() const { return NumJobsSubmitted() == 0 &&
+				NumNodesReady() == 0 && ScriptRunNodeCount() == 0; }
+
+		/** Determine whether the DAG is successfully completed.
+			@return true iff the DAG is successfully completed
+		*/
+	inline bool DoneSuccess() const { return NumNodesDone() == NumNodes(); }
+
+		/** Determine whether the DAG is finished, but failed (because
+			of a node job failure, etc.).
+			@return true iff the DAG is finished but failed
+		*/
+	inline bool DoneFailed() const { return FinishedRunning() &&
+				NumNodesFailed() > 0; }
+
+		/** Determine whether the DAG is finished because of a cycle in
+			the DAG.  (Note that this method sometimes incorrectly returns
+			true for errors other than cycles in the DAG.  wenger 2010-07-30.)
+			@return true iff the DAG is finished but there is a cycle
+		*/
+	inline bool DoneCycle() { return FinishedRunning() &&
+				NumNodesFailed() == 0; }
 
 		/** Submit all ready jobs, provided they are not waiting on a
 			parent job or being throttled.
@@ -375,17 +438,19 @@ class Dag {
         @param datafile The original DAG file
 		@param multiDags Whether we have multiple DAGs
 		@param maxRescueDagNum the maximum legal rescue DAG number
+		@param parseFailed whether parsing the DAG(s) failed
     */
-    void Rescue (const char * datafile, bool multiDags,
-				int maxRescueDagNum) /* const */;
+    void Rescue (const char * dagFile, bool multiDags,
+				int maxRescueDagNum, bool parseFailed = false) /* const */;
 
     /** Creates a DAG file based on the DAG in memory, except all
         completed jobs are premarked as DONE.
         @param rescue_file The name of the rescue file to generate
         @param datafile The original DAG file
+		@param parseFailed whether parsing the DAG(s) failed
     */
     void WriteRescue (const char * rescue_file,
-				const char * datafile) /* const */;
+				const char * dagFile, bool parseFailed = false) /* const */;
 
 	int PreScriptReaper( const char* nodeName, int status );
 	int PostScriptReaper( const char* nodeName, int status );
@@ -400,19 +465,40 @@ class Dag {
     /* Detects cycle within dag submitted by user
 	   @return true if there is cycle
 	*/
-	bool isCycle ();
+	bool isCycle();
 
 	// max number of PRE & POST scripts to run at once (0 means no limit)
     const int _maxPreScripts;
     const int _maxPostScripts;
 
-	void SetDotFileName(char *dot_file_name);
-	void SetDotIncludeFileName(char *include_file_name);
+	void SetDotFileName(const char *dot_file_name);
+	void SetDotIncludeFileName(const char *include_file_name);
 	void SetDotFileUpdate(bool update_dot_file)       { _update_dot_file    = update_dot_file; }
 	void SetDotFileOverwrite(bool overwrite_dot_file) { _overwrite_dot_file = overwrite_dot_file; }
 	bool GetDotFileUpdate(void)                       { return _update_dot_file; }
 	void DumpDotFile(void);
 
+	void SetNodeStatusFileName( const char *statusFileName,
+				int minUpdateTime );
+	void DumpNodeStatus( bool held, bool removed );
+
+		/** Set the reject flag to true for this DAG; if it hasn't been
+			previously set, update the location info for the reject
+			directive.
+			@param location: the file and line # of the REJECT keyword
+		*/
+	void SetReject( const MyString &location );
+
+		/** Get the reject setting of this DAG (true means we shouldn't
+			actually run the DAG).
+			@param firstLocation: the location of the first REJECT
+				directive we encountered in parsing the DAG (including
+				splices)
+		*/
+	bool GetReject( MyString &firstLocation );
+
+	void SetJobstateLogFileName( const char *logFileName );
+
 	void CheckAllJobs();
 
 		/** Returns a delimited string listing the node names of all
@@ -423,10 +509,7 @@ class Dag {
 
 	int NumIdleJobProcs() const { return _numIdleJobProcs; }
 
-		/** Get the number of Stork (nee DaP) log files.
-			@return The number of Stork log files (can be 0).
-		*/
-	int GetStorkLogCount() const { return _storkLogFiles.number(); }
+	int NumHeldJobProcs() const { return _numHeldJobProcs; }
 
 		/** Print the number of deferrals during the run (caused
 		    by MaxJobs, MaxIdle, MaxPre, or MaxPost).
@@ -447,8 +530,9 @@ class Dag {
 		// are normal exe return codes, -1 to -64 represent catching
 		// exe signals 1 to 64, and -1000 and below represent DAGMan,
 		// batch-system, or other external errors
-	const int DAG_ERROR_CONDOR_SUBMIT_FAILED;
-	const int DAG_ERROR_CONDOR_JOB_ABORTED;
+	static const int DAG_ERROR_CONDOR_SUBMIT_FAILED;
+	static const int DAG_ERROR_CONDOR_JOB_ABORTED;
+	static const int DAG_ERROR_LOG_MONITOR_ERROR;
 
 		// The maximum signal we can deal with in the error-reporting
 		// code.
@@ -470,6 +554,12 @@ class Dag {
 		// Node category throttle information for the DAG.
 	ThrottleByCategory		_catThrottles;
 
+		/**	Go through all of the node throttling categories, and print
+			a warning if any categories have no nodes assigned to them,
+			or don't have their throttle set.
+		*/
+	void CheckThrottleCats();
+
 	int MaxJobsSubmitted(void) { return _maxJobsSubmitted; }
 
 	bool AllowLogError(void) { return _allowLogError; }
@@ -494,11 +584,28 @@ class Dag {
 
 	bool SubmitDepthFirst(void) { return _submitDepthFirst; }
 
+	const char *DefaultNodeLog(void) { return _defaultNodeLog; }
+
+	const bool GenerateSubdagSubmits(void) { return _generateSubdagSubmits; }
+
 	StringList& DagFiles(void) { return _dagFiles; }
 
-		// The absolute maximum allowed rescue DAG number (the real maximum
-		// is normally configured lower).
-	static const int ABS_MAX_RESCUE_DAG_NUM;
+	/** Determine whether a job is a NOOP job based on the Condor ID.
+		@param the Condor ID of the job
+		@return true iff the job is a NOOP
+	*/
+	static bool JobIsNoop( const CondorID &id ) {
+		return (id._cluster == 0) && (id._proc == Job::NOOP_NODE_PROCID);
+	}
+
+	/** Get the part of the CondorID that we're indexing by (cluster ID
+		for "normal" jobs, subproc ID for NOOP jobs).
+		@param the Condor ID of the job
+		@return the part of the ID to index by
+	*/
+	static int GetIndexID( const CondorID &id ) {
+		return JobIsNoop( id ) ? id._subproc : id._cluster; 
+	}
 
 	// return same thing as HashTable.insert()
 	int InsertSplice(MyString spliceName, Dag *splice_dag);
@@ -535,7 +642,8 @@ class Dag {
 	OwnedMaterials* RelinquishNodeOwnership(void);
 
 	// Take an array from RelinquishNodeOwnership) and store it in my self.
-	void AssumeOwnershipofNodes(OwnedMaterials *om);
+	void AssumeOwnershipofNodes(const MyString &spliceName,
+				OwnedMaterials *om);
 
 	// This must be called after the toplevel dag has been parsed and
 	// the splices lifted. It will resolve the use of $(JOB) in the value
@@ -553,7 +661,15 @@ class Dag {
 	// node, unless it is an absolute path, in which case we ignore it.
 	void PropogateDirectoryToAllNodes(void);
 
-  protected:
+	/** Set the maximum number of job holds before a node is declared
+		a failure.
+		@param the maximum number of job holds before failure
+	*/
+	void SetMaxJobHolds(int maxJobHolds) { _maxJobHolds = maxJobHolds; }
+
+	JobstateLog &GetJobstateLog() { return _jobstateLog; }
+
+  private:
 
 	// If this DAG is a splice, then this is what the DIR was set to, it 
 	// defaults to ".".
@@ -581,13 +697,6 @@ class Dag {
 	*/
 	void PrintDagFiles( /* const */ StringList &dagFiles );
 
-	/** Find all Condor (not DaP) log files associated with this DAG.
-	    @param The list of DAG files being run.
-		@param useDagDir run DAGs in directories from DAG file paths
-		       if true
-	*/
-	void FindLogFiles( /* const */ StringList &dagFiles, bool useDagDir );
-
     /* Prepares to submit job by running its PRE Script if one exists,
        otherwise adds job to _readyQ and calls SubmitReadyJobs()
 	   @param The job to start
@@ -640,13 +749,21 @@ class Dag {
 	*/
 	void UpdateJobCounts( Job *node, int change );
 
-    // add job to termination queue and report termination to all
-    // child jobs by removing job ID from each child's waiting queue
-    void TerminateJob( Job* job, bool bootstrap = false );
+	/** Add job to termination queue and report termination to all
+		child jobs by removing job ID from each child's waiting queue.
+		Note that this method should be called only in the case of
+		*successful* termination.
+		@param The job (node) just terminated
+		@param Whether we're in recovery mode (re-reading log events)
+		@param Whether we're in bootstrap mode (dealing with jobs marked
+			DONE in a rescue DAG)
+	*/
+    void TerminateJob( Job* job, bool recovery, bool bootstrap = false );
   
 	void PrintEvent( debug_level_t level, const ULogEvent* event,
-					 Job* node );
+					 Job* node, bool recovery );
 
+	// Retry a node that we ran, but which failed.
 	void RestartNode( Job *node, bool recovery );
 
 	/* DFS number the jobs in the DAG in order to detect cycle*/
@@ -679,51 +796,42 @@ class Dag {
 
 		/** Get the appropriate hash table for event ID->node mapping,
 			according to whether this is a Condor or Stork node.
+			@param whether the node is a NOOP node
 			@param the node type/logsource (Condor or Stork) (see
 				Log_source and Job::job_type_t)
 			@return a pointer to the appropriate hash table
 		*/
-	HashTable<int, Job *> *		GetEventIDHash(int jobType);
+	HashTable<int, Job *> *		GetEventIDHash(bool isNoop, int jobType);
 
 		/** Get the appropriate hash table for event ID->node mapping,
 			according to whether this is a Condor or Stork node.
+			@param whether the node is a NOOP node
 			@param the node type/logsource (Condor or Stork) (see
 				Log_source and Job::job_type_t)
 			@return a pointer to the appropriate hash table
 		*/
-	const HashTable<int, Job *> *		GetEventIDHash(int jobType) const;
+	const HashTable<int, Job *> *		GetEventIDHash(bool isNoop,
+				int jobType) const;
 
 	// run DAGs in directories from DAG file paths if true
 	bool _useDagDir;
 
-    // name of consolidated condor log
-	StringList _condorLogFiles;
-
-    // Documentation on ReadUserLog is present in condor_c++_util
+    // Documentation on ReadUserLog is present in condor_utils
 	ReadMultipleUserLogs _condorLogRdr;
 
-    //
-    bool          _condorLogInitialized;
-
-    //-->DAP
-		// The list of all Stork log files (generated from the relevant
-		// submit files).
-	StringList	_storkLogFiles;
-
 		// Object to read events from Stork logs.
 	ReadMultipleUserLogs	_storkLogRdr;
 
-		// Whether the Stork (nee DaP) log(s) have been successfully
-		// initialized.
-    bool          _dapLogInitialized;
-    //<--DAP
-
 		/** Get the total number of node job user log files we'll be
 			accessing.
 			@return The total number of log files.
 		*/
-	int TotalLogFileCount() { return _condorLogFiles.number() +
-				_storkLogFiles.number(); }
+	int TotalLogFileCount() { return CondorLogFileCount() +
+				StorkLogFileCount(); }
+
+	int CondorLogFileCount() { return _condorLogRdr.totalLogFileCount(); }
+
+	int StorkLogFileCount() { return _storkLogRdr.totalLogFileCount(); }
 
     /// List of Job objects
     List<Job>     _jobs;
@@ -740,6 +848,9 @@ class Dag {
 	// procs in the same cluster map to the same node).
 	HashTable<int, Job *>			_storkIDHash;
 
+	// NOOP nodes are indexed by subprocID.
+	HashTable<int, Job *>			_noopIDHash;
+
     // Number of nodes that are done (completed execution)
     int _numNodesDone;
     
@@ -762,6 +873,9 @@ class Dag {
 		// means unlimited.
     const int _maxIdleJobProcs;
 
+		// The number of DAG job procs currently held.
+	int _numHeldJobProcs;
+
 		// Whether to allow the DAG to run even if we have an error
 		// determining the job log files.
 	bool		_allowLogError;
@@ -818,6 +932,20 @@ class Dag {
 	void DumpDotFileArcs(FILE *temp_dot_file);
 	void ChooseDotFileName(MyString &dot_file_name);
 
+		// Name of node status file.
+	char *_statusFileName;
+		
+		// Whether things have changed since the last time the file
+		// was written.
+	bool _statusFileOutdated;
+		
+		// Minimum time between updates (so we can avoid trying to
+		// write the file too often, e.g., for large DAGs).
+	int _minStatusUpdateTime;
+
+		// Last time the status file was written.
+	time_t _lastStatusUpdateTimestamp;
+
 		// Separate event checkers for Condor and Stork here because
 		// IDs could collide.
 	CheckEvents	_checkCondorEvents;
@@ -880,6 +1008,54 @@ class Dag {
 		// retry.
 	static const CondorID	_defaultCondorId;
 
+		// Whether having node job log files on NFS is an error (vs.
+		// just a warning).
+	bool	_nfsLogIsError;
+
+		// The user log file to be used for nodes whose submit files do
+		// not specify a log file.
+	const char *_defaultNodeLog;
+
+		// Whether to generate the .condor.sub files for sub-DAGs
+		// at run time (just before the node is submitted).
+	bool	_generateSubdagSubmits;
+
+		// Options for running condor_submit_dag on nested DAGs.
+	const SubmitDagDeepOptions *_submitDagDeepOpts;
+
+		// Dag objects are used to parse splice files, which are like include
+		// files that ultimately result in a larger in memory dag. To toplevel
+		// dag will have this be false, and any included splices will be true.
+		// We use this knowledge to not allocate precious daemoncore resources
+		// because we know we aren't going to have an "executing" dag object
+		// which is also a splice.
+	bool _isSplice;
+
+		// The splice "scope" for this DAG (e.g., "A+B+", or "root").
+		// This is currently just used for diagnostic messages
+		// (wenger 2010-06-07)
+	MyString _spliceScope;
+
+		// The maximum fake subprocID we see in recovery mode (needed to
+		// initialize the ID for subsequent fake events so IDs don't
+		// collide).
+	int _recoveryMaxfakeID;
+
+		// The maximum number of times a node job can go on hold before
+		// we declare it a failure and remove it; 0 means no limit.
+	int _maxJobHolds;
+
+		// If this flag is set, we shouldn't actually run this DAG file
+		// (this is set when running a "rescue" produced when DAGMan is
+		// run with -DumpRescue and parsing the DAG fails).
+	bool _reject;
+
+		// The file and line number where we first found a REJECT
+		// specification, if any.
+	MyString _firstRejectLoc;
+
+		// The object for logging to the jobstate.log file (for Pegasus).
+	JobstateLog _jobstateLog;
 };
 
 #endif /* #ifndef DAG_H */
