@@ -40,7 +40,7 @@
 
 void ExitSuccess();
 
-	// From condor_c++_util/condor_config.C
+	// From condor_utils/condor_config.C
 	// Note: these functions are declared 'extern "C"' where they're
 	// implemented; if we don't do that here we get a link failure
 	// (I think because of the name mangling).  wenger 2007-02-09.
@@ -49,7 +49,7 @@ extern "C" void process_config_source( char* file, char* name,
 extern "C" bool is_piped_command(const char* filename);
 
 //---------------------------------------------------------------------------
-DECL_SUBSYSTEM( "DAGMAN", SUBSYSTEM_TYPE_DAEMON );
+DECL_SUBSYSTEM( "DAGMAN", SUBSYSTEM_TYPE_DAGMAN );
 
 static char* lockFileName = NULL;
 
@@ -75,6 +75,13 @@ static void Usage() {
             "\t\t[-DoRescueFrom <int N>]\n"
 			"\t\t[-AllowVersionMismatch]\n"
 			"\t\t[-DumpRescue]\n"
+			"\t\t[-Verbose]\n"
+			"\t\t[-Force]\n"
+			"\t\t[-Notification <never|always|complete|error>]\n"
+			"\t\t[-Dagman <dagman_executable>]\n"
+			"\t\t[-Outfile_dir <directory>]\n"
+			"\t\t[-Update_submit]\n"
+			"\t\t[-Import_env]\n"
             "\twhere NAME is the name of your DAG.\n"
             "\tdefault -Debug is -Debug %d\n", DEBUG_NORMAL);
 	DC_Exit( EXIT_ERROR );
@@ -96,8 +103,9 @@ Dagman::Dagman() :
 	storkSubmitExe (NULL),
 	storkRmExe (NULL),
 	submit_delay (0),
-	max_submit_attempts (0),
-	max_submits_per_interval (1000), // so Coverity is happy
+	max_submit_attempts (6),
+	max_submits_per_interval (5), // so Coverity is happy
+	m_user_log_scan_interval (5),
 	primaryDagFile (""),
 	multiDags (false),
 	startup_cycle_detect (false), // so Coverity is happy
@@ -107,19 +115,22 @@ Dagman::Dagman() :
 	retrySubmitFirst (true), // so Coverity is happy
 	retryNodeFirst (false), // so Coverity is happy
 	mungeNodeNames (true), // so Coverity is happy
-	deleteOldLogs (true),
 	prohibitMultiJobs (false), // so Coverity is happy
 	abortDuplicates (true), // so Coverity is happy
 	submitDepthFirst (false), // so Coverity is happy
 	abortOnScarySubmit (true), // so Coverity is happy
-	pendingReportInterval (10 * 60), // so Coverity is happy
+	pendingReportInterval (10 * 60), // 10 minutes
 	_dagmanConfigFile (NULL), // so Coverity is happy
-	autoRescue(false),
+	autoRescue(true),
 	doRescueFrom(0),
-	maxRescueDagNum(ABS_MAX_RESCUE_DAG_NUM),
+	maxRescueDagNum(MAX_RESCUE_DAG_DEFAULT),
 	rescueFileToRun(""),
-	dumpRescueDag(false)
+	dumpRescueDag(false),
+	_defaultNodeLog(NULL),
+	_generateSubdagSubmits(true),
+	_maxJobHolds(100)
 {
+    debug_level = DEBUG_VERBOSE;  // Default debug level is verbose output
 }
 
 
@@ -144,8 +155,8 @@ Dagman::~Dagman()
 bool
 Dagman::Config()
 {
-	int debug_cache_size;
-	bool debug_cache_enabled;
+	int debug_cache_size = (1024*1024)*5; // 5 MB
+	bool debug_cache_enabled = false;
 
 		// Note: debug_printfs are DEBUG_NORMAL here because when we
 		// get here we haven't processed command-line arguments yet.
@@ -160,7 +171,7 @@ Dagman::Config()
 			// message from the config code doesn't show up in dagman.out.
 		if ( access( _dagmanConfigFile, R_OK ) != 0 &&
 					!is_piped_command( _dagmanConfigFile ) ) {
-			debug_printf( DEBUG_NORMAL,
+			debug_printf( DEBUG_QUIET,
 						"ERROR: Can't read DAGMan config file: %s\n",
 						_dagmanConfigFile );
     		DC_Exit( EXIT_ERROR );
@@ -169,31 +180,44 @@ Dagman::Config()
 					NULL, true );
 	}
 
+	debug_level = (debug_level_t)param_integer( "DAGMAN_VERBOSITY",
+				debug_level, DEBUG_SILENT, DEBUG_DEBUG_4 );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_VERBOSITY setting: %d\n",
+				debug_level );
+
 	debug_cache_size = 
-		param_integer( "DAGMAN_DEBUG_CACHE_SIZE", ((1024*1024)*5), 0, INT_MAX);
+		param_integer( "DAGMAN_DEBUG_CACHE_SIZE", debug_cache_size,
+		0, INT_MAX);
 	debug_printf( DEBUG_NORMAL, "DAGMAN_DEBUG_CACHE_SIZE setting: %d\n",
 				debug_cache_size );
 
 	debug_cache_enabled = 
-		param_boolean( "DAGMAN_DEBUG_CACHE_ENABLE", false );
+		param_boolean( "DAGMAN_DEBUG_CACHE_ENABLE", debug_cache_enabled );
 	debug_printf( DEBUG_NORMAL, "DAGMAN_DEBUG_CACHE_ENABLE setting: %s\n",
 				debug_cache_enabled?"True":"False" );
 
-	submit_delay = param_integer( "DAGMAN_SUBMIT_DELAY", 0, 0, 60 );
+	submit_delay = param_integer( "DAGMAN_SUBMIT_DELAY", submit_delay, 0, 60 );
 	debug_printf( DEBUG_NORMAL, "DAGMAN_SUBMIT_DELAY setting: %d\n",
 				submit_delay );
 	max_submit_attempts =
-		param_integer( "DAGMAN_MAX_SUBMIT_ATTEMPTS", 6, 1, 16 );
+		param_integer( "DAGMAN_MAX_SUBMIT_ATTEMPTS", max_submit_attempts,
+		1, 16 );
 	debug_printf( DEBUG_NORMAL, "DAGMAN_MAX_SUBMIT_ATTEMPTS setting: %d\n",
 				max_submit_attempts );
 	startup_cycle_detect =
-		param_boolean( "DAGMAN_STARTUP_CYCLE_DETECT", false );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_STARTUP_CYCLE_DETECT setting: %d\n",
-				startup_cycle_detect );
+		param_boolean( "DAGMAN_STARTUP_CYCLE_DETECT", startup_cycle_detect );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_STARTUP_CYCLE_DETECT setting: %s\n",
+				startup_cycle_detect ? "True" : "False" );
 	max_submits_per_interval =
-		param_integer( "DAGMAN_MAX_SUBMITS_PER_INTERVAL", 5, 1, 1000 );
+		param_integer( "DAGMAN_MAX_SUBMITS_PER_INTERVAL",
+		max_submits_per_interval, 1, 1000 );
 	debug_printf( DEBUG_NORMAL, "DAGMAN_MAX_SUBMITS_PER_INTERVAL setting: %d\n",
 				max_submits_per_interval );
+	m_user_log_scan_interval =
+		param_integer( "DAGMAN_USER_LOG_SCAN_INTERVAL",
+		m_user_log_scan_interval, 1, INT_MAX);
+	debug_printf( DEBUG_NORMAL, "DAGMAN_USER_LOG_SCAN_INTERVAL setting: %d\n",
+				m_user_log_scan_interval );
 
 
 		// Event checking setup...
@@ -237,13 +261,15 @@ Dagman::Config()
 
 		// ...end of event checking setup.
 
-	retrySubmitFirst = param_boolean( "DAGMAN_RETRY_SUBMIT_FIRST", true );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_RETRY_SUBMIT_FIRST setting: %d\n",
+	retrySubmitFirst = param_boolean( "DAGMAN_RETRY_SUBMIT_FIRST",
 				retrySubmitFirst );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_RETRY_SUBMIT_FIRST setting: %s\n",
+				retrySubmitFirst ? "True" : "False" );
 
-	retryNodeFirst = param_boolean( "DAGMAN_RETRY_NODE_FIRST", false );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_RETRY_NODE_FIRST setting: %d\n",
+	retryNodeFirst = param_boolean( "DAGMAN_RETRY_NODE_FIRST",
 				retryNodeFirst );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_RETRY_NODE_FIRST setting: %s\n",
+				retryNodeFirst ? "True" : "False" );
 
 	maxIdle =
 		param_integer( "DAGMAN_MAX_JOBS_IDLE", maxIdle, 0, INT_MAX );
@@ -255,21 +281,34 @@ Dagman::Config()
 	debug_printf( DEBUG_NORMAL, "DAGMAN_MAX_JOBS_SUBMITTED setting: %d\n",
 				maxJobs );
 
-	mungeNodeNames = param_boolean( "DAGMAN_MUNGE_NODE_NAMES", true );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_MUNGE_NODE_NAMES setting: %d\n",
+	maxPreScripts = param_integer( "DAGMAN_MAX_PRE_SCRIPTS", maxPreScripts,
+				0, INT_MAX );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_MAX_PRE_SCRIPTS setting: %d\n",
+				maxPreScripts );
+
+	maxPostScripts = param_integer( "DAGMAN_MAX_POST_SCRIPTS", maxPostScripts,
+				0, INT_MAX );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_MAX_POST_SCRIPTS setting: %d\n",
+				maxPostScripts );
+
+	allowLogError = param_boolean( "DAGMAN_ALLOW_LOG_ERROR", allowLogError );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_ALLOW_LOG_ERROR setting: %s\n",
+				allowLogError ? "True" : "False" );
+
+	mungeNodeNames = param_boolean( "DAGMAN_MUNGE_NODE_NAMES",
 				mungeNodeNames );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_MUNGE_NODE_NAMES setting: %s\n",
+				mungeNodeNames ? "True" : "False" );
 
-	deleteOldLogs = param_boolean( "DAGMAN_DELETE_OLD_LOGS", deleteOldLogs );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_DELETE_OLD_LOGS setting: %d\n",
-				  deleteOldLogs );
-
-	prohibitMultiJobs = param_boolean( "DAGMAN_PROHIBIT_MULTI_JOBS", false );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_PROHIBIT_MULTI_JOBS setting: %d\n",
+	prohibitMultiJobs = param_boolean( "DAGMAN_PROHIBIT_MULTI_JOBS",
 				prohibitMultiJobs );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_PROHIBIT_MULTI_JOBS setting: %s\n",
+				prohibitMultiJobs ? "True" : "False" );
 
-	submitDepthFirst = param_boolean( "DAGMAN_SUBMIT_DEPTH_FIRST", false );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_SUBMIT_DEPTH_FIRST setting: %d\n",
+	submitDepthFirst = param_boolean( "DAGMAN_SUBMIT_DEPTH_FIRST",
 				submitDepthFirst );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_SUBMIT_DEPTH_FIRST setting: %s\n",
+				submitDepthFirst ? "True" : "False" );
 
 	free( condorSubmitExe );
 	condorSubmitExe = param( "DAGMAN_CONDOR_SUBMIT_EXE" );
@@ -299,29 +338,58 @@ Dagman::Config()
 		ASSERT( storkRmExe );
 	}
 
-	abortDuplicates = param_boolean( "DAGMAN_ABORT_DUPLICATES", true );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_ABORT_DUPLICATES setting: %d\n",
+	abortDuplicates = param_boolean( "DAGMAN_ABORT_DUPLICATES",
 				abortDuplicates );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_ABORT_DUPLICATES setting: %s\n",
+				abortDuplicates ? "True" : "False" );
 
-	abortOnScarySubmit = param_boolean( "DAGMAN_ABORT_ON_SCARY_SUBMIT", true );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_ABORT_ON_SCARY_SUBMIT setting: %d\n",
+	abortOnScarySubmit = param_boolean( "DAGMAN_ABORT_ON_SCARY_SUBMIT",
 				abortOnScarySubmit );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_ABORT_ON_SCARY_SUBMIT setting: %s\n",
+				abortOnScarySubmit ? "True" : "False" );
 
-	const int PENDING_REPORT_INT_DEFAULT = 10 * 60; // 10 minutes
 	pendingReportInterval = param_integer( "DAGMAN_PENDING_REPORT_INTERVAL",
-				PENDING_REPORT_INT_DEFAULT );
+				pendingReportInterval );
 	debug_printf( DEBUG_NORMAL, "DAGMAN_PENDING_REPORT_INTERVAL setting: %d\n",
 				pendingReportInterval );
 
-	autoRescue = param_boolean( "DAGMAN_AUTO_RESCUE", true );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_AUTO_RESCUE setting: %d\n",
-				autoRescue );
+	autoRescue = param_boolean( "DAGMAN_AUTO_RESCUE", autoRescue );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_AUTO_RESCUE setting: %s\n",
+				autoRescue ? "True" : "False" );
 	
 	maxRescueDagNum = param_integer( "DAGMAN_MAX_RESCUE_NUM",
-				MAX_RESCUE_DAG_DEFAULT, 0, ABS_MAX_RESCUE_DAG_NUM );
+				maxRescueDagNum, 0, ABS_MAX_RESCUE_DAG_NUM );
 	debug_printf( DEBUG_NORMAL, "DAGMAN_MAX_RESCUE_NUM setting: %d\n",
 				maxRescueDagNum );
 
+	free( _defaultNodeLog );
+	_defaultNodeLog = param( "DAGMAN_DEFAULT_NODE_LOG" );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_DEFAULT_NODE_LOG setting: %s\n",
+				_defaultNodeLog ? _defaultNodeLog : "null" );
+
+	_generateSubdagSubmits = 
+		param_boolean( "DAGMAN_GENERATE_SUBDAG_SUBMITS",
+		_generateSubdagSubmits );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_GENERATE_SUBDAG_SUBMITS setting: %s\n",
+				_generateSubdagSubmits ? "True" : "False" );
+
+	_maxJobHolds = param_integer( "DAGMAN_MAX_JOB_HOLDS", _maxJobHolds,
+				0, 1000000 );
+
+	char *debugSetting = param( "ALL_DEBUG" );
+	debug_printf( DEBUG_NORMAL, "ALL_DEBUG setting: %s\n",
+				debugSetting ? debugSetting : "" );
+	if ( debugSetting ) {
+		free( debugSetting );
+	}
+
+	debugSetting = param( "DAGMAN_DEBUG" );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_DEBUG setting: %s\n",
+				debugSetting ? debugSetting : "" );
+	if ( debugSetting ) {
+		free( debugSetting );
+	}
+
 	// enable up the debug cache if needed
 	if (debug_cache_enabled) {
 		debug_cache_set_size(debug_cache_size);
@@ -333,34 +401,34 @@ Dagman::Config()
 
 
 // NOTE: this is only called on reconfig, not at startup
-int
-main_config( bool )
+void
+main_config()
 {
 		// This is commented out because, even if we get new config
 		// values here, they don't get passed to the Dag object (which
 		// is where most of them actually take effect).  (See Gnats
 		// PR 808.)  wenger 2007-02-09
 	// dagman.Config();
-	return 0;
 }
 
 // this is called by DC when the schedd is shutdown fast
-int
+void
 main_shutdown_fast()
 {
+	dagman.dag->GetJobstateLog().WriteDagmanFinished( EXIT_RESTART );
     DC_Exit( EXIT_RESTART );
-    return FALSE;
 }
 
 // this can be called by other functions, or by DC when the schedd is
 // shutdown gracefully
-int main_shutdown_graceful() {
+void main_shutdown_graceful() {
+	dagman.dag->DumpNodeStatus( true, false );
+	dagman.dag->GetJobstateLog().WriteDagmanFinished( EXIT_RESTART );
     dagman.CleanUp();
 	DC_Exit( EXIT_RESTART );
-    return FALSE;
 }
 
-int main_shutdown_rescue( int exitVal ) {
+void main_shutdown_rescue( int exitVal ) {
 	debug_printf( DEBUG_QUIET, "Aborting DAG...\n" );
 	if( dagman.dag ) {
 			// we write the rescue DAG *before* removing jobs because
@@ -395,29 +463,31 @@ int main_shutdown_rescue( int exitVal ) {
 		}
 		dagman.dag->PrintDeferrals( DEBUG_NORMAL, true );
 	}
+	dagman.dag->DumpNodeStatus( false, true );
+	dagman.dag->GetJobstateLog().WriteDagmanFinished( exitVal );
 	unlink( lockFileName ); 
     dagman.CleanUp();
 	DC_Exit( exitVal );
-	return false;
 }
 
 // this gets called by DC when DAGMan receives a SIGUSR1 -- which,
 // assuming the DAGMan submit file was properly written, is the signal
 // the schedd will send if the DAGMan job is removed from the queue
 int main_shutdown_remove(Service *, int) {
-    debug_printf( DEBUG_NORMAL, "Received SIGUSR1\n" );
+    debug_printf( DEBUG_QUIET, "Received SIGUSR1\n" );
 	main_shutdown_rescue( EXIT_ABORT );
-	return false;
+	return FALSE;
 }
 
 void ExitSuccess() {
+	dagman.dag->DumpNodeStatus( false, false );
+	dagman.dag->GetJobstateLog().WriteDagmanFinished( EXIT_OKAY );
 	unlink( lockFileName ); 
     dagman.CleanUp();
 	DC_Exit( EXIT_OKAY );
 }
 
 void condor_event_timer();
-void print_status();
 
 /****** FOR TESTING *******
 int main_testing_stub( Service *, int ) {
@@ -432,7 +502,7 @@ int main_testing_stub( Service *, int ) {
 ****** FOR TESTING ********/
 
 //---------------------------------------------------------------------------
-int main_init (int argc, char ** const argv) {
+void main_init (int argc, char ** const argv) {
 
 	printf ("Executing condor dagman ... \n");
 
@@ -455,7 +525,6 @@ int main_init (int argc, char ** const argv) {
                                  "main_testing_stub", NULL);
 ****** FOR TESTING ********/
     debug_progname = condor_basename(argv[0]);
-    debug_level = DEBUG_NORMAL;  // Default debug level is normal output
 
 		// condor_submit_dag version from .condor.sub
 	bool allowVerMismatch = false;
@@ -472,6 +541,30 @@ int main_init (int argc, char ** const argv) {
 		// (otherwise it will be set to "-1.-1.-1")
 	dagman.DAGManJobId.SetFromString( getenv( EnvGetName( ENV_ID ) ) );
 
+	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+		// Minimum legal version for a .condor.sub file to be compatible
+		// with this condor_dagman binary.
+
+		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+		// Be sure to change this if the arguments or environment
+		// passed to condor_dagman change in an incompatible way!!
+		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+
+	struct DagVersionData {
+		int majorVer;
+		int minorVer;
+		int subMinorVer;
+	};
+	const DagVersionData MIN_SUBMIT_FILE_VERSION = { 7, 1, 2 };
+
+		// Construct a string of the minimum submit file version.
+	MyString minSubmitVersionStr;
+	minSubmitVersionStr.sprintf( "%d.%d.%d",
+				MIN_SUBMIT_FILE_VERSION.majorVer,
+				MIN_SUBMIT_FILE_VERSION.minorVer,
+				MIN_SUBMIT_FILE_VERSION.subMinorVer );
+	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
     //
     // Process command-line arguments
     //
@@ -586,6 +679,42 @@ int main_init (int argc, char ** const argv) {
         } else if( !strcasecmp( "-DumpRescue", argv[i] ) ) {
 			dagman.dumpRescueDag = true;
 
+        } else if( !strcasecmp( "-verbose", argv[i] ) ) {
+			dagman._submitDagDeepOpts.bVerbose = true;
+
+        } else if( !strcasecmp( "-force", argv[i] ) ) {
+			dagman._submitDagDeepOpts.bForce = true;
+		
+        } else if( !strcasecmp( "-notification", argv[i] ) ) {
+            i++;
+            if( argc <= i || strcmp( argv[i], "" ) == 0 ) {
+                debug_printf( DEBUG_SILENT, "No notification value specified\n" );
+                Usage();
+            }
+			dagman._submitDagDeepOpts.strNotification = argv[i];
+
+        } else if( !strcasecmp( "-dagman", argv[i] ) ) {
+            i++;
+            if( argc <= i || strcmp( argv[i], "" ) == 0 ) {
+                debug_printf( DEBUG_SILENT, "No dagman value specified\n" );
+                Usage();
+            }
+			dagman._submitDagDeepOpts.strDagmanPath = argv[i];
+
+        } else if( !strcasecmp( "-outfile_dir", argv[i] ) ) {
+            i++;
+            if( argc <= i || strcmp( argv[i], "" ) == 0 ) {
+                debug_printf( DEBUG_SILENT, "No outfile_dir value specified\n" );
+                Usage();
+            }
+			dagman._submitDagDeepOpts.strOutfileDir = argv[i];
+
+        } else if( !strcasecmp( "-update_submit", argv[i] ) ) {
+			dagman._submitDagDeepOpts.updateSubmit = true;
+
+        } else if( !strcasecmp( "-import_env", argv[i] ) ) {
+			dagman._submitDagDeepOpts.importEnv = true;
+
         } else {
     		debug_printf( DEBUG_SILENT, "\nUnrecognized argument: %s\n",
 						argv[i] );
@@ -597,22 +726,96 @@ int main_init (int argc, char ** const argv) {
 	dagman.primaryDagFile = dagman.dagFiles.next();
 	dagman.multiDags = (dagman.dagFiles.number() > 1);
 
+	MyString tmpDefaultLog;
+	if ( dagman._defaultNodeLog != NULL ) {
+		tmpDefaultLog = dagman._defaultNodeLog;
+		free( dagman._defaultNodeLog );
+	} else {
+		tmpDefaultLog = dagman.primaryDagFile + ".nodes.log";
+	}
+
+		// Force default log file path to be absolute so it works
+		// with -usedagdir and DIR nodes.
+	CondorError errstack;
+	if ( !MultiLogFiles::makePathAbsolute( tmpDefaultLog, errstack) ) {
+       	debug_printf( DEBUG_QUIET, "Unable to convert default log "
+					"file name to absolute path: %s\n",
+					errstack.getFullText() );
+		dagman.dag->GetJobstateLog().WriteDagmanFinished( EXIT_ERROR );
+		DC_Exit( EXIT_ERROR );
+	}
+	dagman._defaultNodeLog = strdup( tmpDefaultLog.Value() );
+	debug_printf( DEBUG_NORMAL, "Default node log file is: <%s>\n",
+				dagman._defaultNodeLog);
+
     //
     // Check the arguments
     //
-	if( strcmp( CondorVersion(), csdVersion ) != 0 ) {
-		if ( allowVerMismatch ) {
-        	debug_printf( DEBUG_NORMAL, "Warning: version mismatch: "
-						"condor_submit_dag (%s) vs. condor_dagman (%s); "
-						"continuing because of -AllowVersionMismatch flag\n",
-						csdVersion, CondorVersion() );
-		} else {
-        	debug_printf( DEBUG_SILENT, "Version mismatch: condor_submit_dag "
-						"(%s) vs. condor_dagman (%s)\n", csdVersion,
-						CondorVersion() );
+
+	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+	// Checking for version compatibility between the .condor.sub
+	// file and this condor_dagman binary...
+
+	// Note: if we're in recovery mode and the submit file version
+	// causes us to quit, we leave any existing node jobs still
+	// running -- may want to change that eventually.  wenger 2009-10-13.
+
+		// Version of the condor_submit_dag that created our submit file.
+	CondorVersionInfo submitFileVersion( csdVersion );
+
+		// Version of this condor_dagman binary.
+	CondorVersionInfo dagmanVersion;
+
+		// Just generate this message fragment in one place.
+	MyString versionMsg;
+	versionMsg.sprintf("the version (%s) of this DAG's Condor submit "
+				"file (created by condor_submit_dag)", csdVersion );
+
+		// Make sure version in submit file is valid.
+	if( !submitFileVersion.is_valid() ) {
+		if ( !allowVerMismatch ) {
+        	debug_printf( DEBUG_QUIET, "Error: %s is invalid!\n",
+						versionMsg.Value() );
 			DC_Exit( EXIT_ERROR );
+		} else {
+        	debug_printf( DEBUG_NORMAL, "Warning: %s is invalid; "
+						"continuing because of -AllowVersionMismatch flag\n",
+						versionMsg.Value() );
+		}
+
+		// Make sure .condor.sub file is recent enough.
+	} else if ( submitFileVersion.compare_versions(
+				CondorVersion() ) != 0 ) {
+
+		if( !submitFileVersion.built_since_version(
+					MIN_SUBMIT_FILE_VERSION.majorVer,
+					MIN_SUBMIT_FILE_VERSION.minorVer,
+					MIN_SUBMIT_FILE_VERSION.subMinorVer ) ) {
+			if ( !allowVerMismatch ) {
+        		debug_printf( DEBUG_QUIET, "Error: %s is older than "
+							"oldest permissible version (%s)\n",
+							versionMsg.Value(), minSubmitVersionStr.Value() );
+				DC_Exit( EXIT_ERROR );
+			} else {
+        		debug_printf( DEBUG_NORMAL, "Warning: %s is older than "
+							"oldest permissible version (%s); continuing "
+							"because of -AllowVersionMismatch flag\n",
+							versionMsg.Value(), minSubmitVersionStr.Value() );
+			}
+
+			// Warn if .condor.sub file is a newer version than this binary.
+		} else if (dagmanVersion.compare_versions( csdVersion ) > 0 ) {
+        	debug_printf( DEBUG_NORMAL, "Warning: %s is newer than "
+						"condor_dagman version (%s)\n", versionMsg.Value(),
+						CondorVersion() );
+		} else {
+        	debug_printf( DEBUG_NORMAL, "Note: %s differs from "
+						"condor_dagman version (%s), but the "
+						"difference is permissible\n", 
+						versionMsg.Value(), CondorVersion() );
 		}
 	}
+	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
     if( dagman.primaryDagFile == "" ) {
         debug_printf( DEBUG_SILENT, "No DAG file was specified\n" );
@@ -736,17 +939,38 @@ int main_init (int argc, char ** const argv) {
 		}
 	}
 
+		//
+		// Fill in values in the deep submit options that we haven't
+		// already set.
+		//
+	dagman._submitDagDeepOpts.bAllowLogError = dagman.allowLogError;
+	dagman._submitDagDeepOpts.useDagDir = dagman.useDagDir;
+	dagman._submitDagDeepOpts.oldRescue =
+				(dagman.rescueFileToWrite != NULL);
+	dagman._submitDagDeepOpts.autoRescue = dagman.autoRescue;
+	dagman._submitDagDeepOpts.doRescueFrom = dagman.doRescueFrom;
+	dagman._submitDagDeepOpts.allowVerMismatch = allowVerMismatch;
+	dagman._submitDagDeepOpts.recurse = false;
+
     //
     // Create the DAG
     //
 
+	// Note: a bunch of the parameters we pass here duplicate things
+	// in submitDagOpts, but I'm keeping them separate so we don't have to
+	// bother to construct a new SubmitDagOtions object for splices.
+	// wenger 2010-03-25
     dagman.dag = new Dag( dagman.dagFiles, dagman.maxJobs,
 						  dagman.maxPreScripts, dagman.maxPostScripts,
 						  dagman.allowLogError, dagman.useDagDir,
 						  dagman.maxIdle, dagman.retrySubmitFirst,
 						  dagman.retryNodeFirst, dagman.condorRmExe,
 						  dagman.storkRmExe, &dagman.DAGManJobId,
-						  dagman.prohibitMultiJobs, dagman.submitDepthFirst );
+						  dagman.prohibitMultiJobs, dagman.submitDepthFirst,
+						  dagman._defaultNodeLog,
+						  dagman._generateSubdagSubmits,
+						  &dagman._submitDagDeepOpts,
+						  false ); /* toplevel dag! */
 
     if( dagman.dag == NULL ) {
         EXCEPT( "ERROR: out of memory!\n");
@@ -755,6 +979,7 @@ int main_init (int argc, char ** const argv) {
 	dagman.dag->SetAbortOnScarySubmit( dagman.abortOnScarySubmit );
 	dagman.dag->SetAllowEvents( dagman.allow_events );
 	dagman.dag->SetConfigFile( dagman._dagmanConfigFile );
+	dagman.dag->SetMaxJobHolds( dagman._maxJobHolds );
 
     //
     // Parse the input files.  The parse() routine
@@ -778,15 +1003,33 @@ int main_init (int argc, char ** const argv) {
     	debug_printf( DEBUG_VERBOSE, "Parsing %s ...\n", dagFile );
 
     	if( !parse( dagman.dag, dagFile, dagman.useDagDir ) ) {
+			if ( dagman.dumpRescueDag ) {
+					// Dump the rescue DAG so we can see what we got
+					// in the failed parse attempt.
+    			debug_printf( DEBUG_QUIET, "Dumping rescue DAG "
+							"because of -DumpRescue flag\n" );
+				dagman.dag->Rescue( dagman.primaryDagFile.Value(),
+							dagman.multiDags, dagman.maxRescueDagNum,
+							true );
+			}
+
 				// Note: debug_error calls DC_Exit().
         	debug_error( 1, DEBUG_QUIET, "Failed to parse %s\n",
 					 	dagFile );
     	}
 	}
 
+	dagman.dag->GetJobstateLog().WriteDagmanStarted( dagman.DAGManJobId );
+	if ( rescueDagNum > 0 ) {
+			// Get our Pegasus sequence numbers set correctly.
+		dagman.dag->GetJobstateLog().InitializeRescue();
+	}
+
 	// lift the final set of splices into the main dag.
 	dagman.dag->LiftSplices(SELF);
 
+	dagman.dag->CheckThrottleCats();
+
 	// fix up any use of $(JOB) in the vars values for any node
 	dagman.dag->ResolveVarsInterpolations();
 
@@ -802,6 +1045,17 @@ int main_init (int argc, char ** const argv) {
     debug_printf( DEBUG_VERBOSE, "Dag contains %d total jobs\n",
 				  dagman.dag->NumNodes() );
 
+	MyString firstLocation;
+	if ( dagman.dag->GetReject( firstLocation ) ) {
+    	debug_printf( DEBUG_QUIET, "Exiting because of REJECT "
+					"specification in %s.  This most likely means "
+					"that the DAG file was produced with the -DumpRescue "
+					"flag when parsing the original DAG failed.\n",
+					firstLocation.Value() );
+		DC_Exit( EXIT_ERROR );
+		return;
+	}
+
 	dagman.dag->DumpDotFile();
 
 	if ( dagman.dumpRescueDag ) {
@@ -810,7 +1064,7 @@ int main_init (int argc, char ** const argv) {
 		dagman.dag->Rescue( dagman.primaryDagFile.Value(),
 					dagman.multiDags, dagman.maxRescueDagNum );
 		ExitSuccess();
-		return 0;
+		return;
 	}
 
     //------------------------------------------------------------------------
@@ -836,13 +1090,13 @@ int main_init (int argc, char ** const argv) {
 							"currently running on this DAG; if that is "
 							"not the case, delete the lock file (%s) "
 							"and re-submit the DAG.\n", lockFileName );
+					dagman.dag->GetJobstateLog().
+								WriteDagmanFinished( EXIT_RESTART );
     				dagman.CleanUp();
 					DC_Exit( EXIT_ERROR );
 					// We should never get to here!
 				}
 			}
-        } else {
-			dagman.dag->InitializeDagFiles( dagman.deleteOldLogs );
         }
 
 			//
@@ -858,14 +1112,12 @@ int main_init (int argc, char ** const argv) {
         }
     }
 
-    dprintf( D_ALWAYS, "Registering condor_event_timer...\n" );
-    daemonCore->Register_Timer( 1, 5, (TimerHandler)condor_event_timer,
-				"condor_event_timer" );
+    debug_printf( DEBUG_VERBOSE, "Registering condor_event_timer...\n" );
+    daemonCore->Register_Timer( 1, dagman.m_user_log_scan_interval, 
+				condor_event_timer, "condor_event_timer" );
 
 	dagman.dag->SetPendingNodeReportInterval(
 				dagman.pendingReportInterval );
-
-    return 0;
 }
 
 void
@@ -887,6 +1139,8 @@ print_status() {
 
 	debug_printf( DEBUG_VERBOSE, "%5d   %5d    %5d   %5d   %5d      %5d    %5d\n",
 				  done, pre, submitted, post, ready, unready, failed );
+	debug_printf( DEBUG_VERBOSE, "%d job proc(s) currently held\n",
+				dagman.dag->NumHeldJobProcs() );
 	dagman.dag->PrintDeferrals( DEBUG_VERBOSE, false );
 }
 
@@ -916,6 +1170,7 @@ void condor_event_timer () {
     static int prevJobsSubmitted = 0;
     static int prevJobsReady = 0;
     static int prevScriptRunNodes = 0;
+    static int prevJobsHeld = 0;
 
 	int justSubmitted;
 	justSubmitted = dagman.dag->SubmitReadyJobs(dagman);
@@ -926,25 +1181,25 @@ void condor_event_timer () {
 					  justSubmitted, justSubmitted == 1 ? "" : "s" );
 	}
 
-    // If the log has grown
-    if( dagman.dag->DetectCondorLogGrowth() ) {
+	// If the log has grown
+	if( dagman.dag->DetectCondorLogGrowth() ) {
 		if( dagman.dag->ProcessLogEvents( CONDORLOG ) == false ) {
 			dagman.dag->PrintReadyQ( DEBUG_DEBUG_1 );
 			main_shutdown_rescue( EXIT_ERROR );
 			return;
-        }
-    }
+		}
+	}
+
+	if( dagman.dag->DetectDaPLogGrowth() ) {
+		if( dagman.dag->ProcessLogEvents( DAPLOG ) == false ) {
+			debug_printf( DEBUG_NORMAL,
+						"ProcessLogEvents(DAPLOG) returned false\n");
+			dagman.dag->PrintReadyQ( DEBUG_DEBUG_1 );
+			main_shutdown_rescue( EXIT_ERROR );
+			return;
+		}
+	}
 
-    if( dagman.dag->DetectDaPLogGrowth() ) {
-      if( dagman.dag->ProcessLogEvents( DAPLOG ) == false ) {
-	debug_printf( DEBUG_NORMAL,
-			"ProcessLogEvents(DAPLOG) returned false\n");
-	dagman.dag->PrintReadyQ( DEBUG_DEBUG_1 );
-	main_shutdown_rescue( EXIT_ERROR );
-	return;
-      }
-    }
-  
     // print status if anything's changed (or we're in a high debug level)
     if( prevJobsDone != dagman.dag->NumNodesDone()
         || prevJobs != dagman.dag->NumNodes()
@@ -952,27 +1207,32 @@ void condor_event_timer () {
         || prevJobsSubmitted != dagman.dag->NumJobsSubmitted()
         || prevJobsReady != dagman.dag->NumNodesReady()
         || prevScriptRunNodes != dagman.dag->ScriptRunNodeCount()
+		|| prevJobsHeld != dagman.dag->NumHeldJobProcs()
 		|| DEBUG_LEVEL( DEBUG_DEBUG_4 ) ) {
 		print_status();
+
         prevJobsDone = dagman.dag->NumNodesDone();
         prevJobs = dagman.dag->NumNodes();
         prevJobsFailed = dagman.dag->NumNodesFailed();
         prevJobsSubmitted = dagman.dag->NumJobsSubmitted();
         prevJobsReady = dagman.dag->NumNodesReady();
         prevScriptRunNodes = dagman.dag->ScriptRunNodeCount();
+		prevJobsHeld = dagman.dag->NumHeldJobProcs();
 		
 		if( dagman.dag->GetDotFileUpdate() ) {
 			dagman.dag->DumpDotFile();
 		}
 	}
 
+	dagman.dag->DumpNodeStatus( false, false );
+
     ASSERT( dagman.dag->NumNodesDone() + dagman.dag->NumNodesFailed()
 			<= dagman.dag->NumNodes() );
 
     //
     // If DAG is complete, hurray, and exit.
     //
-    if( dagman.dag->Done() ) {
+    if( dagman.dag->DoneSuccess() ) {
         ASSERT( dagman.dag->NumJobsSubmitted() == 0 );
 		dagman.dag->CheckAllJobs();
         debug_printf( DEBUG_NORMAL, "All jobs Completed!\n" );
@@ -987,24 +1247,28 @@ void condor_event_timer () {
     }
 
     //
-
     // If no jobs are submitted and no scripts are running, but the
     // dag is not complete, then at least one job failed, or a cycle
     // exists.
     // 
-    if( dagman.dag->NumJobsSubmitted() == 0 &&
-		dagman.dag->NumNodesReady() == 0 &&
-		dagman.dag->ScriptRunNodeCount() == 0 ) {
-		if( dagman.dag->NumNodesFailed() > 0 ) {
+    if( dagman.dag->FinishedRunning() ) {
+		if( dagman.dag->DoneFailed() ) {
 			if( DEBUG_LEVEL( DEBUG_QUIET ) ) {
 				debug_printf( DEBUG_QUIET,
 							  "ERROR: the following job(s) failed:\n" );
 				dagman.dag->PrintJobList( Job::STATUS_ERROR );
 			}
-		}
-		else {
+		} else {
 			// no jobs failed, so a cycle must exist
-			debug_printf( DEBUG_QUIET, "ERROR: a cycle exists in the DAG\n" );
+			debug_printf( DEBUG_QUIET, "ERROR: DAG finished but not all "
+						"nodes are complete -- checking for a cycle...\n" );
+			if( dagman.dag->isCycle() ) {
+				debug_printf (DEBUG_QUIET, "... ERROR: a cycle exists "
+							"in the dag, plese check input\n");
+			} else {
+				debug_printf (DEBUG_QUIET, "... ERROR: no cycle found; "
+							"unknown error condition\n");
+			}
 			if ( debug_level >= DEBUG_NORMAL ) {
 				dagman.dag->PrintJobList();
 			}
@@ -1020,6 +1284,7 @@ void
 main_pre_dc_init( int, char*[] )
 {
 	DC_Skip_Auth_Init();
+	DC_Skip_Core_Init();
 
 		// Convert the DAGMan log file name to an absolute path if it's
 		// not one already, so that we'll log things to the right file
