@@ -24,6 +24,7 @@
 #include "basename.h"
 #include "tmp_dir.h"
 #include "dagman_multi_dag.h"
+#include "condor_getcwd.h"
 
 // Just so we can link in the ReadMultipleUserLogs class.
 MULTI_LOG_HASH_INSTANCE;
@@ -36,92 +37,6 @@ AppendError(MyString &errMsg, const MyString &newError)
 	errMsg += newError;
 }
 
-//-------------------------------------------------------------------------
-bool
-GetLogFiles(/* const */ StringList &dagFiles, bool useDagDir, 
-			StringList &condorLogFiles, StringList &storkLogFiles,
-			MyString &errMsg)
-{
-	bool		result = true;
-
-	TmpDir		dagDir;
-
-	dagFiles.rewind();
-	char *dagFile;
-	while ( (dagFile = dagFiles.next()) != NULL ) {
-
-		const char *	file;
-		if ( useDagDir ) {
-			MyString	tmpErrMsg;
-			if ( !dagDir.Cd2TmpDirFile( dagFile, tmpErrMsg ) ) {
-				AppendError( errMsg,
-						MyString("Unable to change to DAG directory ") +
-						tmpErrMsg );
-				return false;
-			}
-			file = condor_basename( dagFile );
-		} else {
-			file = dagFile;
-		}
-
-			// Note: this returns absolute paths to the log files.
-		MyString msg = MultiLogFiles::getJobLogsFromSubmitFiles(
-				file, "job", "dir", condorLogFiles);
-		if ( msg != "" ) {
-			AppendError( errMsg,
-					MyString("Failed to locate Condor job log files: ") +
-					msg );
-			result = false;
-		}
-
-			// Note: this returns absolute paths to the log files.
-		msg = MultiLogFiles::getJobLogsFromSubmitFiles(
-				file, "data", "dir", storkLogFiles);
-		if ( msg != "" ) {
-			AppendError( errMsg,
-					MyString("Failed to locate Stork job log files: ") +
-					msg );
-			result = false;
-		}
-
-		MyString	tmpErrMsg;
-		if ( !dagDir.Cd2MainDir( tmpErrMsg ) ) {
-			AppendError( errMsg,
-					MyString("Unable to change to original directory ") +
-					tmpErrMsg );
-			result = false;
-		}
-
-	}
-
-	return result;
-}
-
-//-------------------------------------------------------------------------
-bool
-LogFileNfsError(/* const */ StringList &condorLogFiles,
-			/* const */ StringList &storkLogFiles)
-{
-	condorLogFiles.rewind();
-	storkLogFiles.rewind();
-
-	bool nfsIsError = param_boolean( "DAGMAN_LOG_ON_NFS_IS_ERROR", true );
-
-	if (MultiLogFiles::logFilesOnNFS(condorLogFiles, nfsIsError)) {
-		fprintf( stderr, "Aborting -- "
-				"Condor log files should not be on NFS.\n");
-		return true;
-	}
-
-	if (MultiLogFiles::logFilesOnNFS(storkLogFiles, nfsIsError)) {
-		fprintf( stderr, "Aborting -- "
-				"Stork log files should not be on NFS.\n");
-		return true;
-	}
-
-	return false;
-}
-
 //-------------------------------------------------------------------------
 bool
 GetConfigFile(/* const */ StringList &dagFiles, bool useDagDir, 
@@ -214,11 +129,8 @@ MakePathAbsolute(MyString &filePath, MyString &errMsg)
 
 	if ( !fullpath( filePath.Value() ) ) {
 		MyString    currentDir;
-		char    tmpCwd[PATH_MAX];
-		if ( getcwd(tmpCwd, PATH_MAX) ) {
-			currentDir = tmpCwd;
-		} else {
-			errMsg = MyString( "getcwd() failed with errno " ) +
+		if ( ! condor_getcwd( currentDir ) ) {
+			errMsg = MyString( "condor_getcwd() failed with errno " ) +
 						errno + " (" + strerror(errno) + ") at " + __FILE__
 						+ ":" + __LINE__;
 			result = false;
