@@ -20,6 +20,7 @@
 
 #include "condor_common.h"
 #include "condor_attributes.h"
+#include "condor_string.h"
 #include "util_lib_proto.h"
 #include "my_popen.h"
 
@@ -32,6 +33,7 @@
 #include "util.h"
 #include "debug.h"
 #include "tmp_dir.h"
+#include "write_user_log.h"
 
 typedef bool (* parse_submit_fnc)( const char *buffer, int &jobProcCount,
 			int &cluster );
@@ -48,7 +50,9 @@ typedef bool (* parse_submit_fnc)( const char *buffer, int &jobProcCount,
 static bool
 parse_condor_submit( const char *buffer, int &jobProcCount, int &cluster )
 {
-  if ( 2 != sscanf( buffer, "%d job(s) submitted to cluster %d",
+  // The initial space is to make the sscanf match zero or more leading
+  // whitespace that may exist in the buffer.
+  if ( 2 != sscanf( buffer, " %d job(s) submitted to cluster %d",
   			&jobProcCount, &cluster) ) {
 	debug_printf( DEBUG_QUIET, "ERROR: parse_condor_submit failed:\n\t%s\n",
 				buffer );
@@ -70,7 +74,9 @@ parse_condor_submit( const char *buffer, int &jobProcCount, int &cluster )
 static bool
 parse_stork_submit( const char *buffer, int &jobProcCount, int &cluster )
 {
-  if ( 1 != sscanf( buffer, "Request assigned id: %d", &cluster) ) {
+  // The initial space is to make the sscanf match zero or more leading
+  // whitespace that may exist in the buffer.
+  if ( 1 != sscanf( buffer, " Request assigned id: %d", &cluster) ) {
 	debug_printf( DEBUG_QUIET, "ERROR: parse_stork_submit failed:\n\t%s\n",
 				buffer );
     return false;
@@ -144,11 +150,13 @@ submit_try( ArgList &args, CondorID &condorID, Job::job_type_t type,
 
   MyString  command_output("");
   MyString keyLine("");
-  while (util_getline(fp, buffer, UTIL_MAX_LINE_LENGTH) != EOF) {
-	debug_printf(DEBUG_NORMAL, "From submit: %s\n", buffer);
-	command_output += buffer;
+  while (fgets(buffer, UTIL_MAX_LINE_LENGTH, fp)) {
+    MyString buf_line = buffer;
+	buf_line.chomp();
+	debug_printf(DEBUG_VERBOSE, "From submit: %s\n", buf_line.Value());
+	command_output += buf_line;
     if (strstr(buffer, marker) != NULL) {
-	  keyLine = buffer;
+	  keyLine = buf_line;
 	}
   }
 
@@ -222,7 +230,7 @@ bool
 condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 			   const char* DAGNodeName, MyString DAGParentNodeNames,
 			   List<MyString>* names, List<MyString>* vals,
-			   const char* directory )
+			   const char* directory, const char *logFile )
 {
 	TmpDir		tmpDir;
 	MyString	errMsg;
@@ -272,6 +280,13 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 				"submit_event_notes = DAG Node: " ) + DAGNodeName;
 	args.AppendArg( submitEventNotes.Value() );
 
+	if ( logFile ) {
+		args.AppendArg( "-a" );
+		MyString logFileArg = MyString(
+					"log = " ) + logFile;
+		args.AppendArg( logFileArg.Value() );
+	}
+
 	ArgList parentNameArgs;
 	parentNameArgs.AppendArg( "-a" );
 	MyString parentNodeNames = MyString( "+DAGParentNodeNames = " ) +
@@ -364,3 +379,79 @@ stork_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 
   return success;
 }
+
+//-------------------------------------------------------------------------
+// Subproc ID for "fake" events (for NOOP jobs).
+static int _subprocID = 0;
+
+//-------------------------------------------------------------------------
+void
+set_fake_condorID( int subprocID )
+{
+	_subprocID = subprocID;
+}
+
+//-------------------------------------------------------------------------
+bool
+fake_condor_submit( CondorID& condorID, const char* DAGNodeName, 
+			   const char* directory, const char *logFile, bool logIsXml )
+{
+	TmpDir		tmpDir;
+	MyString	errMsg;
+	if ( !tmpDir.Cd2TmpDir( directory, errMsg ) ) {
+		debug_printf( DEBUG_QUIET,
+				"Could not change to node directory %s: %s\n",
+				directory, errMsg.Value() );
+		return false;
+	}
+
+	_subprocID++;
+		// Special CondorID for NOOP jobs -- actually indexed by
+		// otherwise-unused subprocID.
+	condorID._cluster = 0;
+	condorID._proc = Job::NOOP_NODE_PROCID;
+	condorID._subproc = _subprocID;
+
+
+	WriteUserLog ulog;
+	ulog.setEnableGlobalLog( false );
+	ulog.setUseXML( logIsXml );
+	ulog.initialize( logFile, condorID._cluster, condorID._proc,
+				condorID._subproc, NULL );
+
+
+	SubmitEvent subEvent;
+	subEvent.cluster = condorID._cluster;
+	subEvent.proc = condorID._proc;
+	subEvent.subproc = condorID._subproc;
+
+		// We need some value for submitHost for the event to be read
+		// correctly.
+	sprintf( subEvent.submitHost, "<dummy-submit-for-noop-job>");
+
+	MyString subEventNotes("DAG Node: " );
+	subEventNotes += DAGNodeName;
+		// submitEventLogNotes get deleted in SubmitEvent destructor.
+	subEvent.submitEventLogNotes = strnewp( subEventNotes.Value() );
+
+	if ( !ulog.writeEvent( &subEvent ) ) {
+		EXCEPT( "Error: writing dummy submit event for NOOP node failed!\n" );
+		return false;
+	}
+
+
+	JobTerminatedEvent termEvent;
+	termEvent.cluster = condorID._cluster;
+	termEvent.proc = condorID._proc;
+	termEvent.subproc = condorID._subproc;
+	termEvent.normal = true;
+	termEvent.returnValue = 0;
+	termEvent.signalNumber = 0;
+
+	if ( !ulog.writeEvent( &termEvent ) ) {
+		EXCEPT( "Error: writing dummy terminated event for NOOP node failed!\n" );
+		return false;
+	}
+
+	return true;
+}
