@@ -25,9 +25,16 @@
 #include "dagman_main.h"
 #include "read_multiple_logs.h"
 #include "throttle_by_category.h"
+#include "dag.h"
+#include <set>
+
+static const char *JOB_TAG_NAME = "+job_tag_name";
+static const char *PEGASUS_SITE = "+pegasus_site";
 
 //---------------------------------------------------------------------------
 JobID_t Job::_jobID_counter = 0;  // Initialize the static data memeber
+int Job::NOOP_NODE_PROCID = INT_MAX;
+int Job::_nextJobstateSeqNum = 1;
 
 //---------------------------------------------------------------------------
 // NOTE: this must be kept in sync with the queue_t enum
@@ -68,24 +75,39 @@ Job::~Job() {
     // http://support.microsoft.com/support/kb/articles/Q131/3/22.asp
 	delete [] _jobName;
 	delete [] _logFile;
+
+	varNamesFromDag->Rewind();
+	MyString *name;
+	while ( (name = varNamesFromDag->Next()) ) {
+		delete name;
+	}
 	delete varNamesFromDag;
+
+	varValsFromDag->Rewind();
+	MyString *val;
+	while ( (val = varValsFromDag->Next()) ) {
+		delete val;
+	}
 	delete varValsFromDag;
+
 	delete _scriptPre;
 	delete _scriptPost;
+
+	delete [] _jobTag;
 }
 
-Job::
-Job( const job_type_t jobType, const char* jobName, const char *directory,
-			const char* cmdFile, bool prohibitMultiJobs ) :
+//---------------------------------------------------------------------------
+Job::Job( const job_type_t jobType, const char* jobName,
+			const char *directory, const char* cmdFile,
+			bool prohibitMultiJobs ) :
 	_jobType( jobType )
 {
 	Init( jobName, directory, cmdFile, prohibitMultiJobs );
 }
 
-
-void Job::
-Init( const char* jobName, const char* directory, const char* cmdFile,
-			bool prohibitMultiJobs )
+//---------------------------------------------------------------------------
+void Job::Init( const char* jobName, const char* directory,
+			const char* cmdFile, bool prohibitMultiJobs )
 {
 	ASSERT( jobName != NULL );
 	ASSERT( cmdFile != NULL );
@@ -98,13 +120,14 @@ Init( const char* jobName, const char* directory, const char* cmdFile,
     _Status = STATUS_READY;
 	_isIdle = false;
 	countedAsDone = false;
-	_waitingCount = 0;
 
     _jobName = strnewp (jobName);
 	_directory = strnewp (directory);
     _cmdFile = strnewp (cmdFile);
 	_dagFile = NULL;
 	_throttleInfo = NULL;
+	_logIsMonitored = false;
+	_useDefaultLog = false;
 
 	if ( (_jobType == TYPE_CONDOR) && prohibitMultiJobs ) {
 		MyString	errorMsg;
@@ -112,12 +135,12 @@ Init( const char* jobName, const char* directory, const char* cmdFile,
 					MyString( _cmdFile ), MyString( _directory ),
 					errorMsg );
 		if ( queueCount == -1 ) {
-			debug_printf( DEBUG_NORMAL, "ERROR in "
+			debug_printf( DEBUG_QUIET, "ERROR in "
 						"MultiLogFiles::getQueueCountFromSubmitFile(): %s\n",
 						errorMsg.Value() );
 			main_shutdown_rescue( EXIT_ERROR );
 		} else if ( queueCount != 1 ) {
-			debug_printf( DEBUG_NORMAL, "ERROR: node %s job queues %d "
+			debug_printf( DEBUG_QUIET, "ERROR: node %s job queues %d "
 						"job procs, but DAGMAN_PROHIBIT_MULTI_JOBS is "
 						"set\n", _jobName, queueCount );
 			main_shutdown_rescue( EXIT_ERROR );
@@ -147,19 +170,23 @@ Init( const char* jobName, const char* directory, const char* cmdFile,
 	_hasNodePriority = false;
 	_nodePriority = 0;
 
-		// Note: we use "" for the directory here because when this method
-		// is called we should *already* be in the directory from which
-		// this job is to be run.
-    MyString logFile = MultiLogFiles::loadLogFileNameFromSubFile(_cmdFile, "");
-		// Note: _logFile is needed only for POST script events (as of
-		// 2005-06-23).
-    _logFile = strnewp (logFile.Value());
+    _logFile = NULL;
+	_logFileIsXml = false;
+
+	_noop = false;
+
+	_jobTag = NULL;
+	_jobstateSeqNum = 0;
+	_lastEventTime = 0;
 
 	varNamesFromDag = new List<MyString>;
 	varValsFromDag = new List<MyString>;
 
 	snprintf( error_text, JOB_ERROR_TEXT_MAXLEN, "unknown" );
 
+	_timesHeld = 0;
+	_jobProcsOnHold = 0;
+
 	return;
 }
 //---------------------------------------------------------------------------
@@ -190,32 +217,31 @@ Job::PrefixDirectory(MyString &prefix)
 }
 
 //---------------------------------------------------------------------------
-bool Job::Remove (const queue_t queue, const JobID_t jobID) {
-    _queues[queue].Rewind();
-    JobID_t currentJobID;
-    while(_queues[queue].Next(currentJobID)) {
-        if (currentJobID == jobID) {
-            _queues[queue].DeleteCurrent();
-            return true;
-        }
-    }
-    return false;   // Element Not Found
+bool Job::Remove (const queue_t queue, const JobID_t jobID)
+{
+	if (_queues[queue].erase(jobID) == 0) {
+		return false; // element not found
+	}
+
+	return true;
 }  
 
 //---------------------------------------------------------------------------
 bool
 Job::CheckForLogFile() const
 {
-    MyString logFile = MultiLogFiles::loadLogFileNameFromSubFile( _cmdFile,
-				_directory );
+	bool tmpLogFileIsXml;
+	MyString logFile = MultiLogFiles::loadLogFileNameFromSubFile( _cmdFile,
+				_directory, tmpLogFileIsXml );
 	bool result = (logFile != "");
 	return result;
 }
 
 //---------------------------------------------------------------------------
-void Job::Dump () const {
+void Job::Dump ( const Dag *dag ) const {
     dprintf( D_ALWAYS, "---------------------- Job ----------------------\n");
     dprintf( D_ALWAYS, "      Node Name: %s\n", _jobName );
+    dprintf( D_ALWAYS, "           Noop: %s\n", _noop ? "true" : "false" );
     dprintf( D_ALWAYS, "         NodeID: %d\n", _jobID );
     dprintf( D_ALWAYS, "    Node Status: %s\n", GetStatusName() );
     dprintf( D_ALWAYS, "Node return val: %d\n", retval );
@@ -237,16 +263,17 @@ void Job::Dump () const {
 				 JobTypeString() );
 	}
 	else {
-		dprintf( D_ALWAYS, " %7s Job ID: (%d)\n", JobTypeString(),
-				 _CondorID._cluster );
+		dprintf( D_ALWAYS, " %7s Job ID: (%d.%d.%d)\n", JobTypeString(),
+				 _CondorID._cluster, _CondorID._proc, _CondorID._subproc );
 	}
   
     for (int i = 0 ; i < 3 ; i++) {
         dprintf( D_ALWAYS, "%15s: ", queue_t_names[i] );
-        SimpleListIterator<JobID_t> iList (_queues[i]);
-        JobID_t jobID;
-        while( iList.Next( jobID ) ) {
-			dprintf( D_ALWAYS | D_NOHEADER, "%d, ", jobID );
+
+		set<JobID_t>::const_iterator qit;
+		for (qit = _queues[i].begin(); qit != _queues[i].end(); qit++) {
+			Job *node = dag->Dag::FindNodeByNodeID( *qit );
+			dprintf( D_ALWAYS | D_NOHEADER, "%s, ", node->GetJobName() );
 		}
         dprintf( D_ALWAYS | D_NOHEADER, "<END>\n" );
     }
@@ -263,8 +290,11 @@ void Job::Print (bool condorID) const {
 
 //---------------------------------------------------------------------------
 void job_print (Job * job, bool condorID) {
-    if (job == NULL) dprintf( D_ALWAYS, "(UNKNOWN)");
-    else job->Print(condorID);
+    if (job == NULL) {
+		dprintf( D_ALWAYS, "(UNKNOWN)");
+	} else {
+		job->Print(condorID);
+	}
 }
 
 const char*
@@ -290,12 +320,6 @@ Job::SanityCheck() const
 {
 	bool result = true;
 
-	if( _waitingCount < 0 ) {
-		dprintf( D_ALWAYS, "BADNESS 10000: _waitingCount = %d\n",
-				 _waitingCount );
-		result = false;
-	}
-
 	if( countedAsDone == true && _Status != STATUS_DONE ) {
 		dprintf( D_ALWAYS, "BADNESS 10000: countedAsDone == true but "
 				 "_Status != STATUS_DONE\n" );
@@ -372,7 +396,6 @@ Job::AddParent( Job* parent, MyString &whynot )
 						parent->GetJobName(), GetJobName() );
 			return false;
 		}
-		_waitingCount++;
 	}
 	whynot = "n/a";
     return true;
@@ -427,7 +450,7 @@ Job::AddChild( Job* child, MyString &whynot )
 	}
 
 	if( HasChild( child ) ) {
-		debug_printf( DEBUG_QUIET,
+		debug_printf( DEBUG_NORMAL,
 					"Warning: parent %s already has child %s\n",
 					GetJobName(), child->GetJobName() );
 		return true;
@@ -472,19 +495,24 @@ Job::TerminateFailure()
 bool
 Job::Add( const queue_t queue, const JobID_t jobID )
 {
-	if( _queues[queue].IsMember( jobID ) ) {
+	pair<set<JobID_t>::iterator, bool> ret;
+
+	ret = _queues[queue].insert(jobID);
+
+	if (ret.second == false) {
 		dprintf( D_ALWAYS,
 				 "ERROR: can't add Job ID %d to DAG: already present!",
 				 jobID );
 		return false;
 	}
-	return _queues[queue].Append(jobID);
+
+	return true;
 }
 
 bool
 Job::AddPreScript( const char *cmd, MyString &whynot )
 {
-return AddScript( false, cmd, whynot );
+	return AddScript( false, cmd, whynot );
 }
 
 bool
@@ -544,18 +572,40 @@ Job::GetStatusName() const
 
 bool
 Job::HasChild( Job* child ) {
+	JobID_t cid;
+	set<JobID_t>::iterator it;
+
 	if( !child ) {
 		return false;
 	}
-	return _queues[Q_CHILDREN].IsMember( child->GetJobID() );
+
+	cid = child->GetJobID();
+	it = _queues[Q_CHILDREN].find(cid);
+
+	if (it == _queues[Q_CHILDREN].end()) {
+		return false;
+	}
+
+	return true;
 }
 
 bool
 Job::HasParent( Job* parent ) {
+	JobID_t pid;
+	set<JobID_t>::iterator it;
+
 	if( !parent ) {
 		return false;
 	}
-	return _queues[Q_PARENTS].IsMember( parent->GetJobID() );
+
+	pid = parent->GetJobID();
+	it = _queues[Q_PARENTS].find(pid);
+
+	if (it == _queues[Q_PARENTS].end()) {
+		return false;
+	}
+
+	return true;
 }
 
 
@@ -606,21 +656,14 @@ Job::RemoveDependency( queue_t queue, JobID_t job )
 bool
 Job::RemoveDependency( queue_t queue, JobID_t job, MyString &whynot )
 {
-	JobID_t candidate;
-    _queues[queue].Rewind();
-    while( _queues[queue].Next( candidate ) ) {
-        if( candidate == job ) {
-            _queues[queue].DeleteCurrent();
-			if ( _queues[queue].IsMember( job ) ) {
-				EXCEPT( "Job %d still in queue %d after deletion!!",
-							job, queue );
-			}
-			whynot = "n/a";
-			return true;
-        }
-    }
-	whynot = "no such dependency";
-	return false;
+	if (_queues[queue].erase(job) == 0)
+	{
+		whynot = "no such dependency";
+		return false;
+	}
+
+	whynot = "n/a";
+	return true;
 }
 
 
@@ -649,15 +692,13 @@ const char* Job::JobIdString() const
 int
 Job::NumParents() const
 {
-	int n = _queues[Q_PARENTS].Number();
-	return n;
+	return _queues[Q_PARENTS].size();
 }
 
 int
 Job::NumChildren() const
 {
-	int n = _queues[Q_CHILDREN].Number();
-	return n;
+	return _queues[Q_CHILDREN].size();
 }
 
 void
@@ -667,11 +708,17 @@ Job::SetCategory( const char *categoryName, ThrottleByCategory &catThrottles )
 
 	if ( (_throttleInfo != NULL) &&
 				(tmpName != *(_throttleInfo->_category)) ) {
+			// When we implement the -strict flag (see gittrac # 1755),
+			// should this be a fatal error?
 		debug_printf( DEBUG_NORMAL, "Warning: new category %s for node %s "
 					"overrides old value %s\n", categoryName, GetJobName(),
 					_throttleInfo->_category->Value() );
 	}
 
+		// Note: we must assign a ThrottleInfo here even if the name
+		// already matches, for the case of lifting splices.
+	ThrottleByCategory::ThrottleInfo *oldInfo = _throttleInfo;
+
 	ThrottleByCategory::ThrottleInfo *throttleInfo =
 				catThrottles.GetThrottleInfo( &tmpName );
 	if ( throttleInfo != NULL ) {
@@ -679,6 +726,13 @@ Job::SetCategory( const char *categoryName, ThrottleByCategory &catThrottles )
 	} else {
 		_throttleInfo = catThrottles.AddCategory( &tmpName );
 	}
+
+	if ( oldInfo != _throttleInfo ) {
+		if ( oldInfo != NULL ) {
+			oldInfo->_totalJobs--;
+		}
+		_throttleInfo->_totalJobs++;
+	}
 }
 
 void
@@ -717,3 +771,196 @@ Job::SetDagFile(const char *dagFile)
 	delete _dagFile;
 	_dagFile = strnewp( dagFile );
 }
+
+//---------------------------------------------------------------------------
+bool
+Job::MonitorLogFile( ReadMultipleUserLogs &condorLogReader,
+			ReadMultipleUserLogs &storkLogReader, bool nfsIsError,
+			bool recovery, const char *defaultNodeLog )
+{
+	debug_printf( DEBUG_DEBUG_2,
+				"Attempting to monitor log file for node %s\n",
+				GetJobName() );
+
+	if ( _logIsMonitored ) {
+		debug_printf( DEBUG_DEBUG_1, "Warning: log file for node "
+					"%s is already monitored\n", GetJobName() );
+		return true;
+	}
+
+	ReadMultipleUserLogs &logReader = (_jobType == TYPE_CONDOR) ?
+				condorLogReader : storkLogReader;
+
+    MyString logFileStr;
+	if ( _jobType == TYPE_CONDOR ) {
+    	logFileStr = MultiLogFiles::loadLogFileNameFromSubFile( _cmdFile,
+					_directory, _logFileIsXml );
+	} else {
+		StringList logFiles;
+		MyString tmpResult = MultiLogFiles::loadLogFileNamesFromStorkSubFile(
+					_cmdFile, _directory, logFiles );
+		if ( tmpResult != "" ) {
+			debug_printf( DEBUG_QUIET, "Error getting Stork log file: %s\n",
+						tmpResult.Value() );
+			LogMonitorFailed();
+			return false;
+		} else if ( logFiles.number() != 1 ) {
+			debug_printf( DEBUG_QUIET, "Error: %d Stork log files found "
+						"in submit file %s; we want 1\n",
+						logFiles.number(), _cmdFile );
+			LogMonitorFailed();
+			return false;
+		} else {
+			logFiles.rewind();
+			logFileStr = logFiles.next();
+		}
+	}
+
+	if ( logFileStr == "" ) {
+		logFileStr = defaultNodeLog;
+		_useDefaultLog = true;
+		_logFileIsXml = false;
+		debug_printf( DEBUG_NORMAL, "Unable to get log file from "
+					"submit file %s (node %s); using default (%s)\n",
+					_cmdFile, GetJobName(), logFileStr.Value() );
+	}
+
+		// This function returns true if the log file is on NFS and
+		// that is an error.  If the log file is on NFS, but nfsIsError
+		// is false, it prints a warning but returns false.
+	if ( MultiLogFiles::logFileNFSError( logFileStr.Value(),
+				nfsIsError ) ) {
+		debug_printf( DEBUG_QUIET, "Error: log file %s on NFS\n",
+					logFileStr.Value() );
+		LogMonitorFailed();
+		return false;
+	}
+
+	delete [] _logFile;
+		// Saving log file here in case submit file gets changed.
+	_logFile = strnewp( logFileStr.Value() );
+	debug_printf( DEBUG_DEBUG_2, "Monitoring log file <%s> for node %s\n",
+				GetLogFile(), GetJobName() );
+	CondorError errstack;
+	if ( !logReader.monitorLogFile( GetLogFile(), !recovery, errstack ) ) {
+		errstack.pushf( "DAGMan::Job", DAGMAN_ERR_LOG_FILE,
+					"ERROR: Unable to monitor log file for node %s",
+					GetJobName() );
+		debug_printf( DEBUG_QUIET, "%s\n", errstack.getFullText() );
+		LogMonitorFailed();
+		EXCEPT( "Fatal log file monitoring error!\n" );
+		return false;
+	}
+
+	_logIsMonitored = true;
+
+	return true;
+}
+
+//---------------------------------------------------------------------------
+bool
+Job::UnmonitorLogFile( ReadMultipleUserLogs &condorLogReader,
+			ReadMultipleUserLogs &storkLogReader )
+{
+	debug_printf( DEBUG_DEBUG_2, "Unmonitoring log file <%s> for node %s\n",
+				GetLogFile(), GetJobName() );
+
+	if ( !_logIsMonitored ) {
+		debug_printf( DEBUG_DEBUG_1, "Warning: log file for node "
+					"%s is already unmonitored\n", GetJobName() );
+		return true;
+	}
+
+	ReadMultipleUserLogs &logReader = (_jobType == TYPE_CONDOR) ?
+				condorLogReader : storkLogReader;
+
+	debug_printf( DEBUG_DEBUG_1, "Unmonitoring log file <%s> for node %s\n",
+				GetLogFile(), GetJobName() );
+
+	CondorError errstack;
+	bool result = logReader.unmonitorLogFile( GetLogFile(), errstack );
+	if ( !result ) {
+		errstack.pushf( "DAGMan::Job", DAGMAN_ERR_LOG_FILE,
+					"ERROR: Unable to unmonitor log " "file for node %s",
+					GetJobName() );
+		debug_printf( DEBUG_QUIET, "%s\n", errstack.getFullText() );
+		EXCEPT( "Fatal log file monitoring error!\n" );
+	}
+
+	if ( result ) {
+		delete [] _logFile;
+		_logFile = NULL;
+		_logIsMonitored = false;
+	}
+
+	return result;
+}
+
+//---------------------------------------------------------------------------
+void
+Job::LogMonitorFailed()
+{
+	if ( _Status != Job::STATUS_ERROR ) {
+		_Status = Job::STATUS_ERROR;
+		snprintf( error_text, JOB_ERROR_TEXT_MAXLEN,
+					"Unable to monitor node job log file" );
+		retval = Dag::DAG_ERROR_LOG_MONITOR_ERROR;
+		if ( _scriptPost != NULL) {
+				// let the script know the job's exit status
+			_scriptPost->_retValJob = retval;
+		}
+	}
+}
+
+//---------------------------------------------------------------------------
+const char *
+Job::GetJobstateJobTag()
+{
+	if ( !_jobTag ) {
+		MyString jobTagName = MultiLogFiles::loadValueFromSubFile(
+					_cmdFile, _directory, JOB_TAG_NAME );
+		if ( jobTagName == "" ) {
+			jobTagName = PEGASUS_SITE;
+		} else {
+				// Remove double-quotes
+			int begin = jobTagName[0] == '\"' ? 1 : 0;
+			int last = jobTagName.Length() - 1;
+			int end = jobTagName[last] == '\"' ? last - 1 : last;
+			jobTagName = jobTagName.Substr( begin, end );
+		}
+
+		MyString tmpJobTag = MultiLogFiles::loadValueFromSubFile(
+					_cmdFile, _directory, jobTagName.Value() );
+		if ( tmpJobTag == "" ) {
+			tmpJobTag = "-";
+		} else {
+				// Remove double-quotes
+			int begin = tmpJobTag[0] == '\"' ? 1 : 0;
+			int last = tmpJobTag.Length() - 1;
+			int end = tmpJobTag[last] == '\"' ? last - 1 : last;
+			tmpJobTag = tmpJobTag.Substr( begin, end );
+		}
+		_jobTag = strnewp( tmpJobTag.Value() );
+	}
+
+	return _jobTag;
+}
+
+//---------------------------------------------------------------------------
+int
+Job::GetJobstateSequenceNum()
+{
+	if ( _jobstateSeqNum == 0 ) {
+		_jobstateSeqNum = _nextJobstateSeqNum++;
+	}
+
+	return _jobstateSeqNum;
+}
+
+//---------------------------------------------------------------------------
+void
+Job::SetLastEventTime( const ULogEvent *event )
+{
+	struct tm eventTm = event->eventTime;
+	_lastEventTime = mktime( &eventTm );
+}
