@@ -23,13 +23,19 @@
 
 #include "condor_common.h"      /* for <stdio.h> */
 #include "condor_constants.h"   /* from condor_includes/ directory */
-#include "simplelist.h"         /* from condor_c++_util/ directory */
+#include "simplelist.h"         /* from condor_utils/ directory */
 #include "MyString.h"
 #include "list.h"
 #include "condor_id.h"
 #include "throttle_by_category.h"
+#include "read_multiple_logs.h"
+#include "CondorError.h"
+#include <set>
+
+using namespace std;
 
 class ThrottleByCategory;
+class Dag;
 
 //
 // Local DAGMan includes
@@ -75,7 +81,6 @@ class Job {
 	typedef enum {
 		TYPE_CONDOR,
 		TYPE_STORK,
-		TYPE_NOOP,	// TODO: yet to be implemented...
 	 } job_type_t;
   
     /** Enumeration for specifying which queue for Add() and Remove().
@@ -142,6 +147,8 @@ class Job {
 
 	// explanation text for errors
 	char error_text[JOB_ERROR_TEXT_MAXLEN];
+
+	static int NOOP_NODE_PROCID;
   
     /** Constructor
         @param jobType Type of job in dag file.
@@ -174,11 +181,14 @@ class Job {
 	bool AddPostScript( const char *cmd, MyString &whynot );
 	bool AddScript( bool post, const char *cmd, MyString &whynot );
 
+	void SetNoop( bool value ) { _noop = value; }
+	bool GetNoop( void ) { return _noop; }
+
     Script * _scriptPre;
     Script * _scriptPost;
 
     ///
-    inline SimpleList<JobID_t> & GetQueueRef (const queue_t queue) {
+    inline set<JobID_t> & GetQueueRef (const queue_t queue) {
         return _queues[queue];
     }
 
@@ -190,8 +200,8 @@ class Job {
     */
     bool Add( const queue_t queue, const JobID_t jobID );
 
-    /** Returns true if this job is ready for submittion.
-        @return true if job is submitable, false if not
+    /** Returns true if this job is ready for submission.
+        @return true if job is submittable, false if not
     */
     inline bool CanSubmit () const {
         return (IsEmpty(Job::Q_WAITING) && _Status == STATUS_READY);
@@ -216,7 +226,7 @@ class Job {
         @return true: queue is empty, false: otherwise
     */
     inline bool IsEmpty (const queue_t queue) const {
-        return _queues[queue].IsEmpty();
+        return _queues[queue].empty();
     }
 
     /** Returns the node's current status
@@ -262,9 +272,9 @@ class Job {
 	bool RemoveDependency( queue_t queue, JobID_t job, MyString &whynot );
  
     /** Dump the contents of this Job to stdout for debugging purposes.
-        @param level Only do the dump if the current debug level is >= level
-    */
-    void Dump () const;
+		@param the current DAG (used to translate node ID to node object)
+	*/
+    void Dump ( const Dag *dag ) const;
   
     /** Print the identification info for this Job to stdout.
         @param condorID If true, also print the job's CondorID
@@ -331,6 +341,77 @@ class Job {
 		return _dagFile;
 	}
 
+	/** Monitor this node's Condor or Stork log file with the
+		multiple log reader.  (Must be called before this node's
+		job is submitted.)
+		@param logReader: the multiple log reader
+		@param recovery: whether we're in recovery mode
+		@param defaultNodeLog: the default log file to be used if the
+			node's submit file doesn't define a log file
+		@return true if successful, false if failed
+	*/
+	bool MonitorLogFile( ReadMultipleUserLogs &condorLogReader,
+				ReadMultipleUserLogs &storkLogReader, bool nfsIsError,
+				bool recovery, const char *defaultNodeLog );
+
+	/** Unmonitor this node's Condor or Stork log file with the
+		multiple log reader.  (Must be called after everything is done
+		for this node.)
+		@param logReader: the multiple log reader
+		@return true if successful, false if failed
+	*/
+	bool UnmonitorLogFile( ReadMultipleUserLogs &logReader,
+				ReadMultipleUserLogs &storkLogReader );
+
+		// Whether this node is using the default node log file.
+	bool UsingDefaultLog() { return _useDefaultLog; }
+
+	/** Get the log file for this node.
+		@return the name of this node's log file.
+	*/
+	const char *GetLogFile() const { return _logFile; }
+
+	/** Get whether this node's log file is XML (versus "standard"
+		format).
+		@return true iff the log file is XML.
+	*/
+	bool GetLogFileIsXml() const { return _logFileIsXml; }
+
+	/** Get the jobstate.log job tag for this node.
+		@return The job tag (can be "local"; if no tag is specified,
+			the value will be "-").
+	*/
+	const char *GetJobstateJobTag();
+
+	/** Get the jobstate.log sequence number for this node, assigning one
+		if we haven't already.
+	*/
+	int GetJobstateSequenceNum();
+	
+	/** Reset the jobstate.log sequence number for this node, so we get a
+		new sequence number for node retries, etc.
+	*/
+	void ResetJobstateSequenceNum() { _jobstateSeqNum = 0; }
+
+	/** Set the master jobstate.log sequence number.
+		@param The next sequence number that should be given out.
+	*/
+	static void SetJobstateNextSequenceNum( int nextSeqNum ) {
+		_nextJobstateSeqNum = nextSeqNum;
+	}
+
+	/** Set the last event time for this job to be the time of the given
+		event (this is used as the time for jobstate.log pseudo-events like
+		JOB_SUCCESS).
+		@param The event whose time should be saved.
+	*/
+	void SetLastEventTime( const ULogEvent *event );
+
+	/** Get the time at which the most recent event occurred for this job.
+		@return the last event time.
+	*/
+	time_t GetLastEventTime() { return _lastEventTime; }
+
     /** */ CondorID _CondorID;
     /** */ status_t _Status;
 
@@ -376,11 +457,6 @@ class Job {
 	//DFS ordering of the node
 	int _dfsOrder; 
 
-	//The log file for this job -- needed because we're now allowing
-	//each job to have its own log file.
-	// Note: this is needed only for writing a POST script terminated event.
-	char *_logFile;
-
 	// DAG definition files can now pass named variables into job submit files.
 	// For lack of a pair<> class, I made two lists, and these lists work together
 	// closely. The order of their items defines the mapping between names
@@ -400,6 +476,16 @@ class Job {
 		// Node priority.  Higher number is better priority (submit first).
 	int _nodePriority;
 
+		// The number of times this job has been held.  (Note: the current
+		// implementation counts holds for all procs in a multi-proc cluster
+		// together -- that should get changed eventually.)
+	int _timesHeld;
+
+		// The number of jobs procs of this node that are currently held.
+		// (Note: we may need to track the hold state of each proc in a
+		// cluster separately to correctly deal with multi-proc clusters.)
+	int _jobProcsOnHold;
+
 private:
 
 		// Note: Init moved to private section because calling int more than
@@ -407,6 +493,10 @@ private:
 	void Init( const char* jobName, const char *directory,
 				const char* cmdFile, bool prohibitMultiJobs );
   
+		// Mark this node as failed because of an error in monitoring
+		// the log file.
+  	void LogMonitorFailed();
+
         // strings for job_type_t (e.g., "Condor, "Stork", etc.)
     static const char* _job_type_names[];
 
@@ -429,27 +519,34 @@ private:
   
     /*  Job queues
 	    NOTE: indexed by queue_t
-      
-        parents -> dependencies coming into the Job
-        children -> dependencies going out of the Job
-        waiting -> Jobs on which the current Job is waiting for output 
-    */
-    SimpleList<JobID_t> _queues[3];
-  
-    /*  The ID of this job.  This serves as a primary key for Jobs, where each
-        Job's ID is unique from all the rest
-    */
-    JobID_t _jobID;
+
+		WARNING: The execution order of ready nodes is, and
+		should always be, undefined. This is so noone starts
+		building dag structures that rely on some order of
+		execution. In practice, there is an ordering due to the
+		data structures that hold the information, but we should
+		never rely on that behavior cause the representational
+		structures may change. This comment was written because the
+		data structures *DID* change, and we had to consider the
+		ramifications of it.
+
+		parents -> dependencies coming into the Job 
+		children -> dependencies going out of the Job
+		waiting -> Jobs on which the current Job is waiting for output
+    */ 
+	
+	set<JobID_t> _queues[3];
+
+    /*	The ID of this job.  This serves as a primary key for Jobs, where each
+		Job's ID is unique from all the rest 
+	*/ 
+	JobID_t _jobID;
 
     /*  Ensures that all jobID's are unique.  Starts at zero and increments
         by one for every Job object that is constructed
     */
     static JobID_t _jobID_counter;
 
-		// the number of my parents that have yet to complete
-		// successfully
-	int _waitingCount;
-
 		// True if the node job has been submitted and is idle.
 	bool _isIdle;
 
@@ -457,6 +554,39 @@ private:
 		// ThrottleByCategory object.
 	ThrottleByCategory::ThrottleInfo *_throttleInfo;
 
+		// Whether this node's log file is currently being monitored.
+	bool _logIsMonitored;
+
+		// Whether this node uses the default user log file.
+	bool _useDefaultLog;
+
+		// The log file for this job -- it will be assigned the default
+		// log file name if no log file is specified in the submit file.
+	char *_logFile;
+
+		// Whether the log file is XML.
+	bool _logFileIsXml;
+
+
+		// Whether this is a noop job (shouldn't actually be submitted
+		// to Condor).
+	bool _noop;
+
+		// The job tag for this node ("-" if nothing is specified;
+		// can also be "local").
+	char *_jobTag;
+
+		// The jobstate.log sequence number for this node (used if we are
+		// writing the jobstate.log file for Pegasus or others to read).
+	int _jobstateSeqNum;
+
+		// The next jobstate.log sequence number for the entire DAG.  Note
+		// that, when we run a rescue DAG, we pick up the sequence numbers
+		// from where we left off when we originally ran the DAG.
+	static int _nextJobstateSeqNum;
+
+		// The time of the most recent event related to this job.
+	time_t _lastEventTime;
 };
 
 /** A wrapper function for Job::Print which allows a NULL job pointer.
