@@ -76,14 +76,6 @@ static const char *GMStateNames[] = {
 #define AMAZON_VM_STATE_SHUTTINGDOWN	"shutting-down"
 #define AMAZON_VM_STATE_TERMINATED		"terminated"
 
-// Filenames are case insensitive on Win32, but case sensitive on Unix
-#ifdef WIN32
-#	define file_strcmp _stricmp
-#	define file_contains contains_anycase
-#else
-#	define file_strcmp strcmp
-#	define file_contains contains
-#endif
 
 // TODO: Let the maximum submit attempts be set in the job ad or, better yet,
 // evalute PeriodicHold expression in job ad.
@@ -114,12 +106,12 @@ void AmazonJobReconfig()
 bool AmazonJobAdMatch( const ClassAd *job_ad )
 {
 	int universe;
-	MyString resource;
+	std::string resource;
 	
 	job_ad->LookupInteger( ATTR_JOB_UNIVERSE, universe );
 	job_ad->LookupString( ATTR_GRID_RESOURCE, resource );
 
-	if ( (universe == CONDOR_UNIVERSE_GRID) && (strncasecmp( resource.Value(), "amazon", 6 ) == 0) ) 
+	if ( (universe == CONDOR_UNIVERSE_GRID) && (strncasecmp( resource.c_str(), "amazon", 6 ) == 0) ) 
 	{
 		return true;
 	}
@@ -145,12 +137,13 @@ AmazonJob::AmazonJob( ClassAd *classad )
 {
 dprintf( D_ALWAYS, "================================>  AmazonJob::AmazonJob 1 \n");
 	char buff[16385]; // user data can be 16K, this is 16K+1
-	MyString error_string = "";
+	std::string error_string = "";
 	char *gahp_path = NULL;
 	char *gahp_log = NULL;
-	int gahp_worker_cnt = NULL;
+	int gahp_worker_cnt = 0;
 	char *gahp_debug = NULL;
 	ArgList args;
+	std::string value;
 	
 	remoteJobId = NULL;
 	remoteJobState = "";
@@ -193,14 +186,13 @@ dprintf( D_ALWAYS, "================================>  AmazonJob::AmazonJob 1 \n
 	m_user_data = NULL;
 	m_user_data_file = NULL;	
 	
-	// if user assigns both user_data and user_data_file, only user_data_file
-	// will be used.
+	// if user assigns both user_data and user_data_file, the two will
+	// be concatenated by the gahp
 	if ( jobAd->LookupString( ATTR_AMAZON_USER_DATA_FILE, buff ) ) {
 		m_user_data_file = strdup(buff);	
-	} else {
-		if ( jobAd->LookupString( ATTR_AMAZON_USER_DATA, buff ) ) {
-			m_user_data = strdup(buff);
-		}
+	}
+	if ( jobAd->LookupString( ATTR_AMAZON_USER_DATA, buff ) ) {
+		m_user_data = strdup(buff);
 	}
 	
 	// get VM instance type
@@ -243,14 +235,40 @@ dprintf( D_ALWAYS, "================================>  AmazonJob::AmazonJob 1 \n
 		jobAd->AssignExpr( ATTR_HOLD_REASON, "Undefined" );
 	}
 
+	jobAd->LookupString( ATTR_GRID_RESOURCE, value );
+	if ( value.length() ) {
+		const char *token;
+
+		Tokenize( value );
+
+		token = GetNextToken( " ", false );
+		if ( !token || strcasecmp( token, "amazon" ) ) {
+			sprintf( error_string, "%s not of type amazon",
+								  ATTR_GRID_RESOURCE );
+			goto error_exit;
+		}
+
+		token = GetNextToken( " ", false );
+		if ( token && *token ) {
+			m_serviceUrl = token;
+		} else {
+			sprintf( error_string, "%s missing EC2 service URL",
+								  ATTR_GRID_RESOURCE );
+			goto error_exit;
+		}
+
+	} else {
+		sprintf( error_string, "%s is not set in the job ad",
+							  ATTR_GRID_RESOURCE );
+		goto error_exit;
+	}
+
 	gahp_path = param( "AMAZON_GAHP" );
 	if ( gahp_path == NULL ) {
 		error_string = "AMAZON_GAHP not defined";
 		goto error_exit;
 	}
 
-	snprintf( buff, sizeof(buff), AMAZON_RESOURCE_NAME ); // for client's ID
-
 	gahp_log = param( "AMAZON_GAHP_LOG" );
 	if ( gahp_log == NULL ) {
 		dprintf(D_ALWAYS, "Warning: No AMAZON_GAHP_LOG defined\n");
@@ -277,46 +295,50 @@ dprintf( D_ALWAYS, "================================>  AmazonJob::AmazonJob 1 \n
 		free(gahp_debug);
 	}
 
-	gahp = new GahpClient( buff, gahp_path, &args );
+	gahp = new GahpClient( AMAZON_RESOURCE_NAME, gahp_path, &args );
 	free(gahp_path);
 	gahp->setNotificationTimerId( evaluateStateTid );
 	gahp->setMode( GahpClient::normal );
 	gahp->setTimeout( gahpCallTimeout );
 
-	myResource = AmazonResource::FindOrCreateResource( AMAZON_RESOURCE_NAME, m_public_key_file, m_private_key_file );
+	myResource = AmazonResource::FindOrCreateResource( m_serviceUrl.c_str(), m_public_key_file, m_private_key_file );
 	myResource->RegisterJob( this );
 
-	buff[0] = '\0';
-	jobAd->LookupString( ATTR_GRID_JOB_ID, buff );
-	if ( buff[0] ) {
+	jobAd->LookupString( ATTR_GRID_JOB_ID, value );
+	if ( value.length() ) {
 		const char *token;
-		MyString str = buff;
 
-		str.Tokenize();
+		Tokenize( value );
 
-		token = str.GetNextToken( " ", false );
-		if ( !token || stricmp( token, "amazon" ) ) {
-			error_string.sprintf( "%s not of type amazon",
-								  ATTR_GRID_JOB_ID );
+		token = GetNextToken( " ", false );
+		if ( !token || strcasecmp( token, "amazon" ) ) {
+			sprintf( error_string, "%s not of type amazon", ATTR_GRID_JOB_ID );
 			goto error_exit;
 		}
 
-		token = str.GetNextToken( " ", false );
+			// Skip the service URL
+		GetNextToken( " ", false );
+
+		token = GetNextToken( " ", false );
 		if ( token ) {
 			m_key_pair = token;
 		}
 
-		token = str.GetNextToken( " ", false );
+		token = GetNextToken( " ", false );
 		if ( token ) {
 			remoteJobId = strdup( token );
 		}
 	}
+
+	if ( !m_key_pair.empty() ) {
+		myResource->AlreadySubmitted( this );
+	}
 	
 	jobAd->LookupString( ATTR_GRID_JOB_STATUS, remoteJobState );
 
 	// JEF: Increment a GMSession attribute for use in letting the job
 	// ad crash the gridmanager on request
-	if ( jobAd->Lookup( "CrashGM" ) != NULL ) {
+	if ( jobAd->LookupExpr( "CrashGM" ) != NULL ) {
 		int session = 0;
 		jobAd->LookupInteger( "GMSession", session );
 		session++;
@@ -327,8 +349,8 @@ dprintf( D_ALWAYS, "================================>  AmazonJob::AmazonJob 1 \n
 
  error_exit:
 	gmState = GM_HOLD;
-	if ( !error_string.IsEmpty() ) {
-		jobAd->Assign( ATTR_HOLD_REASON, error_string.Value() );
+	if ( !error_string.empty() ) {
+		jobAd->Assign( ATTR_HOLD_REASON, error_string.c_str() );
 	}
 	
 	return;
@@ -355,13 +377,14 @@ void AmazonJob::Reconfig()
 }
 
 
-int AmazonJob::doEvaluateState()
+void AmazonJob::doEvaluateState()
 {
 	int old_gm_state;
 	bool reevaluate_state = true;
 	time_t now = time(NULL);
 
-	bool done;
+	bool attr_exists;
+	bool attr_dirty;
 	int rc;
 
 	daemonCore->Reset_Timer( evaluateStateTid, TIMER_NEVER );
@@ -385,6 +408,7 @@ int AmazonJob::doEvaluateState()
 		int should_crash = 0;
 		jobAd->Assign( "GMState", gmState );
 		jobAd->SetDirtyFlag( "GMState", false );
+
 		if ( jobAd->EvalBool( "CrashGM", NULL, should_crash ) && should_crash ) {
 			EXCEPT( "Crashing gridmanager at the request of job %d.%d",
 					procID.cluster, procID.proc );
@@ -402,11 +426,13 @@ int AmazonJob::doEvaluateState()
 				// constructor is called while we're connected to the schedd).
 
 				// JEF: Save GMSession to the schedd if needed
-				if ( requestScheddUpdate( this ) == false ) {
+				jobAd->GetDirtyFlag( "GMSession", &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 
-				if ( gahp->Startup() == false ) {
+				if (gahp && gahp->Startup() == false ) {
 					dprintf( D_ALWAYS, "(%d.%d) Error starting GAHP\n", procID.cluster, procID.proc );
 					jobAd->Assign( ATTR_HOLD_REASON, "Failed to start GAHP" );
 					gmState = GM_HOLD;
@@ -464,11 +490,13 @@ int AmazonJob::doEvaluateState()
 				}
 
 				if ( m_key_pair == "" ) {
-					SetKeypairId( build_keypair().Value() );
+					SetKeypairId( build_keypair().c_str() );
 				}
-				if ( requestScheddUpdate( this ) == false ) {
+				jobAd->GetDirtyFlag( ATTR_GRID_JOB_ID, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
 						// The keypair name still needs to be saved to
 						//the schedd
+					requestScheddUpdate( this, true );
 					break;
 				}
 				gmState = GM_CREATE_KEYPAIR;
@@ -513,8 +541,8 @@ int AmazonJob::doEvaluateState()
 					if ( m_group_names == NULL )	m_group_names = build_groupnames();
 					
 					// amazon_vm_start() will check the input arguments
-					rc = gahp->amazon_vm_start( m_public_key_file, m_private_key_file, 
-												m_ami_id.Value(), m_key_pair.Value(), 
+					rc = gahp->amazon_vm_start( m_serviceUrl.c_str(), m_public_key_file, m_private_key_file, 
+												m_ami_id.c_str(), m_key_pair.c_str(), 
 												m_user_data, m_user_data_file, m_instance_type, 
 												*m_group_names, instance_id, gahp_error_code);
 					
@@ -535,8 +563,6 @@ int AmazonJob::doEvaluateState()
 						break;
 					}
 
-					myResource->SubmitComplete( this );
-
 					if ( rc == 0 ) {
 						
 						ASSERT( instance_id != NULL );
@@ -555,7 +581,7 @@ int AmazonJob::doEvaluateState()
 						errorString = gahp->getErrorString();
 						dprintf(D_ALWAYS,"(%d.%d) job submit failed: %s: %s\n",
 								procID.cluster, procID.proc, gahp_error_code,
-								errorString.Value() );
+								errorString.c_str() );
 						gmState = GM_HOLD;
 					}
 					
@@ -582,7 +608,7 @@ int AmazonJob::doEvaluateState()
 				// check if the VM has been started successfully
 				StringList returnStatus;
 							
-				rc = gahp->amazon_vm_vm_keypair_all(m_public_key_file, m_private_key_file,
+				rc = gahp->amazon_vm_vm_keypair_all(m_serviceUrl.c_str(), m_public_key_file, m_private_key_file,
 												    returnStatus, gahp_error_code);
 
 				if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED || rc == GAHPCLIENT_COMMAND_PENDING ) {
@@ -606,7 +632,7 @@ int AmazonJob::doEvaluateState()
 						instance_id = returnStatus.next();
 						keypair_name = returnStatus.next();
 
-						if (strcmp(m_key_pair.Value(), keypair_name) == 0) {
+						if (strcmp(m_key_pair.c_str(), keypair_name) == 0) {
 							is_running = true;
 							break;
 						}
@@ -615,7 +641,6 @@ int AmazonJob::doEvaluateState()
 					if ( is_running ) {
 
 						// there is a running VM instance corresponding to the given SSH keypair
-						myResource->SubmitComplete( this );
 						gmState = GM_SAVE_INSTANCE_ID;
 						// save the instance ID which will be used when delete VM instance
 						SetInstanceId( instance_id );
@@ -632,7 +657,7 @@ int AmazonJob::doEvaluateState()
 					errorString = gahp->getErrorString();
 					dprintf(D_ALWAYS,"(%d.%d) VM check failed: %s: %s\n",
 							procID.cluster, procID.proc, gahp_error_code,
-							errorString.Value() );
+							errorString.c_str() );
 					gmState = GM_HOLD;
 				}
 
@@ -643,9 +668,10 @@ int AmazonJob::doEvaluateState()
 			
 			case GM_SAVE_INSTANCE_ID:
 
-				done = requestScheddUpdate( this );
-				if ( !done ) {
+				jobAd->GetDirtyFlag( ATTR_GRID_JOB_ID, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
 					// Wait for the instance id to be saved to the schedd
+					requestScheddUpdate( this, true );
 					break;
 				}					
 				gmState = GM_SUBMITTED;
@@ -695,8 +721,9 @@ int AmazonJob::doEvaluateState()
 				if ( condorState != HELD && condorState != REMOVED ) {
 					JobTerminated();
 					if ( condorState == COMPLETED ) {
-						done = requestScheddUpdate( this );
-						if ( !done ) {
+						jobAd->GetDirtyFlag( ATTR_JOB_STATUS, &attr_exists, &attr_dirty );
+						if ( attr_exists && attr_dirty ) {
+							requestScheddUpdate( this, true );
 							break;
 						}
 					}
@@ -801,9 +828,11 @@ int AmazonJob::doEvaluateState()
 				// through. However, since we registered update events the
 				// first time, requestScheddUpdate won't return done until
 				// they've been committed to the schedd.
-				done = requestScheddUpdate( this );
-				if ( !done ) {
-
+				const char *name;
+				ExprTree *expr;
+				jobAd->ResetExpr();
+				if ( jobAd->NextDirtyExpr(name, expr) ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 
@@ -827,13 +856,13 @@ int AmazonJob::doEvaluateState()
 				if ( condorState == REMOVED || condorState == HELD ) {
 					gmState = GM_CANCEL;
 				} else {
-					MyString new_status;
-					MyString public_dns;
+					std::string new_status;
+					std::string public_dns;
 					StringList returnStatus;
 
 					// need to call amazon_vm_status(), amazon_vm_status() will check input arguments
 					// The VM status we need is saved in the second string of the returned status StringList
-					rc = gahp->amazon_vm_status(m_public_key_file, m_private_key_file, remoteJobId, returnStatus, gahp_error_code );
+					rc = gahp->amazon_vm_status(m_serviceUrl.c_str(), m_public_key_file, m_private_key_file, remoteJobId, returnStatus, gahp_error_code );
 					
 					if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED || rc == GAHPCLIENT_COMMAND_PENDING ) {
 						break;
@@ -845,7 +874,7 @@ int AmazonJob::doEvaluateState()
 						errorString = gahp->getErrorString();
 						dprintf( D_ALWAYS, "(%d.%d) job probe failed: %s: %s\n",
 								 procID.cluster, procID.proc, gahp_error_code,
-								 errorString.Value() );
+								 errorString.c_str() );
 						gmState = GM_HOLD;
 						break;
 					} else {
@@ -858,12 +887,12 @@ int AmazonJob::doEvaluateState()
 
 						// VM Status is the second value in the return string list
 						returnStatus.rewind();
-						// jump to the value I need
-						for (int i=0; i<1; i++) {
+						if ( returnStatus.number() >= 2 ) {
+							// jump to the value I need
 							returnStatus.next();
+							new_status = returnStatus.next();
 						}
-						new_status = returnStatus.next();
-						
+
 						// if amazon VM's state is "running" or beyond,
 						// change condor job status to Running.
 						if ( new_status != remoteJobState &&
@@ -874,7 +903,7 @@ int AmazonJob::doEvaluateState()
 						}
 												
 						remoteJobState = new_status;
-						SetRemoteJobStatus( new_status.Value() );
+						SetRemoteJobStatus( new_status.c_str() );
 										
 						
 						returnStatus.rewind();
@@ -886,7 +915,7 @@ int AmazonJob::doEvaluateState()
 								returnStatus.next();							
 							}
 							public_dns = returnStatus.next();
-							SetRemoteVMName( public_dns.Value() );
+							SetRemoteVMName( public_dns.c_str() );
 						}
 					}
 
@@ -900,7 +929,7 @@ int AmazonJob::doEvaluateState()
 
 				// need to call amazon_vm_stop(), it will only return STOP operation is success or failed
 				// amazon_vm_stop() will check the input arguments
-				rc = gahp->amazon_vm_stop(m_public_key_file, m_private_key_file, remoteJobId, gahp_error_code);
+				rc = gahp->amazon_vm_stop(m_serviceUrl.c_str(), m_public_key_file, m_private_key_file, remoteJobId, gahp_error_code);
 			
 				if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED || rc == GAHPCLIENT_COMMAND_PENDING ) {
 					break;
@@ -913,7 +942,7 @@ int AmazonJob::doEvaluateState()
 					errorString = gahp->getErrorString();
 					dprintf( D_ALWAYS, "(%d.%d) job cancel failed: %s: %s\n",
 							 procID.cluster, procID.proc, gahp_error_code,
-							 errorString.Value() );
+							 errorString.c_str() );
 					gmState = GM_HOLD;
 				}
 				
@@ -938,8 +967,8 @@ int AmazonJob::doEvaluateState()
 				}
 
 				// now create and register this keypair by using amazon_vm_create_keypair()
-				rc = gahp->amazon_vm_create_keypair(m_public_key_file, m_private_key_file, 
-													m_key_pair.Value(), m_key_pair_file.Value(), gahp_error_code);
+				rc = gahp->amazon_vm_create_keypair(m_serviceUrl.c_str(), m_public_key_file, m_private_key_file, 
+													m_key_pair.c_str(), m_key_pair_file.c_str(), gahp_error_code);
 
 				if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
 					break;
@@ -975,7 +1004,7 @@ int AmazonJob::doEvaluateState()
 					errorString = gahp->getErrorString();
 					dprintf(D_ALWAYS,"(%d.%d) job create keypair failed: %s: %s\n",
 							procID.cluster, procID.proc, gahp_error_code,
-							errorString.Value() );
+							errorString.c_str() );
 					gmState = GM_HOLD;
 					break;
 				}
@@ -989,7 +1018,7 @@ int AmazonJob::doEvaluateState()
 				{
 				// Something went wrong during the submit process and
 				// we need to destroy the keypair
-				rc = gahp->amazon_vm_destroy_keypair(m_public_key_file, m_private_key_file, m_key_pair.Value(), gahp_error_code);
+				rc = gahp->amazon_vm_destroy_keypair(m_serviceUrl.c_str(), m_public_key_file, m_private_key_file, m_key_pair.c_str(), gahp_error_code);
 
 				if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED || rc == GAHPCLIENT_COMMAND_PENDING ) {
 					break;
@@ -997,7 +1026,7 @@ int AmazonJob::doEvaluateState()
 
 				if (rc == 0) {
 					// remove temporary keypair local output file
-					if ( !remove_keypair_file(m_key_pair_file.Value()) ) {
+					if ( !remove_keypair_file(m_key_pair_file.c_str()) ) {
 						dprintf(D_ALWAYS,"(%d.%d) job destroy temporary keypair local file failed.\n", procID.cluster, procID.proc);
 					}
 					if ( condorState == REMOVED || condorState == HELD ) {
@@ -1012,7 +1041,7 @@ int AmazonJob::doEvaluateState()
 					errorString = gahp->getErrorString();
 					dprintf( D_ALWAYS,"(%d.%d) job destroy keypair failed: %s: %s\n",
 							 procID.cluster, procID.proc, gahp_error_code,
-							 errorString.Value() );
+							 errorString.c_str() );
 					gmState = GM_HOLD;
 				}
 									
@@ -1024,7 +1053,7 @@ int AmazonJob::doEvaluateState()
 			case GM_DESTROY_KEYPAIR:
 				{
 				// Yes, now let's destroy the temporary keypair 
-				rc = gahp->amazon_vm_destroy_keypair(m_public_key_file, m_private_key_file, m_key_pair.Value(), gahp_error_code);
+				rc = gahp->amazon_vm_destroy_keypair(m_serviceUrl.c_str(),m_public_key_file, m_private_key_file, m_key_pair.c_str(), gahp_error_code);
 
 				if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED || rc == GAHPCLIENT_COMMAND_PENDING ) {
 					break;
@@ -1032,7 +1061,7 @@ int AmazonJob::doEvaluateState()
 
 				if (rc == 0) {
 					// remove temporary keypair local output file
-					if ( remove_keypair_file(m_key_pair_file.Value()) ) {
+					if ( remove_keypair_file(m_key_pair_file.c_str()) ) {
 						gmState = GM_FAILED;
 					} else {
 						dprintf(D_ALWAYS,"(%d.%d) job destroy keypair local file failed.\n", procID.cluster, procID.proc);
@@ -1043,7 +1072,7 @@ int AmazonJob::doEvaluateState()
 					errorString = gahp->getErrorString();
 					dprintf( D_ALWAYS,"(%d.%d) job destroy keypair failed: %s: %s\n",
 							 procID.cluster, procID.proc, gahp_error_code,
-							 errorString.Value() );
+							 errorString.c_str() );
 					gmState = GM_HOLD;
 				}
 									
@@ -1065,7 +1094,7 @@ int AmazonJob::doEvaluateState()
 					holdReason[sizeof(holdReason)-1] = '\0';
 					jobAd->LookupString( ATTR_HOLD_REASON, holdReason, sizeof(holdReason) - 1 );
 					if ( holdReason[0] == '\0' && errorString != "" ) {
-						strncpy( holdReason, errorString.Value(), sizeof(holdReason) - 1 );
+						strncpy( holdReason, errorString.c_str(), sizeof(holdReason) - 1 );
 					} else if ( holdReason[0] == '\0' ) {
 						strncpy( holdReason, "Unspecified gridmanager error", sizeof(holdReason) - 1 );
 					}
@@ -1121,8 +1150,6 @@ int AmazonJob::doEvaluateState()
 		
 	} // end of do_while
 	while ( reevaluate_state );	
-
-	return TRUE;
 }
 
 
@@ -1141,7 +1168,7 @@ void AmazonJob::SetRemoteVMName(const char * name)
 		jobAd->AssignExpr( ATTR_AMAZON_REMOTE_VM_NAME, "Undefined" );
 	}
 	
-	requestScheddUpdate( this );
+	requestScheddUpdate( this, false );
 }
 
 
@@ -1152,7 +1179,7 @@ void AmazonJob::SetKeypairId( const char *keypair_id )
 	} else {
 		m_key_pair = keypair_id;
 	}
-	SetRemoteJobId( m_key_pair.Value(), remoteJobId );
+	SetRemoteJobId( m_key_pair.c_str(), remoteJobId );
 }
 
 void AmazonJob::SetInstanceId( const char *instance_id )
@@ -1163,20 +1190,20 @@ void AmazonJob::SetInstanceId( const char *instance_id )
 	} else {
 		remoteJobId = NULL;
 	}
-	SetRemoteJobId( m_key_pair.Value(), remoteJobId );
+	SetRemoteJobId( m_key_pair.c_str(), remoteJobId );
 }
 
 // SetRemoteJobId() is used to set the value of global variable "remoteJobID"
 void AmazonJob::SetRemoteJobId( const char *keypair_id, const char *instance_id )
 {
-	MyString full_job_id;
+	std::string full_job_id;
 	if ( keypair_id && keypair_id[0] ) {
-		full_job_id.sprintf( "amazon %s", keypair_id );
+		sprintf( full_job_id, "amazon %s %s", m_serviceUrl.c_str(), keypair_id );
 		if ( instance_id && instance_id[0] ) {
-			full_job_id.sprintf_cat( " %s", instance_id );
+			sprintf_cat( full_job_id, " %s", instance_id );
 		}
 	}
-	BaseJob::SetRemoteJobId( full_job_id.Value() );
+	BaseJob::SetRemoteJobId( full_job_id.c_str() );
 }
 
 
@@ -1184,9 +1211,9 @@ void AmazonJob::SetRemoteJobId( const char *keypair_id, const char *instance_id
 
 // if ami_id is empty, client must have assigned upload file name value
 // otherwise the condor_submit will report an error.
-MyString AmazonJob::build_ami_id()
+std::string AmazonJob::build_ami_id()
 {
-	MyString ami_id;
+	std::string ami_id;
 	char* buffer = NULL;
 	
 	if ( jobAd->LookupString( ATTR_AMAZON_AMI_ID, &buffer ) ) {
@@ -1197,7 +1224,7 @@ MyString AmazonJob::build_ami_id()
 }
 
 
-MyString AmazonJob::build_keypair()
+std::string AmazonJob::build_keypair()
 {
 	// Build a name for the ssh keypair that will be unique to this job.
 	// Our pattern is SSH_<collector name>_<GlobalJobId>
@@ -1215,11 +1242,11 @@ MyString AmazonJob::build_keypair()
 	}
 
 	// use "ATTR_GLOBAL_JOB_ID" to get unique global job id
-	MyString job_id;
+	std::string job_id;
 	jobAd->LookupString( ATTR_GLOBAL_JOB_ID, job_id );
 
-	MyString key_pair;
-	key_pair.sprintf( "SSH_%s_%s", pool_name, job_id.Value() );
+	std::string key_pair;
+	sprintf( key_pair, "SSH_%s_%s", pool_name, job_id.c_str() );
 
 	free( pool_name );
 	return key_pair;
