@@ -24,19 +24,22 @@
 #include "condor_debug.h"
 #include "condor_string.h"	// for strnewp and friends
 #include "condor_daemon_core.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 
 #include "gridmanager.h"
 #include "basejob.h"
 #include "condor_config.h"
 #include "condor_email.h"
 #include "classad_helpers.h"
+#include "classad_merge.h"
 
 #define HASH_TABLE_SIZE			500
 
 
 int BaseJob::periodicPolicyEvalTid = TIMER_UNSET;
 
+int BaseJob::m_checkRemoteStatusTid = TIMER_UNSET;
+
 HashTable<PROC_ID, BaseJob *> BaseJob::JobsByProcId( HASH_TABLE_SIZE,
 													 hashFuncPROC_ID );
 HashTable<HashKey, BaseJob *> BaseJob::JobsByRemoteId( HASH_TABLE_SIZE,
@@ -54,10 +57,15 @@ void BaseJob::BaseJobReconfig()
 	tmp_int = param_integer( "PERIODIC_EXPR_INTERVAL", 300 );
 	if ( tmp_int != 0 ) {
 		periodicPolicyEvalTid = daemonCore->Register_Timer( tmp_int, tmp_int,
-							(TimerHandler)&BaseJob::EvalAllPeriodicJobExprs,
-							"EvalAllPeriodicJobExprs", (Service*)NULL );
+							BaseJob::EvalAllPeriodicJobExprs,
+							"EvalAllPeriodicJobExprs" );
 	}
 
+	if ( m_checkRemoteStatusTid == TIMER_UNSET ) {
+		m_checkRemoteStatusTid = daemonCore->Register_Timer( 5, 60,
+							BaseJob::CheckAllRemoteStatus,
+							"BaseJob::CheckAllRemoteStatus" );
+	}
 }
 
 BaseJob::BaseJob( ClassAd *classad )
@@ -75,6 +83,9 @@ BaseJob::BaseJob( ClassAd *classad )
 
 	exitStatusKnown = false;
 
+	m_lastRemoteStatusUpdate = 0;
+	m_currentStatusUnknown = false;
+
 	deleteFromGridmanager = false;
 	deleteFromSchedd = false;
 
@@ -85,10 +96,10 @@ BaseJob::BaseJob( ClassAd *classad )
 
 	JobsByProcId.insert( procID, this );
 
-	MyString remote_id;
+	std::string remote_id;
 	jobAd->LookupString( ATTR_GRID_JOB_ID, remote_id );
-	if ( !remote_id.IsEmpty() ) {
-		JobsByRemoteId.insert( HashKey( remote_id.Value() ), this );
+	if ( !remote_id.empty() ) {
+		JobsByRemoteId.insert( HashKey( remote_id.c_str() ), this );
 	}
 
 	jobAd->LookupInteger( ATTR_JOB_STATUS, condorState );
@@ -108,6 +119,9 @@ BaseJob::BaseJob( ClassAd *classad )
 	jobLeaseReceivedExpiredTid = TIMER_UNSET;
 	SetJobLeaseTimers();
 
+	jobAd->LookupInteger( ATTR_LAST_REMOTE_STATUS_UPDATE, m_lastRemoteStatusUpdate );
+	jobAd->LookupBool( ATTR_CURRENT_STATUS_UNKNOWN, m_currentStatusUnknown );
+
 	int tmp_int;
 	if ( jobAd->LookupInteger( ATTR_GRID_RESOURCE_UNAVAILABLE_TIME,
 							   tmp_int ) ) {
@@ -131,12 +145,12 @@ BaseJob::~BaseJob()
 	}
 	JobsByProcId.remove( procID );
 
-	MyString remote_id;
+	std::string remote_id;
 	if ( jobAd ) {
 		jobAd->LookupString( ATTR_GRID_JOB_ID, remote_id );
 	}
-	if ( !remote_id.IsEmpty() ) {
-		JobsByRemoteId.remove( HashKey( remote_id.Value() ) );
+	if ( !remote_id.empty() ) {
+		JobsByRemoteId.remove( HashKey( remote_id.c_str() ) );
 	}
 
 	if ( jobAd ) {
@@ -149,11 +163,10 @@ void BaseJob::SetEvaluateState()
 	daemonCore->Reset_Timer( evaluateStateTid, 0 );
 }
 
-int BaseJob::doEvaluateState()
+void BaseJob::doEvaluateState()
 {
 	JobHeld( "the gridmanager can't handle this job type" );
 	DoneWithJob();
-	return TRUE;
 }
 
 BaseResource *BaseJob::GetResource()
@@ -181,7 +194,7 @@ void BaseJob::JobRunning()
 			executeLogged = true;
 		}
 
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 	}
 }
 
@@ -203,7 +216,7 @@ void BaseJob::JobIdle()
 			executeLogged = false;
 		}
 
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 	}
 }
 
@@ -242,7 +255,7 @@ void BaseJob::JobCompleted()
 
 		UpdateRuntimeStats();
 
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 	}
 }
 
@@ -299,12 +312,21 @@ void BaseJob::DoneWithJob()
 		break;
 	}
 
-	requestScheddUpdate( this );
+	requestScheddUpdate( this, false );
 }
 
 void BaseJob::JobHeld( const char *hold_reason, int hold_code,
 					   int hold_sub_code )
 {
+	bool nonessential = false;
+
+	jobAd->LookupBool(ATTR_JOB_NONESSENTIAL,nonessential);
+	if ( nonessential ) {
+		// don't have the gridmanager put a job on  hold if
+		// the job is nonessential.  instead, just remove it.		
+		JobRemoved(hold_reason);
+		return;
+	}
 	if ( condorState != HELD ) {
 			// if the job was in REMOVED state, make certain we return
 			// to the removed state when it is released.
@@ -338,7 +360,7 @@ void BaseJob::JobHeld( const char *hold_reason, int hold_code,
 			holdLogged = true;
 		}
 
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 	}
 }
 
@@ -353,7 +375,7 @@ void BaseJob::JobRemoved( const char *remove_reason )
 
 		UpdateRuntimeStats();
 
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 	}
 }
 
@@ -377,7 +399,7 @@ void BaseJob::UpdateRuntimeStats()
 		num_job_starts++;
 		jobAd->Assign( ATTR_NUM_JOB_STARTS, num_job_starts );
 
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 
 	} else if ( condorState != RUNNING && shadowBirthdate != 0 ) {
 
@@ -390,15 +412,15 @@ void BaseJob::UpdateRuntimeStats()
 		jobAd->Assign( ATTR_JOB_WALL_CLOCK_CKPT,(char *)NULL );
 		jobAd->Assign( ATTR_SHADOW_BIRTHDATE, 0 );
 
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 
 	}
 }
 
 void BaseJob::SetRemoteJobId( const char *job_id )
 {
-	MyString old_job_id;
-	MyString new_job_id;
+	std::string old_job_id;
+	std::string new_job_id;
 	jobAd->LookupString( ATTR_GRID_JOB_ID, old_job_id );
 	if ( job_id != NULL && job_id[0] != '\0' ) {
 		new_job_id = job_id;
@@ -406,35 +428,58 @@ void BaseJob::SetRemoteJobId( const char *job_id )
 	if ( old_job_id == new_job_id ) {
 		return;
 	}
-	if ( !old_job_id.IsEmpty() ) {
-		JobsByRemoteId.remove( HashKey( old_job_id.Value() ) );
+	if ( !old_job_id.empty() ) {
+		JobsByRemoteId.remove( HashKey( old_job_id.c_str() ) );
 		jobAd->AssignExpr( ATTR_GRID_JOB_ID, "Undefined" );
+	} else {
+		//  old job id was NULL
+		m_lastRemoteStatusUpdate = time(NULL);
+		jobAd->Assign( ATTR_LAST_REMOTE_STATUS_UPDATE, m_lastRemoteStatusUpdate );
 	}
-	if ( !new_job_id.IsEmpty() ) {
-		JobsByRemoteId.insert( HashKey( new_job_id.Value() ), this );
-		jobAd->Assign( ATTR_GRID_JOB_ID, new_job_id.Value() );
+	if ( !new_job_id.empty() ) {
+		JobsByRemoteId.insert( HashKey( new_job_id.c_str() ), this );
+		jobAd->Assign( ATTR_GRID_JOB_ID, new_job_id.c_str() );
+	} else {
+		// new job id is NULL
+		m_lastRemoteStatusUpdate = 0;
+		jobAd->Assign( ATTR_LAST_REMOTE_STATUS_UPDATE, 0 );
+		m_currentStatusUnknown = false;
+		jobAd->Assign( ATTR_CURRENT_STATUS_UNKNOWN, false );
 	}
-	requestScheddUpdate( this );
+	requestScheddUpdate( this, false );
 }
 
-void BaseJob::SetRemoteJobStatus( const char *job_status )
+bool BaseJob::SetRemoteJobStatus( const char *job_status )
 {
-	MyString old_job_status;
-	MyString new_job_status;
+	std::string old_job_status;
+	std::string new_job_status;
+
+	if ( job_status ) {
+		m_lastRemoteStatusUpdate = time(NULL);
+		jobAd->Assign( ATTR_LAST_REMOTE_STATUS_UPDATE, m_lastRemoteStatusUpdate );
+		requestScheddUpdate( this, false );
+		if ( m_currentStatusUnknown == true ) {
+			m_currentStatusUnknown = false;
+			jobAd->Assign( ATTR_CURRENT_STATUS_UNKNOWN, false );
+			WriteJobStatusKnownEventToUserLog( jobAd );
+		}
+	}
+
 	jobAd->LookupString( ATTR_GRID_JOB_STATUS, old_job_status );
 	if ( job_status != NULL && job_status[0] != '\0' ) {
 		new_job_status = job_status;
 	}
 	if ( old_job_status == new_job_status ) {
-		return;
+		return false;
 	}
-	if ( !old_job_status.IsEmpty() ) {
+	if ( !old_job_status.empty() ) {
 		jobAd->AssignExpr( ATTR_GRID_JOB_STATUS, "Undefined" );
 	}
-	if ( !new_job_status.IsEmpty() ) {
-		jobAd->Assign( ATTR_GRID_JOB_STATUS, new_job_status.Value() );
+	if ( !new_job_status.empty() ) {
+		jobAd->Assign( ATTR_GRID_JOB_STATUS, new_job_status.c_str() );
 	}
-	requestScheddUpdate( this );
+	requestScheddUpdate( this, false );
+	return true;
 }
 
 void BaseJob::SetJobLeaseTimers()
@@ -512,7 +557,7 @@ dprintf(D_FULLDEBUG,"(%d.%d) UpdateJobLeaseSent(%d)\n",procID.cluster,procID.pro
 						   new_expiration_time );
 		}
 
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 
 		SetJobLeaseTimers();
 	}
@@ -551,7 +596,7 @@ dprintf(D_FULLDEBUG,"(%d.%d) UpdateJobLeaseReceived(%d)\n",procID.cluster,procID
 	}
 }
 
-int BaseJob::JobLeaseSentExpired()
+void BaseJob::JobLeaseSentExpired()
 {
 dprintf(D_FULLDEBUG,"(%d.%d) BaseJob::JobLeaseSentExpired()\n",procID.cluster,procID.proc);
 	if ( jobLeaseSentExpiredTid != TIMER_UNSET ) {
@@ -559,10 +604,9 @@ dprintf(D_FULLDEBUG,"(%d.%d) BaseJob::JobLeaseSentExpired()\n",procID.cluster,pr
 		jobLeaseSentExpiredTid = TIMER_UNSET;
 	}
 	SetEvaluateState();
-	return 0;
 }
 
-int BaseJob::JobLeaseReceivedExpired()
+void BaseJob::JobLeaseReceivedExpired()
 {
 dprintf(D_FULLDEBUG,"(%d.%d) BaseJob::JobLeaseReceivedExpired()\n",procID.cluster,procID.proc);
 	if ( jobLeaseReceivedExpiredTid != TIMER_UNSET ) {
@@ -578,13 +622,12 @@ dprintf(D_FULLDEBUG,"(%d.%d) BaseJob::JobLeaseReceivedExpired()\n",procID.cluste
 
 	UpdateRuntimeStats();
 
-	requestScheddUpdate( this );
+	requestScheddUpdate( this, false );
 
 	SetEvaluateState();
-	return 0;
 }
 
-void BaseJob::JobAdUpdateFromSchedd( const ClassAd *new_ad )
+void BaseJob::JobAdUpdateFromSchedd( const ClassAd *new_ad, bool full_ad )
 {
 	static const char *held_removed_update_attrs[] = {
 		ATTR_JOB_STATUS,
@@ -605,8 +648,9 @@ void BaseJob::JobAdUpdateFromSchedd( const ClassAd *new_ad )
 	new_ad->LookupInteger( ATTR_JOB_STATUS, new_condor_state );
 
 	if ( new_condor_state == condorState ) {
-			// The job state in the sched hasn't changed, so we can ignore
-			// this "update".
+		if ( !full_ad ) {
+			MergeClassAds( jobAd, const_cast<ClassAd*>(new_ad), true, false );
+		}
 		return;
 	}
 
@@ -624,13 +668,11 @@ void BaseJob::JobAdUpdateFromSchedd( const ClassAd *new_ad )
 	if ( new_condor_state == REMOVED || new_condor_state == HELD ) {
 
 		for ( int i = 0; held_removed_update_attrs[i] != NULL; i++ ) {
-			char attr_value[1024];
 			ExprTree *expr;
 
-			if ( (expr = new_ad->Lookup( held_removed_update_attrs[i] )) != NULL ) {
-				attr_value[0] = '\0';
-				expr->RArg()->PrintToStr(attr_value);
-				jobAd->AssignExpr( held_removed_update_attrs[i], attr_value );
+			if ( (expr = new_ad->LookupExpr( held_removed_update_attrs[i] )) != NULL ) {
+				jobAd->Insert( held_removed_update_attrs[i],
+							   expr->Copy() );
 			} else {
 				jobAd->Delete( held_removed_update_attrs[i] );
 			}
@@ -667,11 +709,13 @@ void BaseJob::JobAdUpdateFromSchedd( const ClassAd *new_ad )
 		condorState = new_condor_state;
 			// TODO do we need to update any other attributes?
 		SetEvaluateState();
+	} else if ( !full_ad ) {
+		MergeClassAds( jobAd, const_cast<ClassAd*>(new_ad), true, false );
 	}
 
 }
 
-int BaseJob::EvalAllPeriodicJobExprs(Service *)
+void BaseJob::EvalAllPeriodicJobExprs()
 {
 	BaseJob *curr_job;
 
@@ -681,8 +725,6 @@ int BaseJob::EvalAllPeriodicJobExprs(Service *)
 	while ( JobsByProcId.iterate( curr_job ) != 0  ) {
 		curr_job->EvalPeriodicJobExpr();
 	}
-
-	return 0;
 }
 
 int BaseJob::EvalPeriodicJobExpr()
@@ -699,25 +741,25 @@ int BaseJob::EvalPeriodicJobExpr()
 
 	RestoreJobTime( old_run_time, old_run_time_dirty );
 
-	MyString reason = user_policy.FiringReason();
-	if ( reason == "" ) {
+	const char *reason = user_policy.FiringReason();
+	if ( reason == NULL ) {
 		reason = "Unknown user policy expression";
 	}
 
 	switch( action ) {
 	case UNDEFINED_EVAL:
-		JobHeld( reason.Value() );
+		JobHeld( reason );
 		SetEvaluateState();
 		break;
 	case STAYS_IN_QUEUE:
 			// do nothing
 		break;
 	case REMOVE_FROM_QUEUE:
-		JobRemoved( reason.Value() );
+		JobRemoved( reason );
 		SetEvaluateState();
 		break;
 	case HOLD_IN_QUEUE:
-		JobHeld( reason.Value() );
+		JobHeld( reason );
 		SetEvaluateState();
 		break;
 	case RELEASE_FROM_HOLD:
@@ -770,14 +812,14 @@ int BaseJob::EvalOnExitJobExpr()
 		jobAd->AssignExpr( ATTR_ON_EXIT_SIGNAL, "Undefined" );
 	}
 
-	MyString reason = user_policy.FiringReason();
-	if ( reason == "" ) {
+	const char *reason = user_policy.FiringReason();
+	if ( reason == NULL ) {
 		reason = "Unknown user policy expression";
 	}
 
 	switch( action ) {
 	case UNDEFINED_EVAL:
-		JobHeld( reason.Value() );
+		JobHeld( reason );
 		break;
 	case STAYS_IN_QUEUE:
 			// clean up job but don't set status to complete
@@ -786,7 +828,7 @@ int BaseJob::EvalOnExitJobExpr()
 		JobCompleted();
 		break;
 	case HOLD_IN_QUEUE:
-		JobHeld( reason.Value() );
+		JobHeld( reason );
 		break;
 	default:
 		EXCEPT( "Unknown action (%d) in BaseJob::EvalAtExitJobExpr", 
@@ -796,6 +838,39 @@ int BaseJob::EvalOnExitJobExpr()
 	return 0;
 }
 
+void BaseJob::CheckAllRemoteStatus()
+{
+	BaseJob *curr_job;
+
+	dprintf( D_FULLDEBUG, "Evaluating staleness of remote job statuses.\n" );
+
+		// TODO Reset timer based on shortest time that a job status could
+		//   become stale?
+	JobsByProcId.startIterations();
+	while ( JobsByProcId.iterate( curr_job ) != 0  ) {
+		curr_job->CheckRemoteStatus();
+	}
+}
+
+void BaseJob::CheckRemoteStatus()
+{
+	const int stale_limit = 15*60;
+
+		// TODO return time that this job status could become stale?
+		// TODO compute stale_limit from job's poll interval?
+		// TODO make stale_limit configurable?
+	if ( m_lastRemoteStatusUpdate == 0 || m_currentStatusUnknown == true ) {
+		return;
+	}
+	if ( time(NULL) > m_lastRemoteStatusUpdate + stale_limit ) {
+		m_currentStatusUnknown = true;
+		jobAd->Assign( ATTR_CURRENT_STATUS_UNKNOWN, true );
+		requestScheddUpdate( this, false );
+		WriteJobStatusUnknownEventToUserLog( jobAd );
+		SetEvaluateState();
+	}
+}
+
 /*Before evaluating user policy expressions, temporarily update
   any stale time values.  Currently, this is just RemoteWallClock.
 */
@@ -853,7 +928,7 @@ void BaseJob::NotifyResourceDown()
 		WriteGlobusResourceDownEventToUserLog( jobAd );
 		WriteGridResourceDownEventToUserLog( jobAd );
 		jobAd->Assign( ATTR_GRID_RESOURCE_UNAVAILABLE_TIME, (int)time(NULL) );
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 	}
 	resourceDown = true;
 	if ( resourcePingPending ) {
@@ -871,7 +946,7 @@ void BaseJob::NotifyResourceUp()
 		WriteGlobusResourceUpEventToUserLog( jobAd );
 		WriteGridResourceUpEventToUserLog( jobAd );
 		jobAd->AssignExpr( ATTR_GRID_RESOURCE_UNAVAILABLE_TIME, "Undefined" );
-		requestScheddUpdate( this );
+		requestScheddUpdate( this, false );
 	}
 	resourceDown = false;
 	if ( resourcePingPending ) {
@@ -886,12 +961,12 @@ void BaseJob::NotifyResourceUp()
 // events and must be deleted when you're done.  This returns NULL if
 // the user didn't want a UserLog, so you must check for NULL before
 // using the pointer you get back.
-UserLog*
+WriteUserLog*
 InitializeUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
 	MyString userLogFile;
-	MyString gjid;
+	std::string gjid;
 	bool use_xml = false;
 
 	if( !getPathToUserLog(job_ad, userLogFile) ) {
@@ -904,8 +979,8 @@ InitializeUserLog( ClassAd *job_ad )
 	job_ad->LookupString( ATTR_GLOBAL_JOB_ID, gjid );
 	job_ad->LookupBool( ATTR_ULOG_USE_XML, use_xml );
 
-	UserLog *ULog = new UserLog();
-	ULog->initialize(userLogFile.Value(), cluster, proc, 0, gjid.Value());
+	WriteUserLog *ULog = new WriteUserLog();
+	ULog->initialize(userLogFile.Value(), cluster, proc, 0, gjid.c_str());
 	ULog->setUseXML( use_xml );
 	return ULog;
 }
@@ -916,7 +991,7 @@ WriteExecuteEventToUserLog( ClassAd *job_ad )
 	int cluster, proc;
 	char hostname[128];
 
-	UserLog *ulog = InitializeUserLog( job_ad );
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -953,7 +1028,7 @@ WriteAbortEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
 	char removeReason[256];
-	UserLog *ulog = InitializeUserLog( job_ad );
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -998,7 +1073,7 @@ WriteTerminateEventToUserLog( ClassAd *job_ad )
 	}
 
 	int cluster, proc;
-	UserLog *ulog = InitializeUserLog( job_ad );
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1086,7 +1161,7 @@ bool
 WriteEvictEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
-	UserLog *ulog = InitializeUserLog( job_ad );
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1130,7 +1205,7 @@ WriteHoldEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
 
-	UserLog *ulog = InitializeUserLog( job_ad );
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1166,8 +1241,8 @@ bool
 WriteGlobusResourceUpEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
-	MyString contact;
-	UserLog *ulog = InitializeUserLog( job_ad );
+	std::string contact;
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1183,14 +1258,14 @@ WriteGlobusResourceUpEventToUserLog( ClassAd *job_ad )
 	GlobusResourceUpEvent event;
 
 	job_ad->LookupString( ATTR_GRID_RESOURCE, contact );
-	if ( contact.IsEmpty() ) {
+	if ( contact.empty() ) {
 			// Not a Globus job, don't log the event
 		delete ulog;
 		return true;
 	}
-	contact.Tokenize();
-	contact.GetNextToken( " ", false );
-	event.rmContact =  strnewp(contact.GetNextToken( " ", false ));
+	Tokenize( contact );
+	GetNextToken( " ", false );
+	event.rmContact =  strnewp(GetNextToken( " ", false ));
 
 	int rc = ulog->writeEvent(&event,job_ad);
 	delete ulog;
@@ -1211,8 +1286,8 @@ bool
 WriteGlobusResourceDownEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
-	MyString contact;
-	UserLog *ulog = InitializeUserLog( job_ad );
+	std::string contact;
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1228,14 +1303,14 @@ WriteGlobusResourceDownEventToUserLog( ClassAd *job_ad )
 	GlobusResourceDownEvent event;
 
 	job_ad->LookupString( ATTR_GRID_RESOURCE, contact );
-	if ( contact.IsEmpty() ) {
+	if ( contact.empty() ) {
 			// Not a Globus job, don't log the event
 		delete ulog;
 		return true;
 	}
-	contact.Tokenize();
-	contact.GetNextToken( " ", false );
-	event.rmContact =  strnewp(contact.GetNextToken( " ", false ));
+	Tokenize( contact );
+	GetNextToken( " ", false );
+	event.rmContact =  strnewp(GetNextToken( " ", false ));
 
 	int rc = ulog->writeEvent(&event,job_ad);
 	delete ulog;
@@ -1256,8 +1331,8 @@ bool
 WriteGlobusSubmitEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
-	MyString contact;
-	UserLog *ulog = InitializeUserLog( job_ad );
+	std::string contact;
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1273,16 +1348,16 @@ WriteGlobusSubmitEventToUserLog( ClassAd *job_ad )
 	GlobusSubmitEvent event;
 
 	job_ad->LookupString( ATTR_GRID_RESOURCE, contact );
-	contact.Tokenize();
-	contact.GetNextToken( " ", false );
-	event.rmContact = strnewp(contact.GetNextToken( " ", false ));
+	Tokenize( contact );
+	GetNextToken( " ", false );
+	event.rmContact = strnewp(GetNextToken( " ", false ));
 
 	job_ad->LookupString( ATTR_GRID_JOB_ID, contact );
-	contact.Tokenize();
-	if ( strcasecmp( contact.GetNextToken( " ", false ), "gt2" ) == 0 ) {
-		contact.GetNextToken( " ", false );
+	Tokenize( contact );
+	if ( strcasecmp( GetNextToken( " ", false ), "gt2" ) == 0 ) {
+		GetNextToken( " ", false );
 	}
-	event.jmContact = strnewp(contact.GetNextToken( " ", false ));
+	event.jmContact = strnewp(GetNextToken( " ", false ));
 
 	event.restartableJM = true;
 
@@ -1306,7 +1381,7 @@ WriteGlobusSubmitFailedEventToUserLog( ClassAd *job_ad, int failure_code,
 	int cluster, proc;
 	char buf[1024];
 
-	UserLog *ulog = InitializeUserLog( job_ad );
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1342,8 +1417,8 @@ bool
 WriteGridResourceUpEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
-	MyString contact;
-	UserLog *ulog = InitializeUserLog( job_ad );
+	std::string contact;
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1359,12 +1434,12 @@ WriteGridResourceUpEventToUserLog( ClassAd *job_ad )
 	GridResourceUpEvent event;
 
 	job_ad->LookupString( ATTR_GRID_RESOURCE, contact );
-	if ( contact.IsEmpty() ) {
+	if ( contact.empty() ) {
 		dprintf( D_ALWAYS,
 				 "(%d.%d) %s attribute missing in job ad\n",
 				 cluster, proc, ATTR_GRID_RESOURCE );
 	}
-	event.resourceName =  strnewp( contact.Value() );
+	event.resourceName =  strnewp( contact.c_str() );
 
 	int rc = ulog->writeEvent( &event, job_ad );
 	delete ulog;
@@ -1383,8 +1458,8 @@ bool
 WriteGridResourceDownEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
-	MyString contact;
-	UserLog *ulog = InitializeUserLog( job_ad );
+	std::string contact;
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1400,12 +1475,12 @@ WriteGridResourceDownEventToUserLog( ClassAd *job_ad )
 	GridResourceDownEvent event;
 
 	job_ad->LookupString( ATTR_GRID_RESOURCE, contact );
-	if ( contact.IsEmpty() ) {
+	if ( contact.empty() ) {
 		dprintf( D_ALWAYS,
 				 "(%d.%d) %s attribute missing in job ad\n",
 				 cluster, proc, ATTR_GRID_RESOURCE );
 	}
-	event.resourceName =  strnewp( contact.Value() );
+	event.resourceName =  strnewp( contact.c_str() );
 
 	int rc = ulog->writeEvent(&event,job_ad);
 	delete ulog;
@@ -1424,8 +1499,8 @@ bool
 WriteGridSubmitEventToUserLog( ClassAd *job_ad )
 {
 	int cluster, proc;
-	MyString contact;
-	UserLog *ulog = InitializeUserLog( job_ad );
+	std::string contact;
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
 	if ( ulog == NULL ) {
 		// User doesn't want a log
 		return true;
@@ -1441,10 +1516,10 @@ WriteGridSubmitEventToUserLog( ClassAd *job_ad )
 	GridSubmitEvent event;
 
 	job_ad->LookupString( ATTR_GRID_RESOURCE, contact );
-	event.resourceName = strnewp( contact.Value() );
+	event.resourceName = strnewp( contact.c_str() );
 
 	job_ad->LookupString( ATTR_GRID_JOB_ID, contact );
-	event.jobId = strnewp( contact.Value() );
+	event.jobId = strnewp( contact.c_str() );
 
 	int rc = ulog->writeEvent( &event,job_ad );
 	delete ulog;
@@ -1459,6 +1534,70 @@ WriteGridSubmitEventToUserLog( ClassAd *job_ad )
 	return true;
 }
 
+bool
+WriteJobStatusUnknownEventToUserLog( ClassAd *job_ad )
+{
+	int cluster, proc;
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
+	if ( ulog == NULL ) {
+		// User doesn't want a log
+		return true;
+	}
+
+	job_ad->LookupInteger( ATTR_CLUSTER_ID, cluster );
+	job_ad->LookupInteger( ATTR_PROC_ID, proc );
+
+	dprintf( D_FULLDEBUG, 
+			 "(%d.%d) Writing job status unknown record to user logfile\n",
+			 cluster, proc );
+
+	JobStatusUnknownEvent event;
+
+	int rc = ulog->writeEvent( &event, job_ad );
+	delete ulog;
+
+	if ( !rc ) {
+		dprintf( D_ALWAYS,
+				 "(%d.%d) Unable to log ULOG_JOB_STATUS_UNKNOWN event\n",
+				 cluster, proc );
+		return false;
+	}
+
+	return true;
+}
+
+bool
+WriteJobStatusKnownEventToUserLog( ClassAd *job_ad )
+{
+	int cluster, proc;
+	WriteUserLog *ulog = InitializeUserLog( job_ad );
+	if ( ulog == NULL ) {
+		// User doesn't want a log
+		return true;
+	}
+
+	job_ad->LookupInteger( ATTR_CLUSTER_ID, cluster );
+	job_ad->LookupInteger( ATTR_PROC_ID, proc );
+
+	dprintf( D_FULLDEBUG, 
+			 "(%d.%d) Writing job status known record to user logfile\n",
+			 cluster, proc );
+
+	JobStatusKnownEvent event;
+
+	int rc = ulog->writeEvent( &event, job_ad );
+	delete ulog;
+
+	if ( !rc ) {
+		dprintf( D_ALWAYS,
+				 "(%d.%d) Unable to log ULOG_JOB_STATUS_KNOWN event\n",
+				 cluster, proc );
+		return false;
+	}
+
+	return true;
+}
+
 // TODO: This appears three times in the Condor source.  Unify?
 //   (It only is made visible in condor_shadow.jim's prototypes.h.)
 static char *
@@ -1532,7 +1671,7 @@ EmailTerminateEvent(ClassAd * job_ad, bool exit_status_known)
 
 		// gather all the info out of the job ad which we want to 
 		// put into the email message.
-	MyString JobName;
+	std::string JobName;
 	job_ad->LookupString( ATTR_JOB_CMD, JobName );
 
 	MyString Args;
@@ -1574,8 +1713,8 @@ EmailTerminateEvent(ClassAd * job_ad, bool exit_status_known)
 	time_t now = time(NULL);
 
 	fprintf( mailer, "Your Condor job %d.%d \n", cluster, proc);
-	if ( JobName.Length() ) {
-		fprintf(mailer,"\t%s %s\n",JobName.Value(),Args.Value());
+	if ( JobName.length() ) {
+		fprintf(mailer,"\t%s %s\n",JobName.c_str(),Args.Value());
 	}
 	if(exit_status_known) {
 		fprintf(mailer, "has ");
