@@ -24,7 +24,7 @@
 #include "condor_debug.h"
 #include "condor_string.h"	// for strnewp and friends
 #include "condor_daemon_core.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "daemon.h"
 #include "dc_schedd.h"
 #include "job_lease.h"
@@ -33,6 +33,8 @@
 #include "gridmanager.h"
 #include "condorjob.h"
 #include "condor_config.h"
+#include "file_transfer.h"
+#include "condor_holdcodes.h"
 
 
 // GridManager job states
@@ -87,7 +89,7 @@ static const char *GMStateNames[] = {
 };
 
 #define JOB_STATE_UNKNOWN				-1
-#define JOB_STATE_UNSUBMITTED			UNEXPANDED
+#define JOB_STATE_UNSUBMITTED			0
 
 // TODO: Let the maximum submit attempts be set in the job ad or, better yet,
 // evalute PeriodicHold expression in job ad.
@@ -132,11 +134,11 @@ void CondorJobReconfig()
 
 bool CondorJobAdMatch( const ClassAd *job_ad ) {
 	int universe;
-	MyString resource;
+	std::string resource;
 	if ( job_ad->LookupInteger( ATTR_JOB_UNIVERSE, universe ) &&
 		 universe == CONDOR_UNIVERSE_GRID &&
 		 job_ad->LookupString( ATTR_GRID_RESOURCE, resource ) &&
-		 strncasecmp( resource.Value(), "condor ", 7 ) == 0 ) {
+		 strncasecmp( resource.c_str(), "condor ", 7 ) == 0 ) {
 
 		return true;
 	}
@@ -159,7 +161,7 @@ CondorJob::CondorJob( ClassAd *classad )
 {
 	char buff[4096];
 	ArgList args;
-	MyString error_string = "";
+	std::string error_string = "";
 	char *gahp_path;
 	bool job_already_submitted = false;
 
@@ -178,12 +180,15 @@ CondorJob::CondorJob( ClassAd *classad )
 	submitterId = NULL;
 	connectFailureCount = 0;
 	jobProxy = NULL;
-	remoteProxyExpireTime = 0;
+	lastProxyExpireTime = 0;
+	delegatedProxyExpireTime = 0;
+	delegatedProxyRenewTime = 0;
 	lastProxyRefreshAttempt = 0;
 	myResource = NULL;
 	newRemoteStatusAd = NULL;
 	newRemoteStatusServerTime = 0;
 	doActivePoll = false;
+	int int_value;
 
 	lastRemoteStatusServerTime = 0;
 
@@ -195,46 +200,52 @@ CondorJob::CondorJob( ClassAd *classad )
 		jobAd->AssignExpr( ATTR_HOLD_REASON, "Undefined" );
 	}
 
-	jobProxy = AcquireProxy( jobAd, error_string, evaluateStateTid );
+	jobProxy = AcquireProxy( jobAd, error_string,
+							 (TimerHandlercpp)&BaseJob::SetEvaluateState, this );
 	if ( jobProxy == NULL && error_string != "" ) {
 		goto error_exit;
 	}
 
+	int_value = 0;
+	if ( jobAd->LookupInteger( ATTR_DELEGATED_PROXY_EXPIRATION, int_value ) ) {
+		delegatedProxyExpireTime = (time_t)int_value;
+		delegatedProxyRenewTime = GetDelegatedProxyRenewalTime(delegatedProxyExpireTime);
+	}
+
 	buff[0] = '\0';
 	jobAd->LookupString( ATTR_GRID_RESOURCE, buff );
 	if ( buff[0] != '\0' ) {
 		const char *token;
-		MyString str = buff;
 
-		str.Tokenize();
+		Tokenize( buff );
 
-		token = str.GetNextToken( " ", false );
-		if ( !token || stricmp( token, "condor" ) ) {
-			error_string.sprintf( "%s not of type condor",
+		token = GetNextToken( " ", false );
+		if ( !token || strcasecmp( token, "condor" ) ) {
+			sprintf( error_string, "%s not of type condor",
 								  ATTR_GRID_RESOURCE );
 			goto error_exit;
 		}
 
-		token = str.GetNextToken( " ", false );
+		token = GetNextToken( " ", false );
 		if ( token && *token ) {
 			remoteScheddName = strdup( token );
 		} else {
-			error_string.sprintf( "%s missing schedd name",
+			sprintf( error_string, "%s missing schedd name",
 								  ATTR_GRID_RESOURCE );
 			goto error_exit;
 		}
 
-		token = str.GetNextToken( " ", false );
+		token = GetNextToken( " ", false );
 		if ( token && *token ) {
 			remotePoolName = strdup( token );
 		} else {
-			error_string.sprintf( "%s missing pool name",
+			sprintf( error_string, "%s missing pool name",
 								  ATTR_GRID_RESOURCE );
 			goto error_exit;
 		}
 
 	} else {
-		error_string.sprintf( "%s is not set in the job ad",
+		sprintf( error_string, "%s is not set in the job ad",
 							  ATTR_GRID_RESOURCE );
 		goto error_exit;
 	}
@@ -257,14 +268,14 @@ CondorJob::CondorJob( ClassAd *classad )
 		}
 		submitterId = strdup( buff );
 	} else {
-		error_string.sprintf( "%s is not set in the job ad",
+		sprintf( error_string, "%s is not set in the job ad",
 							  ATTR_GLOBAL_JOB_ID );
 		goto error_exit;
 	}
 
 	myResource = CondorResource::FindOrCreateResource( remoteScheddName,
 													   remotePoolName,
-													   jobProxy ? jobProxy->subject->subject_name : NULL );
+													   jobProxy );
 	myResource->RegisterJob( this, submitterId );
 	if ( job_already_submitted ) {
 		myResource->AlreadySubmitted( this );
@@ -279,7 +290,7 @@ CondorJob::CondorJob( ClassAd *classad )
 		//   a gahp server can handle multiple schedds
 	sprintf( buff, "CONDOR/%s/%s/%s", remotePoolName ? remotePoolName : "NULL",
 			 remoteScheddName,
-			 jobProxy != NULL ? jobProxy->subject->subject_name : "NULL" );
+			 jobProxy != NULL ? jobProxy->subject->fqan : "NULL" );
 	args.AppendArg("-f");
 	args.AppendArg("-s");
 	args.AppendArg(remoteScheddName);
@@ -300,8 +311,8 @@ CondorJob::CondorJob( ClassAd *classad )
 		// We must ensure that the code-path from GM_HOLD doesn't depend
 		// on any initialization that's been skipped.
 	gmState = GM_HOLD;
-	if ( !error_string.IsEmpty() ) {
-		jobAd->Assign( ATTR_HOLD_REASON, error_string.Value() );
+	if ( !error_string.empty() ) {
+		jobAd->Assign( ATTR_HOLD_REASON, error_string.c_str() );
 	}
 	return;
 }
@@ -309,7 +320,7 @@ CondorJob::CondorJob( ClassAd *classad )
 CondorJob::~CondorJob()
 {
 	if ( jobProxy != NULL ) {
-		ReleaseProxy( jobProxy, evaluateStateTid );
+		ReleaseProxy( jobProxy, (TimerHandlercpp)&BaseJob::SetEvaluateState, this );
 	}
 	if ( submitterId != NULL ) {
 		free( submitterId );
@@ -340,7 +351,7 @@ void CondorJob::Reconfig()
 	gahp->setTimeout( gahpCallTimeout );
 }
 
-int CondorJob::JobLeaseSentExpired()
+void CondorJob::JobLeaseSentExpired()
 {
 dprintf(D_FULLDEBUG,"(%d.%d) CondorJob::JobLeaseSentExpired()\n",procID.cluster,procID.proc);
 	BaseJob::JobLeaseSentExpired();
@@ -350,10 +361,9 @@ dprintf(D_FULLDEBUG,"(%d.%d) CondorJob::JobLeaseSentExpired()\n",procID.cluster,
 	if ( gmState != GM_INIT ) {
 		gmState = GM_CLEAR_REQUEST;
 	}
-	return 0;
 }
 
-int CondorJob::doEvaluateState()
+void CondorJob::doEvaluateState()
 {
 	bool connect_failure = false;
 	int old_gm_state;
@@ -361,7 +371,8 @@ int CondorJob::doEvaluateState()
 	bool reevaluate_state = true;
 	time_t now = time(NULL);
 
-	bool done;
+	bool attr_exists;
+	bool attr_dirty;
 	int rc;
 
 	daemonCore->Reset_Timer( evaluateStateTid, TIMER_NEVER );
@@ -436,12 +447,12 @@ int CondorJob::doEvaluateState()
 			int num_ads = 0;
 			int tmp_int = 0;
 			ClassAd **status_ads = NULL;
-			MyString constraint;
-			constraint.sprintf( "%s==%d&&%s==%d", ATTR_CLUSTER_ID,
+			std::string constraint;
+			sprintf( constraint, "%s==%d&&%s==%d", ATTR_CLUSTER_ID,
 								remoteJobId.cluster, ATTR_PROC_ID,
 								remoteJobId.proc );
 			rc = gahp->condor_job_status_constrained( remoteScheddName,
-													  constraint.Value(),
+													  constraint.c_str(),
 													  &num_ads, &status_ads );
 			if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED ||
 				 rc == GAHPCLIENT_COMMAND_PENDING ) {
@@ -541,8 +552,8 @@ int CondorJob::doEvaluateState()
 			// another one.
 			if ( now >= lastSubmitAttempt + submitInterval ) {
 				// Once RequestSubmit() is called at least once, you must
-				// CancelSubmit() or SubmitComplete() once you're done with
-				// the request call
+				// call CancelSubmit() when there's no job left on the
+				// remote host and you don't plan to submit one.
 				if ( myResource->RequestSubmit(this) == false ) {
 					break;
 				}
@@ -568,7 +579,6 @@ int CondorJob::doEvaluateState()
 					 rc == GAHPCLIENT_COMMAND_PENDING ) {
 					break;
 				}
-				myResource->SubmitComplete(this);
 				lastSubmitAttempt = time(NULL);
 				numSubmitAttempts++;
 				if ( rc == GLOBUS_SUCCESS ) {
@@ -642,8 +652,9 @@ int CondorJob::doEvaluateState()
 			if ( condorState == REMOVED || condorState == HELD ) {
 				gmState = GM_CANCEL;
 			} else {
-				done = requestScheddUpdate( this );
-				if ( !done ) {
+				jobAd->GetDirtyFlag( ATTR_GRID_JOB_ID, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 				gmState = GM_STAGE_IN;
@@ -675,7 +686,15 @@ int CondorJob::doEvaluateState()
 				// HELD. By doing an active probe, we'll automatically
 				// ignore any update ads from before the probe.
 				if(jobProxy) {
-					remoteProxyExpireTime = jobProxy->expiration_time;
+					lastProxyExpireTime = jobProxy->expiration_time;
+					delegatedProxyExpireTime = GetDesiredDelegatedJobCredentialExpiration(jobAd);
+					delegatedProxyRenewTime = GetDelegatedProxyRenewalTime(delegatedProxyExpireTime);
+					time_t actual_expiration = delegatedProxyExpireTime;
+					if( actual_expiration == 0 || actual_expiration > jobProxy->expiration_time ) {
+						actual_expiration = jobProxy->expiration_time;
+					}
+					jobAd->Assign( ATTR_DELEGATED_PROXY_EXPIRATION,
+								   (int)actual_expiration );
 				}
 				gmState = GM_POLL_ACTIVE;
 			} else {
@@ -725,7 +744,8 @@ int CondorJob::doEvaluateState()
 				// The job is on hold remotely but not locally. This means
 				// the remote job needs to be released.
 				gmState = GM_RELEASE_REMOTE_JOB;
-			} else if ( jobProxy && remoteProxyExpireTime < jobProxy->expiration_time ) {
+			} else if ( jobProxy && lastProxyExpireTime < jobProxy->expiration_time ||
+						jobProxy && delegatedProxyRenewTime < now ) {
 				int interval = param_integer( "GRIDMANAGER_PROXY_REFRESH_INTERVAL", 10*60 );
 				if ( now >= lastProxyRefreshAttempt + interval ) {
 					gmState = GM_REFRESH_PROXY;
@@ -760,9 +780,11 @@ int CondorJob::doEvaluateState()
 					}
 					errorString = gahp->getErrorString();
 
-					if ( ( remoteProxyExpireTime != 0 &&
-						   remoteProxyExpireTime < now + 60 ) ||
-						 ( remoteProxyExpireTime == 0 &&
+					if ( ( lastProxyExpireTime != 0 &&
+						   lastProxyExpireTime < now + 60 ) ||
+						 ( delegatedProxyExpireTime != 0 &&
+						   delegatedProxyExpireTime < now + 60 ) ||
+						 ( lastProxyExpireTime == 0 &&
 						   jobProxy->near_expired ) ) {
 
 						dprintf( D_ALWAYS,
@@ -772,7 +794,15 @@ int CondorJob::doEvaluateState()
 						break;
 					}
 				} else {
-					remoteProxyExpireTime = jobProxy->expiration_time;
+					lastProxyExpireTime = jobProxy->expiration_time;
+					delegatedProxyExpireTime = GetDesiredDelegatedJobCredentialExpiration(jobAd);
+					delegatedProxyRenewTime = GetDelegatedProxyRenewalTime(delegatedProxyExpireTime);
+					time_t actual_expiration = delegatedProxyExpireTime;
+					if( actual_expiration == 0 || actual_expiration > jobProxy->expiration_time ) {
+						actual_expiration = jobProxy->expiration_time;
+					}
+					jobAd->Assign( ATTR_DELEGATED_PROXY_EXPIRATION,
+								   (int)actual_expiration );
 				}
 				lastProxyRefreshAttempt = time(NULL);
 				gmState = GM_SUBMITTED;
@@ -839,12 +869,12 @@ int CondorJob::doEvaluateState()
 		case GM_POLL_ACTIVE: {
 			int num_ads;
 			ClassAd **status_ads = NULL;
-			MyString constraint;
-			constraint.sprintf( "%s==%d&&%s==%d", ATTR_CLUSTER_ID,
+			std::string constraint;
+			sprintf( constraint, "%s==%d&&%s==%d", ATTR_CLUSTER_ID,
 								remoteJobId.cluster, ATTR_PROC_ID,
 								remoteJobId.proc );
 			rc = gahp->condor_job_status_constrained( remoteScheddName,
-													  constraint.Value(),
+													  constraint.c_str(),
 													  &num_ads, &status_ads );
 			if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED ||
 				 rc == GAHPCLIENT_COMMAND_PENDING ) {
@@ -881,7 +911,7 @@ int CondorJob::doEvaluateState()
 				// retry the poll until the job is no longer on hold to
 				// spool input files. Not the best solution, but it should
 				// work.
-			MyString hold_reason = "";
+			std::string hold_reason = "";
 			status_ads[0]->LookupString( ATTR_HOLD_REASON, hold_reason );
 			if ( hold_reason == "Spooling input data files" ) {
 				dprintf( D_FULLDEBUG, "(%d.%d) Job not yet released from stage-in hold, retrying poll\n",
@@ -935,8 +965,9 @@ int CondorJob::doEvaluateState()
 			// Report job completion to the schedd.
 			JobTerminated();
 			if ( condorState == COMPLETED ) {
-				done = requestScheddUpdate( this );
-				if ( !done ) {
+				jobAd->GetDirtyFlag( ATTR_JOB_STATUS, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 			}
@@ -975,7 +1006,7 @@ int CondorJob::doEvaluateState()
 				// cleared in GM_CLEAR_REQUEST (it might go to GM_HOLD first).
 				SetRemoteJobId( NULL );
 				myResource->CancelSubmit( this );
-				requestScheddUpdate( this );
+				requestScheddUpdate( this, false );
 				gmState = GM_CLEAR_REQUEST;
 			}
 			} break;
@@ -1041,7 +1072,7 @@ int CondorJob::doEvaluateState()
 			// TODO: Let our action here be dictated by the user preference
 			// expressed in the job ad.
 			if( remoteJobId.cluster != 0 ) {
-				errorString.sprintf( "Internal error: Attempting to clear "
+				sprintf( errorString, "Internal error: Attempting to clear "
 									 "request, but remoteJobId.cluster(%d) "
 									 "!= 0, condorState is %s (%d)",
 									 remoteJobId.cluster,
@@ -1073,11 +1104,14 @@ int CondorJob::doEvaluateState()
 			// through. However, since we registered update events the
 			// first time, requestScheddUpdate won't return done until
 			// they've been committed to the schedd.
-			done = requestScheddUpdate( this );
-			if ( !done ) {
+			const char *name;
+			ExprTree *expr;
+			jobAd->ResetExpr();
+			if ( jobAd->NextDirtyExpr(name, expr) ) {
+				requestScheddUpdate( this, true );
 				break;
 			}
-			remoteProxyExpireTime = 0;
+			lastProxyExpireTime = 0;
 			lastProxyRefreshAttempt = 0;
 			submitLogged = false;
 			executeLogged = false;
@@ -1105,7 +1139,7 @@ int CondorJob::doEvaluateState()
 				jobAd->LookupString( ATTR_HOLD_REASON, holdReason,
 								  sizeof(holdReason) - 1 );
 				if ( holdReason[0] == '\0' && errorString != "" ) {
-					strncpy( holdReason, errorString.Value(),
+					strncpy( holdReason, errorString.c_str(),
 							 sizeof(holdReason) - 1 );
 				}
 				if ( holdReason[0] == '\0' ) {
@@ -1170,14 +1204,12 @@ int CondorJob::doEvaluateState()
 			RequestPing();
 		}
 	}
-
-	return TRUE;
 }
 
 void CondorJob::SetRemoteJobId( const char *job_id )
 {
 	int rc;
-	MyString full_job_id;
+	std::string full_job_id;
 
 	if ( job_id ) {
 		rc = sscanf( job_id, "%d.%d", &remoteJobId.cluster,
@@ -1189,13 +1221,13 @@ void CondorJob::SetRemoteJobId( const char *job_id )
 			return;
 		}
 
-		full_job_id.sprintf( "condor %s %s %s", remoteScheddName,
+		sprintf( full_job_id, "condor %s %s %s", remoteScheddName,
 							 remotePoolName, job_id );
 	} else {
 		remoteJobId.cluster = 0;
 	}
 
-	BaseJob::SetRemoteJobId( full_job_id.Value() );
+	BaseJob::SetRemoteJobId( full_job_id.c_str() );
 }
 
 void CondorJob::NotifyNewRemoteStatus( ClassAd *update_ad )
@@ -1207,7 +1239,6 @@ void CondorJob::NotifyNewRemoteStatus( ClassAd *update_ad )
 		dprintf( D_FULLDEBUG, "(%d.%d) Got NULL classad from CondorResource\n",
 				 procID.cluster, procID.proc );
 		doActivePoll = true;
-		delete update_ad;
 		SetEvaluateState();
 		return;
 	}
@@ -1236,7 +1267,6 @@ void CondorJob::NotifyNewRemoteStatus( ClassAd *update_ad )
 void CondorJob::ProcessRemoteAd( ClassAd *remote_ad )
 {
 	int new_remote_state;
-	MyString buff;
 	ExprTree *new_expr, *old_expr;
 
 	int index;
@@ -1267,6 +1297,7 @@ void CondorJob::ProcessRemoteAd( ClassAd *remote_ad )
 		ATTR_JOB_CORE_DUMPED,
 		ATTR_EXECUTABLE_SIZE,
 		ATTR_IMAGE_SIZE,
+		ATTR_SPOOLED_OUTPUT_FILES,
 		NULL };		// list must end with a NULL
 
 	if ( remote_ad == NULL ) {
@@ -1279,7 +1310,7 @@ void CondorJob::ProcessRemoteAd( ClassAd *remote_ad )
 
 	if (config_attrs_to_copy == NULL) {
 		// use the defaults
-		attrs_to_copy = (char **)default_attrs_to_copy;
+		attrs_to_copy = const_cast<char **>(default_attrs_to_copy);
 	} else {
 		StringList sl(NULL, ", ");
 		freeAttrs = true;
@@ -1287,9 +1318,9 @@ void CondorJob::ProcessRemoteAd( ClassAd *remote_ad )
 		sl.rewind();
 		attrs_to_copy = new char *[sl.number() + 1];
 		for (int i = 0; i < sl.number(); i++) {
-			MyString attribute(sl.next());
-			attrs_to_copy[i] = new char[ attribute.Length() + 1 ];
-			strcpy(attrs_to_copy[i], attribute.Value());
+			std::string attribute(sl.next());
+			attrs_to_copy[i] = new char[ attribute.length() + 1 ];
+			strcpy(attrs_to_copy[i], attribute.c_str());
 		}
 		attrs_to_copy[sl.number()] = NULL;
 		free(config_attrs_to_copy);
@@ -1303,7 +1334,7 @@ void CondorJob::ProcessRemoteAd( ClassAd *remote_ad )
 	if ( new_remote_state == IDLE ) {
 		JobIdle();
 	}
-	if ( new_remote_state == RUNNING ) {
+	if ( new_remote_state == RUNNING || new_remote_state == TRANSFERRING_OUTPUT ) {
 		JobRunning();
 	}
 	// If the job has been removed locally, don't propagate a hold from
@@ -1333,11 +1364,11 @@ void CondorJob::ProcessRemoteAd( ClassAd *remote_ad )
 
 	index = -1;
 	while ( attrs_to_copy[++index] != NULL ) {
-		old_expr = jobAd->Lookup( attrs_to_copy[index] );
-		new_expr = remote_ad->Lookup( attrs_to_copy[index] );
+		old_expr = jobAd->LookupExpr( attrs_to_copy[index] );
+		new_expr = remote_ad->LookupExpr( attrs_to_copy[index] );
 
 		if ( new_expr != NULL && ( old_expr == NULL || !(*old_expr == *new_expr) ) ) {
-			jobAd->Insert( new_expr->DeepCopy() );
+			jobAd->Insert( attrs_to_copy[index], new_expr->Copy() );
 		}
 	}
 
@@ -1350,7 +1381,7 @@ void CondorJob::ProcessRemoteAd( ClassAd *remote_ad )
 		delete [] attrs_to_copy;
 	}
 
-	requestScheddUpdate( this );
+	requestScheddUpdate( this, false );
 
 	return;
 }
@@ -1364,7 +1395,7 @@ BaseResource *CondorJob::GetResource()
 ClassAd *CondorJob::buildSubmitAd()
 {
 	int now = time(NULL);
-	MyString expr;
+	std::string expr;
 	ClassAd *submit_ad;
 	ExprTree *next_expr;
 	int tmp_int;
@@ -1376,17 +1407,9 @@ ClassAd *CondorJob::buildSubmitAd()
 	submit_ad->Delete( ATTR_PROC_ID );
 	submit_ad->Delete( ATTR_USER );
 	submit_ad->Delete( ATTR_OWNER );
-	submit_ad->Delete( ATTR_REMOTE_SCHEDD );
-	submit_ad->Delete( ATTR_REMOTE_POOL );
 	submit_ad->Delete( ATTR_GRID_RESOURCE );
 	submit_ad->Delete( ATTR_JOB_MATCHED );
 	submit_ad->Delete( ATTR_JOB_MANAGED );
-	submit_ad->Delete( ATTR_MIRROR_ACTIVE );
-	submit_ad->Delete( ATTR_MIRROR_JOB_ID );
-	submit_ad->Delete( ATTR_MIRROR_LEASE_TIME );
-	submit_ad->Delete( ATTR_MIRROR_RELEASED );
-	submit_ad->Delete( ATTR_MIRROR_REMOTE_LEASE_TIME );
-	submit_ad->Delete( ATTR_MIRROR_SCHEDD );
 	submit_ad->Delete( ATTR_STAGE_IN_FINISH );
 	submit_ad->Delete( ATTR_STAGE_IN_START );
 	submit_ad->Delete( ATTR_SCHEDD_BIRTHDATE );
@@ -1404,7 +1427,6 @@ ClassAd *CondorJob::buildSubmitAd()
 	submit_ad->Delete( ATTR_GLOBAL_JOB_ID );
 	submit_ad->Delete( "CondorPlatform" );
 	submit_ad->Delete( "CondorVersion" );
-	submit_ad->Delete( ATTR_JOB_GRID_TYPE );
 	submit_ad->Delete( ATTR_WANT_CLAIMING );
 	submit_ad->Delete( ATTR_WANT_MATCHING );
 	submit_ad->Delete( ATTR_HOLD_REASON );
@@ -1426,6 +1448,7 @@ ClassAd *CondorJob::buildSubmitAd()
 
 	submit_ad->Assign( ATTR_JOB_STATUS, HELD );
 	submit_ad->Assign( ATTR_HOLD_REASON, "Spooling input data files" );
+	submit_ad->Assign( ATTR_HOLD_REASON_CODE, CONDOR_HOLD_CODE_SpoolingInput );
 	submit_ad->Assign( ATTR_JOB_UNIVERSE, CONDOR_UNIVERSE_VANILLA );
 
 	submit_ad->Assign( ATTR_Q_DATE, now );
@@ -1441,9 +1464,12 @@ ClassAd *CondorJob::buildSubmitAd()
 	submit_ad->Assign( ATTR_NUM_RESTARTS, 0 );
 	submit_ad->Assign( ATTR_NUM_SYSTEM_HOLDS, 0 );
 	submit_ad->Assign( ATTR_JOB_COMMITTED_TIME, 0 );
+	submit_ad->Assign( ATTR_COMMITTED_SLOT_TIME, 0 );
+	submit_ad->Assign( ATTR_CUMULATIVE_SLOT_TIME, 0 );
 	submit_ad->Assign( ATTR_TOTAL_SUSPENSIONS, 0 );
 	submit_ad->Assign( ATTR_LAST_SUSPENSION_TIME, 0 );
 	submit_ad->Assign( ATTR_CUMULATIVE_SUSPENSION_TIME, 0 );
+	submit_ad->Assign( ATTR_COMMITTED_SUSPENSION_TIME, 0 );
 	submit_ad->Assign( ATTR_ON_EXIT_BY_SIGNAL, false );
 	submit_ad->Assign( ATTR_ENTERED_CURRENT_STATUS, now  );
 	submit_ad->Assign( ATTR_JOB_NOTIFICATION, NOTIFY_NEVER );
@@ -1451,54 +1477,53 @@ ClassAd *CondorJob::buildSubmitAd()
 		// If stdout or stderr is not in the job's Iwd, rename them and
 		// add a transfer remap. Otherwise, the file transfer object will
 		// place them in the Iwd when we stage back the job's output files.
-	MyString output_remaps = "";
-	MyString filename = "";
+	std::string output_remaps = "";
+	std::string filename = "";
 	submit_ad->LookupString( ATTR_TRANSFER_OUTPUT_REMAPS, output_remaps );
 
 	jobAd->LookupString( ATTR_JOB_OUTPUT, filename );
-	if ( strcmp( filename.Value(), condor_basename( filename.Value() ) ) &&
-		 !nullFile( filename.Value() ) ) {
+	if ( strcmp( filename.c_str(), condor_basename( filename.c_str() ) ) &&
+		 !nullFile( filename.c_str() ) ) {
 
 		char const *working_name = "_condor_stdout";
-		if ( !output_remaps.IsEmpty() ) output_remaps += ";";
-		output_remaps.sprintf_cat( "%s=%s", working_name, filename.Value() );
+		if ( !output_remaps.empty() ) output_remaps += ";";
+		sprintf_cat( output_remaps, "%s=%s", working_name, filename.c_str() );
 		submit_ad->Assign( ATTR_JOB_OUTPUT, working_name );
 	}
 
 	jobAd->LookupString( ATTR_JOB_ERROR, filename );
-	if ( strcmp( filename.Value(), condor_basename( filename.Value() ) ) &&
-		 !nullFile( filename.Value() ) ) {
+	if ( strcmp( filename.c_str(), condor_basename( filename.c_str() ) ) &&
+		 !nullFile( filename.c_str() ) ) {
 
 		char const *working_name = "_condor_stderr";
-		if ( !output_remaps.IsEmpty() ) output_remaps += ";";
-		output_remaps.sprintf_cat( "%s=%s", working_name, filename.Value() );
+		if ( !output_remaps.empty() ) output_remaps += ";";
+		sprintf_cat( output_remaps, "%s=%s", working_name, filename.c_str() );
 		submit_ad->Assign( ATTR_JOB_ERROR, working_name );
 	}
 
-	if ( !output_remaps.IsEmpty() ) {
+	if ( !output_remaps.empty() ) {
 		submit_ad->Assign( ATTR_TRANSFER_OUTPUT_REMAPS,
-						   output_remaps.Value() );
+						   output_remaps.c_str() );
 	}
 
-	expr.sprintf( "%s = %s == %d", ATTR_JOB_LEAVE_IN_QUEUE, ATTR_JOB_STATUS,
+	sprintf( expr, "%s = %s == %d", ATTR_JOB_LEAVE_IN_QUEUE, ATTR_JOB_STATUS,
 				  COMPLETED );
 
 	if ( jobAd->LookupInteger( ATTR_JOB_LEASE_EXPIRATION, tmp_int ) ) {
 		submit_ad->Assign( ATTR_TIMER_REMOVE_CHECK, tmp_int );
-		expr.sprintf_cat( " && ( %s < %s )", ATTR_CURRENT_TIME,
-						  ATTR_TIMER_REMOVE_CHECK );
+		sprintf_cat( expr, " && ( time() < %s )", ATTR_TIMER_REMOVE_CHECK );
 	}
 
-	submit_ad->Insert( expr.Value() );
+	submit_ad->Insert( expr.c_str() );
 
-	expr.sprintf( "%s = Undefined", ATTR_OWNER );
-	submit_ad->Insert( expr.Value() );
+	sprintf( expr, "%s = Undefined", ATTR_OWNER );
+	submit_ad->Insert( expr.c_str() );
 
 	const int STAGE_IN_TIME_LIMIT  = 60 * 60 * 8; // 8 hours in seconds.
-	expr.sprintf( "%s = (%s > 0) =!= True && CurrentTime > %s + %d",
+	sprintf( expr, "%s = (%s > 0) =!= True && time() > %s + %d",
 				  ATTR_PERIODIC_REMOVE_CHECK, ATTR_STAGE_IN_FINISH,
 				  ATTR_Q_DATE, STAGE_IN_TIME_LIMIT );
-	submit_ad->Insert( expr.Value() );
+	submit_ad->Insert( expr.c_str() );
 
 	submit_ad->Assign( ATTR_SUBMITTER_ID, submitterId );
 
@@ -1509,6 +1534,10 @@ ClassAd *CondorJob::buildSubmitAd()
 		submit_ad->Assign( ATTR_X509_USER_PROXY, jobProxy->proxy_filename );
 		submit_ad->Assign( ATTR_X509_USER_PROXY_SUBJECT,
 						   jobProxy->subject->subject_name );
+		if ( jobProxy->subject->has_voms_attrs ) {
+			submit_ad->Assign( ATTR_X509_USER_PROXY_FQAN,
+							   jobProxy->subject->fqan );
+		}
 	}
 
 	bool cleared_environment = false;
@@ -1528,14 +1557,11 @@ ClassAd *CondorJob::buildSubmitAd()
 	}
 
 	jobAd->ResetExpr();
-	while ( (next_expr = jobAd->NextExpr()) != NULL ) {
-		if ( strncasecmp( ((Variable*)next_expr->LArg())->Name(),
-						  "REMOTE_", 7 ) == 0 &&
-			 strlen( ((Variable*)next_expr->LArg())->Name() ) > 7 ) {
+	while ( jobAd->NextExpr(next_name, next_expr) ) {
+		if ( strncasecmp( next_name, "REMOTE_", 7 ) == 0 &&
+			 strlen( next_name ) > 7 ) {
 
-			char *attr_value;
-			char const *attr_name = &((Variable*)next_expr->LArg())->Name()[7];
-			MyString buf;
+			char const *attr_name = &(next_name[7]);
 
 			if(strcasecmp(attr_name,ATTR_JOB_ENVIRONMENT1) == 0 ||
 			   strcasecmp(attr_name,ATTR_JOB_ENVIRONMENT1_DELIM) == 0 ||
@@ -1567,14 +1593,22 @@ ClassAd *CondorJob::buildSubmitAd()
 				}
 			}
 
-			next_expr->RArg()->PrintToNewStr(&attr_value);
-			buf.sprintf( "%s = %s", attr_name,
-						 attr_value );
-			submit_ad->Insert( buf.Value() );
-			free(attr_value);
+			submit_ad->Insert( attr_name, next_expr->Copy() );
 		}
 	}
 
+	// If the remote job will use a shadow, we always want to set
+	// JobLeaseDuration. Otherwise, starter-shadow reconnect is disabled.
+	// We need to be careful to respect the user's setting of the
+	// attribute, hence why we do this check last.
+	tmp_int = CONDOR_UNIVERSE_VANILLA;
+	submit_ad->LookupInteger( ATTR_JOB_UNIVERSE, tmp_int );
+	if ( universeCanReconnect( tmp_int ) &&
+		 submit_ad->Lookup( ATTR_JOB_LEASE_DURATION ) == NULL ) {
+
+		submit_ad->Assign( ATTR_JOB_LEASE_DURATION, 20 * 60 );
+	}
+
 		// worry about ATTR_JOB_[OUTPUT|ERROR]_ORIG
 
 	return submit_ad;
@@ -1582,7 +1616,6 @@ ClassAd *CondorJob::buildSubmitAd()
 
 ClassAd *CondorJob::buildStageInAd()
 {
-	MyString expr;
 	ClassAd *stage_in_ad;
 
 		// Base the stage in ad on our own job ad
