@@ -42,35 +42,34 @@ const char *CondorResource::HashName( const char *resource_name,
 									  const char *pool_name,
 									  const char *proxy_subject )
 {
-	static MyString hash_name;
+	static std::string hash_name;
 
-	hash_name.sprintf( "%s#%s#%s", resource_name, 
+	sprintf( hash_name, "condor %s %s#%s", resource_name, 
 					   pool_name ? pool_name : "NULL",
 					   proxy_subject ? proxy_subject : "NULL" );
 
-	return hash_name.Value();
+	return hash_name.c_str();
 }
 
 CondorResource *CondorResource::FindOrCreateResource( const char * resource_name,
 													  const char *pool_name,
-													  const char *proxy_subject )
+													  const Proxy *proxy )
 {
 	int rc;
-	MyString resource_key;
 	CondorResource *resource = NULL;
 
 	rc = ResourcesByName.lookup( HashKey( HashName( resource_name,
 													pool_name,
-													proxy_subject ) ),
+													proxy ? proxy->subject->fqan : NULL ) ),
 								 resource );
 	if ( rc != 0 ) {
 		resource = new CondorResource( resource_name, pool_name,
-									   proxy_subject );
+									   proxy );
 		ASSERT(resource);
 		resource->Reconfig();
 		ResourcesByName.insert( HashKey( HashName( resource_name,
 												   pool_name,
-												   proxy_subject ) ),
+												   proxy ? proxy->subject->fqan : NULL ) ),
 								resource );
 	} else {
 		ASSERT(resource);
@@ -80,15 +79,17 @@ CondorResource *CondorResource::FindOrCreateResource( const char * resource_name
 }
 
 CondorResource::CondorResource( const char *resource_name, const char *pool_name,
-								const char *proxy_subject )
+								const Proxy *proxy )
 	: BaseResource( resource_name )
 {
 	hasLeases = true;
 
-	if ( proxy_subject != NULL ) {
-		proxySubject = strdup( proxy_subject );
+	if ( proxy != NULL ) {
+		proxySubject = strdup( proxy->subject->subject_name );
+		proxyFQAN = strdup( proxy->subject->fqan );
 	} else {
-		proxySubject = 0;
+		proxySubject = NULL;
+		proxyFQAN = NULL;
 	}
 	scheddPollTid = TIMER_UNSET;
 	scheddName = strdup( resource_name );
@@ -113,10 +114,10 @@ CondorResource::CondorResource( const char *resource_name, const char *pool_name
 	} else {
 		// TODO remove scheddName from the gahp server key if/when
 		//   a gahp server can handle multiple schedds
-		MyString buff;
+		std::string buff;
 		ArgList args;
-		buff.sprintf( "CONDOR/%s/%s/%s", poolName ? poolName : "NULL",
-					  scheddName, proxySubject ? proxySubject : "NULL" );
+		sprintf( buff, "CONDOR/%s/%s/%s", poolName ? poolName : "NULL",
+					  scheddName, proxyFQAN ? proxyFQAN : "NULL" );
 		args.AppendArg("-f");
 		args.AppendArg("-s");
 		args.AppendArg(scheddName);
@@ -125,17 +126,17 @@ CondorResource::CondorResource( const char *resource_name, const char *pool_name
 			args.AppendArg(poolName);
 		}
 
-		gahp = new GahpClient( buff.Value(), gahp_path, &args );
+		gahp = new GahpClient( buff.c_str(), gahp_path, &args );
 		gahp->setNotificationTimerId( scheddPollTid );
 		gahp->setMode( GahpClient::normal );
 		gahp->setTimeout( CondorJob::gahpCallTimeout );
 
-		ping_gahp = new GahpClient( buff.Value(), gahp_path, &args );
+		ping_gahp = new GahpClient( buff.c_str(), gahp_path, &args );
 		ping_gahp->setNotificationTimerId( pingTimerId );
 		ping_gahp->setMode( GahpClient::normal );
 		ping_gahp->setTimeout( CondorJob::gahpCallTimeout );
 
-		lease_gahp = new GahpClient( buff.Value(), gahp_path, &args );
+		lease_gahp = new GahpClient( buff.c_str(), gahp_path, &args );
 		lease_gahp->setNotificationTimerId( updateLeasesTimerId );
 		lease_gahp->setMode( GahpClient::normal );
 		lease_gahp->setTimeout( CondorJob::gahpCallTimeout );
@@ -148,7 +149,7 @@ CondorResource::~CondorResource()
 {
 	ResourcesByName.remove( HashKey( HashName( resourceName,
 											   poolName,
-											   proxySubject ) ) );
+											   proxyFQAN ) ) );
 
 		// Make sure we don't leak a ScheddPollInfo. If there are other
 		// CondorResources that still want to use it, they'll recreate it.
@@ -168,6 +169,7 @@ CondorResource::~CondorResource()
 	if ( proxySubject != NULL ) {
 		free( proxySubject );
 	}
+	free( proxyFQAN );
 	if ( scheddPollTid != TIMER_UNSET ) {
 		daemonCore->Cancel_Timer( scheddPollTid );
 	}
@@ -198,18 +200,39 @@ const char *CondorResource::ResourceType()
 	return "condor";
 }
 
+const char *CondorResource::GetHashName()
+{
+	return HashName( resourceName, poolName, proxyFQAN );
+}
+
+void CondorResource::PublishResourceAd( ClassAd *resource_ad )
+{
+	BaseResource::PublishResourceAd( resource_ad );
+
+	std::string buff;
+
+	sprintf( buff, "condor %s %s", resourceName, poolName );
+	resource_ad->Assign( ATTR_NAME, buff.c_str() );
+	if ( proxySubject ) {
+		resource_ad->Assign( ATTR_X509_USER_PROXY_SUBJECT, proxySubject );
+	}
+	if ( proxyFQAN ) {
+		resource_ad->Assign( ATTR_X509_USER_PROXY_FQAN, proxyFQAN );
+	}
+}
+
 void CondorResource::RegisterJob( CondorJob *job, const char *submitter_id )
 {
 	BaseResource::RegisterJob( job );
 
 	if ( submitter_ids.contains( submitter_id ) == false ) {
 		submitter_ids.append( submitter_id );
-		if ( submitter_constraint.Length() == 0 ) {
-			submitter_constraint.sprintf( "(%s=?=\"%s\")",
+		if ( submitter_constraint.empty() ) {
+			sprintf( submitter_constraint, "(%s=?=\"%s\")",
 										  ATTR_SUBMITTER_ID,
 										  submitter_id );
 		} else {
-			submitter_constraint.sprintf_cat( "||(%s=?=\"%s\")",
+			sprintf_cat( submitter_constraint, "||(%s=?=\"%s\")",
 											  ATTR_SUBMITTER_ID,
 											  submitter_id );
 		}
@@ -229,7 +252,7 @@ void CondorResource::UnregisterJob( CondorJob *job )
 	BaseResource::UnregisterJob( job );
 }
 
-int CondorResource::DoScheddPoll()
+void CondorResource::DoScheddPoll()
 {
 	int rc;
 	ScheddPollInfo *poll_info = NULL;
@@ -239,17 +262,14 @@ int CondorResource::DoScheddPoll()
 			// No jobs or we can't talk to the schedd, so no point
 			// in polling
 		daemonCore->Reset_Timer( scheddPollTid, scheddPollInterval );
-		return 0;
+		return;
 	}
 
 	if ( gahp->isStarted() == false ) {
-		if ( gahp->Startup() == false ) {
-				// Failed to start the gahp server. Don't do anything
-				// about it. The job objects will also fail on this call
-				// and should go on hold as a result.
-			daemonCore->Reset_Timer( scheddPollTid, scheddPollInterval );
-			return 0;
-		}
+		// The gahp isn't started yet. Wait a few seconds for a CondorJob
+		// object to start it (and possibly initialize x509 credentials).
+		daemonCore->Reset_Timer( scheddPollTid, 5 );
+		return;
 	}
 
 	PollInfoByName.lookup( HashKey( HashName( scheddName, poolName, NULL ) ),
@@ -274,13 +294,13 @@ int CondorResource::DoScheddPoll()
 		if ( poll_info->m_pollActive == true ||
 			 poll_info->m_lastPoll + scheddPollInterval > time(NULL) ) {
 			daemonCore->Reset_Timer( scheddPollTid, scheddPollInterval );
-			return 0;
+			return;
 		}
 
 			// start schedd status command
 		dprintf( D_FULLDEBUG, "Starting collective poll: %s\n",
 				 scheddName );
-		MyString constraint;
+		std::string constraint;
 
 			// create a list of jobs we expect to hear about in our
 			// status command
@@ -293,7 +313,7 @@ int CondorResource::DoScheddPoll()
 		}
 		CondorResource *next_resource;
 		BaseJob *job;
-		MyString job_id;
+		std::string job_id;
 		ResourcesByName.startIterations();
 		while ( ResourcesByName.iterate( next_resource ) != 0 ) {
 			if ( strcmp( scheddName, next_resource->scheddName ) ||
@@ -310,10 +330,10 @@ int CondorResource::DoScheddPoll()
 			}
 		}
 
-		constraint.sprintf( "(%s)", submitter_constraint.Value() );
+		sprintf( constraint, "(%s)", submitter_constraint.c_str() );
 
 		rc = gahp->condor_job_status_constrained( scheddName,
-												  constraint.Value(),
+												  constraint.c_str(),
 												  NULL, NULL );
 
 		if ( rc != GAHPCLIENT_COMMAND_PENDING ) {
@@ -338,7 +358,7 @@ int CondorResource::DoScheddPoll()
 												  &status_ads );
 
 		if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
-			return 0;
+			return;
 		} else if ( rc != 0 ) {
 			dprintf( D_ALWAYS,
 					 "gahp->condor_job_status_constrained returned %d for remote schedd %s\n",
@@ -351,7 +371,7 @@ int CondorResource::DoScheddPoll()
 			for ( int i = 0; i < num_status_ads; i++ ) {
 				int cluster, proc;
 				int rc2;
-				MyString job_id_string;
+				std::string job_id_string;
 				CondorJob *job;
 
 				if( status_ads[i] == NULL ) {
@@ -362,10 +382,10 @@ int CondorResource::DoScheddPoll()
 				status_ads[i]->LookupInteger( ATTR_CLUSTER_ID, cluster );
 				status_ads[i]->LookupInteger( ATTR_PROC_ID, proc );
 
-				job_id_string.sprintf( "condor %s %s %d.%d", scheddName,
+				sprintf( job_id_string, "condor %s %s %d.%d", scheddName,
 									   poolName, cluster, proc );
 
-				rc2 = BaseJob::JobsByRemoteId.lookup( HashKey( job_id_string.Value() ),
+				rc2 = BaseJob::JobsByRemoteId.lookup( HashKey( job_id_string.c_str() ),
 													  (BaseJob*&)job );
 				if ( rc2 == 0 ) {
 					job->NotifyNewRemoteStatus( status_ads[i] );
@@ -386,7 +406,7 @@ int CondorResource::DoScheddPoll()
 			// Check if any jobs were missing from the status result
 		if ( rc == 0 ) {
 			CondorJob *job;
-			MyString job_id;
+			std::string job_id;
 			poll_info->m_submittedJobs.Rewind();
 			while ( ( job = poll_info->m_submittedJobs.Next() ) ) {
 				if ( job->jobAd->LookupString( ATTR_GRID_JOB_ID, job_id ) ) {
@@ -405,8 +425,6 @@ int CondorResource::DoScheddPoll()
 
 		daemonCore->Reset_Timer( scheddPollTid, scheddPollInterval );
 	}
-
-	return 0;
 }
 
 void CondorResource::DoPing( time_t& ping_delay, bool& ping_complete,
@@ -491,12 +509,12 @@ dprintf( D_FULLDEBUG, "*** Lease udpate succeeded!\n" );
 		update_complete = true;
 
 		PROC_ID curr_id;
-		MyString id_str;
+		std::string id_str;
 		updated.Rewind();
 		while ( updated.Next( curr_id ) ) {
-			id_str.sprintf( "condor %s %s %d.%d", scheddName, poolName,
+			sprintf( id_str, "condor %s %s %d.%d", scheddName, poolName,
 							curr_id.cluster, curr_id.proc );
-			if ( BaseJob::JobsByRemoteId.lookup( HashKey( id_str.Value() ),
+			if ( BaseJob::JobsByRemoteId.lookup( HashKey( id_str.c_str() ),
 												 curr_job ) == 0 ) {
 				update_succeeded.Append( curr_job->procID );
 			}
