@@ -27,7 +27,15 @@
 #include "creamjob.h"
 #include "gridmanager.h"
 
-#define DEFAULT_MAX_PENDING_SUBMITS_PER_RESOURCE	5
+#ifdef WIN32
+	#include <sys/types.h> 
+	#include <sys/timeb.h>
+#endif
+
+// Enable more expensive debugging and testing code
+#define DEBUG_CREAM 1
+
+
 #define DEFAULT_MAX_SUBMITTED_JOBS_PER_RESOURCE		100
 
 
@@ -64,11 +72,11 @@ struct CreamProxyDelegation {
 	time_t proxy_expire;
 	time_t last_proxy_refresh;
 	Proxy *proxy;
-	MyString error_message;
+	std::string error_message;
 };
 
 CreamResource *CreamResource::FindOrCreateResource( const char *resource_name,
-													const char *proxy_subject )
+													const Proxy *proxy )
 {
 	int rc;
 	CreamResource *resource = NULL;
@@ -76,12 +84,12 @@ CreamResource *CreamResource::FindOrCreateResource( const char *resource_name,
 	const char *canonical_name = CanonicalName( resource_name );
 	ASSERT(canonical_name);
 
-	const char *hash_name = HashName( canonical_name, proxy_subject );
+	const char *hash_name = HashName( canonical_name, proxy->subject->fqan );
 	ASSERT(hash_name);
 
 	rc = ResourcesByName.lookup( HashKey( hash_name ), resource );
 	if ( rc != 0 ) {
-		resource = new CreamResource( canonical_name, proxy_subject );
+		resource = new CreamResource( canonical_name, proxy );
 		ASSERT(resource);
 		if ( resource->Init() == false ) {
 			delete resource;
@@ -97,11 +105,12 @@ CreamResource *CreamResource::FindOrCreateResource( const char *resource_name,
 }
 
 CreamResource::CreamResource( const char *resource_name,
-							  const char *proxy_subject )
+							  const Proxy *proxy )
 	: BaseResource( resource_name )
 {
 	initialized = false;
-	proxySubject = strdup( proxy_subject );
+	proxySubject = strdup( proxy->subject->subject_name );
+	proxyFQAN = strdup( proxy->subject->fqan );
 	delegationTimerId = daemonCore->Register_Timer( 0,
 							(TimerHandlercpp)&CreamResource::checkDelegation,
 							"CreamResource::checkDelegation", (Service*)this );
@@ -109,10 +118,14 @@ CreamResource::CreamResource( const char *resource_name,
 	delegationServiceUri = NULL;
 	gahp = NULL;
 	deleg_gahp = NULL;
-	submitLimit = DEFAULT_MAX_PENDING_SUBMITS_PER_RESOURCE;
-	jobLimit = DEFAULT_MAX_SUBMITTED_JOBS_PER_RESOURCE;
+	status_gahp = NULL;
+	m_leaseGahp = NULL;
 
-	const char delegservice_name[] = "/ce-cream/services/CREAMDelegation";
+	hasLeases = true;
+	m_hasSharedLeases = true;
+	m_defaultLeaseDuration = 6 * 60 * 60;
+
+	const char delegservice_name[] = "/ce-cream/services/gridsite-delegation";
 	const char *name_ptr;
 	int name_len;
 	if ( strncmp( "https://", resource_name, 8 ) == 0 ) {
@@ -147,11 +160,11 @@ dprintf(D_FULLDEBUG,"*** ~CreamResource\n");
 	CreamProxyDelegation *next_deleg;
 	delegatedProxies.Rewind();
 	while ( (next_deleg = delegatedProxies.Next()) != NULL ) {
-dprintf(D_FULLDEBUG,"    deleting %s\n",next_deleg->deleg_uri);
+dprintf(D_FULLDEBUG,"    deleting %s\n",next_deleg->deleg_uri?next_deleg->deleg_uri:"(undelegated)");
 		delegatedProxies.DeleteCurrent();
 		free( next_deleg->deleg_uri );
-			// unacquire proxy?
-		ReleaseProxy( next_deleg->proxy );
+		ReleaseProxy( next_deleg->proxy,
+					  (TimerHandlercpp)&CreamResource::ProxyCallback, this );
 		delete next_deleg;
 	}
 	if ( delegationServiceUri != NULL ) {
@@ -171,9 +184,12 @@ dprintf(D_FULLDEBUG,"    deleting %s\n",next_deleg->deleg_uri);
 	if ( deleg_gahp != NULL ) {
 		delete deleg_gahp;
 	}
+	delete status_gahp;
+	delete m_leaseGahp;
 	if ( proxySubject ) {
 		free( proxySubject );
 	}
+	free( proxyFQAN );
 }
 
 bool CreamResource::Init()
@@ -184,21 +200,48 @@ bool CreamResource::Init()
 
 		// TODO This assumes that at least one CreamJob has already
 		// initialized the gahp server. Need a better solution.
-	MyString gahp_name;
-	gahp_name.sprintf( "CREAM/%s", proxySubject );
+	std::string gahp_name;
+	sprintf( gahp_name, "CREAM/%s", proxyFQAN );
 
-	gahp = new GahpClient( gahp_name.Value() );
+	gahp = new GahpClient( gahp_name.c_str() );
 
 	gahp->setNotificationTimerId( pingTimerId );
 	gahp->setMode( GahpClient::normal );
 	gahp->setTimeout( gahpCallTimeout );
 
-	deleg_gahp = new GahpClient( gahp_name.Value() );
+	deleg_gahp = new GahpClient( gahp_name.c_str() );
 
 	deleg_gahp->setNotificationTimerId( delegationTimerId );
 	deleg_gahp->setMode( GahpClient::normal );
 	deleg_gahp->setTimeout( gahpCallTimeout );
 
+	status_gahp = new GahpClient( gahp_name.c_str() );
+
+	StartBatchStatusTimer();
+
+	status_gahp->setNotificationTimerId( BatchPollTid() );
+	status_gahp->setMode( GahpClient::normal );
+	status_gahp->setTimeout( gahpCallTimeout );
+
+	m_leaseGahp = new GahpClient( gahp_name.c_str() );
+
+	m_leaseGahp->setNotificationTimerId( updateLeasesTimerId );
+	m_leaseGahp->setMode( GahpClient::normal );
+	m_leaseGahp->setTimeout( gahpCallTimeout );
+
+	char* pool_name = param( "COLLECTOR_HOST" );
+	if ( pool_name ) {
+		StringList collectors( pool_name );
+		free( pool_name );
+		pool_name = collectors.print_to_string();
+	} else {
+		pool_name = strdup( "NoPool" );
+	}
+
+	sprintf( m_leaseId, "Condor#%s#%s#%s", myUserName, ScheddName, pool_name );
+
+	free( pool_name );
+
 	initialized = true;
 
 	Reconfig();
@@ -211,6 +254,9 @@ void CreamResource::Reconfig()
 	BaseResource::Reconfig();
 
 	gahp->setTimeout( gahpCallTimeout );
+	deleg_gahp->setTimeout( gahpCallTimeout );
+	status_gahp->setTimeout( gahpCallTimeout );
+	m_leaseGahp->setTimeout( gahpCallTimeout );
 }
 
 const char *CreamResource::ResourceType()
@@ -221,18 +267,18 @@ const char *CreamResource::ResourceType()
 const char *CreamResource::CanonicalName( const char *name )
 {
 /*
-	static MyString canonical;
+	static std::string canonical;
 	char *host;
 	char *port;
 
 	parse_resource_manager_string( name, &host, &port, NULL, NULL );
 
-	canonical.sprintf( "%s:%s", host, *port ? port : "2119" );
+	sprintf( canonical, "%s:%s", host, *port ? port : "2119" );
 
 	free( host );
 	free( port );
 
-	return canonical.Value();
+	return canonical.c_str();
 */
 	return name;
 }
@@ -240,11 +286,33 @@ const char *CreamResource::CanonicalName( const char *name )
 const char *CreamResource::HashName( const char *resource_name,
 									 const char *proxy_subject )
 {
-	static MyString hash_name;
+	static std::string hash_name;
 
-	hash_name.sprintf( "%s#%s", resource_name, proxy_subject );
+	sprintf( hash_name, "cream %s#%s", resource_name, proxy_subject );
 
-	return hash_name.Value();
+	return hash_name.c_str();
+}
+
+void CreamResource::RegisterJob( CreamJob *job )
+{
+	int job_lease;
+	if ( m_sharedLeaseExpiration == 0 ) {
+		if ( job->jobAd->LookupInteger( ATTR_JOB_LEASE_EXPIRATION, job_lease ) ) {
+			m_sharedLeaseExpiration = job_lease;
+		}
+	} else {
+		if ( job->jobAd->LookupInteger( ATTR_JOB_LEASE_EXPIRATION, job_lease ) ) {
+			job->UpdateJobLeaseSent( m_sharedLeaseExpiration );
+		}
+	}
+
+	// TODO should we also reset the timer if this job has a shorter
+	//   lease duration than all existing jobs?
+	if ( m_sharedLeaseExpiration == 0 ) {
+		daemonCore->Reset_Timer( updateLeasesTimerId, 0 );
+	}
+
+	BaseResource::RegisterJob( job );
 }
 
 void CreamResource::UnregisterJob( CreamJob *job )
@@ -254,7 +322,8 @@ void CreamResource::UnregisterJob( CreamJob *job )
 		CreamJob *next_job;
 		registeredJobs.Rewind();
 		while ( (next_job = (CreamJob*)registeredJobs.Next()) != NULL ) {
-			if ( strcmp( job->delegatedCredentialURI,
+			if ( next_job->delegatedCredentialURI != NULL &&
+				 strcmp( job->delegatedCredentialURI,
 						 next_job->delegatedCredentialURI ) == 0 ) {
 				delete_deleg = false;
 				break;
@@ -262,12 +331,11 @@ void CreamResource::UnregisterJob( CreamJob *job )
 		}
 		if ( delete_deleg ) {
 dprintf(D_FULLDEBUG,"*** deleting delegation %s\n",job->delegatedCredentialURI);
-			bool reacquire_proxy = false;
-			Proxy *proxy_to_reacquire = job->jobProxy;
 			CreamProxyDelegation *next_deleg;
 			delegatedProxies.Rewind();
 			while ( (next_deleg = delegatedProxies.Next()) != NULL ) {
-				if ( strcmp( job->delegatedCredentialURI,
+				if ( next_deleg->deleg_uri != NULL &&
+					 strcmp( job->delegatedCredentialURI,
 							 next_deleg->deleg_uri ) == 0 ) {
 					delegatedProxies.DeleteCurrent();
 					if ( activeDelegationCmd == next_deleg ) {
@@ -275,21 +343,29 @@ dprintf(D_FULLDEBUG,"*** deleting delegation %s\n",job->delegatedCredentialURI);
 						activeDelegationCmd = NULL;
 					}
 					free( next_deleg->deleg_uri );
-						// unacquire proxy?
-					ReleaseProxy( next_deleg->proxy );
+					ReleaseProxy( next_deleg->proxy,
+								  (TimerHandlercpp)&CreamResource::ProxyCallback,
+								  this );
 					delete next_deleg;
-				} else if ( next_deleg->proxy == proxy_to_reacquire ) {
-					reacquire_proxy = true;
 				}
 			}
-			if ( reacquire_proxy ) {
-				AcquireProxy( proxy_to_reacquire, delegationTimerId );
-			}
 		}
 	}
 
 	BaseResource::UnregisterJob( job );
-		// This object may be deleted now. Don't do anything below here!
+}
+
+const char *CreamResource::GetHashName()
+{
+	return HashName( resourceName, proxyFQAN );
+}
+
+void CreamResource::PublishResourceAd( ClassAd *resource_ad )
+{
+	BaseResource::PublishResourceAd( resource_ad );
+
+	resource_ad->Assign( ATTR_X509_USER_PROXY_SUBJECT, proxySubject );
+	resource_ad->Assign( ATTR_X509_USER_PROXY_FQAN, proxyFQAN );
 }
 
 void CreamResource::registerDelegationURI( const char *deleg_uri,
@@ -301,7 +377,8 @@ dprintf(D_FULLDEBUG,"*** registerDelegationURI(%s,%s)\n",deleg_uri,job_proxy->pr
 	delegatedProxies.Rewind();
 
 	while ( ( next_deleg = delegatedProxies.Next() ) != NULL ) {
-		if ( strcmp( deleg_uri, next_deleg->deleg_uri ) == 0 ) {
+		if ( next_deleg->deleg_uri != NULL &&
+			 strcmp( deleg_uri, next_deleg->deleg_uri ) == 0 ) {
 dprintf(D_FULLDEBUG,"    found CreamProxyDelegation\n");
 			return;
 		}
@@ -315,8 +392,7 @@ dprintf(D_FULLDEBUG,"    creating new CreamProxyDelegation\n");
 	next_deleg->last_lifetime_extend = 0;
 	next_deleg->last_proxy_refresh = 0;
 	next_deleg->proxy = job_proxy;
-		// acquire proxy?
-	AcquireProxy( job_proxy, delegationTimerId );
+	AcquireProxy( job_proxy, (TimerHandlercpp)&CreamResource::ProxyCallback, this );
 	delegatedProxies.Append( next_deleg );
 
 		// TODO add smarter timer that delays a few seconds
@@ -347,8 +423,7 @@ dprintf(D_FULLDEBUG,"    creating new CreamProxyDelegation\n");
 	next_deleg->last_lifetime_extend = 0;
 	next_deleg->last_proxy_refresh = 0;
 	next_deleg->proxy = job_proxy;
-		// acquire proxy?
-	AcquireProxy( job_proxy, delegationTimerId );
+	AcquireProxy( job_proxy, (TimerHandlercpp)&CreamResource::ProxyCallback, this );
 	delegatedProxies.Append( next_deleg );
 
 		// TODO add smarter timer that delays a few seconds
@@ -365,10 +440,10 @@ const char *CreamResource::getDelegationError( Proxy *job_proxy )
 
 	while ( ( next_deleg = delegatedProxies.Next() ) != NULL ) {
 		if ( next_deleg->proxy == job_proxy ) {
-			if ( next_deleg->error_message.IsEmpty() ) {
+			if ( next_deleg->error_message.empty() ) {
 				return NULL;
 			} else {
-				return next_deleg->error_message.Value();
+				return next_deleg->error_message.c_str();
 			}
 		}
 	}
@@ -378,7 +453,13 @@ const char *CreamResource::getDelegationError( Proxy *job_proxy )
 	return NULL;
 }
 
-int CreamResource::checkDelegation()
+int CreamResource::ProxyCallback()
+{
+	daemonCore->Reset_Timer( delegationTimerId, 0 );
+	return 0;
+}
+
+void CreamResource::checkDelegation()
 {
 dprintf(D_FULLDEBUG,"*** checkDelegation()\n");
 	bool signal_jobs;
@@ -388,7 +469,7 @@ dprintf(D_FULLDEBUG,"*** checkDelegation()\n");
 	if ( deleg_gahp->isInitialized() == false ) {
 		dprintf( D_ALWAYS,"gahp server not up yet, delaying checkDelegation\n" );
 		daemonCore->Reset_Timer( delegationTimerId, 5 );
-		return 0;
+		return;
 	}
 
 	daemonCore->Reset_Timer( delegationTimerId, CHECK_DELEGATION_INTERVAL );
@@ -407,32 +488,43 @@ dprintf(D_FULLDEBUG,"    new delegation\n");
 		
 				/* TODO generate better id */
 			if ( delegation_uri == "" ) {
+#ifdef WIN32
+				struct _timeb timebuffer;
+				_ftime( &timebuffer );
+				sprintf( delegation_uri, "%d.%d", timebuffer.time, timebuffer.millitm );
+#else
 				struct timeval tv;
 				gettimeofday( &tv, NULL );
-				delegation_uri.sprintf( "%d.%d", tv.tv_sec, tv.tv_usec );
+				sprintf( delegation_uri, "%d.%d", (int)tv.tv_sec, (int)tv.tv_usec );
+#endif
 			}
 
 			deleg_gahp->setDelegProxy( next_deleg->proxy );
 
 			rc = deleg_gahp->cream_delegate(delegationServiceUri,
-											delegation_uri.Value() );
+											delegation_uri.c_str() );
 			
 			if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
 				activeDelegationCmd = next_deleg;
-				return 0;
+				return;
 			}
 			if ( rc != 0 ) {
 					// Failure, what to do?
 				dprintf( D_ALWAYS, "delegate_credentials(%s) failed!\n",
 						 delegationServiceUri );
 				activeDelegationCmd = NULL;
-				next_deleg->error_message = "Failed to create proxy delegation";
+				const char *err = deleg_gahp->getErrorString();
+				if ( err ) {
+					next_deleg->error_message = err;
+				} else {
+					next_deleg->error_message = "Failed to create proxy delegation";
+				}
 				signal_jobs = true;
 			} else {
-dprintf(D_FULLDEBUG,"      %s\n",delegation_uri.Value());
+dprintf(D_FULLDEBUG,"      %s\n",delegation_uri.c_str());
 				activeDelegationCmd = NULL;
 					// we are assuming responsibility to free this
-				next_deleg->deleg_uri = strdup( delegation_uri.Value() );
+				next_deleg->deleg_uri = strdup( delegation_uri.c_str() );
 				next_deleg->proxy_expire = next_deleg->proxy->expiration_time;
 				next_deleg->lifetime = now + 12*60*60;
 				next_deleg->error_message = "";
@@ -446,41 +538,17 @@ dprintf(D_FULLDEBUG,"      %s\n",delegation_uri.Value());
 			 now >= next_deleg->last_proxy_refresh + PROXY_REFRESH_INTERVAL ) {
 dprintf(D_FULLDEBUG,"    refreshing %s\n",next_deleg->deleg_uri);
 			int rc;
-			CreamJob *next_job;
 			deleg_gahp->setDelegProxy( next_deleg->proxy );
 			
-				//create a list of jobIds using the proxy
-			StringList job_ids;
-			registeredJobs.Rewind();
-			while ( (next_job = (CreamJob *)registeredJobs.Next()) != NULL) {
-				if(next_job->remoteJobId != NULL) {
-						//Make sure deleg URI matches w/ job's before adding to list
-						//And make sure the job has been started and is not in terminal state.
-					if ( strcmp( next_job->delegatedCredentialURI, next_deleg->deleg_uri ) == 0 &&
-						 next_job->remoteState != CREAM_JOB_STATE_REGISTERED &&
-						 next_job->remoteState != CREAM_JOB_STATE_CANCELLED &&
-						 next_job->remoteState != CREAM_JOB_STATE_DONE_OK &&
-						 next_job->remoteState != CREAM_JOB_STATE_DONE_FAILED &&
-						 next_job->remoteState != CREAM_JOB_STATE_ABORTED) {
-						job_ids.append(next_job->remoteJobId);
-					}
-				}
-			}
-			
-			rc = deleg_gahp->cream_proxy_renew(serviceUri,
-											   delegationServiceUri,
-											   next_deleg->deleg_uri,
-											   job_ids);
+			rc = deleg_gahp->cream_proxy_renew(delegationServiceUri,
+											   next_deleg->deleg_uri);
 			
 			if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
 				activeDelegationCmd = next_deleg;
-				return 0;
+				return;
 			}
 			next_deleg->last_proxy_refresh = now;
 			if ( rc != 0 ) {
-					/* CREAM will return an error if a single job in the list fails to be
-					   renewed. However, the rest of the jobs will be renewed successfully.
-					*/
 					// Failure, what to do?
 				dprintf( D_ALWAYS, "refresh_credentials(%s) failed!\n",
 						 next_deleg->deleg_uri );
@@ -534,15 +602,14 @@ dprintf(D_FULLDEBUG,"    signalling jobs for %s\n",next_deleg->deleg_uri?next_de
 					 next_deleg->deleg_uri != NULL ) {
 					if ( strcmp( next_job->delegatedCredentialURI,
 								 next_deleg->deleg_uri ) == 0 ) {
-						next_job->SetEvaluateState();
+						next_job->ProxyCallback();
 					}
 				} else if ( next_job->jobProxy == next_deleg->proxy ) {
-					next_job->SetEvaluateState();
+					next_job->ProxyCallback();
 				}
 			}
 		}
 	}
-	return 0;
 }
 
 void CreamResource::DoPing( time_t& ping_delay, bool& ping_complete,
@@ -576,3 +643,149 @@ void CreamResource::DoPing( time_t& ping_delay, bool& ping_complete,
 		ping_succeeded = true;
 	}
 }
+
+
+
+int CreamResource::BatchStatusInterval() const
+{
+	return CreamJob::probeInterval;
+}
+
+CreamResource::BatchStatusResult CreamResource::StartBatchStatus()
+{
+	ASSERT(status_gahp);
+
+	GahpClient::CreamJobStatusMap results;
+	int rc = status_gahp->cream_job_status_all(ResourceName(), results);
+	if(rc == GAHPCLIENT_COMMAND_PENDING) { 
+		return BSR_PENDING;
+	}
+	if(rc != 0) {
+		dprintf(D_ALWAYS, "Error attempting a Cream batch status query: %d\n", rc);
+		return BSR_ERROR;
+	}
+	for(GahpClient::CreamJobStatusMap::const_iterator it = results.begin();
+		it != results.end(); it++) {
+
+		const GahpClient::CreamJobStatus & status = it->second;
+
+		std::string full_job_id = CreamJob::getFullJobId(ResourceName(), status.job_id.c_str());
+		BaseJob * bjob;
+		int rc2 = BaseJob::JobsByRemoteId.lookup( 
+			HashKey( full_job_id.c_str()), bjob);
+		if(rc2 != 0) {
+			// Job not found. Probably okay; we might see jobs
+			// submitted via other means, or jobs we've abandoned.
+			dprintf(D_FULLDEBUG, "Job %s on remote host is unknown. Skipping.\n", status.job_id.c_str());
+			continue;
+		}
+		CreamJob * job = dynamic_cast<CreamJob *>(bjob);
+		ASSERT(job);
+		job->NewCreamState(status.job_status.c_str(), status.exit_code, 
+			status.failure_reason.c_str());
+		dprintf(D_FULLDEBUG, "%d.%d %s new status: %s, %d, %s\n", 
+			job->procID.cluster, job->procID.proc,
+			status.job_id.c_str(),
+			status.job_status.c_str(), 
+			status.exit_code, 
+			status.failure_reason.c_str());
+	}
+
+#if DEBUG_CREAM
+	{
+		CreamJob *job;
+		registeredJobs.Rewind();
+		while ( (job = dynamic_cast<CreamJob*>(registeredJobs.Next())) != NULL ) {
+			if(job->remoteJobId == NULL) { continue; }
+			if(results.find(job->remoteJobId) == results.end()) {
+				dprintf(D_FULLDEBUG, "%d.%d %s NOT updated\n",
+					job->procID.cluster, job->procID.proc,
+					job->remoteJobId);
+			}
+		}
+	}
+#endif
+	return BSR_DONE;
+}
+
+CreamResource::BatchStatusResult CreamResource::FinishBatchStatus()
+{
+	// As it happens, we can use the same code path
+	// for starting and finishing a batch status for
+	// CREAM jobs.  Indeed, doing so is simpler.
+	return StartBatchStatus();
+}
+
+GahpClient * CreamResource::BatchGahp() { return status_gahp; }
+
+const char *CreamResource::getLeaseId()
+{
+	// TODO trigger a DoUpdateLeases() if we don't have a lease set yet
+	if ( m_sharedLeaseExpiration ) {
+		return m_leaseId.c_str();
+	} else {
+		return NULL;
+	}
+}
+
+const char *CreamResource::getLeaseError()
+{
+	// TODO
+	return NULL;
+}
+
+void CreamResource::DoUpdateSharedLease( time_t& update_delay,
+										 bool& update_complete,
+										 bool& update_succeeded )
+{
+	int rc;
+	time_t our_expiration;
+	time_t server_expiration;
+	BaseJob *curr_job;
+
+	// TODO Should we worry about jobs having different lease ids?
+	if ( m_leaseGahp->isStarted() == false ) {
+		dprintf( D_ALWAYS,"gahp server not up yet, delaying lease update\n" );
+		update_delay = 5;
+		return;
+	}
+
+	update_delay = 0;
+
+	our_expiration = m_sharedLeaseExpiration;
+	server_expiration = m_sharedLeaseExpiration;
+
+	rc = m_leaseGahp->cream_set_lease( resourceName, m_leaseId.c_str(),
+									   server_expiration );
+
+	if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
+		update_complete = false;
+	} else if ( rc != 0 ) {
+		dprintf( D_FULLDEBUG, "*** Lease update failed!\n" );
+		const char *err = m_leaseGahp->getErrorString();
+		if ( err ) {
+			m_leaseErrorMsg = err;
+		} else {
+			m_leaseErrorMsg = "Failed to set lease";
+		}
+		update_complete = true;
+		update_succeeded = false;
+	} else {
+		m_leaseErrorMsg = "";
+		update_complete = true;
+		update_succeeded = true;
+
+		m_sharedLeaseExpiration = server_expiration;
+
+		registeredJobs.Rewind();
+		while ( registeredJobs.Next( curr_job ) ) {
+			std::string tmp;
+			if ( !curr_job->jobAd->LookupString( ATTR_GRID_JOB_ID, tmp ) ) {
+				continue;
+			}
+			if ( our_expiration != server_expiration ) {
+				curr_job->UpdateJobLeaseSent( server_expiration );
+			}
+		}
+	}
+}
