@@ -29,6 +29,8 @@
 #include "globus_utils.h"
 #include "proxymanager.h"
 #include "condor_arglist.h"
+#include <map>
+#include <queue>
 
 
 struct GahpProxyInfo
@@ -78,6 +80,8 @@ class GahpServer : public Service {
 	bool Startup();
 	bool Initialize(Proxy * proxy);
 
+	void DeleteMe();
+
 	static const int m_buffer_size;
 	char *m_buffer;
 	int m_buffer_pos;
@@ -85,6 +89,8 @@ class GahpServer : public Service {
 	int buffered_read( int fd, void *buf, int count );
 	int buffered_peek();
 
+	int m_deleteMeTid;
+
 	bool m_in_results;
 
 	static int m_reaperid;
@@ -95,13 +101,14 @@ class GahpServer : public Service {
 	void read_argv(Gahp_Args *g_args) { read_argv(*g_args); }
 	void write_line(const char *command);
 	void write_line(const char *command,int req,const char *args);
-	int pipe_ready();
-	int err_pipe_ready();
+	int pipe_ready(int pipe_end);
+	int err_pipe_ready(int pipe_end);
 
 	void AddGahpClient();
 	void RemoveGahpClient();
 
-	int doProxyCheck();
+	int ProxyCallback();
+	void doProxyCheck();
 	GahpProxyInfo *RegisterProxy( Proxy *proxy );
 	void UnregisterProxy( Proxy *proxy );
 
@@ -130,11 +137,9 @@ class GahpServer : public Service {
 		this method is invoked automatically either by a timer set
 		via setPollInterval or by a Gahp Server async result
 		notification.
-		@return The number of pending commands which have completed
-		since the last poll (note: could easily be zero).	
 		@see setPollInterval
 	*/
-	int poll();
+	void poll();
 
 	void poll_real_soon();
 
@@ -161,13 +166,16 @@ class GahpServer : public Service {
 
 	unsigned int m_reference_count;
 	HashTable<int,GahpClient*> *requestTable;
-	Queue<int> waitingToSubmit;
+	std::queue<int> waitingHighPrio;
+	std::queue<int> waitingMediumPrio;
+	std::queue<int> waitingLowPrio;
 
 	int m_gahp_pid;
 	int m_gahp_readfd;
 	int m_gahp_writefd;
 	int m_gahp_errorfd;
-	MyString m_gahp_error_buffer;
+	std::string m_gahp_error_buffer;
+	bool m_gahp_startup_failed;
 	char m_gahp_version[150];
 	StringList * m_commands_supported;
 	bool use_prefix;
@@ -305,7 +313,7 @@ class GahpClient : public Service {
 
 		Proxy *getMasterProxy();
 
-		bool isStarted() { return server->m_gahp_pid != -1; }
+		bool isStarted() { return server->m_gahp_pid != -1 && !server->m_gahp_startup_failed; }
 		bool isInitialized() { return server->is_initialized; }
 
 		const char *getErrorString();
@@ -323,6 +331,9 @@ class GahpClient : public Service {
 		const char *
 			getGlobusGassServerUrl() { return server->globus_gass_server_url; }
 
+		const char *getGt2CallbackContact()
+			{ return server->globus_gt2_gram_callback_contact; }
+
 		/// cache it from the gahp
 		const char *
 		globus_gram_client_error_string(int error_code);
@@ -338,7 +349,7 @@ class GahpClient : public Service {
 		int 
 		globus_gram_client_job_request(const char * resource_manager_contact,
 			const char * description,
-			const int job_state_mask,
+			const int limited_deleg,
 			const char * callback_contact,
 			char ** job_contact);
 
@@ -380,12 +391,17 @@ class GahpClient : public Service {
 
 		///
 		int
-		globus_gram_client_job_refresh_credentials(const char *job_contact);
+		globus_gram_client_job_refresh_credentials(const char *job_contact,
+												   int limited_deleg);
 
 		///
 		int
 		globus_gass_server_superez_init( char **gass_url, int port );
 
+		///
+		int 
+		globus_gram_client_get_jobmanager_version(const char * resource_manager_contact);
+
 
 		///
 		int
@@ -518,6 +534,11 @@ class GahpClient : public Service {
 		nordugrid_status(const char *hostname, const char *job_id,
 						 char *&status);
 
+		int
+		nordugrid_ldap_query(const char *hostname, const char *ldap_base,
+							 const char *ldap_filter, const char *ldap_attrs,
+							 StringList &results);
+
 		int
 		nordugrid_cancel(const char *hostname, const char *job_id);
 
@@ -542,6 +563,9 @@ class GahpClient : public Service {
 		int
 		nordugrid_ping(const char *hostname);
 
+		int
+		gridftp_transfer(const char *src_url, const char *dst_url);
+
 		///
 		int 
 		unicore_job_create(const char * description,
@@ -569,8 +593,8 @@ class GahpClient : public Service {
 
 		int cream_delegate(const char *delg_service, const char *delg_id);
 		
-		int cream_job_register(const char *service, const char *delg_service, const char *delg_id, 
-							   const char *jdl, time_t lease_time, char **job_id, char **upload_url);
+		int cream_job_register(const char *service, const char *delg_id, 
+							   const char *jdl, const char *lease_id, char **job_id, char **upload_url, char **download_url);
 		
 		int cream_job_start(const char *service, const char *job_id);
 		
@@ -584,23 +608,23 @@ class GahpClient : public Service {
 
 		int cream_job_status(const char *service, const char *job_id, 
 							 char **job_status, int *exit_code, char **failure_reason);
+
+		struct CreamJobStatus {
+			std::string job_id;
+			std::string job_status;
+			int exit_code;
+			std::string failure_reason;
+		};
+		typedef std::map<std::string, CreamJobStatus> CreamJobStatusMap;
+		int cream_job_status_all(const char *service, CreamJobStatusMap & job_ids);
 		
-		int cream_proxy_renew(const char *service, const char *delg_service, const char *delg_id, StringList &job_ids);
+		int cream_proxy_renew(const char *delg_service, const char *delg_id);
 		
 		int cream_ping(const char * service);
 		
-		int cream_job_lease(const char *service, const char *job_id, time_t &lease_incr);
-		
-		//************* Added for Amazon Jobs by fangcao ***************************//
-		
-		/* Phase II work for Amazon jobs (EC2 Part) */
-		
-		/* 
-		** Currently in order to distinguish a job which is submitted from Condor, we will set
-		** is belong to the 'Condor' group. This requires us to create a 'Condor' group before
-		** the start of gahp_server/gahp_client or Condor will create a temporary group name.
-		*/
-		
+		int cream_set_lease(const char *service, const char *lease_id, time_t &lease_expiry);
+
+
 		// 1. Start VM:
 		// AMAZON_COMMAND_VM_START <req_id> <publickeyfile> <privatekeyfile> <ami-id> <keypair> <groupname> <groupname> ...
 		// <keypair> and <groupname> are optional ones.
@@ -610,7 +634,8 @@ class GahpClient : public Service {
 		//		seq_id 0 <instance_id>
 		//		seq_id 1 <error_code> <error_string>
 		//		seq_id 1 
-		int amazon_vm_start( const char * publickeyfile,
+		int amazon_vm_start( const char * service_url,
+							 const char * publickeyfile,
 							 const char * privatekeyfile,
 							 const char * ami_id,
 							 const char * keypair,
@@ -628,11 +653,13 @@ class GahpClient : public Service {
 		//		seq_id 0
 		//		seq_id 1 <error_code> <error_string>
 		//		seq_id 1
-		int amazon_vm_stop( const char * publickeyfile,
+		int amazon_vm_stop( const char * service_url,
+							const char * publickeyfile,
 							const char * privatekeyfile,
 							const char * instance_id,
 							char* & error_code );
 							
+#if 0
 		// 3. Reboot VM:
 		// AMAZON_COMMAND_VM_REBOOT <req_id> <publickeyfile> <privatekeyfile> <instance-id>
 		// return: success/failed
@@ -644,6 +671,7 @@ class GahpClient : public Service {
 							  const char * privatekeyfile,
 							  const char * instance_id,
 							  char* & error_code );		
+#endif
 		
 		// 4. Status VM:
 		// AMAZON_COMMAND_VM_STATUS <req_id> <publickeyfile> <privatekeyfile> <instance-id>
@@ -663,12 +691,14 @@ class GahpClient : public Service {
 		** with "Null", so it means public_dns will always in position 4 and groupname will always 
 		** start from position 7 
 		*/
-		int amazon_vm_status( const char * publickeyfile,
+		int amazon_vm_status( const char * service_url,
+							  const char * publickeyfile,
 							  const char * privatekeyfile,
 							  const char * instance_id,
 							  StringList & returnStatus,
 							  char* & error_code );
 				
+#if 0
 		// 5. Status ALL VM:
 		// AMAZON_COMMAND_VM_STATUS_ALL <req_id> <publickeyfile> <privatekeyfile>
 		// return: success/failed + <instance_id> <status> <ami_id> <instance_id> <status> <ami_id>     NULL
@@ -766,10 +796,12 @@ class GahpClient : public Service {
 								   	  const char * end_port,
 								   	  const char * ip_range,
 								   	  char* & error_code );		
+#endif
 		
 		// 12. Ping
 		// we also need to define a ping function, which will be used by amazon_resource
-		int amazon_ping( const char * publickeyfile,
+		int amazon_ping( const char * service_url,
+						 const char * publickeyfile,
 						 const char * privatekeyfile );
 		
 		//**************************************************************//
@@ -823,7 +855,8 @@ class GahpClient : public Service {
 		//		seq_id 0
 		//		seq_id 1
 		//		seq_id 1 <error_code> <error_string>
-		int amazon_vm_create_keypair( const char * publickeyfile,
+		int amazon_vm_create_keypair( const char * service_url,
+									  const char * publickeyfile,
 								   	  const char * privatekeyfile,
 								   	  const char * keyname,
 								   	  const char * outputfile,
@@ -837,11 +870,13 @@ class GahpClient : public Service {
 		//		seq_id 0
 		//		seq_id 1
 		//		seq_id 1 <error_code> <error_string>
-		int amazon_vm_destroy_keypair( const char * publickeyfile,
+		int amazon_vm_destroy_keypair( const char * service_url,
+									   const char * publickeyfile,
 								   	   const char * privatekeyfile,
 								   	   const char * keyname,
 								   	   char* & error_code );
 								   	   
+#if 0
 		// 15. List all existing SSH Keypair name
 		// AMAZON_COMMAND_VM_KEYPAIR_NAMES <req_id> <publickeyfile> <privatekeyfile>
 		// return: success/failed
@@ -994,6 +1029,7 @@ class GahpClient : public Service {
 										  const char* bucketname,
 										  const char* localdirname,
 										  char* & error_code );
+#endif
 		
 		// 27. check all the running VM instances and their corresponding keypair name								  
 		// AMAZON_COMMAND_VM_KEYPAIR_ALL <req_id> <publickeyfile> <privatekeyfile>
@@ -1002,14 +1038,58 @@ class GahpClient : public Service {
 		//		seq_id 0 <instance_id> <keypair> <instance_id> <keypair> ... 
 		//		seq_id 1 <error_code> <error_string>
 		//		seq_id 1
-		int amazon_vm_vm_keypair_all( const char* publickeyfile,
+		int amazon_vm_vm_keypair_all( const char * service_url,
+									  const char* publickeyfile,
 									  const char* privatekeyfile,
 									  StringList & returnStatus,
 								  	  char* & error_code );
-		
-		//************* End of changes for Amamzon Jobs by fangcao *****************//
 			
 
+		int
+		dcloud_submit( const char *service_url,
+					   const char *username,
+					   const char *password,
+					   const char *image_id,
+					   const char *instance_name,
+					   const char *realm_id,
+					   const char *hwp_id,
+					   const char *hwp_memory,
+					   const char *hwp_cpu,
+					   const char *hwp_storage,
+					   const char *keyname,
+					   const char *userdata,
+					   StringList &attrs );
+
+		int
+		dcloud_status_all( const char *service_url,
+						   const char *username,
+						   const char *password,
+						   StringList &instance_ids,
+						   StringList &statuses );
+
+		int
+		dcloud_action( const char *service_url,
+					   const char *username,
+					   const char *password,
+					   const char *instance_id,
+					   const char *action );
+
+		int
+		dcloud_info( const char *service_url,
+					 const char *username,
+					 const char *password,
+					 const char *instance_id,
+					 StringList &attrs );
+
+		int
+		dcloud_find( const char *service_url,
+					 const char *username,
+					 const char *password,
+					 const char *instance_name,
+					 char **instance_id );
+
+
+
 #ifdef CONDOR_GLOBUS_HELPER_WANT_DUROC
 	// Not yet ready for prime time...
 	globus_duroc_control_barrier_release();
@@ -1025,15 +1105,23 @@ class GahpClient : public Service {
 
 	private:
 
+	/// Enum used by now_pending for the waiting queues
+	enum PrioLevel {
+		/** */ low_prio,
+		/** */ medium_prio,
+		/** */ high_prio
+	};
+
 			// Various Private Methods
 		void clear_pending();
 		bool is_pending(const char *command, const char *buf);
 		void now_pending(const char *command,const char *buf,
-						 GahpProxyInfo *proxy = NULL);
+						 GahpProxyInfo *proxy = NULL,
+						 PrioLevel prio_level = medium_prio);
 		Gahp_Args* get_pending_result(const char *,const char *);
 		bool check_pending_timeout(const char *,const char *);
 		int reset_user_timer(int tid);
-		int reset_user_timer_alarm();
+		void reset_user_timer_alarm();
 
 			// Private Data Members
 		unsigned int m_timeout;
@@ -1049,7 +1137,7 @@ class GahpClient : public Service {
 		GahpProxyInfo *normal_proxy;
 		GahpProxyInfo *deleg_proxy;
 		GahpProxyInfo *pending_proxy;
-		MyString error_string;
+		std::string error_string;
 
 			// These data members all deal with the GAHP
 			// server.  Since there is only one instance of the GAHP
