@@ -33,7 +33,7 @@
 // If the grid_monitor appears hosed, how long do we
 // disable grid_monitoring that site for?
 // (We actually just set the timer for that site to this)
-#define GM_DISABLE_LENGTH (60*60)
+int GlobusResource::monitorDisableLength = DEFAULT_GM_DISABLE_LENGTH;
 
 int GlobusResource::gahpCallTimeout = 300;	// default value
 bool GlobusResource::enableGridMonitor = false;
@@ -47,7 +47,8 @@ HashTable <HashKey, GlobusResource *>
 static unsigned int g_MonitorUID = 0;
 
 GlobusResource *GlobusResource::FindOrCreateResource( const char *resource_name,
-													  const char *proxy_subject )
+													  const Proxy *proxy,
+													  bool is_gt5 )
 {
 	int rc;
 	GlobusResource *resource = NULL;
@@ -55,12 +56,12 @@ GlobusResource *GlobusResource::FindOrCreateResource( const char *resource_name,
 	const char *canonical_name = CanonicalName( resource_name );
 	ASSERT(canonical_name);
 
-	const char *hash_name = HashName( canonical_name, proxy_subject );
+	const char *hash_name = HashName( canonical_name, proxy->subject->fqan );
 	ASSERT(hash_name);
 
 	rc = ResourcesByName.lookup( HashKey( hash_name ), resource );
 	if ( rc != 0 ) {
-		resource = new GlobusResource( canonical_name, proxy_subject );
+		resource = new GlobusResource( canonical_name, proxy, is_gt5 );
 		ASSERT(resource);
 		if ( resource->Init() == false ) {
 			delete resource;
@@ -76,13 +77,23 @@ GlobusResource *GlobusResource::FindOrCreateResource( const char *resource_name,
 }
 
 GlobusResource::GlobusResource( const char *resource_name,
-								const char *proxy_subject )
+								const Proxy *proxy, bool is_gt5 )
 	: BaseResource( resource_name )
 {
 	initialized = false;
-	proxySubject = strdup( proxy_subject );
+	proxySubject = strdup( proxy->subject->subject_name );
+	proxyFQAN = strdup( proxy->subject->fqan );
 
-	jmLimit = DEFAULT_MAX_JOBMANAGERS_PER_RESOURCE;
+	if ( param_boolean( "GRAM_VERSION_DETECTION", true ) ) {
+		m_isGt5 = false;
+		m_versionKnown = false;
+	} else {
+		m_isGt5 = is_gt5;
+		m_versionKnown = true;
+	}
+
+	submitJMLimit = DEFAULT_MAX_JOBMANAGERS_PER_RESOURCE / 2;
+	restartJMLimit = DEFAULT_MAX_JOBMANAGERS_PER_RESOURCE - submitJMLimit;
 
 	checkMonitorTid = daemonCore->Register_Timer( TIMER_NEVER,
 							(TimerHandlercpp)&GlobusResource::CheckMonitor,
@@ -94,17 +105,20 @@ GlobusResource::GlobusResource( const char *resource_name,
 	monitorDirectory = NULL;
 	monitorJobStatusFile = NULL;
 	monitorLogFile = NULL;
-	logFileTimeoutLastReadTime = 0;
+	logFileLastReadTime = 0;
 	jobStatusFileLastUpdate = 0;
 	monitorGramJobId = NULL;
 	gahp = NULL;
 	monitorGahp = NULL;
 	monitorRetryTime = 0;
+	monitorFirstStartup = true;
+	monitorGramJobStatus = GLOBUS_GRAM_PROTOCOL_JOB_STATE_UNKNOWN;
+	monitorGramErrorCode = 0;
 }
 
 GlobusResource::~GlobusResource()
 {
-	ResourcesByName.remove( HashKey( HashName( resourceName, proxySubject ) ) );
+	ResourcesByName.remove( HashKey( HashName( resourceName, proxyFQAN ) ) );
 	if ( checkMonitorTid != TIMER_UNSET ) {
 		daemonCore->Cancel_Timer( checkMonitorTid );
 	}
@@ -118,6 +132,7 @@ GlobusResource::~GlobusResource()
 	if ( proxySubject ) {
 		free( proxySubject );
 	}
+	free( proxyFQAN );
 }
 
 bool GlobusResource::Init()
@@ -126,16 +141,16 @@ bool GlobusResource::Init()
 		return true;
 	}
 
-	MyString gahp_name;
-	gahp_name.sprintf( "GT2/%s", proxySubject );
+	std::string gahp_name;
+	sprintf( gahp_name, "GT2/%s", proxyFQAN );
 
-	gahp = new GahpClient( gahp_name.Value() );
+	gahp = new GahpClient( gahp_name.c_str() );
 
 	gahp->setNotificationTimerId( pingTimerId );
 	gahp->setMode( GahpClient::normal );
 	gahp->setTimeout( gahpCallTimeout );
 
-	monitorGahp = new GahpClient( gahp_name.Value() );
+	monitorGahp = new GahpClient( gahp_name.c_str() );
 
 	monitorGahp->setNotificationTimerId( checkMonitorTid );
 	monitorGahp->setMode( GahpClient::normal );
@@ -150,23 +165,32 @@ bool GlobusResource::Init()
 
 void GlobusResource::Reconfig()
 {
+	int tmp_int;
+
 	BaseResource::Reconfig();
 
 	gahp->setTimeout( gahpCallTimeout );
 
-	jmLimit = param_integer( "GRIDMANAGER_MAX_JOBMANAGERS_PER_RESOURCE",
+	tmp_int = param_integer( "GRIDMANAGER_MAX_JOBMANAGERS_PER_RESOURCE",
 							 DEFAULT_MAX_JOBMANAGERS_PER_RESOURCE );
-	if ( jmLimit == 0 ) {
-		jmLimit = GM_RESOURCE_UNLIMITED;
+	if ( tmp_int == 0 ) {
+		submitJMLimit = GM_RESOURCE_UNLIMITED;
+		restartJMLimit = GM_RESOURCE_UNLIMITED;
+	} else {
+		if ( tmp_int < 2 ) {
+			tmp_int = 2;
+		}
+		submitJMLimit = tmp_int / 2;
+		restartJMLimit = tmp_int - submitJMLimit;
 	}
 
-	// If the jmLimit was widened, move jobs from Wanted to Allowed and
-	// signal them
-	while ( jmsAllowed.Length() < jmLimit && jmsWanted.Length() > 0 ) {
-		GlobusJob *wanted_job = jmsWanted.Head();
-		jmsWanted.Delete( wanted_job );
-		jmsAllowed.Append( wanted_job );
-		wanted_job->SetEvaluateState();
+	// If the jobmanager limits were widened, move jobs from Wanted to
+	// Allowed and signal them
+	while ( ( submitJMsAllowed.Length() + restartJMsAllowed.Length() <
+			  submitJMLimit + restartJMLimit ) &&
+			( submitJMsWanted.Length() != 0 ||
+			  restartJMsWanted.Length() != 0 ) ) {
+		JMComplete( NULL );
 	}
 
 	if ( enableGridMonitor ) {
@@ -189,28 +213,47 @@ const char *GlobusResource::ResourceType()
 
 const char *GlobusResource::CanonicalName( const char *name )
 {
-	static MyString canonical;
+	static std::string canonical;
 	char *host;
 	char *port;
 
 	parse_resource_manager_string( name, &host, &port, NULL, NULL );
 
-	canonical.sprintf( "%s:%s", host, *port ? port : "2119" );
+	sprintf( canonical, "%s:%s", host, *port ? port : "2119" );
 
 	free( host );
 	free( port );
 
-	return canonical.Value();
+	return canonical.c_str();
 }
 
 const char *GlobusResource::HashName( const char *resource_name,
 									  const char *proxy_subject )
 {
-	static MyString hash_name;
+	static std::string hash_name;
 
-	hash_name.sprintf( "%s#%s", resource_name, proxy_subject );
+	sprintf( hash_name, "gt2 %s#%s", resource_name, proxy_subject );
 
-	return hash_name.Value();
+	return hash_name.c_str();
+}
+
+const char *GlobusResource::GetHashName()
+{
+	return HashName( resourceName, proxyFQAN );
+}
+
+void GlobusResource::PublishResourceAd( ClassAd *resource_ad )
+{
+	BaseResource::PublishResourceAd( resource_ad );
+
+	resource_ad->Assign( ATTR_X509_USER_PROXY_SUBJECT, proxySubject );
+	resource_ad->Assign( ATTR_X509_USER_PROXY_FQAN, proxyFQAN );
+	resource_ad->Assign( "SubmitJobmanagerLimit", submitJMLimit );
+	resource_ad->Assign( "SubmitJobmanagersAllowed", submitJMsAllowed.Number() );
+	resource_ad->Assign( "SubmitJobmanagersWanted", submitJMsWanted.Number() );
+	resource_ad->Assign( "RestartJobmanagerLimit", restartJMLimit );
+	resource_ad->Assign( "RestartJobmanagersAllowed", restartJMsAllowed.Number() );
+	resource_ad->Assign( "RestartJobmanagersWanted", restartJMsWanted.Number() );
 }
 
 void GlobusResource::UnregisterJob( GlobusJob *job )
@@ -221,58 +264,116 @@ void GlobusResource::UnregisterJob( GlobusJob *job )
 		// This object may be deleted now. Don't do anything below here!
 }
 
-bool GlobusResource::RequestJM( GlobusJob *job )
+bool GlobusResource::RequestJM( GlobusJob *job, bool is_submit )
 {
 	GlobusJob *jobptr;
 
-	jmsWanted.Rewind();
-	while ( jmsWanted.Next( jobptr ) ) {
-		if ( jobptr == job ) {
+	if ( m_isGt5 ) {
+		return true;
+	}
+
+	if ( is_submit ) {
+		submitJMsWanted.Rewind();
+		while ( submitJMsWanted.Next( jobptr ) ) {
+			if ( jobptr == job ) {
+				return false;
+			}
+		}
+
+		submitJMsAllowed.Rewind();
+		while ( submitJMsAllowed.Next( jobptr ) ) {
+			if ( jobptr == job ) {
+				return true;
+			}
+		}
+
+		if ( submitJMsAllowed.Length() + restartJMsAllowed.Length() <
+			 submitJMLimit + restartJMLimit ) {
+			submitJMsAllowed.Append( job );
+			return true;
+		} else {
+			submitJMsWanted.Append( job );
 			return false;
 		}
-	}
-
-	jmsAllowed.Rewind();
-	while ( jmsAllowed.Next( jobptr ) ) {
-		if ( jobptr == job ) {
-			return true;
-		}
-	}
-
-	if ( jmsAllowed.Length() < jmLimit && jmsWanted.Length() > 0 ) {
-		EXCEPT("In GlobusResource for %s, jmsWanted is not empty and jmsAllowed is not full\n",resourceName);
-	}
-
-	if ( jmsAllowed.Length() < jmLimit ) {
-		jmsAllowed.Append( job );
-		return true;
 	} else {
-		jmsWanted.Append( job );
-		return false;
+		restartJMsWanted.Rewind();
+		while ( restartJMsWanted.Next( jobptr ) ) {
+			if ( jobptr == job ) {
+				return false;
+			}
+		}
+
+		restartJMsAllowed.Rewind();
+		while ( restartJMsAllowed.Next( jobptr ) ) {
+			if ( jobptr == job ) {
+				return true;
+			}
+		}
+
+		if ( submitJMsAllowed.Length() + restartJMsAllowed.Length() <
+			 submitJMLimit + restartJMLimit ) {
+			restartJMsAllowed.Append( job );
+			return true;
+		} else {
+			restartJMsWanted.Append( job );
+			return false;
+		}
 	}
 }
 
 void GlobusResource::JMComplete( GlobusJob *job )
 {
-	if ( jmsAllowed.Delete( job ) ) {
-		if ( jmsAllowed.Length() < jmLimit && jmsWanted.Length() > 0 ) {
-			GlobusJob *queued_job = jmsWanted.Head();
-			jmsWanted.Delete( queued_job );
-			jmsAllowed.Append( queued_job );
-			queued_job->SetEvaluateState();
-		}
-	} else {
-		// We only have to check jmsWanted if the job wasn't in
-		// jmsAllowed.
-		jmsWanted.Delete( job );
+	if ( m_isGt5 ) {
+		return;
 	}
 
-	return;
+	// If the job was in one of the Allowed queues, check whether we
+	// should promote a job from the Wanted queues to take its place.
+	// Also perform the check if NULL is passed.
+	if ( job == NULL || submitJMsAllowed.Delete( job ) ||
+		 restartJMsAllowed.Delete( job ) ) {
+
+		// Check if there's room in the Allowed queues for a job and
+		// if there are any jobs waiting in the Wanted queues.
+		if ( ( submitJMsAllowed.Length() + restartJMsAllowed.Length() <
+			   submitJMLimit + restartJMLimit ) &&
+			 ( submitJMsWanted.Length() != 0 ||
+			   restartJMsWanted.Length() != 0 ) ) {
+
+			// Which Wanted queue should we take a job from? If either
+			// queue is empty, take a job from the other queue.
+			// If neither queue is empty, take a job from the queue
+			// that's under its limit.
+			if ( submitJMsWanted.Length() == 0 ||
+				 ( restartJMsWanted.Length() != 0 &&
+				   restartJMsAllowed.Length() < restartJMLimit ) ) {
+
+				GlobusJob *queued_job = restartJMsWanted.Head();
+				restartJMsWanted.Delete( queued_job );
+				restartJMsAllowed.Append( queued_job );
+				queued_job->SetEvaluateState();
+
+			} else {
+
+				GlobusJob *queued_job = submitJMsWanted.Head();
+				submitJMsWanted.Delete( queued_job );
+				submitJMsAllowed.Append( queued_job );
+				queued_job->SetEvaluateState();
+			}
+		}
+	} else {
+		// We only have to check the Wanted queues if the job wasn't in
+		// the Allowed queues
+		submitJMsWanted.Delete( job );
+		restartJMsWanted.Delete( job );
+	}
 }
 
 void GlobusResource::JMAlreadyRunning( GlobusJob *job )
 {
-	jmsAllowed.Append( job );
+	if ( !m_isGt5 ) {
+		restartJMsAllowed.Append( job );
+	}
 }
 
 void GlobusResource::DoPing( time_t& ping_delay, bool& ping_complete,
@@ -294,7 +395,11 @@ void GlobusResource::DoPing( time_t& ping_delay, bool& ping_complete,
 
 	ping_delay = 0;
 
-	rc = gahp->globus_gram_client_ping( resourceName );
+	if ( m_versionKnown ) {
+		rc = gahp->globus_gram_client_ping( resourceName );
+	} else {
+		rc = gahp->globus_gram_client_get_jobmanager_version( resourceName );
+	}
 
 	if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
 		ping_complete = false;
@@ -304,12 +409,34 @@ void GlobusResource::DoPing( time_t& ping_delay, bool& ping_complete,
 		ping_complete = true;
 		ping_succeeded = false;
 	} else {
+		if ( !m_versionKnown ) {
+			if ( rc == GLOBUS_GRAM_PROTOCOL_ERROR_HTTP_UNPACK_FAILED ) {
+				m_isGt5 = false;
+				m_versionKnown = true;
+			} else if ( rc == GLOBUS_SUCCESS ) {
+				m_isGt5 = true;
+				m_versionKnown = true;
+
+				// Any jobs queued up under gt2 throttles need to signaled.
+				GlobusJob *job;
+				submitJMsWanted.Rewind();
+				while ( submitJMsWanted.Next( job ) ) {
+					job->SetEvaluateState();
+				}
+
+				restartJMsWanted.Rewind();
+				while ( restartJMsWanted.Next( job ) ) {
+					job->SetEvaluateState();
+				}
+			}
+		}
+
 		ping_complete = true;
 		ping_succeeded = true;
 	}
 }
 
-int
+void
 GlobusResource::CheckMonitor()
 {
 	GlobusJob *job;
@@ -322,28 +449,34 @@ GlobusResource::CheckMonitor()
 	dprintf(D_FULLDEBUG, "grid_monitor for %s entering CheckMonitor\n",
 		resourceName);
 
+	if ( m_versionKnown && m_isGt5 ) {
+		dprintf( D_FULLDEBUG, "Disabling grid_monitor for GRAM5 server %s\n",
+				 resourceName );
+		return;
+	}
+
 	if ( monitorGahp->isInitialized() == false ) {
 		dprintf( D_ALWAYS, "GAHP server not initialized yet, not submitting "
 				 "grid_monitor now\n" );
 		daemonCore->Reset_Timer( checkMonitorTid, 5 );
-		return TRUE;
+		return;
 	}
 
 	if ( !enableGridMonitor ) {
-		return TRUE;
+		return;
 	}
 
 	if ( time(NULL) < monitorRetryTime ) {
 		daemonCore->Reset_Timer( checkMonitorTid,
 								 monitorRetryTime - time(NULL) );
-		return TRUE;
+		return;
 	}
 
 	if ( firstPingDone == false ) {
 		dprintf(D_FULLDEBUG,"grid_monitor for %s: first ping not done yet, "
 			"will retry later\n", resourceName);
 		daemonCore->Reset_Timer( checkMonitorTid, 5 );
-		return TRUE;
+		return;
 	}
 
 	if ( monitorSubmitActive ) {
@@ -366,7 +499,7 @@ GlobusResource::CheckMonitor()
 					monitorGahp->globus_gram_client_error_string(rc));
 			monitorSubmitActive = false;
 			AbandonMonitor();
-			return TRUE;
+			return;
 		}
 	}
 
@@ -427,7 +560,7 @@ GlobusResource::CheckMonitor()
 					"file for %s, stopping grid monitor\n", resourceName);
 				// TODO: Try to restart monitor?
 				AbandonMonitor();
-				return TRUE;
+				return;
 
 			} else {
 				EXCEPT("ReadMonitorJobStatusFile returned unexpected %d "
@@ -454,6 +587,7 @@ GlobusResource::CheckMonitor()
 					dprintf(D_ALWAYS, "Successfully started grid_monitor "
 							"for %s\n", resourceName);
 					monitorStarting = false;
+					monitorFirstStartup = false;
 					monitorActive = true;
 					registeredJobs.Rewind();
 					while ( registeredJobs.Next( (BaseJob*&)job ) ) {
@@ -461,7 +595,6 @@ GlobusResource::CheckMonitor()
 					}
 				}
 				logFileLastReadTime = time(NULL);
-				logFileTimeoutLastReadTime = time(NULL);
 				daemonCore->Reset_Timer( checkMonitorTid, 30 );
 				break;
 
@@ -494,16 +627,22 @@ GlobusResource::CheckMonitor()
 
 			}
 
-		} else if ( time(NULL) > logFileLastReadTime + log_file_timeout ) {
+		} else if ( time(NULL) > logFileLastReadTime + log_file_timeout &&
+					!monitorStarting ) {
+			dprintf( D_ALWAYS, "Haven't heard from running grid_monitor "
+					 "at %s for %d seconds, trying new job submission\n",
+					 resourceName, log_file_timeout );
 			if( ! SubmitMonitorJob() ) {
 				dprintf(D_ALWAYS, "Failed to restart grid_monitor.  Giving up on grid_monitor for site %s\n", resourceName);
 				AbandonMonitor();
 			}
 			daemonCore->Reset_Timer( checkMonitorTid, 30);
 
-		} else if ( time(NULL) > logFileTimeoutLastReadTime + monitor_retry_duration) {
-			dprintf(D_ALWAYS, "grid_monitor log file for %s is too old.\n",
-				resourceName);
+		} else if ( monitorStarting &&
+					time(NULL) > logFileLastReadTime + monitor_retry_duration) {
+			dprintf( D_ALWAYS, "Haven't heard from new grid_monitor "
+					 "at %s for %d seconds, giving up\n",
+					 resourceName, monitor_retry_duration );
 			AbandonMonitor();
 
 		} else {
@@ -511,7 +650,7 @@ GlobusResource::CheckMonitor()
 		}
 	}
 
-	return TRUE;
+	return;
 }
 
 void
@@ -519,10 +658,10 @@ GlobusResource::AbandonMonitor()
 {
 	dprintf(D_ALWAYS, "Giving up on grid_monitor for site %s.  "
 		"Will retry in %d seconds (%d minutes)\n",
-		resourceName, GM_DISABLE_LENGTH, GM_DISABLE_LENGTH / 60);
+		resourceName, monitorDisableLength, monitorDisableLength / 60);
 	StopMonitor();
-	monitorRetryTime = time(NULL) + GM_DISABLE_LENGTH;
-	daemonCore->Reset_Timer( checkMonitorTid, GM_DISABLE_LENGTH);
+	monitorRetryTime = time(NULL) + monitorDisableLength;
+	daemonCore->Reset_Timer( checkMonitorTid, monitorDisableLength );
 }
 
 void
@@ -532,10 +671,11 @@ GlobusResource::StopMonitor()
 
 	dprintf(D_ALWAYS, "Stopping grid_monitor for resource %s\n", resourceName);
 
-	monitorStarting = false;
+		// Do we want to notify jobs when the grid monitor fails?
 	monitorActive = false;
-	if ( monitorActive ) {
-		monitorActive = false;
+	monitorFirstStartup = false;
+	monitorStarting = false;
+	if ( monitorActive || monitorFirstStartup ) {
 		registeredJobs.Rewind();
 		while ( registeredJobs.Next( (BaseJob*&)job ) ) {
 			job->SetEvaluateState();
@@ -563,20 +703,22 @@ GlobusResource::CleanupMonitorJob()
 
 		free( monitorGramJobId );
 		monitorGramJobId = NULL;
+		monitorGramJobStatus = GLOBUS_GRAM_PROTOCOL_JOB_STATE_UNKNOWN;
+		monitorGramErrorCode = 0;
 	}
 	if ( monitorDirectory ) {
-		MyString tmp_dir;
+		std::string tmp_dir;
 
-		tmp_dir.sprintf( "%s.remove", monitorDirectory );
+		sprintf( tmp_dir, "%s.remove", monitorDirectory );
 
-		rename( monitorDirectory, tmp_dir.Value() );
+		rename( monitorDirectory, tmp_dir.c_str() );
 		free( monitorDirectory );
 		monitorDirectory = NULL;
 
-		Directory tmp( tmp_dir.Value() );
+		Directory tmp( tmp_dir.c_str() );
 		tmp.Remove_Entire_Directory();
 
-		rmdir( tmp_dir.Value() );
+		rmdir( tmp_dir.c_str() );
 	}
 	if(monitorJobStatusFile)
 	{
@@ -597,18 +739,18 @@ GlobusResource::SubmitMonitorJob()
 	int now = time(NULL);
 	int rc;
 	char *monitor_executable;
-	MyString contact;
-	MyString RSL;
+	std::string contact;
+	std::string RSL;
 
 	StopMonitorJob();
 	
 	/* Create monitor directory and files */
 	g_MonitorUID++;
-	MyString buff;
+	std::string buff;
 
-	buff.sprintf( "%s/grid-monitor.%s.%d", GridmanagerScratchDir,
+	sprintf( buff, "%s/grid-monitor.%s.%d", GridmanagerScratchDir,
 				  resourceName, g_MonitorUID );
-	monitorDirectory = strdup( buff.Value() );
+	monitorDirectory = strdup( buff.c_str() );
 
 	if ( mkdir( monitorDirectory, 0700 ) < 0 ) {
 		dprintf( D_ALWAYS, "SubmitMonitorJob: mkdir(%s,0700) failed, "
@@ -619,11 +761,11 @@ GlobusResource::SubmitMonitorJob()
 		return false;
 	}
 
-	buff.sprintf( "%s/grid-monitor-job-status", monitorDirectory );
-	monitorJobStatusFile = strdup( buff.Value() );
+	sprintf( buff, "%s/grid-monitor-job-status", monitorDirectory );
+	monitorJobStatusFile = strdup( buff.c_str() );
 
-	buff.sprintf( "%s/grid-monitor-log", monitorDirectory );
-	monitorLogFile = strdup( buff.Value() );
+	sprintf( buff, "%s/grid-monitor-log", monitorDirectory );
+	monitorLogFile = strdup( buff.c_str() );
 
 
 	rc = creat( monitorJobStatusFile, S_IREAD|S_IWRITE );
@@ -651,14 +793,6 @@ GlobusResource::SubmitMonitorJob()
 	jobStatusFileLastReadTime = now;
 	logFileLastReadTime = now;
 
-	if( monitorStarting ) {
-		// Anything special to do on a cold start?
-		// (It's possible for this to get called after startup
-		// if someone wants to force a cold restart (say, after
-		// AbandonMonitor())
-		logFileTimeoutLastReadTime = now;
-	}
-
 	monitor_executable = param( "GRID_MONITOR" );
 	if ( monitor_executable == NULL ) {
 		dprintf( D_ALWAYS, "Failed to submit grid_monitor to %s: "
@@ -669,16 +803,17 @@ GlobusResource::SubmitMonitorJob()
 	monitorGahp->setMode( GahpClient::normal );
 
 	const char *gassServerUrl = monitorGahp->getGlobusGassServerUrl();
-	RSL.sprintf( "&(executable=%s%s)(stdout=%s%s)(arguments='--dest-url=%s%s')",
+	sprintf( RSL, "&(executable=%s%s)(stdout=%s%s)(arguments='--dest-url=%s%s')",
 				 gassServerUrl, monitor_executable, gassServerUrl,
 				 monitorLogFile, gassServerUrl, monitorJobStatusFile );
 
 	free( monitor_executable );
 
-	contact.sprintf( "%s/jobmanager-fork", resourceName );
+	sprintf( contact, "%s/jobmanager-fork", resourceName );
 
-	rc = monitorGahp->globus_gram_client_job_request( contact.Value(),
-													  RSL.Value(), 0, NULL,
+	rc = monitorGahp->globus_gram_client_job_request( contact.c_str(),
+													  RSL.c_str(), 1,
+													  monitorGahp->getGt2CallbackContact(),
 													  NULL );
 
 	if ( rc != GAHPCLIENT_COMMAND_PENDING ) {
@@ -780,7 +915,8 @@ GlobusResource::ReadMonitorJobStatusFile()
 	GlobusJob *next_job;
 	registeredJobs.Rewind();
 	while ( (next_job = (GlobusJob *)registeredJobs.Next()) != NULL ) {
-		if ( now > next_job->lastRemoteStatusUpdate + limit ) {
+		if ( next_job->jobContact &&
+			 now > next_job->lastRemoteStatusUpdate + limit ) {
 			next_job->SetEvaluateState();
 		}
 	}
@@ -838,3 +974,13 @@ GlobusResource::ReadMonitorLogFile()
 	return retval;
 }
 
+void
+GlobusResource::gridMonitorCallback( int state, int errorcode )
+{
+	if ( state != monitorGramJobStatus ) {
+		dprintf( D_FULLDEBUG, "grid_monitor for %s: gram callback "
+				 "status=%d errorcode=%d\n", resourceName, state, errorcode );
+		monitorGramJobStatus = state;
+		monitorGramErrorCode = errorcode;
+	}
+}
