@@ -30,6 +30,7 @@
 #include "filename_tools.h"
 #include "classad_helpers.h"
 #include "directory.h"
+#include "condor_version.h"
 
 #include "gridftpmanager.h"
 
@@ -92,14 +93,14 @@ GridftpServer::GridftpServer( Proxy *proxy )
 	m_checkServerTid = daemonCore->Register_Timer( 0,
 								(TimerHandlercpp)&GridftpServer::CheckServer,
 								"GridftpServer::CheckServer", (Service*)this );
-	AcquireProxy( m_proxy, m_checkServerTid );
+	AcquireProxy( m_proxy, (TimerHandlercpp)&GridftpServer::CheckServerSoon, this );
 
 }
 
 GridftpServer::~GridftpServer()
 {
-	m_serversByProxy.remove( HashKey( m_proxy->subject->subject_name ) );
-	ReleaseProxy( m_proxy );
+	m_serversByProxy.remove( HashKey( m_proxy->subject->fqan ) );
+	ReleaseProxy( m_proxy, (TimerHandlercpp)&GridftpServer::CheckServerSoon, this );
 	if ( m_urlBase ) {
 		free( m_urlBase );
 	}
@@ -119,12 +120,12 @@ GridftpServer *GridftpServer::FindOrCreateServer( Proxy *proxy )
 	int rc;
 	GridftpServer *server = NULL;
 
-	rc = m_serversByProxy.lookup( HashKey( proxy->subject->subject_name ),
+	rc = m_serversByProxy.lookup( HashKey( proxy->subject->fqan ),
 								  server );
 	if ( rc != 0 ) {
 		server = new GridftpServer( proxy );
 		ASSERT(server);
-		m_serversByProxy.insert( HashKey( proxy->subject->subject_name ),
+		m_serversByProxy.insert( HashKey( proxy->subject->fqan ),
 								 server );
 	} else {
 		ASSERT(server);
@@ -182,10 +183,10 @@ const char *GridftpServer::GetUrlBase()
 
 const char *GridftpServer::GetErrorMessage()
 {
-	if ( m_errorMessage.IsEmpty() ) {
+	if ( m_errorMessage.empty() ) {
 		return NULL;
 	} else {
-		return m_errorMessage.Value();
+		return m_errorMessage.c_str();
 	}
 }
 
@@ -194,7 +195,7 @@ bool GridftpServer::UseSelfCred()
 	return m_configUrlBase == NULL;
 }
 
-int GridftpServer::UpdateLeases()
+void GridftpServer::UpdateLeases()
 {
 	Qmgr_connection *schedd;
 	bool error = false;
@@ -205,7 +206,7 @@ int GridftpServer::UpdateLeases()
 	dprintf( D_FULLDEBUG, "GridftpServer: Updating job leases for gridftp "
 			 "server jobs\n" );
 
-	schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, false );
+	schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, false, NULL, myUserName, CondorVersion() );
 	if ( !schedd ) {
 		dprintf( D_ALWAYS, "GridftpServer::UpdateLeases: "
 				 "Failed to connect to schedd\n" );
@@ -239,8 +240,6 @@ int GridftpServer::UpdateLeases()
 	} else {
 		daemonCore->Reset_Timer( m_updateLeasesTid, SERVER_JOB_LEASE / 3 );
 	}
-
-	return TRUE;
 }
 
 void GridftpServer::CheckServerSoon( int delta )
@@ -250,9 +249,9 @@ void GridftpServer::CheckServerSoon( int delta )
 	}
 }
 
-int GridftpServer::CheckServer()
+void GridftpServer::CheckServer()
 {
-	bool existing_error = !m_errorMessage.IsEmpty();
+	bool existing_error = !m_errorMessage.empty();
  
 	daemonCore->Reset_Timer( m_checkServerTid, CHECK_SERVER_INTERVAL );
 
@@ -260,14 +259,14 @@ int GridftpServer::CheckServer()
 			// If GRIDFTP_URL_BASE is set in the config file, then never
 			// try to start our own gridftp servers.
 		daemonCore->Reset_Timer( m_checkServerTid, TIMER_NEVER );
-		return TRUE;
+		return;
 	}
 
 	if ( !m_initialScanDone ) {
 		if ( ScanSchedd() ) {
 			m_initialScanDone = true;
 		} else {
-			return TRUE;
+			return;
 		}
 	}
 
@@ -281,7 +280,7 @@ int GridftpServer::CheckServer()
 	if ( IsEmpty() ) {
 		RemoveJob();
 		delete this;
-		return TRUE;
+		return;
 	}
 
 		// TODO wait for an explicit request from a job before
@@ -360,22 +359,20 @@ int GridftpServer::CheckServer()
 		}
 	}
 
-	if ( !existing_error && !m_errorMessage.IsEmpty() ) {
+	if ( !existing_error && !m_errorMessage.empty() ) {
 		int tid;
 		m_registeredClients.Rewind();
 		while ( m_registeredClients.Next( tid ) ) {
 			daemonCore->Reset_Timer( tid, 0 );
 		}
 	}
-
-	return TRUE;
 }
 
 bool GridftpServer::ScanSchedd()
 {
 	Qmgr_connection *schedd;
 	bool error = false;
-	MyString expr;
+	std::string expr;
 	ClassAd *next_ad;
 
 	dprintf( D_FULLDEBUG, "GridftpServer: Scanning schedd for previously "
@@ -383,11 +380,11 @@ bool GridftpServer::ScanSchedd()
 
 	if ( m_updateLeasesTid == TIMER_UNSET ) {
 		m_updateLeasesTid = daemonCore->Register_Timer( 0,
-							(TimerHandler)&GridftpServer::UpdateLeases,
-							"GridftpServer::UpdateLeases", NULL );
+							GridftpServer::UpdateLeases,
+							"GridftpServer::UpdateLeases" );
 	}
 
-	schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, false );
+	schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, false, NULL, myUserName, CondorVersion() );
 	if ( !schedd ) {
 		dprintf( D_ALWAYS, "GridftpServer::ScanSchedd: "
 				 "Failed to connect to schedd\n" );
@@ -406,7 +403,7 @@ bool GridftpServer::ScanSchedd()
 		// jobs to be canceled and resubmitted. Instead, we'll ignore the
 		// restarted server and submit a one that's told to listen on
 		// the old port.
-	expr.sprintf( "%s == \"%s\" && %s =?= True && ( %s == %d || %s == %d ) && "
+	sprintf( expr, "%s == \"%s\" && %s =?= True && ( %s == %d || %s == %d ) && "
 				  "( %s =!= Undefined || ( %s + ( %s == %d ) ) <= 1 )",
 				  ATTR_OWNER, myUserName, ATTR_GRIDFTP_SERVER_JOB,
 				  ATTR_JOB_STATUS, IDLE, ATTR_JOB_STATUS, RUNNING,
@@ -415,10 +412,10 @@ bool GridftpServer::ScanSchedd()
 
 		// TODO check that this didn't return NULL due to a connection
 		//   failure
-	next_ad = GetNextJobByConstraint( expr.Value(), 1 );
+	next_ad = GetNextJobByConstraint( expr.c_str(), 1 );
 	while ( next_ad != NULL ) {
-		MyString buff;
-		MyString buff2;
+		std::string buff;
+		std::string buff2;
 		GridftpServer *server;
 
 			// If we have a Server object for this proxy subect with
@@ -426,14 +423,14 @@ bool GridftpServer::ScanSchedd()
 			// jobs for the same proxy subject, we'll end up picking
 			// the first one, ignoring the requested url base (which
 			// may match one of the later jobs). Tough luck.
-		next_ad->LookupString( ATTR_X509_USER_PROXY_SUBJECT, buff );
-		if ( m_serversByProxy.lookup( HashKey( buff.Value() ), server ) ) {
+		next_ad->LookupString( ATTR_X509_USER_PROXY_FQAN, buff );
+		if ( m_serversByProxy.lookup( HashKey( buff.c_str() ), server ) ) {
 
-			MyString error_str;
+			std::string error_str;
 			Proxy *proxy = AcquireProxy( next_ad, error_str );
 			server = new GridftpServer( proxy );
 			ASSERT(server);
-			m_serversByProxy.insert( HashKey( proxy->subject->subject_name ),
+			m_serversByProxy.insert( HashKey( proxy->subject->fqan ),
 									 server );
 			ReleaseProxy( proxy );
 		}
@@ -451,36 +448,36 @@ bool GridftpServer::ScanSchedd()
 				free( server->m_requestedUrlBase );
 				server->m_requestedUrlBase = NULL;
 			}
-			MyString value;
+			std::string value;
 			next_ad->LookupString( ATTR_JOB_OUTPUT, value );
-			server->m_outputFile = strdup( value.Value() );
+			server->m_outputFile = strdup( value.c_str() );
 			if ( next_ad->LookupString( ATTR_REQUESTED_GRIDFTP_URL_BASE,
 										value ) ) {
-				server->m_requestedUrlBase = strdup( value.Value() );
+				server->m_requestedUrlBase = strdup( value.c_str() );
 			}
 				// TODO should check these SetAttributeString calls
 				//   for errors.
-			buff.sprintf( "SUBMIT_%s", ATTR_JOB_IWD );
+			sprintf( buff, "SUBMIT_%s", ATTR_JOB_IWD );
 			SetAttributeString( server->m_jobId.cluster,
 								server->m_jobId.proc,
-								buff.Value(), GridmanagerScratchDir );
-			buff.sprintf( "SUBMIT_%s", ATTR_JOB_OUTPUT );
-			buff2.sprintf( "%s/%s", GridmanagerScratchDir, STDOUT_NAME );
+								buff.c_str(), GridmanagerScratchDir );
+			sprintf( buff, "SUBMIT_%s", ATTR_JOB_OUTPUT );
+			sprintf( buff2, "%s/%s", GridmanagerScratchDir, STDOUT_NAME );
 			SetAttributeString( server->m_jobId.cluster,
 								server->m_jobId.proc,
-								buff.Value(), buff2.Value() );
-			buff.sprintf( "SUBMIT_%s", ATTR_JOB_ERROR );
-			buff2.sprintf( "%s/%s", GridmanagerScratchDir, STDERR_NAME );
+								buff.c_str(), buff2.c_str() );
+			sprintf( buff, "SUBMIT_%s", ATTR_JOB_ERROR );
+			sprintf( buff2, "%s/%s", GridmanagerScratchDir, STDERR_NAME );
 			SetAttributeString( server->m_jobId.cluster,
 								server->m_jobId.proc,
-								buff.Value(), buff2.Value() );
+								buff.c_str(), buff2.c_str() );
 				// TODO check expiration time on proxy?
 		}
 
 		delete next_ad;
 			// TODO check that this didn't return NULL due to a connection
 			//   failure
-		next_ad = GetNextJobByConstraint( expr.Value(), 0 );
+		next_ad = GetNextJobByConstraint( expr.c_str(), 0 );
 	}
 
 	DisconnectQ( schedd );
@@ -512,12 +509,13 @@ bool GridftpServer::SubmitServerJob()
 	Env env_obj;
 	int low_port, high_port;
 	const char *value;
-	MyString buff;
+	std::string buff;
 	CondorVersionInfo ver_info;
 	ArgList args_list;
+	MyString err_msg;
 
 	dprintf( D_FULLDEBUG, "GridftpServer: Submitting job for proxy '%s'\n",
-			 m_proxy->subject->subject_name );
+			 m_proxy->subject->fqan );
 	if ( m_requestedUrlBase ) {
 		dprintf( D_FULLDEBUG, "   reusing URL %s\n", m_requestedUrlBase );
 	}
@@ -526,7 +524,7 @@ bool GridftpServer::SubmitServerJob()
 	if ( !server_path ) {
 		dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: No gridftp "
 				 "server configured\n" );
-		m_errorMessage.sprintf( "No gridftp server configured" );
+		m_errorMessage = "No gridftp server configured";
 		return false;
 	} else {
 		StatInfo si( server_path );
@@ -534,7 +532,7 @@ bool GridftpServer::SubmitServerJob()
 			dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: "
 					 "GRIDFTP_SERVER file %s is invalid\n",
 					 server_path );
-			m_errorMessage.sprintf( "GRIDFTP_SERVER file is invalid" );
+			m_errorMessage = "GRIDFTP_SERVER file is invalid";
 			goto error_exit;
 		}
 	}
@@ -546,7 +544,7 @@ bool GridftpServer::SubmitServerJob()
 			dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: "
 					 "GRIDFTP_SERVER_WRAPPER file %s is invalid\n",
 					 server_path );
-			m_errorMessage.sprintf( "GRIDFTP_SERVER_WRAPPER file is invalid" );
+			m_errorMessage = "GRIDFTP_SERVER_WRAPPER file is invalid";
 			goto error_exit;
 		}
 	}
@@ -559,17 +557,21 @@ bool GridftpServer::SubmitServerJob()
 	job_ad->Assign( ATTR_X509_USER_PROXY, m_proxy->proxy_filename );
 	job_ad->Assign( ATTR_X509_USER_PROXY_SUBJECT,
 					m_proxy->subject->subject_name );
+		// Always insert FQAN, since that identifies a unique GridftpServer
+		// object, and thus allows us to find find the GridftpServer
+		// object for a given gridftp job later on.
+	job_ad->Assign( ATTR_X509_USER_PROXY_FQAN, m_proxy->subject->fqan );
 	job_ad->Assign( ATTR_JOB_IWD, GridmanagerScratchDir );
-	buff.sprintf( "%s/%s", GridmanagerScratchDir, STDOUT_NAME );
-	job_ad->Assign( ATTR_JOB_OUTPUT, buff.Value() );
-	buff.sprintf( "%s/%s", GridmanagerScratchDir, STDERR_NAME );
-	job_ad->Assign( ATTR_JOB_ERROR, buff.Value() );
+	sprintf( buff, "%s/%s", GridmanagerScratchDir, STDOUT_NAME );
+	job_ad->Assign( ATTR_JOB_OUTPUT, buff.c_str() );
+	sprintf( buff, "%s/%s", GridmanagerScratchDir, STDERR_NAME );
+	job_ad->Assign( ATTR_JOB_ERROR, buff.c_str() );
 
 	job_ad->Assign( ATTR_TIMER_REMOVE_CHECK,
 					(int)time(NULL) + SERVER_JOB_LEASE );
-	buff.sprintf( "%s == %d && %s > CurrentTime", ATTR_JOB_STATUS, COMPLETED,
+	sprintf( buff, "%s == %d && %s > time()", ATTR_JOB_STATUS, COMPLETED,
 				  ATTR_TIMER_REMOVE_CHECK );
-	job_ad->AssignExpr( ATTR_JOB_LEAVE_IN_QUEUE, buff.Value() );
+	job_ad->AssignExpr( ATTR_JOB_LEAVE_IN_QUEUE, buff.c_str() );
 
 	snprintf( mapfile, sizeof(mapfile), "%s/grid-mapfile",
 			  GridmanagerScratchDir );
@@ -606,12 +608,12 @@ bool GridftpServer::SubmitServerJob()
 		// outgoing LOWPORT/HIGHPORT settings, pass them through
 		// to globus via the appropriate environment variables.
 	if ( get_port_range( FALSE, &low_port, &high_port ) == TRUE ) {
-		buff.sprintf( "%d,%d", low_port, high_port );
-		env_obj.SetEnv( "GLOBUS_TCP_PORT_RANGE", buff.Value() );
+		sprintf( buff, "%d,%d", low_port, high_port );
+		env_obj.SetEnv( "GLOBUS_TCP_PORT_RANGE", buff.c_str() );
 	}
 	if ( get_port_range( TRUE, &low_port, &high_port ) == TRUE ) {
-		buff.sprintf( "%d,%d", low_port, high_port );
-		env_obj.SetEnv( "GLOBUS_TCP_SOURCE_RANGE", buff.Value() );
+		sprintf( buff, "%d,%d", low_port, high_port );
+		env_obj.SetEnv( "GLOBUS_TCP_SOURCE_RANGE", buff.c_str() );
 	}
 
 		// TODO Should check config parameter GSI_DAEMON_TRUSTED_CA_DIR?
@@ -620,9 +622,9 @@ bool GridftpServer::SubmitServerJob()
 		env_obj.SetEnv( "X509_CERT_DIR", value );
 	}
 
-	if ( !env_obj.InsertEnvIntoClassAd( job_ad, &buff ) ) {
+	if ( !env_obj.InsertEnvIntoClassAd( job_ad, &err_msg ) ) {
 		dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: Env: %s\n",
-				 buff.Value() );
+				 err_msg.Value() );
 		goto error_exit;
 	}		
 		// TODO what about LD_LIBRARY_PATH?
@@ -653,23 +655,23 @@ bool GridftpServer::SubmitServerJob()
 				url_port = 2811;
 			}
 			args_list.AppendArg( "-p" );
-			buff.sprintf( "%d", url_port );
-			args_list.AppendArg( buff.Value() );
+			sprintf( buff, "%d", url_port );
+			args_list.AppendArg( buff.c_str() );
 
 			job_ad->Assign( ATTR_REQUESTED_GRIDFTP_URL_BASE,
 							m_requestedUrlBase );
 		}
 	}
 
-	if ( !args_list.InsertArgsIntoClassAd( job_ad, &ver_info, &buff ) ) {
+	if ( !args_list.InsertArgsIntoClassAd( job_ad, &ver_info, &err_msg ) ) {
 		dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: ArgList: %s\n",
-				 buff.Value() );
+				 err_msg.Value() );
 		goto error_exit;
 	}
 
 	job_ad->Assign( ATTR_GRIDFTP_SERVER_JOB, true );
 
-	schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, false );
+	schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, false, NULL, myUserName, CondorVersion() );
 	if ( !schedd ) {
 		dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: "
 				 "Failed to connect to schedd\n" );
@@ -698,18 +700,12 @@ bool GridftpServer::SubmitServerJob()
 	job_ad->Assign( ATTR_PROC_ID, proc_id );
 
 	// Set all the classad attribute on the remote classad
+	const char *lhstr, *rhstr;
 	job_ad->ResetExpr();
-	while( (tree = job_ad->NextExpr()) ) {
-		ExprTree *lhs;
-		ExprTree *rhs;
-		char *lhstr, *rhstr;
+	while( job_ad->NextExpr(lhstr, tree) ) {
+		rhstr = ExprTreeToString( tree );
 
-		lhs = NULL, rhs = NULL;
-		rhs = NULL, rhstr = NULL;
-
-		if( (lhs = tree->LArg()) ) { lhs->PrintToNewStr (&lhstr); }
-		if( (rhs = tree->RArg()) ) { rhs->PrintToNewStr (&rhstr); }
-		if( !lhs || !rhs || !lhstr || !rhstr) {
+		if( !lhstr || !rhstr) {
 			dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: Failed to "
 					 "unparse job attribute\n" );
 			goto error_exit;
@@ -717,16 +713,11 @@ bool GridftpServer::SubmitServerJob()
 			dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: Failed to "
 					 "SetAttribute %s=%s for job %d.%d\n",
 					 lhstr, rhstr, cluster_id, proc_id );
-			free( lhstr );
-			free( rhstr );
 			goto error_exit;
 		}
-
-		free( lhstr );
-		free( rhstr );
 	}
 
-	if ( CloseConnection() < 0 ) {
+	if ( RemoteCommitTransaction() < 0 ) {
 		dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: Failed to "
 				 "commit job submission\n" );
 		goto error_exit;
@@ -743,7 +734,7 @@ bool GridftpServer::SubmitServerJob()
 
 	m_outputFile = NULL;
 	while ( m_outputFile == NULL ) {
-		schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, true );
+		schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, true, NULL, myUserName, CondorVersion() );
 		if ( !schedd ) {
 			dprintf( D_ALWAYS, "GridftpServer::SubmitServerJob: "
 					 "Failed to connect to schedd (2nd time)\n" );
@@ -831,22 +822,22 @@ bool GridftpServer::ReadUrlBase()
 		return false;
 	}
 
-	MyString out_name;
+	std::string out_name;
 	FILE *out_fp;
 
-	out_name.sprintf( "%s/%s", GridmanagerScratchDir, STDOUT_NAME );
-	out_fp = safe_fopen_wrapper( out_name.Value(), "r" );
+	sprintf( out_name, "%s/%s", GridmanagerScratchDir, STDOUT_NAME );
+	out_fp = safe_fopen_wrapper( out_name.c_str(), "r" );
 	if ( out_fp == NULL ) {
 		dprintf( D_ALWAYS, "GridftpServer::ReadUrlBase: Failed to open "
-				 "'%s': errno=%d\n", out_name.Value(), errno );
+				 "'%s': errno=%d\n", out_name.c_str(), errno );
 		return false;
 	}
 
 	char buff[1024];
 	if ( fscanf( out_fp, "Server listening at %[^\n]\n", buff ) == 1 ) {
-		MyString buff2;
-		buff2.sprintf( "gsiftp://%s", buff );
-		m_urlBase = strdup( buff2.Value() );
+		std::string buff2;
+		sprintf( buff2, "gsiftp://%s", buff );
+		m_urlBase = strdup( buff2.c_str() );
 	}
 
 	fclose( out_fp );
@@ -869,9 +860,9 @@ bool GridftpServer::ReadUrlBase()
 
 	char buff[1024];
 	if ( fscanf( out, "Server listening at %[^\n]\n", buff ) == 1 ) {
-		MyString buff2;
-		buff2.sprintf( "gsiftp://%s", buff );
-		m_urlBase = strdup( buff2.Value() );
+		std::string buff2;
+		sprintf( buff2, "gsiftp://%s", buff );
+		m_urlBase = strdup( buff2.c_str() );
 	}
 
 	fclose( out );
@@ -914,6 +905,7 @@ void GridftpServer::SetJobStatus( int new_status )
 		m_jobCondition = CONDITION_IDLE;
 		break;
 	case RUNNING:
+	case TRANSFERRING_OUTPUT:
 		m_jobCondition = CONDITION_ACTIVE;
 		break;
 	case REMOVED:
@@ -968,14 +960,14 @@ bool GridftpServer::CheckPortError()
 		return false;
 	}
 
-	MyString err_name;
+	std::string err_name;
 	FILE *err_fp;
 
-	err_name.sprintf( "%s/%s", GridmanagerScratchDir, STDERR_NAME );
-	err_fp = safe_fopen_wrapper( err_name.Value(), "r" );
+	sprintf( err_name, "%s/%s", GridmanagerScratchDir, STDERR_NAME );
+	err_fp = safe_fopen_wrapper( err_name.c_str(), "r" );
 	if ( err_fp == NULL ) {
 		dprintf( D_ALWAYS, "GridftpServer::CheckPortError: Failed to "
-				 "open '%s': errno=%d\n", err_name.Value(), errno );
+				 "open '%s': errno=%d\n", err_name.c_str(), errno );
 		return false;
 	}
 
@@ -1034,12 +1026,12 @@ bool GridftpServer::FetchJobFiles()
 
 	bool result;
 	CondorError errstack;
-	MyString constraint;
+	std::string constraint;
 
-	constraint.sprintf( "%s == %d && %s == %d", ATTR_CLUSTER_ID,
+	sprintf( constraint, "%s == %d && %s == %d", ATTR_CLUSTER_ID,
 						m_jobId.cluster, ATTR_PROC_ID, m_jobId.proc );
 
-	result = ScheddObj->receiveJobSandbox( constraint.Value(), &errstack );
+	result = ScheddObj->receiveJobSandbox( constraint.c_str(), &errstack );
 	if ( result == false ) {
 		dprintf( D_ALWAYS, "GridftpServer::FetchJobFiles: Failed to fetch "
 				 "files: %s\n", errstack.getFullText() );
