@@ -36,20 +36,15 @@
 
 #include "globusjob.h"
 
-#if defined(ORACLE_UNIVERSE)
-#include "oraclejob.h"
-#endif
-
 #include "nordugridjob.h"
 #include "unicorejob.h"
-#include "mirrorjob.h"
 #include "condorjob.h"
 #include "gt4job.h"
 #include "infnbatchjob.h"
+#include "condor_version.h"
 
-// added by fangcao for amazon job 
 #include "amazonjob.h"
-// end of adding by fangcao 
+#include "dcloudjob.h"
 
 #if !defined(WIN32)
 #  include "creamjob.h"
@@ -96,15 +91,22 @@ HashTable <PROC_ID, JobStatusRequest> completedJobStatus( HASH_TABLE_SIZE,
 
 SimpleList<int> scheddUpdateNotifications;
 
-HashTable <PROC_ID, BaseJob *> pendingScheddUpdates( HASH_TABLE_SIZE,
+struct ScheddUpdateRequest {
+	BaseJob *m_job;
+	bool m_notify;
+};
+
+HashTable <PROC_ID, ScheddUpdateRequest *> pendingScheddUpdates( HASH_TABLE_SIZE,
 													 hashFuncPROC_ID );
 bool addJobsSignaled = false;
+bool updateJobsSignaled = false;
 bool checkLeasesSignaled = false;
 int contactScheddTid = TIMER_UNSET;
 int contactScheddDelay;
 time_t lastContactSchedd = 0;
 
 char *ScheddAddr = NULL;
+char *ScheddName = NULL;
 char *ScheddJobConstraint = NULL;
 char *GridmanagerScratchDir = NULL;
 DCSchedd *ScheddObj = NULL;
@@ -114,18 +116,19 @@ int scheddFailureCount = 0;
 int maxScheddFailures = 10;	// Years of careful research...
 
 void RequestContactSchedd();
-int doContactSchedd();
+void doContactSchedd();
 
 // handlers
 int ADD_JOBS_signalHandler( int );
 int REMOVE_JOBS_signalHandler( int );
-int CHECK_LEASES_signalHandler( int );
+void CHECK_LEASES_signalHandler();
+int UPDATE_JOBAD_signalHandler( int );
 
 
 static bool jobExternallyManaged(ClassAd * ad)
 {
 	ASSERT(ad);
-	MyString job_managed;
+	std::string job_managed;
 	if( ! ad->LookupString(ATTR_JOB_MANAGED, job_managed) ) {
 		return false;
 	}
@@ -139,9 +142,9 @@ exposed as a remote call.  Perhaps link it in directly?
 static int tSetAttributeString(int cluster, int proc, 
 	const char * attr_name, const char * attr_value)
 {
-	MyString tmp;
-	tmp.sprintf("\"%s\"", attr_value);
-	return SetAttribute( cluster, proc, attr_name, tmp.Value());
+	std::string tmp;
+	sprintf(tmp, "\"%s\"", attr_value);
+	return SetAttribute( cluster, proc, attr_name, tmp.c_str());
 }
 
 // Check if a job ad needs $$() expansion performed on it. The initial ad
@@ -153,9 +156,9 @@ bool MustExpandJobAd( const ClassAd *job_ad ) {
 
 	job_ad->LookupBool(ATTR_JOB_MUST_EXPAND, must_expand);
 	if ( !must_expand ) {
-		MyString resource_name;
+		std::string resource_name;
 		if ( job_ad->LookupString( ATTR_GRID_RESOURCE, resource_name ) ) {
-			if ( strstr(resource_name.Value(),"$$") ) {
+			if ( strstr(resource_name.c_str(),"$$") ) {
 				must_expand = true;
 			}
 		}
@@ -169,28 +172,37 @@ bool MustExpandJobAd( const ClassAd *job_ad ) {
 // return value of true means requested update has been committed to schedd.
 // return value of false means requested update has been queued, but has not
 //   been committed to the schedd yet
-bool
-requestScheddUpdate( BaseJob *job )
+void
+requestScheddUpdate( BaseJob *job, bool notify )
 {
-	BaseJob *hashed_job;
+	ScheddUpdateRequest *request;
 
 	// Check if there's anything that actually requires contacting the
 	// schedd. If not, just return true (i.e. update is complete)
+
+	const char *name;
+	ExprTree *value;
 	job->jobAd->ResetExpr();
 	if ( job->deleteFromGridmanager == false &&
 		 job->deleteFromSchedd == false &&
-		 job->jobAd->NextDirtyExpr() == NULL ) {
-		return true;
+		 job->jobAd->NextDirtyExpr(name, value) == false ) {
+		job->SetEvaluateState();
+		return;
 	}
 
 	// Check if the job is already in the hash table
-	if ( pendingScheddUpdates.lookup( job->procID, hashed_job ) != 0 ) {
+	if ( pendingScheddUpdates.lookup( job->procID, request ) != 0 ) {
 
-		pendingScheddUpdates.insert( job->procID, job );
+		request = new ScheddUpdateRequest();
+		request->m_job = job;
+		request->m_notify = notify;
+		pendingScheddUpdates.insert( job->procID, request );
 		RequestContactSchedd();
+	} else {
+		if ( request->m_notify == false ) {
+			request->m_notify = notify;
+		}
 	}
-
-	return false;
 }
 
 void
@@ -268,8 +280,8 @@ RequestContactSchedd()
 			delay = (lastContactSchedd + contactScheddDelay) - now;
 		}
 		contactScheddTid = daemonCore->Register_Timer( delay,
-												(TimerHandler)&doContactSchedd,
-												"doContactSchedd", NULL );
+												doContactSchedd,
+												"doContactSchedd" );
 	}
 }
 
@@ -282,14 +294,21 @@ Init()
 	// only set it if it hasn't been set already
 	if ( ScheddAddr == NULL ) {
 		schedd_pid = daemonCore->getppid();
-		ScheddAddr = daemonCore->InfoCommandSinfulString( schedd_pid );
-		if ( ScheddAddr == NULL ) {
+		char const *tmp = daemonCore->InfoCommandSinfulString( schedd_pid );
+		if ( tmp == NULL ) {
 			EXCEPT( "Failed to determine schedd's address" );
 		} else {
-			ScheddAddr = strdup( ScheddAddr );
+			ScheddAddr = strdup( tmp );
 		}
 	}
 
+	const char *val = getenv( "SCHEDD_NAME" );
+	if ( val ) {
+		ScheddName = strdup( val );
+	} else {
+		ScheddName = strdup( "" );
+	}
+
 	if ( ScheddObj == NULL ) {
 		ScheddObj = new DCSchedd( ScheddAddr );
 		ASSERT( ScheddObj );
@@ -298,7 +317,7 @@ Init()
 		}
 	}
 
-	// read config file
+    // read config file
 	// initialize variables
 
 	if ( GridmanagerScratchDir == NULL ) {
@@ -311,16 +330,6 @@ Init()
 
 	JobType *new_type;
 
-#if defined(ORACLE_UNIVERSE)
-	new_type = new JobType;
-	new_type->Name = strdup( "Oracle" );
-	new_type->InitFunc = OracleJobInit;
-	new_type->ReconfigFunc = OracleJobReconfig;
-	new_type->AdMatchFunc = OracleJobAdMatch;
-	new_type->CreateFunc = OracleJobCreate;
-	jobTypes.Append( new_type );
-#endif
-
 #if !defined(WIN32)
 	new_type = new JobType;
 	new_type->Name = strdup( "Cream" );
@@ -339,8 +348,6 @@ Init()
 	new_type->CreateFunc = NordugridJobCreate;
 	jobTypes.Append( new_type );
 	
-	/* ******* added by fangcao for Amazon Job  ********* */ 
-	
 	new_type = new JobType;
 	new_type->Name = strdup( "Amazon" );
 	new_type->InitFunc = AmazonJobInit;
@@ -349,7 +356,13 @@ Init()
 	new_type->CreateFunc = AmazonJobCreate;
 	jobTypes.Append( new_type );
 	
-	/************* end of adding by fangcao ********************/
+	new_type = new JobType;
+	new_type->Name = strdup( "Deltacloud" );
+	new_type->InitFunc = DCloudJobInit;
+	new_type->ReconfigFunc = DCloudJobReconfig;
+	new_type->AdMatchFunc = DCloudJobAdMatch;
+	new_type->CreateFunc = DCloudJobCreate;
+	jobTypes.Append( new_type );
 	
 	new_type = new JobType;
 	new_type->Name = strdup( "Unicore" );
@@ -359,14 +372,6 @@ Init()
 	new_type->CreateFunc = UnicoreJobCreate;
 	jobTypes.Append( new_type );
 
-	new_type = new JobType;
-	new_type->Name = strdup( "Mirror" );
-	new_type->InitFunc = MirrorJobInit;
-	new_type->ReconfigFunc = MirrorJobReconfig;
-	new_type->AdMatchFunc = MirrorJobAdMatch;
-	new_type->CreateFunc = MirrorJobCreate;
-	jobTypes.Append( new_type );
-
 	new_type = new JobType;
 	new_type->Name = strdup( "INFNBatch" );
 	new_type->InitFunc = INFNBatchJobInit;
@@ -417,13 +422,16 @@ Register()
 								 (SignalHandler)&REMOVE_JOBS_signalHandler,
 								 "REMOVE_JOBS_signalHandler", NULL );
 
+	daemonCore->Register_Signal( UPDATE_JOBAD, "UpdateJobAd",
+								 (SignalHandler)&UPDATE_JOBAD_signalHandler,
+								 "UPDATE_JOBAD_signalHandler", NULL );
 /*
 	daemonCore->Register_Signal( GRIDMAN_CHECK_LEASES, "CheckLeases",
 								 (SignalHandler)&CHECK_LEASES_signalHandler,
 								 "CHECK_LEASES_signalHandler", NULL );
 */
-	daemonCore->Register_Timer( 60, 60, (TimerHandler)&CHECK_LEASES_signalHandler,
-								"CHECK_LEASES_signalHandler", NULL );
+	daemonCore->Register_Timer( 60, 60, CHECK_LEASES_signalHandler,
+								"CHECK_LEASES_signalHandler" );
 
 	Reconfig();
 }
@@ -490,6 +498,18 @@ REMOVE_JOBS_signalHandler( int )
 	return TRUE;
 }
 
+int
+UPDATE_JOBAD_signalHandler( int )
+{
+	dprintf(D_FULLDEBUG,"Received UPDATE_JOBAD signal\n");
+	if ( !updateJobsSignaled ) {
+		RequestContactSchedd();
+		updateJobsSignaled = true;
+	}
+
+	return TRUE;
+}
+
 // Call initJobExprs before using any of the expr_*
 // variables.  It is safe to repeatedly call
 // initJobExprs.
@@ -499,25 +519,26 @@ static const char * expr_false = "FALSE";
 //static const char * expr_undefined = "UNDEFINED";
 	// The job is matched, or in unknown match state.
 	// definately unmatched
-static MyString expr_matched_or_undef;
+static std::string expr_matched_or_undef;
 	// Job is being managed by an external process
 	// (probably this gridmanager process)
-static MyString expr_managed;
+static std::string expr_managed;
 	// The job is not being managed by an external
 	// process.  The schedd should be managing it.
 	// It may be done.
-static MyString expr_not_managed;
+static std::string expr_not_managed;
 	// The job is not HELD
-static MyString expr_not_held;
+static std::string expr_not_held;
 	// The constraint passed into the gridmanager
 	// to filter all jobs by
-static MyString expr_schedd_job_constraint;
+static std::string expr_schedd_job_constraint;
 	// The job is marked as completely done by
 	// the gridmanager.  The gridmanager should
 	// never try to manage the job again
-static MyString expr_completely_done;
+static std::string expr_completely_done;
 	// Opposite of expr_completely_done
-static MyString expr_not_completely_done;
+static std::string expr_not_completely_done;
+	// Whether the job has dirty attributes
 
 static void 
 initJobExprs()
@@ -525,21 +546,21 @@ initJobExprs()
 	static bool done = false;
 	if(done) { return; }
 
-	expr_matched_or_undef.sprintf("(%s =!= %s)", ATTR_JOB_MATCHED, expr_false);
-	expr_managed.sprintf("(%s =?= \"%s\")", ATTR_JOB_MANAGED, MANAGED_EXTERNAL);
-	expr_not_managed.sprintf("(%s =!= \"%s\")", ATTR_JOB_MANAGED, MANAGED_EXTERNAL);
-	expr_not_held.sprintf("(%s != %d)", ATTR_JOB_STATUS, HELD);
-	expr_schedd_job_constraint.sprintf("(%s)", ScheddJobConstraint);
+	sprintf(expr_matched_or_undef, "(%s =!= %s)", ATTR_JOB_MATCHED, expr_false);
+	sprintf(expr_managed, "(%s =?= \"%s\")", ATTR_JOB_MANAGED, MANAGED_EXTERNAL);
+	sprintf(expr_not_managed, "(%s =!= \"%s\")", ATTR_JOB_MANAGED, MANAGED_EXTERNAL);
+	sprintf(expr_not_held, "(%s != %d)", ATTR_JOB_STATUS, HELD);
+	sprintf(expr_schedd_job_constraint, "(%s)", ScheddJobConstraint);
 	// The gridmanager never wants to see this job again.
 	// It should be in the process of leaving the queue.
-	expr_completely_done.sprintf("(%s =?= \"%s\")", ATTR_JOB_MANAGED, MANAGED_DONE);
-	expr_not_completely_done.sprintf("(%s =!= \"%s\")", ATTR_JOB_MANAGED, MANAGED_DONE);
+	sprintf(expr_completely_done, "(%s =?= \"%s\")", ATTR_JOB_MANAGED, MANAGED_DONE);
+	sprintf(expr_not_completely_done, "(%s =!= \"%s\")", ATTR_JOB_MANAGED, MANAGED_DONE);
 
 	done = true;
 }
 
-int
-CHECK_LEASES_signalHandler( int )
+void
+CHECK_LEASES_signalHandler()
 {
 	dprintf(D_FULLDEBUG,"Received CHECK_LEASES signal\n");
 
@@ -547,11 +568,9 @@ CHECK_LEASES_signalHandler( int )
 		RequestContactSchedd();
 		checkLeasesSignaled = true;
 	}
-
-	return TRUE;
 }
 
-int
+void
 doContactSchedd()
 {
 	int rc;
@@ -562,10 +581,15 @@ doContactSchedd()
 	bool schedd_updates_complete = false;
 	bool schedd_deletes_complete = false;
 	bool add_remove_jobs_complete = false;
+	bool update_jobs_complete = false;
 	bool commit_transaction = true;
 	int failure_line_num = 0;
 	bool send_reschedule = false;
-	MyString error_str = "";
+	std::string error_str = "";
+	StringList dirty_job_ids;
+	char *job_id_str;
+	PROC_ID job_id;
+	CondorError errstack;
 
 	dprintf(D_FULLDEBUG,"in doContactSchedd()\n");
 
@@ -576,19 +600,18 @@ doContactSchedd()
 	// vacateJobs
 	/////////////////////////////////////////////////////
 	if ( pendingScheddVacates.getNumElements() != 0 ) {
-		MyString buff;
+		std::string buff;
 		StringList job_ids;
 		VacateRequest curr_request;
 
 		int result;
-		CondorError errstack;
 		ClassAd* rval;
 
 		pendingScheddVacates.startIterations();
 		while ( pendingScheddVacates.iterate( curr_request ) != 0 ) {
-			buff.sprintf( "%d.%d", curr_request.job->procID.cluster,
+			sprintf( buff, "%d.%d", curr_request.job->procID.cluster,
 						  curr_request.job->procID.proc );
-			job_ids.append( buff.Value() );
+			job_ids.append( buff.c_str() );
 		}
 
 		char *tmp = job_ids.print_to_string();
@@ -600,15 +623,15 @@ doContactSchedd()
 
 		rval = ScheddObj->vacateJobs( &job_ids, VACATE_FAST, &errstack );
 		if ( rval == NULL ) {
-			error_str.sprintf( "vacateJobs returned NULL, CondorError: %s!",
+			sprintf( error_str, "vacateJobs returned NULL, CondorError: %s!",
 							   errstack.getFullText() );
 			goto contact_schedd_failure;
 		} else {
 			pendingScheddVacates.startIterations();
 			while ( pendingScheddVacates.iterate( curr_request ) != 0 ) {
-				buff.sprintf( "job_%d_%d", curr_request.job->procID.cluster,
+				sprintf( buff, "job_%d_%d", curr_request.job->procID.cluster,
 							  curr_request.job->procID.proc );
-				if ( !rval->LookupInteger( buff.Value(), result ) ) {
+				if ( !rval->LookupInteger( buff.c_str(), result ) ) {
 					dprintf( D_FULLDEBUG, "vacateJobs returned malformed ad\n" );
 					EXCEPT( "vacateJobs returned malformed ad" );
 				} else {
@@ -627,9 +650,9 @@ doContactSchedd()
 	}
 
 
-	schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, false );
+	schedd = ConnectQ( ScheddAddr, QMGMT_TIMEOUT, false, NULL, myUserName, CondorVersion() );
 	if ( !schedd ) {
-		error_str.sprintf( "Failed to connect to schedd!" );
+		error_str = "Failed to connect to schedd!";
 		goto contact_schedd_failure;
 	}
 
@@ -699,21 +722,21 @@ doContactSchedd()
 			// superfluous (by boolean logic), so we drop it.
 			sprintf( expr_buf,
 					 "%s && %s && ((%s && %s) || %s)",
-					 expr_schedd_job_constraint.Value(), 
-					 expr_not_completely_done.Value(),
-					 expr_matched_or_undef.Value(),
-					 expr_not_held.Value(),
-					 expr_managed.Value()
+					 expr_schedd_job_constraint.c_str(), 
+					 expr_not_completely_done.c_str(),
+					 expr_matched_or_undef.c_str(),
+					 expr_not_held.c_str(),
+					 expr_managed.c_str()
 					 );
 		} else {
 			// Grab new jobs for us to manage
 			sprintf( expr_buf,
 					 "%s && %s && %s && %s && %s",
-					 expr_schedd_job_constraint.Value(), 
-					 expr_not_completely_done.Value(),
-					 expr_matched_or_undef.Value(),
-					 expr_not_held.Value(),
-					 expr_not_managed.Value()
+					 expr_schedd_job_constraint.c_str(), 
+					 expr_not_completely_done.c_str(),
+					 expr_matched_or_undef.c_str(),
+					 expr_not_held.c_str(),
+					 expr_not_managed.c_str()
 					 );
 		}
 		dprintf( D_FULLDEBUG,"Using constraint %s\n",expr_buf);
@@ -840,7 +863,7 @@ contact_schedd_next_add_job:
 		// haven't previously indicated that we're done with (by setting
 		// JobManaged to "Schedd".
 		sprintf( expr_buf, "(%s) && (%s) && (%s == %d || %s == %d || (%s == %d && %s =?= \"%s\"))",
-				 ScheddJobConstraint, expr_not_completely_done.Value(),
+				 ScheddJobConstraint, expr_not_completely_done.c_str(),
 				 ATTR_JOB_STATUS, REMOVED,
 				 ATTR_JOB_STATUS, COMPLETED, ATTR_JOB_STATUS, HELD,
 				 ATTR_JOB_MANAGED, MANAGED_EXTERNAL );
@@ -860,7 +883,7 @@ contact_schedd_next_add_job:
 				// Should probably skip jobs we already have marked as
 				// held or removed
 
-				next_job->JobAdUpdateFromSchedd( next_ad );
+				next_job->JobAdUpdateFromSchedd( next_ad, true );
 				num_ads++;
 
 			} else if ( curr_status == REMOVED ) {
@@ -906,7 +929,7 @@ contact_schedd_next_add_job:
 		dprintf(D_FULLDEBUG,"Fetched %d job ads from schedd\n",num_ads);
 	}
 
-	if ( CloseConnection() < 0 ) {
+	if ( RemoteCommitTransaction() < 0 ) {
 		failure_line_num = __LINE__;
 		commit_transaction = false;
 		goto contact_schedd_disconnect;
@@ -914,6 +937,51 @@ contact_schedd_next_add_job:
 
 	add_remove_jobs_complete = true;
 
+
+	// Retrieve dirty attributes
+	/////////////////////////////////////////////////////
+	if ( updateJobsSignaled ) {
+		dprintf( D_FULLDEBUG, "querying for jobs with attribute updates\n" );
+
+		sprintf( expr_buf, "%s && %s && %s && %s",
+				 expr_schedd_job_constraint.c_str(), 
+				 expr_not_completely_done.c_str(),
+				 expr_not_held.c_str(),
+				 expr_managed.c_str()
+				 );
+		dprintf( D_FULLDEBUG,"Using constraint %s\n",expr_buf);
+		next_ad = GetNextDirtyJobByConstraint( expr_buf, 1 );
+		while ( next_ad != NULL ) {
+			ClassAd updates;
+			char str[PROC_ID_STR_BUFLEN];
+			next_ad->LookupInteger( ATTR_CLUSTER_ID, job_id.cluster );
+			next_ad->LookupInteger( ATTR_PROC_ID, job_id.proc );
+			if ( GetDirtyAttributes( job_id.cluster, job_id.proc, &updates ) < 0 ) {
+				dprintf( D_ALWAYS, "Failed to retrieve dirty attributes for job %d.%d\n", job_id.cluster, job_id.proc );
+				failure_line_num = __LINE__;
+				delete next_ad;
+				goto contact_schedd_disconnect;
+		        }
+			else {
+				dprintf (D_FULLDEBUG, "Retrieved updated attributes for job %d.%d\n", job_id.cluster, job_id.proc);
+				updates.dPrint(D_JOB);
+			}
+			if ( BaseJob::JobsByProcId.lookup( job_id, curr_job ) == 0 ) {
+				curr_job->JobAdUpdateFromSchedd( &updates, false );
+				ProcIdToStr( job_id, str );
+				dirty_job_ids.append( str );
+			}
+			else {
+				dprintf( D_ALWAYS, "Don't know about updated job %d.%d. "
+						 "Ignoring it\n",
+						 job_id.cluster, job_id.proc );
+			}
+			delete next_ad;
+			next_ad = GetNextDirtyJobByConstraint( expr_buf, 0 );
+		}
+	}
+	update_jobs_complete = true;
+
 //	if ( BeginTransaction() < 0 ) {
 	errno = 0;
 	BeginTransaction();
@@ -927,7 +995,6 @@ contact_schedd_next_add_job:
 	// requestJobStatus
 	/////////////////////////////////////////////////////
 	if ( pendingJobStatus.getNumElements() != 0 ) {
-		MyString buff;
 		JobStatusRequest curr_request;
 
 		pendingJobStatus.startIterations();
@@ -966,23 +1033,22 @@ contact_schedd_next_add_job:
 
 	// Update existing jobs
 	/////////////////////////////////////////////////////
+	ScheddUpdateRequest *curr_request;
 	pendingScheddUpdates.startIterations();
 
-	while ( pendingScheddUpdates.iterate( curr_job ) != 0 ) {
+	while ( pendingScheddUpdates.iterate( curr_request ) != 0 ) {
 
+		curr_job = curr_request->m_job;
 		dprintf(D_FULLDEBUG,"Updating classad values for %d.%d:\n",
 				curr_job->procID.cluster, curr_job->procID.proc);
-		char attr_name[1024];
-		char attr_value[1024];
+		const char *attr_name;
+		const char *attr_value;
 		ExprTree *expr;
 		bool fake_job_in_queue = false;
 		curr_job->jobAd->ResetExpr();
-		while ( (expr = curr_job->jobAd->NextDirtyExpr()) != NULL &&
+		while ( curr_job->jobAd->NextDirtyExpr(attr_name, expr) == true &&
 				fake_job_in_queue == false ) {
-			attr_name[0] = '\0';
-			attr_value[0] = '\0';
-			expr->LArg()->PrintToStr(attr_name);
-			expr->RArg()->PrintToStr(attr_value);
+			attr_value = ExprTreeToString( expr );
 
 			dprintf(D_FULLDEBUG,"   %s = %s\n",attr_name,attr_value);
 			rc = SetAttribute( curr_job->procID.cluster,
@@ -1010,7 +1076,7 @@ contact_schedd_next_add_job:
 
 	}
 
-	if ( CloseConnection() < 0 ) {
+	if ( RemoteCommitTransaction() < 0 ) {
 		failure_line_num = __LINE__;
 		commit_transaction = false;
 		goto contact_schedd_disconnect;
@@ -1031,8 +1097,9 @@ contact_schedd_next_add_job:
 
 	pendingScheddUpdates.startIterations();
 
-	while ( pendingScheddUpdates.iterate( curr_job ) != 0 ) {
+	while ( pendingScheddUpdates.iterate( curr_request ) != 0 ) {
 
+		curr_job = curr_request->m_job;
 		if ( curr_job->deleteFromSchedd ) {
 			dprintf(D_FULLDEBUG,"Deleting job %d.%d from schedd\n",
 					curr_job->procID.cluster, curr_job->procID.proc);
@@ -1048,7 +1115,7 @@ contact_schedd_next_add_job:
 
 	}
 
-	if ( CloseConnection() < 0 ) {
+	if ( RemoteCommitTransaction() < 0 ) {
 		failure_line_num = __LINE__;
 		commit_transaction = false;
 		goto contact_schedd_disconnect;
@@ -1064,21 +1131,42 @@ contact_schedd_next_add_job:
 		firstScheddContact = false;
 		addJobsSignaled = false;
 	} else {
-		error_str.sprintf( "Schedd connection error during Add/RemoveJobs at line %d!", failure_line_num );
+		sprintf( error_str, "Schedd connection error during Add/RemoveJobs at line %d!", failure_line_num );
+		goto contact_schedd_failure;
+	}
+
+	if ( update_jobs_complete == true ) {
+		updateJobsSignaled = false;
+	} else {
+		sprintf( error_str, "Schedd connection error during dirty attribute update at line %d!", failure_line_num );
 		goto contact_schedd_failure;
 	}
 
 	if ( schedd_updates_complete == false ) {
-		error_str.sprintf( "Schedd connection error during updates at line %d!", failure_line_num );
+		sprintf( error_str, "Schedd connection error during updates at line %d!", failure_line_num );
 		goto contact_schedd_failure;
 	}
 
+	// Clear dirty bits for all jobs updated
+	if ( !dirty_job_ids.isEmpty() ) {
+		ClassAd *rval;
+		dprintf( D_FULLDEBUG, "Calling clearDirtyAttrs on %d jobs\n",
+				 dirty_job_ids.number() );
+		dirty_job_ids.rewind();
+		rval = ScheddObj->clearDirtyAttrs( &dirty_job_ids, &errstack );
+		if ( rval == NULL ) {
+			dprintf(D_ALWAYS, "Failed to notify schedd to clear dirty attributes.  CondorError: %s\n", errstack.getFullText() );
+		}
+		delete rval;
+	}
+
 	// Wake up jobs that had schedd updates pending and delete job
 	// objects that wanted to be deleted
 	pendingScheddUpdates.startIterations();
 
-	while ( pendingScheddUpdates.iterate( curr_job ) != 0 ) {
+	while ( pendingScheddUpdates.iterate( curr_request ) != 0 ) {
 
+		curr_job = curr_request->m_job;
 		curr_job->jobAd->ClearAllDirtyFlags();
 
 		if ( curr_job->deleteFromGridmanager ) {
@@ -1106,9 +1194,12 @@ contact_schedd_next_add_job:
 		} else {
 			pendingScheddUpdates.remove( curr_job->procID );
 
-			curr_job->SetEvaluateState();
+			if ( curr_request->m_notify ) {
+				curr_job->SetEvaluateState();
+			}
 		}
 
+		delete curr_request;
 	}
 
 	// Poke objects that wanted to be notified when a schedd update completed
@@ -1133,14 +1224,23 @@ contact_schedd_next_add_job:
 	lastContactSchedd = time(NULL);
 
 	if ( schedd_deletes_complete == false ) {
-		error_str.sprintf( "Problem using DestroyProc to delete jobs!" );
+		error_str = "Problem using DestroyProc to delete jobs!";
 		goto contact_schedd_failure;
 	}
 
 	scheddFailureCount = 0;
 
+	// For each job that had dirty attributes, re-evaluate the policy
+	dirty_job_ids.rewind();
+	while ( (job_id_str = dirty_job_ids.next()) != NULL ) {
+		StrToProcId(job_id_str, job_id);
+		if ( BaseJob::JobsByProcId.lookup( job_id, curr_job ) == 0 ) {
+			curr_job->EvalPeriodicJobExpr();
+		}
+	}
+
 dprintf(D_FULLDEBUG,"leaving doContactSchedd()\n");
-	return TRUE;
+	return;
 
  contact_schedd_failure:
 	scheddFailureCount++;
@@ -1148,12 +1248,13 @@ dprintf(D_FULLDEBUG,"leaving doContactSchedd()\n");
 		error_str = "Failure in doContactSchedd";
 	}
 	if ( scheddFailureCount >= maxScheddFailures ) {
-		dprintf( D_ALWAYS, "%s\n", error_str.Value() );
+		dprintf( D_ALWAYS, "%s\n", error_str.c_str() );
 		EXCEPT( "Too many failures connecting to schedd!" );
 	}
-	dprintf( D_ALWAYS, "%s Will retry\n", error_str.Value() );
+	dprintf( D_ALWAYS, "%s Will retry\n", error_str.c_str() );
 	lastContactSchedd = time(NULL);
 	RequestContactSchedd();
-	return TRUE;
+	return;
 }
 
+
