@@ -27,7 +27,6 @@
 #include "gt4job.h"
 #include "gridmanager.h"
 
-#define DEFAULT_MAX_PENDING_SUBMITS_PER_RESOURCE	5
 #define DEFAULT_MAX_SUBMITTED_JOBS_PER_RESOURCE		100
 #define DEFAULT_MAX_WS_DESTROYS_PER_RESOURCE		5
 
@@ -51,11 +50,11 @@ struct GT4ProxyDelegation {
 	time_t proxy_expire;
 	time_t last_proxy_refresh;
 	Proxy *proxy;
-	MyString error_message;
+	std::string error_message;
 };
 
 GT4Resource *GT4Resource::FindOrCreateResource( const char *resource_name,
-												const char *proxy_subject )
+												const Proxy *proxy )
 {
 	int rc;
 	GT4Resource *resource = NULL;
@@ -63,12 +62,12 @@ GT4Resource *GT4Resource::FindOrCreateResource( const char *resource_name,
 	const char *canonical_name = CanonicalName( resource_name );
 	ASSERT(canonical_name);
 
-	const char *hash_name = HashName( canonical_name, proxy_subject );
+	const char *hash_name = HashName( canonical_name, proxy->subject->fqan );
 	ASSERT(hash_name);
 
 	rc = ResourcesByName.lookup( HashKey( hash_name ), resource );
 	if ( rc != 0 ) {
-		resource = new GT4Resource( canonical_name, proxy_subject );
+		resource = new GT4Resource( canonical_name, proxy );
 		ASSERT(resource);
 		if ( resource->Init() == false ) {
 			delete resource;
@@ -84,11 +83,12 @@ GT4Resource *GT4Resource::FindOrCreateResource( const char *resource_name,
 }
 
 GT4Resource::GT4Resource( const char *resource_name,
-						  const char *proxy_subject )
+						  const Proxy *proxy )
 	: BaseResource( resource_name )
 {
 	initialized = false;
-	proxySubject = strdup( proxy_subject );
+	proxySubject = strdup( proxy->subject->subject_name );
+	proxyFQAN = strdup( proxy->subject->fqan );
 	delegationTimerId = daemonCore->Register_Timer( 0,
 							(TimerHandlercpp)&GT4Resource::checkDelegation,
 							"GT4Resource::checkDelegation", (Service*)this );
@@ -96,8 +96,6 @@ GT4Resource::GT4Resource( const char *resource_name,
 	delegationServiceUri = NULL;
 	gahp = NULL;
 	deleg_gahp = NULL;
-	submitLimit = DEFAULT_MAX_PENDING_SUBMITS_PER_RESOURCE;
-	jobLimit = DEFAULT_MAX_SUBMITTED_JOBS_PER_RESOURCE;
 	m_isGram42 = false;
 
 	const char service_name[] = "/wsrf/services/DelegationFactoryService";
@@ -127,8 +125,8 @@ dprintf(D_FULLDEBUG,"*** ~GT4Resource\n");
 dprintf(D_FULLDEBUG,"    deleting %s\n",next_deleg->deleg_uri);
 		delegatedProxies.DeleteCurrent();
 		free( next_deleg->deleg_uri );
-			// unacquire proxy?
-		ReleaseProxy( next_deleg->proxy );
+		ReleaseProxy( next_deleg->proxy,
+					  (TimerHandlercpp)&GT4Resource::ProxyCallback, this );
 		delete next_deleg;
 	}
 	if ( delegationServiceUri != NULL ) {
@@ -147,6 +145,7 @@ dprintf(D_FULLDEBUG,"    deleting %s\n",next_deleg->deleg_uri);
 	if ( proxySubject ) {
 		free( proxySubject );
 	}
+	free( proxyFQAN );
 }
 
 bool GT4Resource::Init()
@@ -158,16 +157,16 @@ bool GT4Resource::Init()
 		// TODO This assumes that at least one GT4Job has already
 		// instantiated a GahpClient. Need a better solution.
 /*
-	MyString gahp_name;
-	gahp_name.sprintf( "GT4/%s", proxySubject );
+	std::string gahp_name;
+	sprintf( gahp_name, "GT4/%s", proxyFQAN );
 
-	gahp = new GahpClient( gahp_name.Value() );
+	gahp = new GahpClient( gahp_name.c_str() );
 
 	gahp->setNotificationTimerId( pingTimerId );
 	gahp->setMode( GahpClient::normal );
 	gahp->setTimeout( gahpCallTimeout );
 
-	deleg_gahp = new GahpClient( gahp_name.Value() );
+	deleg_gahp = new GahpClient( gahp_name.c_str() );
 
 	deleg_gahp->setNotificationTimerId( delegationTimerId );
 	deleg_gahp->setMode( GahpClient::normal );
@@ -215,18 +214,18 @@ const char *GT4Resource::ResourceType()
 const char *GT4Resource::CanonicalName( const char *name )
 {
 /*
-	static MyString canonical;
+	static std::string canonical;
 	char *host;
 	char *port;
 
 	parse_resource_manager_string( name, &host, &port, NULL, NULL );
 
-	canonical.sprintf( "%s:%s", host, *port ? port : "2119" );
+	sprintf( canonical, "%s:%s", host, *port ? port : "2119" );
 
 	free( host );
 	free( port );
 
-	return canonical.Value();
+	return canonical.c_str();
 */
 	return name;
 }
@@ -234,11 +233,24 @@ const char *GT4Resource::CanonicalName( const char *name )
 const char *GT4Resource::HashName( const char *resource_name,
 								   const char *proxy_subject )
 {
-	static MyString hash_name;
+	static std::string hash_name;
 
-	hash_name.sprintf( "%s#%s", resource_name, proxy_subject );
+	sprintf( hash_name, "gt4 %s#%s", resource_name, proxy_subject );
 
-	return hash_name.Value();
+	return hash_name.c_str();
+}
+
+const char *GT4Resource::GetHashName()
+{
+	return HashName( resourceName, proxyFQAN );
+}
+
+void GT4Resource::PublishResourceAd( ClassAd *resource_ad )
+{
+	BaseResource::PublishResourceAd( resource_ad );
+
+	resource_ad->Assign( ATTR_X509_USER_PROXY_SUBJECT, proxySubject );
+	resource_ad->Assign( ATTR_X509_USER_PROXY_FQAN, proxyFQAN );
 }
 
 void GT4Resource::UnregisterJob( GT4Job *job )
@@ -256,8 +268,6 @@ void GT4Resource::UnregisterJob( GT4Job *job )
 		}
 		if ( delete_deleg ) {
 dprintf(D_FULLDEBUG,"*** deleting delegation %s\n",job->delegatedCredentialURI);
-			bool reacquire_proxy = false;
-			Proxy *proxy_to_reacquire = job->jobProxy;
 			GT4ProxyDelegation *next_deleg;
 			delegatedProxies.Rewind();
 			while ( (next_deleg = delegatedProxies.Next()) != NULL ) {
@@ -269,16 +279,11 @@ dprintf(D_FULLDEBUG,"*** deleting delegation %s\n",job->delegatedCredentialURI);
 						activeDelegationCmd = NULL;
 					}
 					free( next_deleg->deleg_uri );
-						// unacquire proxy?
-					ReleaseProxy( next_deleg->proxy );
+					ReleaseProxy( next_deleg->proxy,
+								  (TimerHandlercpp)&GT4Resource::ProxyCallback, this );
 					delete next_deleg;
-				} else if ( next_deleg->proxy == proxy_to_reacquire ) {
-					reacquire_proxy = true;
 				}
 			}
-			if ( reacquire_proxy ) {
-				AcquireProxy( proxy_to_reacquire, delegationTimerId );
-			}
 		}
 	}
 
@@ -309,8 +314,7 @@ dprintf(D_FULLDEBUG,"    creating new GT4ProxyDelegation\n");
 	next_deleg->last_lifetime_extend = 0;
 	next_deleg->last_proxy_refresh = 0;
 	next_deleg->proxy = job_proxy;
-		// acquire proxy?
-	AcquireProxy( job_proxy, delegationTimerId );
+	AcquireProxy( job_proxy, (TimerHandlercpp)&GT4Resource::ProxyCallback, this );
 	delegatedProxies.Append( next_deleg );
 
 		// TODO add smarter timer that delays a few seconds
@@ -341,8 +345,7 @@ dprintf(D_FULLDEBUG,"    creating new GT4ProxyDelegation\n");
 	next_deleg->last_lifetime_extend = 0;
 	next_deleg->last_proxy_refresh = 0;
 	next_deleg->proxy = job_proxy;
-		// acquire proxy?
-	AcquireProxy( job_proxy, delegationTimerId );
+	AcquireProxy( job_proxy, (TimerHandlercpp)&GT4Resource::ProxyCallback, this );
 	delegatedProxies.Append( next_deleg );
 
 		// TODO add smarter timer that delays a few seconds
@@ -359,10 +362,10 @@ const char *GT4Resource::getDelegationError( Proxy *job_proxy )
 
 	while ( ( next_deleg = delegatedProxies.Next() ) != NULL ) {
 		if ( next_deleg->proxy == job_proxy ) {
-			if ( next_deleg->error_message.IsEmpty() ) {
+			if ( next_deleg->error_message.empty() ) {
 				return NULL;
 			} else {
-				return next_deleg->error_message.Value();
+				return next_deleg->error_message.c_str();
 			}
 		}
 	}
@@ -372,7 +375,13 @@ const char *GT4Resource::getDelegationError( Proxy *job_proxy )
 	return NULL;
 }
 
-int GT4Resource::checkDelegation()
+int GT4Resource::ProxyCallback()
+{
+	daemonCore->Reset_Timer( delegationTimerId, 0 );
+	return 0;
+}
+
+void GT4Resource::checkDelegation()
 {
 dprintf(D_FULLDEBUG,"*** checkDelegation()\n");
 	bool signal_jobs;
@@ -382,13 +391,13 @@ dprintf(D_FULLDEBUG,"*** checkDelegation()\n");
 	if ( deleg_gahp->isInitialized() == false ) {
 		dprintf( D_ALWAYS,"gahp server not up yet, delaying checkDelegation\n" );
 		daemonCore->Reset_Timer( delegationTimerId, 5 );
-		return 0;
+		return;
 	}
 
 	daemonCore->Reset_Timer( delegationTimerId, CHECK_DELEGATION_INTERVAL );
 
 	if ( resourceDown || !firstPingDone ) {
-		return 0;
+		return;
 	}
 
 	delegatedProxies.Rewind();
@@ -410,7 +419,7 @@ dprintf(D_FULLDEBUG,"    new delegation\n");
 														&delegation_uri );
 			if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
 				activeDelegationCmd = next_deleg;
-				return 0;
+				return;
 			}
 			if ( rc != 0 ) {
 					// Failure, what to do?
@@ -455,7 +464,7 @@ dprintf(D_FULLDEBUG,"    refreshing %s\n",next_deleg->deleg_uri);
 													next_deleg->deleg_uri );
 			if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
 				activeDelegationCmd = next_deleg;
-				return 0;
+				return;
 			}
 			next_deleg->last_proxy_refresh = now;
 			if ( rc != 0 ) {
@@ -485,7 +494,7 @@ dprintf(D_FULLDEBUG,"    extending %s\n",next_deleg->deleg_uri);
 													   new_lifetime );
 			if ( rc == GAHPCLIENT_COMMAND_PENDING ) {
 				activeDelegationCmd = next_deleg;
-				return 0;
+				return;
 			}
 			next_deleg->last_lifetime_extend = now;
 			if ( rc != 0 ) {
@@ -521,7 +530,6 @@ dprintf(D_FULLDEBUG,"    signalling jobs for %s\n",next_deleg->deleg_uri?next_de
 			}
 		}
 	}
-	return 0;
 }
 
 bool GT4Resource::RequestDestroy( GT4Job *job )
@@ -665,20 +673,20 @@ dprintf(D_ALWAYS,"JEF: Telling jobs to switch to Gram 42\n");
 		}
 	}
 
-	MyString gahp_name;
+	std::string gahp_name;
 	if ( m_isGram42 ) {
-		gahp_name.sprintf( "GT42/%s", proxySubject );
+		sprintf( gahp_name, "GT42/%s", proxyFQAN );
 	} else {
-		gahp_name.sprintf( "GT4/%s", proxySubject );
+		sprintf( gahp_name, "GT4/%s", proxyFQAN );
 	}
 
-	new_gahp = new GahpClient( gahp_name.Value() );
+	new_gahp = new GahpClient( gahp_name.c_str() );
 
 	new_gahp->setNotificationTimerId( pingTimerId );
 	new_gahp->setMode( GahpClient::normal );
 	new_gahp->setTimeout( gahpCallTimeout );
 
-	new_deleg_gahp = new GahpClient( gahp_name.Value() );
+	new_deleg_gahp = new GahpClient( gahp_name.c_str() );
 
 	new_deleg_gahp->setNotificationTimerId( delegationTimerId );
 	new_deleg_gahp->setMode( GahpClient::normal );
