@@ -24,7 +24,7 @@
 #include "condor_debug.h"
 #include "condor_string.h"	// for strnewp and friends
 #include "condor_daemon_core.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "daemon.h"
 #include "dc_schedd.h"
 #include "setenv.h"
@@ -76,7 +76,7 @@ static const char *GMStateNames[] = {
 };
 
 #define JOB_STATE_UNKNOWN				-1
-#define JOB_STATE_UNSUBMITTED			UNEXPANDED
+#define JOB_STATE_UNSUBMITTED			0
 
 // TODO: Let the maximum submit attempts be set in the job ad or, better yet,
 // evalute PeriodicHold expression in job ad.
@@ -109,7 +109,7 @@ void INFNBatchJobReconfig()
 
 bool INFNBatchJobAdMatch( const ClassAd *job_ad ) {
 	int universe;
-	MyString resource;
+	std::string resource;
 		// CRUFT: grid-type 'blah' is deprecated. Now, the specific batch
 		//   system names should be used (pbs, lsf). Glite are the only
 		//   people who care about the old value. This changed happend in
@@ -119,11 +119,11 @@ bool INFNBatchJobAdMatch( const ClassAd *job_ad ) {
 	if ( job_ad->LookupInteger( ATTR_JOB_UNIVERSE, universe ) &&
 		 universe == CONDOR_UNIVERSE_GRID &&
 		 job_ad->LookupString( ATTR_GRID_RESOURCE, resource ) &&
-		 ( strncasecmp( resource.Value(), "blah", 4 ) == 0 ||
-		   strncasecmp( resource.Value(), "pbs", 4 ) == 0 ||
-		   strncasecmp( resource.Value(), "lsf", 4 ) == 0 ||
-		   strncasecmp( resource.Value(), "nqs", 4 ) == 0 ||
-		   strncasecmp( resource.Value(), "naregi", 6 ) == 0 ) ) {
+		 ( strncasecmp( resource.c_str(), "blah", 4 ) == 0 ||
+		   strncasecmp( resource.c_str(), "pbs", 4 ) == 0 ||
+		   strncasecmp( resource.c_str(), "lsf", 4 ) == 0 ||
+		   strncasecmp( resource.c_str(), "nqs", 4 ) == 0 ||
+		   strncasecmp( resource.c_str(), "naregi", 6 ) == 0 ) ) {
 
 		return true;
 	}
@@ -145,7 +145,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 	: BaseJob( classad )
 {
 	char buff[4096];
-	MyString error_string = "";
+	std::string error_string = "";
 	char *gahp_path;
 
 	gahpAd = NULL;
@@ -173,7 +173,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 	if ( buff[0] != '\0' ) {
 		batchType = strdup( buff );
 	} else {
-		error_string.sprintf( "%s is not set in the job ad",
+		sprintf( error_string, "%s is not set in the job ad",
 							  ATTR_GRID_RESOURCE );
 		goto error_exit;
 	}
@@ -191,7 +191,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 	sprintf( buff, "%s_GAHP", batchType );
 	gahp_path = param(buff);
 	if ( gahp_path == NULL ) {
-		error_string.sprintf( "%s not defined", buff );
+		sprintf( error_string, "%s not defined", buff );
 		goto error_exit;
 	}
 	gahp = new GahpClient( batchType, gahp_path );
@@ -204,7 +204,8 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 	gahp->setMode( GahpClient::normal );
 	gahp->setTimeout( gahpCallTimeout );
 
-	jobProxy = AcquireProxy( jobAd, error_string, evaluateStateTid );
+	jobProxy = AcquireProxy( jobAd, error_string,
+							 (TimerHandlercpp)&BaseJob::SetEvaluateState, this );
 	if ( jobProxy == NULL && error_string != "" ) {
 		goto error_exit;
 	}
@@ -215,8 +216,8 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 		// We must ensure that the code-path from GM_HOLD doesn't depend
 		// on any initialization that's been skipped.
 	gmState = GM_HOLD;
-	if ( !error_string.IsEmpty() ) {
-		jobAd->Assign( ATTR_HOLD_REASON, error_string.Value() );
+	if ( !error_string.empty() ) {
+		jobAd->Assign( ATTR_HOLD_REASON, error_string.c_str() );
 	}
 	return;
 }
@@ -224,7 +225,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 INFNBatchJob::~INFNBatchJob()
 {
 	if ( jobProxy != NULL ) {
-		ReleaseProxy( jobProxy, evaluateStateTid );
+		ReleaseProxy( jobProxy, (TimerHandlercpp)&BaseJob::SetEvaluateState, this );
 	}
 	if ( batchType != NULL ) {
 		free( batchType );
@@ -246,14 +247,15 @@ void INFNBatchJob::Reconfig()
 	gahp->setTimeout( gahpCallTimeout );
 }
 
-int INFNBatchJob::doEvaluateState()
+void INFNBatchJob::doEvaluateState()
 {
 	int old_gm_state;
 	int old_remote_state;
 	bool reevaluate_state = true;
 	time_t now = time(NULL);
 
-	bool done;
+	bool attr_exists;
+	bool attr_dirty;
 	int rc;
 
 	daemonCore->Reset_Timer( evaluateStateTid, TIMER_NEVER );
@@ -402,8 +404,9 @@ int INFNBatchJob::doEvaluateState()
 			if ( condorState == REMOVED || condorState == HELD ) {
 				gmState = GM_CANCEL;
 			} else {
-				done = requestScheddUpdate( this );
-				if ( !done ) {
+				jobAd->GetDirtyFlag( ATTR_GRID_JOB_ID, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 				gmState = GM_SUBMITTED;
@@ -498,8 +501,9 @@ int INFNBatchJob::doEvaluateState()
 			// Report job completion to the schedd.
 			JobTerminated();
 			if ( condorState == COMPLETED ) {
-				done = requestScheddUpdate( this );
-				if ( !done ) {
+				jobAd->GetDirtyFlag( ATTR_JOB_STATUS, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 			}
@@ -515,7 +519,7 @@ int INFNBatchJob::doEvaluateState()
 				// Clear the contact string here because it may not get
 				// cleared in GM_CLEAR_REQUEST (it might go to GM_HOLD first).
 				SetRemoteJobId( NULL );
-				requestScheddUpdate( this );
+				requestScheddUpdate( this, false );
 				gmState = GM_CLEAR_REQUEST;
 			}
 			} break;
@@ -582,8 +586,11 @@ int INFNBatchJob::doEvaluateState()
 			// through. However, since we registered update events the
 			// first time, requestScheddUpdate won't return done until
 			// they've been committed to the schedd.
-			done = requestScheddUpdate( this );
-			if ( !done ) {
+			const char *name;
+			ExprTree *expr;
+			jobAd->ResetExpr();
+			if ( jobAd->NextDirtyExpr(name, expr) ) {
+				requestScheddUpdate( this, true );
 				break;
 			}
 			remoteProxyExpireTime = 0;
@@ -613,7 +620,7 @@ int INFNBatchJob::doEvaluateState()
 				jobAd->LookupString( ATTR_HOLD_REASON, holdReason,
 									 sizeof(holdReason) - 1 );
 				if ( holdReason[0] == '\0' && errorString != "" ) {
-					strncpy( holdReason, errorString.Value(),
+					strncpy( holdReason, errorString.c_str(),
 							 sizeof(holdReason) - 1 );
 				}
 				if ( holdReason[0] == '\0' ) {
@@ -659,8 +666,6 @@ int INFNBatchJob::doEvaluateState()
 		}
 
 	} while ( reevaluate_state );
-
-	return TRUE;
 }
 
 void INFNBatchJob::SetRemoteJobId( const char *job_id )
@@ -672,17 +677,16 @@ void INFNBatchJob::SetRemoteJobId( const char *job_id )
 		remoteJobId = NULL;
 	}
 
-	MyString full_job_id;
+	std::string full_job_id;
 	if ( job_id ) {
-		full_job_id.sprintf( "%s %s", batchType, job_id );
+		sprintf( full_job_id, "%s %s", batchType, job_id );
 	}
-	BaseJob::SetRemoteJobId( full_job_id.Value() );
+	BaseJob::SetRemoteJobId( full_job_id.c_str() );
 }
 
 void INFNBatchJob::ProcessRemoteAd( ClassAd *remote_ad )
 {
 	int new_remote_state;
-	MyString buff;
 	ExprTree *new_expr, *old_expr;
 
 	int index;
@@ -753,15 +757,15 @@ void INFNBatchJob::ProcessRemoteAd( ClassAd *remote_ad )
 
 	index = -1;
 	while ( attrs_to_copy[++index] != NULL ) {
-		old_expr = jobAd->Lookup( attrs_to_copy[index] );
-		new_expr = remote_ad->Lookup( attrs_to_copy[index] );
+		old_expr = jobAd->LookupExpr( attrs_to_copy[index] );
+		new_expr = remote_ad->LookupExpr( attrs_to_copy[index] );
 
 		if ( new_expr != NULL && ( old_expr == NULL || !(*old_expr == *new_expr) ) ) {
-			jobAd->Insert( new_expr->DeepCopy() );
+			jobAd->Insert( attrs_to_copy[index], new_expr->Copy() );
 		}
 	}
 
-	requestScheddUpdate( this );
+	requestScheddUpdate( this, false );
 
 	return;
 }
@@ -822,8 +826,8 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 
 	index = -1;
 	while ( attrs_to_copy[++index] != NULL ) {
-		if ( ( next_expr = jobAd->Lookup( attrs_to_copy[index] ) ) != NULL ) {
-			submit_ad->Insert( next_expr->DeepCopy() );
+		if ( ( next_expr = jobAd->LookupExpr( attrs_to_copy[index] ) ) != NULL ) {
+			submit_ad->Insert( attrs_to_copy[index], next_expr->Copy() );
 		}
 	}
 
@@ -847,7 +851,7 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 
 		if ( !args.AppendArgsFromClassAd( jobAd, &error_str ) ||
 			 !args.GetArgsStringV1Raw( &value_str, &error_str ) ) {
-			errorString = error_str;
+			errorString = error_str.Value();
 			delete submit_ad;
 			return NULL;
 		}
@@ -856,7 +860,7 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 		value_str = "";
 		if ( !env.MergeFrom( jobAd, &error_str ) ||
 			 !env.getDelimitedStringV1Raw( &value_str, &error_str ) ) {
-			errorString = error_str;
+			errorString = error_str.Value();
 			delete submit_ad;
 			return NULL;
 		}
@@ -879,6 +883,8 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 //	submit_ad->Assign( ATTR_NUM_RESTARTS, 0 );
 //	submit_ad->Assign( ATTR_NUM_SYSTEM_HOLDS, 0 );
 //	submit_ad->Assign( ATTR_JOB_COMMITTED_TIME, 0 );
+//	submit_ad->Assign( ATTR_COMMITTED_SLOT_TIME, 0 );
+//	submit_ad->Assign( ATTR_CUMULATIVE_SLOT_TIME, 0 );
 //	submit_ad->Assign( ATTR_TOTAL_SUSPENSIONS, 0 );
 //	submit_ad->Assign( ATTR_LAST_SUSPENSION_TIME, 0 );
 //	submit_ad->Assign( ATTR_CUMULATIVE_SUSPENSION_TIME, 0 );
@@ -889,7 +895,7 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 //	submit_ad->Assign( ATTR_JOB_LEAVE_IN_QUEUE, true );
 //	submit_ad->Assign( ATTR_SHOULD_TRANSFER_FILES, false );
 
-//	expr.sprintf( "%s = (%s >= %s) =!= True && CurrentTime > %s + %d",
+//	expr.sprintf( "%s = (%s >= %s) =!= True && time() > %s + %d",
 //				  ATTR_PERIODIC_REMOVE_CHECK, ATTR_STAGE_IN_FINISH,
 //				  ATTR_STAGE_IN_START, ATTR_Q_DATE, 1800 );
 //	submit_ad->Insert( expr.Value() );
@@ -911,13 +917,11 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 	}
 
 	jobAd->ResetExpr();
-	while ( (next_expr = jobAd->NextExpr()) != NULL ) {
-		if ( strncasecmp( ((Variable*)next_expr->LArg())->Name(),
-						  "REMOTE_", 7 ) == 0 &&
-			 strlen( ((Variable*)next_expr->LArg())->Name() ) > 7 ) {
+	while ( jobAd->NextExpr(next_name, next_expr) ) {
+		if ( strncasecmp( next_name, "REMOTE_", 7 ) == 0 &&
+			 strlen( next_name ) > 7 ) {
 
-			char *attr_value;
-			char const *attr_name = &((Variable*)next_expr->LArg())->Name()[7];
+			char const *attr_name = &(next_name[7]);
 
 			if(strcasecmp(attr_name,ATTR_JOB_ENVIRONMENT1) == 0 ||
 			   strcasecmp(attr_name,ATTR_JOB_ENVIRONMENT1_DELIM) == 0 ||
@@ -949,9 +953,7 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 				}
 			}
 
-			next_expr->RArg()->PrintToNewStr(&attr_value);
-			submit_ad->AssignExpr( attr_name, attr_value );
-			free(attr_value);
+			submit_ad->Insert( attr_name, next_expr->Copy() );
 		}
 	}
 
