@@ -22,7 +22,7 @@
 #include "condor_common.h"
 #include "condor_debug.h"
 #include "condor_daemon_core.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "condor_config.h"
 #include "classad_hashtable.h"
 #include "util_lib_proto.h"
@@ -62,34 +62,34 @@ int myproxyGetDelegationReaperId = 0;
 
 static int next_proxy_id = 1;
 
-int CheckProxies();
+void CheckProxies();
 
 static bool
 SetMasterProxy( Proxy *master, const Proxy *copy_src )
 {
 	int rc;
-	MyString tmp_file;
+	std::string tmp_file;
 
-	tmp_file.sprintf( "%s.tmp", master->proxy_filename );
+	sprintf( tmp_file, "%s.tmp", master->proxy_filename );
 
-	rc = copy_file( copy_src->proxy_filename, tmp_file.Value() );
+	rc = copy_file( copy_src->proxy_filename, tmp_file.c_str() );
 	if ( rc != 0 ) {
 		return false;
 	}
 
-	rc = rotate_file( tmp_file.Value(), master->proxy_filename );
+	rc = rotate_file( tmp_file.c_str(), master->proxy_filename );
 	if ( rc != 0 ) {
-		unlink( tmp_file.Value() );
+		unlink( tmp_file.c_str() );
 		return false;
 	}
 
 	master->expiration_time = copy_src->expiration_time;
 	master->near_expired = copy_src->near_expired;
 
-	int tid;
-	master->notification_tids.Rewind();
-	while ( master->notification_tids.Next( tid ) ) {
-		daemonCore->Reset_Timer( tid, 0 );
+	Callback cb;
+	master->m_callbacks.Rewind();
+	while ( master->m_callbacks.Next( cb ) ) {
+		((cb.m_data)->*(cb.m_func_ptr))();
 	}
 
 	return true;
@@ -104,8 +104,8 @@ bool InitializeProxyManager( const char *proxy_dir )
 	}
 
 	CheckProxies_tid = daemonCore->Register_Timer( 1, CheckProxies_interval,
-												   (TimerHandler)&CheckProxies,
-												   "CheckProxies", NULL );
+												   CheckProxies,
+												   "CheckProxies" );
 
 	masterProxyDirectory = strdup( proxy_dir );
 
@@ -164,7 +164,8 @@ void ReconfigProxyManager()
 // any proxy-related attributes, AcquireProxy() will store the empty
 // string in the error parameter and return NULL.
 Proxy *
-AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
+AcquireProxy( const ClassAd *job_ad, std::string &error,
+			  TimerHandlercpp func_ptr, Service *data  )
 {
 	if ( proxymanager_initialized == false ) {
 		error = "Internal Error: ProxyManager not initialized";
@@ -175,14 +176,16 @@ AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
 	Proxy *proxy = NULL;
 	ProxySubject *proxy_subject = NULL;
 	char *subject_name = NULL;
-	MyString proxy_path;
-	MyString owner;
+	char *fqan = NULL;
+	std::string proxy_path;
+	std::string owner;
 	char *param_str = NULL;
+	bool has_voms_attrs = false;
 
 	if ( job_ad->LookupString( ATTR_OWNER, owner ) ) {
-		MyString param_name;
-		param_name.sprintf( "JOB_PROXY_OVERRIDE_FILE_%s", owner.Value() );
-		param_str = param( param_name.Value() );
+		std::string param_name;
+		sprintf( param_name, "JOB_PROXY_OVERRIDE_FILE_%s", owner.c_str() );
+		param_str = param( param_name.c_str() );
 	}
 	if ( param_str == NULL ) {
 		param_str = param( "JOB_PROXY_OVERRIDE_FILE" );
@@ -194,23 +197,30 @@ AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
 									  proxy_path ) == 0 ) {
 
 		// Special handling for "use best proxy"
-		if ( job_ad->LookupString( ATTR_X509_USER_PROXY_SUBJECT,
-								   proxy_path ) != 0 ) {
-			subject_name = strdup( proxy_path.Value() );
-			if ( SubjectsByName.lookup( HashKey(subject_name),
+		job_ad->LookupString( ATTR_X509_USER_PROXY_FQAN, &fqan );
+		job_ad->LookupString( ATTR_X509_USER_PROXY_SUBJECT, &subject_name );
+		if ( subject_name ) {
+			if ( fqan == NULL ) {
+				fqan = strdup( subject_name );
+			} else {
+				has_voms_attrs = true;
+			}
+			if ( SubjectsByName.lookup( HashKey(fqan),
 										proxy_subject ) != 0 ) {
 				// We don't know about this proxy subject yet,
 				// create a new ProxySubject and fill it out
 				proxy_subject = new ProxySubject;
 				proxy_subject->subject_name = strdup( subject_name );
+				proxy_subject->fqan = strdup( fqan );
+				proxy_subject->has_voms_attrs = has_voms_attrs;
 
 				// Create a master proxy for our new ProxySubject
 				Proxy *new_master = new Proxy;
 				new_master->id = next_proxy_id++;
-				MyString tmp;
-				tmp.sprintf( "%s/master_proxy.%d", masterProxyDirectory,
+				std::string tmp;
+				sprintf( tmp, "%s/master_proxy.%d", masterProxyDirectory,
 							 new_master->id );
-				new_master->proxy_filename = strdup( tmp.Value() );
+				new_master->proxy_filename = strdup( tmp.c_str() );
 				new_master->num_references = 0;
 				new_master->subject = proxy_subject;
 				//SetMasterProxy( new_master, proxy );
@@ -221,33 +231,44 @@ AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
 
 				proxy_subject->master_proxy = new_master;
 
-				SubjectsByName.insert(HashKey(proxy_subject->subject_name),
+				SubjectsByName.insert(HashKey(proxy_subject->fqan),
 									  proxy_subject);
 			}
 			// Now that we have a proxy_subject, return it's master proxy
 			proxy = proxy_subject->master_proxy;
 			proxy->num_references++;
-			if ( notify_tid > 0 &&
-				 proxy->notification_tids.IsMember( notify_tid ) == false ) {
-				proxy->notification_tids.Append( notify_tid );
+			if ( func_ptr ) {
+				Callback cb;
+				cb.m_func_ptr = func_ptr;
+				cb.m_data = data;
+				if ( proxy->m_callbacks.IsMember( cb ) == false ) {
+					proxy->m_callbacks.Append( cb );
+				}
 			}
 			free( subject_name );
+			free( fqan );
 			return proxy;
 
 		}
 
-		//error.sprintf( "%s is not set in the job ad", ATTR_X509_USER_PROXY );
+		free( subject_name );
+		free( fqan );
+		//sprintf( error, "%s is not set in the job ad", ATTR_X509_USER_PROXY );
 		error = "";
 		return NULL;
 	}
 
-	if ( ProxiesByFilename.lookup( HashKey(proxy_path.Value()), proxy ) == 0 ) {
+	if ( ProxiesByFilename.lookup( HashKey(proxy_path.c_str()), proxy ) == 0 ) {
 		// We already know about this proxy,
 		// use the existing Proxy struct
 		proxy->num_references++;
-		if ( notify_tid > 0 &&
-			 proxy->notification_tids.IsMember( notify_tid ) == false ) {
-			proxy->notification_tids.Append( notify_tid );
+		if ( func_ptr ) {
+			Callback cb;
+			cb.m_func_ptr = func_ptr;
+			cb.m_data = data;
+			if ( proxy->m_callbacks.IsMember( cb ) == false ) {
+				proxy->m_callbacks.Append( cb );
+			}
 		}
 		return proxy;
 
@@ -255,48 +276,72 @@ AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
 
 		// We don't know about this proxy yet,
 		// find the proxy's expiration time and subject name
-		expire_time = x509_proxy_expiration_time( proxy_path.Value() );
+		expire_time = x509_proxy_expiration_time( proxy_path.c_str() );
 		if ( expire_time < 0 ) {
 			dprintf( D_ALWAYS, "Failed to get expiration time of proxy %s\n",
-					 proxy_path.Value() );
-			error.sprintf( "Failed to get expiration time of proxy" );
+					 proxy_path.c_str() );
+			error = "Failed to get expiration time of proxy";
 			return NULL;
 		}
-		subject_name = x509_proxy_identity_name( proxy_path.Value() );
+		subject_name = x509_proxy_identity_name( proxy_path.c_str() );
 		if ( subject_name == NULL ) {
 			dprintf( D_ALWAYS, "Failed to get identity of proxy %s\n",
-					 proxy_path.Value() );
-			error.sprintf( "Failed to get identity of proxy" );
+					 proxy_path.c_str() );
+			error = "Failed to get identity of proxy";
 			return NULL;
 		}
 
+		fqan = NULL;
+#if defined(HAVE_EXT_GLOBUS)
+		int rc = extract_VOMS_info_from_file( proxy_path.c_str(), 0, NULL,
+											  NULL, &fqan );
+		if ( rc != 0 && rc != 1 ) {
+			dprintf( D_ALWAYS, "Failed to get voms info of proxy %s\n",
+					 proxy_path.c_str() );
+			error = "Failed to get voms info of proxy";
+			free( subject_name );
+			return NULL;
+		}
+#endif
+		if ( fqan ) {
+			has_voms_attrs = true;
+		} else {
+			fqan = strdup( subject_name );
+		}
+
 		// Create a Proxy struct for our new proxy and populate it
 		proxy = new Proxy;
-		proxy->proxy_filename = strdup(proxy_path.Value());
+		proxy->proxy_filename = strdup(proxy_path.c_str());
 		proxy->num_references = 1;
 		proxy->expiration_time = expire_time;
 		proxy->near_expired = (expire_time - time(NULL)) <= minProxy_time;
 		proxy->id = next_proxy_id++;
-		if ( notify_tid > 0 &&
-			 proxy->notification_tids.IsMember( notify_tid ) == false ) {
-			proxy->notification_tids.Append( notify_tid );
+		if ( func_ptr ) {
+			Callback cb;
+			cb.m_func_ptr = func_ptr;
+			cb.m_data = data;
+			if ( proxy->m_callbacks.IsMember( cb ) == false ) {
+				proxy->m_callbacks.Append( cb );
+			}
 		}
 
-		ProxiesByFilename.insert(HashKey(proxy_path.Value()), proxy);
+		ProxiesByFilename.insert(HashKey(proxy_path.c_str()), proxy);
 
-		if ( SubjectsByName.lookup( HashKey(subject_name), proxy_subject ) != 0 ) {
+		if ( SubjectsByName.lookup( HashKey(fqan), proxy_subject ) != 0 ) {
 			// We don't know about this proxy subject yet,
 			// create a new ProxySubject and fill it out
 			proxy_subject = new ProxySubject;
 			proxy_subject->subject_name = strdup( subject_name );
+			proxy_subject->fqan = strdup( fqan );
+			proxy_subject->has_voms_attrs = true;
 
 			// Create a master proxy for our new ProxySubject
 			Proxy *new_master = new Proxy;
 			new_master->id = next_proxy_id++;
-			MyString tmp;
-			tmp.sprintf( "%s/master_proxy.%d", masterProxyDirectory,
+			std::string tmp;
+			sprintf( tmp, "%s/master_proxy.%d", masterProxyDirectory,
 						 new_master->id );
-			new_master->proxy_filename = strdup( tmp.Value() );
+			new_master->proxy_filename = strdup( tmp.c_str() );
 			new_master->num_references = 0;
 			new_master->subject = proxy_subject;
 			SetMasterProxy( new_master, proxy );
@@ -305,7 +350,7 @@ AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
 
 			proxy_subject->master_proxy = new_master;
 
-			SubjectsByName.insert(HashKey(proxy_subject->subject_name),
+			SubjectsByName.insert(HashKey(proxy_subject->fqan),
 								  proxy_subject);
 		}
 
@@ -320,10 +365,11 @@ AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
 		}
 
 		free( subject_name );
+		free( fqan );
 	}
 
 		// MyProxy crap
-	MyString buff;
+	std::string buff;
 	if ( job_ad->LookupString( ATTR_MYPROXY_HOST_NAME, buff ) ) {
 
 		int cluster;
@@ -342,17 +388,17 @@ AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
 		myProxyEntry->myproxy_password=NULL;
 		myProxyEntry->myproxy_credential_name=NULL;
 
-		myProxyEntry->myproxy_host=strdup(buff.Value());
+		myProxyEntry->myproxy_host=strdup(buff.c_str());
 		myProxyEntry->cluster_id = cluster;
 		myProxyEntry->proc_id = proc;
 
 		// Get optional MYPROXY_SERVER_DN attribute
 		if (job_ad->LookupString (ATTR_MYPROXY_SERVER_DN, buff)) {
-			myProxyEntry->myproxy_server_dn=strdup(buff.Value());
+			myProxyEntry->myproxy_server_dn=strdup(buff.c_str());
 		}
 
 		if (job_ad->LookupString (ATTR_MYPROXY_CRED_NAME, buff)) {
-			myProxyEntry->myproxy_credential_name=strdup(buff.Value());
+			myProxyEntry->myproxy_credential_name=strdup(buff.c_str());
 		}
 
 		if (job_ad->LookupInteger (ATTR_MYPROXY_REFRESH_THRESHOLD, myProxyEntry->refresh_threshold)) {
@@ -421,12 +467,16 @@ AcquireProxy( const ClassAd *job_ad, MyString &error, int notify_tid )
 }
 
 Proxy *
-AcquireProxy( Proxy *proxy, int notify_tid )
+AcquireProxy( Proxy *proxy, TimerHandlercpp func_ptr, Service *data )
 {
 	proxy->num_references++;
-	if ( notify_tid > 0 &&
-		 proxy->notification_tids.IsMember( notify_tid ) == false ) {
-		proxy->notification_tids.Append( notify_tid );
+	if ( func_ptr ) {
+		Callback cb;
+		cb.m_func_ptr = func_ptr;
+		cb.m_data = data;
+		if ( proxy->m_callbacks.IsMember( cb ) == false ) {
+			proxy->m_callbacks.Append( cb );
+		}
 	}
 	return proxy;
 }
@@ -436,15 +486,18 @@ AcquireProxy( Proxy *proxy, int notify_tid )
 // ProxyManager code will take care of that for you. If you provided a
 // notify_tid to AcquireProxy(), provide it again here.
 void
-ReleaseProxy( Proxy *proxy, int notify_tid )
+ReleaseProxy( Proxy *proxy, TimerHandlercpp func_ptr, Service *data )
 {
 	if ( proxymanager_initialized == false || proxy == NULL ) {
 		return;
 	}
 
 	proxy->num_references--;
-	if ( notify_tid > 0 ) {
-		proxy->notification_tids.Delete( notify_tid );
+	if ( func_ptr ) {
+		Callback cb;
+		cb.m_func_ptr = func_ptr;
+		cb.m_data = data;
+		proxy->m_callbacks.Delete( cb );
 	}
 
 	if ( proxy->num_references < 0 ) {
@@ -485,8 +538,9 @@ ReleaseProxy( Proxy *proxy, int notify_tid )
 			free( proxy_subject->master_proxy->proxy_filename );
 			delete proxy_subject->master_proxy;
 
-			SubjectsByName.remove( HashKey(proxy_subject->subject_name) );
+			SubjectsByName.remove( HashKey(proxy_subject->fqan) );
 			free( proxy_subject->subject_name );
+			free( proxy_subject->fqan );
 			delete proxy_subject;
 		}
 
@@ -565,7 +619,7 @@ void doCheckProxies()
 // This function is called
 // periodically to check for updated proxies. It can be called earlier
 // if a proxy is about to expire.
-int CheckProxies()
+void CheckProxies()
 {
 	int now = time(NULL);
 	int next_check = CheckProxies_interval + now;
@@ -613,10 +667,10 @@ int CheckProxies()
 				curr_proxy->near_expired =
 					(curr_proxy->expiration_time - now) <= minProxy_time;
 
-				int tid;
-				curr_proxy->notification_tids.Rewind();
-				while ( curr_proxy->notification_tids.Next( tid ) ) {
-					daemonCore->Reset_Timer( tid, 0 );
+				Callback cb;
+				curr_proxy->m_callbacks.Rewind();
+				while ( curr_proxy->m_callbacks.Next( cb ) ) {
+					((cb.m_data)->*(cb.m_func_ptr))();
 				}
 
 				if ( curr_proxy->expiration_time > new_master->expiration_time ) {
@@ -625,12 +679,11 @@ int CheckProxies()
 
 			} else if ( curr_proxy->near_expired ) {
 
-				int tid;
-				curr_proxy->notification_tids.Rewind();
-				while ( curr_proxy->notification_tids.Next( tid ) ) {
-					daemonCore->Reset_Timer( tid, 0 );
+				Callback cb;
+				curr_proxy->m_callbacks.Rewind();
+				while ( curr_proxy->m_callbacks.Next( cb ) ) {
+					((cb.m_data)->*(cb.m_func_ptr))();
 				}
-
 			}
 
 			if ( curr_proxy->expiration_time - minProxy_time < next_check &&
@@ -651,8 +704,6 @@ int CheckProxies()
 	// next_check is the absolute time of the next check, convert it to
 	// a relative time (from now)
 	daemonCore->Reset_Timer( CheckProxies_tid, next_check - now );
-
-	return TRUE;
 }
 
 int RefreshProxyThruMyProxy(Proxy * proxy)
@@ -714,19 +765,19 @@ int RefreshProxyThruMyProxy(Proxy * proxy)
 
 	// Set up environnment for myproxy-get-delegation
 	Env myEnv;
-	MyString buff;
+	std::string buff;
 
 	if (myProxyEntry->myproxy_server_dn) {
-		buff.sprintf( "MYPROXY_SERVER_DN=%s",
+		sprintf( buff, "MYPROXY_SERVER_DN=%s",
 				 myProxyEntry->myproxy_server_dn);
-		myEnv.SetEnv(buff.Value());
-		dprintf (D_FULLDEBUG, "%s\n", buff.Value());
+		myEnv.SetEnv(buff.c_str());
+		dprintf (D_FULLDEBUG, "%s\n", buff.c_str());
 	}
 
 
-	buff.sprintf("X509_USER_PROXY=%s", proxy_filename);
-	myEnv.SetEnv (buff.Value());
-	dprintf (D_FULLDEBUG, "%s\n", buff.Value());
+	sprintf(buff, "X509_USER_PROXY=%s", proxy_filename);
+	myEnv.SetEnv (buff.c_str());
+	dprintf (D_FULLDEBUG, "%s\n", buff.c_str());
 
 
 	// Print password (this will end up in stdin for myproxy-get-delegation)
@@ -778,11 +829,15 @@ int RefreshProxyThruMyProxy(Proxy * proxy)
 
 	// Create temporary file to store myproxy-get-delegation's stderr
 	myProxyEntry->get_delegation_err_filename = create_temp_file();
-	chmod (myProxyEntry->get_delegation_err_filename, 0600);
-	myProxyEntry->get_delegation_err_fd = safe_open_wrapper(myProxyEntry->get_delegation_err_filename,O_RDWR);
-	if (myProxyEntry->get_delegation_err_fd == -1) {
-		dprintf (D_ALWAYS, "Error opening file %s\n",
-				 myProxyEntry->get_delegation_err_filename);
+	if(!myProxyEntry->get_delegation_err_filename) {
+		dprintf( D_ALWAYS, "Failed to create temp file");
+	} else {
+		chmod (myProxyEntry->get_delegation_err_filename, 0600);
+		myProxyEntry->get_delegation_err_fd = safe_open_wrapper(myProxyEntry->get_delegation_err_filename,O_RDWR);
+		if (myProxyEntry->get_delegation_err_fd == -1) {
+			dprintf (D_ALWAYS, "Error opening file %s\n",
+					 myProxyEntry->get_delegation_err_filename);
+		}
 	}
 
 
@@ -896,7 +951,7 @@ int MyProxyGetDelegationReaper(Service *, int exitPid, int exitStatus)
 
 		char buff[500];
 		buff[0]='\0';
-		MyString output;
+		std::string output;
 		int fd = safe_open_wrapper(matched_entry->get_delegation_err_filename, O_RDONLY);
 		if (fd != -1) {
 			int bytes_read;
@@ -925,7 +980,7 @@ int MyProxyGetDelegationReaper(Service *, int exitPid, int exitStatus)
 			matched_entry->cluster_id,
 			matched_entry->proc_id,
 			WEXITSTATUS(exitStatus),
-			output.Value());
+			output.c_str());
 
 	}
 
@@ -977,7 +1032,7 @@ int GetMyProxyPasswordFromSchedD (int cluster_id, int proc_id,
 		return FALSE;
 	}
 
-	sock->eom();
+	sock->end_of_message();
 	sock->decode();
 
 	if (!sock->code (*password)) {
@@ -986,7 +1041,7 @@ int GetMyProxyPasswordFromSchedD (int cluster_id, int proc_id,
 
 	}
 
-	sock->eom();
+	sock->end_of_message();
 	sock->close();
 	delete sock;
 	return TRUE;
