@@ -25,7 +25,7 @@
 #include "condor_string.h"	// for strnewp and friends
 #include "condor_daemon_core.h"
 #include "basename.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "condor_xml_classads.h"
 
 #include "gridmanager.h"
@@ -102,11 +102,11 @@ void UnicoreJobReconfig()
 
 bool UnicoreJobAdMatch( const ClassAd *job_ad ) {
 	int universe;
-	MyString resource;
+	std::string resource;
 	if ( job_ad->LookupInteger( ATTR_JOB_UNIVERSE, universe ) &&
 		 universe == CONDOR_UNIVERSE_GRID &&
 		 job_ad->LookupString( ATTR_GRID_RESOURCE, resource ) &&
-		 strncasecmp( resource.Value(), "unicore ", 4 ) == 0 ) {
+		 strncasecmp( resource.c_str(), "unicore ", 4 ) == 0 ) {
 
 		return true;
 	}
@@ -123,7 +123,7 @@ void
 UnicoreJob::UnicoreGahpCallbackHandler( const char *update_ad_string )
 {
 	ClassAd *update_ad = NULL;
-	MyString job_id;
+	std::string job_id;
 	ClassAdXMLParser xml_parser;
 	UnicoreJob *job;
 
@@ -143,7 +143,7 @@ UnicoreJob::UnicoreGahpCallbackHandler( const char *update_ad_string )
 		return;
 	}
 	
-	if ( JobsByUnicoreId.lookup( HashKey( job_id.Value() ), job ) != 0 ||
+	if ( JobsByUnicoreId.lookup( HashKey( job_id.c_str() ), job ) != 0 ||
 		 job == NULL ) {
 		dprintf( D_FULLDEBUG, "UnicoreGahpCallbackHandler: status ad for "
 				 "unknown job, ignoring\n" );
@@ -159,11 +159,12 @@ UnicoreJob::UnicoreGahpCallbackHandler( const char *update_ad_string )
 
 		// If we already have an unprocessed update ad, merge the two,
 		// with the new one overwriting duplicate attributes.
+	const char *new_name;
 	ExprTree *new_expr;
 
 	update_ad->ResetExpr();
-	while ( (new_expr = update_ad->NextExpr()) ) {
-		job->newRemoteStatusAd->Insert( new_expr->DeepCopy() );
+	while ( update_ad->NextExpr( new_name, new_expr ) ) {
+		job->newRemoteStatusAd->Insert( new_name, new_expr->Copy() );
 	}
 
 	job->SetEvaluateState();
@@ -182,7 +183,7 @@ UnicoreJob::UnicoreJob( ClassAd *classad )
 	: BaseJob( classad )
 {
 	MyString buff;
-	MyString error_string = "";
+	std::string error_string = "";
 
 	resourceName = NULL;
 	jobContact = NULL;
@@ -224,19 +225,19 @@ UnicoreJob::UnicoreJob( ClassAd *classad )
 	if ( !buff.IsEmpty() ) {
 		const char *token;
 
-		buff.Tokenize();
+		Tokenize( buff );
 
-		token = buff.GetNextToken( " ", false );
-		if ( !token || stricmp( token, "unicore" ) ) {
-			error_string.sprintf( "%s not of type unicore", ATTR_GRID_JOB_ID );
+		token = GetNextToken( " ", false );
+		if ( !token || strcasecmp( token, "unicore" ) ) {
+			sprintf( error_string, "%s not of type unicore", ATTR_GRID_JOB_ID );
 			goto error_exit;
 		}
 
-		buff.GetNextToken( " ", false );
-		buff.GetNextToken( " ", false );
-		token = buff.GetNextToken( " ", false );
+		GetNextToken( " ", false );
+		GetNextToken( " ", false );
+		token = GetNextToken( " ", false );
 		if ( !token ) {
-			error_string.sprintf( "%s missing job ID",
+			sprintf( error_string, "%s missing job ID",
 								  ATTR_GRID_JOB_ID );
 			goto error_exit;
 		}
@@ -249,8 +250,8 @@ UnicoreJob::UnicoreJob( ClassAd *classad )
 		// We must ensure that the code-path from GM_HOLD doesn't depend
 		// on any initialization that's been skipped.
 	gmState = GM_HOLD;
-	if ( !error_string.IsEmpty() ) {
-		jobAd->Assign( ATTR_HOLD_REASON, error_string.Value() );
+	if ( !error_string.empty() ) {
+		jobAd->Assign( ATTR_HOLD_REASON, error_string.c_str() );
 	}
 	return;
 }
@@ -280,14 +281,15 @@ void UnicoreJob::Reconfig()
 	}
 }
 
-int UnicoreJob::doEvaluateState()
+void UnicoreJob::doEvaluateState()
 {
 	int old_gm_state;
 	int old_unicore_state;
 	bool reevaluate_state = true;
 	time_t now = time(NULL);
 
-	bool done;
+	bool attr_exists;
+	bool attr_dirty;
 	int rc;
 
 	daemonCore->Reset_Timer( evaluateStateTid, TIMER_NEVER );
@@ -460,8 +462,9 @@ int UnicoreJob::doEvaluateState()
 			if ( condorState == REMOVED || condorState == HELD ) {
 				gmState = GM_CANCEL;
 			} else {
-				done = requestScheddUpdate( this );
-				if ( !done ) {
+				jobAd->GetDirtyFlag( ATTR_GRID_JOB_ID, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 				gmState = GM_SUBMIT_COMMIT;
@@ -564,8 +567,9 @@ dprintf(D_FULLDEBUG,"(%d.%d) *** Processing callback ad\n",procID.cluster, procI
 			// Report job completion to the schedd.
 			JobTerminated();
 			if ( condorState == COMPLETED ) {
-				done = requestScheddUpdate( this );
-				if ( !done ) {
+				jobAd->GetDirtyFlag( ATTR_JOB_STATUS, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 			}
@@ -590,13 +594,7 @@ dprintf(D_FULLDEBUG,"(%d.%d) *** Processing callback ad\n",procID.cluster, procI
 			} else {
 				// Clear the contact string here because it may not get
 				// cleared in GM_CLEAR_REQUEST (it might go to GM_HOLD first).
-//				if ( jobContact != NULL ) {
-//					free( jobContact );
-//					jobContact = NULL;
-//					jobAd->Assign( ATTR_GLOBUS_CONTACT_STRING,
-//									   NULL_JOB_CONTACT );
-//					requestScheddUpdate( this );
-//				}
+//				SetRemoteJobId( NULL );
 //				gmState = GM_CLEAR_REQUEST;
 				gmState = GM_HOLD;
 			}
@@ -649,8 +647,11 @@ if ( unicoreState != COMPLETED ) {
 			// through. However, since we registered update events the
 			// first time, requestScheddUpdate won't return done until
 			// they've been committed to the schedd.
-			done = requestScheddUpdate( this );
-			if ( !done ) {
+			const char *name;
+			ExprTree *expr;
+			jobAd->ResetExpr();
+			if ( jobAd->NextDirtyExpr(name, expr) ) {
+				requestScheddUpdate( this, true );
 				break;
 			}
 			executeLogged = false;
@@ -678,7 +679,7 @@ if ( unicoreState != COMPLETED ) {
 				jobAd->LookupString( ATTR_HOLD_REASON, holdReason,
 									 sizeof(holdReason) - 1 );
 				if ( holdReason[0] == '\0' && errorString != "" ) {
-					strncpy( holdReason, errorString.Value(),
+					strncpy( holdReason, errorString.c_str(),
 							 sizeof(holdReason) - 1 );
 				}
 				if ( holdReason[0] == '\0' ) {
@@ -724,8 +725,6 @@ if ( unicoreState != COMPLETED ) {
 		}
 
 	} while ( reevaluate_state );
-
-	return TRUE;
 }
 
 BaseResource *UnicoreJob::GetResource()
@@ -763,7 +762,7 @@ void UnicoreJob::UpdateUnicoreState( ClassAd *update_ad )
 	}
 
 	update_ad->ResetName();
-	while ( ( next_attr_name = update_ad->NextName() ) != NULL ) {
+	while ( ( next_attr_name = update_ad->NextNameOriginal() ) != NULL ) {
 		if ( strcasecmp( next_attr_name, ATTR_MY_TYPE ) == 0 ||
 			 strcasecmp( next_attr_name, ATTR_TARGET_TYPE ) == 0 ||
 			 strcasecmp( next_attr_name, "UnicoreJobId" ) == 0 ) {
@@ -781,8 +780,8 @@ void UnicoreJob::UpdateUnicoreState( ClassAd *update_ad )
 				JobIdle();
 			}
 		}
-		next_expr = update_ad->Lookup( next_attr_name );
-		jobAd->Insert( next_expr->DeepCopy() );
+		next_expr = update_ad->LookupExpr( next_attr_name );
+		jobAd->Insert( next_attr_name, next_expr->Copy() );
 	}
 }
 
@@ -805,11 +804,11 @@ void UnicoreJob::SetRemoteJobId( const char *job_id )
 		jobContact = NULL;
 	}
 
-	MyString full_job_id;
+	std::string full_job_id;
 	if ( job_id ) {
-		full_job_id.sprintf( "%s %s", resourceName, job_id );
+		sprintf( full_job_id, "%s %s", resourceName, job_id );
 	}
-	BaseJob::SetRemoteJobId( full_job_id.Value() );
+	BaseJob::SetRemoteJobId( full_job_id.c_str() );
 }
 
 MyString *UnicoreJob::buildSubmitAd()
