@@ -17,7 +17,9 @@
  *
  ***************************************************************/
 
+#include "condor_common.h"
 #include "condor_daemon_core.h"
+#include "condor_sinful.h"
 // for 'param' function
 #include "condor_config.h"
 // for 'Daemon' class
@@ -77,6 +79,7 @@ AbstractReplicatorStateMachine::initializeReplicationList( char* buffer )
     StringList replicationAddressList;
     char*      replicationAddress    = NULL;
     bool       isMyAddressPresent    = false;
+	Sinful     my_addr( daemonCore->InfoCommandSinfulString( ) );
 
     replicationAddressList.initializeFromString( buffer );
     // initializing a list unrolls it, that's why the rewind is needed to bring
@@ -99,8 +102,7 @@ AbstractReplicatorStateMachine::initializeReplicationList( char* buffer )
 
             continue;
         }
-        if( strcmp( sinfulAddress,
-                    daemonCore->InfoCommandSinfulString( ) ) == 0 ) {
+        if( my_addr.addressPointsToMe( Sinful(sinfulAddress) ) ) {
             isMyAddressPresent = true;
         }
         else {
@@ -131,7 +133,7 @@ AbstractReplicatorStateMachine::reinitialize()
         free( buffer );
     } else {
         utilCrucialError( utilNoParameterError("REPLICATION_LIST", 
-		  								       "REPLICATION").GetCStr( ) );
+		  								       "REPLICATION").Value( ) );
     }
     buffer = param( "HAD_CONNECTION_TIMEOUT" );
 
@@ -140,30 +142,21 @@ AbstractReplicatorStateMachine::reinitialize()
 
         if( errno == ERANGE || m_connectionTimeout <= 0 ) {
         	utilCrucialError( utilConfigurationError( "HAD_CONNECTION_TIMEOUT",
-													  "HAD" ).GetCStr( ) );
+													  "HAD" ).Value( ) );
         }
         free( buffer );
     } else {
         utilCrucialError( utilNoParameterError("HAD_CONNECTION_TIMEOUT",
-										       "HAD").GetCStr( ) );
+										       "HAD").Value( ) );
     }
 
 	buffer = param( "TRANSFERER" );
-	if ( NULL != buffer ) {
-		m_transfererPath = buffer;
-		free( buffer );
-	}
-	else {
-		buffer = param( "SBIN" );
-		if( !buffer ) {
-			utilCrucialError(
-				utilConfigurationError("SBIN","REPLICATION").GetCStr());
-		}
-		else {
-			m_transfererPath.sprintf( "%s/condor_transferer", buffer );
-			free( buffer );
-		}
+	if (!buffer) {
+		utilCrucialError(utilConfigurationError("TRANSFERER",
+												"REPLICATION").Value());
 	}
+	m_transfererPath = buffer;
+	free( buffer );
 
 	char* spoolDirectory = param( "SPOOL" );
     
@@ -190,7 +183,7 @@ AbstractReplicatorStateMachine::reinitialize()
         free( spoolDirectory );
     } else {
         utilCrucialError( utilNoParameterError("SPOOL", "REPLICATION").
-                          GetCStr( ) );
+                          Value( ) );
     }
 	printDataMembers( );
 }
@@ -245,11 +238,11 @@ AbstractReplicatorStateMachine::downloadReplicaTransfererReaper(
     // upon failure we do not synchronize the local version, since such
 	// downloading is considered invalid
 	if( ! FilesOperations::safeRotateFile(
-						  replicatorStateMachine->m_versionFilePath.GetCStr(),
-										  temporaryFilesExtension.GetCStr()) ||
+						  replicatorStateMachine->m_versionFilePath.Value(),
+										  temporaryFilesExtension.Value()) ||
 		! FilesOperations::safeRotateFile(
-						    replicatorStateMachine->m_stateFilePath.GetCStr(),
-										  temporaryFilesExtension.GetCStr()) ) {
+						    replicatorStateMachine->m_stateFilePath.Value(),
+										  temporaryFilesExtension.Value()) ) {
 		return TRANSFERER_FALSE;
 	}
     replicatorStateMachine->m_myVersion.synchronize( false );
@@ -325,27 +318,27 @@ bool
 AbstractReplicatorStateMachine::download( const char* daemonSinfulString )
 {
 	ArgList  processArguments;
-	processArguments.AppendArg( m_transfererPath.GetCStr() );
+	processArguments.AppendArg( m_transfererPath.Value() );
 	processArguments.AppendArg( "-f" );
 	processArguments.AppendArg( "down" );
 	processArguments.AppendArg( daemonSinfulString );
-	processArguments.AppendArg( m_versionFilePath.GetCStr() );
+	processArguments.AppendArg( m_versionFilePath.Value() );
 	processArguments.AppendArg( "1" );
-	processArguments.AppendArg( m_stateFilePath.GetCStr() );
+	processArguments.AppendArg( m_stateFilePath.Value() );
 	// Get arguments from this ArgList object for descriptional purposes.
 	MyString	s;
 	processArguments.GetArgsStringForDisplay( &s );
     dprintf( D_FULLDEBUG,
 			 "AbstractReplicatorStateMachine::download creating "
 			 "downloading condor_transferer process: \n \"%s\"\n",
-			 s.GetCStr( ) );
+			 s.Value( ) );
 
 	// PRIV_ROOT privilege is necessary here to create the process
 	// so we can read GSI certs <sigh>
 	priv_state privilege = PRIV_ROOT;
 
 	int transfererPid = daemonCore->Create_Process(
-        m_transfererPath.GetCStr( ),  // name
+        m_transfererPath.Value( ),    // name
         processArguments,             // args
         privilege,                    // priv
         m_downloadReaperId,           // reaper id
@@ -383,13 +376,13 @@ bool
 AbstractReplicatorStateMachine::upload( const char* daemonSinfulString )
 {
 	ArgList  processArguments;
-	processArguments.AppendArg( m_transfererPath.GetCStr() );
+	processArguments.AppendArg( m_transfererPath.Value() );
 	processArguments.AppendArg( "-f" );
 	processArguments.AppendArg( "up" );
 	processArguments.AppendArg( daemonSinfulString );
-	processArguments.AppendArg( m_versionFilePath.GetCStr() );
+	processArguments.AppendArg( m_versionFilePath.Value() );
 	processArguments.AppendArg( "1" );
-	processArguments.AppendArg( m_stateFilePath.GetCStr() );
+	processArguments.AppendArg( m_stateFilePath.Value() );
 
 	// Get arguments from this ArgList object for descriptional purposes.
 	MyString	s;
@@ -397,14 +390,14 @@ AbstractReplicatorStateMachine::upload( const char* daemonSinfulString )
     dprintf( D_FULLDEBUG,
 			 "AbstractReplicatorStateMachine::upload creating "
 			 "uploading condor_transferer process: \n \"%s\"\n",
-			 s.GetCStr( ) );
+			 s.Value( ) );
 
 	// PRIV_ROOT privilege is necessary here to create the process
 	// so we can read GSI certs <sigh>
 	priv_state privilege = PRIV_ROOT;
 
     int transfererPid = daemonCore->Create_Process(
-        m_transfererPath.GetCStr( ),  // name
+        m_transfererPath.Value( ),    // name
         processArguments,             // args
         privilege,                    // priv
         m_uploadReaperId,             // reaper id
@@ -489,8 +482,8 @@ AbstractReplicatorStateMachine::updateVersionsList( Version& newVersion )
     }
     dprintf( D_FULLDEBUG,
         "AbstractReplicatorStateMachine::updateVersionsList appending %s\n",
-         newVersion.toString( ).GetCStr( ) );
-    m_versionsList.Append( newVersion );
+         newVersion.toString( ).Value( ) );
+    m_versionsList.Append( &newVersion );
     m_versionsList.Rewind( );
 // End of TODO: Atomic operation
 }
@@ -542,8 +535,8 @@ AbstractReplicatorStateMachine::sendCommand(
         return ;
     }
 
-    char* sinfulString = daemonCore->InfoCommandSinfulString();
-    if(! socket.code( sinfulString )/* || ! socket.eom( )*/) {
+    char const* sinfulString = daemonCore->InfoCommandSinfulString();
+    if(! socket.put( sinfulString )/* || ! socket.end_of_message( )*/) {
         dprintf( D_ALWAYS, "AbstractReplicatorStateMachine::sendCommand "
                             "unable to code the local sinful string or eom%s\n",
                    sinfulString );
@@ -564,7 +557,7 @@ AbstractReplicatorStateMachine::sendCommand(
 		return ;
 	}
 // End of Command-specific actions
-	if( ! socket.eom( ) ) {
+	if( ! socket.end_of_message( ) ) {
 		socket.close( );
        	dprintf( D_ALWAYS, "AbstractReplicatorStateMachine::sendCommand "
                             "unable to code the end of message\n" );
@@ -604,7 +597,7 @@ AbstractReplicatorStateMachine::versionAndStateCommand(ReliSock& socket)
     }
 	int stateAsInteger = int( m_state );
 
-    if( ! socket.code( stateAsInteger ) /*|| ! socket.eom( )*/ ) {
+    if( ! socket.code( stateAsInteger ) /*|| ! socket.end_of_message( )*/ ) {
         dprintf( D_NETWORK,
             "AbstractReplicatorStateMachine::versionAndStateCommand "
             "unable to code the state or eom%d\n", m_state );
@@ -641,10 +634,10 @@ AbstractReplicatorStateMachine::killTransferers()
         extension += ".";
         extension += DOWNLOADING_TEMPORARY_FILES_EXTENSION;
 
-        FilesOperations::safeUnlinkFile( m_versionFilePath.GetCStr( ),
-                                         extension.GetCStr( ) );
-        FilesOperations::safeUnlinkFile( m_stateFilePath.GetCStr( ),
-                                         extension.GetCStr( ) );
+        FilesOperations::safeUnlinkFile( m_versionFilePath.Value( ),
+                                         extension.Value( ) );
+        FilesOperations::safeUnlinkFile( m_stateFilePath.Value( ),
+                                         extension.Value( ) );
 		m_downloadTransfererMetadata.set();
     }
 
@@ -671,10 +664,10 @@ AbstractReplicatorStateMachine::killTransferers()
             extension += ".";
             extension += UPLOADING_TEMPORARY_FILES_EXTENSION;
 
-            FilesOperations::safeUnlinkFile( m_versionFilePath.GetCStr( ),
-                                             extension.GetCStr( ) );
-            FilesOperations::safeUnlinkFile( m_stateFilePath.GetCStr( ),
-                                             extension.GetCStr( ) );
+            FilesOperations::safeUnlinkFile( m_versionFilePath.Value( ),
+                                             extension.Value( ) );
+            FilesOperations::safeUnlinkFile( m_stateFilePath.Value( ),
+                                             extension.Value( ) );
 			delete uploadTransfererMetadata;
 			// after deletion the iterator is moved to the previous member
 			// so advancing the iterator twice and missing one entry does not
