@@ -17,10 +17,12 @@
  *
  ***************************************************************/
 
+#include "condor_common.h"
 // for 'daemonCore'
 #include "condor_daemon_core.h"
 // for 'param' function
 #include "condor_config.h"
+#include "my_username.h"
 
 #include "ReplicatorStateMachine.h"
 //#include "HadCommands.h"
@@ -81,7 +83,7 @@ getConfigurationDefaultPositiveIntegerParameter( const char* parameter )
  * Note        : the function may halt the program execution, in case when the
  *               value of a parameter is not properly specified in the 
  *		 		 configuration file - this is the difference between the 
- *		 		 function and 'param_integer' in condor_c++_util/condor_config.C
+ *		 		 function and 'param_integer' in condor_utils/condor_config.C
  */
 static int
 getConfigurationPositiveIntegerParameter( const char* parameter )
@@ -97,7 +99,7 @@ getConfigurationPositiveIntegerParameter( const char* parameter )
 
         if( ! result || parameterValue <= 0 ) {
         	utilCrucialError( utilConfigurationError(parameter,
-                                             "REPLICATION").GetCStr( ) );
+                                             "REPLICATION").Value( ) );
 		}
     } else {
         dprintf( D_ALWAYS, "getConfigurationPositiveIntegerParameter "
@@ -123,6 +125,9 @@ ReplicatorStateMachine::ReplicatorStateMachine()
    	m_newlyJoinedWaitingVersionInterval = -1;
    	m_lastHadAliveTime          = -1;
    	srand( time( NULL ) );
+	m_classAd = NULL;
+	m_updateCollectorTimerId = -1;
+	m_updateInterval = -1;
 }
 // finalizing the delta, belonging to this class only, since the data, belonging
 // to the base class is finalized implicitly
@@ -148,15 +153,32 @@ ReplicatorStateMachine::finalize()
 void
 ReplicatorStateMachine::finalizeDelta( )
 {
-	dprintf( D_ALWAYS, "ReplicatorStateMachine::finalizeDelta started\n" );
+    ClassAd invalidate_ad;
+    MyString line;
+
+    dprintf( D_ALWAYS, "ReplicatorStateMachine::finalizeDelta started\n" );
     utilCancelTimer(m_replicationTimerId);
     utilCancelTimer(m_versionRequestingTimerId);
     utilCancelTimer(m_versionDownloadingTimerId);
+    utilCancelTimer(m_updateCollectorTimerId);
     m_replicationInterval               = -1;
     m_hadAliveTolerance                 = -1;
     m_maxTransfererLifeTime             = -1;
     m_newlyJoinedWaitingVersionInterval = -1;
     m_lastHadAliveTime                  = -1;
+    m_updateInterval                    = -1;
+
+    if ( m_classAd != NULL ) {
+        delete m_classAd;
+        m_classAd = NULL;
+    }
+
+    invalidate_ad.SetMyTypeName( QUERY_ADTYPE );
+    invalidate_ad.SetTargetTypeName( "Replication" );
+    line.sprintf( "TARGET.%s == \"%s\"", ATTR_NAME, m_name.Value( ) );
+    invalidate_ad.AssignExpr( ATTR_REQUIREMENTS, line.Value( ) );
+	invalidate_ad.Assign( ATTR_NAME, m_name.Value() );
+    daemonCore->sendUpdates( INVALIDATE_ADS_GENERIC, &invalidate_ad, NULL, false );
 }
 void
 ReplicatorStateMachine::initialize( )
@@ -211,8 +233,22 @@ ReplicatorStateMachine::reinitialize()
         dprintf( D_FULLDEBUG, "ReplicatorStateMachine::reinitialize %s=%d\n",
                 "HAD_LIST", m_hadAliveTolerance );
     } else {
-        utilCrucialError( utilNoParameterError( "HAD_LIST", "HAD" ).GetCStr( ));
+        utilCrucialError( utilNoParameterError( "HAD_LIST", "HAD" ).Value( ));
     }
+
+    initializeClassAd();
+    int updateInterval = param_integer ( "REPLICATION_UPDATE_INTERVAL", 300 );
+    if ( m_updateInterval != updateInterval ) {
+        m_updateInterval = updateInterval;
+
+        utilCancelTimer(m_updateCollectorTimerId);
+
+        m_updateCollectorTimerId = daemonCore->Register_Timer ( 0,
+               m_updateInterval,
+               (TimerHandlercpp) &ReplicatorStateMachine::updateCollectors,
+               "ReplicatorStateMachine::updateCollectors", this );
+    }
+
     // set a timer to replication routine
     dprintf( D_ALWAYS, "ReplicatorStateMachine::reinitialize setting "
                                       "replication timer\n" );
@@ -237,6 +273,50 @@ ReplicatorStateMachine::reinitialize()
 	
 	beforePassiveStateHandler( );
 }
+
+void
+ReplicatorStateMachine::initializeClassAd()
+{
+    if( m_classAd != NULL) {
+        delete m_classAd;
+        m_classAd = NULL;
+    }
+
+    m_classAd = new ClassAd();
+
+    m_classAd->SetMyTypeName("Replication");
+    m_classAd->SetTargetTypeName("");
+
+    m_name.sprintf( "replication@%s -p %d", my_full_hostname( ),
+				  daemonCore->InfoCommandPort( ) );
+    m_classAd->Assign( ATTR_NAME, m_name.Value( ) );
+    m_classAd->Assign( ATTR_MY_ADDRESS,
+					   daemonCore->InfoCommandSinfulString( ) );
+
+    // publish list of replication nodes
+    char* buffer = param( "REPLICATION_LIST" );
+	if ( NULL == buffer ) {
+		EXCEPT( "ReplicatorStateMachine: No replication list!!\n" );
+	}
+    char* replAddress = NULL;
+    StringList replList;
+    MyString attrReplList;
+    MyString comma;
+
+    replList.initializeFromString( buffer );
+    replList.rewind( );
+
+    while( ( replAddress = replList.next() ) ) {
+        attrReplList += comma;
+        attrReplList += replAddress;
+        comma = ",";
+    }
+    m_classAd->Assign( ATTR_REPLICATION_LIST, attrReplList.Value( ) );
+
+    // publish DC attributes
+    daemonCore->publish(m_classAd);
+	free(buffer);
+}
 // sends the version of the last execution time to all the replication daemons,
 // then asks the pool replication daemons to send their own versions to it,
 // sets a timer to wait till the versions are received
@@ -324,7 +404,7 @@ ReplicatorStateMachine::inLeaderStateHandler( )
             "is set to %s", ctime( &m_lastHadAliveTime ) );
     //if( downloadTransferersNumber( ) == 0 && 
 	// 	  replicaSelectionHandler( newVersion ) ) {
-    //    download( newVersion.getSinfulString( ).GetCStr( ) );
+    //    download( newVersion.getSinfulString( ).Value( ) );
     //}
 }
 
@@ -334,7 +414,7 @@ ReplicatorStateMachine::replicaSelectionHandler( Version& newVersion )
     REPLICATION_ASSERT( m_state == VERSION_DOWNLOADING || m_state == BACKUP );
     dprintf( D_ALWAYS, "ReplicatorStateMachine::replicaSelectionHandler "
 			"started with my version = %s, #versions = %d\n",
-             m_myVersion.toString( ).GetCStr( ), m_versionsList.Number( ) );
+             m_myVersion.toString( ).Value( ), m_versionsList.Number( ) );
     List<Version> actualVersionsList;
     Version myVersionCopy = m_myVersion;
     
@@ -364,11 +444,11 @@ ReplicatorStateMachine::replicaSelectionHandler( Version& newVersion )
     // taking the first actual version as the best version in the meantime
     actualVersionsList.Next( bestVersion );
     dprintf( D_ALWAYS, "ReplicatorStateMachine::replicaSelectionHandler best "
-			"version = %s\n", bestVersion.toString( ).GetCStr( ) );
+			"version = %s\n", bestVersion.toString( ).Value( ) );
     
     while( actualVersionsList.Next( version ) ) {
         dprintf( D_ALWAYS, "ReplicatorStateMachine::replicaSelectionHandler "
-				"actual version = %s\n", version.toString( ).GetCStr( ) );
+				"actual version = %s\n", version.toString( ).Value( ) );
         if( version.isComparable( bestVersion ) && version > bestVersion ) {
             bestVersion = version;
         }
@@ -387,7 +467,7 @@ ReplicatorStateMachine::replicaSelectionHandler( Version& newVersion )
     }
     newVersion = bestVersion;
     dprintf( D_ALWAYS, "ReplicatorStateMachine::replicaSelectionHandler "
-			"best version selected: %s\n", newVersion.toString().GetCStr()); 
+			"best version selected: %s\n", newVersion.toString().Value()); 
     return true;
 }
 // until the state files merging utility is ready, the function is not really
@@ -501,8 +581,8 @@ ReplicatorStateMachine::onLeaderVersion( Stream* stream )
     if( downloadTransferersNumber( ) == 0 && newVersion && downloadNeeded ) {
         dprintf( D_FULLDEBUG, "ReplicatorStateMachine::onLeaderVersion "
 				"downloading from %s\n", 
-				newVersion->getSinfulString( ).GetCStr( ) );
-        download( newVersion->getSinfulString( ).GetCStr( ) );
+				newVersion->getSinfulString( ).Value( ) );
+        download( newVersion->getSinfulString( ).Value( ) );
     }
     // replication leader must not send a version which hasn't been updated
     //assert(downloadNeeded);
@@ -562,7 +642,7 @@ ReplicatorStateMachine::onSolicitVersionReply( Stream* stream )
  * Description: handler of REPLICATION_NEWLY_JOINED_VERSION command; void by now
  */
 void
-ReplicatorStateMachine::onNewlyJoinedVersion( Stream* stream )
+ReplicatorStateMachine::onNewlyJoinedVersion( Stream* /*stream*/ )
 {
     dprintf(D_ALWAYS, "ReplicatorStateMachine::onNewlyJoinedVersion started\n");
     
@@ -579,7 +659,7 @@ ReplicatorStateMachine::onNewlyJoinedVersion( Stream* stream )
  *				state)
  */
 void
-ReplicatorStateMachine::onGivingUpVersion( Stream* stream )
+ReplicatorStateMachine::onGivingUpVersion( Stream* /*stream*/ )
 {
     dprintf( D_ALWAYS, "ReplicatorStateMachine::onGivingUpVersion started\n" );
     
@@ -621,7 +701,9 @@ ReplicatorStateMachine::commandHandler( int command, Stream* stream )
    
     stream->decode( );
 
-    if( ! stream->code( daemonSinfulString ) /*|| ! stream->eom( )*/ ) {
+    if( ! stream->code( daemonSinfulString ) 
+		/*|| ! stream->end_of_message( )*/ ) 
+	{
         dprintf( D_NETWORK, "ReplicatorStateMachine::commandHandler "
                             "cannot read remote daemon sinful string for %s\n",
                  utilToString( command ) );
@@ -731,10 +813,10 @@ ReplicatorStateMachine::killStuckDownloadingTransferer( time_t currentTime )
         extension += ".";
         extension += DOWNLOADING_TEMPORARY_FILES_EXTENSION;
 
-        FilesOperations::safeUnlinkFile( m_versionFilePath.GetCStr( ),
-                                         extension.GetCStr( ) );
-        FilesOperations::safeUnlinkFile( m_stateFilePath.GetCStr( ),
-                                         extension.GetCStr( ) );
+        FilesOperations::safeUnlinkFile( m_versionFilePath.Value( ),
+                                         extension.Value( ) );
+        FilesOperations::safeUnlinkFile( m_stateFilePath.Value( ),
+                                         extension.Value( ) );
 		m_downloadTransfererMetadata.set( );
 	}
 }
@@ -776,10 +858,10 @@ ReplicatorStateMachine::killStuckUploadingTransferers( time_t currentTime )
             extension += ".";
             extension += UPLOADING_TEMPORARY_FILES_EXTENSION;
 
-            FilesOperations::safeUnlinkFile( m_versionFilePath.GetCStr( ),
-                                             extension.GetCStr( ) );
-            FilesOperations::safeUnlinkFile( m_stateFilePath.GetCStr( ),
-                                             extension.GetCStr( ) );
+            FilesOperations::safeUnlinkFile( m_versionFilePath.Value( ),
+                                             extension.Value( ) );
+            FilesOperations::safeUnlinkFile( m_stateFilePath.Value( ),
+                                             extension.Value( ) );
 			delete uploadTransfererMetadata;
 			m_uploadTransfererMetadataList.DeleteCurrent( );
 		}
@@ -867,7 +949,7 @@ ReplicatorStateMachine::versionRequestingTimer( )
     Version updatedVersion;
 
     if( replicaSelectionHandler( updatedVersion ) ) {
-        download( updatedVersion.getSinfulString( ).GetCStr( ) );
+        download( updatedVersion.getSinfulString( ).Value( ) );
         dprintf( D_FULLDEBUG, "ReplicatorStateMachine::versionRequestingTimer "
 				"registering version downloading timer\n" );
         m_versionDownloadingTimerId = daemonCore->Register_Timer( 
@@ -897,3 +979,14 @@ ReplicatorStateMachine::versionDownloadingTimer( )
 
 	m_state = BACKUP;
 }
+
+/* Function    : updateCollectors
+ * Description : sends the classad update to collectors
+ */
+void
+ReplicatorStateMachine::updateCollectors()
+{
+    if (m_classAd) {
+       daemonCore->sendUpdates (UPDATE_AD_GENERIC, m_classAd);
+    }
+}
