@@ -29,11 +29,22 @@
 
 #include "condor_state.h"
 
+#include <vector>
+#include <map>
+#include <set>
+#include <string>
+
+using std::vector;
+using std::map;
+using std::set;
+using std::string;
+
 // this is the required minimum separation between two priorities for them
 // to be considered distinct values
 static const float PriorityDelta = 0.5;
 
 class ClassAdLog;
+struct GroupEntry;
 
 class Accountant {
 
@@ -46,9 +57,10 @@ public:
   Accountant();
   ~Accountant();
 
-  void Initialize();  // Configuration
+  void Initialize(GroupEntry* group);  // Configuration
 
-  int GetResourcesUsed(const MyString& CustomerName); // get # of used resources
+  int GetResourcesUsed(const MyString& CustomerName); // get # of used resources (unweighted by SlotWeight)
+  float GetWeightedResourcesUsed(const MyString& CustomerName);
   float GetPriority(const MyString& CustomerName); // get priority for a customer
   void SetPriority(const MyString& CustomerName, float Priority); // set priority for a customer
 
@@ -65,20 +77,36 @@ public:
   void AddMatch(const MyString& CustomerName, ClassAd* ResourceAd); // Add new match
   void RemoveMatch(const MyString& ResourceName); // remove a match
 
+  float GetSlotWeight(ClassAd *candidate);
   void UpdatePriorities(); // update all the priorities
 
-  void CheckMatches(ClassAdList& ResourceList);  // Remove matches that are not claimed
+  void CheckMatches(ClassAdListDoesNotDeleteAds& ResourceList);  // Remove matches that are not claimed
 
   double GetLimit(const MyString& limit);
   double GetLimitMax(const MyString& limit);
   void ReportLimits(AttrList *attrList);
 
   AttrList* ReportState();
-  AttrList* ReportState(const MyString& CustomerName, int * NumResources = NULL);
+  AttrList* ReportState(const MyString& CustomerName, int * NumResources = NULL, float * NumResourcesRW = NULL);
                                                 
   void DisplayLog();
   void DisplayMatches();
 
+  ClassAd* GetClassAd(const MyString& Key);
+
+  // This maps submitter names to their assigned accounting group.
+  // When called with a defined group name, it maps that group name to itself.
+  GroupEntry* GetAssignedGroup(const MyString& CustomerName);
+  GroupEntry* GetAssignedGroup(const MyString& CustomerName, bool& IsGroup);
+
+  bool UsingWeightedSlots();
+
+  struct ci_less {
+      bool operator()(const string& a, const string& b) const {
+          return strcasecmp(a.c_str(), b.c_str()) < 0;
+      }
+  };
+
 private:
 
   //--------------------------------------------------------
@@ -87,7 +115,7 @@ private:
   
   void RemoveMatch(const MyString& ResourceName, time_t T);
 
-  void LoadLimits(ClassAdList &resourceList);
+  void LoadLimits(ClassAdListDoesNotDeleteAds &resourceList);
   void ClearLimits();
   void DumpLimits();
 
@@ -112,8 +140,8 @@ private:
   float HalfLifePeriod;     // The time in sec in which the priority is halved by aging
   MyString LogFileName;      // Name of Log file
   int	MaxAcctLogSize;		// Max size of log file
-  bool   DiscountSuspendedResources;
-  StringList *GroupNamesList;
+  bool  DiscountSuspendedResources;
+  bool  UseSlotWeights; 
 
   //--------------------------------------------------------
   // Data members
@@ -124,6 +152,9 @@ private:
 
   HashTable<MyString, double> concurrencyLimits;
 
+  GroupEntry* hgq_root_group;
+  map<string, GroupEntry*, ci_less> hgq_submitter_group_map;
+
   //--------------------------------------------------------
   // Static values
   //--------------------------------------------------------
@@ -132,20 +163,6 @@ private:
   static MyString CustomerRecord;
   static MyString ResourceRecord;
 
-  static MyString PriorityAttr;
-  static MyString UnchargedTimeAttr;
-  static MyString ResourcesUsedAttr;
-  static MyString AccumulatedUsageAttr;
-  static MyString BeginUsageTimeAttr;
-  static MyString LastUsageTimeAttr;
-  static MyString PriorityFactorAttr;
-
-  static MyString LastUpdateTimeAttr;
-
-  static MyString RemoteUserAttr;
-  static MyString StartTimeAttr;
-  static MyString Cpus;
-
   //--------------------------------------------------------
   // Utility functions
   //--------------------------------------------------------
@@ -154,10 +171,9 @@ private:
   bool GetResourceState(ClassAd* Resource, State& state);
   int IsClaimed(ClassAd* ResourceAd, MyString& CustomerName);
   int CheckClaimedOrMatched(ClassAd* ResourceAd, const MyString& CustomerName);
-  static ClassAd* FindResourceAd(const MyString& ResourceName, ClassAdList& ResourceList);
+  static ClassAd* FindResourceAd(const MyString& ResourceName, ClassAdListDoesNotDeleteAds& ResourceList);
   static MyString GetDomain(const MyString& CustomerName);
 
-  ClassAd* GetClassAd(const MyString& Key);
   bool DeleteClassAd(const MyString& Key);
 
   void SetAttributeInt(const MyString& Key, const MyString& AttrName, int AttrValue);
@@ -171,4 +187,7 @@ private:
   bool LoadState(const MyString& OldLogFileName);
 };
 
+
+extern void parse_group_name(const string& gname, vector<string>& gpath);
+
 #endif
