@@ -25,7 +25,8 @@
 /****
 ** Queue Manager Commands
 ****/
-#define QMGMT_CMD	1111
+#define QMGMT_READ_CMD	1111
+#define QMGMT_WRITE_CMD	1112
 
 /* Scheduler Commands */
 /*
@@ -37,7 +38,6 @@
 #define CA_AUTH_CMD_BASE	1000
 // beware, QMGMT_CMD is 1111, so we don't want to use 1100...
 #define CA_CMD_BASE			1200
-#define ALT_STARTER_BASE 	70
 
 /*
 **	In the following definitions 'FRGN' does not
@@ -66,7 +66,7 @@
 #define NUM_FRGN_JOBS		(SCHED_VERS+13)
 #define STARTD_INFO			(SCHED_VERS+14)
 #define SCHEDD_INFO			(SCHED_VERS+15)
-#define NEGOTIATE			(SCHED_VERS+16)
+#define NEGOTIATE			(SCHED_VERS+16) // 7.5.4+ negotiation command
 #define SEND_JOB_INFO		(SCHED_VERS+17)
 #define NO_MORE_JOBS		(SCHED_VERS+18)
 #define JOB_INFO			(SCHED_VERS+19)
@@ -143,7 +143,7 @@
 #define CHILD_ON            (SCHED_VERS+90) // Turn my child ON (HAD)
 #define CHILD_OFF           (SCHED_VERS+91) // Turn my child OFF (HAD)
 #define CHILD_OFF_FAST      (SCHED_VERS+92) // Turn my child OFF/Fast (HAD)
-#define NEGOTIATE_WITH_SIGATTRS	(SCHED_VERS+93)	// same as NEGOTIATE, but send sig attrs after Owner
+#define NEGOTIATE_WITH_SIGATTRS	(SCHED_VERS+93)	// pre 7.5.4 NEGOTIATE
 #define SET_ACCUMUSAGE	(SCHED_VERS+94)		// negotiator
 #define SET_BEGINTIME	(SCHED_VERS+95)		// negotiator
 #define SET_LASTTIME	(SCHED_VERS+96)		// negotiator
@@ -161,7 +161,10 @@
 #define TRANSFER_QUEUE_REQUEST (SCHED_VERS+107) // request to do file transfer
 
 #define SET_SHUTDOWN_PROGRAM (SCHED_VERS+108) // Master: Run program at shutdown
+#define GET_JOB_CONNECT_INFO (SCHED_VERS+109) // schedd: get connection information for starter running a job
 
+#define RECYCLE_SHADOW (SCHED_VERS+110) // schedd: get a new job for a shadow
+#define CLEAR_DIRTY_JOB_ATTRS (SCHED_VERS+111) // schedd: clear dirty attributes for a job
 
 // HAD-related commands
 #define HAD_ALIVE_CMD                   (HAD_COMMANDS_BASE + 0)
@@ -284,7 +287,29 @@ const int UPDATE_LEASE_MANAGER_AD		= 64;
 const int QUERY_LEASE_MANAGER_ADS		= 65;
 const int INVALIDATE_LEASE_MANAGER_ADS  = 66;
 
-const int QUERY_GENERIC_ADS = QUERY_ANY_ADS;	// == 48
+const int CCB_REGISTER = 67;
+const int CCB_REQUEST = 68;
+const int CCB_REVERSE_CONNECT = 69;
+
+const int UPDATE_GRID_AD 	= 70;
+const int QUERY_GRID_ADS	= 71;
+const int INVALIDATE_GRID_ADS = 72;
+
+const int MERGE_STARTD_AD = 73;
+const int QUERY_GENERIC_ADS = 74;
+
+const int SHARED_PORT_CONNECT = 75;
+const int SHARED_PORT_PASS_SOCK = 76;
+
+/*
+*** Commands to the starter
+*/
+
+#define STARTER_COMMANDS_BASE 1500
+#define STARTER_HOLD_JOB    (STARTER_COMMANDS_BASE+0)
+#define CREATE_JOB_OWNER_SEC_SESSION (STARTER_COMMANDS_BASE+1)
+#define START_SSHD (STARTER_COMMANDS_BASE+2)
+
 
 /*
 *** Daemon Core Signals
@@ -321,6 +346,8 @@ const int QUERY_GENERIC_ADS = QUERY_ANY_ADS;	// == 48
 #define DC_OFF_PEACEFUL     (DC_BASE+15)
 #define DC_SET_PEACEFUL_SHUTDOWN (DC_BASE+16)
 #define DC_TIME_OFFSET      (DC_BASE+17)
+#define DC_PURGE_LOG        (DC_BASE+18)
+#define DC_SHARE_SOCK       (DC_BASE+19)
 
 /*
 *** Log type supported by DC_FETCH_LOG
@@ -329,6 +356,9 @@ const int QUERY_GENERIC_ADS = QUERY_ANY_ADS;	// == 48
 */
 
 #define DC_FETCH_LOG_TYPE_PLAIN 0
+#define DC_FETCH_LOG_TYPE_HISTORY 1
+#define DC_FETCH_LOG_TYPE_HISTORY_DIR 2
+#define DC_FETCH_LOG_TYPE_HISTORY_PURGE 3
   /* Add more type here... */
 
 /*
@@ -367,6 +397,7 @@ const int QUERY_GENERIC_ADS = QUERY_ANY_ADS;	// == 48
 #define MPI_START_COMRADE      (DCSHADOW_BASE+2)  // for MPI & parallel shadow
 #define GIVE_MATCHES 	       (DCSHADOW_BASE+3)  // for MPI & parallel shadow
 #define RECEIVE_JOBAD		   (DCSHADOW_BASE+4)
+#define UPDATE_JOBAD		   (DCSHADOW_BASE+5)
 
 
 /*
