@@ -21,10 +21,6 @@
 #ifndef CONDOR_DEBUG_H
 #define CONDOR_DEBUG_H
 
-#if !defined(__STDC__) && !defined(__cplusplus)
-#define const
-#endif
-
 /*
 **	Definitions for flags to pass to dprintf
 **  Note: this is a little confusing, since the flags specify both
@@ -42,7 +38,7 @@
 #define D_SYSCALLS		(1<<1)
 #define D_CKPT			(1<<2)
 #define D_HOSTNAME		(1<<3)
-#define D_MALLOC		(1<<4)
+#define D_PERF_TRACE	(1<<4)
 #define D_LOAD			(1<<5)
 #define D_EXPR			(1<<6)
 #define D_PROC			(1<<7)
@@ -51,21 +47,19 @@
 #define D_FULLDEBUG	 	(1<<10)
 #define D_NFS			(1<<11)
 #define D_CONFIG        (1<<12)
-// D_AFS seems to not be used anywhere in Condor. Therefore I decided to recycle
-// this. -- Sonny.
-#define D_NET_REMAP       (1<<13)
-#define D_PREEMPT		(1<<14)
+#define D_UNUSED2       (1<<13)
+#define D_UNUSED3		(1<<14)
 #define D_PROTOCOL		(1<<15)
 #define D_PRIV			(1<<16)
 #define D_SECURITY		(1<<17)
 #define D_DAEMONCORE	(1<<18)
 #define D_COMMAND		(1<<19)
-#define D_BANDWIDTH		(1<<20)
+#define D_MATCH			(1<<20)
 #define D_NETWORK		(1<<21)
 #define D_KEYBOARD		(1<<22)
 #define D_PROCFAMILY	(1<<23)
 #define D_IDLE			(1<<24)
-#define D_MATCH			(1<<25)
+#define D_THREADS		(1<<25)
 #define D_ACCOUNTANT	(1<<26)
 #define D_FAILURE	(1<<27)
 /* 
@@ -74,8 +68,7 @@
 */ 
 #define D_PID           (1<<28)
 #define D_FDS           (1<<29)
-/* D_SECONDS is no longer used */
-#define D_SECONDS       (1<<30)
+#define D_UNUSED4       (1<<30)
 #define D_NOHEADER      (1<<31)
 #define D_ALL           (~(0) & (~(D_NOHEADER)))
 
@@ -85,7 +78,15 @@ extern "C" {
 
 extern int DebugFlags;	/* Bits to look for in dprintf */
 extern int Termlog;		/* Are we logging to a terminal? */
-extern int (*DebugId)(FILE *);		/* set header message */
+extern int DebugShouldLockToAppend; /* Should we lock the file before each write? */
+
+
+/* DebugId is a function that may be registered to be called to insert text
+ * into the header of a line that is about to be logged to the debug log file.
+ * It should treat its arguments similarly to how sprintf_realloc() does.
+ * It should also set its return value similarly to sprintf_realloc().
+ */
+extern int (*DebugId)(char **buf,int *bufpos,int *buflen);
 
 void dprintf ( int flags, const char *fmt, ... ) CHECK_PRINTF_FORMAT(2,3);
 
@@ -95,9 +96,11 @@ int _condor_open_lock_file(const char *filename,int flags, mode_t perm);
 void _EXCEPT_ ( const char *fmt, ... ) CHECK_PRINTF_FORMAT(1,2);
 void Suicide(void);
 void set_debug_flags( const char *strflags );
-void _condor_fd_panic( int line, char *file );
+void _condor_fd_panic( int line, const char *file );
 void _condor_set_debug_flags( const char *strflags );
 
+int  dprintf_config_ContinueOnFailure( int fContinue );
+
 /* must call this before clone(CLONE_VM|CLONE_VFORK) */
 void dprintf_before_shared_mem_clone( void );
 
@@ -118,6 +121,13 @@ void dprintf_dump_stack(void);
 time_t dprintf_last_modification(void);
 void dprintf_touch_log(void);
 
+/* reset statistics about delays acquiring the debug file lock */
+void dprintf_reset_lock_delay(void);
+
+/* return fraction of time spent waiting for debug file lock since
+   start of program or last call to dprintf_reset_lock_delay */
+double dprintf_get_lock_delay(void);
+
 /* wrapper for fclose() that soaks up EINTRs up to maxRetries number of times.
  */
 int fclose_wrapper( FILE *stream, int maxRetries );
@@ -134,7 +144,7 @@ int fclose_wrapper( FILE *stream, int maxRetries );
 /*
 **	Important external variables in libc
 */
-#if !( defined(LINUX) && defined(GLIBC) || defined(Darwin) || defined(CONDOR_FREEBSD) )
+#if !( defined(LINUX) || defined(Darwin) || defined(CONDOR_FREEBSD) )
 extern DLL_IMPORT_MAGIC int		errno;
 extern DLL_IMPORT_MAGIC int		sys_nerr;
 #if _MSC_VER < 1400 /* VC++ 2005 version */
@@ -145,7 +155,7 @@ extern DLL_IMPORT_MAGIC char	*sys_errlist[];
 extern int	_EXCEPT_Line;			/* Line number of the exception    */
 extern const char	*_EXCEPT_File;		/* File name of the exception      */
 extern int	_EXCEPT_Errno;			/* errno from most recent system call */
-extern int (*_EXCEPT_Cleanup)(int,int,char*);	/* Function to call to clean up (or NULL) */
+extern int (*_EXCEPT_Cleanup)(int,int,const char*);	/* Function to call to clean up (or NULL) */
 extern void _EXCEPT_(const char*, ...) CHECK_PRINTF_FORMAT(1,2);
 
 #if defined(__cplusplus)
@@ -186,6 +196,29 @@ char    *mymalloc(), *myrealloc(), *mycalloc();
         (ptr)->ru_stime.tv_usec ); \
 }
 
+#ifndef REMIND
+# ifdef _MSC_VER // for Microsoft C, prefix file and line to the the message
+#  define PRAGMA_QUOTE(x)   #x
+#  define PRAGMA_QQUOTE(y)  PRAGMA_QUOTE(y)
+#  define REMIND(str)       message(__FILE__ "(" PRAGMA_QQUOTE(__LINE__) ") : " str)
+# elif defined __GNUC__ // gcc emits file and line prefix automatically.
+#  define REMIND(str)       message str
+# else 
+# endif
+#endif // REMIND
+
+#if defined _MSC_VER && defined _DEBUG // WIN32
+# ifdef _X86_
+#  define DEBUG_BREAK_INTO_DEBUGGER _asm {int 3}
+# else
+#  define DEBUG_BREAK_INTO_DEBUGGER DebugBreak()
+# endif
+# define DEBUG_WAIT_FOR_DEBUGGER(var,def) { static int var=def; while (var) Sleep(1000); }
+#else
+# define DEBUG_BREAK_INTO_DEBUGGER ((void)0)
+# define DEBUG_WAIT_FOR_DEBUGGER(var,def) ((void)0)
+#endif
+
 #endif /* CONDOR_DEBUG_H */
 
 /* 
