@@ -44,6 +44,8 @@ class Condor_MD_MAC;
 #define GET_FILE_OPEN_FAILED -2
 #define PUT_FILE_OPEN_FAILED -2
 #define GET_FILE_WRITE_FAILED -3
+#define GET_FILE_PLUGIN_FAILED -4
+#define PUT_FILE_PLUGIN_FAILED -4
 
 class ReliSock : public Sock {
 	friend class Authentication;
@@ -53,6 +55,7 @@ class ReliSock : public Sock {
 public:
 
 	friend class DaemonCore;
+	friend class SharedPortEndpoint;
 
 	/*
 	**	Methods
@@ -66,6 +69,8 @@ public:
     ///
 	virtual int end_of_message();
 
+	virtual bool peek_end_of_message();
+
     /** Connect to a host on a port
         @param s can be a hostname or sinful string
         @param port the port to connect to, ignorred if s contains port
@@ -74,13 +79,21 @@ public:
 							bool do_not_block = false);
 
 
+	virtual int do_reverse_connect(char const *ccb_contact,bool nonblocking);
+
+	virtual void cancel_reverse_connect();
+
+	virtual int do_shared_port_local_connect( char const *shared_port_id, bool nonblocking );
+
 		/** Connect this socket to another socket (s).
 			An implementation of socketpair() that works on windows as well
 			as unix (by using the loopback interface).
 			@param sock - the socket to connect this socket to
+			@param use_standard_interface - if true, do not use loopback,
+			                                use normal interface
 			@returns true on success, false on failure.
 		 */
-	bool connect_socketpair(ReliSock &dest);
+	bool connect_socketpair(ReliSock &dest,bool use_standard_interface=false);
 
     ///
 	ReliSock();
@@ -133,9 +146,11 @@ public:
     /// returns <0 on failure, 0 for ok
 	//  failure codes: GET_FILE_OPEN_FAILED  (errno contains specific error)
 	//                 -1                    (all other errors)
-	int get_file( filesize_t *size, const char *destination, bool flush_buffers=false);
+	int get_file( filesize_t *size, const char *destination,
+				  bool flush_buffers=false, bool append=false);
     /// returns -1 on failure, 0 for ok
-	int get_file( filesize_t *size, int fd, bool flush_buffers=false);
+	int get_file( filesize_t *size, int fd,
+				  bool flush_buffers=false, bool append=false);
     /// returns <0 on failure, 0 for ok
 	//  See put_file() for the meaning of specific return codes.
 	int put_file_with_permissions( filesize_t *size, const char *source);
@@ -146,14 +161,18 @@ public:
 	// we can continue talking to the receiver, as though a file had
 	// been successfully sent.  In most cases, the next logical thing
 	// to do is to tell the receiver about the failure.
-	int put_file( filesize_t *size, const char *source);
+	int put_file( filesize_t *size, const char *source, filesize_t offset=0);
     /// returns -1 on failure, 0 for ok
-	int put_file( filesize_t *size, int fd );
+	int put_file( filesize_t *size, int fd, filesize_t offset=0 );
 	/// returns -1 on failure, 0 for ok
 	int get_x509_delegation( filesize_t *size, const char *destination,
 							 bool flush_buffers=false );
 	/// returns -1 on failure, 0 for ok
-	int put_x509_delegation( filesize_t *size, const char *source );
+	// expiration_time: 0 if none; o.w. timestamp of delegated proxy expiration
+	// result_expiration_time: if non-NULL will be set to actual expiration
+	//                         time of delegated proxy; could be shorter than
+	//                         requested time if source proxy expires sooner
+	int put_x509_delegation( filesize_t *size, const char *source, time_t expiration_time, time_t *result_expiration_time );
     ///
 	float get_bytes_sent() { return _bytes_sent; }
     ///
@@ -163,6 +182,13 @@ public:
     ///
 	void reset_bytes_recvd() { _bytes_recvd = 0; }
 
+	/// Used by CCBClient to put this socket in a state that behaves
+	/// like a socket waiting for a non-blocking connection when it
+	/// is actually waiting for a connection _to_ us _from_ the
+	/// desired endpoint.
+	void enter_reverse_connecting_state();
+	void exit_reverse_connecting_state(ReliSock *sock);
+
 #ifndef WIN32
 	// interface no longer supported 
 	int attach_to_file_desc(int);
@@ -189,20 +215,15 @@ public:
     ///
 	int authenticate( const char* methods, CondorError* errstack, int auth_timeout );
     ///
-	int authenticate( KeyInfo *& key, const char* methods, CondorError* errstack, int auth_timeout );
+	int authenticate( KeyInfo *& key, const char* methods, CondorError* errstack, int auth_timeout, char **method_used=NULL );
     ///
-    const char * getHostAddress();
-	
-	int encrypt(bool);
-	///
-	bool is_encrypt();
-	///
-	int hdr_encrypt();
-	///
-	bool is_hdr_encrypt();
-	///
 	int isClient() { return is_client; };
 
+	// Normally, the side of the connection that called connect() is
+	// the client.  The opposite is true for a reversed connection.
+	// This matters for the authentication protocol.
+	void isClient(bool flag) { is_client=flag; };
+
     const char * isIncomingDataMD5ed();
 
 
@@ -228,7 +249,7 @@ protected:
 	int prepare_for_nobuffering( stream_coding = stream_unknown);
 	int perform_authenticate( bool with_key, KeyInfo *& key, 
 							  const char* methods, CondorError* errstack,
-							  int auth_timeout );
+							  int auth_timeout, char **method_used );
 
 	// This is used internally to recover sanity on the stream after
 	// failing to open a file in put_file().
@@ -249,7 +270,7 @@ protected:
 	public:
 		RcvMsg();
                 ~RcvMsg();
-		int rcv_packet(SOCKET, int);
+		int rcv_packet(char const *peer_description, SOCKET, int);
 		void init_parent(ReliSock *tmp){ p_sock = tmp; } 
 
 		ChainBuf	buf;
@@ -265,7 +286,7 @@ protected:
 		SndMsg();
                 ~SndMsg();
 		Buf			buf;
-		int snd_packet(int, int, int);
+		int snd_packet(char const *peer_description, int, int, int);
 
 		//function to support the use of condor_read /write
 		
@@ -284,9 +305,16 @@ protected:
 	int	ignore_next_decode_eom;
 	float _bytes_sent, _bytes_recvd;
 
-	Authentication * authob;
 	int is_client;
 	char *hostAddr;
+	classy_counted_ptr<class CCBClient> m_ccb_client; // for reverse connects
+
+		// after connecting, request to be routed to this daemon
+	char *m_target_shared_port_id;
+
+	virtual void setTargetSharedPortID( char const *id );
+	virtual bool sendTargetSharedPortID();
+	char const *getTargetSharedPortID() { return m_target_shared_port_id; }
 };
 
 #endif
