@@ -32,6 +32,7 @@
 
 // Some error codes.  These should go to condor_errno.h once it is there.
 const int CEDAR_EWOULDBLOCK = 666;
+const int CEDAR_ENOCCB = 667;
 
 #if !defined(WIN32)
 #  ifndef SOCKET
@@ -82,6 +83,7 @@ public:
 	friend class Daemon;
 	friend class SecMan;
 	friend class SecManStartCommand;
+	friend class SharedPortListener;
 
 	/*
 	**	Methods
@@ -135,6 +137,7 @@ public:
 #if defined(WIN32) && defined(_WINSOCK2API_)
 	int assign(LPWSAPROTOCOL_INFO);		// to inherit sockets from other processes
 #endif
+
 	int bind(bool outbound, int port=0, bool loopback=false);
 
 	bool bind_to_loopback(bool outbound=false, int port=0);
@@ -152,9 +155,6 @@ public:
 	*/
 	int set_os_buffers(int desired_size, bool set_write_buf = false);
 
-	static int set_timeout_multiplier(int secs);
-	static int get_timeout_multiplier() { return timeout_multiplier; }
-	
 	inline int bind(bool outbound, char *s) { return bind(outbound, getportbyserv(s)); }
 
 	int close();
@@ -173,6 +173,9 @@ public:
 	*/
 	int timeout_no_timeout_multiplier(int sec);
     
+	/** Returns the timeout with timeout multiplier applied. */
+	int get_timeout_raw();
+
 	/** get the number of bytes available to read without blocking.
 		@return number of bytes, or -1 on failure
 	*/
@@ -182,29 +185,92 @@ public:
 	*/
 	bool readReady();
 
+        //------------------------------------------
+        // Encryption support below
+        //------------------------------------------
+        bool set_crypto_key(bool enable, KeyInfo * key, const char * keyId=0);
+        //------------------------------------------
+        // PURPOSE: set sock to use a particular encryption key
+        // REQUIRE: KeyInfo -- a wrapper for keyData
+        // RETURNS: true -- success; false -- failure
+        //------------------------------------------
+
+        bool wrap(unsigned char* input, int input_len, 
+                  unsigned char*& output, int& outputlen);
+        //------------------------------------------
+        // PURPOSE: encrypt some data
+        // REQUIRE: Protocol, keydata. set_encryption_procotol
+        //          must have been called and encryption_mode is on
+        // RETURNS: TRUE -- success, FALSE -- failure
+        //------------------------------------------
+
+        bool unwrap(unsigned char* input, int input_len, 
+                    unsigned char*& output, int& outputlen);
+        //------------------------------------------
+        // PURPOSE: decrypt some data
+        // REQUIRE: Protocol, keydata. set_encryption_procotol
+        //          must have been called and encryption_mode is on
+        // RETURNS: TRUE -- success, FALSE -- failure
+        //------------------------------------------
+
+        //----------------------------------------------------------------------
+        // MAC/MD related stuff
+        //----------------------------------------------------------------------
+        bool set_MD_mode(CONDOR_MD_MODE mode, KeyInfo * key = 0, const char * keyid = 0);    
+        //virtual bool set_MD_off() = 0;
+        //------------------------------------------
+        // PURPOSE: set mode for MAC (on or off)
+        // REQUIRE: mode -- see the enumeration defined above
+        //          key  -- an optional key for the MAC. if null (by default)
+        //                  all CEDAR does is send a Message Digest over
+        //                  When key is specified, this is essentially a MAC
+        // RETURNS: true -- success; false -- false
+        //------------------------------------------
+
+        bool isOutgoing_MD5_on() const { return (mdMode_ == MD_ALWAYS_ON); }
+        //------------------------------------------
+        // PURPOSE: whether MD is turned on or not
+        // REQUIRE: None
+        // RETURNS: true -- MD is on; 
+        //          false -- MD is off
+        //------------------------------------------
+
+        virtual const char * isIncomingDataMD5ed() = 0;
+        //------------------------------------------
+        // PURPOSE: To check to see if incoming data
+        //          has MD5 checksum/. NOTE! Currently,
+        //          this method should be used with UDP only!
+        // REQUIRE: None
+        // RETURNS: NULL -- data does not contain MD5
+        //          key id -- if the data is checksumed
+        //------------------------------------------
+
 	/*
 	**	Stream protocol
 	*/
 
     /// peer's port and IP address in a struct sockaddr_in.
-	struct sockaddr_in *endpoint();
+	struct sockaddr_in *peer_addr();
 
 	/// peer's port number
-	int endpoint_port();
+	int peer_port();
 
 	/// peer's IP address, string verison (e.g. "128.105.101.17")
-	const char* endpoint_ip_str();
+	const char* peer_ip_str();
 
 	/// peer's IP address, integer version (e.g. 2154390801)
-	unsigned int endpoint_ip_int();
+	unsigned int peer_ip_int();
+
+	/// is peer a local interface, aka did this connection originate from a local process?
+	bool peer_is_local();
 
     /// my port and IP address in a struct sockaddr_in
     /// @args: the address is returned via 'sin'
     /// @ret: 0 if succeed, -1 if failed
-    int mypoint(struct sockaddr_in *sin);
+    int my_addr(struct sockaddr_in *sin);
 
 	/// my IP address, string version (e.g. "128.105.101.17")
-	virtual const char* sender_ip_str();
+	virtual const char* my_ip_str();
 
 	/// local port number
 	int get_port();
@@ -215,9 +281,19 @@ public:
     /// sinful address of mypoint() in the form of "<a.b.c.d:pppp>"
     char * get_sinful();
 
+	/// Sinful address for access from outside of our private network.
+	/// This takes into account TCP_FORWARDING_HOST.
+	char const *get_sinful_public();
+
 	/// sinful address of peer in form of "<a.b.c.d:pppp>"
 	char * get_sinful_peer();
 
+		// Address that was passed to connect().  This is useful in cases
+		// such as CCB or shared port where our peer address is not the
+		// address one would use to actually connect to the peer.
+		// Returns NULL if connect was never called.
+	char const *get_connect_addr();
+
 	/// sinful address of peer, suitable for passing to dprintf() (never NULL)
 	virtual char const *default_peer_description();
 
@@ -225,10 +301,12 @@ public:
 	int get_file_desc() { return _sock; }
 
 	/// is a non-blocking connect outstanding?
-	bool is_connect_pending() { return _state == sock_connect_pending || _state == sock_connect_pending_retry; }
+	bool is_connect_pending() { return _state == sock_connect_pending || _state == sock_connect_pending_retry || _state == sock_reverse_connect_pending; }
+
+	bool is_reverse_connect_pending() { return _state == sock_reverse_connect_pending; }
 
 	/// is the socket connected?
-	bool is_connected() { return _state == sock_connect; }
+	bool is_connected() { return _state == sock_connect; }	
 
     /// 
 	virtual ~Sock();
@@ -238,15 +316,13 @@ public:
 
 	void doNotEnforceMinimalCONNECT_TIMEOUT() ;		// Used by HA Daemon
 
-	void ignoreTimeoutMultiplier() { ignore_timeout_multiplier = true; }
-
-	const char * getFullyQualifiedUser() const { return _fqu; }
+	const char * getFullyQualifiedUser() const;
 
 		/// Get user portion of fqu
-	const char *getOwner() const { return _fqu_user_part; }
+	const char *getOwner() const;
 
 		/// Get domain portion of fqu
-	const char *getDomain() const { return _fqu_domain_part; }
+	const char *getDomain() const;
 
 	void setFullyQualifiedUser(char const *fqu);
 
@@ -261,6 +337,21 @@ public:
 
 	void setTriedAuthentication(bool toggle) { _tried_authentication = toggle; }
 
+		/// Returns true if the fully qualified user name is
+		/// a non-anonymous user name (i.e. something not from
+		/// the unmapped domain)
+	bool isMappedFQU() const;
+
+		/// Returns true if the fully qualified user name was
+		/// authenticated
+	bool isAuthenticated() const;
+
+
+	/// if we are connecting, merges together Stream::get_deadline
+	/// and connect_timeout_time()
+	virtual time_t get_deadline();
+
+
 //	PRIVATE INTERFACE TO ALL SOCKS
 //
 protected:
@@ -272,8 +363,9 @@ protected:
 	*/
 
 	enum sock_state { sock_virgin, sock_assigned, sock_bound, sock_connect,
-						sock_writemsg, sock_readmsg, sock_special, 
-						sock_connect_pending, sock_connect_pending_retry };
+					  sock_writemsg, sock_readmsg, sock_special, 
+					  sock_connect_pending, sock_connect_pending_retry,
+					  sock_reverse_connect_pending };
 
 
 	/*
@@ -302,8 +394,25 @@ protected:
     */
 	int do_connect(char const *host, int port, bool non_blocking_flag = false);
 
+	/**
+	   Called internally by do_connect() to examine the address and
+	   see if we should try a reversed connection (CCB) instead of a
+	   normal forward connection.  If so, initiates the reversed
+	   connection.  If not, returns CEDAR_ENOCCB.  Also checks to
+	   see if we are connecting to a shared port (SharedPortServer)
+       requiring further directions to route us to our final destination.
+	*/
+	int special_connect(char const *host,int port,bool nonblocking);
+
+	virtual int do_reverse_connect(char const *ccb_contact,bool nonblocking) = 0;
+	virtual void cancel_reverse_connect() = 0;
+	virtual int do_shared_port_local_connect( char const *shared_port_id,bool nonblocking ) = 0;
+
+	void set_connect_addr(char const *addr);
+
 	inline SOCKET get_socket (void) { return _sock; }
 	char * serialize(char *);
+	static void close_serialized_socket(char const *buf);
 	char * serialize() const;
     char * serializeCryptoInfo(char * buf);
     char * serializeCryptoInfo() const;
@@ -318,7 +427,8 @@ protected:
     ///
 	virtual int authenticate(const char * auth_methods, CondorError* errstack, int timeout);
     ///
-	virtual int authenticate(KeyInfo *&ki, const char * auth_methods, CondorError* errstack, int timeout);
+	// method_used should be freed by the caller when finished with it
+	virtual int authenticate(KeyInfo *&ki, const char * auth_methods, CondorError* errstack, int timeout, char **method_used=NULL);
     ///
 	virtual bool is_encrypt();
 #ifdef WIN32
@@ -330,16 +440,28 @@ protected:
 
     ///
 	bool test_connection();
-	///
+	/// get timeout time for pending connect operation;
 	time_t connect_timeout_time();
 
-	/// if we are connecting, merges together Stream::get_deadline
-	/// and connect_timeout_time()
-	virtual time_t get_deadline();
-
 	///
 	int move_descriptor_up();
 
+
+    /// called whenever the bound or connected state changes
+    void addr_changed();
+
+	virtual void setTargetSharedPortID( char const *id ) = 0;
+	virtual bool sendTargetSharedPortID() = 0;
+
+	bool enter_connected_state(char const *op="CONNECT");
+
+	virtual bool init_MD(CONDOR_MD_MODE mode, KeyInfo * key, const char * keyId) = 0;
+	virtual bool set_encryption_id(const char * keyId) = 0;
+	const KeyInfo& get_crypto_key() const;
+	const KeyInfo& get_md_key() const;
+	void resetCrypto();
+	virtual bool canEncrypt();
+
 	/*
 	**	Data structures
 	*/
@@ -348,27 +470,35 @@ protected:
 	sock_state		_state;
 	int				_timeout;
 	struct sockaddr_in _who;	// endpoint of "connection"
+	char *			m_connect_addr;
 	char *          _fqu;
 	char *          _fqu_user_part;
 	char *          _fqu_domain_part;
 	bool            _tried_authentication;
 
-	static int timeout_multiplier;
-
 	bool ignore_connect_timeout;	// Used by HA Daemon
 
-	bool ignore_timeout_multiplier;
-
 	// Buffer to hold the string version of our own IP address. 
-	char _sender_ip_buf[IP_STRING_BUF_SIZE];	
+	char _my_ip_buf[IP_STRING_BUF_SIZE];	
+
+	Condor_Crypt_Base * crypto_;         // The actual crypto
+	CONDOR_MD_MODE      mdMode_;        // MAC mode
+	KeyInfo           * mdKey_;
 
 private:
+	bool initialize_crypto(KeyInfo * key);
+        //------------------------------------------
+        // PURPOSE: initialize crypto
+        // REQUIRE: KeyInfo
+        // RETURNS: None
+        //------------------------------------------
+
 	int _condor_read(SOCKET fd, char *buf, int sz, int timeout);
 	int _condor_write(SOCKET fd, char *buf, int sz, int timeout);
 	int bindWithin(const int low, const int high, bool outbound);
 	///
-	// Buffer to hold the string version of our endpoint's IP address. 
-	char _endpoint_ip_buf[IP_STRING_BUF_SIZE];	
+	// Buffer to hold the string version of our peer's IP address. 
+	char _peer_ip_buf[IP_STRING_BUF_SIZE];	
 
 	// Buffer to hold the sinful address of our peer
 	char _sinful_peer_buf[SINFUL_STRING_BUF_SIZE];
@@ -376,6 +506,9 @@ private:
 	// Buffer to hold the sinful address of ourself
 	char _sinful_self_buf[SINFUL_STRING_BUF_SIZE];
 
+	// Buffer to hold the public sinful address of ourself
+	char _sinful_public_buf[SINFUL_STRING_BUF_SIZE];
+
 	// struct to hold state info for do_connect() method
 	struct connect_state_struct {
 			int retry_timeout_interval;   // amount of time to keep retrying
@@ -446,7 +579,6 @@ private:
 	*/
 	int _bind_helper(int fd, SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE addr,
 		SOCKET_LENGTH_TYPE len, bool outbound, bool loopback);
-
 };
 
 #endif /* SOCK_H */
