@@ -27,9 +27,7 @@
 #include "MyString.h"
 #include "CryptKey.h"                 // KeyInfo
 #include "condor_ver_info.h"
-// This #define is a desperate move. GNU g++ seems to choke at runtime on our
-// inline function for eom.  Someday not needed ?
-#define eom end_of_message
+#include "classy_counted_ptr.h"
 
 enum CONDOR_MD_MODE {
     MD_OFF        = 0,         // off
@@ -161,7 +159,7 @@ const condor_mode_t NULL_FILE_PERMISSIONS = (condor_mode_t)0;
 	inline int MyObject::get(Stream &s) { s.decode(); return code(s); }
     </pre>
 */
-class Stream {
+class Stream: public ClassyCountedPtr {
 
 public:
 
@@ -221,7 +219,7 @@ public:
 	int code(long &);
     ///
 	int code(unsigned long &);
-#if !defined(__LP64__)
+#if !defined(__LP64__) || defined(Darwin)
     ///
 	int code(int64_t &);
     ///
@@ -239,6 +237,8 @@ public:
 	int code(char *&);
     ///
 	int code(MyString &);
+    ///
+	int code(std::string &);
     ///
 	int code(char *&, int &);
     ///
@@ -383,7 +383,7 @@ public:
 	int put(unsigned int);
 	int put(long);
 	int put(unsigned long);
-#if !defined(__LP64__)
+#if !defined(__LP64__) || defined(Darwin)
 	int put(int64_t);
 	int put(uint64_t);
 #endif
@@ -393,6 +393,7 @@ public:
 	int put(double);
 	int put(char const *);
 	int put(const MyString &);
+	int put(const std::string &);
 	int put(char const *, int);
 
 
@@ -405,7 +406,7 @@ public:
 	int get(unsigned int &);
 	int get(long &);
 	int get(unsigned long &);
-#if !defined(__LP64__)
+#if !defined(__LP64__) || defined(Darwin)
 	int get(int64_t &);
 	int get(uint64_t &);
 #endif
@@ -415,6 +416,7 @@ public:
 	int get(double &);
 
 	int get(MyString &);
+	int get(std::string &);
 
 		// This function assigns the argument to a freshly mallocated string
 		// or NULL.  The caller should free the string.
@@ -507,7 +509,9 @@ public:
         @return TRUE or FALSE
     */
 	virtual int end_of_message() = 0;
-//	int eom() { return end_of_message(); }
+
+	/* Returns true if at end of message. */
+	virtual bool peek_end_of_message() = 0;
 
 	///
 	virtual void allow_one_empty_message();
@@ -518,9 +522,6 @@ public:
 	/// get number of bytes currently available to read, -1 on failure
 	virtual int bytes_available_to_read() = 0;
 
-	/// get timeout time for pending connect operation;
-	virtual time_t connect_timeout_time() = 0;
-
 	/// set timeout in seconds for sum of all socket operations
 	/// After this amount of time (from now), all operations on
 	/// this socket (including non-blocking read/write) will
@@ -547,10 +548,14 @@ public:
 	bool deadline_expired();
 
 	/// For stream types that support it, this returns the ip address we are connecting from.
-	virtual char const *sender_ip_str() = 0;
+	virtual char const *my_ip_str() = 0;
 
 	/// For stream types that support it, this returns the ip address we are connecting to.
-	virtual char const *endpoint_ip_str() = 0;
+	virtual char const *peer_ip_str() = 0;
+
+	/// For stream types that support it, test if peer is a local interface, aka did this connection 
+	/// originate from a local process?
+	virtual bool peer_is_local() = 0;
 
 	/// For stream types that support it, this is the sinful address of peer.
 	virtual char const *default_peer_description() = 0;
@@ -587,99 +592,36 @@ public:
     ///
 	int rcv_int(int &val, int end_of_record);
 
-        //------------------------------------------
-        // Encryption support below
-        //------------------------------------------
-        bool set_crypto_key(bool enable, KeyInfo * key, const char * keyId=0);
-        //------------------------------------------
-        // PURPOSE: set sock to use a particular encryption key
-        // REQUIRE: KeyInfo -- a wrapper for keyData
-        // RETURNS: true -- success; false -- failure
-        //------------------------------------------
-
-        void set_crypto_mode(bool enable);
-        //------------------------------------------
-        // PURPOSE: enable or disable encryption
-        // REQUIRE: bool, true -- on; false -- off
-        // RETURNS:
-        //------------------------------------------
-
-        bool get_encryption() const;
+	bool get_encryption() const;
         //------------------------------------------
         // PURPOSE: Return encryption mode
         // REQUIRE: None
         // RETURNS: true -- on, false -- off
         //------------------------------------------
 
-        bool wrap(unsigned char* input, int input_len, 
-                  unsigned char*& output, int& outputlen);
+	void set_crypto_mode(bool enable);
         //------------------------------------------
-        // PURPOSE: encrypt some data
-        // REQUIRE: Protocol, keydata. set_encryption_procotol
-        //          must have been called and encryption_mode is on
-        // RETURNS: TRUE -- success, FALSE -- failure
+        // PURPOSE: enable or disable encryption
+        // REQUIRE: bool, true -- on; false -- off
+        // RETURNS:
         //------------------------------------------
 
-        bool unwrap(unsigned char* input, int input_len, 
-                    unsigned char*& output, int& outputlen);
-        //------------------------------------------
-        // PURPOSE: decrypt some data
-        // REQUIRE: Protocol, keydata. set_encryption_procotol
-        //          must have been called and encryption_mode is on
-        // RETURNS: TRUE -- success, FALSE -- failure
-        //------------------------------------------
+	/** Returns true if this stream can turn on encryption. */
+	virtual bool canEncrypt() = 0;
 
-        //----------------------------------------------------------------------
-        // MAC/MD related stuff
-        //----------------------------------------------------------------------
-        bool set_MD_mode(CONDOR_MD_MODE mode, KeyInfo * key = 0, const char * keyid = 0);    
-        //virtual bool set_MD_off() = 0;
-        //------------------------------------------
-        // PURPOSE: set mode for MAC (on or off)
-        // REQUIRE: mode -- see the enumeration defined above
-        //          key  -- an optional key for the MAC. if null (by default)
-        //                  all CEDAR does is send a Message Digest over
-        //                  When key is specified, this is essentially a MAC
-        // RETURNS: true -- success; false -- false
-        //------------------------------------------
+	static int set_timeout_multiplier(int secs);
+	static int get_timeout_multiplier();
 
-        bool isOutgoing_MD5_on() const { return (mdMode_ == MD_ALWAYS_ON); }
-        //------------------------------------------
-        // PURPOSE: whether MD is turned on or not
-        // REQUIRE: None
-        // RETURNS: true -- MD is on; 
-        //          false -- MD is off
-        //------------------------------------------
-
-        virtual const char * isIncomingDataMD5ed() = 0;
-        //------------------------------------------
-        // PURPOSE: To check to see if incoming data
-        //          has MD5 checksum/. NOTE! Currently,
-        //          this method should be used with UDP only!
-        // REQUIRE: None
-        // RETURNS: NULL -- data does not contain MD5
-        //          key id -- if the data is checksumed
-        //------------------------------------------
+	void ignoreTimeoutMultiplier() { ignore_timeout_multiplier = true; }
+	
     //@}
  private:
-        bool initialize_crypto(KeyInfo * key);
-        //------------------------------------------
-        // PURPOSE: initialize crypto
-        // REQUIRE: KeyInfo
-        // RETURNS: None
-        //------------------------------------------
         
 /*
 **		PRIVATE INTERFACE TO ALL STREAMS
 */
 protected:
 
-        virtual bool init_MD(CONDOR_MD_MODE mode, KeyInfo * key, const char * keyId) = 0;
-        virtual bool set_encryption_id(const char * keyId) = 0;
-        const KeyInfo& get_crypto_key() const;
-        const KeyInfo& get_md_key() const;
-      
-        void resetCrypto();
 
 	// serialize object (save/restore object state to an ascii string)
 	//
@@ -704,11 +646,9 @@ protected:
 	**	Data structures
 	*/
 
-        Condor_Crypt_Base * crypto_;         // The actual crypto
-        bool                crypto_mode_;    // true == enabled, false == disabled.
-        CONDOR_MD_MODE      mdMode_;        // MAC mode
-        KeyInfo           * mdKey_;
-        bool                encrypt_;        // Encryption mode
+	bool                encrypt_;        // Encryption mode
+	bool                crypto_mode_;    // true == enabled, false == disabled.
+	bool m_crypto_state_before_secret;
 	stream_code	    _code;
 	stream_coding	    _coding;
 
@@ -717,10 +657,11 @@ protected:
 	char *decrypt_buf;
 	int decrypt_buf_len;
 	char *m_peer_description_str;
-	bool m_crypto_state_before_secret;
 	CondorVersionInfo *m_peer_version;
 
 	time_t m_deadline_time;
+	static int timeout_multiplier;
+	bool ignore_timeout_multiplier;
 };
 
 
