@@ -178,7 +178,7 @@ bool _condorPacket::init_MD(const char * keyId)
  *	@return: true, if this packet is the whole message
  *         false, otherwise
  */
-int _condorPacket::getHeader(int msgsize,
+int _condorPacket::getHeader(int /* msgsize */,
                              bool &last,
                              int &seq,
                              int &len,
@@ -893,6 +893,8 @@ _condorInMsg::_condorInMsg(const _condorMsgID mID,// the id of this message
 	received = 1;
 	lastTime = time(NULL);
 	passed = 0;
+	curData = 0;
+	curPacket = 0;
 
 	// make directory entries as needed
 	headDir = curDir = new _condorDirPage(NULL, 0);
@@ -1181,15 +1183,15 @@ int _condorInMsg::getPtr(void *&buf, char delim)
 
 	if( D_FULLDEBUG & DebugFlags )
 		dprintf(D_NETWORK, "SafeMsg::_longMsg::getPtr:\
-		                    found delim = %c & length = %d\n",
-			  delim, n);
+		                    found delim = %c & length = %lu\n",
+			  delim, (unsigned long)n);
 	if( n > tempBufLen ) {
 		if(tempBuf) {
 			free(tempBuf);
 		}
 		tempBuf = (char *)malloc(n);
 		if(!tempBuf) {
-			dprintf(D_ALWAYS, "getPtr, fail at malloc(%d)\n", n);
+			dprintf(D_ALWAYS, "getPtr, fail at malloc(%lu)\n", (unsigned long)n);
 			tempBufLen = 0;
 			return -1;
 		}
