@@ -50,22 +50,25 @@
 
 MapFile* Authentication::global_map_file = NULL;
 bool Authentication::global_map_file_load_attempted = false;
+bool Authentication::globus_activated = false;
 
-char const *UNMAPPED_DOMAIN = "unmappeduser";
+char const *UNMAPPED_DOMAIN = "unmapped";
 char const *MATCHSESSION_DOMAIN = "matchsession";
+char const *UNAUTHENTICATED_FQU = "unauthenticated@unmapped";
+char const *UNAUTHENTICATED_USER = "unauthenticated";
 extern char const *EXECUTE_SIDE_MATCHSESSION_FQU = "execute-side@matchsession";
 extern char const *SUBMIT_SIDE_MATCHSESSION_FQU = "submit-side@matchsession";
+extern char const *CONDOR_CHILD_FQU = "condor@child";
+extern char const *CONDOR_PARENT_FQU = "condor@parent";
 
 Authentication::Authentication( ReliSock *sock )
 {
-#if !defined(SKIP_AUTHENTICATION)
+// Do this regardless of the state of SKIP_AUTHENTICATION)
+// even if SKIP_AUTHENTICATION is true, we call sock->Timeout later
 	mySock              = sock;
 	auth_status         = CAUTH_NONE;
 	method_used         = NULL;
-	t_mode              = NORMAL;
 	authenticator_      = NULL;
-#endif
-
 }
 
 Authentication::~Authentication()
@@ -225,8 +228,7 @@ int Authentication::authenticate_inner( char *hostAddr, const char* auth_methods
 				break;
  
 			case CAUTH_NONE:
-				dprintf(D_ALWAYS,"AUTHENTICATE: no available authentication methods succeeded, "
-						"failing!\n");
+				dprintf(D_SECURITY|D_FULLDEBUG,"AUTHENTICATE: no available authentication methods succeeded!\n");
 				errstack->push("AUTHENTICATE", AUTHENTICATE_ERR_OUT_OF_METHODS,
 						"Failed to authenticate with any method");
 				return 0;
@@ -247,7 +249,23 @@ int Authentication::authenticate_inner( char *hostAddr, const char* auth_methods
 		//------------------------------------------
 		// Now authenticate
 		//------------------------------------------
-		if (!auth->authenticate(hostAddr, errstack) ) {
+		bool auth_rc = auth->authenticate(hostAddr, errstack);
+
+			// check to see if the auth IP is the same as the socket IP
+		if( auth_rc ) {
+			char const *sockip = mySock->peer_ip_str();
+			char const *authip = auth->getRemoteHost() ;
+
+			auth_rc = !sockip || !authip || !strcmp(sockip,authip);
+
+			if (!auth_rc && !param_boolean( "DISABLE_AUTHENTICATION_IP_CHECK", false)) {
+				errstack->pushf("AUTHENTICATE", AUTHENTICATE_ERR_METHOD_FAILED,
+								"authenticated remote host does not match connection address (%s vs %s)", authip, sockip );
+				dprintf (D_ALWAYS, "AUTHENTICATE: ERROR: authenticated remot ehost does not match connection address (%s vs %s); configure DISABLE_AUTHENTICATION_IP_CHECK=TRUE if this check should be skipped\n",authip,sockip);
+			}
+		}
+
+		if( !auth_rc ) {
 			delete auth;
 			auth = NULL;
 
@@ -362,6 +380,12 @@ int Authentication::authenticate_inner( char *hostAddr, const char* auth_methods
 #endif /* SKIP_AUTHENTICATION */
 }
 
+void Authentication::reconfigMapFile()
+{
+	global_map_file_load_attempted = false;
+}
+
+
 #if !defined(SKIP_AUTHENTICATION)
 // takes the type (as defined in handshake bitmask, CAUTH_*) and result of authentication,
 // and maps it to the cannonical condor name.
@@ -397,12 +421,51 @@ void Authentication::map_authentication_name_to_canonical_name(int authenticatio
 		dprintf (D_SECURITY, "ZKM: map file already loaded.\n");
 	}
 
+#if defined(HAVE_EXT_GLOBUS)
+	if (globus_activated == false) {
+		dprintf (D_FULLDEBUG, "Activating Globus GSI_GSSAPI_ASSIST module.\n");
+		globus_module_activate(GLOBUS_GSI_GSS_ASSIST_MODULE);
+		globus_activated = true;
+	}
+#endif
+      
+
 	dprintf (D_SECURITY, "ZKM: attempting to map '%s'\n", authentication_name);
 
+	// this will hold what we pass to the mapping function
+	MyString auth_name_to_map = authentication_name;
+
+	bool included_voms = false;
+
+#if defined(HAVE_EXT_GLOBUS)
+	// if GSI, try first with the FQAN (dn plus voms attrs)
+	if (authentication_type == CAUTH_GSI) {
+		const char *fqan = ((Condor_Auth_X509*)authenticator_)->getFQAN();
+		if (fqan && fqan[0]) {
+			dprintf (D_ALWAYS, "ZKM: GSI was used, and FQAN is present.\n");
+			auth_name_to_map = fqan;
+			included_voms = true;
+		}
+	}
+#endif
+
 	if (global_map_file) {
 		MyString canonical_user;
-		if (!global_map_file->GetCanonicalization(method_string, authentication_name, canonical_user)) {
+
+		dprintf (D_ALWAYS, "ZKM: 1: attempting to map '%s'\n", auth_name_to_map.Value());
+		bool mapret = global_map_file->GetCanonicalization(method_string, auth_name_to_map.Value(), canonical_user);
+		dprintf (D_ALWAYS, "ZKM: 2: mapret: %i included_voms: %i canonical_user: %s\n", mapret, included_voms, canonical_user.Value());
+
+		// if it did not find a user, and we included voms attrs, try again without voms
+		if (mapret && included_voms) {
+			dprintf (D_ALWAYS, "ZKM: now attempting to map '%s'\n", authentication_name);
+			mapret = global_map_file->GetCanonicalization(method_string, authentication_name, canonical_user);
+			dprintf (D_ALWAYS, "ZKM: now 2: mapret: %i included_voms: %i canonical_user: %s\n", mapret, included_voms, canonical_user.Value());
+		}
+
+		if (!mapret) {
 			// returns true on failure?
+			dprintf (D_ALWAYS, "ZKM: successful mapping to %s\n", canonical_user.Value());
 
 			// there is a switch for GSI to use the default globus function for this, in
 			// case there is some custom globus mapping add-on, or the admin just wants
@@ -423,6 +486,7 @@ void Authentication::map_authentication_name_to_canonical_name(int authenticatio
 				// this api should actually just return the canonical user,
 				// and we should split it into user and domain in this function,
 				// and not invoke setRemoteFoo() directly in nameGssToLocal().
+				dprintf (D_ALWAYS, "ZKM: GRIDMAPPED!\n");
 #else
 				dprintf(D_ALWAYS, "ZKM: GSI not compiled, but was used?!!");
 #endif
@@ -446,6 +510,8 @@ void Authentication::map_authentication_name_to_canonical_name(int authenticatio
 		} else {
 			dprintf (D_ALWAYS, "ZKM: did not find user %s.\n", canonical_user.Value());
 		}
+	} else {
+		dprintf (D_ALWAYS, "ZKM: global_map_file not present!\n");
 	}
 
 }
@@ -652,62 +718,6 @@ bool Authentication :: is_valid()
     return valid;
 }
 
-int Authentication :: encrypt(bool flag)
-{
-    int code = 0;
-#if !defined(SKIP_AUTHENTICATION)
-    if (flag == TRUE) {
-        if (is_valid()){//check for flags to support shd be added 
-            t_mode = ENCRYPT;
-            code = 0;
-        }
-        else{
-            t_mode = NORMAL;
-            code = -1;
-        }
-    }
-    else {
-        t_mode = NORMAL;
-        code = 0;
-    }
-#endif
-    return code;
-}
-
-bool Authentication :: is_encrypt()
-{
-#if defined(SKIP_AUTHENTICATION)
-    return FALSE;
-#else
-    if ( t_mode ==  ENCRYPT || t_mode == ENCRYPT_HDR )
-        return TRUE;
-    return FALSE;
-#endif
-}
-
-int Authentication :: hdr_encrypt()
-{
-#if defined(SKIP_AUTHENTICATION)
-    return FALSE;
-#else
-    if ( t_mode == ENCRYPT){
-        t_mode = ENCRYPT_HDR;
-    }
-    return TRUE;
-#endif
-}
-
-bool Authentication :: is_hdr_encrypt(){
-#if defined(SKIP_AUTHENTICATION)
-    return FALSE;
-#else
-    if (t_mode  == ENCRYPT_HDR)  
-        return TRUE;
-    else 
-        return FALSE;
-#endif
-}
-
 int Authentication :: wrap(char*  input, 
 			   int    input_len, 
 			   char*& output,
