@@ -37,6 +37,12 @@
 #include "globus_utils.h"
 #include "condor_auth_x509.h"
 #include "condor_config.h"
+#include "ccb_client.h"
+#include "condor_sinful.h"
+#include "shared_port_client.h"
+#include "daemon_core_sock_adapter.h"
+#include "condor_netdb.h"
+#include "internet.h"
 
 #ifdef WIN32
 #include <mswsock.h>	// For TransmitFile()
@@ -49,17 +55,23 @@ const unsigned int PUT_FILE_EOM_NUM = 666;
 const int GET_FILE_NULL_FD = -10;
 
 int
-ReliSock::get_file( filesize_t *size, const char *destination, bool flush_buffers)
+ReliSock::get_file( filesize_t *size, const char *destination,
+					bool flush_buffers, bool append )
 {
 	int fd;
 	int result;
+	int flags = O_WRONLY | _O_BINARY | _O_SEQUENTIAL | O_LARGEFILE;
+
+	if ( append ) {
+		flags |= O_APPEND;
+	}
+	else {
+		flags |= O_CREAT | O_TRUNC;
+	}
 
 	// Open the file
 	errno = 0;
-	fd = ::safe_open_wrapper( destination,
-				 O_WRONLY | O_CREAT | O_TRUNC |
-				 _O_BINARY | _O_SEQUENTIAL | O_LARGEFILE,
-				 0600 );
+	fd = ::safe_open_wrapper( destination, flags, 0600 );
 
 	// Handle open failure; it's bad....
 	if ( fd < 0 )
@@ -76,7 +88,7 @@ ReliSock::get_file( filesize_t *size, const char *destination, bool flush_buffer
 
 			// In order to remain in a well-defined state on the wire
 			// protocol, read and throw away the file data.
-		result = get_file( size, GET_FILE_NULL_FD, flush_buffers );
+		result = get_file( size, GET_FILE_NULL_FD, flush_buffers, append );
 		if( result<0 ) {
 				// Failure to read (and throw away) data indicates that
 				// we are in an undefined state on the wire protocol
@@ -89,13 +101,16 @@ ReliSock::get_file( filesize_t *size, const char *destination, bool flush_buffer
 		return GET_FILE_OPEN_FAILED;
 	} 
 
-	dprintf(D_FULLDEBUG,"get_file(): going to write to filename %s\n",
-		destination);
+	dprintf( D_FULLDEBUG,
+			 "get_file(): going to write to filename %s\n",
+			 destination);
 
-	result = get_file( size, fd,flush_buffers);
+	result = get_file( size, fd,flush_buffers, append);
 
 	if(::close(fd)!=0) {
-		dprintf(D_ALWAYS, "ReliSock: get_file: close failed, errno = %d (%s)\n", errno, strerror(errno));
+		dprintf(D_ALWAYS,
+				"ReliSock: get_file: close failed, errno = %d (%s)\n",
+				errno, strerror(errno));
 		return -1;
 	}
 
@@ -105,10 +120,11 @@ ReliSock::get_file( filesize_t *size, const char *destination, bool flush_buffer
 }
 
 int
-ReliSock::get_file( filesize_t *size, int fd, bool flush_buffers )
+ReliSock::get_file( filesize_t *size, int fd,
+					bool flush_buffers, bool append )
 {
 	char buf[65536];
-	filesize_t filesize;
+	filesize_t filesize, bytes_to_receive;
 	unsigned int eom_num;
 	filesize_t total = 0;
 	int retval = 0;
@@ -120,14 +136,18 @@ ReliSock::get_file( filesize_t *size, int fd, bool flush_buffers )
 	// Read the filesize from the other end of the wire
 	if ( !get(filesize) || !end_of_message() ) {
 		dprintf(D_ALWAYS, 
-			"Failed to receive filesize in ReliSock::get_file\n");
+				"Failed to receive filesize in ReliSock::get_file\n");
 		return -1;
 	}
-
+	bytes_to_receive = filesize;
+	if ( append ) {
+		lseek( fd, 0, SEEK_END );
+	}
 
 	// Log what's going on
-	dprintf(D_FULLDEBUG,
-			"get_file: Receiving " FILESIZE_T_FORMAT " bytes\n", filesize );
+	dprintf( D_FULLDEBUG,
+			 "get_file: Receiving " FILESIZE_T_FORMAT " bytes\n",
+			 bytes_to_receive );
 
 		/*
 		  the code used to check for filesize == -1 here, but that's
@@ -139,9 +159,9 @@ ReliSock::get_file( filesize_t *size, int fd, bool flush_buffers )
 		*/
 
 	// Now, read it all in & save it
-	while( total < filesize ) {
-		int	iosize = (int) MIN( (filesize_t) sizeof(buf),
-									filesize - total );
+	while( total < bytes_to_receive ) {
+		int	iosize =
+			(int) MIN( (filesize_t) sizeof(buf), bytes_to_receive - total );
 		int	nbytes = get_bytes_nobuffer( buf, iosize, 0 );
 		if ( nbytes <= 0 ) {
 			break;
@@ -184,7 +204,8 @@ ReliSock::get_file( filesize_t *size, int fd, bool flush_buffers )
 					  do more fancy stuff to handle this case, but
 					  we're probably never going to see this anyway.
 					*/
-				dprintf( D_ALWAYS, "ReliSock::get_file: write() returned 0: "
+				dprintf( D_ALWAYS,
+						 "ReliSock::get_file: write() returned 0: "
 						 "wrote %d out of %d bytes (errno=%d %s)\n",
 						 written, nbytes, errno, strerror(errno) );
 				break;
@@ -197,7 +218,7 @@ ReliSock::get_file( filesize_t *size, int fd, bool flush_buffers )
 
 	if ( filesize == 0 ) {
 		if ( !get(eom_num) || eom_num != PUT_FILE_EOM_NUM ) {
-			dprintf(D_ALWAYS,"get_file: Zero-length file check failed!\n");
+			dprintf( D_ALWAYS, "get_file: Zero-length file check failed!\n" );
 			return -1;
 		}			
 	}
@@ -244,7 +265,7 @@ ReliSock::put_empty_file( filesize_t *size )
 }
 
 int
-ReliSock::put_file( filesize_t *size, const char *source)
+ReliSock::put_file( filesize_t *size, const char *source, filesize_t offset)
 {
 	int fd;
 	int result;
@@ -269,13 +290,15 @@ ReliSock::put_file( filesize_t *size, const char *source)
 		return PUT_FILE_OPEN_FAILED;
 	}
 
-	dprintf(D_FULLDEBUG,"put_file: going to send from filename %s\n",
-		source);
+	dprintf( D_FULLDEBUG,
+			 "put_file: going to send from filename %s\n", source);
 
-	result = put_file( size, fd);
+	result = put_file( size, fd, offset );
 
 	if (::close(fd) < 0) {
-		dprintf(D_ALWAYS, "ReliSock: put_file: close failed, errno = %d (%s)\n", errno, strerror(errno));
+		dprintf(D_ALWAYS,
+				"ReliSock: put_file: close failed, errno = %d (%s)\n",
+				errno, strerror(errno));
 		return -1;
 	}
 
@@ -283,7 +306,7 @@ ReliSock::put_file( filesize_t *size, const char *source)
 }
 
 int
-ReliSock::put_file( filesize_t *size, int fd )
+ReliSock::put_file( filesize_t *size, int fd, filesize_t offset )
 {
 	filesize_t	filesize;
 	filesize_t	total = 0;
@@ -319,26 +342,42 @@ ReliSock::put_file( filesize_t *size, int fd )
 	}
 
 	filesize = filestat.GetFileSize( );
-	dprintf( D_FULLDEBUG, "put_file: Found file size %lld\n", filesize );
+	dprintf( D_FULLDEBUG,
+			 "put_file: Found file size " FILESIZE_T_FORMAT "\n",
+			 filesize );
+	
+	if ( offset > filesize ) {
+		dprintf( D_ALWAYS,
+				 "ReliSock::put_file: offset " FILESIZE_T_FORMAT
+				 " is larger than file " FILESIZE_T_FORMAT "!\n",
+				 offset, filesize );
+	}
+	filesize_t	bytes_to_send = filesize - offset;
 
-	// Send the file size to the reciever
-	if ( !put(filesize) || !end_of_message() ) {
+	// Send the file size to the receiver
+	if ( !put(bytes_to_send) || !end_of_message() ) {
 		dprintf(D_ALWAYS, "ReliSock: put_file: Failed to send filesize.\n");
 		return -1;
 	}
 
+	if ( offset ) {
+		lseek( fd, offset, SEEK_SET );
+	}
+
 	// Log what's going on
 	dprintf(D_FULLDEBUG,
-			"put_file: sending " FILESIZE_T_FORMAT " bytes\n", filesize);
+			"put_file: sending " FILESIZE_T_FORMAT " bytes\n", bytes_to_send );
 
 	// If the file has a non-zero size, send it
-	if ( filesize > 0 ) {
+	if ( bytes_to_send > 0 ) {
 
 #if defined(WIN32)
 		// On Win32, if we don't need encryption, use the super-efficient Win32
 		// TransmitFile system call. Also, TransmitFile does not support
 		// file sizes over 2GB, so we avoid that case as well.
-		if ( !get_encryption() && filesize < INT_MAX ) {
+		if (  (!get_encryption()) &&
+			  (0 == offset) &&
+			  (bytes_to_send < INT_MAX)  ) {
 
 			// First drain outgoing buffers
 			if ( !prepare_for_nobuffering(stream_encode) ) {
@@ -349,14 +388,14 @@ ReliSock::put_file( filesize_t *size, int fd )
 
 			// Now transmit file using special optimized Winsock call
 			if ( TransmitFile(_sock,(HANDLE)_get_osfhandle(fd),
-							  filesize,0,NULL,NULL,0) == FALSE ) {
+							  bytes_to_send,0,NULL,NULL,0) == FALSE ) {
 				dprintf(D_ALWAYS,
 						"ReliSock: put_file: TransmitFile() failed, errno=%d\n",
 						GetLastError() );
 				return -1;
 			} else {
 				// Note that it's been sent, so that we don't try to below
-				total = filesize;
+				total = bytes_to_send;
 			}
 		}
 #endif
@@ -367,7 +406,7 @@ ReliSock::put_file( filesize_t *size, int fd )
 		// On Unix, always send the file using put_bytes_nobuffer().
 		// Note that on Win32, we use this method as well if encryption 
 		// is required.
-		while (total < filesize &&
+		while (total < bytes_to_send &&
 			   (nrd = ::read(fd, buf, sizeof(buf))) > 0) {
 			if ((nbytes = put_bytes_nobuffer(buf, nrd, 0)) < nrd) {
 					// put_bytes_nobuffer() does the appropriate
@@ -392,10 +431,11 @@ ReliSock::put_file( filesize_t *size, int fd )
 	dprintf(D_FULLDEBUG,
 			"ReliSock: put_file: sent " FILESIZE_T_FORMAT " bytes\n", total);
 
-	if (total < filesize) {
-		dprintf(D_ALWAYS,"ReliSock: put_file: only sent " FILESIZE_T_FORMAT 
-					" bytes out of " FILESIZE_T_FORMAT "\n",
-			total, filesize);
+	if (total < bytes_to_send) {
+		dprintf(D_ALWAYS,
+				"ReliSock: put_file: only sent " FILESIZE_T_FORMAT 
+				" bytes out of " FILESIZE_T_FORMAT "\n",
+				total, filesize);
 		return -1;
 	}
 
@@ -426,6 +466,10 @@ ReliSock::get_file_with_permissions( filesize_t *size,
 		return result;
 	}
 
+	if( destination && !strcmp(destination,NULL_FILE) ) {
+		return result;
+	}
+
 		// If the other side told us to ignore its permissions, then we're
 		// done.
 	if ( file_mode == NULL_FILE_PERMISSIONS ) {
@@ -570,7 +614,7 @@ ReliSock::get_x509_delegation( filesize_t *size, const char *destination,
 }
 
 int
-ReliSock::put_x509_delegation( filesize_t *size, const char *source )
+ReliSock::put_x509_delegation( filesize_t *size, const char *source, time_t expiration_time, time_t *result_expiration_time )
 {
 	int in_encode_mode;
 
@@ -584,7 +628,7 @@ ReliSock::put_x509_delegation( filesize_t *size, const char *source )
 		return -1;
 	}
 
-	if ( x509_send_delegation( source, relisock_gsi_get, (void *) this,
+	if ( x509_send_delegation( source, expiration_time, result_expiration_time, relisock_gsi_get, (void *) this,
 							   relisock_gsi_put, (void *) this ) != 0 ) {
 		dprintf( D_ALWAYS, "ReliSock::put_x509_delegation(): delegation "
 				 "failed: %s\n", x509_error_string() );
@@ -665,11 +709,11 @@ int relisock_gsi_put(void *arg,  void *buf, size_t size)
     //if successful, send the data
     if ( stat ) {
         if ( !(stat = sock->code_bytes( buf, ((int) size )) ) ) {
-            dprintf( D_ALWAYS, "failure sending data (%d bytes) over sock\n",size);
+            dprintf( D_ALWAYS, "failure sending data (%lu bytes) over sock\n",(unsigned long)size);
         }
     }
     else {
-        dprintf( D_ALWAYS, "failure sending size (%d) over sock\n", size );
+        dprintf( D_ALWAYS, "failure sending size (%lu) over sock\n", (unsigned long)size );
     }
     
     sock->end_of_message();
@@ -682,38 +726,214 @@ int relisock_gsi_put(void *arg,  void *buf, size_t size)
     return 0;
 }
 
-bool
-Stream::prepare_crypto_for_secret_is_noop()
+
+int Sock::special_connect(char const *host,int /*port*/,bool nonblocking)
 {
-	CondorVersionInfo const *peer_ver = get_peer_version();
-	if( !peer_ver || peer_ver->built_since_version(7,1,3) ) {
-		if( param_boolean("ENCRYPT_SECRETS",true) ) {
-			if( !get_encryption() ) {
-				if( crypto_ ) {
-						// do turn on encryption before sending secret
-					return false;
+	if( !host || *host != '<' ) {
+		return CEDAR_ENOCCB;
+	}
+
+	Sinful sinful(host);
+	if( !sinful.valid() ) {
+		return CEDAR_ENOCCB;
+	}
+
+	char const *shared_port_id = sinful.getSharedPortID();
+	if( shared_port_id ) {
+			// If the port of the SharedPortServer is 0, that means
+			// we do not know the address of the SharedPortServer.
+			// This happens, for example when Create_Process passes
+			// the parent's address to a child or the child's address
+			// to the parent and the SharedPortServer does not exist yet.
+			// So do a local connection bypassing SharedPortServer,
+			// if we are on the same machine.
+
+			// Another case where we want to bypass connecting to
+			// SharedPortServer is if we are the shared port server,
+			// because this causes us to hang.
+
+			// We could additionally bypass using the shared port
+			// server if we use the same shared port server as the
+			// target daemon and we are on the same machine.  However,
+			// this causes us to use an additional ephemeral port than
+			// if we connect to the shared port, so in case that is
+			// important, use the shared port server instead.
+
+		bool no_shared_port_server =
+			sinful.getPort() && strcmp(sinful.getPort(),"0")==0;
+
+		bool same_host = false;
+		char const *my_ip = my_ip_string();
+		if( my_ip && sinful.getHost() && strcmp(my_ip,sinful.getHost())==0 ) {
+			same_host = true;
+		}
+
+		bool i_am_shared_port_server = false;
+		if( daemonCoreSockAdapter.isEnabled() ) {
+			char const *daemon_addr = daemonCoreSockAdapter.publicNetworkIpAddr();
+			if( daemon_addr ) {
+				Sinful my_sinful(daemon_addr);
+				if( my_sinful.getHost() && sinful.getHost() &&
+					strcmp(my_sinful.getHost(),sinful.getHost())==0 &&
+					my_sinful.getPort() && sinful.getPort() &&
+					strcmp(my_sinful.getPort(),sinful.getPort())==0 &&
+					(!my_sinful.getSharedPortID() ||
+					 strcmp(my_sinful.getSharedPortID(),shared_port_id)==0) )
+				{
+					i_am_shared_port_server = true;
+					dprintf(D_FULLDEBUG,"Bypassing connection to shared port server %s, because that is me.\n",daemon_addr);
 				}
 			}
 		}
+		if( (no_shared_port_server && same_host) || i_am_shared_port_server ) {
+
+			if( no_shared_port_server && same_host ) {
+				dprintf(D_FULLDEBUG,"Bypassing connection to shared port server, because its address is not yet established; passing socket directly to %s.\n",host);
+			}
+
+			return do_shared_port_local_connect( shared_port_id,nonblocking );
+		}
 	}
-	return true;
+
+		// Set shared port id even if it is null so we clear whatever may
+		// already be there.  If it is not null, then this information
+		// is saved here and used later after we have connected.
+	setTargetSharedPortID( shared_port_id );
+
+	char const *ccb_contact = sinful.getCCBContact();
+	if( !ccb_contact || !*ccb_contact ) {
+		return CEDAR_ENOCCB;
+	}
+
+	return do_reverse_connect(ccb_contact,nonblocking);
+}
+
+int
+SafeSock::do_reverse_connect(char const *,bool)
+{
+	dprintf(D_ALWAYS,
+			"CCBClient: WARNING: UDP not supported by CCB."
+			"  Will therefore try to send packet directly to %s.\n",
+			peer_description());
+
+	return CEDAR_ENOCCB;
+}
+
+int
+ReliSock::do_reverse_connect(char const *ccb_contact,bool nonblocking)
+{
+	ASSERT( !m_ccb_client.get() ); // only one reverse connect at a time!
+	m_ccb_client =
+		new CCBClient( ccb_contact, (ReliSock *)this );
+
+	if( !m_ccb_client->ReverseConnect(NULL,nonblocking) ) {
+		dprintf(D_ALWAYS,"Failed to reverse connect to %s via CCB.\n",
+				peer_description());
+		return 0;
+	}
+	if( nonblocking ) {
+		return CEDAR_EWOULDBLOCK;
+	}
+
+	m_ccb_client = NULL; // in blocking case, we are done with ccb client
+	return 1;
+}
+
+int
+SafeSock::do_shared_port_local_connect( char const *, bool )
+{
+	dprintf(D_ALWAYS,
+			"SharedPortClient: WARNING: UDP not supported."
+			"  Failing to connect to %s.\n",
+			peer_description());
+
+	return 0;
+}
+
+int
+ReliSock::do_shared_port_local_connect( char const *shared_port_id, bool nonblocking )
+{
+		// Without going through SharedPortServer, we want to connect
+		// to a daemon that is local to this machine and which is set up
+		// to use the local SharedPortServer.  We do this by creating
+		// a connected socket pair and then passing one of those sockets
+		// to the target daemon over its named socket (or whatever mechanism
+		// this OS supports).
+
+	SharedPortClient shared_port_client;
+	ReliSock sock_to_pass;
+		// do not use loopback interface for socketpair unless this happens
+		// to be the standard network interface, because localhost
+		// typically does not happen to be allowed in the authorization policy
+	const bool use_standard_interface = true;
+	if( !connect_socketpair(sock_to_pass,use_standard_interface) ) {
+		dprintf(D_ALWAYS,
+				"Failed to connect to loopback socket, so failing to connect via local shared port access to %s.\n",
+				peer_description());
+		return 0;
+	}
+
+	char const *request_by = "";
+	if( !shared_port_client.PassSocket(&sock_to_pass,shared_port_id,request_by) ) {
+		return 0;
+	}
+
+	if( nonblocking ) {
+			// We must pretend that we are not yet connected so that callers
+			// who want a non-blocking connect will get the expected behavior
+			// from Register_Socket() (register for write rather than read).
+		_state = sock_connect_pending;
+		return CEDAR_EWOULDBLOCK;
+	}
+
+	enter_connected_state();
+	return 1;
 }
 
 void
-Stream::prepare_crypto_for_secret()
-{
-	m_crypto_state_before_secret = true;
-	if( !prepare_crypto_for_secret_is_noop() ) {
-		dprintf(D_NETWORK,"encrypting secret\n");
-		m_crypto_state_before_secret = get_encryption(); // always false
-		set_crypto_mode(true);
-	}
+ReliSock::cancel_reverse_connect() {
+	ASSERT( m_ccb_client.get() );
+	m_ccb_client->CancelReverseConnect();
 }
 
-void
-Stream::restore_crypto_after_secret()
+bool
+ReliSock::sendTargetSharedPortID()
 {
-	if( !m_crypto_state_before_secret ) {
-		set_crypto_mode(false); //restore crypto mode
+	char const *shared_port_id = getTargetSharedPortID();
+	if( !shared_port_id ) {
+		return true;
 	}
+	SharedPortClient shared_port;
+	return shared_port.sendSharedPortID(shared_port_id,this);
+}
+
+char const *
+Sock::get_sinful_public()
+{
+		// In case TCP_FORWARDING_HOST changes, do not cache it.
+	MyString tcp_forwarding_host;
+	param(tcp_forwarding_host,"TCP_FORWARDING_HOST");
+	if (!tcp_forwarding_host.IsEmpty()) {
+		struct sockaddr_in sin;
+		if (!is_ipaddr(tcp_forwarding_host.Value(), &sin.sin_addr)) {
+			struct hostent *he = condor_gethostbyname(tcp_forwarding_host.Value());
+			if (he == NULL) {
+				dprintf(D_ALWAYS,
+					"failed to resolve address of TCP_FORWARDING_HOST=%s\n",
+					tcp_forwarding_host.Value());
+				return NULL;
+			}
+			sin.sin_addr = *(in_addr*)(he->h_addr_list[0]);;
+		}
+		sin.sin_port = htons(get_port());
+		char const *addr = sin_to_string(&sin);
+		if( !addr ) {
+			return NULL;
+		}
+		strncpy(_sinful_public_buf,addr,SINFUL_STRING_BUF_SIZE);
+		_sinful_public_buf[SINFUL_STRING_BUF_SIZE-1] = '\0';
+		return _sinful_public_buf;
+	}
+
+	return get_sinful();
 }
