@@ -188,8 +188,8 @@ int Condor_Auth_Kerberos :: wrap(char*  input,
     out_data.ciphertext.data = (char*)encrypted_data;
 	out_data.ciphertext.length = encrypted_length;
 
-    if (code = krb5_c_encrypt(krb_context_, sessionKey_, 1024, /* key usage */
-				0, &in_data, &out_data)) {			/* 0 = no ivec */
+    if ((code = krb5_c_encrypt(krb_context_, sessionKey_, 1024, /* key usage */
+				0, &in_data, &out_data)) != 0) {			/* 0 = no ivec */
         output     = 0;
         output_len = 0;
         if (out_data.ciphertext.data) {    
@@ -218,9 +218,9 @@ int Condor_Auth_Kerberos :: wrap(char*  input,
     memcpy(output + index, &tmp, sizeof(out_data.ciphertext.length));
     index += sizeof(out_data.ciphertext.length);
 
-    memcpy(output + index, out_data.ciphertext.data, out_data.ciphertext.length);
-
     if (out_data.ciphertext.data) {    
+	memcpy(output + index, out_data.ciphertext.data,
+		out_data.ciphertext.length);
         free(out_data.ciphertext.data);
     }
 
@@ -228,9 +228,9 @@ int Condor_Auth_Kerberos :: wrap(char*  input,
 }
 
 int Condor_Auth_Kerberos :: unwrap(char*  input, 
-                                   int    input_len, 
+                                   int    /* input_len */, 
                                    char*& output, 
-                                   int&   output_len)
+                                   int& output_len)
 {
     krb5_error_code code;
     krb5_data       out_data;
@@ -267,8 +267,8 @@ int Condor_Auth_Kerberos :: unwrap(char*  input,
     out_data.length = enc_data.ciphertext.length;
 	out_data.data = (char*)malloc(out_data.length);
 
-	if (code = krb5_c_decrypt(krb_context_, sessionKey_, 1024, /* key usage */
-				0, &enc_data, &out_data)) {			/* 0 = no ivec */
+	if ((code = krb5_c_decrypt(krb_context_, sessionKey_, 1024, /* key usage */
+				0, &enc_data, &out_data))!=0) {			/* 0 = no ivec */
 
 	//if (code = krb5_decrypt_data(krb_context_, sessionKey_, 0, &enc_data, &out_data)) {
         output_len = 0;
@@ -381,7 +381,7 @@ int Condor_Auth_Kerberos :: init_daemon()
 	dprintf(D_SECURITY, "init_daemon: Trying to get tgt credential for service %s\n", sname.Value());
 
 	priv = set_root_priv();   // Get the old privilige
-	code = krb5_get_init_creds_keytab(krb_context_, creds_, krb_principal_, keytab, 0, (char*)sname.Value(), 0);
+	code = krb5_get_init_creds_keytab(krb_context_, creds_, krb_principal_, keytab, 0, const_cast<char*>(sname.Value()), 0);
 	set_priv(priv);
 	if(code) {
 		goto error;
@@ -1127,9 +1127,9 @@ int Condor_Auth_Kerberos :: init_server_info()
 		if (mySock_->isClient()) {
 			if (instance == 0) {
 				struct hostent * hp;
-				hp = condor_gethostbyaddr((char *) &(mySock_->endpoint())->sin_addr, 
+				hp = condor_gethostbyaddr((char *) &(mySock_->peer_addr())->sin_addr, 
 								   sizeof (struct in_addr),
-								   mySock_->endpoint()->sin_family);
+								   mySock_->peer_addr()->sin_family);
 				instance = hp->h_name;
 			}
 		}
@@ -1169,9 +1169,9 @@ int Condor_Auth_Kerberos :: forward_tgt_creds(krb5_creds      * cred,
     int              message, rc = 1;
     struct hostent * hp;
     
-    hp = condor_gethostbyaddr((char *) &(mySock_->endpoint())->sin_addr, 		  
+    hp = condor_gethostbyaddr((char *) &(mySock_->peer_addr())->sin_addr, 		  
                        sizeof (struct in_addr), 
-                       mySock_->endpoint()->sin_family);
+                       mySock_->peer_addr()->sin_family);
     
     if ((code = krb5_fwd_tgt_creds(krb_context_, 
                                   auth_context_,
@@ -1207,7 +1207,7 @@ int Condor_Auth_Kerberos :: forward_tgt_creds(krb5_creds      * cred,
     return rc;
 }
 
-int Condor_Auth_Kerberos :: receive_tgt_creds(krb5_ticket * ticket)
+int Condor_Auth_Kerberos :: receive_tgt_creds(krb5_ticket * /* ticket */ )
 {
     krb5_error_code  code;
 	//krb5_ccache      ccache;
@@ -1419,10 +1419,9 @@ int Condor_Auth_Kerberos :: isValid() const
     return auth_context_ != NULL;  // This is incorrect!
 }
 
-#endif
-
-
-void Condor_Auth_Kerberos :: dprintf_krb5_principal ( int deblevel, char *fmt, krb5_principal p ) {
+void Condor_Auth_Kerberos :: dprintf_krb5_principal ( int deblevel,
+													  const char *fmt,
+													  krb5_principal p ) {
 
 	if (p) {
 		char * tmpprincname = 0;
@@ -1439,3 +1438,4 @@ void Condor_Auth_Kerberos :: dprintf_krb5_principal ( int deblevel, char *fmt, k
 	}
 }
 
+#endif
