@@ -28,6 +28,16 @@
 #include "condor_string.h"
 #include "CondorError.h"
 #include "setenv.h"
+#include "globus_utils.h"
+#include "condor_gssapi_openssl.h"
+
+#if defined(HAVE_EXT_VOMS)
+extern "C" {
+	#include "voms/voms_apic.h"
+}
+#endif
+
+#define USER_NAME_MAX 256
 
 const char STR_DAEMON_NAME_FORMAT[]="$$(FULL_HOST_NAME)";
 StringList * getDaemonList(ReliSock * sock);
@@ -423,14 +433,24 @@ int Condor_Auth_X509::nameGssToLocal(const char * GSSClientname)
 	//just extract username from /CN=<username>@<domain,etc>
 	OM_uint32 major_status;
 	char *tmp_user = NULL;
-	char local_user[256];
+	char local_user[USER_NAME_MAX];
 
 // windows gsi does not currently include this function.  we use it on
 // unix, but implement our own on windows for now.
 #ifdef WIN32
 	major_status = condor_gss_assist_gridmap(GSSClientname, &tmp_user);
 #else
-	major_status = globus_gss_assist_gridmap((char*)GSSClientname, &tmp_user);
+// Switched the unix map function to _map_and_authorize, which allows access
+// to the Globus callout infrastructure.
+	major_status = globus_gss_assist_map_and_authorize(
+            context_handle,
+            "condor", // Requested service name
+            NULL, // Requested user name; NULL for non-specified
+            local_user,
+            USER_NAME_MAX-1); // Leave one space at end of buffer, just-in-case
+        // Defensive programming: to protect against buffer overruns in the
+        // unknown globus mapping module, make sure we are at least nul-term'd
+        local_user[USER_NAME_MAX-1] = '\0';
 #endif
 
 	if (tmp_user) {
@@ -449,8 +469,8 @@ int Condor_Auth_X509::nameGssToLocal(const char * GSSClientname)
 	MyString domain;
 	Authentication::split_canonical_name( local_user, user, domain );
     
-	setRemoteUser  (user.GetCStr());
-	setRemoteDomain(domain.GetCStr());
+	setRemoteUser  (user.Value());
+	setRemoteDomain(domain.Value());
 	setAuthenticatedName(GSSClientname);
 	return 1;
 }
@@ -461,7 +481,7 @@ StringList * getDaemonList(ReliSock * sock)
     // build a string list, then do a search to see if the target is 
     // in the list
     char * daemonNames = param( "GSI_DAEMON_NAME" );
-    char * fqh         = sin_to_hostname(sock->endpoint(), NULL);
+    char * fqh         = sin_to_hostname(sock->peer_addr(), NULL);
     char * entry       = NULL;
 
 	if (!daemonNames) {
@@ -516,7 +536,7 @@ char * Condor_Auth_X509::get_server_info()
     OM_uint32	major_status = 0;
     OM_uint32	minor_status = 0;            
     OM_uint32   lifetime, flags;
-    gss_name_t  src = NULL, target = NULL;
+    gss_name_t  target = NULL;
     gss_OID     mech, name_type;
     gss_buffer_desc name_buf;
     char *      server = NULL;
@@ -524,7 +544,7 @@ char * Condor_Auth_X509::get_server_info()
     // Now, we do some authorization work 
     major_status = gss_inquire_context(&minor_status,
                                        context_handle,
-                                       &src,    
+                                       NULL,    
                                        &target,
                                        &lifetime,
                                        &mech, 
@@ -536,10 +556,12 @@ char * Condor_Auth_X509::get_server_info()
         return NULL;
     }
     else {
-        if ((major_status = gss_display_name(&minor_status,
+        major_status = gss_display_name(&minor_status,
                                              target,
                                              &name_buf,
-                                             &name_type)) != GSS_S_COMPLETE) {
+                                             &name_type);
+		gss_release_name( &minor_status, &target );
+		if( major_status != GSS_S_COMPLETE) {
             dprintf(D_SECURITY, "Unable to convert target principal name\n");
             return NULL;
         }
@@ -547,6 +569,7 @@ char * Condor_Auth_X509::get_server_info()
             server = new char[name_buf.length+1];
             memset(server, 0, name_buf.length+1);
             memcpy(server, name_buf.value, name_buf.length);
+			gss_release_buffer( &minor_status, &name_buf );
         }
     }
     return server;
@@ -720,6 +743,22 @@ int Condor_Auth_X509::authenticate_client_gss(CondorError* errstack)
                     server);
         }
 
+		// extract and store VOMS attributes
+		if (param_boolean("USE_VOMS_ATTRIBUTES", true)) {
+
+			// get the voms attributes from the peer
+			globus_gsi_cred_handle_t peer_cred = context_handle->peer_cred_handle->cred_handle;
+
+			char * voms_fqan = NULL;
+			int voms_err = extract_VOMS_info(peer_cred, 1, NULL, NULL, &voms_fqan);
+			if (!voms_err) {
+				setFQAN(voms_fqan);
+				free(voms_fqan);
+			} else {
+				// complain!
+				dprintf(D_SECURITY, "ZKM: VOMS FQAN not present (error %i), ignoring.\n", voms_err);
+			}
+		}
 
         StringList * daemonNames = getDaemonList(mySock_);
 
@@ -796,6 +835,21 @@ int Condor_Auth_X509::authenticate_server_gss(CondorError* errstack)
     else {
 		// store the raw subject name for later mapping
 		setAuthenticatedName(GSSClientname);
+		if (param_boolean("USE_VOMS_ATTRIBUTES", true)) {
+
+			// get the voms attributes from the peer
+			globus_gsi_cred_handle_t peer_cred = context_handle->peer_cred_handle->cred_handle;
+
+			char * voms_fqan = NULL;
+			int voms_err = extract_VOMS_info(peer_cred, 1, NULL, NULL, &voms_fqan);
+			if (!voms_err) {
+				setFQAN(voms_fqan);
+				free(voms_fqan);
+			} else {
+				// complain!
+				dprintf(D_SECURITY, "ZKM: VOMS FQAN not present (error %i), ignoring.\n", voms_err);
+			}
+		}
 
         // Try to map DN to local name (in the format of name@domain)
         if ( (status = nameGssToLocal(GSSClientname) ) == 0) {
@@ -847,4 +901,14 @@ int Condor_Auth_X509::authenticate_server_gss(CondorError* errstack)
     return (status == 0) ? FALSE : TRUE;
 }
 
+void Condor_Auth_X509::setFQAN(const char *fqan) {
+	dprintf (D_FULLDEBUG, "ZKM: setting FQAN: %s\n", fqan ? fqan : "");
+	m_fqan = fqan;
+}
+
+const char *Condor_Auth_X509::getFQAN() {
+	return m_fqan.Value();
+}
+
 #endif
+
