@@ -27,6 +27,7 @@
 #include "condor_socket_types.h"
 #include "condor_md.h"
 #include "selector.h"
+#include "ccb_client.h"
 
 #define NORMAL_HEADER_SIZE 5
 #define MAX_HEADER_SIZE MAC_SIZE + NORMAL_HEADER_SIZE
@@ -46,12 +47,12 @@ ReliSock::init()
 	_bytes_recvd = 0.0;
 	_special_state = relisock_none;
 	is_client = 0;
-	authob = NULL;
 	hostAddr = NULL;
 	snd_msg.buf.reset();                                                    
 	rcv_msg.buf.reset();   
 	rcv_msg.init_parent(this);
 	snd_msg.init_parent(this);
+	m_target_shared_port_id = NULL;
 }
 
 
@@ -81,21 +82,24 @@ ReliSock::CloneStream()
 ReliSock::~ReliSock()
 {
 	close();
-	if ( authob ) {
-		delete authob;
-		authob = NULL;
-	}
 	if ( hostAddr ) {
 		free( hostAddr );
 		hostAddr = NULL;
 	}
+	if( m_target_shared_port_id ) {
+		free( m_target_shared_port_id );
+		m_target_shared_port_id = NULL;
+	}
 }
 
 
 int 
 ReliSock::listen()
 {
-	if (_state != sock_bound) return FALSE;
+	if (_state != sock_bound) {
+        dprintf(D_ALWAYS, "Failed to listen on TCP socket, because it is not bound to a port.\n");
+        return FALSE;
+    }
 
 	// many modern OS's now support a >5 backlog, so we ask for 500,
 	// but since we don't know how they behave when you ask for too
@@ -108,6 +112,18 @@ ReliSock::listen()
 		if( ::listen( _sock, 200 ) < 0 ) 
 		if( ::listen( _sock, 100 ) < 0 ) 
 		if( ::listen( _sock, 5 ) < 0 ) {
+
+            char const *self_address = get_sinful();
+            if( !self_address ) {
+                self_address = "<bad address>";
+            }
+#ifdef WIN32
+			int error = WSAGetLastError();
+			dprintf( D_ALWAYS, "Failed to listen on TCP socket %s: WSAError = %d\n", self_address, error );
+#else
+			dprintf(D_ALWAYS, "Failed to listen on TCP socket %s: (errno = %d) %s\n", self_address, errno, strerror(errno));
+#endif
+
 			return FALSE;
 		}
 	}
@@ -154,7 +170,7 @@ ReliSock::accept( ReliSock	&c )
 #ifndef WIN32 /* Unix */
 	errno = 0;
 #endif
-	if ((c_sock = ::accept(_sock, (sockaddr *)&c._who, &addr_sz)) < 0) {
+	if ((c_sock = ::accept(_sock, (sockaddr *)&c._who, (socklen_t*)&addr_sz)) < 0) {
 #ifndef WIN32 /* Unix */
 		if ( errno == EMFILE ) {
 			_condor_fd_panic ( __LINE__, __FILE__ ); /* This calls dprintf_exit! */
@@ -166,7 +182,7 @@ ReliSock::accept( ReliSock	&c )
 
 	c._sock = c_sock;
 	c.move_descriptor_up();	// must be called _after_ we initialize c._sock
-	c._state = sock_connect;
+	c.enter_connected_state("ACCEPT");
 	c.decode();
 
 	int on = 1;
@@ -179,15 +195,6 @@ ReliSock::accept( ReliSock	&c )
 		*/
 	c.setsockopt(IPPROTO_TCP, TCP_NODELAY, (char*)&on, sizeof(on));
 
-	if( DebugFlags & D_NETWORK ) {
-        char* from = strdup( sin_to_string(c.endpoint()) );
-        char* to = strdup(  sock_to_string(_sock) );
-        dprintf( D_NETWORK, "ACCEPT from=%s newfd=%d to=%s\n",
-                 from, c._sock, to );
-        free( from );
-        free( to );
-	}
-	
 	return TRUE;
 }
 
@@ -234,11 +241,6 @@ ReliSock::accept()
 int 
 ReliSock::connect( char	const *host, int port, bool non_blocking_flag )
 {
-	if (authob) {                                                           
-		delete authob;                                                  
-		authob = NULL;
-	}  	                                                                     
-
 	if (hostAddr != NULL)
 	{
 		free(hostAddr);
@@ -289,13 +291,13 @@ ReliSock::get_line_raw( char *buffer, int length )
 int 
 ReliSock::put_bytes_raw( char *buffer, int length )
 {
-	return condor_write(_sock,buffer,length,_timeout);
+	return condor_write(peer_description(),_sock,buffer,length,_timeout);
 }
 
 int 
 ReliSock::get_bytes_raw( char *buffer, int length )
 {
-	return condor_read(_sock,buffer,length,_timeout);
+	return condor_read(peer_description(),_sock,buffer,length,_timeout);
 }
 
 int 
@@ -339,7 +341,7 @@ ReliSock::put_bytes_nobuffer( char *buffer, int length, int send_size )
 	{
 		// If there is less then a page left.
 		if( (length - i) < pagesize ) {
-			result = condor_write(_sock, (char *)cur, (length - i), _timeout);
+			result = condor_write(peer_description(), _sock, (char *)cur, (length - i), _timeout);
 			if( result < 0 ) {
                                 goto error;
 			}
@@ -347,7 +349,7 @@ ReliSock::put_bytes_nobuffer( char *buffer, int length, int send_size )
 			i += (length - i);
 		} else {  
 			// Send another page...
-			result = condor_write(_sock, (char *)cur, pagesize, _timeout);
+			result = condor_write(peer_description(), _sock, (char *)cur, pagesize, _timeout);
 			if( result < 0 ) {
                             goto error;
 			}
@@ -403,7 +405,7 @@ ReliSock::get_bytes_nobuffer(char *buffer, int max_length, int receive_size)
                 goto error;
 	}
 
-	result = condor_read(_sock, buffer, length, _timeout);
+	result = condor_read(peer_description(), _sock, buffer, length, _timeout);
 
 	
 	if( result < 0 ) {
@@ -440,7 +442,7 @@ ReliSock::handle_incoming_packet()
 		return TRUE;
 	}
 
-	if (!rcv_msg.rcv_packet(_sock, _timeout)) {
+	if (!rcv_msg.rcv_packet(peer_description(), _sock, _timeout)) {
 		return FALSE;
 	}
 
@@ -462,7 +464,7 @@ ReliSock::end_of_message()
 				return TRUE;
 			}
 			if (!snd_msg.buf.empty()) {
-				return snd_msg.snd_packet(_sock, TRUE, _timeout);
+				return snd_msg.snd_packet(peer_description(), _sock, TRUE, _timeout);
 			}
 			if ( allow_empty_message_flag ) {
 				allow_empty_message_flag = FALSE;
@@ -499,6 +501,16 @@ ReliSock::end_of_message()
 	return ret_val;
 }
 
+bool
+ReliSock::peek_end_of_message()
+{
+	if ( rcv_msg.ready ) {
+		if ( rcv_msg.buf.consumed() ) {
+			return true;
+		}
+	}
+	return false;
+}
 
 const char * ReliSock :: isIncomingDataMD5ed()
 {
@@ -526,8 +538,8 @@ ReliSock::put_bytes(const void *data, int sz)
             }
         }
         else {
-            dta = (unsigned char *) malloc(sz);
-            memcpy(dta, data, sz);
+            if((dta = (unsigned char *) malloc(sz)) != 0)
+		memcpy(dta, data, sz);
         }
 
 	ignore_next_encode_eom = FALSE;
@@ -535,7 +547,7 @@ ReliSock::put_bytes(const void *data, int sz)
 	for(nw=0;;) {
 		
 		if (snd_msg.buf.full()) {
-			if (!snd_msg.snd_packet(_sock, FALSE, _timeout)) {
+			if (!snd_msg.snd_packet(peer_description(), _sock, FALSE, _timeout)) {
 				if (dta != NULL)
 				{
 					free(dta);
@@ -549,17 +561,14 @@ ReliSock::put_bytes(const void *data, int sz)
 			snd_msg.buf.seek(header_size);
 		}
 		
-		if ((tw = snd_msg.buf.put_max(&((char *)dta)[nw], sz-nw)) < 0) {
-					if (dta != NULL)
-					{
-                    	free(dta);
-						dta = NULL;
-					}
-                    return -1;
+		if (dta && (tw = snd_msg.buf.put_max(&((char *)dta)[nw], sz-nw)) < 0) {
+			free(dta);
+			dta = NULL;
+			return -1;
 		}
 		
 		nw += tw;
-		if (nw == sz) {
+		if (nw >= sz) {
 			break;
 		}
 	}
@@ -569,7 +578,7 @@ ReliSock::put_bytes(const void *data, int sz)
 
 	if (dta != NULL)
 	{
-    	free(dta);
+		free(dta);
 		dta = NULL;
 	}
 
@@ -649,7 +658,8 @@ bool ReliSock::RcvMsg::init_MD(CONDOR_MD_MODE mode, KeyInfo * key)
 ReliSock::RcvMsg :: RcvMsg() : 
     mode_(MD_OFF),
     mdChecker_(0), 
-    ready(0) 
+	p_sock(0),
+    ready(0)
 {
 }
 
@@ -658,7 +668,7 @@ ReliSock::RcvMsg::~RcvMsg()
     delete mdChecker_;
 }
 
-int ReliSock::RcvMsg::rcv_packet( SOCKET _sock, int _timeout)
+int ReliSock::RcvMsg::rcv_packet( char const *peer_description, SOCKET _sock, int _timeout)
 {
 	Buf		*tmp;
 	char	        hdr[MAX_HEADER_SIZE];
@@ -668,7 +678,7 @@ int ReliSock::RcvMsg::rcv_packet( SOCKET _sock, int _timeout)
 
 	header_size = (mode_ != MD_OFF) ? MAX_HEADER_SIZE : NORMAL_HEADER_SIZE;
 
-	int retval = condor_read(_sock,hdr,header_size,_timeout);
+	int retval = condor_read(peer_description,_sock,hdr,header_size,_timeout);
 	if ( retval < 0 && 
 		 retval != -2 ) // -2 means peer just closed the socket
 	{
@@ -705,7 +715,7 @@ int ReliSock::RcvMsg::rcv_packet( SOCKET _sock, int _timeout)
 			len,end);
 		return FALSE;
 	}
-	if ((tmp_len = tmp->read(_sock, len, _timeout)) != len){
+	if ((tmp_len = tmp->read(peer_description, _sock, len, _timeout)) != len){
 		delete tmp;
 		dprintf(D_ALWAYS, "IO: Packet read failed: read %d of %d\n",
 				tmp_len, len);
@@ -736,7 +746,8 @@ int ReliSock::RcvMsg::rcv_packet( SOCKET _sock, int _timeout)
 
 ReliSock::SndMsg::SndMsg() : 
     mode_(MD_OFF), 
-    mdChecker_(0) 
+    mdChecker_(0),
+	p_sock(0)
 {
 }
 
@@ -745,7 +756,7 @@ ReliSock::SndMsg::~SndMsg()
     delete mdChecker_;
 }
 
-int ReliSock::SndMsg::snd_packet( int _sock, int end, int _timeout )
+int ReliSock::SndMsg::snd_packet( char const *peer_description, int _sock, int end, int _timeout )
 {
 	char	        hdr[MAX_HEADER_SIZE];
 	int		len, header_size;
@@ -765,10 +776,13 @@ int ReliSock::SndMsg::snd_packet( int _sock, int end, int _timeout )
             }
         }
 
-        if (buf.flush(_sock, hdr, header_size, _timeout) != (ns+header_size)){
+        if (buf.flush(peer_description, _sock, hdr, header_size, _timeout) != (ns+header_size)){
             return FALSE;
         }
         
+	if( end ) {
+		buf.dealloc_buf(); // save space, now that we are done sending
+	}
 	return TRUE;
 }
 
@@ -853,13 +867,14 @@ ReliSock::serialize(char *buf)
 	// first, let our parent class restore its state
     ptmp = Sock::serialize(buf);
     ASSERT( ptmp );
-
-    sscanf(ptmp,"%d*",(int*)&_special_state);
+    int itmp;
+    sscanf(ptmp,"%d*",&itmp);
+    _special_state = relisock_state(itmp);
     // skip through this
     ptmp = strchr(ptmp, '*');
-    ptmp++;
+    if(ptmp) ptmp++;
     // Now, see if we are 6.3 or 6.2
-    if ((ptr = strchr(ptmp, '*')) != NULL) {
+    if (ptmp && (ptr = strchr(ptmp, '*')) != NULL) {
         // we are 6.3
         memcpy(sinful_string, ptmp, ptr - ptmp);
 
@@ -876,18 +891,12 @@ ReliSock::serialize(char *buf)
             ptmp++;
             memcpy(fqu, ptmp, len);
             if ((fqu[0] != ' ') && (fqu[0] != '\0')) {
-                if (authob && (authob->getFullyQualifiedUser() != NULL)) {
-                    // odd situation!
-                    dprintf(D_SECURITY, "WARNING!!!! Trying to serialize a socket for user %s but the socket is identified with another user: %s", fqu, authob->getFullyQualifiedUser());
-                }
-                else {
                     // We are cozy
-					setFullyQualifiedUser(fqu);
-                }
+				setFullyQualifiedUser(fqu);
             }
         }
     }
-    else {
+    else if(ptmp) {
         // we are 6.2, this is the end of it.
         sscanf(ptmp,"%s",sinful_string);
     }
@@ -914,7 +923,7 @@ ReliSock::prepare_for_nobuffering(stream_coding direction)
 				return TRUE;
 			}
 			if (!snd_msg.buf.empty()) {
-				ret_val = snd_msg.snd_packet(_sock, TRUE, _timeout);
+				ret_val = snd_msg.snd_packet(peer_description(), _sock, TRUE, _timeout);
 			}
 			if ( ret_val ) {
 				ignore_next_encode_eom = TRUE;
@@ -947,110 +956,84 @@ ReliSock::prepare_for_nobuffering(stream_coding direction)
 
 int ReliSock::perform_authenticate(bool with_key, KeyInfo *& key, 
 								   const char* methods, CondorError* errstack,
-								   int auth_timeout)
+								   int auth_timeout, char **method_used)
 {
 	int in_encode_mode;
 	int result;
 
+	if( method_used ) {
+		*method_used = NULL;
+	}
+
     if (!triedAuthentication()) {
+		Authentication authob(this);
 		setTriedAuthentication(true);
-        if ( !authob ) {
-            authob = new Authentication( this );
-        }
-        if ( authob ) {
-				// store if we are in encode or decode mode
-			in_encode_mode = is_encode();
+			// store if we are in encode or decode mode
+		in_encode_mode = is_encode();
 
-				// actually perform the authentication
-			if ( with_key ) {
-				result = authob->authenticate( hostAddr, key, methods, errstack, auth_timeout );
-			} else {
-				result = authob->authenticate( hostAddr, methods, errstack, auth_timeout );
-			}
-				// restore stream mode (either encode or decode)
-			if ( in_encode_mode && is_decode() ) {
-				encode();
-			} else {
-				if ( !in_encode_mode && is_encode() ) { 
-					decode();
-				}
+			// actually perform the authentication
+		if ( with_key ) {
+			result = authob.authenticate( hostAddr, key, methods, errstack, auth_timeout );
+		} else {
+			result = authob.authenticate( hostAddr, methods, errstack, auth_timeout );
+		}
+			// restore stream mode (either encode or decode)
+		if ( in_encode_mode && is_decode() ) {
+			encode();
+		} else {
+			if ( !in_encode_mode && is_encode() ) { 
+				decode();
 			}
+		}
 
-			setFullyQualifiedUser(authob->getFullyQualifiedUser());
-			return result;
-        }
-		setFullyQualifiedUser(NULL);
-        return( 0 );  
+		setFullyQualifiedUser(authob.getFullyQualifiedUser());
+		if( method_used ) {
+			if( authob.getMethodUsed() ) {
+				*method_used = strdup(authob.getMethodUsed());
+			}
+		}
+		return result;
     }
     else {
         return 1;
     }
 }
 
-int ReliSock::authenticate(KeyInfo *& key, const char* methods, CondorError* errstack, int auth_timeout)
+int ReliSock::authenticate(KeyInfo *& key, const char* methods, CondorError* errstack, int auth_timeout, char **method_used)
 {
-	return perform_authenticate(true,key,methods,errstack,auth_timeout);
+	return perform_authenticate(true,key,methods,errstack,auth_timeout,method_used);
 }
 
 int 
 ReliSock::authenticate(const char* methods, CondorError* errstack,int auth_timeout ) 
 {
 	KeyInfo *key = NULL;
-	return perform_authenticate(false,key,methods,errstack,auth_timeout);
-}
-
-const char * ReliSock::getHostAddress() {
-    if (authob) {
-        return ( authob->getRemoteAddress() );
-    }
-    return NULL;
-}
-
-int ReliSock :: encrypt(bool flag)
-{
-	if ( authob ){
-		return(authob -> encrypt( flag ));
-	}
-	return -1;
-}
-
-
-int ReliSock :: hdr_encrypt()
-{
-	if ( authob ){
-		return(authob -> hdr_encrypt());
-	}
-	return -1;
-}
-
-bool ReliSock :: is_hdr_encrypt()
-{
-	if ( authob ){
-		return(authob -> is_hdr_encrypt());
-	}
-	return FALSE;
-}
-
-
-bool ReliSock :: is_encrypt()
-{
-	if ( authob ){
-		return(authob -> is_encrypt());
-	}
-	return FALSE;
+	return perform_authenticate(false,key,methods,errstack,auth_timeout,NULL);
 }
 
 bool
-ReliSock::connect_socketpair(ReliSock &sock)
+ReliSock::connect_socketpair(ReliSock &sock,bool use_standard_interface)
 {
 	ReliSock tmp_srv;
 
-	if( !bind_to_loopback() ) {
+	if( use_standard_interface ) {
+		if( !bind(false) ) {
+			dprintf(D_ALWAYS, "connect_socketpair: failed in bind()\n");
+			return false;
+		}
+	}
+	else if( !bind_to_loopback(false) ) {
 		dprintf(D_ALWAYS, "connect_socketpair: failed in bind_to_loopback()\n");
 		return false;
 	}
 
-	if( !tmp_srv.bind_to_loopback() ) {
+	if( use_standard_interface ) {
+		if( !tmp_srv.bind(false) ) {
+			dprintf(D_ALWAYS, "connect_socketpair: failed in tmp_srv.bind()\n");
+			return false;
+		}
+	}
+	else if( !tmp_srv.bind_to_loopback(false) ) {
 		dprintf(D_ALWAYS, "connect_socketpair: failed in tmp_srv.bind_to_loopback()\n");
 		return false;
 	}
@@ -1060,7 +1043,7 @@ ReliSock::connect_socketpair(ReliSock &sock)
 		return false;
 	}
 
-	if( !connect(tmp_srv.sender_ip_str(),tmp_srv.get_port()) ) {
+	if( !connect(tmp_srv.my_ip_str(),tmp_srv.get_port()) ) {
 		dprintf(D_ALWAYS, "connect_socketpair: failed in tmp_srv.get_port()\n");
 		return false;
 	}
@@ -1072,3 +1055,50 @@ ReliSock::connect_socketpair(ReliSock &sock)
 
 	return true;
 }
+
+void
+ReliSock::enter_reverse_connecting_state()
+{
+	if( _state == sock_assigned ) {
+		// no need for a socket to be allocated while we are waiting
+		// because this socket will be assigned to a new socket
+		// once we accept a connection from the listen socket
+		this->close();
+	}
+	ASSERT( _state == sock_virgin );
+	_state = sock_reverse_connect_pending;
+}
+
+void
+ReliSock::exit_reverse_connecting_state(ReliSock *sock)
+{
+	ASSERT( _state == sock_reverse_connect_pending );
+	_state = sock_virgin;
+
+	if( sock ) {
+		int assign_rc = assign(sock->get_file_desc());
+		ASSERT( assign_rc );
+		isClient(true);
+		if( sock->_state == sock_connect ) {
+			enter_connected_state("REVERSE CONNECT");
+		}
+		else {
+			_state = sock->_state;
+		}
+		sock->_sock = INVALID_SOCKET;
+		sock->close();
+	}
+	m_ccb_client = NULL;
+}
+
+void
+ReliSock::setTargetSharedPortID( char const *id )
+{
+	if( m_target_shared_port_id ) {
+		free( m_target_shared_port_id );
+		m_target_shared_port_id  = NULL;
+	}
+	if( id ) {
+		m_target_shared_port_id = strdup( id );
+	}
+}
