@@ -192,8 +192,24 @@ int SafeSock::end_of_message()
 	return ret_val;
 }
 
+bool
+SafeSock::peek_end_of_message()
+{
+	if(_msgReady) {
+		if(_longMsg) { // long message is ready
+			if(_longMsg->consumed()) {
+				return true;
+			}
+		} else { // short message is ready
+			if(_shortMsg.consumed())
+				return true;
+		}
+	}
+	return false;
+}
+
 const char *
-SafeSock::sender_ip_str()
+SafeSock::my_ip_str()
 {
 	//In order to call getSockAddr(_sock), we would need to call
 	//::connect() on _sock, which changes semantics on what other
@@ -206,9 +222,9 @@ SafeSock::sender_ip_str()
 		return NULL;
 	}
 
-	if(_sender_ip_buf[0]) {
+	if(_my_ip_buf[0]) {
 		// return cached result
-		return &(_sender_ip_buf[0]);
+		return _my_ip_buf;
 	}
 
 	SafeSock s;
@@ -216,7 +232,7 @@ SafeSock::sender_ip_str()
 
 	if (s._state != sock_bound) {
 		dprintf(D_ALWAYS,
-		        "SafeSock::sender_ip_str() failed to bind: _state = %d\n",
+		        "SafeSock::my_ip_str() failed to bind: _state = %d\n",
 			  s._state); 
 		return NULL;
 	}
@@ -224,22 +240,22 @@ SafeSock::sender_ip_str()
 	if(::connect(s._sock, (sockaddr *)&_who, sizeof(sockaddr_in)) != 0) {
 #if defined(WIN32)
 		int lasterr = WSAGetLastError();
-		dprintf( D_ALWAYS, "SafeSock::sender_ip_str() failed to connect, errno = %d\n",
+		dprintf( D_ALWAYS, "SafeSock::my_ip_str() failed to connect, errno = %d\n",
 				 lasterr );
 #else
-		dprintf( D_ALWAYS, "SafeSock::sender_ip_str() failed to connect, errno = %d\n",
+		dprintf( D_ALWAYS, "SafeSock::my_ip_str() failed to connect, errno = %d\n",
 				 errno );
 #endif
 		return NULL;
 	}
 
 	struct sockaddr_in sin;
-	if(s.mypoint(&sin) == -1) {
+	if(s.my_addr(&sin) == -1) {
 		return NULL;
 	}
-	memset(&_sender_ip_buf, 0, IP_STRING_BUF_SIZE );
-	strcpy( _sender_ip_buf, inet_ntoa(sin.sin_addr) );
-	return &(_sender_ip_buf[0]);
+	strncpy( _my_ip_buf, inet_ntoa(sin.sin_addr), IP_STRING_BUF_SIZE );
+    _my_ip_buf[IP_STRING_BUF_SIZE-1] = '\0';
+	return _my_ip_buf;
 }
 
 int SafeSock::connect(
@@ -251,12 +267,44 @@ int SafeSock::connect(
 	struct hostent	*hostp = NULL;
 	unsigned long	inaddr = 0;
 
-	// Remove any cached sender IP info, since the new target may route us
-	// through a different network interface.
-	_sender_ip_buf[0] = '\0';
-
 	if (!host || port < 0) return FALSE;
 
+	memset(&_who, 0, sizeof(sockaddr_in));
+	_who.sin_family = AF_INET;
+	_who.sin_port = htons((u_short)port);
+
+    int ret;
+
+	/* might be in <x.x.x.x:x> notation, i.e. sinfull string */
+	if (host[0] == '<') {
+		string_to_sin(host, &_who);
+		set_connect_addr(host);
+	}
+	/* try to get a decimal notation first 			*/
+	else if( (ret = inet_pton(AF_INET, host, &inaddr)) > 0 ) {
+		memcpy((char *)&_who.sin_addr, &inaddr, sizeof(inaddr));
+		set_connect_addr(sin_to_string(&_who));
+	} else {
+		/* if dotted notation fails, try host database	*/
+		hostp = condor_gethostbyname(host);
+		if( hostp == (struct hostent *)0 ) {
+			return FALSE;
+		} else {
+			memcpy(&_who.sin_addr, hostp->h_addr, sizeof(hostp->h_addr));
+		}
+		set_connect_addr(sin_to_string(&_who));
+	}
+
+    addr_changed();
+
+	// now that we have set _who (useful for getting informative
+	// peer_description), see if we should do a reverse connect
+	// instead of a forward connect
+	int retval=special_connect(host,port,true);
+	if( retval != CEDAR_ENOCCB ) {
+		return retval;
+	}
+
 	/* we bind here so that a sock may be	*/
 	/* assigned to the stream if needed		*/
 	/* TRUE means this is an outgoing connection */
@@ -271,27 +319,6 @@ int SafeSock::connect(
 		return FALSE;
 	}
 	
-	memset(&_who, 0, sizeof(sockaddr_in));
-	_who.sin_family = AF_INET;
-	_who.sin_port = htons((u_short)port);
-
-	/* might be in <x.x.x.x:x> notation, i.e. sinfull string */
-	if (host[0] == '<') {
-		string_to_sin(host, &_who);
-	}
-	/* try to get a decimal notation first 			*/
-	else if( (inaddr=inet_addr(host)) != (unsigned long)(-1L) ) {
-		memcpy((char *)&_who.sin_addr, &inaddr, sizeof(inaddr));
-	} else {
-		/* if dotted notation fails, try host database	*/
-		hostp = condor_gethostbyname(host);
-		if( hostp == (struct hostent *)0 ) {
-			return FALSE;
-		} else {
-			memcpy(&_who.sin_addr, hostp->h_addr, sizeof(hostp->h_addr));
-		}
-	}
-
 	_state = sock_connect;
 	return TRUE;
 }
@@ -500,7 +527,7 @@ int SafeSock::peek(char &c)
 int SafeSock::handle_incoming_packet()
 {
 
-#if defined(Solaris27) || defined(Solaris28) || defined(Solaris29)
+#if defined(Solaris27) || defined(Solaris28) || defined(Solaris29) || defined(Solaris10) || defined(Solaris11)
 	/* SOCKET_ALTERNATE_LENGTH_TYPE is void on this platform, and
 		since noone knows what that void* is supposed to point to
 		in recvfrom, I'm going to predict the "fromlen" variable
@@ -553,7 +580,7 @@ int SafeSock::handle_incoming_packet()
 
 
 	received = recvfrom(_sock, _shortMsg.dataGram, SAFE_MSG_MAX_PACKET_SIZE,
-	                    0, (struct sockaddr *)&_who, fromlen );
+	                    0, (struct sockaddr *)&_who, (socklen_t*)fromlen );
 	if(received < 0) {
 		dprintf(D_NETWORK, "recvfrom failed: errno = %d\n", errno);
 		return FALSE;
@@ -741,18 +768,20 @@ char * SafeSock::serialize(char *buf)
 	// first, let our parent class restore its state
 	ptmp = Sock::serialize(buf);
 	ASSERT( ptmp );
-	sscanf(ptmp,"%d*",(int*)&_special_state);
+	int itmp;
+	sscanf(ptmp,"%d*",&itmp);
+	_special_state=safesock_state(itmp);
     // skip through this
     ptmp = strchr(ptmp, '*');
-    ptmp++;
+    if(ptmp) ptmp++;
 
     // Now, see if we are 6.3 or 6.2
-    if ( (ptr = strchr(ptmp, '*')) != NULL) {
+    if (ptmp && (ptr = strchr(ptmp, '*')) != NULL) {
         // We are 6.3
         memcpy(sinful_string, ptmp, ptr - ptmp);
         ptmp = ++ptr;
     }
-    else {
+    else if(ptmp) {
         sscanf(ptmp,"%s",sinful_string);
     }
 
@@ -877,3 +906,25 @@ void SafeSock::dumpSock()
 	_outMsg.dumpMsg(_outMsgID);
 }
 #endif
+
+void
+SafeSock::cancel_reverse_connect() {
+}
+
+void
+SafeSock::setTargetSharedPortID( char const *id)
+{
+	if( id ) {
+		dprintf(D_ALWAYS,
+			"WARNING: UDP does not support connecting to a shared port! "
+			"(requested address is %s with SharedPortID=%s)\n",
+			peer_description(), id);
+	}
+}
+
+bool
+SafeSock::sendTargetSharedPortID()
+{
+		// do nothing; shared ports are not currently supported by UDP
+	return true;
+}
