@@ -38,13 +38,16 @@
 #include "GCB.h"
 #endif
 
+#ifdef HAVE_EXT_OPENSSL
+#include "condor_crypt_blowfish.h"
+#include "condor_crypt_3des.h"
+#include "condor_md.h"                // Message authentication stuff
+#endif
+
 #if !defined(WIN32)
 #define closesocket close
 #endif
 
-// initialize static data members
-int Sock::timeout_multiplier = 0;
-
 DaemonCoreSockAdapterClass daemonCoreSockAdapter;
 
 Sock::Sock() : Stream() {
@@ -55,12 +58,26 @@ Sock::Sock() : Stream() {
 	_fqu_user_part = NULL;
 	_fqu_domain_part = NULL;
 	_tried_authentication = false;
-	ignore_timeout_multiplier = false;
 	ignore_connect_timeout = FALSE;		// Used by the HA Daemon
+	connect_state.connect_failed = false;
+	connect_state.this_try_timeout_time = 0;
+	connect_state.retry_timeout_time = 0;
+	connect_state.retry_wait_timeout_time = 0;
+	connect_state.failed_once = false;
+	connect_state.connect_refused = false;
+	connect_state.old_timeout_value = 0;
+	connect_state.non_blocking_flag = false;
 	connect_state.host = NULL;
+	connect_state.port = 0;
 	connect_state.connect_failure_reason = NULL;
 	memset(&_who, 0, sizeof(struct sockaddr_in));
-	memset(&_endpoint_ip_buf, 0, IP_STRING_BUF_SIZE);
+
+    crypto_ = NULL;
+    mdMode_ = MD_OFF;
+    mdKey_ = 0;
+
+	m_connect_addr = NULL;
+    addr_changed();
 }
 
 Sock::Sock(const Sock & orig) : Stream() {
@@ -74,10 +91,25 @@ Sock::Sock(const Sock & orig) : Stream() {
 	_fqu_domain_part = NULL;
 	_tried_authentication = false;
 	ignore_timeout_multiplier = orig.ignore_timeout_multiplier;
+	connect_state.connect_failed = false;
+	connect_state.failed_once = false;
+	connect_state.connect_refused = false;
+	connect_state.this_try_timeout_time = 0;
+	connect_state.retry_timeout_time = 0;
+	connect_state.retry_wait_timeout_time = 0;
+	connect_state.old_timeout_value = 0;
+	connect_state.non_blocking_flag = false;
 	connect_state.host = NULL;
+	connect_state.port = 0;
 	connect_state.connect_failure_reason = NULL;
 	memset( &_who, 0, sizeof( struct sockaddr_in ) );
-	memset(	&_endpoint_ip_buf, 0, IP_STRING_BUF_SIZE );
+
+    crypto_ = NULL;
+    mdMode_ = MD_OFF;
+    mdKey_ = 0;
+
+	m_connect_addr = NULL;
+    addr_changed();
 
 	// now duplicate the underlying network socket
 #ifdef WIN32
@@ -115,6 +147,11 @@ Sock::Sock(const Sock & orig) : Stream() {
 
 Sock::~Sock()
 {
+    delete crypto_;
+	crypto_ = NULL;
+    delete mdKey_;
+	mdKey_ = NULL;
+
 	if ( connect_state.host ) free(connect_state.host);
 	if ( connect_state.connect_failure_reason) {
 		free(connect_state.connect_failure_reason);
@@ -131,6 +168,8 @@ Sock::~Sock()
 		free(_fqu_domain_part);
 		_fqu_domain_part = NULL;
 	}
+	free( m_connect_addr );
+	m_connect_addr = NULL;
 }
 
 #if defined(WIN32)
@@ -211,7 +250,7 @@ int Sock::getportbyserv(
 	)
 {
 	servent		*sp;
-	char		*my_prot;
+	const char	*my_prot=0;
 
 	if (!s) return -1;
 
@@ -251,14 +290,10 @@ int Sock::assign(
 			ASSERT(0);
 	}
 
-	_sock = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, 
+	int socket_fd = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, 
 					  FROM_PROTOCOL_INFO, pProtoInfo, 0, 0);
 
-	if ( _sock == INVALID_SOCKET )
-		return FALSE;
-
-	_state = sock_assigned;
-	return TRUE;
+	return assign( socket_fd );
 }
 #endif
 
@@ -342,13 +377,21 @@ int Sock::move_descriptor_up()
 
 int Sock::assign(SOCKET sockd)
 {
-	int		my_type;
+	int my_type = SOCK_DGRAM;
 
 	if (_state != sock_virgin) return FALSE;
 
 	if (sockd != INVALID_SOCKET){
 		_sock = sockd;		/* Could we check for correct protocol ? */
 		_state = sock_assigned;
+
+		memset(&_who, 0, sizeof(struct sockaddr_in));
+		SOCKET_LENGTH_TYPE addrlen = sizeof(_who);
+		getpeername(_sock,(struct sockaddr *)&_who, (socklen_t*)&addrlen);
+
+		if ( _timeout > 0 ) {
+			timeout_no_timeout_multiplier( _timeout );
+		}
 		return TRUE;
 	}
 
@@ -403,6 +446,7 @@ int Sock::assign(SOCKET sockd)
 	if ( _timeout > 0 )
 		timeout_no_timeout_multiplier( _timeout );
 
+    addr_changed();
 	return TRUE;
 }
 
@@ -453,6 +497,8 @@ Sock::bindWithin(const int low_port, const int high_port, bool outbound)
 			(SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE)&sin, 
 			sizeof(sockaddr_in), outbound, false);
 
+        addr_changed();
+
 #ifndef WIN32
 		if (this_trial <= 1024) {
 			set_priv (old_priv);
@@ -491,7 +537,10 @@ int Sock::bind(bool outbound, int port, bool loopback)
 	// Following lines are added because some functions in condor call
 	// this method without checking the port numbers returned from
 	// such as 'getportbyserv'
-	if (port < 0) return FALSE;
+	if (port < 0) {
+        dprintf(D_ALWAYS, "Sock::bind - invalid port %d\n", port);
+        return FALSE;
+    }
 
 	// if stream not assigned to a sock, do it now	*/
 	if (_state == sock_virgin) assign();
@@ -561,7 +610,10 @@ int Sock::bind(bool outbound, int port, bool loopback)
 
 		bind_return_value = _bind_helper(_sock, (sockaddr *)&sin, sizeof(sockaddr_in), outbound, loopback);
 
+        addr_changed();
+
 #ifndef WIN32
+        int bind_errno = errno;
 		if(port > 0 && port < 1024) {
 			set_priv (old_priv);
 		}
@@ -569,9 +621,9 @@ int Sock::bind(bool outbound, int port, bool loopback)
 		if ( bind_return_value < 0) {
 	#ifdef WIN32
 			int error = WSAGetLastError();
-			dprintf( D_ALWAYS, "bind failed: WSAError = %d\n", error );
+			dprintf( D_ALWAYS, "Sock::bind failed: WSAError = %d\n", error );
 	#else
-			dprintf(D_NETWORK, "bind failed errno = %d\n", errno);
+			dprintf(D_ALWAYS, "Sock::bind failed: errno = %d %s\n", bind_errno, strerror(bind_errno));
 	#endif
 			return FALSE;
 		}
@@ -622,7 +674,7 @@ int Sock::set_os_buffers(int desired_size, bool set_write_buf)
 	// Log the current size since Todd is curious.  :^)
 	temp = sizeof(int);
 	::getsockopt(_sock,SOL_SOCKET,command,
-			(char*)&current_size,&temp);
+			(char*)&current_size,(socklen_t*)&temp);
 	dprintf(D_FULLDEBUG,"Current Socket bufsize=%dk\n",
 		current_size / 1024);
 	current_size = 0;
@@ -648,7 +700,7 @@ int Sock::set_os_buffers(int desired_size, bool set_write_buf)
 		previous_size = current_size;
 		temp = sizeof(int);
 		::getsockopt( _sock, SOL_SOCKET, command,
- 					  (char*)&current_size, &temp );
+ 					  (char*)&current_size, (socklen_t*)&temp );
 
 	} while ( ( previous_size < current_size ) &&
 			  ( attempt_size < desired_size  ) );
@@ -669,7 +721,6 @@ int Sock::setsockopt(int level, int optname, const char* optval, int optlen)
 	return TRUE; 
 }
 
-
 int Sock::do_connect(
 	char const	*host,
 	int		port,
@@ -681,6 +732,42 @@ int Sock::do_connect(
 
 	if (!host || port < 0) return FALSE;
 
+	memset(&_who, 0, sizeof(sockaddr_in));
+	_who.sin_family = AF_INET;
+	_who.sin_port = htons((u_short)port);
+
+    int ret;
+
+	/* might be in <x.x.x.x:x> notation				*/
+	if (host[0] == '<') {
+		string_to_sin(host, &_who);
+		set_connect_addr(host);
+	}
+	/* try to get a decimal notation 	 			*/
+	else if ( (ret = inet_pton(AF_INET, host, &inaddr)) > 0 ) {
+		memcpy((char *)&_who.sin_addr, &inaddr, sizeof(inaddr));
+		set_connect_addr(sin_to_string(&_who));
+	}
+	/* if dotted notation fails, try host database	*/
+	else{
+		if ((hostp = condor_gethostbyname(host)) == (hostent *)0) return FALSE;
+		memcpy(&_who.sin_addr, hostp->h_addr, hostp->h_length);
+		set_connect_addr(sin_to_string(&_who));
+	}
+
+    addr_changed();
+
+	// now that we have set _who (useful for getting informative
+	// peer_description), see if we should do a reverse connect
+	// instead of a forward connect.  Also see if we are connecting
+	// to a shared port (SharedPortServer) that needs further information
+	// to route us to the final destination.
+
+	int retval=special_connect(host,port,non_blocking_flag);
+	if( retval != CEDAR_ENOCCB ) {
+		return retval;
+	}
+
 		/* we bind here so that a sock may be	*/
 		/* assigned to the stream if needed		*/
 		/* TRUE means this is an outgoing connection */
@@ -690,24 +777,6 @@ int Sock::do_connect(
 
 	if (_state != sock_bound) return FALSE;
 
-	memset(&_who, 0, sizeof(sockaddr_in));
-	_who.sin_family = AF_INET;
-	_who.sin_port = htons((u_short)port);
-
-	/* might be in <x.x.x.x:x> notation				*/
-	if (host[0] == '<') {
-		string_to_sin(host, &_who);
-	}
-	/* try to get a decimal notation 	 			*/
-	else if ((inaddr = inet_addr(host)) != (unsigned int)-1){
-		memcpy((char *)&_who.sin_addr, &inaddr, sizeof(inaddr));
-	}
-	/* if dotted notation fails, try host database	*/
-	else{
-		if ((hostp = condor_gethostbyname(host)) == (hostent *)0) return FALSE;
-		memcpy(&_who.sin_addr, hostp->h_addr, hostp->h_length);
-	}
-
 	if (_timeout < CONNECT_TIMEOUT) {
 			// NOTE: if _timeout == 0 (no connect() timeout), we still
 			// have a non-zero retry timeout, so we will not keep
@@ -769,7 +838,6 @@ Sock::do_connect_finish()
 
 		if( _state == sock_bound ) {
 			if ( do_connect_tryit() ) {
-				_state = sock_connect;
 				return TRUE;
 			}
 
@@ -861,16 +929,11 @@ Sock::do_connect_finish()
 				break; // done with select() loop
 			}
 			else {
-				_state = sock_connect;
-				if( DebugFlags & D_NETWORK ) {
-					dprintf( D_NETWORK, "CONNECT src=%s fd=%d dst=%s\n",
-							 get_sinful(), _sock, get_sinful_peer() );
-				}
 				if ( connect_state.old_timeout_value != _timeout ) {
 						// Restore old timeout
 					timeout_no_timeout_multiplier(connect_state.old_timeout_value);			
 				}
-				return true;
+				return enter_connected_state();
 			}
 		}
 
@@ -933,6 +996,23 @@ Sock::do_connect_finish()
 	return FALSE;
 }
 
+bool
+Sock::enter_connected_state(char const *op)
+{
+	_state = sock_connect;
+	if( DebugFlags & D_NETWORK ) {
+		dprintf( D_NETWORK, "%s bound to %s fd=%d peer=%s\n",
+				 op, get_sinful(), _sock, get_sinful_peer() );
+	}
+		// if we are connecting to a shared port, send the id of
+		// the daemon we want to be routed to
+	if( !sendTargetSharedPortID() ) {
+		connect_state.connect_refused = true;
+		setConnectFailureReason("Failed to send shared port id.");
+		return false;
+	}
+	return true;
+}
 
 void
 Sock::setConnectFailureReason(char const *reason)
@@ -1058,12 +1138,7 @@ bool Sock::do_connect_tryit()
 			return false;
 		}
 
-		_state = sock_connect;
-		if( DebugFlags & D_NETWORK ) {
-			dprintf( D_NETWORK, "CONNECT src=%s fd=%d dst=%s\n",
-					 get_sinful(), _sock, get_sinful_peer() );
-		}
-		return true;
+		return enter_connected_state();
 	}
 
 #if defined(WIN32)
@@ -1108,9 +1183,6 @@ Sock::cancel_connect()
 		// strange behavior on Solaris as well when we re-use a 
 		// socket after a failed connect.  -Todd 8/00
 		
-		// stash away the descriptor so we can compare later..
-	SOCKET old_sock = _sock;
-
 		// now close the underlying socket.  do not call Sock::close()
 		// here, because we do not want all the CEDAR socket state
 		// (like the _who data member) cleared.
@@ -1126,28 +1198,6 @@ Sock::cancel_connect()
 		return;
 	}
 
-#ifndef WIN32
-		// make certain our descriptor number has not changed,
-		// because parts of Condor may have stashed the old
-		// socket descriptor into data structures.  So if it has
-		// changed, use dup2() to set it the same as before.
-		// NOTE from Dan 2007-12-13: we have no good way to do this
-		// under windows, and it is not necessary anyway, now that
-		// daemonCore avoids stashing fds.  Get rid of this in the
-		// next development branch.
-	if ( _sock != old_sock ) {
-		if ( dup2(_sock,old_sock) < 0 ) {
-			dprintf(D_ALWAYS,
-				"dup2 failed after a failed connect! errno=%d\n", 
-				errno);
-			connect_state.connect_refused = true; // better give up
-			return;
-		}
-		::closesocket(_sock);
-		_sock = old_sock;
-	}
-#endif
-
 	// finally, bind the socket
 	/* TRUE means this is an outgoing connection */
 	if( !bind(true) ) {
@@ -1179,7 +1229,7 @@ Sock::get_deadline()
 	time_t deadline = Stream::get_deadline();
 	if( is_connect_pending() ) {
 		time_t connect_deadline = connect_timeout_time();
-		if( connect_deadline ) {
+		if( connect_deadline && !is_reverse_connect_pending() ) {
 			if( deadline && deadline < connect_deadline ) {
 				return deadline;
 			}
@@ -1203,7 +1253,7 @@ bool Sock::test_connection()
 
 	int error;
     SOCKET_LENGTH_TYPE len = sizeof(error);
-    if (::getsockopt(_sock, SOL_SOCKET, SO_ERROR, (char*)&error, &len) < 0) {
+    if (::getsockopt(_sock, SOL_SOCKET, SO_ERROR, (char*)&error, (socklen_t*)&len) < 0) {
 		connect_state.connect_failed = true;
 #if defined(WIN32)
 		setConnectFailureErrno(WSAGetLastError(),"getsockopt");
@@ -1225,9 +1275,13 @@ bool Sock::test_connection()
 
 int Sock::close()
 {
+	if( _state == sock_reverse_connect_pending ) {
+		cancel_reverse_connect();
+	}
+
 	if (_state == sock_virgin) return FALSE;
 
-	if (type() == Stream::reli_sock) {
+	if (type() == Stream::reli_sock && (DebugFlags & D_NETWORK)) {
 		dprintf( D_NETWORK, "CLOSE %s fd=%d\n", 
 						sock_to_string(_sock), _sock );
 	}
@@ -1243,7 +1297,7 @@ int Sock::close()
     }
 	connect_state.host = NULL;
 	memset(&_who, 0, sizeof( struct sockaddr_in ) );
-	memset(&_endpoint_ip_buf, 0, IP_STRING_BUF_SIZE );
+    addr_changed();
 	
 	return TRUE;
 }
@@ -1253,14 +1307,6 @@ int Sock::close()
 #define ioctlsocket ioctl
 #endif
 
-int
-Sock::set_timeout_multiplier(int secs)
-{
-   int old_val = timeout_multiplier;
-   timeout_multiplier = secs;
-   return old_val;
-}
-
 int
 Sock::bytes_available_to_read()
 {
@@ -1322,6 +1368,12 @@ Sock::readReady() {
 	return selector.has_ready();
 }
 
+int
+Sock::get_timeout_raw()
+{
+	return _timeout;
+}
+
 /* NOTE: on timeout() we return the previous timeout value, or a -1 on an error.
  * Once more: we do _not_ return FALSE on Error like most other CEDAR functions;
  * we return a -1 !! 
@@ -1333,8 +1385,12 @@ Sock::timeout_no_timeout_multiplier(int sec)
 
 	_timeout = sec;
 
-	/* if stream not assigned to a sock, do it now	*/
-	if (_state == sock_virgin) assign();
+	if (_state == sock_virgin) {
+		// Rather than forcing creation of the socket here, we just
+		// return success.  All paths that create the socket also
+		// set the timeout.
+		return t;
+	}
 	if ( (_state != sock_assigned) &&  
 				(_state != sock_connect) &&
 				(_state != sock_bound) )  {
@@ -1596,7 +1652,7 @@ char * Sock::serialize() const
 	char * outbuf = new char[500];
     if (outbuf) {
         memset(outbuf, 0, 500);
-        sprintf(outbuf,"%u*%d*%d*%d*%u*%u*%s*%s*",_sock,_state,_timeout,triedAuthentication(),fqu_len,verstring_len,_fqu ? _fqu : "",verstring ? verstring : "");
+        sprintf(outbuf,"%u*%d*%d*%d*%lu*%lu*%s*%s*",_sock,_state,_timeout,triedAuthentication(),(unsigned long)fqu_len,(unsigned long)verstring_len,_fqu ? _fqu : "",verstring ? verstring : "");
     }
     else {
         dprintf(D_ALWAYS, "Out of memory!\n");
@@ -1605,6 +1661,17 @@ char * Sock::serialize() const
 	return( outbuf );
 }
 
+void
+Sock::close_serialized_socket(char const *buf)
+{
+		// grab the fd from the serialized string and close it
+	SOCKET passed_sock;
+	int i;
+	i = sscanf(buf,"%u*",&passed_sock);
+	ASSERT( i == 1 );
+	::close(passed_sock);
+}
+
 char * Sock::serialize(char *buf)
 {
 	int i;
@@ -1617,7 +1684,7 @@ char * Sock::serialize(char *buf)
 	ASSERT(buf);
 
 	// here we want to restore our state from the incoming buffer
-	i = sscanf(buf,"%u*%d*%d*%d*%u*%u*%n",&passed_sock,(int*)&_state,&_timeout,&tried_authentication,&fqulen,&verstring_len,&pos);
+	i = sscanf(buf,"%u*%d*%d*%d*%lu*%lu*%n",&passed_sock,(int*)&_state,&_timeout,&tried_authentication,(unsigned long *)&fqulen,(unsigned long *)&verstring_len,&pos);
 	if (i!=6) {
 		EXCEPT("Failed to parse serialized socket information (%d,%d): '%s'\n",i,pos,buf);
 	}
@@ -1633,7 +1700,7 @@ char * Sock::serialize(char *buf)
 	free(fqubuf);
 	buf += fqulen;
 	if( *buf != '*' ) {
-		EXCEPT("Failed to parse serialized socket fqu (%d): '%s'\n",fqulen,buf);
+		EXCEPT("Failed to parse serialized socket fqu (%lu): '%s'\n",(unsigned long)fqulen,buf);
 	}
 	buf++;
 
@@ -1653,7 +1720,7 @@ char * Sock::serialize(char *buf)
 	free( verstring );
 	buf += verstring_len;
 	if( *buf != '*' ) {
-		EXCEPT("Failed to parse serialized peer version string (%d): '%s'\n",verstring_len,buf);
+		EXCEPT("Failed to parse serialized peer version string (%lu): '%s'\n",(unsigned long)verstring_len,buf);
 	}
 	buf++;
 
@@ -1693,36 +1760,129 @@ char * Sock::serialize(char *buf)
 	return buf;
 }
 
+void
+Sock::addr_changed()
+{
+    // these are all regenerated whenever they are needed, so when
+    // either the peer's address or our address change, zap them all
+    _my_ip_buf[0] = '\0';
+    _peer_ip_buf[0] = '\0';
+    _sinful_self_buf[0] = '\0';
+    _sinful_public_buf[0] = '\0';
+    _sinful_peer_buf[0] = '\0';
+}
 
 struct sockaddr_in *
-Sock::endpoint()
+Sock::peer_addr()
 {
 	return &_who;
 }
 
 
 int
-Sock::endpoint_port()
+Sock::peer_port()
 {
 	return (int) ntohs( _who.sin_port );
 }
 
 
 unsigned int
-Sock::endpoint_ip_int()
+Sock::peer_ip_int()
 {
 	return (unsigned int) ntohl( _who.sin_addr.s_addr );
 }
 
 
 const char *
-Sock::endpoint_ip_str()
+Sock::peer_ip_str()
 {
-		// We need to recompute this each time because _who might have changed.
-	memset(&_endpoint_ip_buf, 0, IP_STRING_BUF_SIZE );
-	snprintf( _endpoint_ip_buf, IP_STRING_BUF_SIZE, "%s",
-			  inet_ntoa(_who.sin_addr) );
-	return &(_endpoint_ip_buf[0]);
+    if( _peer_ip_buf[0] ) {
+        return _peer_ip_buf;
+    }
+    strncpy( _peer_ip_buf, inet_ntoa(_who.sin_addr), IP_STRING_BUF_SIZE );
+    _peer_ip_buf[IP_STRING_BUF_SIZE-1] = '\0';
+	return _peer_ip_buf;
+}
+
+// is peer a local interface, aka did this connection originate from a local process?
+// return true if peer address corresponds to an interface local to this machine,
+// or false if not or if an error.
+bool 
+Sock::peer_is_local()
+{
+	// Keep a static cache of results, since determining if the peer is local
+	// is somewhat expensive. Flush the cache every 20 min to deal w/
+	// interfaces on the machine being activated/deactivated during runtime.
+	static HashTable<int,bool>* isLocalTable = NULL;
+	static time_t cache_ttl = 0;
+	
+	bool result;
+	int peer_int = peer_ip_int();
+
+	// if there is no peer, bail out now.
+	if ( peer_int == 0 ) {
+		return false;
+	}
+
+	// allocate hashtable dynamically first time we are called, so
+	// we don't bloat private address space for daemons that never
+	// invoke this method.
+	if ( !isLocalTable ) {
+		isLocalTable = new HashTable<int,bool>(hashFuncInt);
+	}
+
+	// check the time to live (ttl) on our cached data
+	time_t now = time(NULL);
+	if ( now >= cache_ttl ) {
+		// ttl has expired; flush the cache and reset ttl
+		isLocalTable->clear();
+		cache_ttl = now + (20 * 60);	// push ttl 20 min into the future
+	}
+
+	// see if our cache already has the answer
+	if (isLocalTable->lookup(peer_int,result) == 0) {
+		// found the answer is in our cache table, just return what we found.
+		return result;
+	} 
+
+	// if we made it here, we don't have a cached answer, so 
+	// we run an experiment to see if the peer address relates
+	// to one of our interfaces.  the experiment is simply try 
+	// and bind a socket to the peer address on an ephemeral port - 
+	// if it works, we must have a local interface w/ that address,
+	// if it fails, assume we don't. 
+	// note: this algorithm may be imperfect if the host is serving
+	// as a NAT gateway.
+
+	int sock = ::socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+    if ( sock < 0 ) {
+		dprintf(D_ALWAYS,"Sock::peer_is_local(): ERROR failed to create socket\n");
+        return false;
+    }
+		// Bind to the *same address* as our peer ....
+	struct sockaddr_in mySockAddr = *( peer_addr() );
+		// ... but use any old ephemeral port.
+    mySockAddr.sin_port = htons( 0 );
+		// invoke OS bind, not cedar bind - cedar bind does not allow us
+		// to specify the local address.
+	if ( ::bind(sock, (SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE) &mySockAddr, 
+		        sizeof(mySockAddr)) < 0 ) 
+	{
+		// failed to bind.  assume we failed  because the peer address is
+		// not local.
+		result = false;        
+	} else {
+		// bind worked, assume address has a local interface.
+		result = true;
+	}
+	// must not forget to close the socket we just created!
+	::closesocket(sock);
+	
+	// Stash our result in the cache.
+	isLocalTable->insert(peer_int,result);
+
+	// return the result to the caller
+	return result;
 }
 
 
@@ -1730,7 +1890,7 @@ Sock::endpoint_ip_str()
 // @args: the address is returned via 'sin'
 // @ret: 0 if succeed, -1 if failed
 int
-Sock::mypoint( struct sockaddr_in *sin )
+Sock::my_addr( struct sockaddr_in *sin )
 {
     struct sockaddr_in *tmp = getSockAddr(_sock);
     if (tmp == NULL) return -1;
@@ -1740,36 +1900,44 @@ Sock::mypoint( struct sockaddr_in *sin )
 }
 
 const char *
-Sock::sender_ip_str()
+Sock::my_ip_str()
 {
-		// We need to recompute this each in case we have reconnected via a different interface
+    if( _my_ip_buf[0] ) {
+        return _my_ip_buf;
+    }
 	struct sockaddr_in sin;
-	if(mypoint(&sin) == -1) {
+	if(my_addr(&sin) == -1) {
 		return NULL;
 	}
-	memset(&_sender_ip_buf, 0, IP_STRING_BUF_SIZE );
-	strcpy( _sender_ip_buf, inet_ntoa(sin.sin_addr) );
-	return &(_sender_ip_buf[0]);
+    strncpy( _my_ip_buf, inet_ntoa(sin.sin_addr), IP_STRING_BUF_SIZE );
+    _my_ip_buf[IP_STRING_BUF_SIZE-1] = '\0';
+	return _my_ip_buf;
 }
 
 char *
 Sock::get_sinful()
 {       
+    if( _sinful_self_buf[0] ) {
+        return _sinful_self_buf;
+    }
     struct sockaddr_in *tmp = getSockAddr(_sock);
     if (tmp == NULL) return NULL;
     char const *s = sin_to_string(tmp);
 	if(!s) {
 		return NULL;
 	}
-	ASSERT(strlen(s) < sizeof(_sinful_self_buf));
-	strcpy(_sinful_self_buf,s);
+	strncpy(_sinful_self_buf,s,SINFUL_STRING_BUF_SIZE);
+    _sinful_self_buf[SINFUL_STRING_BUF_SIZE-1] = '\0';
 	return _sinful_self_buf;
 }
 
 char *
 Sock::get_sinful_peer()
 {       
-    char const *s = sin_to_string(&_who);
+    if( _sinful_peer_buf[0] ) {
+        return _sinful_peer_buf;
+    }
+	char const *s = sin_to_string(&_who);
 	if(!s) {
 		return NULL;
 	}
@@ -1783,7 +1951,7 @@ Sock::default_peer_description()
 {
 	char const *retval = get_sinful_peer();
 	if( !retval ) {
-		return "(unconnected)";
+		return "(unconnected socket)";
 	}
 	return retval;
 }
@@ -2023,7 +2191,7 @@ bool Sock :: is_hdr_encrypt(){
 	return FALSE;
 }
 
-int Sock :: authenticate(KeyInfo *&, const char * /* methods */, CondorError* /* errstack */, int /*timeout*/)
+int Sock :: authenticate(KeyInfo *&, const char * /* methods */, CondorError* /* errstack */, int /*timeout*/, char ** /*method_used*/)
 {
 	return -1;
 }
@@ -2058,7 +2226,7 @@ Sock::_bind_helper(int fd, SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE addr,
 				the functions signatures all the way down to the
 				the Generic_bind() call in the GCB
 				external. */
-			(struct sockaddr*)addr, 
+			const_cast<struct sockaddr*>(addr),
 			len);
 	}
 	else {
@@ -2069,3 +2237,197 @@ Sock::_bind_helper(int fd, SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE addr,
 #endif /* HAVE_EXT_GCB */
 	return rval;
 }
+
+void
+Sock::set_connect_addr(char const *addr)
+{
+	free( m_connect_addr );
+	m_connect_addr = NULL;
+	if( addr ) {
+		m_connect_addr = strdup(addr);
+	}
+}
+
+char const *
+Sock::get_connect_addr()
+{
+	return m_connect_addr;
+}
+
+const char *
+Sock::getFullyQualifiedUser() const {
+	return _fqu ? _fqu : UNAUTHENTICATED_FQU;
+}
+
+const char *
+Sock::getOwner() const {
+	return _fqu_user_part ? _fqu_user_part : UNAUTHENTICATED_USER;
+}
+
+		/// Get domain portion of fqu
+const char *
+Sock::getDomain() const {
+	return _fqu_domain_part ? _fqu_domain_part : UNMAPPED_DOMAIN;
+}
+
+
+bool
+Sock::isMappedFQU() const
+{
+	if( !_fqu_domain_part ) {
+		return false;
+	}
+	return strcmp(_fqu_domain_part,UNMAPPED_DOMAIN) != 0;
+}
+
+bool
+Sock::isAuthenticated() const
+{
+	if( !_fqu ) {
+		return false;
+	}
+	return strcmp(_fqu,UNAUTHENTICATED_FQU) != 0;
+}
+
+bool 
+Sock::wrap(unsigned char* d_in,int l_in, 
+                    unsigned char*& d_out,int& l_out)
+{    
+    bool coded = false;
+#ifdef HAVE_EXT_OPENSSL
+    if (get_encryption()) {
+        coded = crypto_->encrypt(d_in, l_in, d_out, l_out);
+    }
+#endif
+    return coded;
+}
+
+bool 
+Sock::unwrap(unsigned char* d_in,int l_in,
+                      unsigned char*& d_out, int& l_out)
+{
+    bool coded = false;
+#ifdef HAVE_EXT_OPENSSL
+    if (get_encryption()) {
+        coded = crypto_->decrypt(d_in, l_in, d_out, l_out);
+    }
+#endif
+    return coded;
+}
+
+void Sock::resetCrypto()
+{
+#ifdef HAVE_EXT_OPENSSL
+  if (crypto_) {
+    crypto_->resetState();
+  }
+#endif
+}
+
+bool 
+Sock::initialize_crypto(KeyInfo * key) 
+{
+    delete crypto_;
+    crypto_ = 0;
+	crypto_mode_ = false;
+
+    // Will try to do a throw/catch later on
+    if (key) {
+        switch (key->getProtocol()) 
+        {
+#ifdef HAVE_EXT_OPENSSL
+        case CONDOR_BLOWFISH :
+            crypto_ = new Condor_Crypt_Blowfish(*key);
+            break;
+        case CONDOR_3DES:
+            crypto_ = new Condor_Crypt_3des(*key);
+            break;
+#endif
+        default:
+            break;
+        }
+    }
+
+    return (crypto_ != 0);
+}
+
+bool Sock::set_MD_mode(CONDOR_MD_MODE mode, KeyInfo * key, const char * keyId)
+{
+    mdMode_ = mode;
+    delete mdKey_;
+    mdKey_ = 0;
+    if (key) {
+      mdKey_  = new KeyInfo(*key);
+    }
+
+    return init_MD(mode, mdKey_, keyId);
+}
+
+const KeyInfo& Sock :: get_crypto_key() const
+{
+#ifdef HAVE_EXT_OPENSSL
+    if (crypto_) {
+        return crypto_->get_key();
+    }
+#endif
+    ASSERT(0);	// This does not return...
+	return  crypto_->get_key();  // just to make compiler happy...
+}
+
+const KeyInfo& Sock :: get_md_key() const
+{
+#ifdef HAVE_EXT_OPENSSL
+    if (mdKey_) {
+        return *mdKey_;
+    }
+#endif
+    ASSERT(0);
+    return *mdKey_;
+}
+
+
+bool 
+Sock::set_crypto_key(bool enable, KeyInfo * key, const char * keyId)
+{
+    bool inited = true;
+#ifdef HAVE_EXT_OPENSSL
+
+    if (key != 0) {
+        inited = initialize_crypto(key);
+    }
+    else {
+        // We are turning encryption off
+        if (crypto_) {
+            delete crypto_;
+            crypto_ = 0;
+			crypto_mode_ = false;
+        }
+        ASSERT(keyId == 0);
+        ASSERT(enable == false);
+        inited = true;
+    }
+
+    // More check should be done here. what if keyId is NULL?
+    if (inited) {
+		if( enable ) {
+				// We do not set the encryption id if the default crypto
+				// mode is off, because setting the encryption id causes
+				// the UDP packet header to contain the encryption id,
+				// which causes a pre 7.1.3 receiver to think that encryption
+				// is turned on by default, even if that is not what was
+				// previously negotiated.
+			set_encryption_id(keyId);
+		}
+		set_crypto_mode(enable);
+    }
+
+#endif /* HAVE_EXT_OPENSSL */
+
+    return inited;
+}
+
+bool
+Sock::canEncrypt()
+{
+	return crypto_ != NULL;
+}
