@@ -24,18 +24,16 @@
 #include "condor_io.h"
 #include "condor_debug.h"
 #include "MyString.h"
-
-#ifdef HAVE_EXT_OPENSSL
-#include "condor_crypt_blowfish.h"
-#include "condor_crypt_3des.h"
-#include "condor_md.h"                // Message authentication stuff
-#endif
+#include "utilfns.h"
 
 /* The macro definition and file was added for debugging purposes */
 
 int putcount =0;
 int getcount = 0;
 
+// initialize static data members
+int Stream::timeout_multiplier = 0;
+
 #if 0
 static int shipcount =0;
 #define NETWORK_TRACE(s) { shipcount++; nwdump << s << "|"; \
@@ -61,10 +59,9 @@ Stream :: Stream(stream_code c) :
 		// I love individual coding style!
 		// You put _ in the front, I put in the
 		// back, very consistent, isn't it?	
-    crypto_(NULL),
+	encrypt_(false),
     crypto_mode_(false),
-    mdMode_(MD_OFF),
-    mdKey_(0),
+	m_crypto_state_before_secret(false),
     _code(c), 
     _coding(stream_encode),
 	allow_empty_message_flag(FALSE),
@@ -72,7 +69,8 @@ Stream :: Stream(stream_code c) :
 	decrypt_buf_len(0),
 	m_peer_description_str(NULL),
 	m_peer_version(NULL),
-	m_deadline_time(0)
+	m_deadline_time(0),
+	ignore_timeout_multiplier(false)
 {
 }
 
@@ -86,8 +84,6 @@ Stream::code( void *&)
 
 Stream :: ~Stream()
 {
-    delete crypto_;
-    delete mdKey_;
 	if( decrypt_buf ) {
 		free( decrypt_buf );
 	}
@@ -218,7 +214,7 @@ Stream::code( unsigned long	&l)
 }
 
 
-#if !defined(__LP64__)
+#if !defined(__LP64__) || defined(Darwin)
 int 
 Stream::code( int64_t	&l)
 {
@@ -383,6 +379,26 @@ Stream::code( MyString	&s)
 }
 
 
+int 
+Stream::code( std::string	&s)
+{
+	switch(_coding){
+		case stream_encode:
+			return put(s);
+		case stream_decode:
+			return get(s);
+		case stream_unknown:
+			EXCEPT("ERROR: Stream::code(std::string &s) has unknown direction!");
+			break;
+		default:
+			EXCEPT("ERROR: Stream::code(std::string &s)'s _coding is illegal!");
+			break;
+	}
+
+	return FALSE;	/* will never get here	*/
+}
+
+
 int 
 Stream::code( char	*&s, int		&len)
 {
@@ -448,33 +464,6 @@ Stream::code(PROC_ID &id)
 	return TRUE;
 }
 
-#if 0	// hopefully we won't neet to port the PROC structure
-
-/* extern int stream_proc_vers2( Stream *s, V2_PROC *proc ); */
-extern int stream_proc_vers3( Stream *s, PROC *proc );
-
-int 
-Stream::code(PROC &proc)
-{
-	if (!code(proc.version_num))
-		return FALSE;
-	switch( proc.version_num ) {
-/*		case 2:
-			return stream_proc_vers2( this, &proc );
-			break; */
-		case 3:
-			return stream_proc_vers3( this, &proc );
-			break;
-		default:
-			dprintf(D_ALWAYS, "Incorrect PROC version number (%d)\n",
-					proc.version_num );
-			return FALSE;
-	}
-	return TRUE;
-}
-
-#endif
-
 int 
 Stream::code(STARTUP_INFO &start)
 {
@@ -535,8 +524,6 @@ Stream::code(StartdRec &rec)
 	return TRUE;
 }
 
-extern "C" int open_flags_encode( int flags );
-extern "C" int open_flags_decode( int flags );
 
 int 
 Stream::code(open_flags_t &flags)
@@ -556,9 +543,6 @@ Stream::code(open_flags_t &flags)
 	return rval;
 }
 
-extern "C" int errno_num_encode( int errno_num );
-extern "C" int errno_num_decode( int errno_num );
-
 int 
 Stream::code(condor_errno_t &errno_num)
 {
@@ -583,8 +567,7 @@ Stream::code(condor_errno_t &errno_num)
 **	UNIX TYPES
 */
 
-extern "C" int sig_num_encode( int sig_num );
-extern "C" int sig_num_decode( int sig_num );
+
 
 int 
 Stream::code(condor_signal_t &sig_num)
@@ -605,9 +588,6 @@ Stream::code(condor_signal_t &sig_num)
 }
 
 
-extern "C" int fcntl_cmd_encode( int cmd );
-extern "C" int fcntl_cmd_decode( int cmd );
-
 int 
 Stream::code(fcntl_cmd_t &cmd)
 {
@@ -716,7 +696,7 @@ Stream::code(struct statfs &s)
 	STREAM_ASSERT(code(s.f_files));
 	STREAM_ASSERT(code(s.f_ffree));
 
-#if defined(Solaris) || defined(IRIX)
+#if defined(Solaris)
 	STREAM_ASSERT(code(s.f_bfree));
 #else
 	STREAM_ASSERT(code(s.f_bavail));
@@ -755,42 +735,27 @@ Stream::code(struct utimbuf &ut)
 int 
 Stream::code(struct rlimit &rl)
 {
-#ifdef LINUX
-	// Cedar is too damn smart for us.  
-	// The issue is Linux changed the type for rlimits
-	// from a signed long (RedHat 6.2 and before) into
-	// an unsigned long (starting w/ RedHat 7.x).  So if
-	// the shadow is on RedHat 7.2 and sends the results
-	// of getrlimits to a RedHat 6.2 syscall lib, for instance,
-	// it could overflow.  And Cedar is sooo smart.  Sooo very, very
-	// smart.  It return an error if there is an overflow, which
-	// in turn causes an ASSERT to fail in senders/receivers land,
-	// which is bad news for us.  Thus this hack.  -Todd,Erik,Hao Feb 2002
+	// There used to be a piece of code here which implemented backwards
+	// compatibility between redhat 6.2 and 7.x on account of the type of
+	// the rlimit members changing from signed in 6.2 to unsigned in 7.x.
+	// If the shadow thought it was unsigned, but the stduniv executable
+	// thought it was signed, then an overflow error could happen in the 
+	// stduniv executable and this made people upset.
 	//
-	// Note: This has the unfortunate side effect of limitting the 'rlimit'
-	// that Condor supports to 2^31.  -Nick
-	if( is_encode() ) {
-		const unsigned long MAX_RLIMIT = 0x7fffffffLU;
-		long				max_rlimit = (long) MAX_RLIMIT;
+	// Unfortunately, not only did the backwards compatibility code limit
+	// the rlimit to less than 2^31, it also screwed up RLIM_INFINITY, which
+	// is represented as the (now unsigned bit pattern) of -1.
+	//
+	// It has been MANY years since the original backwards compatiblity fix
+	// and I would be very surprised if any redhat 6.2 stduniv executables
+	// are still running! Hence, I've returned the code back to the original
+	// form since both sides will assume the type to be unsigned.
+	//
+	// You can dig into the log history of this file to find the old
+	// compatibility codes.
 	
-		if ( ((unsigned long)rl.rlim_cur) > MAX_RLIMIT ) {
-			STREAM_ASSERT(code(max_rlimit));
-		} else {
-			STREAM_ASSERT(code(rl.rlim_cur));
-		}
-		if ( ((unsigned long)rl.rlim_max) > MAX_RLIMIT ) {
-			STREAM_ASSERT(code(max_rlimit));
-		} else {
-		STREAM_ASSERT(code(rl.rlim_max));
-		}
-	} else {
-		STREAM_ASSERT(code(rl.rlim_cur));
-		STREAM_ASSERT(code(rl.rlim_max));
-	}
-#else
 	STREAM_ASSERT(code(rl.rlim_cur));
 	STREAM_ASSERT(code(rl.rlim_max));
-#endif
 
 	return TRUE;
 }
@@ -847,28 +812,6 @@ Stream::code(struct utsname &n)
 	
 	#if defined(LINUX)
 		n.domainname[0] = 0;
-	#elif defined(IRIX)
-		#if defined(IRIX62)
-			#if defined(m_type)
-				n.__m_type[0] = 0;
-			#else
-				n.m_type[0] = 0;
-			#endif
-			#if defined(base_rel)
-				n.__base_rel[0] = 0;
-			#else
-				n.base_rel[0] = 0;
-			#endif
-		#endif
-		#if defined(IRIX65)
-			#if defined(_ABIAPI)
-				n.__m_type[0] = 0;
-				n.__base_rel[0] = 0;
-			#else
-				n.m_type[0] = 0;
-				n.base_rel[0] = 0;
-			#endif
-		#endif
 	#elif defined(HPUX)
 		n.__idnumber[0] = 0;
 	#endif
@@ -1052,33 +995,6 @@ static unsigned long ntohL(unsigned long netint)
 	return hostint;
 }
 
-// no hton function is provided for ints > 4 bytes by any OS.
-static uint64_t htonLL(uint64_t hostint)
-{
-	uint64_t netint;
-	char *hostp = (char *)&hostint;
-	char *netp = (char *)&netint;
-
-	for (unsigned int i=0, j=sizeof(uint64_t)-1; i < sizeof(uint64_t); i++, j--) {
-		netp[i] = hostp[j];
-	}
-	return netint;
-}
-
-// no ntoh function is provided for ints > 4 bytes by any OS.
-static uint64_t ntohLL(uint64_t netint)
-{
-	uint64_t hostint;
-	char *hostp = (char *)&hostint;
-	char *netp = (char *)&netint;
-
-	for (unsigned int i=0, j=sizeof(uint64_t)-1; i < sizeof(uint64_t); i++, j--) {
-		hostp[i] = netp[j];
-	}
-	return hostint;
-}
-
-
 int 
 Stream::put( long	l)
 {
@@ -1152,7 +1068,36 @@ Stream::put( unsigned long	l)
 }
 
 
-#if !defined(__LP64__)
+#if !defined(__LP64__) || defined(Darwin)
+
+// no hton function is provided for ints > 4 bytes by any OS,
+// but we only use it here.
+static uint64_t htonLL(uint64_t hostint)
+{
+	uint64_t netint;
+	char *hostp = (char *)&hostint;
+	char *netp = (char *)&netint;
+
+	for (unsigned int i=0, j=sizeof(uint64_t)-1; i < sizeof(uint64_t); i++, j--) {
+		netp[i] = hostp[j];
+	}
+	return netint;
+}
+
+// no ntoh function is provided for ints > 4 bytes by any OS,
+// but we only use it here.
+static uint64_t ntohLL(uint64_t netint)
+{
+	uint64_t hostint;
+	char *hostp = (char *)&hostint;
+	char *netp = (char *)&netint;
+
+	for (unsigned int i=0, j=sizeof(uint64_t)-1; i < sizeof(uint64_t); i++, j--) {
+		hostp[i] = netp[j];
+	}
+	return hostint;
+}
+
 int 
 Stream::put( int64_t	l)
 {
@@ -1361,6 +1306,12 @@ Stream::put( const MyString &s)
 	return put( s.Value() );
 }
 
+int 
+Stream::put( const std::string &s)
+{
+	return put( s.c_str() );
+}
+
 int
 Stream::put_secret( char const *s )
 {
@@ -1661,7 +1612,7 @@ Stream::get( unsigned long	&l)
 }
 
 
-#if !defined(__LP64__)
+#if !defined(__LP64__) || defined(Darwin)
 int 
 Stream::get( int64_t	&l)
 {
@@ -2018,6 +1969,25 @@ Stream::get( MyString	&s)
 	return result;
 }
 
+int 
+Stream::get( std::string	&s)
+{
+	char const *ptr = NULL;
+	int result = get_string_ptr(ptr);
+	if( result == TRUE ) {
+		if( ptr ) {
+			s = ptr;
+		}
+		else {
+			s = "";
+		}
+	}
+	else {
+		s = "";
+	}
+	return result;
+}
+
 int 
 Stream::snd_int(
 	int val, 
@@ -2065,11 +2035,10 @@ Stream::allow_one_empty_message()
 	allow_empty_message_flag = TRUE;
 }
 
-
 void 
 Stream::set_crypto_mode(bool enabled)
 {
-	if (crypto_ && enabled) {
+	if (canEncrypt() && enabled) {
 		crypto_mode_ = true;
 	} else {
 		if (enabled) {
@@ -2085,142 +2054,6 @@ Stream::get_encryption() const
     return (crypto_mode_);
 }
 
-bool 
-Stream::wrap(unsigned char* d_in,int l_in, 
-                    unsigned char*& d_out,int& l_out)
-{    
-    bool coded = false;
-#ifdef HAVE_EXT_OPENSSL
-    if (get_encryption()) {
-        coded = crypto_->encrypt(d_in, l_in, d_out, l_out);
-    }
-#endif
-    return coded;
-}
-
-bool 
-Stream::unwrap(unsigned char* d_in,int l_in,
-                      unsigned char*& d_out, int& l_out)
-{
-    bool coded = false;
-#ifdef HAVE_EXT_OPENSSL
-    if (get_encryption()) {
-        coded = crypto_->decrypt(d_in, l_in, d_out, l_out);
-    }
-#endif
-    return coded;
-}
-
-void Stream::resetCrypto()
-{
-#ifdef HAVE_EXT_OPENSSL
-  if (crypto_) {
-    crypto_->resetState();
-  }
-#endif
-}
-
-bool 
-Stream::initialize_crypto(KeyInfo * key) 
-{
-    delete crypto_;
-    crypto_ = 0;
-	crypto_mode_ = false;
-
-    // Will try to do a throw/catch later on
-    if (key) {
-        switch (key->getProtocol()) 
-        {
-#ifdef HAVE_EXT_OPENSSL
-        case CONDOR_BLOWFISH :
-            crypto_ = new Condor_Crypt_Blowfish(*key);
-            break;
-        case CONDOR_3DES:
-            crypto_ = new Condor_Crypt_3des(*key);
-            break;
-#endif
-        default:
-            break;
-        }
-    }
-
-    return (crypto_ != 0);
-}
-
-bool Stream::set_MD_mode(CONDOR_MD_MODE mode, KeyInfo * key, const char * keyId)
-{
-    mdMode_ = mode;
-    delete mdKey_;
-    mdKey_ = 0;
-    if (key) {
-      mdKey_  = new KeyInfo(*key);
-    }
-
-    return init_MD(mode, mdKey_, keyId);
-}
-
-const KeyInfo& Stream :: get_crypto_key() const
-{
-#ifdef HAVE_EXT_OPENSSL
-    if (crypto_) {
-        return crypto_->get_key();
-    }
-#endif
-    ASSERT(0);	// This does not return...
-	return  crypto_->get_key();  // just to make compiler happy...
-}
-
-const KeyInfo& Stream :: get_md_key() const
-{
-#ifdef HAVE_EXT_OPENSSL
-    if (mdKey_) {
-        return *mdKey_;
-    }
-#endif
-    ASSERT(0);
-    return *mdKey_;
-}
-
-
-bool 
-Stream::set_crypto_key(bool enable, KeyInfo * key, const char * keyId)
-{
-    bool inited = true;
-#ifdef HAVE_EXT_OPENSSL
-
-    if (key != 0) {
-        inited = initialize_crypto(key);
-    }
-    else {
-        // We are turning encryption off
-        if (crypto_) {
-            delete crypto_;
-            crypto_ = 0;
-			crypto_mode_ = false;
-        }
-        ASSERT(keyId == 0);
-        ASSERT(enable == false);
-        inited = true;
-    }
-
-    // More check should be done here. what if keyId is NULL?
-    if (inited) {
-		if( enable ) {
-				// We do not set the encryption id if the default crypto
-				// mode is off, because setting the encryption id causes
-				// the UDP packet header to contain the encryption id,
-				// which causes a pre 7.1.3 receiver to think that encryption
-				// is turned on by default, even if that is not what was
-				// previously negotiated.
-			set_encryption_id(keyId);
-		}
-		set_crypto_mode(enable);
-    }
-
-#endif /* HAVE_EXT_OPENSSL */
-
-    return inited;
-}
 
 char const *
 Stream::peer_description() {
@@ -2295,3 +2128,51 @@ Stream::deadline_expired()
 {
 	return m_deadline_time != 0 && time(NULL) > m_deadline_time;
 }
+
+bool
+Stream::prepare_crypto_for_secret_is_noop()
+{
+	CondorVersionInfo const *peer_ver = get_peer_version();
+	if( !peer_ver || peer_ver->built_since_version(7,1,3) ) {
+		if( !get_encryption() ) {
+			if( canEncrypt() ) {
+					// do turn on encryption before sending secret
+				return false;
+			}
+		}
+	}
+	return true;
+}
+
+void
+Stream::prepare_crypto_for_secret()
+{
+	m_crypto_state_before_secret = true;
+	if( !prepare_crypto_for_secret_is_noop() ) {
+		dprintf(D_NETWORK,"encrypting secret\n");
+		m_crypto_state_before_secret = get_encryption(); // always false
+		set_crypto_mode(true);
+	}
+}
+
+void
+Stream::restore_crypto_after_secret()
+{
+	if( !m_crypto_state_before_secret ) {
+		set_crypto_mode(false); //restore crypto mode
+	}
+}
+
+int
+Stream::set_timeout_multiplier(int secs)
+{
+   int old_val = timeout_multiplier;
+   timeout_multiplier = secs;
+   return old_val;
+}
+
+int
+Stream::get_timeout_multiplier()
+{
+	return timeout_multiplier;
+}
