@@ -37,6 +37,9 @@
 #include "file_lock.h"
 #include "classad_newold.h"
 #include "user_job_policy.h"
+#include "get_daemon_name.h"
+#include "filename_tools.h"
+
 
 
 const char JR_ATTR_MAX_JOBS[] = "MaxJobs";
@@ -69,12 +72,16 @@ JobRouter::JobRouter(Scheduler *scheduler): m_jobs(5000,hashFuncStdString,reject
 	m_job_router_entries_refresh = 0;
 	m_job_router_refresh_timer = -1;
 
+        m_public_ad_update_timer = -1;
+        m_public_ad_update_interval = -1;
+
 	m_routes = AllocateRoutingTable();
 	m_poll_count = 0;
 
 	m_router_lock_fd = -1;
 	m_router_lock = NULL;
-	m_custom_attrs = NULL;
+	m_max_jobs = -1;
+	m_max_job_mirror_update_lag = 600;
 
 #if HAVE_JOB_HOOKS
 	m_hook_mgr = NULL;
@@ -98,8 +105,11 @@ JobRouter::~JobRouter() {
 	if(m_router_lock_fd != -1) {
 		close(m_router_lock_fd);
 	}
-	if( m_job_router_refresh_timer >= 0 ) {
-		daemonCore->Cancel_Timer( m_job_router_refresh_timer );
+	if(m_job_router_refresh_timer >= 0) {
+		daemonCore->Cancel_Timer(m_job_router_refresh_timer);
+	}
+	if(m_public_ad_update_timer >= 0) {
+		daemonCore->Cancel_Timer(m_public_ad_update_timer);
 	}
 
 #if HAVE_JOB_HOOKS
@@ -108,6 +118,7 @@ JobRouter::~JobRouter() {
 		delete m_hook_mgr;
 	}
 #endif
+	InvalidatePublicAd();
 }
 
 #include "condor_new_classads.h"
@@ -126,43 +137,11 @@ void
 JobRouter::GetInstanceLock() {
 	std::string lock_fullname;
 	std::string lock_basename;
-	char *lock_dir;
-
-	// Since JobRouterName() may contain characters that are not allowed
-	// (or not portable) in a filename, use an MD5 hash of it for the
-	// lock file name.
-	Condor_MD_MAC MD5;
-	std::string router_name = JobRouterName();
-	unsigned char *md5_str;
-	MD5.addMD((const unsigned char *)router_name.c_str(),router_name.size());
-	md5_str = MD5.computeMD();
-	if(md5_str) {
-		int i;
-		for(i=0;i<MAC_SIZE;i++) {
-			char buf[10];
-			sprintf(buf,"%x",md5_str[i]);
-			lock_basename += buf;
-		}
-		free(md5_str);
-	}
-	else {
-		// MD5 may not be supported in all ports of Condor.  Fake it.
-		unsigned long n = 0;
-		char const *s = router_name.c_str();
-		while(*s) {
-			n += (unsigned char)*s;
-		}
-		char n_buf[100];
-		sprintf(n_buf,"%lu",n);
-		lock_basename = n_buf;
-	}
-	lock_basename += ".JobRouter.lock";
 
 	// We may be an ordinary user, so cannot lock in $(LOCK)
-	lock_dir = "/tmp";
-	lock_fullname = lock_dir;
-	lock_fullname += DIR_DELIM_CHAR;
-	lock_fullname += lock_basename;
+	param(lock_fullname,"JOB_ROUTER_LOCK");
+	ASSERT( !lock_fullname.empty() );
+	canonicalize_dir_delimiters((char *)lock_fullname.c_str());
 
 	m_router_lock_fd = safe_open_wrapper(lock_fullname.c_str(),O_CREAT|O_APPEND|O_WRONLY,0600);
 	if(m_router_lock_fd == -1) {
@@ -203,7 +182,7 @@ JobRouter::config() {
 			daemonCore->Register_Timer(
 				m_job_router_entries_refresh,
 				m_job_router_entries_refresh,
-				(Eventcpp)&JobRouter::config,
+				(TimerHandlercpp)&JobRouter::config,
 				"JobRouter::config", this);
 	}
 
@@ -232,7 +211,10 @@ JobRouter::config() {
 		}
 		free(router_defaults_str);
 	}
-	if(!m_enable_job_routing) return;
+	if(!m_enable_job_routing) {
+		delete new_routes;
+		return;
+	}
 
 	char *routing_str_s = param(PARAM_JOB_ROUTER_ENTRIES);
 	char *routing_file_s = param(PARAM_JOB_ROUTER_ENTRIES_FILE);
@@ -321,8 +303,6 @@ JobRouter::config() {
 		// goes on hold
 	m_release_on_hold = param_boolean("JOB_ROUTER_RELEASE_ON_HOLD", true);
 
-	m_custom_attrs = param("JOB_ROUTER_ATTRS_TO_COPY");
-
 		// default is no maximum (-1)
 	m_max_jobs = param_integer("JOB_ROUTER_MAX_JOBS",-1);
 
@@ -330,6 +310,7 @@ JobRouter::config() {
 
 	Timeslice periodic_interval;
 	periodic_interval.setMinInterval(param_integer("PERIODIC_EXPR_INTERVAL", 60));
+	periodic_interval.setMaxInterval(param_integer("MAX_PERIODIC_EXPR_INTERVAL", 1200) );
 	periodic_interval.setTimeslice(param_double("PERIODIC_EXPR_TIMESLICE", 0.01, 0, 1));
 
 		// read the polling period and if one is not specified use 
@@ -347,12 +328,12 @@ JobRouter::config() {
 	m_job_router_polling_timer = daemonCore->Register_Timer(
 								  0, 
 								  m_job_router_polling_period,
-								  (Eventcpp)&JobRouter::Poll, 
+								  (TimerHandlercpp)&JobRouter::Poll, 
 								  "JobRouter::Poll", this);
 
 	if (periodic_interval.getMinInterval() > 0) {
 		m_periodic_timer_id = daemonCore->Register_Timer(periodic_interval, 
-								(Eventcpp)&JobRouter::EvalAllSrcJobPeriodicExprs,
+								(TimerHandlercpp)&JobRouter::EvalAllSrcJobPeriodicExprs,
 								"JobRouter::EvalAllSrcJobPeriodicExprs",
 								this);
 		dprintf(D_FULLDEBUG, "JobRouter: Registered EvalAllSrcJobPeriodicExprs() to evaluate periodic expressions.\n");
@@ -361,16 +342,59 @@ JobRouter::config() {
 		dprintf(D_FULLDEBUG, "JobRouter: Evaluation of periodic expressions disabled.\n");
 	}
 
-	char *name = param("JOB_ROUTER_NAME");
-	if(name) {
-		m_job_router_name = name;
-		free(name);
+		// NOTE: if you change the default name, then you are breaking
+		// JobRouter's ability to adopt jobs ("orphans") left behind
+		// by the previous version of JobRouter.  At least a warning
+		// in the release notes is warranted.
+	param(m_job_router_name,"JOB_ROUTER_NAME");
+		// In order not to get confused by jobs belonging to
+		// gridmanager in AdoptOphans(), the job router name must not
+		// be empty.
+	if( m_job_router_name.empty() ) {
+		EXCEPT("JOB_ROUTER_NAME must not be empty");
 	}
-	else {
-		m_job_router_name = DaemonIdentityString();
+
+	InitPublicAd();
+
+	int update_interval = param_integer("UPDATE_INTERVAL", 60);
+	if(m_public_ad_update_interval != update_interval) {
+		m_public_ad_update_interval = update_interval;
+
+		if(m_public_ad_update_timer >= 0) {
+			daemonCore->Cancel_Timer(m_public_ad_update_timer);
+			m_public_ad_update_timer = -1;
+		}
+		dprintf(D_FULLDEBUG, "Setting update interval to %d\n",
+			m_public_ad_update_interval);
+		m_public_ad_update_timer = daemonCore->Register_Timer(
+			0,
+			m_public_ad_update_interval,
+			(TimerHandlercpp)&JobRouter::TimerHandler_UpdateCollector,
+			"JobRouter::TimerHandler_UpdateCollector",
+			this);
 	}
 }
 
+void
+JobRouter::InitPublicAd()
+{
+	ASSERT (m_job_router_name.size() > 0);
+
+	char *valid_name = build_valid_daemon_name(m_job_router_name.c_str());
+	ASSERT( valid_name );
+	daemonName = valid_name;
+	delete [] valid_name;
+
+	m_public_ad = ClassAd();
+
+	m_public_ad.SetMyTypeName("Job_Router");
+	m_public_ad.SetTargetTypeName("");
+
+	m_public_ad.Assign(ATTR_NAME,daemonName.c_str());
+
+	daemonCore->publish(&m_public_ad);
+}
+
 void
 JobRouter::EvalAllSrcJobPeriodicExprs()
 {
@@ -390,12 +414,20 @@ JobRouter::EvalAllSrcJobPeriodicExprs()
 			dprintf(D_ALWAYS, "JobRouter failure (%s): Unable to "
 					"evaluate job's periodic policy "
 					"expressions.\n", job->JobDesc().c_str());
+			if( !orig_ad ) {
+				dprintf(D_ALWAYS, "JobRouter failure (%s): "
+					"failed to reset src job "
+					"attributes, because ad not found"
+					"in collection.\n",job->JobDesc().c_str());
+				continue;
+			}
+
 			job->SetSrcJobAd(job->src_key.c_str(), orig_ad, ad_collection);
 			if (false == push_dirty_attributes(job->src_ad,NULL,NULL))
 			{
 				dprintf(D_ALWAYS, "JobRouter failure (%s): "
 						"failed to reset src job "
-						"attributesin the schedd.\n",
+						"attributes in the schedd.\n",
 						job->JobDesc().c_str());
 			}
 			else
@@ -741,8 +773,10 @@ RoutedJob::RoutedJob() {
 	is_success = false;
 	is_sandboxed = false;
 	submission_time = 0;
+	retirement_time = 0;
 	proxy_file_copy_chowned = false;
 	target_universe = CONDOR_UNIVERSE_GRID;
+	saw_dest_job = false;
 }
 RoutedJob::~RoutedJob() {
 }
@@ -828,7 +862,7 @@ JobRouter::AdoptOrphans() {
 
 	// Find all jobs submitted by a JobRouter with same name as me.
 	// Maybe some of them were from a previous life and are now orphans.
-	dest_jobs += "other.ProcId >= 0 && other.";
+	dest_jobs += "target.ProcId >= 0 && target.";
 	dest_jobs += JR_ATTR_ROUTED_BY;
 	dest_jobs += " == \"";
 	dest_jobs += m_job_router_name;
@@ -836,7 +870,7 @@ JobRouter::AdoptOrphans() {
 		//Have observed problems in which we get inconsistent snapshot
 		//of the job queue; ensure that we at least have the Owner
 		//attribute, or we'll run into trouble.
-	dest_jobs += " && other.Owner isnt Undefined";
+	dest_jobs += " && target.Owner isnt Undefined";
 
 	constraint_tree = parser.ParseExpression(dest_jobs.c_str());
 	if(!constraint_tree) {
@@ -938,13 +972,13 @@ JobRouter::AdoptOrphans() {
 	} while (query.Next(dest_key));
 
 
-	src_jobs = "other.Managed == \"External\" && other.ManagedManager == \"";
+	src_jobs = "target.Managed == \"External\" && target.ManagedManager == \"";
 	src_jobs += m_job_router_name;
 	src_jobs += "\"";
 		//Have observed problems in which we get inconsistent snapshot
 		//of the job queue; ensure that we at least have the Owner
 		//attribute, or we'll run into trouble.
-	src_jobs += " && other.Owner isnt Undefined";
+	src_jobs += " && target.Owner isnt Undefined";
 
 	constraint_tree = parser.ParseExpression(src_jobs.c_str());
 	if(!constraint_tree) {
@@ -1058,8 +1092,11 @@ JobRouter::GetCandidateJobs() {
 	umbrella_constraint += " )";
 
 	//Add on basic requirements to keep things sane.
-	//For now, require vanilla universe, but in future this may be removed.
-	umbrella_constraint += " && (other.ProcId >= 0 && other.JobStatus == 1 && other.JobUniverse == 5 && other.Managed isnt \"ScheddDone\" && other.Managed isnt \"External\" && other.Owner isnt Undefined)";
+	umbrella_constraint += " && (target.ProcId >= 0 && target.JobStatus == 1 && target.Managed isnt \"ScheddDone\" && target.Managed isnt \"External\" && target.Owner isnt Undefined && target.";
+	umbrella_constraint += JR_ATTR_ROUTED_BY;
+	umbrella_constraint += " isnt \"";
+	umbrella_constraint += m_job_router_name;
+	umbrella_constraint += "\")";
 
 	if(!can_switch_ids()) {
 			// We are not running as root.  Ensure that we only try to
@@ -1070,13 +1107,13 @@ JobRouter::GetCandidateJobs() {
 
 		ASSERT(username);
 
-		umbrella_constraint += " && (other.";
+		umbrella_constraint += " && (target.";
 		umbrella_constraint += ATTR_OWNER;
 		umbrella_constraint += " == \"";
 		umbrella_constraint += username;
 		umbrella_constraint += "\"";
 		if(domain) {
-			umbrella_constraint += " && other.";
+			umbrella_constraint += " && target.";
 			umbrella_constraint += ATTR_NT_DOMAIN;
 			umbrella_constraint += " == \"";
 			umbrella_constraint += domain;
@@ -1214,12 +1251,14 @@ JobRouter::UpdateRouteStats() {
 	}
 }
 
+/*
 std::string
 JobRouter::DaemonIdentityString() {
 	std::string identity;
 	identity += m_scheduler->Name();
 	return identity;
 }
+*/
 
 void
 JobRouter::TakeOverJob(RoutedJob *job) {
@@ -1563,6 +1602,9 @@ void
 JobRouter::FinishCheckSubmittedJobStatus(RoutedJob *job) {
 	classad::ClassAdCollection *ad_collection = m_scheduler->GetClassAds();
 	classad::ClassAd *src_ad = ad_collection->GetClassAd(job->src_key);
+	std::string keyword;
+	std::string copy_attr_param;
+	char* custom_attrs = NULL;
 
 	if(!src_ad) {
 		dprintf(D_ALWAYS,"JobRouter (%s): failed to find src ad in job collection mirror.\n",job->JobDesc().c_str());
@@ -1599,17 +1641,32 @@ JobRouter::FinishCheckSubmittedJobStatus(RoutedJob *job) {
 
 	if(!ad) {
 		int age = time(NULL) - job->submission_time;
-		if(age > m_max_job_mirror_update_lag) {
-			dprintf(D_ALWAYS,"JobRouter failure (%s): giving up, because submitted job is still not in job queue mirror (submitted %d seconds ago)\n",job->JobDesc().c_str(),age);
+		if(job->SawDestJob()) {
+				// we have seen the dest job before, but now it is gone,
+				// so it must have been removed
+			dprintf(D_ALWAYS,"JobRouter (%s): dest job was removed!\n",job->JobDesc().c_str());
 			GracefullyRemoveJob(job);
 			return;
 		}
-		dprintf(D_FULLDEBUG,"JobRouter (%s): submitted job has not yet appeared in job queue mirror or was removed by user (submitted %d seconds ago)\n",job->JobDesc().c_str(),age);
+		if(age > m_max_job_mirror_update_lag) {
+			dprintf(D_ALWAYS,"JobRouter failure (%s): giving up, because submitted job is still not in job queue mirror (submitted %d seconds ago).  Perhaps it has been removed?\n",job->JobDesc().c_str(),age);
+			GracefullyRemoveJob(job);
+			return;
+		}
+		dprintf(D_FULLDEBUG,"JobRouter (%s): submitted job has not yet appeared in job queue mirror or was removed (submitted %d seconds ago)\n",job->JobDesc().c_str(),age);
 		return;
 	}
 
 	job->SetDestJobAd(ad);
-	if(!update_job_status(*src_ad,job->dest_ad,job->src_ad,m_custom_attrs)) {
+#if HAVE_JOB_HOOKS
+	keyword = m_hook_mgr->getHookKeyword(job->src_ad);
+	if(0 < keyword.length()) {
+		copy_attr_param = keyword;
+		copy_attr_param += "_ATTRS_TO_COPY";
+		custom_attrs = param(copy_attr_param.c_str());
+	}
+#endif
+	if(!update_job_status(*src_ad,job->dest_ad,job->src_ad,custom_attrs)) {
 		dprintf(D_ALWAYS,"JobRouter failure (%s): failed to update job status\n",job->JobDesc().c_str());
 	}
 	else if(ClassAdHasDirtyAttributes(&job->src_ad)) {
@@ -1617,12 +1674,24 @@ JobRouter::FinishCheckSubmittedJobStatus(RoutedJob *job) {
 			dprintf(D_ALWAYS,"JobRouter failure (%s): failed to update src job\n",job->JobDesc().c_str());
 
 			GracefullyRemoveJob(job);
+#if HAVE_JOB_HOOKS
+			if (custom_attrs != NULL) {
+				free(custom_attrs);
+				custom_attrs = NULL;
+			}
+#endif
 			return;
 		}
 		else {
 			dprintf(D_FULLDEBUG,"JobRouter (%s): updated job status\n",job->JobDesc().c_str());
 		}
 	}
+#if HAVE_JOB_HOOKS
+	if (custom_attrs != NULL) {
+		free(custom_attrs);
+		custom_attrs = NULL;
+	}
+#endif
 
 	job_status = 0;
 	if( !ad->EvaluateAttrInt( ATTR_JOB_STATUS, job_status ) ) {
@@ -1630,7 +1699,7 @@ JobRouter::FinishCheckSubmittedJobStatus(RoutedJob *job) {
 		GracefullyRemoveJob(job);
 		return;
 	}
-	job->is_running = (job_status == RUNNING);
+	job->is_running = (job_status == RUNNING || job_status == TRANSFERRING_OUTPUT);
 
 	int job_finished = 0;
 	if( !ad->EvaluateAttrInt( ATTR_JOB_FINISHED_HOOK_DONE, job_finished ) ) {
@@ -1747,7 +1816,7 @@ JobRouter::TestJobSuccess(RoutedJob *job)
 	classad::ClassAd *upd;
 	classad::ClassAdParser parser;
 	std::string upd_str;
-	upd_str = "[leftJobFailureTest = adcl.ad.";
+	upd_str = "[leftJobFailureTest = LEFT.";
 	upd_str += JR_ATTR_JOB_FAILURE_TEST;
 	upd_str += " ;]";
 	upd = parser.ParseClassAd(upd_str);
@@ -1786,7 +1855,7 @@ JobRouter::TestJobSandboxed(RoutedJob *job)
 	classad::ClassAd *upd;
 	classad::ClassAdParser parser;
 	std::string upd_str;
-	upd_str = "[leftJobSandboxedTest = adcl.ad.";
+	upd_str = "[leftJobSandboxedTest = LEFT.";
 	upd_str += JR_ATTR_JOB_SANDBOXED_TEST;
 	upd_str += " ;]";
 	upd = parser.ParseClassAd(upd_str);
@@ -1819,7 +1888,7 @@ JobRoute::EvalUseSharedX509UserProxy(RoutedJob *job)
 	classad::ClassAd *upd;
 	classad::ClassAdParser parser;
 	std::string upd_str;
-	upd_str = "[leftTest = adcl.ad.";
+	upd_str = "[leftTest = LEFT.";
 	upd_str += JR_ATTR_USE_SHARED_X509_USER_PROXY;
 	upd_str += " ;]";
 	upd = parser.ParseClassAd(upd_str);
@@ -1851,7 +1920,7 @@ JobRoute::EvalSharedX509UserProxy(RoutedJob *job,std::string &proxy_file)
 	classad::ClassAd *upd;
 	classad::ClassAdParser parser;
 	std::string upd_str;
-	upd_str = "[leftValue = adcl.ad.";
+	upd_str = "[leftValue = LEFT.";
 	upd_str += JR_ATTR_SHARED_X509_USER_PROXY;
 	upd_str += " ;]";
 	upd = parser.ParseClassAd(upd_str);
@@ -2038,6 +2107,24 @@ JobRouter::CleanupRetiredJob(RoutedJob *job) {
 	}
 }
 
+void
+JobRouter::TimerHandler_UpdateCollector() {
+	daemonCore->sendUpdates(UPDATE_AD_GENERIC, &m_public_ad);
+}
+
+void
+JobRouter::InvalidatePublicAd() {
+	ClassAd invalidate_ad;
+	MyString line;
+
+	invalidate_ad.SetMyTypeName(QUERY_ADTYPE);
+	invalidate_ad.SetTargetTypeName("Job_Router");
+
+	line.sprintf("%s == \"%s\"", ATTR_NAME, daemonName.c_str());
+	invalidate_ad.AssignExpr(ATTR_REQUIREMENTS, line.Value());
+	daemonCore->sendUpdates(INVALIDATE_ADS_GENERIC, &invalidate_ad, NULL, false);
+}
+
 JobRoute::JobRoute() {
 	m_num_jobs = 0;
 	m_num_running_jobs = 0;
@@ -2051,6 +2138,7 @@ JobRoute::JobRoute() {
 	m_throttle = 0;
 	m_override_routing_entry = -1;
 	m_target_universe = CONDOR_UNIVERSE_GRID;
+	m_route_requirements = NULL;
 }
 
 JobRoute::~JobRoute() {
@@ -2121,10 +2209,21 @@ JobRoute::AdjustFailureThrottles() {
 		new_throttle = THROTTLE_UPDATE_INTERVAL * m_failure_rate_threshold / failure_ratio;
 	}
 	else {
-		if (m_recent_jobs_succeeded) {
+        // heuristic for accelerating:
+        //   - if all jobs are succeeding, accel by x5
+        //   - if half are succeeding and half failing, accel x2.5
+        //   - if no failures or successes, accel x2.0
+        //   - if all jobs are failing, do not change
+        float accel = (3.0*m_recent_jobs_succeeded - 2.0*m_recent_jobs_failed);
+        if( accel > 0 ) {
+            accel /= (m_recent_jobs_succeeded + m_recent_jobs_failed);
+        }
+        accel += 2.0;
+		if( accel > 1.0 ) {
 				//Accelerate.
-			new_throttle *= 5;
+			new_throttle *= accel;
 		}
+
 		if( new_throttle > THROTTLE_UPDATE_INTERVAL * m_failure_rate_threshold * 10000 ) {
 				//Things seem to be going fine.  Remove throttle.
 			new_throttle = 0;
@@ -2349,4 +2448,5 @@ RoutedJob::SetDestJobAd(classad::ClassAd const *ad) {
 	// could get deleted before we are done with this ad.
 
 	ASSERT(dest_ad.CopyFromChain(*ad));
+	saw_dest_job = true;
 }
