@@ -96,9 +96,12 @@ bool VanillaToGrid::vanillaToGrid(classad::ClassAd * ad, int target_universe, co
 	ad->InsertAttr(ATTR_NUM_RESTARTS, 0);
 	ad->InsertAttr(ATTR_NUM_SYSTEM_HOLDS, 0);
 	ad->InsertAttr(ATTR_JOB_COMMITTED_TIME, 0);
+	ad->InsertAttr(ATTR_COMMITTED_SLOT_TIME, 0);
+	ad->InsertAttr(ATTR_CUMULATIVE_SLOT_TIME, 0);
 	ad->InsertAttr(ATTR_TOTAL_SUSPENSIONS, 0);
 	ad->InsertAttr(ATTR_LAST_SUSPENSION_TIME, 0);
 	ad->InsertAttr(ATTR_CUMULATIVE_SUSPENSION_TIME, 0);
+	ad->InsertAttr(ATTR_COMMITTED_SUSPENSION_TIME, 0);
 	ad->InsertAttr(ATTR_ON_EXIT_BY_SIGNAL, false);
 
 
@@ -300,6 +303,7 @@ bool update_job_status( classad::ClassAd const & orig, classad::ClassAd & newgri
 		ATTR_EXECUTABLE_SIZE,
 		ATTR_IMAGE_SIZE,
 		ATTR_SHADOW_BIRTHDATE,
+		ATTR_SPOOLED_OUTPUT_FILES,
 		NULL };		// list must end with a NULL
 		// ATTR_JOB_STATUS
 
@@ -328,6 +332,7 @@ bool update_job_status( classad::ClassAd const & orig, classad::ClassAd & newgri
 			set_job_status_idle(orig,update);
 			break;
 		case RUNNING:
+		case TRANSFERRING_OUTPUT:
 			set_job_status_running(orig,update);
 			break;
 		case HELD:
