@@ -28,13 +28,13 @@
 #define WANT_CLASSAD_NAMESPACE
 #include "classad/classad_distribution.h"
 #include "classad_oldnew.h"
-#include "conversion.h"
+#include "../classad_analysis/conversion.h"
 using namespace std;
 
 #include "subsystem_info.h"
 #include "lease_manager.h"
 #include "lease_manager_resources.h"
-#include "newclassad_stream.h"
+#include "newclassad_stream.unix.h"
 #include "debug_timer_dprintf.h"
 
 class LeaseManagerIntervalTimer
@@ -406,7 +406,7 @@ LeaseManager::commandHandler_RenewLease(int command, Stream *stream)
 	}
 	stream->encode( );
 	dprintf (D_FULLDEBUG, "renew (%d): %d leases to renew\n",
-			 command, renew_list.size() );
+			 command, (int) renew_list.size() );
 
 	// Do the actual renewal
 	list <const LeaseManagerLease *> *const_renew_list =
@@ -434,8 +434,9 @@ LeaseManager::commandHandler_RenewLease(int command, Stream *stream)
 		return FALSE;
 	}
 
-	stream->eom();
-	dprintf (D_FULLDEBUG, "renew: %d leases renewed\n", renewed_list.size() );
+	stream->end_of_message();
+	dprintf (D_FULLDEBUG,
+			 "renew: %d leases renewed\n", (int)renewed_list.size() );
 	LeaseManagerLease_FreeList( renewed_list );
 	return TRUE;
 }
@@ -469,11 +470,11 @@ LeaseManager::commandHandler_ReleaseLease(int command, Stream *stream)
 
 	// Free up the list
 	dprintf (D_FULLDEBUG,
-			 "release: %d leases released\n", release_list.size() );
+			 "release: %d leases released\n", (int)release_list.size() );
 	LeaseManagerLease_FreeList( release_list );
 
 	stream->put( OK );
-	stream->eom();
+	stream->end_of_message();
 	return TRUE;
 }
 
@@ -534,7 +535,7 @@ LeaseManager::timerHandler_GetAds ( void )
 int
 LeaseManager::initPublicAd( void )
 {
-	m_publicAd.clear();
+	m_publicAd.Clear();
 
 	m_publicAd.SetMyTypeName( LEASE_MANAGER_ADTYPE );
 	m_publicAd.SetTargetTypeName( "" );
@@ -550,7 +551,7 @@ LeaseManager::initPublicAd( void )
 			EXCEPT( "default_daemon_name() returned NULL" );
 		}
 		m_publicAd.Assign( ATTR_NAME, defaultName );
-		free( const_cast<char *>(defaultName) );
+		delete [] defaultName;
 	}
 
 	m_publicAd.Assign( ATTR_LEASE_MANAGER_IP_ADDR,
