@@ -43,10 +43,16 @@
 #include "store_cred.h"
 #include "setenv.h"
 #include "file_lock.h"
+#include "shared_port_server.h"
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 #include "MasterPlugin.h"
 #endif
+#if defined(WIN32)
+extern int load_master_mgmt(void);
+#endif
+#endif
 
 #if HAVE_EXT_GCB
 #include "GCB.h"
@@ -55,7 +61,7 @@ void gcbBrokerDownCallback();
 
 #ifdef WIN32
 
-#include "windows_firewall.h"
+#include "firewall.WINDOWS.h"
 
 extern void register_service();
 extern void terminate(DWORD);
@@ -72,7 +78,7 @@ extern "C"
 
 // local function prototypes
 void	init_params();
-int		init_daemon_list();
+void	init_daemon_list();
 void	init_classad();
 void	init_firewall_exceptions();
 void	check_uid_for_privsep();
@@ -80,26 +86,21 @@ void	lock_or_except(const char * );
 time_t 	GetTimeStamp(char* file);
 int 	NewExecutable(char* file, time_t* tsp);
 void	RestartMaster();
-int		run_preen(Service*);
+void	run_preen();
 void	usage(const char* );
-int		main_shutdown_graceful();
-int		main_shutdown_fast();
-int		main_config( bool is_full );
-int		agent_starter(ReliSock *);
-int		handle_agent_fetch_log(ReliSock *);
-int		admin_command_handler(Service *, int, Stream *);
-int		handle_subsys_command(int, Stream *);
+void	main_shutdown_graceful();
+void	main_shutdown_fast();
+void	invalidate_ads();
+void	main_config();
+int	agent_starter(ReliSock *);
+int	handle_agent_fetch_log(ReliSock *);
+int	admin_command_handler(Service *, int, Stream *);
+int	handle_subsys_command(int, Stream *);
 int     handle_shutdown_program( int cmd, Stream* stream );
 void	time_skip_handler(void * /*data*/, int delta);
 void	restart_everyone();
 
-extern "C" int	DoCleanup(int,int,char*);
-
-#if 0
-int		GetConfig(char*, char*);
-void	StartConfigServer();
-char*			configServer;
-#endif
+extern "C" int	DoCleanup(int,int,const char*);
 
 // Global variables
 ClassAd	*ad = NULL;				// ClassAd to send to collector
@@ -127,14 +128,12 @@ int		Lines;
 int		AllowAdminCommands = FALSE;
 int		StartDaemons = TRUE;
 int		GotDaemonsOff = FALSE;
+int		MasterShuttingDown = FALSE;
 
 char	*default_daemon_list[] = {
 	"MASTER",
 	"STARTD",
 	"SCHEDD",
-#if defined(OSF1) || defined(IRIX)    // Only need KBDD on alpha and sgi
-	"KBDD",
-#endif
 	0};
 
 // NOTE: When adding something here, also add it to the various condor_config
@@ -142,7 +141,7 @@ char	*default_daemon_list[] = {
 char	default_dc_daemon_list[] =
 "MASTER, STARTD, SCHEDD, KBDD, COLLECTOR, NEGOTIATOR, EVENTD, "
 "VIEW_SERVER, CONDOR_VIEW, VIEW_COLLECTOR, CREDD, HAD, "
-"REPLICATION, DBMSD, QUILL, JOB_ROUTER";
+"REPLICATION, DBMSD, QUILL, JOB_ROUTER, ROOSTER, SHARED_PORT";
 
 // create an object of class daemons.
 class Daemons daemons;
@@ -181,8 +180,10 @@ master_exit(int retval)
 	}
 #endif
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 	MasterPluginManager::Shutdown();
+#endif
 #endif
 
 	DC_Exit(retval, shutdown_program );
@@ -197,7 +198,7 @@ usage( const char* name )
 }
 
 int
-DoCleanup(int,int,char*)
+DoCleanup(int,int,const char*)
 {
 	static int already_excepted = FALSE;
 
@@ -226,7 +227,7 @@ DoCleanup(int,int,char*)
 }
 
 
-int
+void
 main_init( int argc, char* argv[] )
 {
     extern int runfor;
@@ -238,7 +239,7 @@ main_init( int argc, char* argv[] )
 	if ( argc == -1 ) {
 		NT_ServiceFlag = TRUE;
 		register_service();
-		return TRUE;
+		return;
 	}
 #endif
 
@@ -282,8 +283,9 @@ main_init( int argc, char* argv[] )
 		// Grab all parameters needed by the master.
 	init_params();
 		// param() for DAEMON_LIST and initialize our daemons object.
-	if ( init_daemon_list() < 0 ) {
-		EXCEPT( "Daemon list initialization failed" );
+	init_daemon_list();
+	if ( daemons.SetupControllers() < 0 ) {
+		EXCEPT( "Daemon initialization failed" );
 	}
 		// Lookup the paths to all the daemons we now care about.
 	daemons.InitParams();
@@ -296,16 +298,16 @@ main_init( int argc, char* argv[] )
 		// open up the windows firewall 
 	init_firewall_exceptions();
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN)
 	MasterPluginManager::Load();
-
+#elif defined(WIN32)
+	load_master_mgmt();
+#endif
 	MasterPluginManager::Initialize();
 #endif
 
 		// Register admin commands
-	daemonCore->Register_Command( RECONFIG, "RECONFIG",
-								  (CommandHandler)admin_command_handler, 
-								  "admin_command_handler", 0, ADMINISTRATOR );
 	daemonCore->Register_Command( RESTART, "RESTART",
 								  (CommandHandler)admin_command_handler, 
 								  "admin_command_handler", 0, ADMINISTRATOR );
@@ -381,7 +383,6 @@ main_init( int argc, char* argv[] )
 		daemons.StartAllDaemons();
 	}
 	daemons.StartTimers();
-	return TRUE;
 }
 
 
@@ -397,9 +398,6 @@ admin_command_handler( Service*, int cmd, Stream* stream )
 	dprintf( D_FULLDEBUG, 
 			 "Got admin command (%d) and allowing it.\n", cmd );
 	switch( cmd ) {
-	case RECONFIG:
-		daemonCore->Send_Signal( daemonCore->getpid(), SIGHUP );
-		return TRUE;
 	case RESTART:
 		restart_everyone();
 		return TRUE;
@@ -474,7 +472,7 @@ agent_starter( ReliSock * s )
 
 	dprintf ( D_ALWAYS, "Starting agent '%s'\n", subsys );
 
-	if( stricmp(subsys, "fetch_log") == 0 ) {
+	if( strcasecmp(subsys, "fetch_log") == 0 ) {
 		free (subsys);
 		return handle_agent_fetch_log( stream );
 	}
@@ -608,7 +606,7 @@ handle_shutdown_program( int cmd, Stream* stream )
 	MyString	pname;
 	pname =  "master_shutdown_";
 	pname += name;
-	char	*path = param( pname.GetCStr() );
+	char	*path = param( pname.Value() );
 	if ( NULL == path ) {
 		dprintf( D_ALWAYS, "No shutdown program defined for '%s'\n", name );
 		return FALSE;
@@ -631,7 +629,7 @@ handle_shutdown_program( int cmd, Stream* stream )
 		free( shutdown_program );
 	}
 	shutdown_program = path;
-	dprintf( D_FULLDEBUG,
+	dprintf( D_ALWAYS,
 			 "Shutdown program path set to %s\n", shutdown_program );
 	return TRUE;
 }
@@ -746,29 +744,58 @@ init_params()
 }
 
 
-int
+void
 init_daemon_list()
 {
 	char	*daemon_name;
 	class daemon	*new_daemon;
 	StringList daemon_names, dc_daemon_names;
 
+	daemons.ordered_daemon_names.clearAll();
 	char* dc_daemon_list = param("DC_DAEMON_LIST");
-	if( dc_daemon_list ) {
+
+	if( !dc_daemon_list ) {
+		dc_daemon_names.initializeFromString(default_dc_daemon_list);
+	}
+	else {
 		if ( *dc_daemon_list == '+' ) {
 			MyString	dclist;
 			dclist = default_dc_daemon_list;
 			dclist += ", ";
 			dclist += &dc_daemon_list[1];
-			dc_daemon_names.initializeFromString( dclist.GetCStr() );
+			dc_daemon_names.initializeFromString( dclist.Value() );
 		}
 		else {
 			dc_daemon_names.initializeFromString(dc_daemon_list);
+
+			StringList default_list(default_dc_daemon_list);
+			default_list.rewind();
+			char *default_entry;
+			int	missing = 0;
+			while( (default_entry=default_list.next()) ) {
+				if( !dc_daemon_names.contains_anycase(default_entry) ) {
+					dprintf(D_ALWAYS,
+							"WARNING: expected to find %s in"
+							" DC_DAEMON_LIST, but it is not there.\n",
+							default_entry );
+					missing++;
+				}
+			}
+			if ( missing ) {
+				dprintf( D_ALWAYS,
+						 "WARNING: "
+						 "%d entries are missing from DC_DAEMON_LIST.  "
+						 "Unless you know what you are doing, it "
+						 "is best to leave DC_DAEMON_LIST undefined "
+						 "so that the default settings are used, "
+						 "or use the new 'DC_DAEMON_LIST = "
+						 "+<list>' syntax.\n", missing );
+			}
+				
 		}
 		free(dc_daemon_list);
-	} else {
-		dc_daemon_names.initializeFromString(default_dc_daemon_list);
 	}
+
 		// Tolerate a trailing comma in the list
 	dc_daemon_names.remove( "" );
 
@@ -776,16 +803,16 @@ init_daemon_list()
 	if( ha_list ) {
 			// Make MASTER_HA_LIST case insensitive by always converting
 			// what we get to uppercase.
-		StringList	ha_names;
+		StringList ha_names;
 		ha_list = strupr( ha_list );
 		ha_names.initializeFromString(ha_list);
-		free( ha_list );
 			// Tolerate a trailing comma in the list
 		ha_names.remove( "" );
+		daemons.ordered_daemon_names.create_union( ha_names, false );
 
 		ha_names.rewind();
 		while( (daemon_name = ha_names.next()) ) {
-			if(daemons.GetIndex(daemon_name) < 0) {
+			if(daemons.FindDaemon(daemon_name) == NULL) {
 				if( dc_daemon_names.contains(daemon_name) ) {
 					new_daemon = new class daemon(daemon_name, true, true );
 				} else {
@@ -827,9 +854,18 @@ init_daemon_list()
 			daemon_names.insert( "COLLECTOR" );
 		}
 
+			// start shared_port first for a cleaner startup
+		if( daemon_names.contains("SHARED_PORT") ) {
+			daemon_names.deleteCurrent();
+			daemon_names.rewind();
+			daemon_names.next();
+			daemon_names.insert( "SHARED_PORT" );
+		}
+		daemons.ordered_daemon_names.create_union( daemon_names, false );
+
 		daemon_names.rewind();
 		while( (daemon_name = daemon_names.next()) ) {
-			if(daemons.GetIndex(daemon_name) < 0) {
+			if(daemons.FindDaemon(daemon_name) == NULL) {
 				if( dc_daemon_names.contains(daemon_name) ) {
 					new_daemon = new class daemon(daemon_name);
 				} else {
@@ -838,43 +874,12 @@ init_daemon_list()
 			}
 		}
 	} else {
+		daemons.ordered_daemon_names.create_union( dc_daemon_names, false );
 		for(int i = 0; default_daemon_list[i]; i++) {
 			new_daemon = new class daemon(default_daemon_list[i]);
 		}
 	}
 
-	return daemons.SetupControllers( );
-}
-
-
-void
-check_daemon_list()
-{
-	char	*daemon_name;
-	class daemon	*new_daemon;
-	StringList daemon_names;
-	char* daemon_list = param("DAEMON_LIST");
-	if( !daemon_list ) {
-			// Without a daemon list, there's no way it could be
-			// different than what we've got now.
-		return;
-	}
-
-		// Make DAEMON_LIST case insensitive by always converting what
-		// we get to uppercase.
-	daemon_list = strupr( daemon_list );
-
-	daemon_names.initializeFromString(daemon_list);
-	free( daemon_list );
-		// Tolerate a trailing comma in the list
-	daemon_names.remove( "" );
-
-	daemon_names.rewind();
-	while( (daemon_name = daemon_names.next()) ) {
-		if(daemons.GetIndex(daemon_name) < 0) {
-			new_daemon = new class daemon(daemon_name);
-		}
-	}
 }
 
 
@@ -898,15 +903,9 @@ init_classad()
 		delete [] default_name;
 	}
 
-		// CRUFT
-	ad->Assign(ATTR_MASTER_IP_ADDR, daemonCore->InfoCommandSinfulString());
-
 #if !defined(WIN32)
 	ad->Assign(ATTR_REAL_UID, (int)getuid());
 #endif
-
-		// Initialize all the DaemonCore-provided attributes
-	daemonCore->publish( ad ); 	
 }
 
 #ifndef WIN32
@@ -940,28 +939,77 @@ lock_or_except( const char* file_name )
  ** Re read the config file, and send all the daemons a signal telling
  ** them to do so also.
  */
-int
-main_config( bool /* is_full */ )
+void
+main_config()
 {
+	StringList old_daemon_list;
+	char *list = daemons.ordered_daemon_names.print_to_string();
+	char *daemon_name;
+	class daemon	*new_daemon;
+	int new_daemons = 0;
+
+	if( list ) {
+		old_daemon_list.initializeFromString(list);
+		free(list);
+	}
+
 		// Re-read the config files and create a new classad
 	init_classad(); 
 
 		// Reset our config values
 	init_params();
 
+		// Reset the daemon list
+	init_daemon_list();
+
+		// Setup Controllers for new daemons
+	daemons.ordered_daemon_names.rewind();
+	while( ( daemon_name = daemons.ordered_daemon_names.next() ) ) {
+		if( !old_daemon_list.contains(daemon_name) ) {
+			new_daemon = daemons.FindDaemon(daemon_name);
+			if ( new_daemon == NULL ) {
+				dprintf( D_ALWAYS, "Setup for daemon failed\n");
+			}
+			else if ( new_daemon->SetupController() < 0 ) {
+				dprintf( D_ALWAYS,
+						"Setup for daemon %s failed\n",
+						new_daemon->daemon_name );
+			}
+			else {
+				++new_daemons;
+			}
+		}
+	}
+
+	// Remove daemons that should no longer be running
+	old_daemon_list.rewind();
+	while( (daemon_name = old_daemon_list.next()) ) {
+		if( !daemons.ordered_daemon_names.contains(daemon_name) ) {
+			if( NULL != daemons.FindDaemon(daemon_name) ) {
+				daemons.StopDaemon(daemon_name);
+			}
+		}
+	}
+
 		// Re-read the paths to our executables.  If any paths
 		// changed, the daemons will be marked as having a new
 		// executable.
 	daemons.InitParams();
 
 	if( StartDaemons ) {
-			// Restart any daemons who's executables are new or ones 
+			// Restart any daemons who's executables are new or ones
 			// that the path to the executable has changed.  
 		daemons.immediate_restart = TRUE;
 		daemons.CheckForNewExecutable();
 		daemons.immediate_restart = FALSE;
 			// Tell all the daemons that are running to reconfig.
 		daemons.ReconfigAllDaemons();
+
+		if( new_daemons > 0 ) {
+				// Start new daemons
+			daemons.StartAllDaemons();
+		}
+	
 	} else {
 		daemons.DaemonsOff();
 	}
@@ -971,15 +1019,17 @@ main_config( bool /* is_full */ )
 		// changed.
 	daemons.StartTimers();
 	daemons.UpdateCollector();
-	return TRUE;
 }
 
 /*
  ** Kill all daemons and go away.
  */
-int
+void
 main_shutdown_fast()
 {
+	invalidate_ads();
+	
+	MasterShuttingDown = TRUE;
 	daemons.SetAllGoneAction( MASTER_EXIT );
 
 	if( daemons.NumberOfChildren() == 0 ) {
@@ -988,16 +1038,18 @@ main_shutdown_fast()
 
 	daemons.CancelRestartTimers();
 	daemons.StopFastAllDaemons();
-	return TRUE;
 }
 
 
 /*
  ** Cause job(s) to vacate, kill all daemons and go away.
  */
-int
+void
 main_shutdown_graceful()
 {
+	invalidate_ads();
+	
+	MasterShuttingDown = TRUE;
 	daemons.SetAllGoneAction( MASTER_EXIT );
 
 	if( daemons.NumberOfChildren() == 0 ) {
@@ -1006,7 +1058,28 @@ main_shutdown_graceful()
 
 	daemons.CancelRestartTimers();
 	daemons.StopAllDaemons();
-	return TRUE;
+}
+
+void
+invalidate_ads() {
+	ClassAd cmd_ad;
+	cmd_ad.SetMyTypeName( QUERY_ADTYPE );
+	cmd_ad.SetTargetTypeName( MASTER_ADTYPE );
+	
+	MyString line;
+	MyString escaped_name;
+	char* default_name = ::strnewp(MasterName);
+	if(!default_name) {
+		default_name = default_daemon_name();
+	}
+	
+	ClassAd::EscapeStringValue( default_name, escaped_name );
+	line.sprintf( "( TARGET.%s == \"%s\" )", ATTR_NAME, escaped_name.Value() );
+	cmd_ad.AssignExpr( ATTR_REQUIREMENTS, line.Value() );
+	cmd_ad.Assign( ATTR_NAME, default_name );
+	cmd_ad.Assign( ATTR_MY_ADDRESS, daemonCore->publicNetworkIpAddr());
+	daemonCore->sendUpdates( INVALIDATE_MASTER_ADS, &cmd_ad, NULL, false );
+	delete [] default_name;
 }
 
 time_t
@@ -1043,8 +1116,8 @@ NewExecutable(char* file, time_t *tsp)
 	return( cts != *tsp );
 }
 
-int
-run_preen(Service*)
+void
+run_preen()
 {
 	int		child_pid;
 	char *args=NULL;
@@ -1055,7 +1128,7 @@ run_preen(Service*)
 	dprintf(D_FULLDEBUG, "Entered run_preen.\n");
 
 	if( FS_Preen == NULL ) {
-		return 0;
+		return;
 	}
 	preen_base = condor_basename( FS_Preen );
 	arglist.AppendArg(preen_base);
@@ -1073,8 +1146,6 @@ run_preen(Service*)
 					1,				// which reaper ID to use; use default reaper
 					FALSE );		// we do _not_ want this process to have a command port; PREEN is not a daemon core process
 	dprintf( D_ALWAYS, "Preen pid is %d\n", child_pid );
-
-	return child_pid;
 }
 
 
@@ -1085,44 +1156,9 @@ RestartMaster()
 }
 
 
-#if 0
-void StartConfigServer()
-{
-	daemon*			newDaemon;
-	
-	newDaemon = new daemon("CONFIG_SERVER");
-	newDaemon->process_name = param(newDaemon->name_in_config_file);
-	if(newDaemon->process_name == NULL && newDaemon->runs_here)
-	{
-		dprintf(D_ALWAYS, "Process not found in config file: %s\n",
-				newDaemon->name_in_config_file);
-		EXCEPT("Can't continue...");
-	}
-	newDaemon->config_info_file = param("CONFIG_SERVER_FILE");
-	newDaemon->port = param("CONFIG_SERVER_PORT");
-
-	// check that log file is necessary
-	if(newDaemon->log_filename_in_config_file != NULL)
-	{
-		newDaemon->log_name = param(newDaemon->log_filename_in_config_file);
-		if(newDaemon->log_name == NULL && newDaemon->runs_here)
-		{
-			dprintf(D_ALWAYS, "Log file not found in config file: %s\n",
-					newDaemon->log_filename_in_config_file);
-		}
-	}
-
-	newDaemon->StartDaemon();
-	
-	// sleep for a while because we want the config server to stable down
-	// before doing anything else
-	sleep(5); 
-}
-#endif
-
 #if HAVE_EXT_GCB
 void
-gcb_broker_down_handler( Service * )
+gcb_broker_down_handler()
 {
 	int num_slots;
 	const char *our_broker = GetEnv( "GCB_INAGENT" );
@@ -1181,12 +1217,12 @@ gcbBrokerDownCallback()
 		// DaemonCore is blocked on a select() or CEDAR is blocked on a
 		// network operation. So we register a daemoncore timer to do
 		// the real work.
-	daemonCore->Register_Timer( 0, (TimerHandler)gcb_broker_down_handler,
+	daemonCore->Register_Timer( 0, gcb_broker_down_handler,
 								"gcb_broker_down_handler" );
 }
 
 void
-gcb_recovery_failed_handler( Service * )
+gcb_recovery_failed_handler()
 {
 	dprintf(D_ALWAYS, "GCB failed to recover from a failure with the "
 			"Broker. Restarting all daemons\n");
@@ -1200,7 +1236,7 @@ gcbRecoveryFailedCallback()
 		// DaemonCore is blocked on a select() or CEDAR is blocked on a
 		// network operation. So we register a daemoncore timer to do
 		// the real work.
-	daemonCore->Register_Timer( 0, (TimerHandler)gcb_recovery_failed_handler,
+	daemonCore->Register_Timer( 0, gcb_recovery_failed_handler,
 								"gcb_recovery_failed_handler" );
 }
 #endif
@@ -1287,6 +1323,17 @@ main_pre_command_sock_init()
 		}
 	}
 #endif
+
+	MyString daemon_list;
+	if( param(daemon_list,"DAEMON_LIST") ) {
+		StringList sl(daemon_list.Value());
+		if( sl.contains("SHARED_PORT") ) {
+				// in case a shared port address file got left behind by an
+				// unclean shutdown, clean it up now before we create our
+				// command socket to avoid confusion
+			SharedPortServer::RemoveDeadAddressFile();
+		}
+	}
 }
 
 
@@ -1299,7 +1346,7 @@ void init_firewall_exceptions() {
 		 *negotiator_image_path, *collector_image_path, *starter_image_path,
 		 *shadow_image_path, *gridmanager_image_path, *gahp_image_path,
 		 *gahp_worker_image_path, *credd_image_path, 
-		 *vmgahp_image_path, *bin_path;
+		 *vmgahp_image_path, *kbdd_image_path, *hdfs_image_path, *bin_path;
 	const char* dagman_exe = "condor_dagman.exe";
 
 	WindowsFirewallHelper wfh;
@@ -1347,6 +1394,8 @@ void init_firewall_exceptions() {
 	gahp_image_path = param("CONDOR_GAHP");
 	gahp_worker_image_path = param("CONDOR_GAHP_WORKER");
 	credd_image_path = param("CREDD");
+	kbdd_image_path = param("KBDD");
+	hdfs_image_path = param("HDFS");
 	vmgahp_image_path = param("VM_GAHP_SERVER");
 	
 	// We also want to add exceptions for the DAGMan we ship
@@ -1370,14 +1419,14 @@ void init_firewall_exceptions() {
 	}
 
 	// Insert daemons needed on a central manager
-	if ( (! (daemons.GetIndex("NEGOTIATOR") < 0) ) && negotiator_image_path ) {
+	if ( (daemons.FindDaemon("NEGOTIATOR") != NULL) && negotiator_image_path ) {
 		if ( !wfh.addTrusted(negotiator_image_path) ) {
 			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
 				"windows firewall exception list.\n",
 				negotiator_image_path);
 		}
 	}
-	if ( (! (daemons.GetIndex("COLLECTOR") < 0) ) && collector_image_path ) {
+	if ( (daemons.FindDaemon("COLLECTOR") != NULL) && collector_image_path ) {
 		if ( !wfh.addTrusted(collector_image_path) ) {
 			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
 				"windows firewall exception list.\n",
@@ -1386,7 +1435,7 @@ void init_firewall_exceptions() {
 	}
 
 	// Insert daemons needed on a submit node
-	if ( (! (daemons.GetIndex("SCHEDD") < 0) ) && schedd_image_path ) {
+	if ( (daemons.FindDaemon("SCHEDD") != NULL) && schedd_image_path ) {
 		// put in schedd
 		if ( !wfh.addTrusted(schedd_image_path) ) {
 			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
@@ -1423,15 +1472,20 @@ void init_firewall_exceptions() {
 	// Note we include the starter and friends seperately, since the
 	// starter could run on either execute or submit nodes (think 
 	// local universe jobs).
-	if ( (! (daemons.GetIndex("STARTD") < 0) ) && startd_image_path ) {
+	if ( (daemons.FindDaemon("STARTD") != NULL) && startd_image_path ) {
 		if ( !wfh.addTrusted(startd_image_path) ) {
 			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
 				"windows firewall exception list.\n",
 				startd_image_path);
 		}
+		if ( !wfh.addTrusted(kbdd_image_path) ) {
+			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
+				"windows firewall exception list.\n",
+				kbdd_image_path);
+		}
 	}
 
-	if ( (! (daemons.GetIndex("QUILL") < 0) ) && quill_image_path ) {
+	if ( (daemons.FindDaemon("QUILL") != NULL) && quill_image_path ) {
 		if ( !wfh.addTrusted(quill_image_path) ) {
 			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
 				"windows firewall exception list.\n",
@@ -1439,7 +1493,7 @@ void init_firewall_exceptions() {
 		}
 	}
 
-	if ( (! (daemons.GetIndex("DBMSD") < 0) ) && dbmsd_image_path ) {
+	if ( (daemons.FindDaemon("DBMSD") != NULL) && dbmsd_image_path ) {
 		if ( !wfh.addTrusted(dbmsd_image_path) ) {
 			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
 				"windows firewall exception list.\n",
@@ -1455,7 +1509,7 @@ void init_firewall_exceptions() {
 		}
 	}
 
-	if ( (! (daemons.GetIndex("CREDD") < 0) ) && credd_image_path ) {
+	if ( (daemons.FindDaemon("CREDD") != NULL) && credd_image_path ) {
 		if ( !wfh.addTrusted(credd_image_path) ) {
 			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
 				"windows firewall exception list.\n",
@@ -1463,6 +1517,13 @@ void init_firewall_exceptions() {
 		}
 	}
 
+	if ( (daemons.FindDaemon("HDFS") != NULL) && hdfs_image_path ) {
+		if ( !wfh.addTrusted(hdfs_image_path) ) {
+			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
+				"windows firewall exception list.\n", hdfs_image_path);
+		}
+	}
+
 	if ( vmgahp_image_path ) {
 		if ( !wfh.addTrusted(vmgahp_image_path) ) {
 			dprintf(D_FULLDEBUG, "WinFirewall: unable to add %s to the "
@@ -1491,7 +1552,8 @@ void init_firewall_exceptions() {
 	if ( gridmanager_image_path ) { free(gridmanager_image_path); }
 	if ( gahp_image_path ) { free(gahp_image_path); }	
 	if ( credd_image_path ) { free(credd_image_path); }	
-	if ( vmgahp_image_path ) { free(vmgahp_image_path); }		
+	if ( vmgahp_image_path ) { free(vmgahp_image_path); }
+	if ( kbdd_image_path ) { free(kbdd_image_path); }
 #endif
 }
 
