@@ -37,6 +37,7 @@ class daemon : public Service
 {
 public:
 	daemon(char *name, bool is_daemon_core = true, bool is_ha = false );
+	~daemon();
 	daemon_t type;
 	char*	name_in_config_file;
 	char*	daemon_name; 
@@ -68,7 +69,7 @@ public:
 	bool	OnHold( void ) { return on_hold; };
 	void	Stop( bool never_forward = false );
 	void	StopFast( bool never_forward = false );
-	int		StopFastTimer();
+	void	StopFastTimer();
 	void	StopPeaceful();
 	void	HardKill();
 	void	Exited( int );
@@ -78,10 +79,19 @@ public:
 	void	Kill( int );
 	void	KillFamily( void );
 	void	Reconfig();
+	void	InitParams();
 
 	int		SetupController( void );
 	int		RegisterControllee( class daemon * );
 
+	bool	IsHA( void ) { return is_ha; };
+
+	bool WaitBeforeStartingOtherDaemons(bool first_time);
+
+		// true if this daemon needs to run right up until just before
+		// the master shuts down (e.g. shared port server)
+	bool OnlyStopWhenMasterStops() { return m_only_stop_when_master_stops; }
+
 private:
 
 	int		runs_on_this_host();
@@ -91,6 +101,7 @@ private:
 	int		SetupHighAvailability( void );
 	int		HaLockAcquired( LockEventSrc src );
 	int		HaLockLost( LockEventSrc src );
+	void	DoActionAfterStartup();
 
 	int		start_tid;
 	int		recover_tid;
@@ -118,6 +129,12 @@ private:
 
 	int		num_controllees;
 	class daemon  *controllees[MAX_CONTROLLEES];
+
+	MyString m_after_startup_wait_for_file;
+	bool m_reload_shared_port_addr_after_startup;
+	bool m_never_use_shared_port;
+	bool m_waiting_for_startup;
+	bool m_only_stop_when_master_stops;
 };
 
 
@@ -138,7 +155,6 @@ public:
 	Daemons();
 	void	RegisterDaemon(class daemon *);
 	void 	InitParams();
-	int		GetIndex(const char* process_name);
 
 	void	CheckForNewExecutable();
 	void	DaemonsOn();
@@ -147,6 +163,7 @@ public:
 	void 	StartAllDaemons();
 	void	StopAllDaemons();
 	void	StopFastAllDaemons();
+	void	StopDaemon( char* name );
 	void	HardKillAllDaemons();
 	void	StopPeacefulAllDaemons();
 	void	ReconfigAllDaemons();
@@ -183,6 +200,8 @@ public:
 	int		immediate_restart;
 	int		immediate_restart_master;
 
+	StringList	ordered_daemon_names;
+
 	void	Update( ClassAd* );
 	void	UpdateCollector();
 
@@ -190,16 +209,23 @@ public:
 	class daemon*	FindDaemon( const char * );
 
 private:
-	class daemon **daemon_ptr;
-	int	no_daemons;
-	int daemon_list_size;
+	std::map<std::string, class daemon*> daemon_ptr;
+	std::map<int, class daemon*> exit_allowed;
 	int check_new_exec_tid;
 	int update_tid;
 	int preen_tid;
-	int master;  		// index of the master in our daemon table
+	class daemon* master;  	// the master in our daemon table
 	AllGoneT all_daemons_gone_action;
 	ReaperT reaper;
 	int prevLHF;
+	int m_retry_start_all_daemons_tid;
+
+	void ScheduleRetryStartAllDaemons();
+	void CancelRetryStartAllDaemons();
+	void RetryStartAllDaemons();
+
+		// returns true if there are no remaining daemons
+	bool StopDaemonsBeforeMasterStops();
 };
 
 #endif /* _CONDOR_MASTER_H */
