@@ -37,10 +37,15 @@
 #include "condor_netdb.h"
 #include "file_sql.h"
 #include "file_lock.h"
+#include "stat_info.h"
+#include "shared_port_endpoint.h"
+#include "condor_fix_access.h"
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 #include "MasterPlugin.h"
 #endif
+#endif
 
 // these are defined in master.C
 extern int 		MasterLockFD;
@@ -57,7 +62,7 @@ extern int		NT_ServiceFlag; // TRUE if running on NT as an NT Service
 extern time_t	GetTimeStamp(char* file);
 extern int 	   	NewExecutable(char* file, time_t* tsp);
 extern void		tail_log( FILE*, char*, int );
-extern int		run_preen(Service*);
+extern void		run_preen();
 
 extern FILESQL *FILEObj;
 extern int condor_main_argc;
@@ -91,6 +96,7 @@ extern int			Lines;
 extern int			PublishObituaries;
 extern int			StartDaemons;
 extern int			GotDaemonsOff;
+extern int			MasterShuttingDown;
 extern char*		MasterName;
 
 ///////////////////////////////////////////////////////////////////////////
@@ -126,6 +132,11 @@ daemon::daemon(char *name, bool is_daemon_core, bool is_h )
 	// Default to not on hold (will be set to true if controlled (i.e by HAD))
 	on_hold = FALSE;
 
+	m_waiting_for_startup = false;
+	m_reload_shared_port_addr_after_startup = false;
+	m_never_use_shared_port = false;
+	m_only_stop_when_master_stops = false;
+
 	// Handle configuration
 	DoConfig( true );
 
@@ -134,8 +145,9 @@ daemon::daemon(char *name, bool is_daemon_core, bool is_h )
 	log_filename_in_config_file = strdup(buf);
 
 	// Check the process name (is it me?)
+	flag_in_config_file = NULL;
 	process_name = NULL;
-    watch_name = NULL;
+	watch_name = NULL;
 	log_name = NULL;
 	if( strcmp(name, "MASTER") == MATCH ) {
 		runs_here = FALSE;
@@ -166,6 +178,43 @@ daemon::daemon(char *name, bool is_daemon_core, bool is_h )
 	daemons.RegisterDaemon(this);
 }
 
+daemon::~daemon()
+{
+	if( name_in_config_file != NULL ) {
+		free( name_in_config_file );
+	}
+	if( daemon_name != NULL ) {
+		free( daemon_name );
+	}
+	if( log_filename_in_config_file != NULL ) {
+		free( log_filename_in_config_file );
+	}
+	if( flag_in_config_file != NULL ) {
+		free( flag_in_config_file );
+	}
+	if( process_name != NULL ) {
+		free( process_name );
+	}
+	if( watch_name != NULL ) {
+		free( watch_name );
+	}
+	if( log_name != NULL ) {
+		free( log_name );
+	}
+	if( ha_lock != NULL ) {
+		delete( ha_lock );
+	}
+	if( controller_name != NULL ) {
+		free( controller_name );
+	}
+	if( controller != NULL ) {
+		delete( controller );
+	}
+	for( int num = 0;  num < num_controllees;  num++ ) {
+		delete( controllees[num] );
+	}
+}
+
 int
 daemon::runs_on_this_host()
 {
@@ -350,7 +399,7 @@ daemon::DoConfig( bool init )
 	// Previously defind?
 	if ( NULL != flag_in_config_file ) {
 		// Has it changed?
-		if ( strcmp( flag_in_config_file, tmp ) ) {
+		if (tmp && strcmp( flag_in_config_file, tmp ) ) {
 			free( flag_in_config_file );
 			flag_in_config_file = tmp;
 		} else if ( NULL != tmp ) {
@@ -401,6 +450,8 @@ daemon::DoConfig( bool init )
 		}
 	}
 
+	// XXX These defaults look to be very wrong, compare with 
+	// MASTER_BACKOFF_*.
 	sprintf(buf, "MASTER_%s_BACKOFF_CONSTANT", name_in_config_file );
 	m_backoff_constant = param_integer( buf, master_backoff_constant, 1 );
 
@@ -432,6 +483,38 @@ daemon::DoConfig( bool init )
 		*(daemon_name - 2) = '_';
 	} 
 
+	if( strcmp(name_in_config_file,"SHARED_PORT") == 0 ) {
+			// It doesn't make sense for the shared port server to
+			// itself try to exist behind a shared port.  It needs
+			// its own port.
+		m_never_use_shared_port = true;
+
+			// It is not essential to wait for the shared port server
+			// to be ready for other daemons to start, but it makes
+			// for a cleaner start, avoiding a minute or so during
+			// which some daemons advertise themselves without an
+			// address.
+
+		ASSERT( param(m_after_startup_wait_for_file,"SHARED_PORT_DAEMON_AD_FILE") );
+		m_reload_shared_port_addr_after_startup = true;
+
+		if( SharedPortEndpoint::UseSharedPort() ) {
+				// If the master is using the shared port server, stopping
+				// the shared port server would make the master inaccessible,
+				// so never stop it unless the master itself is stopping.
+				// Also, even if the master is shutting down, do not stop
+				// shared port server until all other daemons have exited,
+				// because they may be depending on it.
+
+			m_only_stop_when_master_stops = true;
+		}
+	}
+
+	if( strcmp(name_in_config_file,"COLLECTOR") == 0 ) {
+			// if a collector address file is configured, don't start any
+			// other daemons until the collector has written this file
+		param(m_after_startup_wait_for_file,"COLLECTOR_ADDRESS_FILE");
+	}
 }
 
 void
@@ -497,6 +580,8 @@ int daemon::RealStart( )
 {
 	const char	*shortname;
 	int 	command_port = isDC ? TRUE : FALSE;
+	char const *daemon_sock = NULL;
+	MyString daemon_sock_buf;
 	char	buf[512];
 	ArgList args;
 
@@ -509,6 +594,12 @@ int daemon::RealStart( )
 			// Already running
 		return TRUE;
 	}
+	if( MasterShuttingDown ) {
+		dprintf( D_ALWAYS,
+				 "Master is shutting down, so skipping startup of %s\n",
+				 name_in_config_file );
+		return FALSE;
+	}
 
 	shortname = condor_basename( process_name );
 
@@ -520,6 +611,10 @@ int daemon::RealStart( )
 		return 0;
 	}
 
+	if( !m_after_startup_wait_for_file.IsEmpty() ) {
+		remove( m_after_startup_wait_for_file.Value() );
+	}
+
 		// Collector needs to listen on a well known port, as does the
 		// Negotiator.  
 		// We didn't want them to use root for any reason, but b/c of
@@ -527,18 +622,8 @@ int daemon::RealStart( )
 		// in the keytab file, we still need root afterall. :(
 	bool wants_condor_priv = false;
 	if ( strcmp(name_in_config_file,"COLLECTOR") == 0 ) {
-			// **** OLD
-			// If we're spawning a collector, we can get the right
-			// port by asking the global Collector object for it,
-			// since we've already instantiated that with the info for
-			// the local pool's collector.  This also saves the
-			// trouble of instantiating a new DCCollector object,
-			// which duplicates some effort and is less efficient. 
-			// **** END OLD
 
-			// ckireyev 09/10/04
-			// Now that we have multiple collectors, the way to figure out
-			// the port on this machine, is to go through all of the
+			// Go through all of the
 			// collectors until we find the one for THIS machine. Then
 			// get the port from that entry
 		command_port = -1;
@@ -562,8 +647,19 @@ int daemon::RealStart( )
 
 				if (same_host (my_hostname, 
 							   my_daemon->fullHostname())) {
-					command_port = my_daemon->port();
-					dprintf ( D_FULLDEBUG, "Host name matches." );
+					Sinful sinful( my_daemon->addr() );
+					if( sinful.getSharedPortID() ) {
+							// collector is using a shared port
+						daemon_sock_buf = sinful.getSharedPortID();
+						daemon_sock = daemon_sock_buf.Value();
+						command_port = 1;
+					}
+					else {
+							// collector is using its own port
+						command_port = sinful.getPortNum();
+					}
+					dprintf ( D_FULLDEBUG, "Host name matches collector %s.\n",
+							  sinful.getSinful() ? sinful.getSinful() : "NULL" );
 					break;
 				}
 			}
@@ -587,19 +683,25 @@ int daemon::RealStart( )
 			dprintf (D_ALWAYS, "Collector port not defined, will use default: %d\n", COLLECTOR_PORT);
 		}
 
-		dprintf (D_FULLDEBUG, "Starting Collector on port %d\n", command_port);
+		if( daemon_sock ) {
+			dprintf (D_FULLDEBUG,"Starting collector with shared port id %s\n",
+					 daemon_sock);
+		}
+		else {
+			dprintf (D_FULLDEBUG, "Starting Collector on port %d\n", command_port);
+		}
 
 
 			// We can't do this b/c of needing to read host certs as root 
 			// wants_condor_priv = true;
-	} else if( stricmp(name_in_config_file,"CONDOR_VIEW") == 0 ||
-			   stricmp(name_in_config_file,"VIEW_SERVER") == 0 ) {
+	} else if( strcasecmp(name_in_config_file,"CONDOR_VIEW") == 0 ||
+			   strcasecmp(name_in_config_file,"VIEW_SERVER") == 0 ) {
 		Daemon d( DT_VIEW_COLLECTOR );
 		command_port = d.port();
 			// We can't do this b/c of needing to read host certs as root 
 			// wants_condor_priv = true;
 	} 
-	else if( strcmp(name_in_config_file,"NEGOTIATOR") == 0 ) {
+	else if( strcasecmp(name_in_config_file,"NEGOTIATOR") == 0 ) {
 		char* host = getCmHostFromConfig( "NEGOTIATOR" );
 		if( host ) {
 			free (host);
@@ -685,6 +787,12 @@ int daemon::RealStart( )
 					command_port = atoi(port_arg);
 				}
 			}
+			else if(strncmp( cur_arg, "-sock", strlen(cur_arg)) == 0) {
+				i++;
+				if( i<args.Count() ) {
+					daemon_sock = args.GetArg(i);
+				}
+			}
 		}
     }
 
@@ -694,6 +802,11 @@ int daemon::RealStart( )
 	FamilyInfo fi;
 	fi.max_snapshot_interval = param_integer("PID_SNAPSHOT_INTERVAL", 60);
 
+	int jobopts = 0;
+	if( m_never_use_shared_port ) {
+		jobopts |= DCJOBOPT_NEVER_USE_SHARED_PORT;
+	}
+
 	pid = daemonCore->Create_Process(
 				process_name,	// program to exec
 				args,			// args
@@ -702,7 +815,16 @@ int daemon::RealStart( )
 				command_port,	// port to use for command port; TRUE=choose one dynamically
 				&env,			// environment
 				NULL,			// current working directory
-				&fi);			// we want a new process family
+				&fi,
+				NULL,
+				NULL,
+				NULL,
+				0,
+				NULL,
+				jobopts,
+				NULL,
+				NULL,
+				daemon_sock);
 
 	if ( pid == FALSE ) {
 		// Create_Process failed!
@@ -715,6 +837,8 @@ int daemon::RealStart( )
 		return 0;
 	}
 
+	m_waiting_for_startup = true;
+
 	// if this is a restart, start recover timer
 	if (restarts > 0) {
 		recover_tid = daemonCore->Register_Timer( m_recover_time,
@@ -723,14 +847,25 @@ int daemon::RealStart( )
 		dprintf(D_FULLDEBUG, "start recover timer (%d)\n", recover_tid);
 	}
 
-	if (command_port) {
+	const char	*proc_type = command_port ? "DaemonCore " : "";
+	if ( DebugFlags & D_FULLDEBUG ) {
+		MyString	 args_string, tmp;
+		args.GetArgsStringForDisplay( &tmp, 1 );
+		if( tmp.Length() ) {
+			args_string  = " ";
+			args_string += tmp;
+		}
+		else {
+			args_string = tmp;
+		}
 		dprintf( D_FAILURE|D_ALWAYS,
-				 "Started DaemonCore process \"%s\", pid and pgroup = %d\n",
-				 process_name, pid );
-	} else {
-		dprintf( D_ALWAYS,
-				 "Started process \"%s\", pid and pgroup = %d\n",
-				 process_name, pid );
+				 "Started %sprocess \"%s%s\", pid and pgroup = %d\n",
+				 proc_type, process_name, args_string.Value(), pid );
+	}
+	else {
+		dprintf( D_FAILURE|D_ALWAYS,
+				 "Started %sprocess \"%s\", pid and pgroup = %d\n",
+				 proc_type, process_name, pid );
 	}
 	
 		// Make sure we've got the current timestamp for updates, etc.
@@ -742,14 +877,6 @@ int daemon::RealStart( )
 		// Since we just started it, we know it's not a new executable. 
 	newExec = FALSE;
 
-		// If starting the collector, give it a few seconds to get
-		// going before starting other daemons or talking to ti
-		// ourselves
-	if ( strcmp(name_in_config_file,"COLLECTOR") == 0 ) {
-		dprintf(D_FULLDEBUG, "Pausing to allow the collector to start\n");
-		sleep(3);
-	}
-
 	if( needs_update ) {
 		needs_update = FALSE;
 		daemons.UpdateCollector();
@@ -758,6 +885,43 @@ int daemon::RealStart( )
 	return pid;	
 }
 
+bool
+daemon::WaitBeforeStartingOtherDaemons(bool first_time)
+{
+
+	if( !m_waiting_for_startup ) {
+		return false;
+	}
+
+	bool wait = false;
+	if( !m_after_startup_wait_for_file.IsEmpty() ) {
+		StatInfo si( m_after_startup_wait_for_file.Value() );
+		if( si.Error() != 0 ) {
+			wait = true;
+			dprintf(D_ALWAYS,"Waiting for %s to appear.\n",
+					m_after_startup_wait_for_file.Value() );
+		}
+		else if( !first_time ) {
+			dprintf(D_ALWAYS,"Found %s.\n",
+					m_after_startup_wait_for_file.Value() );
+		}
+	}
+
+	if( !wait && m_waiting_for_startup ) {
+		m_waiting_for_startup = false;
+		DoActionAfterStartup();
+	}
+
+	return wait;
+}
+
+void
+daemon::DoActionAfterStartup()
+{
+	if( m_reload_shared_port_addr_after_startup ) {
+		daemonCore->ReloadSharedPortServerAddr();
+	}
+}
 
 void
 daemon::Stop( bool never_forward )
@@ -841,11 +1005,10 @@ daemon::StopPeaceful()
 	Kill( SIGTERM );
 }
 
-int
+void
 daemon::StopFastTimer()
 {
 	StopFast(false);
-	return TRUE;
 }
 
 void
@@ -1206,6 +1369,71 @@ daemon::Reconfig()
 	Kill( SIGHUP );
 }
 
+
+void
+daemon::InitParams()
+{
+	char* buf;
+	char* tmp = NULL;
+
+	if( process_name ) {
+		tmp = process_name;
+	}
+	process_name = param(name_in_config_file);
+	if( !process_name ) {
+		dprintf( D_ALWAYS, 
+				"%s is in the DAEMON_LIST parameter, but there is no executable path for it defined in the config files!\n", 
+				name_in_config_file ); 
+		EXCEPT( "Must have the path to %s defined.", name_in_config_file ); 
+	}
+	if( tmp && strcmp(process_name, tmp) ) {
+			// The path to this daemon has changed in the config
+			// file, we will need to start the new version.
+		newExec = TRUE;
+	}
+	if (tmp) {
+		free( tmp );
+		tmp = NULL;
+	}
+	if( watch_name ) {
+		tmp = watch_name;
+	}
+			
+	int length = strlen(name_in_config_file) + 32;
+	buf = (char *)malloc(length);
+	snprintf( buf, length, "%s_WATCH_FILE", name_in_config_file );
+	watch_name = param( buf );
+	free(buf);
+	if( !watch_name) {
+		watch_name = strdup(process_name);
+	} 
+
+	if( tmp && strcmp(watch_name, tmp) ) {
+		// tmp is what the old watch_name was 
+		// The path to what we're watching has changed in the 
+		// config file, we will need to start the new version.
+		timeStamp = 0;
+	}
+
+	if (tmp) {
+		free( tmp );
+		tmp = NULL;
+	}
+
+		// check that log file is necessary
+	if ( log_filename_in_config_file != NULL) {
+		if( log_name ) {
+			free( log_name );
+		}
+		log_name = param(log_filename_in_config_file);
+		if ( log_name == NULL && runs_here ) {
+			dprintf(D_ALWAYS, "Log file not found in config file: %s\n", 
+					log_filename_in_config_file);
+		}
+	}
+}
+
+
 int
 daemon::SetupHighAvailability( void )
 {
@@ -1383,9 +1611,6 @@ daemon::RegisterControllee( class daemon *controllee )
 
 Daemons::Daemons()
 {
-	daemon_ptr = NULL;
-	no_daemons  =  0;
-	daemon_list_size = 0;
 	check_new_exec_tid = -1;
 	update_tid = -1;
 	preen_tid = -1;
@@ -1394,44 +1619,33 @@ Daemons::Daemons()
 	immediate_restart = FALSE;
 	immediate_restart_master = FALSE;
 	prevLHF = 0;
+	m_retry_start_all_daemons_tid = -1;
+	master = NULL;
 }
 
 
 void
 Daemons::RegisterDaemon(class daemon *d)
 {
-	int i;
-	if( !daemon_ptr ) {
-		daemon_list_size = 10;
-		daemon_ptr = (class daemon **) malloc(daemon_list_size * sizeof(class daemon *));
-		for( i=0; i<daemon_list_size; i++ ) {
-			daemon_ptr[i] = NULL;
-		}
-	}
-
-	if (no_daemons >= daemon_list_size) {
-		i = daemon_list_size;
-		daemon_list_size *= 2;
-		daemon_ptr = (class daemon **) realloc(daemon_ptr, 
-										daemon_list_size * sizeof(class daemon *));
-		for( ; i<daemon_list_size; i++ ) {
-			daemon_ptr[i] = NULL;
-		}
+	std::pair<std::map<std::string,class daemon*>::iterator,bool> ret;
 
+	ret = daemon_ptr.insert( std::pair<char*, class daemon*>(d->name_in_config_file, d) );
+	if( ret.second == false ) {
+		EXCEPT( "Registering daemon %s failed", d->name_in_config_file );
 	}
-	daemon_ptr[no_daemons] = d;
-	no_daemons++;
 }
 
 int
 Daemons::SetupControllers( void )
 {
 	// Find controlling daemons
-	for( int i=0; i < no_daemons; i++ ) {
-		if ( daemon_ptr[i]->SetupController( ) < 0 ) {
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if ( iter->second->SetupController( ) < 0 ) {
 			dprintf( D_ALWAYS,
 					 "SetupControllers: Setup for daemon %s failed\n",
-					 daemon_ptr[i]->daemon_name );
+					 iter->first.c_str() );
 			return -1;
 		}
 	}
@@ -1441,89 +1655,25 @@ Daemons::SetupControllers( void )
 void
 Daemons::InitParams()
 {
-	char* buf;
-	char* tmp = NULL;
-	for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->process_name ) {
-			tmp = daemon_ptr[i]->process_name;
-		}
-		daemon_ptr[i]->process_name = param(daemon_ptr[i]->name_in_config_file);
-		if( !daemon_ptr[i]->process_name ) {
-			dprintf( D_ALWAYS, 
-					 "%s from your DAEMON_LIST is not defined the config files!\n", 
-					 daemon_ptr[i]->name_in_config_file ); 
-			EXCEPT( "Must have the path to %s defined.", 
-					daemon_ptr[i]->name_in_config_file ); 
-		}
-		if( tmp && strcmp(daemon_ptr[i]->process_name, tmp) ) {
-				// The path to this daemon has changed in the config
-				// file, we will need to start the new version.
-			daemon_ptr[i]->newExec = TRUE;
-		}
-		if (tmp) {
-			free( tmp );
-			tmp = NULL;
-		}
-		if( daemon_ptr[i]->watch_name ) {
-			tmp = daemon_ptr[i]->watch_name;
-		}
-			
-		int length = strlen(daemon_ptr[i]->name_in_config_file) + 32;
-		buf = (char *)malloc(length);
-		snprintf( buf, length, "%s_WATCH_FILE", 
-				daemon_ptr[i]->name_in_config_file );
-		daemon_ptr[i]->watch_name = param( buf );
-		free(buf);
-		if( !daemon_ptr[i]->watch_name)	{
-			daemon_ptr[i]->watch_name = strdup(daemon_ptr[i]->process_name);
-		} 
+	std::map<std::string, class daemon*>::iterator iter;
 
-		if( tmp && strcmp(daemon_ptr[i]->watch_name, tmp) ) {
-			// tmp is what the old watch_name was 
-			// The path to what we're watching has changed in the 
-			// config file, we will need to start the new version.
-			daemon_ptr[i]->timeStamp = 0;
-		}
-
-		if (tmp) {
-			free( tmp );
-			tmp = NULL;
-		}
-
-			// check that log file is necessary
-		if ( daemon_ptr[i]->log_filename_in_config_file != NULL) {
-			if( daemon_ptr[i]->log_name ) {
-				free( daemon_ptr[i]->log_name );
-			}
-			daemon_ptr[i]->log_name = 
-					param(daemon_ptr[i]->log_filename_in_config_file);
-			if ( daemon_ptr[i]->log_name == NULL && daemon_ptr[i]->runs_here ) {
-				dprintf(D_ALWAYS, "Log file not found in config file: %s\n", 
-						daemon_ptr[i]->log_filename_in_config_file);
-			}
-		}
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		iter->second->InitParams();
 	}
 }
 
 
-int
-Daemons::GetIndex(const char* name)
-{
-	for ( int i=0; i < no_daemons; i++)
-		if ( strcmp(daemon_ptr[i]->name_in_config_file,name) == 0 )
-			return i;	
-	return -1;
-}
-
-
 class daemon *
 Daemons::FindDaemon( const char *name )
 {
-	int		index = GetIndex( name );
-	if ( index < 0 ) {
+	std::map<std::string, class daemon*>::iterator iter;
+
+	iter = daemon_ptr.find(name);
+	if( iter != daemon_ptr.end() ) {
+		return iter->second;
+	}
+	else {
 		return NULL;
-	} else {
-		return daemon_ptr[index];
 	}
 }
 
@@ -1532,34 +1682,36 @@ void
 Daemons::CheckForNewExecutable()
 {
 	int found_new = FALSE;
+	std::map<std::string, class daemon*>::iterator iter;
 
 	dprintf(D_FULLDEBUG, "enter Daemons::CheckForNewExecutable\n");
 
-	if( daemon_ptr[master]->newExec ) {
+	if( master->newExec ) {
 			// We already noticed the master has a new binary, so we
 			// already started to restart it.  There's nothing else to
 			// do here.
 		return;
 	}
-    if( NewExecutable( daemon_ptr[master]->watch_name,
-					   &daemon_ptr[master]->timeStamp ) ) {
+	if( NewExecutable( master->watch_name, &master->timeStamp ) ) {
 		dprintf( D_ALWAYS,"%s was modified, restarting %s.\n", 
-				 daemon_ptr[master]->watch_name, 
-				 daemon_ptr[master]->process_name );
-		daemon_ptr[master]->newExec = TRUE;
+				 master->watch_name, 
+				 master->process_name );
+		master->newExec = TRUE;
 			// Begin the master restart procedure.
 		
-        RestartMaster();
+		RestartMaster();
 		return;
-    }
+	}
 
-    for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->runs_here && !daemon_ptr[i]->newExec 
-			&& ! daemon_ptr[i]->OnHold() ) {
-			if( NewExecutable( daemon_ptr[i]->watch_name,
-							   &daemon_ptr[i]->timeStamp ) ) {
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->runs_here && !iter->second->newExec 
+			&& !iter->second->OnHold()
+			&& !iter->second->OnlyStopWhenMasterStops() )
+		{
+			if( NewExecutable( iter->second->watch_name,
+						&iter->second->timeStamp ) ) {
 				found_new = TRUE;
-				daemon_ptr[i]->newExec = TRUE;				
+				iter->second->newExec = TRUE;
 				if( immediate_restart ) {
 						// If we want to avoid the new_binary_delay,
 						// we can just set the newExec flag to false,
@@ -1567,22 +1719,22 @@ Daemons::CheckForNewExecutable()
 						// When it gets restarted, the new binary will
 						// be used, but we won't think it's a new
 						// binary, so we won't use the new_bin_delay.
-					daemon_ptr[i]->newExec = FALSE;
-					daemon_ptr[i]->restarts = 0;
+					iter->second->newExec = FALSE;
+					iter->second->restarts = 0;
 				}
-				if( daemon_ptr[i]->pid ) {
+				if( iter->second->pid ) {
 					dprintf( D_ALWAYS,"%s was modified, killing %s.\n", 
-							 daemon_ptr[i]->watch_name,
-							 daemon_ptr[i]->process_name );
-					daemon_ptr[i]->Stop();
+							 iter->second->watch_name,
+							 iter->second->process_name );
+					iter->second->Stop();
 				} else {
 					if( immediate_restart ) {
 							// This daemon isn't running now, but
 							// there's a new binary.  Cancel the
 							// current start timer and restart it
 							// now. 
-						daemon_ptr[i]->CancelAllTimers();
-						daemon_ptr[i]->Restart();
+						iter->second->CancelAllTimers();
+						iter->second->Restart();
 					}
 				}
 			}
@@ -1632,18 +1784,64 @@ Daemons::DaemonsOffPeaceful( )
 	StopPeacefulAllDaemons();
 }
 
+void
+Daemons::ScheduleRetryStartAllDaemons()
+{
+	if( m_retry_start_all_daemons_tid == -1 ) {
+		m_retry_start_all_daemons_tid = daemonCore->Register_Timer(
+			1,
+			(TimerHandlercpp)&Daemons::RetryStartAllDaemons,
+			"Daemons::RetryStartAllDaemons",
+			this);
+		ASSERT( m_retry_start_all_daemons_tid != -1 );
+	}
+}
+
+void
+Daemons::CancelRetryStartAllDaemons()
+{
+	if( m_retry_start_all_daemons_tid != -1 ) {
+		daemonCore->Cancel_Timer(m_retry_start_all_daemons_tid);
+		m_retry_start_all_daemons_tid = -1;
+	}
+}
+
+void
+Daemons::RetryStartAllDaemons()
+{
+	m_retry_start_all_daemons_tid = -1;
+	StartAllDaemons();
+}
 
 void
 Daemons::StartAllDaemons()
 {
-	for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->pid > 0 ) {
+	char *name;
+	class daemon *daemon;
+
+	ordered_daemon_names.rewind();
+	while( (name = ordered_daemon_names.next()) ) {
+		daemon = FindDaemon( name );
+		if( daemon == NULL ) {
+			EXCEPT("Unable to start daemon %s", name);
+		}
+		if( daemon->pid > 0 ) {
+			if( daemon->WaitBeforeStartingOtherDaemons(false) ) {
+				ScheduleRetryStartAllDaemons();
+				return;
+			}
+
 				// the daemon is already started
 			continue;
 		} 
-		if( ! daemon_ptr[i]->runs_here ) continue;
-		daemon_ptr[i]->Hold( FALSE );
-		daemon_ptr[i]->Start();
+		if( ! daemon->runs_here ) continue;
+		daemon->Hold( FALSE );
+		daemon->Start();
+
+		if( daemon->WaitBeforeStartingOtherDaemons(true) ) {
+			ScheduleRetryStartAllDaemons();
+			return;
+		}
 	}
 }
 
@@ -1651,11 +1849,16 @@ Daemons::StartAllDaemons()
 void
 Daemons::StopAllDaemons() 
 {
+	CancelRetryStartAllDaemons();
 	daemons.SetAllReaper();
 	int running = 0;
-	for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->pid && daemon_ptr[i]->runs_here ) {
-			daemon_ptr[i]->Stop();
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->pid && iter->second->runs_here &&
+			!iter->second->OnlyStopWhenMasterStops() )
+		{
+			iter->second->Stop();
 			running++;
 		}
 	}
@@ -1665,14 +1868,39 @@ Daemons::StopAllDaemons()
 }
 
 
+void
+Daemons::StopDaemon( char* name )
+{
+	std::map<std::string, class daemon*>::iterator iter;
+
+	iter = daemon_ptr.find( name );
+	if( iter != daemon_ptr.end() ) {
+		if( iter->second->pid > 0 ) {
+			exit_allowed.insert( std::pair<int, class daemon*>(iter->second->pid, iter->second) );
+			iter->second->Stop();
+		}
+		else {
+			iter->second->CancelAllTimers();
+			delete iter->second;
+		}
+		daemon_ptr.erase( iter );
+	}
+}
+
+
 void
 Daemons::StopFastAllDaemons()
 {
+	CancelRetryStartAllDaemons();
 	daemons.SetAllReaper();
 	int running = 0;
-	for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->pid && daemon_ptr[i]->runs_here ) {
-			daemon_ptr[i]->StopFast();
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->pid && iter->second->runs_here &&
+			!iter->second->OnlyStopWhenMasterStops() )
+		{
+			iter->second->StopFast();
 			running++;
 		}
 	}
@@ -1684,11 +1912,16 @@ Daemons::StopFastAllDaemons()
 void
 Daemons::StopPeacefulAllDaemons()
 {
+	CancelRetryStartAllDaemons();
 	daemons.SetAllReaper();
 	int running = 0;
-	for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->pid && daemon_ptr[i]->runs_here ) {
-			daemon_ptr[i]->StopPeaceful();
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->pid && iter->second->runs_here &&
+			!iter->second->OnlyStopWhenMasterStops() )
+		{
+			iter->second->StopPeaceful();
 			running++;
 		}
 	}
@@ -1701,11 +1934,16 @@ Daemons::StopPeacefulAllDaemons()
 void
 Daemons::HardKillAllDaemons()
 {
+	CancelRetryStartAllDaemons();
 	daemons.SetAllReaper();
 	int running = 0;
-	for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->pid && daemon_ptr[i]->runs_here ) {
-			daemon_ptr[i]->HardKill();
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->pid && iter->second->runs_here &&
+			!iter->second->OnlyStopWhenMasterStops() )
+		{
+			iter->second->HardKill();
 			running++;
 		}
 	}
@@ -1717,10 +1955,12 @@ Daemons::HardKillAllDaemons()
 void
 Daemons::ReconfigAllDaemons()
 {
+	std::map<std::string, class daemon*>::iterator iter;
 	dprintf( D_ALWAYS, "Reconfiguring all running daemons.\n" );
-	for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->pid && daemon_ptr[i]->runs_here ) {
-			daemon_ptr[i]->Reconfig();
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->pid && iter->second->runs_here ) {
+			iter->second->Reconfig();
 		}
 	}
 }
@@ -1730,24 +1970,21 @@ void
 Daemons::InitMaster()
 {
 		// initialize master data structure
-	master = GetIndex("MASTER");
-	if ( master == -1 ) {
+	master = FindDaemon("MASTER");
+	if ( master == NULL ) {
 		EXCEPT("InitMaster: MASTER not Specified");
 	}
-	daemon_ptr[master]->timeStamp = 
-		GetTimeStamp(daemon_ptr[master]->watch_name);
-	daemon_ptr[master]->startTime = time(0);
-	daemon_ptr[master]->pid = daemonCore->getpid();
+	master->timeStamp = GetTimeStamp(master->watch_name);
+	master->startTime = time(0);
+	master->pid = daemonCore->getpid();
 }
 
 
 void
 Daemons::RestartMaster()
 {
+	MasterShuttingDown = TRUE;
 	immediate_restart_master = immediate_restart;
-	if( NumberOfChildren() == 0 ) {
-		FinishRestartMaster();
-	}
 	all_daemons_gone_action = MASTER_RESTART;
 	StartDaemons = FALSE;
 	StopAllDaemons();
@@ -1756,10 +1993,8 @@ Daemons::RestartMaster()
 void
 Daemons::RestartMasterPeaceful()
 {
+	MasterShuttingDown = TRUE;
 	immediate_restart_master = immediate_restart;
-	if( NumberOfChildren() == 0 ) {
-		FinishRestartMaster();
-	}
 	all_daemons_gone_action = MASTER_RESTART;
 	StartDaemons = FALSE;
 	StopPeacefulAllDaemons();
@@ -1801,17 +2036,22 @@ Daemons::CleanupBeforeRestart()
 		// Release file lock on the log file.
 	if ( MasterLock->release() == FALSE ) {
 		dprintf( D_ALWAYS,
-				 "Can't remove lock on \"%s\"\n",daemon_ptr[master]->log_name);
+				 "Can't remove lock on \"%s\"\n",master->log_name);
 		EXCEPT( "file_lock(%d)", MasterLockFD );
 	}
 	(void)close( MasterLockFD );
 
-		// Now close all sockets and fds so our new invocation of
+		// Now set close-on-exec on all fds so our new invocation of
 		// condor_master does not inherit them.
 		// Note: Not needed (or wanted) on Win32, as CEDAR creates 
 		//		Winsock sockets as non-inheritable by default.
-	for (int i=0; i < max_fds; i++) {
-		close(i);
+		// Also not wanted for stderr, since we might be logging
+		// to that.  No real need for stdin or stdout either.
+	for (int i=3; i < max_fds; i++) {
+		int flag = fcntl(i,F_GETFD,0);
+		if( flag != -1 ) {
+			fcntl(i,F_SETFD,flag | 1);
+		}
 	}
 #endif
 
@@ -1867,7 +2107,9 @@ Daemons::ExecMaster()
 	}
 	argv[i++] = NULL;
 
-	(void)execv(daemon_ptr[master]->process_name, argv);
+	(void)execv(master->process_name, argv);
+
+	free(argv);
 }
 
 // Function that actually does the restart of the master.
@@ -1908,7 +2150,7 @@ Daemons::FinalRestartMaster()
 #endif
 		} else {
 			dprintf( D_ALWAYS, "Doing exec( \"%s\" )\n", 
-				 daemon_ptr[master]->process_name);
+				 master->process_name);
 
 				// It is important to switch to ROOT_PRIV, in case we are
 				// executing a master wrapper script that expects to be
@@ -1919,12 +2161,12 @@ Daemons::FinalRestartMaster()
 
 			set_priv(saved_priv);
 		}
-		daemon_ptr[master]->restarts++;
-		if ( NT_ServiceFlag == TRUE && daemon_ptr[master]->restarts > 7 ) {
+		master->restarts++;
+		if ( NT_ServiceFlag == TRUE && master->restarts > 7 ) {
 			dprintf(D_ALWAYS,"Unable to restart Condor service, aborting.\n");
 			master_exit(1);
 		}
-		i = daemon_ptr[master]->NextStart();
+		i = master->NextStart();
 		dprintf( D_ALWAYS, 
 				 "Cannot execute condor_master (errno=%d), will try again in %d seconds\n", 
 				 errno, i );
@@ -1935,12 +2177,14 @@ Daemons::FinalRestartMaster()
 
 char* Daemons::DaemonLog( int pid )
 {
+	std::map<std::string, class daemon*>::iterator iter;
+
 	// be careful : a pointer to data in this class is returned
 	// posibility of getting tampered
-
-	for ( int i=0; i < no_daemons; i++)
-		if ( daemon_ptr[i]->pid == pid )
-			return (daemon_ptr[i]->log_name);
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if ( iter->second->pid == pid )
+			return (iter->second->log_name);
+	}
 	return "Unknown Program!!!";
 }
 
@@ -1965,8 +2209,11 @@ Daemons::SignalAll( int signal )
 int Daemons::NumberOfChildren()
 {
 	int result = 0;
-	for( int i=0; i < no_daemons; i++) {
-		if( daemon_ptr[i]->runs_here && daemon_ptr[i]->pid ) {
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->runs_here && iter->second->pid
+			&& !iter->second->OnlyStopWhenMasterStops() ) {
 			result++;
 		}
 	}
@@ -1978,17 +2225,31 @@ int Daemons::NumberOfChildren()
 int
 Daemons::AllReaper(int pid, int status)
 {
+	std::map<std::string, class daemon*>::iterator iter;
+	std::map<int, class daemon*>::iterator valid_iter;
+
 		// find out which daemon died
-	for( int i=0; i < no_daemons; i++) {
-		if( pid == daemon_ptr[i]->pid ) {
-			daemon_ptr[i]->Exited( status );
+	valid_iter = exit_allowed.find(pid);
+	if( valid_iter != exit_allowed.end() ) {
+		valid_iter->second->Exited( status );
+ 		delete valid_iter->second;
+		exit_allowed.erase( valid_iter );
+		if( NumberOfChildren() == 0 ) {
+			AllDaemonsGone();
+		}
+		return TRUE;
+	}
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( pid == iter->second->pid ) {
+			iter->second->Exited( status );
 			if( NumberOfChildren() == 0 ) {
 				AllDaemonsGone();
 			}
 			return TRUE;
 		}
 	}
-	dprintf( D_ALWAYS, "Child %d died, but not a daemon -- Ignored\n", pid);
+
 	return TRUE;
 }
 
@@ -1996,17 +2257,27 @@ Daemons::AllReaper(int pid, int status)
 int
 Daemons::DefaultReaper(int pid, int status)
 {
-	for( int i=0; i < no_daemons; i++) {
-		if( pid == daemon_ptr[i]->pid ) {
-			daemon_ptr[i]->Exited( status );
+	std::map<std::string, class daemon*>::iterator iter;
+	std::map<int, class daemon*>::iterator valid_iter;
+
+	valid_iter = exit_allowed.find(pid);
+	if( valid_iter != exit_allowed.end() ) {
+		valid_iter->second->Exited( status );
+ 		delete valid_iter->second;
+		exit_allowed.erase(valid_iter);
+		return TRUE;
+	}
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( pid == iter->second->pid ) {
+			iter->second->Exited( status );
 			if( PublishObituaries ) {
-				daemon_ptr[i]->Obituary( status );
+				iter->second->Obituary( status );
 			}
-			daemon_ptr[i]->Restart();
+			iter->second->Restart();
 			return TRUE;
 		}
 	}
-	dprintf( D_ALWAYS, "Child %d died, but not a daemon -- Ignored\n", pid);
 	return TRUE;
 }
 
@@ -2051,6 +2322,24 @@ Daemons::SetDefaultReaper()
 	reaper = DEFAULT_R;
 }
 
+bool
+Daemons::StopDaemonsBeforeMasterStops()
+{
+		// now shut down all the daemons that should only stop right
+		// before the master stops
+	int running = 0;
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( ( iter->second->pid || iter->second->IsHA() )
+			&& iter->second->runs_here )
+		{
+			iter->second->Stop();
+			running++;
+		}
+	}
+	return !running;
+}
 
 void
 Daemons::AllDaemonsGone()
@@ -2061,10 +2350,16 @@ Daemons::AllDaemonsGone()
 		SetDefaultReaper();
 		break;
 	case MASTER_RESTART:
+		if( !StopDaemonsBeforeMasterStops() ) {
+			return;
+		}
 		dprintf( D_ALWAYS, "All daemons are gone.  Restarting.\n" );
 		FinishRestartMaster();
 		break;
 	case MASTER_EXIT:
+		if( !StopDaemonsBeforeMasterStops() ) {
+			return;
+		}
 		dprintf( D_ALWAYS, "All daemons are gone.  Exiting.\n" );
 		master_exit(0);
 		break;
@@ -2103,7 +2398,7 @@ Daemons::StartTimers()
 	if( new_preen && FS_Preen ) {
 		preen_tid = daemonCore->
 			Register_Timer( first_preen, preen_interval,
-							(TimerHandler)run_preen,
+							run_preen,
 							"run_preen()" );
 	}
 	old_preen_int = preen_interval;
@@ -2160,25 +2455,33 @@ void
 Daemons::Update( ClassAd* ca ) 
 {
 	char buf[128];
-	for( int i=0; i < no_daemons; i++) {
-		if( daemon_ptr[i]->runs_here || i == master ) {
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->runs_here || iter->second == master ) {
 			sprintf( buf, "%s_Timestamp = %ld", 
-					 daemon_ptr[i]->name_in_config_file, 	
-					 (long)daemon_ptr[i]->timeStamp );
+					 iter->second->name_in_config_file, 	
+					 (long)iter->second->timeStamp );
 			ca->Insert( buf );
-			if( daemon_ptr[i]->pid ) {
+			if( iter->second->pid ) {
 				sprintf( buf, "%s_StartTime = %ld", 
-						 daemon_ptr[i]->name_in_config_file, 	
-						 (long)daemon_ptr[i]->startTime );
+						 iter->second->name_in_config_file, 	
+						 (long)iter->second->startTime );
 				ca->Insert( buf );
 			} else {
 					// No pid, but daemon's supposed to be running.
 				sprintf( buf, "%s_StartTime = 0", 
-						 daemon_ptr[i]->name_in_config_file );
+						 iter->second->name_in_config_file );
 				ca->Insert( buf );
 			}
 		}
 	}
+
+		// CRUFT
+	ad->Assign(ATTR_MASTER_IP_ADDR, daemonCore->InfoCommandSinfulString());
+
+		// Initialize all the DaemonCore-provided attributes
+	daemonCore->publish( ad ); 	
 }
 
 
@@ -2192,8 +2495,10 @@ Daemons::UpdateCollector()
     daemonCore->monitor_data.ExportData(ad);
 	daemonCore->sendUpdates(UPDATE_MASTER_AD, ad, NULL, true);
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 	MasterPluginManager::Update(ad);
+#endif
 #endif
 
 		// log classad into sql log so that it can be updated to DB
@@ -2209,9 +2514,11 @@ Daemons::UpdateCollector()
 class daemon*
 Daemons::FindDaemon( daemon_t dt )
 {
-	for( int i=0; i < no_daemons; i++ ) {
-		if( daemon_ptr[i]->type == dt ) {
-			return daemon_ptr[i];
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->type == dt ) {
+			return iter->second;
 		}
 	}
 	return NULL;
@@ -2221,6 +2528,8 @@ Daemons::FindDaemon( daemon_t dt )
 void
 Daemons::CancelRestartTimers( void )
 {
+	std::map<std::string, class daemon*>::iterator iter;
+
 		// We don't need to be checking for new executables anymore. 
 	if( check_new_exec_tid != -1 ) {
 		daemonCore->Cancel_Timer( check_new_exec_tid );
@@ -2232,7 +2541,7 @@ Daemons::CancelRestartTimers( void )
 	}
 
 		// Finally, cancel the start/restart timers for each daemon.  
-	for( int i=0; i < no_daemons; i++ ) {
-		daemon_ptr[i]->CancelRestartTimers();
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		iter->second->CancelRestartTimers();
 	}
 }
