@@ -28,11 +28,15 @@
 #include "dc_collector.h"
 #include "condor_ver_info.h"
 
+#include <vector>
+#include <string>
+#include <map>
+#include <algorithm>
+
+using namespace std;
+
 /* FILESQL include */
 #include "file_sql.h"
-#ifdef WANT_NETMAN
-#include "../condor_netman/netman.h"
-#endif
 
 typedef struct MapEntry {
 	char *remoteHost;
@@ -43,6 +47,46 @@ typedef struct MapEntry {
 //extern ODBC *DBObj;
 /* FILESQL object extern */
 extern FILESQL *FILEObj;
+
+struct GroupEntry {
+    typedef vector<int>::size_type size_type;
+
+	GroupEntry();
+	~GroupEntry();
+
+    // these are set from configuration
+	string name;
+    double config_quota;
+	bool static_quota;
+	bool accept_surplus;
+
+    // current usage information coming into this negotiation cycle
+    double usage;
+    ClassAdList* submitterAds;
+
+    // slot quota as computed by HGQ
+    double quota;
+    // slots requested: jobs submitted against this group
+    double requested;
+    // slots allocated to this group by HGQ
+    double allocated;
+    // sum of slot quotas in this subtree
+    double subtree_quota;
+    // all slots requested by this group and its subtree
+    double subtree_requested;
+    // true if this group got served by most recent round robin
+    bool rr;
+    // timestamp of most recent allocation from round robin
+    double rr_time;
+    double subtree_rr_time;
+
+    // tree structure
+    GroupEntry* parent;
+    vector<GroupEntry*> children;
+    map<string, size_type, Accountant::ci_less> chmap;
+};
+
+
 class Matchmaker : public Service
 {
 	public:
@@ -70,12 +114,9 @@ class Matchmaker : public Service
 		int SET_LASTTIME_commandHandler(int, Stream*);
 		int GET_PRIORITY_commandHandler(int, Stream*);
 		int GET_RESLIST_commandHandler(int, Stream*);
-#ifdef WANT_NETMAN
-		int REQUEST_NETWORK_commandHandler(int, Stream*);
-#endif
 
 		// timeout handler (for periodic negotiations)
-		int negotiationTime ();
+		void negotiationTime ();
 
 			// the order of values in this enumeration is important!
 		enum PreemptState {PRIO_PREEMPTION,RANK_PREEMPTION,NO_PREEMPTION};
@@ -90,6 +131,7 @@ class Matchmaker : public Service
     protected:
 		char * NegotiatorName;
 		int update_interval;
+		
 
 	private:
 		ClassAd * publicAd;
@@ -99,19 +141,19 @@ class Matchmaker : public Service
 		void updateCollector();
 
 		// auxillary functions
-		bool obtainAdsFromCollector (ClassAdList&, ClassAdList&, ClassAdList&, ClaimIdHash& );	
-		char * compute_significant_attrs(ClassAdList & startdAds);
+		bool obtainAdsFromCollector (ClassAdList&, ClassAdListDoesNotDeleteAds&, ClassAdListDoesNotDeleteAds&, ClaimIdHash& );	
+		char * compute_significant_attrs(ClassAdListDoesNotDeleteAds & startdAds);
 		
 		/** Negotiate w/ one schedd for one user, for one 'pie spin'.
 			@param scheddName Name attribute from the submitter ad.
 			@param scheddAddr Sinful string of schedd for this submitter.
 			@param priority Priority of this user from the accountant.
 			@param share Priority w/o up-down (just relative prio factor).
-			@param scheddLimit Give away this many matches max
+			@param submitterLimit Give away this many matches max
 			@param startdAds
 			@param claimIds
 			@param scheddVersion
-			@param ignore_schedd_limit After hit scheddLimit, keep 
+			@param ignore_schedd_limit After hit submitterLimit, keep 
 					negotiating but only consider startd rank.
 			@param numMatched on return this is set to number of machines
 			        matched to this submitter.
@@ -120,25 +162,28 @@ class Matchmaker : public Service
 					MM_DONE if schedd got all the resources it wanted,
 					MM_ERROR if problem negotiating w/ this schedd.
 		**/
-		int negotiate( char *scheddName, const ClassAd *scheddAd, 
+		int negotiate( char const *scheddName, const ClassAd *scheddAd, 
 		   double priority, double share,
-		   int scheddLimit,
-		   ClassAdList &startdAds, ClaimIdHash &claimIds, 
+		   double submitterLimit,
+		   ClassAdListDoesNotDeleteAds &startdAds, ClaimIdHash &claimIds, 
 		   const CondorVersionInfo & scheddVersion,
-		   bool ignore_schedd_limit, time_t startTime, int &numMatched);
+		   bool ignore_schedd_limit, time_t startTime, 
+		   int &numMatched, double &limitUsed, double &pieLeft);
 
 		int negotiateWithGroup ( int untrimmed_num_startds,
-			ClassAdList& startdAds, 
-			ClaimIdHash& claimIds, ClassAdList& scheddAds, 
-			int groupQuota=INT_MAX, const char* groupAccountingName=NULL);
+								 double untrimmedSlotWeightTotal,
+								 double minSlotWeight,
+			ClassAdListDoesNotDeleteAds& startdAds, 
+			ClaimIdHash& claimIds, ClassAdListDoesNotDeleteAds& scheddAds, 
+			float groupQuota=INT_MAX, float groupusage=0, const char* groupAccountingName=NULL);
 
 		
-		ClassAd *matchmakingAlgorithm(char*,const char*,ClassAd&,ClassAdList&,
-									  double=-1.0, double=1.0, bool=false);
+		ClassAd *matchmakingAlgorithm(const char*,const char*,ClassAd&,ClassAdListDoesNotDeleteAds&,
+									  double=-1.0, double=1.0, double=0.0, double=0.0, double=0.0, bool=false);
 		int matchmakingProtocol(ClassAd &request, ClassAd *offer, 
 						ClaimIdHash &claimIds, Sock *sock,
-						char* scheddName, const char* scheddAddr);
-		void calculateNormalizationFactor (ClassAdList &, double &, double &,
+						const char* scheddName, const char* scheddAddr);
+		void calculateNormalizationFactor (ClassAdListDoesNotDeleteAds &, double &, double &,
 										   double &, double &);
 
 		/** Calculate a submitter's share of the pie.
@@ -146,68 +191,70 @@ class Matchmaker : public Service
 			@param scheddName Name attribute from the submitter ad.
 			@param groupAccountingName Group name from the submitter ad.
 			@param groupQuota Usage limit for this group.
-			@param numStartdAds Size of the pie in this spin.
 			@param maxPrioValue Largest prio value of any submitter.
 			@param maxAbsPrioValue Largest prio factor of any submitter
 			@param normalFactor Normalization for prio values
 			@param normalAbsFactor Normalization for prio factors
+			@param slotWeightTotal Size of the pie in this spin.
 
-			@param scheddLimit Resulting submitter share of this pie
-			@param scheddUsage Set to number of slots claimed by this submitter
-			@param scheddShare Resulting fractional share by prio and factor
-			@param scheddAbsShare Resulting fractional share by prio factor
-			@param scheddPrio User priority
-			@param scheddPrioFactor Result is this submitter's prio factor
-			@param scheddLimitRoundoff Difference between ideal share of pie
-			                           and rounded integer share.
+			@param submitterLimit Resulting submitter share of this pie
+			@param submitterUsage Set to number of slots claimed by this submitter
+			@param submitterShare Resulting fractional share by prio and factor
+			@param submitterAbsShare Resulting fractional share by prio factor
+			@param submitterPrio User priority
+			@param submitterPrioFactor Result is this submitter's prio factor
 		**/
-		void calculateScheddLimit(char const *scheddName,
+		void calculateSubmitterLimit(char const *scheddName,
 		                          char const *groupAccountingName,
-		                          int groupQuota,
-		                          int numStartdAds,
+		                          float groupQuota,
+					  float groupusage,
 		                          double maxPrioValue,
 		                          double maxAbsPrioValue,
 		                          double normalFactor,
 		                          double normalAbsFactor,
+								  double slotWeightTotal,
 		                            /* result parameters: */
-		                          int &scheddLimit,
-		                          int &scheddUsage,
-		                          double scheddShare,
-		                          double &scheddAbsShare,
-		                          double &scheddPrio,
-		                          double &scheddPrioFactor,
-		                          double &scheddLimitRoundoff );
+								  double &submitterLimit,
+								  double &submitterUsage,
+		                          double &submitterShare,
+		                          double &submitterAbsShare,
+		                          double &submitterPrio,
+		                          double &submitterPrioFactor);
 
-		/** Calculate a submitter's share of the pie.
+		/** Calculate how much pie might be dished out in this round.
 			@param quiet Do not emitt debug information about the calculation
 			@param scheddAds List of submitters
 			@param groupAccountingName Group name for all of these submitters
 			@param groupQuota Usage limit for this group.
-			@param numStartdAds Size of the pie in this spin.
 			@param maxPrioValue Largest prio value of any submitter.
 			@param maxAbsPrioValue Largest prio factor of any submitter
 			@param normalFactor Normalization for prio values
 			@param normalAbsFactor Normalization for prio factors
-			@param userprioCrumbs Resulting number of batch slots in this
-			                      pie which are left over after handing out
-			                      rounded integer shares.
+			@param pieLeft Sum of submitterLimits
 		**/
-		void calculateUserPrioCrumbs( ClassAdList &scheddAds,
-		                              char const *groupAccountingName,
-		                              int groupQuota,
-		                              int numStartdAds,
-		                              double maxPrioValue,
-		                              double maxAbsPrioValue,
-		                              double normalFactor,
-		                              double normalAbsFactor,
-		                                   /* result parameters: */
-		                              int &userprioCrumbs );
+		void calculatePieLeft( ClassAdListDoesNotDeleteAds &scheddAds,
+		                       char const *groupAccountingName,
+		                       float groupQuota,
+				       float groupusage,
+		                       double maxPrioValue,
+		                       double maxAbsPrioValue,
+		                       double normalFactor,
+		                       double normalAbsFactor,
+		                       double slotWeightTotal,
+		                            /* result parameters: */
+		                       double &pieLeft);
+
+			// rewrite the requirements expression to make matchmaking faster
+		void OptimizeMachineAdForMatchmaking(ClassAd *ad);
+
+			// rewrite the requirements expression to make matchmaking faster
+		void OptimizeJobAdForMatchmaking(ClassAd *ad);
 
 		void MakeClaimIdHash(ClassAdList &startdPvtAdList, ClaimIdHash &claimIds);
 		char const *getClaimId (const char *, const char *, ClaimIdHash &, MyString &);
 		void addRemoteUserPrios( ClassAd* ad );
 		void insertNegotiatorMatchExprs(ClassAd *ad);
-		void insertNegotiatorMatchExprs( ClassAdList &cal );
+		void insertNegotiatorMatchExprs( ClassAdListDoesNotDeleteAds &cal );
 		void reeval( ClassAd *ad );
 		void updateNegCycleEndTime(time_t startTime, ClassAd *submitter);
 		static unsigned int HashFunc(const MyString &Key);
@@ -216,12 +263,18 @@ class Matchmaker : public Service
 
 			// If we are not considering preemption, this function will
 			// trim out startd ads that are not in the Unclaimed state.
-		int trimStartdAds(ClassAdList &startdAds);
+		int trimStartdAds(ClassAdListDoesNotDeleteAds &startdAds);
+
+		bool SubmitterLimitPermits(ClassAd *candidate, double used, double allowed, double pieLeft);
+		double sumSlotWeights(ClassAdListDoesNotDeleteAds &startdAds,double *minSlotWeight, ExprTree* constraint);
 
 		/* ODBC insert functions */
-		void insert_into_rejects(char *userName, ClassAd& job);
-		void insert_into_matches(char * userName, ClassAd& request, ClassAd& offer);
+		void insert_into_rejects(char const *userName, ClassAd& job);
+		void insert_into_matches(char const *userName, ClassAd& request, ClassAd& offer);
 		
+
+		void RegisterAttemptedOfflineMatch( ClassAd *job_ad, ClassAd *startd_ad );
+
 		// configuration information
 		char *AccountantHost;		// who (if at all) is the accountant?
 		int  NegotiatorInterval;	// interval between negotiation cycles
@@ -252,17 +305,12 @@ class Matchmaker : public Service
 		typedef HashTable<MyString, MapEntry*> AdHash;
 		AdHash *stashedAds;			
 
-		typedef HashTable<MyString, int> groupQuotasHashType;
+		typedef HashTable<MyString, float> groupQuotasHashType;
 		groupQuotasHashType *groupQuotasHash;
 
-		bool getGroupInfoFromUserId( const char *user, int & groupQuota, 
-			 int & groupUsage );
+		bool getGroupInfoFromUserId( const char *user, float & groupQuota, 
+			 float & groupUsage );
 		
-#ifdef WANT_NETMAN
-		// allocate network capacity
-		NetworkManager netman;
-		bool allocNetworkShares;
-#endif
 
 
 		// rank condition on matches
@@ -294,6 +342,7 @@ class Matchmaker : public Service
 		int rejPreemptForPrio;	//   - insufficient prio to preempt?
 		int rejPreemptForPolicy; //   - PREEMPTION_REQUIREMENTS == False?
 		int rejPreemptForRank;	//   - startd RANKs new job lower?
+		int rejForSubmitterLimit;   //   - not enough group quota?
 
 
 		// Class used to store each individual entry in the
@@ -317,6 +366,8 @@ class Matchmaker : public Service
 			ClassAd *ad;
 		};
 
+		void DeleteMatchList();
+
 		// List of matches.
 		// This list is essentially a list of sorted matching
 		// machine ads for a job ad of a given autocluster from
@@ -331,7 +382,7 @@ class Matchmaker : public Service
 		class MatchListType
 		{
 		public:
-			
+
 			ClassAd* pop_candidate();
 			bool cache_still_valid(ClassAd &request,ExprTree *preemption_req,
 				ExprTree *preemption_rank,bool preemption_req_unstable, bool preemption_rank_unstable);
@@ -340,13 +391,15 @@ class Matchmaker : public Service
 					int & rejForConcurrencyLimit,
 					int & rejPreemptForPrio,
 					int & rejPreemptForPolicy,
-					int & rejPreemptForRank);
+					int & rejPreemptForRank,
+					int & rejForSubmitterLimit);
 			void set_diagnostics(int rejForNetwork,
 					int rejForNetworkShare,
 					int rejForConcurrencyLimit,
 					int rejPreemptForPrio,
 					int rejPreemptForPolicy,
-					int rejPreemptForRank);
+					int rejPreemptForRank,
+					int rejForSubmitterLimit);
 			void add_candidate(ClassAd* candidate,
 					double candidateRankValue,
 					double candidatePreJobRankValue,
@@ -358,7 +411,9 @@ class Matchmaker : public Service
 
 			MatchListType(int maxlen);
 			~MatchListType();
-			
+
+			void increment_rejForSubmitterLimit() { m_rejForSubmitterLimit++; }
+
 		private:
 
 			// AdListEntry* peek_candidate();
@@ -374,7 +429,10 @@ class Matchmaker : public Service
 			int m_rejForConcurrencyLimit;	//   - limited concurrency?
 			int m_rejPreemptForPrio;	//   - insufficient prio to preempt?
 			int m_rejPreemptForPolicy; //   - PREEMPTION_REQUIREMENTS == False?
-			int m_rejPreemptForRank;	//   - startd RANKs new job lower?
+			int m_rejPreemptForRank;    //   - startd RANKs new job lower?
+			int m_rejForSubmitterLimit;     //  - not enough group quota?
+			float m_submitterLimit;
+			
 			
 		};
 		MatchListType* MatchList;
@@ -384,23 +442,38 @@ class Matchmaker : public Service
 		double cachedPrio;
 		bool cachedOnlyForStartdRank;
 
-		//
-		class SimpleGroupEntry
-		{
-		public:
-			SimpleGroupEntry();
-			~SimpleGroupEntry();
-			char *groupName;
-			float prio;
-			int maxAllowed;
-			int usage;
-			ClassAdList submitterAds;			
-		};
-		static int groupSortCompare(const void*, const void*);
-		
-		char RejectsTable[40];
-		char MatchesTable[40];
+        // set at startup/restart/reinit
+        GroupEntry* hgq_root_group;
+        string hgq_root_name;
+        vector<GroupEntry*> hgq_groups;
+        map<string, GroupEntry*> group_entry_map;
+
+        void hgq_construct_tree();
+        void hgq_assign_quotas(GroupEntry* group, double quota);
+        double hgq_fairshare(GroupEntry* group);
+        double hgq_allocate_surplus(GroupEntry* group, double surplus);
+        double hgq_recover_remainders(GroupEntry* group);
+        double hgq_round_robin(GroupEntry* group, double surplus);
+
+        struct ord_by_rr_time {
+            vector<GroupEntry*>* data;
+            bool operator()(unsigned long const& ja, unsigned long const& jb) const {
+                GroupEntry* a = (*data)[ja];
+                GroupEntry* b = (*data)[jb];
+                if (a->subtree_rr_time != b->subtree_rr_time) return a->subtree_rr_time < b->subtree_rr_time;
+                if (a->subtree_quota != b->subtree_quota) return a->subtree_quota > b->subtree_quota;
+                return a->subtree_requested > b->subtree_requested;
+            }
+        };
+
 		int prevLHF;
+
+		#define MAX_NEGOTIATION_CYCLE_STATS 100
+		class NegotiationCycleStats *negotiation_cycle_stats[MAX_NEGOTIATION_CYCLE_STATS];
+		int num_negotiation_cycle_stats;
+
+		void StartNewNegotiationCycleStat();
+		void publishNegotiationCycleStats( ClassAd *ad );
 };
 
 
