@@ -64,14 +64,6 @@
 #include <sys/user.h>
 #endif 
 
-#ifdef OSF1                // this is for getting physical/available
-#include <sys/table.h>     // memory in OSF1
-#endif
-
-#ifdef IRIX              // this is the include for memory info in Irix.
-#include <sys/sysmp.h>
-#endif
-
 #ifdef AIX
 #include <procinfo.h>
 #include <sys/types.h>
@@ -80,6 +72,8 @@ BEGIN_C_DECLS
 extern int getprocs64(struct procentry64*, int, struct fdsinfo64*, int,
         pid_t*, int);
 END_C_DECLS
+/* AIX # defines the following symbol, which we use later as a formal param*/
+#undef l_name
 #endif
 
 #else // It's WIN32...
@@ -196,8 +190,7 @@ typedef long birthday_t;
     calls don't have to be made.  All OS's support the given information,
     WITH THE EXCEPTION OF MAJ/MIN FAULTS:
     <ul>
-     <li> Linux, Irix, and Hpux all return a reasonable-looking number.
-     <li> They are completely unsupported by OSF/1.
+     <li> Linux and Hpux return a reasonable-looking number.
      <li> Solaris 2.5.1 and 2.6 *Sometimes* returns small number of major 
         faults, and I've only seen a 0 for minor faults.  The documentation
         claims that they are inexact values, anyway.
