@@ -28,7 +28,7 @@
 // operation
 //
 static void
-log_exit(char* op_str, proc_family_error_t error_code)
+log_exit(const char* op_str, proc_family_error_t error_code)
 {
 	int debug_level = D_PROCFAMILY;
 	if (error_code != PROC_FAMILY_ERROR_SUCCESS) {
@@ -254,7 +254,7 @@ ProcFamilyClient::track_family_via_login(pid_t pid,
 
 #if defined(LINUX)
 bool
-ProcFamilyClient::track_family_via_supplementary_group(pid_t pid,
+ProcFamilyClient::track_family_via_allocated_supplementary_group(pid_t pid,
                                                        bool& response,
                                                        gid_t& gid)
 {
@@ -272,7 +272,7 @@ ProcFamilyClient::track_family_via_supplementary_group(pid_t pid,
 	char* ptr = (char*)buffer;
 
 	*(proc_family_command_t*)ptr =
-		PROC_FAMILY_TRACK_FAMILY_VIA_SUPPLEMENTARY_GROUP;
+		PROC_FAMILY_TRACK_FAMILY_VIA_ALLOCATED_SUPPLEMENTARY_GROUP;
 	ptr += sizeof(proc_family_command_t);
 
 	*(pid_t*)ptr = pid;
@@ -306,7 +306,59 @@ ProcFamilyClient::track_family_via_supplementary_group(pid_t pid,
 	}
 	m_client->end_connection();
 
-	log_exit("track_family_via_supplementary_group", err);
+	log_exit("track_family_via_allocated_supplementary_group", err);
+	response = (err == PROC_FAMILY_ERROR_SUCCESS);
+	return true;
+}
+
+bool
+ProcFamilyClient::track_family_via_associated_supplementary_group(pid_t pid,
+                                                       gid_t gid,
+                                                       bool& response)
+{
+	ASSERT(m_initialized);
+
+	dprintf(D_PROCFAMILY,
+	        "About to tell ProcD to track family with root %u "
+	            "via GID %u\n",
+	        pid, 
+			gid);
+
+	int message_len = sizeof(proc_family_command_t) +
+	                  sizeof(pid_t) +
+					  sizeof(gid_t);
+	void* buffer = malloc(message_len);
+	ASSERT(buffer != NULL);
+	char* ptr = (char*)buffer;
+
+	*(proc_family_command_t*)ptr =
+		PROC_FAMILY_TRACK_FAMILY_VIA_ASSOCIATED_SUPPLEMENTARY_GROUP;
+	ptr += sizeof(proc_family_command_t);
+
+	*(pid_t*)ptr = pid;
+	ptr += sizeof(pid_t);
+
+	*(gid_t*)ptr = gid;
+	ptr += sizeof(gid_t);
+
+	ASSERT(ptr - (char*)buffer == message_len);
+	
+	if (!m_client->start_connection(buffer, message_len)) {
+		dprintf(D_ALWAYS,
+		        "ProcFamilyClient: failed to start connection with ProcD\n");
+		free(buffer);
+		return false;
+	}
+	free(buffer);
+	proc_family_error_t err;
+	if (!m_client->read_data(&err, sizeof(proc_family_error_t))) {
+		dprintf(D_ALWAYS,
+		        "ProcFamilyClient: failed to read response from ProcD\n");
+		return false;
+	}
+	m_client->end_connection();
+
+	log_exit("track_family_via_associated_supplementary_group", err);
 	response = (err == PROC_FAMILY_ERROR_SUCCESS);
 	return true;
 }
@@ -639,10 +691,11 @@ ProcFamilyClient::quit(bool& response)
 	response = (err == PROC_FAMILY_ERROR_SUCCESS);
 	return true;
 }
-#if defined(PROCD_DEBUG)
 
-LocalClient*
-ProcFamilyClient::dump(pid_t pid)
+bool
+ProcFamilyClient::dump(pid_t pid,
+                       bool& response,
+                       std::vector<ProcFamilyDump>& vec)
 {
 	ASSERT(m_initialized);
 
@@ -662,16 +715,71 @@ ProcFamilyClient::dump(pid_t pid)
 
 	ASSERT(ptr - (char*)buffer == message_len);
 
-	m_client->start_connection(buffer, message_len);
+	if (!m_client->start_connection(buffer, message_len)) {
+		dprintf(D_ALWAYS,
+		        "ProcFamilyClient: "
+			    "failed to start connection with ProcD\n");
+
+		free(buffer);
+		return false;
+	}
+	free(buffer);
 
 	proc_family_error_t err;
-	m_client->read_data(&err, sizeof(proc_family_error_t));
-	if (err != PROC_FAMILY_ERROR_SUCCESS) {
+	if (!m_client->read_data(&err, sizeof(proc_family_error_t))) {
+		dprintf(D_ALWAYS,
+		        "ProcFamilyClient: "
+			    "failed to read response from ProcD\n");
+		return false;
+	}
+	response = (err == PROC_FAMILY_ERROR_SUCCESS);
+	if (!response) {
 		m_client->end_connection();
-		return NULL;
+		log_exit("dump", err);
+		return true;
 	}
 
-	return m_client;
-}
+	vec.clear();
 
-#endif
+	int family_count;
+	if (!m_client->read_data(&family_count, sizeof(int))) {
+		dprintf(D_ALWAYS,
+		        "ProcFamilyClient: "
+		            "failed to read family count from ProcD\n");
+		return false;
+	}
+	vec.resize(family_count);
+	for (int i = 0; i < family_count; ++i) {
+		if (!m_client->read_data(&vec[i].parent_root, sizeof(pid_t)) ||
+		    !m_client->read_data(&vec[i].root_pid, sizeof(pid_t)) ||
+			!m_client->read_data(&vec[i].watcher_pid, sizeof(pid_t)))
+		{
+			dprintf(D_ALWAYS,
+			        "ProcFamilyClient: "
+			            "failed reading family dump info from ProcD\n");
+			return false;
+		}
+		int proc_count;
+		if (!m_client->read_data(&proc_count, sizeof(int))) {
+			dprintf(D_ALWAYS,
+			        "ProcFamilyClient: "
+			            "failed reading process count from ProcD\n");
+			return false;
+		}
+		vec[i].procs.resize(proc_count);
+		for (int j = 0; j < proc_count; ++j) {
+			if (!m_client->read_data(&vec[i].procs[j],
+			                         sizeof(ProcFamilyProcessDump)))
+			{
+			dprintf(D_ALWAYS,
+			        "ProcFamilyClient: "
+			            "failed reading process dump info from ProcD\n");
+			return false;
+			}
+		}
+	}
+	m_client->end_connection();
+
+	log_exit("dump", err);
+	return true;
+}
