@@ -26,19 +26,21 @@
 #include "parent_tracker.h"
 
 #if !defined(WIN32)
-#include "glexec_kill.h"
+#include "glexec_kill.unix.h"
 #endif
 
 #if defined(LINUX)
-#include "group_tracker.h"
+#include "group_tracker.linux.h"
 #endif
 
 ProcFamilyMonitor::ProcFamilyMonitor(pid_t pid,
                                      birthday_t birthday,
-                                     int snapshot_interval) :
+                                     int snapshot_interval,
+									 bool except_if_pid_dies) :
 	m_everybody_else(NULL),
 	m_family_table(11, pidHashFunc, rejectDuplicateKeys),
-	m_member_table(PHBUCKETS, pidHashFunc, rejectDuplicateKeys)
+	m_member_table(PHBUCKETS, pidHashFunc, rejectDuplicateKeys),
+	m_except_if_pid_dies(except_if_pid_dies)
 {
 	// the snapshot interval must either be non-negative or -1, which
 	// means infinite (higher layers should enforce this)
@@ -71,14 +73,15 @@ ProcFamilyMonitor::ProcFamilyMonitor(pid_t pid,
 	m_everybody_else = new ProcFamily(this);
 	ASSERT(m_everybody_else != NULL);
 
-	// create the "root" family; set the watcher pid to be the same as
-	// the root pid: there's some special logic in delete_unwatched_families
-	// to EXCEPT if we see that the root process has exited
+	// create the "root" family; if we'd like to EXCEPT if the root family
+	// dies, we make the watcher pid the same as the root pid.  Otherwise, we
+	// set it to zero which means that "garbage collection" won't be enabled
+	// for this family and we won't EXCEPT if it dies.
 	//
 	ProcFamily* family = new ProcFamily(this,
 	                                    pid,
 	                                    birthday,
-	                                    pid,
+	                                    except_if_pid_dies ? pid : 0,
 	                                    snapshot_interval);
 	ASSERT(family != NULL);
 
@@ -120,12 +123,14 @@ ProcFamilyMonitor::~ProcFamilyMonitor()
 #if defined(LINUX)
 void
 ProcFamilyMonitor::enable_group_tracking(gid_t min_tracking_gid,
-                                         gid_t max_tracking_gid)
+                                         gid_t max_tracking_gid,
+										 bool allocating)
 {
 	ASSERT(m_group_tracker == NULL);
 	m_group_tracker = new GroupTracker(this,
 	                                   min_tracking_gid,
-	                                   max_tracking_gid);
+	                                   max_tracking_gid,
+									   allocating);
 	ASSERT(m_group_tracker != NULL);
 }
 #endif
@@ -186,9 +191,8 @@ ProcFamilyMonitor::register_subfamily(pid_t root_pid,
 
 	// make sure this process isn't already the root of a family
 	//
-	Tree<ProcFamily*>* tree;
-	ret = m_family_table.lookup(root_pid, tree);
-	if (ret != -1) {
+	Tree<ProcFamily*>* tree = lookup_family(root_pid);
+	if (tree != NULL) {
 		dprintf(D_ALWAYS,
 		        "register_subfamily: pid %u already registered\n",
 		        root_pid);
@@ -213,10 +217,10 @@ ProcFamilyMonitor::register_subfamily(pid_t root_pid,
 	// the parent-child link
 	//
 	pid_t parent_root = member->get_proc_family()->get_root_pid();
-	Tree<ProcFamily*>* parent_tree_node;
-	ret = m_family_table.lookup(parent_root, parent_tree_node);
-	ASSERT(ret != -1);
-	Tree<ProcFamily*>* child_tree_node = parent_tree_node->add_child(family);
+	Tree<ProcFamily*>* parent_tree_node = lookup_family(parent_root);
+	ASSERT(parent_tree_node != NULL);
+	Tree<ProcFamily*>* child_tree_node =
+		parent_tree_node->add_child(family);
 	ASSERT(child_tree_node != NULL);
 
 	// move the new family's root process into the correct family
@@ -240,14 +244,27 @@ ProcFamilyMonitor::register_subfamily(pid_t root_pid,
 	return PROC_FAMILY_ERROR_SUCCESS;
 }
 
+Tree<ProcFamily*>*
+ProcFamilyMonitor::lookup_family(pid_t pid, bool zero_means_root)
+{
+	if (zero_means_root && (pid == 0)) {
+		return m_tree;
+	}
+	Tree<ProcFamily*>* tree;
+	int ret = m_family_table.lookup(pid, tree);
+	if (ret == -1) {
+		return NULL;
+	}
+	return tree;
+}
+
 proc_family_error_t
 ProcFamilyMonitor::track_family_via_environment(pid_t pid, PidEnvID* penvid)
 {
 	// lookup the family
 	//
-	Tree<ProcFamily*>* tree;
-	int ret = m_family_table.lookup(pid, tree);
-	if (ret == -1) {
+	Tree<ProcFamily*>* tree = lookup_family(pid, true);
+	if (tree == NULL) {
 		dprintf(D_ALWAYS,
 		        "track_family_via_environment failure: "
 				    "family with root %u not found\n",
@@ -266,9 +283,8 @@ ProcFamilyMonitor::track_family_via_login(pid_t pid, char* login)
 {
 	// lookup the family
 	//
-	Tree<ProcFamily*>* tree;
-	int ret = m_family_table.lookup(pid, tree);
-	if (ret == -1) {
+	Tree<ProcFamily*>* tree = lookup_family(pid, true);
+	if (tree == NULL) {
 		dprintf(D_ALWAYS,
 		        "track_family_via_login failure: "
 				    "family with root %u not found\n",
@@ -294,9 +310,8 @@ ProcFamilyMonitor::track_family_via_supplementary_group(pid_t pid, gid_t& gid)
 
 	// lookup the family
 	//
-	Tree<ProcFamily*>* tree;
-	int ret = m_family_table.lookup(pid, tree);
-	if (ret == -1) {
+	Tree<ProcFamily*>* tree = lookup_family(pid, true);
+	if (tree == NULL) {
 		dprintf(D_ALWAYS,
 		        "track_family_via_supplementary_group failure: "
 				    "family with root %u not found\n",
@@ -318,9 +333,8 @@ ProcFamilyMonitor::unregister_subfamily(pid_t pid)
 {
 	// lookup the family
 	//
-	Tree<ProcFamily*>* tree;
-	int ret = m_family_table.lookup(pid, tree);
-	if (ret == -1) {
+	Tree<ProcFamily*>* tree = lookup_family(pid);
+	if (tree == NULL) {
 		dprintf(D_ALWAYS,
 		        "unregister_subfamily failure: family with root %u not found\n",
 		        pid);
@@ -390,9 +404,8 @@ ProcFamilyMonitor::signal_process(pid_t pid, int sig)
 {
 	// make sure signals are only sent to subtree roots
 	//
-	Tree<ProcFamily*>* tree;
-	int ret = m_family_table.lookup(pid, tree);
-	if (ret == -1) {
+	Tree<ProcFamily*>* tree = lookup_family(pid, true);
+	if (tree == NULL) {
 		dprintf(D_ALWAYS,
 		        "signal_process failure: family with root %u not found\n",
 				pid);
@@ -414,9 +427,8 @@ ProcFamilyMonitor::signal_family(pid_t pid, int sig)
 
 	// find the family
 	//
-	Tree<ProcFamily*>* tree;
-	int ret = m_family_table.lookup(pid, tree);
-	if (ret == -1) {
+	Tree<ProcFamily*>* tree = lookup_family(pid, true);
+	if (tree == NULL) {
 		dprintf(D_ALWAYS,
 		        "signal_family error: family with root %u not found\n",
 		        pid);
@@ -436,9 +448,8 @@ ProcFamilyMonitor::get_family_usage(pid_t pid, ProcFamilyUsage* usage)
 {
 	// find the family
 	//
-	Tree<ProcFamily*>* tree;
-	int ret = m_family_table.lookup(pid, tree);
-	if (ret == -1) {
+	Tree<ProcFamily*>* tree = lookup_family(pid, true);
+	if (tree == NULL) {
 		dprintf(D_ALWAYS,
 		        "get_family_usage failure: family with root %u not found\n",
 		        pid);
@@ -455,6 +466,7 @@ ProcFamilyMonitor::get_family_usage(pid_t pid, ProcFamilyUsage* usage)
 	usage->sys_cpu_time = 0;
 	usage->percent_cpu = 0.0;
 	usage->total_image_size = 0;
+    usage->total_resident_set_size = 0;
 	usage->num_procs = 0;
 	get_family_usage(tree, usage);
 
@@ -619,7 +631,7 @@ ProcFamilyMonitor::lookup_member(pid_t pid)
 bool
 ProcFamilyMonitor::add_member_to_family(ProcFamily* pf,
                                         procInfo* pi,
-                                        char* method_str)
+                                        const char* method_str)
 {
 	// see if this process has already been inserted into a family
 	//
@@ -645,9 +657,8 @@ ProcFamilyMonitor::add_member_to_family(ProcFamily* pf,
 		// already assigned to (i.e. pf is a "more specific"
 		// match), then move the process
 		//
-		Tree<ProcFamily*>* node;
-		ret = m_family_table.lookup(pf->get_root_pid(), node);
-		ASSERT(ret != -1);
+		Tree<ProcFamily*>* node = lookup_family(pf->get_root_pid());
+		ASSERT(node != NULL);
 		while(node->get_parent() != NULL) {
 			node = node->get_parent();
 			if (node->get_data() == pfm->get_proc_family()) {
@@ -794,11 +805,13 @@ ProcFamilyMonitor::delete_unwatched_families(Tree<ProcFamily*>* tree)
 	}
 
 	// check to see if the current tree node's watcher has exited
-	//
+	// (a watcher PID of 0 means the family is not watched: automatic
+	// unregistering will not happen)
 	pid_t watcher_pid = tree->get_data()->get_watcher_pid();
 	if (watcher_pid == 0) {
 		return;
 	}
+
 	ProcFamilyMember* member;
 	int ret = m_member_table.lookup(watcher_pid, member);
 	if (ret != -1) {
@@ -821,14 +834,16 @@ ProcFamilyMonitor::delete_unwatched_families(Tree<ProcFamily*>* tree)
 		        tree->get_data()->get_root_birthday());
 	}
 
-	// it looks like the watcher has exited; if this family is the
-	// "root family", then this means the Master has gone away and
-	// we should die. otherwise, simply unregister the unwatched
-	// family
+	// it looks like the watcher has exited. if this family is the
+	// "root family" and we should except if it dies, then this means 
+	// the Master has gone away and we should die. 
+	// otherwise, simply unregister the unwatched family.
 	//
-	if (tree->get_parent() == NULL) {
-		EXCEPT("master has exited");
+	if (m_except_if_pid_dies && tree->get_parent() == NULL) {
+		EXCEPT("Watcher pid %lu exited so procd is exiting. Bye.",
+			(unsigned long)watcher_pid);
 	}
+	ASSERT(tree->get_parent() != NULL);
 	pid_t root_pid = tree->get_data()->get_root_pid();
 	unregister_subfamily(tree);
 
@@ -893,59 +908,49 @@ ProcFamilyMonitor::delete_all_families(Tree<ProcFamily*>* tree)
 	delete tree->get_data();
 }
 
-#if defined(PROCD_DEBUG)
-
-void
-ProcFamilyMonitor::output(LocalServer& server, pid_t pid)
+proc_family_error_t
+ProcFamilyMonitor::dump(pid_t pid, std::vector<ProcFamilyDump>& vec)
 {
-	// lookup the family. if the look fails, send "false" to the
+	// lookup the family. if the lookup fails, send "false" to the
 	// client and return. otherwise, send "true" and keep going
 	//
-	Tree<ProcFamily*>* tree;
-	int ret = m_family_table.lookup(pid, tree);
-	bool ok = (ret != -1);
-	server.write_data(&ok, sizeof(bool));
-	if (!ok) {
+	Tree<ProcFamily*>* tree = lookup_family(pid, true);
+	if (tree == NULL) {
 		dprintf(D_ALWAYS,
 		        "output failure: family with root %u not found\n",
 		        pid);
-		return;
+		return PROC_FAMILY_ERROR_FAMILY_NOT_FOUND;
 	}
 
-	// begin recusrsion
+	// clear the passed-in vector and fill it using our helper method
 	//
-	output(server, pid, tree);
+	vec.clear();
+	dump(tree, vec);
 
-	// write a zero when we're done sending family info
-	//
-	int zero = 0;
-	server.write_data(&zero, sizeof(int));
+	return PROC_FAMILY_ERROR_SUCCESS;
 }
 
 void
-ProcFamilyMonitor::output(LocalServer& server,
-                          pid_t pid,
-                          Tree<ProcFamily*>* tree)
+ProcFamilyMonitor::dump(Tree<ProcFamily*>* tree,
+                        std::vector<ProcFamilyDump>& vec)
 {
-	// output the current family first. for each subfamily (i.e. all families
-	// except the one rooted at the given pid), we'll first send back the
-	// family's parent-family's root pid
+	// do the current family first
 	//
-	if (pid != tree->get_data()->get_root_pid()) {
-		Tree<ProcFamily*>* parent = tree->get_parent();
-		ASSERT(parent != NULL);
-		pid_t parent_root_pid = parent->get_data()->get_root_pid();
-		server.write_data(&parent_root_pid, sizeof(pid_t));
+	ProcFamilyDump fam;
+	fam.parent_root = 0;
+	if (tree->get_parent() != NULL) {
+		fam.parent_root = tree->get_parent()->get_data()->get_root_pid();
 	}
-	tree->get_data()->output(server);
+	fam.root_pid = tree->get_data()->get_root_pid();
+	fam.watcher_pid = tree->get_data()->get_watcher_pid();
+	tree->get_data()->dump(fam);
+	vec.push_back(fam);
 
-	// recurse on children
+	// now recurse on children
 	//
 	Tree<ProcFamily*>* child = tree->get_child();
 	while (child != NULL) {
-		output(server, pid, child);
+		dump(child, vec);
 		child = child->get_sibling();
 	}
 }
-
-#endif
