@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2010, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -36,14 +36,16 @@
 #include "dc_collector.h"
 #include "condor_distribution.h"
 #include "CondorError.h"
+#include "str_isxxx.h"
 
 
 char	*MyName;
 char 	*actionReason = NULL;
 JobAction mode;
 bool All = false;
+bool ConstraintArg = false;
+bool UserJobIdArg = false;
 bool had_error = false;
-bool old_messages = false;
 
 DCSchedd* schedd = NULL;
 
@@ -56,7 +58,6 @@ void usage();
 void handleAll();
 void handleConstraints( void );
 ClassAd* doWorkByList( StringList* ids, CondorError * errstack );
-void printOldMessages( ClassAd* result_ad, StringList* ids );
 void printNewMessages( ClassAd* result_ad, StringList* ids );
 bool mayUserForceRm( );
 
@@ -133,7 +134,7 @@ usage()
 		fprintf( stderr,
 				     "  -fast               Use a fast vacate (hardkill)\n" );
 	}
-	fprintf( stderr, " and where [constraints] is one or more of:\n" );
+	fprintf( stderr, " and where [constraints] is one of:\n" );
 	fprintf( stderr, "  cluster.proc        %s the given job\n", word );
 	fprintf( stderr, "  cluster             %s the given cluster of jobs\n", word );
 	fprintf( stderr, "  user                %s all jobs owned by user\n", word );
@@ -237,6 +238,7 @@ main( int argc, char *argv[] )
 				}				
 				args[nArgs] = *argv;
 				nArgs++;
+				ConstraintArg = true;
             } else if (match_prefix(arg, "-all")) {
                 All = true;
             } else if (match_prefix(arg, "-addr")) {
@@ -337,6 +339,7 @@ main( int argc, char *argv[] )
 			}
 			args[nArgs] = arg;
 			nArgs++;
+			UserJobIdArg = true;
 		}
 	}
 
@@ -346,13 +349,9 @@ main( int argc, char *argv[] )
 		usage();
 	}
 
-	old_messages = false;
-	char* tmp = param( "TOOLS_PROVIDE_OLD_MESSAGES" );
-	if( tmp ) {
-		if( tmp[0] == 'T' || tmp[0] == 't' ) {
-			old_messages = true;
-		}
-		free( tmp );
+	if ( ConstraintArg && UserJobIdArg ) {
+		fprintf( stderr, "You can't use both -constraint and usernames or job ids\n" );
+		usage();
 	}
 
 		// Pick the default reason if the user didn't specify one
@@ -389,20 +388,6 @@ main( int argc, char *argv[] )
 		exit( 1 );
 	}
 
-		// If this schedd doesn't support the new protocol, give a
-		// useful error message.
-	CondorVersionInfo ver_info( schedd->version(), "SCHEDD" );
-	if( ! ver_info.built_since_version(6, 3, 3) ) {
-		fprintf( stderr, "The version of the condor_schedd you want to "
-				 "communicate with is:\n%s\n", schedd->version() );
-		fprintf( stderr, "It is too old to support this version of "
-				 "%s:\n%s\n", MyName, CondorVersion() );
-		fprintf( stderr, "To use this version of %s you must upgrade "
-				 "the\n%s to at least version 6.3.3.\nAborting.\n",
-				 MyName, schedd->idStr() ); 
-		exit( 1 );
-	}
-
 		// Special case for condor_rm -forcex: a configuration
 		// setting can disable this functionality.  The real
 		// validation is done in the schedd, but we can catch
@@ -440,12 +425,7 @@ main( int argc, char *argv[] )
 		if (had_error) {
 			fprintf( stderr, "%s\n", errstack.getFullText(true) );
 		}
-		if( old_messages ) {
-			printOldMessages( result_ad, job_ids );
-		} else {
-				// happy day, we can use the new messages! :)
-			printNewMessages( result_ad, job_ids );
-		}
+		printNewMessages( result_ad, job_ids );
 		delete( result_ad );
 	}
 
@@ -472,6 +452,7 @@ bool
 doWorkByConstraint( const char* constraint, CondorError * errstack )
 {
 	ClassAd* ad;
+	int total_jobs = -1;
 	bool rval = true;
 	switch( mode ) {
 	case JA_RELEASE_JOBS:
@@ -501,10 +482,23 @@ doWorkByConstraint( const char* constraint, CondorError * errstack )
 		rval = false;
 	} else {
 		int result = FALSE;
-		if( !ad->LookupInteger(ATTR_ACTION_RESULT, result) || !result ) {
+		if( !ad->LookupInteger(ATTR_ACTION_RESULT, result) ) {
 			had_error = true;
 			rval = false;
 		}
+		else if( !result ) {
+			// There were no ads acted upon, but that doesn't
+			// mean there was an error.  It's possible the schedd
+			// had no jobs
+		        if( !ad->LookupInteger(ATTR_TOTAL_JOB_ADS, total_jobs) || total_jobs > 0 ) {
+				had_error = true;
+			} else {
+				// There were no jobs in the queue, so add a
+				// more meaningful error message
+				errstack->push("condor_rm", 0, "There are no jobs in the queue");
+			}
+			rval = false;
+		}
 	}
 	return rval;
 }
@@ -556,7 +550,7 @@ procArg(const char* arg)
 
 	MyString constraint;
 
-	if(isdigit(*arg))
+	if( str_isint(arg) || str_isreal(arg,true) )
 	// process by cluster/proc #
 	{
 		c = strtol(arg, &tmp, 10);
@@ -572,7 +566,7 @@ procArg(const char* arg)
 			CondorError errstack;
 			constraint.sprintf( "%s == %d", ATTR_CLUSTER_ID, c );
 			if( doWorkByConstraint(constraint.Value(), &errstack) ) {
-				fprintf( old_messages ? stderr : stdout, 
+				fprintf( stdout, 
 						 "Cluster %d %s.\n", c,
 						 (mode == JA_REMOVE_JOBS) ?
 						 "has been marked for removal" :
@@ -581,9 +575,12 @@ procArg(const char* arg)
 						 actionWord(mode,true) );
 			} else {
 				fprintf( stderr, "%s\n", errstack.getFullText(true) );
-				fprintf( stderr, 
+				if (had_error)
+				{
+					fprintf( stderr, 
 						 "Couldn't find/%s all jobs in cluster %d.\n",
 						 actionWord(mode,false), c );
+				}
 			}
 			return;
 		}
@@ -593,7 +590,7 @@ procArg(const char* arg)
 			if(p < 0)
 			{
 				fprintf( stderr, "Invalid proc # from %s.\n", arg);
-				had_error = 1;
+				had_error = true;
 				return;
 			}
 			if(*tmp == '\0')
@@ -609,9 +606,8 @@ procArg(const char* arg)
 		fprintf( stderr, "Warning: unrecognized \"%s\" skipped.\n", arg );
 		return;
 	}
-	else if(isalpha(*arg))
 	// process by user name
-	{
+	else {
 		CondorError errstack;
 		constraint.sprintf("%s == \"%s\"", ATTR_OWNER, arg );
 		if( doWorkByConstraint(constraint.Value(), &errstack) ) {
@@ -623,12 +619,13 @@ procArg(const char* arg)
 					 actionWord(mode,true) );
 		} else {
 			fprintf( stderr, "%s\n", errstack.getFullText(true) );
-			fprintf( stderr, 
+			if (had_error)
+			{
+				fprintf( stderr, 
 					 "Couldn't find/%s all of user %s's job(s).\n",
 					 actionWord(mode,false), arg );
+			}
 		}
-	} else {
-		fprintf( stderr, "Warning: unrecognized \"%s\" skipped\n", arg );
 	}
 }
 
@@ -697,8 +694,11 @@ handleAll()
 				 actionWord(mode,true) );
 	} else {
 		fprintf( stderr, "%s\n", errstack.getFullText(true) );
-		fprintf( stderr, "Could not %s all jobs.\n",
+		if (had_error)
+		{
+			fprintf( stderr, "Could not %s all jobs.\n",
 				 actionWord(mode,false) );
+		}
 	}
 }
 
@@ -722,91 +722,12 @@ handleConstraints( void )
 
 	} else {
 		fprintf( stderr, "%s\n", errstack.getFullText(true) );
-		fprintf( stderr, 
+		if (had_error)
+		{
+			fprintf( stderr, 
 				 "Couldn't find/%s all jobs matching constraint %s\n",
 				 actionWord(mode,false), tmp );
-	}
-}
-
-
-void
-printOldFailure( PROC_ID job_id )
-{
-	fprintf( stderr, "Couldn't find/%s job %d.%d.\n",
-			 actionWord(mode,false), job_id.cluster, job_id.proc );
-}
-
-
-void
-printOldMessage( PROC_ID job_id, action_result_t result )
-{
-	switch( result ) {
-	case AR_SUCCESS:
-		fprintf( stdout, "Job %d.%d %s.\n", 
-				 job_id.cluster, job_id.proc, 
-				 (mode == JA_REMOVE_JOBS) ?
-				 "marked for removal" :
-				 (mode == JA_REMOVE_X_JOBS) ?
-				 "removed locally (remote state unknown)" :
-				 actionWord(mode,true) );
-		break;
-
-	case AR_NOT_FOUND:
-		if( mode==JA_RELEASE_JOBS ) {
-			fprintf( stderr, "Couldn't access job queue for %d.%d\n", 
-					 job_id.cluster, job_id.proc );
-			break;
-		} 
-		printOldFailure( job_id );
-		break;
-
-	case AR_PERMISSION_DENIED: 
-		printOldFailure( job_id );
-		break;
-
-	case AR_BAD_STATUS:
-		if( mode == JA_RELEASE_JOBS ) {
-			fprintf( stderr, "Job %d.%d not held to be released\n", 
-					 job_id.cluster, job_id.proc );
-			break;
-		} 
-		printOldFailure( job_id );
-		break;
-
-	case AR_ALREADY_DONE:
-			// The old tool allowed you to repeatedly hold or remove
-			// the same job over and over again...
-		fprintf( stdout, "Job %d.%d %s.\n", 
-				 job_id.cluster, job_id.proc, 
-				 (mode == JA_REMOVE_JOBS) ?
-				 "marked for removal" :
-				 (mode == JA_REMOVE_X_JOBS) ?
-				 "removed locally (remote state unknown)" :
-				 actionWord(mode,true) );
-		break;
-
-	case AR_ERROR:
-		printOldFailure( job_id );
-		break;
-	}
-}
-
-
-void
-printOldMessages( ClassAd* result_ad, StringList* ids )
-{
-	char* tmp;
-	PROC_ID job_id;
-	action_result_t result;
-
-	JobActionResults results;
-	results.readResults( result_ad );
-
-	ids->rewind();
-	while( (tmp = ids->next()) ) {
-		job_id = getProcByString( tmp );
-		result = results.getResult( job_id );
-		printOldMessage( job_id, result );
+		}
 	}
 }
 
