@@ -46,11 +46,14 @@ class QmgmtPeer {
 		bool set(const struct sockaddr_in *sock, const char *fqOwnerAndDomain);
 		void unset();
 
+		bool setEffectiveOwner(char const *o);
+
 		ReliSock *getReliSock() const { return sock; };
 
 		const char *endpoint_ip_str() const;
 		const struct sockaddr_in *endpoint() const;
 		const char* getOwner() const;
+		const char* getRealOwner() const;
 		const char* getFullyQualifiedUser() const;
 		int isAuthenticated() const;
 
@@ -63,7 +66,7 @@ class QmgmtPeer {
 		ReliSock *sock; 
 
 		Transaction *transaction;
-		int next_proc_num, active_cluster_num, old_cluster_num;
+		int next_proc_num, active_cluster_num;
 		time_t xact_start_time;
 
 	private:
@@ -82,20 +85,31 @@ time_t GetOriginalJobQueueBirthdate();
 void DestroyJobQueue( void );
 int handle_q(Service *, int, Stream *sock);
 void dirtyJobQueue( void );
+void SendDirtyJobAdNotification(char *job_id_str);
+
 bool isQueueSuperUser( const char* user );
+
+// Verify that the user issuing a command (test_owner) is authorized
+// to modify the given job.  In addition to everything OwnerCheck2()
+// does, this also calls IPVerify to check for WRITE authorization.
 bool OwnerCheck( ClassAd *ad, const char *test_owner );
-bool OwnerCheck2( ClassAd *ad, const char *test_owner );
+
+// Verify that the user issuing a command (test_owner) is authorized
+// to modify the given job.  Either ad or job_owner should be given
+// but not both.  If job_owner is NULL, the owner is looked up in the ad.
+bool OwnerCheck2( ClassAd *ad, const char *test_owner, char const *job_owner=NULL );
+
 bool BuildPrioRecArray(bool no_match_found=false);
 void DirtyPrioRecArray();
-extern ClassAd *dollarDollarExpand(int cid, int pid, ClassAd *job, ClassAd *res);
+extern ClassAd *dollarDollarExpand(int cid, int pid, ClassAd *job, ClassAd *res, bool persist_expansions);
+bool rewriteSpooledJobAd(ClassAd *job_ad, int cluster, int proc, bool modify_ad);
+ClassAd* GetJobAd(int cluster_id, int proc_id, bool expStartdAd, bool persist_expansions);
 ClassAd* GetNextJobByCluster( int, int );
 
 int get_myproxy_password_handler(Service *, int, Stream *sock);
 
 QmgmtPeer* getQmgmtConnectionInfo();
 bool OwnerCheck(int,int);
-bool OwnerCheck(ClassAd *, const char *);
-bool OwnerCheck2(ClassAd *, const char *);
 
 // priority records
 extern prio_rec *PrioRec;
@@ -103,9 +117,10 @@ extern int N_PrioRecs;
 extern HashTable<int,int> *PrioRecAutoClusterRejected;
 extern int grow_prio_recs(int);
 
-extern void	FindRunnableJob(PROC_ID & jobid, const ClassAd* my_match_ad, 
-					 char * user);
+extern void	FindRunnableJob(PROC_ID & jobid, ClassAd* my_match_ad, char const * user);
 extern int Runnable(PROC_ID*);
 extern int Runnable(ClassAd*);
 
+extern class ForkWork schedd_forker;
+
 #endif /* _QMGMT_H */
