@@ -36,24 +36,24 @@ SetAttributeInt(int cl, int pr, const char *name, int val, SetAttributeFlags_t f
 	char buf[100];
 	int rval;
 
-	sprintf(buf,"%d",val);
+	snprintf(buf,100,"%d",val);
 	rval = SetAttribute(cl,pr,name,buf,flags);
 	return(rval);
 }
 
 int
-SetAttributeFloat(int cl, int pr, const char *name, float val)
+SetAttributeFloat(int cl, int pr, const char *name, float val, SetAttributeFlags_t flags )
 {
 	char buf[100];
 	int rval;
 
-	sprintf(buf,"%f",val);
-	rval = SetAttribute(cl,pr,name,buf);
+	snprintf(buf,100,"%f",val);
+	rval = SetAttribute(cl,pr,name,buf,flags);
 	return(rval);
 }
 
 int
-SetAttributeString(int cl, int pr, const char *name, const char *val)
+SetAttributeString(int cl, int pr, const char *name, const char *val, SetAttributeFlags_t flags )
 {
 	MyString buf;
 	MyString escape_buf;
@@ -64,35 +64,36 @@ SetAttributeString(int cl, int pr, const char *name, const char *val)
 	buf += '"';
 	buf +=  val;
 	buf += '"';
-	rval = SetAttribute(cl,pr,name,buf.Value());
+	rval = SetAttribute(cl,pr,name,buf.Value(),flags);
 	return(rval);
 }
 
 int
-SetAttributeIntByConstraint(const char *con, const char *name, int val)
+SetAttributeIntByConstraint(const char *con, const char *name, int val, SetAttributeFlags_t flags)
 {
 	char buf[100];
 	int rval;
 
-	sprintf(buf,"%d",val);
-	rval = SetAttributeByConstraint(con,name,buf);
+	snprintf(buf,100,"%d",val);
+	rval = SetAttributeByConstraint(con,name,buf, flags);
 	return(rval);
 }
 
 int
-SetAttributeFloatByConstraint(const char *con, const char *name, float val)
+SetAttributeFloatByConstraint(const char *con, const char *name, float val, SetAttributeFlags_t flags)
 {
 	char buf[100];
 	int rval;
 
-	sprintf(buf,"%f",val);
-	rval = SetAttributeByConstraint(con,name,buf);
+	snprintf(buf,100,"%f",val);
+	rval = SetAttributeByConstraint(con,name,buf, flags);
 	return(rval);
 }
 
 int
 SetAttributeStringByConstraint(const char *con, const char *name,
-							   const char *val)
+							 const char *val,
+							 SetAttributeFlags_t flags)
 {
 	MyString buf;
 	int rval;
@@ -100,7 +101,7 @@ SetAttributeStringByConstraint(const char *con, const char *name,
 	buf += '"';
 	buf +=  val;
 	buf += '"';
-	rval = SetAttributeByConstraint(con,name,buf.Value());
+	rval = SetAttributeByConstraint(con,name,buf.Value(), flags);
 	return(rval);
 }
 
