@@ -38,7 +38,7 @@ extern ReliSock *qmgmt_sock;
 int terrno;
 
 int
-InitializeConnection( const char *owner, const char * /* domain */ )
+InitializeConnection( const char * /*owner*/, const char * /* domain */ )
 {
 	CurrentSysCall = CONDOR_InitializeConnection;
 
@@ -49,7 +49,7 @@ InitializeConnection( const char *owner, const char * /* domain */ )
 }
 
 int
-InitializeReadOnlyConnection( const char *owner )
+InitializeReadOnlyConnection( const char * /*owner*/ )
 {
 	CurrentSysCall = CONDOR_InitializeReadOnlyConnection;
 
@@ -59,6 +59,34 @@ InitializeReadOnlyConnection( const char *owner )
 	return( 0 );
 }
 
+int
+QmgmtSetEffectiveOwner(char const *o)
+{
+	int rval = -1;
+
+	CurrentSysCall = CONDOR_SetEffectiveOwner;
+
+	qmgmt_sock->encode();
+	assert( qmgmt_sock->code(CurrentSysCall) );
+	if( !o ) {
+		o = "";
+	}
+	assert( qmgmt_sock->put(o) );
+	assert( qmgmt_sock->end_of_message() );
+
+	qmgmt_sock->decode();
+	assert( qmgmt_sock->code(rval) );
+	if( rval < 0 ) {
+		assert( qmgmt_sock->code(terrno) );
+		assert( qmgmt_sock->end_of_message() );
+		errno = terrno;
+		return rval;
+	}
+	assert( qmgmt_sock->end_of_message() );
+
+	return 0;
+}
+
 int
 NewCluster()
 {
@@ -192,17 +220,23 @@ DestroyClusterByConstraint( char *constraint )
 
 
 int
-SetAttributeByConstraint( char const *constraint, char const *attr_name, char const *attr_value )
+SetAttributeByConstraint( char const *constraint, char const *attr_name, char const *attr_value, SetAttributeFlags_t flags )
 {
 	int	rval = -1;
 
 		CurrentSysCall = CONDOR_SetAttributeByConstraint;
+		if( flags ) {
+			CurrentSysCall = CONDOR_SetAttributeByConstraint2;
+		}
 
 		qmgmt_sock->encode();
 		assert( qmgmt_sock->code(CurrentSysCall) );
 		assert( qmgmt_sock->put(constraint) );
 		assert( qmgmt_sock->put(attr_value) );
 		assert( qmgmt_sock->put(attr_name) );
+		if( flags ) {
+			assert( qmgmt_sock->code(flags) );
+		}
 		assert( qmgmt_sock->end_of_message() );
 
 		qmgmt_sock->decode();
@@ -243,15 +277,20 @@ SetAttribute( int cluster_id, int proc_id, char const *attr_name, char const *at
 		}
 		assert( qmgmt_sock->end_of_message() );
 
-		qmgmt_sock->decode();
-		assert( qmgmt_sock->code(rval) );
-		if( rval < 0 ) {
-			assert( qmgmt_sock->code(terrno) );
+		if( flags & SetAttribute_NoAck ) {
+			rval = 0;
+		}
+		else {
+			qmgmt_sock->decode();
+			assert( qmgmt_sock->code(rval) );
+			if( rval < 0 ) {
+				assert( qmgmt_sock->code(terrno) );
+				assert( qmgmt_sock->end_of_message() );
+				errno = terrno;
+				return rval;
+			}
 			assert( qmgmt_sock->end_of_message() );
-			errno = terrno;
-			return rval;
 		}
-		assert( qmgmt_sock->end_of_message() );
 
 	return rval;
 }
@@ -347,30 +386,38 @@ AbortTransaction()
 }
 
 int
-CloseConnection()
+RemoteCommitTransaction(SetAttributeFlags_t flags)
 {
 	int	rval = -1;
 
-		CurrentSysCall = CONDOR_CloseConnection;
+	if( flags == 0 ) {
+			// for compatibility with schedd's from before 7.5.0
+		CurrentSysCall = CONDOR_CommitTransactionNoFlags;
+	}
+	else {
+		CurrentSysCall = CONDOR_CommitTransaction;
+	}
 
-		qmgmt_sock->encode();
-		assert( qmgmt_sock->code(CurrentSysCall) );
-		assert( qmgmt_sock->end_of_message() );
+	qmgmt_sock->encode();
+	assert( qmgmt_sock->code(CurrentSysCall) );
+	if( CurrentSysCall == CONDOR_CommitTransaction ) {
+		assert( qmgmt_sock->put((int)flags) );
+	}
+	assert( qmgmt_sock->end_of_message() );
 
-		qmgmt_sock->decode();
-		assert( qmgmt_sock->code(rval) );
-		if( rval < 0 ) {
-			assert( qmgmt_sock->code(terrno) );
-			assert( qmgmt_sock->end_of_message() );
-			errno = terrno;
-			return rval;
-		}
+	qmgmt_sock->decode();
+	assert( qmgmt_sock->code(rval) );
+	if( rval < 0 ) {
+		assert( qmgmt_sock->code(terrno) );
 		assert( qmgmt_sock->end_of_message() );
+		errno = terrno;
+		return rval;
+	}
+	assert( qmgmt_sock->end_of_message() );
 
 	return rval;
 }
 
-
 int
 GetAttributeFloat( int cluster_id, int proc_id, char *attr_name, float *value )
 {
@@ -429,40 +476,13 @@ GetAttributeInt( int cluster_id, int proc_id, char const *attr_name, int *value
 }
 
 
-int
-GetAttributeString( int cluster_id, int proc_id, char *attr_name, char *value )
-{
-	int	rval = -1;
-
-		CurrentSysCall = CONDOR_GetAttributeString;
-
-		qmgmt_sock->encode();
-		assert( qmgmt_sock->code(CurrentSysCall) );
-		assert( qmgmt_sock->code(cluster_id) );
-		assert( qmgmt_sock->code(proc_id) );
-		assert( qmgmt_sock->code(attr_name) );
-		assert( qmgmt_sock->end_of_message() );
-
-		qmgmt_sock->decode();
-		assert( qmgmt_sock->code(rval) );
-		if( rval < 0 ) {
-			assert( qmgmt_sock->code(terrno) );
-			assert( qmgmt_sock->end_of_message() );
-			errno = terrno;
-			return rval;
-		}
-		assert( qmgmt_sock->code(value) );
-		assert( qmgmt_sock->end_of_message() );
-
-	return rval;
-}
-
-
 int
 GetAttributeStringNew( int cluster_id, int proc_id, char const *attr_name, char **val )
 {
 	int	rval = -1;
 
+	*val = NULL;
+
 	CurrentSysCall = CONDOR_GetAttributeString;
 
 	qmgmt_sock->encode();
@@ -475,13 +495,11 @@ GetAttributeStringNew( int cluster_id, int proc_id, char const *attr_name, char
 	qmgmt_sock->decode();
 	assert( qmgmt_sock->code(rval) );
 	if( rval < 0 ) {
-		*val = (char *) calloc(1, sizeof(char));
 		assert( qmgmt_sock->code(terrno) );
 		assert( qmgmt_sock->end_of_message() );
 		errno = terrno;
 		return rval;
 	}
-	*val = NULL;
 	assert( qmgmt_sock->code(*val) );
 	assert( qmgmt_sock->end_of_message() );
 
@@ -490,29 +508,64 @@ GetAttributeStringNew( int cluster_id, int proc_id, char const *attr_name, char
 
 
 int
-GetAttributeExpr( int cluster_id, int proc_id, char const *attr_name, char *value )
+GetAttributeExprNew( int cluster_id, int proc_id, char const *attr_name, char **value )
 {
 	int	rval = -1;
 
-		CurrentSysCall = CONDOR_GetAttributeExpr;
+	CurrentSysCall = CONDOR_GetAttributeExpr;
 
-		qmgmt_sock->encode();
-		assert( qmgmt_sock->code(CurrentSysCall) );
-		assert( qmgmt_sock->code(cluster_id) );
-		assert( qmgmt_sock->code(proc_id) );
-		assert( qmgmt_sock->put(attr_name) );
-		assert( qmgmt_sock->end_of_message() );
+	*value = NULL;
 
-		qmgmt_sock->decode();
-		assert( qmgmt_sock->code(rval) );
-		if( rval < 0 ) {
-			assert( qmgmt_sock->code(terrno) );
-			assert( qmgmt_sock->end_of_message() );
-			errno = terrno;
-			return rval;
-		}
-		assert( qmgmt_sock->code(value) );
+	qmgmt_sock->encode();
+	assert( qmgmt_sock->code(CurrentSysCall) );
+	assert( qmgmt_sock->code(cluster_id) );
+	assert( qmgmt_sock->code(proc_id) );
+	assert( qmgmt_sock->put(attr_name) );
+	assert( qmgmt_sock->end_of_message() );
+
+	qmgmt_sock->decode();
+	assert( qmgmt_sock->code(rval) );
+	if( rval < 0 ) {
+		assert( qmgmt_sock->code(terrno) );
 		assert( qmgmt_sock->end_of_message() );
+		errno = terrno;
+		return rval;
+	}
+	assert( qmgmt_sock->code(*value) );
+	assert( qmgmt_sock->end_of_message() );
+
+	return rval;
+}
+
+
+int
+GetDirtyAttributes(int cluster_id, int proc_id, ClassAd *updated_attrs)
+{
+	int	rval = -1;
+	MyString errs;
+
+	CurrentSysCall = CONDOR_GetDirtyAttributes;
+
+	qmgmt_sock->encode();
+	assert( qmgmt_sock->code(CurrentSysCall) );
+	assert( qmgmt_sock->code(cluster_id) );
+	assert( qmgmt_sock->code(proc_id) );
+	assert( qmgmt_sock->end_of_message() );
+
+	qmgmt_sock->decode();
+	assert( qmgmt_sock->code(rval) );
+	if( rval < 0 ) {
+		assert( qmgmt_sock->code(terrno) );
+		assert( qmgmt_sock->end_of_message() );
+		errno = terrno;
+		return rval;
+	}
+
+	if ( !(updated_attrs->initFromStream(*qmgmt_sock)) ) {
+		errno = ETIMEDOUT;
+		return 0;
+	}
+	assert( qmgmt_sock->end_of_message() != 0 );
 
 	return rval;
 }
@@ -570,6 +623,31 @@ SendSpoolFile( char const *filename )
 	return rval;
 }
 
+int
+SendSpoolFileIfNeeded( ClassAd& ad )
+{
+	int	rval = -1;
+
+	CurrentSysCall = CONDOR_SendSpoolFileIfNeeded;
+
+	qmgmt_sock->encode();
+	assert( qmgmt_sock->code(CurrentSysCall) );
+	assert( ad.put(*qmgmt_sock) );
+	assert( qmgmt_sock->end_of_message() );
+
+	qmgmt_sock->decode();
+	assert( qmgmt_sock->code(rval) );
+	if( rval < 0 ) {
+		assert( qmgmt_sock->code(terrno) );
+		assert( qmgmt_sock->end_of_message() );
+		errno = terrno;
+		return rval;
+	}
+	assert( qmgmt_sock->end_of_message() );
+
+	return rval;
+}
+
 int
 CloseSocket()
 {
@@ -589,7 +667,7 @@ CloseSocket()
 #define assert(x) if (!(x)) { errno = ETIMEDOUT; return NULL; }
 
 ClassAd *
-GetJobAd( int cluster_id, int proc_id, bool /*expStartdAttrs*/ )
+GetJobAd( int cluster_id, int proc_id, bool /*expStartdAttrs*/, bool /*persist_expansions*/ )
 {
 	int	rval = -1;
 
@@ -771,3 +849,38 @@ GetAllJobsByConstraint( char const *constraint, char const *projection, ClassAdL
 {
 	GetAllJobsByConstraint_imp(constraint,projection,list);
 }
+
+ClassAd *
+GetNextDirtyJobByConstraint( char const *constraint, int initScan )
+{
+	int	rval = -1;
+
+	CurrentSysCall = CONDOR_GetNextDirtyJobByConstraint;
+
+	qmgmt_sock->encode();
+	assert( qmgmt_sock->code(CurrentSysCall) );
+	assert( qmgmt_sock->code(initScan) );
+	assert( qmgmt_sock->put(constraint) );
+	assert( qmgmt_sock->end_of_message() );
+
+	qmgmt_sock->decode();
+	assert( qmgmt_sock->code(rval) );
+	if( rval < 0 ) {
+		assert( qmgmt_sock->code(terrno) );
+		assert( qmgmt_sock->end_of_message() );
+		errno = terrno;
+		return NULL;
+	}
+
+	ClassAd *ad = new ClassAd;
+
+	if ( ! (ad->initFromStream(*qmgmt_sock)) ) {
+		delete ad;
+		errno = ETIMEDOUT;
+		return NULL;
+	}
+
+	assert( qmgmt_sock->end_of_message() );
+
+	return ad;
+}
