@@ -25,26 +25,33 @@
 #include "condor_attributes.h"
 #include "condor_classad.h"
 #include "internet.h"
+#include "classad_merge.h"
 
 
 #include "qmgr_job_updater.h"
 #include "condor_qmgr.h"
+#include "dc_schedd.h"
 
 
-QmgrJobUpdater::QmgrJobUpdater( ClassAd* job, const char* schedd_address )
+QmgrJobUpdater::QmgrJobUpdater( ClassAd* job, const char* schedd_address,
+	const char *schedd_version ) : common_job_queue_attrs(0),
+	hold_job_queue_attrs(0),
+	evict_job_queue_attrs(0),
+	remove_job_queue_attrs(0),
+	requeue_job_queue_attrs(0),
+	terminate_job_queue_attrs(0),
+	checkpoint_job_queue_attrs(0),
+	m_pull_attrs(0),
+	job_ad(job), // we do *NOT* want to make our own copy of this ad
+	schedd_addr(schedd_address?strdup(schedd_address):0),
+	schedd_ver(schedd_version?strdup(schedd_version):0),
+	cluster(-1), proc(-1),
+	q_update_tid(-1) 
 {
-	q_update_tid = -1;
-
 	if( ! is_valid_sinful(schedd_address) ) {
 		EXCEPT( "schedd_addr not specified with valid address (%s)",
 				schedd_address );
 	}
-	schedd_addr = strdup( schedd_address );
-
-		// we do *NOT* want to make our own copy of this ad
-	job_ad = job;
-
-	cluster = proc = -1;
 	if( !job_ad->LookupInteger(ATTR_CLUSTER_ID, cluster)) {
 		EXCEPT("Job ad doesn't contain an %s attribute.", ATTR_CLUSTER_ID);
 	}
@@ -53,22 +60,19 @@ QmgrJobUpdater::QmgrJobUpdater( ClassAd* job, const char* schedd_address )
 		EXCEPT("Job ad doesn't contain an %s attribute.", ATTR_PROC_ID);
 	}
 
-	common_job_queue_attrs = NULL;
-	hold_job_queue_attrs = NULL;
-	evict_job_queue_attrs = NULL;
-	remove_job_queue_attrs = NULL;
-	requeue_job_queue_attrs = NULL;
-	terminate_job_queue_attrs = NULL;
-	checkpoint_job_queue_attrs = NULL;
+	// It is safest to read this attribute now, before the ad is
+	// potentially modified.  If it is missing, then SetEffectiveOwner
+	// will just be a no-op.
+	job_ad->LookupString(ATTR_OWNER, m_owner);
+
 	initJobQueueAttrLists();
 
-		// finally, clear all the dirty bits on this jobAd, so we only
-		// update the queue with things that have changed after this
-		// point. 
+	// finally, clear all the dirty bits on this jobAd, so we only
+	// update the queue with things that have changed after this
+	// point. 
 	job_ad->ClearAllDirtyFlags();
 }
 
-
 QmgrJobUpdater::~QmgrJobUpdater()
 {
 	if( q_update_tid >= 0 ) {
@@ -76,6 +80,7 @@ QmgrJobUpdater::~QmgrJobUpdater()
 		q_update_tid = -1;
 	}
 	if( schedd_addr ) { free(schedd_addr); }
+	if (schedd_ver)   {free(schedd_ver); }
 	if( common_job_queue_attrs ) { delete common_job_queue_attrs; }
 	if( hold_job_queue_attrs ) { delete hold_job_queue_attrs; }
 	if( evict_job_queue_attrs ) { delete evict_job_queue_attrs; }
@@ -83,6 +88,7 @@ QmgrJobUpdater::~QmgrJobUpdater()
 	if( requeue_job_queue_attrs ) { delete requeue_job_queue_attrs; }
 	if( terminate_job_queue_attrs ) { delete terminate_job_queue_attrs; }
 	if( checkpoint_job_queue_attrs ) { delete checkpoint_job_queue_attrs; }
+	delete m_pull_attrs;
 }
 
 
@@ -96,19 +102,24 @@ QmgrJobUpdater::initJobQueueAttrLists( void )
 	if( terminate_job_queue_attrs ) { delete terminate_job_queue_attrs; }
 	if( common_job_queue_attrs ) { delete common_job_queue_attrs; }
 	if( checkpoint_job_queue_attrs ) { delete checkpoint_job_queue_attrs; }
+	delete m_pull_attrs;
 
 	common_job_queue_attrs = new StringList();
 	common_job_queue_attrs->insert( ATTR_IMAGE_SIZE );
+	common_job_queue_attrs->insert( ATTR_RESIDENT_SET_SIZE );
 	common_job_queue_attrs->insert( ATTR_DISK_USAGE );
 	common_job_queue_attrs->insert( ATTR_JOB_REMOTE_SYS_CPU );
 	common_job_queue_attrs->insert( ATTR_JOB_REMOTE_USER_CPU );
 	common_job_queue_attrs->insert( ATTR_TOTAL_SUSPENSIONS );
 	common_job_queue_attrs->insert( ATTR_CUMULATIVE_SUSPENSION_TIME );
+	common_job_queue_attrs->insert( ATTR_COMMITTED_SUSPENSION_TIME );
 	common_job_queue_attrs->insert( ATTR_LAST_SUSPENSION_TIME );
 	common_job_queue_attrs->insert( ATTR_BYTES_SENT );
 	common_job_queue_attrs->insert( ATTR_BYTES_RECVD );
 	common_job_queue_attrs->insert( ATTR_LAST_JOB_LEASE_RENEWAL );
 	common_job_queue_attrs->insert( ATTR_JOB_COMMITTED_TIME );
+	common_job_queue_attrs->insert( ATTR_COMMITTED_SLOT_TIME );
+	common_job_queue_attrs->insert( ATTR_DELEGATED_PROXY_EXPIRATION );
 
 	hold_job_queue_attrs = new StringList();
 	hold_job_queue_attrs->insert( ATTR_HOLD_REASON );
@@ -136,6 +147,7 @@ QmgrJobUpdater::initJobQueueAttrLists( void )
 	terminate_job_queue_attrs->insert( ATTR_EXCEPTION_NAME );
 	terminate_job_queue_attrs->insert( ATTR_TERMINATION_PENDING );
 	terminate_job_queue_attrs->insert( ATTR_JOB_CORE_FILENAME );
+	terminate_job_queue_attrs->insert( ATTR_SPOOLED_OUTPUT_FILES );
 
 	checkpoint_job_queue_attrs = new StringList();
 	checkpoint_job_queue_attrs->insert( ATTR_NUM_CKPTS );
@@ -144,6 +156,11 @@ QmgrJobUpdater::initJobQueueAttrLists( void )
 	checkpoint_job_queue_attrs->insert( ATTR_CKPT_OPSYS );
 	checkpoint_job_queue_attrs->insert( ATTR_VM_CKPT_MAC );
 	checkpoint_job_queue_attrs->insert( ATTR_VM_CKPT_IP );
+
+	m_pull_attrs = new StringList();
+	if ( job_ad->LookupExpr( ATTR_TIMER_REMOVE_CHECK ) ) {
+		m_pull_attrs->insert( ATTR_TIMER_REMOVE_CHECK );
+	}
 }
 
 
@@ -168,6 +185,19 @@ QmgrJobUpdater::startUpdateTimer( void )
 }
 
 
+void
+QmgrJobUpdater::resetUpdateTimer( void )
+{
+	if ( q_update_tid < 0 ) {
+		startUpdateTimer();
+		return;
+	}
+
+	int q_interval = param_integer( "SHADOW_QUEUE_UPDATE_INTERVAL", 15*60 );
+	daemonCore->Reset_Timer( q_update_tid, 0, q_interval );
+}
+
+
 /*
   WARNING: This method is BAD NEWS for schedd scalability.  We make a
   whole new qmgmt connection for *every* attribute we update.  Worse
@@ -182,10 +212,11 @@ QmgrJobUpdater::startUpdateTimer( void )
   modify it to be less potentially harmful for schedd scalability.
 */
 bool
-QmgrJobUpdater::updateAttr( const char *name, const char *expr, bool updateMaster )
+QmgrJobUpdater::updateAttr( const char *name, const char *expr, bool updateMaster, bool log )
 {
 	bool result;
 	MyString err_msg;
+	SetAttributeFlags_t flags=0;
 
 	dprintf( D_FULLDEBUG, "QmgrJobUpdater::updateAttr: %s = %s\n",
 			 name, expr );
@@ -198,8 +229,12 @@ QmgrJobUpdater::updateAttr( const char *name, const char *expr, bool updateMaste
 	if (updateMaster) {
 		p = 0;
 	}
-	if( ConnectQ(schedd_addr,SHADOW_QMGMT_TIMEOUT) ) {
-		if( SetAttribute(cluster,p,name,expr) < 0 ) {
+
+	if (log) {
+		flags = SHOULDLOG;
+	}
+	if( ConnectQ(schedd_addr,SHADOW_QMGMT_TIMEOUT,false,NULL,m_owner.Value(),schedd_ver) ) {
+		if( SetAttribute(cluster,p,name,expr,flags) < 0 ) {
 			err_msg = "SetAttribute() failed";
 			result = FALSE;
 		} else {
@@ -220,34 +255,23 @@ QmgrJobUpdater::updateAttr( const char *name, const char *expr, bool updateMaste
 
 
 bool
-QmgrJobUpdater::updateAttr( const char *name, int value, bool updateMaster )
+QmgrJobUpdater::updateAttr( const char *name, int value, bool updateMaster, bool log )
 {
 	MyString buf;
     buf.sprintf("%d", value);
-	return updateAttr(name, buf.Value(), updateMaster);
+	return updateAttr(name, buf.Value(), updateMaster, log);
 }
 
 
 bool
-QmgrJobUpdater::updateJob( update_t type )
+QmgrJobUpdater::updateJob( update_t type, SetAttributeFlags_t commit_flags )
 {
 	ExprTree* tree = NULL;
 	bool is_connected = false;
 	bool had_error = false;
-	bool final_update = false;
-	static bool checked_for_history = false;
-	static bool has_history = false;
-	char* name;
+	const char* name;
+	char *value = NULL;
 	
-	if( ! checked_for_history ) {
-		char* history = param( "HISTORY" );
-		if( history ) {
-			has_history = true;
-			free( history );
-		}
-		checked_for_history = true;
-	}
-
 	StringList* job_queue_attrs = NULL;
 	switch( type ) {
 	case U_HOLD:
@@ -255,14 +279,12 @@ QmgrJobUpdater::updateJob( update_t type )
 		break;
 	case U_REMOVE:
 		job_queue_attrs = remove_job_queue_attrs;
-		final_update = true;
 		break;
 	case U_REQUEUE:
 		job_queue_attrs = requeue_job_queue_attrs;
 		break;
 	case U_TERMINATE:
 		job_queue_attrs = terminate_job_queue_attrs;
-		final_update = true;
 		break;
 	case U_EVICT:
 		job_queue_attrs = evict_job_queue_attrs;
@@ -276,22 +298,12 @@ QmgrJobUpdater::updateJob( update_t type )
 	default:
 		EXCEPT( "QmgrJobUpdater::updateJob: Unknown update type (%d)!", type );
 	}
-	if( final_update && ! has_history ) {
-			// there's no history file on this machine, and this job
-			// is about to leave the queue.  there's no reason to send
-			// this stuff to the schedd, since it's all about to be
-			// flushed, anyway.
-		dprintf( D_FULLDEBUG, "QmgrJobUpdater::updateJob: job leaving "
-				 "queue and schedd has no history file, aborting update\n" );
-		return true;
-	}
 
 	job_ad->ResetExpr();
-	while( (tree = job_ad->NextDirtyExpr()) ) {
-		if( tree->invisible ) {
-			continue;
-		}
-		name = ((Variable*)tree->LArg())->Name();
+	while( job_ad->NextDirtyExpr(name, tree) ) {
+		// There used to be a check for tree->invisible here,
+		// but there are no codepaths that reach here with
+		// private attributes set to invisible.
 
 			// If we have the lists of attributes we care about and
 			// this attribute is in one of the lists, actually do the
@@ -304,18 +316,39 @@ QmgrJobUpdater::updateJob( update_t type )
 			 job_queue_attrs->contains_anycase(name)) ) {
 
 			if( ! is_connected ) {
-				if( ! ConnectQ(schedd_addr, SHADOW_QMGMT_TIMEOUT) ) {
+				if( ! ConnectQ(schedd_addr, SHADOW_QMGMT_TIMEOUT, false, NULL, m_owner.Value(),schedd_ver) ) {
 					return false;
 				}
 				is_connected = true;
 			}
-			if( ! updateExprTree(tree) ) {
+			if( ! updateExprTree(name, tree) ) {
 				had_error = true;
 			}
 		}
 	}
+	m_pull_attrs->rewind();
+	while ( (name = m_pull_attrs->next()) ) {
+		if ( !is_connected ) {
+			if ( !ConnectQ( schedd_addr, SHADOW_QMGMT_TIMEOUT, true, NULL, NULL, schedd_ver ) ) {
+				return false;
+			}
+			is_connected = true;
+		}
+		if ( GetAttributeExprNew( cluster, proc, name, &value ) < 0 ) {
+			had_error = true;
+		} else {
+			job_ad->AssignExpr( name, value );
+		}
+		free( value );
+	}
 	if( is_connected ) {
-		DisconnectQ(NULL);
+		if( !had_error) {
+			if( RemoteCommitTransaction(commit_flags)!=0 ) {
+				dprintf(D_ALWAYS,"Failed to commit job update.\n");
+				had_error = true;
+			}
+		}
+		DisconnectQ(NULL,false);
 	} 
 	if( had_error ) {
 		return false;
@@ -325,31 +358,64 @@ QmgrJobUpdater::updateJob( update_t type )
 }
 
 
-void
-QmgrJobUpdater::periodicUpdateQ( void )
+bool
+QmgrJobUpdater::retrieveJobUpdates( void )
 {
-	updateJob( U_PERIODIC );	
+	ClassAd updates;
+	CondorError errstack;
+	StringList job_ids;
+	char id_str[PROC_ID_STR_BUFLEN];
+	MyString error;
+
+	ProcIdToStr(cluster, proc, id_str);
+	job_ids.insert(id_str);
+
+	if ( !ConnectQ( schedd_addr, SHADOW_QMGMT_TIMEOUT, false ) ) {
+		return false;
+	}
+	if ( GetDirtyAttributes( cluster, proc, &updates ) < 0 ) {
+		DisconnectQ(NULL,false);
+		return false;
+	}
+	DisconnectQ( NULL, false );
+
+	dprintf( D_FULLDEBUG, "Retrieved updated attributes from schedd\n" );
+	updates.dPrint( D_JOB );
+	MergeClassAds( job_ad, &updates, true );
+
+	DCSchedd schedd( schedd_addr );
+	if ( schedd.clearDirtyAttrs( &job_ids, &errstack ) == NULL ) {
+		dprintf( D_ALWAYS, "clearDirtyAttrs() failed: %s\n", errstack.getFullText( ) );
+		return false;
+	}
+	return true;
 }
 
 
+void
+QmgrJobUpdater::periodicUpdateQ( void )
+{
+		// For performance, use a NONDURABLE transaction.
+	updateJob( U_PERIODIC, NONDURABLE );
+}
+
 bool
-QmgrJobUpdater::updateExprTree( ExprTree* tree )
+QmgrJobUpdater::updateExprTree( const char *name, ExprTree* tree )
 {
 	if( ! tree ) {
 		dprintf( D_ALWAYS, "QmgrJobUpdater::updateExprTree: tree is NULL!\n" );
 		return false;
 	}
-	ExprTree *rhs = tree->RArg(), *lhs = tree->LArg();
-	if( ! rhs || ! lhs ) {
-		dprintf( D_ALWAYS,
-				 "QmgrJobUpdater::updateExprTree: tree is invalid!\n" );
-		return false;
-	}
-	char* name = ((Variable*)lhs)->Name();
 	if( ! name ) {
 		dprintf( D_ALWAYS,
 				 "QmgrJobUpdater::updateExprTree: can't find name!\n" );
 		return false;
+	}		
+	const char* value = ExprTreeToString( tree );
+	if( ! value ) {
+		dprintf( D_ALWAYS,
+				 "QmgrJobUpdater::updateExprTree: can't find value!\n" );
+		return false;
 	}		
 		// This code used to be smart about figuring out what type of
 		// expression this is and calling SetAttributeInt(), 
@@ -359,19 +425,20 @@ QmgrJobUpdater::updateExprTree( ExprTree* tree )
 		// and call SetAttribute(), so it was both a waste of effort
 		// here, and made this code needlessly more complex.  
 		// Derek Wright, 3/25/02
-	char* tmp = NULL;
-	rhs->PrintToNewStr( &tmp );
-	if( SetAttribute(cluster, proc, name, tmp) < 0 ) {
+
+		// We use SetAttribute_NoAck to improve performance, since this
+		// avoids a lot of round-trips between the schedd and shadow.
+		// This means we may not detect failure until CommitTransaction()
+		// or the next call to SetAttribute().
+	if( SetAttribute(cluster, proc, name, value, SetAttribute_NoAck) < 0 ) {
 		dprintf( D_ALWAYS, 
 				 "updateExprTree: Failed SetAttribute(%s, %s)\n",
-				 name, tmp );
-		free( tmp );
+				 name, value );
 		return false;
 	}
 	dprintf( D_FULLDEBUG, 
 			 "Updating Job Queue: SetAttribute(%s = %s)\n",
-			 name, tmp );
-	free( tmp );
+			 name, value );
 	return true;
 }
 
