@@ -29,10 +29,7 @@
 #include "scheduler.h"
 #include "basename.h"
 #include "nullfile.h"
-
-extern "C" {
-	char* gen_ckpt_name(char*, int, int, int);
-}
+#include "spooled_job_files.h"
 
 /* In this service function, the client tells the schedd a bunch of jobs
 	it would like to perform a transfer for into/out of a sandbox. The
@@ -92,7 +89,7 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 			respad.Assign(ATTR_TREQ_INVALID_REQUEST, TRUE);
 			respad.Assign(ATTR_TREQ_INVALID_REASON, "Authentication failed.");
 			respad.put(*rsock);
-			rsock->eom();
+			rsock->end_of_message();
 
 			return FALSE;
 		}
@@ -122,7 +119,7 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 	//	ATTR_TREQ_CONSTRAINT
 	//	ATTR_TREQ_XFP
 	reqad.initFromStream(*rsock);
-	rsock->eom();
+	rsock->end_of_message();
 
 	if (reqad.LookupBool(ATTR_TREQ_HAS_CONSTRAINT, has_constraint) == 0) {
 		dprintf(D_ALWAYS, "requestSandBoxLocation(): Client reqad from %s"
@@ -132,7 +129,7 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 		respad.Assign(ATTR_TREQ_INVALID_REQUEST, TRUE);
 		respad.Assign(ATTR_TREQ_INVALID_REASON, "Missing constraint bool.");
 		respad.put(*rsock);
-		rsock->eom();
+		rsock->end_of_message();
 
 		return FALSE;
 	}
@@ -168,8 +165,10 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 			respad.Assign(ATTR_TREQ_INVALID_REQUEST, TRUE);
 			respad.Assign(ATTR_TREQ_INVALID_REASON, "Missing jobid list.");
 			respad.put(*rsock);
-			rsock->eom();
+			rsock->end_of_message();
 
+			delete modify_allow_jobs;
+			delete modify_deny_jobs;
 			return FALSE;
 		}
 
@@ -189,8 +188,10 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 			respad.Assign(ATTR_TREQ_INVALID_REASON, 
 				"No constraint and no jobid list.");
 			respad.put(*rsock);
-			rsock->eom();
+			rsock->end_of_message();
 
+			delete modify_allow_jobs;
+			delete modify_deny_jobs;
 			return FALSE;
 		}
 
@@ -317,8 +318,9 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 		respad.Assign(ATTR_TREQ_INVALID_REASON, 
 			"No file transfer protocol specified.");
 		respad.put(*rsock);
-		rsock->eom();
+		rsock->end_of_message();
 
+		delete modify_allow_jobs;
 		return FALSE;
 	}
 
@@ -337,8 +339,9 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 		respad.Assign(ATTR_TREQ_INVALID_REASON, 
 			"No peer version specified.");
 		respad.put(*rsock);
-		rsock->eom();
+		rsock->end_of_message();
 
+		delete modify_allow_jobs;
 		return FALSE;
 	}
 
@@ -355,8 +358,9 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 		respad.Assign(ATTR_TREQ_INVALID_REASON, 
 			"No file transfer direction specified.");
 		respad.put(*rsock);
-		rsock->eom();
+		rsock->end_of_message();
 
+		delete modify_allow_jobs;
 		return FALSE;
 	}
 
@@ -503,7 +507,7 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 				"getting sandbox info for user.\n", fquser.Value());
 			return FALSE;
 		}
-		rsock->eom();
+		rsock->end_of_message();
 
 		if (td == NULL) {
 			// Create a TransferDaemon object, and hand it to the td
@@ -561,7 +565,7 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 			"getting sandbox info for user.\n", fquser.Value());
 		return FALSE;
 	}
-	rsock->eom();
+	rsock->end_of_message();
 
 	// queue the transfer request to the waiting td who will own the 
 	// transfer request memory which owns the socket.
@@ -583,7 +587,7 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 // open for business.
 ///////////////////////////////////////////////////////////////////////////
 TdAction
-Scheduler::td_register_callback(TransferDaemon *td)
+Scheduler::td_register_callback(TransferDaemon *)
 {
 	dprintf(D_ALWAYS, "Scheduler::td_register_callback() called\n");
 
@@ -595,7 +599,7 @@ Scheduler::td_register_callback(TransferDaemon *td)
 // transferd has died.
 ///////////////////////////////////////////////////////////////////////////
 TdAction
-Scheduler::td_reaper_callback(long pid, int status, TransferDaemon *td)
+Scheduler::td_reaper_callback(long, int, TransferDaemon *)
 {
 	dprintf(D_ALWAYS, "Scheduler::td_reaper_callback() called\n");
 
@@ -613,7 +617,7 @@ Scheduler::td_reaper_callback(long pid, int status, TransferDaemon *td)
 // handler, for all other return values this handler should not delete the treq.
 TreqAction
 Scheduler::treq_upload_pre_push_callback(TransferRequest *treq, 
-	TransferDaemon *td)
+	TransferDaemon *)
 {
 	int cluster;
 	int proc;
@@ -713,7 +717,7 @@ Scheduler::treq_upload_post_push_callback(TransferRequest *treq,
 	dprintf(D_ALWAYS, "Scheduler::treq_post_push_callback() "
 		"Responding to client about sandbox request and closing connection.\n");
 	respad.put(*rsock);
-	rsock->eom();
+	rsock->end_of_message();
 
 	// close the connection to the client now that I've told it where it can
 	// put its files.
@@ -733,7 +737,7 @@ Scheduler::treq_upload_post_push_callback(TransferRequest *treq,
 // for the job and remove it from being on hold.
 TreqAction
 Scheduler::treq_upload_update_callback(TransferRequest *treq, 
-	TransferDaemon *td, ClassAd *update)
+	TransferDaemon *, ClassAd *)
 {
 	int cluster,proc,index;
 	char new_attr_value[500];
@@ -797,7 +801,7 @@ Scheduler::treq_upload_update_callback(TransferRequest *treq,
 		jad = GetJobAd(cluster,proc);
 
 		if ( SpoolSpace ) free(SpoolSpace);
-		SpoolSpace = strdup( gen_ckpt_name(mySpool,cluster,proc,0) );
+		SpoolSpace = gen_ckpt_name(mySpool,cluster,proc,0);
 		ASSERT(SpoolSpace);
 
 		BeginTransaction();
@@ -807,7 +811,7 @@ Scheduler::treq_upload_update_callback(TransferRequest *treq,
 		buf = NULL;
 		jad->LookupString(ATTR_JOB_IWD,&buf);
 		if ( buf ) {
-			sprintf(new_attr_value,"SUBMIT_%s",ATTR_JOB_IWD);
+			snprintf(new_attr_value,500,"SUBMIT_%s",ATTR_JOB_IWD);
 			SetAttributeString(cluster,proc,new_attr_value,buf);
 			free(buf);
 			buf = NULL;
@@ -816,17 +820,14 @@ Scheduler::treq_upload_update_callback(TransferRequest *treq,
 		SetAttributeString(cluster,proc,ATTR_JOB_IWD,SpoolSpace);
 
 			// Backup the original TRANSFER_OUTPUT_REMAPS at submit time
-		expr = jad->Lookup(ATTR_TRANSFER_OUTPUT_REMAPS);
-		sprintf(new_attr_value,"SUBMIT_%s",ATTR_TRANSFER_OUTPUT_REMAPS);
+		expr = jad->LookupExpr(ATTR_TRANSFER_OUTPUT_REMAPS);
+		snprintf(new_attr_value,500,"SUBMIT_%s",ATTR_TRANSFER_OUTPUT_REMAPS);
 		if ( expr ) {
-			char *remap_buf = NULL;
-			ASSERT( expr->RArg() );
-			expr->RArg()->PrintToNewStr(&remap_buf);
+			const char *remap_buf = ExprTreeToString( expr );
 			ASSERT(remap_buf);
 			SetAttribute(cluster,proc,new_attr_value,remap_buf);
-			free(remap_buf);
 		}
-		else if(jad->Lookup(new_attr_value)) {
+		else if(jad->LookupExpr(new_attr_value)) {
 				// SUBMIT_TransferOutputRemaps is defined, but
 				// TransferOutputRemaps is not; disable the former,
 				// so that when somebody fetches the sandbox, nothing
@@ -877,7 +878,7 @@ Scheduler::treq_upload_update_callback(TransferRequest *treq,
 			}
 			if ( changed ) {
 					// Backup original value
-				sprintf(new_attr_value,"SUBMIT_%s",AttrsToModify[index]);
+				snprintf(new_attr_value,500,"SUBMIT_%s",AttrsToModify[index]);
 				SetAttributeString(cluster,proc,new_attr_value,buf);
 					// Store new value
 				char *new_value = new_paths.print_to_string();
@@ -924,7 +925,7 @@ Scheduler::treq_upload_update_callback(TransferRequest *treq,
 // This function does NOT own the memory passed to it ever. If you want a copy
 // of the treq, you must make one.
 TreqAction
-Scheduler::treq_upload_reaper_callback(TransferRequest *treq)
+Scheduler::treq_upload_reaper_callback(TransferRequest *)
 {
 	dprintf(D_ALWAYS, "Scheduler::treq_upload_reaper_callback() called.\n");
 
@@ -947,7 +948,7 @@ Scheduler::treq_upload_reaper_callback(TransferRequest *treq)
 
 TreqAction
 Scheduler::treq_download_pre_push_callback(TransferRequest *treq, 
-	TransferDaemon *td)
+	TransferDaemon *)
 {
 	int cluster;
 	int proc;
@@ -1050,7 +1051,7 @@ Scheduler::treq_download_post_push_callback(TransferRequest *treq,
 	dprintf(D_ALWAYS, "Scheduler::treq_download_post_push_callback() "
 		"Responding to client about sandbox request and closing connection.\n");
 	respad.put(*rsock);
-	rsock->eom();
+	rsock->end_of_message();
 
 	// close the connection to the client now that I've told it where it can
 	// get its files.
@@ -1065,7 +1066,7 @@ Scheduler::treq_download_post_push_callback(TransferRequest *treq,
 
 TreqAction
 Scheduler::treq_download_update_callback(TransferRequest *treq, 
-	TransferDaemon *td, ClassAd *update)
+	TransferDaemon *, ClassAd *)
 {
 	int i;
 	ExtArray<PROC_ID> *jobs;
@@ -1106,7 +1107,7 @@ Scheduler::treq_download_update_callback(TransferRequest *treq,
 // This function does NOT own the memory passed to it. If you want a copy
 // of the treq, you must make one.
 TreqAction
-Scheduler::treq_download_reaper_callback(TransferRequest *treq)
+Scheduler::treq_download_reaper_callback(TransferRequest *)
 {
 	dprintf(D_ALWAYS, "Scheduler::treq_download_reaper_callback() called.\n");
 
@@ -1279,7 +1280,7 @@ Scheduler::uploadGeneralJobFilesWorkerThread(void *arg, Stream* s)
 	}	
 		
 		
-	rsock->eom();
+	rsock->end_of_message();
 
 	int answer;
 
@@ -1287,7 +1288,7 @@ Scheduler::uploadGeneralJobFilesWorkerThread(void *arg, Stream* s)
 
 	answer = OK;
 	rsock->code(answer);
-	rsock->eom();
+	rsock->end_of_message();
 
 	s->timeout(old_timeout);
 
@@ -1363,7 +1364,7 @@ Scheduler::spoolJobFilesReaper(int tid,int exit_status)
 			continue;
 		}
 		if ( SpoolSpace ) free(SpoolSpace);
-		SpoolSpace = strdup( gen_ckpt_name(Spool,cluster,proc,0) );
+		SpoolSpace = gen_ckpt_name(Spool,cluster,proc,0);
 		ASSERT(SpoolSpace);
 
 		BeginTransaction();
@@ -1373,7 +1374,7 @@ Scheduler::spoolJobFilesReaper(int tid,int exit_status)
 		buf = NULL;
 		ad->LookupString(ATTR_JOB_IWD,&buf);
 		if ( buf ) {
-			sprintf(new_attr_value,"SUBMIT_%s",ATTR_JOB_IWD);
+			snprintf(new_attr_value,500,"SUBMIT_%s",ATTR_JOB_IWD);
 			SetAttributeString(cluster,proc,new_attr_value,buf);
 			free(buf);
 			buf = NULL;
@@ -1382,17 +1383,14 @@ Scheduler::spoolJobFilesReaper(int tid,int exit_status)
 		SetAttributeString(cluster,proc,ATTR_JOB_IWD,SpoolSpace);
 
 			// Backup the original TRANSFER_OUTPUT_REMAPS at submit time
-		expr = ad->Lookup(ATTR_TRANSFER_OUTPUT_REMAPS);
-		sprintf(new_attr_value,"SUBMIT_%s",ATTR_TRANSFER_OUTPUT_REMAPS);
+		expr = ad->LookupExpt(ATTR_TRANSFER_OUTPUT_REMAPS);
+		snprintf(new_attr_value,500,"SUBMIT_%s",ATTR_TRANSFER_OUTPUT_REMAPS);
 		if ( expr ) {
-			char *remap_buf = NULL;
-			ASSERT( expr->RArg() );
-			expr->RArg()->PrintToNewStr(&remap_buf);
+			const char *remap_buf = ExprTreeToString( expr );
 			ASSERT(remap_buf);
 			SetAttribute(cluster,proc,new_attr_value,remap_buf);
-			free(remap_buf);
 		}
-		else if(ad->Lookup(new_attr_value)) {
+		else if(ad->LookupExpr(new_attr_value)) {
 				// SUBMIT_TransferOutputRemaps is defined, but
 				// TransferOutputRemaps is not; disable the former,
 				// so that when somebody fetches the sandbox, nothing
@@ -1443,7 +1441,7 @@ Scheduler::spoolJobFilesReaper(int tid,int exit_status)
 			}
 			if ( changed ) {
 					// Backup original value
-				sprintf(new_attr_value,"SUBMIT_%s",AttrsToModify[index]);
+				snprintf(new_attr_value,500,"SUBMIT_%s",AttrsToModify[index]);
 				SetAttributeString(cluster,proc,new_attr_value,buf);
 					// Store new value
 				char *new_value = new_paths.print_to_string();
@@ -1590,7 +1588,7 @@ Scheduler::downloadJobFiles(int mode, Stream* s)
 
 	unsetQSock();
 
-	rsock->eom();
+	rsock->end_of_message();
 
 		// DaemonCore will free the thread_arg for us when the thread
 		// exits, but we need to free anything pointed to by
@@ -1672,7 +1670,7 @@ Scheduler::downloadGeneralJobFilesWorkerThread(void *arg, Stream* s)
 	dprintf(D_FULLDEBUG, "Scheduler::generalJobFilesWorkerThread: "
 		"TRANSFER_DATA/WITH_PERMS: %d jobs to be sent\n", JobAdsArrayLen);
 	rsock->encode();
-	if ( !rsock->code(JobAdsArrayLen) || !rsock->eom() ) {
+	if ( !rsock->code(JobAdsArrayLen) || !rsock->end_of_message() ) {
 		dprintf( D_ALWAYS, "generalJobFilesWorkerThread(): "
 				 "failed to send JobAdsArrayLen (%d) \n",
 				 JobAdsArrayLen );
@@ -1723,7 +1721,7 @@ Scheduler::downloadGeneralJobFilesWorkerThread(void *arg, Stream* s)
 				"failed to send job ad for job %d.%d \n",
 				cluster,proc );
 		} else {
-			rsock->eom();
+			rsock->end_of_message();
 			// and then upload the files
 			result = ftrans.UploadFiles();
 		}
@@ -1739,14 +1737,14 @@ Scheduler::downloadGeneralJobFilesWorkerThread(void *arg, Stream* s)
 	}	
 		
 		
-	rsock->eom();
+	rsock->end_of_message();
 
 	int answer;
 	rsock->decode();
 	answer = -1;
 
 	rsock->code(answer);
-	rsock->eom();
+	rsock->end_of_message();
 	s->timeout(old_timeout);
 
 	if ( peer_version ) {
