@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2010, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -45,10 +45,9 @@
 #include "autocluster.h"
 #include "shadow_mgr.h"
 #include "enum_utils.h"
-//#include "self_draining_queue.h"
 #include "self_draining_queue.h"
-#include "schedd_cronmgr.h"
-#include "condor_classad_namedlist.h"
+#include "schedd_cron_job_mgr.h"
+#include "named_classad_list.h"
 #include "env.h"
 #include "tdman.h"
 #include "condor_crontab.h"
@@ -94,6 +93,13 @@ struct shadow_rec
 		// This ensures that the job can reconnect when we come back up
 		//
 	bool			keepClaimAttributes;
+
+	PROC_ID			prev_job_id;
+	Stream*			recycle_shadow_stream;
+	bool			exit_already_handled;
+
+	shadow_rec();
+	~shadow_rec();
 }; 
 
 struct OwnerData {
@@ -105,15 +111,18 @@ struct OwnerData {
   int JobsFlocked;
   int FlockLevel;
   int OldFlockLevel;
+		// Time of most recent change in flocking level or
+		// successful negotiation at highest current flocking
+		// level.
   time_t NegotiationTimestamp;
   OwnerData() { Name=NULL; Domain=NULL;
-  JobsRunning=JobsIdle=JobsHeld=JobsFlocked=FlockLevel=OldFlockLevel=0; }
+  NegotiationTimestamp=JobsRunning=JobsIdle=JobsHeld=JobsFlocked=FlockLevel=OldFlockLevel=0; }
 };
 
 class match_rec: public ClaimIdParser
 {
  public:
-    match_rec(char*, char*, PROC_ID*, const ClassAd*, char*, char* pool,bool is_dedicated);
+    match_rec(char const*, char const*, PROC_ID*, const ClassAd*, char const*, char const* pool,bool is_dedicated);
 	~match_rec();
 
     char*   		peer; //sinful address of startd
@@ -150,6 +159,8 @@ class match_rec: public ClaimIdParser
 		// punched hole
 	MyString*		auth_hole_id;
 
+	bool m_startd_sends_alives;
+
 		// Set the mrec status to the given value (also updates
 		// entered_current_status)
 	void	setStatus( int stat );
@@ -165,27 +176,37 @@ class UserIdentity {
 			// The default constructor is not recommended as it
 			// has no real identity.  It only exists so
 			// we can put UserIdentities in various templates.
-		UserIdentity() : m_username(""), m_domain("") { }
-		UserIdentity(const char * user, const char * domainname)
-			: m_username(user), m_domain(domainname) { }
-		UserIdentity(const UserIdentity & src)
-			: m_username(src.m_username), m_domain(src.m_domain) { }
+		UserIdentity() : m_username(""), m_domain(""), m_auxid("") { }
+		UserIdentity(const char * user, const char * domainname, const char * aux)
+			: m_username(user), m_domain(domainname), m_auxid(aux) { }		
+		UserIdentity(const UserIdentity & src) {
+			m_username = src.m_username;
+			m_domain = src.m_domain;
+			m_auxid = src.m_auxid;			
+		}
+		UserIdentity(const char * user, const char * domainname, ClassAd * ad);
 		const UserIdentity & operator=(const UserIdentity & src) {
 			m_username = src.m_username;
 			m_domain = src.m_domain;
+			m_auxid = src.m_auxid;
 			return *this;
 		}
 		bool operator==(const UserIdentity & rhs) {
-			return m_username == rhs.m_username && m_domain == rhs.m_domain;
+			return m_username == rhs.m_username && 
+				m_domain == rhs.m_domain && 
+				m_auxid == rhs.m_auxid;
 		}
 		MyString username() const { return m_username; }
 		MyString domain() const { return m_domain; }
+		MyString auxid() const { return m_auxid; }
 
 			// For use in HashTables
 		static unsigned int HashFcn(const UserIdentity & index);
+	
 	private:
 		MyString m_username;
 		MyString m_domain;
+		MyString m_auxid;
 };
 
 
@@ -211,7 +232,6 @@ typedef enum {
 	NO_SHADOW_OLD_VANILLA,
 	NO_SHADOW_RECONNECT,
 	NO_SHADOW_MPI,
-	NO_SHADOW_PRE_6_8_5_STD,
 	NO_SHADOW_VM,
 } NoShadowFailure_t;
 
@@ -219,7 +239,7 @@ typedef enum {
 class ContactStartdArgs
 {
 public:
-	ContactStartdArgs( char* the_claim_id, char* sinful, bool is_dedicated );
+	ContactStartdArgs( char const* the_claim_id, char* sinful, bool is_dedicated );
 	~ContactStartdArgs();
 
 	char*		claimId( void )		{ return csa_claim_id; };
@@ -254,15 +274,20 @@ class Scheduler : public Service
 	void			update_local_ad_file(); // warning, may be removed
 	
 	// negotiation
-	int				doNegotiate(int, Stream *);
 	int				negotiatorSocketHandler(Stream *);
 	int				negotiate(int, Stream *);
 	int				reschedule_negotiator(int, Stream *);
+	void			negotiationFinished( char const *owner, char const *remote_pool, bool satisfied );
 
-	int				reschedule_negotiator_timer() { return reschedule_negotiator(0, NULL); }
+	void				reschedule_negotiator_timer() { reschedule_negotiator(0, NULL); }
 	void			release_claim(int, Stream *);
 	AutoCluster		autocluster;
-	void			sendReschedule( bool checkRecent = true );
+		// send a reschedule command to the negotiatior unless we
+		// have recently sent one and not yet heard from the negotiator
+	void			sendReschedule();
+		// call this when state of job queue has changed in a way that
+		// requires a new round of negotiation
+	void            needReschedule();
 
 	// job managing
 	int				abort_job(int, Stream *);
@@ -274,8 +299,10 @@ class Scheduler : public Service
 	friend	int		updateSchedDInterval( ClassAd* );
 	void			display_shadow_recs();
 	int				actOnJobs(int, Stream *);
+	void            enqueueActOnJobMyself( PROC_ID job_id, JobAction action, bool notify );
+	int             actOnJobMyselfHandler( ServiceData* data );
 	int				updateGSICred(int, Stream* s);
-	void            setNextJobDelay( ClassAd const *job_ad, ClassAd const *machine_ad );
+	void            setNextJobDelay( ClassAd *job_ad, ClassAd *machine_ad );
 	int				spoolJobFiles(int, Stream *);
 	static int		spoolJobFilesWorkerThread(void *, Stream *);
 	static int		transferJobFilesWorkerThread(void *, Stream *);
@@ -285,12 +312,16 @@ class Scheduler : public Service
 	void			PeriodicExprHandler( void );
 	void			addCronTabClassAd( ClassAd* );
 	void			addCronTabClusterId( int );
+	int				RecycleShadow(int cmd, Stream *stream);
+	void			finishRecycleShadow(shadow_rec *srec);
 
 	int				requestSandboxLocation(int mode, Stream* s);
+	int			FindGManagerPid(PROC_ID job_id);
 
 	// match managing
 	int 			publish( ClassAd *ad );
-    match_rec*      AddMrec(char*, char*, PROC_ID*, const ClassAd*, char*, char*, match_rec **pre_existing=NULL);
+	void			OptimizeMachineAdForMatchmaking(ClassAd *ad);
+    match_rec*      AddMrec(char const*, char const*, PROC_ID*, const ClassAd*, char const*, char const*, match_rec **pre_existing=NULL);
 	// All deletions of match records _MUST_ go through DelMrec() to ensure
 	// proper cleanup.
     int         	DelMrec(char const*);
@@ -304,6 +335,7 @@ class Scheduler : public Service
 	void			RemoveShadowRecFromMrec(shadow_rec*);
 	void            sendSignalToShadow(pid_t pid,int sig,PROC_ID proc);
 	int				AlreadyMatched(PROC_ID*);
+	void			ExpediteStartJobs();
 	void			StartJobs();
 	void			StartJob(match_rec *rec);
 	void			StartLocalJobs();
@@ -320,7 +352,8 @@ class Scheduler : public Service
 	bool			availableTransferd( int cluster, int proc, 
 						TransferDaemon *&td_ref ); 
 	bool			startTransferd( int cluster, int proc ); 
-	UserLog*		InitializeUserLog( PROC_ID job_id );
+	WriteUserLog*	InitializeUserLog( PROC_ID job_id );
+	bool			WriteSubmitToUserLog( PROC_ID job_id, bool do_fsync );
 	bool			WriteAbortToUserLog( PROC_ID job_id );
 	bool			WriteHoldToUserLog( PROC_ID job_id );
 	bool			WriteReleaseToUserLog( PROC_ID job_id );
@@ -328,11 +361,9 @@ class Scheduler : public Service
 	bool			WriteEvictToUserLog( PROC_ID job_id, bool checkpointed = false );
 	bool			WriteTerminateToUserLog( PROC_ID job_id, int status );
 	bool			WriteRequeueToUserLog( PROC_ID job_id, int status, const char * reason );
+	bool			WriteAttrChangeToUserLog( const char* job_id_str, const char* attr, const char* attr_value, const char* old_value);
 	int				receive_startd_alive(int cmd, Stream *s);
-#ifdef WANT_NETMAN
-	void			RequestBandwidth(int cluster, int proc, match_rec *rec);
-#endif
-
+	void			InsertMachineAttrs( int cluster, int proc, ClassAd *machine );
 		// Public startd socket management functions
 	void            checkContactQueue();
 
@@ -382,12 +413,18 @@ class Scheduler : public Service
 
 		// Useful public info
 	char*			shadowSockSinful( void ) { return MyShadowSockName; };
-	char*			dcSockSinful( void ) { return MySockName; };
 	int				aliveInterval( void ) { return alive_interval; };
 	char*			uidDomain( void ) { return UidDomain; };
 	int				getJobsTotalAds() { return JobsTotalAds; };
 	int				getMaxJobsSubmitted() { return MaxJobsSubmitted; };
 
+		// Used by the UserIdentity class and some others
+	const ExprTree*	getGridParsedSelectionExpr() const 
+					{ return m_parsed_gridman_selection_expr; };
+	const char*		getGridUnparsedSelectionExpr() const
+					{ return m_unparsed_gridman_selection_expr; };
+
+
 		// Used by the DedicatedScheduler class
 	void 			swap_space_exhausted();
 	void			delete_shadow_rec(int);
@@ -434,8 +471,7 @@ class Scheduler : public Service
 
 		// Used by both the Scheduler and DedicatedScheduler during
 		// negotiation
-	bool canSpawnShadow( int started_jobs, int total_jobs );  
-	void addActiveShadows( int num ) { CurNumActiveShadows += num; };
+	bool canSpawnShadow();
 
 	int				shadow_prio_recs_consistent();
 	void			mail_problem_message();
@@ -451,7 +487,6 @@ private:
 	
 	// information about this scheduler
 	ClassAd*		m_ad;
-	char*			MySockName;		// dhaval
 	Scheduler*		myself;
 
 	// information about the command port which Shadows use
@@ -460,8 +495,8 @@ private:
 	SafeSock*		shadowCommandssock;
 
 	// The "Cron" manager (Hawkeye) & it's classads
-	ScheddCronMgr	*CronMgr;
-	NamedClassAdList extra_ads;
+	ScheddCronJobMgr	*CronJobMgr;
+	NamedClassAdList	 extra_ads;
 
 	// parameters controling the scheduling and starting shadow
 	Timeslice       SchedDInterval;
@@ -471,6 +506,8 @@ private:
 	int             RequestClaimTimeout;
 	int				JobStartDelay;
 	int				JobStartCount;
+	int				JobStopDelay;
+	int				JobStopCount;
 	int             MaxNextJobDelay;
 	int				JobsThisBurst;
 	int				MaxJobsRunning;
@@ -484,7 +521,7 @@ private:
 	int				SwapSpaceExhausted;	// True if job died due to lack of swap space
 	int				ReservedSwap;		// for non-condor users
 	int				MaxShadowsForSwap;
-	int				CurNumActiveShadows;
+	bool			RecentlyWarnedMaxJobsRunning;
 	int				JobsIdle; 
 	int				JobsRunning;
 	int				JobsHeld;
@@ -510,6 +547,8 @@ private:
 	int				jobThrottleNextJobDelay;	// used by jobThrottle()
 
 	int				shadowReaperId; // daemoncore reaper id for shadows
+//	int 				dirtyNoticeId;
+//	int 				dirtyNoticeInterval;
 
 		// Here we enqueue calls to 'contactStartd' when we can't just 
 		// call it any more.  See contactStartd and the call to it...
@@ -524,6 +563,12 @@ private:
 	SimpleList<PROC_ID> jobsToReconnect;
 	int				checkReconnectQueue_tid;
 
+		// queue for sending hold/remove signals to shadows
+	SelfDrainingQueue stop_job_queue;
+		// queue for doing other "act_on_job_myself" calls
+		// such as releasing jobs
+	SelfDrainingQueue act_on_job_myself_queue;
+
 	SelfDrainingQueue job_is_finished_queue;
 	int jobIsFinishedHandler( ServiceData* job_id );
 
@@ -533,6 +578,10 @@ private:
 		// deleting the pointer!
 	GridJobCounts * GetGridJobCounts(UserIdentity user_identity);
 
+		// (un)parsed expressions from condor_config GRIDMANAGER_SELECTION_EXPR
+	ExprTree* m_parsed_gridman_selection_expr;
+	char* m_unparsed_gridman_selection_expr;
+
 	// The object which manages the various transferds.
 	TDMan m_tdman;
 
@@ -549,8 +598,6 @@ private:
 	char*			AccountantName;
     char*			UidDomain;
 
-	time_t last_reschedule_request;
-
 	// connection variables
 	struct sockaddr_in	From;
 	int					Len; 
@@ -616,20 +663,18 @@ private:
 
 	shadow_rec*		StartJob(match_rec*, PROC_ID*);
 	shadow_rec*		start_std(match_rec*, PROC_ID*, int univ);
-	shadow_rec*		start_pvm(match_rec*, PROC_ID*);
 	shadow_rec*		start_sched_universe_job(PROC_ID*);
 	shadow_rec*		start_local_universe_job(PROC_ID*);
 	bool			spawnJobHandlerRaw( shadow_rec* srec, const char* path,
 										ArgList const &args,
 										Env const *env, 
 										const char* name, bool is_dc,
-										bool wants_pipe );
-	void			Relinquish(match_rec*);
+										bool wants_pipe, bool want_udp );
 	void			check_zombie(int, PROC_ID*);
 	void			kill_zombie(int, PROC_ID*);
 	int				is_alive(shadow_rec* srec);
 	shadow_rec*     find_shadow_rec(PROC_ID*);
-	void			NotifyUser(shadow_rec*, char*, int, int);
+	void			NotifyUser(shadow_rec*, const char*, int, int);
 	
 #ifdef CARMI_OPS
 	shadow_rec*		find_shadow_by_cluster( PROC_ID * );
@@ -647,24 +692,34 @@ private:
 	DaemonList		*FlockCollectors, *FlockNegotiators;
 	int				MaxFlockLevel;
 	int				FlockLevel;
-	bool			startd_sends_alives;	
     int         	alive_interval;  // how often to broadcast alive
 		// leaseAliveInterval is the minimum interval we need to send
 		// keepalives based upon ATTR_JOB_LEASE_DURATION...
 	int				leaseAliveInterval;  
 	int				aliveid;	// timer id for sending keepalives to startd
 	int				MaxExceptions;	 // Max shadow excep. before we relinquish
-	bool			ManageBandwidth;
 
 		// put state into ClassAd return it.  Used for condor_squawk
 	int	dumpState(int, Stream *);
 
+		// get connection info for creating sec session to a running job
+		// (e.g. condor_ssh_to_job)
+	int get_job_connect_info_handler(int, Stream* s);
+	int get_job_connect_info_handler_implementation(int, Stream* s);
+
+		// Mark a job as clean
+	int clear_dirty_job_attrs_handler(int, Stream *stream);
+
 		// A bit that says wether or not we've sent email to the admin
 		// about a shadow not starting.
 	int sent_shadow_failure_email;
 
+	bool m_need_reschedule;
+	int m_send_reschedule_timer;
+	Timeslice m_negotiate_timeslice;
+
 	// some stuff about Quill that should go into the ad
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 	int quill_enabled;
 	int quill_is_remotely_queryable;
 	char *quill_name;
@@ -674,6 +729,8 @@ private:
 	int prevLHF;
 #endif
 
+	StringList m_job_machine_attrs;
+	int m_job_machine_attrs_history_length;
 };
 
 
