@@ -33,7 +33,7 @@
 #include "classad_helpers.h"
 #include "log_transaction.h"
 
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "condor_config.h"
 
 #include "loose_file_transfer.h"
@@ -43,7 +43,8 @@
 
 #include "schedd_api.h"
 
-#include "../condor_c++_util/soap_helpers.cpp"
+#include "../condor_utils/soap_helpers.cpp"
+#include "../condor_utils/dc_service.cpp"
 
 #include "qmgmt.h"
 
@@ -176,7 +177,7 @@ extendTransaction(const struct condor__Transaction *transaction)
 	return 0;
 }
 
-static int
+static void
 transtimeout()
 {
 	struct condor__abortTransactionResponse result;
@@ -189,11 +190,7 @@ transtimeout()
 	void *info = daemonCore->GetDataPtr();
 	ASSERT(info);
 	memcpy(&transaction,info,sizeof(condor__Transaction));
-	free(info);
-	info = NULL;
-
 	condor__abortTransaction(NULL, transaction, result);
-	return TRUE;
 }
 
 static void
@@ -484,8 +481,8 @@ condor__beginTransaction(struct soap *soap,
 
 	trans_timer_id =
 		daemonCore->Register_Timer(duration,
-								   (TimerHandler)&transtimeout,
-								   (TimerRelease)&release_data,
+								   transtimeout,
+								   release_data,
 								   "condor_transtimeout");
 	intPtr = (void*) malloc(sizeof(struct condor__Transaction));	
 	memcpy(intPtr,transaction,sizeof(condor__Transaction));
@@ -527,10 +524,7 @@ condor__commitTransaction(struct soap *soap,
 	entry = NULL;	
 
 	current_trans_id = 0;
-	if ( trans_timer_id != -1 ) {
-		daemonCore->Cancel_Timer(trans_timer_id);
-		trans_timer_id = -1;
-	}
+	trans_timer_id = -1;
 
 	result.response.code = SUCCESS;
 	result.response.message = "Success";
@@ -579,10 +573,7 @@ condor__abortTransaction(struct soap *soap,
 	entry = NULL;	
 
 	current_trans_id = 0;
-	if (trans_timer_id != -1) {
-		daemonCore->Cancel_Timer(trans_timer_id);
-		trans_timer_id = -1;
-	}
+	trans_timer_id = -1;
 
 	result.response.code = SUCCESS;
 	result.response.message = "Success";
@@ -694,7 +685,7 @@ condor__removeCluster(struct soap *soap,
 	if (soap->user) {
         ClassAd *an_ad;
         int jobId;
-        if (!(an_ad = GetNextJobByConstraint(constraint.GetCStr(), 1))) {
+        if (!(an_ad = GetNextJobByConstraint(constraint.Value(), 1))) {
 				// Nothing to remove, this is strange
 			result.response.code = FAIL;
 			result.response.message = "Cluster not found";
@@ -719,7 +710,7 @@ condor__removeCluster(struct soap *soap,
         }
 	}
 
-	if (!abortJobsByConstraint(constraint.GetCStr(),
+	if (!abortJobsByConstraint(constraint.Value(),
 							   reason,
 							   transaction->id ? false : true)) {
 		result.response.code = FAIL;
@@ -772,14 +763,17 @@ condor__newJob(struct soap *soap,
 	case -1:
 		result.response.status.code = FAIL;
 		result.response.status.message = "Could not create new job";
+		break;
 	case -2:
 		result.response.status.code =
 			(condor__StatusCode) errstack.code();
 		result.response.status.message =
 			soap_strdup(soap, errstack.message());
+		break;
 	case -3:
 		result.response.status.code = FAIL;
 		result.response.status.message = "Could not record new job";
+		break;
 	default:
 		result.response.status.code = FAIL;
 		result.response.status.message = "Unknown error";
@@ -1402,7 +1396,7 @@ condor__listSpool(struct soap * soap,
 		destroy_job = true;
 	}
 
-	if ((code = job->get_spool_list(files, errstack))) {
+	if (job && (code = job->get_spool_list(files, errstack))) {
 		result.response.status.code =
 			(condor__StatusCode) errstack.code();
 		result.response.status.message =
@@ -1510,7 +1504,7 @@ condor__discoverJobRequirements(struct soap *soap,
 
 		// SimpleInit will bail out if ATTR_JOB_IWD is not set...
 	MyString attribute = MyString(ATTR_JOB_IWD) + " = \"/tmp\"";
-	if (!ad.Insert(attribute.GetCStr())) {
+	if (!ad.Insert(attribute.Value())) {
 		result.response.status.code = FAIL;
 		result.response.status.message = "Failed to setup temporary Iwd attribute.";
 
@@ -1609,7 +1603,7 @@ condor__createJobTemplate(struct soap *soap,
 	if (soapLeaveInQueue) {
 		attribute = attribute + " && (" + soapLeaveInQueue + ")";
 	}
-	job->AssignExpr(ATTR_JOB_LEAVE_IN_QUEUE, attribute.GetCStr());
+	job->AssignExpr(ATTR_JOB_LEAVE_IN_QUEUE, attribute.Value());
 
 	ArgList arglist;
 	MyString arg_errors;
