@@ -403,7 +403,8 @@ sub runCommand
 		if (defined $WantErrorCallback )
 		{
 			debug("Calling error callback!!!!!! at @ $WantErrorCallback\n",6);
-			&$WantErrorCallback( %info )
+			&$WantErrorCallback( %info );
+			return 1;
 		}
 		DumpFailedCommand($command_string);
         return 0;
@@ -419,7 +420,8 @@ sub runCommand
 		if (defined $WantErrorCallback )
 		{
 			debug("Calling error callback!!!!!! at @ $WantErrorCallback\n",6);
-			&$WantErrorCallback( %info )
+			&$WantErrorCallback( %info );
+			return 1;
 		}
 		DumpFailedCommand($command_string);
         return 0;
@@ -601,6 +603,7 @@ sub Monitor
     my $line;
     #my %info;
 	my $timestamp = 0;
+    my $parent_pid = getppid();
 
     debug( "Entering Monitor\n" ,5);
 
@@ -654,6 +657,7 @@ sub Monitor
   LINE:
     while( 1 )
     {
+
 		#print "Now $num_active_jobs is number of active jobs\n";
 	if( $saw_submit && $num_active_jobs == 0 )
 	{
@@ -676,6 +680,12 @@ sub Monitor
 	while( ! defined $line )
 	{
 		sleep 2;
+		if( getppid() != $parent_pid )
+		{
+		    print "\nCondor::Monitor: our parent pid $parent_pid has gone away.  Aborting.\n";
+		    return 0;
+		}
+
 		if(defined $TimedCallback)
 		{
 			CheckTimedCallback();
@@ -899,12 +909,12 @@ sub Monitor
 
 	# 001: job executing
 	elsif( $line =~ 
-	       /^001\s+\(0*(\d+)\.0*(\d+).*<(\d+\.\d+\.\d+\.\d+):(\d+)>/ )
+	       /^001\s+\(0*(\d+)\.0*(\d+).*<([^>]+)>/ )
 	{
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
 	    $info{'host'} = $3;
-	    $info{'sinful'} = "<$3:$4>";
+	    $info{'sinful'} = "<$3>";
 	    
 	    debug( "Saw job executing\n" ,5);
 
@@ -917,12 +927,12 @@ sub Monitor
 
 	# 000: job submitted
 	elsif( $line =~ 
-	       /^000\s+\(0*(\d+)\.0*(\d+).*<(\d+\.\d+\.\d+\.\d+):(\d+)>/ )
+	       /^000\s+\(0*(\d+)\.0*(\d+).*<([^>]+)>/ )
 	{
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
 	    $info{'host'} = $3;
-	    $info{'sinful'} = "<$3:$4>";
+	    $info{'sinful'} = "<$3>";
 
 	    debug( "Saw job submitted\n" ,5);
 	    $submit_info{'cluster'} = $1; # squirrel it away for TimedWait
@@ -960,8 +970,8 @@ sub Monitor
 	{
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
-	    $info{'host'} = $3;
-	    $info{'sinful'} = "<$3:$4>";
+	    #$info{'host'} = $3;
+	    #$info{'sinful'} = "<$3:$4>";
 	    
 	    # read next line to see how job was held
 	    $line = <SUBMIT_LOG>;
@@ -992,8 +1002,8 @@ sub Monitor
 	{
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
-	    $info{'host'} = $3;
-	    $info{'sinful'} = "<$3:$4>";
+	    #$info{'host'} = $3;
+	    #$info{'sinful'} = "<$3:$4>";
 	    
 	    debug( "Saw job released\n" ,5);
 
