@@ -50,13 +50,9 @@
 # Nov 07 : Added repaeating a test n times by adding "-a n" to args
 # Nov 07 : Added condor_personal setup only by adding -p (pretest work);
 # Mar 17 : Added condor cleanup functionality by adding -c option.
-# Dec 08 : Have been working on detecting core/ERRORs after each test
-#	which got us into wrapping tests running in the outer personal condor
-#	in condor_tests/TestingPersonalCondor so we can report back a unique
-#	log directory for each test. This got us into needing to have a server
-#	being started and stoped as needed to collect the publishing of these
-#	log dirs so batch test can still check. This is fairly important as
-#	we now have the -e option to run batches of tests at the same time.
+# Dec 2008: working on concurrency testing. Here with -e 20 we try to keep
+#	20 tests going at a time within the given compiler or toplevel
+#	directory.
 #
 
 #require 5.0;
@@ -64,8 +60,8 @@ use File::Copy;
 use FileHandle;
 use POSIX "sys_wait_h";
 use Cwd;
+use CondorUtils;
 use CondorTest;
-use CondorPubLogdirs;
 use Time::Local;
 use strict;
 use warnings;
@@ -98,39 +94,15 @@ Condor::DebugLevel(1);
 # 	This is all triggered by -w
 #
 #	WRAP_TESTS
-#	SEND_LOGS
 #
 #	We want to search out core files and ERROR prints AND we want
 #	to run many tests at once. The first check method looks for
 #	all logs changed during the test and assigns blame based on that.
-#	Sadly any error from a set of daemons still running will be assigned
-#	to the wrong test. The steps to make this happen now has the following
-#	steps:
 #
 #		Allow for every test which is not wrapped in a personal 
 #		condor to be wrapped. This is done in CondorTest.pm in RunTest
 #		and RunDagTest if WRAP_TESTS is set.
 #
-#		batch_test.pl defines that and also write file
-#		LogServerHandle when the -w option is set. This is so StartCondor 
-#		can report Test name/Log location to a server collecting such things.
-#		I wanted to only use $ENV[SEND_LOGS] but we in the nightlies have
-#		multiple calls of batch_test.pl requiring some way for
-#		it to know if it has already fired off the server to colloect
-#		test log locations.
-#
-#		batch_test will start this server and place connection
-#		information in it. Thus multiple calls to batch_test
-#		will generate exacly one server to collect logs in file
-#		LogDirs.
-#
-#		If set, batch_test will lookup the log location for a test
-#		to do the core/ERROR detection. No log location will
-#		result in a file being created called NoLogDirs where the tests
-#		name will be recorded for later fixing if it should have had
-#		a log location. This file will be restarted with a date at the top
-#		whenevere the server to collect log locations is started
-#
 #################################################################
 
 #my $LogFile = "batch_test.log";
@@ -141,7 +113,12 @@ Condor::DebugLevel(1);
 #select(STDERR); $| = 1;
 #select(STDOUT); $| = 1;
 
-my $iswindows = IsThisWindows();
+my $hostname = `hostname`; chomp $hostname;
+if ( ! ($hostname =~ /\./ ) ) {
+    warn "Warning: Host name '$hostname' is not an FQDN!!";
+    sleep( 10 );
+}
+my $iswindows = CondorTest::IsThisWindows();
 
 # configuration options
 my $test_retirement = 3600;	# seconds for an individual test timeout - 30 minutes
@@ -158,8 +135,8 @@ my $want_core_dumps = 1;
 my $testpersonalcondorlocation = "$BaseDir/TestingPersonalCondor";
 my $wintestpersonalcondorlocation = "";
 if($iswindows == 1) {
-	my $tmp = `cygpath -w $testpersonalcondorlocation`;
-	chomp($tmp);
+	my $tmp = `cygpath -m $testpersonalcondorlocation`;
+	CondorTest::fullchomp($tmp);
 	$wintestpersonalcondorlocation = $tmp;
 }
 
@@ -168,31 +145,19 @@ my $targetconfiglocal = $testpersonalcondorlocation . "/condor_config.local";
 my $condorpidfile = "/tmp/condor.pid.$$";
 my @extracondorargs;
 
-# we want to process and track the collection of cores
-my $coredir = "$BaseDir/Cores";
-if(!(-d $coredir)) {
-	debug("Creating collection directory for cores\n",2);
-	system("mkdir -p $coredir");
-}
-
 my $localdir = $testpersonalcondorlocation . "/local";
 my $installdir;
 my $wininstalldir; # need to have dos type paths for condor
 my $testdir;
-my $configmain;
-my $configlocal;
-my $killlogserver = 1;
+my $configmain = "../condor_examples/condor_config.generic";
+my $configlocal = "../condor_examples/condor_config.local.central.manager";
 
 my $wantcurrentdaemons = 1; # dont set up a new testing pool in condor_tests/TestingPersonalCondor
-my $wantcorechecks = 0;
 my $pretestsetuponly = 0; # only get the personal condor in place
 
 # set up to recover from tests which hang
 $SIG{ALRM} = sub { die "timeout" };
 
-# set up for reading in core/ERROR exemptions
-my $errexempts = "ErrorExemptions";
-my %exemptions;
 my @compilers;
 my @successful_tests;
 my @failed_tests;
@@ -226,7 +191,6 @@ my @testlist;
 # -k[ind]: be kind and submit slowly
 # -b[buildandtest]: set up a personal condor and generic configs
 # -w[wrap]: test in personal condor enable core/ERROR detection
-# -x[execute log dir server]
 # -a[again]: how many times do we run each test?
 # -p[pretest]: get are environment set but run no tests
 # -c[cleanup]: stop condor when test(s) finish.  Not used on windows atm.
@@ -246,7 +210,6 @@ while( $_ = shift( @ARGV ) ) {
 	    print "-s[ort] sort before running tests\n";
 	    print "-b[buildandtest]: set up a personal condor and generic configs\n";
 	    print "-w[wrap]: test in personal condor enable core/ERROR detection\n";
-	    print "-xls: execute log dir server\n";
 	    print "-xml: Output in xml\n";
 	    print "-w[wrap]: test in personal condor enable core/ERROR detection\n";
 	    print "-a[again]: how many times do we run each test?\n";
@@ -273,18 +236,9 @@ while( $_ = shift( @ARGV ) ) {
                 next SWITCH;
         }
         if( /^-w.*/ ) {
-				$wantcorechecks = 1;
 				$ENV{WRAP_TESTS} = "yes";
-				my $server = CondorPubLogdirs::CheckLogServer(); # is it running yet?
-				debug("CheckLogServer says log server is $server\n",1);
-				$ENV{SEND_LOGS} = $server;
                 next SWITCH;
         }
-        if( /^-xls.*/ ) {
-				debug("Stopping LogDir server\n",1);
-				CondorPubLogdirs::StopLogServer(); # is it running yet? kill it
-	    		exit(0);
-        }
         if( /^-d.*/ ) {
                 push(@compilers, shift(@ARGV));
                 next SWITCH;
@@ -330,7 +284,6 @@ while( $_ = shift( @ARGV ) ) {
                 next SWITCH;
         }
         if( /^-t.*/ ) {
-				$killlogserver = 0;
                 push(@testlist, shift(@ARGV));
                 next SWITCH;
         }
@@ -348,10 +301,9 @@ while( $_ = shift( @ARGV ) ) {
                 next SWITCH;
         }
         if( /^-c.*/ ) {
-                # This is not used on windows systems at the moment.
-                $cleanupcondor = 1;
-                push (@extracondorargs, "-pidfile $condorpidfile");
-                next SWITCH;
+			$cleanupcondor = 1;
+			push (@extracondorargs, "-pidfile $condorpidfile");
+			next SWITCH;
         }
   }
 }
@@ -365,13 +317,9 @@ my %test_suite = ();
 my $awkscript = "";
 my $genericconfig = "";
 my $genericlocalconfig = "";
-my $nightly = IsThisNightly($BaseDir);
+my $nightly = CondorTest::IsThisNightly($BaseDir);
 my $res = 0;
 
-if($wantcorechecks == 1) {
-	LoadExemptions();
-}
-
 if(!($wantcurrentdaemons)) {
 
 	$awkscript = "../condor_examples/convert_config_to_win32.awk";
@@ -399,8 +347,8 @@ if(!($wantcurrentdaemons)) {
 	}
 
 	if($iswindows == 1) {
-		my $tmp = `cygpath -w $targetconfig`;
-		chomp($tmp);
+		my $tmp = `cygpath -m $targetconfig`;
+		CondorTest::fullchomp($tmp);
 		$ENV{CONDOR_CONFIG} = $tmp;
 		$res = IsPersonalRunning($tmp);
 	} else {
@@ -418,14 +366,14 @@ if(!($wantcurrentdaemons)) {
 			$mcmd =~ s/\\/\//g;
 			debug( "Starting master like this:\n",2);
 			debug( "\"$mcmd\"\n",2);
-			CondorTest::verbose_system("$mcmd");
+			CondorTest::verbose_system("$mcmd",{emit_output=>0,use_system=>1});
 		} else {
-			CondorTest::verbose_system("$installdir/sbin/condor_master @extracondorargs -f &");
+			CondorTest::verbose_system("$installdir/sbin/condor_master @extracondorargs -f &",{emit_output=>0,use_system=>1});
 		}
 		debug("Done Starting Personal Condor\n",2);
 	}
 		
-	IsRunningYet();
+	CondorPersonal::IsRunningYet() || die "Failed to start Condor";
 
 }
 
@@ -451,7 +399,7 @@ if($#compilers == -1 ) {
 }
 
 if($timestamp == 1) {
-	system("date");
+	print scalar localtime() . "\n";
 }
 
 foreach my $name (@compilers) {
@@ -575,7 +523,7 @@ if( $ignorefile ) {
 my $ResultDir;
 # set up base directory for storing test results
 if ($isXML){
-      CondorTest::verbose_system ("mkdir -p $BaseDir/results");
+      CondorTest::verbose_system ("mkdir -p $BaseDir/results",{emit_output=>0});
       $ResultDir = "$BaseDir/results";
       open( XML, ">$ResultDir/ncondor_testsuite.xml" ) || die "error opening \"ncondor_testsuite.xml\": $!\n";
       print XML "<\?xml version=\"1.0\" \?>\n<test_suite>\n";
@@ -603,7 +551,7 @@ foreach my $compiler (@compilers)
 
 	debug("Compiler/Directory <$compiler> has $testspercompiler tests\n",2); 
     if ($isXML){
-      CondorTest::verbose_system ("mkdir -p $ResultDir/$compiler");
+      CondorTest::verbose_system ("mkdir -p $ResultDir/$compiler",{emit_output=>0});
     } 
 	if($compiler ne "\.") {
 		# Meh, if the directory isn't there, just skip it instead of bailing.
@@ -852,28 +800,56 @@ foreach my $test_name (@failed_tests)
 close OUTF;
 close SUMOUTF;
 
-if( exists $ENV{WRAP_TESTS}) {
-	if(($ENV{WRAP_TESTS} eq "yes") and ($killlogserver == 1)) {
-		debug("Stopping LogDir server\n",1);
-		CondorPubLogdirs::StopLogServer(); # is it running yet? kill it
-	}
-}
 
 if ( $cleanupcondor )
 {
-   if ( $iswindows ) 
-   {
-      # Currently not implemented.
-   }
-   else
-   {
-      my $pid=`cat $condorpidfile`;
-      system("kill -QUIT $pid");
-      system("rm -f $condorpidfile");
-   }
+	my $pid;
+	local *IN;
+	if( open IN, '<', $condorpidfile) {
+		$pid = <IN>;
+		close IN;
+		chomp $pid;
+	}
+	if($pid !~ /^\d+$/) {
+		print STDERR "PID file appears corrupt! Contains: $pid\n";
+		$pid = undef;
+	}
+	if(not defined $pid) {
+		print STDERR "PID file wasn't available; may not be able to shut down Condor.\n";
+	}
+	system("condor_off","-master");
+	if($pid) {
+		if( ! wait_for_process_gone($pid, 5) ) {
+			kill('QUIT', $pid);
+			if( ! wait_for_process_gone($pid, 5) ) {
+				# TODO: More ruthlessly enumerate all of my children and kil them.
+				kill('KILL', $pid);
+				if( ! wait_for_process_gone($pid, 1) ) {
+					print STDERR "Warning: Unable to shut down Condor daemons\n";
+				}
+			}
+		}
+	}
+	unlink($condorpidfile) if -f $condorpidfile;
 }
 exit $num_failed;
 
+# Spin wait until $pid is no longer present or $max_wait (seconds) passes.
+# Returns 1 if process exited, 0 if we timed out.
+# Seconds are the smallest granularity.
+# This uses the "kill(0)" trick; specifically, it's asking "Is this PID alive?"
+# in a loop. On a very heavily loaded system with PIDs being reused very
+# quickly, this could wait on the wrong process or processes.
+sub wait_for_process_gone {
+	my($pid, $max_wait) = @_;
+	my $done_time = time() + $max_wait;
+	while(1) {
+		if(time() >= $done_time) { return 0; }
+		if(kill(0, $pid)) { return 1; }
+		select(undef,undef,undef, 0.01); # Sleep 0.01th of a second
+	}
+}
+
 sub CleanFromPath
 {
 	my $pulldir = shift;
@@ -889,46 +865,6 @@ sub CleanFromPath
 	}
 }
 
-sub IsThisNightly
-{
-	my $mylocation = shift;
-
-	debug("IsThisNightly passed <$mylocation>\n",2);
-	if($mylocation =~ /^.*(\/execute\/).*$/) {
-		print "Nightly testing\n";
-		$configlocal = "../condor_examples/condor_config.local.central.manager";
-		$configmain = "../condor_examples/condor_config.generic";
-		if(!(-f $configmain)) {
-			system("ls ..");
-			system("ls ../condor_examples");
-			die "No base config file!!!!!\n";
-		}
-		return(1);
-	} else {
-		print "Workspace testing\n";
-		$configlocal = "../condor_examples/condor_config.local.central.manager";
-		$configmain = "../condor_examples/condor_config.generic";
-		if(!(-f $configmain)) {
-			system("ls ..");
-			system("ls ../condor_examples");
-			die "No base config file!!!!!\n";
-		}
-		return(0);
-	}
-}
-
-sub IsThisWindows
-{
-	my $path = CondorTest::Which("cygpath");
-	debug("Path return from which cygpath: $path\n",2);
-	if($path =~ /^.*\/bin\/cygpath.*$/ ) {
-		print "This IS windows\n";
-		return(1);
-	}
-	print "This is NOT windows\n";
-	return(0);
-}
-
 sub IsPersonalTestDirThere
 {
 	if( -d $testpersonalcondorlocation ) {
@@ -955,8 +891,8 @@ sub WhereIsInstallDir
 	if($iswindows == 1) {
 		my $top = getcwd();
 		debug( "getcwd says \"$top\"\n",2);
-		my $crunched = `cygpath -w $top`;
-		chomp($crunched);
+		my $crunched = `cygpath -m $top`;
+		CondorTest::fullchomp($crunched);
 		debug( "cygpath changed it to: \"$crunched\"\n",2);
 		my $ppwwdd = `pwd`;
 		debug( "pwd says: $ppwwdd\n",2);
@@ -967,17 +903,22 @@ sub WhereIsInstallDir
 		print STDERR "Unable to find a condor_master in your \$PATH!\n";
 		exit(1);
 	}
-	chomp($tmp);
+	CondorTest::fullchomp($tmp);
 	debug( "Install Directory \"$tmp\"\n",2);
 	if($iswindows == 0) {
-		if($tmp =~ /^(.*)\/sbin\/condor_master\s*$/) {
+	    $tmp =~ s|//|/|g;
+		if( ($tmp =~ /^(.*)\/sbin\/condor_master\s*$/) || \
+		    ($tmp =~ /^(.*)\/bin\/condor_master\s*$/) ) {
 			$installdir = $1;
 			print "Testing This Install Directory: \"$installdir\"\n";
 		}
+		else {
+		    die "'$tmp' didn't match path RE\n";
+		}
 	} else {
 		if($tmp =~ /^(.*)\/bin\/condor_master\s*$/) {
 			$installdir = $1;
-			$tmp = `cygpath -w $1`;
+			$tmp = `cygpath -m $1`;
     		CondorTest::fullchomp($tmp);
 			$wininstalldir = $tmp;
 			$wininstalldir =~ s/\\/\//;
@@ -1016,18 +957,17 @@ sub CreateConfig
 	# file is to set the release-dir and local-dir. (non-windows)
 	# change RELEASE_DIR and LOCAL_DIR    
 	my $currenthost = CondorTest::getFqdnHost();
-	chomp($currenthost);
+	CondorTest::fullchomp($currenthost);
 
 	debug( "Set RELEASE_DIR and LOCAL_DIR\n",2);
 
 	# Windows needs config file preparation, wrapper scripts etc
 	if($iswindows == 1) {
 		# set up job wrapper
-		safe_copy("../condor_scripts/exe_switch.pl", "$wininstalldir/bin/exe_switch.pl") ||
-        die "couldn't copy exe_swtich.pl";
-		open( WRAPPER, ">$wininstalldir/bin/exe_switch.bat" ) || die "Can't open new job wrapper: $!\n";
-    	print WRAPPER "\@c:\\perl\\bin\\perl.exe $wininstalldir/bin/exe_switch.pl %*\n";
-    	close(WRAPPER);
+		# safe_copy("../condor_scripts/exe_switch.pl", "$wininstalldir/bin/exe_switch.pl") || die "couldn't copy exe_swtich.pl";
+		# open( WRAPPER, ">$wininstalldir/bin/exe_switch.bat" ) || die "Can't open new job wrapper: $!\n";
+		# print WRAPPER "\@c:\\perl\\bin\\perl.exe $wininstalldir/bin/exe_switch.pl %*\n";
+		# close(WRAPPER);
 		
 		# pre-process config file src and windowize it
 
@@ -1047,7 +987,7 @@ sub CreateConfig
 	open( NEWFIG, ">$targetconfig" ) 
 		|| die "Can't open new config file: $!\n";    
 	while( <OLDFIG> ) {        
-		chomp;        
+		CondorTest::fullchomp($_);        
 		$line = $_;        
 		if($line =~ /^RELEASE_DIR\s*=.*/) {            
 			debug( "Matching <<$line>>\n",2);
@@ -1075,9 +1015,9 @@ sub CreateConfig
 		} elsif($line =~ /^CONDOR_HOST\s*=.*/) {
 			debug( "Matching <<$line>>\n",2);
 			print NEWFIG "CONDOR_HOST = $currenthost\n";
-		} elsif($line =~ /^HOSTALLOW_WRITE\s*=.*/) {
+		} elsif($line =~ /^ALLOW_WRITE\s*=.*/) {
 			debug( "Matching <<$line>>\n",2);
-			print NEWFIG "HOSTALLOW_WRITE = *\n";
+			print NEWFIG "ALLOW_WRITE = *\n";
 		} elsif($line =~ /NOT_RESPONDING_WANT_CORE\s*=.*/ and $want_core_dumps ) {
 			debug( "Matching <<$line>>\n",2);
 			print NEWFIG "NOT_RESPONDING_WANT_CORE = True\n";
@@ -1128,7 +1068,7 @@ sub CreateLocalConfig
 
 	# ADD size for log files and debug level
 	# default settings are in condor_config, set here to override 
-	print FIX "ALL_DEBUG               = D_FULLDEBUG\n";
+	print FIX "ALL_DEBUG               = D_FULLDEBUG D_SECURITY\n";
 
 	print FIX "MAX_COLLECTOR_LOG       = $logsize\n";
 	print FIX "COLLECTOR_DEBUG         = \n";
@@ -1150,7 +1090,6 @@ sub CreateLocalConfig
 	print FIX "STARTD_DEBUG            = D_COMMAND\n";
 
 	print FIX "MAX_STARTER_LOG         = $logsize\n";
-	print FIX "STARTER_DEBUG           = D_NODATE\n";
 
 	print FIX "MAX_MASTER_LOG          = $logsize\n";
 	print FIX "MASTER_DEBUG            = D_COMMAND\n";
@@ -1159,6 +1098,7 @@ sub CreateLocalConfig
 	print FIX "PERIODIC_EXPR_INTERVAL = 15\n";
 	print FIX "PERIODIC_EXPR_TIMESLICE = .95\n";
 	print FIX "NEGOTIATOR_INTERVAL = 20\n";
+	print FIX "DAGMAN_USER_LOG_SCAN_INTERVAL = 1\n";
 
 	# turn on soap for testing
 	print FIX "ENABLE_SOAP            	= TRUE\n";
@@ -1166,11 +1106,14 @@ sub CreateLocalConfig
 	print FIX "QUEUE_ALL_USERS_TRUSTED 	= TRUE\n";
 
 	# condor_config.generic now contains a special value
-	# for HOSTALLOW_WRITE which causes it to EXCEPT on submit
+	# for ALLOW_WRITE which causes it to EXCEPT on submit
 	# till set to some legal value. Old was most insecure..
-	print FIX "HOSTALLOW_WRITE 			= *\n";
+	print FIX "ALLOW_WRITE 			= *\n";
 	print FIX "NUM_CPUS 			= 15\n";
 
+	if($iswindows == 1) {
+		print FIX "JOB_INHERITS_STARTER_ENVIRONMENT = TRUE\n";
+	}
 	# Allow a default heap size for java(addresses issues on x86_rhas_3)
 	# May address some of the other machines with Java turned off also
 	print FIX "JAVA_MAXHEAP_ARGUMENT = \n";
@@ -1179,43 +1122,53 @@ sub CreateLocalConfig
 	print FIX "RunBenchmarks = false\n";
 	print FIX "JAVA_BENCHMARK_TIME = 0\n";
 
-	# below stolen from condor_configure
 
 	my $jvm = "";
-
-    my @default_jvm_locations = ("/bin/java",
-                 "/usr/bin/java",
-                 "/usr/local/bin/java",
-                 "/s/std/bin/java");
-
-    unless (system ("which java >> /dev/null 2>&1")) {
-    	chomp (my $which_java = CondorTest::Which("java"));
-    	@default_jvm_locations = ($which_java, @default_jvm_locations) unless ($?);
-    }
-
-    my $java_libdir = "";
+	my $java_libdir = "";
+	my $exec_result;
+	my $javabinary = "";
 	if($iswindows == 1) {
-    	$java_libdir = "$wininstalldir/lib";
+
+		$javabinary = "java.exe";
+		my $whichtest = `which $javabinary`;
+	    CondorTest::fullchomp ($whichtest);
+		$whichtest =~ s/Program Files/progra~1/g;
+		$jvm = `cygpath -m $whichtest`;
+		CondorTest::fullchomp($jvm);
+		CondorTest::debug("which java said<<$jvm>>\n",2);
+
+	    $java_libdir = "$wininstalldir/lib";
+
 	} else {
-    	$java_libdir = "$installdir/lib";
+		# below stolen from condor_configure
+
+	    my @default_jvm_locations = ("/bin/java",
+	                 "/usr/bin/java",
+	                 "/usr/local/bin/java",
+	                 "/s/std/bin/java");
+
+		$javabinary = "java";
+	    unless (system ("which java >> /dev/null 2>&1")) {
+	    	CondorTest::fullchomp (my $which_java = CondorTest::Which("$javabinary"));
+			CondorTest::debug("CT::Which for $javabinary said $which_java\n",2);
+	    	@default_jvm_locations = ($which_java, @default_jvm_locations) unless ($?);
+	    }
+
+	    $java_libdir = "$installdir/lib";
+
+	    # check some default locations for java and pick first valid one
+	    foreach my $default_jvm_location (@default_jvm_locations) {
+		    CondorTest::debug("default_jvm_location is <<$default_jvm_location>>\n",2);
+	    	if ( -f $default_jvm_location && -x $default_jvm_location) {
+	        	$jvm = $default_jvm_location;
+				print "Set JAVA to $jvm\n";
+	        	last;
+	    	}
+	    }
 	}
-
-    my $exec_result;
-    my $default_jvm_location;
-
-    # check some default locations for java and pick first valid one
-    foreach $default_jvm_location (@default_jvm_locations) {
-    	if ( -f $default_jvm_location && -x $default_jvm_location) {
-        	$jvm = $default_jvm_location;
-        	last;
-    	}
-    }
-
     # if nothing is found, explain that, otherwise see if they just want to
     # accept what I found.
-
 	debug ("Setting JAVA=$jvm",2);
-
     # Now that we have an executable JVM, see if it is a Sun jvm because that
     # JVM it supports the -Xmx argument then, which is used to specify the
     # maximum size to which the heap can grow.
@@ -1240,7 +1193,12 @@ sub CreateLocalConfig
         $java_jvm_maxmem_arg = "";
     }
 
-	print FIX "JAVA = $jvm\n";
+	if($iswindows == 1){
+		print FIX "JAVA = $jvm\n";
+		print FIX "JAVA_EXTRA_ARGUMENTS = -Xmx1024m\n";
+	} else {
+		print FIX "JAVA = $jvm\n";
+	}
 
 
 	# above stolen from condor_configure
@@ -1267,9 +1225,9 @@ sub CreateLocalConfig
 
 	# Add a job wrapper for windows.... and a few other things which
 	# normally are done by condor_configure for a personal condor
-	if($iswindows == 1) {
-		print FIX "USER_JOB_WRAPPER = $wininstalldir/bin/exe_switch.bat\n";
-	}
+	#if($iswindows == 1) {
+	#	print FIX "USER_JOB_WRAPPER = $wininstalldir/bin/exe_switch.bat\n";
+	#}
 
 	# Tell condor to use the current directory for temp.  This way,
 	# if we get preempted/killed, we don't pollute the global /tmp
@@ -1286,10 +1244,12 @@ sub CreateLocalConfig
 	print FIX "WANT_SUSPEND = FALSE\n";
 	print FIX "WANT_VACATE = FALSE\n";
 	print FIX "COLLECTOR_NAME = Personal Condor for Tests\n";
-	print FIX "ALL_DEBUG = D_FULLDEBUG\n";
+	print FIX "ALL_DEBUG = D_FULLDEBUG D_SECURITY\n";
 	print FIX "SCHEDD_INTERVAL_TIMESLICE = .99\n";
 	#insure path from framework is injected into the new pool
-	print FIX "environment=\"PATH=\'$mypath\'\"\n";
+	if($iswindows == 0) {
+		print FIX "environment=\"PATH=\'$mypath\'\"\n";
+	}
 	print FIX "SUBMIT_EXPRS=environment\n";
 	print FIX "PROCD_LOG = \$(LOG)/ProcLog\n";
 	if($iswindows == 1) {
@@ -1312,8 +1272,7 @@ sub IsPersonalRunning
 		$pathtoconfig =~ s/\\/\\\\/g;
 	}
 
-    open(CONFIG, "condor_config_val -config -master log 2>&1 |") || die "condor_config_val: $
-!\n";
+    open(CONFIG, "condor_config_val -config -master log 2>&1 |") || die "condor_config_val: $!\n";
     while(<CONFIG>) {
         CondorTest::fullchomp($_);
         $line = $_;
@@ -1327,7 +1286,9 @@ sub IsPersonalRunning
 			last;
         }
     }
-    close(CONFIG);
+    if ( close(CONFIG) && ($? != 13) ) {	# Ignore SIGPIPE
+	warn "Error executing condor_config_val: '$?' '$!'"
+    }
 
     if( $matchedconfig eq "" ) {
         die	"lost: config does not match expected config setting......\n";
@@ -1353,125 +1314,6 @@ sub IsPersonalRunning
 	close(MADDR);
 }
 
-sub IsRunningYet
-{
-
-    my $daemonlist = `condor_config_val daemon_list`;
-    CondorTest::fullchomp($daemonlist);
-
-	if($daemonlist =~ /.*MASTER.*/) {
-		# now wait for the master to start running... get address file loc
-		# and wait for file to exist
-		# Give the master time to start before jobs are submitted.
-		my $masteradr = `condor_config_val MASTER_ADDRESS_FILE`;
-		$masteradr =~ s/\012+$//;
-		$masteradr =~ s/\015+$//;
-		debug( "MASTER_ADDRESS_FILE is <<<<<$masteradr>>>>>\n",2);
-    	debug( "We are waiting for the file to exist\n",2);
-    	# Where is the master address file? wait for it to exist
-    	my $havemasteraddr = "no";
-    	while($havemasteraddr ne "yes") {
-        	debug( "Looking for $masteradr\n",2);
-        	if( -f $masteradr ) {
-            	debug("Found it!!!! master address file \n",2);
-            	$havemasteraddr = "yes";
-        	} else {
-            	sleep 1;
-        	}
-    	}
-	}
-
-	if($daemonlist =~ /.*COLLECTOR.*/) {
-		# now wait for the collector to start running... get address file loc
-		# and wait for file to exist
-		# Give the master time to start before jobs are submitted.
-		my $collectoradr = `condor_config_val COLLECTOR_ADDRESS_FILE`;
-		$collectoradr =~ s/\012+$//;
-		$collectoradr =~ s/\015+$//;
-		debug( "COLLECTOR_ADDRESS_FILE is <<<<<$collectoradr>>>>>\n",2);
-    	debug( "We are waiting for the file to exist\n",2);
-    	# Where is the collector address file? wait for it to exist
-    	my $havecollectoraddr = "no";
-    	while($havecollectoraddr ne "yes") {
-        	debug( "Looking for $collectoradr\n",2);
-        	if( -f $collectoradr ) {
-            	debug("Found it!!!! collector address file\n",2);
-            	$havecollectoraddr = "yes";
-        	} else {
-            	sleep 1;
-        	}
-    	}
-	}
-
-	if($daemonlist =~ /.*NEGOTIATOR.*/) {
-		# now wait for the negotiator to start running... get address file loc
-		# and wait for file to exist
-		# Give the master time to start before jobs are submitted.
-		my $negotiatoradr = `condor_config_val NEGOTIATOR_ADDRESS_FILE`;
-		$negotiatoradr =~ s/\012+$//;
-		$negotiatoradr =~ s/\015+$//;
-		debug( "NEGOTIATOR_ADDRESS_FILE is <<<<<$negotiatoradr>>>>>\n",2);
-    	debug( "We are waiting for the file to exist\n",2);
-    	# Where is the negotiator address file? wait for it to exist
-    	my $havenegotiatoraddr = "no";
-    	while($havenegotiatoraddr ne "yes") {
-        	debug( "Looking for $negotiatoradr\n",2);
-        	if( -f $negotiatoradr ) {
-            	debug("Found it!!!! negotiator address file\n",2);
-            	$havenegotiatoraddr = "yes";
-        	} else {
-            	sleep 1;
-        	}
-    	}
-	}
-
-	if($daemonlist =~ /.*STARTD.*/) {
-		# now wait for the startd to start running... get address file loc
-		# and wait for file to exist
-		# Give the master time to start before jobs are submitted.
-		my $startdadr = `condor_config_val STARTD_ADDRESS_FILE`;
-		$startdadr =~ s/\012+$//;
-		$startdadr =~ s/\015+$//;
-		debug( "STARTD_ADDRESS_FILE is <<<<<$startdadr>>>>>\n",2);
-    	debug( "We are waiting for the file to exist\n",2);
-    	# Where is the startd address file? wait for it to exist
-    	my $havestartdaddr = "no";
-    	while($havestartdaddr ne "yes") {
-        	debug( "Looking for $startdadr\n",2);
-        	if( -f $startdadr ) {
-            	debug("Found it!!!! startd address file\n",2);
-            	$havestartdaddr = "yes";
-        	} else {
-            	sleep 1;
-        	}
-    	}
-	}
-
-	if($daemonlist =~ /.*SCHEDD.*/) {
-		# now wait for the schedd to start running... get address file loc
-		# and wait for file to exist
-		# Give the master time to start before jobs are submitted.
-		my $scheddadr = `condor_config_val SCHEDD_ADDRESS_FILE`;
-		$scheddadr =~ s/\012+$//;
-		$scheddadr =~ s/\015+$//;
-		debug( "SCHEDD_ADDRESS_FILE is <<<<<$scheddadr>>>>>\n",2);
-    	debug( "We are waiting for the file to exist\n",2);
-    	# Where is the schedd address file? wait for it to exist
-    	my $havescheddaddr = "no";
-    	while($havescheddaddr ne "yes") {
-        	debug( "Looking for $scheddadr\n",2);
-        	if( -f $scheddadr ) {
-            	debug("Found it!!!! schedd address file\n",2);
-            	$havescheddaddr = "yes";
-        	} else {
-            	sleep 1;
-        	}
-    	}
-	}
-
-    return(1);
-}
-	
 # StartTestOutput($compiler,$test_program);
 sub StartTestOutput
 {
@@ -1549,9 +1391,6 @@ sub DoChild
 	# we know where the published directories are if we ask by name
 	# and they are relevant for the entire test time. We need ask
 	# and check only once.
-	#if($wantcorechecks) {
-		#CoreCheck($test_starttime);
-	#}
 	debug( "Test start @ $test_starttime \n",2);
 	sleep(1);
 	# add test core file
@@ -1562,9 +1401,9 @@ sub DoChild
 	my $save = $testname . ".saveme";
 	my $piddir = $save . "/$$";
 	# make sure pid storage directory exists
-	CondorTest::verbose_system("mkdir -p $save");
+	verbose_system("mkdir -p $save",{emit_output=>0});
 	my $pidcmd = "mkdir -p " . $save . "/" . "$$";
-	CondorTest::verbose_system("$pidcmd");
+	verbose_system("$pidcmd",{emit_output=>0});
 
 	my $log = $testname . ".log";
 	my $cmd = $testname . ".cmd";
@@ -1575,7 +1414,7 @@ sub DoChild
 
 	# before starting test clean trace of earlier run
 	my $rmcmd = "rm -f $log $out $err $runout $cmdout";
-	CondorTest::verbose_system("$rmcmd");
+	CondorTest::verbose_system("$rmcmd",{emit_output=>0});
 
 	my $newlog =  $piddir . "/" . $log;
 	my $newcmd =  $piddir . "/" . $cmd;
@@ -1620,14 +1459,6 @@ sub DoChild
 				#print "Perl test($test_program) returned <<$res>>!!! \n"; 
 				exit(1); 
 			}
-			if($wantcorechecks) {
-				# send off the name without the .run attached
-				$corecount = CoreCheck($testname);
-				if($corecount != 0) {
-					print "\n ************ -Core/ERROR found- *************\n";
-					exit(1);
-				}
-			}
 			exit(0);
 		};
 
@@ -1661,6 +1492,18 @@ sub DebugOff
 {
 }
 
+# yates_shuffle(\@foo) random shuffle of array
+sub yates_shuffle
+{
+    my $array = shift;
+    my $i;
+    for($i = @$array; --$i; ) {
+        my $j = int rand ($i+1);
+        next if $i == $j;
+        @$array[$i,$j] = @$array[$j,$i];
+    }
+}
+
 sub timestamp {
     return scalar localtime();
 }
@@ -1677,252 +1520,4 @@ sub safe_copy {
     }
 }
 
-sub CoreCheck {
-	my $test = shift;
-	# get the most recent list of tests and their log directories
-	CondorPubLogdirs::LoadPublished();
-	my $publishedarrayref = CondorPubLogdirs::ReturnPublished($test);
-	my $logdircount = $#{$publishedarrayref};
-	debug("In searching for logs for test <$test> count is <$logdircount>\n",2);
-	if($logdircount == -1) {
-		open(NOLOGS,">>LogDirsMissing") or die "Can not add that <$test> has no log dir:$!\n";
-		debug("No logs for test <$test>\n",2);
-		print NOLOGS "$test\n";
-		return(0);
-	}
-
-	my $count = 0;
-	my $scancount = 0;
-	my $fullpath = "";
-	foreach my $logdir (@{$publishedarrayref}) {
-		debug("Checking <$logdir> for test <$test>\n",2);
-		my @files = `ls $logdir`;
-		foreach my $perp (@files) {
-			chomp($perp);
-			$fullpath = $logdir . "/" . $perp;
-			if(-f $fullpath) {
-					if($fullpath =~ /^.*\/(core.*)$/) {
-						# returns printable string
-						debug("Checking <$logdir> for test <$test> Found Core <$fullpath>\n",2);
-						my $filechange = GetFileTime($fullpath);
-						my $newname = MoveCoreFile($fullpath,$coredir);
-						AddFileTrace($fullpath,$filechange,$newname);
-						$count += 1;
-					} else {
-						debug("Checking <$fullpath> for test <$test> for ERROR\n",2);
-						$scancount = ScanForERROR($fullpath,$test);
-						$count += $scancount;
-						debug("After ScanForERROR error count <$scancount>\n",2);
-					}
-			} else {
-				debug( "Not File: $fullpath\n",2);
-			}
-		}
-	}
-	return($count);
-}
-
-sub ScanForERROR
-{
-	my $daemonlog = shift;
-	my $testname = shift;
-	my $count = 0;
-	my $ignore = 1;
-	open(MDL,"<$daemonlog") or die "Can not open daemon log<$daemonlog>:$!\n";
-	my $line = "";
-	while(<MDL>) {
-		chomp();
-		$line = $_;
-		# ERROR preceeded by white space and trailed by white space, :, ; or -
-		if($line =~ /^\s*(\d+\/\d+\s+\d+:\d+:\d+)\s+ERROR[\s;:\-!].*$/){
-			debug("$line TStamp $1\n",2);
-			$ignore = IgnoreError($testname,$line);
-			if($ignore == 0) {
-				$count += 1;
-				AddFileTrace($daemonlog, $1, $line);
-			}
-		} elsif($line =~ /^\s*(\d+\/\d+\s+\d+:\d+:\d+)\s+.*\s+ERROR[\s;:\-!].*$/){
-			debug("$line TStamp $1\n",2);
-			$ignore = IgnoreError($testname,$line);
-			if($ignore == 0) {
-				$count += 1;
-				AddFileTrace($daemonlog, $1, $line);
-			}
-		} elsif($line =~ /^.*ERROR.*$/){
-			debug("Skipping this error<<$line>> \n",2);
-		}
-	}
-	close(MDL);
-	return($count);
-}
-
-sub CheckTriggerTime
-{	
-	my $teststartstamp = shift;
-	my $timestring = shift;
-	my $tsmon = 0;
-
-	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
-
-	if($timestring =~ /^(\d+)\/(\d+)\s+(\d+):(\d+):(\d+)$/) {
-		$tsmon = $1 - 1;
-		my $timeloc = timelocal($5,$4,$3,$mday,$tsmon,$year,0,0,$isdst);
-		#print "timestamp from test start is $teststartstamp\n";
-		#print "timestamp fromlocaltime is $timeloc \n";
-		if($timeloc > $teststartstamp) {
-			return(1);
-		}
-	}
-}
-
-sub GetFileChangeTime
-{
-	my $file = shift;
-	my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($file);
-
-	return($ctime);
-}
-
-sub GetFileTime
-{
-	my $file = shift;
-	my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($file);
-
-	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($ctime);
-
-	$mon = $mon + 1;
-	$year = $year + 1900;
-
-	return("$mon/$mday $hour:$min:$sec");
-}
-
-sub AddFileTrace
-{
-	my $file = shift;
-	my $time = shift;
-	my $entry = shift;
-
-	my $tracefile = $coredir . "/core_error_trace";
-	my $newtracefile = $coredir . "/core_error_trace.new";
-
-	# make sure the trace file exists
-	if(!(-f $tracefile)) {
-		open(TF,">$tracefile") or die "Can not create ERROR/CORE trace file<$tracefile>:$!\n";
-		print TF "Tracking file for core files and ERROR prints in daemonlogs\n";
-		close(TF);
-	}
-	open(TF,"<$tracefile") or die "Can not create ERROR/CORE trace file<$tracefile>:$!\n";
-	open(NTF,">$newtracefile") or die "Can not create ERROR/CORE trace file<$newtracefile>:$!\n";
-	while(<TF>) {
-		print NTF "$_";
-	}
-	close(TF);
-	my $buildentry = "$time	$file	$entry\n";
-	print NTF "$buildentry";
-	debug("\n$buildentry",2);
-	close(NTF);
-	system("mv $newtracefile $tracefile");
-
-}
-
-sub MoveCoreFile
-{
-	my $oldname = shift;
-	my $targetdir = shift;
-	my $newname = "";
-	# get number for core file rename into trace dir
-	my $entries = CountFileTrace();
-	if($oldname =~ /^.*\/(core.*)\s*.*$/) {
-		$newname = $coredir . "/" . $1 . "_$entries";
-		system("mv $oldname $newname");
-		#system("rm $oldname");
-		return($newname);
-	} else {
-		debug("Only move core files<$oldname>\n",2);
-		return("badmoverequest");
-	}
-}
-
-sub CountFileTrace
-{
-	my $tracefile = $coredir . "/core_error_trace";
-	my $count = 0;
-
-	open(CT,"<$tracefile") or die "Can not count<$tracefile>:$!\n";
-	while(<CT>) {
-		$count += 1;
-	}
-	close(CT);
-	return($count);
-}
-
-# yates_shuffle(\@foo) random shuffle of array
-sub yates_shuffle
-{
-    my $array = shift;
-    my $i;
-    for($i = @$array; --$i; ) {
-        my $j = int rand ($i+1);
-        next if $i == $j;
-        @$array[$i,$j] = @$array[$j,$i];
-    }
-}
-
-sub LoadExemptions
-{
-	my $line = "";
-	open(EE,"<$errexempts") or die "Can not open $errexempts:$!\n";
-	while(<EE>) {
-    	chomp();
-    	$line = $_;
-		debug("$line\n",2);
-    	my ($test, $required, $message) = split /,/, $line;
-    	my $save = $required . "," . $message;
-    	if(exists $exemptions{$test}) {
-        	push @{$exemptions{$test}}, $save;
-    	} else {
-        	$exemptions{$test} = ();
-        	push @{$exemptions{$test}}, $save;
-    	}
-	}
-	#DropExemptions();
-}
-
-sub IgnoreError
-{
-	my $testname = shift;
-	my $errorstring = shift;
-
-	# no no.... must acquire array for test and check all substrings
-	# against current large string.... see DropExemptions below
-	debug("IgnoreError called for test <$testname> and string <$errorstring>\n",2);
-	# get list of per/test specs
-	my @testarray = @{$exemptions{$testname}};
-	foreach my $oneexemption (@testarray) {
-		my( $must, $partialstr) = split /,/,  $oneexemption;
-		my $quoted = quotemeta($partialstr);
-		debug("Looking for <$quoted> in this error <$errorstring>\n",2);
-		if($errorstring =~ m/$quoted/) {
-			debug("IgnoreError: Ignore ******** <<$quoted>> ******** \n",2);
-			return(1);
-		} 
-	}
-	# no exemption for this one
-	return(0);
-}
-
-sub DropExemptions
-{
-	foreach my $key (sort keys %exemptions) {
-    	print "$key\n";
-    	my @array = @{$exemptions{$key}};
-    	foreach my $p (@array) {
-        	print "$p\n";
-    	}
-	}
-}
-
-### Local Variables: ***
-### mode:perl ***
-### tab-width: 4  ***
-### End: ***
+1;
