@@ -31,16 +31,19 @@
 #include "enum_utils.h"
 #include "condor_holdcodes.h"
 #include "classad_helpers.h"
+#include "classad_merge.h"
+#include "dc_startd.h"
 
 #include <math.h>
 
 // these are declared static in baseshadow.h; allocate space here
-UserLog BaseShadow::uLog;
+WriteUserLog BaseShadow::uLog;
 BaseShadow* BaseShadow::myshadow_ptr = NULL;
 
 
 // this appears at the bottom of this file:
-extern "C" int display_dprintf_header(FILE *fp);
+extern "C" int display_dprintf_header(char **buf,int *bufpos,int *buflen);
+extern bool sendUpdatesToSchedd;
 
 // some helper functions
 int getJobAdExitCode(ClassAd *jad, int &exit_code);
@@ -50,8 +53,6 @@ int getJobAdExitSignal(ClassAd *jad, int &exit_signal);
 BaseShadow::BaseShadow() {
 	spool = NULL;
 	fsDomain = uidDomain = NULL;
-	ckptServerHost = NULL;
-	useAFS = useNFS = useCkptServer = false;
 	jobAd = NULL;
 	remove_requested = false;
 	cluster = proc = -1;
@@ -63,12 +64,20 @@ BaseShadow::BaseShadow() {
 	myshadow_ptr = this;
 	exception_already_logged = false;
 	began_execution = FALSE;
+	reconnect_e_factor = 0.0;
+	reconnect_ceiling = 300;
+	prev_run_bytes_sent = 0.0;
+	prev_run_bytes_recvd = 0.0;
+	m_num_cleanup_retries = 0;
+	m_max_cleanup_retries = 5;
+	m_lazy_queue_update = true;
+	m_cleanup_retry_tid = -1;
+	m_cleanup_retry_delay = 30;
 }
 
 BaseShadow::~BaseShadow() {
 	if (spool) free(spool);
 	if (fsDomain) free(fsDomain);
-	if (ckptServerHost) free(ckptServerHost);
 	if (jobAd) FreeJobAd(jobAd);
 	if (gjid) free(gjid); 
 	if (scheddAddr) free(scheddAddr);
@@ -85,10 +94,10 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 	}
 	jobAd = job_ad;
 
-	if( ! is_valid_sinful(schedd_addr) ) {
+	if (sendUpdatesToSchedd && ! is_valid_sinful(schedd_addr)) {
 		EXCEPT("schedd_addr not specified with valid address");
 	}
-	scheddAddr = strdup( schedd_addr );
+	scheddAddr = sendUpdatesToSchedd ? strdup( schedd_addr ) : strdup("noschedd");
 
 	m_xfer_queue_contact_info = xfer_queue_contact_info;
 
@@ -151,11 +160,6 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 		// Make sure we've got enough swap space to run
 	checkSwap();
 
-		// register SIGUSR1 (condor_rm) for shutdown...
-	daemonCore->Register_Signal( SIGUSR1, "SIGUSR1", 
-		(SignalHandlercpp)&BaseShadow::handleJobRemoval, "HandleJobRemoval", 
-		this);
-
 	// handle system calls with Owner's privilege
 // XXX this belong here?  We'll see...
 	if ( !init_user_ids(owner.Value(), domain.Value())) {
@@ -175,19 +179,20 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 		// permanent job queue.  this clears all the dirty bits on our
 		// copy of the classad, so anything we touch after this will
 		// be updated to the schedd when appropriate.
-	job_updater = new QmgrJobUpdater( jobAd, scheddAddr );
+
+		// Unless we got a command line arg asking us not to
+	if (sendUpdatesToSchedd) {
+		// the usual case
+		job_updater = new QmgrJobUpdater( jobAd, scheddAddr, CondorVersion() );
+	} else {
+		job_updater = new NullQmgrJobUpdater( jobAd, scheddAddr, CondorVersion() );
+	}
 
 		// change directory; hold on failure
 	if ( cdToIwd() == -1 ) {
 		EXCEPT("Could not cd to initial working directory");
 	}
 
-		// CRUFT
-		// we want this *after* we clear the dirty flags so that if we
-		// change anything, we consider that change dirty so it'll get
-		// updated the next time we connect to the job queue...
-	checkFileTransferCruft();
-
 		// check to see if this invocation of the shadow is just to write
 		// a terminate event and exit since this job had been recorded as
 		// pending termination, but somehow the job didn't leave the queue
@@ -200,89 +205,49 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 			this->terminateJob(US_TERMINATE_PENDING);
 		}
 	}
+
+		// If we need to claim the startd before activating the claim
+	int wantClaiming = 0;
+	jobAd->LookupBool(ATTR_CLAIM_STARTD, wantClaiming);
+	if (wantClaiming) {
+		MyString startdSinful;
+		MyString claimid;
+
+			// Pull startd addr and claimid out of the jobad
+		jobAd->LookupString(ATTR_STARTD_IP_ADDR, startdSinful);
+		jobAd->LookupString(ATTR_CLAIM_ID, claimid);
+
+		dprintf(D_ALWAYS, "%s is true, trying to claim startd %s\n", ATTR_CLAIM_STARTD, startdSinful.Value());
+
+		classy_counted_ptr<DCStartd> startd = new DCStartd("description", NULL, startdSinful.Value(), claimid.Value());
+	
+		classy_counted_ptr<DCMsgCallback> cb = 
+			new DCMsgCallback((DCMsgCallback::CppFunction)&BaseShadow::startdClaimedCB,
+			this, jobAd);
+																 
+			// this can't fail, will always call the callback
+		startd->asyncRequestOpportunisticClaim(jobAd, 
+											   "description", 
+											   daemonCore->InfoCommandSinfulString(), 
+											   1200 /*alive interval*/, 
+											   20 /* net timeout*/, 
+											   100 /*total timeout*/, 
+											   cb);
+	}
 }
 
+	// We land in this callback when we need to claim the startd
+	// when we get here, the claiming is finished, successful
+	// or not
+void BaseShadow::startdClaimedCB(DCMsgCallback *) {
 
-void
-BaseShadow::checkFileTransferCruft()
-{
-		/*
-		  If this job was a) submitted by a pre-6.3.3 condor_submit,
-		  b) unix and c) vanilla, it was submitted with an incorrect
-		  default value of "ON_EXIT" for ATTR_TRANSFER_FILES.  So, if
-		  all of those conditions are met, we want to change the value
-		  to be "NEVER" instead, so that we treat this like the old
-		  shadow would treat it and rely on a shared file system.
-		*/
-#ifndef WIN32
-	int universe; 
-	char* version = NULL;
-	bool is_old = false;
-	if( ! jobAd->LookupInteger(ATTR_JOB_UNIVERSE, universe) ) {
-		universe = CONDOR_UNIVERSE_VANILLA;
-	}
-	if( universe != CONDOR_UNIVERSE_VANILLA ) {
-			// nothing to do
-		return;
-	}
-	jobAd->LookupString( ATTR_VERSION, &version );
-	if( version ) {
-		CondorVersionInfo ver( version, "JOB" );
-		if( ! ver.built_since_version(6,3,3) ) {
-			is_old = true;
-		}
-		free( version );
-		version = NULL;
-	} else {
-		dprintf( D_FULLDEBUG, "Job has no %s, assuming pre version 6.3.3\n",
-				 ATTR_VERSION ); 
-		is_old = true;
-	}	
-	if( ! is_old ) {
-			// if we're new enough, nothing else to do
-		return;
-	}
-
-		// see if ATTR_TRANSFER_FILES is already set to "NEVER"... 
-	bool already_never;
-	char* tmp = NULL;
-	jobAd->LookupString( ATTR_TRANSFER_FILES, &tmp );
-	if( tmp ) {
-		already_never = ( stricmp(tmp, "NEVER") == 0 );
-		free( tmp );
-		if( already_never ) {
-				// already have the right value, don't bother changing
-				// it and updating the job queue, etc.
-			return;
-		}
-	}
-
-		// if we're still here, we've hit the nasty case, so change
-		// the value...
-	MyString new_attr;
-	new_attr += ATTR_TRANSFER_FILES;
-	new_attr += " = \"NEVER\"";
-
-	dprintf( D_FULLDEBUG, "Unix Vanilla job is pre version 6.3.3, "
-			 "setting '%s'\n", new_attr.Value() );
-
-	if( ! jobAd->Insert(new_attr.Value()) ) {
-		EXCEPT( "Insert of '%s' into job ad failed!", new_attr.Value() );
-	}
-
-		// also, add it to the list of attributes we want to update,
-		// so we change it in the job queue, too.
-	job_updater->watchAttribute( ATTR_TRANSFER_FILES );
-
-#endif /* ! WIN32 */
-
+	// We've claimed the startd, the following kicks off the
+	// activation of the claim, and runs the job
+	this->spawn();
 }
 
-
 void BaseShadow::config()
 {
-	char *tmp;
-
 	if (spool) free(spool);
 	spool = param("SPOOL");
 	if (!spool) {
@@ -301,37 +266,11 @@ void BaseShadow::config()
 		EXCEPT("UID_DOMAIN not specified in config file.");
 	}
 
-	tmp = param("USE_AFS");
-	if (tmp && (tmp[0] == 'T' || tmp[0] == 't')) {
-		useAFS = true;
-	} else {
-		useAFS = false;
-	}
-	if (tmp) free(tmp);
-
-	tmp = param("USE_NFS");
-	if (tmp && (tmp[0] == 'T' || tmp[0] == 't')) {
-		useNFS = true;
-	} else {
-		useNFS = false;
-	}
-	if (tmp) free(tmp);
-
-	if (ckptServerHost) free(ckptServerHost);
-	ckptServerHost = param("CKPT_SERVER_HOST");
-	tmp = param("USE_CKPT_SERVER");
-	if (tmp && ckptServerHost && (tmp[0] == 'T' || tmp[0] == 't')) {
-		useCkptServer = true;
-	} else {
-		useCkptServer = false;
-	}
-	if (tmp) free(tmp);
-
 	reconnect_ceiling = param_integer( "RECONNECT_BACKOFF_CEILING", 300 );
 
 	reconnect_e_factor = 0.0;
 	reconnect_e_factor = param_double( "RECONNECT_BACKOFF_FACTOR", 2.0, 0.0 );
-	if( !reconnect_e_factor ) {
+	if( reconnect_e_factor < -1e-4 || reconnect_e_factor > 1e-4) {
     	reconnect_e_factor = 2.0;
     }
 
@@ -452,6 +391,51 @@ BaseShadow::holdJob( const char* reason, int hold_reason_code, int hold_reason_s
 	DC_Exit( JOB_SHOULD_HOLD );
 }
 
+void
+BaseShadow::mockTerminateJob( MyString exit_reason, 
+		bool exited_by_signal, int exit_code, int exit_signal, 
+		bool core_dumped )
+{
+	if (exit_reason == "") {
+		exit_reason = "Exited normally";
+	}
+	
+	dprintf( D_ALWAYS, "Mock terminating job %d.%d: "
+			"exited_by_signal=%s, exit_code=%d OR exit_signal=%d, "
+			"core_dumped=%s, exit_reason=\"%s\"\n", 
+			 getCluster(),
+			 getProc(), 
+			 exited_by_signal ? "TRUE" : "FALSE",
+			 exit_code,
+			 exit_signal,
+			 core_dumped ? "TRUE" : "FALSE",
+			 exit_reason.Value());
+
+	if( ! jobAd ) {
+		dprintf(D_ALWAYS, "BaseShadow::mockTerminateJob(): NULL JobAd! "
+			"Holding Job!");
+		DC_Exit( JOB_SHOULD_HOLD );
+	}
+
+	// Insert the various exit attributes into our job ad.
+	jobAd->Assign( ATTR_JOB_CORE_DUMPED, core_dumped );
+	jobAd->Assign( ATTR_ON_EXIT_BY_SIGNAL, exited_by_signal );
+
+	if (exited_by_signal) {
+		jobAd->Assign( ATTR_ON_EXIT_SIGNAL, exit_signal );
+	} else {
+		jobAd->Assign( ATTR_ON_EXIT_CODE, exit_code );
+	}
+
+	jobAd->Assign( ATTR_EXIT_REASON, exit_reason );
+
+		// update the job queue for the attributes we care about
+	if( !updateJobInQueue(U_TERMINATE) ) {
+			// trouble!  TODO: should we do anything else?
+		dprintf( D_ALWAYS, "Failed to update job queue!\n" );
+	}
+}
+
 
 void
 BaseShadow::removeJob( const char* reason )
@@ -613,10 +597,19 @@ BaseShadow::terminateJob( update_style_t kind ) // has a default argument of US_
     if( int_value > 0 ) {
         int job_committed_time = 0;
         jobAd->LookupInteger(ATTR_JOB_COMMITTED_TIME, job_committed_time);
-        job_committed_time += (int)time(NULL) - int_value;
+		int delta = (int)time(NULL) - int_value;
+        job_committed_time += delta;
         jobAd->Assign(ATTR_JOB_COMMITTED_TIME, job_committed_time);
+
+		float slot_weight = 1;
+		jobAd->LookupFloat(ATTR_JOB_MACHINE_ATTR_SLOT_WEIGHT0, slot_weight);
+		float slot_time = 0;
+		jobAd->LookupFloat(ATTR_COMMITTED_SLOT_TIME, slot_time);
+		slot_time += slot_weight * delta;
+		jobAd->Assign(ATTR_COMMITTED_SLOT_TIME, slot_time);
     }
 
+	CommitSuspensionTime(jobAd);
 
 	// update the job ad in the queue with some important final
 	// attributes so we know what happened to the job when using
@@ -655,6 +648,12 @@ BaseShadow::terminateJob( update_style_t kind ) // has a default argument of US_
 		reason = JOB_EXITED_AND_CLAIM_CLOSING;
 	}
 
+	// try to get a new job for this shadow
+	if( recycleShadow(reason) ) {
+		// recycleShadow delete's this, so we must return immediately
+		return;
+	}
+
 	// does not return.
 	DC_Exit( reason );
 }
@@ -770,8 +769,13 @@ void BaseShadow::initUserLog()
 {
 	MyString logfilename;
 	int  use_xml;
+	bool result;
+
 	if ( getPathToUserLog(jobAd, logfilename) ) {
-		uLog.initialize (owner.Value(), domain.Value(), logfilename.Value(), cluster, proc, 0, gjid);
+		result = uLog.initialize (owner.Value(), domain.Value(), logfilename.Value(), cluster, proc, 0, gjid);
+		if ( result == false ) {
+			EXCEPT("Failed to initialize user log to %s",logfilename.Value());
+		}
 		if (jobAd->LookupBool(ATTR_ULOG_USE_XML, use_xml)
 			&& use_xml) {
 			uLog.setUseXML(true);
@@ -831,7 +835,7 @@ BaseShadow::logTerminateEvent( int exitReason, update_style_t kind )
 		break;
 	default:
 		dprintf( D_ALWAYS, 
-				 "logTerminateEvent with unknown reason (%d), aborting\n",
+				 "UserLog logTerminateEvent with unknown reason (%d), aborting\n",
 				 exitReason ); 
 		return;
 	}
@@ -885,6 +889,7 @@ BaseShadow::logTerminateEvent( int exitReason, update_style_t kind )
 		if (!uLog.writeEvent (&event,jobAd)) {
 			dprintf (D_ALWAYS,"Unable to log "
 				 	"ULOG_JOB_TERMINATED event\n");
+			EXCEPT("UserLog Unable to log ULOG_JOB_TERMINATED event");
 		}
 
 		return;
@@ -926,6 +931,7 @@ BaseShadow::logTerminateEvent( int exitReason, update_style_t kind )
 	if (!uLog.writeEvent (&event,jobAd)) {
 		dprintf (D_ALWAYS,"Unable to log "
 				 "ULOG_JOB_TERMINATED event\n");
+		EXCEPT("UserLog Unable to log ULOG_JOB_TERMINATED event");
 	}
 }
 
@@ -1026,7 +1032,7 @@ BaseShadow::checkSwap( void )
 {
 	int	reserved_swap, free_swap;
 		// Reserved swap is specified in megabytes
-	reserved_swap = param_integer( "RESERVED_SWAP", 5 );
+	reserved_swap = param_integer( "RESERVED_SWAP", 0 );
 	reserved_swap *= 1024;
 
 	if( reserved_swap == 0 ) {
@@ -1077,7 +1083,7 @@ BaseShadow::log_except(const char *msg)
 		event.sent_bytes = 0.0;
 	}
 
-	if (!exception_already_logged && !uLog.writeEvent (&event,NULL))
+	if (!exception_already_logged && !uLog.writeEventNoFsync (&event,NULL))
 	{
 		::dprintf (D_ALWAYS, "Unable to log ULOG_SHADOW_EXCEPTION event\n");
 	}
@@ -1085,16 +1091,16 @@ BaseShadow::log_except(const char *msg)
 
 
 bool
-BaseShadow::updateJobAttr( const char *name, const char *expr )
+BaseShadow::updateJobAttr( const char *name, const char *expr, bool log )
 {
-	return job_updater->updateAttr( name, expr, false );
+	return job_updater->updateAttr( name, expr, false, log );
 }
 
 
 bool
-BaseShadow::updateJobAttr( const char *name, int value )
+BaseShadow::updateJobAttr( const char *name, int value, bool log )
 {
-	return job_updater->updateAttr( name, value, false );
+	return job_updater->updateAttr( name, value, false, log );
 }
 
 
@@ -1227,14 +1233,14 @@ BaseShadow::publishShadowAttrs( ClassAd* ad )
 }
 
 
-void BaseShadow::dprintf_va( int flags, char* fmt, va_list args )
+void BaseShadow::dprintf_va( int flags, const char* fmt, va_list args )
 {
 		// Print nothing in this version.  A subclass like MPIShadow
 		// might like to say ::dprintf( flags, "(res %d)"
 	::_condor_dprintf_va( flags, fmt, args );
 }
 
-void BaseShadow::dprintf( int flags, char* fmt, ... )
+void BaseShadow::dprintf( int flags, const char* fmt, ... )
 {
 	va_list args;
 	va_start( args, fmt );
@@ -1251,28 +1257,28 @@ extern BaseShadow *Shadow;
 // and pid in our log entries. 
 extern "C" 
 int
-display_dprintf_header(FILE *fp)
+display_dprintf_header(char **buf,int *bufpos,int *buflen)
 {
 	static pid_t mypid = 0;
-	static int mycluster = -1;
-	static int myproc = -1;
+	int mycluster = -1;
+	int myproc = -1;
 
 	if (!mypid) {
 		mypid = daemonCore->getpid();
 	}
 
-	if (mycluster == -1) {
+	if (Shadow) {
 		mycluster = Shadow->getCluster();
 		myproc = Shadow->getProc();
 	}
 
 	if ( mycluster != -1 ) {
-		fprintf( fp, "(%d.%d) (%ld): ", mycluster, myproc, (long)mypid );
+		return sprintf_realloc( buf, bufpos, buflen, "(%d.%d) (%ld): ", mycluster, myproc, (long)mypid );
 	} else {
-		fprintf( fp, "(?.?) (%ld): ", (long)mypid );
+		return sprintf_realloc( buf, bufpos, buflen, "(?.?) (%ld): ", (long)mypid );
 	}	
 
-	return TRUE;
+	return 0;
 }
 
 bool
@@ -1280,3 +1286,33 @@ BaseShadow::getMachineName( MyString & /*machineName*/ )
 {
 	return false;
 }
+
+void
+BaseShadow::CommitSuspensionTime(ClassAd *jobAd)
+{
+	int uncommitted_suspension_time = 0;
+	jobAd->LookupInteger(ATTR_UNCOMMITTED_SUSPENSION_TIME,uncommitted_suspension_time);
+	if( uncommitted_suspension_time > 0 ) {
+		int committed_suspension_time = 0;
+		jobAd->LookupInteger( ATTR_COMMITTED_SUSPENSION_TIME,
+							  committed_suspension_time );
+		committed_suspension_time += uncommitted_suspension_time;
+		jobAd->Assign( ATTR_COMMITTED_SUSPENSION_TIME, committed_suspension_time );
+		jobAd->Assign( ATTR_UNCOMMITTED_SUSPENSION_TIME, 0 );
+	}
+}
+
+int
+BaseShadow::handleUpdateJobAd( int sig )
+{
+	dprintf ( D_FULLDEBUG, "In handleUpdateJobAd, sig %d\n", sig );
+	if (!job_updater->retrieveJobUpdates()) {
+		dprintf(D_ALWAYS, "Error: Failed to update JobAd\n");
+		return -1;
+	}
+
+	// Attributes might have changed that would cause the job policy
+	// to evaluate differently, so evaluate now.
+	shadow_user_policy.checkPeriodic();
+	return 0;
+}
