@@ -28,7 +28,7 @@
 #include "write_user_log.h"
 #include "exit.h"
 #include "internet.h"
-#include "qmgr_job_updater.h"
+#include "../condor_schedd.V6/qmgr_job_updater.h"
 #include "condor_update_style.h"
 
 /* Forward declaration to prevent loops... */
@@ -40,7 +40,7 @@ class RemoteResource;
 	make sure you know how these classes interact.  If it's a general
 	shadow thing, you very well may want to add it to this class.  
 	However, if it is specific to one remote resource, you want to
-	look at RemoteResource.  If it is single, MPI, or PVM (one day!)
+	look at RemoteResource.  If it is single, MPI
 	specific, make the change in that derived class. <p>
 
 	More to come...<p>
@@ -113,6 +113,8 @@ class BaseShadow : public Service
 		*/
 	void reconnectFailed( const char* reason ); 
 
+	virtual bool shouldAttemptReconnect(RemoteResource *) { return true;};
+
 		/** Here, we param for lots of stuff in the config file.  Things
 			param'ed for are: SPOOL, FILESYSTEM_DOMAIN, UID_DOMAIN, 
 			USE_AFS, USE_NFS, and CKPT_SERVER_HOST.
@@ -174,6 +176,19 @@ class BaseShadow : public Service
 		*/
 	void terminateJob( update_style_t kind = US_NORMAL );
 
+		/** The job exited, but it _isn't_ ready to leave the queue.
+
+			For example, the job segfaulted and didn't produce all of the
+			output files specifically labeled in transfer_output_files, so
+			this causes file transfer to fail which puts the job on hold
+			(as of 7.4.4). However, the user would like to write periodic or
+			exit policies about the job and for that we need to know exactly
+			how the job exited. So this call places how the job exited into
+			the jobad, but doesn't write any events about it.
+		*/
+	void mockTerminateJob( MyString exit_reason, bool exited_by_signal, 
+		int exit_code, int exit_signal, bool core_dumped );
+
 		/** Set a timer to call terminateJob() so we retry
 		    our attempts to update the job queue etc.
 		*/
@@ -215,6 +230,8 @@ class BaseShadow : public Service
 
 	virtual bool claimIsClosing( void ) = 0;
 
+	static void CommitSuspensionTime(ClassAd *jobAd);
+
 		/** Initializes the user log.  'Nuff said. 
 		 */
 	void initUserLog();
@@ -229,6 +246,10 @@ class BaseShadow : public Service
 		 */
 	virtual int handleJobRemoval(int sig) = 0;
 
+		/** Update this job.
+		 */
+	int handleUpdateJobAd(int sig);
+
 		/** This function returns a file pointer that one can 
 			write an email message into.
 			@return A mail message file pointer.
@@ -239,11 +260,11 @@ class BaseShadow : public Service
 			after the header and before the text of a dprintf
 			message.
 		*/
-	virtual void dprintf_va( int flags, char* fmt, va_list args );
+	virtual void dprintf_va( int flags, const char* fmt, va_list args );
 
 		/** A local dprintf maker that uses dprintf_va...
 		 */
-	void dprintf( int flags, char* fmt, ... ) CHECK_PRINTF_FORMAT(3,4);
+	void dprintf( int flags, const char* fmt, ... ) CHECK_PRINTF_FORMAT(3,4);
 
 		/// Returns the jobAd for this job
 	ClassAd *getJobAd() { return jobAd; }
@@ -303,10 +324,10 @@ class BaseShadow : public Service
 	bool updateJobInQueue( update_t type );
 
 		/** Connect to the job queue and update one attribute */
-	virtual bool updateJobAttr( const char *name, const char *expr );
+	virtual bool updateJobAttr( const char *name, const char *expr, bool log=false );
 
 		/** Connect to the job queue and update one integer attribute */
-	virtual bool updateJobAttr( const char *name, int value );
+	virtual bool updateJobAttr( const char *name, int value, bool log=false );
 
 		/** Do whatever cleanup (like killing starter(s)) that's
 			required before the shadow can exit.
@@ -331,7 +352,7 @@ class BaseShadow : public Service
 	virtual int exitCode( void ) = 0;
 
 		// make UserLog static so it can be accessed by EXCEPTION handler
-	static UserLog uLog;
+	static WriteUserLog uLog;
 
 	void evalPeriodicUserPolicy( void );
 
@@ -389,6 +410,7 @@ class BaseShadow : public Service
 
 	virtual void emailTerminateEvent( int exitReason, update_style_t kind = US_NORMAL ) = 0;
 
+	void startdClaimedCB(DCMsgCallback *cb);
 	bool m_lazy_queue_update;
 
  private:
@@ -400,22 +422,10 @@ class BaseShadow : public Service
 		*/
 	void checkSwap( void );
 
-		/** See if the job is a) vanilla, b) unix and c) submitted
-			with a 6.3.1 or earlier condor_submit.  if so, it'll have
-			an incorrect default value for ATTR_TRANSFER_FILES of
-			"ON_EXIT" that will cause all sorts of problems.  If all
-			those conditions are met, we change the value to "NEVER".
-		*/
-	void checkFileTransferCruft( void );
-
 	// config file parameters
 	char *spool;
 	char *fsDomain;
 	char *uidDomain;
-	char *ckptServerHost;
-	bool useAFS;
-	bool useNFS;
-	bool useCkptServer;
 	int reconnect_ceiling;
 	double reconnect_e_factor;
 
@@ -427,7 +437,6 @@ class BaseShadow : public Service
 	MyString domain;
 	MyString iwd;
 	char *scheddAddr;
-	bool jobExitedGracefully;
 	char *core_file_name;
 	MyString m_xfer_queue_contact_info;
 
@@ -451,6 +460,11 @@ class BaseShadow : public Service
 
 extern void dumpClassad( const char*, ClassAd*, int );
 
+// Register the shadow "exit status" for the previous job
+// and restart this shadow with a new job.
+// Returns false if no new job found.
+extern bool recycleShadow(int previous_job_exit_reason);
+
 extern BaseShadow *Shadow;
 
 #endif
