@@ -51,7 +51,10 @@ MPIShadow::~MPIShadow() {
     for ( int i=0 ; i<=ResourceList.getlast() ; i++ ) {
         delete ResourceList[i];
     }
-#if ! MPI_USES_RSH
+	daemonCore->Cancel_Command( SHADOW_UPDATEINFO );
+#if MPI_USES_RSH
+	daemonCore->Cancel_Command( MPI_START_COMRADE );
+#else
 	if( master_addr ) {
 		free( master_addr );
 	}
@@ -103,31 +106,23 @@ MPIShadow::init( ClassAd* job_ad, const char* schedd_addr, const char *xfer_queu
         // make first remote resource the "master".  Put it first in list.
     MpiResource *rr = new MpiResource( this );
 
-    ClassAd *temp = new ClassAd( *(getJobAd() ) );
-
     sprintf( buf, "%s = %s", ATTR_MPI_IS_MASTER, "TRUE" );
-    if( !temp->Insert(buf) ) {
+    if( !job_ad->Insert(buf) ) {
         dprintf( D_ALWAYS, "Failed to insert %s into jobAd.\n", buf );
         shutDown( JOB_NOT_STARTED );
     }
 
-	replaceNode( temp, 0 );
+	replaceNode( job_ad, 0 );
 	rr->setNode( 0 );
 	sprintf( buf, "%s = 0", ATTR_NODE );
-	temp->InsertOrUpdate( buf );
-    rr->setJobAd( temp );
+	job_ad->InsertOrUpdate( buf );
+    rr->setJobAd( job_ad );
 
-	rr->setStartdInfo( temp );
+	rr->setStartdInfo( job_ad );
 
-	temp->Assign( ATTR_JOB_STATUS, RUNNING );
+	job_ad->Assign( ATTR_JOB_STATUS, RUNNING );
     ResourceList[ResourceList.getlast()+1] = rr;
 
-		// now, we want to re-initialize the shadow_user_policy object
-		// with the ClassAd for our master node, since the one sitting
-		// in the Shadow object itself will never get updated with
-		// exit status, info about the run, etc, etc.
-	shadow_user_policy.init( temp, this );
-
 }
 
 
@@ -192,7 +187,6 @@ MPIShadow::getResources( void )
     int numProcs=0;    // the # of procs to come
     int numInProc=0;   // the # in a particular proc.
 	ClassAd *job_ad = NULL;
-	ClassAd *tmp_ad = NULL;
 	int nodenum = 1;
 	ReliSock* sock;
 
@@ -270,6 +264,7 @@ MPIShadow::getResources( void )
                 free( claim_id );
                 host = NULL;
                 claim_id = NULL;
+				delete job_ad;
                 continue;
             }
 
@@ -280,15 +275,14 @@ MPIShadow::getResources( void )
 				// hostname... 
 			rr->setMachineName( host );
 
-			tmp_ad = new ClassAd ( *job_ad );
-			replaceNode ( tmp_ad, nodenum );
+			replaceNode ( job_ad, nodenum );
 			rr->setNode( nodenum );
 			sprintf( buf, "%s = %d", ATTR_NODE, nodenum );
-			tmp_ad->InsertOrUpdate( buf );
+			job_ad->InsertOrUpdate( buf );
 			sprintf( buf, "%s = \"%s\"", ATTR_MY_ADDRESS,
 					 daemonCore->InfoCommandSinfulString() );
-			tmp_ad->InsertOrUpdate( buf );
-			rr->setJobAd( tmp_ad );
+			job_ad->InsertOrUpdate( buf );
+			rr->setJobAd( job_ad );
 			nodenum++;
 
             ResourceList[ResourceList.getlast()+1] = rr;
@@ -300,9 +294,6 @@ MPIShadow::getResources( void )
             claim_id = NULL;
 
         } // end of for loop for this proc
-        
-		delete job_ad;
-		job_ad = NULL;
 
     } // end of for loop on all procs...
 
@@ -971,22 +962,16 @@ MPIShadow::handleJobRemoval( int sig ) {
 void
 MPIShadow::replaceNode ( ClassAd *ad, int nodenum ) {
 
-	ExprTree *tree = NULL, *rhs = NULL, *lhs = NULL;
+	ExprTree *tree = NULL;
 	char node[9];
+	const char *lhstr, *rhstr;
 
 	sprintf( node, "%d", nodenum );
 
 	ad->ResetExpr();
-	while( (tree = ad->NextExpr()) ) {
-		MyString rhstr;
-		MyString lhstr;
-		if( (lhs = tree->LArg()) ) {
-			lhs->PrintToStr (lhstr);
-		}
-		if( (rhs = tree->RArg()) ) {
-			rhs->PrintToStr (rhstr);
-		}
-		if( !lhs || !rhs ) {
+	while( ad->NextExpr(lhstr, tree) ) {
+		rhstr = ExprTreeToString(tree);
+		if( !lhstr || !rhstr ) {
 			dprintf( D_ALWAYS, "Could not replace $(NODE) in ad!\n" );
 			return;
 		}
@@ -994,9 +979,9 @@ MPIShadow::replaceNode ( ClassAd *ad, int nodenum ) {
 		MyString strRh(rhstr);
 		if (strRh.replaceString("#MpInOdE#", node))
 		{
-			ad->AssignExpr(lhstr.Value(), strRh.Value());
+			ad->AssignExpr(lhstr, strRh.Value());
 			dprintf( D_FULLDEBUG, "Replaced $(NODE), now using: %s = %s\n", 
-					 lhstr.Value(), strRh.Value() );
+					 lhstr, strRh.Value() );
 		}
 	}	
 }
