@@ -30,7 +30,7 @@
 #include "daemon.h"
 #include "env.h"
 #include "condor_config.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "condor_claimid_parser.h"
 
 RemoteResource *parallelMasterResource = NULL;
@@ -52,6 +52,7 @@ ParallelShadow::~ParallelShadow() {
     for ( int i=0 ; i<=ResourceList.getlast() ; i++ ) {
         delete ResourceList[i];
     }
+	daemonCore->Cancel_Command( SHADOW_UPDATEINFO );
 }
 
 void 
@@ -81,39 +82,32 @@ ParallelShadow::init( ClassAd* job_ad, const char* schedd_addr, const char *xfer
     MpiResource *rr = new MpiResource( this );
 	parallelMasterResource = rr;
 
-    ClassAd *temp = new ClassAd( *(getJobAd() ) );
-
-
 	char buffer[1024];
-	sprintf (buffer, "%s = \"%s%c%s\"", ATTR_REMOTE_SPOOL_DIR, 
-		param("SPOOL"), DIR_DELIM_CHAR, 
-		gen_ckpt_name(0, getCluster(), 0, 0));
-	temp->Insert(buffer);
+	char *lspool = param("SPOOL");
+	char *dir = gen_ckpt_name(lspool, getCluster(), 0, 0);
+	free(lspool);
+	job_ad->Assign(ATTR_REMOTE_SPOOL_DIR,dir);
+	free(dir); dir = NULL;
+	job_ad->Insert(buffer);
 
-    sprintf( buf, "%s = %s", ATTR_MPI_IS_MASTER, "TRUE" );
-    if( !temp->Insert(buf) ) {
+    snprintf( buf, 256, "%s = %s", ATTR_MPI_IS_MASTER, "TRUE" );
+    if( !job_ad->Insert(buf) ) {
         dprintf( D_ALWAYS, "Failed to insert %s into jobAd.\n", buf );
         shutDown( JOB_NOT_STARTED );
     }
 
-	replaceNode( temp, 0 );
+	replaceNode( job_ad, 0 );
 	rr->setNode( 0 );
-	sprintf( buf, "%s = 0", ATTR_NODE );
-	temp->InsertOrUpdate( buf );
-    rr->setJobAd( temp );
+	snprintf( buf, 256, "%s = 0", ATTR_NODE );
+	job_ad->InsertOrUpdate( buf );
+    rr->setJobAd( job_ad );
 
-	rr->setStartdInfo( temp );
+	rr->setStartdInfo( job_ad );
 
-	temp->Assign( ATTR_JOB_STATUS, RUNNING );
+	job_ad->Assign( ATTR_JOB_STATUS, RUNNING );
 
     ResourceList[ResourceList.getlast()+1] = rr;
 
-		// now, we want to re-initialize the shadow_user_policy object
-		// with the ClassAd for our master node, since the one sitting
-		// in the Shadow object itself will never get updated with
-		// exit status, info about the run, etc, etc.
-	shadow_user_policy.init( temp, this );
-
 	shutdownPolicy = ParallelShadow::WAIT_FOR_NODE0;
 
 	MyString policy;
@@ -125,6 +119,19 @@ ParallelShadow::init( ClassAd* job_ad, const char* schedd_addr, const char *xfer
 }
 
 
+bool
+ParallelShadow::shouldAttemptReconnect(RemoteResource *r) {
+	if (shutdownPolicy == WAIT_FOR_ALL) {
+		return true;
+	}
+
+	if ((shutdownPolicy == WAIT_FOR_NODE0) && (r != ResourceList[0])) {
+		return false;
+	}
+
+	return true;
+}
+
 void
 ParallelShadow::reconnect( void )
 {
@@ -197,7 +204,6 @@ ParallelShadow::getResources( void )
     int numProcs=0;    // the # of procs to come
     int numInProc=0;   // the # in a particular proc.
 	ClassAd *job_ad = NULL;
-	ClassAd *tmp_ad = NULL;
 	int nodenum = 1;
 	ReliSock* sock;
 
@@ -276,6 +282,7 @@ ParallelShadow::getResources( void )
                 free( claim_id );
                 host = NULL;
                 claim_id = NULL;
+				delete job_ad;
                 continue;
             }
 
@@ -286,22 +293,28 @@ ParallelShadow::getResources( void )
 				// hostname... 
 			rr->setMachineName( host );
 
-			tmp_ad = new ClassAd ( *job_ad );
-			replaceNode ( tmp_ad, nodenum );
+			replaceNode ( job_ad, nodenum );
 			rr->setNode( nodenum );
-			sprintf( buf, "%s = %d", ATTR_NODE, nodenum );
-			tmp_ad->InsertOrUpdate( buf );
-			sprintf( buf, "%s = \"%s\"", ATTR_MY_ADDRESS,
+			snprintf( buf, 128, "%s = %d", ATTR_NODE, nodenum );
+			job_ad->InsertOrUpdate( buf );
+			snprintf( buf, 128, "%s = \"%s\"", ATTR_MY_ADDRESS,
 					 daemonCore->InfoCommandSinfulString() );
-			tmp_ad->InsertOrUpdate( buf );
+			job_ad->InsertOrUpdate( buf );
 
 			char buffer[1024];
-			sprintf (buffer, "%s = \"%s%c%s\"", ATTR_REMOTE_SPOOL_DIR, 
-				param("SPOOL"), DIR_DELIM_CHAR, 
-				gen_ckpt_name(0, job_cluster, 0, 0));
-			tmp_ad->Insert(buffer);
+			char *lspool = param("SPOOL");
+			char *dir = gen_ckpt_name(lspool, job_cluster, 0, 0);
+			free(lspool);
+			job_ad->Assign(ATTR_REMOTE_SPOOL_DIR, dir);
+			free(dir); dir = NULL;
+			job_ad->Insert(buffer);
 
-			rr->setJobAd( tmp_ad );
+				// Put the correct claim id into this ad's ClaimId attribute.
+				// Otherwise, it is the claim id of the master proc.
+				// This is how the starter finds out about the claim id.
+			job_ad->Assign(ATTR_CLAIM_ID,claim_id);
+
+			rr->setJobAd( job_ad );
 			nodenum++;
 
             ResourceList[ResourceList.getlast()+1] = rr;
@@ -313,9 +326,6 @@ ParallelShadow::getResources( void )
             claim_id = NULL;
 
         } // end of for loop for this proc
-        
-		delete job_ad;
-		job_ad = NULL;
 
     } // end of for loop on all procs...
 
@@ -378,6 +388,7 @@ ParallelShadow::spawnNode( MpiResource* rr )
 			// First, contact the startd to spawn the job
 		if( rr->activateClaim() != OK ) {
 			shutDown( JOB_NOT_STARTED );
+			
 		}
 	}
 
@@ -494,6 +505,25 @@ ParallelShadow::emailTerminateEvent( int exitReason, update_style_t kind )
 void 
 ParallelShadow::shutDown( int exitReason )
 {
+	if (exitReason != JOB_NOT_STARTED) {
+		if (shutdownPolicy == WAIT_FOR_ALL) {
+			for ( int i=0 ; i<=ResourceList.getlast() ; i++ ) {
+				RemoteResource *r = ResourceList[i];
+				// If the policy is wait for all nodes to exit
+				// see if any are still running.  If so,
+				// just return, and wait for them all to go
+				if (r->getResourceState() != RR_FINISHED) {
+					return;
+				}
+			}
+			
+		}
+			// If node0 is still running, don't really shut down
+		RemoteResource *r =  ResourceList[0];
+		if (r->getResourceState() != RR_FINISHED) {
+			return;
+		}
+	}
 		/* With many resources, we have to figure out if all of
 		   them are done, and we have to figure out if we need
 		   to kill others.... */
@@ -507,23 +537,6 @@ ParallelShadow::shutDown( int exitReason )
 	}
 */
 
-	if (shutdownPolicy == WAIT_FOR_ALL) {
-    	for ( int i=0 ; i<=ResourceList.getlast() ; i++ ) {
-			RemoteResource *r = ResourceList[i];
-			// If the policy is wait for all nodes to exit
-			// see if any are still running.  If so,
-			// just return, and wait for them all to go
-			if (r->getResourceState() != RR_FINISHED) {
-				return;
-			}
-		}
-		
-	}
-		// If node0 is still running, don't really shut down
-	RemoteResource *r =  ResourceList[0];
-	if (r->getResourceState() != RR_FINISHED) {
-		return;
-	}
 	handleJobRemoval(0);
 
 		/* if we're still here, we can call BaseShadow::shutDown() to
@@ -657,22 +670,16 @@ ParallelShadow::handleJobRemoval( int sig ) {
 void
 ParallelShadow::replaceNode ( ClassAd *ad, int nodenum ) {
 
-	ExprTree *tree = NULL, *rhs = NULL, *lhs = NULL;
+	ExprTree *tree = NULL;
 	char node[9];
+	const char *lhstr, *rhstr;
 
-	sprintf( node, "%d", nodenum );
+	snprintf( node, 9, "%d", nodenum );
 
 	ad->ResetExpr();
-	while( (tree = ad->NextExpr()) ) {
-		MyString rhstr;
-		MyString lhstr;
-		if( (lhs = tree->LArg()) ) {
-			lhs->PrintToStr (lhstr);
-		}
-		if( (rhs = tree->RArg()) ) {
-			rhs->PrintToStr (rhstr);
-		}
-		if( !lhs || !rhs ) {
+	while( ad->NextExpr(lhstr, tree) ) {
+		rhstr = ExprTreeToString(tree);
+		if( !lhstr || !rhstr ) {
 			dprintf( D_ALWAYS, "Could not replace $(NODE) in ad!\n" );
 			return;
 		}
@@ -680,9 +687,9 @@ ParallelShadow::replaceNode ( ClassAd *ad, int nodenum ) {
 		MyString strRh(rhstr);
 		if (strRh.replaceString("#pArAlLeLnOdE#", node))
 		{
-			ad->AssignExpr( lhstr.Value(), strRh.Value() );
+			ad->AssignExpr( lhstr, strRh.Value() );
 			dprintf( D_FULLDEBUG, "Replaced $(NODE), now using: %s = %s\n", 
-					 lhstr.Value(), strRh.Value() );
+					 lhstr, strRh.Value() );
 		}
 	}	
 }
@@ -937,11 +944,11 @@ ParallelShadow::logDisconnectedEvent( const char* reason )
 	}
 	event.setStartdAddr( dc_startd->addr() );
 	event.setStartdName( dc_startd->name() );
-*/
 
 	if( !uLog.writeEvent(&event,jobAd) ) {
 		dprintf( D_ALWAYS, "Unable to log ULOG_JOB_DISCONNECTED event\n" );
 	}
+*/
 }
 
 
@@ -994,14 +1001,14 @@ ParallelShadow::logReconnectFailedEvent( const char* reason )
 }
 
 bool
-ParallelShadow::updateJobAttr( const char *name, const char *expr )
+ParallelShadow::updateJobAttr( const char *name, const char *expr, bool log )
 {
-	return job_updater->updateAttr( name, expr, true );
+	return job_updater->updateAttr( name, expr, true, log );
 }
 
 bool
-ParallelShadow::updateJobAttr( const char *name, int value )
+ParallelShadow::updateJobAttr( const char *name, int value, bool log )
 {
-	return job_updater->updateAttr( name, value, true );
+	return job_updater->updateAttr( name, value, true, log );
 }
 
