@@ -36,10 +36,10 @@ extern BaseShadow *Shadow;
 extern RemoteResource *thisRemoteResource;
 extern RemoteResource *parallelMasterResource;
 
-static void append_buffer_info( MyString &url, char *method, char const *path );
-static int use_append( char *method, const char *path );
-static int use_compress( char *method, const char *path );
-static int use_fetch( char *method, const char *path );
+static void append_buffer_info( MyString &url, const char *method, char const *path );
+static int use_append( const char *method, const char *path );
+static int use_compress( const char *method, const char *path );
+static int use_fetch( const char *method, const char *path );
 static int use_local_access( const char *file );
 static int use_special_access( const char *file );
 static int access_via_afs( const char *file );
@@ -162,6 +162,31 @@ pseudo_job_exit(int status, int reason, ClassAd* ad)
 	return 0;
 }
 
+int 
+pseudo_job_termination( ClassAd *ad )
+{
+	bool exited_by_signal = false;
+	bool core_dumped = false;
+	int exit_signal = 0;
+	int exit_code = 0;
+	MyString exit_reason;
+
+	ad->LookupBool(ATTR_ON_EXIT_BY_SIGNAL,exited_by_signal);
+	ad->LookupBool(ATTR_JOB_CORE_DUMPED,core_dumped);
+	ad->LookupString(ATTR_EXIT_REASON,exit_reason);
+
+	// Only one of these next two exist.
+	ad->LookupInteger(ATTR_ON_EXIT_SIGNAL,exit_signal);
+	ad->LookupInteger(ATTR_ON_EXIT_CODE,exit_code);
+
+	// This will utilize only the correct arguments depending on if the
+	// process exited with a signal or not.
+	Shadow->mockTerminateJob( exit_reason, exited_by_signal, exit_code,
+		exit_signal, core_dumped );
+
+	return 0;
+}
+
 
 int
 pseudo_register_mpi_master_info( ClassAd* ad ) 
@@ -218,7 +243,7 @@ int pseudo_get_file_info_new( const char *logical_name, char *&actual_url )
 	MyString	full_path;
 	MyString	remap;
 	MyString urlbuf;
-	char	*method;
+	const char	*method;
 
 	dprintf( D_SYSCALLS, "\tlogical_name = \"%s\"\n", logical_name );
 
@@ -302,7 +327,7 @@ int pseudo_get_file_info_new( const char *logical_name, char *&actual_url )
 	return 0;
 }
 
-static void append_buffer_info( MyString &url, char *method, char const *path )
+static void append_buffer_info( MyString &url, const char *method, char const *path )
 {
 	MyString buffer_list;
 	MyString buffer_string;
@@ -364,17 +389,17 @@ static int attr_list_has_file( const char *attr, const char *path )
 	}
 }
 
-static int use_append( char * /* method */, const char *path )
+static int use_append( const char * /* method */, const char *path )
 {
 	return attr_list_has_file( ATTR_APPEND_FILES, path );
 }
 
-static int use_compress( char * /* method */, const char *path )
+static int use_compress( const char * /* method */, const char *path )
 {
 	return attr_list_has_file( ATTR_COMPRESS_FILES, path );
 }
 
-static int use_fetch( char * /* method */, const char *path )
+static int use_fetch( const char * /* method */, const char *path )
 {
 	return attr_list_has_file( ATTR_FETCH_FILES, path );
 }
@@ -467,6 +492,7 @@ static int access_via_afs( const char * /* file */ )
 	done:
 	dprintf(D_SYSCALLS,"\taccess_via_afs() returning %s\n", result ? "TRUE" : "FALSE" );
 	if(my_fs_domain) free(my_fs_domain);
+	if(remote_fs_domain) free(remote_fs_domain);
 	return result;
 }
 
@@ -537,6 +563,8 @@ static int access_via_nfs( const char * /* file */ )
 
 	done:
 	dprintf( D_SYSCALLS, "\taccess_via_NFS() returning %s\n", result ? "TRUE" : "FALSE" );
+	if (remote_fs_domain) free(remote_fs_domain);
+	if (remote_uid_domain) free(remote_uid_domain);
 	return result;
 }
 
@@ -577,7 +605,7 @@ pseudo_ulog( ClassAd *ad )
 		dprintf(
 		  D_ALWAYS,
 		  "invalid event ClassAd in pseudo_ulog: %s\n",
-		  add_str.GetCStr());
+		  add_str.Value());
 		return -1;
 	}
 
@@ -603,12 +631,11 @@ pseudo_ulog( ClassAd *ad )
 
 		if(err->isCriticalError()) {
 			CriticalErrorBuf.sprintf(
-			  "Error from %s on %s: %s",
-			  err->getDaemonName(),
+			  "Error from %s: %s",
 			  err->getExecuteHost(),
 			  err->getErrorText());
 
-			critical_error = CriticalErrorBuf.GetCStr();
+			critical_error = CriticalErrorBuf.Value();
 			if(!hold_reason) {
 				hold_reason = critical_error;
 			}
@@ -628,7 +655,7 @@ pseudo_ulog( ClassAd *ad )
 		dprintf(
 		  D_ALWAYS,
 		  "unable to log event in pseudo_ulog: %s\n",
-		  add_str.GetCStr());
+		  add_str.Value());
 		result = -1;
 	}
 
@@ -653,28 +680,6 @@ pseudo_ulog( ClassAd *ad )
 	return result;
 }
 
-int
-pseudo_get_job_attr( const char *name, char *expr )
-{
-	RemoteResource *remote;
-	if (parallelMasterResource == NULL) {
-		remote = thisRemoteResource;
-	} else {
-		remote = parallelMasterResource;
-	}
-	ClassAd *ad = remote->getJobAd();
-	ExprTree *e = ad->Lookup(name);
-	if(e) {
-		expr[0] = 0;
-		e->RArg()->PrintToStr(expr);
-		dprintf(D_SYSCALLS,"pseudo_get_job_attr(%s) = %s\n",name,expr);
-		return 0;
-	} else {
-		dprintf(D_SYSCALLS,"pseudo_get_job_attr(%s) failed\n",name);
-		return -1;
-	}
-}
-
 int
 pseudo_get_job_attr( const char *name, MyString &expr )
 {
@@ -686,19 +691,20 @@ pseudo_get_job_attr( const char *name, MyString &expr )
 	}
 	ClassAd *ad = remote->getJobAd();
 
-	ExprTree *e = ad->Lookup(name);
+	ExprTree *e = ad->LookupExpr(name);
 	if(e) {
-		e->RArg()->PrintToStr(expr);
+		expr = ExprTreeToString(e);
 		dprintf(D_SYSCALLS,"pseudo_get_job_attr(%s) = %s\n",name,expr.Value());
 		return 0;
 	} else {
-		dprintf(D_SYSCALLS,"pseudo_get_job_attr(%s) failed\n",name);
-		return -1;
+		dprintf(D_SYSCALLS,"pseudo_get_job_attr(%s) is UNDEFINED\n",name);
+		expr = "UNDEFINED";
+		return 0;
 	}
 }
 
 int
-pseudo_set_job_attr( const char *name, const char *expr )
+pseudo_set_job_attr( const char *name, const char *expr, bool log )
 {
 	RemoteResource *remote;
 	if (parallelMasterResource == NULL) {
@@ -706,7 +712,7 @@ pseudo_set_job_attr( const char *name, const char *expr )
 	} else {
 		remote = parallelMasterResource;
 	}
-	if(Shadow->updateJobAttr(name,expr)) {
+	if(Shadow->updateJobAttr(name,expr,log)) {
 		dprintf(D_SYSCALLS,"pseudo_set_job_attr(%s,%s) succeeded\n",name,expr);
 		ClassAd *ad = remote->getJobAd();
 		ASSERT(ad);
