@@ -32,6 +32,7 @@
 #include "directory.h"
 #include "condor_claimid_parser.h"
 #include "authentication.h"
+#include "globus_utils.h"
 
 extern const char* public_schedd_addr;	// in shadow_v61_main.C
 
@@ -42,7 +43,7 @@ extern int do_REMOTE_syscall();
 ReliSock *syscall_sock;
 RemoteResource *thisRemoteResource;
 
-static char *Resource_State_String [] = {
+static const char *Resource_State_String [] = {
 	"PRE", 
 	"EXECUTING", 
 	"PENDING_DEATH", 
@@ -81,16 +82,26 @@ RemoteResource::RemoteResource( BaseShadow *shad )
 	next_reconnect_tid = -1;
 	proxy_check_tid = -1;
 	last_proxy_timestamp = time(0); // We haven't sent the proxy to the starter yet, so anything before "now" means it hasn't changed.
+	m_remote_proxy_expiration = 0;
+	m_remote_proxy_renew_time = 0;
 	reconnect_attempts = 0;
 
 	lease_duration = -1;
 	already_killed_graceful = false;
 	already_killed_fast = false;
+	m_want_chirp = false;
+	m_want_streaming_io = false;
 }
 
 
 RemoteResource::~RemoteResource()
 {
+	if( syscall_sock == claim_sock ) {
+		syscall_sock = NULL;
+	}
+	if( thisRemoteResource == this ) {
+		thisRemoteResource = NULL;
+	}
 	if ( dc_startd     ) delete dc_startd;
 	if ( machineName   ) delete [] machineName;
 	if ( starterAddress) delete [] starterAddress;
@@ -99,12 +110,23 @@ RemoteResource::~RemoteResource()
 	if ( starter_version ) delete [] starter_version;
 	if ( uid_domain	   ) delete [] uid_domain;
 	if ( fs_domain     ) delete [] fs_domain;
-	if ( claim_sock    ) delete claim_sock;
-	if ( jobAd         ) delete jobAd;
+	closeClaimSock();
+	if ( jobAd && jobAd != shadow->getJobAd() ) {
+		delete jobAd;
+	}
 	if( proxy_check_tid != -1) {
 		daemonCore->Cancel_Timer(proxy_check_tid);
 		proxy_check_tid = -1;
 	}
+
+	if( param_boolean("SEC_ENABLE_MATCH_PASSWORD_AUTHENTICATION",false) ) {
+		if( m_claim_session.secSessionId() ) {
+			daemonCore->getSecMan()->invalidateKey( m_claim_session.secSessionId() );
+		}
+		if( m_filetrans_session.secSessionId() ) {
+			daemonCore->getSecMan()->invalidateKey( m_filetrans_session.secSessionId() );
+		}
+	}
 }
 
 
@@ -138,7 +160,8 @@ RemoteResource::activateClaim( int starterVersion )
 	    (jobAd->LookupString( ATTR_X509_USER_PROXY, &proxy ) == 1) ) {
 		dprintf( D_FULLDEBUG,
 	                 "trying early delegation (for glexec) of proxy: %s\n", proxy );
-		int dReply = dc_startd->delegateX509Proxy( proxy );
+		time_t expiration_time = GetDesiredDelegatedJobCredentialExpiration(jobAd);
+		int dReply = dc_startd->delegateX509Proxy( proxy, expiration_time, NULL );
 		if( dReply == OK ) {
 			// early delegation was successful. this means the startd
 			// is going to launch the starter using glexec, so we need
@@ -189,6 +212,15 @@ RemoteResource::activateClaim( int starterVersion )
 				   "HandleSyscalls", this );
 			setResourceState( RR_STARTUP );		
 			hadContact();
+
+				// This expiration time we calculate here may be
+				// sooner than the proxy, because the proxy may be
+				// delegated some time in the future when file
+				// transfer happens.  However, that just means we may
+				// renew the proxy a few seconds earlier than we would
+				// if we were more accurate, so it is ok.
+			setRemoteProxyRenewTime();
+
 			return true;
 			break;
 		case CONDOR_TRY_AGAIN:
@@ -272,6 +304,12 @@ RemoteResource::killStarter( bool graceful )
 				 graceful ? "graceful" : "fast", addr );
 	}
 
+	int wantReleaseClaim = 0;
+	jobAd->LookupBool(ATTR_RELEASE_CLAIM, wantReleaseClaim);
+	if (wantReleaseClaim) {
+		ClassAd replyAd;
+		dc_startd->releaseClaim(VACATE_FAST, &replyAd);
+	}
 	return true;
 }
 
@@ -324,8 +362,7 @@ RemoteResource::handleSysCalls( Stream * /* sock */ )
 		shadow->dprintf(D_SYSCALLS,"Shadow: do_REMOTE_syscall returned < 0\n");
 			// we call our shadow's shutdown method:
 		shadow->shutDown( exit_reason );
-			// close sock on this end...the starter has gone away.
-		return TRUE;
+		return KEEP_STREAM;
 	}
 	hadContact();
 	return KEEP_STREAM;
@@ -564,13 +601,6 @@ RemoteResource::setStartdInfo( ClassAd* ad )
 		EXCEPT( "ad does not include %s!", ATTR_CLAIM_ID );
 	}
 
-		// Delete the claim id from the shadow's copy of the job ad,
-		// now that we have extracted it.  When we send this ad over
-		// the wire to the starter, there is no need for the claimid
-		// to be in the job ad, so better to remove it than to have
-		// to worry about encrypting that transmission.
-	ad->Delete( ATTR_CLAIM_ID );
-
 	char* addr = NULL;
 	ad->LookupString( ATTR_STARTD_IP_ADDR, &addr );
 	if( ! addr ) {
@@ -932,6 +962,23 @@ RemoteResource::setJobAd( ClassAd *jA )
 	if( jA->LookupInteger(ATTR_LAST_JOB_LEASE_RENEWAL, int_value) ) {
 		last_job_lease_renewal = (time_t)int_value;
 	}
+
+	jA->LookupBool( ATTR_WANT_IO_PROXY, m_want_chirp );
+
+	bool stream_input=false, stream_output=false, stream_error=false;
+	jA->LookupBool(ATTR_STREAM_INPUT,stream_input);
+	jA->LookupBool(ATTR_STREAM_OUTPUT,stream_output);
+	jA->LookupBool(ATTR_STREAM_ERROR,stream_error);
+
+	m_want_streaming_io = stream_input || stream_output || stream_error;
+	if( m_want_chirp || m_want_streaming_io ) {
+		dprintf(D_FULLDEBUG,
+			"Enabling remote IO syscalls (want chirp=%s,want streaming=%s).\n",
+			m_want_chirp ? "true" : "false",
+			m_want_streaming_io ? "true" : "false");
+	}
+
+	jA->LookupString(ATTR_X509_USER_PROXY, proxy_path);
 }
 
 
@@ -967,7 +1014,11 @@ RemoteResource::updateFromStarter( ClassAd* update_ad )
 			jobAd->Assign(ATTR_IMAGE_SIZE, int_value);
 		}
 	}
-			
+
+	if( update_ad->LookupInteger(ATTR_RESIDENT_SET_SIZE, int_value) ) {
+	    jobAd->Assign(ATTR_RESIDENT_SET_SIZE, int_value);
+	}
+
 	if( update_ad->LookupInteger(ATTR_DISK_USAGE, int_value) ) {
 		if( int_value > disk_usage ) {
 			disk_usage = int_value;
@@ -1010,17 +1061,29 @@ RemoteResource::updateFromStarter( ClassAd* update_ad )
 		jobAd->Assign(ATTR_JOB_CORE_DUMPED, (bool)int_value);
 	}
 
+		// The starter sends this attribute whether or not we are spooling
+		// output (because it doesn't know if we are).  Technically, we
+		// only need to write this attribute into the job ClassAd if we
+		// are spooling output.  However, it doesn't hurt to have it there
+		// otherwise.
+	if( update_ad->LookupString(ATTR_SPOOLED_OUTPUT_FILES,string_value) ) {
+		jobAd->Assign(ATTR_SPOOLED_OUTPUT_FILES,string_value.Value());
+	}
+	else if( jobAd->LookupString(ATTR_SPOOLED_OUTPUT_FILES,string_value) ) {
+		jobAd->AssignExpr(ATTR_SPOOLED_OUTPUT_FILES,"UNDEFINED");
+	}
+
 	char* job_state = NULL;
 	ResourceState new_state = state;
 	update_ad->LookupString( ATTR_JOB_STATE, &job_state );
 	if( job_state ) { 
 			// The starter told us the job state, see what it is and
 			// if we need to log anything to the UserLog
-		if( stricmp(job_state, "Suspended") == MATCH ) {
+		if( strcasecmp(job_state, "Suspended") == MATCH ) {
 			new_state = RR_SUSPENDED;
-		} else if ( stricmp(job_state, "Running") == MATCH ) {
+		} else if ( strcasecmp(job_state, "Running") == MATCH ) {
 			new_state = RR_EXECUTING;
-		} else if ( stricmp(job_state, "Checkpointed") == MATCH ) {
+		} else if ( strcasecmp(job_state, "Checkpointed") == MATCH ) {
 			new_state = RR_CHECKPOINTED;
 		} else { 
 				// For our purposes in here, we don't care about any
@@ -1057,6 +1120,16 @@ RemoteResource::updateFromStarter( ClassAd* update_ad )
 			setResourceState( new_state );
 		}
 	}
+
+	MyString starter_addr;
+	update_ad->LookupString( ATTR_STARTER_IP_ADDR, starter_addr );
+	if( !starter_addr.IsEmpty() ) {
+		// The starter sends updated contact info along with the job
+		// update (useful if CCB info changes).  It's a bit of a hack
+		// to do it through this channel, but better than nothing.
+		setStarterAddress( starter_addr.Value() );
+	}
+
 		// now that we've gotten an update, we should evaluate our
 		// periodic user policy again, since we have new information
 		// and something might now evaluate to true which we should
@@ -1147,6 +1220,12 @@ RemoteResource::recordResumeEvent( ClassAd* /* update_ad */ )
 	if( last_suspension_time > 0 ) {
 		// There was a real job suspension.
 		cumulative_suspension_time += now - last_suspension_time;
+
+		int uncommitted_suspension_time = 0;
+		jobAd->LookupInteger( ATTR_UNCOMMITTED_SUSPENSION_TIME,
+							  uncommitted_suspension_time );
+		uncommitted_suspension_time += now - last_suspension_time;
+		jobAd->Assign(ATTR_UNCOMMITTED_SUSPENSION_TIME, uncommitted_suspension_time);
 	}
 
 	sprintf( tmp, "%s = %d", ATTR_CUMULATIVE_SUSPENSION_TIME,
@@ -1194,7 +1273,7 @@ RemoteResource::recordCheckpointEvent( ClassAd* update_ad )
 	event.sent_bytes = recv_bytes - last_recv_bytes;
 	last_recv_bytes = recv_bytes;
 
-	if( !writeULogEvent(&event) ) {
+	if( !shadow->uLog.writeEventNoFsync(&event, jobAd) ) {
 		dprintf( D_ALWAYS, "Unable to log ULOG_CHECKPOINTED event\n" );
 		rval = false;
 	}
@@ -1227,6 +1306,13 @@ RemoteResource::recordCheckpointEvent( ClassAd* update_ad )
 		jobAd->LookupInteger(ATTR_JOB_COMMITTED_TIME, job_committed_time);
 		job_committed_time += now - int_value;
 		jobAd->Assign(ATTR_JOB_COMMITTED_TIME, job_committed_time);
+
+		float slot_weight = 1;
+		jobAd->LookupFloat(ATTR_JOB_MACHINE_ATTR_SLOT_WEIGHT0, slot_weight);
+		float slot_time = 0;
+		jobAd->LookupFloat(ATTR_COMMITTED_SLOT_TIME, slot_time);
+		slot_time += slot_weight * (now - int_value);
+		jobAd->Assign(ATTR_COMMITTED_SLOT_TIME, slot_time);
 	}
 
 	// Update timestamp of the last checkpoint
@@ -1248,6 +1334,8 @@ RemoteResource::recordCheckpointEvent( ClassAd* update_ad )
 		jobAd->Assign(ATTR_VM_CKPT_IP, string_value.Value());
 	}
 
+	shadow->CommitSuspensionTime(jobAd);
+
 		// Log stuff so we can check our sanity
 	printCheckpointStats( D_FULLDEBUG );
 
@@ -1311,6 +1399,13 @@ RemoteResource::printCheckpointStats( int debug_level )
 	jobAd->LookupInteger(ATTR_JOB_COMMITTED_TIME, int_value);
 	dprintf( debug_level, "%s = %d\n", ATTR_JOB_COMMITTED_TIME, int_value);
 
+	int committed_suspension_time = 0;
+	jobAd->LookupInteger( ATTR_COMMITTED_SUSPENSION_TIME, 
+						  committed_suspension_time );
+	dprintf( debug_level, "%s = %d\n",
+			 ATTR_COMMITTED_SUSPENSION_TIME,
+			 committed_suspension_time );
+
 	// timestamp of the last checkpoint
 	int_value = 0;
 	jobAd->LookupInteger( ATTR_LAST_CKPT_TIME, int_value);
@@ -1401,6 +1496,21 @@ rrStateToString( ResourceState s )
 	return Resource_State_String[s];
 }
 
+void
+RemoteResource::startCheckingProxy()
+{
+	if( proxy_check_tid != -1 ) {
+		return; // already running
+	}
+	if( !proxy_path.IsEmpty() ) {
+		// This job has a proxy.  We need to check it regularlly to
+		// potentially upload a renewed one.
+		int PROXY_CHECK_INTERVAL = param_integer("SHADOW_CHECKPROXY_INTERVAL",60*10,1);
+		proxy_check_tid = daemonCore->Register_Timer( 0, PROXY_CHECK_INTERVAL,
+							(TimerHandlercpp)&RemoteResource::checkX509Proxy,
+							"RemoteResource::checkX509Proxy()", this );
+    }
+}
 
 void 
 RemoteResource::beginExecution( void )
@@ -1412,17 +1522,9 @@ RemoteResource::beginExecution( void )
 
 	began_execution = true;
 	setResourceState( RR_EXECUTING );
-	
-    if( jobAd->LookupString(ATTR_X509_USER_PROXY, proxy_path) ) {
-		// This job has a proxy.  We need to check it regularlly to
-		// potentially upload a renewed one.
-		const int PROXY_CHECK_INTERVAL = 60; 
-		ASSERT(proxy_check_tid == -1);
-		proxy_check_tid = daemonCore->Register_Timer( 0, PROXY_CHECK_INTERVAL,
-							(TimerHandlercpp)&RemoteResource::checkX509Proxy,
-							"RemoteResource::checkX509Proxy()", this );
-    }
 
+	startCheckingProxy();
+	
 		// Let our shadow know so it can make global decisions (for
 		// example, should it log a JOB_EXECUTE event)
 	shadow->resourceBeganExecution( this );
@@ -1687,7 +1789,7 @@ RemoteResource::requestReconnect( void )
 
 		// And also put in the request the ATTR_TRANS_SOCK and
 		// the ATTR_TRANS_KEY --- the starter will need these two
-		// atttribute value in order to re-establish the FileTransfer
+		// attribute value in order to re-establish the FileTransfer
 		// objects.  To get the values for these, just instantiate a 
 		// FileTransfer server object right here.  No worries if
 		// one already exists, the FileTransfer object will just
@@ -1800,6 +1902,12 @@ RemoteResource::requestReconnect( void )
 	reconnect_attempts = 0;
 	hadContact();
 
+	int proxy_expiration = 0;
+	if( jobAd->LookupInteger(ATTR_DELEGATED_PROXY_EXPIRATION,proxy_expiration) ) {
+		setRemoteProxyRenewTime(proxy_expiration);
+	}
+	startCheckingProxy();
+
 		// Tell the Shadow object so it can take care of the rest.
 	shadow->resourceReconnected( this );
 
@@ -1808,6 +1916,45 @@ RemoteResource::requestReconnect( void )
 	return;
 }
 
+void
+RemoteResource::setRemoteProxyRenewTime(time_t expiration_time)
+{
+	m_remote_proxy_expiration = expiration_time;
+	m_remote_proxy_renew_time = GetDelegatedProxyRenewalTime(expiration_time);
+	jobAd->Assign(ATTR_DELEGATED_PROXY_EXPIRATION, (int)expiration_time);
+}
+
+void
+RemoteResource::setRemoteProxyRenewTime()
+{
+		// When the proxy was (initially) delegated via the file
+		// transfer object, we have to do our best here to compute the
+		// expiration time of the delegated proxy.  To know what it
+		// actually is, we would need to have a better interface for
+		// obtaining that information from the file transfer object.
+
+	if( proxy_path.IsEmpty() ) {
+		return;
+	}
+
+	time_t desired_expiration_time = GetDesiredDelegatedJobCredentialExpiration(jobAd);
+	time_t proxy_expiration_time = x509_proxy_expiration_time(proxy_path.Value());
+	time_t expiration_time = desired_expiration_time;
+
+	if( proxy_expiration_time == (time_t)-1 ) {
+		char const *errmsg = x509_error_string();
+		dprintf(D_ALWAYS,"setRemoteProxyRenewTime: failed to get proxy expiration time for %s: %s\n",
+				proxy_path.Value(),
+				errmsg ? errmsg : "");
+	}
+	else {
+		if( proxy_expiration_time < desired_expiration_time || desired_expiration_time == 0 ) {
+			expiration_time = proxy_expiration_time;
+		}
+	}
+
+	setRemoteProxyRenewTime(expiration_time);
+}
 
 bool
 RemoteResource::updateX509Proxy(const char * filename)
@@ -1822,7 +1969,12 @@ RemoteResource::updateX509Proxy(const char * filename)
 
 	DCStarter::X509UpdateStatus ret = DCStarter::XUS_Error;
 	if ( param_boolean( "DELEGATE_JOB_GSI_CREDENTIALS", true ) == true ) {
-		ret = starter.delegateX509Proxy(filename, m_claim_session.secSessionId());
+		time_t expiration_time = GetDesiredDelegatedJobCredentialExpiration(jobAd);
+		time_t result_expiration_time = 0;
+		ret = starter.delegateX509Proxy(filename, expiration_time, m_claim_session.secSessionId(),&result_expiration_time);
+		if( ret == DCStarter::XUS_Okay ) {
+			setRemoteProxyRenewTime(result_expiration_time);
+		}
 	}
 	if ( ret != DCStarter::XUS_Okay ) {
 		ret = starter.updateX509Proxy(filename, m_claim_session.secSessionId());
@@ -1847,6 +1999,10 @@ RemoteResource::updateX509Proxy(const char * filename)
 void 
 RemoteResource::checkX509Proxy( void )
 {
+	if( state != RR_EXECUTING ) {
+		dprintf(D_FULLDEBUG,"checkX509Proxy() doing nothing, because resource is not in EXECUTING state.\n");
+		return;
+	}
 	if(proxy_path.IsEmpty()) {
 		/* Harmless, but suspicious. */
 		return;
@@ -1855,8 +2011,18 @@ RemoteResource::checkX509Proxy( void )
 	StatInfo si(proxy_path.Value());
 	time_t lastmod = si.GetModifyTime();
 	dprintf(D_FULLDEBUG, "Proxy timestamps: remote estimated %ld, local %ld (%ld difference)\n",
-		(long)last_proxy_timestamp, (long)lastmod, lastmod - last_proxy_timestamp);
-	if(lastmod <= last_proxy_timestamp) {
+		(long)last_proxy_timestamp, (long)lastmod,lastmod - last_proxy_timestamp);
+
+	if( m_remote_proxy_renew_time ) {
+		dprintf(D_FULLDEBUG, "Remote short-lived proxy remaining lifetime %lds, to be redelegated in %lds\n",
+				(long)(m_remote_proxy_expiration-time(NULL)),
+				(long)(m_remote_proxy_renew_time-time(NULL)));
+	}
+
+	if( m_remote_proxy_renew_time && m_remote_proxy_renew_time <= time(NULL) ) {
+		dprintf(D_ALWAYS,"Time to redelegate short-lived proxy to starter.\n");
+	}
+	else if(lastmod <= last_proxy_timestamp) {
 		// No change.
 		return;
 	}
@@ -1868,11 +2034,11 @@ RemoteResource::checkX509Proxy( void )
 
 bool
 RemoteResource::getSecSessionInfo(
-	char const *starter_reconnect_session_info,
+	char const *,
 	MyString &reconnect_session_id,
 	MyString &reconnect_session_info,
 	MyString &reconnect_session_key,
-	char const *starter_filetrans_session_info,
+	char const *,
 	MyString &filetrans_session_id,
 	MyString &filetrans_session_info,
 	MyString &filetrans_session_key)
@@ -1898,3 +2064,45 @@ RemoteResource::getSecSessionInfo(
 
 	return true;
 }
+
+bool
+RemoteResource::allowRemoteReadFileAccess( char const * filename )
+{
+	bool response = m_want_chirp || m_want_streaming_io;
+	logRemoteAccessCheck(response,"read access to file",filename);
+	return response;
+}
+
+bool
+RemoteResource::allowRemoteWriteFileAccess( char const * filename )
+{
+	bool response = m_want_chirp || m_want_streaming_io;
+	logRemoteAccessCheck(response,"write access to file",filename);
+	return response;
+}
+
+bool
+RemoteResource::allowRemoteReadAttributeAccess( char const * name )
+{
+	bool response = m_want_chirp;
+	logRemoteAccessCheck(response,"read access to attribute",name);
+	return response;
+}
+
+bool
+RemoteResource::allowRemoteWriteAttributeAccess( char const * name )
+{
+	bool response = m_want_chirp;
+	logRemoteAccessCheck(response,"write access to attribute",name);
+	return response;
+}
+
+void
+RemoteResource::logRemoteAccessCheck(bool allow,char const *op,char const *name)
+{
+	int debug_level = allow ? D_FULLDEBUG : D_ALWAYS;
+	dprintf(debug_level,"%s remote request for %s %s\n",
+			allow ? "ALLOWING" : "DENYING",
+			op,
+			name);
+}
