@@ -37,6 +37,7 @@ UniShadow::UniShadow() {
 
 UniShadow::~UniShadow() {
 	if ( remRes ) delete remRes;
+	daemonCore->Cancel_Command( SHADOW_UPDATEINFO );
 }
 
 
@@ -77,10 +78,11 @@ UniShadow::updateFromStarterClassAd(ClassAd* update_ad) {
 	remRes->updateFromStarter(update_ad);
 
 	int cur_image = getImageSize();
-	if (cur_image > prev_image) {
+	if (cur_image != prev_image) {
 		JobImageSizeEvent event;
 		event.size = cur_image;
-		if (!uLog.writeEvent(&event, job_ad)) {
+			// for performance, do not bother fsyncing this event
+		if (!uLog.writeEventNoFsync(&event, job_ad)) {
 			dprintf(D_ALWAYS, "Unable to log ULOG_IMAGE_SIZE event\n");
 		}
 	}
@@ -381,7 +383,7 @@ UniShadow::logDisconnectedEvent( const char* reason )
 	event.setStartdAddr( dc_startd->addr() );
 	event.setStartdName( dc_startd->name() );
 
-	if( !uLog.writeEvent(&event,getJobAd()) ) {
+	if( !uLog.writeEventNoFsync(&event,getJobAd()) ) {
 		dprintf( D_ALWAYS, "Unable to log ULOG_JOB_DISCONNECTED event\n" );
 	}
 }
@@ -405,7 +407,7 @@ UniShadow::logReconnectedEvent( void )
 	delete [] starter;
 	starter = NULL;
 
-	if( !uLog.writeEvent(&event,getJobAd()) ) {
+	if( !uLog.writeEventNoFsync(&event,getJobAd()) ) {
 		dprintf( D_ALWAYS, "Unable to log ULOG_JOB_RECONNECTED event\n" );
 	}
 }
@@ -424,7 +426,7 @@ UniShadow::logReconnectFailedEvent( const char* reason )
 	}
 	event.setStartdName( dc_startd->name() );
 
-	if( !uLog.writeEvent(&event,getJobAd()) ) {
+	if( !uLog.writeEventNoFsync(&event,getJobAd()) ) {
 		dprintf( D_ALWAYS, "Unable to log ULOG_JOB_RECONNECT_FAILED event\n" );
 	}
 }
