@@ -27,7 +27,7 @@
 #include "../condor_syscall_lib/syscall_param_sizes.h"
 #include "my_hostname.h"
 #include "pseudo_ops.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "condor_ckpt_mode.h"
 #include "../condor_ckpt_server/server_interface.h"
 #include "internet.h"
@@ -62,11 +62,6 @@ extern "C" {
 	void HoldJob( const char* long_reason, const char* short_reason,
 				  int reason_code, int reason_subcode );
 	void reaper();
-	#if defined(DUX4)
-		int statfs(const char *, struct statfs*);
-	#elif defined(DUX5)
-		int _F64_statfs(char *, struct statfs*, ...);
-	#endif
 }
 
 extern int JobStatus;
@@ -130,13 +125,6 @@ const int MaxRetryWait = 3600;
 static bool CkptWanted = true;	// WantCheckpoint from Job ClassAd
 static bool RestoreCkptWithNoScheddName = false; // compat with old naming
 
-#ifdef WANT_NETMAN
-bool ManageBandwidth = false;   // notify negotiator about network usage?
-int	 NetworkHorizon = 0;		// how often do we request network bandwidth?
-static Daemon Negotiator(DT_NEGOTIATOR);
-void RequestCkptBandwidth();
-#endif
-
 int
 pseudo_register_ckpt_platform( const char *platform, int /*len*/ )
 {
@@ -534,7 +522,8 @@ pseudo_work_request( PROC *p, char *&a_out, char *&targ, char *&orig, int *kill_
 	if( stat(ICkptName,&st_buf) == 0 ) {
 		a_out = strdup( ICkptName );
 	} else {
-		EXCEPT( "Can't find a.out file" );
+		EXCEPT( "Can't find initial checkpoint file %s",
+			ICkptName?ICkptName:"(null pointer)");
 	}
 
 		/* Copy current checkpoint name into space provided */
@@ -543,7 +532,10 @@ pseudo_work_request( PROC *p, char *&a_out, char *&targ, char *&orig, int *kill_
 	} else if( stat(ICkptName,&st_buf) == 0 ) {
 		orig = strdup( ICkptName );
 	} else {
-		EXCEPT( "Can't find checkpoint file" );
+		EXCEPT( "Can't find any checkpoint file: CkptFile='%s' "
+			"or ICkptFile='%s'", 
+			CkptName?CkptName:"(null pointer)",
+			ICkptName?ICkptName:"(null pointer)" );
 	}
 
 		/* Copy next checkpoint name into space provided */
@@ -567,9 +559,17 @@ is_ckpt_file(const char path[])
 	char *test_path;
 
 	test_path = gen_ckpt_name( Spool, Proc->id.cluster, Proc->id.proc, 0 );
-	if (strcmp(path, test_path) == 0) return true;
+	if (strcmp(path, test_path) == 0) {
+		free(test_path); test_path = NULL;
+		return true;
+	}
 	strcat(test_path, ".tmp");
-	if (strcmp(path, test_path) == 0) return true;
+	if (strcmp(path, test_path) == 0) {
+		free(test_path); test_path = NULL;
+		return true;
+	}
+
+	free(test_path); test_path = NULL;
 	return false;
 }
 
@@ -584,30 +584,16 @@ is_ickpt_file(const char path[])
 	char *test_path;
 
 	test_path = gen_ckpt_name( Spool, Proc->id.cluster, ICKPT, 0 );
-	if (strcmp(path, test_path) == 0) return true;
+	if (strcmp(path, test_path) == 0) {
+		free(test_path); test_path = NULL;
+		return true;
+	}
+
+	free(test_path); test_path = NULL;
 	return false;
 }
 
 
-#if WANT_NETMAN
-enum file_stream_transfer_type {
-	INITIAL_CHECKPOINT, CHECKPOINT_RESTART,
-	PERIODIC_CHECKPOINT, VACATE_CHECKPOINT, COREFILE_TRANSFER
-};
-static 
-struct _file_stream_info {
-	_file_stream_info() : active(false) {}
-	bool active;
-	bool remote;
-	pid_t pid;
-	file_stream_transfer_type type;
-	char src[16], dst[16];
-	int bytes_so_far;
-	int total_bytes;
-	time_t last_update;
-} file_stream_info;
-#endif
-
 /*
    rename() becomes a psuedo system call because the file may be a checkpoint
    stored on the checkpoint server.  If it is a checkpoint file and we're
@@ -648,11 +634,6 @@ pseudo_rename(char *from, char *to)
 				LastCkptServer = NULL; // stored ckpt file on local disk
 				LastCkptTime = time(0);
 				NumCkpts++;
-#if WANT_NETMAN
-				file_stream_info.bytes_so_far = file_stream_info.total_bytes;
-				RequestCkptBandwidth();
-				file_stream_info.active = false;
-#endif
 				commit_rusage();
 					// We just successfully wrote a local checkpoint,
 					// so we should remove the checkpoint we left on
@@ -707,11 +688,6 @@ pseudo_rename(char *from, char *to)
 		if (CkptServerHost) LastCkptServer = strdup(CkptServerHost);
 		LastCkptTime = time(0);
 		NumCkpts++;
-#if WANT_NETMAN
-		file_stream_info.bytes_so_far = file_stream_info.total_bytes;
-		RequestCkptBandwidth();
-		file_stream_info.active = false;
-#endif
 		commit_rusage();
 		return 0;
 
@@ -740,25 +716,11 @@ pseudo_get_file_stream(
 	bool	CkptFile = is_ckpt_file(file);
 	bool	ICkptFile = is_ickpt_file(file);
 	priv_state	priv;
+	struct in_addr taddr;
+	MyString buf;
 
 	dprintf( D_ALWAYS, "\tEntering pseudo_get_file_stream\n" );
 	dprintf( D_ALWAYS, "\tfile = \"%s\"\n", file );
-#if WANT_NETMAN
-	if (file_stream_info.active) {
-#if !defined(PVM_RECEIVE)
-		if (!file_stream_info.remote) {
-			reaper();
-		}
-#endif
-		if (file_stream_info.active) {
-			dprintf(D_ALWAYS, "\tWarning: previous file_stream transfer "
-					"still active!  Cleaning up...\n");
-			file_stream_info.bytes_so_far = file_stream_info.total_bytes;
-			RequestCkptBandwidth();
-			file_stream_info.active = false;
-		}
-	}
-#endif
 
 	*len = 0;
 
@@ -773,33 +735,22 @@ pseudo_get_file_stream(
 								  								  scheddName,
 								  file,len,
 								  (struct in_addr*)ip_addr,port);
+			taddr.s_addr = *ip_addr;
+			buf = inet_ntoa(taddr);
+
 			if (rval) { // network error, try again
-				dprintf(D_ALWAYS, "ckpt server restore failed, trying again"
-						" in %d seconds\n", retry_wait);
+				dprintf(D_ALWAYS, "ckpt server restore to %s failed, "
+						"trying again in %d seconds\n", 
+						buf.Value(),
+						retry_wait);
 				sleep(retry_wait);
 				retry_wait *= 2;
 				if (retry_wait > MaxRetryWait) {
-					EXCEPT("ckpt server restore failed");
+					EXCEPT("ckpt server restore to %s failed", buf.Value());
 				}
 			}
 		} while (rval);
 
-#if WANT_NETMAN
-		file_stream_info.active = true;
-		file_stream_info.remote = true;
-		file_stream_info.pid = getpid();
-		file_stream_info.type = CHECKPOINT_RESTART;
-		struct in_addr sin;
-		memcpy(&sin, ip_addr, sizeof(struct in_addr));
-		strcpy(file_stream_info.src, inet_ntoa(sin));
-		strncpy(file_stream_info.dst, ExecutingHost+1, 16);
-		char *str = strchr(file_stream_info.dst, ':');
-		*str = '\0';
-		file_stream_info.bytes_so_far = 0;
-		file_stream_info.total_bytes = *len;
-		RequestCkptBandwidth();
-#endif
-
 		*ip_addr = ntohl( *ip_addr );
 		*port = ntohs( *port );
 		dprintf( D_ALWAYS, "RestoreRequest returned address:\n");
@@ -831,21 +782,6 @@ pseudo_get_file_stream(
     create_tcp_port( ip_addr, port, &connect_sock );
     dprintf( D_NETWORK, "\taddr = %s\n", ipport_to_string(htonl(*ip_addr), htons(*port)));
 		
-#if WANT_NETMAN
-	if (CkptFile || ICkptFile) {
-		file_stream_info.active = true;
-		file_stream_info.remote = false;
-		file_stream_info.type = CkptFile ? CHECKPOINT_RESTART :
-			INITIAL_CHECKPOINT;
-		strcpy(file_stream_info.src, inet_ntoa(*my_sin_addr()));
-		strncpy(file_stream_info.dst, ExecutingHost+1, 16);
-		char *str = strchr(file_stream_info.dst, ':');
-		*str = '\0';
-		file_stream_info.bytes_so_far = 0;
-		file_stream_info.total_bytes = *len;
-	}
-#endif
-
 	switch( child_pid = fork() ) {
 	case -1:	/* error */
 		dprintf( D_ALWAYS, "fork() failed, errno = %d\n", errno );
@@ -875,10 +811,6 @@ pseudo_get_file_stream(
 		if (CkptFile || ICkptFile) {
 			set_priv(priv);
 			if (CkptFile) NumRestarts++;
-#if WANT_NETMAN
-			file_stream_info.pid = child_pid;
-			RequestCkptBandwidth();
-#endif
 		}
 		return 0;
 	}
@@ -916,27 +848,13 @@ pseudo_put_file_stream(
 	int		retry_wait = 5;
 	priv_state	priv;
 	mode_t	omask;
+	struct in_addr taddr;
+	MyString buf;
 
 	dprintf( D_ALWAYS, "\tEntering pseudo_put_file_stream\n" );
 	dprintf( D_ALWAYS, "\tfile = \"%s\"\n", file );
 	dprintf( D_ALWAYS, "\tlen = %u\n", len );
 	dprintf( D_ALWAYS, "\towner = %s\n", p->owner );
-#if WANT_NETMAN
-	if (file_stream_info.active) {
-#if !defined(PVM_RECEIVE)
-		if (!file_stream_info.remote) {
-			reaper();
-		}
-#endif
-		if (file_stream_info.active) {
-			dprintf(D_ALWAYS, "\tWarning: previous file_stream transfer "
-					"still active!  Cleaning up...\n");
-			file_stream_info.bytes_so_far = file_stream_info.total_bytes;
-			RequestCkptBandwidth();
-			file_stream_info.active = false;
-		}
-	}
-#endif
 
     // ip_addr will be updated down below because I changed create_tcp_port so that
     // ip address the socket is bound to is determined by that function. Therefore,
@@ -951,33 +869,22 @@ pseudo_put_file_stream(
             // ip_addr is set within RequestStore
 			rval = RequestStore(p->owner, scheddName, file, len,
 								(struct in_addr*)ip_addr, port);
+			taddr.s_addr = *ip_addr;
+			buf = inet_ntoa(taddr);
+
 			if (rval) {	/* request denied or network error, try again */
-				dprintf(D_ALWAYS, "store request to ckpt server failed, "
-						"trying again in %d seconds\n", retry_wait);
+				dprintf(D_ALWAYS, "store request to ckpt server %s failed, "
+						"trying again in %d seconds\n", 
+						buf.Value(),
+						retry_wait);
 				sleep(retry_wait);
 				retry_wait *= 2;
 				if (retry_wait > MaxRetryWait) {
-					EXCEPT("ckpt server store failed");
+					EXCEPT("ckpt server store to %s failed", buf.Value());
 				}
 			}
 		} while (rval);
 
-#if WANT_NETMAN
-		file_stream_info.active = true;
-		file_stream_info.remote = true;
-		file_stream_info.pid = getpid();
-		file_stream_info.type = (LastCkptSig == SIGUSR2) ?
-			PERIODIC_CHECKPOINT : VACATE_CHECKPOINT;
-		struct in_addr sin;
-		memcpy(&sin, ip_addr, sizeof(struct in_addr));
-		strcpy(file_stream_info.dst, inet_ntoa(sin));
-		strncpy(file_stream_info.src, ExecutingHost+1, 16);
-		char *str = strchr(file_stream_info.src, ':');
-		*str = '\0';
-		file_stream_info.bytes_so_far = 0;
-		file_stream_info.total_bytes = len;
-#endif
-
 		*ip_addr = ntohl(*ip_addr);
 		*port = ntohs( *port );
 		dprintf(D_ALWAYS,  "StoreRequest returned addr:\n");
@@ -991,12 +898,6 @@ pseudo_put_file_stream(
 		if (len > 0) {
 				// need to handle the len == -1 case someday
 			BytesRecvd += len;
-#if WANT_NETMAN		
-			if (CkptFile && ManageBandwidth) {
-					// tell negotiator that the job is writing a checkpoint
-				RequestCkptBandwidth();
-			}
-#endif
 		}
 		return 0;
 	}
@@ -1011,7 +912,31 @@ pseudo_put_file_stream(
 		// effects us, the shadow.  -Derek Wright 6/11/98
 	omask = umask( 022 );
 
-	if( (file_fd=safe_open_wrapper(file,O_WRONLY|O_CREAT|O_TRUNC,0664)) < 0 ) {
+	int open_attempts=0;
+	for(open_attempts=0;open_attempts<100;open_attempts++) {
+		file_fd=safe_open_wrapper(file,O_WRONLY|O_CREAT|O_TRUNC,0664);
+
+		if( file_fd < 0 && errno == ENOENT && (CkptFile || ICkptFile) ) {
+
+				// Perhaps the shared spool directory hierarchy does
+				// not yet exist for this job.  In case of a race in
+				// which we create the directory and something else
+				// removes it, we try multiple times.
+
+			if( !SpooledJobFiles::createJobSpoolDirectory( JobAd, PRIV_CONDOR ) ) {
+				dprintf(D_ALWAYS,"Failed to create parent spool directory for %s\n",file);
+				break;
+			}
+			continue;
+		}
+		break;
+	}
+
+	if( file_fd < 0 && open_attempts > 0 ) {
+		dprintf(D_ALWAYS,"Failed to open spool file %s after %d tries.\n",file,open_attempts);
+	}
+
+	if( file_fd < 0 ) {
 		if (CkptFile || ICkptFile) set_priv(priv);	// restore user privileges
 		(void)umask(omask);
 		return -1;
@@ -1025,20 +950,6 @@ pseudo_put_file_stream(
 	create_tcp_port(ip_addr, port, &connect_sock);
     dprintf(D_NETWORK, "\taddr = %s\n", ipport_to_string(htonl(*ip_addr), htons(*port)));
 	
-#ifdef WANT_NETMAN
-	file_stream_info.active = true;
-	file_stream_info.remote = false;
-	file_stream_info.type = (CkptFile || ICkptFile) ?
-		((LastCkptSig == SIGUSR2) ?	PERIODIC_CHECKPOINT : VACATE_CHECKPOINT) :
-		COREFILE_TRANSFER;
-	strcpy(file_stream_info.dst, inet_ntoa(*my_sin_addr()));
-	strncpy(file_stream_info.src, ExecutingHost+1, 16);
-	char *str = strchr(file_stream_info.src, ':');
-	*str = '\0';
-	file_stream_info.bytes_so_far = 0;
-	file_stream_info.total_bytes = len;
-#endif
-
 	switch( child_pid = fork() ) {
 	  case -1:	/* error */
 		dprintf( D_ALWAYS, "fork() failed, errno = %d\n", errno );
@@ -1069,20 +980,11 @@ pseudo_put_file_stream(
 		close( connect_sock );
 		if (CkptFile || ICkptFile) {
 			set_priv(priv);	// restore user privileges
-#ifdef WANT_NETMAN
-			file_stream_info.pid = child_pid;
-#endif
 			if (CkptFile) NumRestarts++;
 		}
 		if (len > 0) {
 				// need to handle the len == -1 case someday
 			BytesRecvd += len;
-#ifdef WANT_NETMAN
-			if (CkptFile && ManageBandwidth) {
-					// tell negotiator that the job is writing a checkpoint
-				RequestCkptBandwidth();
-			}
-#endif
 		}
 		return 0;
 	}
@@ -1091,328 +993,6 @@ pseudo_put_file_stream(
 	return -1;
 }
 
-#ifdef WANT_NETMAN
-/* This function is called whenever a child process exits.   */
-void
-file_stream_child_exit(pid_t pid)
-{
-	if (file_stream_info.active && !file_stream_info.remote &&
-		file_stream_info.pid == pid) {
-		switch(file_stream_info.type) {
-		case INITIAL_CHECKPOINT:
-			file_stream_info.bytes_so_far = file_stream_info.total_bytes;
-			RequestCkptBandwidth();
-			file_stream_info.active = false;
-			break;
-		case COREFILE_TRANSFER:
-			file_stream_info.active = false;
-			break;
-		case CHECKPOINT_RESTART:
-				// handled in pseudo_get_ckpt_mode()
-		case PERIODIC_CHECKPOINT:
-		case VACATE_CHECKPOINT:
-				// handled in rename() (yuk!)
-			break;
-		default:
-			dprintf(D_ALWAYS, "Bad file_stream_transfer_type %d in "
-					"file_stream_child_exit!\n", file_stream_info.type);
-			return;
-		}
-	}
-}
-
-extern "C" {
-void
-file_stream_progress_report(int bytes_moved)
-{
-	if (!file_stream_info.active) {
-		static bool warned_already = false;
-		if (!warned_already) {
-			dprintf(D_ALWAYS, "Warning: file_stream_progress_report() called "
-					"with no active file_stream_transfer.\n");
-		}
-		warned_already = true;
-		return;
-	}
-	if (file_stream_info.remote) {
-		static bool warned_already = false;
-		if (!warned_already) {
-			dprintf(D_ALWAYS, "Warning: file_stream_progress_report() called "
-					"with active REMOTE file_stream_transfer.\n");
-		}
-		warned_already = true;
-		return;
-	}
-	file_stream_info.bytes_so_far = bytes_moved;
-	time_t current_time = time(0);
-	if (current_time >= file_stream_info.last_update + (NetworkHorizon/5)) {
-		RequestCkptBandwidth();
-	}
-}
-}
-
-void
-abort_file_stream_transfer()
-{
-	if (file_stream_info.active) {
-		if (!file_stream_info.remote) {
-			priv_state priv = set_condor_priv();
-			kill(file_stream_info.pid, SIGKILL);
-			set_priv(priv);
-		}
-			// invalidate our network bandwidth allocation
-		file_stream_info.bytes_so_far = file_stream_info.total_bytes;
-		RequestCkptBandwidth();
-		file_stream_info.active = false;
-	}
-}
-
-/* 
-   We allocate bandwidth for checkpoint and executable transfers here
-   (i.e., anything transferred using the file_stream protocol).  We
-   include the startd slot id so the matchmaker can uniquely identify
-   checkpoint transfers to/from SMP machines.  The matchmaker may have
-   already allocated the bandwidth for this transfer.  In that case,
-   our request serves as an update on the amount of data we're
-   expecting to transfer.  We can update our request at any time
-   during the transfer.  We must send a final update when the transfer
-   completes, so the matchmaker can free the allocation.  The
-   allocation will timeout if our update is lost.
-*/
-void
-RequestCkptBandwidth()
-{
-	ClassAd request;
-	char buf[100], owner[100], user[100];
-	int nice_user = 0, slot_id = 1;
-
-	if (!ManageBandwidth) return;
-
-	if (!file_stream_info.active) {
-		dprintf(D_ALWAYS, "Warning: RequestCkptBandwidth() called with no "
-				"active file_stream_transfer.\n");
-		return;
-	}
-	if (file_stream_info.total_bytes < 0) return; // TODO (someday)
-	
-	switch(file_stream_info.type) {
-	case INITIAL_CHECKPOINT:
-		sprintf(buf, "%s = \"InitialCheckpoint\"", ATTR_TRANSFER_TYPE);
-		dprintf(D_ALWAYS, "Transferring InitialCkpt from %s to %s: "
-				"%d bytes to go.\n", file_stream_info.src,
-				file_stream_info.dst,
-				file_stream_info.total_bytes-file_stream_info.bytes_so_far);
-		break;
-	case CHECKPOINT_RESTART:
-		sprintf(buf, "%s = \"CheckpointRestart\"", ATTR_TRANSFER_TYPE);
-		dprintf(D_ALWAYS, "Transferring RestartCkpt from %s to %s: "
-				"%d bytes to go.\n", file_stream_info.src,
-				file_stream_info.dst,
-				file_stream_info.total_bytes-file_stream_info.bytes_so_far);
-		break;
-	case PERIODIC_CHECKPOINT:
-		sprintf(buf, "%s = \"PeriodicCheckpoint\"", ATTR_TRANSFER_TYPE);
-		dprintf(D_ALWAYS, "Transferring PeriodicCkpt from %s to %s: "
-				"%d bytes to go.\n", file_stream_info.src,
-				file_stream_info.dst,
-				file_stream_info.total_bytes-file_stream_info.bytes_so_far);
-		break;
-	case VACATE_CHECKPOINT:
-		sprintf(buf, "%s = \"VacateCheckpoint\"", ATTR_TRANSFER_TYPE);
-		dprintf(D_ALWAYS, "Transferring VacateCkpt from %s to %s: "
-				"%d bytes to go.\n", file_stream_info.src,
-				file_stream_info.dst,
-				file_stream_info.total_bytes-file_stream_info.bytes_so_far);
-		break;
-	case COREFILE_TRANSFER:
-		return;					// ignore for now
-	default:
-		dprintf(D_ALWAYS, "Bad file_stream_transfer_type %d in "
-				"RequestCkptBandwidth!\n", file_stream_info.type);
-		return;
-	}
-	request.Insert(buf);
-	JobAd->LookupInteger(ATTR_NICE_USER, nice_user);
-	JobAd->LookupString(ATTR_OWNER, owner);
-	sprintf(user, "%s%s@%s", (nice_user) ? "nice-user." : "", owner,
-			My_UID_Domain);
-	sprintf(buf, "%s = \"%s\"", ATTR_USER, user);
-	request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_CLUSTER_ID, Proc->id.cluster);
-	request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_PROC_ID, Proc->id.proc);
-	request.Insert(buf);
-	if (param_boolean("ALLOW_VM_CRUFT", true)) {
-		if (!JobAd->LookupInteger(ATTR_REMOTE_SLOT_ID, slot_id)) {
-			JobAd->LookupInteger(ATTR_REMOTE_VIRTUAL_MACHINE_ID, slot_id);
-		}
-	} else {
-		JobAd->LookupInteger(ATTR_REMOTE_SLOT_ID, slot_id);
-	}
-	dprintf(D_ALWAYS, "slot_id = %d\n", slot_id);
- 	sprintf(buf, "%s = %d", ATTR_SLOT_ID, slot_id);
- 	request.Insert(buf);
-	if (param_boolean("ALLOW_VM_CRUFT", true)) {
-		sprintf(buf, "%s = %d", ATTR_VIRTUAL_MACHINE_ID, slot_id);
-		request.Insert(buf);
-	}
-    // Using ip:pid as an identity of the process is not complete because of private
-    // addresses are reusable. TODO (someday)
-	sprintf(buf, "%s = \"%u.%d\"", ATTR_TRANSFER_KEY, my_ip_addr(),
-			file_stream_info.pid);
- 	request.Insert(buf);
-	sprintf(buf, "%s = 1", ATTR_FORCE);
-	request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_SOURCE, file_stream_info.src);
-	request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_REMOTE_HOST, ExecutingHost);
-	request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_DESTINATION, file_stream_info.dst);
-	request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_REQUESTED_CAPACITY,
-			file_stream_info.total_bytes-file_stream_info.bytes_so_far);
-	request.Insert(buf);
-
-    SafeSock sock;
-	sock.timeout(10);
-	if (!sock.connect(Negotiator.addr())) {
-		dprintf(D_ALWAYS, "Couldn't connect to negotiator (%s)!\n",
-				Negotiator.addr());
-		return;
-	}
-
-	Negotiator.startCommand (REQUEST_NETWORK, &sock);
-
-	sock.put(1);
-	request.put(sock);
-	sock.end_of_message();
-
-
-	buf[0] = '\0';
-	if (JobAd->LookupString(ATTR_REMOTE_POOL, buf)) {
-		Daemon FlockNegotiator(DT_NEGOTIATOR, buf, buf);
-        if (!sock.connect(FlockNegotiator.addr())) {
-			dprintf(D_ALWAYS, "Couldn't connect to flock negotiator (%s)!\n",
-					FlockNegotiator.addr());
-			return;
-		}
-
-		FlockNegotiator.startCommand(REQUEST_NETWORK, &sock);
-
-		sock.put(1);
-		request.put(sock);
-		sock.end_of_message();
-	}
-	file_stream_info.last_update = time(0);
-}
-
-/* 
-   We allocate bandwidth for RSC here.  Since we don't know how much
-   RSC I/O we will generate in the future, we must make some
-   prediction.  We use an exponential average of our past network
-   usage for RIO to predict future RIO needs.  We can track our
-   prediction error as we go.
-*/
-
-void
-RequestRSCBandwidth()
-{
-	static float prev_bytes_sent=0.0, prev_bytes_recvd=0.0;
-	static int send_estimate=0, recv_estimate=0;
-	ClassAd send_request, recv_request;
-	char buf[100], endpoint[100], owner[100], user[100], *str;
-	int nice_user = 0;
-
-	if (!syscall_sock || !ManageBandwidth) return;
-
-	int bytes_sent = (int)(syscall_sock->get_bytes_sent()-prev_bytes_sent);
-	int bytes_recvd = (int)(syscall_sock->get_bytes_recvd()-prev_bytes_recvd);
-
-	prev_bytes_sent = syscall_sock->get_bytes_sent();
-	prev_bytes_recvd = syscall_sock->get_bytes_recvd();
-
-	int prev_send_estimate = send_estimate;
-	int prev_recv_estimate = recv_estimate;
-
-	send_estimate = (bytes_sent + send_estimate) / 2;
-	recv_estimate = (bytes_recvd + recv_estimate) / 2;
-
-	// don't bother allocating anything under 1KB
-	if (send_estimate < 1024.0 && recv_estimate < 1024.0) return;
-
-    SafeSock sock;
-	sock.timeout(10);
-	if (!sock.connect(Negotiator.addr())) {
-		dprintf(D_ALWAYS, "Couldn't connect to negotiator!\n");
-		return;
-	}
-
-	Negotiator.startCommand (REQUEST_NETWORK, &sock);
-	sock.put(2);
-
-	// these attributes are only required in the first network req. ClassAd
-	sprintf(buf, "%s = \"RemoteSyscalls\"", ATTR_TRANSFER_TYPE);
-	send_request.Insert(buf);
-	JobAd->LookupInteger(ATTR_NICE_USER, nice_user);
-	JobAd->LookupString(ATTR_OWNER, owner);
-	sprintf(user, "%s%s@%s", (nice_user) ? "nice-user." : "", owner,
-			My_UID_Domain);
-	sprintf(buf, "%s = \"%s\"", ATTR_USER, user);
-	send_request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_BYTES_SENT, bytes_sent);
-	send_request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_BYTES_RECVD, bytes_recvd);
-	send_request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_PREV_SEND_ESTIMATE, prev_send_estimate);
-	send_request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_PREV_RECV_ESTIMATE, prev_recv_estimate);
-	send_request.Insert(buf);
-	strcpy(endpoint, ExecutingHost+1);
-	str = strchr(endpoint, ':');
-	*str = '\0';
-	sprintf(buf, "%s = \"%s\"", ATTR_REMOTE_HOST, ExecutingHost);
-	send_request.Insert(buf);
-
-	// these attributes must be defined for both network req. ClassAds
-	sprintf(buf, "%s = 1", ATTR_FORCE);
-	send_request.Insert(buf);
-	recv_request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_SOURCE, endpoint);
-	send_request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_DESTINATION, inet_ntoa(*my_sin_addr()));
-	send_request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_REQUESTED_CAPACITY, recv_estimate);
-	send_request.Insert(buf);
-	send_request.put(sock);
-	sprintf(buf, "%s = \"%s\"", ATTR_DESTINATION, endpoint);
-	recv_request.Insert(buf);
-	sprintf(buf, "%s = \"%s\"", ATTR_SOURCE, inet_ntoa(*my_sin_addr()));
-	recv_request.Insert(buf);
-	sprintf(buf, "%s = %d", ATTR_REQUESTED_CAPACITY, send_estimate);
-	recv_request.Insert(buf);
-	recv_request.put(sock);
-	sock.end_of_message();
-
-
-	buf[0] = '\0';
-	if (JobAd->LookupString(ATTR_REMOTE_POOL, buf)) {
-		Daemon FlockNegotiator(DT_NEGOTIATOR, buf, buf);
-
-		if (!sock.connect(FlockNegotiator.addr())) {
-			dprintf(D_ALWAYS, "Couldn't connect to flock negotiator (%s)!\n",
-					FlockNegotiator.addr());
-			return;
-		}
-        FlockNegotiator.startCommand(REQUEST_NETWORK, &sock);
-
-		sock.put(2);
-		send_request.put(sock);
-		recv_request.put(sock);
-		sock.end_of_message();
-	}
-}
-#endif
-
 /*
   Accept a TCP connection on the connect_sock.  Close the connect_sock,
   and return the new fd.
@@ -1483,7 +1063,8 @@ create_tcp_port( unsigned int *ip, u_short *port, int *fd )
 
         /* create a tcp socket */
     if( (*fd=socket(AF_INET,SOCK_STREAM,0)) < 0 ) {
-        EXCEPT( "socket" );
+        EXCEPT( "create_tcp_port(): socket() failed: %d(%s)", 
+			errno, strerror(errno) );
     }
     dprintf( D_FULLDEBUG, "\tconnect_sock = %d\n", *fd );
 
@@ -1491,20 +1072,22 @@ create_tcp_port( unsigned int *ip, u_short *port, int *fd )
 
         /* FALSE means this is an incoming connection */
     if( ! _condor_local_bind(FALSE, *fd) ) {
-        EXCEPT( "bind" );
+        EXCEPT( "create_tcp_port(): bind() failed: %d(%s)",
+			errno, strerror(errno) );
     }
 
         /* determine which local port number we were assigned to */
     struct sockaddr_in *tmp = getSockAddr(*fd);
     if (tmp == NULL) {
-        EXCEPT("getSockAddr failed");
+        EXCEPT("create_tcp_port(): getSockAddr() failed");
     }
     *ip = ntohl(tmp->sin_addr.s_addr);
     *port = ntohs(tmp->sin_port);
 
         /* Get ready to accept a connection */
     if( listen(*fd,1) < 0 ) {
-        EXCEPT( "listen" );
+        EXCEPT( "create_tcp_port(): listen() failed: %d(%s)", 
+			errno, strerror(errno) );
     }
     dprintf( D_FULLDEBUG, "\tListening...\n" );
 
@@ -2032,20 +1615,6 @@ pseudo_get_ckpt_mode( int sig )
 		if (PeriodicSync) mode |= CKPT_MODE_MSYNC;
 	} else if (sig == 0) {		// restart
 		if (PeriodicSync) mode |= CKPT_MODE_MSYNC;
-#ifdef WANT_NETMAN
-			// The job has completed its checkpoint restart now, so we
-			// can release the network bandwidth allocation (by
-			// updating our bandwidth request).
-		if (file_stream_info.active &&
-			file_stream_info.type == CHECKPOINT_RESTART) {
-			file_stream_info.bytes_so_far = file_stream_info.total_bytes;
-			RequestCkptBandwidth();
-			file_stream_info.active = false;
-		} else {
-			dprintf( D_ALWAYS, "Warning: received get_ckpt_mode(0) with no "
-					 "info about active checkpoint restart transfer!\n" );
-		}
-#endif
 	} else {
 		dprintf( D_ALWAYS,
 				 "pseudo_get_ckpt_mode called with unknown signal %d\n", sig );
@@ -2320,12 +1889,12 @@ has_ckpt_file()
 			}
 		}
 		if(rval == -1 && LastCkptServer && accum_usage > MaxDiscardedRunTime) {
-			dprintf(D_ALWAYS, "failed to contact ckpt server, trying again"
-					" in %d seconds\n", retry_wait);
+			dprintf(D_ALWAYS, "failed to contact last ckpt server at %s, "
+					"trying again in %d seconds\n", LastCkptServer, retry_wait);
 			sleep(retry_wait);
 			retry_wait *= 2;
 			if (retry_wait > MaxRetryWait) {
-				EXCEPT("failed to contact ckpt server");
+				EXCEPT("failed to contact last ckpt server %s", LastCkptServer);
 			}
 		}
 	} while(rval == -1 && LastCkptServer && accum_usage > MaxDiscardedRunTime);
@@ -2385,13 +1954,13 @@ simp_log( const char *msg )
 
 	if( !fp ) {
 		if( (fd=safe_open_wrapper(name,O_WRONLY|O_CREAT,0666)) < 0 ) {
-			EXCEPT( name );
+			EXCEPT( "Can't create/open \"%s\"", name );
 		}
 		close( fd );
 
 		fp = safe_fopen_wrapper( name, "a" );
 		if( fp == NULL ) {
-			EXCEPT( "Can't open \"%s\" for appending\n", name );
+			EXCEPT( "Can't open \"%s\" for appending", name );
 		}
 	}
 
@@ -2624,10 +2193,8 @@ pseudo_statfs( const char *path, struct statfs *buf )
 		 me. */
 	str = strdup(path);
 
-#if defined(Solaris) || defined(IRIX)
+#if defined(Solaris)
 	ret = statfs( str, buf, 0, 0);
-#elif defined DUX5
-	ret = _F64_statfs( str, buf );
 #else
 	ret = statfs( str, buf );
 #endif
