@@ -20,8 +20,8 @@
 
 #include "condor_common.h"
 #include "condor_classad.h"
-#include "condor_io.h"
-#include "condor_ckpt_name.h"
+#include "std_univ_io.h"
+#include "spooled_job_files.h"
 #include "condor_debug.h"
 #include "internet.h"
 #include "condor_uid.h"
@@ -34,7 +34,7 @@
 #include "../condor_ckpt_server/server_interface.h"
 #include "sig_install.h"
 #include "job_report.h"
-#include "../condor_c++_util/directory.h"
+#include "../condor_utils/directory.h"
 #include "condor_distribution.h"
 #include "condor_environ.h"
 #include "condor_holdcodes.h"
@@ -51,6 +51,7 @@
 #include "exit.h"
 
 #include "file_sql.h"
+#include "spool_version.h"
 
 /* XXX This should not be here */
 #if !defined( WCOREDUMP )
@@ -86,7 +87,7 @@ extern "C" {
 	void get_local_rusage( struct rusage *bsd_rusage );
 	void NotifyUser( char *buf, PROC *proc );
 	FILE	*fdopen(int, const char *);
-	int		whoami(FILE*);
+	int		whoami(char **buf,int *bufpos,int *buflen);
 	void update_job_status( struct rusage *localp, struct rusage *remotep );
 	void update_job_rusage( struct rusage *localp, struct rusage *remotep );
 	int DoCleanup();
@@ -117,7 +118,7 @@ char *strcpy();
 #endif
 
 #include "write_user_log.h"
-UserLog ULog;
+WriteUserLog ULog;
 
 char * My_Filesystem_Domain;
 char * My_UID_Domain;
@@ -141,14 +142,6 @@ int  CommittedTime = 0;			// run-time committed in checkpoints
 // last known checkpointing signature of a successful checkpoint
 extern char *LastCkptPlatform;
 
-#ifdef WANT_NETMAN
-extern bool ManageBandwidth;	// notify negotiator about network usage?
-extern int NetworkHorizon;		// how often do we request network bandwidth?
-extern "C" void file_stream_child_exit(pid_t pid);
-extern "C" void abort_file_stream_transfer();
-extern "C" void RequestRSCBandwidth();
-#endif
-
 extern char *Executing_Arch, *Executing_OpSys;
 
 extern char *IpcFile;
@@ -166,7 +159,7 @@ GENERIC_PROC GenProc;
 
 extern "C"  void initializeUserLog();
 extern "C"  void log_termination(struct rusage *, struct rusage *);
-extern "C"  void log_except(char *);
+extern "C"  void log_except(const char *);
 
 char	*Spool = NULL;
 char	*ExecutingHost = NULL;
@@ -209,11 +202,11 @@ volatile int check_static_policy = 1;	/* don't check if condor_rm'ed */
 int JobExitStatus = 0;                 /* the job's exit status */
 int MaxDiscardedRunTime = 3600;
 
-extern "C" int ExceptCleanup(int,int,char*);
+extern "C" int ExceptCleanup(int,int,const char*);
 extern int Termlog;
 
-ReliSock	*sock_RSC1 = NULL, *RSC_ShadowInit(int rscsock, int errsock);
-ReliSock	*RSC_MyShadowInit(int rscsock, int errsock);
+StdUnivSock	*sock_RSC1 = NULL, *RSC_ShadowInit(int rscsock, int errsock);
+StdUnivSock	*RSC_MyShadowInit(int rscsock, int errsock);
 int HandleLog();
 void RemoveNewShadowDroppings(char *cluster, char *proc);
 
@@ -272,7 +265,7 @@ main(int argc, char *argv[] )
 	int		i;
 
 	myDistro->Init( argc, argv );
-	if( argc == 2 && strincmp(argv[1], "-cl", 3) == MATCH ) {
+	if( argc == 2 && strncasecmp(argv[1], "-cl", 3) == MATCH ) {
 		printClassAd();
 		exit( 0 );
 	}
@@ -318,7 +311,7 @@ main(int argc, char *argv[] )
 	dprintf( D_ALWAYS, "** %s\n", CondorPlatform() );
 	dprintf( D_ALWAYS, "*******************************************\n" );
 
-	reserved_swap = param_integer("RESERVED_SWAP", 5);
+	reserved_swap = param_integer("RESERVED_SWAP", 0);
 	reserved_swap *= 1024; /* megabytes -> kb */
 
 	bool use_sql_log = param_boolean("QUILL_USE_SQL_LOG", false);
@@ -328,7 +321,7 @@ main(int argc, char *argv[] )
 
 	dprintf( D_FULLDEBUG, "*** Reserved Swap = %d\n", reserved_swap );
 	dprintf( D_FULLDEBUG, "*** Free Swap = %d\n", free_swap );
-	if( free_swap < reserved_swap ) {
+	if( reserved_swap && free_swap < reserved_swap ) {
 		dprintf( D_ALWAYS, "Not enough reserved swap space\n" );
 		if(FILEObj) {
 		  delete FILEObj;
@@ -359,6 +352,8 @@ main(int argc, char *argv[] )
 		}
 	}
 
+	CheckSpoolVersion(SPOOL_MIN_VERSION_SHADOW_SUPPORTS,SPOOL_CUR_VERSION_SHADOW_SUPPORTS);
+
 	if( strcmp("-pipe",argv[1]) == 0 ) {
 		bogus_capability = argv[2];
 		cluster = argv[3];
@@ -476,11 +471,6 @@ main(int argc, char *argv[] )
 
 	SlowCkptSpeed = param_integer( "SLOW_CKPT_SPEED", 0 );
 
-#ifdef WANT_NETMAN
-	ManageBandwidth = param_boolean("MANAGE_BANDWIDTH",false);
-	NetworkHorizon = param_integer("NETWORK_HORIZON",300);
-#endif
-
 	// Install signal handlers such that all signals are blocked when inside
 	// the handler.
 	sigset_t fullset;
@@ -625,9 +615,6 @@ HandleSyscalls()
 
 	time_t current_time = time(0);
 	time_t next_periodic_update = current_time + periodic_interval_len;
-#if WANT_NETMAN
-	time_t next_bw_update = current_time + NetworkHorizon;
-#endif
 	
 	for(;;) {	/* get a request and fulfill it */
 
@@ -639,13 +626,6 @@ HandleSyscalls()
 		timer.tv_sec = next_periodic_update - current_time;
 		timer.tv_usec = 0;
 		ptimer = &timer;
-#if WANT_NETMAN
-		if (ManageBandwidth && next_bw_update > current_time) {
-			timer.tv_sec = next_bw_update - current_time;
-			timer.tv_usec = 0;
-			ptimer = &timer;
-		}
-#endif
 		/* if the current timer is set for a time longer than this, than
 			truncate the timer required to the periodic limit. After 
 			inspection of the bandwidth timer, it seems that it will recorrect
@@ -658,7 +638,7 @@ HandleSyscalls()
 
 		unblock_signal(SIGCHLD);
 		unblock_signal(SIGUSR1);
-#if defined(LINUX) || defined(IRIX) || defined(Solaris)
+#if defined(LINUX) || defined(Solaris)
 		cnt = select(nfds, &readfds, (fd_set *)0, (fd_set *)0, ptimer);
 #else
 		cnt = select(nfds, &readfds, 0, 0, ptimer);
@@ -711,13 +691,6 @@ HandleSyscalls()
 			}
 		}
 
-#if WANT_NETMAN
-		if (ManageBandwidth && current_time >= next_bw_update) {
-			RequestRSCBandwidth();
-			next_bw_update = current_time + NetworkHorizon;
-		}
-#endif
-
 #if defined(SYSCALL_DEBUG)
 		strcpy( SyscallLabel, "shadow" );
 #endif
@@ -1082,6 +1055,19 @@ update_job_status( struct rusage *localp, struct rusage *remotep )
 							 ATTR_LAST_VACATE_TIME, time(0) );
 		}
 
+		if( ExitReason == JOB_CKPTED || LastCkptTime > LastRestartTime ) {
+			int uncommitted_suspension_time = 0;
+			JobAd->LookupInteger(ATTR_UNCOMMITTED_SUSPENSION_TIME, uncommitted_suspension_time);
+			if( uncommitted_suspension_time > 0 ) {
+				int committed_suspension_time = 0;
+				GetAttributeInt(Proc->id.cluster, Proc->id.proc,
+								ATTR_COMMITTED_SUSPENSION_TIME, &committed_suspension_time);
+				committed_suspension_time += uncommitted_suspension_time;
+				SetAttributeInt(Proc->id.cluster, Proc->id.proc,
+								ATTR_COMMITTED_SUSPENSION_TIME, committed_suspension_time);
+			}
+		}
+
 		// if we had checkpointed, then save all of these attributes as well.
 		if (LastCkptTime > LastRestartTime) {
 			SetAttributeInt(Proc->id.cluster, Proc->id.proc,
@@ -1320,17 +1306,19 @@ start_job( char *cluster_id, char *proc_id )
 		tmp = gen_ckpt_name( Spool, Proc->id.cluster, Proc->id.proc, 0 );
 		snprintf( CkptName, MAXPATHLEN, "%s", tmp );
 		sprintf( TmpCkptName, "%s.tmp", CkptName );
+		free(tmp); tmp = NULL;
 	}
 
 	tmp = gen_ckpt_name( Spool, Proc->id.cluster, ICKPT, 0 );
 	snprintf( ICkptName, MAXPATHLEN, "%s", tmp );
+	free(tmp); tmp = NULL;
 
 	strcpy( RCkptName, CkptName );
 }
 
 extern "C" {
 int
-ExceptCleanup(int, int, char *buf)
+ExceptCleanup(int, int, const char *buf)
 {
   log_except(buf);
   return DoCleanup();
@@ -1348,10 +1336,6 @@ DoCleanup()
 		(void) unlink_local_or_ckpt_server( TmpCkptName );
 	}
 
-#if WANT_NETMAN
-	abort_file_stream_transfer(); // if we've got one
-#endif
-
 	return 0;
 }
 
@@ -1494,11 +1478,6 @@ reaper(int unused)
 				pid, WTERMSIG(status)
 			);
 		}
-#if WANT_NETMAN
-			// tell the file_stream code whenever a child exits, so it
-			// can do some cleanup with its children
-		file_stream_child_exit(pid);
-#endif
 	}
 
 #ifdef HPUX
@@ -1645,7 +1624,8 @@ void RemoveNewShadowDroppings(char *cluster, char *proc)
 	strcpy(names[0], ckpt_name);
 	strcpy(names[1], ckpt_name);
 	strcat(names[1], ".tmp");
-	
+	free(ckpt_name); ckpt_name = NULL;
+
 	for (j = 0; j < 2; j++)
 	{
 		if (stat(names[j], &buf) == 0) {
