@@ -28,7 +28,7 @@
 #include "condor_common.h"
 #include "condor_classad.h"
 #include "condor_io.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "condor_debug.h"
 #include "internet.h"
 #include "condor_uid.h"
@@ -63,7 +63,7 @@ extern "C" {
 				  int reason_code, int reason_subcode );
 	char *d_format_time( double dsecs );
 	int unlink_local_or_ckpt_server( char *file );
-	int whoami( FILE *fp );
+	int whoami( char **buf,int *bufpos,int *buflen);
 	void get_local_rusage( struct rusage *bsd_rusage );
 	void handle_termination( PROC *proc, char *notification,
 				int *jobstatus, char const *coredir );
@@ -462,12 +462,12 @@ unlink_local_or_ckpt_server( char *file )
 ** dprintf().
 */
 int
-whoami( FILE *fp )
+whoami( char **buf,int *bufpos,int *buflen)
 {
         if ((Proc) && (Proc->id.cluster || Proc->id.proc)) {
-                fprintf( fp, "(%d.%d) (%d):", Proc->id.cluster, Proc->id.proc, MyPid );
+                return sprintf_realloc( buf, bufpos, buflen, "(%d.%d) (%d):", Proc->id.cluster, Proc->id.proc, MyPid );
         } else {
-                fprintf( fp, "(?.?) (%d):", MyPid );
+                return sprintf_realloc( buf, bufpos, buflen, "(?.?) (%d):", MyPid );
         }
 		return 0;
 }
@@ -481,15 +481,11 @@ Convert a time value from the POSIX style "clock_t" to a BSD style
 void
 clock_t_to_timeval( clock_t ticks, struct timeval *tv )
 {
-#if defined(OSF1)
-        static long clock_tick = CLK_TCK;
-#else
         static long clock_tick = 0;
 
         if( !clock_tick ) {
                 clock_tick = sysconf( _SC_CLK_TCK );
         }
-#endif
 
         tv->tv_sec = ticks / clock_tick;
         tv->tv_usec = ticks % clock_tick;
@@ -903,14 +899,14 @@ part_send_job(
 	  // communication.
   char sinfulstring[SINFUL_STRING_BUF_SIZE];
 
-  snprintf(sinfulstring, SINFUL_STRING_BUF_SIZE, "<%s:%d>", sock->endpoint_ip_str(), ports.port1);
+  snprintf(sinfulstring, SINFUL_STRING_BUF_SIZE, "<%s:%d>", sock->peer_ip_str(), ports.port1);
 
   if( (sd1 = do_connect(sinfulstring, (char *)0, (u_short)ports.port1)) < 0 ) {
     dprintf( D_ALWAYS, "failed to connect to scheduler on %s\n", sinfulstring );
 	goto returnfailure;
   }
  
-  snprintf(sinfulstring, SINFUL_STRING_BUF_SIZE, "<%s:%d>", sock->endpoint_ip_str(), ports.port2);
+  snprintf(sinfulstring, SINFUL_STRING_BUF_SIZE, "<%s:%d>", sock->peer_ip_str(), ports.port2);
   if( (sd2 = do_connect(sinfulstring, (char *)0, (u_short)ports.port2)) < 0 ) {
     dprintf( D_ALWAYS, "failed to connect to scheduler on %s\n", sinfulstring );
 	close(sd1);
@@ -1055,33 +1051,15 @@ MakeProc(ClassAd *ad, PROC *p)
 	ad->LookupString(ATTR_JOB_IWD, buf);
 	p->iwd = strdup(buf);
 
-	e = ad->Lookup(ATTR_REQUIREMENTS);
+	e = ad->LookupExpr(ATTR_REQUIREMENTS);
 	if (e) {
-		buf[0] = '\0';
-		e->PrintToStr(buf);
-		s = strchr(buf, '=');
-		if (s) {
-			s++;
-			p->requirements = strdup(s);
-		} 
-		else {
-			p->requirements = strdup(buf);
-		}
+		p->requirements = strdup(ExprTreeToString(e));
 	} else {
 	   p->requirements = NULL;
 	}
-	e = ad->Lookup(ATTR_RANK);
+	e = ad->LookupExpr(ATTR_RANK);
 	if (e) {
-		buf[0] = '\0';
-		e->PrintToStr(buf);
-		s = strchr(buf, '=');
-		if (s) {
-			s++;
-			p->preferences = strdup(s);
-		} 
-		else {
-			p->preferences = strdup(buf);
-		}
+		p->preferences = strdup(ExprTreeToString(e));
 	} else {
 		p->preferences = NULL;
 	}
