@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2011, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -28,8 +28,10 @@
 #ifndef _RES_ATTRIBUTES_H
 #define _RES_ATTRIBUTES_H
 
+#include "condor_common.h"
+#include "condor_classad.h"
+
 class Resource;
-class ClassAd;
 
 typedef int amask_t;
 
@@ -93,8 +95,9 @@ public:
 	void publish( ClassAd*, amask_t );  // Publish desired info to given CA
 	void compute( amask_t );			  // Actually recompute desired stats
 
-		// Compute kflops and mips on the given resource
-	void benchmark( Resource*, int force = 0 );	
+		// Initiate benchmark computations benchmarks on the given resource
+	void start_benchmarks( Resource*, int &count );	
+	void benchmarks_finished( Resource* );	
 
 #if defined(WIN32)
 		// For testing communication with the CredD, if one is
@@ -118,6 +121,7 @@ public:
 	time_t		console_idle()	{ return m_console_idle; };
 
 private:
+
 		// Dynamic info
 	float			m_load;
 	float			m_condor_load;
@@ -127,7 +131,7 @@ private:
 	time_t			m_console_idle;
 	int				m_mips;
 	int				m_kflops;
-	int				m_last_benchmark;   // Last time we computed benchmarks
+	time_t			m_last_benchmark;   // Last time we computed benchmarks
 	time_t			m_last_keypress; 	// Last time m_idle decreased
 	bool			m_seen_keypress;    // Have we seen our first keypress yet?
 	int				m_clock_day;
@@ -186,7 +190,7 @@ public:
 	int	type() { return c_type; };
 
 	void display( amask_t );
-	void dprintf( int, char*, ... );
+	void dprintf( int, const char*, ... );
 	void show_totals( int );
 
 	float get_disk() { return c_disk; }
@@ -222,9 +226,9 @@ private:
 
 		// These hold the fractions of shared, dynamic resources
 		// that are allocated to this CPU.
-	float			c_virt_mem_fraction;
+	double			c_virt_mem_fraction;
 
-	float			c_disk_fraction; // share of execute dir partition
+	double			c_disk_fraction; // share of execute dir partition
 	MyString        c_execute_dir;
 	MyString        c_execute_partition_id;  // unique id for partition
 
@@ -265,8 +269,4 @@ private:
 	AvailDiskPartition &GetAvailDiskPartition(MyString const &execute_partition_id);
 };
 
-
-void deal_with_benchmarks( Resource* );
-
 #endif /* _RES_ATTRIBUTES_H */
-
