@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2011, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -26,6 +26,7 @@ ResState::ResState( Resource* res_ip )
 	r_state = owner_state;
 	r_destination = no_state;
 	r_act = idle_act;
+	r_act_was_benchmark = false;
 	m_atime = time(NULL);
 	m_stime = m_atime;
 	this->rip = res_ip;
@@ -48,19 +49,13 @@ ResState::ResState( Resource* res_ip )
 void
 ResState::publish( ClassAd* cp, amask_t how_much ) 
 {
-	char tmp[80];
+	cp->Assign( ATTR_STATE, state_to_string(r_state) );
 
-	sprintf( tmp, "%s=\"%s\"", ATTR_STATE, state_to_string(r_state) );
-	cp->InsertOrUpdate( tmp );
+	cp->Assign( ATTR_ENTERED_CURRENT_STATE, (int)m_stime );
 
-	sprintf( tmp, "%s=%d", ATTR_ENTERED_CURRENT_STATE, (int)m_stime );
-	cp->InsertOrUpdate( tmp );
+	cp->Assign( ATTR_ACTIVITY, activity_to_string(r_act) );
 
-	sprintf( tmp, "%s=\"%s\"", ATTR_ACTIVITY, activity_to_string(r_act) );
-	cp->InsertOrUpdate( tmp );
-
-	sprintf( tmp, "%s=%d", ATTR_ENTERED_CURRENT_ACTIVITY, (int)m_atime );
-	cp->InsertOrUpdate(tmp);
+	cp->Assign( ATTR_ENTERED_CURRENT_ACTIVITY, (int)m_atime );
 
 		// Conditionally publish any attributes about time spent in
 		// each of the following state/activity combinations.
@@ -80,7 +75,7 @@ ResState::publish( ClassAd* cp, amask_t how_much )
 }
 
 
-int
+void
 ResState::change( State new_state, Activity new_act )
 {
 	bool statechange = false, actchange = false;
@@ -93,14 +88,14 @@ ResState::change( State new_state, Activity new_act )
 		actchange = true;
 	}
 	if( ! (actchange || statechange) ) {
-		return TRUE;   // If we're not changing anything, return
+		return;   // If we're not changing anything, return
 	}
 
 		// leave_action and enter_action return TRUE if they result in
 		// a state or activity change.  In these cases, we want to
 		// abort the current state change.
 	if( leave_action( r_state, r_act, new_state, new_act, statechange ) ) {
-		return TRUE;
+		return;
 	}
 
 	if( statechange && !actchange ) {
@@ -136,12 +131,13 @@ ResState::change( State new_state, Activity new_act )
 		}
 	}
 	if( actchange ) {
+		r_act_was_benchmark = ( r_act == benchmarking_act );
 		r_act = new_act;
 		m_atime = now;
 	}
 
 	if( enter_action( r_state, r_act, statechange, actchange ) ) {
-		return TRUE;
+		return;
 	}
 
 		// Update resource availability statistics on state changes
@@ -161,7 +157,7 @@ ResState::change( State new_state, Activity new_act )
 		  if we're in Backfill, we can immediately return now...
 		*/
 	if( r_state == backfill_state ) {
-		return TRUE;
+		return;
 	}
 #endif /* HAVE_BACKFILL */
 
@@ -171,30 +167,30 @@ ResState::change( State new_state, Activity new_act )
 		this->eval();
 	}
 
-	return TRUE;
+	return;
 }
 
 
-int
+void
 ResState::change( Activity new_act )
 {
-	return change( r_state, new_act );
+	change( r_state, new_act );
 }
 
 
-int
+void
 ResState::change( State new_state )
 {
 	if( new_state == preempting_state ) {
 			// wants_vacate dprintf's about what it decides and the
 			// implications on state changes...
 		if( !rip->preemptWasTrue() || rip->wants_vacate() ) {
-			return change( new_state, vacating_act );
+			change( new_state, vacating_act );
 		} else {
-			return change( new_state, killing_act );
+			change( new_state, killing_act );
 		}
 	} else {
-		return change( new_state, idle_act );
+		change( new_state, idle_act );
 	}
 }
 
@@ -244,7 +240,8 @@ ResState::eval( void )
 					// change to the preempting state.
 				dprintf( D_ALWAYS, "State change: claim retirement ended/expired\n" );
 				// STATE TRANSITION #18
-				return change( preempting_state );
+				change( preempting_state );
+				return TRUE; // XXX: change TRUE
 			}
 		}
 		want_suspend = rip->wants_suspend();
@@ -271,18 +268,21 @@ ResState::eval( void )
 			if( rip->mayUnretire() ) {
 				dprintf( D_ALWAYS, "State change: unretiring because no preempting claim exists\n" );
 				// STATE TRANSITION #13
-				return change( busy_act );
+				change( busy_act );
+				return TRUE; // XXX: change TRUE
 			}
 			if( rip->retirementExpired() ) {
 				dprintf( D_ALWAYS, "State change: retirement ended/expired\n" );
-				return change( preempting_state );
+				change( preempting_state );
+				return TRUE; // XXX: change TRUE
 			}
 		}
 		if( (r_act == busy_act || r_act == retiring_act) && want_suspend ) {
 			if( rip->eval_suspend() ) {
 				// STATE TRANSITION #14 or #17
 				dprintf( D_ALWAYS, "State change: SUSPEND is TRUE\n" );
-				return change( suspended_act );
+				change( suspended_act );
+				return TRUE; // XXX: change TRUE
 			}
 		}
 		if( r_act == suspended_act ) {
@@ -290,11 +290,13 @@ ResState::eval( void )
 				// STATE TRANSITION #15
 				dprintf( D_ALWAYS, "State change: CONTINUE is TRUE\n" );
 				if( !rip->inRetirement() ) {
-					return change( busy_act );
+					change( busy_act );
+					return TRUE; // XXX: change TRUE
 				}
 				else {
 					// STATE TRANSITION #16
-					return change( retiring_act );
+					change( retiring_act );
+					return TRUE; // XXX: change TRUE
 				}
 			}
 		}
@@ -302,11 +304,13 @@ ResState::eval( void )
 			dprintf( D_ALWAYS, "State change: retiring due to preempting claim\n" );
 			// reversible retirement (e.g. if preempting claim goes away)
 			// STATE TRANSITION #12
-			return change( retiring_act );
+			change( retiring_act );
+			return TRUE; // XXX: change TRUE
 		}
 		if( (r_act == idle_act) && rip->hasPreemptingClaim() ) {
 			dprintf( D_ALWAYS, "State change: preempting idle claim\n" );
-			return change( preempting_state );
+			change( preempting_state );
+			return TRUE; // XXX: change TRUE
 		}
 		if( (r_act == idle_act) && (rip->eval_start() == 0) ) {
 				// START evaluates to False, so return to the owner
@@ -316,11 +320,13 @@ ResState::eval( void )
 				// we're in the claimed state, so we'll have a job ad
 				// to evaluate against.
 			dprintf( D_ALWAYS, "State change: START is false\n" );
-			return change( preempting_state ); 
+			change( preempting_state ); 
+			return TRUE; // XXX: change TRUE
 		}
 		if( (r_act == idle_act) && rip->claimWorklifeExpired() ) {
 			dprintf( D_ALWAYS, "State change: idle claim shutting down due to CLAIM_WORKLIFE\n" );
-			return change( preempting_state );
+			change( preempting_state );
+			return TRUE; // XXX: change TRUE
 		}
 		if( (r_act == busy_act || r_act == retiring_act) && (rip->wants_pckpt()) ) {
 			rip->periodic_checkpoint();
@@ -346,23 +352,39 @@ ResState::eval( void )
 			if( rip->eval_kill() ) {
 				dprintf( D_ALWAYS, "State change: KILL is TRUE\n" );
 					// STATE TRANSITION #19
-				return change( killing_act );
+				change( killing_act );
+				return TRUE; // XXX: change TRUE
 			}
 		}
 		break;	// case preempting_state:
 
 	case unclaimed_state:
 		if( Resource::DYNAMIC_SLOT == rip->get_feature() ) {
-			return change( delete_state );
+#if HAVE_JOB_HOOKS
+				// If we're currently fetching we can't delete
+				// ourselves. If we do when the hook returns we won't
+				// be around to handle the response.
+			if( rip->isCurrentlyFetching() ) {
+				dprintf(D_ALWAYS, "State change: Unclaimed -> Deleted delayed for outstanding work fetch\n");
+				break;
+			}
+#endif
+			change( delete_state );
+			return TRUE; // XXX: change TRUE
 		}
 
 		// See if we should be owner or unclaimed
 		if( rip->eval_is_owner() ) {
 			dprintf( D_ALWAYS, "State change: IS_OWNER is TRUE\n" );
-			return change( owner_state );
+			change( owner_state );
+			return TRUE; // XXX: change TRUE
 		}
+
 			// Check to see if we should run benchmarks
-		deal_with_benchmarks( rip );
+		if ( ! r_act_was_benchmark ) {
+			int num_started;
+			resmgr->m_attr->start_benchmarks( rip, num_started );
+		}
 
 #if HAVE_JOB_HOOKS
 			// If we're compiled to support fetching work
@@ -377,7 +399,8 @@ ResState::eval( void )
 			// instantiated, and b) START_BACKFILL evals to TRUE
 		if( resmgr->m_backfill_mgr && rip->eval_start_backfill() > 0 ) {
 			dprintf( D_ALWAYS, "State change: START_BACKFILL is TRUE\n" );
-			return change( backfill_state, idle_act );
+			change( backfill_state, idle_act );
+			return TRUE; // XXX: change TRUE
 		}
 #endif /* HAVE_BACKFILL */
 
@@ -392,6 +415,7 @@ ResState::eval( void )
 		if( ! rip->eval_is_owner() ) {
 			dprintf( D_ALWAYS, "State change: IS_OWNER is false\n" );
 			change( unclaimed_state );
+			return TRUE; // change() can delete rip
 		}
 #if HAVE_JOB_HOOKS
 			// If we're compiled to support fetching work
@@ -435,7 +459,8 @@ ResState::eval( void )
 			if( r_act == idle_act ) {
 					// no sense going to killing if we're already
 					// idle, go to owner immediately.
-				return change( owner_state );
+				change( owner_state );
+				return TRUE; // XXX: change TRUE
 			}
 				// we can change into Backfill/Killing then set our
 				// destination, since set_dest() won't take any
@@ -496,7 +521,7 @@ act_to_load( Activity act )
 
 int
 ResState::leave_action( State cur_s, Activity cur_a, State new_s,
-						Activity, bool statechange )
+						Activity new_a, bool statechange )
 {
 	switch( cur_s ) {
 	case preempting_state:
@@ -531,25 +556,28 @@ ResState::leave_action( State cur_s, Activity cur_a, State new_s,
 				rip->r_cur->starterKillPg( SIGKILL );
 				dprintf( D_ALWAYS,
 						 "State change: Error sending signals to starter\n" );
-				if( new_s != owner_state ) {
+				if( new_s != preempting_state ) {
+					change( preempting_state );
+					return TRUE; // XXX: change TRUE
+				} else {
 						/*
 						  if we're already trying to get into the
-						  owner state (because of an error like this,
+						  preempting state (because of an error like this,
 						  either suspending or resuming), we do NOT
 						  want to officially call ResState::change()
 						  again, or we just get into an infinite loop.
 						  instead, just return FALSE (that we didn't
 						  already change the state), and allow the
-						  previous attempt to change into owner_state
+						  previous attempt to change into preempting_state
 						  to continue...
 						*/
-					return change( owner_state );
-				} else {
 					return FALSE;
 				}
 			}
 		}
-		if( statechange ) {
+
+		//TODO: tstclair evaluate whether we should even be doing this in ResState
+		if( statechange && new_a != vacating_act ) {
 			rip->r_cur->cancelLeaseTimer();	
 		}
 		break;
@@ -593,7 +621,8 @@ ResState::enter_action( State s, Activity a,
 		if( ! rip->eval_is_owner() ) {
 				// Really want to be in unclaimed.
 			dprintf( D_ALWAYS, "State change: IS_OWNER is false\n" );
-			return change( unclaimed_state );
+			change( unclaimed_state );
+			return TRUE; // XXX: change TRUE
 		}
 		rip->r_reqexp->restore();		
 		break;
@@ -612,7 +641,8 @@ ResState::enter_action( State s, Activity a,
 				rip->r_cur->starterKillPg( SIGKILL );
 				dprintf( D_ALWAYS,
 						 "State change: Error sending signals to starter\n" );
-				return change( owner_state );
+				change( preempting_state );
+				return TRUE; // XXX: change TRUE
 			}
 		}
 		else if (a == busy_act) {
@@ -624,14 +654,16 @@ ResState::enter_action( State s, Activity a,
 				// suspension) and there is a preempting claim or we
 				// are in irreversible retirement, so retire.
 
-				return change( retiring_act );
+				change( retiring_act );
+				return TRUE; // XXX: change TRUE
 			}
 		}
 		else if (a == retiring_act) {
 			if( ! rip->claimIsActive() ) {
 				// The starter exited by the time we got here.
 				// No need to wait around in retirement.
-				return change( preempting_state );
+				change( preempting_state );
+				return TRUE; // XXX: change TRUE
 			}
 		}
 #if HAVE_JOB_HOOKS
@@ -718,7 +750,8 @@ ResState::enter_action( State s, Activity a,
 						// to the starter's process group.
 					dprintf( D_ALWAYS,
 							 "State change: Error sending signals to starter\n" );
-					return change( owner_state );
+					rip->leave_preempting_state();
+					return TRUE; // XXX: change TRUE
 				}
 			} else {
 				rip->leave_preempting_state();
@@ -728,11 +761,15 @@ ResState::enter_action( State s, Activity a,
 
 		case vacating_act:
 			if( rip->claimIsActive() ) {
-				if( ! rip->r_cur->starterKillSoft() ) {
+				if( rip->preemptWasTrue() && rip->wants_hold() ) {
+					rip->hold_job();
+				}
+				else if( ! rip->r_cur->starterKillSoft() ) {
 					rip->r_cur->starterKillPg( SIGKILL );
 					dprintf( D_ALWAYS,
 							 "State change: Error sending signals to starter\n" );
-					return change( owner_state );
+					change( owner_state );
+					return TRUE; // XXX: change TRUE
 				}
 			} else {
 				rip->leave_preempting_state();
@@ -762,7 +799,7 @@ ResState::enter_action( State s, Activity a,
 
 
 void
-ResState::dprintf( int flags, char* fmt, ... )
+ResState::dprintf( int flags, const char* fmt, ... )
 {
 	va_list args;
 	va_start( args, fmt );
@@ -884,7 +921,8 @@ ResState::starterExited( void )
 			// for all 3 of these, once the starter is gone, we can
 			// enter the destination directly.
 		dprintf( D_ALWAYS, "State change: starter exited\n" );
-		return change( r_destination );
+		change( r_destination );
+		return TRUE; // XXX: change TRUE
 		break;
 
 	case claimed_state:
@@ -898,7 +936,8 @@ ResState::starterExited( void )
 			return TRUE;
 		} else {
 			r_destination = no_state;
-			return change( owner_state );
+			change( owner_state );
+			return TRUE; // XXX: change TRUE
 		}
 		break;
 
@@ -931,7 +970,8 @@ ResState::starterExited( void )
 			return FALSE;
 		}
 		dprintf( D_ALWAYS, "State change: Backfill starter exited\n" );
-		return change( idle_act );
+		change( idle_act );
+		return TRUE; // XXX: change TRUE
 		break;
 #endif /* HAVE_BACKFILL */
 
