@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2011, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  *
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -25,10 +25,14 @@
 #include "vm_common.h"
 #include "VMRegister.h"
 #include "file_sql.h"
+#include "condor_holdcodes.h"
+#include "startd_bench_job.h"
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 #include "StartdPlugin.h"
 #endif
+#endif
 
 extern FILESQL *FILEObj;
 
@@ -48,7 +52,7 @@ Resource::Resource( CpuAttributes* cap, int rid, Resource* _parent )
 	}
 	if( _parent ) {
 		r_sub_id = _parent->m_id_dispenser->next();
-		tmp.sprintf_cat( "%d.%d", r_id, r_sub_id );
+		tmp.sprintf_cat( "%d_%d", r_id, r_sub_id );
 	} else {
 		tmp.sprintf_cat( "%d", r_id );
 	}
@@ -62,7 +66,7 @@ Resource::Resource( CpuAttributes* cap, int rid, Resource* _parent )
 
 		// we need this before we instantiate the Reqexp object...
 	tmp.sprintf( "SLOT_TYPE_%d_PARTITIONABLE", type() );
-	if( param_boolean( tmp.GetCStr(), false ) ) {
+	if( param_boolean( tmp.Value(), false ) ) {
 		set_feature( PARTITIONABLE_SLOT );
 
 		m_id_dispenser = new IdDispenser( 3, 1 );
@@ -158,12 +162,20 @@ Resource::~Resource()
 		m_next_fetch_work_tid = -1;
 	}
 	if (m_hook_keyword) {
-		free(m_hook_keyword);
+		free(m_hook_keyword); m_hook_keyword = NULL;
 	}
 #endif /* HAVE_JOB_HOOKS */
 
+		// Note on "&& !m_currently_fetching": A DYNAMIC slot will
+		// defer its destruction while it is waiting on a fetch work
+		// hook. The only time when a slot with a parent will be
+		// destroyed while waiting on a hook is during
+		// shutdown. During shutdown there is no need to give
+		// resources back to the parent slot, and doing so may
+		// actually be dangerous if our parent was deleted first.
+
 		// If we have a parent, return our resources to it
-	if( m_parent ) {
+	if( m_parent && !m_currently_fetching ) {
 		*(m_parent->r_attr) += *(r_attr);
 		m_parent->m_id_dispenser->insert( r_sub_id );
 		m_parent->update();
@@ -175,21 +187,21 @@ Resource::~Resource()
 		m_id_dispenser = NULL;
 	}
 
-	delete r_state;
-	delete r_classad;
-	delete r_cur;
+	delete r_state; r_state = NULL;
+	delete r_classad; r_classad = NULL;
+	delete r_cur; r_cur = NULL;
 	if( r_pre ) {
-		delete r_pre;
+		delete r_pre; r_pre = NULL;
 	}
 	if( r_pre_pre ) {
-		delete r_pre_pre;
+		delete r_pre_pre; r_pre_pre = NULL;
 	}
-	delete r_cod_mgr;
-	delete r_reqexp;
-	delete r_attr;
-	delete r_load_queue;
-	free( r_name );
-	free( r_id_str );
+	delete r_cod_mgr; r_cod_mgr = NULL;
+	delete r_reqexp; r_reqexp = NULL;
+	delete r_attr; r_attr = NULL;
+	delete r_load_queue; r_load_queue = NULL;
+	free( r_name ); r_name = NULL;
+	free( r_id_str ); r_id_str = NULL;
 }
 
 
@@ -224,15 +236,17 @@ Resource::retire_claim( void )
 				r_cur->setRetirePeacefully(true);
 			}
 		}
-		return change_state( retiring_act );
+		change_state( retiring_act );
+		break;
 	case matched_state:
-		return change_state( owner_state );
+		change_state( owner_state );
+		break;
 #if HAVE_BACKFILL
 	case backfill_state:
 			// we don't want retirement to mean anything special for
 			// backfill jobs... they should be killed immediately
 		set_destination_state( owner_state );
-		return TRUE;
+		break;
 #endif /* HAVE_BACKFILL */
 	default:
 			// For good measure, try directly killing the starter if
@@ -250,18 +264,20 @@ Resource::release_claim( void )
 {
 	switch( state() ) {
 	case claimed_state:
-		return change_state( preempting_state, vacating_act );
+		change_state( preempting_state, vacating_act );
+		break;
 	case preempting_state:
 		if( activity() != killing_act ) {
-			return change_state( preempting_state, vacating_act );
+			change_state( preempting_state, vacating_act );
 		}
 		break;
 	case matched_state:
-		return change_state( owner_state );
+		change_state( owner_state );
+		break;
 #if HAVE_BACKFILL
 	case backfill_state:
 		set_destination_state( owner_state );
-		return TRUE;
+		break;
 #endif /* HAVE_BACKFILL */
 	default:
 		return (int)r_cur->starterKillHard();
@@ -279,13 +295,15 @@ Resource::kill_claim( void )
 			// We might be in preempting/vacating, in which case we'd
 			// still want to do the activity change into killing...
 			// Added 4/26/00 by Derek Wright <wright@cs.wisc.edu>
-		return change_state( preempting_state, killing_act );
+		change_state( preempting_state, killing_act );
+		break;
 	case matched_state:
-		return change_state( owner_state );
+		change_state( owner_state );
+		break;
 #if HAVE_BACKFILL
 	case backfill_state:
 		set_destination_state( owner_state );
-		return TRUE;
+		break;
 #endif /* HAVE_BACKFILL */
 	default:
 			// In other states, try direct kill.  See above.
@@ -309,7 +327,7 @@ Resource::got_alive( void )
 		dprintf( D_ALWAYS, "Got keep alive with no current client object.\n" );
 		return FALSE;
 	}
-	r_cur->alive();
+	r_cur->alive( true );
 	return TRUE;
 }
 
@@ -393,38 +411,46 @@ Resource::removeClaim( Claim* c )
 }
 
 
-int
+void
 Resource::releaseAllClaims( void )
 {
-	return shutdownAllClaims( true );
+	shutdownAllClaims( true );
 }
 
 
-int
+void
 Resource::killAllClaims( void )
 {
-	return shutdownAllClaims( false );
+	shutdownAllClaims( false );
 }
 
 
-int
+void
 Resource::shutdownAllClaims( bool graceful )
 {
 		// shutdown the COD claims
 	r_cod_mgr->shutdownAllClaims( graceful );
 
-	if( graceful ) {
-		retire_claim();
+	if( Resource::DYNAMIC_SLOT == get_feature() ) {
+		if( graceful ) {
+			void_retire_claim();
+		} else {
+			void_kill_claim();
+		}
+
+		// We have deleted ourself and can't send any updates.
 	} else {
-		kill_claim();
+		if( graceful ) {
+			void_retire_claim();
+		} else {
+			void_kill_claim();
+		}
+
+			// Tell the negotiator not to match any new jobs to this slot,
+			// since they would just be rejected by the startd anyway.
+		r_reqexp->unavail();
+		update();
 	}
-
-		// Tell the negotiator not to match any new jobs to this slot,
-		// since they would just be rejected by the startd anyway.
-	r_reqexp->unavail();
-	update();
-
-	return TRUE;
 }
 
 bool
@@ -493,7 +519,8 @@ Resource::suspendForCOD( void )
     case CLAIM_RUNNING:
 		dprintf( D_ALWAYS, "State change: Suspending because a COD "
 				 "job is now running\n" );
-		did_update = change_state( suspended_act );
+		change_state( suspended_act );
+		did_update = TRUE;
 		break;
 
     case CLAIM_VACATING:
@@ -556,7 +583,8 @@ Resource::resumeForCOD( void )
     case CLAIM_SUSPENDED:
 		dprintf( D_ALWAYS, "State Change: No running COD job, "
 				 "resuming opportunistic claim\n" );
-		did_update = change_state( busy_act );
+		change_state( busy_act );
+		did_update = TRUE;
 		break;
 
     case CLAIM_IDLE:
@@ -578,17 +606,11 @@ Resource::hackLoadForCOD( void )
 		return;
 	}
 
-	char float_buf[64];
+	MyString load;
+	load.sprintf( "%s=%.2f", ATTR_LOAD_AVG, r_pre_cod_total_load );
 
-	MyString load = ATTR_LOAD_AVG;
-	load += '=';
-	sprintf( float_buf, "%.2f", r_pre_cod_total_load );
-	load += float_buf;
-
-	MyString c_load = ATTR_CONDOR_LOAD_AVG;
-	c_load += '=';
-	sprintf( float_buf, "%.2f", r_pre_cod_condor_load );
-	c_load += float_buf;
+	MyString c_load;
+	c_load.sprintf( "%s=%.2f", ATTR_CONDOR_LOAD_AVG, r_pre_cod_condor_load );
 
 	if( DebugFlags & D_FULLDEBUG && DebugFlags & D_LOAD ) {
 		if( r_cod_mgr->isRunning() ) {
@@ -604,13 +626,9 @@ Resource::hackLoadForCOD( void )
 	r_classad->Insert( load.Value() );
 	r_classad->Insert( c_load.Value() );
 
-	MyString line = ATTR_CPU_IS_BUSY;
-	line += "=False";
-	r_classad->Insert( line.Value() );
+	r_classad->Assign( ATTR_CPU_IS_BUSY, false );
 
-	line = ATTR_CPU_BUSY_TIME;
-	line += "=0";
-	r_classad->Insert( line.Value() );
+	r_classad->Assign( ATTR_CPU_BUSY_TIME, 0 );
 }
 
 
@@ -847,7 +865,7 @@ Resource::leave_preempting_state( void )
 }
 
 
-int
+void
 Resource::init_classad( void )
 {
 	ASSERT( resmgr->config_classad );
@@ -860,8 +878,6 @@ Resource::init_classad( void )
 		// init_classad is being called, we don't necessarily have
 		// classads for the other slots, yet we'll publish the SHARED_SLOT
 		// attrs after this...
-
-	return TRUE;
 }
 
 
@@ -878,15 +894,21 @@ Resource::refresh_classad( amask_t mask )
 
 
 int
-Resource::force_benchmark( void )
+Resource::benchmarks_started( void )
 {
-		// Force this resource to run benchmarking.
-	resmgr->m_attr->benchmark( this, 1 );
-	return TRUE;
+	return 0;
 }
 
-
 int
+Resource::benchmarks_finished( void )
+{
+	resmgr->m_attr->benchmarks_finished( this );
+	time_t last_benchmark = time(NULL);
+	r_classad->Assign( ATTR_LAST_BENCHMARK, (unsigned)last_benchmark );
+	return 0;
+}
+
+void
 Resource::reconfig( void )
 {
 #if HAVE_JOB_HOOKS
@@ -896,16 +918,13 @@ Resource::reconfig( void )
 	}
 	m_hook_keyword_initialized = false;
 #endif /* HAVE_JOB_HOOKS */
-	// This bogus return makes the prototype happy for ResMgr::walk().
-	return TRUE;
 }
 
 
-int
+void
 Resource::update( void )
 {
 	int timeout = 3;
-	int ret_value = TRUE;
 
 	if ( update_tid == -1 ) {
 			// Send no more than 16 ClassAds per second to help
@@ -928,13 +947,10 @@ Resource::update( void )
 	if ( update_tid < 0 ) {
 		// Somehow, the timer could not be set.  Ick!
 		update_tid = -1;
-		ret_value = FALSE;
 	}
-
-	return ret_value;
 }
 
-int
+void
 Resource::do_update( void )
 {
 	int rval;
@@ -944,8 +960,10 @@ Resource::do_update( void )
         // Get the public and private ads
     publish_for_update( &public_ad, &private_ad );
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 	StartdPluginManager::Update(&public_ad, &private_ad);
+#endif
 #endif
 
 		// Send class ads to collector(s)
@@ -960,8 +978,6 @@ Resource::do_update( void )
 	// We _must_ reset update_tid to -1 before we return so
 	// the class knows there is no pending update.
 	update_tid = -1;
-
-	return rval;
 }
 
 void
@@ -991,19 +1007,28 @@ void
 Resource::final_update( void )
 {
 	ClassAd invalidate_ad;
-	char line[256];
+	MyString line;
+	MyString escaped_name;
 
 		// Set the correct types
 	invalidate_ad.SetMyTypeName( QUERY_ADTYPE );
 	invalidate_ad.SetTargetTypeName( STARTD_ADTYPE );
 
-		// We only want to invalidate this slot.
-	sprintf( line, "%s = %s == \"%s\"", ATTR_REQUIREMENTS, ATTR_NAME,
-			 r_name );
-	invalidate_ad.Insert( line );
+	/*
+	 * NOTE: the collector depends on the data below for performance reasons
+	 * if you change here you will need to CollectorEngine::remove (AdTypes t_AddType, const ClassAd & c_query)
+	 * the IP was added to allow the collector to create a hash key to delete in O(1).
+     */
+	 ClassAd::EscapeStringValue( r_name, escaped_name );
+     line.sprintf( "( TARGET.%s == \"%s\" )", ATTR_NAME, escaped_name.Value() );
+     invalidate_ad.AssignExpr( ATTR_REQUIREMENTS, line.Value() );
+     invalidate_ad.Assign( ATTR_NAME, r_name );
+     invalidate_ad.Assign( ATTR_MY_ADDRESS, daemonCore->publicNetworkIpAddr());
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 	StartdPluginManager::Invalidate(&invalidate_ad);
+#endif
 #endif
 
 	resmgr->send_update( INVALIDATE_STARTD_ADS, &invalidate_ad, NULL, false );
@@ -1076,7 +1101,7 @@ Resource::update_with_ack( void )
 
     }
 
-    if ( !socket->eom () ) {
+    if ( !socket->end_of_message() ) {
 
         dprintf (
             D_FULLDEBUG,
@@ -1111,24 +1136,53 @@ Resource::update_with_ack( void )
 
     }
 
-    socket->eom ();
+    socket->end_of_message();
 
     return success;
 
 }
 
+void
+Resource::hold_job( )
+{
+	MyString hold_reason;
+	int hold_subcode = 0;
+
+	r_classad->EvalString("WANT_HOLD_REASON",r_cur->ad(),hold_reason);
+	if( hold_reason.IsEmpty() ) {
+		ExprTree *want_hold_expr;
+		MyString want_hold_str;
+
+		want_hold_expr = r_classad->LookupExpr("WANT_HOLD");
+		if( want_hold_expr ) {
+			want_hold_str.sprintf( "%s = %s", "WANT_HOLD",
+								   ExprTreeToString( want_hold_expr ) );
+		}
+
+		hold_reason = "The startd put this job on hold.  (At preemption time, WANT_HOLD evaluated to true: ";
+		hold_reason += want_hold_str;
+		hold_reason += ")";
+	}
+
+	r_classad->EvalInteger("WANT_HOLD_SUBCODE",r_cur->ad(),hold_subcode);
+
+	r_cur->starterHoldJob(hold_reason.Value(),CONDOR_HOLD_CODE_StartdHeldJob,hold_subcode);
+}
 
 int
-Resource::eval_and_update( void )
+Resource::wants_hold( void )
 {
-		// Evaluate the state of this resource.
-	eval_state();
+	int want_hold = eval_expr("WANT_HOLD",false,false);
 
-		// If we didn't update b/c of the eval_state, we need to
-		// actually do the update now.
-	update();
-
-	return TRUE;
+	if( want_hold == -1 ) {
+		want_hold = 0;
+		dprintf(D_ALWAYS,"State change: WANT_HOLD is undefined --> FALSE.\n");
+	}
+	else {
+		dprintf( D_ALWAYS, "State change: WANT_HOLD is %s\n",
+				 want_hold ? "TRUE" : "FALSE" );
+	}
+	return want_hold;
 }
 
 
@@ -1220,10 +1274,8 @@ Resource::wants_suspend( void )
 		if( r_classad->EvalBool( "WANT_SUSPEND",
 								   r_cur->ad(),
 								   want_suspend ) == 0) {
-				// This should never happen, since we already check
-				// when we're constructing the internal config classad
-				// if we've got this defined. -Derek Wright 4/12/00
-			EXCEPT( "Can't find WANT_SUSPEND in internal ClassAd" );
+				// UNDEFINED means FALSE for WANT_SUSPEND
+			want_suspend = false;
 		}
 	}
 	return want_suspend;
@@ -1307,7 +1359,7 @@ Resource::claimWorklifeExpired()
 		int ClaimWorklife = 0;
 
 		//look up the maximum retirement time specified by the startd
-		if(!r_classad->LookupElem("CLAIM_WORKLIFE") ||
+		if(!r_classad->LookupExpr("CLAIM_WORKLIFE") ||
 		   !r_classad->EvalInteger(
 				  "CLAIM_WORKLIFE",
 		          NULL,
@@ -1337,7 +1389,7 @@ Resource::retirementExpired()
 
 	if (r_cur && r_cur->isActive() && r_cur->ad()) {
 		//look up the maximum retirement time specified by the startd
-		if(!r_classad->LookupElem(ATTR_MAX_JOB_RETIREMENT_TIME) ||
+		if(!r_classad->LookupExpr(ATTR_MAX_JOB_RETIREMENT_TIME) ||
 		   !r_classad->EvalInteger(
 		          ATTR_MAX_JOB_RETIREMENT_TIME,
 		          r_cur->ad(),
@@ -1350,7 +1402,7 @@ Resource::retirementExpired()
 			MaxJobRetirementTime = INT_MAX;
 		}
 		//look up the maximum retirement time specified by the job
-		if(r_cur->ad()->LookupElem(ATTR_MAX_JOB_RETIREMENT_TIME) &&
+		if(r_cur->ad()->LookupExpr(ATTR_MAX_JOB_RETIREMENT_TIME) &&
 		   r_cur->ad()->EvalInteger(
 		          ATTR_MAX_JOB_RETIREMENT_TIME,
 		          r_classad,
@@ -1380,7 +1432,7 @@ int
 Resource::eval_expr( const char* expr_name, bool fatal, bool check_vanilla )
 {
 	int tmp;
-	if( check_vanilla && r_cur->universe() == CONDOR_UNIVERSE_VANILLA ) {
+	if( check_vanilla && r_cur && r_cur->universe() == CONDOR_UNIVERSE_VANILLA ) {
 		MyString tmp_expr_name = expr_name;
 		tmp_expr_name += "_VANILLA";
 		tmp = eval_expr( tmp_expr_name.Value(), false, false );
@@ -1390,7 +1442,7 @@ Resource::eval_expr( const char* expr_name, bool fatal, bool check_vanilla )
 		}
 			// otherwise, fall through and try the non-vanilla version
 	}
-	if( check_vanilla && r_cur->universe() == CONDOR_UNIVERSE_VM ) {
+	if( check_vanilla && r_cur && r_cur->universe() == CONDOR_UNIVERSE_VM ) {
 		MyString tmp_expr_name = expr_name;
 		tmp_expr_name += "_VM";
 		tmp = eval_expr( tmp_expr_name.Value(), false, false );
@@ -1400,12 +1452,16 @@ Resource::eval_expr( const char* expr_name, bool fatal, bool check_vanilla )
 		}
 			// otherwise, fall through and try the non-vm version
 	}
-	if( (r_classad->EvalBool(expr_name, r_cur->ad(), tmp) ) == 0 ) {
+	if( (r_classad->EvalBool(expr_name, r_cur ? r_cur->ad() : NULL , tmp) ) == 0 ) {
 		if( fatal ) {
 			dprintf(D_ALWAYS, "Can't evaluate %s in the context of following ads\n", expr_name );
 			r_classad->dPrint(D_ALWAYS);
 			dprintf(D_ALWAYS, "=============================\n");
-			r_cur->ad()->dPrint(D_ALWAYS);
+			if ( r_cur && r_cur->ad() ) {
+				r_cur->ad()->dPrint(D_ALWAYS);
+			} else {
+				dprintf( D_ALWAYS, "<no job ad>\n" );
+			}
 			EXCEPT( "Can't evaluate %s", expr_name );
 		} else {
 				// anything else for here?
@@ -1563,8 +1619,6 @@ Resource::hardkill_backfill( void )
 void
 Resource::publish( ClassAd* cap, amask_t mask )
 {
-	char line[256];
-	MyString my_line;
 	State s;
 	char* ptr;
 
@@ -1577,12 +1631,10 @@ Resource::publish( ClassAd* cap, amask_t mask )
 
 	if( IS_STATIC(mask) ) {
 			// We need these for both public and private ads
-		sprintf( line, "%s = \"%s\"", ATTR_STARTD_IP_ADDR,
+		cap->Assign( ATTR_STARTD_IP_ADDR,
 				 daemonCore->InfoCommandSinfulString() );
-		cap->Insert( line );
 
-		sprintf( line, "%s = \"%s\"", ATTR_NAME, r_name );
-		cap->Insert( line );
+		cap->Assign( ATTR_NAME, r_name );
 	}
 
 	if( IS_PUBLIC(mask) && IS_STATIC(mask) ) {
@@ -1593,8 +1645,7 @@ Resource::publish( ClassAd* cap, amask_t mask )
 			// value, since we don't want to use the job ClassAd's
 			// Rank expression when we evaluate our Rank value.
 		if( !caInsert(cap, r_classad, ATTR_RANK) ) {
-			sprintf( line, "%s = 0.0", ATTR_RANK );
-			cap->Insert( line );
+			cap->Assign( ATTR_RANK, 0.0 );
 		}
 
 			// Similarly, the CpuBusy expression only lives in the
@@ -1608,15 +1659,31 @@ Resource::publish( ClassAd* cap, amask_t mask )
 					"should be added by ResMgr!", ATTR_CPU_BUSY );
 		}
 
+		caInsert(cap, r_classad, ATTR_SLOT_WEIGHT);
+
+#if HAVE_HIBERNATION
+		caInsert(cap, r_classad, ATTR_UNHIBERNATE);
+#endif
+
 			// Include everything from STARTD_EXPRS.
 			// And then include everything from SLOTx_STARTD_EXPRS
 		daemonCore->publish(cap);
-		config_fill_ad( cap, r_id_str );
+
+		// config_fill_ad can not take strings with "." in it's prefix
+		// e.g. slot1.1, instead needs to be slot1
+		MyString szTmp(r_id_str);
+		int iPeriodPos = szTmp.find(".");
+
+		if ( iPeriodPos >=0 ) {
+			szTmp.setChar ( iPeriodPos,  '\0' );
+		}
+		
+		config_fill_ad( cap, szTmp.Value() );
 
 			// Also, include a slot ID attribute, since it's handy for
 			// defining expressions, and other things.
 		cap->Assign(ATTR_SLOT_ID, r_id);
-		if (param_boolean("ALLOW_VM_CRUFT", true)) {
+		if (param_boolean("ALLOW_VM_CRUFT", false)) {
 			cap->Assign(ATTR_VIRTUAL_MACHINE_ID, r_id);
 		}
 
@@ -1627,6 +1694,8 @@ Resource::publish( ClassAd* cap, amask_t mask )
 		case DYNAMIC_SLOT:
 			cap->AssignExpr(ATTR_SLOT_DYNAMIC, "TRUE");
 			break;
+		default:
+			break; // Do nothing
 		}
 	}		
 
@@ -1660,13 +1729,9 @@ Resource::publish( ClassAd* cap, amask_t mask )
 		// If this is a public ad, publish anything we had to evaluate
 		// to "compute"
 	if( IS_PUBLIC(mask) && IS_EVALUATED(mask) ) {
-		sprintf( line, "%s=%d", ATTR_CPU_BUSY_TIME,
-				 (int)cpu_busy_time() );
-		cap->Insert(line);
+		cap->Assign( ATTR_CPU_BUSY_TIME, (int)cpu_busy_time() );
 
-		sprintf( line, "%s=%s", ATTR_CPU_IS_BUSY,
-				 r_cpu_busy ? "True" : "False" );
-		cap->Insert(line);
+		cap->Assign( ATTR_CPU_IS_BUSY, r_cpu_busy ? true : false );
 
         publishDeathTime( cap );
 	}
@@ -1679,34 +1744,29 @@ Resource::publish( ClassAd* cap, amask_t mask )
 
 		// Put in max job retirement time expression
 	ptr = param(ATTR_MAX_JOB_RETIREMENT_TIME);
-	if(!ptr) {
-		ptr = strdup("0");
-	} else if ( !*ptr ) {
+	if( ptr && !*ptr ) {
 		free(ptr);
-		ptr = strdup("0");
+		ptr = NULL;
 	}
-	my_line.sprintf("%s=%s",ATTR_MAX_JOB_RETIREMENT_TIME,ptr);
-	cap->Insert(my_line.Value());
+	cap->AssignExpr( ATTR_MAX_JOB_RETIREMENT_TIME, ptr ? ptr : "0" );
 
 	free(ptr);
 
 #if HAVE_JOB_HOOKS
 	if (IS_PUBLIC(mask)) {
-		my_line.sprintf("%s=%d", ATTR_LAST_FETCH_WORK_SPAWNED,
-						(int)m_last_fetch_work_spawned);
-		cap->Insert(my_line.Value());
-		my_line.sprintf("%s=%d", ATTR_LAST_FETCH_WORK_COMPLETED,
-						(int)m_last_fetch_work_completed);
-		cap->Insert(my_line.Value());
-		my_line.sprintf("%s=%d", ATTR_NEXT_FETCH_WORK_DELAY,
-						m_next_fetch_work_delay);
-		cap->Insert(my_line.Value());
+		cap->Assign( ATTR_LAST_FETCH_WORK_SPAWNED, 
+					 (int)m_last_fetch_work_spawned );
+		cap->Assign( ATTR_LAST_FETCH_WORK_COMPLETED,
+					 (int)m_last_fetch_work_completed );
+		cap->Assign( ATTR_NEXT_FETCH_WORK_DELAY,
+					 m_next_fetch_work_delay );
 	}
 #endif /* HAVE_JOB_HOOKS */
 
 		// Update info from the current Claim object, if it exists.
 	if( r_cur ) {
 		r_cur->publish( cap, mask );
+        if (state() == claimed_state)  cap->Assign(ATTR_PUBLIC_CLAIM_ID, r_cur->publicClaimId());
 	}
 	if( r_pre ) {
 		r_pre->publishPreemptingClaim( cap, mask );
@@ -1718,7 +1778,7 @@ Resource::publish( ClassAd* cap, amask_t mask )
 	r_cod_mgr->publish( cap, mask );
 
 	// Publish the supplemental Class Ads
-	resmgr->adlist_publish( cap, mask );
+	resmgr->adlist_publish( r_id, cap, mask );
 
     // Publish the monitoring information
     daemonCore->monitor_data.ExportData( cap );
@@ -1726,6 +1786,10 @@ Resource::publish( ClassAd* cap, amask_t mask )
 	if( IS_PUBLIC(mask) && IS_SHARED_SLOT(mask) ) {
 		resmgr->publishSlotAttrs( cap );
 	}
+
+#if !defined(WANT_OLD_CLASSADS)
+	cap->AddTargetRefs( TargetJobAttrs, false );
+#endif
 }
 
 void
@@ -1886,7 +1950,7 @@ Resource::compute( amask_t mask )
 
 
 void
-Resource::dprintf_va( int flags, char* fmt, va_list args )
+Resource::dprintf_va( int flags, const char* fmt, va_list args )
 {
 	if( resmgr->is_smp() ) {
 		MyString fmt_str( r_id_str );
@@ -1900,7 +1964,7 @@ Resource::dprintf_va( int flags, char* fmt, va_list args )
 
 
 void
-Resource::dprintf( int flags, char* fmt, ... )
+Resource::dprintf( int flags, const char* fmt, ... )
 {
 	va_list args;
 	va_start( args, fmt );
@@ -2310,6 +2374,17 @@ Resource::fetchCompleted(void)
 		// Now that a fetch hook returned, (re)set our timer to try
 		// fetching again based on the delay expression.
 	resetFetchWorkTimer();
+
+		// If we are a dynamically created slot, it is possible that
+		// we became unclaimed while waiting for the fetch to
+		// complete. Now that it has we can reevaluate our state and
+		// potentially delete ourself.
+	if ( get_feature() == DYNAMIC_SLOT ) {
+		// WARNING: This must be the last thing done in response to a
+		// hook exiting, if it isn't then there is a chance we will be
+		// referenced after we are deleted.
+		eval_state();
+	}
 }
 
 
@@ -2337,19 +2412,19 @@ Resource::evalNextFetchWorkDelay(void)
 }
 
 
-bool
+void
 Resource::tryFetchWork(void)
 {
 		// First, make sure we're configured for fetching at all.
 	if (!getHookKeyword()) {
 			// No hook keyword for ths slot, bail out.
-		return false;
+		return;
 	}
 
 		// Then, make sure we're not currently fetching.
 	if (m_currently_fetching) {
 			// No need to log a message about this, it's not an error.
-		return false;
+		return;
 	}
 
 		// Now, make sure we  haven't fetched too recently.
@@ -2363,7 +2438,7 @@ Resource::tryFetchWork(void)
 				// our timer to go off again when we think we'd be
 				// ready, and bail out.
 			resetFetchWorkTimer(m_next_fetch_work_delay - delta);
-			return false;
+			return;
 		}
 	}
 
@@ -2374,13 +2449,13 @@ Resource::tryFetchWork(void)
 			// fetching delay was already reached, so we should reset
 			// our timer for another full delay.
 		resetFetchWorkTimer();
-		return false;
+		return;
 	}
 
 		// We're ready to invoke the hook. The timer to re-fetch will
 		// be reset once the hook completes.
 	resmgr->m_hook_mgr->invokeHookFetchWork(this);
-	return true;
+	return;
 }
 
 
@@ -2443,7 +2518,7 @@ void Resource::disable()
 {
 
     /* kill the claim */
-	kill_claim ();
+	void_kill_claim ();
 
 	/* let the negotiator know not to match any new jobs to
     this slot */
