@@ -33,9 +33,12 @@
 #include "setenv.h"
 #include "my_popen.h"
 #include "basename.h"
+#include "dc_starter.h"
+#include "classadHistory.h"
+#include "classad_helpers.h"
 
 #if defined(LINUX)
-#include "glexec_starter.h"
+#include "glexec_starter.linux.h"
 #endif
 
 #ifdef WIN32
@@ -94,6 +97,16 @@ Starter::initRunData( void )
 	s_is_boinc = false;
 #endif /* HAVE_BOINC */
 	s_job_update_sock = NULL;
+
+	m_hold_job_cb = NULL;
+
+		// XXX: ProcFamilyUsage needs a constructor
+	s_usage.max_image_size = 0;
+	s_usage.num_procs = 0;
+	s_usage.percent_cpu = 0.0;
+	s_usage.sys_cpu_time = 0;
+	s_usage.total_image_size = 0;
+	s_usage.user_cpu_time = 0;
 }
 
 
@@ -111,6 +124,11 @@ Starter::~Starter()
 		daemonCore->Cancel_Socket( s_job_update_sock );
 		delete s_job_update_sock;
 	}
+
+	if( m_hold_job_cb ) {
+		m_hold_job_cb->cancelCallback();
+		m_hold_job_cb = NULL;
+	}
 }
 
 
@@ -211,44 +229,31 @@ Starter::publish( ClassAd* ad, amask_t mask, StringList* list )
 		ignored_attr_list->append(ATTR_STARTER_IGNORED_ATTRS);
 	}
 
-	ExprTree *tree, *lhs;
-	char *expr_str = NULL, *lhstr = NULL;
+	ExprTree *tree;
+	const char *lhstr = NULL;
 	s_ad->ResetExpr();
-	while( (tree = s_ad->NextExpr()) ) {
-		if( (lhs = tree->LArg()) ) {
-			lhs->PrintToNewStr( &lhstr );
-		} else {
-			dprintf( D_ALWAYS, 
-					 "ERROR parsing Starter classad attribute!\n" );
-			continue;
-		}
-		tree->PrintToNewStr( &expr_str );
+	while( s_ad->NextExpr(lhstr, tree) ) {
 
 		if (ignored_attr_list) {
 				// insert every attr that's not in the ignored_attr_list
 			if (!ignored_attr_list->contains(lhstr)) {
-				ad->Insert(expr_str);
-				if (strincmp(lhstr, "Has", 3) == MATCH) {
+				ad->Insert(lhstr, tree->Copy());
+				if (strncasecmp(lhstr, "Has", 3) == MATCH) {
 					list->append(lhstr);
 				}
 			}
 		}
 		else {
 				// no list of attrs to ignore - fallback on old behavior
-			if( strincmp(lhstr, "Has", 3) == MATCH ) {
-				ad->Insert( expr_str );
+			if( strncasecmp(lhstr, "Has", 3) == MATCH ) {
+				ad->Insert( lhstr, tree->Copy() );
 				if( list ) {
 					list->append( lhstr );
 				}
-			} else if( strincmp(lhstr, "Java", 4) == MATCH ) {
-				ad->Insert( expr_str );
+			} else if( strncasecmp(lhstr, "Java", 4) == MATCH ) {
+				ad->Insert( lhstr, tree->Copy() );
 			}
 		}
-
-		free( expr_str );
-		expr_str = NULL;
-		free( lhstr );
-		lhstr = NULL;
 	}
 
 	if (ignored_attr_list) {
@@ -286,8 +291,7 @@ Starter::reallykill( int signo, int type )
 #endif
 	int 		ret = 0, sig = 0;
 	priv_state	priv;
-	char		signame[32];
-	signame[0]='\0';
+	const char *signame = "";
 
 	if ( s_pid == 0 ) {
 			// If there's no starter, just return.  We've done our task.
@@ -296,27 +300,27 @@ Starter::reallykill( int signo, int type )
 
 	switch( signo ) {
 	case DC_SIGSUSPEND:
-		sprintf( signame, "DC_SIGSUSPEND" );
+		signame = "DC_SIGSUSPEND";
 		break;
 	case DC_SIGHARDKILL:
 		signo = SIGQUIT;
-		sprintf( signame, "SIGQUIT" );
+		signame = "SIGQUIT";
 		break;
 	case DC_SIGSOFTKILL:
 		signo = SIGTERM;
-		sprintf( signame, "SIGTERM" );
+		signame = "SIGTERM";
 		break;
 	case DC_SIGPCKPT:
-		sprintf( signame, "DC_SIGPCKPT" );
+		signame = "DC_SIGPCKPT";
 		break;
 	case DC_SIGCONTINUE:
-		sprintf( signame, "DC_SIGCONTINUE" );
+		signame = "DC_SIGCONTINUE";
 		break;
 	case SIGHUP:
-		sprintf( signame, "SIGHUP" );
+		signame = "SIGHUP";
 		break;
 	case SIGKILL:
-		sprintf( signame, "SIGKILL" );
+		signame = "SIGKILL";
 		break;
 	default:
 		EXCEPT( "Unknown signal (%d) in Starter::reallykill", signo );
@@ -402,7 +406,7 @@ Starter::reallykill( int signo, int type )
 		case EACCES:
 			needs_stat = FALSE;
 			break;
-#if defined(OSF1) || defined(Darwin) || defined(CONDOR_FREEBSD)
+#if defined(Darwin) || defined(CONDOR_FREEBSD)
 				// dux 4.0 doesn't have ENOLINK for stat().  It does
 				// have ESTALE, which means our binaries live on a
 				// stale NFS mount.  So, we can at least EXCEPT with a
@@ -449,6 +453,7 @@ Starter::reallykill( int signo, int type )
 		dprintf( D_FULLDEBUG, 
 				 "In Starter::killpg() with pid %d, sig %d (%s)\n", 
 				 s_pid, signo, signame );
+		break;
 	case 2:
 		dprintf( D_FULLDEBUG, 
 				 "In Starter::kill_kids() with pid %d, sig %d (%s)\n", 
@@ -468,7 +473,9 @@ Starter::reallykill( int signo, int type )
 		if( sig != SIGSTOP && sig != SIGCONT && sig != SIGKILL ) {
 			if( type == 1 ) { 
 				ret = ::kill( -(s_pid), SIGCONT );
-			} else if( type == 0 ) {
+			} else if( type == 0 && 
+						!daemonCore->ProcessExitedButNotReaped(s_pid)) 
+			{
 				ret = ::kill( (s_pid), SIGCONT );
 			}
 		}
@@ -490,7 +497,9 @@ Starter::reallykill( int signo, int type )
 		} 
 #ifndef WIN32
 		else {
-			ret = ::kill( (s_pid), sig );
+			if (!daemonCore->ProcessExitedButNotReaped(s_pid)) {
+				ret = ::kill( (s_pid), sig );
+			}
 			break;
 		}
 #endif /* ! WIN32 */
@@ -589,8 +598,50 @@ Starter::spawn( time_t now, Stream* s )
 }
 
 void
-Starter::exited()
+Starter::exited(int status)
 {
+	ClassAd *jobAd = NULL;
+	bool jobAdNeedsFree = true;
+
+	if (s_claim && s_claim->ad()) {
+		// real jobs in the startd have claims and job ads, boinc and perhaps others won't
+		jobAd = s_claim->ad();
+		jobAdNeedsFree = false;
+	} else {
+		// Dummy up an ad
+		int now = (int) time(0);
+		jobAd = new ClassAd();
+		jobAd->SetMyTypeName("Job");
+		jobAd->SetTargetTypeName("Machine");
+		jobAd->Assign(ATTR_CLUSTER_ID, now);
+		jobAd->Assign(ATTR_PROC_ID, 1);
+		jobAd->Assign(ATTR_OWNER, "boinc");
+		jobAd->Assign(ATTR_Q_DATE, (int)s_birthdate);
+		jobAd->Assign(ATTR_JOB_PRIO, 0);
+		jobAd->Assign(ATTR_IMAGE_SIZE, 0);
+		jobAd->Assign(ATTR_JOB_CMD, "boinc");
+		MyString gjid;
+		gjid.sprintf("%s#%d#%d#%d", my_hostname(), now, 1, now);
+		jobAd->Assign(ATTR_GLOBAL_JOB_ID, gjid);
+	}
+
+	// First, patch up the ad a little bit 
+	jobAd->Assign(ATTR_COMPLETION_DATE, (int)time(0));
+	int runtime = time(0) - s_birthdate;
+	
+	jobAd->Assign(ATTR_JOB_REMOTE_WALL_CLOCK, runtime);
+	int jobStatus = COMPLETED;
+	if (WIFSIGNALED(status)) {
+		jobStatus = REMOVED;
+	}
+	jobAd->Assign(ATTR_JOB_STATUS, jobStatus);
+	AppendHistory(jobAd);
+	WritePerJobHistoryFile(jobAd, true /* use gjid for filename*/);
+
+	if (jobAdNeedsFree) {
+		delete jobAd;
+	}
+
 		// Make sure our time isn't going to go off.
 	cancelKillTimer();
 
@@ -748,6 +799,14 @@ Starter::receiveJobClassAdUpdate( Stream *stream )
 		dprintf(D_FULLDEBUG, "Received job ClassAd update from starter.\n");
 		update_ad.dPrint( D_JOB );
 
+		// In addition to new info about the job, the starter also
+		// inserts contact info for itself (important for CCB and
+		// shadow-starter reconnect, because startd needs to relay
+		// starter's full contact info to the shadow when queried).
+		// It's a bit of a hack to do it through this channel, but
+		// better than nothing.
+		update_ad.LookupString(ATTR_STARTER_IP_ADDR,m_starter_addr);
+
 		if( s_claim ) {
 			s_claim->receiveJobClassAdUpdate( update_ad );
 		}
@@ -803,6 +862,10 @@ Starter::execDCStarter( ArgList const &args, Env const *env,
 	}
 	inherit_list[0] = &child_job_update_sock;
 
+	// Pass the machine ad to the starter
+	if (s_claim) 
+		s_claim->writeMachAd( s_job_update_sock );
+
 	if( daemonCore->Register_Socket(
 			s_job_update_sock,
 			"starter ClassAd update socket",
@@ -1008,7 +1071,7 @@ Starter::cleanupAfterGlexec()
 {
 	// remove the copy of the user proxy that we own
 	// (the starter should remove the one glexec created for it)
-	if( s_claim->client()->proxyFile() != NULL ) {
+	if( s_claim && s_claim->client()->proxyFile() != NULL ) {
 		if ( unlink( s_claim->client()->proxyFile() ) == -1) {
 			dprintf( D_ALWAYS,
 			         "error removing temporary proxy %s: %s (%d)\n",
@@ -1039,7 +1102,7 @@ Starter::active()
 	
 
 void
-Starter::dprintf( int flags, char* fmt, ... )
+Starter::dprintf( int flags, const char* fmt, ... )
 {
 	va_list args;
 	va_start( args, fmt );
@@ -1108,12 +1171,23 @@ Starter::printInfo( int debug_level )
 }
 
 
-char*
+char const*
 Starter::getIpAddr( void )
 {
 	if( ! s_pid ) {
 		return NULL;
 	}
+	if( !m_starter_addr.IsEmpty() ) {
+		return m_starter_addr.Value();
+	}
+
+	// Fall back on the raw contact string known to daemonCore.
+	// Unfortunately, that doesn't include any of the fancy
+	// stuff like private network info and CCB.
+	dprintf(D_ALWAYS,
+			"Warning: giving raw address in response to starter address query,"
+			"because update from starter not received yet.\n");
+
 	return daemonCore->InfoCommandSinfulString( s_pid );
 }
 
@@ -1124,11 +1198,14 @@ Starter::killHard( void )
 	if( ! active() ) {
 		return true;
 	}
+	
 	if( ! kill(DC_SIGHARDKILL) ) {
 		killpg( SIGKILL );
 		return false;
 	}
-	startKillTimer();
+	dprintf(D_FULLDEBUG, "in starter:killHard starting kill timer\n");
+	startKillTimer();	
+
 	return true;
 }
 
@@ -1182,7 +1259,7 @@ Starter::startKillTimer( void )
 			// Timer already started.
 		return TRUE;
 	}
-
+ 
 	int tmp_killing_timeout = killing_timeout;
 	if( s_claim && (s_claim->universe() == CONDOR_UNIVERSE_VM) ) {
 		// For vm universe, we need longer killing_timeout
@@ -1225,7 +1302,7 @@ Starter::cancelKillTimer( void )
 }
 
 
-bool
+void
 Starter::sigkillStarter( void )
 {
 		// In case the kill fails for some reason, we are on a periodic
@@ -1241,8 +1318,42 @@ Starter::sigkillStarter( void )
 		killkids( SIGKILL );
 
 			// Kill the starter's entire process group.
-		return killpg( SIGKILL );
+		killpg( SIGKILL );
 	}
+}
+
+bool
+Starter::holdJob(char const *hold_reason,int hold_code,int hold_subcode)
+{
+	if( !s_is_dc ) {
+		return false;  // this starter does not support putting jobs on hold
+	}
+	if( m_hold_job_cb ) {
+		dprintf(D_ALWAYS,"holdJob() called when operation already in progress (starter pid %d).\n", s_pid);
+		return true;
+	}
+
+	classy_counted_ptr<DCStarter> starter = new DCStarter(getIpAddr());
+	classy_counted_ptr<StarterHoldJobMsg> msg = new StarterHoldJobMsg(hold_reason,hold_code,hold_subcode);
+
+	m_hold_job_cb = new DCMsgCallback( (DCMsgCallback::CppFunction)&Starter::holdJobCallback, this );
+
+	msg->setCallback( m_hold_job_cb );
+
+	starter->sendMsg(msg.get());
+
 	return true;
 }
 
+void
+Starter::holdJobCallback(DCMsgCallback *cb)
+{
+	ASSERT( m_hold_job_cb == cb );
+	m_hold_job_cb = NULL;
+
+	ASSERT( cb->getMessage() );
+	if( cb->getMessage()->deliveryStatus() != DCMsg::DELIVERY_SUCCEEDED ) {
+		dprintf(D_ALWAYS,"Failed to hold job (starter pid %d), so killing it.\n", s_pid);
+		killSoft();
+	}
+}
