@@ -84,6 +84,8 @@ Claim::Claim( Resource* res_ip, ClaimType claim_type, int lease_duration )
 	c_retire_peacefully = false;
 	c_preempt_was_true = false;
 	c_schedd_closed_claim = false;
+
+	c_last_state = CLAIM_UNCLAIMED;
 }
 
 
@@ -208,6 +210,15 @@ Claim::publish( ClassAd* cad, amask_t how_much )
 		if (tmp) {
 			cad->Assign(ATTR_CONCURRENCY_LIMITS, tmp);
 		}
+
+		int numJobPids = c_client->numPids();
+		
+		//In standard universe, numJobPids should be 1
+		if(c_universe == CONDOR_UNIVERSE_STANDARD) {
+			numJobPids = 1;
+		}
+		line.sprintf("%s=%d", ATTR_NUM_PIDS, numJobPids);
+		cad->Insert( line.Value() );
 	}
 
 	if( (c_cluster > 0) && (c_proc >= 0) ) {
@@ -251,7 +262,7 @@ Claim::publish( ClassAd* cad, amask_t how_much )
 }
 
 void
-Claim::publishPreemptingClaim( ClassAd* cad, amask_t how_much )
+Claim::publishPreemptingClaim( ClassAd* cad, amask_t /*how_much*/ /*UNUSED*/ )
 {
 	MyString line;
 	char* tmp;
@@ -376,7 +387,7 @@ Claim::publishCOD( ClassAd* cad )
 		}
 		char buf[128];
 		if( (c_cluster > 0) && (c_proc >= 0) ) {
-			sprintf( buf, "%d.%d", c_cluster, c_proc );
+			snprintf( buf, 128, "%d.%d", c_cluster, c_proc );
 		} else {
 			strcpy( buf, "1.0" );
 		}
@@ -459,7 +470,7 @@ Claim::publishStateTimes( ClassAd* cad )
 
 
 void
-Claim::dprintf( int flags, char* fmt, ... )
+Claim::dprintf( int flags, const char* fmt, ... )
 {
 	va_list args;
 	va_start( args, fmt );
@@ -538,7 +549,7 @@ Claim::cancel_match_timer()
 }
 
 
-int
+void
 Claim::match_timed_out()
 {
 	char* my_id = id();
@@ -549,14 +560,14 @@ Claim::match_timed_out()
 			// up, too, and we don't want to seg fault.
 		::dprintf( D_FAILURE|D_ALWAYS,
 				   "ERROR: Match timed out but there's no ClaimId\n" );
-		return FALSE;
+		return;
 	}
 		
 	Resource* res_ip = resmgr->get_by_any_id( my_id );
 	if( !res_ip ) {
 		::dprintf( D_FAILURE|D_ALWAYS,
 				   "ERROR: Can't find resource of expired match\n" );
-		return FALSE;
+		return;
 	}
 
 	if( res_ip->r_cur->idMatches( id() ) ) {
@@ -579,7 +590,7 @@ Claim::match_timed_out()
 					 "backfill jobs can evict themselves.\n", 
 					 state_to_string(res_ip->state()), match_timeout );
 			res_ip->set_destination_state( owner_state );
-			return FALSE;
+			return;
 		}
 #endif /* HAVE_BACKFILL */
 		if( res_ip->state() != matched_state ) {
@@ -599,7 +610,7 @@ Claim::match_timed_out()
 			dprintf( D_FAILURE|D_FULLDEBUG, 
 					 "WARNING: Current match timed out but in %s state.\n",
 					 state_to_string(res_ip->state()) );
-			return FALSE;
+			return;
 		}
 		delete res_ip->r_cur;
 			// once we've done this delete, the this pointer is now in
@@ -626,7 +637,7 @@ Claim::match_timed_out()
 		res_ip->r_reqexp->restore();
 		res_ip->update();
 	}		
-	return TRUE;
+	return;
 }
 
 
@@ -694,6 +705,18 @@ Claim::loadRequestInfo()
 	}
 }
 
+void
+Claim::loadStatistics()
+{
+		// Stash the ATTR_NUM_PIDS, necessary to advertise
+		// them if they exist
+	if ( c_client ) {
+		int numJobPids = 0;
+		c_ad->LookupInteger(ATTR_NUM_PIDS, numJobPids);
+		c_client->setNumPids(numJobPids);
+	}
+}
+
 void
 Claim::beginActivation( time_t now )
 {
@@ -818,7 +841,21 @@ Claim::startLeaseTimer()
 		EXCEPT( "Couldn't register timer (out of memory)." );
 	}
 	
-	if ( param_boolean("STARTD_SENDS_ALIVES",false) &&
+	// Figure out who's sending 
+	bool startd_sends_alives;
+	std::string value;
+	param( value, "STARTD_SENDS_ALIVES", "peer" );
+	if ( c_ad && c_ad->LookupBool( ATTR_STARTD_SENDS_ALIVES, startd_sends_alives ) ) {
+		// Use value from ad
+	} else if ( strcasecmp( value.c_str(), "false" ) == 0 ) {
+		startd_sends_alives = false;
+	} else if ( strcasecmp( value.c_str(), "true" ) == 0 ) {
+		startd_sends_alives = true;
+	} else {
+		// No direction from the schedd or config file. 
+		startd_sends_alives = false;
+	}
+	if ( startd_sends_alives &&
 		 c_type != CLAIM_COD &&
 		 c_lease_duration > 0 )	// prevent divide by zero
 	{
@@ -863,7 +900,7 @@ Claim::cancelLeaseTimer()
 
 }
 
-int
+void
 Claim::sendAlive()
 {
 	const char* c_addr = NULL;
@@ -874,12 +911,12 @@ Claim::sendAlive()
 
 	if( !c_addr ) {
 			// Client not really set, nothing to do.
-		return FALSE;
+		return;
 	}
 
 	if ( c_alive_inprogress_sock ) {
 			// already did it
-		return FALSE;
+		return;
 	}
 
 	DCSchedd matched_schedd ( c_addr, NULL );
@@ -889,15 +926,15 @@ Claim::sendAlive()
 
 	int connect_timeout = MAX(20, ((c_lease_duration / 3)-3) );
 
-	if (!(sock = matched_schedd.reliSock( connect_timeout, NULL, true ))) {
+	if (!(sock = matched_schedd.reliSock( connect_timeout, 0, NULL, true ))) {
 		dprintf( D_FAILURE|D_ALWAYS, 
 				"Alive failed - couldn't initiate connection to %s\n",
 		         c_addr );
-		return FALSE;
+		return;
 	}
 
 	char to_schedd[256];
-	sprintf ( to_schedd, "Alive to schedd %s", c_addr );
+	snprintf ( to_schedd, 256, "Alive to schedd %s", c_addr );
 
 	int reg_rc = daemonCore->
 			Register_Socket( sock, "<Alive Contact Socket>",
@@ -910,12 +947,10 @@ Claim::sendAlive()
 		         "Register_Socket returned %d.\n",
 		         c_addr,reg_rc);
 		delete sock;
-		return FALSE;
+		return;
 	}
 
 	c_alive_inprogress_sock = sock;
-
-	return TRUE;
 }
 
 /* ALIVE_BAILOUT def:
@@ -929,7 +964,7 @@ Claim::sendAlive()
 int
 Claim::sendAliveConnectHandler(Stream *s)
 {
-	char* c_addr = "(unknown)";
+	const char* c_addr = "(unknown)";
 	if ( c_client ) {
 		c_addr = c_client->addr();
 	}
@@ -983,7 +1018,7 @@ Claim::sendAliveConnectHandler(Stream *s)
 	daemonCore->Cancel_Socket( sock ); //Allow us to re-register this socket.
 
 	char to_schedd[256];
-	sprintf ( to_schedd, "Alive to schedd %s", c_addr );
+	snprintf ( to_schedd, 256, "Alive to schedd %s", c_addr );
 	int reg_rc = daemonCore->
 			Register_Socket( sock, "<Alive Contact Socket>",
 			  (SocketHandlercpp)&Claim::sendAliveResponseHandler,
@@ -1010,7 +1045,7 @@ int
 Claim::sendAliveResponseHandler( Stream *sock )
 {
 	int reply;
-	char* c_addr = "(unknown)";
+	const char* c_addr = "(unknown)";
 
 	if ( c_client ) {
 		c_addr = c_client->addr();
@@ -1038,6 +1073,7 @@ Claim::sendAliveResponseHandler( Stream *sock )
 	if ( reply == -1 ) {
 		dprintf(D_FAILURE|D_ALWAYS,"State change: claim no longer recognized "
 			 "by the schedd - removing claim\n" );
+		c_alive_inprogress_sock = NULL;
 		finishKillClaim();	// get rid of the claim
 		return FALSE;
 	}
@@ -1051,7 +1087,7 @@ Claim::sendAliveResponseHandler( Stream *sock )
 }
 
 
-int
+void
 Claim::leaseExpired()
 {
 	c_lease_tid = -1;
@@ -1062,9 +1098,11 @@ Claim::leaseExpired()
 		if( removeClaim(false) ) {
 				// There is no starter, so remove immediately.
 				// Otherwise, we will be removed when starter exits.
-			getCODMgr()->removeClaim(this);
+			CODMgr* pCODMgr = getCODMgr();
+			if (pCODMgr)
+				pCODMgr->removeClaim(this);
 		}
-		return TRUE;
+		return;
 	}
 
 	dprintf( D_FAILURE|D_ALWAYS, "State change: claim lease expired "
@@ -1072,7 +1110,7 @@ Claim::leaseExpired()
 
 		// Kill the claim.
 	finishKillClaim();
-	return TRUE;
+	return;
 }
 
 int
@@ -1096,10 +1134,11 @@ Claim::finishKillClaim()
 }
 
 void
-Claim::alive()
+Claim::alive( bool alive_from_schedd )
 {
-	dprintf( D_PROTOCOL, "Keep alive for ClaimId %s job %d.%d\n", 
-		publicClaimId(), c_cluster, c_proc );
+	dprintf( D_PROTOCOL, "Keep alive for ClaimId %s job %d.%d%s\n", 
+			 publicClaimId(), c_cluster, c_proc,
+			 alive_from_schedd ? ", received from schedd" : "" );
 
 		// Process an alive command.  This is called whenever we
 		// "heard" from the schedd since a claim was created, 
@@ -1121,9 +1160,17 @@ Claim::alive()
 		// it is possible that c_lease_duration changed on activation
 		// of a claim, so our timer reset here will handle that case
 		// as well since alive() is called upon claim activation.
-	if ( c_sendalive_tid ) {
-		daemonCore->Reset_Timer(c_sendalive_tid, c_lease_duration / 3, 
-							c_lease_duration / 3);
+		// If we got an alive message from the schedd, send our own
+		// alive message soon, since that should cause the schedd to
+		// stop sending them (since we're supposed to be sending them).
+	if ( c_sendalive_tid != -1 ) {
+		if ( alive_from_schedd ) {
+			daemonCore->Reset_Timer(c_sendalive_tid, 10, 
+									c_lease_duration / 3);
+		} else {
+			daemonCore->Reset_Timer(c_sendalive_tid, c_lease_duration / 3, 
+									c_lease_duration / 3);
+		}
 	}
 }
 
@@ -1324,7 +1371,7 @@ Claim::setStarter( Starter* s )
 
 
 void
-Claim::starterExited( void )
+Claim::starterExited( int status )
 {
 		// Now that the starter is gone, we need to change our state
 	changeState( CLAIM_IDLE );
@@ -1332,7 +1379,7 @@ Claim::starterExited( void )
 		// Notify our starter object that its starter exited, so it
 		// can cancel timers any pending timers, cleanup the starter's
 		// execute directory, and do any other cleanup. 
-	c_starter->exited();
+	c_starter->exited(status);
 	
 		// Now, clear out this claim with all the starter-specific
 		// info, including the starter object itself.
@@ -1521,6 +1568,19 @@ Claim::starterKillHard( void )
 }
 
 
+void
+Claim::starterHoldJob( char const *hold_reason,int hold_code,int hold_subcode )
+{
+	if( c_starter ) {
+		if( c_starter->holdJob(hold_reason,hold_code,hold_subcode) ) {
+			return;
+		}
+		dprintf(D_ALWAYS,"Starter unable to hold job, so evicting job instead.\n");
+	}
+
+	starterKillSoft();
+}
+
 void
 Claim::makeStarterArgs( ArgList &args )
 {
@@ -1620,7 +1680,7 @@ Claim::publishStarterAd( ClassAd* cad )
 		return false;
 	}
 
-	char* ip_addr = c_starter->getIpAddr();
+	char const* ip_addr = c_starter->getIpAddr();
 	if( ip_addr ) {
 		line = ATTR_STARTER_IP_ADDR;
 		line += "=\"";
@@ -1908,6 +1968,18 @@ Claim::writeJobAd( int pipe_end )
 	return true;
 }
 
+bool
+Claim::writeMachAd( Stream* stream )
+{
+	dprintf(D_FULLDEBUG | D_JOB, "Sending Machine Ad to Starter\n");
+	c_rip->r_classad->dPrint(D_JOB);
+	if (!c_rip->r_classad->put(*stream) || !stream->end_of_message()) {
+		dprintf(D_ALWAYS, "writeMachAd: Failed to write machine ClassAd to stream\n");
+		return false;
+	}
+	return true;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 // Client
 ///////////////////////////////////////////////////////////////////////////
@@ -1921,6 +1993,7 @@ Client::Client()
 	c_host = NULL;
 	c_proxyfile = NULL;
 	c_concurrencyLimits = NULL;
+	c_numPids = 0;
 }
 
 
@@ -2029,6 +2102,12 @@ Client::setConcurrencyLimits( const char* limits )
 	}
 }
 
+void
+Client::setNumPids( int numJobPids )
+{
+	c_numPids = numJobPids;
+}
+
 void
 Client::vacate(char* id)
 {
@@ -2054,7 +2133,7 @@ Client::vacate(char* id)
 	}
 	if( !sock->put_secret( id ) ) {
 		dprintf(D_ALWAYS, "Can't send ClaimId to client\n");
-	} else if( !sock->eom() ) {
+	} else if( !sock->end_of_message() ) {
 		dprintf(D_ALWAYS, "Can't send EOM to client\n");
 	}
 
@@ -2079,7 +2158,18 @@ newIdString( char** id_str_ptr )
 	sequence_num++;
 
 	MyString id;
-	id += daemonCore->InfoCommandSinfulString();
+	// Keeping with tradition, we insert the startd's address in
+	// the claim id.  As of condor 7.2, nothing relies on this.
+	// Strip out CCB and other special info so we don't get any
+	// '#' characters in the address.
+
+	Sinful my_sin( daemonCore->publicNetworkIpAddr() );
+	my_sin.clearParams();
+	char const *my_addr = my_sin.getSinful();
+
+	ASSERT( my_addr && !strchr(my_addr,'#') );
+
+	id += my_addr;
 	id += '#';
 	id += (int)startd_startup;
 	id += '#';
@@ -2089,11 +2179,8 @@ newIdString( char** id_str_ptr )
 		// keylen is 20 in order to avoid generating claim ids that
 		// overflow the 80 byte buffer in pre-7.1.3 negotiators
 	const size_t keylen = 20;
-	unsigned char *keybuf = Condor_Crypt_Base::randomKey(keylen);
-	int i;
-	for(i=0;i<keylen;i++) {
-		id.sprintf_cat("%02x",keybuf[i]);
-	}
+	char *keybuf = Condor_Crypt_Base::randomHexKey(keylen);
+	id += keybuf;
 	free( keybuf );
 
 	*id_str_ptr = strdup( id.Value() );
@@ -2101,13 +2188,13 @@ newIdString( char** id_str_ptr )
 }
 
 
-ClaimId::ClaimId( ClaimType claim_type, char const *slotname )
+ClaimId::ClaimId( ClaimType claim_type, char const * /*slotname*/ /*UNUSED*/ )
 {
 	int num = newIdString( &c_id );
 	claimid_parser.setClaimId(c_id);
 	if( claim_type == CLAIM_COD ) { 
 		char buf[64];
-		sprintf( buf, "COD%d", num );
+		snprintf( buf, 64, "COD%d", num );
 		c_cod_id = strdup( buf );
 	} else {
 		c_cod_id = NULL;
@@ -2237,13 +2324,11 @@ Claim::receiveJobClassAdUpdate( ClassAd &update_ad )
 	ASSERT( c_ad );
 
 	update_ad.ResetExpr();
+	const char *name;
 	ExprTree *expr;
-	while( (expr=update_ad.NextExpr()) != NULL ) {
+	while( update_ad.NextExpr(name, expr) ) {
 
-		ASSERT( expr->MyType() == LX_ASSIGN &&
-				expr->LArg()->MyType() == LX_VARIABLE );
-
-		char const *name = ((Variable *)expr->LArg())->Name();
+		ASSERT( name );
 		if( !strcmp(name,ATTR_MY_TYPE) ||
 			!strcmp(name,ATTR_TARGET_TYPE) )
 		{
@@ -2252,12 +2337,13 @@ Claim::receiveJobClassAdUpdate( ClassAd &update_ad )
 		}
 
 			// replace expression in current ad with expression from update ad
-		ExprTree *new_expr = expr->DeepCopy();
+		ExprTree *new_expr = expr->Copy();
 		ASSERT( new_expr );
-		if( !c_ad->Insert( new_expr ) ) {
+		if( !c_ad->Insert( name, new_expr ) ) {
 			delete new_expr;
 		}
 	}
+	loadStatistics();
 	if( DebugFlags & D_JOB ) {
 		dprintf(D_JOB,"Updated job ClassAd:\n");
 		c_ad->dPrint(D_JOB);
