@@ -19,6 +19,7 @@
 
 
 #include "condor_common.h"
+#include "condor_classad_util.h"
 #include "condor_mkstemp.h"
 #include "startd.h"
 #include "vm_common.h"
@@ -40,6 +41,7 @@ command_handler( Service*, int cmd, Stream* stream )
 	int rval = FALSE;
 	Resource* rip;
 	if( ! (rip = stream_to_rip(stream)) ) {
+		dprintf(D_ALWAYS, "Error: problem finding resource for %d (%s)\n", cmd, getCommandString(cmd));
 		return FALSE;
 	}
 	State s = rip->state();
@@ -77,6 +79,24 @@ deactivate_claim(Stream *stream, Resource *rip, bool graceful)
 {
 	int rval;
 	bool claim_is_closing = rip->curClaimIsClosing();
+
+		// send response to shadow before killing starter to avoid a
+		// 3-way deadlock (under windows) where startd blocks trying to
+		// authenticate DC_RAISESIGNAL to starter while starter is
+		// blocking trying to send update to shadow.
+	stream->encode();
+
+	ClassAd response_ad;
+	response_ad.Assign(ATTR_START,!claim_is_closing);
+	if( !response_ad.put(*stream) || !stream->end_of_message() ) {
+		dprintf(D_FULLDEBUG,"Failed to send response ClassAd in deactivate_claim.\n");
+			// Prior to 7.0.5, no response ClassAd was expected.
+			// Anyway, failure to send it is not (currently) critical
+			// in any way.
+
+		claim_is_closing = false;
+	}
+
 	if(graceful) {
 		rval = rip->deactivate_claim();
 	}
@@ -84,24 +104,12 @@ deactivate_claim(Stream *stream, Resource *rip, bool graceful)
 		rval = rip->deactivate_claim_forcibly();
 	}
 
-	stream->encode();
-
-	ClassAd response_ad;
-	response_ad.Assign(ATTR_START,!claim_is_closing);
-	if( !response_ad.put(*stream) || !stream->eom() ) {
-		dprintf(D_FULLDEBUG,"Failed to send response ClassAd in deactivate_claim.\n");
-			// Prior to 7.0.5, no response ClassAd was expected.
-			// Anyway, failure to send it is not (currently) critical
-			// in any way.
-	}
-	else {
-		if( claim_is_closing && rip->r_cur ) {
-				// We told the submit-side this claim is closing, so there is
-				// no need to exchange RELEASE_CLAIM messages.  Behave as
-				// though the schedd has already sent us RELEASE_CLAIM.
-			rip->r_cur->scheddClosedClaim();
-			rip->release_claim();
-		}
+	if( claim_is_closing && rip->r_cur ) {
+			// We told the submit-side this claim is closing, so there is
+			// no need to exchange RELEASE_CLAIM messages.  Behave as
+			// though the schedd has already sent us RELEASE_CLAIM.
+		rip->r_cur->scheddClosedClaim();
+		rip->void_release_claim();
 	}
 
 	return rval;
@@ -122,7 +130,7 @@ command_activate_claim( Service*, int cmd, Stream* stream )
 	if( !rip ) {
 		ClaimIdParser idp( id );
 		dprintf( D_ALWAYS, 
-				 "Error: can't find resource with ClaimId (%s)\n", idp.publicClaimId() );
+				 "Error: can't find resource with ClaimId (%s) for %d (%s)\n", idp.publicClaimId(), cmd, getCommandString(cmd) );
 		free( id );
 		stream->end_of_message();
 		reply( stream, NOT_OK );
@@ -186,11 +194,11 @@ command_vacate_all( Service*, int cmd, Stream* )
 	switch( cmd ) {
 	case VACATE_ALL_CLAIMS:
 		dprintf( D_ALWAYS, "State change: received VACATE_ALL_CLAIMS command\n" );
-		resmgr->walk( &Resource::retire_claim );
+		resmgr->walk( &Resource::void_retire_claim );
 		break;
 	case VACATE_ALL_FAST:
 		dprintf( D_ALWAYS, "State change: received VACATE_ALL_FAST command\n" );
-		resmgr->walk( &Resource::kill_claim );
+		resmgr->walk( &Resource::void_kill_claim );
 		break;
 	default:
 		EXCEPT( "Unknown command (%d) in command_vacate_all", cmd );
@@ -204,7 +212,7 @@ int
 command_pckpt_all( Service*, int, Stream* ) 
 {
 	dprintf( D_ALWAYS, "command_pckpt_all() called.\n" );
-	resmgr->walk( &Resource::periodic_checkpoint );
+	resmgr->walk( &Resource::void_periodic_checkpoint );
 	return TRUE;
 }
 
@@ -214,7 +222,18 @@ command_x_event( Service*, int, Stream* s )
 {
 	dprintf( D_FULLDEBUG, "command_x_event() called.\n" );
 
-	sysapi_last_xevent();
+		// Only trust events over the network if the network message
+		// originated from our local machine.
+	if ( !s ||							// trust calls from within the startd
+		 (s && s->peer_is_local())		// trust only sockets from local machine
+	   ) 
+	{
+		sysapi_last_xevent();
+	} else {
+		dprintf( D_ALWAYS, 
+			"ERROR command_x_event received from %s is not local - discarded\n",
+			s->peer_ip_str() );
+	}
 
 	if( s ) {
 		s->end_of_message();
@@ -226,14 +245,18 @@ command_x_event( Service*, int, Stream* s )
 int
 command_give_state( Service*, int, Stream* stream ) 
 {
+	int rval = TRUE;
 	char* tmp;
 	dprintf( D_FULLDEBUG, "command_give_state() called.\n" );
 	stream->encode();
 	tmp = strdup( state_to_string(resmgr->state()) );
-	stream->code( tmp );
-	stream->end_of_message();
+	if ( ! stream->code( tmp ) ||
+		 ! stream->end_of_message() ) {
+		dprintf( D_FULLDEBUG, "command_give_state(): failed to send state\n" );
+		rval = FALSE;
+	}
 	free( tmp );
-	return TRUE;
+	return rval;
 }
 
 int
@@ -271,7 +294,7 @@ command_request_claim( Service*, int cmd, Stream* stream )
 	if( !rip ) {
 		ClaimIdParser idp( id );
 		dprintf( D_ALWAYS, 
-				 "Error: can't find resource with ClaimId (%s)\n", idp.publicClaimId() );
+				 "Error: can't find resource with ClaimId (%s) for %d (%s)\n", idp.publicClaimId(), cmd, getCommandString(cmd) );
 		free( id );
 		refuse( stream );
 		return FALSE;
@@ -338,8 +361,8 @@ command_release_claim( Service*, int cmd, Stream* stream )
 	rip = resmgr->get_by_any_id( id );
 	if( !rip ) {
 		ClaimIdParser idp( id );
-		dprintf( D_FULLDEBUG, 
-				 "Can't find resource with ClaimId (%s); perhaps this claim was removed already.\n", idp.publicClaimId() );
+		dprintf( D_ALWAYS, 
+				 "Error: can't find resource with ClaimId (%s) for %d (%s); perhaps this claim was removed already.\n", idp.publicClaimId(), cmd, getCommandString(cmd) );
 		free( id );
 		refuse( stream );
 		return FALSE;
@@ -350,6 +373,7 @@ command_release_claim( Service*, int cmd, Stream* stream )
 		dprintf( D_ALWAYS,
 				 "Error: can't read end of message for RELEASE_CLAIM %s.\n",
 				 idp.publicClaimId() );
+		free( id );
 		return FALSE;
 	}
 
@@ -486,6 +510,7 @@ command_match_info( Service*, int cmd, Stream* stream )
 	}
 	if( !stream->end_of_message() ) {
 		dprintf( D_ALWAYS, "Error: can't read end of message for MATCH_INFO.\n" );
+		free( id );
 		return FALSE;
 	}
 
@@ -571,7 +596,7 @@ command_query_ads( Service*, int, Stream* stream)
 	ClassAdList ads;
 	int more = 1, num_ads = 0;
    
-	dprintf( D_ALWAYS, "In command_query_ads\n" );
+	dprintf( D_FULLDEBUG, "In command_query_ads\n" );
 
 	stream->decode();
     stream->timeout(15);
@@ -580,6 +605,10 @@ command_query_ads( Service*, int, Stream* stream)
 		return FALSE;
 	}
 
+#if !defined(WANT_OLD_CLASSADS)
+	queryAd.AddExplicitTargetRefs();
+#endif
+
 		// Construct a list of all our ClassAds:
 	resmgr->makeAdList( &ads );
 	
@@ -587,7 +616,7 @@ command_query_ads( Service*, int, Stream* stream)
 	stream->encode();
 	ads.Open();
 	while( (ad = ads.Next()) ) {
-		if( (*ad) >= queryAd ) {
+		if( IsAHalfMatch( &queryAd, ad ) ) {
 			if( !stream->code(more) || !ad->put(*stream) ) {
 				dprintf (D_ALWAYS, 
 						 "Error sending query result to client -- aborting\n");
@@ -843,7 +872,7 @@ command_delegate_gsi_cred( Service*, int, Stream* stream )
 	         proxy_file.Value() );
 
 	// sender decides whether to use delegation or simply copy
-	int use_delegation;
+	int use_delegation = 0;
 	if( ! sock->code(use_delegation) ) {
 		dprintf( D_ALWAYS, "error reading delegation request\n" );
 		return FALSE;
@@ -1007,6 +1036,10 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		ABORT;
 	}
 
+#if !defined(WANT_OLD_CLASSADS)
+	req_classad->AddTargetRefs( TargetMachineAttrs );
+#endif
+
 		// Try now to read the schedd addr and aline interval.
 		// Do _not_ abort if we fail, since older (pre v6.1.11) schedds do 
 		// not send this information until after we accept the request, and we
@@ -1080,7 +1113,7 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		} else {
 				// some memory size must be available else we cannot
 				// match, plus a job ad without ATTR_MEMORY is sketchy
-			rip->dprintf( D_FULLDEBUG,
+			rip->dprintf( D_ALWAYS,
 						  "No memory request in incoming ad, aborting...\n" );
 			ABORT;
 		}
@@ -1099,9 +1132,9 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		}
 
 		rip->dprintf( D_FULLDEBUG,
-					  "Match requesting resources: %s\n", type.GetCStr() );
+					  "Match requesting resources: %s\n", type.Value() );
 
-		type_list.initializeFromString( type.GetCStr() );
+		type_list.initializeFromString( type.Value() );
 		cpu_attrs = resmgr->buildSlot( rip->r_id, &type_list, -1, false );
 		if( ! cpu_attrs ) {
 			rip->dprintf( D_ALWAYS,
@@ -1349,7 +1382,7 @@ abort_accept_claim( Resource* rip, Stream* stream )
 bool
 accept_request_claim( Resource* rip )
 {
-	int interval;
+	int interval = -1;
 	char *client_addr = NULL, *client_host, *full_client_host, *tmp;
 	char RemoteOwner[512];
 	RemoteOwner[0] = '\0';
@@ -1367,7 +1400,7 @@ accept_request_claim( Resource* rip )
 		abort_accept_claim( rip, stream );
 		return false;
 	}
-	if( !stream->eom() ) {
+	if( !stream->end_of_message() ) {
 		rip->dprintf( D_ALWAYS, "Can't to send eom to schedd.\n" );
 		abort_accept_claim( rip, stream );
 		return false;
@@ -1406,8 +1439,8 @@ accept_request_claim( Resource* rip )
 	}
 
 		// Figure out the hostname of our client.
-	if( ! (tmp = sin_to_hostname(sock->endpoint(), NULL)) ) {
-		char *sinful = sin_to_string(sock->endpoint());
+	if( ! (tmp = sin_to_hostname(sock->peer_addr(), NULL)) ) {
+		char *sinful = sin_to_string(sock->peer_addr());
 		char *ip = string_to_ipstr(sinful);
 		rip->dprintf( D_FULLDEBUG,
 					  "Can't find hostname of client machine %s\n", ip );
@@ -1435,7 +1468,7 @@ accept_request_claim( Resource* rip )
 				 ATTR_USER );
 		RemoteOwner[0] = '\0';
 	}
-	if( RemoteOwner ) {
+	if( '\0' != RemoteOwner[0] ) {
 		rip->r_cur->client()->setowner( RemoteOwner );
 			// For now, we say the remote user is the same as the
 			// remote owner.  In the future, we might decide to leave
@@ -1487,9 +1520,9 @@ activate_claim( Resource* rip, Stream* stream )
 	int len = sizeof frm;	
 	StartdRec stRec;
 #endif
-	int starter;
+	int starter = MAX_STARTERS;
 	Sock* sock = (Sock*)stream;
-	char* shadow_addr = strdup( sin_to_string( sock->endpoint() ));
+	char* shadow_addr = strdup( sin_to_string( sock->peer_addr() ));
 
 	if( rip->state() != claimed_state ) {
 		rip->dprintf( D_ALWAYS, "Not in claimed state, aborting.\n" );
@@ -1521,10 +1554,14 @@ activate_claim( Resource* rip, Stream* stream )
 		ABORT;
 	}
 	if (!stream->end_of_message()) {
-		rip->dprintf( D_ALWAYS, "Can't receive eom() from shadow.\n" );
+		rip->dprintf( D_ALWAYS, "Can't receive end_of_message() from shadow.\n" );
 		ABORT;
 	}
 
+#if !defined(WANT_OLD_CLASSADS)
+	req_classad->AddTargetRefs( TargetMachineAttrs );
+#endif
+
 	rip->dprintf( D_FULLDEBUG, "Read request ad and starter from shadow.\n" );
 
 		// Now, ask the ResMgr to recompute so we have totally
@@ -1628,7 +1665,7 @@ activate_claim( Resource* rip, Stream* stream )
 			ABORT;
 		}
 
-		if (!stream->eom()) {
+		if (!stream->end_of_message()) {
 			ABORT;
 		}
 
@@ -1823,11 +1860,11 @@ match_info( Resource* rip, char* id )
 int
 caRequestCODClaim( Stream *s, char* cmd_str, ClassAd* req_ad )
 {
-	char* requirements_str = NULL;
+	const char* requirements_str = NULL;
 	Resource* rip;
 	Claim* claim;
 	MyString err_msg;
-	ExprTree *tree, *rhs;
+	ExprTree *tree;
 	ReliSock* rsock = (ReliSock*)s;
 	int lease_duration = 0;
 	const char* owner = rsock->getOwner();
@@ -1844,18 +1881,15 @@ caRequestCODClaim( Stream *s, char* cmd_str, ClassAd* req_ad )
 			 owner );
 
 		// Make sure the ad's got a requirements expression at all.
-	tree = req_ad->Lookup( ATTR_REQUIREMENTS );
+	tree = req_ad->LookupExpr( ATTR_REQUIREMENTS );
 	if( ! tree ) {
 		dprintf( D_FULLDEBUG, 
 				 "Request did not contain %s, assuming TRUE\n",
 				 ATTR_REQUIREMENTS );
-		requirements_str = strdup( "TRUE" );
+		requirements_str = "TRUE";
 		req_ad->Insert( "Requirements = TRUE" );
 	} else {
-		rhs = tree->RArg();
-		if( rhs ) {
-			rhs->PrintToNewStr( &requirements_str );
-		}
+		requirements_str = ExprTreeToString( tree );
 	}
 
 		// Find the right resource for this claim
@@ -1867,13 +1901,9 @@ caRequestCODClaim( Stream *s, char* cmd_str, ClassAd* req_ad )
 		err_msg += " (";
 		err_msg += requirements_str;
 		err_msg += ')';
-		free( requirements_str );
 		return sendErrorReply( s, cmd_str, CA_FAILURE, err_msg.Value() );
 	}
 
-		// done with this now, so don't leak it
-	free( requirements_str );
-
 	if( !req_ad->LookupInteger(ATTR_JOB_LEASE_DURATION, lease_duration) ) {
 		lease_duration = -1;
 	}
@@ -1887,7 +1917,7 @@ caRequestCODClaim( Stream *s, char* cmd_str, ClassAd* req_ad )
 		// Stash some info about who made this request in the Claim  
 	claim->client()->setuser( owner );
 	claim->client()->setowner(owner );
-	claim->client()->sethost( rsock->endpoint_ip_str() );
+	claim->client()->sethost( rsock->peer_ip_str() );
 
 		// now, we just fill in the reply ad appropriately.  publish
 		// a complete resource ad (like what we'd send to the
@@ -1974,6 +2004,7 @@ caLocateStarter( Stream *s, char* cmd_str, ClassAd* req_ad )
 	Claim* claim = NULL;
 	int rval = TRUE;
 	ClassAd reply;
+	std::string startd_sends_alives;
 
 	req_ad->LookupString(ATTR_CLAIM_ID, &claimid);
 	req_ad->LookupString(ATTR_GLOBAL_JOB_ID, &global_job_id);
@@ -1997,8 +2028,9 @@ caLocateStarter( Stream *s, char* cmd_str, ClassAd* req_ad )
 		// if startd is sending keepalives to the schedd,
 		// then we _must_ be passed the address of the schedd
 		// since it likely changed.
+	param( startd_sends_alives, "STARTD_SENDS_ALIVES", "peer" );
 	if ( (!schedd_addr) && 
-		 (param_boolean("STARTD_SENDS_ALIVES",false)) )
+		 strcasecmp( startd_sends_alives.c_str(), "false" ) )
 	{
 		MyString err_msg;
 		err_msg.sprintf("Required %s, not found in request",
@@ -2048,18 +2080,22 @@ cleanup:
 int
 command_classad_handler( Service*, int dc_cmd, Stream* s )
 {
-	int rval;
+	int rval=0;
 	ClassAd ad;
 	ReliSock* rsock = (ReliSock*)s;
 	int cmd = 0;
 	char* cmd_str = NULL;
 
-
 	if( dc_cmd == CA_AUTH_CMD ) {
 		cmd = getCmdFromReliSock( rsock, &ad, true );
 	} else {
 		cmd = getCmdFromReliSock( rsock, &ad, false );
 	}
+
+#if !defined(WANT_OLD_CLASSADS)
+	ad.AddTargetRefs( TargetMachineAttrs );
+#endif
+
 		// since we really care about the command string for a lot of
 		// things, let's just grab it out of the classad once right
 		// here.
