@@ -1,6 +1,6 @@
 /***************************************************************
  *
- * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
+ * Copyright (C) 1990-2011, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -30,10 +30,16 @@
 #include "vm_common.h"
 #include "VMManager.h"
 #include "VMRegister.h"
+#include "classadHistory.h"
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 #include "StartdPlugin.h"
 #endif
+#if defined(WIN32)
+extern int load_startd_mgmt(void);
+#endif
+#endif
 
 // Define global variables
 
@@ -48,6 +54,7 @@ ResMgr*	resmgr;			// Pointer to the resource manager object
 // Polling variables
 int	polling_interval = 0;	// Interval for polling when there are resources in use
 int	update_interval = 0;	// Interval to update CM
+int	update_offset = 0;		// Interval offset to update CM
 
 // String Lists
 StringList *startd_job_exprs = NULL;
@@ -92,20 +99,23 @@ DECL_SUBSYSTEM( "STARTD", SUBSYSTEM_TYPE_STARTD );
 int main_reaper = 0;
 
 // Cron stuff
-StartdCronMgr	*Cronmgr;
+StartdCronJobMgr	*cron_job_mgr;
+
+// Benchmark stuff
+StartdBenchJobMgr	*bench_job_mgr;
 
 /*
  * Prototypes of static functions.
  */
 
 void usage( char* );
-int main_init( int argc, char* argv[] );
+void main_init( int argc, char* argv[] );
 int init_params(int);
-int main_config( bool is_full );
-int finish_main_config();
-int main_shutdown_fast();
-int main_shutdown_graceful();
-extern "C" int do_cleanup(int,int,char*);
+void main_config();
+void finish_main_config();
+void main_shutdown_fast();
+void main_shutdown_graceful();
+extern "C" int do_cleanup(int,int,const char*);
 int reaper( Service*, int pid, int status);
 int	shutdown_reaper( Service*, int pid, int status ); 
 
@@ -120,15 +130,15 @@ usage( char* MyName)
 }
 
 
-int
+void
 main_init( int, char* argv[] )
 {
 	int		skip_benchmarks = FALSE;
-	char*	tmp = NULL;
 	char**	ptr; 
 
-	// Reset the cron manager to a known state
-	Cronmgr = NULL;
+	// Reset the cron & benchmark managers to a known state
+	cron_job_mgr = NULL;
+	bench_job_mgr = NULL;
 
 		// Process command line args.
 	for(ptr = argv + 1; *ptr; ptr++) {
@@ -171,7 +181,7 @@ main_init( int, char* argv[] )
 		// Now starter has codes for vm universe.
 		resmgr->m_vmuniverse_mgr.setStarterAbility(true);
 		// check whether vm universe is available through vmgahp server
-		resmgr->m_vmuniverse_mgr.init();
+		resmgr->m_vmuniverse_mgr.checkVMUniverse();
 	}
 
 		// Read in global parameters from the config file.
@@ -181,10 +191,8 @@ main_init( int, char* argv[] )
 		// Resource's LoadQueue object.
 	init_params(1);		// The 1 indicates that this is the first time
 
-#if defined( OSF1 ) || defined (IRIX62) || defined(WIN32)
-		// Pretend we just got an X event so we think our console idle
-		// is something, even if we haven't heard from the kbdd yet.
-		// We do this on Win32 as well, since Win32 uses last_x_event
+#if defined(WIN32)
+		// We do this on Win32 since Win32 uses last_x_event
 		// variable in a similar fasion to the X11 condor_kbdd, and
 		// thus it must be initialized.
 	command_x_event( 0, 0, 0 );
@@ -202,26 +210,15 @@ main_init( int, char* argv[] )
 		// Compute all attributes
 	resmgr->compute( A_ALL );
 
-	if( (tmp = param("RunBenchmarks")) ) {
-		if( (!skip_benchmarks) &&
-			(*tmp != 'F' && *tmp != 'f') ) {
-			// There's an expression defined to have us periodically
-			// run benchmarks, so run them once here at the start.
-			// Added check so if people want no benchmarks at all,
-			// they just comment RunBenchmarks out of their config
-			// file, or set it to "False". -Derek Wright 10/20/98
-			dprintf( D_ALWAYS, "About to run initial benchmarks.\n" ); 
-			resmgr->force_benchmark();
-			dprintf( D_ALWAYS, "Completed initial benchmarks.\n" );
-		}
-		free( tmp );
-	}
-
 	resmgr->walk( &Resource::init_classad );
 
-	// Startup Cron
-	Cronmgr = new StartdCronMgr( );
-	Cronmgr->Initialize( );
+		// Startup Cron
+	cron_job_mgr = new StartdCronJobMgr( );
+	cron_job_mgr->Initialize( "startd" );
+
+		// Startup benchmarking
+	bench_job_mgr = new StartdBenchJobMgr( );
+	bench_job_mgr->Initialize( "benchmarks" );
 
 		// Now that we have our classads, we can compute things that
 		// need to be evaluated
@@ -296,7 +293,7 @@ main_init( int, char* argv[] )
 	daemonCore->Register_Command( X_EVENT_NOTIFICATION,
 								  "X_EVENT_NOTIFICATION",
 								  (CommandHandler)command_x_event,
-								  "command_x_event", 0, DAEMON,
+								  "command_x_event", 0, ALLOW,
 								  D_FULLDEBUG ); 
 	daemonCore->Register_Command( PCKPT_ALL_JOBS, "PCKPT_ALL_JOBS", 
 								  (CommandHandler)command_pckpt_all,
@@ -383,10 +380,10 @@ main_init( int, char* argv[] )
 
 	daemonCore->Set_Default_Reaper( main_reaper );
 
-#if defined( OSF1 ) || defined (IRIX) || defined(WIN32)
+#if defined(WIN32)
 		// Pretend we just got an X event so we think our console idle
 		// is something, even if we haven't heard from the kbdd yet.
-		// We do this on Win32 as well, since Win32 uses last_x_event
+		// We do this on Win32 since Win32 uses last_x_event
 		// variable in a similar fasion to the X11 condor_kbdd, and
 		// thus it must be initialized.
 	command_x_event( 0, 0, 0 );
@@ -401,20 +398,22 @@ main_init( int, char* argv[] )
 		// Evaluate the state of all resources and update CM 
 		// We don't just call eval_and_update_all() b/c we don't need
 		// to recompute anything.
-	resmgr->first_eval_and_update_all();
+		// This is now called by a timer registered by start_update_timer()
+	//resmgr->update_all();
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN)
    StartdPluginManager::Load();
-
+#elif defined(WIN32)
+	load_startd_mgmt();
+#endif
    StartdPluginManager::Initialize();
 #endif
-
-	return TRUE;
 }
 
 
-int
-main_config( bool /* is_full */ )
+void
+main_config()
 {
 	bool done_allocating;
 
@@ -423,13 +422,12 @@ main_config( bool /* is_full */ )
 		// Process any changes in the slot type specifications
 	done_allocating = resmgr->reconfig_resources();
 	if( done_allocating ) {
-		return finish_main_config();
+		finish_main_config();
 	}
-	return TRUE;
 }
 
 
-int
+void
 finish_main_config( void ) 
 {
 #if defined(WIN32)
@@ -443,7 +441,8 @@ finish_main_config( void )
 	resmgr->reset_timers();
 
 	dprintf( D_FULLDEBUG, "MainConfig finish\n" );
-	Cronmgr->Reconfig(  );
+	cron_job_mgr->Reconfig(  );
+	bench_job_mgr->Reconfig(  );
 	resmgr->starter_mgr.init();
 
 #if HAVE_HIBERNATION
@@ -453,8 +452,8 @@ finish_main_config( void )
 		// Re-evaluate and update the CM for each resource (again, we
 		// don't need to recompute, since we just did that, so we call
 		// the special case version).
-	resmgr->first_eval_and_update_all();
-	return TRUE;
+		// This is now called by a timer registered by reset_timers()
+	//resmgr->update_all();
 }
 
 
@@ -469,6 +468,7 @@ init_params( int /* first_time */)
 	polling_interval = param_integer( "POLLING_INTERVAL", 5 );
 
 	update_interval = param_integer( "UPDATE_INTERVAL", 300, 1 );
+	update_offset = param_integer( "UPDATE_OFFSET", 0, 0 );
 
 	if( accountant_host ) {
 		free( accountant_host );
@@ -505,7 +505,7 @@ init_params( int /* first_time */)
 	if (!tmp) {
 		tmp = param( "STARTD_SLOT_EXPRS" );
 	}
-	if (param_boolean("ALLOW_VM_CRUFT", true) && !tmp) {
+	if (param_boolean("ALLOW_VM_CRUFT", false) && !tmp) {
 		tmp = param( "STARTD_VM_ATTRS" );
 		if (!tmp) {
 			tmp = param( "STARTD_VM_EXPRS" );
@@ -584,6 +584,8 @@ init_params( int /* first_time */)
 		free(tmp);
 	}
 
+	InitJobHistoryFile( "STARTD_HISTORY" , "STARTD_PER_JOB_HISTORY_DIR");
+
 	return TRUE;
 }
 
@@ -592,10 +594,17 @@ void
 startd_exit() 
 {
 	// Shut down the cron logic
-	if( Cronmgr ) {
-		dprintf( D_ALWAYS, "Deleting Cronmgr\n" );
-		Cronmgr->Shutdown( true );
-		delete Cronmgr;
+	if( cron_job_mgr ) {
+		dprintf( D_ALWAYS, "Deleting cron job manager\n" );
+		cron_job_mgr->Shutdown( true );
+		delete cron_job_mgr;
+	}
+
+	// Shut down the benchmark job manager
+	if( bench_job_mgr ) {
+		dprintf( D_ALWAYS, "Deleting benchmark job mgr\n" );
+		bench_job_mgr->Shutdown( true );
+		delete bench_job_mgr;
 	}
 
 	// Cleanup the resource manager
@@ -632,22 +641,29 @@ startd_exit()
 	systray_notifier.notifyCondorOff();
 #endif
 
-#if HAVE_DLOPEN
+#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
+#if defined(HAVE_DLOPEN) || defined(WIN32)
 	StartdPluginManager::Shutdown();
+#endif
 #endif
 
 	dprintf( D_ALWAYS, "All resources are free, exiting.\n" );
 	DC_Exit(0);
 }
 
-int
+void
 main_shutdown_fast()
 {
 	dprintf( D_ALWAYS, "shutdown fast\n" );
 
-	// Shut down the cron logic
-	if( Cronmgr ) {
-		Cronmgr->Shutdown( true );
+		// Shut down the cron logic
+	if( cron_job_mgr ) {
+		cron_job_mgr->Shutdown( true );
+	}
+
+		// Shut down the benchmark logic
+	if( bench_job_mgr ) {
+		bench_job_mgr->Shutdown( true );
 	}
 
 		// If the machine is free, we can just exit right away.
@@ -665,20 +681,24 @@ main_shutdown_fast()
 	resmgr->walk( &Resource::killAllClaims );
 
 	daemonCore->Register_Timer( 0, 5, 
-								(TimerHandler)startd_check_free,
+								startd_check_free,
 								 "startd_check_free" );
-	return TRUE;
 }
 
 
-int
+void
 main_shutdown_graceful()
 {
 	dprintf( D_ALWAYS, "shutdown graceful\n" );
 
-	// Shut down the cron logic
-	if( Cronmgr ) {
-		Cronmgr->Shutdown( false );
+		// Shut down the cron logic
+	if( cron_job_mgr ) {
+		cron_job_mgr->Shutdown( false );
+	}
+
+		// Shut down the benchmark logic
+	if( bench_job_mgr ) {
+		bench_job_mgr->Shutdown( false );
 	}
 
 		// If the machine is free, we can just exit right away.
@@ -696,9 +716,8 @@ main_shutdown_graceful()
 	resmgr->walk( &Resource::releaseAllClaims );
 
 	daemonCore->Register_Timer( 0, 5, 
-								(TimerHandler)startd_check_free,
+								startd_check_free,
 								 "startd_check_free" );
-	return TRUE;
 }
 
 
@@ -720,7 +739,7 @@ reaper(Service *, int pid, int status)
 
 	foo = resmgr->getClaimByPid(pid);
 	if( foo ) {
-		foo->starterExited();
+		foo->starterExited(status);
 	}		
 	return TRUE;
 }
@@ -736,7 +755,7 @@ shutdown_reaper(Service *, int pid, int status)
 
 
 int
-do_cleanup(int,int,char*)
+do_cleanup(int,int,const char*)
 {
 	static int already_excepted = FALSE;
 
@@ -745,7 +764,7 @@ do_cleanup(int,int,char*)
 			// If the machine is already free, we can exit right away.
 		startd_check_free();		
 			// Otherwise, quickly kill all the active starters.
-		resmgr->walk( &Resource::kill_claim );
+		resmgr->walk( &Resource::void_kill_claim );
 		dprintf( D_FAILURE|D_ALWAYS, "startd exiting because of fatal exception.\n" );
 	}
 
@@ -753,11 +772,14 @@ do_cleanup(int,int,char*)
 }
 
 
-int
+void
 startd_check_free()
 {	
-	if ( Cronmgr && ( ! Cronmgr->ShutdownOk() ) ) {
-		return FALSE;
+	if ( cron_job_mgr && ( ! cron_job_mgr->ShutdownOk() ) ) {
+		return;
+	}
+	if ( bench_job_mgr && ( ! bench_job_mgr->ShutdownOk() ) ) {
+		return;
 	}
 	if ( ! resmgr ) {
 		startd_exit();
@@ -765,7 +787,7 @@ startd_check_free()
 	if( ! resmgr->hasAnyClaim() ) {
 		startd_exit();
 	}
-	return TRUE;
+	return;
 }
 
 
