@@ -37,6 +37,8 @@
 #include "vm_univ_utils.h"
 #include "setenv.h"
 
+extern ResMgr* resmgr;
+
 static unsigned long get_image_size(procInfo& pi)
 {
 #if defined(WIN32)
@@ -64,21 +66,21 @@ VMStarterInfo::updateUsageOfVM(void)
 	}
 
 	int proc_status = PROCAPI_OK;
-	struct procInfo pinfo;
-	memset(&pinfo, 0, sizeof(pinfo));
 
-	piPTR pi = &pinfo;
+	piPTR pi = NULL;
 	if( ProcAPI::getProcInfo(m_vm_pid, pi, proc_status) == 
 			PROCAPI_SUCCESS ) {
-		memcpy(&m_vm_alive_pinfo, &pinfo, sizeof(pinfo));
+		memcpy(&m_vm_alive_pinfo, pi, sizeof(struct procInfo));
 		if( (DebugFlags & D_FULLDEBUG) && (DebugFlags & D_LOAD) ) {
 			dprintf(D_FULLDEBUG,"Usage of process[%d] for a VM is updated\n", 
 					m_vm_pid);
 			dprintf(D_FULLDEBUG,"sys_time=%lu, user_time=%lu, image_size=%lu\n", 
-					pinfo.sys_time, pinfo.user_time, get_image_size(pinfo));
+					pi->sys_time, pi->user_time, get_image_size(*pi));
 		}
+		delete pi;
 		return true;
 	}
+	if (pi) delete pi;
 	return false;
 }
 
@@ -103,8 +105,10 @@ VMStarterInfo::getUsageOfVM(ProcFamilyUsage &usage)
 
 	if( updated ) {
 		usage.total_image_size = m_vm_alive_pinfo.imgsize;
+		usage.total_resident_set_size = m_vm_alive_pinfo.rssize;
 	}else {
 		usage.total_image_size = 0;
+        usage.total_resident_set_size = 0;
 	}
 
 	if( (DebugFlags & D_FULLDEBUG) && (DebugFlags & D_LOAD) ) {
@@ -158,7 +162,7 @@ VMStarterInfo::addMACForVM(const char* mac)
 const char *
 VMStarterInfo::getMACForVM(void)
 {
-	return m_vm_mac.GetCStr();
+	return m_vm_mac.Value();
 }
 
 void
@@ -170,7 +174,7 @@ VMStarterInfo::addIPForVM(const char* ip)
 const char *
 VMStarterInfo::getIPForVM(void)
 {
-	return m_vm_ip.GetCStr();
+	return m_vm_ip.Value();
 }
 
 void 
@@ -271,6 +275,7 @@ VMUniverseMgr::init( void )
 		m_vm_type = "";
 		return false;
 	}
+
 	return true;
 }
 
@@ -307,21 +312,17 @@ VMUniverseMgr::publish( ClassAd* ad, amask_t mask )
 	// we will publish all information provided by vmgahp server
 	m_vmgahp_info.ResetExpr();
 
-	MyString line;
 	ExprTree* expr = NULL;
-	char *attr_name = NULL;
-	while((expr = m_vmgahp_info.NextExpr()) != NULL) {
-		attr_name = ((Variable*)expr->LArg())->Name();
-
+	const char *attr_name = NULL;
+	while(m_vmgahp_info.NextExpr(attr_name, expr)) {
 		// we need to adjust available vm memory
-		if( stricmp(attr_name, ATTR_VM_MEMORY) == MATCH ) {
+		if( strcasecmp(attr_name, ATTR_VM_MEMORY) == MATCH ) {
 			int freemem = getFreeVMMemSize();
 			ad->Assign(ATTR_VM_MEMORY, freemem);
-		}else if( stricmp(attr_name, ATTR_VM_NETWORKING) == MATCH ) {
+		}else if( strcasecmp(attr_name, ATTR_VM_NETWORKING) == MATCH ) {
 			ad->Assign(ATTR_VM_NETWORKING, m_vm_networking); 
 		}else {
-			expr->PrintToStr(line);
-			ad->Insert(line.Value());
+			ad->Insert(attr_name, expr->Copy());
 		}
 	}
 
@@ -416,7 +417,7 @@ VMUniverseMgr::testVMGahp(const char* gahppath, const char* vmtype)
 #endif
 
 	priv_state prev_priv;
-	if( strcasecmp(vmtype, CONDOR_VM_UNIVERSE_XEN) == MATCH ) {
+	if( (strcasecmp(vmtype, CONDOR_VM_UNIVERSE_XEN) == MATCH) || (strcasecmp(vmtype, CONDOR_VM_UNIVERSE_KVM) == MATCH) ) {
 		// Xen requires root privilege
 		prev_priv = set_root_priv();
 	}else {
@@ -435,7 +436,7 @@ VMUniverseMgr::testVMGahp(const char* gahppath, const char* vmtype)
 	bool read_something = false;
 	char buf[2048];
 
-	m_vmgahp_info.clear();
+	m_vmgahp_info.Clear();
 	while( fgets(buf, 2048, fp) ) {
 		if( !m_vmgahp_info.Insert(buf) ) {
 			dprintf( D_ALWAYS, "Failed to insert \"%s\" into VMInfo, "
@@ -451,7 +452,7 @@ VMUniverseMgr::testVMGahp(const char* gahppath, const char* vmtype)
 		dprintf( D_ALWAYS, 
 				 "Warning: '%s' did not produce any valid output.\n", 
 				 args_string.Value());
-		if( strcasecmp(vmtype, CONDOR_VM_UNIVERSE_XEN) == 0 ) {
+		if( (strcasecmp(vmtype, CONDOR_VM_UNIVERSE_XEN) == 0) ) {
 			MyString err_msg;
 			err_msg += "\n#######################################################\n";
 			err_msg += "##### Make sure the followings ";
@@ -463,6 +464,18 @@ VMUniverseMgr::testVMGahp(const char* gahppath, const char* vmtype)
 			err_msg += "not allowed.\n";
 			err_msg += "#########################################################\n";
 			dprintf( D_ALWAYS, "%s", err_msg.Value());
+		} else if( (strcasecmp(vmtype, CONDOR_VM_UNIVERSE_KVM) == 0)) {
+		        MyString err_msg;
+			err_msg += "\n#######################################################\n";
+			err_msg += "##### Make sure the followings ";
+			err_msg += "to use VM universe for KVM\n";
+			err_msg += "### - The owner of script progrm like ";
+			err_msg += "'condor_vm_xen.sh' must be root\n";
+			err_msg += "### - The script program must be executable\n";
+			err_msg += "### - Other writable bit for the above files is ";
+			err_msg += "not allowed.\n";
+			err_msg += "#########################################################\n";
+			dprintf( D_ALWAYS, "%s", err_msg.Value());
 		}else if( strcasecmp(vmtype, CONDOR_VM_UNIVERSE_VMWARE ) == 0 ) {
 			MyString err_msg;
 			MyString err_msg2;
@@ -502,36 +515,15 @@ VMUniverseMgr::testVMGahp(const char* gahppath, const char* vmtype)
 	dprintf( D_ALWAYS, "VMType('%s') is supported\n", vmtype);
 
 	// Read vm_memory
-	int tmp_mem = 0;
-	if( m_vmgahp_info.LookupInteger(ATTR_VM_MEMORY, tmp_mem) != 1 ) {
+	if( m_vmgahp_info.LookupInteger(ATTR_VM_MEMORY, m_vm_max_memory) != 1 ) {
 		dprintf( D_ALWAYS, "There is no %s in the output of vmgahp\n",ATTR_VM_MEMORY);
 		return false;
 	}
-	if( tmp_mem == 0 ) {
+	if( m_vm_max_memory == 0 ) {
 		dprintf( D_ALWAYS, "There is no sufficient memory for virtual machines\n");
 		return false;
 	}
 
-	// VM_MEMORY in condor config should be less than 
-	// VM_MEMORY provided by vmgahp */
-	char *vmtmp = NULL;
-	vmtmp = param( "VM_MEMORY" );
-	if(vmtmp) {
-		int vmem = (int)strtol(vmtmp, (char **)NULL, 10);
-		if( (vmem <= 0) || (vmem > tmp_mem)) {
-			m_vm_max_memory = tmp_mem;
-			dprintf( D_ALWAYS, "Warning: Even though '%s = %d' is defined "
-					"in condor config file, the amount of memory for "
-					"vm universe is set to %d MB, because vmgahp says "
-					"(%d MB) as the maximum memory for vm universe\n", 
-					"VM_MEMORY", vmem, tmp_mem, tmp_mem);
-		}else {
-			m_vm_max_memory = vmem;
-		}
-		free(vmtmp);
-	} else {
-		m_vm_max_memory = tmp_mem;
-	}
 	dprintf( D_ALWAYS, "The maximum available memory for vm universe is "
 			"set to %d MB\n", m_vm_max_memory);
 
@@ -612,7 +604,7 @@ VMUniverseMgr::canCreateVM(ClassAd *jobAd)
 	// is conflict with running VMs.
 	MyString string_value;
 	if( jobAd->LookupString(ATTR_VM_CKPT_MAC, string_value) == 1 ) {
-		if( findVMStarterInfoWithMac(string_value.GetCStr()) ) {
+		if( findVMStarterInfoWithMac(string_value.Value()) ) {
 			dprintf(D_ALWAYS, "MAC address[%s] for VM is already being used "
 					"by other VM\n", string_value.Value());
 			return false;
@@ -802,10 +794,11 @@ VMUniverseMgr::checkVMUniverse(void)
 void 
 VMUniverseMgr::docheckVMUniverse(void)
 {
+	char *vm_type = param( "VM_TYPE" );
 	dprintf( D_ALWAYS, "VM universe will be tested "
-			"to check if it is still available\n");
-	if( init() == false ) {
-		// VM universe is not available
+			"to check if it is available\n");
+	if( init() == false && vm_type ) {
+		// VM universe is desired, but not available
 
 		// In VMware, some errors may be transient.
 		// For example, when VMware fails to start a new VM 
@@ -814,12 +807,28 @@ VMUniverseMgr::docheckVMUniverse(void)
 		// But after some time, we are able to run it again.
 		// So we register a timer to call this function later.
 		m_check_interval = param_integer("VM_RECHECK_INTERVAL", 600);
-		m_check_tid = daemonCore->Register_Timer(m_check_interval,
-				(TimerHandlercpp)&VMUniverseMgr::init,
-				"VMUniverseMgr::init", this);
+		if ( m_check_tid >= 0 ) {
+			daemonCore->Reset_Timer( m_check_tid, m_check_interval );
+		} else {
+			m_check_tid = daemonCore->Register_Timer(m_check_interval,
+				(TimerHandlercpp)&VMUniverseMgr::docheckVMUniverse,
+				"VMUniverseMgr::docheckVMUniverse", this);
+		}
 		dprintf( D_ALWAYS, "Started a timer to test VM universe after "
 			   "%d(seconds)\n", m_check_interval);
+	} else {
+		if ( m_check_tid >= 0 ) {
+			daemonCore->Cancel_Timer( m_check_tid );
+			m_check_tid = -1;
+
+			// in the case where we had to use the timer,
+			// make certain we publish our changes.  
+			if( resmgr ) {
+				resmgr->eval_and_update_all();
+			}	
+		}
 	}
+	free( vm_type );
 }
 
 void 
@@ -835,39 +844,15 @@ VMUniverseMgr::numOfRunningVM(void)
 }
 
 void
-VMUniverseMgr::killVM(VMStarterInfo *info)
+VMUniverseMgr::killVM(const char *matchstring)
 {
-	if( !info ) {
+	if ( !matchstring ) {
 		return;
 	}
 	if( !m_vm_type.Length() || !m_vmgahp_server.Length() ) {
 		return;
 	}
 
-	if( info->m_vm_pid > 0 ) {
-		dprintf( D_ALWAYS, "In VMUniverseMgr::killVM(), "
-				"Sending SIGKILL to Process[%d]\n", (int)info->m_vm_pid);
-		daemonCore->Send_Signal(info->m_vm_pid, SIGKILL);
-	}
-
-	MyString matchstring;
-	MyString workingdir;
-
-	workingdir.sprintf("%s%cdir_%ld", info->m_execute_dir.Value(),
-	                   DIR_DELIM_CHAR, (long)info->m_pid);
-
-	if( strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN ) == MATCH ) {
-		if( create_name_for_VM(&info->m_job_ad, matchstring) == false ) {
-			dprintf(D_ALWAYS, "VMUniverseMgr::killVM() : "
-					"cannot make the name of VM\n");
-			return;
-		}
-	}else {
-		// Except Xen, we need the path of working directory of Starter
-		// in order to destroy VM.
-		matchstring = workingdir;
-	}
-
 	// vmgahp is daemonCore, so we need to add -f -t options of daemonCore.
 	// Then, try to execute vmgahp with 
 	// vmtype <vmtype> match <string>"
@@ -903,7 +888,7 @@ VMUniverseMgr::killVM(VMStarterInfo *info)
 
 	if( ret == 0 ) {
 		dprintf( D_ALWAYS, "VMUniverseMgr::killVM() is called with "
-						"'%s'\n", matchstring.Value());
+						"'%s'\n", matchstring );
 	}else {
 		dprintf( D_ALWAYS, "VMUniverseMgr::killVM() failed!\n");
 	}
@@ -911,6 +896,43 @@ VMUniverseMgr::killVM(VMStarterInfo *info)
 	return;
 }
 
+void
+VMUniverseMgr::killVM(VMStarterInfo *info)
+{
+	if( !info ) {
+		return;
+	}
+	if( !m_vm_type.Length() || !m_vmgahp_server.Length() ) {
+		return;
+	}
+
+	if( info->m_vm_pid > 0 ) {
+		dprintf( D_ALWAYS, "In VMUniverseMgr::killVM(), "
+				"Sending SIGKILL to Process[%d]\n", (int)info->m_vm_pid);
+		daemonCore->Send_Signal(info->m_vm_pid, SIGKILL);
+	}
+
+	MyString matchstring;
+	MyString workingdir;
+
+	workingdir.sprintf("%s%cdir_%ld", info->m_execute_dir.Value(),
+	                   DIR_DELIM_CHAR, (long)info->m_pid);
+
+	if( (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN ) == MATCH) || (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_KVM) == 0)) {
+		if( create_name_for_VM(&info->m_job_ad, matchstring) == false ) {
+			dprintf(D_ALWAYS, "VMUniverseMgr::killVM() : "
+					"cannot make the name of VM\n");
+			return;
+		}
+	}else {
+		// Except Xen, we need the path of working directory of Starter
+		// in order to destroy VM.
+		matchstring = workingdir;
+	}
+
+	killVM( matchstring.Value() );
+}
+
 bool 
 VMUniverseMgr::isStarterForVM(pid_t s_pid)
 {
@@ -941,6 +963,7 @@ VMUniverseMgr::getUsageForVM(pid_t s_pid, ProcFamilyUsage &usage)
 		usage.max_image_size = vm_usage.max_image_size;
 	}
 	usage.total_image_size += vm_usage.total_image_size;
+	usage.total_resident_set_size += vm_usage.total_resident_set_size;
 	return true;
 }
 
