@@ -45,9 +45,17 @@
 #include "exit.h"
 #include "condor_auth_x509.h"
 #include "setenv.h"
+#include "condor_claimid_parser.h"
+#include "condor_version.h"
+#include "sshd_proc.h"
+#include "condor_base64.h"
+#include "my_username.h"
 
 extern "C" int get_random_int();
-extern int main_shutdown_fast();
+extern void main_shutdown_fast();
+
+const char* JOB_AD_FILENAME = ".job.ad";
+const char* MACHINE_AD_FILENAME = ".machine.ad";
 
 /* CStarter class implementation */
 
@@ -68,6 +76,8 @@ CStarter::CStarter()
 	suspended = false;
 	m_privsep_helper = NULL;
 	m_configured = false;
+	m_job_environment_is_ready = false;
+	m_all_jobs_done = false;
 }
 
 
@@ -155,6 +165,9 @@ CStarter::Init( JobInfoCommunicator* my_jic, const char* original_cwd,
 	daemonCore->Register_Signal(DC_SIGREMOVE, "DC_SIGREMOVE",
 		(SignalHandlercpp)&CStarter::RemoteRemove, "RemoteRemove",
 		this);
+	daemonCore->Register_Signal(SIGUSR1, "SIGUSR1",
+		(SignalHandlercpp)&CStarter::RemoteRemove, "RemoteRemove",
+		this);
 	daemonCore->Register_Signal(DC_SIGHOLD, "DC_SIGHOLD",
 		(SignalHandlercpp)&CStarter::RemoteHold, "RemoteHold",
 		this);
@@ -184,6 +197,24 @@ CStarter::Init( JobInfoCommunicator* my_jic, const char* original_cwd,
 						  "DELEGATE_GSI_CRED_STARTER",
 						  (CommandHandlercpp)&CStarter::updateX509Proxy,
 						  "CStarter::updateX509Proxy", this, WRITE );
+	daemonCore->
+		Register_Command( STARTER_HOLD_JOB,
+						  "STARTER_HOLD_JOB",
+						  (CommandHandlercpp)&CStarter::remoteHoldCommand,
+						  "CStarter::remoteHoldCommand", this, DAEMON );
+	daemonCore->
+		Register_Command( CREATE_JOB_OWNER_SEC_SESSION,
+						  "CREATE_JOB_OWNER_SEC_SESSION",
+						  (CommandHandlercpp)&CStarter::createJobOwnerSecSession,
+						  "CStarter::createJobOwnerSecSession", this, DAEMON );
+
+		// Job-owner commands are registered at READ authorization level.
+		// Why?  See the explanation in createJobOwnerSecSession().
+	daemonCore->
+		Register_Command( START_SSHD,
+						  "START_SSHD",
+						  (CommandHandlercpp)&CStarter::startSSHD,
+						  "CStarter::startSSHD", this, READ );
 
 	sysapi_set_resource_limits();
 
@@ -207,7 +238,7 @@ CStarter::Init( JobInfoCommunicator* my_jic, const char* original_cwd,
 void
 CStarter::StarterExit( int code )
 {
-	removeTempExecuteDir();
+	FinalCleanup();
 #if !defined(WIN32)
 	if ( GetEnv( "CONDOR_GLEXEC_STARTER_CLEANUP_FLAG" ) ) {
 		exitAfterGlexec( code );
@@ -216,6 +247,12 @@ CStarter::StarterExit( int code )
 	DC_Exit( code );
 }
 
+void CStarter::FinalCleanup()
+{
+	RemoveRecoveryFile();
+	removeTempExecuteDir();
+}
+
 
 void
 CStarter::Config()
@@ -496,6 +533,749 @@ CStarter::RemoteHold( int )
 	return ( false );
 }
 
+int 
+CStarter::createJobOwnerSecSession( int /*cmd*/, Stream* s )
+{
+		// A Condor daemon on the submit side (e.g. schedd) wishes to
+		// create a security session for use by a user
+		// (e.g. condor_rsh_to_job).  Why doesn't the user's tool just
+		// connect and create a security session directly?  Because
+		// the starter is not necessarily set up to authenticate the
+		// owner of the job.  So the schedd authenticates the owner of
+		// the job and facilitates the creation of a security session
+		// between the starter and the tool.
+
+	MyString fqu;
+	getJobOwnerFQUOrDummy(fqu);
+	ASSERT( !fqu.IsEmpty() );
+
+	MyString error_msg;
+	ClassAd input;
+	s->decode();
+	if( !input.initFromStream(*s) || !s->end_of_message() ) {
+		dprintf(D_ALWAYS,"Failed to read request in createJobOwnerSecSession()\n");
+		return FALSE;
+	}
+
+		// In order to ensure that we are really talking to the schedd
+		// that is managing this job, check that the schedd has provided
+		// the correct secret claim id.
+	MyString job_claim_id;
+	MyString input_claim_id;
+	getJobClaimId(job_claim_id);
+	input.LookupString(ATTR_CLAIM_ID,input_claim_id);
+	if( job_claim_id != input_claim_id || job_claim_id.IsEmpty() ) {
+		dprintf(D_ALWAYS,
+				"Claim ID provided to createJobOwnerSecSession does not match "
+				"expected value!  Rejecting connection from %s\n",
+				s->peer_description());
+		return FALSE;
+	}
+
+	char *session_id = Condor_Crypt_Base::randomHexKey();
+	char *session_key = Condor_Crypt_Base::randomHexKey();
+
+	MyString session_info;
+	input.LookupString(ATTR_SESSION_INFO,session_info);
+
+		// Create an authorization rule so that the job owner can
+		// access commands that the job owner should be able to
+		// access.  All such commands are registered at READ level.
+		// Why?  Because all such commands do a special check to make
+		// sure the authenticated name matches the job owner name.
+		// Therefore, to give the job owner as little power as
+		// necessary, these commands are registered as READ (rather
+		// than something more powerful such as DAEMON) and we leave
+		// it up to the job-owner commands themselves to be more
+		// selective about who gets to successfully run the command.
+		// Ideally, we would have a first-class OWNER access level
+		// instead.
+
+	IpVerify* ipv = daemonCore->getSecMan()->getIpVerify();
+	bool rc = ipv->PunchHole(READ, fqu);
+	if( !rc ) {
+		error_msg = "Starter failed to create authorization entry for job owner.";
+	}
+
+	if( rc ) {
+		rc = daemonCore->getSecMan()->CreateNonNegotiatedSecuritySession(
+			READ,
+			session_id,
+			session_key,
+			session_info.Value(),
+			fqu.Value(),
+			NULL,
+			0 );
+	}
+	if( rc ) {
+			// get the final session parameters that were chosen
+		session_info = "";
+		rc = daemonCore->getSecMan()->ExportSecSessionInfo(
+			session_id,
+			session_info );
+	}
+
+	ClassAd response;
+	response.Assign(ATTR_VERSION,CondorVersion());
+	if( !rc ) {
+		if( error_msg.IsEmpty() ) {
+			error_msg = "Failed to create security session.";
+		}
+		response.Assign(ATTR_RESULT,false);
+		response.Assign(ATTR_ERROR_STRING,error_msg);
+		dprintf(D_ALWAYS,
+				"createJobOwnerSecSession failed: %s\n", error_msg.Value());
+	}
+	else {
+		// We use a "claim id" string to hold the security session info,
+		// because it is a convenient container.
+
+		ClaimIdParser claimid(session_id,session_info.Value(),session_key);
+		response.Assign(ATTR_RESULT,true);
+		response.Assign(ATTR_CLAIM_ID,claimid.claimId());
+		response.Assign(ATTR_STARTER_IP_ADDR,daemonCore->publicNetworkIpAddr());
+
+		dprintf(D_FULLDEBUG,"Created security session for job owner (%s).\n",
+				fqu.Value());
+	}
+
+	if( !response.put(*s) || !s->end_of_message() ) {
+		dprintf(D_ALWAYS,
+				"createJobOwnerSecSession failed to send response\n");
+	}
+
+	free( session_id );
+	free( session_key );
+
+	return TRUE;
+}
+
+int
+CStarter::vSSHDFailed(Stream *s,bool retry,char const *fmt,va_list args)
+{
+	MyString error_msg;
+	error_msg.vsprintf( fmt, args );
+
+		// old classads cannot handle a string ending in a double quote
+		// followed by a newline, so strip off any trailing newline
+	error_msg.trim();
+
+	dprintf(D_ALWAYS,"START_SSHD failed: %s\n",error_msg.Value());
+
+	ClassAd response;
+	response.Assign(ATTR_RESULT,false);
+	response.Assign(ATTR_ERROR_STRING,error_msg);
+	if( retry ) {
+		response.Assign(ATTR_RETRY,retry);
+	}
+
+	s->encode();
+	if( !response.put(*s) || !s->end_of_message() ) {
+		dprintf(D_ALWAYS,"Failed to send response to START_SSHD.\n");
+	}
+
+	return FALSE;
+}
+
+int
+CStarter::SSHDRetry(Stream *s,char const *fmt,...)
+{
+	va_list args;
+	va_start( args, fmt );
+	vSSHDFailed(s,true,fmt,args);
+	va_end( args );
+
+	return FALSE;
+}
+int
+CStarter::SSHDFailed(Stream *s,char const *fmt,...)
+{
+	va_list args;
+	va_start( args, fmt );
+	vSSHDFailed(s,false,fmt,args);
+	va_end( args );
+
+	return FALSE;
+}
+
+// The following functions are only needed if SSH_TO_JOB is supported.
+
+#if HAVE_SSH_TO_JOB
+
+// returns position of str in buffer or -1 if not found
+static int find_str_in_buffer(
+	char const *buffer,
+	int buffer_len,
+	char const *str)
+{
+	int str_len = strlen(str);
+    int i;
+	for(i=0; i+str_len <= buffer_len; i++) {
+		if( memcmp(buffer+i,str,str_len)==0 ) {
+			return i;
+		}
+	}
+	return -1;
+}
+
+static bool extract_delimited_data_as_base64(
+	char const *input_buffer,
+	int input_len,
+	char const *begin_marker,
+	char const *end_marker,
+	MyString &output_buffer,
+	MyString *error_msg)
+{
+	int start = find_str_in_buffer(input_buffer,input_len,begin_marker);
+	int end = find_str_in_buffer(input_buffer,input_len,end_marker);
+	if( start < 0 ) {
+		if( error_msg ) {
+			error_msg->sprintf("Failed to find '%s' in input: %.*s",
+							   begin_marker,input_len,input_buffer);
+		}
+		return false;
+	}
+	start += strlen(begin_marker);
+	if( end < 0 || end < start ) {
+		if( error_msg ) {
+			error_msg->sprintf("Failed to find '%s' in input: %.*s",
+							   end_marker,input_len,input_buffer);
+		}
+		return false;
+	}
+	char *encoded = condor_base64_encode((unsigned char const *)input_buffer+start,end-start);
+	output_buffer = encoded;
+	free(encoded);
+	return true;
+}
+
+
+static bool extract_delimited_data(
+	char const *input_buffer,
+	int input_len,
+	char const *begin_marker,
+	char const *end_marker,
+	MyString &output_buffer,
+	MyString *error_msg)
+{
+	int start = find_str_in_buffer(input_buffer,input_len,begin_marker);
+	int end = find_str_in_buffer(input_buffer,input_len,end_marker);
+	if( start < 0 ) {
+		if( error_msg ) {
+			error_msg->sprintf("Failed to find '%s' in input: %.*s",
+							   begin_marker,input_len,input_buffer);
+		}
+		return false;
+	}
+	start += strlen(begin_marker);
+	if( end < 0 || end < start ) {
+		if( error_msg ) {
+			error_msg->sprintf("Failed to find '%s' in input: %.*s",
+							   end_marker,input_len,input_buffer);
+		}
+		return false;
+	}
+	output_buffer.sprintf("%.*s",end-start,input_buffer+start);
+	return true;
+}
+
+#endif
+
+int
+CStarter::startSSHD( int /*cmd*/, Stream* s )
+{
+		// This command should only be allowed by the job owner.
+	MyString error_msg;
+	Sock *sock = (Sock*)s;
+	char const *fqu = sock->getFullyQualifiedUser();
+	MyString job_owner;
+	getJobOwnerFQUOrDummy(job_owner);
+	if( !fqu || job_owner != fqu ) {
+		dprintf(D_ALWAYS,"Unauthorized attempt to start sshd by '%s'\n",
+				fqu ? fqu : "");
+		return FALSE;
+	}
+
+	ClassAd input;
+	s->decode();
+	if( !input.initFromStream(*s) || !s->end_of_message() ) {
+		dprintf(D_ALWAYS,"Failed to read request in START_SSHD.\n");
+		return FALSE;
+	}
+
+#if !defined(HAVE_SSH_TO_JOB)
+	return SSHDFailed(s,"This version of condor_starter does not support ssh access.");
+#else
+
+	ClassAd *jobad = NULL;
+	ClassAd *machinead = NULL;
+	if( jic ) {
+		jobad = jic->jobClassAd();
+		machinead = jic->machClassAd();
+	}
+
+	bool enabled = param_boolean("ENABLE_SSH_TO_JOB",true,true,machinead,jobad);
+	if( !enabled ) {
+		return SSHDFailed(s,"Rejecting request, because ENABLE_SSH_TO_JOB=false");
+	}
+
+	if( !jic || !jobad ) {
+		return SSHDRetry(s,"Rejecting request, because job not yet initialized.");
+	}
+	if( !m_job_environment_is_ready ) {
+			// This can happen if file transfer is still in progress.
+			// At this stage, the sandbox might not even be owned by the user.
+		return SSHDRetry(s,"Rejecting request, because the job execution environment is not yet ready.");
+	}
+	if( m_all_jobs_done ) {
+		return SSHDFailed(s,"Rejecting request, because the job is finished.");
+	}
+	if( suspended ) {
+			// Better to reject them with a clear explanation rather
+			// than to allow them to connect and then immediately
+			// suspend them without any explanation.
+		return SSHDRetry(s,"This slot is currently suspended.");
+	}
+
+	MyString preferred_shells;
+	input.LookupString(ATTR_SHELL,preferred_shells);
+
+	MyString slot_name;
+	input.LookupString(ATTR_NAME,slot_name);
+
+	char *username;
+	if( !jic->userPrivInitialized() ) {
+		username = NULL;
+	}
+	else if( condorPrivSepHelper() != NULL ) {
+		username = strdup(condorPrivSepHelper()->get_user_name());
+	}
+	else if( can_switch_ids() ) {
+		username = strdup(get_user_loginname());
+	}
+	else {
+		username = my_username();
+	}
+	if( !username ) {
+		return SSHDRetry(s,"Rejecting request, because job execution account not yet established.");
+	}
+
+	MyString libexec;
+	if( !param(libexec,"LIBEXEC") ) {
+		if(username) free(username);
+		return SSHDFailed(s,"LIBEXEC not defined, so cannot find condor_ssh_to_job_sshd_setup");
+	}
+	MyString ssh_to_job_sshd_setup;
+	MyString ssh_to_job_shell_setup;
+	ssh_to_job_sshd_setup.sprintf(
+		"%s%ccondor_ssh_to_job_sshd_setup",libexec.Value(),DIR_DELIM_CHAR);
+	ssh_to_job_shell_setup.sprintf(
+		"%s%ccondor_ssh_to_job_shell_setup",libexec.Value(),DIR_DELIM_CHAR);
+
+	if( access(ssh_to_job_sshd_setup.Value(),X_OK)!=0 ) {
+		if(username) free(username);
+		return SSHDFailed(s,"Cannot execute %s: %s",
+						  ssh_to_job_sshd_setup.Value(),strerror(errno));
+	}
+	if( access(ssh_to_job_shell_setup.Value(),X_OK)!=0 ) {
+		if(username) free(username);
+		return SSHDFailed(s,"Cannot execute %s: %s",
+						  ssh_to_job_shell_setup.Value(),strerror(errno));
+	}
+
+	MyString sshd_config_template;
+	if( !param(sshd_config_template,"SSH_TO_JOB_SSHD_CONFIG_TEMPLATE") ) {
+		if( param(sshd_config_template,"LIB") ) {
+			sshd_config_template.sprintf_cat("%ccondor_ssh_to_job_sshd_config_template",DIR_DELIM_CHAR);
+		}
+		else {
+			if(username) free(username);
+			return SSHDFailed(s,"SSH_TO_JOB_SSHD_CONFIG_TEMPLATE and LIB are not defined.  At least one of them is required.");
+		}
+	}
+	if( access(sshd_config_template.Value(),F_OK)!=0 ) {
+		if(username) free(username);
+		return SSHDFailed(s,"%s does not exist!",sshd_config_template.Value());
+	}
+
+	MyString ssh_keygen;
+	MyString ssh_keygen_args;
+	ArgList ssh_keygen_arglist;
+	param(ssh_keygen,"SSH_TO_JOB_SSH_KEYGEN","/usr/bin/ssh-keygen");
+	param(ssh_keygen_args,"SSH_TO_JOB_SSH_KEYGEN_ARGS","\"-N '' -C '' -q -f %f -t rsa\"");
+	ssh_keygen_arglist.AppendArg(ssh_keygen.Value());
+	if( !ssh_keygen_arglist.AppendArgsV2Quoted(ssh_keygen_args.Value(),&error_msg) ) {
+		if(username) free(username);
+		return SSHDFailed(s,
+						  "SSH_TO_JOB_SSH_KEYGEN_ARGS is misconfigured: %s",
+						  error_msg.Value());
+	}
+
+	MyString client_keygen_args;
+	input.LookupString(ATTR_SSH_KEYGEN_ARGS,client_keygen_args);
+	if( !ssh_keygen_arglist.AppendArgsV2Raw(client_keygen_args.Value(),&error_msg) ) {
+		return SSHDFailed(s,
+						  "Failed to produce ssh-keygen arg list: %s",
+						  error_msg.Value());
+	}
+
+	MyString ssh_keygen_cmd;
+	if(!ssh_keygen_arglist.GetArgsStringSystem(&ssh_keygen_cmd,0,&error_msg)) {
+		return SSHDFailed(s,
+						  "Failed to produce ssh-keygen command string: %s",
+						  error_msg.Value());
+	}
+
+	int setup_pipe_fds[2];
+	setup_pipe_fds[0] = -1;
+	setup_pipe_fds[1] = -1;
+	if( !daemonCore->Create_Pipe(setup_pipe_fds) ) {
+		return SSHDFailed(
+			s,"Failed to create pipe for condor_ssh_to_job_sshd_setup.");
+	}
+	int setup_std_fds[3];
+	setup_std_fds[0] = 0;
+	setup_std_fds[1] = setup_pipe_fds[1]; // write end of pipe
+	setup_std_fds[2] = setup_pipe_fds[1];
+
+		// Pass the job environment to the sshd setup script.  It will
+		// save the environment so that it can be restored when the
+		// ssh session starts and our shell setup script is called.
+	Env setup_env;
+	if( !GetJobEnv( jobad, &setup_env, &error_msg ) ) {
+		return SSHDFailed(
+			s,"Failed to get job environment: %s",error_msg.Value());
+	}
+
+	if( !slot_name.IsEmpty() ) {
+		setup_env.SetEnv("_CONDOR_SLOT_NAME",slot_name.Value());
+	}
+
+	if( !preferred_shells.IsEmpty() ) {
+		dprintf(D_FULLDEBUG,
+				"Checking preferred shells: %s\n",preferred_shells.Value());
+		StringList shells(preferred_shells.Value(),",");
+		shells.rewind();
+		char *shell;
+		while( (shell=shells.next()) ) {
+			if( access(shell,X_OK)==0 ) {
+				dprintf(D_FULLDEBUG,"Will use shell %s\n",shell);
+				setup_env.SetEnv("_CONDOR_SHELL",shell);
+				break;
+			}
+		}
+	}
+
+	ArgList setup_args;
+	setup_args.AppendArg(ssh_to_job_sshd_setup.Value());
+	setup_args.AppendArg(GetWorkingDir());
+	setup_args.AppendArg(ssh_to_job_shell_setup.Value());
+	setup_args.AppendArg(sshd_config_template.Value());
+	setup_args.AppendArg(ssh_keygen_cmd.Value());
+
+		// Would like to use my_popen here, but we need to support glexec.
+		// Use the default reaper, even though it doesn't know anything
+		// about this task.  We avoid needing to know the final exit status
+		// by checking for a magic success string at the end of the output.
+	int setup_reaper = 1;
+	int setup_pid;
+	if( privSepHelper() ) {
+		setup_pid = privSepHelper()->create_process(
+			ssh_to_job_sshd_setup.Value(),
+			setup_args,
+			setup_env,
+			GetWorkingDir(),
+			setup_std_fds,
+			NULL,
+			0,
+			NULL,
+			setup_reaper,
+			0,
+			NULL);
+	}
+	else {
+		setup_pid = daemonCore->Create_Process(
+			ssh_to_job_sshd_setup.Value(),
+			setup_args,
+			PRIV_USER_FINAL,
+			setup_reaper,
+			FALSE,
+			&setup_env,
+			GetWorkingDir(),
+			NULL,
+			NULL,
+			setup_std_fds);
+	}
+
+	daemonCore->Close_Pipe(setup_pipe_fds[1]); // write-end of pipe
+
+		// NOTE: the output from the script may contain binary data,
+		// so we can't just slurp it in as a C string.
+	char *setup_output = NULL;
+	int setup_output_len = 0;
+	char pipe_buf[1024];
+	while( true ) {
+		int n = daemonCore->Read_Pipe(setup_pipe_fds[0],pipe_buf,1024);
+		if( n <= 0 ) {
+			break;
+		}
+		char *old_setup_output = setup_output;
+		setup_output = (char *)realloc(setup_output,setup_output_len+n+1);
+		if( !setup_output ) {
+			free( old_setup_output );
+			daemonCore->Close_Pipe(setup_pipe_fds[0]); // read-end of pipe
+			return SSHDFailed(s,"Out of memory");
+		}
+		memcpy(setup_output+setup_output_len,pipe_buf,n);
+		setup_output_len += n;
+			// for easier debugging, append a null to the end,
+			// so we can try to print the buffer as a string
+		setup_output[setup_output_len] = '\0';
+	}
+
+	daemonCore->Close_Pipe(setup_pipe_fds[0]); // read-end of pipe
+
+		// look for magic success string
+	if( find_str_in_buffer(setup_output,setup_output_len,"condor_ssh_to_job_sshd_setup SUCCESS") < 0 ) {
+		error_msg.sprintf("condor_ssh_to_job_sshd_setup failed: %s",
+						  setup_output);
+		free( setup_output );
+		return SSHDFailed(s,"%s",error_msg.Value());
+	}
+
+		// Since in privsep situations, we cannot directly access the
+		// ssh key files that sshd_setup prepared, we slurp them in
+		// from the pipe to sshd_setup.
+
+	bool rc = true;
+	MyString session_dir;
+	if( rc ) {
+		rc = extract_delimited_data(
+			setup_output,
+			setup_output_len,
+			"condor_ssh_to_job_sshd_setup SSHD DIR BEGIN\n",
+			"\ncondor_ssh_to_job_sshd_setup SSHD DIR END\n",
+			session_dir,
+			&error_msg);
+	}
+
+	MyString public_host_key;
+	if( rc ) {
+		rc = extract_delimited_data_as_base64(
+			setup_output,
+			setup_output_len,
+			"condor_ssh_to_job_sshd_setup PUBLIC SERVER KEY BEGIN\n",
+			"condor_ssh_to_job_sshd_setup PUBLIC SERVER KEY END\n",
+			public_host_key,
+			&error_msg);
+	}
+
+	MyString private_client_key;
+	if( rc ) {
+		rc = extract_delimited_data_as_base64(
+			setup_output,
+			setup_output_len,
+			"condor_ssh_to_job_sshd_setup AUTHORIZED CLIENT KEY BEGIN\n",
+			"condor_ssh_to_job_sshd_setup AUTHORIZED CLIENT KEY END\n",
+			private_client_key,
+			&error_msg);
+	}
+
+	free( setup_output );
+
+	if( !rc ) {
+		MyString msg;
+		if(username) free(username);
+		return SSHDFailed(s,
+			"Failed to parse output of condor_ssh_to_job_sshd_setup: %s",
+			error_msg.Value());
+	}
+
+	dprintf(D_FULLDEBUG,"StartSSHD: session_dir='%s'\n",session_dir.Value());
+
+	MyString sshd_config_file;
+	sshd_config_file.sprintf("%s%csshd_config",session_dir.Value(),DIR_DELIM_CHAR);
+
+
+
+	MyString sshd;
+	param(sshd,"SSH_TO_JOB_SSHD","/usr/sbin/sshd");
+	if( access(sshd.Value(),X_OK)!=0 ) {
+		if(username) free(username);
+		return SSHDFailed(s,"Failed, because sshd not correctly configured (SSH_TO_JOB_SSHD=%s): %s.",sshd.Value(),strerror(errno));
+	}
+
+	ArgList sshd_arglist;
+	MyString sshd_arg_string;
+	param(sshd_arg_string,"SSH_TO_JOB_SSHD_ARGS","\"-i -e -f %f\"");
+	if( !sshd_arglist.AppendArgsV2Quoted(sshd_arg_string.Value(),&error_msg) )
+	{
+		if(username) free(username);
+		return SSHDFailed(s,"Invalid SSH_TO_JOB_SSHD_ARGS (%s): %s",
+						  sshd_arg_string.Value(),error_msg.Value());
+	}
+
+	char **argarray = sshd_arglist.GetStringArray();
+	sshd_arglist.Clear();
+	for(int i=0; argarray[i]; i++) {
+		char const *ptr;
+		MyString new_arg;
+		for(ptr=argarray[i]; *ptr; ptr++) {
+			if( *ptr == '%' ) {
+				ptr += 1;
+				if( *ptr == '%' ) {
+					new_arg += '%';
+				}
+				else if( *ptr == 'f' ) {
+					new_arg += sshd_config_file.Value();
+				}
+				else {
+					if(username) free(username);
+					return SSHDFailed(s,
+							"Unexpected %%%c in SSH_TO_JOB_SSHD_ARGS: %s\n",
+							*ptr ? *ptr : ' ', sshd_arg_string.Value());
+				}
+			}
+			else {
+				new_arg += *ptr;
+			}
+		}
+		sshd_arglist.AppendArg(new_arg.Value());
+	}
+	deleteStringArray(argarray);
+	argarray = NULL;
+
+
+	ClassAd sshd_ad;
+	sshd_ad.CopyAttribute(ATTR_REMOTE_USER,jobad);
+	sshd_ad.CopyAttribute(ATTR_JOB_RUNAS_OWNER,jobad);
+	sshd_ad.Assign(ATTR_JOB_CMD,sshd.Value());
+	CondorVersionInfo ver_info;
+	if( !sshd_arglist.InsertArgsIntoClassAd(&sshd_ad,&ver_info,&error_msg) ) {
+		if(username) free(username);
+		return SSHDFailed(s,
+			"Failed to insert args into sshd job description: %s",
+			error_msg.Value());
+	}
+		// Since sshd clenses the user's environment, it doesn't really
+		// matter what we pass here.  Instead, we depend on sshd_shell_init
+		// to restore the environment that was saved by sshd_setup.
+		// However, we may as well pass the desired environment.
+	if( !setup_env.InsertEnvIntoClassAd(&sshd_ad,&error_msg,NULL,&ver_info) ) {
+		if(username) free(username);
+		return SSHDFailed(s,
+			"Failed to insert environment into sshd job description: %s",
+			error_msg.Value());
+	}
+
+
+
+		// Now send the expected ClassAd response to the caller.
+		// This is the last exchange before switching the connection
+		// over to the ssh protocol.
+	ClassAd response;
+	response.Assign(ATTR_RESULT,true);
+	response.Assign(ATTR_REMOTE_USER,username);
+	response.Assign(ATTR_SSH_PUBLIC_SERVER_KEY,public_host_key.Value());
+	response.Assign(ATTR_SSH_PRIVATE_CLIENT_KEY,private_client_key.Value());
+
+	free(username);
+	username = NULL;
+	s->encode();
+	if( !response.put(*s) || !s->end_of_message() ) {
+		dprintf(D_ALWAYS,"Failed to send response to START_SSHD.\n");
+		return FALSE;
+	}
+
+
+
+	MyString sshd_log_fname;
+	sshd_log_fname.sprintf(
+		"%s%c%s",session_dir.Value(),DIR_DELIM_CHAR,"sshd.log");
+
+
+	int std[3];
+	char const *std_fname[3];
+	std[0] = sock->get_file_desc();
+	std_fname[0] = "stdin";
+	std[1] = sock->get_file_desc();
+	std_fname[1] = "stdout";
+	std[2] = -1;
+	std_fname[2] = sshd_log_fname.Value();
+
+
+	SSHDProc *proc = new SSHDProc(&sshd_ad);
+	if( !proc ) {
+		dprintf(D_ALWAYS,"Failed to create SSHDProc.\n");
+		return FALSE;
+	}
+	if( !proc->StartJob(std,std_fname) ) {
+		dprintf(D_ALWAYS,"Failed to start sshd.\n");
+		return FALSE;
+	}
+	m_job_list.Append(proc);
+	if( this->suspended ) {
+		proc->Suspend();
+	}
+
+	return TRUE;
+#endif
+}
+
+/**
+ * Hold Job Command (sent by startd)
+ * Unlike the DC signal, this includes a hold reason and hold code.
+ * Also unlike the DC signal, this _puts_ the job on hold, rather than
+ * just passively informing the JIC of the hold.
+ * 
+ * @param command (not used)
+ * @return true if ????, otherwise false
+ */ 
+int 
+CStarter::remoteHoldCommand( int /*cmd*/, Stream* s )
+{
+	MyString hold_reason;
+	int hold_code;
+	int hold_subcode;
+
+	s->decode();
+	if( !s->get(hold_reason) ||
+		!s->get(hold_code) ||
+		!s->get(hold_subcode) ||
+		!s->end_of_message() )
+	{
+		dprintf(D_ALWAYS,"Failed to read message from %s in CStarter::remoteHoldCommand()\n", s->peer_description());
+		return FALSE;
+	}
+
+		// Put the job on hold on the remote side.
+	if( jic ) {
+		jic->holdJob(hold_reason.Value(),hold_code,hold_subcode);
+	}
+
+	int reply = 1;
+	s->encode();
+	if( !s->put(reply) || !s->end_of_message()) {
+		dprintf(D_ALWAYS,"Failed to send response to startd in CStarter::remoteHoldCommand()\n");
+	}
+
+		//
+		// If this method returns true, that means there were no
+		// jobs running. We'll send out a little debug message 
+		// and let ourselves know that all the jobs are done
+		// Note that this used to be in the Hold() call but
+		// this causes problems if the Hold is coming from ourself
+		// For instance, if the OnExitHold evaluates to true for
+		// a local universe job
+		//
+	if ( this->Hold( ) ) {
+		dprintf( D_FULLDEBUG, "Got Hold when no jobs running\n" );
+		this->allJobsDone();
+		return ( true );
+	}	
+	return ( false );
+}
+
 /**
  * Attempts to put all the jobs on hold
  * If a job has already been put on hold, then the job is removed
@@ -587,8 +1367,8 @@ CStarter::createTempExecuteDir( void )
 	CondorPrivSepHelper* cpsh = condorPrivSepHelper();
 	if (cpsh != NULL) {
 		cpsh->initialize_sandbox(WorkingDir.Value());
-	}
-	else {
+		WriteAdFiles();
+	} else {
 		if( mkdir(WorkingDir.Value(), 0777) < 0 ) {
 			dprintf( D_FAILURE|D_ALWAYS,
 			         "couldn't create dir %s: %s\n",
@@ -597,6 +1377,7 @@ CStarter::createTempExecuteDir( void )
 			set_priv( priv );
 			return false;
 		}
+		WriteAdFiles();
 #if !defined(WIN32)
 		if (use_chown) {
 			priv_state p = set_root_priv();
@@ -757,6 +1538,8 @@ CStarter::jobEnvironmentReady( void )
 		}
 	}
 
+	m_job_environment_is_ready = true;
+
 		//
 		// The Starter will determine when the job 
 		// should be started. This method will always return 
@@ -813,7 +1596,7 @@ CStarter::jobWaitUntilExecuteTime( void )
 	int deferralOffset = 0;
 	int deltaT = 0;
 	int deferralWindow = 0;
-	if ( jobAd->Lookup( ATTR_DEFERRAL_TIME ) != NULL ) {
+	if ( jobAd->LookupExpr( ATTR_DEFERRAL_TIME ) != NULL ) {
 			//
 		 	// Make sure that the expression evaluated and we 
 		 	// got a positive integer. Otherwise we'll have to kick out
@@ -863,7 +1646,7 @@ CStarter::jobWaitUntilExecuteTime( void )
 				// So if the deferralTime is less than the currenTime,
 				// but within this window, we'll still run the job
 				//
-			if ( jobAd->Lookup( ATTR_DEFERRAL_WINDOW ) != NULL &&
+			if ( jobAd->LookupExpr( ATTR_DEFERRAL_WINDOW ) != NULL &&
 				 jobAd->EvalInteger( ATTR_DEFERRAL_WINDOW, NULL, deferralWindow ) ) {
 				dprintf( D_FULLDEBUG, "Job %d.%d has a deferral window of "
 				                      "%d seconds\n", 
@@ -1020,7 +1803,7 @@ CStarter::removeDeferredJobs() {
  * 
  * return true if no errors occured
  **/
-int
+void
 CStarter::SpawnPreScript( void )
 {
 		//
@@ -1038,7 +1821,7 @@ CStarter::SpawnPreScript( void )
 
 	attr = "Pre";
 	attr += ATTR_JOB_CMD;
-	if( jobAd->LookupString(attr.GetCStr(), &tmp) ) {
+	if( jobAd->LookupString(attr.Value(), &tmp) ) {
 		free( tmp );
 		tmp = NULL;
 		pre_script = new ScriptProc( jobAd, "Pre" );
@@ -1046,7 +1829,7 @@ CStarter::SpawnPreScript( void )
 
 	attr = "Post";
 	attr += ATTR_JOB_CMD;
-	if( jobAd->LookupString(attr.GetCStr(), &tmp) ) {
+	if( jobAd->LookupString(attr.Value(), &tmp) ) {
 		free( tmp );
 		tmp = NULL;
 		post_script = new ScriptProc( jobAd, "Post" );
@@ -1059,18 +1842,41 @@ CStarter::SpawnPreScript( void )
 				// if it's running, all we can do is return to
 				// DaemonCore and wait for the it to exit.  the
 				// reaper will then do the right thing
-			return TRUE;
+			return;
 		} else {
 			dprintf( D_ALWAYS, "Failed to start prescript, exiting\n" );
 				// TODO notify the JIC somehow?
 			main_shutdown_fast();
-			return FALSE;
+			return;
 		}
 	}
 
 		// if there's no pre-script, we can go directly to trying to
-		// spawn the main job
-	return SpawnJob();
+		// spawn the main job. if we fail to spawn the job we exit
+	if( ! SpawnJob() ) {
+		dprintf( D_ALWAYS, "Failed to start main job, exiting.\n" );
+		main_shutdown_fast();
+	}
+}
+
+void CStarter::getJobOwnerFQUOrDummy(MyString &result)
+{
+	ClassAd *jobAd = jic ? jic->jobClassAd() : NULL;
+	if( jobAd ) {
+		jobAd->LookupString(ATTR_USER,result);
+	}
+	if( result.IsEmpty() ) {
+		result = "job-owner@submit-domain";
+	}
+}
+
+bool CStarter::getJobClaimId(MyString &result)
+{
+	ClassAd *jobAd = jic ? jic->jobClassAd() : NULL;
+	if( jobAd ) {
+		return jobAd->LookupString(ATTR_CLAIM_ID,result);
+	}
+	return false;
 }
 
 /**
@@ -1194,6 +2000,56 @@ CStarter::SpawnJob( void )
 	}
 }
 
+void
+CStarter::WriteRecoveryFile( ClassAd *recovery_ad )
+{
+	MyString tmp_file;
+	FILE *tmp_fp;
+
+	if ( recovery_ad == NULL ) {
+		return;
+	}
+
+	if ( m_recoveryFile.Length() == 0 ) {
+		m_recoveryFile.sprintf( "%s%cdir_%ld.recover", Execute,
+								DIR_DELIM_CHAR, (long)daemonCore->getpid() );
+	}
+
+	tmp_file.sprintf( "%s.tmp", m_recoveryFile.Value() );
+
+	tmp_fp = safe_fcreate_replace_if_exists( tmp_file.Value(), "w" );
+	if ( tmp_fp == NULL ) {
+		dprintf( D_ALWAYS, "Failed to open recovery file %s\n", tmp_file.Value() );
+		return;
+	}
+
+	if ( recovery_ad->fPrint( tmp_fp ) == FALSE ) {
+		dprintf( D_ALWAYS, "Failed to write recovery file\n" );
+		fclose( tmp_fp );
+		return;
+	}
+
+	if ( fclose( tmp_fp ) != 0 ) {
+		dprintf( D_ALWAYS, "Failed close recovery file\n" );
+		unlink( tmp_file.Value() );
+		return;
+	}
+
+	if ( rotate_file( tmp_file.Value(), m_recoveryFile.Value() ) != 0 ) {
+		dprintf( D_ALWAYS, "Failed to rename recovery file\n" );
+		unlink( tmp_file.Value() );
+	}
+}
+
+void
+CStarter::RemoveRecoveryFile()
+{
+	if ( m_recoveryFile.Length() > 0 ) {
+		unlink( m_recoveryFile.Value() );
+		m_recoveryFile = "";
+	}
+}
+
 /**
  * DC Suspend Job Wrapper
  * We notify our JIC that we got a suspend call
@@ -1448,6 +2304,7 @@ CStarter::Reaper(int pid, int exit_status)
 				// so, we can directly call allJobsDone() to do final
 				// cleanup.
 			if( !allJobsDone() ) {
+				dprintf(D_ALWAYS, "Returning from CStarter::JobReaper()\n");
 				return 0;
 			}
 		}
@@ -1457,6 +2314,7 @@ CStarter::Reaper(int pid, int exit_status)
 		dprintf(D_ALWAYS,"Last process exited, now Starter is exiting\n");
 		StarterExit(0);
 	}
+
 	return 0;
 }
 
@@ -1464,6 +2322,8 @@ CStarter::Reaper(int pid, int exit_status)
 bool
 CStarter::allJobsDone( void )
 {
+	m_all_jobs_done = true;
+
 		// now that all user processes are complete, change the
 		// sandbox ownership back over to condor. if this is a VM
 		// universe job, this chown will have already been
@@ -1489,7 +2349,39 @@ CStarter::allJobsDone( void )
 bool
 CStarter::transferOutput( void )
 {
-	if (!jic->transferOutput()) {
+	UserProc *job;
+	bool transient_failure = false;
+
+	if (jic->transferOutput(transient_failure) == false) {
+
+		if( transient_failure ) {
+				// we will retry the transfer when (if) the shadow reconnects
+			return false;
+		}
+
+		// Send, if the JIC thinks it is talking to a shadow of the right
+		// version, a message about the termination of the job to the shadow in
+		// the event of a file transfer failure.  The UserProc's classad which
+		// has an ATTR_JOB_PID attribute is the actual job the starter ran on
+		// behalf of the user. For other types of jobs like pre/post scripts,
+		// that attribute is name mangled and won't be present as ATTR_JOB_PID
+		// in the published ad.
+		//
+		// See the usage of the "name" variable in user_proc.h/cpp
+
+		m_reaped_job_list.Rewind();
+		while ((job = m_reaped_job_list.Next()) != NULL) {
+			ClassAd ad;
+			int pid;
+			job->PublishUpdateAd(&ad);
+			if (ad.LookupInteger(ATTR_JOB_PID, pid)) {
+				jic->notifyJobTermination(job);
+				break;
+			}
+		}
+
+		jic->transferOutputMopUp();
+
 			/*
 			  there was an error with the JIC in this step.  at this
 			  point, the only possible reason is if we're talking to a
@@ -1503,6 +2395,8 @@ CStarter::transferOutput( void )
 		return false;
 	}
 
+	jic->transferOutputMopUp();
+
 		// If we're here, the JIC successfully transfered output.
 		// We're ready to move on to the next cleanup stage.
 	return cleanupJobs();
@@ -1595,7 +2489,38 @@ CStarter::publishPostScriptUpdateAd( ClassAd* ad )
 	}
 	return false;
 }
-	
+
+bool
+CStarter::GetJobEnv( ClassAd *jobad, Env *job_env, MyString *env_errors )
+{
+	char *env_str = param( "STARTER_JOB_ENVIRONMENT" );
+
+	ASSERT( jobad );
+	ASSERT( job_env );
+	if( !job_env->MergeFromV1RawOrV2Quoted(env_str,env_errors) ) {
+		if( env_errors ) {
+			env_errors->sprintf_cat(
+				" The full value for STARTER_JOB_ENVIRONMENT: %s\n",env_str);
+		}
+		free(env_str);
+		return false;
+	}
+	free(env_str);
+
+	if(!job_env->MergeFrom(jobad,env_errors)) {
+		if( env_errors ) {
+			env_errors->sprintf_cat(
+				" (This error was from the environment string in the job "
+				"ClassAd.)");
+		}
+		return false;
+	}
+
+		// Now, let the starter publish any env vars it wants to into
+		// the mainjob's env...
+	PublishToEnv( job_env );
+	return true;
+}	
 
 void
 CStarter::PublishToEnv( Env* proc_env )
@@ -1608,11 +2533,23 @@ CStarter::PublishToEnv( Env* proc_env )
 		// after everything else, so there's not going to be any info
 		// about it to pass until it's already done.
 
+		// used by sshd_shell_setup script to cd to the job working dir
+	proc_env->SetEnv("_CONDOR_JOB_IWD",jic->jobRemoteIWD());
+
+	MyString job_pids;
 	UserProc* uproc;
 	m_job_list.Rewind();
 	while ((uproc = m_job_list.Next()) != NULL) {
 		uproc->PublishToEnv( proc_env );
+
+		if( ! job_pids.IsEmpty() ) {
+			job_pids += " ";
+		}
+		job_pids.sprintf_cat("%d",uproc->GetJobPid());		
 	}
+		// put the pid of the job in the environment, used by sshd and hooks
+	proc_env->SetEnv("_CONDOR_JOB_PIDS",job_pids);
+
 	m_reaped_job_list.Rewind();
 	while ((uproc = m_reaped_job_list.Next()) != NULL) {
 		uproc->PublishToEnv( proc_env );
@@ -1641,24 +2578,24 @@ CStarter::PublishToEnv( Env* proc_env )
 		// path to the output ad, if any
 	const char* output_ad = jic->getOutputAdFile();
 	if( output_ad && !(output_ad[0] == '-' && output_ad[1] == '\0') ) {
-		env_name = base.GetCStr();
+		env_name = base.Value();
 		env_name += "OUTPUT_CLASSAD";
-		proc_env->SetEnv( env_name.GetCStr(), output_ad );
+		proc_env->SetEnv( env_name.Value(), output_ad );
 	}
 	
 		// job scratch space
-	env_name = base.GetCStr();
+	env_name = base.Value();
 	env_name += "SCRATCH_DIR";
-	proc_env->SetEnv( env_name.GetCStr(), GetWorkingDir() );
+	proc_env->SetEnv( env_name.Value(), GetWorkingDir() );
 
 		// slot identifier
-	env_name = base.GetCStr();
+	env_name = base.Value();
 	env_name += "SLOT";
 	int slot = getMySlotNumber();
 	if (!slot) {
 		slot = 1;
 	}
-	proc_env->SetEnv(env_name.GetCStr(), slot);
+	proc_env->SetEnv(env_name.Value(), slot);
 
 		// pass through the pidfamily ancestor env vars this process
 		// currently has to the job.
@@ -1669,15 +2606,23 @@ CStarter::PublishToEnv( Env* proc_env )
 		MyString tmp_port_number;
 
 		tmp_port_number = high;
-		env_name = base.GetCStr();
+		env_name = base.Value();
 		env_name += "HIGHPORT";
-		proc_env->SetEnv( env_name.GetCStr(), tmp_port_number.GetCStr() );
+		proc_env->SetEnv( env_name.Value(), tmp_port_number.Value() );
 
 		tmp_port_number = low;
-		env_name = base.GetCStr();
+		env_name = base.Value();
 		env_name += "LOWPORT";
-		proc_env->SetEnv( env_name.GetCStr(), tmp_port_number.GetCStr() );
+		proc_env->SetEnv( env_name.Value(), tmp_port_number.Value() );
     }
+
+		// set environment variables for temporary directories
+		// Condor will clean these up on job exits, and there's
+		// no chance of file collisions with other running slots
+
+	proc_env->SetEnv("TMPDIR", GetWorkingDir());
+	proc_env->SetEnv("TEMP", GetWorkingDir()); // Windows
+	proc_env->SetEnv("TMP", GetWorkingDir()); // Windows
 }
 
 
@@ -1860,3 +2805,65 @@ CStarter::exitAfterGlexec( int code )
 	exit( code );
 }
 #endif
+
+bool
+CStarter::WriteAdFiles()
+{
+
+	ClassAd* ad;
+	const char* dir = this->GetWorkingDir();
+	MyString ad_str, filename;
+	FILE* fp;
+	bool ret_val = true;
+
+	// Write the job ad first
+	ad = this->jic->jobClassAd();
+	if (ad != NULL)
+	{
+		filename.sprintf("%s%c%s", dir, DIR_DELIM_CHAR, JOB_AD_FILENAME);
+		fp = safe_fopen_wrapper(filename.Value(), "w");
+		if (!fp)
+		{
+			dprintf(D_ALWAYS, "Failed to open \"%s\" for to write job ad: "
+						"%s (errno %d)\n", filename.Value(),
+						strerror(errno), errno);
+			ret_val = false;
+		}
+		else
+		{
+			ad->SetPrivateAttributesInvisible(true);
+			ad->fPrint(fp);
+			ad->SetPrivateAttributesInvisible(false);
+			fclose(fp);
+		}
+	}
+	else
+	{
+		// If there is no job ad, this is a problem
+		ret_val = false;
+	}
+
+	// Write the machine ad
+	ad = this->jic->machClassAd();
+	if (ad != NULL)
+	{
+		filename.sprintf("%s%c%s", dir, DIR_DELIM_CHAR, MACHINE_AD_FILENAME);
+		fp = safe_fopen_wrapper(filename.Value(), "w");
+		if (!fp)
+		{
+			dprintf(D_ALWAYS, "Failed to open \"%s\" for to write machine "
+						"ad: %s (errno %d)\n", filename.Value(),
+					strerror(errno), errno);
+			ret_val = false;
+		}
+		else
+		{
+			ad->SetPrivateAttributesInvisible(true);
+			ad->fPrint(fp);
+			ad->SetPrivateAttributesInvisible(false);
+			fclose(fp);
+		}
+	}
+
+	return ret_val;
+}
