@@ -112,7 +112,7 @@ JICLocal::init( void )
 void
 JICLocal::config( void ) 
 { 
-		// nothing to look up just for JICLocal
+	// Nothing for us to do
 }
 
 
@@ -236,6 +236,11 @@ JICLocal::addToOutputFiles( const char* /*filename*/ )
 		// to the user, it's already going to be in the job's iwd...
 }
 
+void
+JICLocal::removeFromOutputFiles( const char* /*filename*/ )
+{
+		// there's no file transfer, so there is nothing to clean up.
+}
 
 bool
 JICLocal::registerStarterInfo( void )
@@ -337,7 +342,7 @@ bool
 JICLocal::initJobInfo( void ) 
 {
 		// Give our base class a chance.
-	JobInfoCommunicator::initJobInfo();
+	if (!JobInfoCommunicator::initJobInfo()) return false;
 
 	char *orig_job_iwd;
 
@@ -385,7 +390,7 @@ JICLocal::initJobInfo( void )
 		iwd_str += "=\"";
 		iwd_str += Starter->origCwd();
 		iwd_str += '"';
-		job_ad->InsertOrUpdate( iwd_str.GetCStr() );
+		job_ad->InsertOrUpdate( iwd_str.Value() );
 	}
 	job_ad->LookupString( ATTR_JOB_IWD, &job_iwd );
 	if( ! job_iwd ) {
@@ -505,7 +510,6 @@ JICLocal::checkUniverse( int univ )
 		return true;
 
 	case CONDOR_UNIVERSE_STANDARD:
-	case CONDOR_UNIVERSE_PVM:
 	case CONDOR_UNIVERSE_SCHEDULER:
 	case CONDOR_UNIVERSE_MPI:
 	case CONDOR_UNIVERSE_GRID:
