@@ -40,10 +40,21 @@
 #include "stream_handler.h"
 #include "condor_vm_universe_types.h"
 #include "authentication.h"
+#include "condor_mkstemp.h"
 
 extern CStarter *Starter;
 ReliSock *syscall_sock = NULL;
+extern const char* JOB_AD_FILENAME;
+extern const char* MACHINE_AD_FILENAME;
 
+// Filenames are case insensitive on Win32, but case sensitive on Unix
+#ifdef WIN32
+#	define file_contains contains_anycase
+#	define file_remove remove_anycase
+#else
+#	define file_contains contains
+#	define file_remove remove
+#endif
 
 JICShadow::JICShadow( const char* shadow_name ) : JobInfoCommunicator()
 {
@@ -55,9 +66,12 @@ JICShadow::JICShadow( const char* shadow_name ) : JobInfoCommunicator()
 	shadow = NULL;
 	shadow_version = NULL;
 	filetrans = NULL;
+	m_did_transfer = false;
 	m_filetrans_sec_session = NULL;
 	m_reconnect_sec_session = NULL;
 	
+		// just in case transferOutputMopUp gets called before transferOutput
+	m_ft_rval = true;
 	trust_uid_domain = false;
 	uid_domain = NULL;
 	fs_domain = NULL;
@@ -132,6 +146,11 @@ JICShadow::init( void )
 		return false;
 	}
 
+	if ( m_job_startd_update_sock )
+	{
+		receiveMachineAd(m_job_startd_update_sock);
+	}
+
 		// stash a copy of the unmodified job ad in case we decide
 		// below that we want to write out an execution visa
 	ClassAd orig_ad = *job_ad;	
@@ -351,15 +370,33 @@ JICShadow::Continue( void )
 }
 
 
-bool
-JICShadow::transferOutput( void )
+bool JICShadow::allJobsDone( void )
 {
-		// make sure we only do this step once.
-	static bool did_transfer = false;
-	if (did_transfer) {
+	bool r1, r2;
+	ClassAd update_ad;
+
+	r1 = JobInfoCommunicator::allJobsDone();
+
+	publishJobExitAd( &update_ad );
+	r2 = updateShadow( &update_ad, true );
+
+	return r1;
+}
+
+
+bool
+JICShadow::transferOutput( bool &transient_failure )
+{
+	dprintf(D_FULLDEBUG, "Inside JICShadow::transferOutput(void)\n");
+
+	transient_failure = false;
+
+	if (m_did_transfer) {
 		return true;
 	}
 
+	dprintf(D_FULLDEBUG, "JICShadow::transferOutput(void): Transferring...\n");
+
 		// transfer output files back if requested job really
 		// finished.  may as well do this in the foreground,
 		// since we do not want to be interrupted by anything
@@ -373,6 +410,18 @@ JICShadow::transferOutput( void )
 		while ((filename = m_added_output_files.next()) != NULL) {
 			filetrans->addOutputFile(filename);
 		}
+
+			// remove any dynamically-removed output files from
+			// the ft's list (i.e. a renamed Windows script)
+		m_removed_output_files.rewind();
+		while ((filename = m_removed_output_files.next()) != NULL) {
+			filetrans->addFileToExeptionList(filename);
+		}
+
+		// remove the job and machine classad files from the
+		// ft list
+		filetrans->addFileToExeptionList(JOB_AD_FILENAME);
+		filetrans->addFileToExeptionList(MACHINE_AD_FILENAME);
 	
 			// true if job exited on its own
 		bool final_transfer = (requested_exit == false);	
@@ -388,44 +437,51 @@ JICShadow::transferOutput( void )
 			// by the user, so set_user_priv here.
 		priv_state saved_priv = set_user_priv();
 
+		dprintf( D_FULLDEBUG, "Begin transfer of sandbox to shadow.\n");
 			// this will block
-		bool rval = filetrans->UploadFiles( true, final_transfer );
+		m_ft_rval = filetrans->UploadFiles( true, final_transfer );
+		m_ft_info = filetrans->GetInfo();
+		dprintf( D_FULLDEBUG, "End transfer of sandbox to shadow.\n");
 		set_priv(saved_priv);
 
-		if( ! rval ) {
-				// Failed to transfer.  See if there is a reason to put
-				// the job on hold.
-			FileTransfer::FileTransferInfo ft_info = filetrans->GetInfo();
-			if(!ft_info.success && !ft_info.try_again) {
-				ASSERT(ft_info.hold_code != 0);
-				notifyStarterError(ft_info.error_desc.Value(), true,
-				                   ft_info.hold_code,ft_info.hold_subcode);
-				return false;
-			}
+		if( m_ft_rval ) {
+			job_ad->Assign(ATTR_SPOOLED_OUTPUT_FILES, 
+							m_ft_info.spooled_files.Value());
+		} else {
+			dprintf( D_FULLDEBUG, "Sandbox transfer failed.\n");
+			// Failed to transfer.
+			// JICShadow::transferOutputMopUp() will figure out what to do
+			// when you call it after JICShadow::transferOutput() returns.
 
-				// Some other kind of error.  Would like to know
-				// for sure whether this really means we are disconnected
-				// from the shadow, but for now, force the socket to
-				// disconnect by closing it.
+			if( !m_ft_info.success && m_ft_info.try_again ) {
+				// Some kind of transient error, such as a timeout or
+				// disconnect.  Would like to know for sure whether
+				// this really means we are disconnected from the
+				// shadow, but for now, force the socket to disconnect
+				// by closing it.
 
 				// Please forgive us these hacks
 				// as we forgive those who hack against us
-			static int timesCalled = 0;
-			timesCalled++;
-			if (timesCalled < 5) {
+
+				static int timesCalled = 0;
+				timesCalled++;
+				if (timesCalled < 5) {
 					dprintf(D_ALWAYS,"File transfer failed, forcing disconnect.\n");
 
 					if (syscall_sock != NULL) {
-							syscall_sock->close();
+						syscall_sock->close();
 					}
 
+						// trigger retransfer on reconnect
 					job_cleanup_disconnected = true;
-					return false;
+
+						// inform our caller that transfer will be retried
+						// when the shadow reconnects
+					transient_failure = true;
+				}
 			}
 
-				// We tried 5 times and kept failing
-				// now just tell the user we're giving up
-			notifyStarterError("Repeated attempts to transfer output failed for unknown reasons", true,0,0);
+			m_did_transfer = false;
 			return false;
 		}
 	}
@@ -433,7 +489,44 @@ JICShadow::transferOutput( void )
 		// In both cases, we should record that we were successful so
 		// that if we ever come through here again to retry the whole
 		// job cleanup process we don't attempt to transfer again.
-	did_transfer = true;
+	m_did_transfer = true;
+	return true;
+}
+
+bool
+JICShadow::transferOutputMopUp(void)
+{
+
+	dprintf(D_FULLDEBUG, "Inside JICShadow::transferOutputMopUp(void)\n");
+
+	if (m_did_transfer) {
+		/* nothing was marked wrong if we were able to do the transfer */
+		return true;
+	}
+
+	// We saved the return value of the last filetransfer attempt...
+	// We also saved the ft_info structure when we did the file transfer.
+	if( ! m_ft_rval ) {
+		dprintf(D_FULLDEBUG, "JICShadow::transferOutputMopUp(void): "
+			"Mopping up failed transfer...\n");
+
+		// Failed to transfer.  See if there is a reason to put
+		// the job on hold.
+		if(!m_ft_info.success && !m_ft_info.try_again) {
+			ASSERT(m_ft_info.hold_code != 0);
+			// The shadow will immediately cut the connection to the
+			// starter when this is called. 
+			notifyStarterError(m_ft_info.error_desc.Value(), true,
+			                   m_ft_info.hold_code,m_ft_info.hold_subcode);
+			return false;
+		}
+
+		// We hit some "transient" error, but we've retried too many times,
+		// so tell the shadow we are giving up.
+		notifyStarterError("Repeated attempts to transfer output failed for unknown reasons", true,0,0);
+		return false;
+	}
+
 	return true;
 }
 
@@ -518,8 +611,8 @@ JICShadow::reconnect( ReliSock* s, ClassAd* ad )
 		// Destroy our old DCShadow object and make a new one with the
 		// current info.
 	dprintf( D_ALWAYS, "Accepted request to reconnect from <%s:%d>\n",
-			 syscall_sock->endpoint_ip_str(), 
-			 syscall_sock->endpoint_port() );
+			 syscall_sock->peer_ip_str(), 
+			 syscall_sock->peer_port() );
 	dprintf( D_ALWAYS, "Ignoring old shadow %s\n", shadow->addr() );
 	delete shadow;
 	shadow = new DCShadow;
@@ -543,14 +636,14 @@ JICShadow::reconnect( ReliSock* s, ClassAd* ad )
 
 		// switch over to the new syscall_sock
 	dprintf( D_FULLDEBUG, "Closing old syscall sock <%s:%d>\n",
-			 syscall_sock->endpoint_ip_str(), 
-			 syscall_sock->endpoint_port() );
+			 syscall_sock->peer_ip_str(), 
+			 syscall_sock->peer_port() );
 	delete syscall_sock;
 	syscall_sock = s;
 	syscall_sock->timeout(param_integer( "STARTER_UPLOAD_TIMEOUT", 300));
 	dprintf( D_FULLDEBUG, "Using new syscall sock <%s:%d>\n",
-			 syscall_sock->endpoint_ip_str(), 
-			 syscall_sock->endpoint_port() );
+			 syscall_sock->peer_ip_str(), 
+			 syscall_sock->peer_port() );
 
 	initMatchSecuritySession();
 
@@ -621,44 +714,15 @@ JICShadow::notifyJobPreSpawn( void )
 
 bool
 JICShadow::notifyJobExit( int exit_status, int reason, UserProc*
-						  user_proc )
+						  /* user_proc */ )
 {
 	static bool wrote_local_log_event = false;
-	bool job_exit_wants_ad = true;
-
-		// protocol changed w/ v6.3.0 so the Update Ad is sent
-		// with the final REMOTE_CONDOR_job_exit system call.
-		// to keep things backwards compatible, do not send the 
-		// ad with this system call if the shadow is older.
-
-		// However, b/c the shadow didn't start sending it's version
-		// to the starter until 6.3.2, we confuse 6.3.0 and 6.3.1
-		// shadows with 6.2.X shadows that don't support the new
-		// protocol.  Luckily, we never released 6.3.0 or 6.3.1 for
-		// windoze, and we never released any part of the new
-		// shadow/starter for Unix until 6.3.0.  So, we only have to
-		// do this compatibility check on windoze, and we don't have
-		// to worry about it not being able to tell the difference
-		// between 6.2.X, 6.3.0, and 6.3.1, since we never released
-		// 6.3.0 or 6.3.1. :) Derek <wright@cs.wisc.edu> 1/25/02
-
-#ifdef WIN32		
-	job_exit_wants_ad = false;
-
-
-	if( shadow_version && shadow_version->built_since_version(6,3,0) ) {
-
-		job_exit_wants_ad = true;	// new shadow; send ad
-
-	}
-#endif		
 
 	ClassAd ad;
-	ClassAd *ad_to_send;
 
 		// We want the update ad anyway, in case we want it for the
 		// LocalUserLog
-	user_proc->PublishUpdateAd( &ad );
+	publishUpdateAd( &ad );
 
 		// depending on the exit reason, we want a different event. 
 		// however, don't write multiple events if we've already been
@@ -672,23 +736,37 @@ JICShadow::notifyJobExit( int exit_status, int reason, UserProc*
 
 	updateStartd(&ad, true);
 
-	if ( job_exit_wants_ad ) {
-		ad_to_send = &ad;
-	} else {
-		dprintf( D_FULLDEBUG,
-				 "Shadow is pre-v6.3.0 - not sending final update ad\n" ); 
-		ad_to_send = NULL;
-	}
-			
-	if( REMOTE_CONDOR_job_exit(exit_status, reason, ad_to_send) < 0 ) {    
-		dprintf( D_ALWAYS, "Failed to send job exit status to shadow\n" );
-		job_cleanup_disconnected = true;
-		return false;
+	if( !had_hold ) {
+		if( REMOTE_CONDOR_job_exit(exit_status, reason, &ad) < 0 ) {    
+			dprintf( D_ALWAYS, "Failed to send job exit status to shadow\n" );
+			job_cleanup_disconnected = true;
+			return false;
+		}
 	}
 
 	return true;
 }
 
+int
+JICShadow::notifyJobTermination( UserProc *user_proc )
+{
+	int rval = 0;
+	ClassAd ad;
+
+	dprintf(D_FULLDEBUG, "Inside JICShadow::notifyJobTermination()\n");
+
+	if (shadow_version && shadow_version->built_since_version(7,4,4)) {
+		dprintf(D_ALWAYS, "JICShadow::notifyJobTermination(): "
+			"Sending mock terminate event.\n");
+
+		user_proc->PublishUpdateAd( &ad );
+
+		rval = REMOTE_CONDOR_job_termination(&ad);
+	}
+
+	return rval;
+}
+
 void
 JICShadow::updateStartd( ClassAd *ad, bool final_update )
 {
@@ -732,6 +810,12 @@ JICShadow::notifyStarterError( const char* err_msg, bool critical, int hold_reas
 	return true;
 }
 
+bool
+JICShadow::holdJob( const char* hold_reason, int hold_reason_code, int hold_reason_subcode )
+{
+	gotHold();
+	return notifyStarterError( hold_reason, true, hold_reason_code, hold_reason_subcode );
+}
 
 bool
 JICShadow::registerStarterInfo( void )
@@ -742,25 +826,10 @@ JICShadow::registerStarterInfo( void )
 		EXCEPT( "registerStarterInfo called with NULL DCShadow object" );
 	}
 
-		// CRUFT!
-		// If the shadow is older than 6.3.3, we need to use the
-		// CONDOR_register_machine_info method, which sends a bunch of
-		// strings over the wire.  If we're 6.3.3 or later, we can use
-		// CONDOR_register_starter_info, which just sends a ClassAd
-		// with all the relevent info.
-	if( shadow_version && shadow_version->built_since_version(6,3,3) ) {
-		ClassAd starter_info;
-		publishStarterInfo( &starter_info );
-		rval = REMOTE_CONDOR_register_starter_info( &starter_info );
+	ClassAd starter_info;
+	publishStarterInfo( &starter_info );
+	rval = REMOTE_CONDOR_register_starter_info( &starter_info );
 
-	} else {
-			// We've got to use the old method.
-		char *mfhn = strnewp ( my_full_hostname() );
-		rval = REMOTE_CONDOR_register_machine_info( uid_domain,
-			     fs_domain, daemonCore->InfoCommandSinfulString(), 
-				 mfhn, 0 );
-		delete [] mfhn;
-	}
 	if( rval < 0 ) {
 		return false;
 	}
@@ -799,12 +868,7 @@ JICShadow::publishStarterInfo( ClassAd* ad )
 	line += '"';
 	ad->Insert( line.Value() );
 
-	tmp_val = daemonCore->InfoCommandSinfulString();
-	size = strlen(tmp_val) + strlen(ATTR_STARTER_IP_ADDR) + 5;
-	tmp = (char*) malloc( size * sizeof(char) );
-	sprintf( tmp, "%s=\"%s\"", ATTR_STARTER_IP_ADDR, tmp_val );
-	ad->Insert( tmp );
-	free( tmp );
+	ad->Assign(ATTR_STARTER_IP_ADDR, daemonCore->InfoCommandSinfulString() );
 
 	tmp_val = param( "ARCH" );
 	size = strlen(tmp_val) + strlen(ATTR_ARCH) + 5;
@@ -845,7 +909,18 @@ JICShadow::publishStarterInfo( ClassAd* ad )
 void
 JICShadow::addToOutputFiles( const char* filename )
 {
-	m_added_output_files.append(filename);
+	if (!m_removed_output_files.file_contains(filename)) {
+		m_added_output_files.append(filename);
+	}
+}
+
+void
+JICShadow::removeFromOutputFiles( const char* filename )
+{
+	if (m_added_output_files.file_contains(filename)) {
+		m_added_output_files.file_remove(filename);
+	}
+	m_removed_output_files.append(filename);
 }
 
 bool
@@ -977,23 +1052,15 @@ JICShadow::initUserPriv( void )
 				setExecuteAccountIsDedicated( owner.Value() );
 			}
 		}
-		else if( stricmp(vm_univ_type.Value(), CONDOR_VM_UNIVERSE_VMWARE) == MATCH ) {
+		else if( strcasecmp(vm_univ_type.Value(), CONDOR_VM_UNIVERSE_VMWARE) == MATCH ) {
 			// For VMware vm universe, we can't use SOFT_UID_DOMAIN
 			run_as_owner = false;
 		}
 		else {
 				// There's a problem, maybe SOFT_UID_DOMAIN can help.
-			bool shadow_is_old = true;
 			bool try_soft_uid = param_boolean( "SOFT_UID_DOMAIN", false );
 
-			if( try_soft_uid ) {
-					// first, see if the shadow is new enough to
-					// support the RSC we need to do...
-				if( shadow_version && 
-					shadow_version->built_since_version(6,3,3) ) {
-						shadow_is_old = false;
-				}
-			} else {
+			if( !try_soft_uid ) {
 					// No soft_uid_domain or it's set to False.  No
 					// need to do the RSC, we can just fail.
 				dprintf( D_ALWAYS, "ERROR: Uid for \"%s\" not found in "
@@ -1002,24 +1069,10 @@ JICShadow::initUserPriv( void )
 				return false;
             }
 
-				// if the shadow is old, we have to just print an error
-				// message and fail, since we can't do the RSC we need
-				// to find out the right uid/gid.
-			if( shadow_is_old ) {
-				dprintf( D_ALWAYS, "ERROR: Uid for \"%s\" not found in "
-						 "passwd file, SOFT_UID_DOMAIN is True, but the "
-						 "condor_shadow on the submitting host is too old "
-						 "to support SOFT_UID_DOMAIN.  You must upgrade "
-						 "Condor on the submitting host to at least "
-						 "version 6.3.3.\n", owner.Value() ); 
-				return false;
-			}
-
 				// if we're here, it means that 1) the owner we want
-				// isn't in the passwd file, 2) SOFT_UID_DOMAIN is
-				// True, and 3) the shadow we're talking to can
-				// support the CONDOR_REMOTE_get_user_info RSC.  So,
-				// we'll do that call to get the uid/gid pair we need
+				// isn't in the passwd file, and 2) SOFT_UID_DOMAIN is
+				// True. So, we'll do a CONDOR_REMOTE_get_user_info RSC
+				// to get the uid/gid pair we need
 				// and initialize user priv with that. 
 
 			ClassAd user_info;
@@ -1028,10 +1081,8 @@ JICShadow::initUserPriv( void )
 						 "REMOTE_CONDOR_get_user_info() failed\n" );
 				dprintf( D_ALWAYS, "ERROR: Uid for \"%s\" not found in "
 						 "passwd file, SOFT_UID_DOMAIN is True, but the "
-						 "condor_shadow on the submitting host cannot "
-						 "support SOFT_UID_DOMAIN.  You must upgrade "
-						 "Condor on the submitting host to at least "
-						 "version 6.3.3.\n", owner.Value() );
+						 "condor_shadow failed to send the required Uid.\n",
+						 owner.Value() );
 				return false;
 			}
 
@@ -1070,7 +1121,8 @@ JICShadow::initUserPriv( void )
 
        // first see if we define SLOTx_USER in the config file
         char *nobody_user = NULL;
-        char paramer[20];
+			// 20 is the longest param: len(VM_UNIV_NOBODY_USER) + 1
+        char nobody_param[20];
 		int slot = Starter->getMySlotNumber();
 		if( ! slot ) {
 			slot = 1;
@@ -1079,33 +1131,33 @@ JICShadow::initUserPriv( void )
 		if( job_universe == CONDOR_UNIVERSE_VM ) {
 			// If "VM_UNIV_NOBODY_USER" is defined in Condor configuration file, 
 			// we will use it. 
-        	nobody_user = param("VM_UNIV_NOBODY_USER");
-			sprintf( paramer, "VM_UNIV_NOBODY_USER" );
+			snprintf( nobody_param, 20, "VM_UNIV_NOBODY_USER" );
+			nobody_user = param(nobody_param);
 			if( nobody_user == NULL ) {
 				// "VM_UNIV_NOBODY_USER" is NOT defined.
 				// Next, we will try to use SLOTx_VMUSER
-				sprintf( paramer, "SLOT%d_VMUSER", slot );
-				nobody_user = param(paramer);
+				snprintf( nobody_param, 20, "SLOT%d_VMUSER", slot );
+				nobody_user = param(nobody_param);
 			}
 		}
 		if( nobody_user == NULL ) {
-			sprintf( paramer, "SLOT%d_USER", slot );
-			nobody_user = param(paramer);
-			if (!nobody_user && param_boolean("ALLOW_VM_CRUFT", true)) {
-				sprintf( paramer, "VM%d_USER", slot );
-				nobody_user = param(paramer);
+			snprintf( nobody_param, 20, "SLOT%d_USER", slot );
+			nobody_user = param(nobody_param);
+			if (!nobody_user && param_boolean("ALLOW_VM_CRUFT", false)) {
+				snprintf( nobody_param, 20, "VM%d_USER", slot );
+				nobody_user = param(nobody_param);
 			}
 		}
 
         if ( nobody_user != NULL ) {
             if ( strcmp(nobody_user, "root") == MATCH ) {
                 dprintf(D_ALWAYS, "WARNING: %s set to root, which is not "
-                       "allowed. Ignoring.\n", paramer);
+                       "allowed. Ignoring.\n", nobody_param);
                 free(nobody_user);
                 nobody_user = strdup("nobody");
             } else {
                 dprintf(D_ALWAYS, "%s set, so running job as %s\n",
-                        paramer, nobody_user);
+                        nobody_param, nobody_user);
             }
         } else {
             nobody_user = strdup("nobody");
@@ -1151,7 +1203,7 @@ bool
 JICShadow::initJobInfo( void ) 
 {
 		// Give our base class a chance.
-	JobInfoCommunicator::initJobInfo();
+	if (!JobInfoCommunicator::initJobInfo()) return false;
 
 	char *orig_job_iwd;
 
@@ -1325,8 +1377,12 @@ JICShadow::initWithFileTransfer()
 					 "ad, aborting\n" );
 			return false;
 		}
+
+		char firstCharOfTF = tmp[0];
+		free(tmp);
+
 			// if set to "ALWAYS", then set transfer_at_vacate to true
-		switch ( tmp[0] ) {
+		switch ( firstCharOfTF ) {
 		case 'a':
 		case 'A':
 			transfer_at_vacate = true;
@@ -1336,7 +1392,6 @@ JICShadow::initWithFileTransfer()
 			return initNoFileTransfer();
 			break;
 		}
-		free( tmp );
 	}
 
 		// if we're here, it means we're transfering files, so we need
@@ -1453,7 +1508,7 @@ JICShadow::sameUidDomain( void )
 	dprintf( D_FULLDEBUG, " Local UidDomain: \"%s\"\n",
 			 uid_domain );
 
-	if( stricmp(job_uid_domain, uid_domain) == MATCH ) {
+	if( strcasecmp(job_uid_domain, uid_domain) == MATCH ) {
 		same_domain = true;
 	}
 
@@ -1520,7 +1575,7 @@ JICShadow::sameFSDomain( void )
 	dprintf( D_FULLDEBUG, " Local FsDomain: \"%s\"\n",
 			 fs_domain );
 
-	if( stricmp(job_fs_domain, fs_domain) == MATCH ) {
+	if( strcasecmp(job_fs_domain, fs_domain) == MATCH ) {
 		same_domain = true;
 	}
 
@@ -1546,7 +1601,7 @@ refuse(ReliSock * s)
 	s->encode();
 	int i = 0; // == failure;
 	s->code(i); // == failure
-	s->eom();
+	s->end_of_message();
 }
 
 // Based on Scheduler::updateGSICred
@@ -1559,12 +1614,38 @@ updateX509Proxy(int cmd, ReliSock * rsock, const char * path)
 	rsock->timeout(10);
 	rsock->decode();
 
-	dprintf(D_FULLDEBUG, "Remote side requests to update X509 proxy at %s\n", path);
+	dprintf(D_FULLDEBUG,
+	        "Remote side requests to update X509 proxy at %s\n",
+	        path);
 
-	MyString tmp_path(path);
-	tmp_path += ".tmp";
-
-	rsock->decode();
+	MyString tmp_path;
+#if defined(LINUX)
+	GLExecPrivSepHelper* gpsh = Starter->glexecPrivSepHelper();
+#else
+	// dummy for non-linux platforms.
+	int* gpsh = NULL;
+#endif
+	if (gpsh != NULL) {
+		// in glexec mode, we may not have permission to write the
+		// new proxy directly into the sandbox, so we stage it into
+		// /tmp first, then use a GLExec helper script
+		//
+		char tmp[] = "/tmp/condor_proxy_XXXXXX";
+		int fd = condor_mkstemp(tmp);
+		if (fd == -1) {
+			dprintf(D_ALWAYS,
+			        "updateX509Proxy: error creating temp file "
+			            "for proxy: %s\n",
+			        strerror(errno));
+			return 0;
+		}
+		close(fd);
+		tmp_path = tmp;
+	}
+	else {
+		tmp_path = path;
+		tmp_path += ".tmp";
+	}
 
 	priv_state old_priv = set_priv(PRIV_USER);
 
@@ -1576,22 +1657,43 @@ updateX509Proxy(int cmd, ReliSock * rsock, const char * path)
 	} else if ( cmd == DELEGATE_GSI_CRED_STARTER ) {
 		rc = rsock->get_x509_delegation(&size,tmp_path.Value());
 	} else {
-		dprintf( D_ALWAYS, "unknown CEDAR command %d in updateX509Proxy\n",
-				 cmd );
+		dprintf( D_ALWAYS,
+		         "unknown CEDAR command %d in updateX509Proxy\n",
+		         cmd );
 		rc = -1;
 	}
 	if ( rc < 0 ) {
 			// transfer failed
 		reply = 0; // == failure
 	} else {
-			// transfer worked, now rename the file to final_proxy_path
-		if ( rotate_file(tmp_path.Value(), path) < 0 ) 
-		{
-				// the rename failed!!?!?!
-			dprintf( D_ALWAYS, "updateX509Proxy failed, could not rename file\n");
-			reply = 0; // == failure
-		} else {
-			reply = 1; // == success
+		if (gpsh != NULL) {
+#if defined(LINUX)
+			// use our glexec helper object, which will
+			// call out to GLExec
+			//
+			if (gpsh->update_proxy(tmp_path.Value())) {
+				reply = 1;
+			}
+			else {
+				reply = 0;
+			}
+#else
+			EXCEPT("not on a linux platform and encounterd GLEXEC code!");
+#endif
+		}
+		else {
+				// transfer worked, now rename the file to
+				// final_proxy_path
+			if ( rotate_file(tmp_path.Value(), path) < 0 ) 
+			{
+					// the rename failed!!?!?!
+				dprintf( D_ALWAYS,
+				         "updateX509Proxy failed, "
+				             "could not rename file\n");
+				reply = 0; // == failure
+			} else {
+				reply = 1; // == success
+			}
 		}
 	}
 	set_priv(old_priv);
@@ -1599,12 +1701,14 @@ updateX509Proxy(int cmd, ReliSock * rsock, const char * path)
 		// Send our reply back to the client
 	rsock->encode();
 	rsock->code(reply);
-	rsock->eom();
+	rsock->end_of_message();
 
 	if(reply) {
-		dprintf(D_FULLDEBUG, "Attempt to refresh X509 proxy succeeded.\n");
+		dprintf(D_FULLDEBUG,
+		        "Attempt to refresh X509 proxy succeeded.\n");
 	} else {
-		dprintf(D_ALWAYS, "Attempt to refresh X509 proxy FAILED.\n");
+		dprintf(D_ALWAYS,
+		        "Attempt to refresh X509 proxy FAILED.\n");
 	}
 	
 	return reply;
@@ -1617,7 +1721,7 @@ JICShadow::updateX509Proxy(int cmd, ReliSock * s)
 		s->encode();
 		int i = 2; // == success, but please don't call any more.
 		s->code(i); // == success, but please don't call any more.
-		s->eom();
+		s->end_of_message();
 		refuse(s);
 		return false;
 	}
@@ -1646,7 +1750,22 @@ JICShadow::publishUpdateAd( ClassAd* ad )
 		execsz = starter_dir.GetDirectorySize();
 		sprintf( buf, "%s=%lu", ATTR_DISK_USAGE, (long unsigned)((execsz+1023)/1024) ); 
 		ad->InsertOrUpdate( buf );
+
 	}
+	MyString spooled_files;
+	if( job_ad->LookupString(ATTR_SPOOLED_OUTPUT_FILES,spooled_files) && spooled_files.Length() > 0 )
+	{
+		ad->Assign(ATTR_SPOOLED_OUTPUT_FILES,spooled_files);
+	}
+
+	// Insert the starter's address into the update ad, because all
+	// parties who subscribe to updates (shadow & startd) also should
+	// be informed of any changes in the starter's contact info
+	// (important for CCB and shadow-starter reconnect, because startd
+	// needs to relay starter's full contact info to the shadow when
+	// queried).  It's a bit of a hack to do it through this channel,
+	// but better than nothing.
+	ad->Assign( ATTR_STARTER_IP_ADDR, daemonCore->publicNetworkIpAddr() );
 
 		// Now, get our Starter object to publish, as well.  This will
 		// walk through all the UserProcs and have those publish, as
@@ -1656,6 +1775,46 @@ JICShadow::publishUpdateAd( ClassAd* ad )
 }
 
 
+bool
+JICShadow::publishJobExitAd( ClassAd* ad )
+{
+	filesize_t execsz = 0;
+	char buf[200];
+
+	// if there is a filetrans object, then let's send the current
+	// size of the starter execute directory back to the shadow.  this
+	// way the ATTR_DISK_USAGE will be updated, and we won't end
+	// up on a machine without enough local disk space.
+	if ( filetrans ) {
+		Directory starter_dir( Starter->GetWorkingDir(), PRIV_USER );
+		execsz = starter_dir.GetDirectorySize();
+		sprintf( buf, "%s=%lu", ATTR_DISK_USAGE, (long unsigned)((execsz+1023)/1024) ); 
+		ad->InsertOrUpdate( buf );
+
+	}
+	MyString spooled_files;
+	if( job_ad->LookupString(ATTR_SPOOLED_OUTPUT_FILES,spooled_files) && spooled_files.Length() > 0 )
+	{
+		ad->Assign(ATTR_SPOOLED_OUTPUT_FILES,spooled_files);
+	}
+
+	// Insert the starter's address into the update ad, because all
+	// parties who subscribe to updates (shadow & startd) also should
+	// be informed of any changes in the starter's contact info
+	// (important for CCB and shadow-starter reconnect, because startd
+	// needs to relay starter's full contact info to the shadow when
+	// queried).  It's a bit of a hack to do it through this channel,
+	// but better than nothing.
+	ad->Assign( ATTR_STARTER_IP_ADDR, daemonCore->publicNetworkIpAddr() );
+
+		// Now, get our Starter object to publish, as well.  This will
+		// walk through all the UserProcs and have those publish, as
+		// well.  It returns true if there was anything published,
+		// false if not.
+	return Starter->publishJobExitAd( ad );
+}
+
+
 bool
 JICShadow::periodicJobUpdate( ClassAd* update_ad, bool insure_update )
 {
@@ -1738,6 +1897,13 @@ JICShadow::beginFileTransfer( void )
 		}
 
 		filetrans = new FileTransfer();
+
+			// In the starter, we never want to use
+			// SpooledOutputFiles, because we are not reading the
+			// output from the spool.  We always want to use
+			// TransferOutputFiles instead.
+		job_ad->Delete(ATTR_SPOOLED_OUTPUT_FILES);
+
 		ASSERT( filetrans->Init(job_ad, false, PRIV_USER) );
 		filetrans->setSecuritySession(m_filetrans_sec_session);
 		filetrans->RegisterCallback(
@@ -1786,6 +1952,10 @@ JICShadow::transferCompleted( FileTransfer *ftrans )
 		if (job_ad->LookupString(ATTR_JOB_CMD, cmd) &&
 		    (cmd == CONDOR_EXEC))
 		{
+				// if we are running as root, the files were downloaded
+				// as PRIV_USER, so switch to that priv level to do chmod
+			priv_state saved_priv = set_priv( PRIV_USER );
+
 			if (chmod(CONDOR_EXEC, 0755) == -1) {
 				dprintf(D_ALWAYS,
 				        "warning: unable to chmod %s to "
@@ -1793,6 +1963,8 @@ JICShadow::transferCompleted( FileTransfer *ftrans )
 				        CONDOR_EXEC,
 				        strerror(errno));
 			}
+
+			set_priv( saved_priv );
 		}
 	}
 
@@ -1855,6 +2027,16 @@ JICShadow::initIOProxy( void )
 	int want_io_proxy = 0;
 	MyString io_proxy_config_file;
 
+		// the admin should have the final say over whether
+		// chirp is enabled
+    bool enableIOProxy = true;
+	enableIOProxy = param_boolean("ENABLE_CHIRP", true);
+	
+	if (!enableIOProxy) {
+		dprintf(D_ALWAYS, "ENABLE_CHIRP is false in config file, not enabling chirp\n");
+		return false;
+	}
+
 	if( job_ad->LookupBool( ATTR_WANT_IO_PROXY, want_io_proxy ) < 1 ) {
 		dprintf( D_FULLDEBUG, "JICShadow::initIOProxy(): "
 				 "Job does not define %s\n", ATTR_WANT_IO_PROXY );
@@ -1901,8 +2083,8 @@ JICShadow::initMatchSecuritySession()
 		// options here if we wanted to get an effect similar to
 		// security negotiation, but currently we just take the
 		// defaults from the shadow.
-	char *starter_reconnect_session_info = "";
-	char *starter_filetrans_session_info = "";
+	const char *starter_reconnect_session_info = "";
+	const char *starter_filetrans_session_info = "";
 
 	int rc = REMOTE_CONDOR_get_sec_session_info(
 		starter_reconnect_session_info,
@@ -1977,3 +2159,27 @@ JICShadow::initMatchSecuritySession()
 		}
 	}
 }
+
+bool
+JICShadow::receiveMachineAd( Stream *stream )
+{
+        bool ret_val = true;
+
+	dprintf(D_FULLDEBUG, "Entering JICShadow::receiveMachineAd\n");
+	mach_ad = new ClassAd();
+
+	stream->decode();
+	if (!mach_ad->initFromStream(*stream))
+	{
+		dprintf(D_ALWAYS, "Received invalid machine ad.  Discarding\n");
+		delete mach_ad;
+		mach_ad = NULL;
+		ret_val = false;
+	}
+	else
+	{
+		mach_ad->dPrint(D_JOB);
+	}
+
+	return ret_val;
+}
