@@ -87,11 +87,12 @@ public:
 		// Job Actions
 		// // // // // // // // // // // //
 
+	virtual bool holdJob( const char*, int hold_reason_code, int hold_reason_subcode );
+
 		/** 
 		 * JICShadow does not need to implement these 
 		 * at this time
 		 **/ 
-	virtual bool holdJob( const char*, int /*hold_reason_code*/, int /*hold_reason_subcode*/ ) { return ( false ); }
 	virtual bool removeJob( const char* ) { return ( false ); }
 	virtual bool terminateJob( const char* ) { return ( false ); }
 	virtual bool requeueJob( const char* ) { return ( false ); }
@@ -110,10 +111,29 @@ public:
 		*/
 	void Continue( void );
 
+		/** All the jobs are done. We need to send an update ad to the
+		 *  shadow before starting the transfer of output files.
+		 */
+	bool allJobsDone( void );
+
 		/** Once all the jobs are done, and after the optional
 			HOOK_JOB_EXIT has returned, initiate the file transfer.
+			If you call this, and transient_failure is not true, you
+			MUST use transferOutputMopUp() afterwards to handle
+			problems the file transfer may have had.
 		*/
-	bool transferOutput( void );
+	bool transferOutput( bool &transient_failure );
+
+		/** After transferOutput returns, we need to handle what happens
+			if the transfer actually failed. This call is separate from the
+			one above since it allows us to talk to the shadow in the time
+			between the failure of the transfer and the disconnection of the
+			shadow when the starter notifies it of the file transfer error.
+			This call will put the job on hold and cause the shadow to
+			disconnect from the starter if something went wrong. If 
+			the file transfer went right, then it is a noop.
+		*/
+	bool transferOutputMopUp( void );
 
 		/** The last job this starter is controlling has been
    			completely cleaned up.  We don't care, since we just wait
@@ -141,13 +161,31 @@ public:
 		 */
 	void notifyJobPreSpawn( void );
 
-		/** Notify the shadow that the job exited
+		/** Notify the shadow that the job exited. This will not only
+			update the job ad with the termination information of the job,
+			but also write terminate events into the job log, and do the
+			rest of the machinery for when a job finishes and is likely to
+			leave the queue.
+
 			@param exit_status The exit status from wait()
 			@param reason The Condor-defined exit reason
 			@param user_proc The UserProc that was running the job
 		*/
 	bool notifyJobExit( int exit_status, int reason,
 						UserProc* user_proc );
+	
+		/** The sutble distinction between notifyJobTermination() and
+			notifyJobExit() is that notifyJobTermination() will only update the
+			U_TERMINATE attributes in the job ad on the submit machine based
+			upon the exit_status and reason and do nothing else! An example
+			use is when a job coredumps and due to this causes the 
+			file transfer to fail since some files might not exist if 
+			directly specified in transfer_output_files. This allows the
+			job ad to contain the exit status of the job while it is on hold.
+			Basically this is used to preserve the termination of the job in
+			the face of Condor then messing stuff up terribly around it.
+		*/
+	int notifyJobTermination( UserProc* user_proc );
 
 		/** Tell the shadow to log an error event in the user log.
 			@param err_msg A desription of the error
@@ -185,7 +223,16 @@ public:
 		*/
 	void addToOutputFiles( const char* filename );
 
-		/** Send modified files in a job working direcotry 
+		/** Make sure the given filename will be excluded from the
+			list of files that the job sends back to the submitter.
+			If the file has already been added to the output list,
+			it is removed and added to the exception list; otherwise,
+			it is added to the exception list directly.
+			@param filename File to remove from the job's output list 
+		*/
+	void removeFromOutputFiles( const char* filename );
+
+		/** Send modified files in a job working directory 
 		    to shadow by using file transfer
 		*/
 	bool uploadWorkingFiles(void);
@@ -206,6 +253,8 @@ private:
 		*/ 
 	bool publishUpdateAd( ClassAd* ad );
 
+	bool publishJobExitAd( ClassAd* ad );
+
 		/** Send an update ClassAd to the shadow.  The "insure_update"
 			just means do we make sure the update gets there.  It has
 			nothing to do with the "insure" memory analysis tool.
@@ -234,6 +283,9 @@ private:
 		/// Do the RSC to get the job classad from the shadow
 	bool getJobAdFromShadow( void );
 
+		/// Get the machine classad from the given stream
+	bool receiveMachineAd( Stream *stream );
+
 		/** Initialize information about the shadow's version and
 			sinful string from the given ClassAd.  At startup, we just
 			pass the job ad, since that should have everything in it.
@@ -390,6 +442,10 @@ private:
 	IOProxy io_proxy;
 
 	FileTransfer *filetrans;
+	bool m_ft_rval;
+	FileTransfer::FileTransferInfo m_ft_info;
+	bool m_did_transfer;
+
 
 		// specially made security sessions if we are doing
 		// SEC_ENABLE_MATCH_PASSWORD_AUTHENTICATION
@@ -419,6 +475,11 @@ private:
 		    (e.g. a core file dumped by the job)
 		*/
 	StringList m_added_output_files;
+
+		/** A list of files that should NOT be transfered back to the
+			job submitter. (e.g. the job's executable itself)
+		*/
+	StringList m_removed_output_files;
 };
 
 
