@@ -39,6 +39,7 @@ extern CStarter *Starter;
 JobInfoCommunicator::JobInfoCommunicator()
 {
 	job_ad = NULL;
+	mach_ad = NULL;
 	job_universe = CONDOR_UNIVERSE_VANILLA;
 	job_cluster = -1;
 	job_proc = -1;
@@ -63,6 +64,7 @@ JobInfoCommunicator::JobInfoCommunicator()
 #endif
 	m_periodic_job_update_tid = -1;
 	m_allJobsDone_finished = false;
+	m_enforce_limits = false;
 }
 
 
@@ -71,6 +73,9 @@ JobInfoCommunicator::~JobInfoCommunicator()
 	if( job_ad ) {
 		delete job_ad;
 	}
+	if( mach_ad ) {
+		delete mach_ad;
+	}
 	if( u_log ) {
 		delete u_log;
 	}
@@ -215,6 +220,13 @@ JobInfoCommunicator::jobClassAd( void )
 }
 
 
+ClassAd*
+JobInfoCommunicator::machClassAd( void )
+{
+	return mach_ad;
+}
+
+
 int
 JobInfoCommunicator::jobUniverse( void )
 {
@@ -408,7 +420,7 @@ JobInfoCommunicator::initOutputAdFile( void )
 		path += DIR_DELIM_CHAR;
 		path += job_output_ad_file;
 		free( job_output_ad_file );
-		job_output_ad_file = strdup( path.GetCStr() );
+		job_output_ad_file = strdup( path.Value() );
 	}
 	dprintf( D_ALWAYS, "Will write job output ClassAd to \"%s\"\n",
 			 job_output_ad_is_stdout ? "STDOUT" : job_output_ad_file );
@@ -471,8 +483,8 @@ JobInfoCommunicator::allowRunAsOwner( bool default_allow, bool default_request )
 		// Eval as an expression so a policy such as this can be specified:
 		// TARGET.RunAsOwner =?= True
 
-	bool run_as_owner = param_boolean_expr( "STARTER_ALLOW_RUNAS_OWNER",
-	                                        default_allow, NULL, job_ad );
+	bool run_as_owner = param_boolean( "STARTER_ALLOW_RUNAS_OWNER",
+                                       default_allow, true, NULL, job_ad );
 
 		// Next check if the job has requested runas_owner
 	if( run_as_owner ) {
@@ -700,7 +712,7 @@ JobInfoCommunicator::initJobInfo( void )
 {
 #if HAVE_JOB_HOOKS
 	m_hook_mgr = new StarterHookMgr;
-	m_hook_mgr->initialize(job_ad);
+	return m_hook_mgr->initialize(job_ad);
 #endif
 	return true;
 }
@@ -857,7 +869,7 @@ JobInfoCommunicator::periodicJobUpdateTimerHandler( void )
 
 
 bool
-JobInfoCommunicator::periodicJobUpdate(ClassAd* update_ad, bool insure_update)
+JobInfoCommunicator::periodicJobUpdate(ClassAd* update_ad, bool)
 {
 #if HAVE_JOB_HOOKS
 	if (m_hook_mgr) {
