@@ -143,6 +143,9 @@ public:
 		/// Return a pointer to the ClassAd for our job.
 	virtual ClassAd* jobClassAd( void );
 
+		/// Return a pointer to the ClassAd for the machine.
+	virtual ClassAd* machClassAd( void );
+
 		/// Return the job's universe integer.
 	int jobUniverse( void );
 
@@ -207,8 +210,12 @@ public:
 			internal file transfer for the output.  This only makes
 			sense for JICShadow, but we need this step to be included
 			in all JICs so that the code path during cleanup is sane.
+			Returns false on failure and sets transient_failure to
+			true if the failure is deemed transient and will therefore
+			be automatically tried again (e.g. when the shadow reconnects).
 		*/
-	virtual bool transferOutput( void ) = 0;
+	virtual bool transferOutput( bool &transient_failure ) = 0;
+	virtual bool transferOutputMopUp( void ) = 0;
 
 		/** The last job this starter is controlling has been
 			completely cleaned up.  Do whatever final work we want to
@@ -256,6 +263,8 @@ public:
 	virtual bool notifyJobExit( int exit_status, int reason, 
 								UserProc* user_proc ) = 0;
 
+	virtual int notifyJobTermination( UserProc* user_proc ) = 0;
+
 	virtual bool notifyStarterError( const char* err_msg, bool critical, int hold_reason_code, int hold_reason_subcode ) = 0;
 
 
@@ -305,6 +314,12 @@ public:
 		*/
 	virtual void addToOutputFiles( const char* filename ) = 0;
 
+		/** Make sure the given filename will be excluded from the
+			list of files that the job sends back to the submitter.  
+			@param filename File to remove from the job's output list 
+		*/
+	virtual void removeFromOutputFiles( const char* filename ) = 0;
+
 		/// Has user_priv been initialized yet?
 	bool userPrivInitialized( void ); 
 
@@ -341,7 +356,6 @@ public:
 		/* Update Job ClassAd with checkpoint info and log it */
 	virtual void updateCkptInfo(void) {};
 
-
 protected:
 
 		// // // // // // // // // // // //
@@ -455,6 +469,9 @@ protected:
 		/// The ClassAd for our job.  We control the memory for this.
 	ClassAd* job_ad;
 
+		// The Machine ClassAd running the job.
+	ClassAd* mach_ad;
+
 		/// The universe of the job.
 	int job_universe;
 
@@ -481,6 +498,8 @@ protected:
 	StarterHookMgr* m_hook_mgr;
 #endif /* HAVE_JOB_HOOKS */
 
+	bool m_enforce_limits;
+
 private:
 		/// Start a timer for the periodic job updates
 	void startUpdateTimer( void );
