@@ -43,6 +43,10 @@
 #endif
 
 extern CStarter *Starter;
+extern const char* JOB_WRAPPER_FAILURE_FILE;
+
+extern const char* JOB_AD_FILENAME;
+extern const char* MACHINE_AD_FILENAME;
 
 
 /* OsProc class implementation */
@@ -136,22 +140,17 @@ OsProc::StartJob(FamilyInfo* family_info)
 		// to successfully merge them with the additional wrapper args.
 	args.SetArgV1SyntaxToCurrentPlatform();
 
-		// First, put "condor_exec" at the front of Args, since that
-		// will become argv[0] of what we exec(), either the wrapper
-		// or the actual job.
-		//
-		// The Java universe cannot tolerate an incorrect argv[0].
-		// For Java, set it correctly.  In a future version, we
-		// may consider removing the CONDOR_EXEC feature entirely.
-		//
-	if( (job_universe == CONDOR_UNIVERSE_JAVA) ) {
+		// First, put "condor_exec" or whatever at the front of Args,
+		// since that will become argv[0] of what we exec(), either
+		// the wrapper or the actual job.
+
+	if( !getArgv0() ) {
 		args.AppendArg(JobName.Value());
 	} else {
-		args.AppendArg(CONDOR_EXEC);
+		args.AppendArg(getArgv0());
 	}
-
+	
 		// Support USER_JOB_WRAPPER parameter...
-
 	char *wrapper = NULL;
 	if( (wrapper=param("USER_JOB_WRAPPER")) ) {
 
@@ -171,7 +170,31 @@ OsProc::StartJob(FamilyInfo* family_info)
 		args.AppendArg(JobName.Value());
 		JobName = wrapper;
 		free(wrapper);
-	} 
+	}
+	
+		// Support USE_PARROT 
+	bool use_parrot = false;
+	if( JobAd->LookupBool( ATTR_USE_PARROT, use_parrot) ) {
+			// Check for parrot executable
+		char *parrot = NULL;
+		if( (parrot=param("PARROT")) ) {
+			if( access(parrot,X_OK) < 0 ) {
+				dprintf( D_ALWAYS, "Unable to use parrot(Cannot find/execute "
+					"at %s(%s)).\n", parrot, strerror(errno) );
+				free( parrot );
+				return 0;
+			} else {
+				args.AppendArg(JobName.Value());
+				JobName = parrot;
+				free( parrot );
+			}
+		} else {
+			dprintf( D_ALWAYS, "Unable to use parrot(Undefined path in config"
+			" file)" );
+			return 0;
+		}
+	}
+
 		// Either way, we now have to add the user-specified args as
 		// the rest of the Args string.
 	MyString args_error;
@@ -189,28 +212,12 @@ OsProc::StartJob(FamilyInfo* family_info)
 		// environment as needed.
 	Env job_env;
 
-	char *env_str = param( "STARTER_JOB_ENVIRONMENT" );
-
 	MyString env_errors;
-	if( ! job_env.MergeFromV1RawOrV2Quoted(env_str,&env_errors) ) {
-		dprintf( D_ALWAYS, "Aborting OSProc::StartJob: "
-				 "%s\nThe full value for STARTER_JOB_ENVIRONMENT: "
-				 "%s\n",env_errors.Value(),env_str);
-		free(env_str);
+	if( !Starter->GetJobEnv(JobAd,&job_env,&env_errors) ) {
+		dprintf( D_ALWAYS, "Aborting OSProc::StartJob: %s\n",
+				 env_errors.Value());
 		return 0;
 	}
-	free(env_str);
-
-	if(!job_env.MergeFrom(JobAd,&env_errors)) {
-		dprintf( D_ALWAYS, "Invalid environment found in JobAd.  "
-		         "Aborting OsProc::StartJob: %s\n",
-		         env_errors.Value());
-		return 0;
-	}
-
-		// Now, let the starter publish any env vars it wants to into
-		// the mainjob's env...
-	Starter->PublishToEnv( &job_env );
 
 
 		// // // // // // 
@@ -294,7 +301,9 @@ OsProc::StartJob(FamilyInfo* family_info)
 		// Misc + Exec
 		// // // // // // 
 
-	Starter->jic->notifyJobPreSpawn();
+	if( !ThisProcRunsAlongsideMainProc() ) {
+		Starter->jic->notifyJobPreSpawn();
+	}
 
 	// compute job's renice value by evaluating the machine's
 	// JOB_RENICE_INCREMENT in the context of the job ad...
@@ -344,18 +353,40 @@ OsProc::StartJob(FamilyInfo* family_info)
 		// is sometimes condor_exec, in the Args string. 
 
 	MyString args_string;
-	int start_arg = has_wrapper ? 2 : 1;
-	args.GetArgsStringForDisplay(&args_string,start_arg);
+	args.GetArgsStringForDisplay(&args_string, 1);
 	if( has_wrapper ) { 
 			// print out exactly what we're doing so folks can debug
 			// it, if they need to.
 		dprintf( D_ALWAYS, "Using wrapper %s to exec %s\n", JobName.Value(), 
 				 args_string.Value() );
+
+		MyString wrapper_err;
+		wrapper_err.sprintf("%s%c%s", Starter->GetWorkingDir(),
+				 	DIR_DELIM_CHAR,
+					JOB_WRAPPER_FAILURE_FILE);
+		if( ! job_env.SetEnv("_CONDOR_WRAPPER_ERROR_FILE", wrapper_err.Value()) ) {
+			dprintf( D_ALWAYS, "Failed to set _CONDOR_WRAPPER_ERROR_FILE environment variable\n");
+		}
 	} else {
 		dprintf( D_ALWAYS, "About to exec %s %s\n", JobName.Value(),
 				 args_string.Value() );
 	}
 
+	MyString path;
+	path.sprintf("%s%c%s", Starter->GetWorkingDir(),
+			 	DIR_DELIM_CHAR,
+				MACHINE_AD_FILENAME);
+	if( ! job_env.SetEnv("_CONDOR_MACHINE_AD", path.Value()) ) {
+		dprintf( D_ALWAYS, "Failed to set _CONDOR_MACHINE_AD environment variable\n");
+	}
+
+	path.sprintf("%s%c%s", Starter->GetWorkingDir(),
+			 	DIR_DELIM_CHAR,
+				JOB_AD_FILENAME);
+	if( ! job_env.SetEnv("_CONDOR_JOB_AD", path.Value()) ) {
+		dprintf( D_ALWAYS, "Failed to set _CONDOR_JOB_AD environment variable\n");
+	}
+
 		// Grab the full environment back out of the Env object 
 	if(DebugFlags & D_FULLDEBUG) {
 		MyString env_string;
@@ -391,6 +422,8 @@ OsProc::StartJob(FamilyInfo* family_info)
 		core_size_ptr = &core_size;
 	}
 
+	int *affinity_mask = makeCpuAffinityMask(Starter->getMySlotNumber());
+
 #if defined ( WIN32 )
     owner_profile_.update ();
     /*************************************************************
@@ -420,6 +453,10 @@ OsProc::StartJob(FamilyInfo* family_info)
 
 	set_priv ( priv );
 
+    // use this to return more detailed and reliable error message info
+    // from create-process operation.
+    MyString create_process_err_msg;
+
 	if (privsep_helper != NULL) {
 		const char* std_file_names[3] = {
 			privsep_stdin_name.Value(),
@@ -436,7 +473,8 @@ OsProc::StartJob(FamilyInfo* family_info)
 		                                        core_size_ptr,
 		                                        1,
 		                                        job_opt_mask,
-		                                        family_info);
+		                                        family_info,
+												affinity_mask);
 	}
 	else {
 		JobPid = daemonCore->Create_Process( JobName.Value(),
@@ -453,14 +491,24 @@ OsProc::StartJob(FamilyInfo* family_info)
 		                                     nice_inc,
 		                                     NULL,
 		                                     job_opt_mask, 
-		                                     core_size_ptr );
+		                                     core_size_ptr,
+                                             affinity_mask,
+											 NULL,
+                                             &create_process_err_msg);
 	}
 
-	//NOTE: Create_Process() saves the errno for us if it is an
-	//"interesting" error.
-	char const *create_process_error = NULL;
+	// Create_Process() saves the errno for us if it is an "interesting" error.
 	int create_process_errno = errno;
-	if(JobPid == FALSE && errno) create_process_error = strerror(errno);
+	char const *create_process_error = NULL;
+
+    // errno is 0 in the privsep case.  This executes for the daemon core create-process logic
+    if ((FALSE == JobPid) && (0 != create_process_errno)) {
+        if (create_process_err_msg != "") create_process_err_msg += " ";
+        MyString errbuf;
+        errbuf.sprintf("(errno=%d: '%s')", create_process_errno, strerror(create_process_errno));
+        create_process_err_msg += errbuf;
+        create_process_error = create_process_err_msg.Value();       
+    }
 
 	// now close the descriptors in fds array.  our child has inherited
 	// them already, so we should close them so we do not leak descriptors.
@@ -500,14 +548,22 @@ OsProc::StartJob(FamilyInfo* family_info)
 			}
 			err_msg += ": ";
 			err_msg += create_process_error;
-			Starter->jic->notifyStarterError( err_msg.Value(),
-			                                  true,
-			                                  CONDOR_HOLD_CODE_FailedToCreateProcess,
-			                                  create_process_errno );
+			if( !ThisProcRunsAlongsideMainProc() ) {
+				Starter->jic->notifyStarterError( err_msg.Value(),
+			    	                              true,
+			        	                          CONDOR_HOLD_CODE_FailedToCreateProcess,
+			            	                      create_process_errno );
+			}
 		}
 
-		EXCEPT("Create_Process(%s,%s, ...) failed",
-			JobName.Value(), args_string.Value() );
+		if( !ThisProcRunsAlongsideMainProc() ) {
+			EXCEPT("Create_Process(%s,%s, ...) failed: %s",
+				JobName.Value(), args_string.Value(), (create_process_error ? create_process_error : ""));
+		}
+		else {
+			dprintf(D_ALWAYS,"Create_Process(%s,%s, ...) failed: %s",
+				JobName.Value(), args_string.Value(), (create_process_error ? create_process_error : ""));
+		}
 		return 0;
 	}
 
@@ -793,3 +849,54 @@ OsProc::PublishUpdateAd( ClassAd* ad )
 
 	return UserProc::PublishUpdateAd( ad );
 }
+
+int *
+OsProc::makeCpuAffinityMask(int slotId) {
+	bool useAffinity = param_boolean("ENFORCE_CPU_AFFINITY", false);
+
+	if (!useAffinity) {
+		dprintf(D_FULLDEBUG, "ENFORCE_CPU_AFFINITY not true, not setting affinity\n");	
+		return NULL;
+	}
+
+	// slotID of 0 means "no slot".  Punt.
+	if (slotId < 1) {
+		return NULL;
+	}
+
+	MyString affinityParam;
+
+	affinityParam.sprintf("SLOT%d_CPU_AFFINITY", slotId);
+	char *affinityParamResult = param(affinityParam.Value());
+
+	if (affinityParamResult == NULL) {
+		// No specific cpu, assume one-to-one mapping from slotid
+		// to cpu affinity
+
+		dprintf(D_FULLDEBUG, "Setting cpu affinity to %d\n", slotId - 1);	
+		int *mask = (int *) malloc(sizeof(int) * 2);
+		mask[0] = 2;
+		mask[1] = slotId - 1; // slots start at 1, cpus at 0
+		return mask;
+	}
+
+	StringList cpus(affinityParamResult);
+
+	if (cpus.number() < 1) {
+		dprintf(D_ALWAYS, "Could not parse affinity string %s, not setting affinity\n", affinityParamResult);
+		free(affinityParamResult);
+		return NULL;
+	}
+
+	int *mask = (int *) malloc(sizeof(int) * (cpus.number() + 1));
+	mask[0] = cpus.number() + 1;
+	cpus.rewind();
+	char *cpu;
+	int index = 1;
+	while ((cpu = cpus.next())) {
+		mask[index++] = atoi(cpu);
+	}
+
+	free(affinityParamResult);
+	return mask;
+}
