@@ -28,7 +28,7 @@
 #include "condor_privsep_helper.h"
 
 #if defined(LINUX)
-#include "glexec_privsep_helper.h"
+#include "glexec_privsep_helper.linux.h"
 #endif
 
 /** The starter class.  Basically, this class does some initialization
@@ -60,12 +60,25 @@ public:
 		*/
 	virtual void StarterExit( int code );
 
+		/** Do any potential cleanup before exiting. Used both in 
+		    successful exits (StarterExit()) and EXCEPT()ions.
+			Do not call this function lightly!
+
+			It is conceivable that this function will be called
+			several times if, say, an EXCEPT()ion happens while
+			shutting down, so be careful in the implementation.
+		*/
+	virtual void FinalCleanup();
+
 		/** Params for "EXECUTE" and other useful stuff 
 		 */
 	virtual void Config();
 
 	virtual int SpawnJob( void );
 
+	virtual void WriteRecoveryFile( ClassAd *recovery_ad );
+	virtual void RemoveRecoveryFile();
+
 		/*************************************************************
 		 * Starter Commands
 		 * We now have two versions of the old commands that the Starter
@@ -94,6 +107,7 @@ public:
 	virtual bool Remove( void );
 
 		/** Call Hold() on all elements in m_job_list */
+	int remoteHoldCommand( int cmd, Stream* s );
 	virtual int RemoteHold(int);
 	virtual bool Hold( void );
 
@@ -140,7 +154,7 @@ public:
 		 * 
 		 * 
 		 **/
-	virtual int SpawnPreScript( void );
+	virtual void SpawnPreScript( void );
 
 		/** Does initial cleanup once all the jobs (and post script, if
 			any) have completed.  This notifies the JIC so it can
@@ -206,6 +220,11 @@ public:
 			@param proc_env The environment to publish to
 		*/
 	void PublishToEnv( Env* proc_env );
+
+		/** Set up the complete environment for the job.  This includes
+			STARTER_JOB_ENVIRONMENT, the job ClassAd, and PublishToEnv()
+		*/
+	bool GetJobEnv( ClassAd *jobad, Env *job_env, MyString *env_errors );
 	
 		/** Pointer to our JobInfoCommuniator object, which abstracts
 			away any details about our communications with whatever
@@ -236,6 +255,13 @@ public:
 
 	int updateX509Proxy( int cmd, Stream* );
 
+	int createJobOwnerSecSession( int cmd, Stream* s );
+
+	int startSSHD( int /*cmd*/, Stream* s );
+	int SSHDFailed(Stream *s,char const *fmt,...) CHECK_PRINTF_FORMAT(3,4);
+	int SSHDRetry(Stream *s,char const *fmt,...) CHECK_PRINTF_FORMAT(3,4);
+	int vSSHDFailed(Stream *s,bool retry,char const *fmt,va_list args);
+
 		/** This will return NULL if we're not using either
 		    PrivSep or GLExec */
 	PrivSepHelper* privSepHelper()
@@ -287,7 +313,20 @@ private:
 		*/
 	bool publishJobInfoAd(List<UserProc>* proc_list, ClassAd* ad);
 
+		/*
+		  @param result Buffer in which to store fully-qualified user name of the job owner
+		  If no job owner can be found, substitute a suitable dummy user name.
+		 */
+	void getJobOwnerFQUOrDummy(MyString &result);
 
+		/*
+		  @param result Buffer in which to store claim id string from job.
+		  Returns false if no claim id could be found.
+		 */
+	bool getJobClaimId(MyString &result);
+
+
+	bool WriteAdFiles();
 		// // // // // // // //
 		// Private Data Members
 		// // // // // // // //
@@ -297,6 +336,7 @@ private:
 	char *Execute;
 	MyString WorkingDir; // The iwd given to the job
 	char *orig_cwd;
+	MyString m_recoveryFile;
 	bool is_gridshell;
 	int ShuttingDown;
 	int starter_stdin_fd;
@@ -329,6 +369,12 @@ private:
 		// Flag to indicate whether Config() has been run
 		//
 	bool m_configured;
+
+		// true if jobEnvironmentReady() has been called
+	bool m_job_environment_is_ready;
+
+		// true if allJobsDone() has been called
+	bool m_all_jobs_done;
 };
 
 #endif
