@@ -39,7 +39,7 @@
 #include "condor_getcwd.h"
 
 
-extern "C" int exception_cleanup(int,int,char*);	/* Our function called by EXCEPT */
+extern "C" int exception_cleanup(int,int,const char*);	/* Our function called by EXCEPT */
 JobInfoCommunicator* parseArgs( int argc, char* argv [] );
 
 static CStarter StarterObj;
@@ -49,7 +49,7 @@ extern int Foreground;	// from daemoncore
 static bool is_gridshell = false;
 
 // this appears at the bottom of this file:
-extern "C" int display_dprintf_header(FILE *fp);
+extern "C" int display_dprintf_header(char **buf,int *bufpos,int *buflen);
 static char* dprintf_header = NULL;
 
 int my_argc;
@@ -119,6 +119,12 @@ printClassAd( void )
 			str = 0;
 			gotone++;
 		}
+		if(ad->LookupString("JavaSpecificationVersion",&str)) {
+			printf("JavaSpecificationVersion = \"%s\"\n",str);
+			free(str);
+			str = 0;
+			gotone++;
+		}
 		if(ad->LookupFloat(ATTR_JAVA_MFLOPS,mflops)) {
 			printf("%s = %f\n", ATTR_JAVA_MFLOPS,mflops);
 			gotone++;
@@ -157,7 +163,7 @@ main_pre_dc_init( int argc, char* argv[] )
 	const char* base = condor_basename(argv[0]);
 	char const *tmp;
 	tmp = strrchr(base, '_' );
-	if( tmp && strincmp(tmp, "_gridshell", 10) == MATCH ) {
+	if( tmp && strncasecmp(tmp, "_gridshell", 10) == MATCH ) {
 		get_mySubSystem()->setName( "GRIDSHELL" );
 		is_gridshell = true;
 	} else { 
@@ -169,7 +175,7 @@ main_pre_dc_init( int argc, char* argv[] )
 					// confused with just "-" or something
 				continue;
 			}
-			if( strincmp(argv[i], "-gridshell", MIN(len,10)) == MATCH ) {
+			if( strncasecmp(argv[i], "-gridshell", MIN(len,10)) == MATCH ) {
 				get_mySubSystem()->setName( "GRIDSHELL" );
 				is_gridshell = true;
 				break;
@@ -184,7 +190,7 @@ main_pre_dc_init( int argc, char* argv[] )
 		// exit, without going back to daemoncore or anything.  we
 		// need to do this *after* we set get_mySubSystem(), since this ends
 		// up calling functions that rely on it being defined...  
-	if( argc == 2 && strincmp(argv[1],"-cla",4) == MATCH ) {
+	if( argc == 2 && strncasecmp(argv[1],"-cla",4) == MATCH ) {
 			// needed for Java stuff
 		config(true);
 
@@ -230,7 +236,7 @@ main_pre_dc_init( int argc, char* argv[] )
 }
 
 
-int
+void
 main_init(int argc, char *argv[])
 {
 	my_argc = argc;
@@ -271,8 +277,6 @@ main_init(int argc, char *argv[])
 		dprintf(D_ALWAYS, "Unable to start job.\n");
 		DC_Exit(1);
 	}
-
-	return 0;
 }
 
 
@@ -616,6 +620,11 @@ parseArgs( int argc, char* argv [] )
 		jic = new JICShadow( shadow_host );
 		free( shadow_host );
 		shadow_host = NULL;
+		free( schedd_addr );
+		free( job_output_ad );
+		free( job_stdin );
+		free( job_stdout );
+		free( job_stderr );
 		return jic;
 	}
 
@@ -678,15 +687,14 @@ parseArgs( int argc, char* argv [] )
 }
 
 
-int
-main_config( bool /*is_full*/ )
+void
+main_config()
 {
 	Starter->Config();
-	return 0;
 }
 
 
-int
+void
 main_shutdown_fast()
 {
 	if ( Starter->RemoteShutdownFast(0) ) {
@@ -694,11 +702,10 @@ main_shutdown_fast()
 		// no jobs to shutdown.  No need to stick around.
 		Starter->StarterExit(0);
 	}
-	return 0;
 }
 
 
-int
+void
 main_shutdown_graceful()
 {
 	if ( Starter->RemoteShutdownGraceful(0) ) {
@@ -706,15 +713,15 @@ main_shutdown_graceful()
 		// there are no jobs to shutdown.  No need to stick around.
 		Starter->StarterExit(0);
 	}
-	return 0;
 }
 
 extern "C" 
-int exception_cleanup(int,int,char*errmsg)
+int exception_cleanup(int,int,const char*errmsg)
 {
 	_EXCEPT_Cleanup = NULL;
 	Starter->jic->notifyStarterError(errmsg,true,0,0);
 	Starter->RemoteShutdownFast(0);
+	Starter->FinalCleanup();
 	return 0;
 }
 
@@ -726,10 +733,10 @@ main_pre_command_sock_init( )
 
 extern "C" 
 int
-display_dprintf_header(FILE *fp)
+display_dprintf_header(char **buf,int *bufpos,int *buflen)
 {
 	if( dprintf_header ) {
-		fprintf( fp, "%s ", dprintf_header );
+		return sprintf_realloc( buf, bufpos, buflen, "%s ", dprintf_header );
 	}
-	return TRUE;
+	return 0;
 }
