@@ -289,7 +289,7 @@ VMProc::StartJob()
 				CONDOR_HOLD_CODE_FailedToCreateProcess, 0);
 		return false;
 	}
-	vm_type_name.strlwr();
+	vm_type_name.lower_case();
 	m_vm_type = vm_type_name;
 
 	// get vm checkpoint flag from ClassAd
@@ -309,6 +309,17 @@ VMProc::StartJob()
 	}
 	*/
 
+	ClassAd recovery_ad = *JobAd;
+	MyString vm_name;
+	if ( strcasecmp( m_vm_type.Value(), CONDOR_VM_UNIVERSE_KVM ) == MATCH ||
+		 strcasecmp( m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN ) == MATCH ) {
+		ASSERT( create_name_for_VM( JobAd, vm_name ) );
+	} else {
+		vm_name = Starter->GetWorkingDir();
+	}
+	recovery_ad.Assign( "JobVMId", vm_name.Value() );
+	Starter->WriteRecoveryFile( &recovery_ad );
+
 	// //
 	// Now everything is ready to start a vmgahp server 
 	// //
@@ -668,7 +679,7 @@ VMProc::CheckStatus()
 		return;
 	}
 
-	 m_status_req->setNotificationTimerId(m_vmstatus_notify_tid);
+	m_status_req->setNotificationTimerId(m_vmstatus_notify_tid);
 
 	int p_result;
 	p_result = m_status_req->vmStatus(m_vm_id);
@@ -698,10 +709,8 @@ VMProc::CheckStatus()
 		if( m_vmstatus_notify_tid != -1 ) {
 			daemonCore->Reset_Timer(m_vmstatus_notify_tid, 0);
 		}else {
-			if(m_status_req) {
-				delete m_status_req;
-				m_status_req = NULL;
-			}
+			delete m_status_req;
+			m_status_req = NULL;
 		}
 		return;
 	}
@@ -1040,7 +1049,7 @@ VMProc::Ckpt()
 		return false;
 	}
 
-	if( strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN) == MATCH ) {
+	if( (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN) == MATCH) || (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_KVM) == MATCH) ) {
 		if( !m_is_vacate_ckpt ) {
 			// Xen doesn't support periodic checkpoint
 			return false;
@@ -1183,7 +1192,7 @@ VMProc::PublishUpdateAd( ClassAd* ad )
 	dprintf( D_FULLDEBUG, "Inside VMProc::PublishUpdateAd()\n" );
 
 	MyString buf;
-	if( strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN) == MATCH ) {
+	if( (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN) == MATCH) || (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_KVM) == MATCH) ) {
 		float sys_time = m_vm_cputime;
 		float user_time = 0.0;
 
@@ -1191,19 +1200,29 @@ VMProc::PublishUpdateAd( ClassAd* ad )
 		ad->Assign(ATTR_JOB_REMOTE_SYS_CPU, sys_time );
 		ad->Assign(ATTR_JOB_REMOTE_USER_CPU, user_time );
 		ad->Assign(ATTR_IMAGE_SIZE, (int)0);
+		ad->Assign(ATTR_RESIDENT_SET_SIZE, (int)0);
 	}else {
 		// Update usage of process for VM
 		long sys_time = 0;
 		long user_time = 0;
 		unsigned long max_image = 0;
+        unsigned long rss = 0;
 
-		getUsageOfVM(sys_time, user_time, max_image);
+		getUsageOfVM(sys_time, user_time, max_image, rss);
+		
+		// Added to update CPU Usage of VM in ESX
+		if ( long(m_vm_cputime) > user_time ) {
+			user_time = long(m_vm_cputime);
+		}
 
 		// Publish it into the ad.
-		ad->Assign(ATTR_JOB_REMOTE_SYS_CPU, (float)sys_time );
-		ad->Assign(ATTR_JOB_REMOTE_USER_CPU, (float)user_time );
+		ad->Assign(ATTR_JOB_REMOTE_SYS_CPU, float(sys_time));
+		ad->Assign(ATTR_JOB_REMOTE_USER_CPU, float(user_time));
+
 		buf.sprintf("%s=%lu", ATTR_IMAGE_SIZE, max_image );
 		ad->InsertOrUpdate( buf.Value());
+		buf.sprintf("%s=%lu", ATTR_RESIDENT_SET_SIZE, rss );
+		ad->InsertOrUpdate( buf.Value());
 	}
 
 	if( m_vm_checkpoint ) {
@@ -1276,7 +1295,7 @@ VMProc::reportErrorToStartd()
 
 	ssock.code(buffer);
 
-	if( !ssock.eom() ) {
+	if( !ssock.end_of_message() ) {
 		dprintf( D_FULLDEBUG, "Failed to send EOM to local startd %s\n", addr);
 		free(buffer);
 		return false;
@@ -1333,7 +1352,7 @@ VMProc::reportVMInfoToStartd(int cmd, const char *value)
 	ASSERT(vm_value);
 	ssock.code(vm_value);
 
-	if( !ssock.eom() ) {
+	if( !ssock.end_of_message() ) {
 		dprintf( D_FULLDEBUG, "Failed to send EOM to local startd %s\n", addr);
 		free(starter_pid);
 		free(vm_value);
@@ -1450,12 +1469,15 @@ VMProc::updateUsageOfVM()
 }
 
 void
-VMProc::getUsageOfVM(long &sys_time, long& user_time, unsigned long &max_image)
+VMProc::getUsageOfVM(long &sys_time, long& user_time, unsigned long &max_image, unsigned long& rss)
 {
 	updateUsageOfVM();
 	sys_time = m_vm_exited_pinfo.sys_time + m_vm_alive_pinfo.sys_time;
 	user_time = m_vm_exited_pinfo.user_time + m_vm_alive_pinfo.user_time;
 
+	rss = (m_vm_exited_pinfo.rssize > m_vm_alive_pinfo.rssize) ? 
+		   m_vm_exited_pinfo.rssize : m_vm_alive_pinfo.rssize;
+
 #if defined(WIN32)
 	max_image = (m_vm_exited_pinfo.rssize > m_vm_alive_pinfo.rssize) ? 
 		m_vm_exited_pinfo.rssize : m_vm_alive_pinfo.rssize;
