@@ -28,7 +28,7 @@
 #include "proto.h"
 #include "name_tab.h"
 
-#include "state_machine_driver.h"
+#include "state_machine_driver.unix.h"
 
 #include "starter.h"
 #include "fileno.h"
@@ -68,7 +68,7 @@ DECL_SUBSYSTEM( "STARTER", SUBSYSTEM_TYPE_STARTER );
 	// Constants
 const pid_t	ANY_PID = -1;		// arg to waitpid() for any process
 
-ReliSock	*SyscallStream = NULL;	// stream to shadow for remote system calls
+StdUnivSock	*SyscallStream = NULL;	// stream to shadow for remote system calls
 List<UserProc>	UProcList;		// List of user processes
 char	*Execute;				// Name of directory where user procs execute
 int		ExecTransferAttempts;	// How many attempts at getting the initial ckpt
@@ -121,7 +121,7 @@ int
 main( int argc, char *argv[] )
 {
 	myDistro->Init( argc, argv );
-	if( argc == 2 && strincmp(argv[1], "-cl", 3) == MATCH ) {
+	if( argc == 2 && strncasecmp(argv[1], "-cl", 3) == MATCH ) {
 		printClassAd();
 		exit( 0 );
 	}
@@ -1022,7 +1022,8 @@ get_job_info()
 			break;
 #endif
 		default:
-			u_proc = new UserProc( s );
+			if(s.cmd)
+				u_proc = new UserProc( s );
 			break;
 	}
 
@@ -1089,7 +1090,7 @@ cleanup()
 */
 extern "C" {
 int
-exception_cleanup(int,int,char*)
+exception_cleanup(int,int,const char*)
 {
 	sigset_t	mask;
 
@@ -1115,7 +1116,7 @@ exception_cleanup(int,int,char*)
 int	AvoidNFS = 0;
 
 
-#if defined(OSF1) || defined(HPUX)
+#if defined(HPUX)
 /*
   None of this stuff is ever used, but it must be here so that we
   can link with the remote system call library without getting
