@@ -98,7 +98,7 @@ extern sigset_t	ChildSigMask;
 extern NameTable SigNames;
 extern char *ThisHost;
 extern char *InitiatingHost;
-extern ReliSock	*SyscallStream;	// stream to shadow for remote system calls
+extern StdUnivSock	*SyscallStream;	// stream to shadow for remote system calls
 extern int EventSigs[];
 int UserProc::proc_index = 1;
 
@@ -258,6 +258,7 @@ UserProc::expand_exec_name( int &on_this_host )
 	free( m_a_out );
 	m_a_out = strdup( tmp );
 	FREE( tmp );
+	free(a_out);
 			
 
 	if( on_this_host ) {
@@ -621,7 +622,7 @@ UserProc::execute()
 			// the coresize to the requested amount. Otherwise, just
 			// use whatever the current default is.
 		if (coredump_limit_exists == TRUE) {
-			limit( RLIMIT_CORE, coredump_limit, CONDOR_HARD_LIMIT );
+			limit( RLIMIT_CORE, coredump_limit, CONDOR_HARD_LIMIT, "max core size" );
 		}
 
 			// child process should have only it's submitting uid, and cannot
@@ -939,20 +940,20 @@ UserProc::handle_termination( int exit_st )
 
 				/* this gets deleted when this objects destructs */
 				core_name = new char [ corebuf.Length() + 1 ];
-				strcpy( core_name, corebuf.Value() );
+				if(core_name != NULL) strcpy( core_name,corebuf.Value() );
 
 				/* core_is_valid checks to make sure it isn't a symlink and
 					returns false if it is */
-		    	if( core_is_valid(core_name) == TRUE) {
+					if( core_is_valid(corebuf.Value()) == TRUE) {
 					dprintf( D_FAILURE|D_ALWAYS, 
-						"A core file was created: %s\n", core_name );
+						"A core file was created: %s\n", corebuf.Value());
 					core_created = TRUE;
 				} else {
 					dprintf( D_FULLDEBUG, "No core file was created\n" );
 					core_created = FALSE;
 					if (core_name != NULL) {
 						// remove any incomplete core, or possible symlink
-						(void)unlink( core_name );	
+						(void)unlink(corebuf.Value());	
 					}
 				}
 			}
@@ -1331,7 +1332,7 @@ open_std_file( int fd )
 		MyString err;
 		err.sprintf("Can't open \"%s\": %s", file_name,strerror(errno));
 		dprintf(D_ALWAYS,"%s\n",err.Value());
-		REMOTE_CONDOR_report_error((char *)err.Value());
+		REMOTE_CONDOR_report_error(const_cast<char *>(err.Value()));
 		exit( 4 );
 	} else {
 		if( real_fd != fd ) {
@@ -1491,7 +1492,7 @@ set_iwd()
 		MyString err;
 		err.sprintf( "Can't open working directory \"%s\": %s", iwd,
 			    strerror(errno) );
-		REMOTE_CONDOR_report_error( (char *)err.Value() );
+		REMOTE_CONDOR_report_error(const_cast<char *>(err.Value()));
 		exit( 4 );
 	}
 	free( iwd );
