@@ -19,10 +19,12 @@
 
 #include "condor_common.h"
 #include "condor_api.h"
+#include "condor_adtypes.h"
 #include "status_types.h"
 #include "totals.h"
 #include "format_time.h"
 #include "condor_xml_classads.h"
+#include "string_list.h"
 
 extern ppOption				ppStyle;
 extern AttrListPrintMask 	pm;
@@ -38,17 +40,18 @@ static int stashed_now = 0;
 void printStartdNormal 	(ClassAd *);
 void printScheddNormal 	(ClassAd *);
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 void printQuillNormal 	(ClassAd *);
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 void printScheddSubmittors(ClassAd *);
 void printMasterNormal 	(ClassAd *);
 void printCollectorNormal (ClassAd *);
 void printCkptSrvrNormal(ClassAd *);
-void printStorageNormal(ClassAd *);
+void printStorageNormal (ClassAd *);
 void printNegotiatorNormal (ClassAd *);
-void printAnyNormal(ClassAd *);
+void printGridNormal 	(ClassAd *);
+void printAnyNormal 	(ClassAd *);
 void printServer 		(ClassAd *);
 void printRun    		(ClassAd *);
 void printCOD    		(ClassAd *);
@@ -72,6 +75,16 @@ prettyPrint (ClassAdList &adList, TrackTotals *totals)
 	adList.Open();
 	while ((ad = adList.Next())) {
 		if (!wantOnlyTotals) {
+				// CRUFT: Before 7.3.2, submitter ads had a MyType of
+				//   "Scheduler". The only way to tell the difference
+				//   was that submitter ads didn't have ATTR_NUM_USERS.
+				//   Coerce MyStype to "Submitter" for ads coming from
+				//   these older schedds. This used to be done inside
+				//   the ClassAd library.
+			if ( !strcmp( ad->GetMyTypeName(), SCHEDD_ADTYPE ) &&
+				 !ad->LookupExpr( ATTR_NUM_USERS ) ) {
+				ad->SetMyTypeName( SUBMITTER_ADTYPE );
+			}
 			switch (ppStyle) {
 			  case PP_STARTD_NORMAL:
 				printStartdNormal (ad);
@@ -93,11 +106,11 @@ prettyPrint (ClassAdList &adList, TrackTotals *totals)
 				printState( ad );
 				break;
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 			  case PP_QUILL_NORMAL:
 				printQuillNormal (ad);
 				break;
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 			  case PP_SCHEDD_NORMAL:
 				printScheddNormal (ad);
@@ -117,9 +130,9 @@ prettyPrint (ClassAdList &adList, TrackTotals *totals)
 				break;
 
 			  case PP_XML:
-				  printXML (ad, (classad_index == 0),
-							(classad_index == last_classad_index));
-				  break;
+				printXML (ad, (classad_index == 0),
+					(classad_index == last_classad_index));
+				break;
 
 			  case PP_MASTER_NORMAL:
 				printMasterNormal(ad);
@@ -137,6 +150,10 @@ prettyPrint (ClassAdList &adList, TrackTotals *totals)
 				printStorageNormal(ad);
 				break;
 
+			  case PP_GRID_NORMAL:
+				printGridNormal(ad);
+				break;
+
 			  case PP_GENERIC_NORMAL:
 			  case PP_GENERIC:
 			  case PP_ANY_NORMAL:
@@ -168,8 +185,6 @@ prettyPrint (ClassAdList &adList, TrackTotals *totals)
 		printXML (NULL, true, true);
 	}
 
-	printf ("\n");
-
 	// if totals are required, display totals
 	if (adList.MyLength() > 0 && totals) totals->displayTotals(stdout, 20);
 }
@@ -183,28 +198,28 @@ printStartdNormal (ClassAd *ad)
 	int    now;
 	int	   actvty;
 
-	char *opsys_attr, *arch_attr, *mem_attr;
-	char *opsys_name, *arch_name, *mem_name;
-
+	const char *opsys_attr, *arch_attr, *mem_attr;
+	const char *opsys_name, *arch_name, *mem_name;
+ 
 	mem_name = "Mem";
-	mem_attr = (char*) ATTR_MEMORY;
+	mem_attr = ATTR_MEMORY;
 
 	if(javaMode) {
-		opsys_name = opsys_attr = (char*) ATTR_JAVA_VENDOR;
+		opsys_name = opsys_attr = ATTR_JAVA_VENDOR;
 		arch_name = "Ver";
-		arch_attr = (char*) ATTR_JAVA_VERSION;	
+		arch_attr = ATTR_JAVA_VERSION;	
 	} else if(vmMode) {
 		// For vm universe, we will print ATTR_VM_MEMORY
 		// instead of ATTR_MEMORY
 		opsys_name = "VMType";
-		opsys_attr = (char*) ATTR_VM_TYPE;
-		arch_name = "Ver";
-		arch_attr = (char*) ATTR_VM_VERSION;	
+		opsys_attr = ATTR_VM_TYPE;
+		arch_name = "Network";
+		arch_attr = ATTR_VM_NETWORKING_TYPES;
 		mem_name = "VMMem";
-		mem_attr = (char*) ATTR_VM_MEMORY;
+		mem_attr = ATTR_VM_MEMORY;
 	} else {
-		opsys_name = opsys_attr = (char*) ATTR_OPSYS;
-		arch_name = arch_attr = (char*) ATTR_ARCH;
+		opsys_name = opsys_attr = ATTR_OPSYS;
+		arch_name = arch_attr = ATTR_ARCH;
 	}
 
 	if (ad)
@@ -224,20 +239,20 @@ printStartdNormal (ClassAd *ad)
 						ATTR_LOAD_AVG, mem_name, "ActvtyTime");
 			}
 		
-			alpm.registerFormat("%-18.18s ", ATTR_NAME, "[???????????] ");
+			alpm.registerFormat("%-18.18s ", ATTR_NAME, "[????????????????] ");
 
 			if( vmMode ) {
 				alpm.registerFormat("%-6.6s " , opsys_attr, "[????] ");
 				alpm.registerFormat("%-10.10s " , arch_attr, "[????????] ");
 			}else {
-				alpm.registerFormat("%-10.10s " , opsys_attr, "[?????????] ");
+				alpm.registerFormat("%-10.10s " , opsys_attr, "[????????] ");
 				alpm.registerFormat("%-6.6s " , arch_attr, "[????] ");
 			}
 
-			alpm.registerFormat("%-9.9s ",  ATTR_STATE), "[????????] ";
-			alpm.registerFormat("%-8.8s ",  ATTR_ACTIVITY, "[????????] ");
+			alpm.registerFormat("%-9.9s ",  ATTR_STATE, "[???????] ");
+			alpm.registerFormat("%-8.8s ",  ATTR_ACTIVITY, "[??????] ");
 			alpm.registerFormat("%.3f  ",  ATTR_LOAD_AVG, "[???]  ");
-			alpm.registerFormat("%4d",  mem_attr, "[??]  ");
+			alpm.registerFormat("%4d",  mem_attr, "[??]");
 
 			first = false;
 		}
@@ -252,19 +267,6 @@ printStartdNormal (ClassAd *ad)
 			printf("   [Unknown]");
 		}
 
-		if( vmMode ) {
-			bool vm_networking = false;
-			MyString networking_types;
-
-			ad->LookupBool(ATTR_VM_NETWORKING, vm_networking);
-			if( vm_networking && ( ad->LookupString( ATTR_VM_NETWORKING_TYPES,
-							networking_types) == 1 )) {
-				printf( " %s", networking_types.Value());
-			}else {
-				printf(" [Not-Supported]");
-			}
-		}
-
 		printf("\n");
 	}
 }
@@ -291,7 +293,7 @@ printServer (ClassAd *ad)
 			alpm.registerFormat("%-6.6s " , ATTR_ARCH, "[????] ");
 			alpm.registerFormat("%.3f  ",  ATTR_LOAD_AVG, "[???]  ");
 			alpm.registerFormat("%6d  ",  ATTR_MEMORY, "[????]  ");
-			alpm.registerFormat("%7d ",  ATTR_DISK, "[?????]");
+			alpm.registerFormat("%7d ",  ATTR_DISK, "[?????] ");
 			alpm.registerFormat("%10d ", ATTR_MIPS, "[????????] ");
 			alpm.registerFormat("%10d\n", ATTR_KFLOPS, "[????????]\n");
 
@@ -321,20 +323,20 @@ printState (ClassAd *ad)
 				ATTR_ACTIVITY, "ActvtyTime");
 		
 			alpm.registerFormat("%-10.10s ", ATTR_NAME, "[????????] ");
-			alpm.registerFormat("%3d " , ATTR_CPUS, "[??] ");
+			alpm.registerFormat("%3d " , ATTR_CPUS, "[?] ");
 			alpm.registerFormat("%5d " , ATTR_MEMORY, "[???] ");
-			alpm.registerFormat("%.3f ", ATTR_LOAD_AVG, "[???]  ");
+			alpm.registerFormat("%.3f ", ATTR_LOAD_AVG, "[???] ");
 			alpm.registerFormat( (IntCustomFmt) format_time,
 									ATTR_KEYBOARD_IDLE,
-									"[??????????] ");
-			alpm.registerFormat(" %-7.7s ",  ATTR_STATE, "[????????]  ");
+									"[??????????]");
+			alpm.registerFormat(" %-7.7s ",  ATTR_STATE, " [?????] ");
 			alpm.registerFormat( (IntCustomFmt) formatElapsedTime,
 									ATTR_ENTERED_CURRENT_STATE,
-									"[??????????] ");
-			alpm.registerFormat(" %-4.4s ",  ATTR_ACTIVITY, "[????????]  ");
+									"[??????????]");
+			alpm.registerFormat(" %-4.4s ",  ATTR_ACTIVITY, " [??] ");
 			alpm.registerFormat( (IntCustomFmt) formatElapsedTime,
 									ATTR_ENTERED_CURRENT_ACTIVITY,
-									"[??????????] ");
+									"[??????????]");
 			alpm.registerFormat("\n", "*bogus*", "\n");  // force newline
 
 			first = false;
@@ -350,21 +352,21 @@ printRun (ClassAd *ad)
 	static bool first = true;
 	static AttrListPrintMask alpm;
 
-	char *opsys_attr, *arch_attr;
-	char *opsys_name, *arch_name;
+	const char *opsys_attr, *arch_attr;
+	const char *opsys_name, *arch_name;
 
 	if(javaMode) {
-		opsys_name = opsys_attr = (char*) ATTR_JAVA_VENDOR;
+		opsys_name = opsys_attr = ATTR_JAVA_VENDOR;
 		arch_name = "Ver";
-		arch_attr = (char*) ATTR_JAVA_VERSION;	
+		arch_attr = ATTR_JAVA_VERSION;	
 	} else if(vmMode) {
 		opsys_name = "VMType";
-		opsys_attr = (char*) ATTR_VM_TYPE;
-		arch_name = "Ver";
-		arch_attr = (char*) ATTR_VM_VERSION;	
+		opsys_attr = ATTR_VM_TYPE;
+		arch_name = "Network";
+		arch_attr = ATTR_VM_NETWORKING_TYPES;
 	} else {
-		opsys_name = opsys_attr = (char*) ATTR_OPSYS;
-		arch_name = arch_attr = (char*) ATTR_ARCH;
+		opsys_name = opsys_attr = ATTR_OPSYS;
+		arch_name = arch_attr = ATTR_ARCH;
 	}
 
 	if (ad)
@@ -373,11 +375,11 @@ printRun (ClassAd *ad)
 		if (first)
 		{
 			if(vmMode) {
-				printf ("\n%-13.13s %-11.11s %-6.6s %-6.6s %-20.20s %-15.15s\n\n",
+				printf ("\n%-13.13s %-6.6s %-11.11s %-6.6s %-20.20s %-15.15s\n\n",
 						ATTR_NAME, opsys_name, arch_name, ATTR_LOAD_AVG,
 						ATTR_REMOTE_USER, ATTR_CLIENT_MACHINE);
 			}else {
-				printf ("\n%-13.13s %-6.6s %-11.11s %-6.6s %-20.20s %-15.15s\n\n",
+				printf ("\n%-13.13s %-11.11s %-6.6s %-6.6s %-20.20s %-15.15s\n\n",
 						ATTR_NAME, opsys_name, arch_name, ATTR_LOAD_AVG,
 						ATTR_REMOTE_USER, ATTR_CLIENT_MACHINE);
 			}
@@ -480,7 +482,7 @@ printCOD (ClassAd *ad)
 	}
 }
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 void
 printQuillNormal (ClassAd *ad) {
 	static bool first = true;
@@ -509,7 +511,7 @@ printQuillNormal (ClassAd *ad) {
 		alpm.display (stdout, ad);
 	}
 }
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 void
 printScheddNormal (ClassAd *ad)
@@ -637,8 +639,8 @@ printCkptSrvrNormal(ClassAd *ad)
 		{
 			printf ("\n%-20.20s %-9.9s %-11.11s\n\n", ATTR_NAME,
 					"AvailDisk", ATTR_SUBNET);
-			alpm.registerFormat("%-20.20s ", ATTR_NAME, "[??????????????????]");
-			alpm.registerFormat("%9d ", ATTR_DISK, "[?????]");
+			alpm.registerFormat("%-20.20s ", ATTR_NAME, "[??????????????????] ");
+			alpm.registerFormat("%9d ", ATTR_DISK, "[???????] ");
 			alpm.registerFormat("%-11s\n", ATTR_SUBNET, "[?????]\n");
 			first = false;
 		}
@@ -661,8 +663,8 @@ printStorageNormal(ClassAd *ad)
 		{
 			printf ("\n%-30.30s %-9.9s %-11.11s\n\n", ATTR_NAME,
 					"AvailDisk", ATTR_SUBNET);
-			alpm.registerFormat("%-30.30s ", ATTR_NAME, "[??????????????????]");
-			alpm.registerFormat("%9d ", ATTR_DISK, "[?????]");
+			alpm.registerFormat("%-30.30s ", ATTR_NAME, "[????????????????????????????] ");
+			alpm.registerFormat("%9d ", ATTR_DISK, "[???????] ");
 			alpm.registerFormat("%-11s\n", ATTR_SUBNET, "[?????]\n");
 			first = false;
 		}
@@ -671,6 +673,40 @@ printStorageNormal(ClassAd *ad)
 	}
 }
 
+void
+printGridNormal(ClassAd *ad)
+{
+    static bool first = true;
+	static AttrListPrintMask alpm;
+
+    if (ad)
+    {
+        // print header if necessary
+        if (first)
+        {
+            printf ("\n%-35.35s %-7.7s %-7.7s %-7.7s %-7.7s %-7.7s\n\n",
+				ATTR_NAME, "NumJobs", "Allowed", "Wanted", "Running", "Idle" );
+			
+			alpm.registerFormat("%-35.35s ", ATTR_NAME, 
+				"[?????????????????????????????????] " );
+			alpm.registerFormat ( "%-7d ", "NumJobs",
+				"[?????] " );
+			alpm.registerFormat ( "%-7d ", "SubmitsAllowed",
+				"[?????] " );
+			alpm.registerFormat ( "%-7d ", "SubmitsWanted",
+				"[?????] " );
+			alpm.registerFormat ( "%-7d ", ATTR_RUNNING_JOBS,
+				"[?????] " );
+			alpm.registerFormat ( "%-7d\n", ATTR_IDLE_JOBS,
+				"[?????]\n" );
+
+            first = false;
+        }
+        
+        alpm.display (stdout, ad);
+	}
+}
+
 void
 printNegotiatorNormal(ClassAd *ad)
 {
@@ -687,7 +723,7 @@ printNegotiatorNormal(ClassAd *ad)
 		
 			alpm.registerFormat("%-20.20s ", ATTR_NAME,
 													"[??????????????????] ");
-			alpm.registerFormat("%-20.20s ", ATTR_MACHINE,
+			alpm.registerFormat("%-20.20s\n", ATTR_MACHINE,
 													"[??????????????????] ");
 
 			first = false;
