@@ -59,6 +59,7 @@ bool            javaMode = false;
 bool			vmMode = false;
 char 		*target = NULL;
 ClassAd		*targetAd = NULL;
+ArgList projList;		// Attributes that we want the server to send us
 
 // instantiate templates
 
@@ -72,9 +73,9 @@ int  lessThanFunc(AttrList*,AttrList*,void*);
 int  customLessThanFunc(AttrList*,AttrList*,void*);
 
 extern "C" int SetSyscalls (int) {return 0;}
-extern	void setPPstyle (ppOption, int, char *);
-extern	void setType    (char *, int, char *);
-extern	void setMode 	(Mode, int, char *);
+extern	void setPPstyle (ppOption, int, const char *);
+extern	void setType    (const char *, int, const char *);
+extern	void setMode 	(Mode, int, const char *);
 
 int
 main (int argc, char *argv[])
@@ -110,11 +111,11 @@ main (int argc, char *argv[])
 	// can override it
 	switch (type)
 	{
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 	  case QUILL_AD:
 		setPPstyle(PP_QUILL_NORMAL, 0, DEFAULT);
 		break;
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 	  case STARTD_AD:
 		setPPstyle(PP_STARTD_NORMAL, 0, DEFAULT);
@@ -144,6 +145,10 @@ main (int argc, char *argv[])
 		setPPstyle(PP_NEGOTIATOR_NORMAL, 0, DEFAULT);
 		break;
 
+      case GRID_AD:
+        setPPstyle(PP_GRID_NORMAL, 0, DEFAULT);
+		break;
+
 	  case GENERIC_AD:
 		setPPstyle(PP_GENERIC, 0, DEFAULT);
 		break;
@@ -158,9 +163,9 @@ main (int argc, char *argv[])
 
 	// set the constraints implied by the mode
 	switch (mode) {
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 	  case MODE_QUILL_NORMAL:
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 	  case MODE_STARTD_NORMAL:
 	  case MODE_MASTER_NORMAL:
@@ -172,6 +177,8 @@ main (int argc, char *argv[])
 	  case MODE_STORAGE_NORMAL:
 	  case MODE_GENERIC_NORMAL:
 	  case MODE_ANY_NORMAL:
+	  case MODE_GRID_NORMAL:
+	  case MODE_HAD_NORMAL:
 		break;
 
 	  case MODE_OTHER:
@@ -183,7 +190,7 @@ main (int argc, char *argv[])
 
 	  case MODE_STARTD_AVAIL:
 			  // For now, -avail shows you machines avail to anyone.
-		sprintf (buffer, "TARGET.%s == \"%s\"", ATTR_STATE,
+		sprintf (buffer, "%s == \"%s\"", ATTR_STATE,
 					state_to_string(unclaimed_state));
 		if (diagnose) {
 			printf ("Adding constraint [%s]\n", buffer);
@@ -193,7 +200,7 @@ main (int argc, char *argv[])
 
 
 	  case MODE_STARTD_RUN:
-		sprintf (buffer, "TARGET.%s == \"%s\"", ATTR_STATE,
+		sprintf (buffer, "%s == \"%s\"", ATTR_STATE,
 					state_to_string(claimed_state));
 		if (diagnose) {
 			printf ("Adding constraint [%s]\n", buffer);
@@ -202,7 +209,7 @@ main (int argc, char *argv[])
 		break;
 
 	  case MODE_STARTD_COD:
-	    sprintf (buffer, "TARGET.%s > 0", ATTR_NUM_COD_CLAIMS );
+	    sprintf (buffer, "%s > 0", ATTR_NUM_COD_CLAIMS );
 		if (diagnose) {
 			printf ("Adding constraint [%s]\n", buffer);
 		}
@@ -214,21 +221,39 @@ main (int argc, char *argv[])
 	}	
 
 	if(javaMode) {
-		sprintf( buffer, "TARGET.%s == TRUE", ATTR_HAS_JAVA );
+		sprintf( buffer, "%s == TRUE", ATTR_HAS_JAVA );
 		if (diagnose) {
 			printf ("Adding constraint [%s]\n", buffer);
 		}
 		query->addANDConstraint (buffer);
+		
+		projList.AppendArg(ATTR_HAS_JAVA);
+		projList.AppendArg(ATTR_JAVA_MFLOPS);
+		projList.AppendArg(ATTR_JAVA_VENDOR);
+		projList.AppendArg(ATTR_JAVA_VERSION);
+
 	}
 
 	if(vmMode) {
-		sprintf( buffer, "TARGET.%s == TRUE", ATTR_HAS_VM);
+		sprintf( buffer, "%s == TRUE", ATTR_HAS_VM);
 		if (diagnose) {
 			printf ("Adding constraint [%s]\n", buffer);
 		}
 		query->addANDConstraint (buffer);
+
+		projList.AppendArg(ATTR_VM_TYPE);
+		projList.AppendArg(ATTR_VM_MEMORY);
+		projList.AppendArg(ATTR_VM_NETWORKING);
+		projList.AppendArg(ATTR_VM_NETWORKING_TYPES);
+		projList.AppendArg(ATTR_VM_HARDWARE_VT);
+		projList.AppendArg(ATTR_VM_AVAIL_NUM);
+		projList.AppendArg(ATTR_VM_ALL_GUEST_MACS);
+		projList.AppendArg(ATTR_VM_ALL_GUEST_IPS);
+		projList.AppendArg(ATTR_VM_GUEST_MAC);
+		projList.AppendArg(ATTR_VM_GUEST_IP);
+
 	}
-									
+
 	// second pass:  add regular parameters and constraints
 	if (diagnose) {
 		printf ("----------\n");
@@ -242,6 +267,38 @@ main (int argc, char *argv[])
 	// fetch the query
 	QueryResult q;
 
+	if ((mode == MODE_STARTD_NORMAL) && (ppStyle == PP_STARTD_NORMAL)) {
+		projList.AppendArg("Name");
+		projList.AppendArg("Machine");
+		projList.AppendArg("Opsys");
+		projList.AppendArg("Arch");
+		projList.AppendArg("State");
+		projList.AppendArg("Activity");
+		projList.AppendArg("LoadAvg");
+		projList.AppendArg("Memory");
+		projList.AppendArg("ActvtyTime");
+		projList.AppendArg("MyCurrentTime");
+		projList.AppendArg("EnteredCurrentActivity");
+	}
+
+	
+	
+	// Calculate the projected arguments, and insert into
+	// the projection query attribute
+
+	
+	MyString quotedProjStr;
+	MyString projStrError;
+
+	projList.GetArgsStringV2Quoted(&quotedProjStr, &projStrError);
+
+	MyString projStr("projection = ");
+	projStr += quotedProjStr;
+		// If it is empty, it's just quotes
+	if (quotedProjStr.Length() > 2) {
+		query->addExtraAttribute(projStr.Value());
+	}
+
 	// if diagnose was requested, just print the query ad
 	if (diagnose) {
 		ClassAd 	queryAd;
@@ -260,7 +317,14 @@ main (int argc, char *argv[])
 		exit (1);
 	}
 
-	char* addr = pool ? pool->addr() : NULL;
+        // Address (host:port) is taken from requested pool, if given.
+	char* addr = (NULL != pool) ? pool->addr() : NULL;
+        Daemon* requested_daemon = pool;
+
+        // If we're in "direct" mode, then we attempt to locate the daemon
+	// associated with the requested subsystem (here encoded by value of mode)
+        // In this case the host:port of pool (if given) denotes which
+        // pool is being consulted
 	if( direct ) {
 		Daemon *d = NULL;
 		switch( mode ) {
@@ -274,11 +338,11 @@ main (int argc, char *argv[])
 			d = new Daemon( DT_STARTD, direct, addr );
 			break;
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		case MODE_QUILL_NORMAL:
 			d = new Daemon( DT_QUILL, direct, addr );
 			break;
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 		case MODE_SCHEDD_NORMAL:
 		case MODE_SCHEDD_SUBMITTORS:
@@ -294,6 +358,8 @@ main (int argc, char *argv[])
 		case MODE_GENERIC_NORMAL:
 		case MODE_ANY_NORMAL:
 		case MODE_OTHER:
+		case MODE_GRID_NORMAL:
+		case MODE_HAD_NORMAL:
 				// These have to go to the collector, anyway.
 			break;
 		default:
@@ -301,41 +367,72 @@ main (int argc, char *argv[])
 			exit( 1 );
 			break;
 		}
-		if( d ) {
+
+                // Here is where we actually override 'addr', if we can obtain
+                // address of the requested daemon/subsys.  If it can't be
+                // located, then fail with error msg.
+                // 'd' will be null (unset) if mode is one of above that must go to
+                // collector (MODE_ANY_NORMAL, MODE_COLLECTOR_NORMAL, etc)
+		if (NULL != d) {
 			if( d->locate() ) {
 				addr = d->addr();
+				requested_daemon = d;
 			} else {
-				fprintf( stderr, "%s: %s\n", myName, d->error() );
+			        char* id = const_cast<char*>(d->idStr());
+                                if (NULL == id) id = const_cast<char*>(d->name());
+				if (NULL == id) id = "daemon";
+           	                fprintf(stderr, "Error: Failed to locate %s\n", id);
+                                fprintf(stderr, "%s\n", d->error());
 				exit( 1 );
 			}
 		}
 	}
 
-	CondorError errstack;	
-	if (addr) {
+	CondorError errstack;
+	if (NULL != addr) {
+	        // this case executes if pool was provided, or if in "direct" mode with
+	        // subsystem that corresponds to a daemon (above).
+                // Here 'addr' represents either the host:port of requested pool, or
+                // alternatively the host:port of daemon associated with requested subsystem (direct mode)
 		q = query->fetchAds (result, addr, &errstack);
 	} else {
+                // otherwise obtain list of collectors and submit query that way
 		CollectorList * collectors = CollectorList::create();
 		q = collectors->query (*query, result, &errstack);
 		delete collectors;
 	}
 		
 
-	if (q != Q_OK) {
-		if (q == Q_COMMUNICATION_ERROR) {
-			fprintf( stderr, "%s\n", errstack.getFullText(true) );
-				// if we're not an expert, we want verbose output
-			if (errstack.code(0) == CEDAR_ERR_CONNECT_FAILED) {
-				printNoCollectorContact( stderr, addr, !expert );
-			}
+	// if any error was encountered during the query, report it and exit 
+        if (Q_OK != q) {
+                // we can always provide these messages:
+	        fprintf( stderr, "Error: %s\n", getStrQueryResult(q) );
+		fprintf( stderr, "%s\n", errstack.getFullText(true) );
+
+	        if ((NULL != requested_daemon) && ((Q_NO_COLLECTOR_HOST == q) || (requested_daemon->type() == DT_COLLECTOR))) {
+                        // Specific long message if connection to collector failed.
+		        char* fullhost = requested_daemon->fullHostname();
+                        if (NULL == fullhost) fullhost = "<unknown_host>";
+                        char* daddr = requested_daemon->addr();
+                        if (NULL == daddr) daddr = "<unknown>";
+                        char info[1000];
+                        sprintf(info, "%s (%s)", fullhost, daddr);
+		        printNoCollectorContact( stderr, info, !expert );                        
+	        } else if ((NULL != requested_daemon) && (Q_COMMUNICATION_ERROR == q)) {
+                        // more helpful message for failure to connect to some daemon/subsys
+			char* id = const_cast<char*>(requested_daemon->idStr());
+                        if (NULL == id) id = const_cast<char*>(requested_daemon->name());
+			if (NULL == id) id = "daemon";
+                        char* daddr = requested_daemon->addr();
+                        if (NULL == daddr) daddr = "<unknown>";
+           	        fprintf(stderr, "Error: Failed to contact %s at %s\n", id, daddr);
 		}
-		else {
-			fprintf (stderr, "Error:  Could not fetch ads --- %s\n",
-					 getStrQueryResult(q));
-		}
-		exit (1);
+
+                // fail
+                exit (1);
 	}
 
+
 	// sort the ad
 	if( sortLessThanExprs.getlast() > -1 ) {
 		result.Sort((SortFunctionType) customLessThanFunc );
@@ -343,9 +440,11 @@ main (int argc, char *argv[])
 		result.Sort ((SortFunctionType)lessThanFunc);
 	}
 
+	
 	// output result
 	prettyPrint (result, &totals);
 	
+
 	// be nice ...
 	{
 		int last = sortLessThanExprs.getlast();
@@ -373,7 +472,7 @@ usage ()
 		"\t-ckptsrvr\t\tDisplay checkpoint server attributes\n"
 		"\t-claimed\t\tPrint information about claimed resources\n"
 		"\t-cod\t\t\tDisplay Computing On Demand (COD) jobs\n"
-//		"\t-collector\t\tSame as -world\n"
+		"\t-collector\t\tDisplay collector daemon attributes\n"
 		"\t-debug\t\t\tDisplay debugging info to console\n"
 		"\t-direct <host>\t\tGet attributes directly from the given daemon\n"
 		"\t-java\t\t\tDisplay Java-capable hosts\n"
@@ -381,15 +480,16 @@ usage ()
 		"\t-license\t\tDisplay attributes of licenses\n"
 		"\t-master\t\t\tDisplay daemon master attributes\n"
 		"\t-pool <name>\t\tGet information from collector <name>\n"
+        "\t-grid\t\t\tDisplay grid resources\n"
 		"\t-run\t\t\tSame as -claimed [deprecated]\n"
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		"\t-quill\t\t\tDisplay attributes of quills\n"
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 		"\t-schedd\t\t\tDisplay attributes of schedds\n"
 		"\t-server\t\t\tDisplay important attributes of resources\n"
 		"\t-startd\t\t\tDisplay resource attributes\n"
-		"\t-generic\t\t\tDisplay attributes of 'generic' ads\n"
-		"\t-subsystem <type>\t\tDisplay classads of the given type\n"
+		"\t-generic\t\tDisplay attributes of 'generic' ads\n"
+		"\t-subsystem <type>\tDisplay classads of the given type\n"
 		"\t-negotiator\t\tDisplay negotiator attributes\n"
 		"\t-storage\t\tDisplay network storage resources\n"
 		"\t-any\t\t\tDisplay any resources\n"
@@ -402,6 +502,7 @@ usage ()
 		"\t-total\t\t\tDisplay totals only\n"
 		"\t-verbose\t\tSame as -long\n"
 		"\t-xml\t\t\tDisplay entire classads, but in XML\n"
+		"\t-attributes X,Y,...\tAttributes to show in -xml or -long \n"
 		"\t-expert\t\t\tDisplay shorter error messages\n"
 		"    and [custom-opts ...] are one or more of\n"
 		"\t-constraint <const>\tAdd constraint on classads\n"
@@ -481,6 +582,7 @@ firstPass (int argc, char *argv[])
 			FILE *targetFile = safe_fopen_wrapper(target, "r");
 			int iseof, iserror, empty;
 			targetAd = new ClassAd(targetFile, "\n\n", iseof, iserror, empty);
+			fclose(targetFile);
 		} else
 		if (matchPrefix (argv[i], "-constraint", 4)) {
 			// can add constraints on second pass only
@@ -524,6 +626,9 @@ firstPass (int argc, char *argv[])
 		if (matchPrefix (argv[i],"-xml", 2)){
 			setPPstyle (PP_XML, i, argv[i]);
 		} else
+		if (matchPrefix (argv[i],"-attributes", 3)){
+			// we don't do anything right here ... see prefix check in secondPass
+		} else	
 		if (matchPrefix (argv[i], "-run", 2) || matchPrefix(argv[i], "-claimed", 3)) {
 			setMode (MODE_STARTD_RUN, i, argv[i]);
 		} else
@@ -548,6 +653,9 @@ firstPass (int argc, char *argv[])
 		if (matchPrefix (argv[i], "-schedd", 3)) {
 			setMode (MODE_SCHEDD_NORMAL, i, argv[i]);
 		} else
+		if (matchPrefix (argv[i], "-grid", 2)) {
+			setMode (MODE_GRID_NORMAL, i, argv[i]);
+		} else
 		if (matchPrefix (argv[i], "-subsystem", 5)) {
 			i++;
 			if( !argv[i] ) {
@@ -577,6 +685,9 @@ firstPass (int argc, char *argv[])
 			if (matchPrefix (argv[i], "generic", 7)) {
 				setMode (MODE_GENERIC_NORMAL, i, argv[i]);
 			} else
+			if (matchPrefix (argv[i], "had", 3)) {
+				setMode (MODE_HAD_NORMAL, i, argv[i]);
+			} else
 			if (*argv[i] == '-') {
 				fprintf(stderr, "%s: -subsystem requires another argument\n",
 						myName);
@@ -587,11 +698,11 @@ firstPass (int argc, char *argv[])
 				setMode (MODE_OTHER, i, argv[i]);
 			}
 		} else
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		if (matchPrefix (argv[i], "-quill", 2)) {
 			setMode (MODE_QUILL_NORMAL, i, argv[i]);
 		} else
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 		if (matchPrefix (argv[i], "-license", 3)) {
 			setMode (MODE_LICENSE_NORMAL, i, argv[i]);
 		} else
@@ -619,13 +730,13 @@ firstPass (int argc, char *argv[])
 			ExprTree	*sortExpr;
 			exprString[0] = '\0';
 			sprintf( exprString, "MY.%s < TARGET.%s", argv[i], argv[i] );
-			if( Parse( exprString, sortExpr ) ) {
+			if( ParseClassAdRvalExpr( exprString, sortExpr ) ) {
 				fprintf( stderr, "Error:  Parse error of: %s\n", exprString );
 				exit( 1 );
 			}
 			sortLessThanExprs[sortLessThanExprs.getlast()+1] = sortExpr;
 			sprintf( exprString, "MY.%s == TARGET.%s", argv[i], argv[i] );
-			if( Parse( exprString, sortExpr ) ) {
+			if( ParseClassAdRvalExpr( exprString, sortExpr ) ) {
 				fprintf( stderr, "Error:  Parse error of: %s\n", exprString );
 				exit( 1 );
 			}
@@ -694,6 +805,7 @@ secondPass (int argc, char *argv[])
 		}
 		if (matchPrefix (argv[i], "-format", 2)) {
 			pm.registerFormat (argv[i+1], argv[i+2]);
+			projList.AppendArg(argv[i+2]);
 			if (diagnose) {
 				printf ("Arg %d --- register format [%s] for [%s]\n",
 						i, argv[i+1], argv[i+2]);
@@ -707,10 +819,23 @@ secondPass (int argc, char *argv[])
 		}
 		if( matchPrefix(argv[i], "-sort", 3) ) {
 			i++;
-			sprintf( buffer, "TARGET.%s =!= UNDEFINED", argv[i] );
+			sprintf( buffer, "%s =!= UNDEFINED", argv[i] );
 			query->addANDConstraint( buffer );
 			continue;
 		}
+		
+		if (matchPrefix (argv[i], "-attributes", 3) ) {
+			// parse attributes to be selected and split them along ","
+			StringList more_attrs(argv[i+1],",");
+			char const *s;
+			more_attrs.rewind();
+			while( (s=more_attrs.next()) ) {
+				projList.AppendArg(s);
+			}
+			i += 2;
+			continue;
+		}
+		
 
 
 		// figure out what the other parameters should do
@@ -739,9 +864,9 @@ secondPass (int argc, char *argv[])
 			switch (mode) {
 			  case MODE_STARTD_NORMAL:
 			  case MODE_STARTD_COD:
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 			  case MODE_QUILL_NORMAL:
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 			  case MODE_SCHEDD_NORMAL:
 			  case MODE_SCHEDD_SUBMITTORS:
 			  case MODE_MASTER_NORMAL:
@@ -753,7 +878,9 @@ secondPass (int argc, char *argv[])
 			  case MODE_GENERIC_NORMAL:
 			  case MODE_STARTD_AVAIL:
 			  case MODE_OTHER:
-				sprintf(buffer,"(TARGET.%s==\"%s\") || (TARGET.%s==\"%s\")",
+			  case MODE_GRID_NORMAL:
+			  case MODE_HAD_NORMAL:
+			  	sprintf(buffer,"(%s==\"%s\") || (%s==\"%s\")",
 						ATTR_NAME, daemonname, ATTR_MACHINE, daemonname );
 				if (diagnose) {
 					printf ("[%s]\n", buffer);
@@ -762,7 +889,7 @@ secondPass (int argc, char *argv[])
 				break;
 
 			  case MODE_STARTD_RUN:
-				sprintf (buffer,"TARGET.%s == \"%s\"",ATTR_REMOTE_USER,argv[i]);
+				sprintf (buffer,"%s == \"%s\"",ATTR_REMOTE_USER,argv[i]);
 				if (diagnose) {
 					printf ("[%s]\n", buffer);
 				}
@@ -850,19 +977,20 @@ customLessThanFunc( AttrList *ad1, AttrList *ad2, void *)
 	int			last = sortLessThanExprs.getlast();
 
 	for( int i = 0 ; i <= last ; i++ ) {
-		sortLessThanExprs[i]->EvalTree( ad1, ad2, &lt_result );
-		if( lt_result.type == LX_INTEGER ) {
+		if(EvalExprTree( sortLessThanExprs[i], ad1, ad2, &lt_result)
+			&& lt_result.type == LX_INTEGER ) {
 			if( lt_result.i ) {
 				return 1;
 			} else {
-				sortEqualExprs[i]->EvalTree( ad1, ad2, &lt_result );
-				if( lt_result.type != LX_INTEGER || !lt_result.i )
+				if(EvalExprTree( sortEqualExprs[i], ad1,
+					ad2, &lt_result ) &&
+				(( lt_result.type != LX_INTEGER || !lt_result.i ))){
 					return 0;
+				}
 			}
 		} else {
 			return 0;
 		}
 	}
-
 	return 0;
 }
