@@ -78,7 +78,7 @@ static int is_number(const char *str);
 
 /* we must now use the kstat interface to get this information under later
 	releases of Solaris */
-#if defined(Solaris28) || defined(Solaris29)
+#if defined(Solaris28) || defined(Solaris29) || defined(Solaris10) || defined(Solaris11)
 static time_t solaris_kbd_idle(void);
 static time_t solaris_mouse_idle(void);
 #endif
@@ -94,42 +94,8 @@ static time_t solaris_mouse_idle(void);
 void
 calc_idle_time_cpp( time_t * user_idle, time_t * console_idle)
 {
-	LASTINPUTINFO lii;
-	static POINT previous_pos = { 0, 0 };   // stored position of cursor
-	static DWORD previous_input_tick = 0;
     time_t now = time( 0 );
 	
-	
-	lii.cbSize = sizeof(LASTINPUTINFO);
-	lii.dwTime = 0;
-	
-	if ( !GetLastInputInfo(&lii) ) {
-		dprintf(D_ALWAYS, "calc_idle_time: GetLastInputInfo()"
-			" failed with err=%d\n", GetLastError());
-	}
-	
-	if ( lii.dwTime != previous_input_tick ) {
-		_sysapi_last_x_event = now;
-		previous_input_tick = lii.dwTime;
-	} else {
-		// no keypress detected, test if mouse moved
-		CURSORINFO cursor_inf;
-		cursor_inf.cbSize = sizeof(CURSORINFO);
-		if ( ! GetCursorInfo(&cursor_inf) ) {
-			dprintf(D_ALWAYS,"GetCursorInfo() failed (err=%li)\n",
-				GetLastError());
-		} else {
-			if ( (cursor_inf.ptScreenPos.x != previous_pos.x) || 
-				(cursor_inf.ptScreenPos.y != previous_pos.y) ) {
-				// the mouse has moved!
-				// stash new position
-				previous_pos.x = cursor_inf.ptScreenPos.x; 
-				previous_pos.y = cursor_inf.ptScreenPos.y; 
-				_sysapi_last_x_event = now;
-			}
-		}
-	}
-	
 	*user_idle = now - _sysapi_last_x_event;
 	*console_idle = *user_idle;
 	
@@ -198,10 +164,13 @@ calc_idle_time_cpp( time_t & m_idle, time_t & m_console_idle )
 	m_idle = MIN( now - _sysapi_last_x_event, m_idle );
 
 		// If last_x_event != 0, then condor_kbdd told us someone did
-		// something on the console, but always believe a /dev device
-		// over the kbdd, so if console_idle is set, leave it alone.
-	if( (m_console_idle == -1) && (_sysapi_last_x_event != 0) ) {
-		m_console_idle = now - _sysapi_last_x_event;
+		// something on the console.
+	if ( _sysapi_last_x_event ) {
+		if( m_console_idle != -1 ) {
+			m_console_idle = MIN( now - _sysapi_last_x_event, m_console_idle );
+		}else {
+			m_console_idle = now - _sysapi_last_x_event;
+		}
 	}
 
 #if defined(LINUX)
@@ -236,16 +205,13 @@ calc_idle_time_cpp( time_t & m_idle, time_t & m_console_idle )
 #include <utmp.h>
 #define UTMP_KIND utmp
 
-#if defined(OSF1)
-static char *UtmpName = "/var/adm/utmp";
-static char *AltUtmpName = "/etc/utmp";
-#elif defined(LINUX)
+#if defined(LINUX)
 static char *UtmpName = "/var/run/utmp";
 static char *AltUtmpName = "/var/adm/utmp";
 #elif defined(CONDOR_FREEBSD)
 static char *UtmpName = "/var/run/utmp";
 static char *AltUtmpName = "";
-#elif defined(Solaris28) || defined(Solaris29)
+#elif defined(Solaris28) || defined(Solaris29) || defined(Solaris10) || defined(Solaris11)
 #include <utmpx.h>
 static char *UtmpName = "/etc/utmpx";
 static char *AltUtmpName = "/var/adm/utmpx";
@@ -297,7 +263,7 @@ utmp_pty_idle_time( time_t now )
 	}
 
 	while (fread((char *)&utmp_info, sizeof(struct UTMP_KIND), 1, fp)) {
-#if defined(AIX) || defined(LINUX) || defined(OSF1) || defined(IRIX65)
+#if defined(AIX) || defined(LINUX)
 		if (utmp_info.ut_type != USER_PROCESS)
 #else
 			if (utmp_info.ut_name[0] == '\0')
@@ -435,7 +401,7 @@ all_pty_idle_time( time_t now )
 
 #ifdef LINUX
 #include <sys/sysmacros.h>  /* needed for major() below */
-#elif defined( OSF1 ) || defined(Darwin) || defined(CONDOR_FREEBSD)
+#elif defined(Darwin) || defined(CONDOR_FREEBSD)
 #include <sys/types.h>
 #elif defined( HPUX )
 #include <sys/sysmacros.h>
@@ -451,9 +417,9 @@ dev_idle_time( const char *path, time_t now )
 {
 	struct stat	buf;
 	time_t answer;
-	#if defined(Solaris28) || defined(Solaris29)
+#  if defined(Solaris28) || defined(Solaris29) || defined(Solaris10) || defined(Solaris11)
 	time_t kstat_answer;
-	#endif
+#  endif
 	static char pathname[100] = "/dev/";
 	static int null_major_device = -1;
 
@@ -499,7 +465,7 @@ dev_idle_time( const char *path, time_t now )
 		In this case, buf.st_atime would already be set to 0 above.
 	*/
 	if ( buf.st_atime != 0 && null_major_device > -1 &&
-							null_major_device == major(buf.st_rdev) ) {
+							null_major_device == int(major(buf.st_rdev))) {
 		// this device is related to /dev/null, it should not count
 		buf.st_atime = 0;
 	}
@@ -515,7 +481,7 @@ dev_idle_time( const char *path, time_t now )
 		solaris 8 update 6+ or solaris 9. stat() will return 0
 		for the access time no matter what. */
 
-	#if defined(Solaris28) || defined(Solaris29)
+#  if defined(Solaris28) || defined(Solaris29) || defined(Solaris10) || defined(Solaris11)
 	
 		/* if I happen not to be dealing with a console device, we better 
 			choose what the stat() answer would have been, so initialize this
@@ -537,7 +503,7 @@ dev_idle_time( const char *path, time_t now )
 		then, well, someone is using it */
 	answer = MAX(answer, kstat_answer);
 
-	#endif
+#  endif
 
 	if( (DebugFlags & D_FULLDEBUG) && (DebugFlags & D_IDLE) ) {
         dprintf( D_IDLE, "%s: %d secs\n", pathname, (int)answer );
@@ -595,30 +561,30 @@ get_keyboard_info(idle_t *fill_me)
 
 	fgets(buf, BUFFER_SIZE, intr_fs);  /* Ignore header line */
 	while (!result && (fgets(buf, BUFFER_SIZE, intr_fs) != NULL)) {
-	    if (strstr(buf, "i8042") != NULL || strstr(buf, "keyboard") != NULL){
+		if (strstr(buf, "i8042") != NULL || strstr(buf, "keyboard") != NULL){
 
 			if( (DebugFlags & D_IDLE) && (DebugFlags & D_FULLDEBUG) ) {
 				dprintf( D_IDLE, "Keyboard IRQ: %d\n", atoi(buf) );
 			}
-		tok = strtok_r(buf, DELIMS, &tok_loc);  /* Ignore [IRQ #]: */
-		do {
-		    tok = strtok_r(NULL, DELIMS, &tok_loc);
-		    if (is_number(tok)) {
-					/* It is ok if this overflows */
-				fill_me->num_key_intr += strtoul(tok, NULL, 10);
-				if( (DebugFlags & D_IDLE) && (DebugFlags & D_FULLDEBUG) ) {
-					dprintf( D_FULLDEBUG, 
-							 "Add %lu keyboard interrupts.  Total: %lu\n",
-							 strtoul(tok, NULL, 10), fill_me->num_key_intr );
-				}
-		    } else {
-			break;  /* device type column */
-		    }
-		} while (tok != NULL);		
-		result = TRUE;
-	    }
+			tok = strtok_r(buf, DELIMS, &tok_loc);  /* Ignore [IRQ #]: */
+			if(tok != NULL)
+				do {
+					tok = strtok_r(NULL, DELIMS, &tok_loc);
+					if (tok && is_number(tok)) {
+						/* It is ok if this overflows */
+						fill_me->num_key_intr += strtoul(tok, NULL, 10);
+						if( (DebugFlags & D_IDLE) && (DebugFlags & D_FULLDEBUG) ) {
+							dprintf( D_FULLDEBUG, 
+									"Add %lu keyboard interrupts.  Total: %lu\n",
+									strtoul(tok, NULL, 10), fill_me->num_key_intr );
+						}
+					} else {
+						break;  /* device type column */
+					}
+				} while (tok != NULL);		
+			result = TRUE;
+		}
 	}
-
 	fclose(intr_fs);	
 	return result;
 }
@@ -928,7 +894,7 @@ extract_idle_time(
 	actual kbd or mouse device no longer returns(by design according to 
 	solaris) the access time. These are localized functions static to this
 	object file. */
-#if defined(Solaris28) || defined(Solaris29)
+#if defined(Solaris28) || defined(Solaris29) || defined(Solaris10) || defined(Solaris11)
 static time_t solaris_kbd_idle(void)
 {
 	kstat_ctl_t     *kc = NULL;  /* libkstat cookie */ 
@@ -1152,10 +1118,9 @@ static time_t solaris_mouse_idle(void)
  out of this file. This will get a little ugly. The sysapi entry points
  are C linkage */
 
-BEGIN_C_DECLS
+extern "C" {
 
-void
-sysapi_idle_time_raw(time_t *m_idle, time_t *m_console_idle)
+void sysapi_idle_time_raw(time_t *m_idle, time_t *m_console_idle)
 {
 
 
@@ -1174,12 +1139,11 @@ sysapi_idle_time_raw(time_t *m_idle, time_t *m_console_idle)
 #endif
 }
 
-void
-sysapi_idle_time(time_t *m_idle, time_t *m_console_idle)
+void sysapi_idle_time(time_t *m_idle, time_t *m_console_idle)
 {
 	sysapi_internal_reconfig();
 
 	sysapi_idle_time_raw(m_idle, m_console_idle);
 }
 
-END_C_DECLS
+}
