@@ -41,6 +41,7 @@ usage( char *cmd )
 	fprintf(stderr,"    -pool <hostname>  Use this central manager\n");
 	fprintf(stderr,"    -debug            Show extra debugging info\n");
 	fprintf(stderr,"    -tcp              Ship classad via TCP (default is UDP)\n");
+	fprintf(stderr,"    -multiple         Publish multiple ads, separated by blank lines\n");
 	fprintf(stderr,"\nExample: %s -debug UPDATE_STORAGE_AD adfile\n\n",cmd);
 }
 
@@ -57,6 +58,8 @@ int main( int argc, char *argv[] )
 	int command=-1;
 	int i;
 	bool use_tcp = false;
+	bool with_ack = false;
+	bool allow_multiple = false;
 
 
 	myDistro->Init( argc, argv );
@@ -76,6 +79,11 @@ int main( int argc, char *argv[] )
 			pool = argv[i];
 		} else if(!strncmp(argv[i],"-tcp",strlen(argv[i]))) {
 			use_tcp = true;
+		} else if(!strncmp(argv[i],"-multiple",strlen(argv[i]))) {
+				// We don't set allow_multiple=true by default, because
+				// existing users (e.g. glideinWMS) have stray blank lines
+				// in the input file.
+			allow_multiple = true;
 		} else if(!strcmp(argv[i],"-version")) {
 			version();
 			exit(0);
@@ -106,10 +114,19 @@ int main( int argc, char *argv[] )
 	}
 
 	FILE *file;
-	ClassAd *ad;
+	ClassAdList ads;
 	Daemon *collector;
 	Sock *sock;
-	int eof,error,empty;
+
+	switch( command ) {
+	case UPDATE_STARTD_AD_WITH_ACK:
+		with_ack = true;
+		break;
+	}
+
+	if( with_ack ) {
+		use_tcp =  true;
+	}
 
 	if(!filename || !strcmp(filename,"-")) {
 		file = stdin;
@@ -122,30 +139,35 @@ int main( int argc, char *argv[] )
 		return 1;
 	}
 
-	ad = new ClassAd(file,"***",eof,error,empty);
-	if(error) {
-		fprintf(stderr,"couldn't parse ClassAd in %s\n",filename);
-		delete ad;
-		return 1;
+	while(!feof(file)) {
+		int eof=0,error=0,empty=0;
+		char const *delim = "\n";
+		if( !allow_multiple ) {
+			delim = "***";
+		}
+		ClassAd *ad = new ClassAd(file,const_cast<char *>(delim),eof,error,empty);
+		if(error) {
+			fprintf(stderr,"couldn't parse ClassAd in %s\n",filename);
+			delete ad;
+			return 1;
+		}
+		if( empty ) {
+			delete ad;
+			break;
+		}
+		if( !allow_multiple && ads.Length() > 0 ) {
+			fprintf(stderr,"ERROR: failed to parse '%s' as a ClassAd attribute\n",delim);
+			delete ad;
+			return 1;
+		}
+		ads.Insert(ad);
 	}
 
-	if(empty) {
+	if(ads.Length() == 0) {
 		fprintf(stderr,"%s is empty\n",filename);
-		delete ad;
 		return 1;
 	}
 
-	// If there's no "MyAddress", generate one..
-	ExprTree *tree = ad->Lookup( ATTR_MY_ADDRESS );
-	MyString tmp = "";
-	if ( tree ) {
-		tmp = ((MyString *)tree->RArg())->Value();
-	}
-	if ( tmp.Length() == 0 ) {
-		tmp.sprintf( "%s = \"<%s:0>\"", ATTR_MY_ADDRESS, my_ip_string() );
-		ad->Insert( tmp.GetCStr() );
-	}
-
 	CollectorList * collectors;
 	if ( pool ) {
 		collector = new Daemon( DT_COLLECTOR, pool, 0 );
@@ -170,30 +192,82 @@ int main( int argc, char *argv[] )
 
 		dprintf(D_FULLDEBUG,"collector is %s located at %s\n",
 				collector->hostname(),collector->addr());
-	
-		if ( use_tcp ) {
-			sock = collector->startCommand(command,Stream::reli_sock,20);
-		} else {
-			sock = collector->startCommand(command,Stream::safe_sock,20);
-		}
 
-		int result = 0;
-		if ( sock ) {
-			result += ad->put( *sock );
-			result += sock->end_of_message();
-		}
-		if ( result != 2 ) {
-			fprintf(stderr,"failed to send classad to %s\n",collector->addr());
-			had_error = true;
-			delete sock;
-			continue;
-		}
+		sock = NULL;
 
+		ClassAd *ad;
+		int success_count = 0;
+		int failure_count = 0;
+		ads.Rewind();
+		while( (ad=ads.Next()) ) {
+
+				// If there's no "MyAddress", generate one..
+			if( !ad->Lookup( ATTR_MY_ADDRESS ) ) {
+				MyString tmp;
+				tmp.sprintf( "<%s:0>", my_ip_string() );
+				ad->Assign( ATTR_MY_ADDRESS, tmp.Value() );
+			}
+
+			if ( use_tcp ) {
+				if( !sock ) {
+					sock = collector->startCommand(command,Stream::reli_sock,20);
+				}
+				else {
+						// Use existing connection.
+					sock->encode();
+					sock->put(command);
+				}
+			} else {
+					// We must open a new UDP socket each time.
+				delete sock;
+				sock = collector->startCommand(command,Stream::safe_sock,20);
+			}
+
+			int result = 0;
+			if ( sock ) {
+				result += ad->put( *sock );
+				result += sock->end_of_message();
+			}
+			if ( result != 2 ) {
+				fprintf(stderr,"failed to send classad to %s\n",collector->addr());
+				had_error = true;
+				failure_count++;
+				delete sock;
+				sock = NULL;
+				continue;
+			}
+
+			if( with_ack ) {
+				sock->decode();
+				int ok = 0;
+				if( !sock->get(ok) || !sock->end_of_message() ) {
+					fprintf(stderr,"failed to get ack from %s\n",collector->addr());
+					had_error = true;
+					failure_count++;
+					delete sock;
+					sock = NULL;
+					continue;
+				}
+
+					// ack protocol does not allow for multiple updates,
+					// so close the socket now
+				delete sock;
+				sock = NULL;
+			}
+
+			success_count++;
+		}
 		delete sock;
+		sock = NULL;
+
+		printf("Sent %d of %d ad%s to %s.\n",
+			   success_count,
+			   success_count + failure_count,
+			   success_count+failure_count == 1 ? "" : "s",
+			   collector->name());
 	}
 
 	delete collectors;
-	delete ad;
 
 	return (had_error)?1:0;
 }
