@@ -66,11 +66,11 @@ VacateType vacate_type = VACATE_GRACEFUL;
 
 
 // protoypes of interest
-void usage( char* );
+void usage( const char* );
 void version( void );
-void invalid( char* opt );
-void ambiguous( char* opt );
-void another( char* opt );
+void invalid( const char* opt );
+void ambiguous( const char* opt );
+void another( const char* opt );
 void parseCOpt( char* opt, char* arg );
 void parsePOpt( char* opt, char* arg );
 void parseArgv( int argc, char* argv[] );
@@ -114,7 +114,7 @@ main( int argc, char *argv[] )
 		exit( 1 );
 	}
 
-	bool rval;
+	bool rval = FALSE;
 	int irval;
 	ClassAd reply;
 	ClassAd ad;
@@ -145,7 +145,7 @@ main( int argc, char *argv[] )
 		rval = startd.renewLeaseForClaim( &reply, timeout );
 		break;
 	case DELEGATE_GSI_CRED_STARTD:
-		irval = startd.delegateX509Proxy( proxy_file );
+		irval = startd.delegateX509Proxy( proxy_file, 0, NULL );
 		rval = (irval == OK);
 		break;
 	}
@@ -231,11 +231,11 @@ printOutput( ClassAd* reply, DCStartd* startd )
 void
 fillRequirements( ClassAd* req )
 {
-	MyString jic_req;
+	MyString jic_req = "TARGET.";
 	if (jobad_path) {
-		jic_req = ATTR_HAS_JIC_LOCAL_STDIN;
+		jic_req += ATTR_HAS_JIC_LOCAL_STDIN;
 	} else {
-		jic_req = ATTR_HAS_JIC_LOCAL_CONFIG;
+		jic_req += ATTR_HAS_JIC_LOCAL_CONFIG;
 	}
 	jic_req += "==TRUE";
 
@@ -254,12 +254,13 @@ fillRequirements( ClassAd* req )
 		}
 	}
 	if (slot_id > 0) {
+		require += "TARGET.";
 		require += ATTR_SLOT_ID;
 		require += "==";
 		require += slot_id;
 		require += ")&&(";
 	}
-	else if (param_boolean("ALLOW_VM_CRUFT", true)) {
+	else if (param_boolean("ALLOW_VM_CRUFT", false)) {
 		int vm_id = 0;
 		if (name) {
 			if (sscanf(name, "vm%d@", &vm_id) != 1) { 
@@ -267,6 +268,7 @@ fillRequirements( ClassAd* req )
 			}
 		}
 		if (vm_id > 0) {
+			require += "TARGET.";
 			require += ATTR_VIRTUAL_MACHINE_ID;
 			require += "==";
 			require += vm_id;
@@ -282,6 +284,13 @@ fillRequirements( ClassAd* req )
 				 requirements );
 		exit( 1 );
 	}
+
+#if !defined(WANT_OLD_CLASSADS)
+		// The user may have entered some references to machine attributes
+		// without explicitly specifying the TARGET scope.
+	req->AddTargetRefs( TargetMachineAttrs );
+#endif
+
 }
 
 
@@ -359,6 +368,13 @@ dumpAdIntoRequest( ClassAd* req )
 		}
     }
 	fclose( JOBAD_PATH );
+
+#if !defined(WANT_OLD_CLASSADS)
+		// The user may have entered some references to machine attributes
+		// without explicitly specifying the TARGET scope.
+	req->AddTargetRefs( TargetMachineAttrs );
+#endif
+
 	return read_something;
 }
 
@@ -381,7 +397,7 @@ getCommandFromArgv( int argc, char* argv[] )
 		// now, make sure we're not looking at "condor_cod" or
 		// something.  if cmd_str is pointing at cod, we want to move
 		// beyond that...
-	if( cmd_str && !strincmp(cmd_str, "_cod", 4) ) {
+	if( cmd_str && !strncasecmp(cmd_str, "_cod", 4) ) {
 		if( cmd_str[4] ) {
 			cmd_str = &cmd_str[4];
 		} else {
@@ -452,7 +468,7 @@ version()
 }
 
 void
-invalid( char* opt )
+invalid( const char* opt )
 {
 	fprintf( stderr, "%s: '%s' is invalid\n", my_name, opt );
 	usage( my_name );
@@ -460,7 +476,7 @@ invalid( char* opt )
 
 
 void
-ambiguous( char* opt )
+ambiguous( const char* opt )
 {
 	fprintf( stderr, "%s: '%s' is ambiguous\n", my_name, opt ); 
 	usage( my_name );
@@ -468,7 +484,7 @@ ambiguous( char* opt )
 
 
 void
-another( char* opt )
+another( const char* opt )
 {
 	fprintf( stderr, "%s: '%s' requires another argument\n", my_name,
 			 opt ); 
@@ -925,7 +941,7 @@ printFast( void )
 
 
 void
-usage( char *str )
+usage( const char *str )
 {
 	bool has_cmd_opt = true;
 	bool needsID = true;
