@@ -111,13 +111,14 @@ usage()
 	fprintf( stderr, "   -unset\t\t(unset a persistent config file expression)\n" );
 	fprintf( stderr, "   -runset\t\t(unset a runtime config file expression)\n" );
 
-	fprintf( stderr, "   -master\t\t(query the master [default])\n" );
+	fprintf( stderr, "   -master\t\t(query the master)\n" );
 	fprintf( stderr, "   -schedd\t\t(query the schedd)\n" );
 	fprintf( stderr, "   -startd\t\t(query the startd)\n" );
 	fprintf( stderr, "   -collector\t\t(query the collector)\n" );
 	fprintf( stderr, "   -negotiator\t\t(query the negotiator)\n" );
 	fprintf( stderr, "   -tilde\t\t(return the path to the Condor home directory)\n" );
 	fprintf( stderr, "   -owner\t\t(return the owner of the condor_config_val process)\n" );
+	fprintf( stderr, "   -local-name name\t(Specify a local name for use with the config system)\n" );
 	fprintf( stderr, "   -verbose\t\t(print information about where variables are defined)\n" );
 	fprintf( stderr, "   -dump\t\t(print locally defined variables)\n" );
 	fprintf( stderr, "   -config\t\t(print the locations of found config files)\n" );
@@ -134,11 +135,14 @@ main( int argc, char* argv[] )
 {
 	char	*value, *tmp, *host = NULL;
 	char	*addr = NULL, *name = NULL, *pool = NULL;
+	char	*local_name = NULL;
 	int		i;
 	bool	ask_a_daemon = false;
 	bool    verbose = false;
 	bool    dump_all_variables = false;
 	bool    print_config_sources = false;
+	bool	write_config = false;
+	bool	debug = false;
 	
 	PrintType pt = CONDOR_NONE;
 	ModeType mt = CONDOR_QUERY;
@@ -188,6 +192,13 @@ main( int argc, char* argv[] )
 			} else {
 				usage();
 			}
+		} else if( match_prefix( argv[i], "-local-name" ) ) {
+			if( argv[i + 1] ) {
+				i++;
+				local_name = argv[i];
+			} else {
+				usage();
+			}
 		} else if( match_prefix( argv[i], "-owner" ) ) {
 			pt = CONDOR_OWNER;
 		} else if( match_prefix( argv[i], "-tilde" ) ) {
@@ -219,14 +230,21 @@ main( int argc, char* argv[] )
 			verbose = true;
 		} else if( match_prefix( argv[i], "-dump" ) ) {
 			dump_all_variables = true;
+		} else if( match_prefix( argv[i], "-writeconfig" ) ) {
+			write_config = true;
 		} else if( match_prefix( argv[i], "-debug" ) ) {
 				// dprintf to console
-			Termlog = 1;
-			dprintf_config( "TOOL" );
+			debug = true;
 		} else if( match_prefix( argv[i], "-" ) ) {
 			usage();
 		} else {
-			params.append( strdup( argv[i] ) );
+			MyString str;
+			str = argv[i];
+			// remove any case sensitivity, this is done mostly so output
+			// later can look nice. The param() subsystem inherently assumes
+			// case insensitivity, so this is perfectly fine to do here.
+			str.upper_case();
+			params.append( strdup( str.Value() ) ) ;
 		}
 	}
 
@@ -236,6 +254,12 @@ main( int argc, char* argv[] )
 		get_mySubSystem()->setName( "TOOL" );
 	}
 
+		// Honor any local name we might want to use for the param system
+		// while looking up variables.
+	if( local_name != NULL ) {
+		get_mySubSystem()->setLocalName( local_name );
+	}
+
 		// We need to do this before we call config().  A) we don't
 		// need config() to find this out and B) config() will fail if
 		// all the config sources aren't setup yet, and we use
@@ -270,6 +294,11 @@ main( int argc, char* argv[] )
 		}
 	}
 
+	if (debug) {
+		Termlog = 1;
+		dprintf_config( "TOOL" );
+	}
+
 	/* XXX -dump only currently spits out variables found through the
 		configuration file(s) available to this tool. Environment overloads
 		would be reflected. */
@@ -321,7 +350,7 @@ main( int argc, char* argv[] )
 				fprintf(stdout, "# Line %d, File %s\n", 
 					pv.lnum, pv.filename.Value());
 			}
-			fprintf(stdout, "%s = %s\n\n", upname.Value(), pv.value.Value());
+			fprintf(stdout, "%s = %s\n", upname.Value(), pv.value.Value());
 			
 		}
 		fflush( stdout );
@@ -335,12 +364,21 @@ main( int argc, char* argv[] )
 		my_exit( 0 );
 
 	}
-
+	
+	if(write_config == true) {
+		write_config_file("static_condor_config");
+	}
+	
 	if( pool && ! name ) {
 		fprintf( stderr, "Error: you must specify -name with -pool\n" );
 		my_exit( 1 );
 	}
 
+	params.rewind();
+	if( ! params.number() && !print_config_sources ) {
+		usage();
+	}
+
 	if( name || addr || mt != CONDOR_QUERY || dt != DT_MASTER ) {
 		ask_a_daemon = true;
 	}
@@ -356,8 +394,15 @@ main( int argc, char* argv[] )
 			} else { 
 				CollectorList * collectors = CollectorList::create();
 				Daemon * collector = NULL;
+				ReliSock sock;
 				while (collectors->next (collector)) {
-					if (collector->locate()) break;
+					if (collector->locate() &&
+					    sock.connect((char*) collector->addr(), 0)) {
+						// If we can connect to the
+						// collector, then we accept
+						// it as valid
+						break;
+					}
 				}
 				if( (!collector) || (!collector->addr()) ) {
 					fprintf( stderr, 
@@ -379,12 +424,6 @@ main( int argc, char* argv[] )
 		}
 	}
 
-	params.rewind();
-
-	if( ! params.number() && !print_config_sources ) {
-		usage();
-	}
-
 	while( (tmp = params.next()) ) {
 		if( mt == CONDOR_SET || mt == CONDOR_RUNTIME_SET ||
 			mt == CONDOR_UNSET || mt == CONDOR_RUNTIME_UNSET ) {
@@ -410,10 +449,10 @@ main( int argc, char* argv[] )
 					int      line_number;
 					param_get_location(tmp, filename, line_number);
 					if (line_number == -1) {
-						printf("  Defined in '%s'.\n\n", filename.GetCStr());
+						printf("  Defined in '%s'.\n\n", filename.Value());
 					} else {
 						printf("  Defined in '%s', line %d.\n\n",
-							   filename.GetCStr(), line_number);
+							   filename.Value(), line_number);
 					}
 				}
 			}
@@ -438,13 +477,23 @@ GetRemoteParam( Daemon* target, char* param_name )
 		// tools for fear that someone's ASCII parser will break, i'm
 		// just cheating and being lazy here by replicating the old
 		// behavior...
-	char* addr = target->addr();
-	char* name = target->name();
-	if( !name ) {
-		name = "";
-	}
+	char* addr;
+	char* name;
+	bool connect_error = true;
+	do {
+		addr = target->addr();
+		name = target->name();
+		if( !name ) {
+			name = "";
+		}
 
-    if( ! s.connect( addr, 0 ) ) {
+		if( s.connect( addr, 0 ) ) {
+			connect_error = false;
+			break;
+		}
+	} while( target->nextValidCm() == true );
+
+	if( connect_error == true ) {
 		fprintf( stderr, "Can't connect to %s on %s %s\n", 
 				 daemonString(dt), name, addr );
 		my_exit(1);
@@ -480,7 +529,7 @@ GetRemoteParam( Daemon* target, char* param_name )
 void
 SetRemoteParam( Daemon* target, char* param_value, ModeType mt )
 {
-	int cmd, rval;
+	int cmd = DC_NOP, rval;
 	ReliSock s;
 	bool set = false;
 
@@ -491,11 +540,9 @@ SetRemoteParam( Daemon* target, char* param_value, ModeType mt )
 		// tools for fear that someone's ASCII parser will break, i'm
 		// just cheating and being lazy here by replicating the old
 		// behavior...
-	char* addr = target->addr();
-	char* name = target->name();
-	if( !name ) {
-		name = "";
-	}
+	char* addr;
+	char* name;
+	bool connect_error = true;
 
 		// We need to know two things: what command to send, and (for
 		// error messages) if we're setting or unsetting.  Since our
@@ -558,10 +605,10 @@ SetRemoteParam( Daemon* target, char* param_value, ModeType mt )
 		// At this point, in either set or unset mode, param_name
 		// should hold a valid name, so do a final check to make sure
 		// there are no spaces.
-	if( (tmp = strchr(param_name, ' ')) ) {
+	if( !is_valid_param_name(param_name) ) {
 		fprintf( stderr, 
-				 "%s: Error: Configuration variable names cannot contain spaces\n",
-				 MyName );
+				 "%s: Error: Configuration variable name (%s) is not valid, alphanumeric and _ only\n",
+				 MyName, param_name );
 		my_exit( 1 );
 	}
 
@@ -574,8 +621,21 @@ SetRemoteParam( Daemon* target, char* param_value, ModeType mt )
 	char* buf = (char*)malloc( strlen(param_value) + 2 );
 	sprintf( buf, "%s\n", param_value );
 
-    s.timeout( 30 );
-	if( ! s.connect( addr, 0 ) ) {
+	s.timeout( 30 );
+	do {
+		addr = target->addr();
+		name = target->name();
+		if( !name ) {
+			name = "";
+		}
+
+		if( s.connect( addr, 0 ) ) {
+			connect_error = false;
+			break;
+		}
+	} while( target->nextValidCm() == true );
+
+	if( connect_error == true ) {
 		fprintf( stderr, "Can't connect to %s on %s %s\n", 
 				 daemonString(dt), name, addr );
 		my_exit(1);
