@@ -17,8 +17,8 @@
  *
  ***************************************************************/
 
-#ifdef WANT_QUILL
 #include "condor_common.h"
+#ifdef HAVE_EXT_POSTGRESQL
 #include "condor_config.h"
 #include "condor_classad.h"
 #include "condor_debug.h"
@@ -521,7 +521,7 @@ static long findLastDelimiter(FILE *fd, char *filename)
                 break;
 	  
             // If line starts with *** and its last line of file
-            if (strncmp(buf.GetCStr(), "***", 3) == 0 && buf.readLine(fd) == false) {
+            if (strncmp(buf.Value(), "***", 3) == 0 && buf.readLine(fd) == false) {
                 found = true;
                 break;
             }
@@ -559,7 +559,7 @@ static long findPrevDelimiter(FILE *fd, char* filename, long currOffset)
     // *** ProcId = a ClusterId = b Owner = "cde" CompletionDate = f
     // For the moment, owner and completionDate are just parsed in, reserved for future functionalities. 
 
-    sscanf(buf.GetCStr(), "%*s %*s %*s %ld %*s %*s %d %*s %*s %d %*s %*s %s %*s %*s %ld", 
+    sscanf(buf.Value(), "%*s %*s %*s %ld %*s %*s %d %*s %*s %d %*s %*s %s %*s %*s %ld", 
            &prevOffset, &clusterId, &procId, owner, &completionDate);
 
     if (prevOffset == -1 && clusterId == -1 && procId == -1) {
@@ -587,7 +587,7 @@ static long findPrevDelimiter(FILE *fd, char* filename, long currOffset)
             
             owner = (char *) realloc (owner, buf.Length() * sizeof(char));
       
-            sscanf(buf.GetCStr(), "%*s %*s %*s %ld %*s %*s %d %*s %*s %d %*s %*s %s %*s %*s %ld", 
+            sscanf(buf.Value(), "%*s %*s %*s %ld %*s %*s %d %*s %*s %d %*s %*s %s %*s %*s %ld", 
                    &prevOffset, &clusterId, &procId, owner, &completionDate);
         }
     }
@@ -704,5 +704,5 @@ static void readHistoryFromFile(char *JobHistoryFileName, char* constraint, Expr
     return;
 }
 
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
