@@ -35,13 +35,14 @@
 #include "match_prefix.h"
 #include "condor_xml_classads.h"
 #include "subsystem_info.h"
+#include "historyFileFinder.h"
 
 #include "history_utils.h"
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 #include "sqlquery.h"
 #include "historysnapshot.h"
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 #define NUM_PARAMETERS 3
 
@@ -58,7 +59,7 @@ static void Usage(char* name)
 		"\t\t-format <fmt> <attr>\tPrint attribute attr using format fmt\n"		
 		"\t\t-l\t\t\tVerbose output (entire classads)\n"
 		"\t\t-constraint <expr>\tAdd constraint on classads\n"
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		"\t\t-name <schedd-name>\tRead history data from Quill database\n"
 		"\t\t-completedsince <time>\tDisplay jobs completed on/after time\n"
 #endif
@@ -71,21 +72,18 @@ static void Usage(char* name)
   exit(1);
 }
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 static char * getDBConnStr(char *&quillName, char *&databaseIp, char *&databaseName, char *&queryPassword);
 static bool checkDBconfig();
-#endif /* WANT_QUILL */
+static	QueryResult result;
+#endif /* HAVE_EXT_POSTGRESQL */
 
 static void readHistoryFromFiles(char *JobHistoryFileName, char* constraint, ExprTree *constraintExpr);
-static char **findHistoryFiles(int *numHistoryFiles);
-static bool isHistoryBackup(const char *fullFilename, time_t *backup_time);
-static int compareHistoryFilenames(const void *item1, const void *item2);
 static void readHistoryFromFile(char *JobHistoryFileName, char* constraint, ExprTree *constraintExpr);
 
 //------------------------------------------------------------------------
 
 static CollectorList * Collectors = NULL;
-static	QueryResult result;
 static	CondorQuery	quillQuery(QUILL_AD);
 static	ClassAdList	quillList;
 static  bool longformat=false;
@@ -93,7 +91,6 @@ static  bool use_xml=false;
 static  bool customFormat=false;
 static  bool backwards=false;
 static  AttrListPrintMask mask;
-static  char *BaseJobHistoryFileName = NULL;
 static int cluster=-1, proc=-1;
 static int specifiedMatch = 0, matchCount = 0;
 
@@ -102,30 +99,30 @@ main(int argc, char* argv[])
 {
   Collectors = NULL;
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
   HistorySnapshot *historySnapshot;
   SQLQuery queryhor;
   SQLQuery queryver;
   QuillErrCode st;
-#endif /* WANT_QUILL */
+  bool remotequill=false;
+  char *quillName=NULL;
+  AttrList *ad=0;
+  int flag = 1;
+#endif /* HAVE_EXT_POSTGRESQL */
 
   void **parameters;
   char *dbconn=NULL;
   char *completedsince = NULL;
   char *owner=NULL;
-  bool readfromfile = false,remotequill=false;
+  bool readfromfile = false;
 
   char* JobHistoryFileName=NULL;
-  char *dbIpAddr=NULL, *dbName=NULL,*queryPassword=NULL,*quillName=NULL;
+  char *dbIpAddr=NULL, *dbName=NULL,*queryPassword=NULL;
 
 
   char* constraint=NULL;
   ExprTree *constraintExpr=NULL;
 
-  AttrList *ad=0;
-
-  int flag = 1;
-
   char tmp[512];
 
   int i;
@@ -134,10 +131,10 @@ main(int argc, char* argv[])
 
   config();
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
   queryhor.setQuery(HISTORY_ALL_HOR, NULL);
   queryver.setQuery(HISTORY_ALL_VER, NULL);
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
   for(i=1; i<argc; i++) {
     if (strcmp(argv[i],"-l")==0) {
@@ -168,7 +165,7 @@ main(int argc, char* argv[])
         specifiedMatch = atoi(argv[i]);
     }
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
     else if(match_prefix(argv[i], "-name")) {
 		i++;
 		if (argc <= i) {
@@ -205,7 +202,7 @@ main(int argc, char* argv[])
 		remotequill = false;
 		readfromfile = false;
     }
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
     else if (strcmp(argv[i],"-f")==0) {
 		if (i+1==argc || JobHistoryFileName) break;
 		i++;
@@ -235,7 +232,7 @@ main(int argc, char* argv[])
 		i++;
 		//readfromfile = true;
     }
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
     else if (match_prefix(argv[i],"-completedsince")) {
 		i++;
 		if (argc <= i) {
@@ -254,29 +251,35 @@ main(int argc, char* argv[])
 		queryhor.setQuery(HISTORY_COMPLETEDSINCE_HOR,parameters);
 		queryver.setQuery(HISTORY_COMPLETEDSINCE_VER,parameters);
     }
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
     else if (sscanf (argv[i], "%d.%d", &cluster, &proc) == 2) {
-		if (constraint) break;
+		if (constraint) {
+			fprintf(stderr, "Error: Cannot provide both -constraint and <cluster>.<proc>\n");
+			break;
+		}
 		sprintf (tmp, "((%s == %d) && (%s == %d))", 
 				 ATTR_CLUSTER_ID, cluster,ATTR_PROC_ID, proc);
 		constraint=tmp;
 		parameters[0] = &cluster;
 		parameters[1] = &proc;
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		queryhor.setQuery(HISTORY_CLUSTER_PROC_HOR, parameters);
 		queryver.setQuery(HISTORY_CLUSTER_PROC_VER, parameters);
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
     }
     else if (sscanf (argv[i], "%d", &cluster) == 1) {
-		if (constraint) break;
+		if (constraint) {
+			fprintf(stderr, "Error: Cannot provide both -constraint and <cluster>\n");
+			break;
+		}
 		sprintf (tmp, "(%s == %d)", ATTR_CLUSTER_ID, cluster);
 		constraint=tmp;
 		parameters[0] = &cluster;
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		queryhor.setQuery(HISTORY_CLUSTER_HOR, parameters);
 		queryver.setQuery(HISTORY_CLUSTER_VER, parameters);
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
     }
     else if (strcmp(argv[i],"-debug")==0) {
           // dprintf to console
@@ -284,27 +287,30 @@ main(int argc, char* argv[])
           dprintf_config ("TOOL");
     }
     else {
-		if (constraint) break;
+		if (constraint) {
+			fprintf(stderr, "Error: Cannot provide both -constraint and <owner>\n");
+			break;
+		}
 		owner = (char *) malloc(512 * sizeof(char));
 		sscanf(argv[i], "%s", owner);	
 		sprintf(tmp, "(%s == \"%s\")", ATTR_OWNER, owner);
 		constraint=tmp;
 		parameters[0] = owner;
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		queryhor.setQuery(HISTORY_OWNER_HOR, parameters);
 		queryver.setQuery(HISTORY_OWNER_VER, parameters);
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
     }
   }
   if (i<argc) Usage(argv[0]);
   
   
-  if( constraint && Parse( constraint, constraintExpr ) ) {
+  if( constraint && ParseClassAdRvalExpr( constraint, constraintExpr ) ) {
 	  fprintf( stderr, "Error:  could not parse constraint %s\n", constraint );
 	  exit( 1 );
   }
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 	/* This call must happen AFTER config() is called */
   if (checkDBconfig() == true && !readfromfile) {
   	readfromfile = false;
@@ -313,10 +319,10 @@ main(int argc, char* argv[])
   }
 #else 
   readfromfile = true;
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
   if(readfromfile == false) {
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 	  if(remotequill) {
 		  if (Collectors == NULL) {
 			  Collectors = CollectorList::create();
@@ -415,7 +421,7 @@ main(int argc, char* argv[])
 	  }
 	  historySnapshot->release();
 	  delete(historySnapshot);
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
   }
   
   if(readfromfile == true) {
@@ -436,7 +442,7 @@ main(int argc, char* argv[])
 
 //------------------------------------------------------------------------
 
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 
 /* this function for checking whether database can be used for 
    querying in local machine */
@@ -556,7 +562,7 @@ static char * getDBConnStr(char *&quillName,
   return dbconn;
 }
 
-#endif /* WANT_QUILL */
+#endif /* HAVE_EXT_POSTGRESQL */
 
 // Read the history from the specified history file, or from all the history files.
 // There are multiple history files because we do rotation. 
@@ -576,7 +582,17 @@ static void readHistoryFromFiles(char *JobHistoryFileName, char* constraint, Exp
         int numHistoryFiles;
         char **historyFiles;
 
-        historyFiles = findHistoryFiles(&numHistoryFiles);
+        historyFiles = findHistoryFiles("HISTORY", &numHistoryFiles);
+		if (!historyFiles) {
+			fprintf( stderr, "Error: No history file is defined\n");
+			fprintf(stderr, "\n");
+			print_wrapped_text("Extra Info: " 
+						   "The variable HISTORY is not defined in "
+						   "your config file. If you want Condor to "
+						   "keep a history of past jobs, you must "
+						   "define HISTORY in your config file", stderr );
+			exit(1);
+		}
         if (historyFiles && numHistoryFiles > 0) {
             int fileIndex;
             if (backwards) { // Reverse reading of history files array
@@ -597,126 +613,6 @@ static void readHistoryFromFiles(char *JobHistoryFileName, char* constraint, Exp
     return;
 }
 
-// Find all of the history files that the schedd created, and put them
-// in order by time that they were created. The time comes from a
-// timestamp in the file name.
-static char **findHistoryFiles(int *numHistoryFiles)
-{
-    int  fileIndex;
-    char **historyFiles = NULL;
-    char *historyDir;
-
-    BaseJobHistoryFileName = param("HISTORY");
-	if ( BaseJobHistoryFileName == NULL ) {
-		fprintf( stderr, "Error: No history file is defined\n");
-		fprintf(stderr, "\n");
-		print_wrapped_text("Extra Info: " 
-						   "The variable HISTORY is not defined in "
-						   "your config file. If you want Condor to "
-						   "keep a history of past jobs, you must "
-						   "define HISTORY in your config file", stderr );
-
-		exit( 1 );    
-	}
-    historyDir = condor_dirname(BaseJobHistoryFileName);
-
-    *numHistoryFiles = 0;
-    if (historyDir != NULL) {
-        Directory dir(historyDir);
-        const char *current_filename;
-
-        // We walk through once and count the number of history file backups
-         for (current_filename = dir.Next(); 
-             current_filename != NULL; 
-             current_filename = dir.Next()) {
-            
-            if (isHistoryBackup(current_filename, NULL)) {
-                (*numHistoryFiles)++;
-            }
-        }
-
-        // Add one for the current history file
-        (*numHistoryFiles)++;
-
-        // Make space for the filenames
-        historyFiles = (char **) malloc(sizeof(char*) * (*numHistoryFiles));
-
-        // Walk through again to fill in the names
-        // Note that we won't get the current history file
-        dir.Rewind();
-        for (fileIndex = 0, current_filename = dir.Next(); 
-             current_filename != NULL; 
-             current_filename = dir.Next()) {
-            
-            if (isHistoryBackup(current_filename, NULL)) {
-                historyFiles[fileIndex++] = strdup(dir.GetFullPath());
-            }
-        }
-        historyFiles[fileIndex] = strdup(BaseJobHistoryFileName);
-
-        if ((*numHistoryFiles) > 2) {
-            // Sort the backup files so that they are in the proper 
-            // order. The current history file is already in the right place.
-            qsort(historyFiles, (*numHistoryFiles)-1, sizeof(char*), compareHistoryFilenames);
-        }
-        
-        free(historyDir);
-    }
-    return historyFiles;
-}
-
-// Returns true if the filename is a history file, false otherwise.
-// If backup_time is not NULL, returns the time from the timestamp in
-// the file.
-static bool isHistoryBackup(const char *fullFilename, time_t *backup_time)
-{
-    bool       is_history_filename;
-    const char *filename;
-    const char *history_base;
-    int        history_base_length;
-
-    if (backup_time != NULL) {
-        *backup_time = -1;
-    }
-    
-    is_history_filename = false;
-    history_base        = condor_basename(BaseJobHistoryFileName);
-    history_base_length = strlen(history_base);
-    filename            = condor_basename(fullFilename);
-
-    if (   !strncmp(filename, history_base, history_base_length)
-        && filename[history_base_length] == '.') {
-        // The filename begins correctly, now see if it ends in an 
-        // ISO time
-        struct tm file_time;
-        bool is_utc;
-
-        iso8601_to_time(filename + history_base_length + 1, &file_time, &is_utc);
-        if (   file_time.tm_year != -1 && file_time.tm_mon != -1 
-            && file_time.tm_mday != -1 && file_time.tm_hour != -1
-            && file_time.tm_min != -1  && file_time.tm_sec != -1
-            && !is_utc) {
-            // This appears to be a proper history file backup.
-            is_history_filename = true;
-            if (backup_time != NULL) {
-                *backup_time = mktime(&file_time);
-            }
-        }
-    }
-
-    return is_history_filename;
-}
-
-// Used by qsort in findHistoryFiles() to sort history files. 
-static int compareHistoryFilenames(const void *item1, const void *item2)
-{
-    time_t time1, time2;
-
-    isHistoryBackup((const char *) item1, &time1);
-    isHistoryBackup((const char *) item2, &time2);
-    return time1 - time2;
-}
-
 // Given a history file, returns the position offset of the last delimiter
 // The last delimiter will be found in the last line of file, 
 // and will start with the "***" character string 
@@ -744,7 +640,7 @@ static long findLastDelimiter(FILE *fd, char *filename)
                 break;
 	  
             // If line starts with *** and its last line of file
-            if (strncmp(buf.GetCStr(), "***", 3) == 0 && buf.readLine(fd) == false) {
+            if (strncmp(buf.Value(), "***", 3) == 0 && buf.readLine(fd) == false) {
                 found = true;
                 break;
             }
@@ -782,7 +678,7 @@ static long findPrevDelimiter(FILE *fd, char* filename, long currOffset)
     // *** ProcId = a ClusterId = b Owner = "cde" CompletionDate = f
     // For the moment, owner and completionDate are just parsed in, reserved for future functionalities. 
 
-    sscanf(buf.GetCStr(), "%*s %*s %*s %ld %*s %*s %d %*s %*s %d %*s %*s %s %*s %*s %ld", 
+    sscanf(buf.Value(), "%*s %*s %*s %ld %*s %*s %d %*s %*s %d %*s %*s %s %*s %*s %ld", 
            &prevOffset, &clusterId, &procId, owner, &completionDate);
 
     if (prevOffset == -1 && clusterId == -1 && procId == -1) {
@@ -799,7 +695,7 @@ static long findPrevDelimiter(FILE *fd, char* filename, long currOffset)
         // Ok if only clusterId specified
         while (clusterId != cluster || (proc != -1 && procId != proc)) {
 	  
-            if (prevOffset == 0) { // no match
+            if (prevOffset <= 0) { // no match
                 free(owner);
                 return -1;
             }
@@ -814,7 +710,7 @@ static long findPrevDelimiter(FILE *fd, char* filename, long currOffset)
 			clusterId = -1;
 			procId = -1;
 
-            sscanf(buf.GetCStr(), "%*s %*s %*s %ld %*s %*s %d %*s %*s %d %*s %*s %s %*s %*s %ld", 
+            sscanf(buf.Value(), "%*s %*s %*s %ld %*s %*s %d %*s %*s %d %*s %*s %s %*s %*s %ld", 
                    &prevOffset, &clusterId, &procId, owner, &completionDate);
 
 			if (prevOffset == -1 && clusterId == -1 && procId == -1) {
