@@ -44,6 +44,11 @@
 #include "condor_distribution.h"
 #include "basename.h" // for condor_basename 
 #include "extArray.h"
+#include "link.h"
+#include "shared_port_endpoint.h"
+#include "file_lock.h"
+#include "../condor_privsep/condor_privsep.h"
+#include "filename_tools.h"
 
 State get_machine_state();
 
@@ -54,6 +59,7 @@ int			MaxCkptInterval;	// max time between ckpts on this machine
 char		*Spool;				// dir for condor job queue
 StringList   ExecuteDirs;		// dirs for execution of condor jobs
 char		*Log;				// dir for condor program logs
+char		*DaemonSockDir;     // dir for daemon named sockets
 char		*PreenAdmin;		// who to send mail to in case of trouble
 char		*MyName;			// name this program was invoked by
 char        *ValidSpoolFiles;   // well known files in the spool dir
@@ -71,15 +77,21 @@ void init_params();
 void check_spool_dir();
 void check_execute_dir();
 void check_log_dir();
+void rec_lock_cleanup(const char *path, int depth, bool remove_self = false);
+void check_tmp_dir();
+void check_daemon_sock_dir();
 void bad_file( const char *, const char *, Directory & );
 void good_file( const char *, const char * );
 void produce_output();
+BOOLEAN is_valid_shared_exe( const char *name );
 BOOLEAN is_ckpt_file( const char *name );
 BOOLEAN is_v2_ckpt( const char *name );
 BOOLEAN is_v3_ckpt( const char *name );
 BOOLEAN cluster_exists( int );
 BOOLEAN proc_exists( int, int );
 BOOLEAN is_myproxy_file( const char *name );
+BOOLEAN is_ccb_file( const char *name );
+BOOLEAN touched_recently(char const *fname,time_t delta);
 
 /*
   Tell folks how to use this program.
@@ -138,7 +150,8 @@ main( int argc, char *argv[] )
 	check_spool_dir();
 	check_execute_dir();
 	check_log_dir();
-
+	check_daemon_sock_dir();
+	check_tmp_dir();
 
 		// Produce output, either on stdout or by mail
 	if( !BadFiles->isEmpty() ) {
@@ -204,6 +217,60 @@ produce_output()
 	}
 }
 
+bool
+check_job_spool_hierarchy( char const *parent, char const *child, StringList &bad_spool_files )
+{
+	ASSERT( parent );
+	ASSERT( child );
+
+		// We expect directories of the form produced by gen_ckpt_name().
+		// e.g. $(SPOOL)/<cluster mod 10000>/<proc mod 10000>/cluster<cluster>.proc<proc>.subproc<proc>
+		// or $(SPOOL)/<cluster mod 10000>/cluster<cluster>.ickpt.subproc<subproc>
+
+	char *end=NULL;
+	strtol(child,&end,10);
+	if( !end || *end != '\0' ) {
+		return false; // not part of the job spool hierarchy
+	}
+
+	std::string topdir;
+	sprintf(topdir,"%s%c%s",parent,DIR_DELIM_CHAR,child);
+	Directory dir(topdir.c_str(),PRIV_ROOT);
+	char const *f;
+	while( (f=dir.Next()) ) {
+
+			// see if it's a legitimate job spool file/directory
+		if( is_ckpt_file(f) ) {
+			good_file( topdir.c_str(), f );
+			continue;
+		}
+
+		if( IsDirectory(dir.GetFullPath()) && !IsSymlink(dir.GetFullPath()) ) {
+			if( check_job_spool_hierarchy( topdir.c_str(), f, bad_spool_files ) ) {
+				good_file( topdir.c_str(), f );
+				continue;
+			}
+		}
+
+		bad_spool_files.append( dir.GetFullPath() );
+	}
+
+		// By returning true here, we indicate that this directory is
+		// part of the spool directory hierarchy and should not be
+		// deleted.  We do return true even if the directory is empty.
+		// If we wanted to remove empty directories of this type, we
+		// would need to do so with rmdir(), not a recursive remove,
+		// because new job directories might show up inside this
+		// directory between the time we examine the directory
+		// contents and the time we remove the directory.  We don't
+		// bother dealing with that case, because if something ever
+		// causes the self-cleaning of these directories to fail, the
+		// self-cleaning will be tried again the next time the
+		// directory is used.  In the worst case, the spool hierarchy
+		// is self-limited in how many entries will be made anyway.
+
+	return true;
+}
 
 /*
   Check the condor spool directory for extraneous files.  Files found
@@ -216,21 +283,31 @@ produce_output()
 void
 check_spool_dir()
 {
-    unsigned int	history_length;
+    unsigned int	history_length, startd_history_length;
 	const char  	*f;
-    const char      *history;
+    const char      *history, *startd_history;
 	Directory  		dir(Spool, PRIV_ROOT);
 	StringList 		well_known_list, bad_spool_files;
 	Qmgr_connection *qmgr;
 
+	if ( ValidSpoolFiles == NULL ) {
+		dprintf( D_ALWAYS, "Not cleaning spool directory: No VALID_SPOOL_FILES defined\n");
+		return;
+	}
+
     history = param("HISTORY");
-    history = condor_basename(history);
+    history = condor_basename(history); // condor_basename never returns NULL
     history_length = strlen(history);
 
+    startd_history = param("STARTD_HISTORY");
+   	startd_history = condor_basename(startd_history);
+   	startd_history_length = strlen(startd_history);
+
 	well_known_list.initializeFromString (ValidSpoolFiles);
 		// add some reasonable defaults that we never want to remove
 	well_known_list.append( "job_queue.log" );
 	well_known_list.append( "job_queue.log.tmp" );
+	well_known_list.append( "spool_version" );
 	well_known_list.append( "Accountant.log" );
 	well_known_list.append( "Accountantnew.log" );
 	well_known_list.append( "local_univ_execute" );
@@ -238,10 +315,13 @@ check_spool_dir()
 		// SCHEDD.lock: High availability lock file.  Current
 		// manual recommends putting it in the spool, so avoid it.
 	well_known_list.append( "SCHEDD.lock" );
+		// These are Quill-related files
+	well_known_list.append( ".quillwritepassword" );
+	well_known_list.append( ".pgpass" );
 
 	// connect to the Q manager
 	if (!(qmgr = ConnectQ (0))) {
-		dprintf( D_ALWAYS, "Not cleaning spool directory.\n" );
+		dprintf( D_ALWAYS, "Not cleaning spool directory: Can't contact schedd\n" );
 		return;
 	}
 
@@ -271,6 +351,21 @@ check_spool_dir()
             continue;
         }
 
+			// if startd_history is defined, so if it's one of those
+		if ( startd_history_length > 0 &&
+			strlen(f) >= startd_history_length &&
+			strncmp(f, startd_history, startd_history_length) == 0) {
+
+            good_file( Spool, f );
+            continue;
+		}
+
+			// see it it's an in-use shared executable
+		if( is_valid_shared_exe(f) ) {
+			good_file( Spool, f );
+			continue;
+		}
+
 			// see if it's a legitimate checkpoint
 		if( is_ckpt_file(f) ) {
 			good_file( Spool, f );
@@ -283,6 +378,19 @@ check_spool_dir()
 			continue;
 		}
 
+			// See if it's a CCB server file
+		if ( is_ccb_file( f ) ) {
+			good_file( Spool, f );
+			continue;
+		}
+
+		if( IsDirectory( dir.GetFullPath() ) && !IsSymlink( dir.GetFullPath() ) ) {
+			if( check_job_spool_hierarchy( Spool, f, bad_spool_files ) ) {
+				good_file( Spool, f );
+				continue;
+			}
+		}
+
 			// We think it's bad.  For now, just append it to a
 			// StringList, instead of actually deleting it.  This way,
 			// if DisconnectQ() fails, we can abort and not actually
@@ -304,6 +412,26 @@ check_spool_dir()
 	}
 }
 
+/*
+*/
+BOOLEAN
+is_valid_shared_exe( const char *name )
+{
+	if ((strlen(name) < 4) || (strncmp(name, "exe-", 4) != 0)) {
+		return FALSE;
+	}
+	MyString path;
+	path.sprintf("%s/%s", Spool, name);
+	int count = link_count(path.Value());
+	if (count == 1) {
+		return FALSE;
+	}
+	if (count == -1) {
+		dprintf(D_ALWAYS, "link_count error on %s; not deleting\n", name);
+	}
+	return TRUE;
+}
+
 /*
   Given the name of a file in the spool directory, return TRUE if it's a
   legitimate checkpoint file, and FALSE otherwise.  If the name starts
@@ -413,6 +541,19 @@ is_myproxy_file( const char *name )
 	return proc_exists( cluster, proc );
 }
 
+/*
+  Check whether the given file could be a valid MyProxy password file
+  for a queued job.
+*/
+BOOLEAN
+is_ccb_file( const char *name )
+{
+	if( strstr(name,".ccb_reconnect") ) {
+		return TRUE;
+	}
+	return FALSE;
+}
+
 /*
   Check to see whether a given cluster number exists in the job queue.
 */
@@ -508,7 +649,7 @@ check_log_dir()
 	Directory dir(Log, PRIV_ROOT);
 	StringList invalid;
 
-	invalid.initializeFromString (InvalidLogFiles);
+	invalid.initializeFromString (InvalidLogFiles ? InvalidLogFiles : "");
 
 	while( (f = dir.Next()) ) {
 		if( invalid.contains(f) ) {
@@ -518,7 +659,89 @@ check_log_dir()
 		}
 	}
 }
+
+void
+check_daemon_sock_dir()
+{
+	const char	*f;
+	Directory dir(DaemonSockDir, PRIV_ROOT);
+
+	time_t stale_age = SharedPortEndpoint::TouchSocketInterval()*100;
+
+	while( (f = dir.Next()) ) {
+		MyString full_path;
+		full_path.sprintf("%s%c%s",DaemonSockDir,DIR_DELIM_CHAR,f);
+
+			// daemon sockets are touched periodically to mark them as
+			// still in use
+		if( touched_recently( full_path.Value(), stale_age ) ) {
+			good_file( DaemonSockDir, f );
+		}
+		else {
+			bad_file( DaemonSockDir, f, dir );
+		}
+	}
+}
+
+void rec_lock_cleanup(const char *path, int depth, bool remove_self) {
+#if !defined(WIN32)
+	FileLock *lock = NULL;
+	if (depth == 0) {
+		lock = new FileLock(path, true, true);
+		delete lock;
+		return ;
+	}
+	Directory *dir = new Directory(path);
+	if (dir == NULL) {
+		// that may be ok as the path could already be cleaned up.
+		return;
+	}
+	const char *entry;
+	while ((entry = dir->Next()) != 0) {
+		if (!dir->IsDirectory() && depth > 1) { // clean up files floating around randomly -- maybe from older releases
+			lock = new FileLock(path, false, true);
+			bool result = lock->obtain(WRITE_LOCK);
+			if (!result) {
+					dprintf(D_FULLDEBUG, "Cannot lock %s\n", path);
+			}
+			int res = unlink(dir->GetFullPath());
+			if (res != 0) {
+				dprintf(D_FULLDEBUG, "Cannot delete %s (%s) \n", path, strerror(errno));
+			}
+			delete lock;
+		} else {
+			rec_lock_cleanup(dir->GetFullPath(), --depth, true);
+		}
+	}
+	// make sure, orphaned directories will be deleted as well.
+	if (remove_self) {		
+		int res = rmdir(path);
+		if (res != 0) {
+			dprintf(D_FULLDEBUG, "Directory %s could not be removed.\n", path);
+		}
+	}
 	
+	delete dir;
+#endif
+}	
+
+void check_tmp_dir(){
+#if !defined(WIN32)
+	const char *tmpDir = NULL;
+	bool newLock = param_boolean("CREATE_LOCKS_ON_LOCAL_DISK", true);
+	if (newLock) {
+				// create a dummy FileLock for TmpPath access
+		FileLock *lock = new FileLock(-1, NULL, NULL);
+		tmpDir = lock->GetTempPath();	
+		delete lock;
+		rec_lock_cleanup(tmpDir, 4);
+		if (tmpDir != NULL)
+			delete []tmpDir;
+	}
+  
+#endif	
+}
+
 
 extern "C" int
 SetSyscalls( int foo ) { return foo; }
@@ -539,6 +762,11 @@ init_params()
         EXCEPT( "LOG not specified in config file\n" );
     }
 
+	DaemonSockDir = param("DAEMON_SOCKET_DIR");
+	if( DaemonSockDir == NULL ) {
+		EXCEPT("DAEMON_SOCKET_DIR not defined\n");
+	}
+
 	char *Execute = param("EXECUTE");
 	if( Execute ) {
 		ExecuteDirs.append(Execute);
@@ -570,13 +798,9 @@ init_params()
 		}
     }
 
-	if( (ValidSpoolFiles = param("VALID_SPOOL_FILES")) == NULL ) {
-		EXCEPT ( "VALID_SPOOL_FILES not specified in config file" );
-	}
+	ValidSpoolFiles = param("VALID_SPOOL_FILES");
 
-	if( (InvalidLogFiles = param("INVALID_LOG_FILES")) == NULL ) {
-		EXCEPT ( "INVALID_LOG_FILES not specified in config file" );
-	}
+	InvalidLogFiles = param("INVALID_LOG_FILES");
 
 	if( (MailPrg = param("MAIL")) == NULL ) {
 		EXCEPT ( "MAIL not specified in config file" );
@@ -606,14 +830,28 @@ bad_file( const char *dirpath, const char *name, Directory & dir )
 	MyString	pathname;
 	MyString	buf;
 
-	pathname.sprintf( "%s%c%s", dirpath, DIR_DELIM_CHAR, name );
+	if( is_relative_to_cwd( name ) ) {
+		pathname.sprintf( "%s%c%s", dirpath, DIR_DELIM_CHAR, name );
+	}
+	else {
+		pathname = name;
+	}
 
 	if( VerboseFlag ) {
 		printf( "%s - BAD\n", pathname.Value() );
 	}
 
 	if( RmFlag ) {
-		if( dir.Remove_Full_Path( pathname.Value() ) ) {
+		bool removed = dir.Remove_Full_Path( pathname.Value() );
+		if( !removed && privsep_enabled() ) {
+			removed = privsep_remove_dir( pathname.Value() );
+			if( VerboseFlag ) {
+				if( removed ) {
+					printf( "%s - failed to remove directly, but succeeded via privsep switchboard\n", pathname.Value() );
+				}
+			}
+		}
+		if( removed ) {
 			buf.sprintf( "%s - Removed", pathname.Value() );
 		} else {
 			buf.sprintf( "%s - Can't Remove", pathname.Value() );
@@ -646,7 +884,7 @@ get_machine_state()
 		return _error_state_;
 	}
 
-	sock->eom();
+	sock->end_of_message();
 	sock->decode();
 	if( !sock->code( state_str ) || !sock->end_of_message() ) {
 		dprintf( D_ALWAYS, "Can't read state/eom from startd.\n" );
@@ -662,3 +900,17 @@ get_machine_state()
 	return s;
 }
 
+BOOLEAN
+touched_recently(char const *fname,time_t delta)
+{
+	StatInfo statinfo(fname);
+	if( statinfo.Error() != 0 ) {
+		return false;
+	}
+		// extend the window of what it means to have been touched "recently"
+		// both forwards and backwards in time to handle system clock jumps.
+	if( abs((int)(time(NULL)-statinfo.GetModifyTime())) > delta ) {
+		return false;
+	}
+	return true;
+}
