@@ -67,6 +67,7 @@ bool fast = false;
 bool peaceful_shutdown = false;
 bool full = false;
 bool all = false;
+char* constraint = NULL;
 const char* subsys = NULL;
 const char* exec_program = NULL;
 int takes_subsys = 0;
@@ -74,6 +75,8 @@ int cmd_set = 0;
 char *subsys_arg = NULL;
 bool IgnoreMissingDaemon = false;
 
+bool all_good = true;
+
 HashTable<MyString, bool> addresses_sent( 100, MyStringHash );
 
 
@@ -102,7 +105,7 @@ usage( char *str )
 
 	fprintf( stderr, "[general-options] [targets]" );
 	if( takes_subsys ) {
-		fprintf( stderr, " [subsystem]" );
+		fprintf( stderr, " [daemon]" );
 	}
 	fprintf( stderr, "\nwhere [general-options] can be zero or more of:\n" );
 	fprintf( stderr, "    -help\t\tgives this usage information\n" );
@@ -121,10 +124,6 @@ usage( char *str )
 		fprintf( stderr, 
 				 "    -fast\t\tquickly vacate the jobs (no checkpointing)\n" );
 	}
-	if( cmd == DC_RECONFIG ) {
-		fprintf( stderr, 
-				 "    -full\t\tPerform a full reconfig\n" );
-	}
 	fprintf( stderr, "where [targets] can be zero or more of:\n" );
 	fprintf( stderr, 
 			 "    -all\t\tall hosts in your pool (overrides other targets)\n" );
@@ -133,14 +132,15 @@ usage( char *str )
 	fprintf( stderr,
 			 "  (for compatibility with other Condor tools, you can also use:)\n" );
 	fprintf( stderr, "    -name name\tgiven host\n" );
+	fprintf( stderr, "    -constraint constraint\tconstraint\n" );
 	fprintf( stderr, "    -addr <addr:port>\tgiven \"sinful string\"\n" );
 	fprintf( stderr, "  (if no targets are specified, the local host is used)\n" );
 	if( takes_subsys ) {
-		fprintf( stderr, "where [subsystem] can be one of:\n" );
+		fprintf( stderr, "where [daemon] can be one of:\n" );
 		fprintf( stderr, 
-			 "    -subsystem <name>\tspecify the target subsystem by name.\n" );
+			 "    -daemon <name>\tspecify the target daemon by name.\n" );
 		fprintf( stderr,
-			"    The following named subsystem options are deprecated, and\n");
+			"    The following named daemon options are deprecated, and\n");
 		fprintf( stderr, "    may be discontinued in a future release:\n");
 		if( cmd == DAEMONS_OFF || cmd == DAEMON_OFF ) {
 			fprintf( stderr, "    -master\n" );
@@ -152,7 +152,7 @@ usage( char *str )
 		fprintf( stderr, "    -collector\n" );
 		fprintf( stderr, "    -negotiator\n" );
 		fprintf( stderr, "    -kbdd\n" );
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		fprintf( stderr, "    -quill\n" );
 #endif
 	}
@@ -169,7 +169,7 @@ usage( char *str )
 		fprintf( stderr, "  %s turns off the specified daemon.\n", 
 				 str );
 		fprintf( stderr, 
-				 "  If no subsystem is given, everything except the master is shut down.\n" );
+				 "  If no daemon is given, everything except the master is shut down.\n" );
 		break;
 	case RESTART:
 		fprintf( stderr, "  %s causes specified daemon to restart itself.\n", str );
@@ -177,7 +177,7 @@ usage( char *str )
 				 "  If sent to the master, all daemons on that host will restart.\n" );
 		break;
 
-	case DC_RECONFIG:
+	case DC_RECONFIG_FULL:
 		fprintf( stderr, 
 				 "  %s causes the specified daemon to reconfigure itself.\n", 
 				 str );
@@ -217,7 +217,7 @@ usage( char *str )
 	default:
 		fprintf( stderr, "  Valid commands are:\n%s%s",
 				 "\toff, on, restart, reconfig, reschedule, ",
-				 "vacate, checkpoint\n\n" );
+				 "vacate, checkpoint, set_shutdown\n\n" );
 		fprintf( stderr, "  Use \"%s [command] -help\" for more information %s\n", 
 				 str, "on a given command." );
 		break;
@@ -239,7 +239,7 @@ pool_target_usage( void )
 }
 
 
-char*
+const char*
 cmdToStr( int c )
 {
 	switch( c ) {
@@ -286,10 +286,8 @@ cmdToStr( int c )
 		return "Checkpoint-All-Jobs";
 	case RESCHEDULE:
 		return "Reschedule";
-	case DC_RECONFIG:
-		return "Reconfig";
 	case DC_RECONFIG_FULL:
-		return "Full-Reconfig";
+		return "Reconfig";
 	case SQUAWK:
 		return "Squawk";
 	case SET_SHUTDOWN_PROGRAM:
@@ -307,12 +305,12 @@ subsys_check( char* MyName )
 {
 	if( ! takes_subsys ) {
 		fprintf( stderr, 
-				 "ERROR: Can't specify a subsystem flag with %s.\n",  
+				 "ERROR: Can't specify a daemon flag with %s.\n",  
 				 MyName );
 		usage( MyName );
 	}
 	if( dt ) {
-		fprintf( stderr, "ERROR: can only specify one subsystem flag.\n" );
+		fprintf( stderr, "ERROR: can only specify one daemon flag.\n" );
 		usage( MyName );
 	}
 	subsys = (char*)1;
@@ -372,10 +370,10 @@ main( int argc, char *argv[] )
 	if( !strncmp_auto( cmd_str, "_reconfig_schedd" ) ) {
 		fprintf( stderr, "WARNING: condor_reconfig_schedd is deprecated.\n" );
 		fprintf( stderr, "\t Use: \"condor_reconfig -schedd\" instead.\n" );
-		cmd = DC_RECONFIG;
+		cmd = DC_RECONFIG_FULL;
 		dt = DT_SCHEDD;
     } else if( !strncmp_auto( cmd_str, "_reconfig" ) ) {
-		cmd = DC_RECONFIG;
+		cmd = DC_RECONFIG_FULL;
 		takes_subsys = 1;
 	} else if( !strncmp_auto( cmd_str, "_restart" ) ) {
 		cmd = RESTART;
@@ -419,7 +417,7 @@ main( int argc, char *argv[] )
 		case 'v':
 			version();
 			break;
-#ifdef WANT_QUILL
+#ifdef HAVE_EXT_POSTGRESQL
 		case 'q':
 			subsys_check( MyName );
 			dt = DT_QUILL;
@@ -466,9 +464,9 @@ main( int argc, char *argv[] )
 			if( (*tmp)[2] ) {
 				switch( (*tmp)[2] ) {
 				case 'u':
-					if( cmd == DC_RECONFIG ) {
-						full = true;
-					} else {
+					// CRUFT: -full is a deprecated argument to
+					//   condor_reconfig. It was removed in 7.5.3.
+					if( cmd != DC_RECONFIG_FULL ) {
 						fprintf( stderr, "ERROR: \"-full\" "
 								 "is not valid with %s\n", MyName );
 						usage( NULL );
@@ -506,8 +504,32 @@ main( int argc, char *argv[] )
 			}
 			break;
 		case 'd':
-			Termlog = 1;
-			dprintf_config ("TOOL");
+			if (!(*tmp)[2] || (*tmp)[2] == 'e') {
+				Termlog = 1;
+				dprintf_config ("TOOL");
+			} else if ((*tmp)[2] == 'a')  {
+				subsys_check( MyName );
+					// We got a "-daemon", make sure we've got 
+					// something else after it
+				tmp++;
+				if( tmp && *tmp ) {
+					subsys_arg = *tmp;
+					dt = stringToDaemonType(subsys_arg);
+					if( dt == DT_NONE ) {
+						dt = DT_GENERIC;
+					}
+				} else {
+					fprintf( stderr, 
+							 "ERROR: -daemon requires another argument\n" ); 
+					usage( NULL );
+					exit( 1 );
+				}
+			} else {
+				fprintf( stderr, 
+						 "ERROR: unknown parameter: \"%s\"\n",
+						 *tmp );  
+				usage( NULL );
+			}
 			break;
 		case 'e':
 			if ( strcmp( *tmp, "-exec" ) ) {
@@ -629,9 +651,40 @@ main( int argc, char *argv[] )
 					}
 					break;
 				case 'o':
+				  if( (*tmp)[3] ) {
+				    switch( (*tmp)[3] ) {
+				    case 'n': 
+				      // We got a "-constraint", make sure we've got 
+				      // something else after it
+				      tmp++;
+				      if( tmp && *tmp ) {
+					constraint = *tmp;
+				      } else {
+					fprintf( stderr, "ERROR: -constraint requires another argument\n" );
+					usage( NULL );
+				      }
+
+				      break;
+				    case 'l':
 					subsys_check( MyName );
 					dt = DT_COLLECTOR;
 					break;
+				    default:
+				      fprintf( stderr, 
+					       "ERROR: unknown parameter: \"%s\"\n",
+					       *tmp );  
+				      usage( NULL );
+				      break;
+				    }
+				  } else {
+				    fprintf( stderr, 
+					     "ERROR: ambigous parameter: \"%s\"\n",
+					     *tmp );  
+				    fprintf( stderr, 
+					     "Please specify \"-collector\" or \"-constraint\"\n" );
+				    usage( NULL );
+				  }
+				  break;
 				default:
 					fprintf( stderr, 
 							 "ERROR: unknown parameter: \"%s\"\n",
@@ -640,17 +693,10 @@ main( int argc, char *argv[] )
 					break;
 				}
 			} else {
-					// Since -cmd is a developer-only, hidden
-					// option, just treat "-c" as "-collector". 
-#if 0
 				fprintf( stderr, 
 						 "ERROR: ambiguous parameter: \"%s\"\n",
 						 *tmp ); 
 				usage( NULL );
-#else
-				subsys_check( MyName );
-				dt = DT_COLLECTOR;
-#endif
 			}
 			break;
 		case 'k':
@@ -676,11 +722,11 @@ main( int argc, char *argv[] )
 						subsys_arg = *tmp;
 						dt = stringToDaemonType(subsys_arg);
 						if( dt == DT_NONE ) {
-							dt = DT_ANY;
+							dt = DT_GENERIC;
 						}
 					} else {
 						fprintf( stderr, 
-							 "ERROR: -subsystem requires another argument\n" ); 
+							 "ERROR: -daemon requires another argument\n" ); 
 						usage( NULL );
 						exit( 1 );
 					}
@@ -697,7 +743,7 @@ main( int argc, char *argv[] )
 						 "ERROR: ambiguous argument \"%s\"\n",
 						 *tmp );
 				fprintf( stderr, 
-				"Please specify \"-subsystem\", \"-startd\" or \"-schedd\"\n" );
+				"Please specify \"-daemon\", \"-startd\" or \"-schedd\"\n" );
 				usage( NULL );
 			}
 			break;
@@ -718,12 +764,12 @@ main( int argc, char *argv[] )
 	computeRealAction();
 
 		// While we're parsing command-line args, when we notice a
-		// subsystem flag, we just set subsys to non-NULL to know we
-		// found it, and record the actual subsystem with "dt".  Now
+		// daemon flag, we just set subsys to non-NULL to know we
+		// found it, and record the actual daemon with "dt".  Now
 		// that we know the true target daemon type for whatever
 		// command we're using, want the real string to use.
 	if( subsys ) {
-		if( dt == DT_ANY && subsys_arg ) {
+		if( (dt == DT_ANY || dt == DT_GENERIC) && subsys_arg ) {
 			subsys = subsys_arg; 
 		} else { 
 			subsys = daemonString( dt );
@@ -797,7 +843,7 @@ doCommands(int /*argc*/,char * argv[],char *MyName)
 	char *daemonname;
 	bool found_one = false;
 
-	if( all ) {
+	if( all || (constraint!=NULL) ) {
 			// If we were told -all, we can just ignore any other
 			// options and send the specifed command to every machine
 			// in the pool.
@@ -825,6 +871,7 @@ doCommands(int /*argc*/,char * argv[],char *MyName)
 				// with those again and skip over the arguments to
 				// options we've handled.  this includes:
 				//  -pool XXX    (but not "-peaceful")
+				//  -constraint XXX    (but not "-collector")
 				//  -cmd XXX     (but not "-collector")
 				//  -subsys XXX  (but not "-schedd" or "-startd")
 			switch( (*argv)[1] ) {
@@ -838,6 +885,15 @@ doCommands(int /*argc*/,char * argv[],char *MyName)
 				if( (*argv)[2] == 'm' ) {
 						// this is -cmd, skip the next one.
 					argv++;
+				} else if( (*argv)[3] == 'n' ) {
+						// this is -constraint, skip the next one.
+					argv++;
+				}
+				break;
+			case 'd':
+				if( (*argv)[2] == 'a' ) {
+						// this is -daemon, skip the next one.
+					argv++;
 				}
 				break;
 			case 's':
@@ -865,6 +921,7 @@ doCommands(int /*argc*/,char * argv[],char *MyName)
 				fprintf( stderr, "Address %s is not valid.\n", *argv );
 				fprintf( stderr, "Should be of the form <ip.address.here:port>.\n" );
 				fprintf( stderr, "For example: <123.456.789.123:6789>\n" );
+				all_good = false;
 				continue;
 			}
 			break;
@@ -874,6 +931,7 @@ doCommands(int /*argc*/,char * argv[],char *MyName)
 			if( (daemonname = get_daemon_name(*argv)) == NULL ) {
 				fprintf( stderr, "%s: unknown host %s\n", MyName, 
 						 get_host_part(*argv) );
+				all_good = false;
 				continue;
 			}
 			names.append( daemonname );
@@ -896,6 +954,9 @@ doCommands(int /*argc*/,char * argv[],char *MyName)
 			// to talk to a remote pool, they just want to send their
 			// command to the local host. 
 		Daemon local_d( real_dt, NULL );
+		if( real_dt == DT_GENERIC ) {
+			local_d.setSubsystem( subsys );
+		}
 		if( ! local_d.locate() ) {
 			if( IgnoreMissingDaemon ) {
 				return 0;
@@ -905,7 +966,7 @@ doCommands(int /*argc*/,char * argv[],char *MyName)
 			return 1;
 		}
 		doCommand( &local_d );
-		return 0;
+		return all_good ? 0 : 1;
 	}
 
 		// If we got here, there were some targets specified on the
@@ -933,9 +994,12 @@ doCommands(int /*argc*/,char * argv[],char *MyName)
 		// Now, send commands to all the daemons we know about.
 	daemons.rewind();	
 	while( daemons.next(d) ) {
+		if( real_dt == DT_GENERIC ) {
+			d->setSubsystem( subsys );
+		}
 		doCommand( d );
 	}
-	return 0;
+	return all_good ? 0 : 1;
 }
 
 
@@ -945,14 +1009,14 @@ computeRealAction( void )
 	real_cmd = -1;
 	switch( cmd ) {
 
-	case DC_RECONFIG:
+	case DC_RECONFIG_FULL:
 			// no magic
 		break;
 
 	case RESTART:
 		if( subsys && dt != DT_MASTER ) {
 				// We're trying to restart something and we were told
-				// a specific subsystem to restart.  So, just send a
+				// a specific daemon to restart.  So, just send a
 				// DC_OFF to that daemon, and the master will restart
 				// for us.  Note: we don't want to do this if we were
 				// told to restart the master, since if we send this,
@@ -1043,7 +1107,7 @@ resolveNames( DaemonList* daemon_list, StringList* name_list )
 		return true;
 	}
 
-	AdTypes	adtype;
+	AdTypes	adtype = MASTER_AD;
 	switch( real_dt ) {
 	case DT_MASTER:
 		adtype = MASTER_AD;
@@ -1054,6 +1118,30 @@ resolveNames( DaemonList* daemon_list, StringList* name_list )
 	case DT_SCHEDD:
 		adtype = SCHEDD_AD;
 		break;
+	case DT_CLUSTER:
+		adtype = CLUSTER_AD;
+		break;
+	case DT_COLLECTOR:
+		adtype = COLLECTOR_AD;
+		break;
+	case DT_NEGOTIATOR:
+		adtype = NEGOTIATOR_AD;
+		break;
+	case DT_CREDD:
+		adtype = CREDD_AD;
+		break;
+	case DT_QUILL:
+		adtype = QUILL_AD;
+		break;
+	case DT_LEASE_MANAGER:
+		adtype = LEASE_MANAGER_AD;
+		break;
+	case DT_GENERIC:
+		adtype = GENERIC_AD;
+		break;
+	case DT_HAD:
+		adtype = HAD_AD;
+		break;
 	default:
 			// TODO: can we do better than this?
 		fprintf( stderr, "Unrecognized daemon type while resolving names\n" );
@@ -1067,6 +1155,16 @@ resolveNames( DaemonList* daemon_list, StringList* name_list )
 
 	CondorError errstack;
 	QueryResult q_result;
+	MyString buffer;
+
+	if (adtype == GENERIC_AD) {
+		query.setGenericQueryType(subsys);
+	}
+	if (constraint!=NULL) {
+	  query.addANDConstraint(constraint);
+	}
+
+
 	if (pool_addr) {
 		q_result = query.fetchAds(ads, pool_addr, &errstack);
 	} else {
@@ -1090,8 +1188,13 @@ resolveNames( DaemonList* daemon_list, StringList* name_list )
 	}
 	if( had_error ) {
 		if( ! name_list ) {
-			fprintf( stderr, "Can't find addresses for %s's for -all\n", 
-					 real_dt ? daemonString(real_dt) : "daemon" );
+		  if ( constraint!=NULL ) {
+		    fprintf( stderr, "Can't find addresses for %s's for constraint '%s'\n", 
+			     real_dt ? daemonString(real_dt) : "daemon", constraint );
+		  } else {
+		    fprintf( stderr, "Can't find addresses for %s's for -all\n", 
+			     real_dt ? daemonString(real_dt) : "daemon" );
+		  }
 		} else {
 			name_list->rewind();
 			while( (name = name_list->next()) ) {
@@ -1140,7 +1243,7 @@ resolveNames( DaemonList* daemon_list, StringList* name_list )
 						// should we print a warning?
 					continue;
 				}
-				if( stricmp(tmp, host) ) {		// no match
+				if( strcasecmp(tmp, host) ) {		// no match
 					free( tmp );
 					tmp = NULL;
 					continue;
@@ -1163,11 +1266,7 @@ resolveNames( DaemonList* daemon_list, StringList* name_list )
 					  version and the ip/port, and those will
 					  obviously be the same for all slots)
 					*/
-				MyString attr = ATTR_NAME;
-				attr += "=\"";
-				attr += name;
-				attr += '"';
-				ad->Insert( attr.GetCStr() );
+				ad->Assign( ATTR_NAME, name);
 				d = new Daemon( ad, real_dt, pool_addr );
 				free( tmp );
 				tmp = NULL;
@@ -1180,7 +1279,12 @@ resolveNames( DaemonList* daemon_list, StringList* name_list )
 						// should we print a warning?
 					continue;
 				}
-				if( ! stricmp(tmp, name) ) {
+				if( ! strchr(name, '@') ) {
+					host = get_host_part( tmp );
+				} else {
+					host = tmp;
+				}
+				if( ! strcasecmp(host, name) ) {
 					d = new Daemon( ad, real_dt, pool_addr );
 				}
 				free( tmp );
@@ -1202,6 +1306,7 @@ resolveNames( DaemonList* daemon_list, StringList* name_list )
 	if( had_error ) {
 		fprintf( stderr,
 				 "Perhaps you need to query another pool.\n" );
+		all_good = false;
 	}
 	return true;
 }
@@ -1212,12 +1317,20 @@ doCommand( Daemon* d )
 {
 	bool done = false;
 	int	my_cmd = real_cmd;
-
-		// Grab some info about the daemon which is frequently used.
-	char* name = d->name();
-	daemon_t d_type = d->type();
-	bool is_local = d->isLocal();
 	CondorError errstack;
+	bool error = true;
+	char* name;
+	bool is_local;
+	daemon_t d_type;
+
+	do {
+		// Grab some info about the daemon which is frequently used.
+		name = d->name();
+		d_type = d->type();
+		is_local = d->isLocal();
+		ReliSock sock;
+		done = false;
+		my_cmd = real_cmd;
 
 		// If we're trying to send either vacate or checkpoint we have
 		// two possible commands to send, the machine-wide version and
@@ -1228,14 +1341,14 @@ doCommand( Daemon* d )
 		// it's a local vacate, or if we only have a hostname but no
 		// "slotX@...", we want to send a machine-wide command, not the
 		// per-claim one.
-	bool is_per_claim_startd_cmd = false;
-	if( real_cmd == VACATE_CLAIM || real_cmd == PCKPT_JOB ) {
-		if( !all && d_type != DT_ANY && !is_local &&
-			(name && strchr(name, '@')) )
-		{
-			is_per_claim_startd_cmd = true;
+		bool is_per_claim_startd_cmd = false;
+		if( real_cmd == VACATE_CLAIM || real_cmd == PCKPT_JOB ) {
+			if( !all && d_type != DT_ANY && !is_local &&
+				(name && strchr(name, '@')) )
+			{
+				is_per_claim_startd_cmd = true;
+			}
 		}
-	}
 
 		// in general, we never want to send the same command to the
 		// same address.  the only exception is if we're doing a
@@ -1243,212 +1356,222 @@ doCommand( Daemon* d )
 		// the requests even though we're talking to the same address,
 		// since we'll send the claim-id after the command and it
 		// won't be duplication of effort.
-	if( ! is_per_claim_startd_cmd ) {
-		MyString hash_key = d->addr();
-		bool sent_it = false;
-		if( addresses_sent.lookup(hash_key, sent_it) >= 0 && sent_it ) {
+		if( ! is_per_claim_startd_cmd ) {
+			MyString hash_key = d->addr();
+			bool sent_it = false;
+			if( addresses_sent.lookup(hash_key, sent_it) >= 0 && sent_it ) {
+				return;
+			}
+			addresses_sent.insert( hash_key, true );
+		}
+
+			/* Squawking does its own connect... */
+		if( real_cmd == SQUAWK ) {
+			doSquawk( d->addr() );
+			printf ( "Bye!\n" );
 			return;
 		}
-		addresses_sent.insert( hash_key, true );
-	}
 
-		/* Squawking does its own connect... */
-	if( real_cmd == SQUAWK ) {
-		doSquawk( d->addr() );
-		printf ( "Bye!\n" );
-		return;
-	}
-
-	/* Connect to the daemon */
-	ReliSock sock;
-	if( !sock.connect(d->addr()) ) {
-		fprintf( stderr, "Can't connect to %s\n", d->idStr() );
-		return;
-	}
-
-	char	*psubsys = const_cast<char *>(subsys);
-	switch(real_cmd) {
-	case VACATE_CLAIM:
-		if( is_per_claim_startd_cmd ) {
-				// we've got a specific slot, so send the claim after
-				// the command.
-			if( fast ) {
-				my_cmd = VACATE_CLAIM_FAST;
-			}
-			if (!d->startCommand(my_cmd, &sock, 0, &errstack)) {
-				fprintf(stderr, "ERROR\n%s\n", errstack.getFullText(true));
-			}
-			if( !sock.code(name) || !sock.eom() ) {
-				fprintf( stderr, "Can't send %s command to %s\n", 
-							 cmdToStr(my_cmd), d->idStr() );
-				return;
-			} else {
-				done = true;
-			}
+		/* Connect to the daemon */
+		if( sock.connect(d->addr()) ) {
+			error = false;
+//			break;
 		} else {
-			if( fast ) {
-				my_cmd = VACATE_ALL_FAST;
-			} else {
-				my_cmd = VACATE_ALL_CLAIMS;
-			}
+			continue;
 		}
-		break;
 
-	case PCKPT_JOB:
-		if( is_per_claim_startd_cmd ) {
-				// we've got a specific slot, so send the claim after
-				// the command.
-			if( !d->startCommand(my_cmd, &sock, 0, &errstack) ) {
-				fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true));
+		char	*psubsys = const_cast<char *>(subsys);
+		switch(real_cmd) {
+		case VACATE_CLAIM:
+			if( is_per_claim_startd_cmd ) {
+					// we've got a specific slot, so send the claim after
+					// the command.
+				if( fast ) {
+					my_cmd = VACATE_CLAIM_FAST;
+				}
+				if (!d->startCommand(my_cmd, &sock, 0, &errstack)) {
+					fprintf(stderr, "ERROR\n%s\n", errstack.getFullText(true));
+				}
+				if( !sock.code(name) || !sock.end_of_message() ) {
+					fprintf( stderr, "Can't send %s command to %s\n", 
+								 cmdToStr(my_cmd), d->idStr() );
+					all_good = false;
+					return;
+				} else {
+					done = true;
+				}
+			} else {
+				if( fast ) {
+					my_cmd = VACATE_ALL_FAST;
+				} else {
+					my_cmd = VACATE_ALL_CLAIMS;
+				}
 			}
-			if( !sock.code(name) || !sock.eom() ) {
+			break;
+
+		case PCKPT_JOB:
+			if( is_per_claim_startd_cmd ) {
+					// we've got a specific slot, so send the claim after
+					// the command.
+				if( !d->startCommand(my_cmd, &sock, 0, &errstack) ) {
+					fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true));
+				}
+				if( !sock.code(name) || !sock.end_of_message() ) {
+					fprintf( stderr, "Can't send %s command to %s\n",
+								 cmdToStr(my_cmd), d->idStr() );
+					all_good = false;
+					return;
+				} else {
+					done = true;
+				}
+			} else {
+				my_cmd = PCKPT_ALL_JOBS;
+			}
+			break;
+
+		case DAEMON_OFF:
+				// if -fast is used, we need to send a different command.
+			if( fast ) {
+				my_cmd = DAEMON_OFF_FAST;
+			} else if( peaceful_shutdown ) {
+				my_cmd = DAEMON_OFF_PEACEFUL;
+			}
+			if( !d->startCommand( my_cmd, &sock, 0, &errstack) ) {
+				fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true) );
+			}
+			if( !sock.code( psubsys ) || !sock.end_of_message() ) {
 				fprintf( stderr, "Can't send %s command to %s\n",
-							 cmdToStr(my_cmd), d->idStr() );
+							cmdToStr(my_cmd), d->idStr() );
+				all_good = false;
 				return;
 			} else {
 				done = true;
 			}
-		} else {
-			my_cmd = PCKPT_ALL_JOBS;
-		}
-		break;
+			break;
 
-	case DAEMON_OFF:
-			// if -fast is used, we need to send a different command.
-		if( fast ) {
-			my_cmd = DAEMON_OFF_FAST;
-		} else if( peaceful_shutdown ) {
-			my_cmd = DAEMON_OFF_PEACEFUL;
-		}
-		if( !d->startCommand( my_cmd, &sock, 0, &errstack) ) {
-			fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true) );
-		}
-		if( !sock.code( psubsys ) || !sock.eom() ) {
-			fprintf( stderr, "Can't send %s command to %s\n",
-						cmdToStr(my_cmd), d->idStr() );
-			return;
-		} else {
-			done = true;
-		}
-		break;
+		case DAEMON_ON:
+			if( !d->startCommand(my_cmd, &sock, 0, &errstack) ) {
+				fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true) );
+			}
+			if( !sock.code( psubsys ) || !sock.end_of_message() ) {
+				fprintf( stderr, "Can't send %s command to %s\n",
+						 cmdToStr(my_cmd), d->idStr() );
+				all_good = false;
+				return;
+			} else {
+				done = true;
+			}
+			break;
 
-	case DAEMON_ON:
-		if( !d->startCommand(my_cmd, &sock, 0, &errstack) ) {
-			fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true) );
-		}
-		if( !sock.code( psubsys ) || !sock.eom() ) {
-			fprintf( stderr, "Can't send %s command to %s\n",
-					 cmdToStr(my_cmd), d->idStr() );
-			return;
-		} else {
-			done = true;
-		}
-		break;
+		case RESTART:
+			if( peaceful_shutdown ) {
+				my_cmd = RESTART_PEACEFUL;
+			}
+			break;
 
-	case RESTART:
-		if( peaceful_shutdown ) {
-			my_cmd = RESTART_PEACEFUL;
-		}
-		break;
+		case DAEMONS_OFF:
+				// if -fast is used, we need to send a different command.
+			if( fast ) {
+				my_cmd = DAEMONS_OFF_FAST;
+			} else if( peaceful_shutdown ) {
+				my_cmd = DAEMONS_OFF_PEACEFUL;
+			}
+			if( d_type != DT_MASTER ) {
+ 					// if we're trying to send this to anything other than
+ 					// a master (for example, if we were just given a
+ 					// sinful string and we don't know the daemon type)
+ 					// we've got to send a different command. 
+				if( fast ) {
+					my_cmd = DC_OFF_FAST;
+				} else if( peaceful_shutdown ) {
+					my_cmd = DC_OFF_PEACEFUL;
+				} else {
+					my_cmd = DC_OFF_GRACEFUL;
+				}
+			}
+			break;
 
-	case DAEMONS_OFF:
-			// if -fast is used, we need to send a different command.
-		if( fast ) {
-			my_cmd = DAEMONS_OFF_FAST;
-		} else if( peaceful_shutdown ) {
-			my_cmd = DAEMONS_OFF_PEACEFUL;
-		}
-		if( d_type != DT_MASTER ) {
- 				// if we're trying to send this to anything other than
- 				// a master (for example, if we were just given a
- 				// sinful string and we don't know the daemon type)
- 				// we've got to send a different command. 
+		case DC_OFF_GRACEFUL:
+				// if -fast is used, we need to send a different command.
 			if( fast ) {
 				my_cmd = DC_OFF_FAST;
 			} else if( peaceful_shutdown ) {
 				my_cmd = DC_OFF_PEACEFUL;
+			}
+			break;
+
+		case DC_RECONFIG_FULL:
+				// Nothing to do
+			break;
+		case SET_SHUTDOWN_PROGRAM:
+		{
+			char	*pexec = const_cast<char *>(exec_program); 
+			if( !d->startCommand(my_cmd, &sock, 0, &errstack) ) {
+				fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true) );
+			}
+			if( !sock.code( pexec ) || !sock.end_of_message() ) {
+				fprintf( stderr, "Can't send %s command to %s\n",
+						 cmdToStr(my_cmd), d->idStr() );
+				all_good = false;
+				return;
 			} else {
-				my_cmd = DC_OFF_GRACEFUL;
+				done = true;
+			}
+			break;
+		}
+
+		default:
+			break;
+		}
+
+		if( !done ) {
+			if( !d->sendCommand(my_cmd, &sock, 0, &errstack) ) {
+				fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true) );
+				fprintf( stderr, "Can't send %s command to %s\n",
+							 cmdToStr(my_cmd), d->idStr() );
+				all_good = false;
+				return;
 			}
 		}
-		break;
-
-	case DC_OFF_GRACEFUL:
-			// if -fast is used, we need to send a different command.
-		if( fast ) {
-			my_cmd = DC_OFF_FAST;
-		} else if( peaceful_shutdown ) {
-			my_cmd = DC_OFF_PEACEFUL;
-		}
-		break;
-
-	case DC_RECONFIG:
-			// if -full is used, we need to send a different command.
-		if( full ) {
-			my_cmd = DC_RECONFIG_FULL;
-		}
-		break;
-
-	case SET_SHUTDOWN_PROGRAM:
-	{
-		char	*pexec = const_cast<char *>(exec_program); 
-		if( !d->startCommand(my_cmd, &sock, 0, &errstack) ) {
-			fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true) );
-		}
-		if( !sock.code( pexec ) || !sock.eom() ) {
-			fprintf( stderr, "Can't send %s command to %s\n",
-					 cmdToStr(my_cmd), d->idStr() );
-			return;
-		} else {
-			done = true;
-		}
-		break;
-	}
-
-	default:
-		break;
-	}
-
-	if( !done ) {
-		if( !d->sendCommand(my_cmd, &sock, 0, &errstack) ) {
-			fprintf( stderr, "ERROR\n%s\n", errstack.getFullText(true) );
-			fprintf( stderr, "Can't send %s command to %s\n",
-						 cmdToStr(my_cmd), d->idStr() );
-			return;
-		}
-	}
 	
-		// for the purposes of reporting, if what we're trying to do
-		// is a restart, we want to print that here.  so, set my_cmd
-		// back to RESTART.
-	if( cmd == RESTART ) {
-		my_cmd = RESTART;
-	}
-
-		// now, print out the right thing depending on what we did
-	if( my_cmd == DAEMON_ON || my_cmd == DAEMON_OFF || 
-		my_cmd == DAEMON_OFF_FAST || 
-		((my_cmd == DC_OFF_GRACEFUL || my_cmd == DC_OFF_FAST) && 
-		 real_dt == DT_MASTER) )
-	{
-		if( d_type == DT_ANY ) {
-			printf( "Sent \"%s\" command to %s\n",
-					cmdToStr(my_cmd), d->idStr() );
-		} else {
-			printf( "Sent \"%s\" command for \"%s\" to %s\n",
-					cmdToStr(my_cmd), 
-					subsys_arg ? subsys_arg : daemonString(dt),
-					d->idStr() );
+			// for the purposes of reporting, if what we're trying to do
+			// is a restart, we want to print that here.  so, set my_cmd
+			// back to RESTART.
+		if( cmd == RESTART ) {
+			my_cmd = RESTART;
 		}
-	} else if( d_type == DT_STARTD && all ) {
-			// we want to special case printing about this, since
-			// we're doing a machine-wide command...
-		printf( "Sent \"%s\" command to startd %s\n", cmdToStr(my_cmd),
-				d->fullHostname() );
-	} else if( cmd_set ) {
-		printf( "Sent command \"%d\" to %s\n", my_cmd, d->idStr() );
-	} else {
-		printf( "Sent \"%s\" command to %s\n", cmdToStr(my_cmd), d->idStr() );
+
+			// now, print out the right thing depending on what we did
+		if( my_cmd == DAEMON_ON || my_cmd == DAEMON_OFF || 
+			my_cmd == DAEMON_OFF_FAST || 
+			((my_cmd == DC_OFF_GRACEFUL || my_cmd == DC_OFF_FAST) && 
+			 real_dt == DT_MASTER) )
+		{
+			if( d_type == DT_ANY ) {
+				printf( "Sent \"%s\" command to %s\n",
+						cmdToStr(my_cmd), d->idStr() );
+			} else {
+				printf( "Sent \"%s\" command for \"%s\" to %s\n",
+						cmdToStr(my_cmd), 
+						subsys_arg ? subsys_arg : daemonString(dt),
+						d->idStr() );
+			}
+		} else if( d_type == DT_STARTD && all ) {
+				// we want to special case printing about this, since
+				// we're doing a machine-wide command...
+			printf( "Sent \"%s\" command to startd %s\n", cmdToStr(my_cmd),
+					d->fullHostname() );
+		} else if( cmd_set ) {
+			printf( "Sent command \"%d\" to %s\n", my_cmd, d->idStr() );
+		} else {
+			printf( "Sent \"%s\" command to %s\n", cmdToStr(my_cmd), d->idStr() );
+		}
+		sock.close();
+	} while(d->nextValidCm() == true);
+	if( error == true ) {
+		fprintf( stderr, "Can't connect to %s\n", d->idStr() );
+		all_good = false;
+		return;
 	}
 }
 
@@ -1461,21 +1584,29 @@ version()
 }
 
 
-// Want to send a command to all hosts in the pool.
+// Want to send a command to all hosts in the pool or
+// modulo the constraint, of course
 void
 handleAll()
 {
 	DaemonList daemons;
 
 	if( ! resolveNames(&daemons, NULL) ) {
-		fprintf( stderr, "ERROR: Failed to find daemons for -all\n" );
-		exit( 1 );
+	  if ( constraint!=NULL ) {
+	    fprintf( stderr, "ERROR: Failed to find daemons matching the constraint\n" );
+	  } else {
+	    fprintf( stderr, "ERROR: Failed to find daemons for -all\n" );
+	  }
+	  exit( 1 );
 	}
 
 		// Now, send commands to all the daemons we know about.
 	Daemon* d;
 	daemons.rewind();	
 	while( daemons.next(d) ) {
+		if( real_dt == DT_GENERIC ) {
+			d->setSubsystem( subsys );
+		}
 		doCommand( d );
 	}
 }
@@ -1570,7 +1701,7 @@ handleSquawk( char *line, char *addr ) {
 
 		sock.encode();
 		sock.code( signal );
-		sock.eom();
+		sock.end_of_message();
 		
 		return TRUE;
 	}
@@ -1607,7 +1738,7 @@ handleSquawk( char *line, char *addr ) {
 				}
 			}
 		}
-		sock.eom();
+		sock.end_of_message();
 
 		return TRUE;
 	}
@@ -1706,8 +1837,12 @@ doSquawkReconnect( char *addr ) {
 		}
 	}
 	Daemon d( dt, hostname, pool ? pool->addr() : NULL );
+	if( real_dt == DT_GENERIC ) {
+		d.setSubsystem( subsys );
+	}
 	if( ! d.locate() ) {
 		printf ( "Failed to contact daemon.\n" );
+		delete [] hostname;
 		return FALSE;
 	}
 	strcpy ( addr, d.addr() );
