@@ -40,10 +40,13 @@ struct LineRec {
   MyString Name;
   float Priority;
   int Res;
+  float wtRes;
   float AccUsage;
   float Factor;
   int BeginUsage;
   int LastUsage;
+  MyString AcctGroup;
+  bool IsAcctGroup;
 };
 
 //-----------------------------------------------------------------
@@ -441,7 +444,7 @@ main(int argc, char* argv[])
     // get reply
     sock->decode();
     AttrList* ad=new AttrList();
-    if (!ad->initFromStream(*sock) ||
+    if (!ad->initAttrListFromStream(*sock) ||
         !sock->end_of_message()) {
       fprintf( stderr, "failed to get classad from negotiator\n" );
       exit(1);
@@ -464,12 +467,12 @@ main(int argc, char* argv[])
     }
 
 	// ship it out
-	sock->eom();
+	sock->end_of_message();
 
     // get reply
     sock->decode();
     AttrList* ad=new AttrList();
-    if (!ad->initFromStream(*sock) ||
+    if (!ad->initAttrListFromStream(*sock) ||
         !sock->end_of_message()) {
       fprintf( stderr, "failed to get classad from negotiator\n" );
       exit(1);
@@ -534,12 +537,17 @@ int CompPrio(const void * ina, const void * inb)
 
 static void CollectInfo(int numElem, AttrList* ad, LineRec* LR)
 {
-  char  attrName[32], attrPrio[32], attrResUsed[32], attrFactor[32], attrBeginUsage[32], attrAccUsage[32];
+  char  attrName[32], attrPrio[32], attrResUsed[32], attrWtResUsed[32], attrFactor[32], attrBeginUsage[32], attrAccUsage[42];
   char  attrLastUsage[32];
+  MyString attrAcctGroup;
+  MyString attrIsAcctGroup;
   char  name[128];
-  float priority, Factor, AccUsage;
-  int   resUsed, BeginUsage;
-  int   LastUsage;
+  float priority, Factor, AccUsage = -1;
+  int   resUsed = 0, BeginUsage = 0;
+  int   LastUsage = 0;
+  float wtResUsed;
+  MyString AcctGroup;
+  bool IsAcctGroup;
 
   for( int i=1; i<=numElem; i++) {
     LR[i-1].Priority=0;
@@ -547,10 +555,13 @@ static void CollectInfo(int numElem, AttrList* ad, LineRec* LR)
     sprintf( attrName , "Name%d", i );
     sprintf( attrPrio , "Priority%d", i );
     sprintf( attrResUsed , "ResourcesUsed%d", i );
+    sprintf( attrWtResUsed , "WeightedResourcesUsed%d", i );
     sprintf( attrFactor , "PriorityFactor%d", i );
     sprintf( attrBeginUsage , "BeginUsageTime%d", i );
     sprintf( attrLastUsage , "LastUsageTime%d", i );
-    sprintf( attrAccUsage , "AccumulatedUsage%d", i );
+    sprintf( attrAccUsage , "WeightedAccumulatedUsage%d", i );
+    attrAcctGroup.sprintf("AccountingGroup%d", i);
+    attrIsAcctGroup.sprintf("IsAccountingGroup%d", i);
 
     if( !ad->LookupString	( attrName, name ) 		|| 
 		!ad->LookupFloat	( attrPrio, priority ) )
@@ -563,16 +574,29 @@ static void CollectInfo(int numElem, AttrList* ad, LineRec* LR)
 		!ad->LookupInteger	( attrResUsed, resUsed ) ) 
 			DetailFlag=0;
 
+	if( !ad->LookupFloat( attrWtResUsed, wtResUsed ) ) {
+		wtResUsed = resUsed;
+	}
+
+    if (!ad->LookupString(attrAcctGroup.Value(), AcctGroup)) {
+        AcctGroup = "<none>";
+    }
+    if (!ad->LookupBool(attrIsAcctGroup.Value(), IsAcctGroup)) {
+        IsAcctGroup = false;
+    }
+
 	if (LastUsage==0) LastUsage=-1;
 
     LR[i-1].Name=name;
     LR[i-1].Priority=priority;
     LR[i-1].Res=resUsed;
+    LR[i-1].wtRes=wtResUsed;
     LR[i-1].Factor=Factor;
     LR[i-1].BeginUsage=BeginUsage;
     LR[i-1].LastUsage=LastUsage;
     LR[i-1].AccUsage=AccUsage;
-
+    LR[i-1].AcctGroup=AcctGroup;
+    LR[i-1].IsAcctGroup=IsAcctGroup;
   }
  
   // ad->fPrint(stdout);
@@ -591,17 +615,18 @@ static void PrintInfo(AttrList* ad, LineRec* LR, int NumElem)
 
   LineRec Totals;
   Totals.Res=0;
+  Totals.wtRes=0.0;
   Totals.BeginUsage=0;
   Totals.AccUsage=0;
   
-  char* Fmt1="%-30s %14.2f\n";  // Data line format
-  char* Fmt2="%-30s %14s\n";    // Title and separator line format
-  char* Fmt3="Number of users shown: %-13d %14s\n";    // Totals line format
+  const char* Fmt1="%-30s %14.2f\n";  // Data line format
+  const char* Fmt2="%-30s %14s\n";    // Title and separator line format
+  const char* Fmt3="Number of users shown: %-13d %14s\n";  // Totals line format
 
   if (DetailFlag==1) {
-    Fmt1="%-30s %14.2f %8.2f %12.2f %4d %12.2f %14s %14s\n"; 
+    Fmt1="%-30s %14.2f %8.2f %12.2f %4.0f %12.2f %14s %14s\n"; 
     Fmt2="%-30s %14s %8s %12s %4s %12s %14s %14s\n"; 
-    Fmt3="Number of users: %-13d %14s %8s %12s %4d %12.2f %14s %14s\n"; 
+    Fmt3="Number of users: %-13d %14s %8s %12s %4.0f %12.2f %14s %14s\n"; 
   }
   else if (DetailFlag==2) {
     Fmt1="%-30s %12.2f %14s %14s\n"; 
@@ -610,39 +635,47 @@ static void PrintInfo(AttrList* ad, LineRec* LR, int NumElem)
   }
 
   if (DetailFlag==2) {
-    printf(Fmt2,"         ","Accumulated","     Usage    ","     Last     ");
-    printf(Fmt2,"User Name","Usage (hrs)","  Start Time  ","  Usage Time  ");
+    printf(Fmt2,"         ","Total Usage","     Usage    ","     Last     ");
+    printf(Fmt2,"User Name","(wghted-hrs)","  Start Time  ","  Usage Time  ");
     printf(Fmt2,"------------------------------","-----------","----------------","----------------");
   }
   else {
-    printf(Fmt2,"         ","Effective","  Real  ","  Priority  ","Res ","Accumulated","      Usage     ","      Last      ");
-    printf(Fmt2,"User Name","Priority ","Priority","   Factor   ","Used","Usage (hrs)","   Start Time   ","   Usage Time   ");
+    printf(Fmt2,"         ","Effective","  Real  ","  Priority  ","Res ","Total Usage","      Usage     ","      Last      ");
+    printf(Fmt2,"User Name","Priority ","Priority","   Factor   ","Used","(wghted-hrs)","   Start Time   ","   Usage Time   ");
     printf(Fmt2,"------------------------------","---------","--------","------------","----","-----------","----------------","----------------");
   }
 
   int UserCount=0;
   for (int i=0; i<NumElem; i++) {
+    // We want to avoid counting totals twice for acct group records
+    bool is_group = LR[i].IsAcctGroup;
+
 	if (LR[i].LastUsage<MinLastUsageTime) continue;
-    UserCount++;
+
+    if (!is_group) UserCount++;
+
     strcpy(LastUsageStr,format_date_year(LR[i].LastUsage));
     if (LR[i].Name.Length()>30) LR[i].Name=LR[i].Name.Substr(0,29);
     if (DetailFlag==2)
       printf(Fmt1,LR[i].Name.Value(),LR[i].AccUsage/3600.0,format_date_year(LR[i].BeginUsage),LastUsageStr);
     else 
-      printf(Fmt1,LR[i].Name.Value(),LR[i].Priority,(LR[i].Priority/LR[i].Factor),LR[i].Factor,LR[i].Res,LR[i].AccUsage/3600.0,format_date_year(LR[i].BeginUsage),LastUsageStr);
-    Totals.Res+=LR[i].Res;
-    Totals.AccUsage+=LR[i].AccUsage;
-    if (LR[i].BeginUsage<Totals.BeginUsage || Totals.BeginUsage==0) Totals.BeginUsage=LR[i].BeginUsage;
+      printf(Fmt1,LR[i].Name.Value(),LR[i].Priority,(LR[i].Priority/LR[i].Factor),LR[i].Factor,LR[i].wtRes,LR[i].AccUsage/3600.0,format_date_year(LR[i].BeginUsage),LastUsageStr);
+
+    if (!is_group) {
+      Totals.wtRes+=LR[i].wtRes;
+      Totals.AccUsage+=LR[i].AccUsage;
+      if (LR[i].BeginUsage<Totals.BeginUsage || Totals.BeginUsage==0) Totals.BeginUsage=LR[i].BeginUsage;
+    }
   }
 
   strcpy(LastUsageStr,format_date_year(MinLastUsageTime));
   if (DetailFlag==2) {
     printf(Fmt2,"------------------------------","-----------","----------------","----------------");
-    printf(Fmt3,NumElem,Totals.AccUsage/3600.0,format_date_year(Totals.BeginUsage),LastUsageStr);
+    printf(Fmt3,UserCount,Totals.AccUsage/3600.0,format_date_year(Totals.BeginUsage),LastUsageStr);
   }
   else {
     printf(Fmt2,"------------------------------","---------","--------","------------","----","-----------","----------------","----------------");
-    printf(Fmt3,UserCount,"","","",Totals.Res,Totals.AccUsage/3600.0,format_date_year(Totals.BeginUsage),LastUsageStr);
+    printf(Fmt3,UserCount,"","","",Totals.wtRes,Totals.AccUsage/3600.0,format_date_year(Totals.BeginUsage),LastUsageStr);
   }
 
   return;
