@@ -355,7 +355,7 @@ static void change_snapshot_vmsd_file(const char *file, StringList *parent_filen
 		}
 
 		MyString tmp_name = name;
-		tmp_name.strlwr();
+		tmp_name.lower_case();
 		if( tmp_name.find( ".filename", 0 ) > 0 ) {
 			if( parent_filenames->contains(condor_basename(value.Value()))) {
 				if( use_fullpath ) {
@@ -416,6 +416,9 @@ VMwareType::VMwareType(const char* prog_for_script, const char* scriptname,
 	VMType(prog_for_script, scriptname, workingpath, ad)
 {
 	m_vmtype = CONDOR_VM_UNIVERSE_VMWARE;
+
+	//m_cputime_before_suspend = 0;
+
 	m_need_snapshot = false;
 	m_restart_with_ckpt = false;
 	m_vmware_transfer = false;
@@ -436,6 +439,12 @@ VMwareType::~VMwareType()
 	setVMStatus(VM_STOPPED);
 }
 
+void
+VMwareType::Config()
+{
+	// Nothing to do
+}
+
 void
 VMwareType::adjustConfigDiskPath()
 {
@@ -491,7 +500,7 @@ VMwareType::adjustConfigDiskPath()
 		if( !strncasecmp(name.Value(), "scsi", strlen("scsi")) ||
 				!strncasecmp(name.Value(), "ide", strlen("ide"))) {
 			MyString tmp_name = name;
-			tmp_name.strlwr();
+			tmp_name.lower_case();
 			if( tmp_name.find( ".filename", 0 ) > 0 ) {
 				if( has_suffix(value.Value(), ".vmdk") && 
 						(fullpath(value.Value()) == false) ) {
@@ -642,7 +651,7 @@ VMwareType::adjustCkptConfig(const char* vmconfig)
 				MyString tmp_string2;
 
 				tmp_string2 = m_vm_networking_type;
-				tmp_string2.strupr();
+				tmp_string2.upper_case();
 
 				tmp_string.sprintf("VMWARE_%s_NETWORKING_TYPE", tmp_string2.Value());
 
@@ -761,13 +770,15 @@ VMwareType::readVMXfile(const char *filename, const char *dirpath)
 		}
 
 		MyString tmp_name = name;
-		tmp_name.strlwr();
+		tmp_name.lower_case();
 		if( tmp_name.find( ".devicetype", 0 ) > 0 ) {
 			// find cdrom device
 #define CDROM_TYPE1		"atapi-cdrom"
 #define CDROM_TYPE2		"cdrom-raw"
+#define CDROM_TYPE3		"cdrom-image"
 			if( (strcasecmp(value.Value(), CDROM_TYPE1) == 0 ) ||
-					(strcasecmp(value.Value(), CDROM_TYPE2) == 0 )) {
+				(strcasecmp(value.Value(), CDROM_TYPE2) == 0 ) ||
+				(strcasecmp(value.Value(), CDROM_TYPE3) == 0 )) {
 				pos = name.FindChar('.', 0);
 				if( pos > 0 ) {
 					name.setChar(pos, '\0');
@@ -813,7 +824,7 @@ VMwareType::readVMXfile(const char *filename, const char *dirpath)
 		}else if( !strncasecmp(line, "scsi", strlen("scsi")) ||
 				!strncasecmp(line, "ide", strlen("ide"))) {
 			MyString tmp_name = name;
-			tmp_name.strlwr();
+			tmp_name.lower_case();
 			if( tmp_name.find( ".filename", 0 ) > 0 ) {
 
 				// Adjust filename
@@ -854,7 +865,7 @@ VMwareType::readVMXfile(const char *filename, const char *dirpath)
 						tmp_line.sprintf("%s = \"%s\"", name.Value(), 
 								tmp_fullname.Value());
 
-						if( check_vm_read_access_file(tmp_fullname.Value()) 
+						if( !(*dirpath) || check_vm_read_access_file(tmp_fullname.Value()) 
 								== false ) {
 							vmprintf(D_ALWAYS, "file(%s) in a vmx file cannot "
 									"be read\n", tmp_fullname.Value());
@@ -877,7 +888,7 @@ VMwareType::readVMXfile(const char *filename, const char *dirpath)
 				continue;
 			}else if( (tmp_name.find(".mode", 0 ) > 0) && (value.Length() > 0)) {
 				MyString tmp_value = value;
-				tmp_value.strlwr();
+				tmp_value.lower_case();
 				if( tmp_value.find( "independent", 0 ) >= 0 ) {
 					if( !m_vmware_transfer ) {
 						// In VMware, independent disks are not affected 
@@ -899,40 +910,6 @@ VMwareType::readVMXfile(const char *filename, const char *dirpath)
 		}
 	}
 
-	if( m_iso_file.IsEmpty() == false ) {
-		// There is an ISO iamge. So we use it as a CDROM
-		if( cdrom_devices.isEmpty() ) {
-			vmprintf(D_ALWAYS, "Job user defined files for a CDROM, "
-					"but no CDROM device is found in a vmx file\n");
-			m_result_msg = VMGAHP_ERR_JOBCLASSAD_VMWARE_NO_CDROM_DEVICE;
-			return false;
-		}
-
-		cdrom_devices.rewind();
-		// Obtain the first device name for CDROM
-		cdrom = cdrom_devices.next();
-
-		MyString tmp_string;
-		tmp_string.sprintf("%s.present = \"TRUE\"", cdrom);
-		m_configVars.append(tmp_string.Value());
-
-		if( m_local_iso ) {
-			// use basename
-			tmp_string.sprintf("%s.fileName = \"%s\"", cdrom, 
-					condor_basename(m_iso_file.Value()));
-		}else {
-			tmp_string.sprintf("%s.fileName = \"%s\"", cdrom, 
-					m_iso_file.Value());
-		}
-		m_configVars.append(tmp_string.Value());
-
-		tmp_string.sprintf("%s.deviceType = \"cdrom-image\"", cdrom);
-		m_configVars.append(tmp_string.Value());
-
-		tmp_string.sprintf("%s.autodetect = \"TRUE\"", cdrom);
-		m_configVars.append(tmp_string.Value());
-	}
-
 	return true;
 }
 
@@ -952,7 +929,7 @@ VMwareType::CombineDisks()
 	systemcmd.AppendArg("commit");
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false);
+	int result = systemCommand(systemcmd, m_file_owner);
 	if( result != 0 ) {
 		return false;
 	}
@@ -976,7 +953,7 @@ VMwareType::Unregister()
 	systemcmd.AppendArg("unregister");
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false);
+	int result = systemCommand(systemcmd, m_file_owner);
 	if( result != 0 ) {
 		return false;
 	}
@@ -1002,7 +979,7 @@ VMwareType::Snapshot()
 	systemcmd.AppendArg("snapshot");
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false, &cmd_out);
+	int result = systemCommand(systemcmd, m_file_owner, &cmd_out);
 	if( result != 0 ) {
 		char *temp = cmd_out.print_to_delimed_string("/");
 		m_result_msg = temp;
@@ -1055,6 +1032,7 @@ VMwareType::Start()
 					m_result_msg = VMGAHP_ERR_CRITICAL;
 					return false;
 				}
+				m_file_owner = PRIV_CONDOR;
 			}
 #endif
 
@@ -1081,6 +1059,7 @@ VMwareType::Start()
 			m_result_msg = VMGAHP_ERR_CRITICAL;
 			return false;
 		}
+		m_file_owner = PRIV_USER;
 	}
 #endif
 
@@ -1099,7 +1078,7 @@ VMwareType::Start()
 	systemcmd.AppendArg("start");
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false, &cmd_out);
+	int result = systemCommand(systemcmd, m_file_owner, &cmd_out);
 	if( result != 0 ) {
 		Unregister();
 		char *temp = cmd_out.print_to_delimed_string("/");
@@ -1123,6 +1102,7 @@ VMwareType::Start()
 
 	setVMStatus(VM_RUNNING);
 	m_start_time.getTime();
+    //m_cpu_time = 0;
 	return true;
 }
 
@@ -1159,7 +1139,7 @@ VMwareType::ShutdownGraceful()
 	systemcmd.AppendArg("stop");
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false);
+	int result = systemCommand(systemcmd, m_file_owner);
 	if( result != 0 ) {
 		return false; 
 	}
@@ -1184,6 +1164,7 @@ VMwareType::Shutdown()
 			m_result_msg = VMGAHP_ERR_CRITICAL;
 			return false;
 		}
+		m_file_owner = PRIV_CONDOR;
 	}
 #endif
 
@@ -1370,7 +1351,7 @@ VMwareType::Suspend()
 	systemcmd.AppendArg("suspend");
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false, &cmd_out);
+	int result = systemCommand(systemcmd, m_file_owner, &cmd_out);
 	if( result != 0 ) {
 		char *temp = cmd_out.print_to_delimed_string("/");
 		m_result_msg = temp;
@@ -1381,6 +1362,8 @@ VMwareType::Suspend()
 	// Suspend succeeds. So there is no process for VM.
 	m_vm_pid = 0;
 	setVMStatus(VM_SUSPENDED);
+	//m_cputime_before_suspend += m_cpu_time;
+	//m_cpu_time = 0;
 	return true;
 }
 
@@ -1417,7 +1400,7 @@ VMwareType::Resume()
 	systemcmd.AppendArg("resume");
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false, &cmd_out);
+	int result = systemCommand(systemcmd, m_file_owner, &cmd_out);
 	if( result != 0 ) {
 		char *temp = cmd_out.print_to_delimed_string("/");
 		m_result_msg = temp;
@@ -1490,7 +1473,7 @@ VMwareType::Status()
 	}
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false, &cmd_out);
+	int result = systemCommand(systemcmd, m_file_owner, &cmd_out);
 	if( result != 0 ) {
 		char *temp = cmd_out.print_to_delimed_string("/");
 		m_result_msg = temp;
@@ -1506,6 +1489,7 @@ VMwareType::Status()
 
 	MyString vm_status;
 	int vm_pid = 0;
+	float cputime = 0;
 	cmd_out.rewind();
 	while( (next_line = cmd_out.next()) != NULL ) {
 		one_line = next_line;
@@ -1524,6 +1508,14 @@ VMwareType::Status()
 		if( !name.Length() || !value.Length() ) {
 			continue;
 		}
+		
+		if( !strcasecmp(name.Value(), VMGAHP_STATUS_COMMAND_CPUTIME)) {
+			cputime = (float)strtod(value.Value(), (char **)NULL);
+			if( cputime <= 0 ) {
+				cputime = 0;
+			}
+			continue;
+		}
 
 		if( !strcasecmp(name.Value(), VMGAHP_STATUS_COMMAND_STATUS)) {
 			vm_status = value;
@@ -1597,6 +1589,17 @@ VMwareType::Status()
 		m_result_msg += VMGAHP_STATUS_COMMAND_PID;
 		m_result_msg += "=";
 		m_result_msg += m_vm_pid;
+		if( cputime > 0 ) {
+			// Update vm running time
+			m_cpu_time = cputime;
+
+			m_result_msg += " ";
+			m_result_msg += VMGAHP_STATUS_COMMAND_CPUTIME;
+			m_result_msg += "=";
+			m_result_msg += m_cpu_time;
+			//m_result_msg += (double)(m_cpu_time + m_cputime_before_suspend);
+		}
+
 		return true;
 
 	}else if( strcasecmp(vm_status.Value(), "Suspended") == 0 ) {
@@ -1656,7 +1659,7 @@ VMwareType::getPIDofVM(int &vm_pid)
 	systemcmd.AppendArg("getpid");
 	systemcmd.AppendArg(m_configfile);
 
-	int result = systemCommand(systemcmd, false, &cmd_out);
+	int result = systemCommand(systemcmd, m_file_owner, &cmd_out);
 	if( result != 0 ) {
 		return false;
 	}
@@ -1697,12 +1700,8 @@ VMwareType::CreateConfigFile()
 	m_classAd.LookupBool(VMPARAM_VMWARE_SNAPSHOTDISK, m_vmware_snapshot_disk);
 
 	// Read the directory where vmware files are on a submit machine
-	if( m_classAd.LookupString(VMPARAM_VMWARE_DIR, m_vmware_dir) != 1 ) {
-		vmprintf(D_ALWAYS, "%s cannot be found in job classAd\n", 
-							VMPARAM_VMWARE_DIR);
-		m_result_msg = VMGAHP_ERR_JOBCLASSAD_NO_VMWARE_DIR_PARAM;
-		return false;
-	}
+	m_vmware_dir = "";
+	m_classAd.LookupString(VMPARAM_VMWARE_DIR, m_vmware_dir);
 	m_vmware_dir.trim();
 
 	// Read the parameter of vmware vmx file
@@ -1760,16 +1759,6 @@ VMwareType::CreateConfigFile()
 		}
 	}
 
-	if( (m_vm_cdrom_files.isEmpty() == false) && !m_has_iso ) {
-		// Create ISO file
-		if( createISO() == false ) {
-			vmprintf(D_ALWAYS, "Cannot create a ISO file for CDROM\n");
-			m_iso_file = "";
-			m_result_msg = VMGAHP_ERR_CANNOT_CREATE_ISO_FILE;
-			return false;
-		}
-	}
-
 	// Create vm config file
 	if( createTempFile(VMWARE_TMP_TEMPLATE, VMWARE_TMP_CONFIG_SUFFIX, 
 				tmp_config_name) == false ) {
@@ -1813,7 +1802,7 @@ VMwareType::CreateConfigFile()
 		MyString tmp_string2;
 
 		tmp_string2 = m_vm_networking_type;
-		tmp_string2.strupr();
+		tmp_string2.upper_case();
 
 		tmp_string.sprintf("VMWARE_%s_NETWORKING_TYPE", tmp_string2.Value());
 
@@ -1836,7 +1825,20 @@ VMwareType::CreateConfigFile()
 		tmp_line.sprintf("ethernet0.connectionType = \"%s\"", 
 				networking_type.Value());
 		m_configVars.append(tmp_line.Value());
-		m_configVars.append("ethernet0.addressType = \"generated\"");
+        if (!m_vm_job_mac.IsEmpty())
+        {
+            vmprintf(D_FULLDEBUG, "mac address is %s\n", m_vm_job_mac.Value());
+            m_configVars.append("ethernet0.addressType = \"static\"");
+            tmp_line.sprintf("ethernet0.address = \"%s\"", m_vm_job_mac.Value());
+            m_configVars.append(tmp_line.Value());
+            //**********************************************************************
+            // LIMITATION: the mac address has to be in the range
+            // 00:50:56:00:00:00 - 00:50:56:3F:FF:FF
+            // This is a vmware limitation and I can't find a way to circumvent it.
+            //**********************************************************************
+        } else {
+    		m_configVars.append("ethernet0.addressType = \"generated\"");
+        }
 	}
 
 	// Add uuid option
@@ -1940,7 +1942,6 @@ VMwareType::createCkptFiles()
 					!has_suffix(tmp_file, ".log") &&
 					!has_suffix(tmp_file, VMWARE_WRITELOCK_SUFFIX ) &&
 					!has_suffix(tmp_file, VMWARE_READLOCK_SUFFIX ) &&
-					!filelist_contains_file(tmp_file, &m_vm_cdrom_files, true) &&
 					strcmp(condor_basename(tmp_file), m_vmware_vmx.Value())) {
 				// We update mtime and atime of all files 
 				// except vmdk, iso, log, lock files, cdrom file, and 
@@ -2042,7 +2043,7 @@ VMwareType::testVMware(VMGahpConfig* config)
 	systemcmd.AppendArg(config->m_vm_script);
 	systemcmd.AppendArg("check");
 
-	int result = systemCommand(systemcmd, false);
+	int result = systemCommand(systemcmd, PRIV_USER);
 	if( result != 0 ) {
 		vmprintf( D_ALWAYS, "VMware script check failed:\n" );
 		return false;
@@ -2085,7 +2086,7 @@ VMwareType::killVMFast(const char* prog_for_script, const char* script,
 	systemcmd.AppendArg("killvm");
 	systemcmd.AppendArg(matchstring);
 
-	int result = systemCommand(systemcmd, is_root);
+	int result = systemCommand(systemcmd, is_root ? PRIV_ROOT : PRIV_USER);
 	if( result != 0 ) {
 		return false;
 	}
