@@ -1,4 +1,4 @@
-/* Modified from Globus 4.2.0 for use with the NorduGrid GAHP server. */
+/* Modified from Globus 5.0.0 for use with the NorduGrid GAHP server. */
 /*
  * Copyright 1999-2006 University of Chicago
  * 
@@ -21,10 +21,13 @@
  * Globus FTP Client Library
  *
  * $RCSfile: globus_i_ftp_client.h,v $
- * $Revision: 1.45 $
- * $Date: 2008/04/04 01:51:47 $
+ * $Revision: 1.47 $
+ * $Date: 2008/10/29 22:59:05 $
  */
 
+#ifndef GLOBUS_L_INCLUDE_FTP_CLIENT_H
+#define GLOBUS_L_INCLUDE_FTP_CLIENT_H
+
 #include "globus_common.h"
 #include "globus_ftp_client.h"
 #include "globus_ftp_client_plugin.h"
@@ -33,8 +36,7 @@
 
 #define SSH_EXEC_SCRIPT "gridftp-ssh"
 
-#ifndef GLOBUS_L_INCLUDE_FTP_CLIENT_H
-#define GLOBUS_L_INCLUDE_FTP_CLIENT_H
+#define GLOBUS_L_FTP_CLIENT_CLIENTINFO_APPNAME "libglobus_ftp_client"
 
 #ifndef EXTERN_C_BEGIN
 #ifdef __cplusplus
@@ -104,6 +106,9 @@ do { \
 } while (0)
 
 #define globus_i_ftp_client_debug_states(level, handle)                     \
+  do {                                                                      \
+  if(handle != NULL)                                                        \
+  {                                                                         \
     globus_i_ftp_client_debug_printf(level, (stderr,                        \
         "   handle state = %s\n"                                            \
         "   source state = %s\n"                                            \
@@ -114,7 +119,13 @@ do { \
             : "NULL",                                                       \
         handle->dest                                                        \
             ? globus_i_ftp_target_state_to_string(handle->dest->state)      \
-            : "NULL"))
+            : "NULL"));                                                     \
+   }                                                                        \
+   else                                                                     \
+   {                                                                        \
+    globus_i_ftp_client_debug_printf(level, (stderr, "handle is null\n"));  \
+   }                                                                        \
+} while(0)
             
 #else
 #define globus_i_ftp_client_debug_printf(level, message)
@@ -157,6 +168,7 @@ typedef struct globus_i_ftp_client_operationattr_t
     char *                                      net_stack_str;
     char *                                      disk_stack_str;
     char *                                      module_alg_str;
+    char *                                      clientinfo_argstr;
 }
 globus_i_ftp_client_operationattr_t;
 
@@ -238,6 +250,11 @@ typedef struct globus_i_ftp_client_handleattr_t
     globus_bool_t                               nl_ftp;
     globus_bool_t                               nl_io;
     globus_netlogger_handle_t *                 nl_handle;
+    
+    /* client info */
+    char *                                      clientinfo_app_name;
+    char *                                      clientinfo_app_ver;
+    char *                                      clientinfo_other;
 }
 globus_i_ftp_client_handleattr_t;
 
@@ -306,6 +323,8 @@ typedef enum
     GLOBUS_FTP_CLIENT_TARGET_SITE_HELP,
     GLOBUS_FTP_CLIENT_TARGET_FEAT,
     GLOBUS_FTP_CLIENT_TARGET_SETUP_CONNECTION,
+    GLOBUS_FTP_CLIENT_TARGET_SETUP_CLIENTINFO,
+    GLOBUS_FTP_CLIENT_TARGET_CLIENTINFO,
     GLOBUS_FTP_CLIENT_TARGET_SETUP_TYPE,
     GLOBUS_FTP_CLIENT_TARGET_TYPE,
     GLOBUS_FTP_CLIENT_TARGET_SETUP_MODE,
@@ -701,6 +720,8 @@ typedef struct globus_i_ftp_client_target_s
     globus_bool_t                               dst_command_sent;
     
     globus_list_t *                             net_stack_list;
+    
+    char *                                      clientinfo_argstr;
 } globus_i_ftp_client_target_t;
 
 /**
